:root {
    color-scheme: light;
    --pf: 259 94% 44%;
    --sf: 314 100% 40%;
    --af: 174 75% 39%;
    --nf: 214 20% 14%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 259 94% 51%;
    --pc: 259 96% 91%;
    --s: 314 100% 47%;
    --sc: 314 100% 91%;
    --a: 174 75% 46%;
    --ac: 174 75% 11%;
    --n: 214 20% 21%;
    --nc: 212 19% 87%;
    --b1: 0 0% 100%;
    --b2: 0 0% 95%;
    --b3: 180 2% 90%;
    --bc: 215 28% 17%
}
@media (prefers-color-scheme: dark) {
    :root {
        color-scheme: dark;
        --pf: 262 80% 43%;
        --sf: 316 70% 43%;
        --af: 175 70% 34%;
        --in: 198 93% 60%;
        --su: 158 64% 52%;
        --wa: 43 96% 56%;
        --er: 0 91% 71%;
        --inc: 198 100% 12%;
        --suc: 158 100% 10%;
        --wac: 43 100% 11%;
        --erc: 0 100% 14%;
        --rounded-box: 1rem;
        --rounded-btn: 0.5rem;
        --rounded-badge: 1.9rem;
        --animation-btn: 0.25s;
        --animation-input: .2s;
        --btn-text-case: uppercase;
        --btn-focus-scale: 0.95;
        --border-btn: 1px;
        --tab-border: 1px;
        --tab-radius: 0.5rem;
        --p: 262 80% 50%;
        --pc: 0 0% 100%;
        --s: 316 70% 50%;
        --sc: 0 0% 100%;
        --a: 175 70% 41%;
        --ac: 0 0% 100%;
        --n: 213 18% 20%;
        --nf: 212 17% 17%;
        --nc: 220 13% 69%;
        --b1: 212 18% 14%;
        --b2: 213 18% 12%;
        --b3: 213 18% 10%;
        --bc: 220 13% 69%
    }
}
[data-theme=light] {
    color-scheme: light;
    --pf: 259 94% 44%;
    --sf: 314 100% 40%;
    --af: 174 75% 39%;
    --nf: 214 20% 14%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 259 94% 51%;
    --pc: 259 96% 91%;
    --s: 314 100% 47%;
    --sc: 314 100% 91%;
    --a: 174 75% 46%;
    --ac: 174 75% 11%;
    --n: 214 20% 21%;
    --nc: 212 19% 87%;
    --b1: 0 0% 100%;
    --b2: 0 0% 95%;
    --b3: 180 2% 90%;
    --bc: 215 28% 17%
}
[data-theme=dark] {
    color-scheme: dark;
    --pf: 262 80% 43%;
    --sf: 316 70% 43%;
    --af: 175 70% 34%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 262 80% 50%;
    --pc: 0 0% 100%;
    --s: 316 70% 50%;
    --sc: 0 0% 100%;
    --a: 175 70% 41%;
    --ac: 0 0% 100%;
    --n: 213 18% 20%;
    --nf: 212 17% 17%;
    --nc: 220 13% 69%;
    --b1: 212 18% 14%;
    --b2: 213 18% 12%;
    --b3: 213 18% 10%;
    --bc: 220 13% 69%
}
[data-theme=cupcake] {
    color-scheme: light;
    --pf: 183 47% 52%;
    --sf: 338 71% 71%;
    --af: 39 84% 51%;
    --nf: 280 46% 7%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --pc: 183 20% 13%;
    --sc: 340 15% 16%;
    --ac: 37 41% 13%;
    --nc: 283 9% 81%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --p: 183 47% 59%;
    --s: 338 71% 78%;
    --a: 39 84% 58%;
    --n: 280 46% 14%;
    --b1: 24 33% 97%;
    --b2: 27 22% 92%;
    --b3: 23 14% 89%;
    --bc: 280 46% 14%;
    --rounded-btn: 1.9rem;
    --tab-border: 2px;
    --tab-radius: .5rem
}
[data-theme=bumblebee] {
    color-scheme: light;
    --pf: 50 94% 51%;
    --sf: 41 74% 46%;
    --af: 24 67% 52%;
    --nf: 240 33% 7%;
    --b2: 0 0% 93%;
    --b3: 0 0% 86%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --bc: 146 0% 19%;
    --ac: 23 34% 13%;
    --nc: 247 7% 81%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 50 94% 58%;
    --pc: 240 33% 14%;
    --s: 41 74% 53%;
    --sc: 240 33% 14%;
    --a: 24 67% 59%;
    --n: 240 33% 14%;
    --b1: 0 0% 100%
}
[data-theme=emerald] {
    color-scheme: light;
    --pf: 141 50% 53%;
    --sf: 219 96% 53%;
    --af: 10 81% 49%;
    --nf: 219 20% 18%;
    --b2: 0 0% 93%;
    --b3: 0 0% 86%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --btn-text-case: uppercase;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 141 50% 60%;
    --pc: 151 28% 19%;
    --s: 219 96% 60%;
    --sc: 210 20% 98%;
    --a: 10 81% 56%;
    --ac: 210 20% 98%;
    --n: 219 20% 25%;
    --nc: 210 20% 98%;
    --b1: 0 0% 100%;
    --bc: 219 20% 25%;
    --animation-btn: 0;
    --animation-input: 0;
    --btn-focus-scale: 1
}
[data-theme=corporate] {
    color-scheme: light;
    --pf: 229 96% 57%;
    --sf: 215 26% 52%;
    --af: 154 49% 53%;
    --nf: 233 27% 6%;
    --b2: 0 0% 93%;
    --b3: 0 0% 86%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --pc: 243 100% 94%;
    --sc: 216 13% 13%;
    --ac: 151 21% 13%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --btn-text-case: uppercase;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 229 96% 64%;
    --s: 215 26% 59%;
    --a: 154 49% 60%;
    --n: 233 27% 13%;
    --nc: 210 38% 95%;
    --b1: 0 0% 100%;
    --bc: 233 27% 13%;
    --rounded-box: 0.25rem;
    --rounded-btn: .125rem;
    --rounded-badge: .125rem;
    --animation-btn: 0;
    --animation-input: 0;
    --btn-focus-scale: 1
}
[data-theme=synthwave] {
    color-scheme: dark;
    --pf: 321 70% 62%;
    --sf: 197 87% 58%;
    --af: 48 89% 50%;
    --nf: 253 59% 13%;
    --b2: 253 58% 8%;
    --b3: 253 58% 1%;
    --pc: 323 23% 15%;
    --sc: 199 28% 14%;
    --ac: 45 42% 13%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 321 70% 69%;
    --s: 197 87% 65%;
    --a: 48 89% 57%;
    --n: 253 59% 20%;
    --nc: 260 60% 98%;
    --b1: 253 58% 15%;
    --bc: 260 60% 98%;
    --in: 199 87% 64%;
    --inc: 257 63% 17%;
    --su: 168 74% 68%;
    --suc: 257 63% 17%;
    --wa: 48 89% 57%;
    --wac: 257 63% 17%;
    --er: 352 74% 57%;
    --erc: 260 60% 98%
}
[data-theme=retro] {
    color-scheme: light;
    --pf: 3 74% 69%;
    --sf: 145 27% 65%;
    --af: 24 67% 52%;
    --nf: 340 7% 10%;
    --inc: 239 85% 93%;
    --suc: 126 38% 89%;
    --wac: 29 59% 11%;
    --erc: 11 100% 91%;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 3 74% 76%;
    --pc: 345 5% 15%;
    --s: 145 27% 72%;
    --sc: 345 5% 15%;
    --a: 24 67% 59%;
    --ac: 345 5% 15%;
    --n: 340 7% 17%;
    --nc: 43 41% 88%;
    --b1: 45 47% 80%;
    --b2: 44 47% 73%;
    --b3: 44 47% 68%;
    --bc: 345 5% 15%;
    --in: 221 83% 53%;
    --su: 142 76% 36%;
    --wa: 32 95% 44%;
    --er: 0 72% 51%;
    --rounded-box: 0.4rem;
    --rounded-btn: 0.4rem;
    --rounded-badge: 0.4rem
}
[data-theme=cyberpunk] {
    color-scheme: light;
    --pf: 345 100% 66%;
    --sf: 195 80% 63%;
    --af: 276 74% 64%;
    --nf: 57 100% 6%;
    --b2: 56 100% 43%;
    --b3: 56 100% 36%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --bc: 53 46% 13%;
    --pc: 348 27% 15%;
    --sc: 196 23% 15%;
    --ac: 277 22% 15%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    font-family: ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,Liberation Mono,Courier New,monospace;
    --p: 345 100% 73%;
    --s: 195 80% 70%;
    --a: 276 74% 71%;
    --n: 57 100% 13%;
    --nc: 56 100% 50%;
    --b1: 56 100% 50%;
    --rounded-box: 0;
    --rounded-btn: 0;
    --rounded-badge: 0;
    --tab-radius: 0
}
[data-theme=valentine] {
    color-scheme: light;
    --pf: 353 74% 60%;
    --sf: 254 86% 70%;
    --af: 181 56% 63%;
    --nf: 336 43% 41%;
    --b2: 318 46% 82%;
    --b3: 318 46% 75%;
    --pc: 356 26% 14%;
    --sc: 256 20% 15%;
    --ac: 181 16% 15%;
    --inc: 239 85% 93%;
    --suc: 126 38% 89%;
    --wac: 29 59% 11%;
    --erc: 11 100% 91%;
    --rounded-box: 1rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 353 74% 67%;
    --s: 254 86% 77%;
    --a: 181 56% 70%;
    --n: 336 43% 48%;
    --nc: 318 46% 89%;
    --b1: 318 46% 89%;
    --bc: 344 38% 28%;
    --in: 221 83% 53%;
    --su: 142 76% 36%;
    --wa: 32 95% 44%;
    --er: 0 72% 51%;
    --rounded-btn: 1.9rem
}
[data-theme=halloween] {
    color-scheme: dark;
    --pf: 32 89% 45%;
    --sf: 271 46% 35%;
    --af: 91 100% 26%;
    --nf: 31 81% 3%;
    --b2: 0 0% 6%;
    --b3: 0 0% 0%;
    --bc: 145 0% 81%;
    --sc: 275 36% 88%;
    --nc: 26 11% 80%;
    --inc: 239 85% 93%;
    --suc: 126 38% 89%;
    --wac: 29 59% 11%;
    --erc: 11 100% 91%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 32 89% 52%;
    --pc: 180 7% 8%;
    --s: 271 46% 42%;
    --a: 91 100% 33%;
    --ac: 0 0% 0%;
    --n: 31 81% 10%;
    --b1: 0 0% 13%;
    --in: 221 83% 53%;
    --su: 142 76% 36%;
    --wa: 32 95% 44%;
    --er: 0 72% 51%
}
[data-theme=garden] {
    color-scheme: light;
    --pf: 331 100% 41%;
    --sf: 334 37% 34%;
    --af: 139 16% 36%;
    --nf: 44 100% 1%;
    --b2: 0 4% 84%;
    --b3: 0 4% 77%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --pc: 346 100% 93%;
    --sc: 340 30% 88%;
    --ac: 136 12% 88%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 331 100% 48%;
    --s: 334 37% 41%;
    --a: 139 16% 43%;
    --n: 44 100% 8%;
    --nc: 0 4% 91%;
    --b1: 0 4% 91%;
    --bc: 0 3% 6%
}
[data-theme=forest] {
    color-scheme: dark;
    --pf: 141 72% 35%;
    --sf: 164 73% 35%;
    --af: 175 73% 35%;
    --nf: 161 37% 8%;
    --b2: 0 12% 1%;
    --b3: 0 0% 0%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --bc: 360 1% 79%;
    --sc: 158 32% 11%;
    --ac: 172 31% 11%;
    --nc: 157 7% 81%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 141 72% 42%;
    --pc: 0 0% 0%;
    --s: 164 73% 42%;
    --a: 175 73% 42%;
    --n: 161 37% 15%;
    --b1: 0 12% 8%;
    --rounded-btn: 1.9rem
}
[data-theme=aqua] {
    color-scheme: dark;
    --pf: 182 93% 42%;
    --sf: 274 31% 50%;
    --af: 47 100% 73%;
    --nf: 205 54% 43%;
    --b2: 219 53% 36%;
    --b3: 219 53% 29%;
    --bc: 228 38% 89%;
    --sc: 276 17% 12%;
    --ac: 46 19% 16%;
    --nc: 212 51% 91%;
    --inc: 239 85% 93%;
    --suc: 126 38% 89%;
    --wac: 29 59% 11%;
    --erc: 11 100% 91%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 182 93% 49%;
    --pc: 181 100% 17%;
    --s: 274 31% 57%;
    --a: 47 100% 80%;
    --n: 205 54% 50%;
    --b1: 219 53% 43%;
    --in: 221 83% 53%;
    --su: 142 76% 36%;
    --wa: 32 95% 44%;
    --er: 0 72% 51%
}
[data-theme=lofi] {
    color-scheme: light;
    --pf: 0 0% 0%;
    --sf: 0 2% 3%;
    --af: 0 0% 8%;
    --nf: 0 0% 0%;
    --btn-text-case: uppercase;
    --border-btn: 1px;
    --tab-border: 1px;
    --p: 0 0% 5%;
    --pc: 0 0% 100%;
    --s: 0 2% 10%;
    --sc: 0 0% 100%;
    --a: 0 0% 15%;
    --ac: 0 0% 100%;
    --n: 0 0% 0%;
    --nc: 0 0% 100%;
    --b1: 0 0% 100%;
    --b2: 0 0% 95%;
    --b3: 0 2% 90%;
    --bc: 0 0% 0%;
    --in: 212 100% 48%;
    --inc: 0 0% 100%;
    --su: 137 72% 46%;
    --suc: 0 0% 0%;
    --wa: 5 100% 66%;
    --wac: 0 0% 100%;
    --er: 325 78% 49%;
    --erc: 0 0% 100%;
    --rounded-box: 0.25rem;
    --rounded-btn: 0.125rem;
    --rounded-badge: 0.125rem;
    --animation-btn: 0;
    --animation-input: 0;
    --btn-focus-scale: 1;
    --tab-radius: 0
}
[data-theme=pastel] {
    color-scheme: light;
    --pf: 284 22% 73%;
    --sf: 352 70% 81%;
    --af: 158 55% 74%;
    --nf: 199 44% 54%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --bc: 146 0% 19%;
    --pc: 284 4% 16%;
    --sc: 352 7% 17%;
    --ac: 158 10% 16%;
    --nc: 200 19% 13%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 284 22% 80%;
    --s: 352 70% 88%;
    --a: 158 55% 81%;
    --n: 199 44% 61%;
    --b1: 0 0% 100%;
    --b2: 210 20% 98%;
    --b3: 216 12% 84%;
    --rounded-btn: 1.9rem
}
[data-theme=fantasy] {
    color-scheme: light;
    --pf: 296 83% 18%;
    --sf: 200 100% 30%;
    --af: 31 94% 44%;
    --nf: 215 28% 10%;
    --b2: 0 0% 93%;
    --b3: 0 0% 86%;
    --in: 198 93% 60%;
    --su: 158 64% 52%;
    --wa: 43 96% 56%;
    --er: 0 91% 71%;
    --pc: 302 27% 85%;
    --sc: 212 51% 90%;
    --ac: 28 57% 12%;
    --nc: 218 6% 82%;
    --inc: 198 100% 12%;
    --suc: 158 100% 10%;
    --wac: 43 100% 11%;
    --erc: 0 100% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 296 83% 25%;
    --s: 200 100% 37%;
    --a: 31 94% 51%;
    --n: 215 28% 17%;
    --b1: 0 0% 100%;
    --bc: 215 28% 17%
}
[data-theme=wireframe] {
    color-scheme: light;
    --pf: 0 0% 65%;
    --sf: 0 0% 65%;
    --af: 0 0% 65%;
    --nf: 0 0% 85%;
    --bc: 146 0% 19%;
    --pc: 145 0% 15%;
    --sc: 145 0% 15%;
    --ac: 145 0% 15%;
    --nc: 145 0% 18%;
    --inc: 263 100% 91%;
    --suc: 105 32% 85%;
    --wac: 58 21% 11%;
    --erc: 17 100% 90%;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    font-family: Chalkboard,comic sans ms,"sanssecondaryerif";
    --p: 0 0% 72%;
    --s: 0 0% 72%;
    --a: 0 0% 72%;
    --n: 0 0% 92%;
    --b1: 0 0% 100%;
    --b2: 0 0% 93%;
    --b3: 0 0% 87%;
    --in: 240 100% 50%;
    --su: 120 100% 25%;
    --wa: 60 30% 50%;
    --er: 0 100% 50%;
    --rounded-box: 0.2rem;
    --rounded-btn: 0.2rem;
    --rounded-badge: 0.2rem;
    --tab-radius: 0.2rem
}
[data-theme=black] {
    color-scheme: dark;
    --pf: 0 2% 13%;
    --sf: 0 2% 13%;
    --af: 0 2% 13%;
    --bc: 145 0% 78%;
    --pc: 0 1% 82%;
    --sc: 0 1% 82%;
    --ac: 0 1% 82%;
    --nc: 0 0% 81%;
    --inc: 263 100% 91%;
    --suc: 105 32% 85%;
    --wac: 58 45% 13%;
    --erc: 17 100% 90%;
    --border-btn: 1px;
    --tab-border: 1px;
    --p: 0 2% 20%;
    --s: 0 2% 20%;
    --a: 0 2% 20%;
    --b1: 0 0% 0%;
    --b2: 0 0% 5%;
    --b3: 0 2% 10%;
    --n: 0 1% 15%;
    --nf: 0 2% 20%;
    --in: 240 100% 50%;
    --su: 120 100% 25%;
    --wa: 60 100% 50%;
    --er: 0 100% 50%;
    --rounded-box: 0;
    --rounded-btn: 0;
    --rounded-badge: 0;
    --animation-btn: 0;
    --animation-input: 0;
    --btn-text-case: lowercase;
    --btn-focus-scale: 1;
    --tab-radius: 0
}
[data-theme=luxury] {
    color-scheme: dark;
    --pf: 0 0% 93%;
    --sf: 218 54% 11%;
    --af: 319 22% 19%;
    --nf: 28 100% 3%;
    --pc: 146 0% 19%;
    --sc: 227 12% 82%;
    --ac: 322 9% 84%;
    --inc: 205 27% 15%;
    --suc: 88 35% 12%;
    --wac: 52 28% 14%;
    --erc: 3 31% 15%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 0 0% 100%;
    --s: 218 54% 18%;
    --a: 319 22% 26%;
    --n: 28 100% 10%;
    --nc: 44 100% 82%;
    --b1: 240 10% 4%;
    --b2: 270 4% 9%;
    --b3: 270 2% 18%;
    --bc: 37 67% 58%;
    --in: 202 100% 70%;
    --su: 89 62% 52%;
    --wa: 54 69% 64%;
    --er: 0 100% 72%
}
[data-theme=dracula] {
    color-scheme: dark;
    --pf: 326 100% 67%;
    --sf: 265 89% 71%;
    --af: 31 100% 64%;
    --nf: 230 15% 23%;
    --b2: 231 15% 11%;
    --b3: 231 15% 4%;
    --pc: 328 26% 15%;
    --sc: 266 19% 16%;
    --ac: 30 30% 15%;
    --nc: 232 7% 85%;
    --inc: 191 20% 16%;
    --suc: 128 30% 14%;
    --wac: 64 20% 15%;
    --erc: 5 39% 14%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 326 100% 74%;
    --s: 265 89% 78%;
    --a: 31 100% 71%;
    --n: 230 15% 30%;
    --b1: 231 15% 18%;
    --bc: 60 30% 96%;
    --in: 191 97% 77%;
    --su: 135 94% 65%;
    --wa: 65 92% 76%;
    --er: 0 100% 67%
}
[data-theme=cmyk] {
    color-scheme: light;
    --pf: 203 83% 53%;
    --sf: 335 78% 53%;
    --af: 56 100% 53%;
    --nf: 0 0% 3%;
    --b2: 0 0% 93%;
    --b3: 0 0% 86%;
    --bc: 146 0% 19%;
    --pc: 207 32% 14%;
    --sc: 344 100% 93%;
    --ac: 54 41% 14%;
    --nc: 145 0% 80%;
    --inc: 194 26% 12%;
    --suc: 295 30% 87%;
    --wac: 24 46% 13%;
    --erc: 12 100% 91%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 203 83% 60%;
    --s: 335 78% 60%;
    --a: 56 100% 60%;
    --n: 0 0% 10%;
    --b1: 0 0% 100%;
    --in: 192 48% 52%;
    --su: 291 48% 38%;
    --wa: 25 85% 57%;
    --er: 4 81% 56%
}
[data-theme=autumn] {
    color-scheme: light;
    --pf: 344 96% 21%;
    --sf: 0 63% 51%;
    --af: 27 56% 56%;
    --nf: 22 17% 37%;
    --b2: 0 0% 88%;
    --b3: 0 0% 81%;
    --bc: 145 0% 18%;
    --pc: 2 46% 87%;
    --sc: 6 87% 92%;
    --ac: 27 25% 13%;
    --nc: 21 15% 88%;
    --inc: 188 26% 12%;
    --suc: 161 25% 89%;
    --wac: 28 55% 12%;
    --erc: 8 100% 91%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 344 96% 28%;
    --s: 0 63% 58%;
    --a: 27 56% 63%;
    --n: 22 17% 44%;
    --b1: 0 0% 95%;
    --in: 187 48% 50%;
    --su: 165 34% 43%;
    --wa: 30 84% 50%;
    --er: 354 79% 49%
}
[data-theme=business] {
    color-scheme: dark;
    --pf: 210 64% 24%;
    --sf: 200 13% 48%;
    --af: 13 80% 53%;
    --nf: 213 14% 9%;
    --b2: 0 0% 6%;
    --b3: 0 0% 0%;
    --bc: 145 0% 80%;
    --pc: 219 26% 86%;
    --sc: 200 7% 12%;
    --ac: 14 40% 13%;
    --nc: 214 3% 81%;
    --inc: 210 64% 91%;
    --suc: 141 16% 12%;
    --wac: 37 30% 13%;
    --erc: 11 59% 89%;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 210 64% 31%;
    --s: 200 13% 55%;
    --a: 13 80% 60%;
    --n: 213 14% 16%;
    --b1: 0 0% 13%;
    --in: 199 100% 42%;
    --su: 144 31% 56%;
    --wa: 39 64% 60%;
    --er: 6 56% 43%;
    --rounded-box: 0.25rem;
    --rounded-btn: .125rem;
    --rounded-badge: .125rem
}
[data-theme=acid] {
    color-scheme: light;
    --pf: 303 100% 43%;
    --sf: 27 100% 43%;
    --af: 72 98% 43%;
    --nf: 238 43% 10%;
    --b2: 0 0% 91%;
    --b3: 0 0% 84%;
    --bc: 145 0% 19%;
    --pc: 302 100% 93%;
    --sc: 25 62% 12%;
    --ac: 73 44% 13%;
    --nc: 248 11% 82%;
    --inc: 217 36% 14%;
    --suc: 145 23% 13%;
    --wac: 50 42% 13%;
    --erc: 15 100% 90%;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 303 100% 50%;
    --s: 27 100% 50%;
    --a: 72 98% 50%;
    --n: 238 43% 17%;
    --b1: 0 0% 98%;
    --in: 210 92% 58%;
    --su: 149 50% 58%;
    --wa: 53 93% 57%;
    --er: 1 100% 45%;
    --rounded-box: 1.25rem;
    --rounded-btn: 1rem;
    --rounded-badge: 1rem
}
[data-theme=lemonade] {
    color-scheme: light;
    --pf: 89 96% 24%;
    --sf: 60 81% 48%;
    --af: 63 80% 81%;
    --nf: 238 43% 10%;
    --b2: 0 0% 93%;
    --b3: 0 0% 86%;
    --bc: 146 0% 19%;
    --pc: 89 39% 87%;
    --sc: 58 39% 13%;
    --ac: 62 8% 17%;
    --nc: 248 11% 82%;
    --inc: 192 5% 17%;
    --suc: 74 15% 16%;
    --wac: 49 21% 15%;
    --erc: 2 11% 16%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 89 96% 31%;
    --s: 60 81% 55%;
    --a: 63 80% 88%;
    --n: 238 43% 17%;
    --b1: 0 0% 100%;
    --in: 192 39% 85%;
    --su: 74 76% 79%;
    --wa: 50 87% 75%;
    --er: 1 70% 83%
}
[data-theme=night] {
    color-scheme: dark;
    --pf: 198 93% 53%;
    --sf: 234 89% 67%;
    --af: 329 86% 63%;
    --b2: 222 47% 4%;
    --b3: 0 0% 0%;
    --bc: 229 7% 80%;
    --pc: 202 34% 14%;
    --sc: 239 22% 15%;
    --ac: 332 26% 15%;
    --nc: 221 7% 82%;
    --suc: 169 31% 13%;
    --wac: 39 36% 14%;
    --erc: 354 28% 15%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 198 93% 60%;
    --s: 234 89% 74%;
    --a: 329 86% 70%;
    --n: 217 33% 17%;
    --nf: 217 30% 22%;
    --b1: 222 47% 11%;
    --in: 198 90% 48%;
    --inc: 0 0% 0%;
    --su: 172 66% 50%;
    --wa: 41 88% 64%;
    --er: 351 95% 71%
}
[data-theme=coffee] {
    color-scheme: dark;
    --pf: 30 67% 51%;
    --sf: 182 25% 13%;
    --af: 194 74% 18%;
    --nf: 0 0% 0%;
    --b2: 306 19% 4%;
    --b3: 0 0% 0%;
    --pc: 28 35% 13%;
    --sc: 182 6% 83%;
    --ac: 199 20% 85%;
    --nc: 300 1% 79%;
    --inc: 170 12% 14%;
    --suc: 92 11% 13%;
    --wac: 41 33% 14%;
    --erc: 11 25% 15%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 30 67% 58%;
    --s: 182 25% 20%;
    --a: 194 74% 25%;
    --n: 300 20% 6%;
    --b1: 306 19% 11%;
    --bc: 37 8% 42%;
    --in: 171 37% 67%;
    --su: 93 25% 62%;
    --wa: 43 100% 69%;
    --er: 10 95% 75%
}
[data-theme=winter] {
    color-scheme: light;
    --pf: 212 100% 44%;
    --sf: 247 47% 36%;
    --af: 310 49% 45%;
    --nf: 217 92% 3%;
    --pc: 231 100% 93%;
    --sc: 256 40% 88%;
    --ac: 316 56% 91%;
    --nc: 229 10% 80%;
    --inc: 192 18% 16%;
    --suc: 181 16% 14%;
    --wac: 32 9% 16%;
    --erc: 2 19% 15%;
    --rounded-box: 1rem;
    --rounded-btn: 0.5rem;
    --rounded-badge: 1.9rem;
    --animation-btn: 0.25s;
    --animation-input: .2s;
    --btn-text-case: uppercase;
    --btn-focus-scale: 0.95;
    --border-btn: 1px;
    --tab-border: 1px;
    --tab-radius: 0.5rem;
    --p: 212 100% 51%;
    --s: 247 47% 43%;
    --a: 310 49% 52%;
    --n: 217 92% 10%;
    --b1: 0 0% 100%;
    --b2: 217 100% 97%;
    --b3: 219 44% 92%;
    --bc: 214 30% 32%;
    --in: 192 93% 78%;
    --su: 182 47% 66%;
    --wa: 32 62% 84%;
    --er: 0 63% 72%
}
/*
! tailwindcss v3.3.3 | MIT License | https://tailwindcss.com
*/
/*
1. Prevent padding and border from affecting element width. (https://github.com/mozdevs/cssremedy/issues/4)
2. Allow adding a border to an element by just adding a border-width. (https://github.com/tailwindcss/tailwindcss/pull/116)
*/
*,
::before,
::after {
  box-sizing: border-box; /* 1 */
  border-width: 0; /* 2 */
  border-style: solid; /* 2 */
  border-color: currentColor; /* 2 */
}
::before,
::after {
  --tw-content: '';
}
/*
1. Use a consistent sensible line-height in all browsers.
2. Prevent adjustments of font size after orientation changes in iOS.
3. Use a more readable tab size.
4. Use the user's configured `sans` font-family by default.
5. Use the user's configured `sans` font-feature-settings by default.
6. Use the user's configured `sans` font-variation-settings by default.
*/
html {
  line-height: 1.5; /* 1 */
  -webkit-text-size-adjust: 100%; /* 2 */ /* 3 */
  tab-size: 4; /* 3 */
  font-family: ui-sans-serif, system-ui, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji"; /* 4 */
  font-feature-settings: normal; /* 5 */
  font-variation-settings: normal; /* 6 */
}
/*
1. Remove the margin in all browsers.
2. Inherit line-height from `html` so users can set them as a class directly on the `html` element.
*/
body {
  margin: 0; /* 1 */
  line-height: inherit; /* 2 */
}
/*
1. Add the correct height in Firefox.
2. Correct the inheritance of border color in Firefox. (https://bugzilla.mozilla.org/show_bug.cgi?id=190655)
3. Ensure horizontal rules are visible by default.
*/
hr {
  height: 0; /* 1 */
  color: inherit; /* 2 */
  border-top-width: 1px; /* 3 */
}
/*
Add the correct text decoration in Chrome, Edge, and Safari.
*/
abbr:where([title]) {
  text-decoration: underline dotted;
}
/*
Remove the default font size and weight for headings.
*/
h1,
h2,
h3,
h4,
h5,
h6 {
  font-size: inherit;
  font-weight: inherit;
}
/*
Reset links to optimize for opt-in styling instead of opt-out.
*/
a {
  color: inherit;
  text-decoration: inherit;
}
/*
Add the correct font weight in Edge and Safari.
*/
b,
strong {
  font-weight: bolder;
}
/*
1. Use the user's configured `mono` font family by default.
2. Correct the odd `em` font sizing in all browsers.
*/
code,
kbd,
samp,
pre {
  font-family: ui-monospace, SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace; /* 1 */
  font-size: 1em; /* 2 */
}
/*
Add the correct font size in all browsers.
*/
small {
  font-size: 80%;
}
/*
Prevent `sub` and `sup` elements from affecting the line height in all browsers.
*/
sub,
sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline;
}
sub {
  bottom: -0.25em;
}
sup {
  top: -0.5em;
}
/*
1. Remove text indentation from table contents in Chrome and Safari. (https://bugs.chromium.org/p/chromium/issues/detail?id=999088, https://bugs.webkit.org/show_bug.cgi?id=201297)
2. Correct table border color inheritance in all Chrome and Safari. (https://bugs.chromium.org/p/chromium/issues/detail?id=935729, https://bugs.webkit.org/show_bug.cgi?id=195016)
3. Remove gaps between table borders by default.
*/
table {
  text-indent: 0; /* 1 */
  border-color: inherit; /* 2 */
  border-collapse: collapse; /* 3 */
}
/*
1. Change the font styles in all browsers.
2. Remove the margin in Firefox and Safari.
3. Remove default padding in all browsers.
*/
button,
input,
optgroup,
select,
textarea {
  font-family: inherit; /* 1 */
  font-feature-settings: inherit; /* 1 */
  font-variation-settings: inherit; /* 1 */
  font-size: 100%; /* 1 */
  font-weight: inherit; /* 1 */
  line-height: inherit; /* 1 */
  color: inherit; /* 1 */
  margin: 0; /* 2 */
  padding: 0; /* 3 */
}
/*
Remove the inheritance of text transform in Edge and Firefox.
*/
button,
select {
  text-transform: none;
}
/*
1. Correct the inability to style clickable types in iOS and Safari.
2. Remove default button styles.
*/
button,
[type='button'],
[type='reset'],
[type='submit'] {
  -webkit-appearance: button; /* 1 */
  background-color: transparent; /* 2 */
  background-image: none; /* 2 */
}
/*
Use the modern Firefox focus style for all focusable elements.
*/
:-moz-focusring {
  outline: auto;
}
/*
Remove the additional `:invalid` styles in Firefox. (https://github.com/mozilla/gecko-dev/blob/2f9eacd9d3d995c937b4251a5557d95d494c9be1/layout/style/res/forms.css#L728-L737)
*/
:-moz-ui-invalid {
  box-shadow: none;
}
/*
Add the correct vertical alignment in Chrome and Firefox.
*/
progress {
  vertical-align: baseline;
}
/*
Correct the cursor style of increment and decrement buttons in Safari.
*/
::-webkit-inner-spin-button,
::-webkit-outer-spin-button {
  height: auto;
}
/*
1. Correct the odd appearance in Chrome and Safari.
2. Correct the outline style in Safari.
*/
[type='search'] {
  -webkit-appearance: textfield; /* 1 */
  outline-offset: -2px; /* 2 */
}
/*
Remove the inner padding in Chrome and Safari on macOS.
*/
::-webkit-search-decoration {
  -webkit-appearance: none;
}
/*
1. Correct the inability to style clickable types in iOS and Safari.
2. Change font properties to `inherit` in Safari.
*/
::-webkit-file-upload-button {
  -webkit-appearance: button; /* 1 */
  font: inherit; /* 2 */
}
/*
Add the correct display in Chrome and Safari.
*/
summary {
  display: list-item;
}
/*
Removes the default spacing and border for appropriate elements.
*/
blockquote,
dl,
dd,
h1,
h2,
h3,
h4,
h5,
h6,
hr,
figure,
p,
pre {
  margin: 0;
}
fieldset {
  margin: 0;
  padding: 0;
}
legend {
  padding: 0;
}
ol,
ul,
menu {
  list-style: none;
  margin: 0;
  padding: 0;
}
/*
Reset default styling for dialogs.
*/
dialog {
  padding: 0;
}
/*
Prevent resizing textareas horizontally by default.
*/
textarea {
  resize: vertical;
}
/*
1. Reset the default placeholder opacity in Firefox. (https://github.com/tailwindlabs/tailwindcss/issues/3300)
2. Set the default placeholder color to the user's configured gray 400 color.
*/
input::placeholder,
textarea::placeholder {
  opacity: 1; /* 1 */
  color: #9ca3af; /* 2 */
}
/*
Set the default cursor for buttons.
*/
button,
[role="button"] {
  cursor: pointer;
}
/*
Make sure disabled buttons don't get the pointer cursor.
*/
:disabled {
  cursor: default;
}
/*
1. Make replaced elements `display: block` by default. (https://github.com/mozdevs/cssremedy/issues/14)
2. Add `vertical-align: middle` to align replaced elements more sensibly by default. (https://github.com/jensimmons/cssremedy/issues/14#issuecomment-634934210)
   This can trigger a poorly considered lint error in some tools but is included by design.
*/
img,
svg,
video,
canvas,
audio,
iframe,
embed,
object {
  display: block; /* 1 */
  vertical-align: middle; /* 2 */
}
/*
Constrain images and videos to the parent width and preserve their intrinsic aspect ratio. (https://github.com/mozdevs/cssremedy/issues/14)
*/
img,
video {
  max-width: 100%;
  height: auto;
}
/* Make elements with the HTML hidden attribute stay hidden by default */
[hidden] {
  display: none;
}
:root,
[data-theme] {
    background-color: hsl(var(--b1) / var(--tw-bg-opacity, 1));
    color: hsl(var(--bc) / var(--tw-text-opacity, 1))}
html {
    -webkit-tap-highlight-color: transparent}
*, ::before, ::after {
    --tw-gradient-from-position:  ;
    --tw-gradient-via-position:  ;
    --tw-gradient-to-position:  ;
    --tw-ring-inset:  ;
    --tw-ring-offset-width: 0px;
    --tw-ring-offset-color: #fff;
    --tw-ring-color: rgb(59 130 246 / 0.5);
    --tw-ring-offset-shadow: 0 0 #0000;
    --tw-ring-shadow: 0 0 #0000;
    --tw-shadow: 0 0 #0000;
    --tw-shadow-colored: 0 0 #0000}
::backdrop {
    --tw-gradient-from-position:  ;
    --tw-gradient-via-position:  ;
    --tw-gradient-to-position:  ;
    --tw-ring-inset:  ;
    --tw-ring-offset-width: 0px;
    --tw-ring-offset-color: #fff;
    --tw-ring-color: rgb(59 130 246 / 0.5);
    --tw-ring-offset-shadow: 0 0 #0000;
    --tw-ring-shadow: 0 0 #0000;
    --tw-shadow: 0 0 #0000;
    --tw-shadow-colored: 0 0 #0000}
.alert {
    display: grid;
    width: 100%;
    grid-auto-flow: row;
    align-content: flex-start;
    align-items: center;
    justify-items: center;
    gap: 1rem;
    text-align: center;
    border-width: 1px;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    padding: 1rem;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    border-radius: var(--rounded-box, 1rem);
    --alert-bg: hsl(var(--b2));
    --alert-bg-mix: hsl(var(--b1));
    background-color: var(--alert-bg)}
@media (min-width: 640px) {
    .alert {
        grid-auto-flow: column;
        grid-template-columns: auto minmax(auto,1fr);
        justify-items: start;
        text-align: left}}
.artboard {
    width: 100%}
.avatar {
    position: relative;
    display: inline-flex}
.avatar > div {
    display: block;
    aspect-ratio: 1 / 1;
    overflow: hidden}
.avatar img {
    height: 100%;
    width: 100%;
    object-fit: cover}
.avatar.placeholder > div {
    display: flex;
    align-items: center;
    justify-content: center}
.badge {
    display: inline-flex;
    align-items: center;
    justify-content: center;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms;
    height: 1.25rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    width: fit-content;
    padding-left: 0.563rem;
    padding-right: 0.563rem;
    border-width: 1px;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    border-radius: var(--rounded-badge, 1.9rem)}
.btm-nav {
    position: fixed;
    bottom: 0px;
    left: 0px;
    right: 0px;
    display: flex;
    width: 100%;
    flex-direction: row;
    align-items: center;
    justify-content: space-around;
    padding-bottom: env(safe-area-inset-bottom);
    height: 4rem;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    color: currentColor}
.btm-nav > * {
    position: relative;
    display: flex;
    height: 100%;
    flex-basis: 100%;
    cursor: pointer;
    flex-direction: column;
    align-items: center;
    justify-content: center;
    gap: 0.25rem;
    border-color: currentColor}
.breadcrumbs {
    max-width: 100%;
    overflow-x: auto;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem}
.breadcrumbs > ul,
  .breadcrumbs > ol {
    display: flex;
    align-items: center;
    white-space: nowrap;
    min-height: min-content}
.breadcrumbs > ul > li, .breadcrumbs > ol > li {
    display: flex;
    align-items: center}
.breadcrumbs > ul > li > a, .breadcrumbs > ol > li > a {
    display: flex;
    cursor: pointer;
    align-items: center}
@media (hover:hover) {
    .breadcrumbs > ul > li > a:hover, .breadcrumbs > ol > li > a:hover {
        text-decoration-line: underline}
    .link-hover:hover {
        text-decoration-line: underline}
    .checkbox-primary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--p) / var(--tw-border-opacity))}
    .checkbox-secondary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--s) / var(--tw-border-opacity))}
    .checkbox-accent:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--a) / var(--tw-border-opacity))}
    .checkbox-success:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity))}
    .checkbox-warning:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity))}
    .checkbox-info:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity))}
    .checkbox-error:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity))}
    .label a:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--bc) / var(--tw-text-opacity))}
    .link-primary:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--pf) / var(--tw-text-opacity))}
    .link-secondary:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--sf) / var(--tw-text-opacity))}
    .link-accent:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--af) / var(--tw-text-opacity))}
    .link-neutral:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--nf) / var(--tw-text-opacity))}
    .link-success:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--su) / var(--tw-text-opacity))}
    .link-info:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--in) / var(--tw-text-opacity))}
    .link-warning:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--wa) / var(--tw-text-opacity))}
    .link-error:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--er) / var(--tw-text-opacity))}
    .menu li > *:not(ul):not(.menu-title):not(details):active,
.menu li > *:not(ul):not(.menu-title):not(details).active,
.menu li > details > summary:active {
        --tw-bg-opacity: 1;
        background-color: hsl(var(--n) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--nc) / var(--tw-text-opacity))}
    .radio-primary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--p) / var(--tw-border-opacity))}
    .radio-secondary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--s) / var(--tw-border-opacity))}
    .radio-accent:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--a) / var(--tw-border-opacity))}
    .radio-success:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity))}
    .radio-warning:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity))}
    .radio-info:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity))}
    .radio-error:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity))}
    .tab:hover {
        --tw-text-opacity: 1}
    .tabs-boxed .tab-active:not(.tab-disabled):not([disabled]):hover {
        --tw-text-opacity: 1;
        color: hsl(var(--pc) / var(--tw-text-opacity))}
    .table tr.hover:hover,
  .table tr.hover:nth-child(even):hover {
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b2) / var(--tw-bg-opacity))}
    .table-zebra tr.hover:hover,
  .table-zebra tr.hover:nth-child(even):hover {
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b3) / var(--tw-bg-opacity))}}
.btn {
    display: inline-flex;
    flex-shrink: 0;
    cursor: pointer;
    user-select: none;
    flex-wrap: wrap;
    align-items: center;
    justify-content: center;
    border-color: transparent;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    text-align: center;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms;
    border-radius: var(--rounded-btn, 0.5rem);
    height: 3rem;
    padding-left: 1rem;
    padding-right: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 1em;
    min-height: 3rem;
    gap: 0.5rem;
    font-weight: 600;
    text-decoration-line: none;
    border-width: var(--border-btn, 1px);
    animation: button-pop var(--animation-btn, 0.25s) ease-out;
    text-transform: var(--btn-text-case, uppercase);
    --tw-border-opacity: 1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    outline-color: hsl(var(--bc) / 1)}
.btn-disabled,
  .btn[disabled],
  .btn:disabled {
    pointer-events: none}
.btn-square {
    height: 3rem;
    width: 3rem;
    padding: 0px}
.btn-circle {
    height: 3rem;
    width: 3rem;
    border-radius: 9999px;
    padding: 0px}
.btn-group {
    display: inline-flex}
.btn-group > input[type="radio"].btn {
    appearance: none}
.btn-group > input[type="radio"].btn:before {
    content: attr(data-title)}
.btn:is(input[type="checkbox"]),
.btn:is(input[type="radio"]) {
    width: auto;
    appearance: none}
.btn:is(input[type="checkbox"]):after,
.btn:is(input[type="radio"]):after {
    --tw-content: attr(aria-label);
    content: var(--tw-content)}
.card {
    position: relative;
    display: flex;
    flex-direction: column;
    border-radius: var(--rounded-box, 1rem)}
.card:focus {
    outline: 2px solid transparent;
    outline-offset: 2px}
.card-body {
    display: flex;
    flex: 1 1 auto;
    flex-direction: column;
    padding: var(--padding-card, 2rem);
    gap: 0.5rem}
.card-body :where(p) {
    flex-grow: 1}
.card-actions {
    display: flex;
    flex-wrap: wrap;
    align-items: flex-start;
    gap: 0.5rem}
.card figure {
    display: flex;
    align-items: center;
    justify-content: center}
.card.image-full {
    display: grid}
.card.image-full:before {
    position: relative;
    content: "";
    z-index: 10;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    opacity: 0.75;
    border-radius: var(--rounded-box, 1rem)}
.card.image-full:before,
    .card.image-full > * {
    grid-column-start: 1;
    grid-row-start: 1}
.card.image-full > figure img {
    height: 100%;
    object-fit: cover}
.card.image-full > .card-body {
    position: relative;
    z-index: 20;
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity))}
.carousel {
    display: inline-flex;
    overflow-x: scroll;
    scroll-snap-type: x mandatory;
    scroll-behavior: smooth;
    -ms-overflow-style: none;
    scrollbar-width: none}
.carousel-vertical {
    flex-direction: column;
    overflow-y: scroll;
    scroll-snap-type: y mandatory}
.carousel-item {
    box-sizing: content-box;
    display: flex;
    flex: none;
    scroll-snap-align: start}
.carousel-center .carousel-item {
    scroll-snap-align: center}
.carousel-end .carousel-item {
    scroll-snap-align: end}
.chat {
    display: grid;
    grid-template-columns: repeat(2, minmax(0, 1fr));
    column-gap: 0.75rem;
    padding-top: 0.25rem;
    padding-bottom: 0.25rem}
.chat-image {
    grid-row: span 2 / span 2;
    align-self: flex-end}
.chat-header {
    grid-row-start: 1;
    font-size: 0.875rem;
    line-height: 1.25rem}
.chat-footer {
    grid-row-start: 3;
    font-size: 0.875rem;
    line-height: 1.25rem}
.chat-bubble {
    position: relative;
    display: block;
    width: fit-content;
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    max-width: 90%;
    border-radius: var(--rounded-box, 1rem);
    min-height: 2.75rem;
    min-width: 2.75rem;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity))}
.chat-bubble:before {
    position: absolute;
    bottom: 0px;
    height: 0.75rem;
    width: 0.75rem;
    background-color: inherit;
    content: "";
    -webkit-mask-size: contain;
            mask-size: contain;
    -webkit-mask-repeat: no-repeat;
            mask-repeat: no-repeat;
    -webkit-mask-position: center;
            mask-position: center}
.chat-start {
    place-items: start;
    grid-template-columns: auto 1fr}
.chat-start .chat-header {
    grid-column-start: 2}
.chat-start .chat-footer {
    grid-column-start: 2}
.chat-start .chat-image {
    grid-column-start: 1}
.chat-start .chat-bubble {
    grid-column-start: 2;
    border-bottom-left-radius: 0px}
.chat-start .chat-bubble:before {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 3 3 L 3 0 C 3 1 1 3 0 3'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 3 3 L 3 0 C 3 1 1 3 0 3'/%3e%3c/svg%3e");
    left: -0.75rem}
[dir="rtl"] .chat-start .chat-bubble:before {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 1 3 L 3 3 C 2 3 0 1 0 0'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 1 3 L 3 3 C 2 3 0 1 0 0'/%3e%3c/svg%3e")}
.chat-end {
    place-items: end;
    grid-template-columns: 1fr auto}
.chat-end .chat-header {
    grid-column-start: 1}
.chat-end .chat-footer {
    grid-column-start: 1}
.chat-end .chat-image {
    grid-column-start: 2}
.chat-end .chat-bubble {
    grid-column-start: 1;
    border-bottom-right-radius: 0px}
.chat-end .chat-bubble:before {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 1 3 L 3 3 C 2 3 0 1 0 0'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 1 3 L 3 3 C 2 3 0 1 0 0'/%3e%3c/svg%3e");
    left: 100%}
[dir="rtl"] .chat-end .chat-bubble:before {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 3 3 L 3 0 C 3 1 1 3 0 3'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='3' height='3' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m 0 3 L 3 3 L 3 0 C 3 1 1 3 0 3'/%3e%3c/svg%3e")}
.checkbox {
    flex-shrink: 0;
    --chkbg: var(--bc);
    --chkfg: var(--b1);
    height: 1.5rem;
    width: 1.5rem;
    cursor: pointer;
    appearance: none;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0.2;
    border-radius: var(--rounded-btn, 0.5rem)}
.collapse:not(td):not(tr):not(colgroup) {
    visibility: visible}
.collapse {
    position: relative;
    display: grid;
    overflow: hidden;
    grid-template-rows: auto 0fr;
    transition: grid-template-rows 0.2s;
    width: 100%;
    border-radius: var(--rounded-box, 1rem)}
.collapse-title,
.collapse > input[type="checkbox"],
.collapse > input[type="radio"],
.collapse-content {
    grid-column-start: 1;
    grid-row-start: 1}
.collapse > input[type="checkbox"],
.collapse > input[type="radio"] {
    appearance: none;
    opacity: 0}
.collapse-content {
    visibility: hidden;
    grid-column-start: 1;
    grid-row-start: 2;
    min-height: 0px;
    transition: visibility 0.2s;
    transition: padding 0.2s ease-out, background-color 0.2s ease-out;
    padding-left: 1rem;
    padding-right: 1rem;
    cursor: unset}
.collapse[open],
.collapse-open,
.collapse:focus:not(.collapse-close) {
    grid-template-rows: auto 1fr}
.collapse:not(.collapse-close):has(> input[type="checkbox"]:checked),
.collapse:not(.collapse-close):has(> input[type="radio"]:checked) {
    grid-template-rows: auto 1fr}
.collapse[open] .collapse-content,
.collapse-open .collapse-content,
.collapse:focus:not(.collapse-close) .collapse-content,
.collapse:not(.collapse-close) input[type="checkbox"]:checked ~ .collapse-content,
.collapse:not(.collapse-close) input[type="radio"]:checked ~ .collapse-content {
    visibility: visible;
    min-height: fit-content}
:root .countdown {
    line-height: 1em}
.countdown {
    display: inline-flex}
.countdown > * {
    height: 1em;
    display: inline-block;
    overflow-y: hidden}
.countdown > *:before {
    position: relative;
    content: "00\A 01\A 02\A 03\A 04\A 05\A 06\A 07\A 08\A 09\A 10\A 11\A 12\A 13\A 14\A 15\A 16\A 17\A 18\A 19\A 20\A 21\A 22\A 23\A 24\A 25\A 26\A 27\A 28\A 29\A 30\A 31\A 32\A 33\A 34\A 35\A 36\A 37\A 38\A 39\A 40\A 41\A 42\A 43\A 44\A 45\A 46\A 47\A 48\A 49\A 50\A 51\A 52\A 53\A 54\A 55\A 56\A 57\A 58\A 59\A 60\A 61\A 62\A 63\A 64\A 65\A 66\A 67\A 68\A 69\A 70\A 71\A 72\A 73\A 74\A 75\A 76\A 77\A 78\A 79\A 80\A 81\A 82\A 83\A 84\A 85\A 86\A 87\A 88\A 89\A 90\A 91\A 92\A 93\A 94\A 95\A 96\A 97\A 98\A 99\A";
    white-space: pre;
    top: calc(var(--value) * -1em);
    text-align: center;
    transition: all 1s cubic-bezier(1, 0, 0, 1)}
.divider {
    display: flex;
    flex-direction: row;
    align-items: center;
    align-self: stretch;
    margin-top: 1rem;
    margin-bottom: 1rem;
    height: 1rem;
    white-space: nowrap}
.divider:before,
  .divider:after {
    content: "";
    flex-grow: 1;
    height: 0.125rem;
    width: 100%}
.drawer {
    position: relative;
    display: grid;
    grid-auto-columns: max-content auto;
    width: 100%}
.drawer-content {
    grid-column-start: 2;
    grid-row-start: 1}
.drawer-side {
    pointer-events: none;
    position: fixed;
    top: 0px;
    left: 0px;
    grid-column-start: 1;
    grid-row-start: 1;
    display: grid;
    width: 100%;
    grid-template-columns: repeat(1, minmax(0, 1fr));
    grid-template-rows: repeat(1, minmax(0, 1fr));
    align-items: flex-start;
    justify-items: start;
    overflow-y: auto;
    overscroll-behavior: contain;
    height: 100vh;
    height: 100dvh;
    scrollbar-width: none}
.drawer-side::-webkit-scrollbar {
    display: none}
.drawer-side > .drawer-overlay {
    position: sticky;
    top: 0px;
    place-self: stretch;
    cursor: pointer;
    background-color: transparent;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms}
.drawer-side > * {
    grid-column-start: 1;
    grid-row-start: 1}
.drawer-side > *:not(.drawer-overlay) {
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 300ms;
    transform: translateX(-100%)}
[dir="rtl"] .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(100%)}
.drawer-toggle {
    position: fixed;
    height: 0px;
    width: 0px;
    appearance: none;
    opacity: 0}
.drawer-toggle:checked ~ .drawer-side {
    pointer-events: auto;
    visibility: visible}
.drawer-toggle:checked ~ .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(0%)}
.drawer-end {
    grid-auto-columns: auto max-content}
.drawer-end .drawer-toggle ~ .drawer-content {
    grid-column-start: 1}
.drawer-end .drawer-toggle ~ .drawer-side {
    justify-items: end}
.drawer-end .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(100%)}
[dir="rtl"] .drawer-end .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(-100%)}
.drawer-end .drawer-toggle:checked ~ .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(0%)}
.dropdown {
    position: relative;
    display: inline-block}
.dropdown > *:focus {
    outline: 2px solid transparent;
    outline-offset: 2px}
.dropdown .dropdown-content {
    position: absolute}
.dropdown:is(:not(details)) .dropdown-content {
    visibility: hidden;
    opacity: 0;
    transform-origin: top;
    --tw-scale-x: .95;
    --tw-scale-y: .95;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms}
.dropdown-end .dropdown-content {
    right: 0px}
.dropdown-left .dropdown-content {
    top: 0px;
    right: 100%;
    bottom: auto;
    transform-origin: right}
.dropdown-right .dropdown-content {
    left: 100%;
    top: 0px;
    bottom: auto;
    transform-origin: left}
.dropdown-bottom .dropdown-content {
    bottom: auto;
    top: 100%;
    transform-origin: top}
.dropdown-top .dropdown-content {
    bottom: 100%;
    top: auto;
    transform-origin: bottom}
.dropdown-end.dropdown-right .dropdown-content {
    bottom: 0px;
    top: auto}
.dropdown-end.dropdown-left .dropdown-content {
    bottom: 0px;
    top: auto}
.dropdown.dropdown-open .dropdown-content,
.dropdown:not(.dropdown-hover):focus .dropdown-content,
.dropdown:focus-within .dropdown-content {
    visibility: visible;
    opacity: 1}
@media (hover: hover) {
    .dropdown.dropdown-hover:hover .dropdown-content {
        visibility: visible;
        opacity: 1}
    .btm-nav > *.disabled:hover,
      .btm-nav > *[disabled]:hover {
        pointer-events: none;
        --tw-border-opacity: 0;
        background-color: hsl(var(--n) / var(--tw-bg-opacity));
        --tw-bg-opacity: 0.1;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        --tw-text-opacity: 0.2}
    .btn:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--b3) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b3) / var(--tw-bg-opacity))}
    .btn-primary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--pf) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--pf) / var(--tw-bg-opacity))}
    .btn-secondary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--sf) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--sf) / var(--tw-bg-opacity))}
    .btn-accent:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--af) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--af) / var(--tw-bg-opacity))}
    .btn-neutral:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--nf) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--nf) / var(--tw-bg-opacity))}
    .btn-info:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--in) / var(--tw-bg-opacity))}
    .btn-success:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--su) / var(--tw-bg-opacity))}
    .btn-warning:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
    .btn-error:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--er) / var(--tw-bg-opacity))}
    .btn.glass:hover {
        --glass-opacity: 25%;
        --glass-border-opacity: 15%}
    .btn-ghost:hover {
        --tw-border-opacity: 0;
        background-color: hsl(var(--bc) / var(--tw-bg-opacity));
        --tw-bg-opacity: 0.2}
    .btn-link:hover {
        border-color: transparent;
        background-color: transparent;
        text-decoration-line: underline}
    .btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--bc) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--bc) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--b1) / var(--tw-text-opacity))}
    .btn-outline.btn-primary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--pf) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--pf) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--pc) / var(--tw-text-opacity))}
    .btn-outline.btn-secondary:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--sf) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--sf) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--sc) / var(--tw-text-opacity))}
    .btn-outline.btn-accent:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--af) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--af) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--ac) / var(--tw-text-opacity))}
    .btn-outline.btn-success:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--su) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--suc) / var(--tw-text-opacity))}
    .btn-outline.btn-info:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--in) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--inc) / var(--tw-text-opacity))}
    .btn-outline.btn-warning:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--wa) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--wac) / var(--tw-text-opacity))}
    .btn-outline.btn-error:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--er) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--erc) / var(--tw-text-opacity))}
    .btn-disabled:hover,
    .btn[disabled]:hover,
    .btn:disabled:hover {
        --tw-border-opacity: 0;
        background-color: hsl(var(--n) / var(--tw-bg-opacity));
        --tw-bg-opacity: 0.2;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        --tw-text-opacity: 0.2}
    .btn:is(input[type="checkbox"]:checked):hover, .btn:is(input[type="radio"]:checked):hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--pf) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--pf) / var(--tw-bg-opacity))}
    .dropdown.dropdown-hover:hover .dropdown-content {
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
    :where(.menu li:not(.menu-title):not(.disabled) > *:not(ul):not(details):not(.menu-title)):not(.active):hover, :where(.menu li:not(.menu-title):not(.disabled) > details > summary:not(.menu-title)):not(.active):hover {
        cursor: pointer;
        background-color: hsl(var(--bc) / 0.1);
        --tw-text-opacity: 1;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        outline: 2px solid transparent;
        outline-offset: 2px}
    .tab[disabled],
    .tab[disabled]:hover {
        cursor: not-allowed;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        --tw-text-opacity: 0.2}}
.dropdown:is(details) summary::-webkit-details-marker {
    display: none}
.file-input {
    height: 3rem;
    flex-shrink: 1;
    padding-right: 1rem;
    font-size: 1rem;
    line-height: 2;
    line-height: 1.5rem;
    overflow: hidden;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    border-radius: var(--rounded-btn, 0.5rem)}
.file-input::file-selector-button {
    margin-right: 1rem;
    display: inline-flex;
    height: 2.875rem;
    min-height: 2.875rem;
    flex-shrink: 0;
    cursor: pointer;
    user-select: none;
    flex-wrap: wrap;
    align-items: center;
    justify-content: center;
    padding-left: 1rem;
    padding-right: 1rem;
    text-align: center;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 1em;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms;
    border-style: solid;
    --tw-border-opacity: 1;
    border-color: hsl(var(--n) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    font-weight: 600;
    text-transform: uppercase;
    text-transform: var(--btn-text-case, uppercase);
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity));
    text-decoration-line: none;
    border-width: var(--border-btn, 1px);
    animation: button-pop var(--animation-btn, 0.25s) ease-out}
.footer {
    display: grid;
    width: 100%;
    grid-auto-flow: row;
    place-items: start;
    row-gap: 2.5rem;
    column-gap: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem}
.footer > * {
    display: grid;
    place-items: start;
    gap: 0.5rem}
.footer-center {
    place-items: center;
    text-align: center}
.footer-center > * {
    place-items: center}
@media (min-width: 48rem) {
    .footer {
        grid-auto-flow: column}
    .footer-center {
        grid-auto-flow: row dense}}
.form-control {
    display: flex;
    flex-direction: column}
.label {
    display: flex;
    user-select: none;
    align-items: center;
    justify-content: space-between;
    padding-left: 0.25rem;
    padding-right: 0.25rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem}
.hero {
    display: grid;
    width: 100%;
    place-items: center;
    background-size: cover;
    background-position: center}
.hero > * {
    grid-column-start: 1;
    grid-row-start: 1}
.hero-overlay {
    grid-column-start: 1;
    grid-row-start: 1;
    height: 100%;
    width: 100%;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.5}
.hero-content {
    z-index: 0;
    display: flex;
    align-items: center;
    justify-content: center;
    max-width: 80rem;
    gap: 1rem;
    padding: 1rem}
.indicator {
    position: relative;
    display: inline-flex;
    width: max-content}
.indicator :where(.indicator-item) {
    z-index: 1;
    position: absolute;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    white-space: nowrap}
.input {
    flex-shrink: 1;
    height: 3rem;
    padding-left: 1rem;
    padding-right: 1rem;
    font-size: 1rem;
    line-height: 2;
    line-height: 1.5rem;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    border-radius: var(--rounded-btn, 0.5rem)}
.input-group {
    display: flex;
    width: 100%;
    align-items: stretch}
.input-group > .input {
    isolation: isolate}
.input-group > *,
  .input-group > .input,
  .input-group > .textarea,
  .input-group > .select {
    border-radius: 0px}
.input-group-md {
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2}
.input-group-lg {
    font-size: 1.125rem;
    line-height: 1.75rem;
    line-height: 2}
.input-group-sm {
    font-size: 0.875rem;
    line-height: 2rem}
.input-group-xs {
    font-size: 0.75rem;
    line-height: 1rem;
    line-height: 1.625}
.input-group :where(span) {
    display: flex;
    align-items: center;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity));
    padding-left: 1rem;
    padding-right: 1rem}
.input-group > :first-child {
    border-top-left-radius: var(--rounded-btn, 0.5rem);
    border-top-right-radius: 0;
    border-bottom-left-radius: var(--rounded-btn, 0.5rem);
    border-bottom-right-radius: 0}
.input-group > :last-child {
    border-top-left-radius: 0;
    border-top-right-radius: var(--rounded-btn, 0.5rem);
    border-bottom-left-radius: 0;
    border-bottom-right-radius: var(--rounded-btn, 0.5rem)}
.input-group-vertical {
    flex-direction: column}
.input-group-vertical :first-child {
    border-top-left-radius: var(--rounded-btn, 0.5rem);
    border-top-right-radius: var(--rounded-btn, 0.5rem);
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0}
.input-group-vertical :last-child {
    border-top-left-radius: 0;
    border-top-right-radius: 0;
    border-bottom-left-radius: var(--rounded-btn, 0.5rem);
    border-bottom-right-radius: var(--rounded-btn, 0.5rem)}
.join {
    display: inline-flex;
    align-items: stretch;
    border-radius: var(--rounded-btn, 0.5rem)}
.join :where(.join-item) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
    border-bottom-left-radius: 0;
    border-top-left-radius: 0}
.join .join-item:not(:first-child):not(:last-child),
  .join *:not(:first-child):not(:last-child) .join-item {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
    border-bottom-left-radius: 0;
    border-top-left-radius: 0}
.join .join-item:first-child:not(:last-child),
  .join *:first-child:not(:last-child) .join-item {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0}
.join :where(.join-item:first-child:not(:last-child)),
  .join :where(*:first-child:not(:last-child) .join-item) {
    border-bottom-left-radius: inherit;
    border-top-left-radius: inherit}
.join .join-item:last-child:not(:first-child),
  .join *:last-child:not(:first-child) .join-item {
    border-bottom-left-radius: 0;
    border-top-left-radius: 0}
.join :where(.join-item:last-child:not(:first-child)),
  .join :where(*:last-child:not(:first-child) .join-item) {
    border-top-right-radius: inherit;
    border-bottom-right-radius: inherit}
:where(.join *) {
    border-radius: inherit}
.kbd {
    display: inline-flex;
    align-items: center;
    justify-content: center;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0.2;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    border-radius: var(--rounded-btn, 0.5rem);
    border-bottom-width: 2px;
    min-height: 2.2em;
    min-width: 2.2em}
.link {
    cursor: pointer;
    text-decoration-line: underline}
.link-hover {
    text-decoration-line: none}
.mask {
    -webkit-mask-size: contain;
            mask-size: contain;
    -webkit-mask-repeat: no-repeat;
            mask-repeat: no-repeat;
    -webkit-mask-position: center;
            mask-position: center}
.mask-half-1 {
    -webkit-mask-size: 200%;
            mask-size: 200%;
    -webkit-mask-position: left;
            mask-position: left}
.mask-half-2 {
    -webkit-mask-size: 200%;
            mask-size: 200%;
    -webkit-mask-position: right;
            mask-position: right}
.menu {
    display: flex;
    flex-direction: column;
    flex-wrap: wrap;
    font-size: 0.875rem;
    line-height: 1.25rem;
    padding: 0.5rem}
.menu :where(li ul) {
    position: relative;
    white-space: nowrap;
    margin-left: 1rem;
    padding-left: 0.5rem}
.menu :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),
  .menu :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
    display: grid;
    grid-auto-flow: column;
    align-content: flex-start;
    align-items: center;
    gap: 0.5rem;
    grid-auto-columns: minmax(auto, max-content) auto max-content;
    user-select: none}
.menu li.disabled {
    cursor: not-allowed;
    user-select: none;
    color: hsl(var(--bc) / 0.3)}
.menu :where(li > .menu-dropdown:not(.menu-dropdown-show)) {
    display: none}
:where(.menu li) {
    position: relative;
    display: flex;
    flex-shrink: 0;
    flex-direction: column;
    flex-wrap: wrap;
    align-items: stretch}
:where(.menu li) .badge {
    justify-self: end}
.mockup-code {
    position: relative;
    overflow: hidden;
    overflow-x: auto;
    min-width: 18rem;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    padding-top: 1.25rem;
    padding-bottom: 1.25rem;
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity));
    border-radius: var(--rounded-box, 1rem)}
.mockup-code pre[data-prefix]:before {
    content: attr(data-prefix);
    display: inline-block;
    text-align: right;
    width: 2rem;
    opacity: 0.5}
.mockup-window {
    position: relative;
    overflow: hidden;
    overflow-x: auto;
    padding-top: 1.25rem;
    border-radius: var(--rounded-box, 1rem)}
.mockup-window pre[data-prefix]:before {
    content: attr(data-prefix);
    display: inline-block;
    text-align: right}
.mockup-browser {
    position: relative;
    overflow: hidden;
    overflow-x: auto;
    border-radius: var(--rounded-box, 1rem)}
.mockup-browser pre[data-prefix]:before {
    content: attr(data-prefix);
    display: inline-block;
    text-align: right}
.modal {
    pointer-events: none;
    position: fixed;
    inset: 0px;
    margin: 0px;
    display: grid;
    height: 100%;
    max-height: none;
    width: 100%;
    max-width: none;
    justify-items: center;
    padding: 0px;
    opacity: 0;
    overscroll-behavior: contain;
    z-index: 999;
    background-color: transparent;
    color: inherit;
    transition-duration: 200ms;
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-property: transform, opacity, visibility;
    overflow-y: hidden}
.modal-scroll {
    overscroll-behavior: auto}
:where(.modal) {
    align-items: center}
.modal-box {
    max-height: calc(100vh - 5em);
    grid-column-start: 1;
    grid-row-start: 1;
    width: 91.666667%;
    max-width: 32rem;
    --tw-scale-x: .9;
    --tw-scale-y: .9;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    padding: 1.5rem;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms;
    border-top-left-radius: var(--rounded-box, 1rem);
    border-top-right-radius: var(--rounded-box, 1rem);
    border-bottom-left-radius: var(--rounded-box, 1rem);
    border-bottom-right-radius: var(--rounded-box, 1rem);
    box-shadow: rgba(0, 0, 0, 0.25) 0px 25px 50px -12px;
    overflow-y: auto;
    overscroll-behavior: contain}
.modal-open,
.modal:target,
.modal-toggle:checked + .modal,
.modal[open] {
    pointer-events: auto;
    visibility: visible;
    opacity: 1}
.modal-action {
    display: flex;
    margin-top: 1.5rem;
    justify-content: flex-end}
.modal-toggle {
    position: fixed;
    height: 0px;
    width: 0px;
    appearance: none;
    opacity: 0}
:root:has(:is(.modal-open, .modal:target, .modal-toggle:checked + .modal, .modal[open])) {
    overflow: hidden}
.navbar {
    display: flex;
    align-items: center;
    padding: var(--navbar-padding, 0.5rem);
    min-height: 4rem;
    width: 100%}
:where(.navbar > *) {
    display: inline-flex;
    align-items: center}
.navbar-start {
    width: 50%;
    justify-content: flex-start}
.navbar-center {
    flex-shrink: 0}
.navbar-end {
    width: 50%;
    justify-content: flex-end}
.progress {
    position: relative;
    width: 100%;
    appearance: none;
    overflow: hidden;
    height: 0.5rem;
    background-color: hsl(var(--bc) / 0.2);
    border-radius: var(--rounded-box, 1rem)}
.radial-progress {
    position: relative;
    display: inline-grid;
    height: var(--size);
    width: var(--size);
    place-content: center;
    border-radius: 9999px;
    background-color: transparent;
    vertical-align: middle;
    box-sizing: content-box;
    --value: 0;
    --size: 5rem;
    --thickness: calc(var(--size) / 10)}
.radial-progress::-moz-progress-bar {
    appearance: none;
    background-color: transparent}
.radial-progress::-webkit-progress-value {
    appearance: none;
    background-color: transparent}
.radial-progress::-webkit-progress-bar {
    appearance: none;
    background-color: transparent}
.radial-progress:before,
.radial-progress:after {
    position: absolute;
    border-radius: 9999px;
    content: ""}
.radial-progress:before {
    inset: 0px;
    background: radial-gradient(farthest-side, currentColor 98%, #0000) top/var(--thickness)
      var(--thickness) no-repeat,
    conic-gradient(currentColor calc(var(--value) * 1%), #0000 0);
    -webkit-mask: radial-gradient(
    farthest-side,
    #0000 calc(99% - var(--thickness)),
    #000 calc(100% - var(--thickness))
  );
    mask: radial-gradient(
    farthest-side,
    #0000 calc(99% - var(--thickness)),
    #000 calc(100% - var(--thickness))
  )}
.radial-progress:after {
    inset: calc(50% - var(--thickness) / 2);
    transform: rotate(calc(var(--value) * 3.6deg - 90deg)) translate(calc(var(--size) / 2 - 50%));
    background-color: currentColor}
.radio {
    flex-shrink: 0;
    --chkbg: var(--bc);
    height: 1.5rem;
    width: 1.5rem;
    cursor: pointer;
    appearance: none;
    border-radius: 9999px;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0.2}
.range {
    height: 1.5rem;
    width: 100%;
    cursor: pointer;
    appearance: none;
    -webkit-appearance: none;
    --range-shdw: var(--bc);
    overflow: hidden;
    background-color: transparent;
    border-radius: var(--rounded-box, 1rem)}
.range:focus {
    outline: none}
.rating {
    position: relative;
    display: inline-flex}
.rating :where(input) {
    cursor: pointer;
    border-radius: 0px;
    animation: rating-pop var(--animation-input, 0.25s) ease-out;
    height: 1.5rem;
    width: 1.5rem;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    --tw-bg-opacity: 1}
.select {
    display: inline-flex;
    cursor: pointer;
    user-select: none;
    appearance: none;
    height: 3rem;
    padding-left: 1rem;
    padding-right: 2.5rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2;
    min-height: 3rem;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    font-weight: 600;
    border-radius: var(--rounded-btn, 0.5rem);
    background-image: linear-gradient(45deg, transparent 50%, currentColor 50%),
    linear-gradient(135deg, currentColor 50%, transparent 50%);
    background-position: calc(100% - 20px) calc(1px + 50%), calc(100% - 16px) calc(1px + 50%);
    background-size: 4px 4px, 4px 4px;
    background-repeat: no-repeat}
.select[multiple] {
    height: auto}
.stack {
    display: inline-grid;
    place-items: center;
    align-items: flex-end}
.stack > * {
    grid-column-start: 1;
    grid-row-start: 1;
    transform: translateY(10%) scale(0.9);
    z-index: 1;
    width: 100%;
    opacity: 0.6}
.stack > *:nth-child(2) {
    transform: translateY(5%) scale(0.95);
    z-index: 2;
    opacity: 0.8}
.stack > *:nth-child(1) {
    transform: translateY(0) scale(1);
    z-index: 3;
    opacity: 1}
.stats {
    display: inline-grid;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    border-radius: var(--rounded-box, 1rem)}
:where(.stats) {
    grid-auto-flow: column;
    overflow-x: auto}
.stat {
    display: inline-grid;
    width: 100%;
    grid-template-columns: repeat(1, 1fr);
    column-gap: 1rem;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    padding-top: 1rem;
    padding-bottom: 1rem}
.stat-figure {
    grid-column-start: 2;
    grid-row: span 3 / span 3;
    grid-row-start: 1;
    place-self: center;
    justify-self: end}
.stat-title {
    grid-column-start: 1;
    white-space: nowrap;
    color: hsl(var(--bc) / 0.6)}
.stat-value {
    grid-column-start: 1;
    white-space: nowrap;
    font-size: 2.25rem;
    line-height: 2.5rem;
    font-weight: 800}
.stat-desc {
    grid-column-start: 1;
    white-space: nowrap;
    font-size: 0.75rem;
    line-height: 1rem;
    color: hsl(var(--bc) / 0.6)}
.stat-actions {
    grid-column-start: 1;
    white-space: nowrap;
    margin-top: 1rem}
.steps {
    display: inline-grid;
    grid-auto-flow: column;
    overflow: hidden;
    overflow-x: auto;
    counter-reset: step;
    grid-auto-columns: 1fr}
.steps .step {
    display: grid;
    grid-template-columns: repeat(1, minmax(0, 1fr));
    grid-template-columns: auto;
    grid-template-rows: repeat(2, minmax(0, 1fr));
    grid-template-rows: 40px 1fr;
    place-items: center;
    text-align: center;
    min-width: 4rem}
.swap {
    position: relative;
    display: inline-grid;
    user-select: none;
    place-content: center;
    cursor: pointer}
.swap > * {
    grid-column-start: 1;
    grid-row-start: 1;
    transition-duration: 300ms;
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-property: transform, opacity}
.swap input {
    appearance: none}
.swap .swap-on,
.swap .swap-indeterminate,
.swap input:indeterminate ~ .swap-on {
    opacity: 0}
.swap input:checked ~ .swap-off,
.swap.swap-active .swap-off,
.swap input:indeterminate ~ .swap-off {
    opacity: 0}
.swap input:checked ~ .swap-on,
.swap-active .swap-on,
.swap input:indeterminate ~ .swap-indeterminate {
    opacity: 1}
.tabs {
    display: flex;
    flex-wrap: wrap;
    align-items: flex-end}
.tab {
    position: relative;
    display: inline-flex;
    cursor: pointer;
    user-select: none;
    flex-wrap: wrap;
    align-items: center;
    justify-content: center;
    text-align: center;
    height: 2rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2;
    --tab-padding: 1rem;
    --tw-text-opacity: 0.5;
    --tab-color: hsl(var(--bc) / var(--tw-text-opacity, 1));
    --tab-bg: hsl(var(--b1) / var(--tw-bg-opacity, 1));
    --tab-border-color: hsl(var(--b3) / var(--tw-bg-opacity, 1));
    color: var(--tab-color);
    padding-left: var(--tab-padding, 1rem);
    padding-right: var(--tab-padding, 1rem)}
.table {
    position: relative;
    width: 100%;
    text-align: left;
    font-size: 0.875rem;
    line-height: 1.25rem;
    border-radius: var(--rounded-box, 1rem)}
.table :where(.table-pin-rows thead tr) {
    position: sticky;
    top: 0px;
    z-index: 1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity))}
.table :where(.table-pin-rows tfoot tr) {
    position: sticky;
    bottom: 0px;
    z-index: 1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity))}
.table :where(.table-pin-cols tr th) {
    position: sticky;
    left: 0px;
    right: 0px;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity))}
.table-zebra tbody tr:nth-child(even) :where(.table-pin-cols tr th) {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity))}
.textarea {
    flex-shrink: 1;
    min-height: 3rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    padding-left: 1rem;
    padding-right: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    border-radius: var(--rounded-btn, 0.5rem)}
.toast {
    position: fixed;
    display: flex;
    min-width: fit-content;
    flex-direction: column;
    white-space: nowrap;
    gap: 0.5rem;
    padding: 1rem}
.toggle {
    flex-shrink: 0;
    --tglbg: hsl(var(--b1));
    --handleoffset: 1.5rem;
    --handleoffsetcalculator: calc(var(--handleoffset) * -1);
    --togglehandleborder: 0 0;
    height: 1.5rem;
    width: 3rem;
    cursor: pointer;
    appearance: none;
    border-width: 1px;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0.2;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.5;
    border-radius: var(--rounded-badge, 1.9rem);
    transition: background, box-shadow var(--animation-input, 0.2s) ease-out;
    box-shadow: var(--handleoffsetcalculator) 0 0 2px var(--tglbg) inset, 0 0 0 2px var(--tglbg) inset,
    var(--togglehandleborder)}
.alert-info {
    border-color: hsl(var(--in) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--in));
    --alert-bg-mix: hsl(var(--b1))}
.alert-success {
    border-color: hsl(var(--su) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--su));
    --alert-bg-mix: hsl(var(--b1))}
.alert-warning {
    border-color: hsl(var(--wa) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--wa));
    --alert-bg-mix: hsl(var(--b1))}
.alert-error {
    border-color: hsl(var(--er) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--er));
    --alert-bg-mix: hsl(var(--b1))}
.avatar-group {
    display: flex;
    overflow: hidden}
.avatar-group :where(.avatar) {
    overflow: hidden;
    border-radius: 9999px;
    border-width: 4px;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b1) / var(--tw-border-opacity))}
.badge-neutral {
    --tw-border-opacity: 1;
    border-color: hsl(var(--n) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity))}
.badge-primary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.badge-secondary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.badge-accent {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.badge-info {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.badge-success {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.badge-warning {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.badge-error {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.badge-ghost {
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity))}
.badge-outline {
    border-color: currentColor;
    --tw-border-opacity: 0.5;
    background-color: transparent;
    color: currentColor}
.badge-outline.badge-neutral {
    --tw-text-opacity: 1;
    color: hsl(var(--n) / var(--tw-text-opacity))}
.badge-outline.badge-primary {
    --tw-text-opacity: 1;
    color: hsl(var(--p) / var(--tw-text-opacity))}
.badge-outline.badge-secondary {
    --tw-text-opacity: 1;
    color: hsl(var(--s) / var(--tw-text-opacity))}
.badge-outline.badge-accent {
    --tw-text-opacity: 1;
    color: hsl(var(--a) / var(--tw-text-opacity))}
.badge-outline.badge-info {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.badge-outline.badge-success {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.badge-outline.badge-warning {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.badge-outline.badge-error {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.btm-nav > *:not(.active) {
    padding-top: 0.125rem}
.btm-nav > *:where(.active) {
    border-top-width: 2px;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity))}
.btm-nav > *.disabled,
    .btm-nav > *[disabled] {
    pointer-events: none;
    --tw-border-opacity: 0;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    --tw-text-opacity: 0.2}
.btm-nav > * .label {
    font-size: 1rem;
    line-height: 1.5rem}
.breadcrumbs > ul > li > a:focus, .breadcrumbs > ol > li > a:focus {
    outline: 2px solid transparent;
    outline-offset: 2px}
.breadcrumbs > ul > li > a:focus-visible, .breadcrumbs > ol > li > a:focus-visible {
    outline: 2px solid currentColor;
    outline-offset: 2px}
.breadcrumbs > ul > li + *:before, .breadcrumbs > ol > li + *:before {
    content: "";
    margin-left: 0.5rem;
    margin-right: 0.75rem;
    display: block;
    height: 0.375rem;
    width: 0.375rem;
    --tw-rotate: 45deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    opacity: 0.4;
    border-top: 1px solid;
    border-right: 1px solid;
    background-color: transparent}
[dir="rtl"] .breadcrumbs > ul > li + *:before,
[dir="rtl"] .breadcrumbs > ol > li + *:before {
    --tw-rotate: -45deg}
.btn:active:hover,
  .btn:active:focus {
    animation: button-pop 0s ease-out;
    transform: scale(var(--btn-focus-scale, 0.97))}
.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--b3) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity))}
.btn:focus-visible {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px}
.btn-primary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity));
    outline-color: hsl(var(--p) / 1)}
.btn-primary.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--pf) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--pf) / var(--tw-bg-opacity))}
.btn-secondary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity));
    outline-color: hsl(var(--s) / 1)}
.btn-secondary.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--sf) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--sf) / var(--tw-bg-opacity))}
.btn-accent {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity));
    outline-color: hsl(var(--a) / 1)}
.btn-accent.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--af) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--af) / var(--tw-bg-opacity))}
.btn-neutral {
    --tw-border-opacity: 1;
    border-color: hsl(var(--n) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity));
    outline-color: hsl(var(--n) / 1)}
.btn-neutral.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--nf) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--nf) / var(--tw-bg-opacity))}
.btn-info {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity));
    outline-color: hsl(var(--in) / 1)}
.btn-info.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.btn-success {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity));
    outline-color: hsl(var(--su) / 1)}
.btn-success.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.btn-warning {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity));
    outline-color: hsl(var(--wa) / 1)}
.btn-warning.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.btn-error {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity));
    outline-color: hsl(var(--er) / 1)}
.btn-error.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.btn.glass {
    --tw-shadow: 0 0 #0000;
    --tw-shadow-colored: 0 0 #0000;
    box-shadow: var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow);
    outline-color: currentColor}
.btn.glass.btn-active {
    --glass-opacity: 25%;
    --glass-border-opacity: 15%}
.btn-ghost {
    border-width: 1px;
    border-color: transparent;
    background-color: transparent;
    color: currentColor;
    --tw-shadow: 0 0 #0000;
    --tw-shadow-colored: 0 0 #0000;
    box-shadow: var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow);
    outline-color: currentColor}
.btn-ghost.btn-active {
    --tw-border-opacity: 0;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.2}
.btn-link {
    border-color: transparent;
    background-color: transparent;
    --tw-text-opacity: 1;
    color: hsl(var(--p) / var(--tw-text-opacity));
    text-decoration-line: underline;
    --tw-shadow: 0 0 #0000;
    --tw-shadow-colored: 0 0 #0000;
    box-shadow: var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow);
    outline-color: currentColor}
.btn-link.btn-active {
    border-color: transparent;
    background-color: transparent;
    text-decoration-line: underline}
.btn-outline {
    border-color: currentColor;
    background-color: transparent;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    --tw-shadow: 0 0 #0000;
    --tw-shadow-colored: 0 0 #0000;
    box-shadow: var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow)}
.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--b1) / var(--tw-text-opacity))}
.btn-outline.btn-primary {
    --tw-text-opacity: 1;
    color: hsl(var(--p) / var(--tw-text-opacity))}
.btn-outline.btn-primary.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--pf) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--pf) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.btn-outline.btn-secondary {
    --tw-text-opacity: 1;
    color: hsl(var(--s) / var(--tw-text-opacity))}
.btn-outline.btn-secondary.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--sf) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--sf) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.btn-outline.btn-accent {
    --tw-text-opacity: 1;
    color: hsl(var(--a) / var(--tw-text-opacity))}
.btn-outline.btn-accent.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--af) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--af) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.btn-outline.btn-success {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.btn-outline.btn-success.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.btn-outline.btn-info {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.btn-outline.btn-info.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.btn-outline.btn-warning {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.btn-outline.btn-warning.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.btn-outline.btn-error {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.btn-outline.btn-error.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.btn.btn-disabled,
  .btn[disabled],
  .btn:disabled {
    --tw-border-opacity: 0;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.2;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    --tw-text-opacity: 0.2}
.btn-group > input[type="radio"]:checked.btn,
  .btn-group > .btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.btn-group > input[type="radio"]:checked.btn:focus-visible, .btn-group > .btn-active:focus-visible {
    outline-style: solid;
    outline-width: 2px;
    outline-color: hsl(var(--p) / 1)}
.btn:is(input[type="checkbox"]:checked),
.btn:is(input[type="radio"]:checked) {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.btn:is(input[type="checkbox"]:checked):focus-visible, .btn:is(input[type="radio"]:checked):focus-visible {
    outline-color: hsl(var(--p) / 1)}
@keyframes button-pop {
    0% {
        transform: scale(var(--btn-focus-scale, 0.98))}
    40% {
        transform: scale(1.02)}
    100% {
        transform: scale(1)}}
.card :where(figure:first-child) {
    overflow: hidden;
    border-start-start-radius: inherit;
    border-start-end-radius: inherit;
    border-end-start-radius: unset;
    border-end-end-radius: unset}
.card :where(figure:last-child) {
    overflow: hidden;
    border-start-start-radius: unset;
    border-start-end-radius: unset;
    border-end-start-radius: inherit;
    border-end-end-radius: inherit}
.card:focus-visible {
    outline: 2px solid currentColor;
    outline-offset: 2px}
.card.bordered {
    border-width: 1px;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity))}
.card-bordered {
    border-width: 1px;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity))}
.card.compact .card-body {
    padding: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem}
.card-title {
    display: flex;
    align-items: center;
    gap: 0.5rem;
    font-size: 1.25rem;
    line-height: 1.75rem;
    font-weight: 600}
.card.image-full :where(figure) {
    overflow: hidden;
    border-radius: inherit}
.carousel::-webkit-scrollbar {
    display: none}
.chat-bubble-primary {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.chat-bubble-secondary {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.chat-bubble-accent {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.chat-bubble-info {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.chat-bubble-success {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.chat-bubble-warning {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.chat-bubble-error {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.checkbox:focus-visible {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 1)}
.checkbox:checked,
  .checkbox[checked="true"],
  .checkbox[aria-checked="true"] {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    background-repeat: no-repeat;
    animation: checkmark var(--animation-input, 0.2s) ease-out;
    background-image: linear-gradient(-45deg, transparent 65%, hsl(var(--chkbg)) 65.99%),
      linear-gradient(45deg, transparent 75%, hsl(var(--chkbg)) 75.99%),
      linear-gradient(-45deg, hsl(var(--chkbg)) 40%, transparent 40.99%),
      linear-gradient(
        45deg,
        hsl(var(--chkbg)) 30%,
        hsl(var(--chkfg)) 30.99%,
        hsl(var(--chkfg)) 40%,
        transparent 40.99%
      ),
      linear-gradient(-45deg, hsl(var(--chkfg)) 50%, hsl(var(--chkbg)) 50.99%)}
.checkbox:indeterminate {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    background-repeat: no-repeat;
    animation: checkmark var(--animation-input, 0.2s) ease-out;
    background-image: linear-gradient(90deg, transparent 80%, hsl(var(--chkbg)) 80%),
      linear-gradient(-90deg, transparent 80%, hsl(var(--chkbg)) 80%),
      linear-gradient(
        0deg,
        hsl(var(--chkbg)) 43%,
        hsl(var(--chkfg)) 43%,
        hsl(var(--chkfg)) 57%,
        hsl(var(--chkbg)) 57%
      )}
.checkbox-primary {
    --chkbg: var(--p);
    --chkfg: var(--pc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity))}
.checkbox-primary:focus-visible {
    outline-color: hsl(var(--p) / 1)}
.checkbox-primary:checked,
    .checkbox-primary[checked="true"],
    .checkbox-primary[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.checkbox-secondary {
    --chkbg: var(--s);
    --chkfg: var(--sc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity))}
.checkbox-secondary:focus-visible {
    outline-color: hsl(var(--s) / 1)}
.checkbox-secondary:checked,
    .checkbox-secondary[checked="true"],
    .checkbox-secondary[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.checkbox-accent {
    --chkbg: var(--a);
    --chkfg: var(--ac);
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity))}
.checkbox-accent:focus-visible {
    outline-color: hsl(var(--a) / 1)}
.checkbox-accent:checked,
    .checkbox-accent[checked="true"],
    .checkbox-accent[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.checkbox-success {
    --chkbg: var(--su);
    --chkfg: var(--suc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.checkbox-success:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.checkbox-success:checked,
    .checkbox-success[checked="true"],
    .checkbox-success[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.checkbox-warning {
    --chkbg: var(--wa);
    --chkfg: var(--wac);
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.checkbox-warning:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.checkbox-warning:checked,
    .checkbox-warning[checked="true"],
    .checkbox-warning[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.checkbox-info {
    --chkbg: var(--in);
    --chkfg: var(--inc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.checkbox-info:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.checkbox-info:checked,
    .checkbox-info[checked="true"],
    .checkbox-info[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.checkbox-error {
    --chkbg: var(--er);
    --chkfg: var(--erc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.checkbox-error:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.checkbox-error:checked,
    .checkbox-error[checked="true"],
    .checkbox-error[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.checkbox:disabled {
    cursor: not-allowed;
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    opacity: 0.2}
@keyframes checkmark {
    0% {
        background-position-y: 5px}
    50% {
        background-position-y: -2px}
    100% {
        background-position-y: 0}}
.checkbox-mark {
    display: none}
[dir="rtl"] .checkbox:checked,
    [dir="rtl"] .checkbox[checked="true"],
    [dir="rtl"] .checkbox[aria-checked="true"] {
    background-image: linear-gradient(45deg, transparent 65%, hsl(var(--chkbg)) 65.99%),
        linear-gradient(-45deg, transparent 75%, hsl(var(--chkbg)) 75.99%),
        linear-gradient(45deg, hsl(var(--chkbg)) 40%, transparent 40.99%),
        linear-gradient(
          -45deg,
          hsl(var(--chkbg)) 30%,
          hsl(var(--chkfg)) 30.99%,
          hsl(var(--chkfg)) 40%,
          transparent 40.99%
        ),
        linear-gradient(45deg, hsl(var(--chkfg)) 50%, hsl(var(--chkbg)) 50.99%)}
details.collapse {
    width: 100%}
details.collapse summary {
    position: relative;
    display: block;
    outline: 2px solid transparent;
    outline-offset: 2px}
details.collapse summary::-webkit-details-marker {
    display: none}
.collapse:focus-visible {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 1)}
.collapse:has(.collapse-title:focus-visible),
.collapse:has(> input[type="checkbox"]:focus-visible),
.collapse:has(> input[type="radio"]:focus-visible) {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 1)}
.collapse-arrow .collapse-title:after {
    position: absolute;
    display: block;
    height: 0.5rem;
    width: 0.5rem;
    --tw-translate-y: -100%;
    --tw-rotate: 45deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    transition-property: all;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 150ms;
    transition-duration: 0.2s;
    top: 50%;
    right: 1.4rem;
    content: "";
    transform-origin: 75% 75%;
    box-shadow: 2px 2px;
    pointer-events: none}
[dir="rtl"] .collapse-arrow .collapse-title:after {
    --tw-rotate: -45deg}
.collapse-plus .collapse-title:after {
    position: absolute;
    display: block;
    height: 0.5rem;
    width: 0.5rem;
    transition-property: all;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 300ms;
    top: 0.9rem;
    right: 1.4rem;
    content: "+";
    pointer-events: none}
.collapse:not(.collapse-open):not(.collapse-close) input[type="checkbox"],
.collapse:not(.collapse-open):not(.collapse-close) input[type="radio"]:not(:checked),
.collapse:not(.collapse-open):not(.collapse-close) .collapse-title {
    cursor: pointer}
.collapse:focus:not(.collapse-open):not(.collapse-close):not(.collapse[open]) .collapse-title {
    cursor: unset}
.collapse-title {
    position: relative}
:where(.collapse > input[type="checkbox"]),
:where(.collapse > input[type="radio"]) {
    z-index: 1}
.collapse-title,
:where(.collapse > input[type="checkbox"]),
:where(.collapse > input[type="radio"]) {
    width: 100%;
    padding: 1rem;
    padding-right: 3rem;
    min-height: 3.75rem;
    transition: background-color 0.2s ease-out}
.collapse[open] :where(.collapse-content),
.collapse-open :where(.collapse-content),
.collapse:focus:not(.collapse-close) :where(.collapse-content),
.collapse:not(.collapse-close) :where(input[type="checkbox"]:checked ~ .collapse-content),
.collapse:not(.collapse-close) :where(input[type="radio"]:checked ~ .collapse-content) {
    padding-bottom: 1rem;
    transition: padding 0.2s ease-out, background-color 0.2s ease-out}
.collapse[open].collapse-arrow .collapse-title:after,
.collapse-open.collapse-arrow .collapse-title:after,
.collapse-arrow:focus:not(.collapse-close) .collapse-title:after,
.collapse-arrow:not(.collapse-close) input[type="checkbox"]:checked ~ .collapse-title:after,
.collapse-arrow:not(.collapse-close) input[type="radio"]:checked ~ .collapse-title:after {
    --tw-translate-y: -50%;
    --tw-rotate: 225deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
[dir="rtl"] .collapse[open].collapse-arrow .collapse-title:after,
[dir="rtl"] .collapse-open.collapse-arrow .collapse-title:after,
[dir="rtl"] .collapse-arrow:focus:not(.collapse-close) .collapse-title:after,
[dir="rtl"]
  .collapse-arrow:not(.collapse-close)
  input[type="checkbox"]:checked
  ~ .collapse-title:after {
    --tw-rotate: 135deg}
.collapse[open].collapse-plus .collapse-title:after,
.collapse-open.collapse-plus .collapse-title:after,
.collapse-plus:focus:not(.collapse-close) .collapse-title:after,
.collapse-plus:not(.collapse-close) input[type="checkbox"]:checked ~ .collapse-title:after,
.collapse-plus:not(.collapse-close) input[type="radio"]:checked ~ .collapse-title:after {
    content: "−"}
.divider:before {
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.1}
.divider:after {
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.1}
.divider:not(:empty) {
    gap: 1rem}
.drawer-toggle:checked ~ .drawer-side > .drawer-overlay {
    background-color: hsl(0 0% 0%/0.4)}
.drawer-toggle:focus-visible ~ .drawer-content label.drawer-button {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px}
.dropdown.dropdown-open .dropdown-content,
.dropdown:focus .dropdown-content,
.dropdown:focus-within .dropdown-content {
    --tw-scale-x: 1;
    --tw-scale-y: 1;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.file-input-bordered {
    --tw-border-opacity: 0.2}
.file-input:focus {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 0.2)}
.file-input-ghost {
    --tw-bg-opacity: 0.05}
.file-input-ghost:focus {
    --tw-bg-opacity: 1;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    box-shadow: none}
.file-input-ghost::file-selector-button {
    border-width: 1px;
    border-color: transparent;
    background-color: transparent;
    color: currentColor}
.file-input-primary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity))}
.file-input-primary:focus {
    outline-color: hsl(var(--p) / 1)}
.file-input-primary::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.file-input-secondary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity))}
.file-input-secondary:focus {
    outline-color: hsl(var(--s) / 1)}
.file-input-secondary::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.file-input-accent {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity))}
.file-input-accent:focus {
    outline-color: hsl(var(--a) / 1)}
.file-input-accent::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.file-input-info {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.file-input-info:focus {
    outline-color: hsl(var(--in) / 1)}
.file-input-info::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.file-input-success {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.file-input-success:focus {
    outline-color: hsl(var(--su) / 1)}
.file-input-success::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.file-input-warning {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.file-input-warning:focus {
    outline-color: hsl(var(--wa) / 1)}
.file-input-warning::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.file-input-error {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.file-input-error:focus {
    outline-color: hsl(var(--er) / 1)}
.file-input-error::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.file-input-disabled,
  .file-input[disabled] {
    cursor: not-allowed;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    --tw-text-opacity: 0.2}
.file-input-disabled::placeholder,
  .file-input[disabled]::placeholder {
    color: hsl(var(--bc) / var(--tw-placeholder-opacity));
    --tw-placeholder-opacity: 0.2}
.file-input-disabled::file-selector-button, .file-input[disabled]::file-selector-button {
    --tw-border-opacity: 0;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-bg-opacity: 0.2;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    --tw-text-opacity: 0.2}
.footer-title {
    margin-bottom: 0.5rem;
    font-weight: 700;
    text-transform: uppercase;
    opacity: 0.5}
.label-text {
    font-size: 0.875rem;
    line-height: 1.25rem;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity))}
.label-text-alt {
    font-size: 0.75rem;
    line-height: 1rem;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity))}
.input[list]::-webkit-calendar-picker-indicator {
    line-height: 1em}
.input-bordered {
    --tw-border-opacity: 0.2}
.input:focus {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 0.2)}
.input-ghost {
    --tw-bg-opacity: 0.05}
.input-ghost:focus {
    --tw-bg-opacity: 1;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    box-shadow: none}
.input-primary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity))}
.input-primary:focus {
    outline-color: hsl(var(--p) / 1)}
.input-secondary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity))}
.input-secondary:focus {
    outline-color: hsl(var(--s) / 1)}
.input-accent {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity))}
.input-accent:focus {
    outline-color: hsl(var(--a) / 1)}
.input-info {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.input-info:focus {
    outline-color: hsl(var(--in) / 1)}
.input-success {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.input-success:focus {
    outline-color: hsl(var(--su) / 1)}
.input-warning {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.input-warning:focus {
    outline-color: hsl(var(--wa) / 1)}
.input-error {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.input-error:focus {
    outline-color: hsl(var(--er) / 1)}
.input-disabled,
  .input:disabled,
  .input[disabled] {
    cursor: not-allowed;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    --tw-text-opacity: 0.2}
.input-disabled::placeholder,
  .input:disabled::placeholder,
  .input[disabled]::placeholder {
    color: hsl(var(--bc) / var(--tw-placeholder-opacity));
    --tw-placeholder-opacity: 0.2}
.join > :where(*:not(:first-child)) {
    margin-top: 0px;
    margin-bottom: 0px;
    margin-left: -1px}
.join-item:focus {
    isolation: isolate}
.link-primary {
    --tw-text-opacity: 1;
    color: hsl(var(--p) / var(--tw-text-opacity))}
.link-secondary {
    --tw-text-opacity: 1;
    color: hsl(var(--s) / var(--tw-text-opacity))}
.link-accent {
    --tw-text-opacity: 1;
    color: hsl(var(--a) / var(--tw-text-opacity))}
.link-neutral {
    --tw-text-opacity: 1;
    color: hsl(var(--n) / var(--tw-text-opacity))}
.link-success {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.link-info {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.link-warning {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.link-error {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.link:focus {
    outline: 2px solid transparent;
    outline-offset: 2px}
.link:focus-visible {
    outline: 2px solid currentColor;
    outline-offset: 2px}
.loading {
    pointer-events: none;
    display: inline-block;
    aspect-ratio: 1 / 1;
    width: 1.5rem;
    background-color: currentColor;
    -webkit-mask-size: 100%;
            mask-size: 100%;
    -webkit-mask-repeat: no-repeat;
            mask-repeat: no-repeat;
    -webkit-mask-position: center;
            mask-position: center;
    -webkit-mask-image: url("data:image/svg+xml,%3Csvg width='24' height='24' stroke='%23000' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_V8m1%7Btransform-origin:center;animation:spinner_zKoa 2s linear infinite%7D.spinner_V8m1 circle%7Bstroke-linecap:round;animation:spinner_YpZS 1.5s ease-out infinite%7D%40keyframes spinner_zKoa%7B100%25%7Btransform:rotate(360deg)%7D%7D%40keyframes spinner_YpZS%7B0%25%7Bstroke-dasharray:0 150;stroke-dashoffset:0%7D47.5%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-16%7D95%25%2C100%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-59%7D%7D%3C%2Fstyle%3E%3Cg class='spinner_V8m1'%3E%3Ccircle cx='12' cy='12' r='9.5' fill='none' stroke-width='3'%3E%3C%2Fcircle%3E%3C%2Fg%3E%3C%2Fsvg%3E");
            mask-image: url("data:image/svg+xml,%3Csvg width='24' height='24' stroke='%23000' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_V8m1%7Btransform-origin:center;animation:spinner_zKoa 2s linear infinite%7D.spinner_V8m1 circle%7Bstroke-linecap:round;animation:spinner_YpZS 1.5s ease-out infinite%7D%40keyframes spinner_zKoa%7B100%25%7Btransform:rotate(360deg)%7D%7D%40keyframes spinner_YpZS%7B0%25%7Bstroke-dasharray:0 150;stroke-dashoffset:0%7D47.5%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-16%7D95%25%2C100%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-59%7D%7D%3C%2Fstyle%3E%3Cg class='spinner_V8m1'%3E%3Ccircle cx='12' cy='12' r='9.5' fill='none' stroke-width='3'%3E%3C%2Fcircle%3E%3C%2Fg%3E%3C%2Fsvg%3E")}
.loading-spinner {
    -webkit-mask-image: url("data:image/svg+xml,%3Csvg width='24' height='24' stroke='%23000' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_V8m1%7Btransform-origin:center;animation:spinner_zKoa 2s linear infinite%7D.spinner_V8m1 circle%7Bstroke-linecap:round;animation:spinner_YpZS 1.5s ease-out infinite%7D%40keyframes spinner_zKoa%7B100%25%7Btransform:rotate(360deg)%7D%7D%40keyframes spinner_YpZS%7B0%25%7Bstroke-dasharray:0 150;stroke-dashoffset:0%7D47.5%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-16%7D95%25%2C100%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-59%7D%7D%3C%2Fstyle%3E%3Cg class='spinner_V8m1'%3E%3Ccircle cx='12' cy='12' r='9.5' fill='none' stroke-width='3'%3E%3C%2Fcircle%3E%3C%2Fg%3E%3C%2Fsvg%3E");
            mask-image: url("data:image/svg+xml,%3Csvg width='24' height='24' stroke='%23000' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_V8m1%7Btransform-origin:center;animation:spinner_zKoa 2s linear infinite%7D.spinner_V8m1 circle%7Bstroke-linecap:round;animation:spinner_YpZS 1.5s ease-out infinite%7D%40keyframes spinner_zKoa%7B100%25%7Btransform:rotate(360deg)%7D%7D%40keyframes spinner_YpZS%7B0%25%7Bstroke-dasharray:0 150;stroke-dashoffset:0%7D47.5%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-16%7D95%25%2C100%25%7Bstroke-dasharray:42 150;stroke-dashoffset:-59%7D%7D%3C%2Fstyle%3E%3Cg class='spinner_V8m1'%3E%3Ccircle cx='12' cy='12' r='9.5' fill='none' stroke-width='3'%3E%3C%2Fcircle%3E%3C%2Fg%3E%3C%2Fsvg%3E")}
.loading-dots {
    -webkit-mask-image: url("data:image/svg+xml,%3Csvg width='24' height='24' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_qM83%7Banimation:spinner_8HQG 1.05s infinite%7D.spinner_oXPr%7Banimation-delay:.1s%7D.spinner_ZTLf%7Banimation-delay:.2s%7D@keyframes spinner_8HQG%7B0%25,57.14%25%7Banimation-timing-function:cubic-bezier(0.33,.66,.66,1);transform:translate(0)%7D28.57%25%7Banimation-timing-function:cubic-bezier(0.33,0,.66,.33);transform:translateY(-6px)%7D100%25%7Btransform:translate(0)%7D%7D%3C/style%3E%3Ccircle class='spinner_qM83' cx='4' cy='12' r='3'/%3E%3Ccircle class='spinner_qM83 spinner_oXPr' cx='12' cy='12' r='3'/%3E%3Ccircle class='spinner_qM83 spinner_ZTLf' cx='20' cy='12' r='3'/%3E%3C/svg%3E");
            mask-image: url("data:image/svg+xml,%3Csvg width='24' height='24' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_qM83%7Banimation:spinner_8HQG 1.05s infinite%7D.spinner_oXPr%7Banimation-delay:.1s%7D.spinner_ZTLf%7Banimation-delay:.2s%7D@keyframes spinner_8HQG%7B0%25,57.14%25%7Banimation-timing-function:cubic-bezier(0.33,.66,.66,1);transform:translate(0)%7D28.57%25%7Banimation-timing-function:cubic-bezier(0.33,0,.66,.33);transform:translateY(-6px)%7D100%25%7Btransform:translate(0)%7D%7D%3C/style%3E%3Ccircle class='spinner_qM83' cx='4' cy='12' r='3'/%3E%3Ccircle class='spinner_qM83 spinner_oXPr' cx='12' cy='12' r='3'/%3E%3Ccircle class='spinner_qM83 spinner_ZTLf' cx='20' cy='12' r='3'/%3E%3C/svg%3E")}
.loading-ring {
    -webkit-mask-image: url("data:image/svg+xml,%3Csvg width='44' height='44' viewBox='0 0 44 44' xmlns='http://www.w3.org/2000/svg' stroke='%23fff'%3E%3Cg fill='none' fill-rule='evenodd' stroke-width='2'%3E%3Ccircle cx='22' cy='22' r='1'%3E%3Canimate attributeName='r' begin='0s' dur='1.8s' values='1; 20' calcMode='spline' keyTimes='0; 1' keySplines='0.165, 0.84, 0.44, 1' repeatCount='indefinite' /%3E%3Canimate attributeName='stroke-opacity' begin='0s' dur='1.8s' values='1; 0' calcMode='spline' keyTimes='0; 1' keySplines='0.3, 0.61, 0.355, 1' repeatCount='indefinite' /%3E%3C/circle%3E%3Ccircle cx='22' cy='22' r='1'%3E%3Canimate attributeName='r' begin='-0.9s' dur='1.8s' values='1; 20' calcMode='spline' keyTimes='0; 1' keySplines='0.165, 0.84, 0.44, 1' repeatCount='indefinite' /%3E%3Canimate attributeName='stroke-opacity' begin='-0.9s' dur='1.8s' values='1; 0' calcMode='spline' keyTimes='0; 1' keySplines='0.3, 0.61, 0.355, 1' repeatCount='indefinite' /%3E%3C/circle%3E%3C/g%3E%3C/svg%3E");
            mask-image: url("data:image/svg+xml,%3Csvg width='44' height='44' viewBox='0 0 44 44' xmlns='http://www.w3.org/2000/svg' stroke='%23fff'%3E%3Cg fill='none' fill-rule='evenodd' stroke-width='2'%3E%3Ccircle cx='22' cy='22' r='1'%3E%3Canimate attributeName='r' begin='0s' dur='1.8s' values='1; 20' calcMode='spline' keyTimes='0; 1' keySplines='0.165, 0.84, 0.44, 1' repeatCount='indefinite' /%3E%3Canimate attributeName='stroke-opacity' begin='0s' dur='1.8s' values='1; 0' calcMode='spline' keyTimes='0; 1' keySplines='0.3, 0.61, 0.355, 1' repeatCount='indefinite' /%3E%3C/circle%3E%3Ccircle cx='22' cy='22' r='1'%3E%3Canimate attributeName='r' begin='-0.9s' dur='1.8s' values='1; 20' calcMode='spline' keyTimes='0; 1' keySplines='0.165, 0.84, 0.44, 1' repeatCount='indefinite' /%3E%3Canimate attributeName='stroke-opacity' begin='-0.9s' dur='1.8s' values='1; 0' calcMode='spline' keyTimes='0; 1' keySplines='0.3, 0.61, 0.355, 1' repeatCount='indefinite' /%3E%3C/circle%3E%3C/g%3E%3C/svg%3E")}
.loading-ball {
    -webkit-mask-image: url("data:image/svg+xml,%0A%3Csvg width='24' height='24' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_rXNP%7Banimation:spinner_YeBj .8s infinite%7D@keyframes spinner_YeBj%7B0%25%7Banimation-timing-function:cubic-bezier(0.33,0,.66,.33);cy:5px%7D46.875%25%7Bcy:20px;rx:4px;ry:4px%7D50%25%7Banimation-timing-function:cubic-bezier(0.33,.66,.66,1);cy:20.5px;rx:4.8px;ry:3px%7D53.125%25%7Brx:4px;ry:4px%7D100%25%7Bcy:5px%7D%7D%3C/style%3E%3Cellipse class='spinner_rXNP' cx='12' cy='5' rx='4' ry='4'/%3E%3C/svg%3E");
            mask-image: url("data:image/svg+xml,%0A%3Csvg width='24' height='24' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_rXNP%7Banimation:spinner_YeBj .8s infinite%7D@keyframes spinner_YeBj%7B0%25%7Banimation-timing-function:cubic-bezier(0.33,0,.66,.33);cy:5px%7D46.875%25%7Bcy:20px;rx:4px;ry:4px%7D50%25%7Banimation-timing-function:cubic-bezier(0.33,.66,.66,1);cy:20.5px;rx:4.8px;ry:3px%7D53.125%25%7Brx:4px;ry:4px%7D100%25%7Bcy:5px%7D%7D%3C/style%3E%3Cellipse class='spinner_rXNP' cx='12' cy='5' rx='4' ry='4'/%3E%3C/svg%3E")}
.loading-bars {
    -webkit-mask-image: url("data:image/svg+xml,%0A%3Csvg width='24' height='24' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_hzlK%7Banimation:spinner_vc4H .8s linear infinite;animation-delay:-.8s%7D.spinner_koGT%7Banimation-delay:-.65s%7D.spinner_YF1u%7Banimation-delay:-.5s%7D@keyframes spinner_vc4H%7B0%25%7By:1px;height:22px%7D93.75%25%7By:5px;height:14px;opacity:.2%7D%7D%3C/style%3E%3Crect class='spinner_hzlK' x='1' y='1' width='6' height='22'/%3E%3Crect class='spinner_hzlK spinner_koGT' x='9' y='1' width='6' height='22'/%3E%3Crect class='spinner_hzlK spinner_YF1u' x='17' y='1' width='6' height='22'/%3E%3C/svg%3E");
            mask-image: url("data:image/svg+xml,%0A%3Csvg width='24' height='24' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg'%3E%3Cstyle%3E.spinner_hzlK%7Banimation:spinner_vc4H .8s linear infinite;animation-delay:-.8s%7D.spinner_koGT%7Banimation-delay:-.65s%7D.spinner_YF1u%7Banimation-delay:-.5s%7D@keyframes spinner_vc4H%7B0%25%7By:1px;height:22px%7D93.75%25%7By:5px;height:14px;opacity:.2%7D%7D%3C/style%3E%3Crect class='spinner_hzlK' x='1' y='1' width='6' height='22'/%3E%3Crect class='spinner_hzlK spinner_koGT' x='9' y='1' width='6' height='22'/%3E%3Crect class='spinner_hzlK spinner_YF1u' x='17' y='1' width='6' height='22'/%3E%3C/svg%3E")}
.loading-infinity {
    -webkit-mask-image: url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' style='shape-rendering: auto;' width='200px' height='200px' viewBox='0 0 100 100' preserveAspectRatio='xMidYMid'%3E%3Cpath fill='none' stroke='%230a0a0a' stroke-width='10' stroke-dasharray='205.271142578125 51.317785644531256' d='M24.3 30C11.4 30 5 43.3 5 50s6.4 20 19.3 20c19.3 0 32.1-40 51.4-40 C88.6 30 95 43.3 95 50s-6.4 20-19.3 20C56.4 70 43.6 30 24.3 30z' stroke-linecap='round' style='transform:scale(0.8);transform-origin:50px 50px'%3E%3Canimate attributeName='stroke-dashoffset' repeatCount='indefinite' dur='2s' keyTimes='0;1' values='0;256.58892822265625'%3E%3C/animate%3E%3C/path%3E%3C/svg%3E");
            mask-image: url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' style='shape-rendering: auto;' width='200px' height='200px' viewBox='0 0 100 100' preserveAspectRatio='xMidYMid'%3E%3Cpath fill='none' stroke='%230a0a0a' stroke-width='10' stroke-dasharray='205.271142578125 51.317785644531256' d='M24.3 30C11.4 30 5 43.3 5 50s6.4 20 19.3 20c19.3 0 32.1-40 51.4-40 C88.6 30 95 43.3 95 50s-6.4 20-19.3 20C56.4 70 43.6 30 24.3 30z' stroke-linecap='round' style='transform:scale(0.8);transform-origin:50px 50px'%3E%3Canimate attributeName='stroke-dashoffset' repeatCount='indefinite' dur='2s' keyTimes='0;1' values='0;256.58892822265625'%3E%3C/animate%3E%3C/path%3E%3C/svg%3E")}
.loading-xs {
    width: 1rem}
.loading-sm {
    width: 1.25rem}
.loading-md {
    width: 1.5rem}
.loading-lg {
    width: 2.5rem}
.mask-squircle {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M100 0C20 0 0 20 0 100s20 100 100 100 100-20 100-100S180 0 100 0Z'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M100 0C20 0 0 20 0 100s20 100 100 100 100-20 100-100S180 0 100 0Z'/%3e%3c/svg%3e")}
.mask-decagon {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='192' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 0 58.779 19.098 36.327 50v61.804l-36.327 50L96 200l-58.779-19.098-36.327-50V69.098l36.327-50z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='192' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 0 58.779 19.098 36.327 50v61.804l-36.327 50L96 200l-58.779-19.098-36.327-50V69.098l36.327-50z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-diamond {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m100 0 100 100-100 100L0 100z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m100 0 100 100-100 100L0 100z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-heart {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='185' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M100 184.606a15.384 15.384 0 0 1-8.653-2.678C53.565 156.28 37.205 138.695 28.182 127.7 8.952 104.264-.254 80.202.005 54.146.308 24.287 24.264 0 53.406 0c21.192 0 35.869 11.937 44.416 21.879a2.884 2.884 0 0 0 4.356 0C110.725 11.927 125.402 0 146.594 0c29.142 0 53.098 24.287 53.4 54.151.26 26.061-8.956 50.122-28.176 73.554-9.023 10.994-25.383 28.58-63.165 54.228a15.384 15.384 0 0 1-8.653 2.673Z' fill='black' fill-rule='nonzero'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='185' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M100 184.606a15.384 15.384 0 0 1-8.653-2.678C53.565 156.28 37.205 138.695 28.182 127.7 8.952 104.264-.254 80.202.005 54.146.308 24.287 24.264 0 53.406 0c21.192 0 35.869 11.937 44.416 21.879a2.884 2.884 0 0 0 4.356 0C110.725 11.927 125.402 0 146.594 0c29.142 0 53.098 24.287 53.4 54.151.26 26.061-8.956 50.122-28.176 73.554-9.023 10.994-25.383 28.58-63.165 54.228a15.384 15.384 0 0 1-8.653 2.673Z' fill='black' fill-rule='nonzero'/%3e%3c/svg%3e")}
.mask-hexagon {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='182' height='201' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M.3 65.486c0-9.196 6.687-20.063 14.211-25.078l61.86-35.946c8.36-5.016 20.899-5.016 29.258 0l61.86 35.946c8.36 5.015 14.211 15.882 14.211 25.078v71.055c0 9.196-6.687 20.063-14.211 25.079l-61.86 35.945c-8.36 4.18-20.899 4.18-29.258 0L14.51 161.62C6.151 157.44.3 145.737.3 136.54V65.486Z' fill='black' fill-rule='nonzero'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='182' height='201' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M.3 65.486c0-9.196 6.687-20.063 14.211-25.078l61.86-35.946c8.36-5.016 20.899-5.016 29.258 0l61.86 35.946c8.36 5.015 14.211 15.882 14.211 25.078v71.055c0 9.196-6.687 20.063-14.211 25.079l-61.86 35.945c-8.36 4.18-20.899 4.18-29.258 0L14.51 161.62C6.151 157.44.3 145.737.3 136.54V65.486Z' fill='black' fill-rule='nonzero'/%3e%3c/svg%3e")}
.mask-hexagon-2 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='182' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M64.786 181.4c-9.196 0-20.063-6.687-25.079-14.21L3.762 105.33c-5.016-8.36-5.016-20.9 0-29.259l35.945-61.86C44.723 5.851 55.59 0 64.786 0h71.055c9.196 0 20.063 6.688 25.079 14.211l35.945 61.86c4.18 8.36 4.18 20.899 0 29.258l-35.945 61.86c-4.18 8.36-15.883 14.211-25.079 14.211H64.786Z' fill='black' fill-rule='nonzero'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='182' xmlns='http://www.w3.org/2000/svg'%3e%3cpath d='M64.786 181.4c-9.196 0-20.063-6.687-25.079-14.21L3.762 105.33c-5.016-8.36-5.016-20.9 0-29.259l35.945-61.86C44.723 5.851 55.59 0 64.786 0h71.055c9.196 0 20.063 6.688 25.079 14.211l35.945 61.86c4.18 8.36 4.18 20.899 0 29.258l-35.945 61.86c-4.18 8.36-15.883 14.211-25.079 14.211H64.786Z' fill='black' fill-rule='nonzero'/%3e%3c/svg%3e")}
.mask-circle {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3ccircle fill='black' cx='100' cy='100' r='100' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3ccircle fill='black' cx='100' cy='100' r='100' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-parallelogram {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='154' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M46.154 0H200l-46.154 153.846H0z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='154' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M46.154 0H200l-46.154 153.846H0z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-parallelogram-2 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='154' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M153.846 0H0l46.154 153.846H200z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='154' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M153.846 0H0l46.154 153.846H200z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-parallelogram-3 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='154' height='201' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M.077 47.077v153.846l153.846-46.154V.923z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='154' height='201' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M.077 47.077v153.846l153.846-46.154V.923z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-parallelogram-4 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='154' height='201' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M153.923 47.077v153.846L.077 154.77V.923z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='154' height='201' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M153.923 47.077v153.846L.077 154.77V.923z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-pentagon {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='192' height='181' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 0 95.106 69.098-36.327 111.804H37.22L.894 69.098z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='192' height='181' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 0 95.106 69.098-36.327 111.804H37.22L.894 69.098z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-square {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M0 0h200v200H0z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='200' height='200' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M0 0h200v200H0z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-star {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='192' height='180' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 137.263-58.779 42.024 22.163-68.389L.894 68.481l72.476-.243L96 0l22.63 68.238 72.476.243-58.49 42.417 22.163 68.389z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='192' height='180' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 137.263-58.779 42.024 22.163-68.389L.894 68.481l72.476-.243L96 0l22.63 68.238 72.476.243-58.49 42.417 22.163 68.389z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-star-2 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='192' height='180' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 153.044-58.779 26.243 7.02-63.513L.894 68.481l63.117-13.01L96 0l31.989 55.472 63.117 13.01-43.347 47.292 7.02 63.513z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='192' height='180' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m96 153.044-58.779 26.243 7.02-63.513L.894 68.481l63.117-13.01L96 0l31.989 55.472 63.117 13.01-43.347 47.292 7.02 63.513z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-triangle {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='174' height='149' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m87 148.476-86.603.185L43.86 74.423 87 0l43.14 74.423 43.463 74.238z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='174' height='149' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m87 148.476-86.603.185L43.86 74.423 87 0l43.14 74.423 43.463 74.238z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-triangle-2 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='174' height='150' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m87 .738 86.603-.184-43.463 74.238L87 149.214 43.86 74.792.397.554z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='174' height='150' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m87 .738 86.603-.184-43.463 74.238L87 149.214 43.86 74.792.397.554z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-triangle-3 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='150' height='174' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m149.369 87.107.185 86.603-74.239-43.463L.893 87.107l74.422-43.14L149.554.505z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='150' height='174' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='m149.369 87.107.185 86.603-74.239-43.463L.893 87.107l74.422-43.14L149.554.505z' fill-rule='evenodd'/%3e%3c/svg%3e")}
.mask-triangle-4 {
    -webkit-mask-image: url("data:image/svg+xml,%3csvg width='150' height='174' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M.631 87.107.446.505l74.239 43.462 74.422 43.14-74.422 43.14L.446 173.71z' fill-rule='evenodd'/%3e%3c/svg%3e");
            mask-image: url("data:image/svg+xml,%3csvg width='150' height='174' xmlns='http://www.w3.org/2000/svg'%3e%3cpath fill='black' d='M.631 87.107.446.505l74.239 43.462 74.422 43.14-74.422 43.14L.446 173.71z' fill-rule='evenodd'/%3e%3c/svg%3e")}
:where(.menu li:empty) {
    background-color: hsl(var(--bc) / 0.1);
    margin: 0.5rem 1rem;
    height: 1px}
.menu :where(li ul):before {
    position: absolute;
    left: 0px;
    top: 0.75rem;
    bottom: 0.75rem;
    width: 1px;
    background-color: hsl(var(--bc) / 0.1);
    content: ""}
.menu :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),
.menu :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    text-align: left;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 200ms;
    border-radius: var(--rounded-btn, 0.5rem);
    text-wrap: balance}
:where(.menu li:not(.menu-title):not(.disabled) > *:not(ul):not(details):not(.menu-title)):not(summary):not(.active).focus,
  :where(.menu li:not(.menu-title):not(.disabled) > *:not(ul):not(details):not(.menu-title)):not(summary):not(.active):focus,
  :where(.menu li:not(.menu-title):not(.disabled) > *:not(ul):not(details):not(.menu-title)):is(summary):not(.active):focus-visible,
  :where(.menu li:not(.menu-title):not(.disabled) > details > summary:not(.menu-title)):not(summary):not(.active).focus,
  :where(.menu li:not(.menu-title):not(.disabled) > details > summary:not(.menu-title)):not(summary):not(.active):focus,
  :where(.menu li:not(.menu-title):not(.disabled) > details > summary:not(.menu-title)):is(summary):not(.active):focus-visible {
    cursor: pointer;
    background-color: hsl(var(--bc) / 0.1);
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    outline: 2px solid transparent;
    outline-offset: 2px}
.menu li > *:not(ul):not(.menu-title):not(details):active,
.menu li > *:not(ul):not(.menu-title):not(details).active,
.menu li > details > summary:active {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity))}
.menu :where(li > details > summary)::-webkit-details-marker {
    display: none}
.menu :where(li > details > summary):after,
.menu :where(li > .menu-dropdown-toggle):after {
    justify-self: end;
    display: block;
    margin-top: -0.5rem;
    height: 0.5rem;
    width: 0.5rem;
    transform: rotate(45deg);
    transition-property: transform, margin-top;
    transition-duration: 0.3s;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    content: "";
    transform-origin: 75% 75%;
    box-shadow: 2px 2px;
    pointer-events: none}
.menu :where(li > details[open] > summary):after,
.menu :where(li > .menu-dropdown-toggle.menu-dropdown-show):after {
    transform: rotate(225deg);
    margin-top: 0}
.menu-title {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    font-weight: 700;
    color: hsl(var(--bc) / 0.4)}
.mockup-code:before {
    content: "";
    margin-bottom: 1rem;
    display: block;
    height: 0.75rem;
    width: 0.75rem;
    border-radius: 9999px;
    opacity: 0.3;
    box-shadow: 1.4em 0, 2.8em 0, 4.2em 0}
.mockup-code pre {
    padding-right: 1.25rem}
.mockup-code pre:before {
    content: "";
    margin-right: 2ch}
.mockup-window:before {
    content: "";
    margin-bottom: 1rem;
    display: block;
    aspect-ratio: 1 / 1;
    height: 0.75rem;
    border-radius: 9999px;
    opacity: 0.3;
    box-shadow: 1.4em 0, 2.8em 0, 4.2em 0}
.mockup-phone {
    display: inline-block;
    border: 4px solid #444;
    border-radius: 50px;
    background-color: #000;
    padding: 10px;
    margin: 0 auto;
    overflow: hidden}
.mockup-phone .camera {
    position: relative;
    top: 0px;
    left: 0px;
    background: #000;
    height: 25px;
    width: 150px;
    margin: 0 auto;
    border-bottom-left-radius: 17px;
    border-bottom-right-radius: 17px;
    z-index: 11}
.mockup-phone .camera:before {
    content: "";
    position: absolute;
    top: 35%;
    left: 50%;
    width: 50px;
    height: 4px;
    border-radius: 5px;
    background-color: #0c0b0e;
    transform: translate(-50%, -50%)}
.mockup-phone .camera:after {
    content: "";
    position: absolute;
    top: 20%;
    left: 70%;
    width: 8px;
    height: 8px;
    border-radius: 5px;
    background-color: #0f0b25}
.mockup-phone .display {
    overflow: hidden;
    border-radius: 40px;
    margin-top: -25px}
.mockup-browser .mockup-browser-toolbar {
    margin-top: 0.75rem;
    margin-bottom: 0.75rem;
    display: inline-flex;
    width: 100%;
    align-items: center;
    padding-right: 1.4em}
.mockup-browser .mockup-browser-toolbar:before {
    content: "";
    margin-right: 4.8rem;
    display: inline-block;
    aspect-ratio: 1 / 1;
    height: 0.75rem;
    border-radius: 9999px;
    opacity: 0.3;
    box-shadow: 1.4em 0, 2.8em 0, 4.2em 0}
.mockup-browser .mockup-browser-toolbar .input {
    position: relative;
    margin-left: auto;
    margin-right: auto;
    display: block;
    height: 1.75rem;
    width: 24rem;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    padding-left: 2rem}
.mockup-browser .mockup-browser-toolbar .input:before {
    content: "";
    position: absolute;
    top: 50%;
    left: 0.5rem;
    aspect-ratio: 1 / 1;
    height: 0.75rem;
    --tw-translate-y: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    border-radius: 9999px;
    border-width: 2px;
    border-color: currentColor;
    opacity: 0.6}
.mockup-browser .mockup-browser-toolbar .input:after {
    content: "";
    position: absolute;
    top: 50%;
    left: 1.25rem;
    height: 0.5rem;
    --tw-translate-y: 25%;
    --tw-rotate: -45deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    border-radius: 9999px;
    border-width: 1px;
    border-color: currentColor;
    opacity: 0.6}
.modal:not(dialog:not(.modal-open)),
  .modal::backdrop {
    background-color: rgba(0, 0, 0, 0.3);
    animation: modal-pop 0.2s ease-out}
.modal-backdrop {
    z-index: -1;
    grid-column-start: 1;
    grid-row-start: 1;
    display: grid;
    align-self: stretch;
    justify-self: stretch;
    color: transparent}
.modal-open .modal-box,
.modal-toggle:checked + .modal .modal-box,
.modal:target .modal-box,
.modal[open] .modal-box {
    --tw-translate-y: 0px;
    --tw-scale-x: 1;
    --tw-scale-y: 1;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.modal-action > :not([hidden]) ~ :not([hidden]) {
    --tw-space-x-reverse: 0;
    margin-right: calc(0.5rem * var(--tw-space-x-reverse));
    margin-left: calc(0.5rem * calc(1 - var(--tw-space-x-reverse)))}
@keyframes modal-pop {
    0% {
        opacity: 0}}
.progress::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity))}
.progress-primary::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity))}
.progress-secondary::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity))}
.progress-accent::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity))}
.progress-info::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.progress-success::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.progress-warning::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.progress-error::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.progress:indeterminate {
    --progress-color: hsl(var(--bc));
    background-image: repeating-linear-gradient(
    90deg,
    var(--progress-color) -1%,
    var(--progress-color) 10%,
    transparent 10%,
    transparent 90%
  );
    background-size: 200%;
    background-position-x: 15%;
    animation: progress-loading 5s ease-in-out infinite}
.progress-primary:indeterminate {
    --progress-color: hsl(var(--p))}
.progress-secondary:indeterminate {
    --progress-color: hsl(var(--s))}
.progress-accent:indeterminate {
    --progress-color: hsl(var(--a))}
.progress-info:indeterminate {
    --progress-color: hsl(var(--in))}
.progress-success:indeterminate {
    --progress-color: hsl(var(--su))}
.progress-warning:indeterminate {
    --progress-color: hsl(var(--wa))}
.progress-error:indeterminate {
    --progress-color: hsl(var(--er))}
.progress::-webkit-progress-bar {
    background-color: transparent;
    border-radius: var(--rounded-box, 1rem)}
.progress::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    border-radius: var(--rounded-box, 1rem)}
.progress-primary::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity))}
.progress-secondary::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity))}
.progress-accent::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity))}
.progress-info::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.progress-success::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.progress-warning::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.progress-error::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.progress:indeterminate::-moz-progress-bar {
    background-color: transparent;
    background-image: repeating-linear-gradient(
    90deg,
    var(--progress-color) -1%,
    var(--progress-color) 10%,
    transparent 10%,
    transparent 90%
  );
    background-size: 200%;
    background-position-x: 15%;
    animation: progress-loading 5s ease-in-out infinite}
@keyframes progress-loading {
    50% {
        background-position-x: -115%}}
.radio:focus-visible {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 1)}
.radio:checked,
  .radio[aria-checked="true"] {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--bc) / var(--tw-bg-opacity));
    animation: radiomark var(--animation-input, 0.2s) ease-out;
    box-shadow: 0 0 0 4px hsl(var(--b1)) inset, 0 0 0 4px hsl(var(--b1)) inset}
.radio-primary {
    --chkbg: var(--p);
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity))}
.radio-primary:focus-visible {
    outline-color: hsl(var(--p) / 1)}
.radio-primary:checked,
    .radio-primary[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.radio-secondary {
    --chkbg: var(--s);
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity))}
.radio-secondary:focus-visible {
    outline-color: hsl(var(--s) / 1)}
.radio-secondary:checked,
    .radio-secondary[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.radio-accent {
    --chkbg: var(--a);
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity))}
.radio-accent:focus-visible {
    outline-color: hsl(var(--a) / 1)}
.radio-accent:checked,
    .radio-accent[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.radio-success {
    --chkbg: var(--su);
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.radio-success:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.radio-success:checked,
    .radio-success[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.radio-warning {
    --chkbg: var(--wa);
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.radio-warning:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.radio-warning:checked,
    .radio-warning[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.radio-info {
    --chkbg: var(--in);
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.radio-info:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.radio-info:checked,
    .radio-info[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.radio-error {
    --chkbg: var(--er);
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.radio-error:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.radio-error:checked,
    .radio-error[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.radio:disabled {
    cursor: not-allowed;
    opacity: 0.2}
@keyframes radiomark {
    0% {
        box-shadow: 0 0 0 12px hsl(var(--b1)) inset, 0 0 0 12px hsl(var(--b1)) inset}
    50% {
        box-shadow: 0 0 0 3px hsl(var(--b1)) inset, 0 0 0 3px hsl(var(--b1)) inset}
    100% {
        box-shadow: 0 0 0 4px hsl(var(--b1)) inset, 0 0 0 4px hsl(var(--b1)) inset}}
.radio-mark {
    display: none}
.range:focus-visible::-webkit-slider-thumb {
    --focus-shadow: 0 0 0 6px hsl(var(--b1)) inset, 0 0 0 2rem hsl(var(--range-shdw)) inset}
.range:focus-visible::-moz-range-thumb {
    --focus-shadow: 0 0 0 6px hsl(var(--b1)) inset, 0 0 0 2rem hsl(var(--range-shdw)) inset}
.range::-webkit-slider-runnable-track {
    height: 0.5rem;
    width: 100%;
    background-color: hsl(var(--bc) / 0.1);
    border-radius: var(--rounded-box, 1rem)}
.range::-moz-range-track {
    height: 0.5rem;
    width: 100%;
    background-color: hsl(var(--bc) / 0.1);
    border-radius: var(--rounded-box, 1rem)}
.range::-webkit-slider-thumb {
    position: relative;
    height: 1.5rem;
    width: 1.5rem;
    border-style: none;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    border-radius: var(--rounded-box, 1rem);
    appearance: none;
    -webkit-appearance: none;
    top: 50%;
    color: hsl(var(--range-shdw));
    transform: translateY(-50%);
    --filler-size: 100rem;
    --filler-offset: 0.6rem;
    box-shadow: 0 0 0 3px hsl(var(--range-shdw)) inset, var(--focus-shadow, 0 0),
      calc(var(--filler-size) * -1 - var(--filler-offset)) 0 0 var(--filler-size)}
.range::-moz-range-thumb {
    position: relative;
    height: 1.5rem;
    width: 1.5rem;
    border-style: none;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    border-radius: var(--rounded-box, 1rem);
    top: 50%;
    color: hsl(var(--range-shdw));
    --filler-size: 100rem;
    --filler-offset: 0.5rem;
    box-shadow: 0 0 0 3px hsl(var(--range-shdw)) inset, var(--focus-shadow, 0 0),
      calc(var(--filler-size) * -1 - var(--filler-offset)) 0 0 var(--filler-size)}
.range-primary {
    --range-shdw: var(--p)}
.range-secondary {
    --range-shdw: var(--s)}
.range-accent {
    --range-shdw: var(--a)}
.range-success {
    --range-shdw: var(--su)}
.range-warning {
    --range-shdw: var(--wa)}
.range-info {
    --range-shdw: var(--in)}
.range-error {
    --range-shdw: var(--er)}
.rating input {
    appearance: none;
    -webkit-appearance: none}
.rating .rating-hidden {
    width: 0.5rem;
    background-color: transparent}
.rating input:checked ~ input,
  .rating input[aria-checked="true"] ~ input {
    --tw-bg-opacity: 0.2}
.rating input:focus-visible {
    transition-property: transform;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0, 0, 0.2, 1);
    transition-duration: 300ms;
    transform: translateY(-0.125em)}
.rating input:active:focus {
    animation: none;
    transform: translateY(-0.125em)}
.rating-half :where(input:not(.rating-hidden)) {
    width: 0.75rem}
@keyframes rating-pop {
    0% {
        transform: translateY(-0.125em)}
    40% {
        transform: translateY(-0.125em)}
    100% {
        transform: translateY(0)}}
.select-bordered {
    --tw-border-opacity: 0.2}
.select:focus {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 0.2)}
.select-ghost {
    --tw-bg-opacity: 0.05}
.select-ghost:focus {
    --tw-bg-opacity: 1;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity))}
.select-primary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity))}
.select-primary:focus {
    outline-color: hsl(var(--p) / 1)}
.select-secondary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity))}
.select-secondary:focus {
    outline-color: hsl(var(--s) / 1)}
.select-accent {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity))}
.select-accent:focus {
    outline-color: hsl(var(--a) / 1)}
.select-info {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.select-info:focus {
    outline-color: hsl(var(--in) / 1)}
.select-success {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.select-success:focus {
    outline-color: hsl(var(--su) / 1)}
.select-warning {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.select-warning:focus {
    outline-color: hsl(var(--wa) / 1)}
.select-error {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.select-error:focus {
    outline-color: hsl(var(--er) / 1)}
.select-disabled,
  .select:disabled,
  .select[disabled] {
    cursor: not-allowed;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    --tw-text-opacity: 0.2}
.select-disabled::placeholder,
  .select:disabled::placeholder,
  .select[disabled]::placeholder {
    color: hsl(var(--bc) / var(--tw-placeholder-opacity));
    --tw-placeholder-opacity: 0.2}
.select-multiple,
  .select[multiple],
  .select[size].select:not([size="1"]) {
    background-image: none;
    padding-right: 1rem}
[dir="rtl"] .select {
    background-position: calc(0% + 12px) calc(1px + 50%), calc(0% + 16px) calc(1px + 50%)}
:where(.stats) > :not([hidden]) ~ :not([hidden]) {
    --tw-divide-x-reverse: 0;
    border-right-width: calc(1px * var(--tw-divide-x-reverse));
    border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
    --tw-divide-y-reverse: 0;
    border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
    border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}
.steps .step:before {
    top: 0px;
    grid-column-start: 1;
    grid-row-start: 1;
    height: 0.5rem;
    width: 100%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    content: "";
    margin-left: -100%}
.steps .step:after {
    content: counter(step);
    counter-increment: step;
    z-index: 1;
    position: relative;
    grid-column-start: 1;
    grid-row-start: 1;
    display: grid;
    height: 2rem;
    width: 2rem;
    place-items: center;
    place-self: center;
    border-radius: 9999px;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity))}
.steps .step:first-child:before {
    content: none}
.steps .step[data-content]:after {
    content: attr(data-content)}
.steps .step-neutral + .step-neutral:before,
  .steps .step-neutral:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--n) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--nc) / var(--tw-text-opacity))}
.steps .step-primary + .step-primary:before,
  .steps .step-primary:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.steps .step-secondary + .step-secondary:before,
  .steps .step-secondary:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.steps .step-accent + .step-accent:before,
  .steps .step-accent:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.steps .step-info + .step-info:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.steps .step-info:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.steps .step-success + .step-success:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.steps .step-success:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.steps .step-warning + .step-warning:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.steps .step-warning:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.steps .step-error + .step-error:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.steps .step-error:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.swap-rotate .swap-on,
.swap-rotate .swap-indeterminate,
.swap-rotate input:indeterminate ~ .swap-on {
    --tw-rotate: 45deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.swap-rotate input:checked ~ .swap-off,
.swap-rotate.swap-active .swap-off,
.swap-rotate input:indeterminate ~ .swap-off {
    --tw-rotate: -45deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.swap-rotate input:checked ~ .swap-on,
.swap-rotate.swap-active .swap-on,
.swap-rotate input:indeterminate ~ .swap-indeterminate {
    --tw-rotate: 0deg;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.swap-flip {
    transform-style: preserve-3d;
    perspective: 16em}
.swap-flip .swap-on,
.swap-flip .swap-indeterminate,
.swap-flip input:indeterminate ~ .swap-on {
    transform: rotateY(180deg);
    backface-visibility: hidden;
    opacity: 1}
.swap-flip input:checked ~ .swap-off,
.swap-flip.swap-active .swap-off,
.swap-flip input:indeterminate ~ .swap-off {
    transform: rotateY(-180deg);
    backface-visibility: hidden;
    opacity: 1}
.swap-flip input:checked ~ .swap-on,
.swap-flip.swap-active .swap-on,
.swap-flip input:indeterminate ~ .swap-indeterminate {
    transform: rotateY(0deg)}
.tab.tab-active:not(.tab-disabled):not([disabled]) {
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 1;
    --tw-text-opacity: 1}
.tab:focus {
    outline: 2px solid transparent;
    outline-offset: 2px}
.tab:focus-visible {
    outline: 2px solid currentColor;
    outline-offset: -3px}
.tab:focus-visible.tab-lifted {
    border-bottom-right-radius: var(--tab-radius, 0.5rem);
    border-bottom-left-radius: var(--tab-radius, 0.5rem)}
.tab-disabled,
  .tab[disabled] {
    cursor: not-allowed;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    --tw-text-opacity: 0.2}
.tab-bordered {
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    --tw-border-opacity: 0.2;
    border-style: solid;
    border-bottom-width: calc(var(--tab-border, 1px) + 1px)}
.tab-lifted {
    border: var(--tab-border, 1px) solid transparent;
    border-width: 0 0 var(--tab-border, 1px) 0;
    border-top-left-radius: var(--tab-radius, 0.5rem);
    border-top-right-radius: var(--tab-radius, 0.5rem);
    border-bottom-color: var(--tab-border-color);
    padding-left: var(--tab-padding, 1rem);
    padding-right: var(--tab-padding, 1rem);
    padding-top: var(--tab-border, 1px)}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled]) {
    background-color: var(--tab-bg);
    border-width: var(--tab-border, 1px) var(--tab-border, 1px) 0 var(--tab-border, 1px);
    border-left-color: var(--tab-border-color);
    border-right-color: var(--tab-border-color);
    border-top-color: var(--tab-border-color);
    padding-left: calc(var(--tab-padding, 1rem) - var(--tab-border, 1px));
    padding-right: calc(var(--tab-padding, 1rem) - var(--tab-border, 1px));
    padding-bottom: var(--tab-border, 1px);
    padding-top: 0}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled]):before,
    .tab-lifted.tab-active:not(.tab-disabled):not([disabled]):after {
    z-index: 1;
    content: "";
    display: block;
    position: absolute;
    width: var(--tab-radius, 0.5rem);
    height: var(--tab-radius, 0.5rem);
    bottom: 0;
    --tab-grad: calc(68% - var(--tab-border, 1px));
    --tab-corner-bg: radial-gradient(
        circle at var(--circle-pos),
        transparent var(--tab-grad),
        var(--tab-border-color) calc(var(--tab-grad) + 0.3px),
        var(--tab-border-color) calc(var(--tab-grad) + var(--tab-border, 1px)),
        var(--tab-bg) calc(var(--tab-grad) + var(--tab-border, 1px) + 0.3px)
      )}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled]):before {
    left: calc(var(--tab-radius, 0.5rem) * -1);
    --circle-pos: top left;
    background-image: var(--tab-corner-bg)}
[dir="rtl"] .tab-lifted.tab-active:not(.tab-disabled):not([disabled]):before {
    --circle-pos: top right}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled]):after {
    right: calc(var(--tab-radius, 0.5rem) * -1);
    --circle-pos: top right;
    background-image: var(--tab-corner-bg)}
[dir="rtl"] .tab-lifted.tab-active:not(.tab-disabled):not([disabled]):after {
    --circle-pos: top left}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled]):first-child:before {
    background: none}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled]):last-child:after {
    background: none}
.tab-lifted.tab-active:not(.tab-disabled):not([disabled])
  + .tab-lifted.tab-active:not(.tab-disabled):not([disabled]):before {
    background: none}
.tabs-boxed {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    padding: 0.25rem;
    border-radius: var(--rounded-btn, 0.5rem)}
.tabs-boxed .tab {
    border-radius: var(--rounded-btn, 0.5rem)}
.tabs-boxed .tab-active:not(.tab-disabled):not([disabled]) {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.table :where(th, td) {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.75rem;
    padding-bottom: 0.75rem;
    vertical-align: middle}
.table tr.active,
  .table tr.active:nth-child(even),
  .table-zebra tbody tr:nth-child(even) {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity))}
.table-zebra tr.active,
    .table-zebra tr.active:nth-child(even),
    .table-zebra-zebra tbody tr:nth-child(even) {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity))}
.table :where(thead, tbody) :where(tr:not(:last-child)),
    .table :where(thead, tbody) :where(tr:first-child:last-child) {
    border-bottom-width: 1px;
    --tw-border-opacity: 1;
    border-bottom-color: hsl(var(--b2) / var(--tw-border-opacity))}
.table :where(thead, tfoot) {
    white-space: nowrap;
    font-size: 0.75rem;
    line-height: 1rem;
    font-weight: 700;
    color: hsl(var(--bc) / 0.6)}
.textarea-bordered {
    --tw-border-opacity: 0.2}
.textarea:focus {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 0.2)}
.textarea-ghost {
    --tw-bg-opacity: 0.05}
.textarea-ghost:focus {
    --tw-bg-opacity: 1;
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    box-shadow: none}
.textarea-primary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--p) / var(--tw-border-opacity))}
.textarea-primary:focus {
    outline-color: hsl(var(--p) / 1)}
.textarea-secondary {
    --tw-border-opacity: 1;
    border-color: hsl(var(--s) / var(--tw-border-opacity))}
.textarea-secondary:focus {
    outline-color: hsl(var(--s) / 1)}
.textarea-accent {
    --tw-border-opacity: 1;
    border-color: hsl(var(--a) / var(--tw-border-opacity))}
.textarea-accent:focus {
    outline-color: hsl(var(--a) / 1)}
.textarea-info {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.textarea-info:focus {
    outline-color: hsl(var(--in) / 1)}
.textarea-success {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.textarea-success:focus {
    outline-color: hsl(var(--su) / 1)}
.textarea-warning {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.textarea-warning:focus {
    outline-color: hsl(var(--wa) / 1)}
.textarea-error {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.textarea-error:focus {
    outline-color: hsl(var(--er) / 1)}
.textarea-disabled,
  .textarea:disabled,
  .textarea[disabled] {
    cursor: not-allowed;
    --tw-border-opacity: 1;
    border-color: hsl(var(--b2) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b2) / var(--tw-bg-opacity));
    --tw-text-opacity: 0.2}
.textarea-disabled::placeholder,
  .textarea:disabled::placeholder,
  .textarea[disabled]::placeholder {
    color: hsl(var(--bc) / var(--tw-placeholder-opacity));
    --tw-placeholder-opacity: 0.2}
.toast > * {
    animation: toast-pop 0.25s ease-out}
@keyframes toast-pop {
    0% {
        transform: scale(0.9);
        opacity: 0}
    100% {
        transform: scale(1);
        opacity: 1}}
[dir="rtl"] .toggle {
    --handleoffsetcalculator: calc(var(--handleoffset) * 1)}
.toggle:focus-visible {
    outline-style: solid;
    outline-width: 2px;
    outline-offset: 2px;
    outline-color: hsl(var(--bc) / 0.2)}
.toggle:checked,
  .toggle[checked="true"],
  .toggle[aria-checked="true"] {
    --handleoffsetcalculator: var(--handleoffset);
    --tw-border-opacity: 1;
    --tw-bg-opacity: 1}
[dir="rtl"] .toggle:checked, [dir="rtl"] .toggle[checked="true"], [dir="rtl"] .toggle[aria-checked="true"] {
    --handleoffsetcalculator: calc(var(--handleoffset) * -1)}
.toggle:indeterminate {
    --tw-border-opacity: 1;
    --tw-bg-opacity: 1;
    box-shadow: calc(var(--handleoffset) / 2) 0 0 2px var(--tglbg) inset,
      calc(var(--handleoffset) / -2) 0 0 2px var(--tglbg) inset, 0 0 0 2px var(--tglbg) inset}
[dir="rtl"] .toggle:indeterminate {
    box-shadow: calc(var(--handleoffset) / 2) 0 0 2px var(--tglbg) inset,
        calc(var(--handleoffset) / -2) 0 0 2px var(--tglbg) inset, 0 0 0 2px var(--tglbg) inset}
.toggle-primary:focus-visible {
    outline-color: hsl(var(--p) / 1)}
.toggle-primary:checked,
    .toggle-primary[checked="true"],
    .toggle-primary[aria-checked="true"] {
    border-color: hsl(var(--p) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--p) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--pc) / var(--tw-text-opacity))}
.toggle-secondary:focus-visible {
    outline-color: hsl(var(--s) / 1)}
.toggle-secondary:checked,
    .toggle-secondary[checked="true"],
    .toggle-secondary[aria-checked="true"] {
    border-color: hsl(var(--s) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--s) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--sc) / var(--tw-text-opacity))}
.toggle-accent:focus-visible {
    outline-color: hsl(var(--a) / 1)}
.toggle-accent:checked,
    .toggle-accent[checked="true"],
    .toggle-accent[aria-checked="true"] {
    border-color: hsl(var(--a) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--a) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--ac) / var(--tw-text-opacity))}
.toggle-success:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.toggle-success:checked,
    .toggle-success[checked="true"],
    .toggle-success[aria-checked="true"] {
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.toggle-warning:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.toggle-warning:checked,
    .toggle-warning[checked="true"],
    .toggle-warning[aria-checked="true"] {
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.toggle-info:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.toggle-info:checked,
    .toggle-info[checked="true"],
    .toggle-info[aria-checked="true"] {
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.toggle-error:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.toggle-error:checked,
    .toggle-error[checked="true"],
    .toggle-error[aria-checked="true"] {
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.toggle:disabled {
    cursor: not-allowed;
    --tw-border-opacity: 1;
    border-color: hsl(var(--bc) / var(--tw-border-opacity));
    background-color: transparent;
    opacity: 0.3;
    --togglehandleborder: 0 0 0 3px hsl(var(--bc)) inset,
      var(--handleoffsetcalculator) 0 0 3px hsl(var(--bc)) inset}
.toggle-mark {
    display: none}
.tooltip.tooltip-primary {
    --tooltip-color: hsl(var(--p));
    --tooltip-text-color: hsl(var(--pc))}
.tooltip.tooltip-secondary {
    --tooltip-color: hsl(var(--s));
    --tooltip-text-color: hsl(var(--sc))}
.tooltip.tooltip-accent {
    --tooltip-color: hsl(var(--a));
    --tooltip-text-color: hsl(var(--ac))}
.tooltip.tooltip-info {
    --tooltip-color: hsl(var(--in));
    --tooltip-text-color: hsl(var(--inc))}
.tooltip.tooltip-success {
    --tooltip-color: hsl(var(--su));
    --tooltip-text-color: hsl(var(--suc))}
.tooltip.tooltip-warning {
    --tooltip-color: hsl(var(--wa));
    --tooltip-text-color: hsl(var(--wac))}
.tooltip.tooltip-error {
    --tooltip-color: hsl(var(--er));
    --tooltip-text-color: hsl(var(--erc))}
:root .prose {
    --tw-prose-body: hsl(var(--bc) / 0.8);
    --tw-prose-headings: hsl(var(--bc));
    --tw-prose-lead: hsl(var(--bc));
    --tw-prose-links: hsl(var(--bc));
    --tw-prose-bold: hsl(var(--bc));
    --tw-prose-counters: hsl(var(--bc));
    --tw-prose-bullets: hsl(var(--bc) / 0.5);
    --tw-prose-hr: hsl(var(--bc) / 0.2);
    --tw-prose-quotes: hsl(var(--bc));
    --tw-prose-quote-borders: hsl(var(--bc) / 0.2);
    --tw-prose-captions: hsl(var(--bc) / 0.5);
    --tw-prose-code: hsl(var(--bc));
    --tw-prose-pre-code: hsl(var(--nc));
    --tw-prose-pre-bg: hsl(var(--n));
    --tw-prose-th-borders: hsl(var(--bc) / 0.5);
    --tw-prose-td-borders: hsl(var(--bc) / 0.2)}
.prose :where(code):not(:where([class~="not-prose"] *)) {
    padding: 2px 8px;
    border-radius: var(--rounded-badge)}
.prose code:after,
  .prose code:before {
    content: none}
.prose pre code {
    border-radius: 0;
    padding: 0}
.prose :where(tbody tr, thead):not(:where([class~="not-prose"] *)) {
    border-bottom-color: hsl(var(--bc) / 20%)}
@keyframes bounce {
    0%, 100% {
        transform: translateY(-25%);
        animation-timing-function: cubic-bezier(0.8,0,1,1)}
    50% {
        transform: none;
        animation-timing-function: cubic-bezier(0,0,0.2,1)}}
.animate-bounce {
    animation: bounce 1s infinite}
.animate-none {
    animation: none}
@keyframes ping {
    75%, 100% {
        transform: scale(2);
        opacity: 0}}
.animate-ping {
    animation: ping 1s cubic-bezier(0, 0, 0.2, 1) infinite}
@keyframes pulse {
    50% {
        opacity: .5}}
.animate-pulse {
    animation: pulse 2s cubic-bezier(0.4, 0, 0.6, 1) infinite}
@keyframes spin {
    to {
        transform: rotate(360deg)}}
.animate-spin {
    animation: spin 1s linear infinite}
.divide-accent > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 1)}
.divide-accent-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 1)}
.divide-accent-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0)}
.divide-accent-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.1)}
.divide-accent-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 1)}
.divide-accent-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.2)}
.divide-accent-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.25)}
.divide-accent-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.3)}
.divide-accent-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.4)}
.divide-accent-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.05)}
.divide-accent-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.5)}
.divide-accent-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.6)}
.divide-accent-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.7)}
.divide-accent-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.75)}
.divide-accent-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.8)}
.divide-accent-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.9)}
.divide-accent-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--ac) / 0.95)}
.divide-accent-focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 1)}
.divide-accent-focus\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0)}
.divide-accent-focus\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.1)}
.divide-accent-focus\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 1)}
.divide-accent-focus\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.2)}
.divide-accent-focus\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.25)}
.divide-accent-focus\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.3)}
.divide-accent-focus\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.4)}
.divide-accent-focus\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.05)}
.divide-accent-focus\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.5)}
.divide-accent-focus\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.6)}
.divide-accent-focus\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.7)}
.divide-accent-focus\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.75)}
.divide-accent-focus\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.8)}
.divide-accent-focus\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.9)}
.divide-accent-focus\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--af) / 0.95)}
.divide-accent\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0)}
.divide-accent\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.1)}
.divide-accent\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 1)}
.divide-accent\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.2)}
.divide-accent\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.25)}
.divide-accent\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.3)}
.divide-accent\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.4)}
.divide-accent\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.05)}
.divide-accent\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.5)}
.divide-accent\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.6)}
.divide-accent\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.7)}
.divide-accent\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.75)}
.divide-accent\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.8)}
.divide-accent\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.9)}
.divide-accent\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--a) / 0.95)}
.divide-base-100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 1)}
.divide-base-100\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0)}
.divide-base-100\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.1)}
.divide-base-100\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 1)}
.divide-base-100\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.2)}
.divide-base-100\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.25)}
.divide-base-100\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.3)}
.divide-base-100\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.4)}
.divide-base-100\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.05)}
.divide-base-100\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.5)}
.divide-base-100\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.6)}
.divide-base-100\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.7)}
.divide-base-100\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.75)}
.divide-base-100\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.8)}
.divide-base-100\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.9)}
.divide-base-100\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.95)}
.divide-base-200 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 1)}
.divide-base-200\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0)}
.divide-base-200\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.1)}
.divide-base-200\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 1)}
.divide-base-200\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.2)}
.divide-base-200\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.25)}
.divide-base-200\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.3)}
.divide-base-200\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.4)}
.divide-base-200\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.05)}
.divide-base-200\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.5)}
.divide-base-200\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.6)}
.divide-base-200\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.7)}
.divide-base-200\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.75)}
.divide-base-200\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.8)}
.divide-base-200\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.9)}
.divide-base-200\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.95)}
.divide-base-300 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 1)}
.divide-base-300\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0)}
.divide-base-300\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.1)}
.divide-base-300\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 1)}
.divide-base-300\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.2)}
.divide-base-300\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.25)}
.divide-base-300\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.3)}
.divide-base-300\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.4)}
.divide-base-300\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.05)}
.divide-base-300\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.5)}
.divide-base-300\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.6)}
.divide-base-300\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.7)}
.divide-base-300\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.75)}
.divide-base-300\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.8)}
.divide-base-300\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.9)}
.divide-base-300\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.95)}
.divide-base-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 1)}
.divide-base-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0)}
.divide-base-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.1)}
.divide-base-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 1)}
.divide-base-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.2)}
.divide-base-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.25)}
.divide-base-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.3)}
.divide-base-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.4)}
.divide-base-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.05)}
.divide-base-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.5)}
.divide-base-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.6)}
.divide-base-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.7)}
.divide-base-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.75)}
.divide-base-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.8)}
.divide-base-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.9)}
.divide-base-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.95)}
.divide-current > :not([hidden]) ~ :not([hidden]) {
    border-color: currentColor}
.divide-error > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 1)}
.divide-error-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 1)}
.divide-error-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0)}
.divide-error-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.1)}
.divide-error-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 1)}
.divide-error-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.2)}
.divide-error-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.25)}
.divide-error-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.3)}
.divide-error-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.4)}
.divide-error-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.05)}
.divide-error-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.5)}
.divide-error-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.6)}
.divide-error-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.7)}
.divide-error-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.75)}
.divide-error-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.8)}
.divide-error-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.9)}
.divide-error-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.95)}
.divide-error\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0)}
.divide-error\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.1)}
.divide-error\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 1)}
.divide-error\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.2)}
.divide-error\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.25)}
.divide-error\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.3)}
.divide-error\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.4)}
.divide-error\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.05)}
.divide-error\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.5)}
.divide-error\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.6)}
.divide-error\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.7)}
.divide-error\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.75)}
.divide-error\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.8)}
.divide-error\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.9)}
.divide-error\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.95)}
.divide-info > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 1)}
.divide-info-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 1)}
.divide-info-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0)}
.divide-info-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.1)}
.divide-info-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 1)}
.divide-info-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.2)}
.divide-info-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.25)}
.divide-info-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.3)}
.divide-info-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.4)}
.divide-info-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.05)}
.divide-info-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.5)}
.divide-info-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.6)}
.divide-info-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.7)}
.divide-info-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.75)}
.divide-info-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.8)}
.divide-info-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.9)}
.divide-info-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.95)}
.divide-info\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0)}
.divide-info\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.1)}
.divide-info\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 1)}
.divide-info\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.2)}
.divide-info\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.25)}
.divide-info\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.3)}
.divide-info\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.4)}
.divide-info\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.05)}
.divide-info\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.5)}
.divide-info\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.6)}
.divide-info\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.7)}
.divide-info\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.75)}
.divide-info\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.8)}
.divide-info\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.9)}
.divide-info\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.95)}
.divide-neutral > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 1)}
.divide-neutral-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 1)}
.divide-neutral-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0)}
.divide-neutral-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.1)}
.divide-neutral-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 1)}
.divide-neutral-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.2)}
.divide-neutral-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.25)}
.divide-neutral-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.3)}
.divide-neutral-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.4)}
.divide-neutral-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.05)}
.divide-neutral-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.5)}
.divide-neutral-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.6)}
.divide-neutral-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.7)}
.divide-neutral-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.75)}
.divide-neutral-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.8)}
.divide-neutral-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.9)}
.divide-neutral-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nc) / 0.95)}
.divide-neutral-focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 1)}
.divide-neutral-focus\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0)}
.divide-neutral-focus\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.1)}
.divide-neutral-focus\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 1)}
.divide-neutral-focus\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.2)}
.divide-neutral-focus\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.25)}
.divide-neutral-focus\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.3)}
.divide-neutral-focus\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.4)}
.divide-neutral-focus\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.05)}
.divide-neutral-focus\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.5)}
.divide-neutral-focus\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.6)}
.divide-neutral-focus\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.7)}
.divide-neutral-focus\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.75)}
.divide-neutral-focus\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.8)}
.divide-neutral-focus\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.9)}
.divide-neutral-focus\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--nf) / 0.95)}
.divide-neutral\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0)}
.divide-neutral\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.1)}
.divide-neutral\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 1)}
.divide-neutral\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.2)}
.divide-neutral\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.25)}
.divide-neutral\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.3)}
.divide-neutral\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.4)}
.divide-neutral\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.05)}
.divide-neutral\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.5)}
.divide-neutral\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.6)}
.divide-neutral\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.7)}
.divide-neutral\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.75)}
.divide-neutral\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.8)}
.divide-neutral\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.9)}
.divide-neutral\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--n) / 0.95)}
.divide-primary > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 1)}
.divide-primary-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 1)}
.divide-primary-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0)}
.divide-primary-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.1)}
.divide-primary-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 1)}
.divide-primary-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.2)}
.divide-primary-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.25)}
.divide-primary-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.3)}
.divide-primary-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.4)}
.divide-primary-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.05)}
.divide-primary-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.5)}
.divide-primary-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.6)}
.divide-primary-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.7)}
.divide-primary-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.75)}
.divide-primary-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.8)}
.divide-primary-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.9)}
.divide-primary-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pc) / 0.95)}
.divide-primary-focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 1)}
.divide-primary-focus\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0)}
.divide-primary-focus\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.1)}
.divide-primary-focus\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 1)}
.divide-primary-focus\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.2)}
.divide-primary-focus\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.25)}
.divide-primary-focus\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.3)}
.divide-primary-focus\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.4)}
.divide-primary-focus\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.05)}
.divide-primary-focus\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.5)}
.divide-primary-focus\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.6)}
.divide-primary-focus\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.7)}
.divide-primary-focus\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.75)}
.divide-primary-focus\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.8)}
.divide-primary-focus\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.9)}
.divide-primary-focus\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--pf) / 0.95)}
.divide-primary\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0)}
.divide-primary\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.1)}
.divide-primary\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 1)}
.divide-primary\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.2)}
.divide-primary\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.25)}
.divide-primary\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.3)}
.divide-primary\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.4)}
.divide-primary\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.05)}
.divide-primary\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.5)}
.divide-primary\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.6)}
.divide-primary\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.7)}
.divide-primary\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.75)}
.divide-primary\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.8)}
.divide-primary\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.9)}
.divide-primary\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--p) / 0.95)}
.divide-secondary > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 1)}
.divide-secondary-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 1)}
.divide-secondary-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0)}
.divide-secondary-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.1)}
.divide-secondary-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 1)}
.divide-secondary-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.2)}
.divide-secondary-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.25)}
.divide-secondary-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.3)}
.divide-secondary-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.4)}
.divide-secondary-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.05)}
.divide-secondary-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.5)}
.divide-secondary-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.6)}
.divide-secondary-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.7)}
.divide-secondary-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.75)}
.divide-secondary-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.8)}
.divide-secondary-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.9)}
.divide-secondary-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sc) / 0.95)}
.divide-secondary-focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 1)}
.divide-secondary-focus\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0)}
.divide-secondary-focus\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.1)}
.divide-secondary-focus\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 1)}
.divide-secondary-focus\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.2)}
.divide-secondary-focus\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.25)}
.divide-secondary-focus\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.3)}
.divide-secondary-focus\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.4)}
.divide-secondary-focus\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.05)}
.divide-secondary-focus\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.5)}
.divide-secondary-focus\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.6)}
.divide-secondary-focus\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.7)}
.divide-secondary-focus\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.75)}
.divide-secondary-focus\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.8)}
.divide-secondary-focus\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.9)}
.divide-secondary-focus\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--sf) / 0.95)}
.divide-secondary\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0)}
.divide-secondary\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.1)}
.divide-secondary\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 1)}
.divide-secondary\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.2)}
.divide-secondary\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.25)}
.divide-secondary\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.3)}
.divide-secondary\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.4)}
.divide-secondary\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.05)}
.divide-secondary\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.5)}
.divide-secondary\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.6)}
.divide-secondary\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.7)}
.divide-secondary\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.75)}
.divide-secondary\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.8)}
.divide-secondary\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.9)}
.divide-secondary\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--s) / 0.95)}
.divide-success > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 1)}
.divide-success-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 1)}
.divide-success-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0)}
.divide-success-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.1)}
.divide-success-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 1)}
.divide-success-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.2)}
.divide-success-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.25)}
.divide-success-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.3)}
.divide-success-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.4)}
.divide-success-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.05)}
.divide-success-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.5)}
.divide-success-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.6)}
.divide-success-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.7)}
.divide-success-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.75)}
.divide-success-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.8)}
.divide-success-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.9)}
.divide-success-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.95)}
.divide-success\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0)}
.divide-success\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.1)}
.divide-success\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 1)}
.divide-success\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.2)}
.divide-success\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.25)}
.divide-success\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.3)}
.divide-success\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.4)}
.divide-success\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.05)}
.divide-success\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.5)}
.divide-success\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.6)}
.divide-success\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.7)}
.divide-success\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.75)}
.divide-success\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.8)}
.divide-success\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.9)}
.divide-success\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.95)}
.divide-transparent > :not([hidden]) ~ :not([hidden]) {
    border-color: transparent}
.divide-transparent\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0)}
.divide-transparent\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.1)}
.divide-transparent\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 1)}
.divide-transparent\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.2)}
.divide-transparent\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.25)}
.divide-transparent\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.3)}
.divide-transparent\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.4)}
.divide-transparent\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.05)}
.divide-transparent\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.5)}
.divide-transparent\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.6)}
.divide-transparent\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.7)}
.divide-transparent\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.75)}
.divide-transparent\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.8)}
.divide-transparent\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.9)}
.divide-transparent\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: rgb(0 0 0 / 0.95)}
.divide-warning > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 1)}
.divide-warning-content > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 1)}
.divide-warning-content\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0)}
.divide-warning-content\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.1)}
.divide-warning-content\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 1)}
.divide-warning-content\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.2)}
.divide-warning-content\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.25)}
.divide-warning-content\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.3)}
.divide-warning-content\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.4)}
.divide-warning-content\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.05)}
.divide-warning-content\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.5)}
.divide-warning-content\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.6)}
.divide-warning-content\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.7)}
.divide-warning-content\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.75)}
.divide-warning-content\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.8)}
.divide-warning-content\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.9)}
.divide-warning-content\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.95)}
.divide-warning\/0 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0)}
.divide-warning\/10 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.1)}
.divide-warning\/100 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 1)}
.divide-warning\/20 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.2)}
.divide-warning\/25 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.25)}
.divide-warning\/30 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.3)}
.divide-warning\/40 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.4)}
.divide-warning\/5 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.05)}
.divide-warning\/50 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.5)}
.divide-warning\/60 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.6)}
.divide-warning\/70 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.7)}
.divide-warning\/75 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.75)}
.divide-warning\/80 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.8)}
.divide-warning\/90 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.9)}
.divide-warning\/95 > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.95)}
.border-accent {
    border-color: hsl(var(--a) / 1)}
.border-accent-content {
    border-color: hsl(var(--ac) / 1)}
.border-accent-content\/0 {
    border-color: hsl(var(--ac) / 0)}
.border-accent-content\/10 {
    border-color: hsl(var(--ac) / 0.1)}
.border-accent-content\/100 {
    border-color: hsl(var(--ac) / 1)}
.border-accent-content\/20 {
    border-color: hsl(var(--ac) / 0.2)}
.border-accent-content\/25 {
    border-color: hsl(var(--ac) / 0.25)}
.border-accent-content\/30 {
    border-color: hsl(var(--ac) / 0.3)}
.border-accent-content\/40 {
    border-color: hsl(var(--ac) / 0.4)}
.border-accent-content\/5 {
    border-color: hsl(var(--ac) / 0.05)}
.border-accent-content\/50 {
    border-color: hsl(var(--ac) / 0.5)}
.border-accent-content\/60 {
    border-color: hsl(var(--ac) / 0.6)}
.border-accent-content\/70 {
    border-color: hsl(var(--ac) / 0.7)}
.border-accent-content\/75 {
    border-color: hsl(var(--ac) / 0.75)}
.border-accent-content\/80 {
    border-color: hsl(var(--ac) / 0.8)}
.border-accent-content\/90 {
    border-color: hsl(var(--ac) / 0.9)}
.border-accent-content\/95 {
    border-color: hsl(var(--ac) / 0.95)}
.border-accent-focus {
    border-color: hsl(var(--af) / 1)}
.border-accent-focus\/0 {
    border-color: hsl(var(--af) / 0)}
.border-accent-focus\/10 {
    border-color: hsl(var(--af) / 0.1)}
.border-accent-focus\/100 {
    border-color: hsl(var(--af) / 1)}
.border-accent-focus\/20 {
    border-color: hsl(var(--af) / 0.2)}
.border-accent-focus\/25 {
    border-color: hsl(var(--af) / 0.25)}
.border-accent-focus\/30 {
    border-color: hsl(var(--af) / 0.3)}
.border-accent-focus\/40 {
    border-color: hsl(var(--af) / 0.4)}
.border-accent-focus\/5 {
    border-color: hsl(var(--af) / 0.05)}
.border-accent-focus\/50 {
    border-color: hsl(var(--af) / 0.5)}
.border-accent-focus\/60 {
    border-color: hsl(var(--af) / 0.6)}
.border-accent-focus\/70 {
    border-color: hsl(var(--af) / 0.7)}
.border-accent-focus\/75 {
    border-color: hsl(var(--af) / 0.75)}
.border-accent-focus\/80 {
    border-color: hsl(var(--af) / 0.8)}
.border-accent-focus\/90 {
    border-color: hsl(var(--af) / 0.9)}
.border-accent-focus\/95 {
    border-color: hsl(var(--af) / 0.95)}
.border-accent\/0 {
    border-color: hsl(var(--a) / 0)}
.border-accent\/10 {
    border-color: hsl(var(--a) / 0.1)}
.border-accent\/100 {
    border-color: hsl(var(--a) / 1)}
.border-accent\/20 {
    border-color: hsl(var(--a) / 0.2)}
.border-accent\/25 {
    border-color: hsl(var(--a) / 0.25)}
.border-accent\/30 {
    border-color: hsl(var(--a) / 0.3)}
.border-accent\/40 {
    border-color: hsl(var(--a) / 0.4)}
.border-accent\/5 {
    border-color: hsl(var(--a) / 0.05)}
.border-accent\/50 {
    border-color: hsl(var(--a) / 0.5)}
.border-accent\/60 {
    border-color: hsl(var(--a) / 0.6)}
.border-accent\/70 {
    border-color: hsl(var(--a) / 0.7)}
.border-accent\/75 {
    border-color: hsl(var(--a) / 0.75)}
.border-accent\/80 {
    border-color: hsl(var(--a) / 0.8)}
.border-accent\/90 {
    border-color: hsl(var(--a) / 0.9)}
.border-accent\/95 {
    border-color: hsl(var(--a) / 0.95)}
.border-base-100 {
    border-color: hsl(var(--b1) / 1)}
.border-base-100\/0 {
    border-color: hsl(var(--b1) / 0)}
.border-base-100\/10 {
    border-color: hsl(var(--b1) / 0.1)}
.border-base-100\/100 {
    border-color: hsl(var(--b1) / 1)}
.border-base-100\/20 {
    border-color: hsl(var(--b1) / 0.2)}
.border-base-100\/25 {
    border-color: hsl(var(--b1) / 0.25)}
.border-base-100\/30 {
    border-color: hsl(var(--b1) / 0.3)}
.border-base-100\/40 {
    border-color: hsl(var(--b1) / 0.4)}
.border-base-100\/5 {
    border-color: hsl(var(--b1) / 0.05)}
.border-base-100\/50 {
    border-color: hsl(var(--b1) / 0.5)}
.border-base-100\/60 {
    border-color: hsl(var(--b1) / 0.6)}
.border-base-100\/70 {
    border-color: hsl(var(--b1) / 0.7)}
.border-base-100\/75 {
    border-color: hsl(var(--b1) / 0.75)}
.border-base-100\/80 {
    border-color: hsl(var(--b1) / 0.8)}
.border-base-100\/90 {
    border-color: hsl(var(--b1) / 0.9)}
.border-base-100\/95 {
    border-color: hsl(var(--b1) / 0.95)}
.border-base-200 {
    border-color: hsl(var(--b2) / 1)}
.border-base-200\/0 {
    border-color: hsl(var(--b2) / 0)}
.border-base-200\/10 {
    border-color: hsl(var(--b2) / 0.1)}
.border-base-200\/100 {
    border-color: hsl(var(--b2) / 1)}
.border-base-200\/20 {
    border-color: hsl(var(--b2) / 0.2)}
.border-base-200\/25 {
    border-color: hsl(var(--b2) / 0.25)}
.border-base-200\/30 {
    border-color: hsl(var(--b2) / 0.3)}
.border-base-200\/40 {
    border-color: hsl(var(--b2) / 0.4)}
.border-base-200\/5 {
    border-color: hsl(var(--b2) / 0.05)}
.border-base-200\/50 {
    border-color: hsl(var(--b2) / 0.5)}
.border-base-200\/60 {
    border-color: hsl(var(--b2) / 0.6)}
.border-base-200\/70 {
    border-color: hsl(var(--b2) / 0.7)}
.border-base-200\/75 {
    border-color: hsl(var(--b2) / 0.75)}
.border-base-200\/80 {
    border-color: hsl(var(--b2) / 0.8)}
.border-base-200\/90 {
    border-color: hsl(var(--b2) / 0.9)}
.border-base-200\/95 {
    border-color: hsl(var(--b2) / 0.95)}
.border-base-300 {
    border-color: hsl(var(--b3) / 1)}
.border-base-300\/0 {
    border-color: hsl(var(--b3) / 0)}
.border-base-300\/10 {
    border-color: hsl(var(--b3) / 0.1)}
.border-base-300\/100 {
    border-color: hsl(var(--b3) / 1)}
.border-base-300\/20 {
    border-color: hsl(var(--b3) / 0.2)}
.border-base-300\/25 {
    border-color: hsl(var(--b3) / 0.25)}
.border-base-300\/30 {
    border-color: hsl(var(--b3) / 0.3)}
.border-base-300\/40 {
    border-color: hsl(var(--b3) / 0.4)}
.border-base-300\/5 {
    border-color: hsl(var(--b3) / 0.05)}
.border-base-300\/50 {
    border-color: hsl(var(--b3) / 0.5)}
.border-base-300\/60 {
    border-color: hsl(var(--b3) / 0.6)}
.border-base-300\/70 {
    border-color: hsl(var(--b3) / 0.7)}
.border-base-300\/75 {
    border-color: hsl(var(--b3) / 0.75)}
.border-base-300\/80 {
    border-color: hsl(var(--b3) / 0.8)}
.border-base-300\/90 {
    border-color: hsl(var(--b3) / 0.9)}
.border-base-300\/95 {
    border-color: hsl(var(--b3) / 0.95)}
.border-base-content {
    border-color: hsl(var(--bc) / 1)}
.border-base-content\/0 {
    border-color: hsl(var(--bc) / 0)}
.border-base-content\/10 {
    border-color: hsl(var(--bc) / 0.1)}
.border-base-content\/100 {
    border-color: hsl(var(--bc) / 1)}
.border-base-content\/20 {
    border-color: hsl(var(--bc) / 0.2)}
.border-base-content\/25 {
    border-color: hsl(var(--bc) / 0.25)}
.border-base-content\/30 {
    border-color: hsl(var(--bc) / 0.3)}
.border-base-content\/40 {
    border-color: hsl(var(--bc) / 0.4)}
.border-base-content\/5 {
    border-color: hsl(var(--bc) / 0.05)}
.border-base-content\/50 {
    border-color: hsl(var(--bc) / 0.5)}
.border-base-content\/60 {
    border-color: hsl(var(--bc) / 0.6)}
.border-base-content\/70 {
    border-color: hsl(var(--bc) / 0.7)}
.border-base-content\/75 {
    border-color: hsl(var(--bc) / 0.75)}
.border-base-content\/80 {
    border-color: hsl(var(--bc) / 0.8)}
.border-base-content\/90 {
    border-color: hsl(var(--bc) / 0.9)}
.border-base-content\/95 {
    border-color: hsl(var(--bc) / 0.95)}
.border-current {
    border-color: currentColor}
.border-error {
    border-color: hsl(var(--er) / 1)}
.border-error-content {
    border-color: hsl(var(--erc) / 1)}
.border-error-content\/0 {
    border-color: hsl(var(--erc) / 0)}
.border-error-content\/10 {
    border-color: hsl(var(--erc) / 0.1)}
.border-error-content\/100 {
    border-color: hsl(var(--erc) / 1)}
.border-error-content\/20 {
    border-color: hsl(var(--erc) / 0.2)}
.border-error-content\/25 {
    border-color: hsl(var(--erc) / 0.25)}
.border-error-content\/30 {
    border-color: hsl(var(--erc) / 0.3)}
.border-error-content\/40 {
    border-color: hsl(var(--erc) / 0.4)}
.border-error-content\/5 {
    border-color: hsl(var(--erc) / 0.05)}
.border-error-content\/50 {
    border-color: hsl(var(--erc) / 0.5)}
.border-error-content\/60 {
    border-color: hsl(var(--erc) / 0.6)}
.border-error-content\/70 {
    border-color: hsl(var(--erc) / 0.7)}
.border-error-content\/75 {
    border-color: hsl(var(--erc) / 0.75)}
.border-error-content\/80 {
    border-color: hsl(var(--erc) / 0.8)}
.border-error-content\/90 {
    border-color: hsl(var(--erc) / 0.9)}
.border-error-content\/95 {
    border-color: hsl(var(--erc) / 0.95)}
.border-error\/0 {
    border-color: hsl(var(--er) / 0)}
.border-error\/10 {
    border-color: hsl(var(--er) / 0.1)}
.border-error\/100 {
    border-color: hsl(var(--er) / 1)}
.border-error\/20 {
    border-color: hsl(var(--er) / 0.2)}
.border-error\/25 {
    border-color: hsl(var(--er) / 0.25)}
.border-error\/30 {
    border-color: hsl(var(--er) / 0.3)}
.border-error\/40 {
    border-color: hsl(var(--er) / 0.4)}
.border-error\/5 {
    border-color: hsl(var(--er) / 0.05)}
.border-error\/50 {
    border-color: hsl(var(--er) / 0.5)}
.border-error\/60 {
    border-color: hsl(var(--er) / 0.6)}
.border-error\/70 {
    border-color: hsl(var(--er) / 0.7)}
.border-error\/75 {
    border-color: hsl(var(--er) / 0.75)}
.border-error\/80 {
    border-color: hsl(var(--er) / 0.8)}
.border-error\/90 {
    border-color: hsl(var(--er) / 0.9)}
.border-error\/95 {
    border-color: hsl(var(--er) / 0.95)}
.border-info {
    border-color: hsl(var(--in) / 1)}
.border-info-content {
    border-color: hsl(var(--inc) / 1)}
.border-info-content\/0 {
    border-color: hsl(var(--inc) / 0)}
.border-info-content\/10 {
    border-color: hsl(var(--inc) / 0.1)}
.border-info-content\/100 {
    border-color: hsl(var(--inc) / 1)}
.border-info-content\/20 {
    border-color: hsl(var(--inc) / 0.2)}
.border-info-content\/25 {
    border-color: hsl(var(--inc) / 0.25)}
.border-info-content\/30 {
    border-color: hsl(var(--inc) / 0.3)}
.border-info-content\/40 {
    border-color: hsl(var(--inc) / 0.4)}
.border-info-content\/5 {
    border-color: hsl(var(--inc) / 0.05)}
.border-info-content\/50 {
    border-color: hsl(var(--inc) / 0.5)}
.border-info-content\/60 {
    border-color: hsl(var(--inc) / 0.6)}
.border-info-content\/70 {
    border-color: hsl(var(--inc) / 0.7)}
.border-info-content\/75 {
    border-color: hsl(var(--inc) / 0.75)}
.border-info-content\/80 {
    border-color: hsl(var(--inc) / 0.8)}
.border-info-content\/90 {
    border-color: hsl(var(--inc) / 0.9)}
.border-info-content\/95 {
    border-color: hsl(var(--inc) / 0.95)}
.border-info\/0 {
    border-color: hsl(var(--in) / 0)}
.border-info\/10 {
    border-color: hsl(var(--in) / 0.1)}
.border-info\/100 {
    border-color: hsl(var(--in) / 1)}
.border-info\/20 {
    border-color: hsl(var(--in) / 0.2)}
.border-info\/25 {
    border-color: hsl(var(--in) / 0.25)}
.border-info\/30 {
    border-color: hsl(var(--in) / 0.3)}
.border-info\/40 {
    border-color: hsl(var(--in) / 0.4)}
.border-info\/5 {
    border-color: hsl(var(--in) / 0.05)}
.border-info\/50 {
    border-color: hsl(var(--in) / 0.5)}
.border-info\/60 {
    border-color: hsl(var(--in) / 0.6)}
.border-info\/70 {
    border-color: hsl(var(--in) / 0.7)}
.border-info\/75 {
    border-color: hsl(var(--in) / 0.75)}
.border-info\/80 {
    border-color: hsl(var(--in) / 0.8)}
.border-info\/90 {
    border-color: hsl(var(--in) / 0.9)}
.border-info\/95 {
    border-color: hsl(var(--in) / 0.95)}
.border-neutral {
    border-color: hsl(var(--n) / 1)}
.border-neutral-content {
    border-color: hsl(var(--nc) / 1)}
.border-neutral-content\/0 {
    border-color: hsl(var(--nc) / 0)}
.border-neutral-content\/10 {
    border-color: hsl(var(--nc) / 0.1)}
.border-neutral-content\/100 {
    border-color: hsl(var(--nc) / 1)}
.border-neutral-content\/20 {
    border-color: hsl(var(--nc) / 0.2)}
.border-neutral-content\/25 {
    border-color: hsl(var(--nc) / 0.25)}
.border-neutral-content\/30 {
    border-color: hsl(var(--nc) / 0.3)}
.border-neutral-content\/40 {
    border-color: hsl(var(--nc) / 0.4)}
.border-neutral-content\/5 {
    border-color: hsl(var(--nc) / 0.05)}
.border-neutral-content\/50 {
    border-color: hsl(var(--nc) / 0.5)}
.border-neutral-content\/60 {
    border-color: hsl(var(--nc) / 0.6)}
.border-neutral-content\/70 {
    border-color: hsl(var(--nc) / 0.7)}
.border-neutral-content\/75 {
    border-color: hsl(var(--nc) / 0.75)}
.border-neutral-content\/80 {
    border-color: hsl(var(--nc) / 0.8)}
.border-neutral-content\/90 {
    border-color: hsl(var(--nc) / 0.9)}
.border-neutral-content\/95 {
    border-color: hsl(var(--nc) / 0.95)}
.border-neutral-focus {
    border-color: hsl(var(--nf) / 1)}
.border-neutral-focus\/0 {
    border-color: hsl(var(--nf) / 0)}
.border-neutral-focus\/10 {
    border-color: hsl(var(--nf) / 0.1)}
.border-neutral-focus\/100 {
    border-color: hsl(var(--nf) / 1)}
.border-neutral-focus\/20 {
    border-color: hsl(var(--nf) / 0.2)}
.border-neutral-focus\/25 {
    border-color: hsl(var(--nf) / 0.25)}
.border-neutral-focus\/30 {
    border-color: hsl(var(--nf) / 0.3)}
.border-neutral-focus\/40 {
    border-color: hsl(var(--nf) / 0.4)}
.border-neutral-focus\/5 {
    border-color: hsl(var(--nf) / 0.05)}
.border-neutral-focus\/50 {
    border-color: hsl(var(--nf) / 0.5)}
.border-neutral-focus\/60 {
    border-color: hsl(var(--nf) / 0.6)}
.border-neutral-focus\/70 {
    border-color: hsl(var(--nf) / 0.7)}
.border-neutral-focus\/75 {
    border-color: hsl(var(--nf) / 0.75)}
.border-neutral-focus\/80 {
    border-color: hsl(var(--nf) / 0.8)}
.border-neutral-focus\/90 {
    border-color: hsl(var(--nf) / 0.9)}
.border-neutral-focus\/95 {
    border-color: hsl(var(--nf) / 0.95)}
.border-neutral\/0 {
    border-color: hsl(var(--n) / 0)}
.border-neutral\/10 {
    border-color: hsl(var(--n) / 0.1)}
.border-neutral\/100 {
    border-color: hsl(var(--n) / 1)}
.border-neutral\/20 {
    border-color: hsl(var(--n) / 0.2)}
.border-neutral\/25 {
    border-color: hsl(var(--n) / 0.25)}
.border-neutral\/30 {
    border-color: hsl(var(--n) / 0.3)}
.border-neutral\/40 {
    border-color: hsl(var(--n) / 0.4)}
.border-neutral\/5 {
    border-color: hsl(var(--n) / 0.05)}
.border-neutral\/50 {
    border-color: hsl(var(--n) / 0.5)}
.border-neutral\/60 {
    border-color: hsl(var(--n) / 0.6)}
.border-neutral\/70 {
    border-color: hsl(var(--n) / 0.7)}
.border-neutral\/75 {
    border-color: hsl(var(--n) / 0.75)}
.border-neutral\/80 {
    border-color: hsl(var(--n) / 0.8)}
.border-neutral\/90 {
    border-color: hsl(var(--n) / 0.9)}
.border-neutral\/95 {
    border-color: hsl(var(--n) / 0.95)}
.border-primary {
    border-color: hsl(var(--p) / 1)}
.border-primary-content {
    border-color: hsl(var(--pc) / 1)}
.border-primary-content\/0 {
    border-color: hsl(var(--pc) / 0)}
.border-primary-content\/10 {
    border-color: hsl(var(--pc) / 0.1)}
.border-primary-content\/100 {
    border-color: hsl(var(--pc) / 1)}
.border-primary-content\/20 {
    border-color: hsl(var(--pc) / 0.2)}
.border-primary-content\/25 {
    border-color: hsl(var(--pc) / 0.25)}
.border-primary-content\/30 {
    border-color: hsl(var(--pc) / 0.3)}
.border-primary-content\/40 {
    border-color: hsl(var(--pc) / 0.4)}
.border-primary-content\/5 {
    border-color: hsl(var(--pc) / 0.05)}
.border-primary-content\/50 {
    border-color: hsl(var(--pc) / 0.5)}
.border-primary-content\/60 {
    border-color: hsl(var(--pc) / 0.6)}
.border-primary-content\/70 {
    border-color: hsl(var(--pc) / 0.7)}
.border-primary-content\/75 {
    border-color: hsl(var(--pc) / 0.75)}
.border-primary-content\/80 {
    border-color: hsl(var(--pc) / 0.8)}
.border-primary-content\/90 {
    border-color: hsl(var(--pc) / 0.9)}
.border-primary-content\/95 {
    border-color: hsl(var(--pc) / 0.95)}
.border-primary-focus {
    border-color: hsl(var(--pf) / 1)}
.border-primary-focus\/0 {
    border-color: hsl(var(--pf) / 0)}
.border-primary-focus\/10 {
    border-color: hsl(var(--pf) / 0.1)}
.border-primary-focus\/100 {
    border-color: hsl(var(--pf) / 1)}
.border-primary-focus\/20 {
    border-color: hsl(var(--pf) / 0.2)}
.border-primary-focus\/25 {
    border-color: hsl(var(--pf) / 0.25)}
.border-primary-focus\/30 {
    border-color: hsl(var(--pf) / 0.3)}
.border-primary-focus\/40 {
    border-color: hsl(var(--pf) / 0.4)}
.border-primary-focus\/5 {
    border-color: hsl(var(--pf) / 0.05)}
.border-primary-focus\/50 {
    border-color: hsl(var(--pf) / 0.5)}
.border-primary-focus\/60 {
    border-color: hsl(var(--pf) / 0.6)}
.border-primary-focus\/70 {
    border-color: hsl(var(--pf) / 0.7)}
.border-primary-focus\/75 {
    border-color: hsl(var(--pf) / 0.75)}
.border-primary-focus\/80 {
    border-color: hsl(var(--pf) / 0.8)}
.border-primary-focus\/90 {
    border-color: hsl(var(--pf) / 0.9)}
.border-primary-focus\/95 {
    border-color: hsl(var(--pf) / 0.95)}
.border-primary\/0 {
    border-color: hsl(var(--p) / 0)}
.border-primary\/10 {
    border-color: hsl(var(--p) / 0.1)}
.border-primary\/100 {
    border-color: hsl(var(--p) / 1)}
.border-primary\/20 {
    border-color: hsl(var(--p) / 0.2)}
.border-primary\/25 {
    border-color: hsl(var(--p) / 0.25)}
.border-primary\/30 {
    border-color: hsl(var(--p) / 0.3)}
.border-primary\/40 {
    border-color: hsl(var(--p) / 0.4)}
.border-primary\/5 {
    border-color: hsl(var(--p) / 0.05)}
.border-primary\/50 {
    border-color: hsl(var(--p) / 0.5)}
.border-primary\/60 {
    border-color: hsl(var(--p) / 0.6)}
.border-primary\/70 {
    border-color: hsl(var(--p) / 0.7)}
.border-primary\/75 {
    border-color: hsl(var(--p) / 0.75)}
.border-primary\/80 {
    border-color: hsl(var(--p) / 0.8)}
.border-primary\/90 {
    border-color: hsl(var(--p) / 0.9)}
.border-primary\/95 {
    border-color: hsl(var(--p) / 0.95)}
.border-secondary {
    border-color: hsl(var(--s) / 1)}
.border-secondary-content {
    border-color: hsl(var(--sc) / 1)}
.border-secondary-content\/0 {
    border-color: hsl(var(--sc) / 0)}
.border-secondary-content\/10 {
    border-color: hsl(var(--sc) / 0.1)}
.border-secondary-content\/100 {
    border-color: hsl(var(--sc) / 1)}
.border-secondary-content\/20 {
    border-color: hsl(var(--sc) / 0.2)}
.border-secondary-content\/25 {
    border-color: hsl(var(--sc) / 0.25)}
.border-secondary-content\/30 {
    border-color: hsl(var(--sc) / 0.3)}
.border-secondary-content\/40 {
    border-color: hsl(var(--sc) / 0.4)}
.border-secondary-content\/5 {
    border-color: hsl(var(--sc) / 0.05)}
.border-secondary-content\/50 {
    border-color: hsl(var(--sc) / 0.5)}
.border-secondary-content\/60 {
    border-color: hsl(var(--sc) / 0.6)}
.border-secondary-content\/70 {
    border-color: hsl(var(--sc) / 0.7)}
.border-secondary-content\/75 {
    border-color: hsl(var(--sc) / 0.75)}
.border-secondary-content\/80 {
    border-color: hsl(var(--sc) / 0.8)}
.border-secondary-content\/90 {
    border-color: hsl(var(--sc) / 0.9)}
.border-secondary-content\/95 {
    border-color: hsl(var(--sc) / 0.95)}
.border-secondary-focus {
    border-color: hsl(var(--sf) / 1)}
.border-secondary-focus\/0 {
    border-color: hsl(var(--sf) / 0)}
.border-secondary-focus\/10 {
    border-color: hsl(var(--sf) / 0.1)}
.border-secondary-focus\/100 {
    border-color: hsl(var(--sf) / 1)}
.border-secondary-focus\/20 {
    border-color: hsl(var(--sf) / 0.2)}
.border-secondary-focus\/25 {
    border-color: hsl(var(--sf) / 0.25)}
.border-secondary-focus\/30 {
    border-color: hsl(var(--sf) / 0.3)}
.border-secondary-focus\/40 {
    border-color: hsl(var(--sf) / 0.4)}
.border-secondary-focus\/5 {
    border-color: hsl(var(--sf) / 0.05)}
.border-secondary-focus\/50 {
    border-color: hsl(var(--sf) / 0.5)}
.border-secondary-focus\/60 {
    border-color: hsl(var(--sf) / 0.6)}
.border-secondary-focus\/70 {
    border-color: hsl(var(--sf) / 0.7)}
.border-secondary-focus\/75 {
    border-color: hsl(var(--sf) / 0.75)}
.border-secondary-focus\/80 {
    border-color: hsl(var(--sf) / 0.8)}
.border-secondary-focus\/90 {
    border-color: hsl(var(--sf) / 0.9)}
.border-secondary-focus\/95 {
    border-color: hsl(var(--sf) / 0.95)}
.border-secondary\/0 {
    border-color: hsl(var(--s) / 0)}
.border-secondary\/10 {
    border-color: hsl(var(--s) / 0.1)}
.border-secondary\/100 {
    border-color: hsl(var(--s) / 1)}
.border-secondary\/20 {
    border-color: hsl(var(--s) / 0.2)}
.border-secondary\/25 {
    border-color: hsl(var(--s) / 0.25)}
.border-secondary\/30 {
    border-color: hsl(var(--s) / 0.3)}
.border-secondary\/40 {
    border-color: hsl(var(--s) / 0.4)}
.border-secondary\/5 {
    border-color: hsl(var(--s) / 0.05)}
.border-secondary\/50 {
    border-color: hsl(var(--s) / 0.5)}
.border-secondary\/60 {
    border-color: hsl(var(--s) / 0.6)}
.border-secondary\/70 {
    border-color: hsl(var(--s) / 0.7)}
.border-secondary\/75 {
    border-color: hsl(var(--s) / 0.75)}
.border-secondary\/80 {
    border-color: hsl(var(--s) / 0.8)}
.border-secondary\/90 {
    border-color: hsl(var(--s) / 0.9)}
.border-secondary\/95 {
    border-color: hsl(var(--s) / 0.95)}
.border-success {
    border-color: hsl(var(--su) / 1)}
.border-success-content {
    border-color: hsl(var(--suc) / 1)}
.border-success-content\/0 {
    border-color: hsl(var(--suc) / 0)}
.border-success-content\/10 {
    border-color: hsl(var(--suc) / 0.1)}
.border-success-content\/100 {
    border-color: hsl(var(--suc) / 1)}
.border-success-content\/20 {
    border-color: hsl(var(--suc) / 0.2)}
.border-success-content\/25 {
    border-color: hsl(var(--suc) / 0.25)}
.border-success-content\/30 {
    border-color: hsl(var(--suc) / 0.3)}
.border-success-content\/40 {
    border-color: hsl(var(--suc) / 0.4)}
.border-success-content\/5 {
    border-color: hsl(var(--suc) / 0.05)}
.border-success-content\/50 {
    border-color: hsl(var(--suc) / 0.5)}
.border-success-content\/60 {
    border-color: hsl(var(--suc) / 0.6)}
.border-success-content\/70 {
    border-color: hsl(var(--suc) / 0.7)}
.border-success-content\/75 {
    border-color: hsl(var(--suc) / 0.75)}
.border-success-content\/80 {
    border-color: hsl(var(--suc) / 0.8)}
.border-success-content\/90 {
    border-color: hsl(var(--suc) / 0.9)}
.border-success-content\/95 {
    border-color: hsl(var(--suc) / 0.95)}
.border-success\/0 {
    border-color: hsl(var(--su) / 0)}
.border-success\/10 {
    border-color: hsl(var(--su) / 0.1)}
.border-success\/100 {
    border-color: hsl(var(--su) / 1)}
.border-success\/20 {
    border-color: hsl(var(--su) / 0.2)}
.border-success\/25 {
    border-color: hsl(var(--su) / 0.25)}
.border-success\/30 {
    border-color: hsl(var(--su) / 0.3)}
.border-success\/40 {
    border-color: hsl(var(--su) / 0.4)}
.border-success\/5 {
    border-color: hsl(var(--su) / 0.05)}
.border-success\/50 {
    border-color: hsl(var(--su) / 0.5)}
.border-success\/60 {
    border-color: hsl(var(--su) / 0.6)}
.border-success\/70 {
    border-color: hsl(var(--su) / 0.7)}
.border-success\/75 {
    border-color: hsl(var(--su) / 0.75)}
.border-success\/80 {
    border-color: hsl(var(--su) / 0.8)}
.border-success\/90 {
    border-color: hsl(var(--su) / 0.9)}
.border-success\/95 {
    border-color: hsl(var(--su) / 0.95)}
.border-transparent {
    border-color: transparent}
.border-transparent\/0 {
    border-color: rgb(0 0 0 / 0)}
.border-transparent\/10 {
    border-color: rgb(0 0 0 / 0.1)}
.border-transparent\/100 {
    border-color: rgb(0 0 0 / 1)}
.border-transparent\/20 {
    border-color: rgb(0 0 0 / 0.2)}
.border-transparent\/25 {
    border-color: rgb(0 0 0 / 0.25)}
.border-transparent\/30 {
    border-color: rgb(0 0 0 / 0.3)}
.border-transparent\/40 {
    border-color: rgb(0 0 0 / 0.4)}
.border-transparent\/5 {
    border-color: rgb(0 0 0 / 0.05)}
.border-transparent\/50 {
    border-color: rgb(0 0 0 / 0.5)}
.border-transparent\/60 {
    border-color: rgb(0 0 0 / 0.6)}
.border-transparent\/70 {
    border-color: rgb(0 0 0 / 0.7)}
.border-transparent\/75 {
    border-color: rgb(0 0 0 / 0.75)}
.border-transparent\/80 {
    border-color: rgb(0 0 0 / 0.8)}
.border-transparent\/90 {
    border-color: rgb(0 0 0 / 0.9)}
.border-transparent\/95 {
    border-color: rgb(0 0 0 / 0.95)}
.border-warning {
    border-color: hsl(var(--wa) / 1)}
.border-warning-content {
    border-color: hsl(var(--wac) / 1)}
.border-warning-content\/0 {
    border-color: hsl(var(--wac) / 0)}
.border-warning-content\/10 {
    border-color: hsl(var(--wac) / 0.1)}
.border-warning-content\/100 {
    border-color: hsl(var(--wac) / 1)}
.border-warning-content\/20 {
    border-color: hsl(var(--wac) / 0.2)}
.border-warning-content\/25 {
    border-color: hsl(var(--wac) / 0.25)}
.border-warning-content\/30 {
    border-color: hsl(var(--wac) / 0.3)}
.border-warning-content\/40 {
    border-color: hsl(var(--wac) / 0.4)}
.border-warning-content\/5 {
    border-color: hsl(var(--wac) / 0.05)}
.border-warning-content\/50 {
    border-color: hsl(var(--wac) / 0.5)}
.border-warning-content\/60 {
    border-color: hsl(var(--wac) / 0.6)}
.border-warning-content\/70 {
    border-color: hsl(var(--wac) / 0.7)}
.border-warning-content\/75 {
    border-color: hsl(var(--wac) / 0.75)}
.border-warning-content\/80 {
    border-color: hsl(var(--wac) / 0.8)}
.border-warning-content\/90 {
    border-color: hsl(var(--wac) / 0.9)}
.border-warning-content\/95 {
    border-color: hsl(var(--wac) / 0.95)}
.border-warning\/0 {
    border-color: hsl(var(--wa) / 0)}
.border-warning\/10 {
    border-color: hsl(var(--wa) / 0.1)}
.border-warning\/100 {
    border-color: hsl(var(--wa) / 1)}
.border-warning\/20 {
    border-color: hsl(var(--wa) / 0.2)}
.border-warning\/25 {
    border-color: hsl(var(--wa) / 0.25)}
.border-warning\/30 {
    border-color: hsl(var(--wa) / 0.3)}
.border-warning\/40 {
    border-color: hsl(var(--wa) / 0.4)}
.border-warning\/5 {
    border-color: hsl(var(--wa) / 0.05)}
.border-warning\/50 {
    border-color: hsl(var(--wa) / 0.5)}
.border-warning\/60 {
    border-color: hsl(var(--wa) / 0.6)}
.border-warning\/70 {
    border-color: hsl(var(--wa) / 0.7)}
.border-warning\/75 {
    border-color: hsl(var(--wa) / 0.75)}
.border-warning\/80 {
    border-color: hsl(var(--wa) / 0.8)}
.border-warning\/90 {
    border-color: hsl(var(--wa) / 0.9)}
.border-warning\/95 {
    border-color: hsl(var(--wa) / 0.95)}
.border-x-accent {
    border-left-color: hsl(var(--a) / 1);
    border-right-color: hsl(var(--a) / 1)}
.border-x-accent-content {
    border-left-color: hsl(var(--ac) / 1);
    border-right-color: hsl(var(--ac) / 1)}
.border-x-accent-content\/0 {
    border-left-color: hsl(var(--ac) / 0);
    border-right-color: hsl(var(--ac) / 0)}
.border-x-accent-content\/10 {
    border-left-color: hsl(var(--ac) / 0.1);
    border-right-color: hsl(var(--ac) / 0.1)}
.border-x-accent-content\/100 {
    border-left-color: hsl(var(--ac) / 1);
    border-right-color: hsl(var(--ac) / 1)}
.border-x-accent-content\/20 {
    border-left-color: hsl(var(--ac) / 0.2);
    border-right-color: hsl(var(--ac) / 0.2)}
.border-x-accent-content\/25 {
    border-left-color: hsl(var(--ac) / 0.25);
    border-right-color: hsl(var(--ac) / 0.25)}
.border-x-accent-content\/30 {
    border-left-color: hsl(var(--ac) / 0.3);
    border-right-color: hsl(var(--ac) / 0.3)}
.border-x-accent-content\/40 {
    border-left-color: hsl(var(--ac) / 0.4);
    border-right-color: hsl(var(--ac) / 0.4)}
.border-x-accent-content\/5 {
    border-left-color: hsl(var(--ac) / 0.05);
    border-right-color: hsl(var(--ac) / 0.05)}
.border-x-accent-content\/50 {
    border-left-color: hsl(var(--ac) / 0.5);
    border-right-color: hsl(var(--ac) / 0.5)}
.border-x-accent-content\/60 {
    border-left-color: hsl(var(--ac) / 0.6);
    border-right-color: hsl(var(--ac) / 0.6)}
.border-x-accent-content\/70 {
    border-left-color: hsl(var(--ac) / 0.7);
    border-right-color: hsl(var(--ac) / 0.7)}
.border-x-accent-content\/75 {
    border-left-color: hsl(var(--ac) / 0.75);
    border-right-color: hsl(var(--ac) / 0.75)}
.border-x-accent-content\/80 {
    border-left-color: hsl(var(--ac) / 0.8);
    border-right-color: hsl(var(--ac) / 0.8)}
.border-x-accent-content\/90 {
    border-left-color: hsl(var(--ac) / 0.9);
    border-right-color: hsl(var(--ac) / 0.9)}
.border-x-accent-content\/95 {
    border-left-color: hsl(var(--ac) / 0.95);
    border-right-color: hsl(var(--ac) / 0.95)}
.border-x-accent-focus {
    border-left-color: hsl(var(--af) / 1);
    border-right-color: hsl(var(--af) / 1)}
.border-x-accent-focus\/0 {
    border-left-color: hsl(var(--af) / 0);
    border-right-color: hsl(var(--af) / 0)}
.border-x-accent-focus\/10 {
    border-left-color: hsl(var(--af) / 0.1);
    border-right-color: hsl(var(--af) / 0.1)}
.border-x-accent-focus\/100 {
    border-left-color: hsl(var(--af) / 1);
    border-right-color: hsl(var(--af) / 1)}
.border-x-accent-focus\/20 {
    border-left-color: hsl(var(--af) / 0.2);
    border-right-color: hsl(var(--af) / 0.2)}
.border-x-accent-focus\/25 {
    border-left-color: hsl(var(--af) / 0.25);
    border-right-color: hsl(var(--af) / 0.25)}
.border-x-accent-focus\/30 {
    border-left-color: hsl(var(--af) / 0.3);
    border-right-color: hsl(var(--af) / 0.3)}
.border-x-accent-focus\/40 {
    border-left-color: hsl(var(--af) / 0.4);
    border-right-color: hsl(var(--af) / 0.4)}
.border-x-accent-focus\/5 {
    border-left-color: hsl(var(--af) / 0.05);
    border-right-color: hsl(var(--af) / 0.05)}
.border-x-accent-focus\/50 {
    border-left-color: hsl(var(--af) / 0.5);
    border-right-color: hsl(var(--af) / 0.5)}
.border-x-accent-focus\/60 {
    border-left-color: hsl(var(--af) / 0.6);
    border-right-color: hsl(var(--af) / 0.6)}
.border-x-accent-focus\/70 {
    border-left-color: hsl(var(--af) / 0.7);
    border-right-color: hsl(var(--af) / 0.7)}
.border-x-accent-focus\/75 {
    border-left-color: hsl(var(--af) / 0.75);
    border-right-color: hsl(var(--af) / 0.75)}
.border-x-accent-focus\/80 {
    border-left-color: hsl(var(--af) / 0.8);
    border-right-color: hsl(var(--af) / 0.8)}
.border-x-accent-focus\/90 {
    border-left-color: hsl(var(--af) / 0.9);
    border-right-color: hsl(var(--af) / 0.9)}
.border-x-accent-focus\/95 {
    border-left-color: hsl(var(--af) / 0.95);
    border-right-color: hsl(var(--af) / 0.95)}
.border-x-accent\/0 {
    border-left-color: hsl(var(--a) / 0);
    border-right-color: hsl(var(--a) / 0)}
.border-x-accent\/10 {
    border-left-color: hsl(var(--a) / 0.1);
    border-right-color: hsl(var(--a) / 0.1)}
.border-x-accent\/100 {
    border-left-color: hsl(var(--a) / 1);
    border-right-color: hsl(var(--a) / 1)}
.border-x-accent\/20 {
    border-left-color: hsl(var(--a) / 0.2);
    border-right-color: hsl(var(--a) / 0.2)}
.border-x-accent\/25 {
    border-left-color: hsl(var(--a) / 0.25);
    border-right-color: hsl(var(--a) / 0.25)}
.border-x-accent\/30 {
    border-left-color: hsl(var(--a) / 0.3);
    border-right-color: hsl(var(--a) / 0.3)}
.border-x-accent\/40 {
    border-left-color: hsl(var(--a) / 0.4);
    border-right-color: hsl(var(--a) / 0.4)}
.border-x-accent\/5 {
    border-left-color: hsl(var(--a) / 0.05);
    border-right-color: hsl(var(--a) / 0.05)}
.border-x-accent\/50 {
    border-left-color: hsl(var(--a) / 0.5);
    border-right-color: hsl(var(--a) / 0.5)}
.border-x-accent\/60 {
    border-left-color: hsl(var(--a) / 0.6);
    border-right-color: hsl(var(--a) / 0.6)}
.border-x-accent\/70 {
    border-left-color: hsl(var(--a) / 0.7);
    border-right-color: hsl(var(--a) / 0.7)}
.border-x-accent\/75 {
    border-left-color: hsl(var(--a) / 0.75);
    border-right-color: hsl(var(--a) / 0.75)}
.border-x-accent\/80 {
    border-left-color: hsl(var(--a) / 0.8);
    border-right-color: hsl(var(--a) / 0.8)}
.border-x-accent\/90 {
    border-left-color: hsl(var(--a) / 0.9);
    border-right-color: hsl(var(--a) / 0.9)}
.border-x-accent\/95 {
    border-left-color: hsl(var(--a) / 0.95);
    border-right-color: hsl(var(--a) / 0.95)}
.border-x-base-100 {
    border-left-color: hsl(var(--b1) / 1);
    border-right-color: hsl(var(--b1) / 1)}
.border-x-base-100\/0 {
    border-left-color: hsl(var(--b1) / 0);
    border-right-color: hsl(var(--b1) / 0)}
.border-x-base-100\/10 {
    border-left-color: hsl(var(--b1) / 0.1);
    border-right-color: hsl(var(--b1) / 0.1)}
.border-x-base-100\/100 {
    border-left-color: hsl(var(--b1) / 1);
    border-right-color: hsl(var(--b1) / 1)}
.border-x-base-100\/20 {
    border-left-color: hsl(var(--b1) / 0.2);
    border-right-color: hsl(var(--b1) / 0.2)}
.border-x-base-100\/25 {
    border-left-color: hsl(var(--b1) / 0.25);
    border-right-color: hsl(var(--b1) / 0.25)}
.border-x-base-100\/30 {
    border-left-color: hsl(var(--b1) / 0.3);
    border-right-color: hsl(var(--b1) / 0.3)}
.border-x-base-100\/40 {
    border-left-color: hsl(var(--b1) / 0.4);
    border-right-color: hsl(var(--b1) / 0.4)}
.border-x-base-100\/5 {
    border-left-color: hsl(var(--b1) / 0.05);
    border-right-color: hsl(var(--b1) / 0.05)}
.border-x-base-100\/50 {
    border-left-color: hsl(var(--b1) / 0.5);
    border-right-color: hsl(var(--b1) / 0.5)}
.border-x-base-100\/60 {
    border-left-color: hsl(var(--b1) / 0.6);
    border-right-color: hsl(var(--b1) / 0.6)}
.border-x-base-100\/70 {
    border-left-color: hsl(var(--b1) / 0.7);
    border-right-color: hsl(var(--b1) / 0.7)}
.border-x-base-100\/75 {
    border-left-color: hsl(var(--b1) / 0.75);
    border-right-color: hsl(var(--b1) / 0.75)}
.border-x-base-100\/80 {
    border-left-color: hsl(var(--b1) / 0.8);
    border-right-color: hsl(var(--b1) / 0.8)}
.border-x-base-100\/90 {
    border-left-color: hsl(var(--b1) / 0.9);
    border-right-color: hsl(var(--b1) / 0.9)}
.border-x-base-100\/95 {
    border-left-color: hsl(var(--b1) / 0.95);
    border-right-color: hsl(var(--b1) / 0.95)}
.border-x-base-200 {
    border-left-color: hsl(var(--b2) / 1);
    border-right-color: hsl(var(--b2) / 1)}
.border-x-base-200\/0 {
    border-left-color: hsl(var(--b2) / 0);
    border-right-color: hsl(var(--b2) / 0)}
.border-x-base-200\/10 {
    border-left-color: hsl(var(--b2) / 0.1);
    border-right-color: hsl(var(--b2) / 0.1)}
.border-x-base-200\/100 {
    border-left-color: hsl(var(--b2) / 1);
    border-right-color: hsl(var(--b2) / 1)}
.border-x-base-200\/20 {
    border-left-color: hsl(var(--b2) / 0.2);
    border-right-color: hsl(var(--b2) / 0.2)}
.border-x-base-200\/25 {
    border-left-color: hsl(var(--b2) / 0.25);
    border-right-color: hsl(var(--b2) / 0.25)}
.border-x-base-200\/30 {
    border-left-color: hsl(var(--b2) / 0.3);
    border-right-color: hsl(var(--b2) / 0.3)}
.border-x-base-200\/40 {
    border-left-color: hsl(var(--b2) / 0.4);
    border-right-color: hsl(var(--b2) / 0.4)}
.border-x-base-200\/5 {
    border-left-color: hsl(var(--b2) / 0.05);
    border-right-color: hsl(var(--b2) / 0.05)}
.border-x-base-200\/50 {
    border-left-color: hsl(var(--b2) / 0.5);
    border-right-color: hsl(var(--b2) / 0.5)}
.border-x-base-200\/60 {
    border-left-color: hsl(var(--b2) / 0.6);
    border-right-color: hsl(var(--b2) / 0.6)}
.border-x-base-200\/70 {
    border-left-color: hsl(var(--b2) / 0.7);
    border-right-color: hsl(var(--b2) / 0.7)}
.border-x-base-200\/75 {
    border-left-color: hsl(var(--b2) / 0.75);
    border-right-color: hsl(var(--b2) / 0.75)}
.border-x-base-200\/80 {
    border-left-color: hsl(var(--b2) / 0.8);
    border-right-color: hsl(var(--b2) / 0.8)}
.border-x-base-200\/90 {
    border-left-color: hsl(var(--b2) / 0.9);
    border-right-color: hsl(var(--b2) / 0.9)}
.border-x-base-200\/95 {
    border-left-color: hsl(var(--b2) / 0.95);
    border-right-color: hsl(var(--b2) / 0.95)}
.border-x-base-300 {
    border-left-color: hsl(var(--b3) / 1);
    border-right-color: hsl(var(--b3) / 1)}
.border-x-base-300\/0 {
    border-left-color: hsl(var(--b3) / 0);
    border-right-color: hsl(var(--b3) / 0)}
.border-x-base-300\/10 {
    border-left-color: hsl(var(--b3) / 0.1);
    border-right-color: hsl(var(--b3) / 0.1)}
.border-x-base-300\/100 {
    border-left-color: hsl(var(--b3) / 1);
    border-right-color: hsl(var(--b3) / 1)}
.border-x-base-300\/20 {
    border-left-color: hsl(var(--b3) / 0.2);
    border-right-color: hsl(var(--b3) / 0.2)}
.border-x-base-300\/25 {
    border-left-color: hsl(var(--b3) / 0.25);
    border-right-color: hsl(var(--b3) / 0.25)}
.border-x-base-300\/30 {
    border-left-color: hsl(var(--b3) / 0.3);
    border-right-color: hsl(var(--b3) / 0.3)}
.border-x-base-300\/40 {
    border-left-color: hsl(var(--b3) / 0.4);
    border-right-color: hsl(var(--b3) / 0.4)}
.border-x-base-300\/5 {
    border-left-color: hsl(var(--b3) / 0.05);
    border-right-color: hsl(var(--b3) / 0.05)}
.border-x-base-300\/50 {
    border-left-color: hsl(var(--b3) / 0.5);
    border-right-color: hsl(var(--b3) / 0.5)}
.border-x-base-300\/60 {
    border-left-color: hsl(var(--b3) / 0.6);
    border-right-color: hsl(var(--b3) / 0.6)}
.border-x-base-300\/70 {
    border-left-color: hsl(var(--b3) / 0.7);
    border-right-color: hsl(var(--b3) / 0.7)}
.border-x-base-300\/75 {
    border-left-color: hsl(var(--b3) / 0.75);
    border-right-color: hsl(var(--b3) / 0.75)}
.border-x-base-300\/80 {
    border-left-color: hsl(var(--b3) / 0.8);
    border-right-color: hsl(var(--b3) / 0.8)}
.border-x-base-300\/90 {
    border-left-color: hsl(var(--b3) / 0.9);
    border-right-color: hsl(var(--b3) / 0.9)}
.border-x-base-300\/95 {
    border-left-color: hsl(var(--b3) / 0.95);
    border-right-color: hsl(var(--b3) / 0.95)}
.border-x-base-content {
    border-left-color: hsl(var(--bc) / 1);
    border-right-color: hsl(var(--bc) / 1)}
.border-x-base-content\/0 {
    border-left-color: hsl(var(--bc) / 0);
    border-right-color: hsl(var(--bc) / 0)}
.border-x-base-content\/10 {
    border-left-color: hsl(var(--bc) / 0.1);
    border-right-color: hsl(var(--bc) / 0.1)}
.border-x-base-content\/100 {
    border-left-color: hsl(var(--bc) / 1);
    border-right-color: hsl(var(--bc) / 1)}
.border-x-base-content\/20 {
    border-left-color: hsl(var(--bc) / 0.2);
    border-right-color: hsl(var(--bc) / 0.2)}
.border-x-base-content\/25 {
    border-left-color: hsl(var(--bc) / 0.25);
    border-right-color: hsl(var(--bc) / 0.25)}
.border-x-base-content\/30 {
    border-left-color: hsl(var(--bc) / 0.3);
    border-right-color: hsl(var(--bc) / 0.3)}
.border-x-base-content\/40 {
    border-left-color: hsl(var(--bc) / 0.4);
    border-right-color: hsl(var(--bc) / 0.4)}
.border-x-base-content\/5 {
    border-left-color: hsl(var(--bc) / 0.05);
    border-right-color: hsl(var(--bc) / 0.05)}
.border-x-base-content\/50 {
    border-left-color: hsl(var(--bc) / 0.5);
    border-right-color: hsl(var(--bc) / 0.5)}
.border-x-base-content\/60 {
    border-left-color: hsl(var(--bc) / 0.6);
    border-right-color: hsl(var(--bc) / 0.6)}
.border-x-base-content\/70 {
    border-left-color: hsl(var(--bc) / 0.7);
    border-right-color: hsl(var(--bc) / 0.7)}
.border-x-base-content\/75 {
    border-left-color: hsl(var(--bc) / 0.75);
    border-right-color: hsl(var(--bc) / 0.75)}
.border-x-base-content\/80 {
    border-left-color: hsl(var(--bc) / 0.8);
    border-right-color: hsl(var(--bc) / 0.8)}
.border-x-base-content\/90 {
    border-left-color: hsl(var(--bc) / 0.9);
    border-right-color: hsl(var(--bc) / 0.9)}
.border-x-base-content\/95 {
    border-left-color: hsl(var(--bc) / 0.95);
    border-right-color: hsl(var(--bc) / 0.95)}
.border-x-current {
    border-left-color: currentColor;
    border-right-color: currentColor}
.border-x-error {
    border-left-color: hsl(var(--er) / 1);
    border-right-color: hsl(var(--er) / 1)}
.border-x-error-content {
    border-left-color: hsl(var(--erc) / 1);
    border-right-color: hsl(var(--erc) / 1)}
.border-x-error-content\/0 {
    border-left-color: hsl(var(--erc) / 0);
    border-right-color: hsl(var(--erc) / 0)}
.border-x-error-content\/10 {
    border-left-color: hsl(var(--erc) / 0.1);
    border-right-color: hsl(var(--erc) / 0.1)}
.border-x-error-content\/100 {
    border-left-color: hsl(var(--erc) / 1);
    border-right-color: hsl(var(--erc) / 1)}
.border-x-error-content\/20 {
    border-left-color: hsl(var(--erc) / 0.2);
    border-right-color: hsl(var(--erc) / 0.2)}
.border-x-error-content\/25 {
    border-left-color: hsl(var(--erc) / 0.25);
    border-right-color: hsl(var(--erc) / 0.25)}
.border-x-error-content\/30 {
    border-left-color: hsl(var(--erc) / 0.3);
    border-right-color: hsl(var(--erc) / 0.3)}
.border-x-error-content\/40 {
    border-left-color: hsl(var(--erc) / 0.4);
    border-right-color: hsl(var(--erc) / 0.4)}
.border-x-error-content\/5 {
    border-left-color: hsl(var(--erc) / 0.05);
    border-right-color: hsl(var(--erc) / 0.05)}
.border-x-error-content\/50 {
    border-left-color: hsl(var(--erc) / 0.5);
    border-right-color: hsl(var(--erc) / 0.5)}
.border-x-error-content\/60 {
    border-left-color: hsl(var(--erc) / 0.6);
    border-right-color: hsl(var(--erc) / 0.6)}
.border-x-error-content\/70 {
    border-left-color: hsl(var(--erc) / 0.7);
    border-right-color: hsl(var(--erc) / 0.7)}
.border-x-error-content\/75 {
    border-left-color: hsl(var(--erc) / 0.75);
    border-right-color: hsl(var(--erc) / 0.75)}
.border-x-error-content\/80 {
    border-left-color: hsl(var(--erc) / 0.8);
    border-right-color: hsl(var(--erc) / 0.8)}
.border-x-error-content\/90 {
    border-left-color: hsl(var(--erc) / 0.9);
    border-right-color: hsl(var(--erc) / 0.9)}
.border-x-error-content\/95 {
    border-left-color: hsl(var(--erc) / 0.95);
    border-right-color: hsl(var(--erc) / 0.95)}
.border-x-error\/0 {
    border-left-color: hsl(var(--er) / 0);
    border-right-color: hsl(var(--er) / 0)}
.border-x-error\/10 {
    border-left-color: hsl(var(--er) / 0.1);
    border-right-color: hsl(var(--er) / 0.1)}
.border-x-error\/100 {
    border-left-color: hsl(var(--er) / 1);
    border-right-color: hsl(var(--er) / 1)}
.border-x-error\/20 {
    border-left-color: hsl(var(--er) / 0.2);
    border-right-color: hsl(var(--er) / 0.2)}
.border-x-error\/25 {
    border-left-color: hsl(var(--er) / 0.25);
    border-right-color: hsl(var(--er) / 0.25)}
.border-x-error\/30 {
    border-left-color: hsl(var(--er) / 0.3);
    border-right-color: hsl(var(--er) / 0.3)}
.border-x-error\/40 {
    border-left-color: hsl(var(--er) / 0.4);
    border-right-color: hsl(var(--er) / 0.4)}
.border-x-error\/5 {
    border-left-color: hsl(var(--er) / 0.05);
    border-right-color: hsl(var(--er) / 0.05)}
.border-x-error\/50 {
    border-left-color: hsl(var(--er) / 0.5);
    border-right-color: hsl(var(--er) / 0.5)}
.border-x-error\/60 {
    border-left-color: hsl(var(--er) / 0.6);
    border-right-color: hsl(var(--er) / 0.6)}
.border-x-error\/70 {
    border-left-color: hsl(var(--er) / 0.7);
    border-right-color: hsl(var(--er) / 0.7)}
.border-x-error\/75 {
    border-left-color: hsl(var(--er) / 0.75);
    border-right-color: hsl(var(--er) / 0.75)}
.border-x-error\/80 {
    border-left-color: hsl(var(--er) / 0.8);
    border-right-color: hsl(var(--er) / 0.8)}
.border-x-error\/90 {
    border-left-color: hsl(var(--er) / 0.9);
    border-right-color: hsl(var(--er) / 0.9)}
.border-x-error\/95 {
    border-left-color: hsl(var(--er) / 0.95);
    border-right-color: hsl(var(--er) / 0.95)}
.border-x-info {
    border-left-color: hsl(var(--in) / 1);
    border-right-color: hsl(var(--in) / 1)}
.border-x-info-content {
    border-left-color: hsl(var(--inc) / 1);
    border-right-color: hsl(var(--inc) / 1)}
.border-x-info-content\/0 {
    border-left-color: hsl(var(--inc) / 0);
    border-right-color: hsl(var(--inc) / 0)}
.border-x-info-content\/10 {
    border-left-color: hsl(var(--inc) / 0.1);
    border-right-color: hsl(var(--inc) / 0.1)}
.border-x-info-content\/100 {
    border-left-color: hsl(var(--inc) / 1);
    border-right-color: hsl(var(--inc) / 1)}
.border-x-info-content\/20 {
    border-left-color: hsl(var(--inc) / 0.2);
    border-right-color: hsl(var(--inc) / 0.2)}
.border-x-info-content\/25 {
    border-left-color: hsl(var(--inc) / 0.25);
    border-right-color: hsl(var(--inc) / 0.25)}
.border-x-info-content\/30 {
    border-left-color: hsl(var(--inc) / 0.3);
    border-right-color: hsl(var(--inc) / 0.3)}
.border-x-info-content\/40 {
    border-left-color: hsl(var(--inc) / 0.4);
    border-right-color: hsl(var(--inc) / 0.4)}
.border-x-info-content\/5 {
    border-left-color: hsl(var(--inc) / 0.05);
    border-right-color: hsl(var(--inc) / 0.05)}
.border-x-info-content\/50 {
    border-left-color: hsl(var(--inc) / 0.5);
    border-right-color: hsl(var(--inc) / 0.5)}
.border-x-info-content\/60 {
    border-left-color: hsl(var(--inc) / 0.6);
    border-right-color: hsl(var(--inc) / 0.6)}
.border-x-info-content\/70 {
    border-left-color: hsl(var(--inc) / 0.7);
    border-right-color: hsl(var(--inc) / 0.7)}
.border-x-info-content\/75 {
    border-left-color: hsl(var(--inc) / 0.75);
    border-right-color: hsl(var(--inc) / 0.75)}
.border-x-info-content\/80 {
    border-left-color: hsl(var(--inc) / 0.8);
    border-right-color: hsl(var(--inc) / 0.8)}
.border-x-info-content\/90 {
    border-left-color: hsl(var(--inc) / 0.9);
    border-right-color: hsl(var(--inc) / 0.9)}
.border-x-info-content\/95 {
    border-left-color: hsl(var(--inc) / 0.95);
    border-right-color: hsl(var(--inc) / 0.95)}
.border-x-info\/0 {
    border-left-color: hsl(var(--in) / 0);
    border-right-color: hsl(var(--in) / 0)}
.border-x-info\/10 {
    border-left-color: hsl(var(--in) / 0.1);
    border-right-color: hsl(var(--in) / 0.1)}
.border-x-info\/100 {
    border-left-color: hsl(var(--in) / 1);
    border-right-color: hsl(var(--in) / 1)}
.border-x-info\/20 {
    border-left-color: hsl(var(--in) / 0.2);
    border-right-color: hsl(var(--in) / 0.2)}
.border-x-info\/25 {
    border-left-color: hsl(var(--in) / 0.25);
    border-right-color: hsl(var(--in) / 0.25)}
.border-x-info\/30 {
    border-left-color: hsl(var(--in) / 0.3);
    border-right-color: hsl(var(--in) / 0.3)}
.border-x-info\/40 {
    border-left-color: hsl(var(--in) / 0.4);
    border-right-color: hsl(var(--in) / 0.4)}
.border-x-info\/5 {
    border-left-color: hsl(var(--in) / 0.05);
    border-right-color: hsl(var(--in) / 0.05)}
.border-x-info\/50 {
    border-left-color: hsl(var(--in) / 0.5);
    border-right-color: hsl(var(--in) / 0.5)}
.border-x-info\/60 {
    border-left-color: hsl(var(--in) / 0.6);
    border-right-color: hsl(var(--in) / 0.6)}
.border-x-info\/70 {
    border-left-color: hsl(var(--in) / 0.7);
    border-right-color: hsl(var(--in) / 0.7)}
.border-x-info\/75 {
    border-left-color: hsl(var(--in) / 0.75);
    border-right-color: hsl(var(--in) / 0.75)}
.border-x-info\/80 {
    border-left-color: hsl(var(--in) / 0.8);
    border-right-color: hsl(var(--in) / 0.8)}
.border-x-info\/90 {
    border-left-color: hsl(var(--in) / 0.9);
    border-right-color: hsl(var(--in) / 0.9)}
.border-x-info\/95 {
    border-left-color: hsl(var(--in) / 0.95);
    border-right-color: hsl(var(--in) / 0.95)}
.border-x-neutral {
    border-left-color: hsl(var(--n) / 1);
    border-right-color: hsl(var(--n) / 1)}
.border-x-neutral-content {
    border-left-color: hsl(var(--nc) / 1);
    border-right-color: hsl(var(--nc) / 1)}
.border-x-neutral-content\/0 {
    border-left-color: hsl(var(--nc) / 0);
    border-right-color: hsl(var(--nc) / 0)}
.border-x-neutral-content\/10 {
    border-left-color: hsl(var(--nc) / 0.1);
    border-right-color: hsl(var(--nc) / 0.1)}
.border-x-neutral-content\/100 {
    border-left-color: hsl(var(--nc) / 1);
    border-right-color: hsl(var(--nc) / 1)}
.border-x-neutral-content\/20 {
    border-left-color: hsl(var(--nc) / 0.2);
    border-right-color: hsl(var(--nc) / 0.2)}
.border-x-neutral-content\/25 {
    border-left-color: hsl(var(--nc) / 0.25);
    border-right-color: hsl(var(--nc) / 0.25)}
.border-x-neutral-content\/30 {
    border-left-color: hsl(var(--nc) / 0.3);
    border-right-color: hsl(var(--nc) / 0.3)}
.border-x-neutral-content\/40 {
    border-left-color: hsl(var(--nc) / 0.4);
    border-right-color: hsl(var(--nc) / 0.4)}
.border-x-neutral-content\/5 {
    border-left-color: hsl(var(--nc) / 0.05);
    border-right-color: hsl(var(--nc) / 0.05)}
.border-x-neutral-content\/50 {
    border-left-color: hsl(var(--nc) / 0.5);
    border-right-color: hsl(var(--nc) / 0.5)}
.border-x-neutral-content\/60 {
    border-left-color: hsl(var(--nc) / 0.6);
    border-right-color: hsl(var(--nc) / 0.6)}
.border-x-neutral-content\/70 {
    border-left-color: hsl(var(--nc) / 0.7);
    border-right-color: hsl(var(--nc) / 0.7)}
.border-x-neutral-content\/75 {
    border-left-color: hsl(var(--nc) / 0.75);
    border-right-color: hsl(var(--nc) / 0.75)}
.border-x-neutral-content\/80 {
    border-left-color: hsl(var(--nc) / 0.8);
    border-right-color: hsl(var(--nc) / 0.8)}
.border-x-neutral-content\/90 {
    border-left-color: hsl(var(--nc) / 0.9);
    border-right-color: hsl(var(--nc) / 0.9)}
.border-x-neutral-content\/95 {
    border-left-color: hsl(var(--nc) / 0.95);
    border-right-color: hsl(var(--nc) / 0.95)}
.border-x-neutral-focus {
    border-left-color: hsl(var(--nf) / 1);
    border-right-color: hsl(var(--nf) / 1)}
.border-x-neutral-focus\/0 {
    border-left-color: hsl(var(--nf) / 0);
    border-right-color: hsl(var(--nf) / 0)}
.border-x-neutral-focus\/10 {
    border-left-color: hsl(var(--nf) / 0.1);
    border-right-color: hsl(var(--nf) / 0.1)}
.border-x-neutral-focus\/100 {
    border-left-color: hsl(var(--nf) / 1);
    border-right-color: hsl(var(--nf) / 1)}
.border-x-neutral-focus\/20 {
    border-left-color: hsl(var(--nf) / 0.2);
    border-right-color: hsl(var(--nf) / 0.2)}
.border-x-neutral-focus\/25 {
    border-left-color: hsl(var(--nf) / 0.25);
    border-right-color: hsl(var(--nf) / 0.25)}
.border-x-neutral-focus\/30 {
    border-left-color: hsl(var(--nf) / 0.3);
    border-right-color: hsl(var(--nf) / 0.3)}
.border-x-neutral-focus\/40 {
    border-left-color: hsl(var(--nf) / 0.4);
    border-right-color: hsl(var(--nf) / 0.4)}
.border-x-neutral-focus\/5 {
    border-left-color: hsl(var(--nf) / 0.05);
    border-right-color: hsl(var(--nf) / 0.05)}
.border-x-neutral-focus\/50 {
    border-left-color: hsl(var(--nf) / 0.5);
    border-right-color: hsl(var(--nf) / 0.5)}
.border-x-neutral-focus\/60 {
    border-left-color: hsl(var(--nf) / 0.6);
    border-right-color: hsl(var(--nf) / 0.6)}
.border-x-neutral-focus\/70 {
    border-left-color: hsl(var(--nf) / 0.7);
    border-right-color: hsl(var(--nf) / 0.7)}
.border-x-neutral-focus\/75 {
    border-left-color: hsl(var(--nf) / 0.75);
    border-right-color: hsl(var(--nf) / 0.75)}
.border-x-neutral-focus\/80 {
    border-left-color: hsl(var(--nf) / 0.8);
    border-right-color: hsl(var(--nf) / 0.8)}
.border-x-neutral-focus\/90 {
    border-left-color: hsl(var(--nf) / 0.9);
    border-right-color: hsl(var(--nf) / 0.9)}
.border-x-neutral-focus\/95 {
    border-left-color: hsl(var(--nf) / 0.95);
    border-right-color: hsl(var(--nf) / 0.95)}
.border-x-neutral\/0 {
    border-left-color: hsl(var(--n) / 0);
    border-right-color: hsl(var(--n) / 0)}
.border-x-neutral\/10 {
    border-left-color: hsl(var(--n) / 0.1);
    border-right-color: hsl(var(--n) / 0.1)}
.border-x-neutral\/100 {
    border-left-color: hsl(var(--n) / 1);
    border-right-color: hsl(var(--n) / 1)}
.border-x-neutral\/20 {
    border-left-color: hsl(var(--n) / 0.2);
    border-right-color: hsl(var(--n) / 0.2)}
.border-x-neutral\/25 {
    border-left-color: hsl(var(--n) / 0.25);
    border-right-color: hsl(var(--n) / 0.25)}
.border-x-neutral\/30 {
    border-left-color: hsl(var(--n) / 0.3);
    border-right-color: hsl(var(--n) / 0.3)}
.border-x-neutral\/40 {
    border-left-color: hsl(var(--n) / 0.4);
    border-right-color: hsl(var(--n) / 0.4)}
.border-x-neutral\/5 {
    border-left-color: hsl(var(--n) / 0.05);
    border-right-color: hsl(var(--n) / 0.05)}
.border-x-neutral\/50 {
    border-left-color: hsl(var(--n) / 0.5);
    border-right-color: hsl(var(--n) / 0.5)}
.border-x-neutral\/60 {
    border-left-color: hsl(var(--n) / 0.6);
    border-right-color: hsl(var(--n) / 0.6)}
.border-x-neutral\/70 {
    border-left-color: hsl(var(--n) / 0.7);
    border-right-color: hsl(var(--n) / 0.7)}
.border-x-neutral\/75 {
    border-left-color: hsl(var(--n) / 0.75);
    border-right-color: hsl(var(--n) / 0.75)}
.border-x-neutral\/80 {
    border-left-color: hsl(var(--n) / 0.8);
    border-right-color: hsl(var(--n) / 0.8)}
.border-x-neutral\/90 {
    border-left-color: hsl(var(--n) / 0.9);
    border-right-color: hsl(var(--n) / 0.9)}
.border-x-neutral\/95 {
    border-left-color: hsl(var(--n) / 0.95);
    border-right-color: hsl(var(--n) / 0.95)}
.border-x-primary {
    border-left-color: hsl(var(--p) / 1);
    border-right-color: hsl(var(--p) / 1)}
.border-x-primary-content {
    border-left-color: hsl(var(--pc) / 1);
    border-right-color: hsl(var(--pc) / 1)}
.border-x-primary-content\/0 {
    border-left-color: hsl(var(--pc) / 0);
    border-right-color: hsl(var(--pc) / 0)}
.border-x-primary-content\/10 {
    border-left-color: hsl(var(--pc) / 0.1);
    border-right-color: hsl(var(--pc) / 0.1)}
.border-x-primary-content\/100 {
    border-left-color: hsl(var(--pc) / 1);
    border-right-color: hsl(var(--pc) / 1)}
.border-x-primary-content\/20 {
    border-left-color: hsl(var(--pc) / 0.2);
    border-right-color: hsl(var(--pc) / 0.2)}
.border-x-primary-content\/25 {
    border-left-color: hsl(var(--pc) / 0.25);
    border-right-color: hsl(var(--pc) / 0.25)}
.border-x-primary-content\/30 {
    border-left-color: hsl(var(--pc) / 0.3);
    border-right-color: hsl(var(--pc) / 0.3)}
.border-x-primary-content\/40 {
    border-left-color: hsl(var(--pc) / 0.4);
    border-right-color: hsl(var(--pc) / 0.4)}
.border-x-primary-content\/5 {
    border-left-color: hsl(var(--pc) / 0.05);
    border-right-color: hsl(var(--pc) / 0.05)}
.border-x-primary-content\/50 {
    border-left-color: hsl(var(--pc) / 0.5);
    border-right-color: hsl(var(--pc) / 0.5)}
.border-x-primary-content\/60 {
    border-left-color: hsl(var(--pc) / 0.6);
    border-right-color: hsl(var(--pc) / 0.6)}
.border-x-primary-content\/70 {
    border-left-color: hsl(var(--pc) / 0.7);
    border-right-color: hsl(var(--pc) / 0.7)}
.border-x-primary-content\/75 {
    border-left-color: hsl(var(--pc) / 0.75);
    border-right-color: hsl(var(--pc) / 0.75)}
.border-x-primary-content\/80 {
    border-left-color: hsl(var(--pc) / 0.8);
    border-right-color: hsl(var(--pc) / 0.8)}
.border-x-primary-content\/90 {
    border-left-color: hsl(var(--pc) / 0.9);
    border-right-color: hsl(var(--pc) / 0.9)}
.border-x-primary-content\/95 {
    border-left-color: hsl(var(--pc) / 0.95);
    border-right-color: hsl(var(--pc) / 0.95)}
.border-x-primary-focus {
    border-left-color: hsl(var(--pf) / 1);
    border-right-color: hsl(var(--pf) / 1)}
.border-x-primary-focus\/0 {
    border-left-color: hsl(var(--pf) / 0);
    border-right-color: hsl(var(--pf) / 0)}
.border-x-primary-focus\/10 {
    border-left-color: hsl(var(--pf) / 0.1);
    border-right-color: hsl(var(--pf) / 0.1)}
.border-x-primary-focus\/100 {
    border-left-color: hsl(var(--pf) / 1);
    border-right-color: hsl(var(--pf) / 1)}
.border-x-primary-focus\/20 {
    border-left-color: hsl(var(--pf) / 0.2);
    border-right-color: hsl(var(--pf) / 0.2)}
.border-x-primary-focus\/25 {
    border-left-color: hsl(var(--pf) / 0.25);
    border-right-color: hsl(var(--pf) / 0.25)}
.border-x-primary-focus\/30 {
    border-left-color: hsl(var(--pf) / 0.3);
    border-right-color: hsl(var(--pf) / 0.3)}
.border-x-primary-focus\/40 {
    border-left-color: hsl(var(--pf) / 0.4);
    border-right-color: hsl(var(--pf) / 0.4)}
.border-x-primary-focus\/5 {
    border-left-color: hsl(var(--pf) / 0.05);
    border-right-color: hsl(var(--pf) / 0.05)}
.border-x-primary-focus\/50 {
    border-left-color: hsl(var(--pf) / 0.5);
    border-right-color: hsl(var(--pf) / 0.5)}
.border-x-primary-focus\/60 {
    border-left-color: hsl(var(--pf) / 0.6);
    border-right-color: hsl(var(--pf) / 0.6)}
.border-x-primary-focus\/70 {
    border-left-color: hsl(var(--pf) / 0.7);
    border-right-color: hsl(var(--pf) / 0.7)}
.border-x-primary-focus\/75 {
    border-left-color: hsl(var(--pf) / 0.75);
    border-right-color: hsl(var(--pf) / 0.75)}
.border-x-primary-focus\/80 {
    border-left-color: hsl(var(--pf) / 0.8);
    border-right-color: hsl(var(--pf) / 0.8)}
.border-x-primary-focus\/90 {
    border-left-color: hsl(var(--pf) / 0.9);
    border-right-color: hsl(var(--pf) / 0.9)}
.border-x-primary-focus\/95 {
    border-left-color: hsl(var(--pf) / 0.95);
    border-right-color: hsl(var(--pf) / 0.95)}
.border-x-primary\/0 {
    border-left-color: hsl(var(--p) / 0);
    border-right-color: hsl(var(--p) / 0)}
.border-x-primary\/10 {
    border-left-color: hsl(var(--p) / 0.1);
    border-right-color: hsl(var(--p) / 0.1)}
.border-x-primary\/100 {
    border-left-color: hsl(var(--p) / 1);
    border-right-color: hsl(var(--p) / 1)}
.border-x-primary\/20 {
    border-left-color: hsl(var(--p) / 0.2);
    border-right-color: hsl(var(--p) / 0.2)}
.border-x-primary\/25 {
    border-left-color: hsl(var(--p) / 0.25);
    border-right-color: hsl(var(--p) / 0.25)}
.border-x-primary\/30 {
    border-left-color: hsl(var(--p) / 0.3);
    border-right-color: hsl(var(--p) / 0.3)}
.border-x-primary\/40 {
    border-left-color: hsl(var(--p) / 0.4);
    border-right-color: hsl(var(--p) / 0.4)}
.border-x-primary\/5 {
    border-left-color: hsl(var(--p) / 0.05);
    border-right-color: hsl(var(--p) / 0.05)}
.border-x-primary\/50 {
    border-left-color: hsl(var(--p) / 0.5);
    border-right-color: hsl(var(--p) / 0.5)}
.border-x-primary\/60 {
    border-left-color: hsl(var(--p) / 0.6);
    border-right-color: hsl(var(--p) / 0.6)}
.border-x-primary\/70 {
    border-left-color: hsl(var(--p) / 0.7);
    border-right-color: hsl(var(--p) / 0.7)}
.border-x-primary\/75 {
    border-left-color: hsl(var(--p) / 0.75);
    border-right-color: hsl(var(--p) / 0.75)}
.border-x-primary\/80 {
    border-left-color: hsl(var(--p) / 0.8);
    border-right-color: hsl(var(--p) / 0.8)}
.border-x-primary\/90 {
    border-left-color: hsl(var(--p) / 0.9);
    border-right-color: hsl(var(--p) / 0.9)}
.border-x-primary\/95 {
    border-left-color: hsl(var(--p) / 0.95);
    border-right-color: hsl(var(--p) / 0.95)}
.border-x-secondary {
    border-left-color: hsl(var(--s) / 1);
    border-right-color: hsl(var(--s) / 1)}
.border-x-secondary-content {
    border-left-color: hsl(var(--sc) / 1);
    border-right-color: hsl(var(--sc) / 1)}
.border-x-secondary-content\/0 {
    border-left-color: hsl(var(--sc) / 0);
    border-right-color: hsl(var(--sc) / 0)}
.border-x-secondary-content\/10 {
    border-left-color: hsl(var(--sc) / 0.1);
    border-right-color: hsl(var(--sc) / 0.1)}
.border-x-secondary-content\/100 {
    border-left-color: hsl(var(--sc) / 1);
    border-right-color: hsl(var(--sc) / 1)}
.border-x-secondary-content\/20 {
    border-left-color: hsl(var(--sc) / 0.2);
    border-right-color: hsl(var(--sc) / 0.2)}
.border-x-secondary-content\/25 {
    border-left-color: hsl(var(--sc) / 0.25);
    border-right-color: hsl(var(--sc) / 0.25)}
.border-x-secondary-content\/30 {
    border-left-color: hsl(var(--sc) / 0.3);
    border-right-color: hsl(var(--sc) / 0.3)}
.border-x-secondary-content\/40 {
    border-left-color: hsl(var(--sc) / 0.4);
    border-right-color: hsl(var(--sc) / 0.4)}
.border-x-secondary-content\/5 {
    border-left-color: hsl(var(--sc) / 0.05);
    border-right-color: hsl(var(--sc) / 0.05)}
.border-x-secondary-content\/50 {
    border-left-color: hsl(var(--sc) / 0.5);
    border-right-color: hsl(var(--sc) / 0.5)}
.border-x-secondary-content\/60 {
    border-left-color: hsl(var(--sc) / 0.6);
    border-right-color: hsl(var(--sc) / 0.6)}
.border-x-secondary-content\/70 {
    border-left-color: hsl(var(--sc) / 0.7);
    border-right-color: hsl(var(--sc) / 0.7)}
.border-x-secondary-content\/75 {
    border-left-color: hsl(var(--sc) / 0.75);
    border-right-color: hsl(var(--sc) / 0.75)}
.border-x-secondary-content\/80 {
    border-left-color: hsl(var(--sc) / 0.8);
    border-right-color: hsl(var(--sc) / 0.8)}
.border-x-secondary-content\/90 {
    border-left-color: hsl(var(--sc) / 0.9);
    border-right-color: hsl(var(--sc) / 0.9)}
.border-x-secondary-content\/95 {
    border-left-color: hsl(var(--sc) / 0.95);
    border-right-color: hsl(var(--sc) / 0.95)}
.border-x-secondary-focus {
    border-left-color: hsl(var(--sf) / 1);
    border-right-color: hsl(var(--sf) / 1)}
.border-x-secondary-focus\/0 {
    border-left-color: hsl(var(--sf) / 0);
    border-right-color: hsl(var(--sf) / 0)}
.border-x-secondary-focus\/10 {
    border-left-color: hsl(var(--sf) / 0.1);
    border-right-color: hsl(var(--sf) / 0.1)}
.border-x-secondary-focus\/100 {
    border-left-color: hsl(var(--sf) / 1);
    border-right-color: hsl(var(--sf) / 1)}
.border-x-secondary-focus\/20 {
    border-left-color: hsl(var(--sf) / 0.2);
    border-right-color: hsl(var(--sf) / 0.2)}
.border-x-secondary-focus\/25 {
    border-left-color: hsl(var(--sf) / 0.25);
    border-right-color: hsl(var(--sf) / 0.25)}
.border-x-secondary-focus\/30 {
    border-left-color: hsl(var(--sf) / 0.3);
    border-right-color: hsl(var(--sf) / 0.3)}
.border-x-secondary-focus\/40 {
    border-left-color: hsl(var(--sf) / 0.4);
    border-right-color: hsl(var(--sf) / 0.4)}
.border-x-secondary-focus\/5 {
    border-left-color: hsl(var(--sf) / 0.05);
    border-right-color: hsl(var(--sf) / 0.05)}
.border-x-secondary-focus\/50 {
    border-left-color: hsl(var(--sf) / 0.5);
    border-right-color: hsl(var(--sf) / 0.5)}
.border-x-secondary-focus\/60 {
    border-left-color: hsl(var(--sf) / 0.6);
    border-right-color: hsl(var(--sf) / 0.6)}
.border-x-secondary-focus\/70 {
    border-left-color: hsl(var(--sf) / 0.7);
    border-right-color: hsl(var(--sf) / 0.7)}
.border-x-secondary-focus\/75 {
    border-left-color: hsl(var(--sf) / 0.75);
    border-right-color: hsl(var(--sf) / 0.75)}
.border-x-secondary-focus\/80 {
    border-left-color: hsl(var(--sf) / 0.8);
    border-right-color: hsl(var(--sf) / 0.8)}
.border-x-secondary-focus\/90 {
    border-left-color: hsl(var(--sf) / 0.9);
    border-right-color: hsl(var(--sf) / 0.9)}
.border-x-secondary-focus\/95 {
    border-left-color: hsl(var(--sf) / 0.95);
    border-right-color: hsl(var(--sf) / 0.95)}
.border-x-secondary\/0 {
    border-left-color: hsl(var(--s) / 0);
    border-right-color: hsl(var(--s) / 0)}
.border-x-secondary\/10 {
    border-left-color: hsl(var(--s) / 0.1);
    border-right-color: hsl(var(--s) / 0.1)}
.border-x-secondary\/100 {
    border-left-color: hsl(var(--s) / 1);
    border-right-color: hsl(var(--s) / 1)}
.border-x-secondary\/20 {
    border-left-color: hsl(var(--s) / 0.2);
    border-right-color: hsl(var(--s) / 0.2)}
.border-x-secondary\/25 {
    border-left-color: hsl(var(--s) / 0.25);
    border-right-color: hsl(var(--s) / 0.25)}
.border-x-secondary\/30 {
    border-left-color: hsl(var(--s) / 0.3);
    border-right-color: hsl(var(--s) / 0.3)}
.border-x-secondary\/40 {
    border-left-color: hsl(var(--s) / 0.4);
    border-right-color: hsl(var(--s) / 0.4)}
.border-x-secondary\/5 {
    border-left-color: hsl(var(--s) / 0.05);
    border-right-color: hsl(var(--s) / 0.05)}
.border-x-secondary\/50 {
    border-left-color: hsl(var(--s) / 0.5);
    border-right-color: hsl(var(--s) / 0.5)}
.border-x-secondary\/60 {
    border-left-color: hsl(var(--s) / 0.6);
    border-right-color: hsl(var(--s) / 0.6)}
.border-x-secondary\/70 {
    border-left-color: hsl(var(--s) / 0.7);
    border-right-color: hsl(var(--s) / 0.7)}
.border-x-secondary\/75 {
    border-left-color: hsl(var(--s) / 0.75);
    border-right-color: hsl(var(--s) / 0.75)}
.border-x-secondary\/80 {
    border-left-color: hsl(var(--s) / 0.8);
    border-right-color: hsl(var(--s) / 0.8)}
.border-x-secondary\/90 {
    border-left-color: hsl(var(--s) / 0.9);
    border-right-color: hsl(var(--s) / 0.9)}
.border-x-secondary\/95 {
    border-left-color: hsl(var(--s) / 0.95);
    border-right-color: hsl(var(--s) / 0.95)}
.border-x-success {
    border-left-color: hsl(var(--su) / 1);
    border-right-color: hsl(var(--su) / 1)}
.border-x-success-content {
    border-left-color: hsl(var(--suc) / 1);
    border-right-color: hsl(var(--suc) / 1)}
.border-x-success-content\/0 {
    border-left-color: hsl(var(--suc) / 0);
    border-right-color: hsl(var(--suc) / 0)}
.border-x-success-content\/10 {
    border-left-color: hsl(var(--suc) / 0.1);
    border-right-color: hsl(var(--suc) / 0.1)}
.border-x-success-content\/100 {
    border-left-color: hsl(var(--suc) / 1);
    border-right-color: hsl(var(--suc) / 1)}
.border-x-success-content\/20 {
    border-left-color: hsl(var(--suc) / 0.2);
    border-right-color: hsl(var(--suc) / 0.2)}
.border-x-success-content\/25 {
    border-left-color: hsl(var(--suc) / 0.25);
    border-right-color: hsl(var(--suc) / 0.25)}
.border-x-success-content\/30 {
    border-left-color: hsl(var(--suc) / 0.3);
    border-right-color: hsl(var(--suc) / 0.3)}
.border-x-success-content\/40 {
    border-left-color: hsl(var(--suc) / 0.4);
    border-right-color: hsl(var(--suc) / 0.4)}
.border-x-success-content\/5 {
    border-left-color: hsl(var(--suc) / 0.05);
    border-right-color: hsl(var(--suc) / 0.05)}
.border-x-success-content\/50 {
    border-left-color: hsl(var(--suc) / 0.5);
    border-right-color: hsl(var(--suc) / 0.5)}
.border-x-success-content\/60 {
    border-left-color: hsl(var(--suc) / 0.6);
    border-right-color: hsl(var(--suc) / 0.6)}
.border-x-success-content\/70 {
    border-left-color: hsl(var(--suc) / 0.7);
    border-right-color: hsl(var(--suc) / 0.7)}
.border-x-success-content\/75 {
    border-left-color: hsl(var(--suc) / 0.75);
    border-right-color: hsl(var(--suc) / 0.75)}
.border-x-success-content\/80 {
    border-left-color: hsl(var(--suc) / 0.8);
    border-right-color: hsl(var(--suc) / 0.8)}
.border-x-success-content\/90 {
    border-left-color: hsl(var(--suc) / 0.9);
    border-right-color: hsl(var(--suc) / 0.9)}
.border-x-success-content\/95 {
    border-left-color: hsl(var(--suc) / 0.95);
    border-right-color: hsl(var(--suc) / 0.95)}
.border-x-success\/0 {
    border-left-color: hsl(var(--su) / 0);
    border-right-color: hsl(var(--su) / 0)}
.border-x-success\/10 {
    border-left-color: hsl(var(--su) / 0.1);
    border-right-color: hsl(var(--su) / 0.1)}
.border-x-success\/100 {
    border-left-color: hsl(var(--su) / 1);
    border-right-color: hsl(var(--su) / 1)}
.border-x-success\/20 {
    border-left-color: hsl(var(--su) / 0.2);
    border-right-color: hsl(var(--su) / 0.2)}
.border-x-success\/25 {
    border-left-color: hsl(var(--su) / 0.25);
    border-right-color: hsl(var(--su) / 0.25)}
.border-x-success\/30 {
    border-left-color: hsl(var(--su) / 0.3);
    border-right-color: hsl(var(--su) / 0.3)}
.border-x-success\/40 {
    border-left-color: hsl(var(--su) / 0.4);
    border-right-color: hsl(var(--su) / 0.4)}
.border-x-success\/5 {
    border-left-color: hsl(var(--su) / 0.05);
    border-right-color: hsl(var(--su) / 0.05)}
.border-x-success\/50 {
    border-left-color: hsl(var(--su) / 0.5);
    border-right-color: hsl(var(--su) / 0.5)}
.border-x-success\/60 {
    border-left-color: hsl(var(--su) / 0.6);
    border-right-color: hsl(var(--su) / 0.6)}
.border-x-success\/70 {
    border-left-color: hsl(var(--su) / 0.7);
    border-right-color: hsl(var(--su) / 0.7)}
.border-x-success\/75 {
    border-left-color: hsl(var(--su) / 0.75);
    border-right-color: hsl(var(--su) / 0.75)}
.border-x-success\/80 {
    border-left-color: hsl(var(--su) / 0.8);
    border-right-color: hsl(var(--su) / 0.8)}
.border-x-success\/90 {
    border-left-color: hsl(var(--su) / 0.9);
    border-right-color: hsl(var(--su) / 0.9)}
.border-x-success\/95 {
    border-left-color: hsl(var(--su) / 0.95);
    border-right-color: hsl(var(--su) / 0.95)}
.border-x-transparent {
    border-left-color: transparent;
    border-right-color: transparent}
.border-x-transparent\/0 {
    border-left-color: rgb(0 0 0 / 0);
    border-right-color: rgb(0 0 0 / 0)}
.border-x-transparent\/10 {
    border-left-color: rgb(0 0 0 / 0.1);
    border-right-color: rgb(0 0 0 / 0.1)}
.border-x-transparent\/100 {
    border-left-color: rgb(0 0 0 / 1);
    border-right-color: rgb(0 0 0 / 1)}
.border-x-transparent\/20 {
    border-left-color: rgb(0 0 0 / 0.2);
    border-right-color: rgb(0 0 0 / 0.2)}
.border-x-transparent\/25 {
    border-left-color: rgb(0 0 0 / 0.25);
    border-right-color: rgb(0 0 0 / 0.25)}
.border-x-transparent\/30 {
    border-left-color: rgb(0 0 0 / 0.3);
    border-right-color: rgb(0 0 0 / 0.3)}
.border-x-transparent\/40 {
    border-left-color: rgb(0 0 0 / 0.4);
    border-right-color: rgb(0 0 0 / 0.4)}
.border-x-transparent\/5 {
    border-left-color: rgb(0 0 0 / 0.05);
    border-right-color: rgb(0 0 0 / 0.05)}
.border-x-transparent\/50 {
    border-left-color: rgb(0 0 0 / 0.5);
    border-right-color: rgb(0 0 0 / 0.5)}
.border-x-transparent\/60 {
    border-left-color: rgb(0 0 0 / 0.6);
    border-right-color: rgb(0 0 0 / 0.6)}
.border-x-transparent\/70 {
    border-left-color: rgb(0 0 0 / 0.7);
    border-right-color: rgb(0 0 0 / 0.7)}
.border-x-transparent\/75 {
    border-left-color: rgb(0 0 0 / 0.75);
    border-right-color: rgb(0 0 0 / 0.75)}
.border-x-transparent\/80 {
    border-left-color: rgb(0 0 0 / 0.8);
    border-right-color: rgb(0 0 0 / 0.8)}
.border-x-transparent\/90 {
    border-left-color: rgb(0 0 0 / 0.9);
    border-right-color: rgb(0 0 0 / 0.9)}
.border-x-transparent\/95 {
    border-left-color: rgb(0 0 0 / 0.95);
    border-right-color: rgb(0 0 0 / 0.95)}
.border-x-warning {
    border-left-color: hsl(var(--wa) / 1);
    border-right-color: hsl(var(--wa) / 1)}
.border-x-warning-content {
    border-left-color: hsl(var(--wac) / 1);
    border-right-color: hsl(var(--wac) / 1)}
.border-x-warning-content\/0 {
    border-left-color: hsl(var(--wac) / 0);
    border-right-color: hsl(var(--wac) / 0)}
.border-x-warning-content\/10 {
    border-left-color: hsl(var(--wac) / 0.1);
    border-right-color: hsl(var(--wac) / 0.1)}
.border-x-warning-content\/100 {
    border-left-color: hsl(var(--wac) / 1);
    border-right-color: hsl(var(--wac) / 1)}
.border-x-warning-content\/20 {
    border-left-color: hsl(var(--wac) / 0.2);
    border-right-color: hsl(var(--wac) / 0.2)}
.border-x-warning-content\/25 {
    border-left-color: hsl(var(--wac) / 0.25);
    border-right-color: hsl(var(--wac) / 0.25)}
.border-x-warning-content\/30 {
    border-left-color: hsl(var(--wac) / 0.3);
    border-right-color: hsl(var(--wac) / 0.3)}
.border-x-warning-content\/40 {
    border-left-color: hsl(var(--wac) / 0.4);
    border-right-color: hsl(var(--wac) / 0.4)}
.border-x-warning-content\/5 {
    border-left-color: hsl(var(--wac) / 0.05);
    border-right-color: hsl(var(--wac) / 0.05)}
.border-x-warning-content\/50 {
    border-left-color: hsl(var(--wac) / 0.5);
    border-right-color: hsl(var(--wac) / 0.5)}
.border-x-warning-content\/60 {
    border-left-color: hsl(var(--wac) / 0.6);
    border-right-color: hsl(var(--wac) / 0.6)}
.border-x-warning-content\/70 {
    border-left-color: hsl(var(--wac) / 0.7);
    border-right-color: hsl(var(--wac) / 0.7)}
.border-x-warning-content\/75 {
    border-left-color: hsl(var(--wac) / 0.75);
    border-right-color: hsl(var(--wac) / 0.75)}
.border-x-warning-content\/80 {
    border-left-color: hsl(var(--wac) / 0.8);
    border-right-color: hsl(var(--wac) / 0.8)}
.border-x-warning-content\/90 {
    border-left-color: hsl(var(--wac) / 0.9);
    border-right-color: hsl(var(--wac) / 0.9)}
.border-x-warning-content\/95 {
    border-left-color: hsl(var(--wac) / 0.95);
    border-right-color: hsl(var(--wac) / 0.95)}
.border-x-warning\/0 {
    border-left-color: hsl(var(--wa) / 0);
    border-right-color: hsl(var(--wa) / 0)}
.border-x-warning\/10 {
    border-left-color: hsl(var(--wa) / 0.1);
    border-right-color: hsl(var(--wa) / 0.1)}
.border-x-warning\/100 {
    border-left-color: hsl(var(--wa) / 1);
    border-right-color: hsl(var(--wa) / 1)}
.border-x-warning\/20 {
    border-left-color: hsl(var(--wa) / 0.2);
    border-right-color: hsl(var(--wa) / 0.2)}
.border-x-warning\/25 {
    border-left-color: hsl(var(--wa) / 0.25);
    border-right-color: hsl(var(--wa) / 0.25)}
.border-x-warning\/30 {
    border-left-color: hsl(var(--wa) / 0.3);
    border-right-color: hsl(var(--wa) / 0.3)}
.border-x-warning\/40 {
    border-left-color: hsl(var(--wa) / 0.4);
    border-right-color: hsl(var(--wa) / 0.4)}
.border-x-warning\/5 {
    border-left-color: hsl(var(--wa) / 0.05);
    border-right-color: hsl(var(--wa) / 0.05)}
.border-x-warning\/50 {
    border-left-color: hsl(var(--wa) / 0.5);
    border-right-color: hsl(var(--wa) / 0.5)}
.border-x-warning\/60 {
    border-left-color: hsl(var(--wa) / 0.6);
    border-right-color: hsl(var(--wa) / 0.6)}
.border-x-warning\/70 {
    border-left-color: hsl(var(--wa) / 0.7);
    border-right-color: hsl(var(--wa) / 0.7)}
.border-x-warning\/75 {
    border-left-color: hsl(var(--wa) / 0.75);
    border-right-color: hsl(var(--wa) / 0.75)}
.border-x-warning\/80 {
    border-left-color: hsl(var(--wa) / 0.8);
    border-right-color: hsl(var(--wa) / 0.8)}
.border-x-warning\/90 {
    border-left-color: hsl(var(--wa) / 0.9);
    border-right-color: hsl(var(--wa) / 0.9)}
.border-x-warning\/95 {
    border-left-color: hsl(var(--wa) / 0.95);
    border-right-color: hsl(var(--wa) / 0.95)}
.border-y-accent {
    border-top-color: hsl(var(--a) / 1);
    border-bottom-color: hsl(var(--a) / 1)}
.border-y-accent-content {
    border-top-color: hsl(var(--ac) / 1);
    border-bottom-color: hsl(var(--ac) / 1)}
.border-y-accent-content\/0 {
    border-top-color: hsl(var(--ac) / 0);
    border-bottom-color: hsl(var(--ac) / 0)}
.border-y-accent-content\/10 {
    border-top-color: hsl(var(--ac) / 0.1);
    border-bottom-color: hsl(var(--ac) / 0.1)}
.border-y-accent-content\/100 {
    border-top-color: hsl(var(--ac) / 1);
    border-bottom-color: hsl(var(--ac) / 1)}
.border-y-accent-content\/20 {
    border-top-color: hsl(var(--ac) / 0.2);
    border-bottom-color: hsl(var(--ac) / 0.2)}
.border-y-accent-content\/25 {
    border-top-color: hsl(var(--ac) / 0.25);
    border-bottom-color: hsl(var(--ac) / 0.25)}
.border-y-accent-content\/30 {
    border-top-color: hsl(var(--ac) / 0.3);
    border-bottom-color: hsl(var(--ac) / 0.3)}
.border-y-accent-content\/40 {
    border-top-color: hsl(var(--ac) / 0.4);
    border-bottom-color: hsl(var(--ac) / 0.4)}
.border-y-accent-content\/5 {
    border-top-color: hsl(var(--ac) / 0.05);
    border-bottom-color: hsl(var(--ac) / 0.05)}
.border-y-accent-content\/50 {
    border-top-color: hsl(var(--ac) / 0.5);
    border-bottom-color: hsl(var(--ac) / 0.5)}
.border-y-accent-content\/60 {
    border-top-color: hsl(var(--ac) / 0.6);
    border-bottom-color: hsl(var(--ac) / 0.6)}
.border-y-accent-content\/70 {
    border-top-color: hsl(var(--ac) / 0.7);
    border-bottom-color: hsl(var(--ac) / 0.7)}
.border-y-accent-content\/75 {
    border-top-color: hsl(var(--ac) / 0.75);
    border-bottom-color: hsl(var(--ac) / 0.75)}
.border-y-accent-content\/80 {
    border-top-color: hsl(var(--ac) / 0.8);
    border-bottom-color: hsl(var(--ac) / 0.8)}
.border-y-accent-content\/90 {
    border-top-color: hsl(var(--ac) / 0.9);
    border-bottom-color: hsl(var(--ac) / 0.9)}
.border-y-accent-content\/95 {
    border-top-color: hsl(var(--ac) / 0.95);
    border-bottom-color: hsl(var(--ac) / 0.95)}
.border-y-accent-focus {
    border-top-color: hsl(var(--af) / 1);
    border-bottom-color: hsl(var(--af) / 1)}
.border-y-accent-focus\/0 {
    border-top-color: hsl(var(--af) / 0);
    border-bottom-color: hsl(var(--af) / 0)}
.border-y-accent-focus\/10 {
    border-top-color: hsl(var(--af) / 0.1);
    border-bottom-color: hsl(var(--af) / 0.1)}
.border-y-accent-focus\/100 {
    border-top-color: hsl(var(--af) / 1);
    border-bottom-color: hsl(var(--af) / 1)}
.border-y-accent-focus\/20 {
    border-top-color: hsl(var(--af) / 0.2);
    border-bottom-color: hsl(var(--af) / 0.2)}
.border-y-accent-focus\/25 {
    border-top-color: hsl(var(--af) / 0.25);
    border-bottom-color: hsl(var(--af) / 0.25)}
.border-y-accent-focus\/30 {
    border-top-color: hsl(var(--af) / 0.3);
    border-bottom-color: hsl(var(--af) / 0.3)}
.border-y-accent-focus\/40 {
    border-top-color: hsl(var(--af) / 0.4);
    border-bottom-color: hsl(var(--af) / 0.4)}
.border-y-accent-focus\/5 {
    border-top-color: hsl(var(--af) / 0.05);
    border-bottom-color: hsl(var(--af) / 0.05)}
.border-y-accent-focus\/50 {
    border-top-color: hsl(var(--af) / 0.5);
    border-bottom-color: hsl(var(--af) / 0.5)}
.border-y-accent-focus\/60 {
    border-top-color: hsl(var(--af) / 0.6);
    border-bottom-color: hsl(var(--af) / 0.6)}
.border-y-accent-focus\/70 {
    border-top-color: hsl(var(--af) / 0.7);
    border-bottom-color: hsl(var(--af) / 0.7)}
.border-y-accent-focus\/75 {
    border-top-color: hsl(var(--af) / 0.75);
    border-bottom-color: hsl(var(--af) / 0.75)}
.border-y-accent-focus\/80 {
    border-top-color: hsl(var(--af) / 0.8);
    border-bottom-color: hsl(var(--af) / 0.8)}
.border-y-accent-focus\/90 {
    border-top-color: hsl(var(--af) / 0.9);
    border-bottom-color: hsl(var(--af) / 0.9)}
.border-y-accent-focus\/95 {
    border-top-color: hsl(var(--af) / 0.95);
    border-bottom-color: hsl(var(--af) / 0.95)}
.border-y-accent\/0 {
    border-top-color: hsl(var(--a) / 0);
    border-bottom-color: hsl(var(--a) / 0)}
.border-y-accent\/10 {
    border-top-color: hsl(var(--a) / 0.1);
    border-bottom-color: hsl(var(--a) / 0.1)}
.border-y-accent\/100 {
    border-top-color: hsl(var(--a) / 1);
    border-bottom-color: hsl(var(--a) / 1)}
.border-y-accent\/20 {
    border-top-color: hsl(var(--a) / 0.2);
    border-bottom-color: hsl(var(--a) / 0.2)}
.border-y-accent\/25 {
    border-top-color: hsl(var(--a) / 0.25);
    border-bottom-color: hsl(var(--a) / 0.25)}
.border-y-accent\/30 {
    border-top-color: hsl(var(--a) / 0.3);
    border-bottom-color: hsl(var(--a) / 0.3)}
.border-y-accent\/40 {
    border-top-color: hsl(var(--a) / 0.4);
    border-bottom-color: hsl(var(--a) / 0.4)}
.border-y-accent\/5 {
    border-top-color: hsl(var(--a) / 0.05);
    border-bottom-color: hsl(var(--a) / 0.05)}
.border-y-accent\/50 {
    border-top-color: hsl(var(--a) / 0.5);
    border-bottom-color: hsl(var(--a) / 0.5)}
.border-y-accent\/60 {
    border-top-color: hsl(var(--a) / 0.6);
    border-bottom-color: hsl(var(--a) / 0.6)}
.border-y-accent\/70 {
    border-top-color: hsl(var(--a) / 0.7);
    border-bottom-color: hsl(var(--a) / 0.7)}
.border-y-accent\/75 {
    border-top-color: hsl(var(--a) / 0.75);
    border-bottom-color: hsl(var(--a) / 0.75)}
.border-y-accent\/80 {
    border-top-color: hsl(var(--a) / 0.8);
    border-bottom-color: hsl(var(--a) / 0.8)}
.border-y-accent\/90 {
    border-top-color: hsl(var(--a) / 0.9);
    border-bottom-color: hsl(var(--a) / 0.9)}
.border-y-accent\/95 {
    border-top-color: hsl(var(--a) / 0.95);
    border-bottom-color: hsl(var(--a) / 0.95)}
.border-y-base-100 {
    border-top-color: hsl(var(--b1) / 1);
    border-bottom-color: hsl(var(--b1) / 1)}
.border-y-base-100\/0 {
    border-top-color: hsl(var(--b1) / 0);
    border-bottom-color: hsl(var(--b1) / 0)}
.border-y-base-100\/10 {
    border-top-color: hsl(var(--b1) / 0.1);
    border-bottom-color: hsl(var(--b1) / 0.1)}
.border-y-base-100\/100 {
    border-top-color: hsl(var(--b1) / 1);
    border-bottom-color: hsl(var(--b1) / 1)}
.border-y-base-100\/20 {
    border-top-color: hsl(var(--b1) / 0.2);
    border-bottom-color: hsl(var(--b1) / 0.2)}
.border-y-base-100\/25 {
    border-top-color: hsl(var(--b1) / 0.25);
    border-bottom-color: hsl(var(--b1) / 0.25)}
.border-y-base-100\/30 {
    border-top-color: hsl(var(--b1) / 0.3);
    border-bottom-color: hsl(var(--b1) / 0.3)}
.border-y-base-100\/40 {
    border-top-color: hsl(var(--b1) / 0.4);
    border-bottom-color: hsl(var(--b1) / 0.4)}
.border-y-base-100\/5 {
    border-top-color: hsl(var(--b1) / 0.05);
    border-bottom-color: hsl(var(--b1) / 0.05)}
.border-y-base-100\/50 {
    border-top-color: hsl(var(--b1) / 0.5);
    border-bottom-color: hsl(var(--b1) / 0.5)}
.border-y-base-100\/60 {
    border-top-color: hsl(var(--b1) / 0.6);
    border-bottom-color: hsl(var(--b1) / 0.6)}
.border-y-base-100\/70 {
    border-top-color: hsl(var(--b1) / 0.7);
    border-bottom-color: hsl(var(--b1) / 0.7)}
.border-y-base-100\/75 {
    border-top-color: hsl(var(--b1) / 0.75);
    border-bottom-color: hsl(var(--b1) / 0.75)}
.border-y-base-100\/80 {
    border-top-color: hsl(var(--b1) / 0.8);
    border-bottom-color: hsl(var(--b1) / 0.8)}
.border-y-base-100\/90 {
    border-top-color: hsl(var(--b1) / 0.9);
    border-bottom-color: hsl(var(--b1) / 0.9)}
.border-y-base-100\/95 {
    border-top-color: hsl(var(--b1) / 0.95);
    border-bottom-color: hsl(var(--b1) / 0.95)}
.border-y-base-200 {
    border-top-color: hsl(var(--b2) / 1);
    border-bottom-color: hsl(var(--b2) / 1)}
.border-y-base-200\/0 {
    border-top-color: hsl(var(--b2) / 0);
    border-bottom-color: hsl(var(--b2) / 0)}
.border-y-base-200\/10 {
    border-top-color: hsl(var(--b2) / 0.1);
    border-bottom-color: hsl(var(--b2) / 0.1)}
.border-y-base-200\/100 {
    border-top-color: hsl(var(--b2) / 1);
    border-bottom-color: hsl(var(--b2) / 1)}
.border-y-base-200\/20 {
    border-top-color: hsl(var(--b2) / 0.2);
    border-bottom-color: hsl(var(--b2) / 0.2)}
.border-y-base-200\/25 {
    border-top-color: hsl(var(--b2) / 0.25);
    border-bottom-color: hsl(var(--b2) / 0.25)}
.border-y-base-200\/30 {
    border-top-color: hsl(var(--b2) / 0.3);
    border-bottom-color: hsl(var(--b2) / 0.3)}
.border-y-base-200\/40 {
    border-top-color: hsl(var(--b2) / 0.4);
    border-bottom-color: hsl(var(--b2) / 0.4)}
.border-y-base-200\/5 {
    border-top-color: hsl(var(--b2) / 0.05);
    border-bottom-color: hsl(var(--b2) / 0.05)}
.border-y-base-200\/50 {
    border-top-color: hsl(var(--b2) / 0.5);
    border-bottom-color: hsl(var(--b2) / 0.5)}
.border-y-base-200\/60 {
    border-top-color: hsl(var(--b2) / 0.6);
    border-bottom-color: hsl(var(--b2) / 0.6)}
.border-y-base-200\/70 {
    border-top-color: hsl(var(--b2) / 0.7);
    border-bottom-color: hsl(var(--b2) / 0.7)}
.border-y-base-200\/75 {
    border-top-color: hsl(var(--b2) / 0.75);
    border-bottom-color: hsl(var(--b2) / 0.75)}
.border-y-base-200\/80 {
    border-top-color: hsl(var(--b2) / 0.8);
    border-bottom-color: hsl(var(--b2) / 0.8)}
.border-y-base-200\/90 {
    border-top-color: hsl(var(--b2) / 0.9);
    border-bottom-color: hsl(var(--b2) / 0.9)}
.border-y-base-200\/95 {
    border-top-color: hsl(var(--b2) / 0.95);
    border-bottom-color: hsl(var(--b2) / 0.95)}
.border-y-base-300 {
    border-top-color: hsl(var(--b3) / 1);
    border-bottom-color: hsl(var(--b3) / 1)}
.border-y-base-300\/0 {
    border-top-color: hsl(var(--b3) / 0);
    border-bottom-color: hsl(var(--b3) / 0)}
.border-y-base-300\/10 {
    border-top-color: hsl(var(--b3) / 0.1);
    border-bottom-color: hsl(var(--b3) / 0.1)}
.border-y-base-300\/100 {
    border-top-color: hsl(var(--b3) / 1);
    border-bottom-color: hsl(var(--b3) / 1)}
.border-y-base-300\/20 {
    border-top-color: hsl(var(--b3) / 0.2);
    border-bottom-color: hsl(var(--b3) / 0.2)}
.border-y-base-300\/25 {
    border-top-color: hsl(var(--b3) / 0.25);
    border-bottom-color: hsl(var(--b3) / 0.25)}
.border-y-base-300\/30 {
    border-top-color: hsl(var(--b3) / 0.3);
    border-bottom-color: hsl(var(--b3) / 0.3)}
.border-y-base-300\/40 {
    border-top-color: hsl(var(--b3) / 0.4);
    border-bottom-color: hsl(var(--b3) / 0.4)}
.border-y-base-300\/5 {
    border-top-color: hsl(var(--b3) / 0.05);
    border-bottom-color: hsl(var(--b3) / 0.05)}
.border-y-base-300\/50 {
    border-top-color: hsl(var(--b3) / 0.5);
    border-bottom-color: hsl(var(--b3) / 0.5)}
.border-y-base-300\/60 {
    border-top-color: hsl(var(--b3) / 0.6);
    border-bottom-color: hsl(var(--b3) / 0.6)}
.border-y-base-300\/70 {
    border-top-color: hsl(var(--b3) / 0.7);
    border-bottom-color: hsl(var(--b3) / 0.7)}
.border-y-base-300\/75 {
    border-top-color: hsl(var(--b3) / 0.75);
    border-bottom-color: hsl(var(--b3) / 0.75)}
.border-y-base-300\/80 {
    border-top-color: hsl(var(--b3) / 0.8);
    border-bottom-color: hsl(var(--b3) / 0.8)}
.border-y-base-300\/90 {
    border-top-color: hsl(var(--b3) / 0.9);
    border-bottom-color: hsl(var(--b3) / 0.9)}
.border-y-base-300\/95 {
    border-top-color: hsl(var(--b3) / 0.95);
    border-bottom-color: hsl(var(--b3) / 0.95)}
.border-y-base-content {
    border-top-color: hsl(var(--bc) / 1);
    border-bottom-color: hsl(var(--bc) / 1)}
.border-y-base-content\/0 {
    border-top-color: hsl(var(--bc) / 0);
    border-bottom-color: hsl(var(--bc) / 0)}
.border-y-base-content\/10 {
    border-top-color: hsl(var(--bc) / 0.1);
    border-bottom-color: hsl(var(--bc) / 0.1)}
.border-y-base-content\/100 {
    border-top-color: hsl(var(--bc) / 1);
    border-bottom-color: hsl(var(--bc) / 1)}
.border-y-base-content\/20 {
    border-top-color: hsl(var(--bc) / 0.2);
    border-bottom-color: hsl(var(--bc) / 0.2)}
.border-y-base-content\/25 {
    border-top-color: hsl(var(--bc) / 0.25);
    border-bottom-color: hsl(var(--bc) / 0.25)}
.border-y-base-content\/30 {
    border-top-color: hsl(var(--bc) / 0.3);
    border-bottom-color: hsl(var(--bc) / 0.3)}
.border-y-base-content\/40 {
    border-top-color: hsl(var(--bc) / 0.4);
    border-bottom-color: hsl(var(--bc) / 0.4)}
.border-y-base-content\/5 {
    border-top-color: hsl(var(--bc) / 0.05);
    border-bottom-color: hsl(var(--bc) / 0.05)}
.border-y-base-content\/50 {
    border-top-color: hsl(var(--bc) / 0.5);
    border-bottom-color: hsl(var(--bc) / 0.5)}
.border-y-base-content\/60 {
    border-top-color: hsl(var(--bc) / 0.6);
    border-bottom-color: hsl(var(--bc) / 0.6)}
.border-y-base-content\/70 {
    border-top-color: hsl(var(--bc) / 0.7);
    border-bottom-color: hsl(var(--bc) / 0.7)}
.border-y-base-content\/75 {
    border-top-color: hsl(var(--bc) / 0.75);
    border-bottom-color: hsl(var(--bc) / 0.75)}
.border-y-base-content\/80 {
    border-top-color: hsl(var(--bc) / 0.8);
    border-bottom-color: hsl(var(--bc) / 0.8)}
.border-y-base-content\/90 {
    border-top-color: hsl(var(--bc) / 0.9);
    border-bottom-color: hsl(var(--bc) / 0.9)}
.border-y-base-content\/95 {
    border-top-color: hsl(var(--bc) / 0.95);
    border-bottom-color: hsl(var(--bc) / 0.95)}
.border-y-current {
    border-top-color: currentColor;
    border-bottom-color: currentColor}
.border-y-error {
    border-top-color: hsl(var(--er) / 1);
    border-bottom-color: hsl(var(--er) / 1)}
.border-y-error-content {
    border-top-color: hsl(var(--erc) / 1);
    border-bottom-color: hsl(var(--erc) / 1)}
.border-y-error-content\/0 {
    border-top-color: hsl(var(--erc) / 0);
    border-bottom-color: hsl(var(--erc) / 0)}
.border-y-error-content\/10 {
    border-top-color: hsl(var(--erc) / 0.1);
    border-bottom-color: hsl(var(--erc) / 0.1)}
.border-y-error-content\/100 {
    border-top-color: hsl(var(--erc) / 1);
    border-bottom-color: hsl(var(--erc) / 1)}
.border-y-error-content\/20 {
    border-top-color: hsl(var(--erc) / 0.2);
    border-bottom-color: hsl(var(--erc) / 0.2)}
.border-y-error-content\/25 {
    border-top-color: hsl(var(--erc) / 0.25);
    border-bottom-color: hsl(var(--erc) / 0.25)}
.border-y-error-content\/30 {
    border-top-color: hsl(var(--erc) / 0.3);
    border-bottom-color: hsl(var(--erc) / 0.3)}
.border-y-error-content\/40 {
    border-top-color: hsl(var(--erc) / 0.4);
    border-bottom-color: hsl(var(--erc) / 0.4)}
.border-y-error-content\/5 {
    border-top-color: hsl(var(--erc) / 0.05);
    border-bottom-color: hsl(var(--erc) / 0.05)}
.border-y-error-content\/50 {
    border-top-color: hsl(var(--erc) / 0.5);
    border-bottom-color: hsl(var(--erc) / 0.5)}
.border-y-error-content\/60 {
    border-top-color: hsl(var(--erc) / 0.6);
    border-bottom-color: hsl(var(--erc) / 0.6)}
.border-y-error-content\/70 {
    border-top-color: hsl(var(--erc) / 0.7);
    border-bottom-color: hsl(var(--erc) / 0.7)}
.border-y-error-content\/75 {
    border-top-color: hsl(var(--erc) / 0.75);
    border-bottom-color: hsl(var(--erc) / 0.75)}
.border-y-error-content\/80 {
    border-top-color: hsl(var(--erc) / 0.8);
    border-bottom-color: hsl(var(--erc) / 0.8)}
.border-y-error-content\/90 {
    border-top-color: hsl(var(--erc) / 0.9);
    border-bottom-color: hsl(var(--erc) / 0.9)}
.border-y-error-content\/95 {
    border-top-color: hsl(var(--erc) / 0.95);
    border-bottom-color: hsl(var(--erc) / 0.95)}
.border-y-error\/0 {
    border-top-color: hsl(var(--er) / 0);
    border-bottom-color: hsl(var(--er) / 0)}
.border-y-error\/10 {
    border-top-color: hsl(var(--er) / 0.1);
    border-bottom-color: hsl(var(--er) / 0.1)}
.border-y-error\/100 {
    border-top-color: hsl(var(--er) / 1);
    border-bottom-color: hsl(var(--er) / 1)}
.border-y-error\/20 {
    border-top-color: hsl(var(--er) / 0.2);
    border-bottom-color: hsl(var(--er) / 0.2)}
.border-y-error\/25 {
    border-top-color: hsl(var(--er) / 0.25);
    border-bottom-color: hsl(var(--er) / 0.25)}
.border-y-error\/30 {
    border-top-color: hsl(var(--er) / 0.3);
    border-bottom-color: hsl(var(--er) / 0.3)}
.border-y-error\/40 {
    border-top-color: hsl(var(--er) / 0.4);
    border-bottom-color: hsl(var(--er) / 0.4)}
.border-y-error\/5 {
    border-top-color: hsl(var(--er) / 0.05);
    border-bottom-color: hsl(var(--er) / 0.05)}
.border-y-error\/50 {
    border-top-color: hsl(var(--er) / 0.5);
    border-bottom-color: hsl(var(--er) / 0.5)}
.border-y-error\/60 {
    border-top-color: hsl(var(--er) / 0.6);
    border-bottom-color: hsl(var(--er) / 0.6)}
.border-y-error\/70 {
    border-top-color: hsl(var(--er) / 0.7);
    border-bottom-color: hsl(var(--er) / 0.7)}
.border-y-error\/75 {
    border-top-color: hsl(var(--er) / 0.75);
    border-bottom-color: hsl(var(--er) / 0.75)}
.border-y-error\/80 {
    border-top-color: hsl(var(--er) / 0.8);
    border-bottom-color: hsl(var(--er) / 0.8)}
.border-y-error\/90 {
    border-top-color: hsl(var(--er) / 0.9);
    border-bottom-color: hsl(var(--er) / 0.9)}
.border-y-error\/95 {
    border-top-color: hsl(var(--er) / 0.95);
    border-bottom-color: hsl(var(--er) / 0.95)}
.border-y-info {
    border-top-color: hsl(var(--in) / 1);
    border-bottom-color: hsl(var(--in) / 1)}
.border-y-info-content {
    border-top-color: hsl(var(--inc) / 1);
    border-bottom-color: hsl(var(--inc) / 1)}
.border-y-info-content\/0 {
    border-top-color: hsl(var(--inc) / 0);
    border-bottom-color: hsl(var(--inc) / 0)}
.border-y-info-content\/10 {
    border-top-color: hsl(var(--inc) / 0.1);
    border-bottom-color: hsl(var(--inc) / 0.1)}
.border-y-info-content\/100 {
    border-top-color: hsl(var(--inc) / 1);
    border-bottom-color: hsl(var(--inc) / 1)}
.border-y-info-content\/20 {
    border-top-color: hsl(var(--inc) / 0.2);
    border-bottom-color: hsl(var(--inc) / 0.2)}
.border-y-info-content\/25 {
    border-top-color: hsl(var(--inc) / 0.25);
    border-bottom-color: hsl(var(--inc) / 0.25)}
.border-y-info-content\/30 {
    border-top-color: hsl(var(--inc) / 0.3);
    border-bottom-color: hsl(var(--inc) / 0.3)}
.border-y-info-content\/40 {
    border-top-color: hsl(var(--inc) / 0.4);
    border-bottom-color: hsl(var(--inc) / 0.4)}
.border-y-info-content\/5 {
    border-top-color: hsl(var(--inc) / 0.05);
    border-bottom-color: hsl(var(--inc) / 0.05)}
.border-y-info-content\/50 {
    border-top-color: hsl(var(--inc) / 0.5);
    border-bottom-color: hsl(var(--inc) / 0.5)}
.border-y-info-content\/60 {
    border-top-color: hsl(var(--inc) / 0.6);
    border-bottom-color: hsl(var(--inc) / 0.6)}
.border-y-info-content\/70 {
    border-top-color: hsl(var(--inc) / 0.7);
    border-bottom-color: hsl(var(--inc) / 0.7)}
.border-y-info-content\/75 {
    border-top-color: hsl(var(--inc) / 0.75);
    border-bottom-color: hsl(var(--inc) / 0.75)}
.border-y-info-content\/80 {
    border-top-color: hsl(var(--inc) / 0.8);
    border-bottom-color: hsl(var(--inc) / 0.8)}
.border-y-info-content\/90 {
    border-top-color: hsl(var(--inc) / 0.9);
    border-bottom-color: hsl(var(--inc) / 0.9)}
.border-y-info-content\/95 {
    border-top-color: hsl(var(--inc) / 0.95);
    border-bottom-color: hsl(var(--inc) / 0.95)}
.border-y-info\/0 {
    border-top-color: hsl(var(--in) / 0);
    border-bottom-color: hsl(var(--in) / 0)}
.border-y-info\/10 {
    border-top-color: hsl(var(--in) / 0.1);
    border-bottom-color: hsl(var(--in) / 0.1)}
.border-y-info\/100 {
    border-top-color: hsl(var(--in) / 1);
    border-bottom-color: hsl(var(--in) / 1)}
.border-y-info\/20 {
    border-top-color: hsl(var(--in) / 0.2);
    border-bottom-color: hsl(var(--in) / 0.2)}
.border-y-info\/25 {
    border-top-color: hsl(var(--in) / 0.25);
    border-bottom-color: hsl(var(--in) / 0.25)}
.border-y-info\/30 {
    border-top-color: hsl(var(--in) / 0.3);
    border-bottom-color: hsl(var(--in) / 0.3)}
.border-y-info\/40 {
    border-top-color: hsl(var(--in) / 0.4);
    border-bottom-color: hsl(var(--in) / 0.4)}
.border-y-info\/5 {
    border-top-color: hsl(var(--in) / 0.05);
    border-bottom-color: hsl(var(--in) / 0.05)}
.border-y-info\/50 {
    border-top-color: hsl(var(--in) / 0.5);
    border-bottom-color: hsl(var(--in) / 0.5)}
.border-y-info\/60 {
    border-top-color: hsl(var(--in) / 0.6);
    border-bottom-color: hsl(var(--in) / 0.6)}
.border-y-info\/70 {
    border-top-color: hsl(var(--in) / 0.7);
    border-bottom-color: hsl(var(--in) / 0.7)}
.border-y-info\/75 {
    border-top-color: hsl(var(--in) / 0.75);
    border-bottom-color: hsl(var(--in) / 0.75)}
.border-y-info\/80 {
    border-top-color: hsl(var(--in) / 0.8);
    border-bottom-color: hsl(var(--in) / 0.8)}
.border-y-info\/90 {
    border-top-color: hsl(var(--in) / 0.9);
    border-bottom-color: hsl(var(--in) / 0.9)}
.border-y-info\/95 {
    border-top-color: hsl(var(--in) / 0.95);
    border-bottom-color: hsl(var(--in) / 0.95)}
.border-y-neutral {
    border-top-color: hsl(var(--n) / 1);
    border-bottom-color: hsl(var(--n) / 1)}
.border-y-neutral-content {
    border-top-color: hsl(var(--nc) / 1);
    border-bottom-color: hsl(var(--nc) / 1)}
.border-y-neutral-content\/0 {
    border-top-color: hsl(var(--nc) / 0);
    border-bottom-color: hsl(var(--nc) / 0)}
.border-y-neutral-content\/10 {
    border-top-color: hsl(var(--nc) / 0.1);
    border-bottom-color: hsl(var(--nc) / 0.1)}
.border-y-neutral-content\/100 {
    border-top-color: hsl(var(--nc) / 1);
    border-bottom-color: hsl(var(--nc) / 1)}
.border-y-neutral-content\/20 {
    border-top-color: hsl(var(--nc) / 0.2);
    border-bottom-color: hsl(var(--nc) / 0.2)}
.border-y-neutral-content\/25 {
    border-top-color: hsl(var(--nc) / 0.25);
    border-bottom-color: hsl(var(--nc) / 0.25)}
.border-y-neutral-content\/30 {
    border-top-color: hsl(var(--nc) / 0.3);
    border-bottom-color: hsl(var(--nc) / 0.3)}
.border-y-neutral-content\/40 {
    border-top-color: hsl(var(--nc) / 0.4);
    border-bottom-color: hsl(var(--nc) / 0.4)}
.border-y-neutral-content\/5 {
    border-top-color: hsl(var(--nc) / 0.05);
    border-bottom-color: hsl(var(--nc) / 0.05)}
.border-y-neutral-content\/50 {
    border-top-color: hsl(var(--nc) / 0.5);
    border-bottom-color: hsl(var(--nc) / 0.5)}
.border-y-neutral-content\/60 {
    border-top-color: hsl(var(--nc) / 0.6);
    border-bottom-color: hsl(var(--nc) / 0.6)}
.border-y-neutral-content\/70 {
    border-top-color: hsl(var(--nc) / 0.7);
    border-bottom-color: hsl(var(--nc) / 0.7)}
.border-y-neutral-content\/75 {
    border-top-color: hsl(var(--nc) / 0.75);
    border-bottom-color: hsl(var(--nc) / 0.75)}
.border-y-neutral-content\/80 {
    border-top-color: hsl(var(--nc) / 0.8);
    border-bottom-color: hsl(var(--nc) / 0.8)}
.border-y-neutral-content\/90 {
    border-top-color: hsl(var(--nc) / 0.9);
    border-bottom-color: hsl(var(--nc) / 0.9)}
.border-y-neutral-content\/95 {
    border-top-color: hsl(var(--nc) / 0.95);
    border-bottom-color: hsl(var(--nc) / 0.95)}
.border-y-neutral-focus {
    border-top-color: hsl(var(--nf) / 1);
    border-bottom-color: hsl(var(--nf) / 1)}
.border-y-neutral-focus\/0 {
    border-top-color: hsl(var(--nf) / 0);
    border-bottom-color: hsl(var(--nf) / 0)}
.border-y-neutral-focus\/10 {
    border-top-color: hsl(var(--nf) / 0.1);
    border-bottom-color: hsl(var(--nf) / 0.1)}
.border-y-neutral-focus\/100 {
    border-top-color: hsl(var(--nf) / 1);
    border-bottom-color: hsl(var(--nf) / 1)}
.border-y-neutral-focus\/20 {
    border-top-color: hsl(var(--nf) / 0.2);
    border-bottom-color: hsl(var(--nf) / 0.2)}
.border-y-neutral-focus\/25 {
    border-top-color: hsl(var(--nf) / 0.25);
    border-bottom-color: hsl(var(--nf) / 0.25)}
.border-y-neutral-focus\/30 {
    border-top-color: hsl(var(--nf) / 0.3);
    border-bottom-color: hsl(var(--nf) / 0.3)}
.border-y-neutral-focus\/40 {
    border-top-color: hsl(var(--nf) / 0.4);
    border-bottom-color: hsl(var(--nf) / 0.4)}
.border-y-neutral-focus\/5 {
    border-top-color: hsl(var(--nf) / 0.05);
    border-bottom-color: hsl(var(--nf) / 0.05)}
.border-y-neutral-focus\/50 {
    border-top-color: hsl(var(--nf) / 0.5);
    border-bottom-color: hsl(var(--nf) / 0.5)}
.border-y-neutral-focus\/60 {
    border-top-color: hsl(var(--nf) / 0.6);
    border-bottom-color: hsl(var(--nf) / 0.6)}
.border-y-neutral-focus\/70 {
    border-top-color: hsl(var(--nf) / 0.7);
    border-bottom-color: hsl(var(--nf) / 0.7)}
.border-y-neutral-focus\/75 {
    border-top-color: hsl(var(--nf) / 0.75);
    border-bottom-color: hsl(var(--nf) / 0.75)}
.border-y-neutral-focus\/80 {
    border-top-color: hsl(var(--nf) / 0.8);
    border-bottom-color: hsl(var(--nf) / 0.8)}
.border-y-neutral-focus\/90 {
    border-top-color: hsl(var(--nf) / 0.9);
    border-bottom-color: hsl(var(--nf) / 0.9)}
.border-y-neutral-focus\/95 {
    border-top-color: hsl(var(--nf) / 0.95);
    border-bottom-color: hsl(var(--nf) / 0.95)}
.border-y-neutral\/0 {
    border-top-color: hsl(var(--n) / 0);
    border-bottom-color: hsl(var(--n) / 0)}
.border-y-neutral\/10 {
    border-top-color: hsl(var(--n) / 0.1);
    border-bottom-color: hsl(var(--n) / 0.1)}
.border-y-neutral\/100 {
    border-top-color: hsl(var(--n) / 1);
    border-bottom-color: hsl(var(--n) / 1)}
.border-y-neutral\/20 {
    border-top-color: hsl(var(--n) / 0.2);
    border-bottom-color: hsl(var(--n) / 0.2)}
.border-y-neutral\/25 {
    border-top-color: hsl(var(--n) / 0.25);
    border-bottom-color: hsl(var(--n) / 0.25)}
.border-y-neutral\/30 {
    border-top-color: hsl(var(--n) / 0.3);
    border-bottom-color: hsl(var(--n) / 0.3)}
.border-y-neutral\/40 {
    border-top-color: hsl(var(--n) / 0.4);
    border-bottom-color: hsl(var(--n) / 0.4)}
.border-y-neutral\/5 {
    border-top-color: hsl(var(--n) / 0.05);
    border-bottom-color: hsl(var(--n) / 0.05)}
.border-y-neutral\/50 {
    border-top-color: hsl(var(--n) / 0.5);
    border-bottom-color: hsl(var(--n) / 0.5)}
.border-y-neutral\/60 {
    border-top-color: hsl(var(--n) / 0.6);
    border-bottom-color: hsl(var(--n) / 0.6)}
.border-y-neutral\/70 {
    border-top-color: hsl(var(--n) / 0.7);
    border-bottom-color: hsl(var(--n) / 0.7)}
.border-y-neutral\/75 {
    border-top-color: hsl(var(--n) / 0.75);
    border-bottom-color: hsl(var(--n) / 0.75)}
.border-y-neutral\/80 {
    border-top-color: hsl(var(--n) / 0.8);
    border-bottom-color: hsl(var(--n) / 0.8)}
.border-y-neutral\/90 {
    border-top-color: hsl(var(--n) / 0.9);
    border-bottom-color: hsl(var(--n) / 0.9)}
.border-y-neutral\/95 {
    border-top-color: hsl(var(--n) / 0.95);
    border-bottom-color: hsl(var(--n) / 0.95)}
.border-y-primary {
    border-top-color: hsl(var(--p) / 1);
    border-bottom-color: hsl(var(--p) / 1)}
.border-y-primary-content {
    border-top-color: hsl(var(--pc) / 1);
    border-bottom-color: hsl(var(--pc) / 1)}
.border-y-primary-content\/0 {
    border-top-color: hsl(var(--pc) / 0);
    border-bottom-color: hsl(var(--pc) / 0)}
.border-y-primary-content\/10 {
    border-top-color: hsl(var(--pc) / 0.1);
    border-bottom-color: hsl(var(--pc) / 0.1)}
.border-y-primary-content\/100 {
    border-top-color: hsl(var(--pc) / 1);
    border-bottom-color: hsl(var(--pc) / 1)}
.border-y-primary-content\/20 {
    border-top-color: hsl(var(--pc) / 0.2);
    border-bottom-color: hsl(var(--pc) / 0.2)}
.border-y-primary-content\/25 {
    border-top-color: hsl(var(--pc) / 0.25);
    border-bottom-color: hsl(var(--pc) / 0.25)}
.border-y-primary-content\/30 {
    border-top-color: hsl(var(--pc) / 0.3);
    border-bottom-color: hsl(var(--pc) / 0.3)}
.border-y-primary-content\/40 {
    border-top-color: hsl(var(--pc) / 0.4);
    border-bottom-color: hsl(var(--pc) / 0.4)}
.border-y-primary-content\/5 {
    border-top-color: hsl(var(--pc) / 0.05);
    border-bottom-color: hsl(var(--pc) / 0.05)}
.border-y-primary-content\/50 {
    border-top-color: hsl(var(--pc) / 0.5);
    border-bottom-color: hsl(var(--pc) / 0.5)}
.border-y-primary-content\/60 {
    border-top-color: hsl(var(--pc) / 0.6);
    border-bottom-color: hsl(var(--pc) / 0.6)}
.border-y-primary-content\/70 {
    border-top-color: hsl(var(--pc) / 0.7);
    border-bottom-color: hsl(var(--pc) / 0.7)}
.border-y-primary-content\/75 {
    border-top-color: hsl(var(--pc) / 0.75);
    border-bottom-color: hsl(var(--pc) / 0.75)}
.border-y-primary-content\/80 {
    border-top-color: hsl(var(--pc) / 0.8);
    border-bottom-color: hsl(var(--pc) / 0.8)}
.border-y-primary-content\/90 {
    border-top-color: hsl(var(--pc) / 0.9);
    border-bottom-color: hsl(var(--pc) / 0.9)}
.border-y-primary-content\/95 {
    border-top-color: hsl(var(--pc) / 0.95);
    border-bottom-color: hsl(var(--pc) / 0.95)}
.border-y-primary-focus {
    border-top-color: hsl(var(--pf) / 1);
    border-bottom-color: hsl(var(--pf) / 1)}
.border-y-primary-focus\/0 {
    border-top-color: hsl(var(--pf) / 0);
    border-bottom-color: hsl(var(--pf) / 0)}
.border-y-primary-focus\/10 {
    border-top-color: hsl(var(--pf) / 0.1);
    border-bottom-color: hsl(var(--pf) / 0.1)}
.border-y-primary-focus\/100 {
    border-top-color: hsl(var(--pf) / 1);
    border-bottom-color: hsl(var(--pf) / 1)}
.border-y-primary-focus\/20 {
    border-top-color: hsl(var(--pf) / 0.2);
    border-bottom-color: hsl(var(--pf) / 0.2)}
.border-y-primary-focus\/25 {
    border-top-color: hsl(var(--pf) / 0.25);
    border-bottom-color: hsl(var(--pf) / 0.25)}
.border-y-primary-focus\/30 {
    border-top-color: hsl(var(--pf) / 0.3);
    border-bottom-color: hsl(var(--pf) / 0.3)}
.border-y-primary-focus\/40 {
    border-top-color: hsl(var(--pf) / 0.4);
    border-bottom-color: hsl(var(--pf) / 0.4)}
.border-y-primary-focus\/5 {
    border-top-color: hsl(var(--pf) / 0.05);
    border-bottom-color: hsl(var(--pf) / 0.05)}
.border-y-primary-focus\/50 {
    border-top-color: hsl(var(--pf) / 0.5);
    border-bottom-color: hsl(var(--pf) / 0.5)}
.border-y-primary-focus\/60 {
    border-top-color: hsl(var(--pf) / 0.6);
    border-bottom-color: hsl(var(--pf) / 0.6)}
.border-y-primary-focus\/70 {
    border-top-color: hsl(var(--pf) / 0.7);
    border-bottom-color: hsl(var(--pf) / 0.7)}
.border-y-primary-focus\/75 {
    border-top-color: hsl(var(--pf) / 0.75);
    border-bottom-color: hsl(var(--pf) / 0.75)}
.border-y-primary-focus\/80 {
    border-top-color: hsl(var(--pf) / 0.8);
    border-bottom-color: hsl(var(--pf) / 0.8)}
.border-y-primary-focus\/90 {
    border-top-color: hsl(var(--pf) / 0.9);
    border-bottom-color: hsl(var(--pf) / 0.9)}
.border-y-primary-focus\/95 {
    border-top-color: hsl(var(--pf) / 0.95);
    border-bottom-color: hsl(var(--pf) / 0.95)}
.border-y-primary\/0 {
    border-top-color: hsl(var(--p) / 0);
    border-bottom-color: hsl(var(--p) / 0)}
.border-y-primary\/10 {
    border-top-color: hsl(var(--p) / 0.1);
    border-bottom-color: hsl(var(--p) / 0.1)}
.border-y-primary\/100 {
    border-top-color: hsl(var(--p) / 1);
    border-bottom-color: hsl(var(--p) / 1)}
.border-y-primary\/20 {
    border-top-color: hsl(var(--p) / 0.2);
    border-bottom-color: hsl(var(--p) / 0.2)}
.border-y-primary\/25 {
    border-top-color: hsl(var(--p) / 0.25);
    border-bottom-color: hsl(var(--p) / 0.25)}
.border-y-primary\/30 {
    border-top-color: hsl(var(--p) / 0.3);
    border-bottom-color: hsl(var(--p) / 0.3)}
.border-y-primary\/40 {
    border-top-color: hsl(var(--p) / 0.4);
    border-bottom-color: hsl(var(--p) / 0.4)}
.border-y-primary\/5 {
    border-top-color: hsl(var(--p) / 0.05);
    border-bottom-color: hsl(var(--p) / 0.05)}
.border-y-primary\/50 {
    border-top-color: hsl(var(--p) / 0.5);
    border-bottom-color: hsl(var(--p) / 0.5)}
.border-y-primary\/60 {
    border-top-color: hsl(var(--p) / 0.6);
    border-bottom-color: hsl(var(--p) / 0.6)}
.border-y-primary\/70 {
    border-top-color: hsl(var(--p) / 0.7);
    border-bottom-color: hsl(var(--p) / 0.7)}
.border-y-primary\/75 {
    border-top-color: hsl(var(--p) / 0.75);
    border-bottom-color: hsl(var(--p) / 0.75)}
.border-y-primary\/80 {
    border-top-color: hsl(var(--p) / 0.8);
    border-bottom-color: hsl(var(--p) / 0.8)}
.border-y-primary\/90 {
    border-top-color: hsl(var(--p) / 0.9);
    border-bottom-color: hsl(var(--p) / 0.9)}
.border-y-primary\/95 {
    border-top-color: hsl(var(--p) / 0.95);
    border-bottom-color: hsl(var(--p) / 0.95)}
.border-y-secondary {
    border-top-color: hsl(var(--s) / 1);
    border-bottom-color: hsl(var(--s) / 1)}
.border-y-secondary-content {
    border-top-color: hsl(var(--sc) / 1);
    border-bottom-color: hsl(var(--sc) / 1)}
.border-y-secondary-content\/0 {
    border-top-color: hsl(var(--sc) / 0);
    border-bottom-color: hsl(var(--sc) / 0)}
.border-y-secondary-content\/10 {
    border-top-color: hsl(var(--sc) / 0.1);
    border-bottom-color: hsl(var(--sc) / 0.1)}
.border-y-secondary-content\/100 {
    border-top-color: hsl(var(--sc) / 1);
    border-bottom-color: hsl(var(--sc) / 1)}
.border-y-secondary-content\/20 {
    border-top-color: hsl(var(--sc) / 0.2);
    border-bottom-color: hsl(var(--sc) / 0.2)}
.border-y-secondary-content\/25 {
    border-top-color: hsl(var(--sc) / 0.25);
    border-bottom-color: hsl(var(--sc) / 0.25)}
.border-y-secondary-content\/30 {
    border-top-color: hsl(var(--sc) / 0.3);
    border-bottom-color: hsl(var(--sc) / 0.3)}
.border-y-secondary-content\/40 {
    border-top-color: hsl(var(--sc) / 0.4);
    border-bottom-color: hsl(var(--sc) / 0.4)}
.border-y-secondary-content\/5 {
    border-top-color: hsl(var(--sc) / 0.05);
    border-bottom-color: hsl(var(--sc) / 0.05)}
.border-y-secondary-content\/50 {
    border-top-color: hsl(var(--sc) / 0.5);
    border-bottom-color: hsl(var(--sc) / 0.5)}
.border-y-secondary-content\/60 {
    border-top-color: hsl(var(--sc) / 0.6);
    border-bottom-color: hsl(var(--sc) / 0.6)}
.border-y-secondary-content\/70 {
    border-top-color: hsl(var(--sc) / 0.7);
    border-bottom-color: hsl(var(--sc) / 0.7)}
.border-y-secondary-content\/75 {
    border-top-color: hsl(var(--sc) / 0.75);
    border-bottom-color: hsl(var(--sc) / 0.75)}
.border-y-secondary-content\/80 {
    border-top-color: hsl(var(--sc) / 0.8);
    border-bottom-color: hsl(var(--sc) / 0.8)}
.border-y-secondary-content\/90 {
    border-top-color: hsl(var(--sc) / 0.9);
    border-bottom-color: hsl(var(--sc) / 0.9)}
.border-y-secondary-content\/95 {
    border-top-color: hsl(var(--sc) / 0.95);
    border-bottom-color: hsl(var(--sc) / 0.95)}
.border-y-secondary-focus {
    border-top-color: hsl(var(--sf) / 1);
    border-bottom-color: hsl(var(--sf) / 1)}
.border-y-secondary-focus\/0 {
    border-top-color: hsl(var(--sf) / 0);
    border-bottom-color: hsl(var(--sf) / 0)}
.border-y-secondary-focus\/10 {
    border-top-color: hsl(var(--sf) / 0.1);
    border-bottom-color: hsl(var(--sf) / 0.1)}
.border-y-secondary-focus\/100 {
    border-top-color: hsl(var(--sf) / 1);
    border-bottom-color: hsl(var(--sf) / 1)}
.border-y-secondary-focus\/20 {
    border-top-color: hsl(var(--sf) / 0.2);
    border-bottom-color: hsl(var(--sf) / 0.2)}
.border-y-secondary-focus\/25 {
    border-top-color: hsl(var(--sf) / 0.25);
    border-bottom-color: hsl(var(--sf) / 0.25)}
.border-y-secondary-focus\/30 {
    border-top-color: hsl(var(--sf) / 0.3);
    border-bottom-color: hsl(var(--sf) / 0.3)}
.border-y-secondary-focus\/40 {
    border-top-color: hsl(var(--sf) / 0.4);
    border-bottom-color: hsl(var(--sf) / 0.4)}
.border-y-secondary-focus\/5 {
    border-top-color: hsl(var(--sf) / 0.05);
    border-bottom-color: hsl(var(--sf) / 0.05)}
.border-y-secondary-focus\/50 {
    border-top-color: hsl(var(--sf) / 0.5);
    border-bottom-color: hsl(var(--sf) / 0.5)}
.border-y-secondary-focus\/60 {
    border-top-color: hsl(var(--sf) / 0.6);
    border-bottom-color: hsl(var(--sf) / 0.6)}
.border-y-secondary-focus\/70 {
    border-top-color: hsl(var(--sf) / 0.7);
    border-bottom-color: hsl(var(--sf) / 0.7)}
.border-y-secondary-focus\/75 {
    border-top-color: hsl(var(--sf) / 0.75);
    border-bottom-color: hsl(var(--sf) / 0.75)}
.border-y-secondary-focus\/80 {
    border-top-color: hsl(var(--sf) / 0.8);
    border-bottom-color: hsl(var(--sf) / 0.8)}
.border-y-secondary-focus\/90 {
    border-top-color: hsl(var(--sf) / 0.9);
    border-bottom-color: hsl(var(--sf) / 0.9)}
.border-y-secondary-focus\/95 {
    border-top-color: hsl(var(--sf) / 0.95);
    border-bottom-color: hsl(var(--sf) / 0.95)}
.border-y-secondary\/0 {
    border-top-color: hsl(var(--s) / 0);
    border-bottom-color: hsl(var(--s) / 0)}
.border-y-secondary\/10 {
    border-top-color: hsl(var(--s) / 0.1);
    border-bottom-color: hsl(var(--s) / 0.1)}
.border-y-secondary\/100 {
    border-top-color: hsl(var(--s) / 1);
    border-bottom-color: hsl(var(--s) / 1)}
.border-y-secondary\/20 {
    border-top-color: hsl(var(--s) / 0.2);
    border-bottom-color: hsl(var(--s) / 0.2)}
.border-y-secondary\/25 {
    border-top-color: hsl(var(--s) / 0.25);
    border-bottom-color: hsl(var(--s) / 0.25)}
.border-y-secondary\/30 {
    border-top-color: hsl(var(--s) / 0.3);
    border-bottom-color: hsl(var(--s) / 0.3)}
.border-y-secondary\/40 {
    border-top-color: hsl(var(--s) / 0.4);
    border-bottom-color: hsl(var(--s) / 0.4)}
.border-y-secondary\/5 {
    border-top-color: hsl(var(--s) / 0.05);
    border-bottom-color: hsl(var(--s) / 0.05)}
.border-y-secondary\/50 {
    border-top-color: hsl(var(--s) / 0.5);
    border-bottom-color: hsl(var(--s) / 0.5)}
.border-y-secondary\/60 {
    border-top-color: hsl(var(--s) / 0.6);
    border-bottom-color: hsl(var(--s) / 0.6)}
.border-y-secondary\/70 {
    border-top-color: hsl(var(--s) / 0.7);
    border-bottom-color: hsl(var(--s) / 0.7)}
.border-y-secondary\/75 {
    border-top-color: hsl(var(--s) / 0.75);
    border-bottom-color: hsl(var(--s) / 0.75)}
.border-y-secondary\/80 {
    border-top-color: hsl(var(--s) / 0.8);
    border-bottom-color: hsl(var(--s) / 0.8)}
.border-y-secondary\/90 {
    border-top-color: hsl(var(--s) / 0.9);
    border-bottom-color: hsl(var(--s) / 0.9)}
.border-y-secondary\/95 {
    border-top-color: hsl(var(--s) / 0.95);
    border-bottom-color: hsl(var(--s) / 0.95)}
.border-y-success {
    border-top-color: hsl(var(--su) / 1);
    border-bottom-color: hsl(var(--su) / 1)}
.border-y-success-content {
    border-top-color: hsl(var(--suc) / 1);
    border-bottom-color: hsl(var(--suc) / 1)}
.border-y-success-content\/0 {
    border-top-color: hsl(var(--suc) / 0);
    border-bottom-color: hsl(var(--suc) / 0)}
.border-y-success-content\/10 {
    border-top-color: hsl(var(--suc) / 0.1);
    border-bottom-color: hsl(var(--suc) / 0.1)}
.border-y-success-content\/100 {
    border-top-color: hsl(var(--suc) / 1);
    border-bottom-color: hsl(var(--suc) / 1)}
.border-y-success-content\/20 {
    border-top-color: hsl(var(--suc) / 0.2);
    border-bottom-color: hsl(var(--suc) / 0.2)}
.border-y-success-content\/25 {
    border-top-color: hsl(var(--suc) / 0.25);
    border-bottom-color: hsl(var(--suc) / 0.25)}
.border-y-success-content\/30 {
    border-top-color: hsl(var(--suc) / 0.3);
    border-bottom-color: hsl(var(--suc) / 0.3)}
.border-y-success-content\/40 {
    border-top-color: hsl(var(--suc) / 0.4);
    border-bottom-color: hsl(var(--suc) / 0.4)}
.border-y-success-content\/5 {
    border-top-color: hsl(var(--suc) / 0.05);
    border-bottom-color: hsl(var(--suc) / 0.05)}
.border-y-success-content\/50 {
    border-top-color: hsl(var(--suc) / 0.5);
    border-bottom-color: hsl(var(--suc) / 0.5)}
.border-y-success-content\/60 {
    border-top-color: hsl(var(--suc) / 0.6);
    border-bottom-color: hsl(var(--suc) / 0.6)}
.border-y-success-content\/70 {
    border-top-color: hsl(var(--suc) / 0.7);
    border-bottom-color: hsl(var(--suc) / 0.7)}
.border-y-success-content\/75 {
    border-top-color: hsl(var(--suc) / 0.75);
    border-bottom-color: hsl(var(--suc) / 0.75)}
.border-y-success-content\/80 {
    border-top-color: hsl(var(--suc) / 0.8);
    border-bottom-color: hsl(var(--suc) / 0.8)}
.border-y-success-content\/90 {
    border-top-color: hsl(var(--suc) / 0.9);
    border-bottom-color: hsl(var(--suc) / 0.9)}
.border-y-success-content\/95 {
    border-top-color: hsl(var(--suc) / 0.95);
    border-bottom-color: hsl(var(--suc) / 0.95)}
.border-y-success\/0 {
    border-top-color: hsl(var(--su) / 0);
    border-bottom-color: hsl(var(--su) / 0)}
.border-y-success\/10 {
    border-top-color: hsl(var(--su) / 0.1);
    border-bottom-color: hsl(var(--su) / 0.1)}
.border-y-success\/100 {
    border-top-color: hsl(var(--su) / 1);
    border-bottom-color: hsl(var(--su) / 1)}
.border-y-success\/20 {
    border-top-color: hsl(var(--su) / 0.2);
    border-bottom-color: hsl(var(--su) / 0.2)}
.border-y-success\/25 {
    border-top-color: hsl(var(--su) / 0.25);
    border-bottom-color: hsl(var(--su) / 0.25)}
.border-y-success\/30 {
    border-top-color: hsl(var(--su) / 0.3);
    border-bottom-color: hsl(var(--su) / 0.3)}
.border-y-success\/40 {
    border-top-color: hsl(var(--su) / 0.4);
    border-bottom-color: hsl(var(--su) / 0.4)}
.border-y-success\/5 {
    border-top-color: hsl(var(--su) / 0.05);
    border-bottom-color: hsl(var(--su) / 0.05)}
.border-y-success\/50 {
    border-top-color: hsl(var(--su) / 0.5);
    border-bottom-color: hsl(var(--su) / 0.5)}
.border-y-success\/60 {
    border-top-color: hsl(var(--su) / 0.6);
    border-bottom-color: hsl(var(--su) / 0.6)}
.border-y-success\/70 {
    border-top-color: hsl(var(--su) / 0.7);
    border-bottom-color: hsl(var(--su) / 0.7)}
.border-y-success\/75 {
    border-top-color: hsl(var(--su) / 0.75);
    border-bottom-color: hsl(var(--su) / 0.75)}
.border-y-success\/80 {
    border-top-color: hsl(var(--su) / 0.8);
    border-bottom-color: hsl(var(--su) / 0.8)}
.border-y-success\/90 {
    border-top-color: hsl(var(--su) / 0.9);
    border-bottom-color: hsl(var(--su) / 0.9)}
.border-y-success\/95 {
    border-top-color: hsl(var(--su) / 0.95);
    border-bottom-color: hsl(var(--su) / 0.95)}
.border-y-transparent {
    border-top-color: transparent;
    border-bottom-color: transparent}
.border-y-transparent\/0 {
    border-top-color: rgb(0 0 0 / 0);
    border-bottom-color: rgb(0 0 0 / 0)}
.border-y-transparent\/10 {
    border-top-color: rgb(0 0 0 / 0.1);
    border-bottom-color: rgb(0 0 0 / 0.1)}
.border-y-transparent\/100 {
    border-top-color: rgb(0 0 0 / 1);
    border-bottom-color: rgb(0 0 0 / 1)}
.border-y-transparent\/20 {
    border-top-color: rgb(0 0 0 / 0.2);
    border-bottom-color: rgb(0 0 0 / 0.2)}
.border-y-transparent\/25 {
    border-top-color: rgb(0 0 0 / 0.25);
    border-bottom-color: rgb(0 0 0 / 0.25)}
.border-y-transparent\/30 {
    border-top-color: rgb(0 0 0 / 0.3);
    border-bottom-color: rgb(0 0 0 / 0.3)}
.border-y-transparent\/40 {
    border-top-color: rgb(0 0 0 / 0.4);
    border-bottom-color: rgb(0 0 0 / 0.4)}
.border-y-transparent\/5 {
    border-top-color: rgb(0 0 0 / 0.05);
    border-bottom-color: rgb(0 0 0 / 0.05)}
.border-y-transparent\/50 {
    border-top-color: rgb(0 0 0 / 0.5);
    border-bottom-color: rgb(0 0 0 / 0.5)}
.border-y-transparent\/60 {
    border-top-color: rgb(0 0 0 / 0.6);
    border-bottom-color: rgb(0 0 0 / 0.6)}
.border-y-transparent\/70 {
    border-top-color: rgb(0 0 0 / 0.7);
    border-bottom-color: rgb(0 0 0 / 0.7)}
.border-y-transparent\/75 {
    border-top-color: rgb(0 0 0 / 0.75);
    border-bottom-color: rgb(0 0 0 / 0.75)}
.border-y-transparent\/80 {
    border-top-color: rgb(0 0 0 / 0.8);
    border-bottom-color: rgb(0 0 0 / 0.8)}
.border-y-transparent\/90 {
    border-top-color: rgb(0 0 0 / 0.9);
    border-bottom-color: rgb(0 0 0 / 0.9)}
.border-y-transparent\/95 {
    border-top-color: rgb(0 0 0 / 0.95);
    border-bottom-color: rgb(0 0 0 / 0.95)}
.border-y-warning {
    border-top-color: hsl(var(--wa) / 1);
    border-bottom-color: hsl(var(--wa) / 1)}
.border-y-warning-content {
    border-top-color: hsl(var(--wac) / 1);
    border-bottom-color: hsl(var(--wac) / 1)}
.border-y-warning-content\/0 {
    border-top-color: hsl(var(--wac) / 0);
    border-bottom-color: hsl(var(--wac) / 0)}
.border-y-warning-content\/10 {
    border-top-color: hsl(var(--wac) / 0.1);
    border-bottom-color: hsl(var(--wac) / 0.1)}
.border-y-warning-content\/100 {
    border-top-color: hsl(var(--wac) / 1);
    border-bottom-color: hsl(var(--wac) / 1)}
.border-y-warning-content\/20 {
    border-top-color: hsl(var(--wac) / 0.2);
    border-bottom-color: hsl(var(--wac) / 0.2)}
.border-y-warning-content\/25 {
    border-top-color: hsl(var(--wac) / 0.25);
    border-bottom-color: hsl(var(--wac) / 0.25)}
.border-y-warning-content\/30 {
    border-top-color: hsl(var(--wac) / 0.3);
    border-bottom-color: hsl(var(--wac) / 0.3)}
.border-y-warning-content\/40 {
    border-top-color: hsl(var(--wac) / 0.4);
    border-bottom-color: hsl(var(--wac) / 0.4)}
.border-y-warning-content\/5 {
    border-top-color: hsl(var(--wac) / 0.05);
    border-bottom-color: hsl(var(--wac) / 0.05)}
.border-y-warning-content\/50 {
    border-top-color: hsl(var(--wac) / 0.5);
    border-bottom-color: hsl(var(--wac) / 0.5)}
.border-y-warning-content\/60 {
    border-top-color: hsl(var(--wac) / 0.6);
    border-bottom-color: hsl(var(--wac) / 0.6)}
.border-y-warning-content\/70 {
    border-top-color: hsl(var(--wac) / 0.7);
    border-bottom-color: hsl(var(--wac) / 0.7)}
.border-y-warning-content\/75 {
    border-top-color: hsl(var(--wac) / 0.75);
    border-bottom-color: hsl(var(--wac) / 0.75)}
.border-y-warning-content\/80 {
    border-top-color: hsl(var(--wac) / 0.8);
    border-bottom-color: hsl(var(--wac) / 0.8)}
.border-y-warning-content\/90 {
    border-top-color: hsl(var(--wac) / 0.9);
    border-bottom-color: hsl(var(--wac) / 0.9)}
.border-y-warning-content\/95 {
    border-top-color: hsl(var(--wac) / 0.95);
    border-bottom-color: hsl(var(--wac) / 0.95)}
.border-y-warning\/0 {
    border-top-color: hsl(var(--wa) / 0);
    border-bottom-color: hsl(var(--wa) / 0)}
.border-y-warning\/10 {
    border-top-color: hsl(var(--wa) / 0.1);
    border-bottom-color: hsl(var(--wa) / 0.1)}
.border-y-warning\/100 {
    border-top-color: hsl(var(--wa) / 1);
    border-bottom-color: hsl(var(--wa) / 1)}
.border-y-warning\/20 {
    border-top-color: hsl(var(--wa) / 0.2);
    border-bottom-color: hsl(var(--wa) / 0.2)}
.border-y-warning\/25 {
    border-top-color: hsl(var(--wa) / 0.25);
    border-bottom-color: hsl(var(--wa) / 0.25)}
.border-y-warning\/30 {
    border-top-color: hsl(var(--wa) / 0.3);
    border-bottom-color: hsl(var(--wa) / 0.3)}
.border-y-warning\/40 {
    border-top-color: hsl(var(--wa) / 0.4);
    border-bottom-color: hsl(var(--wa) / 0.4)}
.border-y-warning\/5 {
    border-top-color: hsl(var(--wa) / 0.05);
    border-bottom-color: hsl(var(--wa) / 0.05)}
.border-y-warning\/50 {
    border-top-color: hsl(var(--wa) / 0.5);
    border-bottom-color: hsl(var(--wa) / 0.5)}
.border-y-warning\/60 {
    border-top-color: hsl(var(--wa) / 0.6);
    border-bottom-color: hsl(var(--wa) / 0.6)}
.border-y-warning\/70 {
    border-top-color: hsl(var(--wa) / 0.7);
    border-bottom-color: hsl(var(--wa) / 0.7)}
.border-y-warning\/75 {
    border-top-color: hsl(var(--wa) / 0.75);
    border-bottom-color: hsl(var(--wa) / 0.75)}
.border-y-warning\/80 {
    border-top-color: hsl(var(--wa) / 0.8);
    border-bottom-color: hsl(var(--wa) / 0.8)}
.border-y-warning\/90 {
    border-top-color: hsl(var(--wa) / 0.9);
    border-bottom-color: hsl(var(--wa) / 0.9)}
.border-y-warning\/95 {
    border-top-color: hsl(var(--wa) / 0.95);
    border-bottom-color: hsl(var(--wa) / 0.95)}
.border-b-accent {
    border-bottom-color: hsl(var(--a) / 1)}
.border-b-accent-content {
    border-bottom-color: hsl(var(--ac) / 1)}
.border-b-accent-content\/0 {
    border-bottom-color: hsl(var(--ac) / 0)}
.border-b-accent-content\/10 {
    border-bottom-color: hsl(var(--ac) / 0.1)}
.border-b-accent-content\/100 {
    border-bottom-color: hsl(var(--ac) / 1)}
.border-b-accent-content\/20 {
    border-bottom-color: hsl(var(--ac) / 0.2)}
.border-b-accent-content\/25 {
    border-bottom-color: hsl(var(--ac) / 0.25)}
.border-b-accent-content\/30 {
    border-bottom-color: hsl(var(--ac) / 0.3)}
.border-b-accent-content\/40 {
    border-bottom-color: hsl(var(--ac) / 0.4)}
.border-b-accent-content\/5 {
    border-bottom-color: hsl(var(--ac) / 0.05)}
.border-b-accent-content\/50 {
    border-bottom-color: hsl(var(--ac) / 0.5)}
.border-b-accent-content\/60 {
    border-bottom-color: hsl(var(--ac) / 0.6)}
.border-b-accent-content\/70 {
    border-bottom-color: hsl(var(--ac) / 0.7)}
.border-b-accent-content\/75 {
    border-bottom-color: hsl(var(--ac) / 0.75)}
.border-b-accent-content\/80 {
    border-bottom-color: hsl(var(--ac) / 0.8)}
.border-b-accent-content\/90 {
    border-bottom-color: hsl(var(--ac) / 0.9)}
.border-b-accent-content\/95 {
    border-bottom-color: hsl(var(--ac) / 0.95)}
.border-b-accent-focus {
    border-bottom-color: hsl(var(--af) / 1)}
.border-b-accent-focus\/0 {
    border-bottom-color: hsl(var(--af) / 0)}
.border-b-accent-focus\/10 {
    border-bottom-color: hsl(var(--af) / 0.1)}
.border-b-accent-focus\/100 {
    border-bottom-color: hsl(var(--af) / 1)}
.border-b-accent-focus\/20 {
    border-bottom-color: hsl(var(--af) / 0.2)}
.border-b-accent-focus\/25 {
    border-bottom-color: hsl(var(--af) / 0.25)}
.border-b-accent-focus\/30 {
    border-bottom-color: hsl(var(--af) / 0.3)}
.border-b-accent-focus\/40 {
    border-bottom-color: hsl(var(--af) / 0.4)}
.border-b-accent-focus\/5 {
    border-bottom-color: hsl(var(--af) / 0.05)}
.border-b-accent-focus\/50 {
    border-bottom-color: hsl(var(--af) / 0.5)}
.border-b-accent-focus\/60 {
    border-bottom-color: hsl(var(--af) / 0.6)}
.border-b-accent-focus\/70 {
    border-bottom-color: hsl(var(--af) / 0.7)}
.border-b-accent-focus\/75 {
    border-bottom-color: hsl(var(--af) / 0.75)}
.border-b-accent-focus\/80 {
    border-bottom-color: hsl(var(--af) / 0.8)}
.border-b-accent-focus\/90 {
    border-bottom-color: hsl(var(--af) / 0.9)}
.border-b-accent-focus\/95 {
    border-bottom-color: hsl(var(--af) / 0.95)}
.border-b-accent\/0 {
    border-bottom-color: hsl(var(--a) / 0)}
.border-b-accent\/10 {
    border-bottom-color: hsl(var(--a) / 0.1)}
.border-b-accent\/100 {
    border-bottom-color: hsl(var(--a) / 1)}
.border-b-accent\/20 {
    border-bottom-color: hsl(var(--a) / 0.2)}
.border-b-accent\/25 {
    border-bottom-color: hsl(var(--a) / 0.25)}
.border-b-accent\/30 {
    border-bottom-color: hsl(var(--a) / 0.3)}
.border-b-accent\/40 {
    border-bottom-color: hsl(var(--a) / 0.4)}
.border-b-accent\/5 {
    border-bottom-color: hsl(var(--a) / 0.05)}
.border-b-accent\/50 {
    border-bottom-color: hsl(var(--a) / 0.5)}
.border-b-accent\/60 {
    border-bottom-color: hsl(var(--a) / 0.6)}
.border-b-accent\/70 {
    border-bottom-color: hsl(var(--a) / 0.7)}
.border-b-accent\/75 {
    border-bottom-color: hsl(var(--a) / 0.75)}
.border-b-accent\/80 {
    border-bottom-color: hsl(var(--a) / 0.8)}
.border-b-accent\/90 {
    border-bottom-color: hsl(var(--a) / 0.9)}
.border-b-accent\/95 {
    border-bottom-color: hsl(var(--a) / 0.95)}
.border-b-base-100 {
    border-bottom-color: hsl(var(--b1) / 1)}
.border-b-base-100\/0 {
    border-bottom-color: hsl(var(--b1) / 0)}
.border-b-base-100\/10 {
    border-bottom-color: hsl(var(--b1) / 0.1)}
.border-b-base-100\/100 {
    border-bottom-color: hsl(var(--b1) / 1)}
.border-b-base-100\/20 {
    border-bottom-color: hsl(var(--b1) / 0.2)}
.border-b-base-100\/25 {
    border-bottom-color: hsl(var(--b1) / 0.25)}
.border-b-base-100\/30 {
    border-bottom-color: hsl(var(--b1) / 0.3)}
.border-b-base-100\/40 {
    border-bottom-color: hsl(var(--b1) / 0.4)}
.border-b-base-100\/5 {
    border-bottom-color: hsl(var(--b1) / 0.05)}
.border-b-base-100\/50 {
    border-bottom-color: hsl(var(--b1) / 0.5)}
.border-b-base-100\/60 {
    border-bottom-color: hsl(var(--b1) / 0.6)}
.border-b-base-100\/70 {
    border-bottom-color: hsl(var(--b1) / 0.7)}
.border-b-base-100\/75 {
    border-bottom-color: hsl(var(--b1) / 0.75)}
.border-b-base-100\/80 {
    border-bottom-color: hsl(var(--b1) / 0.8)}
.border-b-base-100\/90 {
    border-bottom-color: hsl(var(--b1) / 0.9)}
.border-b-base-100\/95 {
    border-bottom-color: hsl(var(--b1) / 0.95)}
.border-b-base-200 {
    border-bottom-color: hsl(var(--b2) / 1)}
.border-b-base-200\/0 {
    border-bottom-color: hsl(var(--b2) / 0)}
.border-b-base-200\/10 {
    border-bottom-color: hsl(var(--b2) / 0.1)}
.border-b-base-200\/100 {
    border-bottom-color: hsl(var(--b2) / 1)}
.border-b-base-200\/20 {
    border-bottom-color: hsl(var(--b2) / 0.2)}
.border-b-base-200\/25 {
    border-bottom-color: hsl(var(--b2) / 0.25)}
.border-b-base-200\/30 {
    border-bottom-color: hsl(var(--b2) / 0.3)}
.border-b-base-200\/40 {
    border-bottom-color: hsl(var(--b2) / 0.4)}
.border-b-base-200\/5 {
    border-bottom-color: hsl(var(--b2) / 0.05)}
.border-b-base-200\/50 {
    border-bottom-color: hsl(var(--b2) / 0.5)}
.border-b-base-200\/60 {
    border-bottom-color: hsl(var(--b2) / 0.6)}
.border-b-base-200\/70 {
    border-bottom-color: hsl(var(--b2) / 0.7)}
.border-b-base-200\/75 {
    border-bottom-color: hsl(var(--b2) / 0.75)}
.border-b-base-200\/80 {
    border-bottom-color: hsl(var(--b2) / 0.8)}
.border-b-base-200\/90 {
    border-bottom-color: hsl(var(--b2) / 0.9)}
.border-b-base-200\/95 {
    border-bottom-color: hsl(var(--b2) / 0.95)}
.border-b-base-300 {
    border-bottom-color: hsl(var(--b3) / 1)}
.border-b-base-300\/0 {
    border-bottom-color: hsl(var(--b3) / 0)}
.border-b-base-300\/10 {
    border-bottom-color: hsl(var(--b3) / 0.1)}
.border-b-base-300\/100 {
    border-bottom-color: hsl(var(--b3) / 1)}
.border-b-base-300\/20 {
    border-bottom-color: hsl(var(--b3) / 0.2)}
.border-b-base-300\/25 {
    border-bottom-color: hsl(var(--b3) / 0.25)}
.border-b-base-300\/30 {
    border-bottom-color: hsl(var(--b3) / 0.3)}
.border-b-base-300\/40 {
    border-bottom-color: hsl(var(--b3) / 0.4)}
.border-b-base-300\/5 {
    border-bottom-color: hsl(var(--b3) / 0.05)}
.border-b-base-300\/50 {
    border-bottom-color: hsl(var(--b3) / 0.5)}
.border-b-base-300\/60 {
    border-bottom-color: hsl(var(--b3) / 0.6)}
.border-b-base-300\/70 {
    border-bottom-color: hsl(var(--b3) / 0.7)}
.border-b-base-300\/75 {
    border-bottom-color: hsl(var(--b3) / 0.75)}
.border-b-base-300\/80 {
    border-bottom-color: hsl(var(--b3) / 0.8)}
.border-b-base-300\/90 {
    border-bottom-color: hsl(var(--b3) / 0.9)}
.border-b-base-300\/95 {
    border-bottom-color: hsl(var(--b3) / 0.95)}
.border-b-base-content {
    border-bottom-color: hsl(var(--bc) / 1)}
.border-b-base-content\/0 {
    border-bottom-color: hsl(var(--bc) / 0)}
.border-b-base-content\/10 {
    border-bottom-color: hsl(var(--bc) / 0.1)}
.border-b-base-content\/100 {
    border-bottom-color: hsl(var(--bc) / 1)}
.border-b-base-content\/20 {
    border-bottom-color: hsl(var(--bc) / 0.2)}
.border-b-base-content\/25 {
    border-bottom-color: hsl(var(--bc) / 0.25)}
.border-b-base-content\/30 {
    border-bottom-color: hsl(var(--bc) / 0.3)}
.border-b-base-content\/40 {
    border-bottom-color: hsl(var(--bc) / 0.4)}
.border-b-base-content\/5 {
    border-bottom-color: hsl(var(--bc) / 0.05)}
.border-b-base-content\/50 {
    border-bottom-color: hsl(var(--bc) / 0.5)}
.border-b-base-content\/60 {
    border-bottom-color: hsl(var(--bc) / 0.6)}
.border-b-base-content\/70 {
    border-bottom-color: hsl(var(--bc) / 0.7)}
.border-b-base-content\/75 {
    border-bottom-color: hsl(var(--bc) / 0.75)}
.border-b-base-content\/80 {
    border-bottom-color: hsl(var(--bc) / 0.8)}
.border-b-base-content\/90 {
    border-bottom-color: hsl(var(--bc) / 0.9)}
.border-b-base-content\/95 {
    border-bottom-color: hsl(var(--bc) / 0.95)}
.border-b-current {
    border-bottom-color: currentColor}
.border-b-error {
    border-bottom-color: hsl(var(--er) / 1)}
.border-b-error-content {
    border-bottom-color: hsl(var(--erc) / 1)}
.border-b-error-content\/0 {
    border-bottom-color: hsl(var(--erc) / 0)}
.border-b-error-content\/10 {
    border-bottom-color: hsl(var(--erc) / 0.1)}
.border-b-error-content\/100 {
    border-bottom-color: hsl(var(--erc) / 1)}
.border-b-error-content\/20 {
    border-bottom-color: hsl(var(--erc) / 0.2)}
.border-b-error-content\/25 {
    border-bottom-color: hsl(var(--erc) / 0.25)}
.border-b-error-content\/30 {
    border-bottom-color: hsl(var(--erc) / 0.3)}
.border-b-error-content\/40 {
    border-bottom-color: hsl(var(--erc) / 0.4)}
.border-b-error-content\/5 {
    border-bottom-color: hsl(var(--erc) / 0.05)}
.border-b-error-content\/50 {
    border-bottom-color: hsl(var(--erc) / 0.5)}
.border-b-error-content\/60 {
    border-bottom-color: hsl(var(--erc) / 0.6)}
.border-b-error-content\/70 {
    border-bottom-color: hsl(var(--erc) / 0.7)}
.border-b-error-content\/75 {
    border-bottom-color: hsl(var(--erc) / 0.75)}
.border-b-error-content\/80 {
    border-bottom-color: hsl(var(--erc) / 0.8)}
.border-b-error-content\/90 {
    border-bottom-color: hsl(var(--erc) / 0.9)}
.border-b-error-content\/95 {
    border-bottom-color: hsl(var(--erc) / 0.95)}
.border-b-error\/0 {
    border-bottom-color: hsl(var(--er) / 0)}
.border-b-error\/10 {
    border-bottom-color: hsl(var(--er) / 0.1)}
.border-b-error\/100 {
    border-bottom-color: hsl(var(--er) / 1)}
.border-b-error\/20 {
    border-bottom-color: hsl(var(--er) / 0.2)}
.border-b-error\/25 {
    border-bottom-color: hsl(var(--er) / 0.25)}
.border-b-error\/30 {
    border-bottom-color: hsl(var(--er) / 0.3)}
.border-b-error\/40 {
    border-bottom-color: hsl(var(--er) / 0.4)}
.border-b-error\/5 {
    border-bottom-color: hsl(var(--er) / 0.05)}
.border-b-error\/50 {
    border-bottom-color: hsl(var(--er) / 0.5)}
.border-b-error\/60 {
    border-bottom-color: hsl(var(--er) / 0.6)}
.border-b-error\/70 {
    border-bottom-color: hsl(var(--er) / 0.7)}
.border-b-error\/75 {
    border-bottom-color: hsl(var(--er) / 0.75)}
.border-b-error\/80 {
    border-bottom-color: hsl(var(--er) / 0.8)}
.border-b-error\/90 {
    border-bottom-color: hsl(var(--er) / 0.9)}
.border-b-error\/95 {
    border-bottom-color: hsl(var(--er) / 0.95)}
.border-b-info {
    border-bottom-color: hsl(var(--in) / 1)}
.border-b-info-content {
    border-bottom-color: hsl(var(--inc) / 1)}
.border-b-info-content\/0 {
    border-bottom-color: hsl(var(--inc) / 0)}
.border-b-info-content\/10 {
    border-bottom-color: hsl(var(--inc) / 0.1)}
.border-b-info-content\/100 {
    border-bottom-color: hsl(var(--inc) / 1)}
.border-b-info-content\/20 {
    border-bottom-color: hsl(var(--inc) / 0.2)}
.border-b-info-content\/25 {
    border-bottom-color: hsl(var(--inc) / 0.25)}
.border-b-info-content\/30 {
    border-bottom-color: hsl(var(--inc) / 0.3)}
.border-b-info-content\/40 {
    border-bottom-color: hsl(var(--inc) / 0.4)}
.border-b-info-content\/5 {
    border-bottom-color: hsl(var(--inc) / 0.05)}
.border-b-info-content\/50 {
    border-bottom-color: hsl(var(--inc) / 0.5)}
.border-b-info-content\/60 {
    border-bottom-color: hsl(var(--inc) / 0.6)}
.border-b-info-content\/70 {
    border-bottom-color: hsl(var(--inc) / 0.7)}
.border-b-info-content\/75 {
    border-bottom-color: hsl(var(--inc) / 0.75)}
.border-b-info-content\/80 {
    border-bottom-color: hsl(var(--inc) / 0.8)}
.border-b-info-content\/90 {
    border-bottom-color: hsl(var(--inc) / 0.9)}
.border-b-info-content\/95 {
    border-bottom-color: hsl(var(--inc) / 0.95)}
.border-b-info\/0 {
    border-bottom-color: hsl(var(--in) / 0)}
.border-b-info\/10 {
    border-bottom-color: hsl(var(--in) / 0.1)}
.border-b-info\/100 {
    border-bottom-color: hsl(var(--in) / 1)}
.border-b-info\/20 {
    border-bottom-color: hsl(var(--in) / 0.2)}
.border-b-info\/25 {
    border-bottom-color: hsl(var(--in) / 0.25)}
.border-b-info\/30 {
    border-bottom-color: hsl(var(--in) / 0.3)}
.border-b-info\/40 {
    border-bottom-color: hsl(var(--in) / 0.4)}
.border-b-info\/5 {
    border-bottom-color: hsl(var(--in) / 0.05)}
.border-b-info\/50 {
    border-bottom-color: hsl(var(--in) / 0.5)}
.border-b-info\/60 {
    border-bottom-color: hsl(var(--in) / 0.6)}
.border-b-info\/70 {
    border-bottom-color: hsl(var(--in) / 0.7)}
.border-b-info\/75 {
    border-bottom-color: hsl(var(--in) / 0.75)}
.border-b-info\/80 {
    border-bottom-color: hsl(var(--in) / 0.8)}
.border-b-info\/90 {
    border-bottom-color: hsl(var(--in) / 0.9)}
.border-b-info\/95 {
    border-bottom-color: hsl(var(--in) / 0.95)}
.border-b-neutral {
    border-bottom-color: hsl(var(--n) / 1)}
.border-b-neutral-content {
    border-bottom-color: hsl(var(--nc) / 1)}
.border-b-neutral-content\/0 {
    border-bottom-color: hsl(var(--nc) / 0)}
.border-b-neutral-content\/10 {
    border-bottom-color: hsl(var(--nc) / 0.1)}
.border-b-neutral-content\/100 {
    border-bottom-color: hsl(var(--nc) / 1)}
.border-b-neutral-content\/20 {
    border-bottom-color: hsl(var(--nc) / 0.2)}
.border-b-neutral-content\/25 {
    border-bottom-color: hsl(var(--nc) / 0.25)}
.border-b-neutral-content\/30 {
    border-bottom-color: hsl(var(--nc) / 0.3)}
.border-b-neutral-content\/40 {
    border-bottom-color: hsl(var(--nc) / 0.4)}
.border-b-neutral-content\/5 {
    border-bottom-color: hsl(var(--nc) / 0.05)}
.border-b-neutral-content\/50 {
    border-bottom-color: hsl(var(--nc) / 0.5)}
.border-b-neutral-content\/60 {
    border-bottom-color: hsl(var(--nc) / 0.6)}
.border-b-neutral-content\/70 {
    border-bottom-color: hsl(var(--nc) / 0.7)}
.border-b-neutral-content\/75 {
    border-bottom-color: hsl(var(--nc) / 0.75)}
.border-b-neutral-content\/80 {
    border-bottom-color: hsl(var(--nc) / 0.8)}
.border-b-neutral-content\/90 {
    border-bottom-color: hsl(var(--nc) / 0.9)}
.border-b-neutral-content\/95 {
    border-bottom-color: hsl(var(--nc) / 0.95)}
.border-b-neutral-focus {
    border-bottom-color: hsl(var(--nf) / 1)}
.border-b-neutral-focus\/0 {
    border-bottom-color: hsl(var(--nf) / 0)}
.border-b-neutral-focus\/10 {
    border-bottom-color: hsl(var(--nf) / 0.1)}
.border-b-neutral-focus\/100 {
    border-bottom-color: hsl(var(--nf) / 1)}
.border-b-neutral-focus\/20 {
    border-bottom-color: hsl(var(--nf) / 0.2)}
.border-b-neutral-focus\/25 {
    border-bottom-color: hsl(var(--nf) / 0.25)}
.border-b-neutral-focus\/30 {
    border-bottom-color: hsl(var(--nf) / 0.3)}
.border-b-neutral-focus\/40 {
    border-bottom-color: hsl(var(--nf) / 0.4)}
.border-b-neutral-focus\/5 {
    border-bottom-color: hsl(var(--nf) / 0.05)}
.border-b-neutral-focus\/50 {
    border-bottom-color: hsl(var(--nf) / 0.5)}
.border-b-neutral-focus\/60 {
    border-bottom-color: hsl(var(--nf) / 0.6)}
.border-b-neutral-focus\/70 {
    border-bottom-color: hsl(var(--nf) / 0.7)}
.border-b-neutral-focus\/75 {
    border-bottom-color: hsl(var(--nf) / 0.75)}
.border-b-neutral-focus\/80 {
    border-bottom-color: hsl(var(--nf) / 0.8)}
.border-b-neutral-focus\/90 {
    border-bottom-color: hsl(var(--nf) / 0.9)}
.border-b-neutral-focus\/95 {
    border-bottom-color: hsl(var(--nf) / 0.95)}
.border-b-neutral\/0 {
    border-bottom-color: hsl(var(--n) / 0)}
.border-b-neutral\/10 {
    border-bottom-color: hsl(var(--n) / 0.1)}
.border-b-neutral\/100 {
    border-bottom-color: hsl(var(--n) / 1)}
.border-b-neutral\/20 {
    border-bottom-color: hsl(var(--n) / 0.2)}
.border-b-neutral\/25 {
    border-bottom-color: hsl(var(--n) / 0.25)}
.border-b-neutral\/30 {
    border-bottom-color: hsl(var(--n) / 0.3)}
.border-b-neutral\/40 {
    border-bottom-color: hsl(var(--n) / 0.4)}
.border-b-neutral\/5 {
    border-bottom-color: hsl(var(--n) / 0.05)}
.border-b-neutral\/50 {
    border-bottom-color: hsl(var(--n) / 0.5)}
.border-b-neutral\/60 {
    border-bottom-color: hsl(var(--n) / 0.6)}
.border-b-neutral\/70 {
    border-bottom-color: hsl(var(--n) / 0.7)}
.border-b-neutral\/75 {
    border-bottom-color: hsl(var(--n) / 0.75)}
.border-b-neutral\/80 {
    border-bottom-color: hsl(var(--n) / 0.8)}
.border-b-neutral\/90 {
    border-bottom-color: hsl(var(--n) / 0.9)}
.border-b-neutral\/95 {
    border-bottom-color: hsl(var(--n) / 0.95)}
.border-b-primary {
    border-bottom-color: hsl(var(--p) / 1)}
.border-b-primary-content {
    border-bottom-color: hsl(var(--pc) / 1)}
.border-b-primary-content\/0 {
    border-bottom-color: hsl(var(--pc) / 0)}
.border-b-primary-content\/10 {
    border-bottom-color: hsl(var(--pc) / 0.1)}
.border-b-primary-content\/100 {
    border-bottom-color: hsl(var(--pc) / 1)}
.border-b-primary-content\/20 {
    border-bottom-color: hsl(var(--pc) / 0.2)}
.border-b-primary-content\/25 {
    border-bottom-color: hsl(var(--pc) / 0.25)}
.border-b-primary-content\/30 {
    border-bottom-color: hsl(var(--pc) / 0.3)}
.border-b-primary-content\/40 {
    border-bottom-color: hsl(var(--pc) / 0.4)}
.border-b-primary-content\/5 {
    border-bottom-color: hsl(var(--pc) / 0.05)}
.border-b-primary-content\/50 {
    border-bottom-color: hsl(var(--pc) / 0.5)}
.border-b-primary-content\/60 {
    border-bottom-color: hsl(var(--pc) / 0.6)}
.border-b-primary-content\/70 {
    border-bottom-color: hsl(var(--pc) / 0.7)}
.border-b-primary-content\/75 {
    border-bottom-color: hsl(var(--pc) / 0.75)}
.border-b-primary-content\/80 {
    border-bottom-color: hsl(var(--pc) / 0.8)}
.border-b-primary-content\/90 {
    border-bottom-color: hsl(var(--pc) / 0.9)}
.border-b-primary-content\/95 {
    border-bottom-color: hsl(var(--pc) / 0.95)}
.border-b-primary-focus {
    border-bottom-color: hsl(var(--pf) / 1)}
.border-b-primary-focus\/0 {
    border-bottom-color: hsl(var(--pf) / 0)}
.border-b-primary-focus\/10 {
    border-bottom-color: hsl(var(--pf) / 0.1)}
.border-b-primary-focus\/100 {
    border-bottom-color: hsl(var(--pf) / 1)}
.border-b-primary-focus\/20 {
    border-bottom-color: hsl(var(--pf) / 0.2)}
.border-b-primary-focus\/25 {
    border-bottom-color: hsl(var(--pf) / 0.25)}
.border-b-primary-focus\/30 {
    border-bottom-color: hsl(var(--pf) / 0.3)}
.border-b-primary-focus\/40 {
    border-bottom-color: hsl(var(--pf) / 0.4)}
.border-b-primary-focus\/5 {
    border-bottom-color: hsl(var(--pf) / 0.05)}
.border-b-primary-focus\/50 {
    border-bottom-color: hsl(var(--pf) / 0.5)}
.border-b-primary-focus\/60 {
    border-bottom-color: hsl(var(--pf) / 0.6)}
.border-b-primary-focus\/70 {
    border-bottom-color: hsl(var(--pf) / 0.7)}
.border-b-primary-focus\/75 {
    border-bottom-color: hsl(var(--pf) / 0.75)}
.border-b-primary-focus\/80 {
    border-bottom-color: hsl(var(--pf) / 0.8)}
.border-b-primary-focus\/90 {
    border-bottom-color: hsl(var(--pf) / 0.9)}
.border-b-primary-focus\/95 {
    border-bottom-color: hsl(var(--pf) / 0.95)}
.border-b-primary\/0 {
    border-bottom-color: hsl(var(--p) / 0)}
.border-b-primary\/10 {
    border-bottom-color: hsl(var(--p) / 0.1)}
.border-b-primary\/100 {
    border-bottom-color: hsl(var(--p) / 1)}
.border-b-primary\/20 {
    border-bottom-color: hsl(var(--p) / 0.2)}
.border-b-primary\/25 {
    border-bottom-color: hsl(var(--p) / 0.25)}
.border-b-primary\/30 {
    border-bottom-color: hsl(var(--p) / 0.3)}
.border-b-primary\/40 {
    border-bottom-color: hsl(var(--p) / 0.4)}
.border-b-primary\/5 {
    border-bottom-color: hsl(var(--p) / 0.05)}
.border-b-primary\/50 {
    border-bottom-color: hsl(var(--p) / 0.5)}
.border-b-primary\/60 {
    border-bottom-color: hsl(var(--p) / 0.6)}
.border-b-primary\/70 {
    border-bottom-color: hsl(var(--p) / 0.7)}
.border-b-primary\/75 {
    border-bottom-color: hsl(var(--p) / 0.75)}
.border-b-primary\/80 {
    border-bottom-color: hsl(var(--p) / 0.8)}
.border-b-primary\/90 {
    border-bottom-color: hsl(var(--p) / 0.9)}
.border-b-primary\/95 {
    border-bottom-color: hsl(var(--p) / 0.95)}
.border-b-secondary {
    border-bottom-color: hsl(var(--s) / 1)}
.border-b-secondary-content {
    border-bottom-color: hsl(var(--sc) / 1)}
.border-b-secondary-content\/0 {
    border-bottom-color: hsl(var(--sc) / 0)}
.border-b-secondary-content\/10 {
    border-bottom-color: hsl(var(--sc) / 0.1)}
.border-b-secondary-content\/100 {
    border-bottom-color: hsl(var(--sc) / 1)}
.border-b-secondary-content\/20 {
    border-bottom-color: hsl(var(--sc) / 0.2)}
.border-b-secondary-content\/25 {
    border-bottom-color: hsl(var(--sc) / 0.25)}
.border-b-secondary-content\/30 {
    border-bottom-color: hsl(var(--sc) / 0.3)}
.border-b-secondary-content\/40 {
    border-bottom-color: hsl(var(--sc) / 0.4)}
.border-b-secondary-content\/5 {
    border-bottom-color: hsl(var(--sc) / 0.05)}
.border-b-secondary-content\/50 {
    border-bottom-color: hsl(var(--sc) / 0.5)}
.border-b-secondary-content\/60 {
    border-bottom-color: hsl(var(--sc) / 0.6)}
.border-b-secondary-content\/70 {
    border-bottom-color: hsl(var(--sc) / 0.7)}
.border-b-secondary-content\/75 {
    border-bottom-color: hsl(var(--sc) / 0.75)}
.border-b-secondary-content\/80 {
    border-bottom-color: hsl(var(--sc) / 0.8)}
.border-b-secondary-content\/90 {
    border-bottom-color: hsl(var(--sc) / 0.9)}
.border-b-secondary-content\/95 {
    border-bottom-color: hsl(var(--sc) / 0.95)}
.border-b-secondary-focus {
    border-bottom-color: hsl(var(--sf) / 1)}
.border-b-secondary-focus\/0 {
    border-bottom-color: hsl(var(--sf) / 0)}
.border-b-secondary-focus\/10 {
    border-bottom-color: hsl(var(--sf) / 0.1)}
.border-b-secondary-focus\/100 {
    border-bottom-color: hsl(var(--sf) / 1)}
.border-b-secondary-focus\/20 {
    border-bottom-color: hsl(var(--sf) / 0.2)}
.border-b-secondary-focus\/25 {
    border-bottom-color: hsl(var(--sf) / 0.25)}
.border-b-secondary-focus\/30 {
    border-bottom-color: hsl(var(--sf) / 0.3)}
.border-b-secondary-focus\/40 {
    border-bottom-color: hsl(var(--sf) / 0.4)}
.border-b-secondary-focus\/5 {
    border-bottom-color: hsl(var(--sf) / 0.05)}
.border-b-secondary-focus\/50 {
    border-bottom-color: hsl(var(--sf) / 0.5)}
.border-b-secondary-focus\/60 {
    border-bottom-color: hsl(var(--sf) / 0.6)}
.border-b-secondary-focus\/70 {
    border-bottom-color: hsl(var(--sf) / 0.7)}
.border-b-secondary-focus\/75 {
    border-bottom-color: hsl(var(--sf) / 0.75)}
.border-b-secondary-focus\/80 {
    border-bottom-color: hsl(var(--sf) / 0.8)}
.border-b-secondary-focus\/90 {
    border-bottom-color: hsl(var(--sf) / 0.9)}
.border-b-secondary-focus\/95 {
    border-bottom-color: hsl(var(--sf) / 0.95)}
.border-b-secondary\/0 {
    border-bottom-color: hsl(var(--s) / 0)}
.border-b-secondary\/10 {
    border-bottom-color: hsl(var(--s) / 0.1)}
.border-b-secondary\/100 {
    border-bottom-color: hsl(var(--s) / 1)}
.border-b-secondary\/20 {
    border-bottom-color: hsl(var(--s) / 0.2)}
.border-b-secondary\/25 {
    border-bottom-color: hsl(var(--s) / 0.25)}
.border-b-secondary\/30 {
    border-bottom-color: hsl(var(--s) / 0.3)}
.border-b-secondary\/40 {
    border-bottom-color: hsl(var(--s) / 0.4)}
.border-b-secondary\/5 {
    border-bottom-color: hsl(var(--s) / 0.05)}
.border-b-secondary\/50 {
    border-bottom-color: hsl(var(--s) / 0.5)}
.border-b-secondary\/60 {
    border-bottom-color: hsl(var(--s) / 0.6)}
.border-b-secondary\/70 {
    border-bottom-color: hsl(var(--s) / 0.7)}
.border-b-secondary\/75 {
    border-bottom-color: hsl(var(--s) / 0.75)}
.border-b-secondary\/80 {
    border-bottom-color: hsl(var(--s) / 0.8)}
.border-b-secondary\/90 {
    border-bottom-color: hsl(var(--s) / 0.9)}
.border-b-secondary\/95 {
    border-bottom-color: hsl(var(--s) / 0.95)}
.border-b-success {
    border-bottom-color: hsl(var(--su) / 1)}
.border-b-success-content {
    border-bottom-color: hsl(var(--suc) / 1)}
.border-b-success-content\/0 {
    border-bottom-color: hsl(var(--suc) / 0)}
.border-b-success-content\/10 {
    border-bottom-color: hsl(var(--suc) / 0.1)}
.border-b-success-content\/100 {
    border-bottom-color: hsl(var(--suc) / 1)}
.border-b-success-content\/20 {
    border-bottom-color: hsl(var(--suc) / 0.2)}
.border-b-success-content\/25 {
    border-bottom-color: hsl(var(--suc) / 0.25)}
.border-b-success-content\/30 {
    border-bottom-color: hsl(var(--suc) / 0.3)}
.border-b-success-content\/40 {
    border-bottom-color: hsl(var(--suc) / 0.4)}
.border-b-success-content\/5 {
    border-bottom-color: hsl(var(--suc) / 0.05)}
.border-b-success-content\/50 {
    border-bottom-color: hsl(var(--suc) / 0.5)}
.border-b-success-content\/60 {
    border-bottom-color: hsl(var(--suc) / 0.6)}
.border-b-success-content\/70 {
    border-bottom-color: hsl(var(--suc) / 0.7)}
.border-b-success-content\/75 {
    border-bottom-color: hsl(var(--suc) / 0.75)}
.border-b-success-content\/80 {
    border-bottom-color: hsl(var(--suc) / 0.8)}
.border-b-success-content\/90 {
    border-bottom-color: hsl(var(--suc) / 0.9)}
.border-b-success-content\/95 {
    border-bottom-color: hsl(var(--suc) / 0.95)}
.border-b-success\/0 {
    border-bottom-color: hsl(var(--su) / 0)}
.border-b-success\/10 {
    border-bottom-color: hsl(var(--su) / 0.1)}
.border-b-success\/100 {
    border-bottom-color: hsl(var(--su) / 1)}
.border-b-success\/20 {
    border-bottom-color: hsl(var(--su) / 0.2)}
.border-b-success\/25 {
    border-bottom-color: hsl(var(--su) / 0.25)}
.border-b-success\/30 {
    border-bottom-color: hsl(var(--su) / 0.3)}
.border-b-success\/40 {
    border-bottom-color: hsl(var(--su) / 0.4)}
.border-b-success\/5 {
    border-bottom-color: hsl(var(--su) / 0.05)}
.border-b-success\/50 {
    border-bottom-color: hsl(var(--su) / 0.5)}
.border-b-success\/60 {
    border-bottom-color: hsl(var(--su) / 0.6)}
.border-b-success\/70 {
    border-bottom-color: hsl(var(--su) / 0.7)}
.border-b-success\/75 {
    border-bottom-color: hsl(var(--su) / 0.75)}
.border-b-success\/80 {
    border-bottom-color: hsl(var(--su) / 0.8)}
.border-b-success\/90 {
    border-bottom-color: hsl(var(--su) / 0.9)}
.border-b-success\/95 {
    border-bottom-color: hsl(var(--su) / 0.95)}
.border-b-transparent {
    border-bottom-color: transparent}
.border-b-transparent\/0 {
    border-bottom-color: rgb(0 0 0 / 0)}
.border-b-transparent\/10 {
    border-bottom-color: rgb(0 0 0 / 0.1)}
.border-b-transparent\/100 {
    border-bottom-color: rgb(0 0 0 / 1)}
.border-b-transparent\/20 {
    border-bottom-color: rgb(0 0 0 / 0.2)}
.border-b-transparent\/25 {
    border-bottom-color: rgb(0 0 0 / 0.25)}
.border-b-transparent\/30 {
    border-bottom-color: rgb(0 0 0 / 0.3)}
.border-b-transparent\/40 {
    border-bottom-color: rgb(0 0 0 / 0.4)}
.border-b-transparent\/5 {
    border-bottom-color: rgb(0 0 0 / 0.05)}
.border-b-transparent\/50 {
    border-bottom-color: rgb(0 0 0 / 0.5)}
.border-b-transparent\/60 {
    border-bottom-color: rgb(0 0 0 / 0.6)}
.border-b-transparent\/70 {
    border-bottom-color: rgb(0 0 0 / 0.7)}
.border-b-transparent\/75 {
    border-bottom-color: rgb(0 0 0 / 0.75)}
.border-b-transparent\/80 {
    border-bottom-color: rgb(0 0 0 / 0.8)}
.border-b-transparent\/90 {
    border-bottom-color: rgb(0 0 0 / 0.9)}
.border-b-transparent\/95 {
    border-bottom-color: rgb(0 0 0 / 0.95)}
.border-b-warning {
    border-bottom-color: hsl(var(--wa) / 1)}
.border-b-warning-content {
    border-bottom-color: hsl(var(--wac) / 1)}
.border-b-warning-content\/0 {
    border-bottom-color: hsl(var(--wac) / 0)}
.border-b-warning-content\/10 {
    border-bottom-color: hsl(var(--wac) / 0.1)}
.border-b-warning-content\/100 {
    border-bottom-color: hsl(var(--wac) / 1)}
.border-b-warning-content\/20 {
    border-bottom-color: hsl(var(--wac) / 0.2)}
.border-b-warning-content\/25 {
    border-bottom-color: hsl(var(--wac) / 0.25)}
.border-b-warning-content\/30 {
    border-bottom-color: hsl(var(--wac) / 0.3)}
.border-b-warning-content\/40 {
    border-bottom-color: hsl(var(--wac) / 0.4)}
.border-b-warning-content\/5 {
    border-bottom-color: hsl(var(--wac) / 0.05)}
.border-b-warning-content\/50 {
    border-bottom-color: hsl(var(--wac) / 0.5)}
.border-b-warning-content\/60 {
    border-bottom-color: hsl(var(--wac) / 0.6)}
.border-b-warning-content\/70 {
    border-bottom-color: hsl(var(--wac) / 0.7)}
.border-b-warning-content\/75 {
    border-bottom-color: hsl(var(--wac) / 0.75)}
.border-b-warning-content\/80 {
    border-bottom-color: hsl(var(--wac) / 0.8)}
.border-b-warning-content\/90 {
    border-bottom-color: hsl(var(--wac) / 0.9)}
.border-b-warning-content\/95 {
    border-bottom-color: hsl(var(--wac) / 0.95)}
.border-b-warning\/0 {
    border-bottom-color: hsl(var(--wa) / 0)}
.border-b-warning\/10 {
    border-bottom-color: hsl(var(--wa) / 0.1)}
.border-b-warning\/100 {
    border-bottom-color: hsl(var(--wa) / 1)}
.border-b-warning\/20 {
    border-bottom-color: hsl(var(--wa) / 0.2)}
.border-b-warning\/25 {
    border-bottom-color: hsl(var(--wa) / 0.25)}
.border-b-warning\/30 {
    border-bottom-color: hsl(var(--wa) / 0.3)}
.border-b-warning\/40 {
    border-bottom-color: hsl(var(--wa) / 0.4)}
.border-b-warning\/5 {
    border-bottom-color: hsl(var(--wa) / 0.05)}
.border-b-warning\/50 {
    border-bottom-color: hsl(var(--wa) / 0.5)}
.border-b-warning\/60 {
    border-bottom-color: hsl(var(--wa) / 0.6)}
.border-b-warning\/70 {
    border-bottom-color: hsl(var(--wa) / 0.7)}
.border-b-warning\/75 {
    border-bottom-color: hsl(var(--wa) / 0.75)}
.border-b-warning\/80 {
    border-bottom-color: hsl(var(--wa) / 0.8)}
.border-b-warning\/90 {
    border-bottom-color: hsl(var(--wa) / 0.9)}
.border-b-warning\/95 {
    border-bottom-color: hsl(var(--wa) / 0.95)}
.border-e-accent {
    border-inline-end-color: hsl(var(--a) / 1)}
.border-e-accent-content {
    border-inline-end-color: hsl(var(--ac) / 1)}
.border-e-accent-content\/0 {
    border-inline-end-color: hsl(var(--ac) / 0)}
.border-e-accent-content\/10 {
    border-inline-end-color: hsl(var(--ac) / 0.1)}
.border-e-accent-content\/100 {
    border-inline-end-color: hsl(var(--ac) / 1)}
.border-e-accent-content\/20 {
    border-inline-end-color: hsl(var(--ac) / 0.2)}
.border-e-accent-content\/25 {
    border-inline-end-color: hsl(var(--ac) / 0.25)}
.border-e-accent-content\/30 {
    border-inline-end-color: hsl(var(--ac) / 0.3)}
.border-e-accent-content\/40 {
    border-inline-end-color: hsl(var(--ac) / 0.4)}
.border-e-accent-content\/5 {
    border-inline-end-color: hsl(var(--ac) / 0.05)}
.border-e-accent-content\/50 {
    border-inline-end-color: hsl(var(--ac) / 0.5)}
.border-e-accent-content\/60 {
    border-inline-end-color: hsl(var(--ac) / 0.6)}
.border-e-accent-content\/70 {
    border-inline-end-color: hsl(var(--ac) / 0.7)}
.border-e-accent-content\/75 {
    border-inline-end-color: hsl(var(--ac) / 0.75)}
.border-e-accent-content\/80 {
    border-inline-end-color: hsl(var(--ac) / 0.8)}
.border-e-accent-content\/90 {
    border-inline-end-color: hsl(var(--ac) / 0.9)}
.border-e-accent-content\/95 {
    border-inline-end-color: hsl(var(--ac) / 0.95)}
.border-e-accent-focus {
    border-inline-end-color: hsl(var(--af) / 1)}
.border-e-accent-focus\/0 {
    border-inline-end-color: hsl(var(--af) / 0)}
.border-e-accent-focus\/10 {
    border-inline-end-color: hsl(var(--af) / 0.1)}
.border-e-accent-focus\/100 {
    border-inline-end-color: hsl(var(--af) / 1)}
.border-e-accent-focus\/20 {
    border-inline-end-color: hsl(var(--af) / 0.2)}
.border-e-accent-focus\/25 {
    border-inline-end-color: hsl(var(--af) / 0.25)}
.border-e-accent-focus\/30 {
    border-inline-end-color: hsl(var(--af) / 0.3)}
.border-e-accent-focus\/40 {
    border-inline-end-color: hsl(var(--af) / 0.4)}
.border-e-accent-focus\/5 {
    border-inline-end-color: hsl(var(--af) / 0.05)}
.border-e-accent-focus\/50 {
    border-inline-end-color: hsl(var(--af) / 0.5)}
.border-e-accent-focus\/60 {
    border-inline-end-color: hsl(var(--af) / 0.6)}
.border-e-accent-focus\/70 {
    border-inline-end-color: hsl(var(--af) / 0.7)}
.border-e-accent-focus\/75 {
    border-inline-end-color: hsl(var(--af) / 0.75)}
.border-e-accent-focus\/80 {
    border-inline-end-color: hsl(var(--af) / 0.8)}
.border-e-accent-focus\/90 {
    border-inline-end-color: hsl(var(--af) / 0.9)}
.border-e-accent-focus\/95 {
    border-inline-end-color: hsl(var(--af) / 0.95)}
.border-e-accent\/0 {
    border-inline-end-color: hsl(var(--a) / 0)}
.border-e-accent\/10 {
    border-inline-end-color: hsl(var(--a) / 0.1)}
.border-e-accent\/100 {
    border-inline-end-color: hsl(var(--a) / 1)}
.border-e-accent\/20 {
    border-inline-end-color: hsl(var(--a) / 0.2)}
.border-e-accent\/25 {
    border-inline-end-color: hsl(var(--a) / 0.25)}
.border-e-accent\/30 {
    border-inline-end-color: hsl(var(--a) / 0.3)}
.border-e-accent\/40 {
    border-inline-end-color: hsl(var(--a) / 0.4)}
.border-e-accent\/5 {
    border-inline-end-color: hsl(var(--a) / 0.05)}
.border-e-accent\/50 {
    border-inline-end-color: hsl(var(--a) / 0.5)}
.border-e-accent\/60 {
    border-inline-end-color: hsl(var(--a) / 0.6)}
.border-e-accent\/70 {
    border-inline-end-color: hsl(var(--a) / 0.7)}
.border-e-accent\/75 {
    border-inline-end-color: hsl(var(--a) / 0.75)}
.border-e-accent\/80 {
    border-inline-end-color: hsl(var(--a) / 0.8)}
.border-e-accent\/90 {
    border-inline-end-color: hsl(var(--a) / 0.9)}
.border-e-accent\/95 {
    border-inline-end-color: hsl(var(--a) / 0.95)}
.border-e-base-100 {
    border-inline-end-color: hsl(var(--b1) / 1)}
.border-e-base-100\/0 {
    border-inline-end-color: hsl(var(--b1) / 0)}
.border-e-base-100\/10 {
    border-inline-end-color: hsl(var(--b1) / 0.1)}
.border-e-base-100\/100 {
    border-inline-end-color: hsl(var(--b1) / 1)}
.border-e-base-100\/20 {
    border-inline-end-color: hsl(var(--b1) / 0.2)}
.border-e-base-100\/25 {
    border-inline-end-color: hsl(var(--b1) / 0.25)}
.border-e-base-100\/30 {
    border-inline-end-color: hsl(var(--b1) / 0.3)}
.border-e-base-100\/40 {
    border-inline-end-color: hsl(var(--b1) / 0.4)}
.border-e-base-100\/5 {
    border-inline-end-color: hsl(var(--b1) / 0.05)}
.border-e-base-100\/50 {
    border-inline-end-color: hsl(var(--b1) / 0.5)}
.border-e-base-100\/60 {
    border-inline-end-color: hsl(var(--b1) / 0.6)}
.border-e-base-100\/70 {
    border-inline-end-color: hsl(var(--b1) / 0.7)}
.border-e-base-100\/75 {
    border-inline-end-color: hsl(var(--b1) / 0.75)}
.border-e-base-100\/80 {
    border-inline-end-color: hsl(var(--b1) / 0.8)}
.border-e-base-100\/90 {
    border-inline-end-color: hsl(var(--b1) / 0.9)}
.border-e-base-100\/95 {
    border-inline-end-color: hsl(var(--b1) / 0.95)}
.border-e-base-200 {
    border-inline-end-color: hsl(var(--b2) / 1)}
.border-e-base-200\/0 {
    border-inline-end-color: hsl(var(--b2) / 0)}
.border-e-base-200\/10 {
    border-inline-end-color: hsl(var(--b2) / 0.1)}
.border-e-base-200\/100 {
    border-inline-end-color: hsl(var(--b2) / 1)}
.border-e-base-200\/20 {
    border-inline-end-color: hsl(var(--b2) / 0.2)}
.border-e-base-200\/25 {
    border-inline-end-color: hsl(var(--b2) / 0.25)}
.border-e-base-200\/30 {
    border-inline-end-color: hsl(var(--b2) / 0.3)}
.border-e-base-200\/40 {
    border-inline-end-color: hsl(var(--b2) / 0.4)}
.border-e-base-200\/5 {
    border-inline-end-color: hsl(var(--b2) / 0.05)}
.border-e-base-200\/50 {
    border-inline-end-color: hsl(var(--b2) / 0.5)}
.border-e-base-200\/60 {
    border-inline-end-color: hsl(var(--b2) / 0.6)}
.border-e-base-200\/70 {
    border-inline-end-color: hsl(var(--b2) / 0.7)}
.border-e-base-200\/75 {
    border-inline-end-color: hsl(var(--b2) / 0.75)}
.border-e-base-200\/80 {
    border-inline-end-color: hsl(var(--b2) / 0.8)}
.border-e-base-200\/90 {
    border-inline-end-color: hsl(var(--b2) / 0.9)}
.border-e-base-200\/95 {
    border-inline-end-color: hsl(var(--b2) / 0.95)}
.border-e-base-300 {
    border-inline-end-color: hsl(var(--b3) / 1)}
.border-e-base-300\/0 {
    border-inline-end-color: hsl(var(--b3) / 0)}
.border-e-base-300\/10 {
    border-inline-end-color: hsl(var(--b3) / 0.1)}
.border-e-base-300\/100 {
    border-inline-end-color: hsl(var(--b3) / 1)}
.border-e-base-300\/20 {
    border-inline-end-color: hsl(var(--b3) / 0.2)}
.border-e-base-300\/25 {
    border-inline-end-color: hsl(var(--b3) / 0.25)}
.border-e-base-300\/30 {
    border-inline-end-color: hsl(var(--b3) / 0.3)}
.border-e-base-300\/40 {
    border-inline-end-color: hsl(var(--b3) / 0.4)}
.border-e-base-300\/5 {
    border-inline-end-color: hsl(var(--b3) / 0.05)}
.border-e-base-300\/50 {
    border-inline-end-color: hsl(var(--b3) / 0.5)}
.border-e-base-300\/60 {
    border-inline-end-color: hsl(var(--b3) / 0.6)}
.border-e-base-300\/70 {
    border-inline-end-color: hsl(var(--b3) / 0.7)}
.border-e-base-300\/75 {
    border-inline-end-color: hsl(var(--b3) / 0.75)}
.border-e-base-300\/80 {
    border-inline-end-color: hsl(var(--b3) / 0.8)}
.border-e-base-300\/90 {
    border-inline-end-color: hsl(var(--b3) / 0.9)}
.border-e-base-300\/95 {
    border-inline-end-color: hsl(var(--b3) / 0.95)}
.border-e-base-content {
    border-inline-end-color: hsl(var(--bc) / 1)}
.border-e-base-content\/0 {
    border-inline-end-color: hsl(var(--bc) / 0)}
.border-e-base-content\/10 {
    border-inline-end-color: hsl(var(--bc) / 0.1)}
.border-e-base-content\/100 {
    border-inline-end-color: hsl(var(--bc) / 1)}
.border-e-base-content\/20 {
    border-inline-end-color: hsl(var(--bc) / 0.2)}
.border-e-base-content\/25 {
    border-inline-end-color: hsl(var(--bc) / 0.25)}
.border-e-base-content\/30 {
    border-inline-end-color: hsl(var(--bc) / 0.3)}
.border-e-base-content\/40 {
    border-inline-end-color: hsl(var(--bc) / 0.4)}
.border-e-base-content\/5 {
    border-inline-end-color: hsl(var(--bc) / 0.05)}
.border-e-base-content\/50 {
    border-inline-end-color: hsl(var(--bc) / 0.5)}
.border-e-base-content\/60 {
    border-inline-end-color: hsl(var(--bc) / 0.6)}
.border-e-base-content\/70 {
    border-inline-end-color: hsl(var(--bc) / 0.7)}
.border-e-base-content\/75 {
    border-inline-end-color: hsl(var(--bc) / 0.75)}
.border-e-base-content\/80 {
    border-inline-end-color: hsl(var(--bc) / 0.8)}
.border-e-base-content\/90 {
    border-inline-end-color: hsl(var(--bc) / 0.9)}
.border-e-base-content\/95 {
    border-inline-end-color: hsl(var(--bc) / 0.95)}
.border-e-current {
    border-inline-end-color: currentColor}
.border-e-error {
    border-inline-end-color: hsl(var(--er) / 1)}
.border-e-error-content {
    border-inline-end-color: hsl(var(--erc) / 1)}
.border-e-error-content\/0 {
    border-inline-end-color: hsl(var(--erc) / 0)}
.border-e-error-content\/10 {
    border-inline-end-color: hsl(var(--erc) / 0.1)}
.border-e-error-content\/100 {
    border-inline-end-color: hsl(var(--erc) / 1)}
.border-e-error-content\/20 {
    border-inline-end-color: hsl(var(--erc) / 0.2)}
.border-e-error-content\/25 {
    border-inline-end-color: hsl(var(--erc) / 0.25)}
.border-e-error-content\/30 {
    border-inline-end-color: hsl(var(--erc) / 0.3)}
.border-e-error-content\/40 {
    border-inline-end-color: hsl(var(--erc) / 0.4)}
.border-e-error-content\/5 {
    border-inline-end-color: hsl(var(--erc) / 0.05)}
.border-e-error-content\/50 {
    border-inline-end-color: hsl(var(--erc) / 0.5)}
.border-e-error-content\/60 {
    border-inline-end-color: hsl(var(--erc) / 0.6)}
.border-e-error-content\/70 {
    border-inline-end-color: hsl(var(--erc) / 0.7)}
.border-e-error-content\/75 {
    border-inline-end-color: hsl(var(--erc) / 0.75)}
.border-e-error-content\/80 {
    border-inline-end-color: hsl(var(--erc) / 0.8)}
.border-e-error-content\/90 {
    border-inline-end-color: hsl(var(--erc) / 0.9)}
.border-e-error-content\/95 {
    border-inline-end-color: hsl(var(--erc) / 0.95)}
.border-e-error\/0 {
    border-inline-end-color: hsl(var(--er) / 0)}
.border-e-error\/10 {
    border-inline-end-color: hsl(var(--er) / 0.1)}
.border-e-error\/100 {
    border-inline-end-color: hsl(var(--er) / 1)}
.border-e-error\/20 {
    border-inline-end-color: hsl(var(--er) / 0.2)}
.border-e-error\/25 {
    border-inline-end-color: hsl(var(--er) / 0.25)}
.border-e-error\/30 {
    border-inline-end-color: hsl(var(--er) / 0.3)}
.border-e-error\/40 {
    border-inline-end-color: hsl(var(--er) / 0.4)}
.border-e-error\/5 {
    border-inline-end-color: hsl(var(--er) / 0.05)}
.border-e-error\/50 {
    border-inline-end-color: hsl(var(--er) / 0.5)}
.border-e-error\/60 {
    border-inline-end-color: hsl(var(--er) / 0.6)}
.border-e-error\/70 {
    border-inline-end-color: hsl(var(--er) / 0.7)}
.border-e-error\/75 {
    border-inline-end-color: hsl(var(--er) / 0.75)}
.border-e-error\/80 {
    border-inline-end-color: hsl(var(--er) / 0.8)}
.border-e-error\/90 {
    border-inline-end-color: hsl(var(--er) / 0.9)}
.border-e-error\/95 {
    border-inline-end-color: hsl(var(--er) / 0.95)}
.border-e-info {
    border-inline-end-color: hsl(var(--in) / 1)}
.border-e-info-content {
    border-inline-end-color: hsl(var(--inc) / 1)}
.border-e-info-content\/0 {
    border-inline-end-color: hsl(var(--inc) / 0)}
.border-e-info-content\/10 {
    border-inline-end-color: hsl(var(--inc) / 0.1)}
.border-e-info-content\/100 {
    border-inline-end-color: hsl(var(--inc) / 1)}
.border-e-info-content\/20 {
    border-inline-end-color: hsl(var(--inc) / 0.2)}
.border-e-info-content\/25 {
    border-inline-end-color: hsl(var(--inc) / 0.25)}
.border-e-info-content\/30 {
    border-inline-end-color: hsl(var(--inc) / 0.3)}
.border-e-info-content\/40 {
    border-inline-end-color: hsl(var(--inc) / 0.4)}
.border-e-info-content\/5 {
    border-inline-end-color: hsl(var(--inc) / 0.05)}
.border-e-info-content\/50 {
    border-inline-end-color: hsl(var(--inc) / 0.5)}
.border-e-info-content\/60 {
    border-inline-end-color: hsl(var(--inc) / 0.6)}
.border-e-info-content\/70 {
    border-inline-end-color: hsl(var(--inc) / 0.7)}
.border-e-info-content\/75 {
    border-inline-end-color: hsl(var(--inc) / 0.75)}
.border-e-info-content\/80 {
    border-inline-end-color: hsl(var(--inc) / 0.8)}
.border-e-info-content\/90 {
    border-inline-end-color: hsl(var(--inc) / 0.9)}
.border-e-info-content\/95 {
    border-inline-end-color: hsl(var(--inc) / 0.95)}
.border-e-info\/0 {
    border-inline-end-color: hsl(var(--in) / 0)}
.border-e-info\/10 {
    border-inline-end-color: hsl(var(--in) / 0.1)}
.border-e-info\/100 {
    border-inline-end-color: hsl(var(--in) / 1)}
.border-e-info\/20 {
    border-inline-end-color: hsl(var(--in) / 0.2)}
.border-e-info\/25 {
    border-inline-end-color: hsl(var(--in) / 0.25)}
.border-e-info\/30 {
    border-inline-end-color: hsl(var(--in) / 0.3)}
.border-e-info\/40 {
    border-inline-end-color: hsl(var(--in) / 0.4)}
.border-e-info\/5 {
    border-inline-end-color: hsl(var(--in) / 0.05)}
.border-e-info\/50 {
    border-inline-end-color: hsl(var(--in) / 0.5)}
.border-e-info\/60 {
    border-inline-end-color: hsl(var(--in) / 0.6)}
.border-e-info\/70 {
    border-inline-end-color: hsl(var(--in) / 0.7)}
.border-e-info\/75 {
    border-inline-end-color: hsl(var(--in) / 0.75)}
.border-e-info\/80 {
    border-inline-end-color: hsl(var(--in) / 0.8)}
.border-e-info\/90 {
    border-inline-end-color: hsl(var(--in) / 0.9)}
.border-e-info\/95 {
    border-inline-end-color: hsl(var(--in) / 0.95)}
.border-e-neutral {
    border-inline-end-color: hsl(var(--n) / 1)}
.border-e-neutral-content {
    border-inline-end-color: hsl(var(--nc) / 1)}
.border-e-neutral-content\/0 {
    border-inline-end-color: hsl(var(--nc) / 0)}
.border-e-neutral-content\/10 {
    border-inline-end-color: hsl(var(--nc) / 0.1)}
.border-e-neutral-content\/100 {
    border-inline-end-color: hsl(var(--nc) / 1)}
.border-e-neutral-content\/20 {
    border-inline-end-color: hsl(var(--nc) / 0.2)}
.border-e-neutral-content\/25 {
    border-inline-end-color: hsl(var(--nc) / 0.25)}
.border-e-neutral-content\/30 {
    border-inline-end-color: hsl(var(--nc) / 0.3)}
.border-e-neutral-content\/40 {
    border-inline-end-color: hsl(var(--nc) / 0.4)}
.border-e-neutral-content\/5 {
    border-inline-end-color: hsl(var(--nc) / 0.05)}
.border-e-neutral-content\/50 {
    border-inline-end-color: hsl(var(--nc) / 0.5)}
.border-e-neutral-content\/60 {
    border-inline-end-color: hsl(var(--nc) / 0.6)}
.border-e-neutral-content\/70 {
    border-inline-end-color: hsl(var(--nc) / 0.7)}
.border-e-neutral-content\/75 {
    border-inline-end-color: hsl(var(--nc) / 0.75)}
.border-e-neutral-content\/80 {
    border-inline-end-color: hsl(var(--nc) / 0.8)}
.border-e-neutral-content\/90 {
    border-inline-end-color: hsl(var(--nc) / 0.9)}
.border-e-neutral-content\/95 {
    border-inline-end-color: hsl(var(--nc) / 0.95)}
.border-e-neutral-focus {
    border-inline-end-color: hsl(var(--nf) / 1)}
.border-e-neutral-focus\/0 {
    border-inline-end-color: hsl(var(--nf) / 0)}
.border-e-neutral-focus\/10 {
    border-inline-end-color: hsl(var(--nf) / 0.1)}
.border-e-neutral-focus\/100 {
    border-inline-end-color: hsl(var(--nf) / 1)}
.border-e-neutral-focus\/20 {
    border-inline-end-color: hsl(var(--nf) / 0.2)}
.border-e-neutral-focus\/25 {
    border-inline-end-color: hsl(var(--nf) / 0.25)}
.border-e-neutral-focus\/30 {
    border-inline-end-color: hsl(var(--nf) / 0.3)}
.border-e-neutral-focus\/40 {
    border-inline-end-color: hsl(var(--nf) / 0.4)}
.border-e-neutral-focus\/5 {
    border-inline-end-color: hsl(var(--nf) / 0.05)}
.border-e-neutral-focus\/50 {
    border-inline-end-color: hsl(var(--nf) / 0.5)}
.border-e-neutral-focus\/60 {
    border-inline-end-color: hsl(var(--nf) / 0.6)}
.border-e-neutral-focus\/70 {
    border-inline-end-color: hsl(var(--nf) / 0.7)}
.border-e-neutral-focus\/75 {
    border-inline-end-color: hsl(var(--nf) / 0.75)}
.border-e-neutral-focus\/80 {
    border-inline-end-color: hsl(var(--nf) / 0.8)}
.border-e-neutral-focus\/90 {
    border-inline-end-color: hsl(var(--nf) / 0.9)}
.border-e-neutral-focus\/95 {
    border-inline-end-color: hsl(var(--nf) / 0.95)}
.border-e-neutral\/0 {
    border-inline-end-color: hsl(var(--n) / 0)}
.border-e-neutral\/10 {
    border-inline-end-color: hsl(var(--n) / 0.1)}
.border-e-neutral\/100 {
    border-inline-end-color: hsl(var(--n) / 1)}
.border-e-neutral\/20 {
    border-inline-end-color: hsl(var(--n) / 0.2)}
.border-e-neutral\/25 {
    border-inline-end-color: hsl(var(--n) / 0.25)}
.border-e-neutral\/30 {
    border-inline-end-color: hsl(var(--n) / 0.3)}
.border-e-neutral\/40 {
    border-inline-end-color: hsl(var(--n) / 0.4)}
.border-e-neutral\/5 {
    border-inline-end-color: hsl(var(--n) / 0.05)}
.border-e-neutral\/50 {
    border-inline-end-color: hsl(var(--n) / 0.5)}
.border-e-neutral\/60 {
    border-inline-end-color: hsl(var(--n) / 0.6)}
.border-e-neutral\/70 {
    border-inline-end-color: hsl(var(--n) / 0.7)}
.border-e-neutral\/75 {
    border-inline-end-color: hsl(var(--n) / 0.75)}
.border-e-neutral\/80 {
    border-inline-end-color: hsl(var(--n) / 0.8)}
.border-e-neutral\/90 {
    border-inline-end-color: hsl(var(--n) / 0.9)}
.border-e-neutral\/95 {
    border-inline-end-color: hsl(var(--n) / 0.95)}
.border-e-primary {
    border-inline-end-color: hsl(var(--p) / 1)}
.border-e-primary-content {
    border-inline-end-color: hsl(var(--pc) / 1)}
.border-e-primary-content\/0 {
    border-inline-end-color: hsl(var(--pc) / 0)}
.border-e-primary-content\/10 {
    border-inline-end-color: hsl(var(--pc) / 0.1)}
.border-e-primary-content\/100 {
    border-inline-end-color: hsl(var(--pc) / 1)}
.border-e-primary-content\/20 {
    border-inline-end-color: hsl(var(--pc) / 0.2)}
.border-e-primary-content\/25 {
    border-inline-end-color: hsl(var(--pc) / 0.25)}
.border-e-primary-content\/30 {
    border-inline-end-color: hsl(var(--pc) / 0.3)}
.border-e-primary-content\/40 {
    border-inline-end-color: hsl(var(--pc) / 0.4)}
.border-e-primary-content\/5 {
    border-inline-end-color: hsl(var(--pc) / 0.05)}
.border-e-primary-content\/50 {
    border-inline-end-color: hsl(var(--pc) / 0.5)}
.border-e-primary-content\/60 {
    border-inline-end-color: hsl(var(--pc) / 0.6)}
.border-e-primary-content\/70 {
    border-inline-end-color: hsl(var(--pc) / 0.7)}
.border-e-primary-content\/75 {
    border-inline-end-color: hsl(var(--pc) / 0.75)}
.border-e-primary-content\/80 {
    border-inline-end-color: hsl(var(--pc) / 0.8)}
.border-e-primary-content\/90 {
    border-inline-end-color: hsl(var(--pc) / 0.9)}
.border-e-primary-content\/95 {
    border-inline-end-color: hsl(var(--pc) / 0.95)}
.border-e-primary-focus {
    border-inline-end-color: hsl(var(--pf) / 1)}
.border-e-primary-focus\/0 {
    border-inline-end-color: hsl(var(--pf) / 0)}
.border-e-primary-focus\/10 {
    border-inline-end-color: hsl(var(--pf) / 0.1)}
.border-e-primary-focus\/100 {
    border-inline-end-color: hsl(var(--pf) / 1)}
.border-e-primary-focus\/20 {
    border-inline-end-color: hsl(var(--pf) / 0.2)}
.border-e-primary-focus\/25 {
    border-inline-end-color: hsl(var(--pf) / 0.25)}
.border-e-primary-focus\/30 {
    border-inline-end-color: hsl(var(--pf) / 0.3)}
.border-e-primary-focus\/40 {
    border-inline-end-color: hsl(var(--pf) / 0.4)}
.border-e-primary-focus\/5 {
    border-inline-end-color: hsl(var(--pf) / 0.05)}
.border-e-primary-focus\/50 {
    border-inline-end-color: hsl(var(--pf) / 0.5)}
.border-e-primary-focus\/60 {
    border-inline-end-color: hsl(var(--pf) / 0.6)}
.border-e-primary-focus\/70 {
    border-inline-end-color: hsl(var(--pf) / 0.7)}
.border-e-primary-focus\/75 {
    border-inline-end-color: hsl(var(--pf) / 0.75)}
.border-e-primary-focus\/80 {
    border-inline-end-color: hsl(var(--pf) / 0.8)}
.border-e-primary-focus\/90 {
    border-inline-end-color: hsl(var(--pf) / 0.9)}
.border-e-primary-focus\/95 {
    border-inline-end-color: hsl(var(--pf) / 0.95)}
.border-e-primary\/0 {
    border-inline-end-color: hsl(var(--p) / 0)}
.border-e-primary\/10 {
    border-inline-end-color: hsl(var(--p) / 0.1)}
.border-e-primary\/100 {
    border-inline-end-color: hsl(var(--p) / 1)}
.border-e-primary\/20 {
    border-inline-end-color: hsl(var(--p) / 0.2)}
.border-e-primary\/25 {
    border-inline-end-color: hsl(var(--p) / 0.25)}
.border-e-primary\/30 {
    border-inline-end-color: hsl(var(--p) / 0.3)}
.border-e-primary\/40 {
    border-inline-end-color: hsl(var(--p) / 0.4)}
.border-e-primary\/5 {
    border-inline-end-color: hsl(var(--p) / 0.05)}
.border-e-primary\/50 {
    border-inline-end-color: hsl(var(--p) / 0.5)}
.border-e-primary\/60 {
    border-inline-end-color: hsl(var(--p) / 0.6)}
.border-e-primary\/70 {
    border-inline-end-color: hsl(var(--p) / 0.7)}
.border-e-primary\/75 {
    border-inline-end-color: hsl(var(--p) / 0.75)}
.border-e-primary\/80 {
    border-inline-end-color: hsl(var(--p) / 0.8)}
.border-e-primary\/90 {
    border-inline-end-color: hsl(var(--p) / 0.9)}
.border-e-primary\/95 {
    border-inline-end-color: hsl(var(--p) / 0.95)}
.border-e-secondary {
    border-inline-end-color: hsl(var(--s) / 1)}
.border-e-secondary-content {
    border-inline-end-color: hsl(var(--sc) / 1)}
.border-e-secondary-content\/0 {
    border-inline-end-color: hsl(var(--sc) / 0)}
.border-e-secondary-content\/10 {
    border-inline-end-color: hsl(var(--sc) / 0.1)}
.border-e-secondary-content\/100 {
    border-inline-end-color: hsl(var(--sc) / 1)}
.border-e-secondary-content\/20 {
    border-inline-end-color: hsl(var(--sc) / 0.2)}
.border-e-secondary-content\/25 {
    border-inline-end-color: hsl(var(--sc) / 0.25)}
.border-e-secondary-content\/30 {
    border-inline-end-color: hsl(var(--sc) / 0.3)}
.border-e-secondary-content\/40 {
    border-inline-end-color: hsl(var(--sc) / 0.4)}
.border-e-secondary-content\/5 {
    border-inline-end-color: hsl(var(--sc) / 0.05)}
.border-e-secondary-content\/50 {
    border-inline-end-color: hsl(var(--sc) / 0.5)}
.border-e-secondary-content\/60 {
    border-inline-end-color: hsl(var(--sc) / 0.6)}
.border-e-secondary-content\/70 {
    border-inline-end-color: hsl(var(--sc) / 0.7)}
.border-e-secondary-content\/75 {
    border-inline-end-color: hsl(var(--sc) / 0.75)}
.border-e-secondary-content\/80 {
    border-inline-end-color: hsl(var(--sc) / 0.8)}
.border-e-secondary-content\/90 {
    border-inline-end-color: hsl(var(--sc) / 0.9)}
.border-e-secondary-content\/95 {
    border-inline-end-color: hsl(var(--sc) / 0.95)}
.border-e-secondary-focus {
    border-inline-end-color: hsl(var(--sf) / 1)}
.border-e-secondary-focus\/0 {
    border-inline-end-color: hsl(var(--sf) / 0)}
.border-e-secondary-focus\/10 {
    border-inline-end-color: hsl(var(--sf) / 0.1)}
.border-e-secondary-focus\/100 {
    border-inline-end-color: hsl(var(--sf) / 1)}
.border-e-secondary-focus\/20 {
    border-inline-end-color: hsl(var(--sf) / 0.2)}
.border-e-secondary-focus\/25 {
    border-inline-end-color: hsl(var(--sf) / 0.25)}
.border-e-secondary-focus\/30 {
    border-inline-end-color: hsl(var(--sf) / 0.3)}
.border-e-secondary-focus\/40 {
    border-inline-end-color: hsl(var(--sf) / 0.4)}
.border-e-secondary-focus\/5 {
    border-inline-end-color: hsl(var(--sf) / 0.05)}
.border-e-secondary-focus\/50 {
    border-inline-end-color: hsl(var(--sf) / 0.5)}
.border-e-secondary-focus\/60 {
    border-inline-end-color: hsl(var(--sf) / 0.6)}
.border-e-secondary-focus\/70 {
    border-inline-end-color: hsl(var(--sf) / 0.7)}
.border-e-secondary-focus\/75 {
    border-inline-end-color: hsl(var(--sf) / 0.75)}
.border-e-secondary-focus\/80 {
    border-inline-end-color: hsl(var(--sf) / 0.8)}
.border-e-secondary-focus\/90 {
    border-inline-end-color: hsl(var(--sf) / 0.9)}
.border-e-secondary-focus\/95 {
    border-inline-end-color: hsl(var(--sf) / 0.95)}
.border-e-secondary\/0 {
    border-inline-end-color: hsl(var(--s) / 0)}
.border-e-secondary\/10 {
    border-inline-end-color: hsl(var(--s) / 0.1)}
.border-e-secondary\/100 {
    border-inline-end-color: hsl(var(--s) / 1)}
.border-e-secondary\/20 {
    border-inline-end-color: hsl(var(--s) / 0.2)}
.border-e-secondary\/25 {
    border-inline-end-color: hsl(var(--s) / 0.25)}
.border-e-secondary\/30 {
    border-inline-end-color: hsl(var(--s) / 0.3)}
.border-e-secondary\/40 {
    border-inline-end-color: hsl(var(--s) / 0.4)}
.border-e-secondary\/5 {
    border-inline-end-color: hsl(var(--s) / 0.05)}
.border-e-secondary\/50 {
    border-inline-end-color: hsl(var(--s) / 0.5)}
.border-e-secondary\/60 {
    border-inline-end-color: hsl(var(--s) / 0.6)}
.border-e-secondary\/70 {
    border-inline-end-color: hsl(var(--s) / 0.7)}
.border-e-secondary\/75 {
    border-inline-end-color: hsl(var(--s) / 0.75)}
.border-e-secondary\/80 {
    border-inline-end-color: hsl(var(--s) / 0.8)}
.border-e-secondary\/90 {
    border-inline-end-color: hsl(var(--s) / 0.9)}
.border-e-secondary\/95 {
    border-inline-end-color: hsl(var(--s) / 0.95)}
.border-e-success {
    border-inline-end-color: hsl(var(--su) / 1)}
.border-e-success-content {
    border-inline-end-color: hsl(var(--suc) / 1)}
.border-e-success-content\/0 {
    border-inline-end-color: hsl(var(--suc) / 0)}
.border-e-success-content\/10 {
    border-inline-end-color: hsl(var(--suc) / 0.1)}
.border-e-success-content\/100 {
    border-inline-end-color: hsl(var(--suc) / 1)}
.border-e-success-content\/20 {
    border-inline-end-color: hsl(var(--suc) / 0.2)}
.border-e-success-content\/25 {
    border-inline-end-color: hsl(var(--suc) / 0.25)}
.border-e-success-content\/30 {
    border-inline-end-color: hsl(var(--suc) / 0.3)}
.border-e-success-content\/40 {
    border-inline-end-color: hsl(var(--suc) / 0.4)}
.border-e-success-content\/5 {
    border-inline-end-color: hsl(var(--suc) / 0.05)}
.border-e-success-content\/50 {
    border-inline-end-color: hsl(var(--suc) / 0.5)}
.border-e-success-content\/60 {
    border-inline-end-color: hsl(var(--suc) / 0.6)}
.border-e-success-content\/70 {
    border-inline-end-color: hsl(var(--suc) / 0.7)}
.border-e-success-content\/75 {
    border-inline-end-color: hsl(var(--suc) / 0.75)}
.border-e-success-content\/80 {
    border-inline-end-color: hsl(var(--suc) / 0.8)}
.border-e-success-content\/90 {
    border-inline-end-color: hsl(var(--suc) / 0.9)}
.border-e-success-content\/95 {
    border-inline-end-color: hsl(var(--suc) / 0.95)}
.border-e-success\/0 {
    border-inline-end-color: hsl(var(--su) / 0)}
.border-e-success\/10 {
    border-inline-end-color: hsl(var(--su) / 0.1)}
.border-e-success\/100 {
    border-inline-end-color: hsl(var(--su) / 1)}
.border-e-success\/20 {
    border-inline-end-color: hsl(var(--su) / 0.2)}
.border-e-success\/25 {
    border-inline-end-color: hsl(var(--su) / 0.25)}
.border-e-success\/30 {
    border-inline-end-color: hsl(var(--su) / 0.3)}
.border-e-success\/40 {
    border-inline-end-color: hsl(var(--su) / 0.4)}
.border-e-success\/5 {
    border-inline-end-color: hsl(var(--su) / 0.05)}
.border-e-success\/50 {
    border-inline-end-color: hsl(var(--su) / 0.5)}
.border-e-success\/60 {
    border-inline-end-color: hsl(var(--su) / 0.6)}
.border-e-success\/70 {
    border-inline-end-color: hsl(var(--su) / 0.7)}
.border-e-success\/75 {
    border-inline-end-color: hsl(var(--su) / 0.75)}
.border-e-success\/80 {
    border-inline-end-color: hsl(var(--su) / 0.8)}
.border-e-success\/90 {
    border-inline-end-color: hsl(var(--su) / 0.9)}
.border-e-success\/95 {
    border-inline-end-color: hsl(var(--su) / 0.95)}
.border-e-transparent {
    border-inline-end-color: transparent}
.border-e-transparent\/0 {
    border-inline-end-color: rgb(0 0 0 / 0)}
.border-e-transparent\/10 {
    border-inline-end-color: rgb(0 0 0 / 0.1)}
.border-e-transparent\/100 {
    border-inline-end-color: rgb(0 0 0 / 1)}
.border-e-transparent\/20 {
    border-inline-end-color: rgb(0 0 0 / 0.2)}
.border-e-transparent\/25 {
    border-inline-end-color: rgb(0 0 0 / 0.25)}
.border-e-transparent\/30 {
    border-inline-end-color: rgb(0 0 0 / 0.3)}
.border-e-transparent\/40 {
    border-inline-end-color: rgb(0 0 0 / 0.4)}
.border-e-transparent\/5 {
    border-inline-end-color: rgb(0 0 0 / 0.05)}
.border-e-transparent\/50 {
    border-inline-end-color: rgb(0 0 0 / 0.5)}
.border-e-transparent\/60 {
    border-inline-end-color: rgb(0 0 0 / 0.6)}
.border-e-transparent\/70 {
    border-inline-end-color: rgb(0 0 0 / 0.7)}
.border-e-transparent\/75 {
    border-inline-end-color: rgb(0 0 0 / 0.75)}
.border-e-transparent\/80 {
    border-inline-end-color: rgb(0 0 0 / 0.8)}
.border-e-transparent\/90 {
    border-inline-end-color: rgb(0 0 0 / 0.9)}
.border-e-transparent\/95 {
    border-inline-end-color: rgb(0 0 0 / 0.95)}
.border-e-warning {
    border-inline-end-color: hsl(var(--wa) / 1)}
.border-e-warning-content {
    border-inline-end-color: hsl(var(--wac) / 1)}
.border-e-warning-content\/0 {
    border-inline-end-color: hsl(var(--wac) / 0)}
.border-e-warning-content\/10 {
    border-inline-end-color: hsl(var(--wac) / 0.1)}
.border-e-warning-content\/100 {
    border-inline-end-color: hsl(var(--wac) / 1)}
.border-e-warning-content\/20 {
    border-inline-end-color: hsl(var(--wac) / 0.2)}
.border-e-warning-content\/25 {
    border-inline-end-color: hsl(var(--wac) / 0.25)}
.border-e-warning-content\/30 {
    border-inline-end-color: hsl(var(--wac) / 0.3)}
.border-e-warning-content\/40 {
    border-inline-end-color: hsl(var(--wac) / 0.4)}
.border-e-warning-content\/5 {
    border-inline-end-color: hsl(var(--wac) / 0.05)}
.border-e-warning-content\/50 {
    border-inline-end-color: hsl(var(--wac) / 0.5)}
.border-e-warning-content\/60 {
    border-inline-end-color: hsl(var(--wac) / 0.6)}
.border-e-warning-content\/70 {
    border-inline-end-color: hsl(var(--wac) / 0.7)}
.border-e-warning-content\/75 {
    border-inline-end-color: hsl(var(--wac) / 0.75)}
.border-e-warning-content\/80 {
    border-inline-end-color: hsl(var(--wac) / 0.8)}
.border-e-warning-content\/90 {
    border-inline-end-color: hsl(var(--wac) / 0.9)}
.border-e-warning-content\/95 {
    border-inline-end-color: hsl(var(--wac) / 0.95)}
.border-e-warning\/0 {
    border-inline-end-color: hsl(var(--wa) / 0)}
.border-e-warning\/10 {
    border-inline-end-color: hsl(var(--wa) / 0.1)}
.border-e-warning\/100 {
    border-inline-end-color: hsl(var(--wa) / 1)}
.border-e-warning\/20 {
    border-inline-end-color: hsl(var(--wa) / 0.2)}
.border-e-warning\/25 {
    border-inline-end-color: hsl(var(--wa) / 0.25)}
.border-e-warning\/30 {
    border-inline-end-color: hsl(var(--wa) / 0.3)}
.border-e-warning\/40 {
    border-inline-end-color: hsl(var(--wa) / 0.4)}
.border-e-warning\/5 {
    border-inline-end-color: hsl(var(--wa) / 0.05)}
.border-e-warning\/50 {
    border-inline-end-color: hsl(var(--wa) / 0.5)}
.border-e-warning\/60 {
    border-inline-end-color: hsl(var(--wa) / 0.6)}
.border-e-warning\/70 {
    border-inline-end-color: hsl(var(--wa) / 0.7)}
.border-e-warning\/75 {
    border-inline-end-color: hsl(var(--wa) / 0.75)}
.border-e-warning\/80 {
    border-inline-end-color: hsl(var(--wa) / 0.8)}
.border-e-warning\/90 {
    border-inline-end-color: hsl(var(--wa) / 0.9)}
.border-e-warning\/95 {
    border-inline-end-color: hsl(var(--wa) / 0.95)}
.border-l-accent {
    border-left-color: hsl(var(--a) / 1)}
.border-l-accent-content {
    border-left-color: hsl(var(--ac) / 1)}
.border-l-accent-content\/0 {
    border-left-color: hsl(var(--ac) / 0)}
.border-l-accent-content\/10 {
    border-left-color: hsl(var(--ac) / 0.1)}
.border-l-accent-content\/100 {
    border-left-color: hsl(var(--ac) / 1)}
.border-l-accent-content\/20 {
    border-left-color: hsl(var(--ac) / 0.2)}
.border-l-accent-content\/25 {
    border-left-color: hsl(var(--ac) / 0.25)}
.border-l-accent-content\/30 {
    border-left-color: hsl(var(--ac) / 0.3)}
.border-l-accent-content\/40 {
    border-left-color: hsl(var(--ac) / 0.4)}
.border-l-accent-content\/5 {
    border-left-color: hsl(var(--ac) / 0.05)}
.border-l-accent-content\/50 {
    border-left-color: hsl(var(--ac) / 0.5)}
.border-l-accent-content\/60 {
    border-left-color: hsl(var(--ac) / 0.6)}
.border-l-accent-content\/70 {
    border-left-color: hsl(var(--ac) / 0.7)}
.border-l-accent-content\/75 {
    border-left-color: hsl(var(--ac) / 0.75)}
.border-l-accent-content\/80 {
    border-left-color: hsl(var(--ac) / 0.8)}
.border-l-accent-content\/90 {
    border-left-color: hsl(var(--ac) / 0.9)}
.border-l-accent-content\/95 {
    border-left-color: hsl(var(--ac) / 0.95)}
.border-l-accent-focus {
    border-left-color: hsl(var(--af) / 1)}
.border-l-accent-focus\/0 {
    border-left-color: hsl(var(--af) / 0)}
.border-l-accent-focus\/10 {
    border-left-color: hsl(var(--af) / 0.1)}
.border-l-accent-focus\/100 {
    border-left-color: hsl(var(--af) / 1)}
.border-l-accent-focus\/20 {
    border-left-color: hsl(var(--af) / 0.2)}
.border-l-accent-focus\/25 {
    border-left-color: hsl(var(--af) / 0.25)}
.border-l-accent-focus\/30 {
    border-left-color: hsl(var(--af) / 0.3)}
.border-l-accent-focus\/40 {
    border-left-color: hsl(var(--af) / 0.4)}
.border-l-accent-focus\/5 {
    border-left-color: hsl(var(--af) / 0.05)}
.border-l-accent-focus\/50 {
    border-left-color: hsl(var(--af) / 0.5)}
.border-l-accent-focus\/60 {
    border-left-color: hsl(var(--af) / 0.6)}
.border-l-accent-focus\/70 {
    border-left-color: hsl(var(--af) / 0.7)}
.border-l-accent-focus\/75 {
    border-left-color: hsl(var(--af) / 0.75)}
.border-l-accent-focus\/80 {
    border-left-color: hsl(var(--af) / 0.8)}
.border-l-accent-focus\/90 {
    border-left-color: hsl(var(--af) / 0.9)}
.border-l-accent-focus\/95 {
    border-left-color: hsl(var(--af) / 0.95)}
.border-l-accent\/0 {
    border-left-color: hsl(var(--a) / 0)}
.border-l-accent\/10 {
    border-left-color: hsl(var(--a) / 0.1)}
.border-l-accent\/100 {
    border-left-color: hsl(var(--a) / 1)}
.border-l-accent\/20 {
    border-left-color: hsl(var(--a) / 0.2)}
.border-l-accent\/25 {
    border-left-color: hsl(var(--a) / 0.25)}
.border-l-accent\/30 {
    border-left-color: hsl(var(--a) / 0.3)}
.border-l-accent\/40 {
    border-left-color: hsl(var(--a) / 0.4)}
.border-l-accent\/5 {
    border-left-color: hsl(var(--a) / 0.05)}
.border-l-accent\/50 {
    border-left-color: hsl(var(--a) / 0.5)}
.border-l-accent\/60 {
    border-left-color: hsl(var(--a) / 0.6)}
.border-l-accent\/70 {
    border-left-color: hsl(var(--a) / 0.7)}
.border-l-accent\/75 {
    border-left-color: hsl(var(--a) / 0.75)}
.border-l-accent\/80 {
    border-left-color: hsl(var(--a) / 0.8)}
.border-l-accent\/90 {
    border-left-color: hsl(var(--a) / 0.9)}
.border-l-accent\/95 {
    border-left-color: hsl(var(--a) / 0.95)}
.border-l-base-100 {
    border-left-color: hsl(var(--b1) / 1)}
.border-l-base-100\/0 {
    border-left-color: hsl(var(--b1) / 0)}
.border-l-base-100\/10 {
    border-left-color: hsl(var(--b1) / 0.1)}
.border-l-base-100\/100 {
    border-left-color: hsl(var(--b1) / 1)}
.border-l-base-100\/20 {
    border-left-color: hsl(var(--b1) / 0.2)}
.border-l-base-100\/25 {
    border-left-color: hsl(var(--b1) / 0.25)}
.border-l-base-100\/30 {
    border-left-color: hsl(var(--b1) / 0.3)}
.border-l-base-100\/40 {
    border-left-color: hsl(var(--b1) / 0.4)}
.border-l-base-100\/5 {
    border-left-color: hsl(var(--b1) / 0.05)}
.border-l-base-100\/50 {
    border-left-color: hsl(var(--b1) / 0.5)}
.border-l-base-100\/60 {
    border-left-color: hsl(var(--b1) / 0.6)}
.border-l-base-100\/70 {
    border-left-color: hsl(var(--b1) / 0.7)}
.border-l-base-100\/75 {
    border-left-color: hsl(var(--b1) / 0.75)}
.border-l-base-100\/80 {
    border-left-color: hsl(var(--b1) / 0.8)}
.border-l-base-100\/90 {
    border-left-color: hsl(var(--b1) / 0.9)}
.border-l-base-100\/95 {
    border-left-color: hsl(var(--b1) / 0.95)}
.border-l-base-200 {
    border-left-color: hsl(var(--b2) / 1)}
.border-l-base-200\/0 {
    border-left-color: hsl(var(--b2) / 0)}
.border-l-base-200\/10 {
    border-left-color: hsl(var(--b2) / 0.1)}
.border-l-base-200\/100 {
    border-left-color: hsl(var(--b2) / 1)}
.border-l-base-200\/20 {
    border-left-color: hsl(var(--b2) / 0.2)}
.border-l-base-200\/25 {
    border-left-color: hsl(var(--b2) / 0.25)}
.border-l-base-200\/30 {
    border-left-color: hsl(var(--b2) / 0.3)}
.border-l-base-200\/40 {
    border-left-color: hsl(var(--b2) / 0.4)}
.border-l-base-200\/5 {
    border-left-color: hsl(var(--b2) / 0.05)}
.border-l-base-200\/50 {
    border-left-color: hsl(var(--b2) / 0.5)}
.border-l-base-200\/60 {
    border-left-color: hsl(var(--b2) / 0.6)}
.border-l-base-200\/70 {
    border-left-color: hsl(var(--b2) / 0.7)}
.border-l-base-200\/75 {
    border-left-color: hsl(var(--b2) / 0.75)}
.border-l-base-200\/80 {
    border-left-color: hsl(var(--b2) / 0.8)}
.border-l-base-200\/90 {
    border-left-color: hsl(var(--b2) / 0.9)}
.border-l-base-200\/95 {
    border-left-color: hsl(var(--b2) / 0.95)}
.border-l-base-300 {
    border-left-color: hsl(var(--b3) / 1)}
.border-l-base-300\/0 {
    border-left-color: hsl(var(--b3) / 0)}
.border-l-base-300\/10 {
    border-left-color: hsl(var(--b3) / 0.1)}
.border-l-base-300\/100 {
    border-left-color: hsl(var(--b3) / 1)}
.border-l-base-300\/20 {
    border-left-color: hsl(var(--b3) / 0.2)}
.border-l-base-300\/25 {
    border-left-color: hsl(var(--b3) / 0.25)}
.border-l-base-300\/30 {
    border-left-color: hsl(var(--b3) / 0.3)}
.border-l-base-300\/40 {
    border-left-color: hsl(var(--b3) / 0.4)}
.border-l-base-300\/5 {
    border-left-color: hsl(var(--b3) / 0.05)}
.border-l-base-300\/50 {
    border-left-color: hsl(var(--b3) / 0.5)}
.border-l-base-300\/60 {
    border-left-color: hsl(var(--b3) / 0.6)}
.border-l-base-300\/70 {
    border-left-color: hsl(var(--b3) / 0.7)}
.border-l-base-300\/75 {
    border-left-color: hsl(var(--b3) / 0.75)}
.border-l-base-300\/80 {
    border-left-color: hsl(var(--b3) / 0.8)}
.border-l-base-300\/90 {
    border-left-color: hsl(var(--b3) / 0.9)}
.border-l-base-300\/95 {
    border-left-color: hsl(var(--b3) / 0.95)}
.border-l-base-content {
    border-left-color: hsl(var(--bc) / 1)}
.border-l-base-content\/0 {
    border-left-color: hsl(var(--bc) / 0)}
.border-l-base-content\/10 {
    border-left-color: hsl(var(--bc) / 0.1)}
.border-l-base-content\/100 {
    border-left-color: hsl(var(--bc) / 1)}
.border-l-base-content\/20 {
    border-left-color: hsl(var(--bc) / 0.2)}
.border-l-base-content\/25 {
    border-left-color: hsl(var(--bc) / 0.25)}
.border-l-base-content\/30 {
    border-left-color: hsl(var(--bc) / 0.3)}
.border-l-base-content\/40 {
    border-left-color: hsl(var(--bc) / 0.4)}
.border-l-base-content\/5 {
    border-left-color: hsl(var(--bc) / 0.05)}
.border-l-base-content\/50 {
    border-left-color: hsl(var(--bc) / 0.5)}
.border-l-base-content\/60 {
    border-left-color: hsl(var(--bc) / 0.6)}
.border-l-base-content\/70 {
    border-left-color: hsl(var(--bc) / 0.7)}
.border-l-base-content\/75 {
    border-left-color: hsl(var(--bc) / 0.75)}
.border-l-base-content\/80 {
    border-left-color: hsl(var(--bc) / 0.8)}
.border-l-base-content\/90 {
    border-left-color: hsl(var(--bc) / 0.9)}
.border-l-base-content\/95 {
    border-left-color: hsl(var(--bc) / 0.95)}
.border-l-current {
    border-left-color: currentColor}
.border-l-error {
    border-left-color: hsl(var(--er) / 1)}
.border-l-error-content {
    border-left-color: hsl(var(--erc) / 1)}
.border-l-error-content\/0 {
    border-left-color: hsl(var(--erc) / 0)}
.border-l-error-content\/10 {
    border-left-color: hsl(var(--erc) / 0.1)}
.border-l-error-content\/100 {
    border-left-color: hsl(var(--erc) / 1)}
.border-l-error-content\/20 {
    border-left-color: hsl(var(--erc) / 0.2)}
.border-l-error-content\/25 {
    border-left-color: hsl(var(--erc) / 0.25)}
.border-l-error-content\/30 {
    border-left-color: hsl(var(--erc) / 0.3)}
.border-l-error-content\/40 {
    border-left-color: hsl(var(--erc) / 0.4)}
.border-l-error-content\/5 {
    border-left-color: hsl(var(--erc) / 0.05)}
.border-l-error-content\/50 {
    border-left-color: hsl(var(--erc) / 0.5)}
.border-l-error-content\/60 {
    border-left-color: hsl(var(--erc) / 0.6)}
.border-l-error-content\/70 {
    border-left-color: hsl(var(--erc) / 0.7)}
.border-l-error-content\/75 {
    border-left-color: hsl(var(--erc) / 0.75)}
.border-l-error-content\/80 {
    border-left-color: hsl(var(--erc) / 0.8)}
.border-l-error-content\/90 {
    border-left-color: hsl(var(--erc) / 0.9)}
.border-l-error-content\/95 {
    border-left-color: hsl(var(--erc) / 0.95)}
.border-l-error\/0 {
    border-left-color: hsl(var(--er) / 0)}
.border-l-error\/10 {
    border-left-color: hsl(var(--er) / 0.1)}
.border-l-error\/100 {
    border-left-color: hsl(var(--er) / 1)}
.border-l-error\/20 {
    border-left-color: hsl(var(--er) / 0.2)}
.border-l-error\/25 {
    border-left-color: hsl(var(--er) / 0.25)}
.border-l-error\/30 {
    border-left-color: hsl(var(--er) / 0.3)}
.border-l-error\/40 {
    border-left-color: hsl(var(--er) / 0.4)}
.border-l-error\/5 {
    border-left-color: hsl(var(--er) / 0.05)}
.border-l-error\/50 {
    border-left-color: hsl(var(--er) / 0.5)}
.border-l-error\/60 {
    border-left-color: hsl(var(--er) / 0.6)}
.border-l-error\/70 {
    border-left-color: hsl(var(--er) / 0.7)}
.border-l-error\/75 {
    border-left-color: hsl(var(--er) / 0.75)}
.border-l-error\/80 {
    border-left-color: hsl(var(--er) / 0.8)}
.border-l-error\/90 {
    border-left-color: hsl(var(--er) / 0.9)}
.border-l-error\/95 {
    border-left-color: hsl(var(--er) / 0.95)}
.border-l-info {
    border-left-color: hsl(var(--in) / 1)}
.border-l-info-content {
    border-left-color: hsl(var(--inc) / 1)}
.border-l-info-content\/0 {
    border-left-color: hsl(var(--inc) / 0)}
.border-l-info-content\/10 {
    border-left-color: hsl(var(--inc) / 0.1)}
.border-l-info-content\/100 {
    border-left-color: hsl(var(--inc) / 1)}
.border-l-info-content\/20 {
    border-left-color: hsl(var(--inc) / 0.2)}
.border-l-info-content\/25 {
    border-left-color: hsl(var(--inc) / 0.25)}
.border-l-info-content\/30 {
    border-left-color: hsl(var(--inc) / 0.3)}
.border-l-info-content\/40 {
    border-left-color: hsl(var(--inc) / 0.4)}
.border-l-info-content\/5 {
    border-left-color: hsl(var(--inc) / 0.05)}
.border-l-info-content\/50 {
    border-left-color: hsl(var(--inc) / 0.5)}
.border-l-info-content\/60 {
    border-left-color: hsl(var(--inc) / 0.6)}
.border-l-info-content\/70 {
    border-left-color: hsl(var(--inc) / 0.7)}
.border-l-info-content\/75 {
    border-left-color: hsl(var(--inc) / 0.75)}
.border-l-info-content\/80 {
    border-left-color: hsl(var(--inc) / 0.8)}
.border-l-info-content\/90 {
    border-left-color: hsl(var(--inc) / 0.9)}
.border-l-info-content\/95 {
    border-left-color: hsl(var(--inc) / 0.95)}
.border-l-info\/0 {
    border-left-color: hsl(var(--in) / 0)}
.border-l-info\/10 {
    border-left-color: hsl(var(--in) / 0.1)}
.border-l-info\/100 {
    border-left-color: hsl(var(--in) / 1)}
.border-l-info\/20 {
    border-left-color: hsl(var(--in) / 0.2)}
.border-l-info\/25 {
    border-left-color: hsl(var(--in) / 0.25)}
.border-l-info\/30 {
    border-left-color: hsl(var(--in) / 0.3)}
.border-l-info\/40 {
    border-left-color: hsl(var(--in) / 0.4)}
.border-l-info\/5 {
    border-left-color: hsl(var(--in) / 0.05)}
.border-l-info\/50 {
    border-left-color: hsl(var(--in) / 0.5)}
.border-l-info\/60 {
    border-left-color: hsl(var(--in) / 0.6)}
.border-l-info\/70 {
    border-left-color: hsl(var(--in) / 0.7)}
.border-l-info\/75 {
    border-left-color: hsl(var(--in) / 0.75)}
.border-l-info\/80 {
    border-left-color: hsl(var(--in) / 0.8)}
.border-l-info\/90 {
    border-left-color: hsl(var(--in) / 0.9)}
.border-l-info\/95 {
    border-left-color: hsl(var(--in) / 0.95)}
.border-l-neutral {
    border-left-color: hsl(var(--n) / 1)}
.border-l-neutral-content {
    border-left-color: hsl(var(--nc) / 1)}
.border-l-neutral-content\/0 {
    border-left-color: hsl(var(--nc) / 0)}
.border-l-neutral-content\/10 {
    border-left-color: hsl(var(--nc) / 0.1)}
.border-l-neutral-content\/100 {
    border-left-color: hsl(var(--nc) / 1)}
.border-l-neutral-content\/20 {
    border-left-color: hsl(var(--nc) / 0.2)}
.border-l-neutral-content\/25 {
    border-left-color: hsl(var(--nc) / 0.25)}
.border-l-neutral-content\/30 {
    border-left-color: hsl(var(--nc) / 0.3)}
.border-l-neutral-content\/40 {
    border-left-color: hsl(var(--nc) / 0.4)}
.border-l-neutral-content\/5 {
    border-left-color: hsl(var(--nc) / 0.05)}
.border-l-neutral-content\/50 {
    border-left-color: hsl(var(--nc) / 0.5)}
.border-l-neutral-content\/60 {
    border-left-color: hsl(var(--nc) / 0.6)}
.border-l-neutral-content\/70 {
    border-left-color: hsl(var(--nc) / 0.7)}
.border-l-neutral-content\/75 {
    border-left-color: hsl(var(--nc) / 0.75)}
.border-l-neutral-content\/80 {
    border-left-color: hsl(var(--nc) / 0.8)}
.border-l-neutral-content\/90 {
    border-left-color: hsl(var(--nc) / 0.9)}
.border-l-neutral-content\/95 {
    border-left-color: hsl(var(--nc) / 0.95)}
.border-l-neutral-focus {
    border-left-color: hsl(var(--nf) / 1)}
.border-l-neutral-focus\/0 {
    border-left-color: hsl(var(--nf) / 0)}
.border-l-neutral-focus\/10 {
    border-left-color: hsl(var(--nf) / 0.1)}
.border-l-neutral-focus\/100 {
    border-left-color: hsl(var(--nf) / 1)}
.border-l-neutral-focus\/20 {
    border-left-color: hsl(var(--nf) / 0.2)}
.border-l-neutral-focus\/25 {
    border-left-color: hsl(var(--nf) / 0.25)}
.border-l-neutral-focus\/30 {
    border-left-color: hsl(var(--nf) / 0.3)}
.border-l-neutral-focus\/40 {
    border-left-color: hsl(var(--nf) / 0.4)}
.border-l-neutral-focus\/5 {
    border-left-color: hsl(var(--nf) / 0.05)}
.border-l-neutral-focus\/50 {
    border-left-color: hsl(var(--nf) / 0.5)}
.border-l-neutral-focus\/60 {
    border-left-color: hsl(var(--nf) / 0.6)}
.border-l-neutral-focus\/70 {
    border-left-color: hsl(var(--nf) / 0.7)}
.border-l-neutral-focus\/75 {
    border-left-color: hsl(var(--nf) / 0.75)}
.border-l-neutral-focus\/80 {
    border-left-color: hsl(var(--nf) / 0.8)}
.border-l-neutral-focus\/90 {
    border-left-color: hsl(var(--nf) / 0.9)}
.border-l-neutral-focus\/95 {
    border-left-color: hsl(var(--nf) / 0.95)}
.border-l-neutral\/0 {
    border-left-color: hsl(var(--n) / 0)}
.border-l-neutral\/10 {
    border-left-color: hsl(var(--n) / 0.1)}
.border-l-neutral\/100 {
    border-left-color: hsl(var(--n) / 1)}
.border-l-neutral\/20 {
    border-left-color: hsl(var(--n) / 0.2)}
.border-l-neutral\/25 {
    border-left-color: hsl(var(--n) / 0.25)}
.border-l-neutral\/30 {
    border-left-color: hsl(var(--n) / 0.3)}
.border-l-neutral\/40 {
    border-left-color: hsl(var(--n) / 0.4)}
.border-l-neutral\/5 {
    border-left-color: hsl(var(--n) / 0.05)}
.border-l-neutral\/50 {
    border-left-color: hsl(var(--n) / 0.5)}
.border-l-neutral\/60 {
    border-left-color: hsl(var(--n) / 0.6)}
.border-l-neutral\/70 {
    border-left-color: hsl(var(--n) / 0.7)}
.border-l-neutral\/75 {
    border-left-color: hsl(var(--n) / 0.75)}
.border-l-neutral\/80 {
    border-left-color: hsl(var(--n) / 0.8)}
.border-l-neutral\/90 {
    border-left-color: hsl(var(--n) / 0.9)}
.border-l-neutral\/95 {
    border-left-color: hsl(var(--n) / 0.95)}
.border-l-primary {
    border-left-color: hsl(var(--p) / 1)}
.border-l-primary-content {
    border-left-color: hsl(var(--pc) / 1)}
.border-l-primary-content\/0 {
    border-left-color: hsl(var(--pc) / 0)}
.border-l-primary-content\/10 {
    border-left-color: hsl(var(--pc) / 0.1)}
.border-l-primary-content\/100 {
    border-left-color: hsl(var(--pc) / 1)}
.border-l-primary-content\/20 {
    border-left-color: hsl(var(--pc) / 0.2)}
.border-l-primary-content\/25 {
    border-left-color: hsl(var(--pc) / 0.25)}
.border-l-primary-content\/30 {
    border-left-color: hsl(var(--pc) / 0.3)}
.border-l-primary-content\/40 {
    border-left-color: hsl(var(--pc) / 0.4)}
.border-l-primary-content\/5 {
    border-left-color: hsl(var(--pc) / 0.05)}
.border-l-primary-content\/50 {
    border-left-color: hsl(var(--pc) / 0.5)}
.border-l-primary-content\/60 {
    border-left-color: hsl(var(--pc) / 0.6)}
.border-l-primary-content\/70 {
    border-left-color: hsl(var(--pc) / 0.7)}
.border-l-primary-content\/75 {
    border-left-color: hsl(var(--pc) / 0.75)}
.border-l-primary-content\/80 {
    border-left-color: hsl(var(--pc) / 0.8)}
.border-l-primary-content\/90 {
    border-left-color: hsl(var(--pc) / 0.9)}
.border-l-primary-content\/95 {
    border-left-color: hsl(var(--pc) / 0.95)}
.border-l-primary-focus {
    border-left-color: hsl(var(--pf) / 1)}
.border-l-primary-focus\/0 {
    border-left-color: hsl(var(--pf) / 0)}
.border-l-primary-focus\/10 {
    border-left-color: hsl(var(--pf) / 0.1)}
.border-l-primary-focus\/100 {
    border-left-color: hsl(var(--pf) / 1)}
.border-l-primary-focus\/20 {
    border-left-color: hsl(var(--pf) / 0.2)}
.border-l-primary-focus\/25 {
    border-left-color: hsl(var(--pf) / 0.25)}
.border-l-primary-focus\/30 {
    border-left-color: hsl(var(--pf) / 0.3)}
.border-l-primary-focus\/40 {
    border-left-color: hsl(var(--pf) / 0.4)}
.border-l-primary-focus\/5 {
    border-left-color: hsl(var(--pf) / 0.05)}
.border-l-primary-focus\/50 {
    border-left-color: hsl(var(--pf) / 0.5)}
.border-l-primary-focus\/60 {
    border-left-color: hsl(var(--pf) / 0.6)}
.border-l-primary-focus\/70 {
    border-left-color: hsl(var(--pf) / 0.7)}
.border-l-primary-focus\/75 {
    border-left-color: hsl(var(--pf) / 0.75)}
.border-l-primary-focus\/80 {
    border-left-color: hsl(var(--pf) / 0.8)}
.border-l-primary-focus\/90 {
    border-left-color: hsl(var(--pf) / 0.9)}
.border-l-primary-focus\/95 {
    border-left-color: hsl(var(--pf) / 0.95)}
.border-l-primary\/0 {
    border-left-color: hsl(var(--p) / 0)}
.border-l-primary\/10 {
    border-left-color: hsl(var(--p) / 0.1)}
.border-l-primary\/100 {
    border-left-color: hsl(var(--p) / 1)}
.border-l-primary\/20 {
    border-left-color: hsl(var(--p) / 0.2)}
.border-l-primary\/25 {
    border-left-color: hsl(var(--p) / 0.25)}
.border-l-primary\/30 {
    border-left-color: hsl(var(--p) / 0.3)}
.border-l-primary\/40 {
    border-left-color: hsl(var(--p) / 0.4)}
.border-l-primary\/5 {
    border-left-color: hsl(var(--p) / 0.05)}
.border-l-primary\/50 {
    border-left-color: hsl(var(--p) / 0.5)}
.border-l-primary\/60 {
    border-left-color: hsl(var(--p) / 0.6)}
.border-l-primary\/70 {
    border-left-color: hsl(var(--p) / 0.7)}
.border-l-primary\/75 {
    border-left-color: hsl(var(--p) / 0.75)}
.border-l-primary\/80 {
    border-left-color: hsl(var(--p) / 0.8)}
.border-l-primary\/90 {
    border-left-color: hsl(var(--p) / 0.9)}
.border-l-primary\/95 {
    border-left-color: hsl(var(--p) / 0.95)}
.border-l-secondary {
    border-left-color: hsl(var(--s) / 1)}
.border-l-secondary-content {
    border-left-color: hsl(var(--sc) / 1)}
.border-l-secondary-content\/0 {
    border-left-color: hsl(var(--sc) / 0)}
.border-l-secondary-content\/10 {
    border-left-color: hsl(var(--sc) / 0.1)}
.border-l-secondary-content\/100 {
    border-left-color: hsl(var(--sc) / 1)}
.border-l-secondary-content\/20 {
    border-left-color: hsl(var(--sc) / 0.2)}
.border-l-secondary-content\/25 {
    border-left-color: hsl(var(--sc) / 0.25)}
.border-l-secondary-content\/30 {
    border-left-color: hsl(var(--sc) / 0.3)}
.border-l-secondary-content\/40 {
    border-left-color: hsl(var(--sc) / 0.4)}
.border-l-secondary-content\/5 {
    border-left-color: hsl(var(--sc) / 0.05)}
.border-l-secondary-content\/50 {
    border-left-color: hsl(var(--sc) / 0.5)}
.border-l-secondary-content\/60 {
    border-left-color: hsl(var(--sc) / 0.6)}
.border-l-secondary-content\/70 {
    border-left-color: hsl(var(--sc) / 0.7)}
.border-l-secondary-content\/75 {
    border-left-color: hsl(var(--sc) / 0.75)}
.border-l-secondary-content\/80 {
    border-left-color: hsl(var(--sc) / 0.8)}
.border-l-secondary-content\/90 {
    border-left-color: hsl(var(--sc) / 0.9)}
.border-l-secondary-content\/95 {
    border-left-color: hsl(var(--sc) / 0.95)}
.border-l-secondary-focus {
    border-left-color: hsl(var(--sf) / 1)}
.border-l-secondary-focus\/0 {
    border-left-color: hsl(var(--sf) / 0)}
.border-l-secondary-focus\/10 {
    border-left-color: hsl(var(--sf) / 0.1)}
.border-l-secondary-focus\/100 {
    border-left-color: hsl(var(--sf) / 1)}
.border-l-secondary-focus\/20 {
    border-left-color: hsl(var(--sf) / 0.2)}
.border-l-secondary-focus\/25 {
    border-left-color: hsl(var(--sf) / 0.25)}
.border-l-secondary-focus\/30 {
    border-left-color: hsl(var(--sf) / 0.3)}
.border-l-secondary-focus\/40 {
    border-left-color: hsl(var(--sf) / 0.4)}
.border-l-secondary-focus\/5 {
    border-left-color: hsl(var(--sf) / 0.05)}
.border-l-secondary-focus\/50 {
    border-left-color: hsl(var(--sf) / 0.5)}
.border-l-secondary-focus\/60 {
    border-left-color: hsl(var(--sf) / 0.6)}
.border-l-secondary-focus\/70 {
    border-left-color: hsl(var(--sf) / 0.7)}
.border-l-secondary-focus\/75 {
    border-left-color: hsl(var(--sf) / 0.75)}
.border-l-secondary-focus\/80 {
    border-left-color: hsl(var(--sf) / 0.8)}
.border-l-secondary-focus\/90 {
    border-left-color: hsl(var(--sf) / 0.9)}
.border-l-secondary-focus\/95 {
    border-left-color: hsl(var(--sf) / 0.95)}
.border-l-secondary\/0 {
    border-left-color: hsl(var(--s) / 0)}
.border-l-secondary\/10 {
    border-left-color: hsl(var(--s) / 0.1)}
.border-l-secondary\/100 {
    border-left-color: hsl(var(--s) / 1)}
.border-l-secondary\/20 {
    border-left-color: hsl(var(--s) / 0.2)}
.border-l-secondary\/25 {
    border-left-color: hsl(var(--s) / 0.25)}
.border-l-secondary\/30 {
    border-left-color: hsl(var(--s) / 0.3)}
.border-l-secondary\/40 {
    border-left-color: hsl(var(--s) / 0.4)}
.border-l-secondary\/5 {
    border-left-color: hsl(var(--s) / 0.05)}
.border-l-secondary\/50 {
    border-left-color: hsl(var(--s) / 0.5)}
.border-l-secondary\/60 {
    border-left-color: hsl(var(--s) / 0.6)}
.border-l-secondary\/70 {
    border-left-color: hsl(var(--s) / 0.7)}
.border-l-secondary\/75 {
    border-left-color: hsl(var(--s) / 0.75)}
.border-l-secondary\/80 {
    border-left-color: hsl(var(--s) / 0.8)}
.border-l-secondary\/90 {
    border-left-color: hsl(var(--s) / 0.9)}
.border-l-secondary\/95 {
    border-left-color: hsl(var(--s) / 0.95)}
.border-l-success {
    border-left-color: hsl(var(--su) / 1)}
.border-l-success-content {
    border-left-color: hsl(var(--suc) / 1)}
.border-l-success-content\/0 {
    border-left-color: hsl(var(--suc) / 0)}
.border-l-success-content\/10 {
    border-left-color: hsl(var(--suc) / 0.1)}
.border-l-success-content\/100 {
    border-left-color: hsl(var(--suc) / 1)}
.border-l-success-content\/20 {
    border-left-color: hsl(var(--suc) / 0.2)}
.border-l-success-content\/25 {
    border-left-color: hsl(var(--suc) / 0.25)}
.border-l-success-content\/30 {
    border-left-color: hsl(var(--suc) / 0.3)}
.border-l-success-content\/40 {
    border-left-color: hsl(var(--suc) / 0.4)}
.border-l-success-content\/5 {
    border-left-color: hsl(var(--suc) / 0.05)}
.border-l-success-content\/50 {
    border-left-color: hsl(var(--suc) / 0.5)}
.border-l-success-content\/60 {
    border-left-color: hsl(var(--suc) / 0.6)}
.border-l-success-content\/70 {
    border-left-color: hsl(var(--suc) / 0.7)}
.border-l-success-content\/75 {
    border-left-color: hsl(var(--suc) / 0.75)}
.border-l-success-content\/80 {
    border-left-color: hsl(var(--suc) / 0.8)}
.border-l-success-content\/90 {
    border-left-color: hsl(var(--suc) / 0.9)}
.border-l-success-content\/95 {
    border-left-color: hsl(var(--suc) / 0.95)}
.border-l-success\/0 {
    border-left-color: hsl(var(--su) / 0)}
.border-l-success\/10 {
    border-left-color: hsl(var(--su) / 0.1)}
.border-l-success\/100 {
    border-left-color: hsl(var(--su) / 1)}
.border-l-success\/20 {
    border-left-color: hsl(var(--su) / 0.2)}
.border-l-success\/25 {
    border-left-color: hsl(var(--su) / 0.25)}
.border-l-success\/30 {
    border-left-color: hsl(var(--su) / 0.3)}
.border-l-success\/40 {
    border-left-color: hsl(var(--su) / 0.4)}
.border-l-success\/5 {
    border-left-color: hsl(var(--su) / 0.05)}
.border-l-success\/50 {
    border-left-color: hsl(var(--su) / 0.5)}
.border-l-success\/60 {
    border-left-color: hsl(var(--su) / 0.6)}
.border-l-success\/70 {
    border-left-color: hsl(var(--su) / 0.7)}
.border-l-success\/75 {
    border-left-color: hsl(var(--su) / 0.75)}
.border-l-success\/80 {
    border-left-color: hsl(var(--su) / 0.8)}
.border-l-success\/90 {
    border-left-color: hsl(var(--su) / 0.9)}
.border-l-success\/95 {
    border-left-color: hsl(var(--su) / 0.95)}
.border-l-transparent {
    border-left-color: transparent}
.border-l-transparent\/0 {
    border-left-color: rgb(0 0 0 / 0)}
.border-l-transparent\/10 {
    border-left-color: rgb(0 0 0 / 0.1)}
.border-l-transparent\/100 {
    border-left-color: rgb(0 0 0 / 1)}
.border-l-transparent\/20 {
    border-left-color: rgb(0 0 0 / 0.2)}
.border-l-transparent\/25 {
    border-left-color: rgb(0 0 0 / 0.25)}
.border-l-transparent\/30 {
    border-left-color: rgb(0 0 0 / 0.3)}
.border-l-transparent\/40 {
    border-left-color: rgb(0 0 0 / 0.4)}
.border-l-transparent\/5 {
    border-left-color: rgb(0 0 0 / 0.05)}
.border-l-transparent\/50 {
    border-left-color: rgb(0 0 0 / 0.5)}
.border-l-transparent\/60 {
    border-left-color: rgb(0 0 0 / 0.6)}
.border-l-transparent\/70 {
    border-left-color: rgb(0 0 0 / 0.7)}
.border-l-transparent\/75 {
    border-left-color: rgb(0 0 0 / 0.75)}
.border-l-transparent\/80 {
    border-left-color: rgb(0 0 0 / 0.8)}
.border-l-transparent\/90 {
    border-left-color: rgb(0 0 0 / 0.9)}
.border-l-transparent\/95 {
    border-left-color: rgb(0 0 0 / 0.95)}
.border-l-warning {
    border-left-color: hsl(var(--wa) / 1)}
.border-l-warning-content {
    border-left-color: hsl(var(--wac) / 1)}
.border-l-warning-content\/0 {
    border-left-color: hsl(var(--wac) / 0)}
.border-l-warning-content\/10 {
    border-left-color: hsl(var(--wac) / 0.1)}
.border-l-warning-content\/100 {
    border-left-color: hsl(var(--wac) / 1)}
.border-l-warning-content\/20 {
    border-left-color: hsl(var(--wac) / 0.2)}
.border-l-warning-content\/25 {
    border-left-color: hsl(var(--wac) / 0.25)}
.border-l-warning-content\/30 {
    border-left-color: hsl(var(--wac) / 0.3)}
.border-l-warning-content\/40 {
    border-left-color: hsl(var(--wac) / 0.4)}
.border-l-warning-content\/5 {
    border-left-color: hsl(var(--wac) / 0.05)}
.border-l-warning-content\/50 {
    border-left-color: hsl(var(--wac) / 0.5)}
.border-l-warning-content\/60 {
    border-left-color: hsl(var(--wac) / 0.6)}
.border-l-warning-content\/70 {
    border-left-color: hsl(var(--wac) / 0.7)}
.border-l-warning-content\/75 {
    border-left-color: hsl(var(--wac) / 0.75)}
.border-l-warning-content\/80 {
    border-left-color: hsl(var(--wac) / 0.8)}
.border-l-warning-content\/90 {
    border-left-color: hsl(var(--wac) / 0.9)}
.border-l-warning-content\/95 {
    border-left-color: hsl(var(--wac) / 0.95)}
.border-l-warning\/0 {
    border-left-color: hsl(var(--wa) / 0)}
.border-l-warning\/10 {
    border-left-color: hsl(var(--wa) / 0.1)}
.border-l-warning\/100 {
    border-left-color: hsl(var(--wa) / 1)}
.border-l-warning\/20 {
    border-left-color: hsl(var(--wa) / 0.2)}
.border-l-warning\/25 {
    border-left-color: hsl(var(--wa) / 0.25)}
.border-l-warning\/30 {
    border-left-color: hsl(var(--wa) / 0.3)}
.border-l-warning\/40 {
    border-left-color: hsl(var(--wa) / 0.4)}
.border-l-warning\/5 {
    border-left-color: hsl(var(--wa) / 0.05)}
.border-l-warning\/50 {
    border-left-color: hsl(var(--wa) / 0.5)}
.border-l-warning\/60 {
    border-left-color: hsl(var(--wa) / 0.6)}
.border-l-warning\/70 {
    border-left-color: hsl(var(--wa) / 0.7)}
.border-l-warning\/75 {
    border-left-color: hsl(var(--wa) / 0.75)}
.border-l-warning\/80 {
    border-left-color: hsl(var(--wa) / 0.8)}
.border-l-warning\/90 {
    border-left-color: hsl(var(--wa) / 0.9)}
.border-l-warning\/95 {
    border-left-color: hsl(var(--wa) / 0.95)}
.border-r-accent {
    border-right-color: hsl(var(--a) / 1)}
.border-r-accent-content {
    border-right-color: hsl(var(--ac) / 1)}
.border-r-accent-content\/0 {
    border-right-color: hsl(var(--ac) / 0)}
.border-r-accent-content\/10 {
    border-right-color: hsl(var(--ac) / 0.1)}
.border-r-accent-content\/100 {
    border-right-color: hsl(var(--ac) / 1)}
.border-r-accent-content\/20 {
    border-right-color: hsl(var(--ac) / 0.2)}
.border-r-accent-content\/25 {
    border-right-color: hsl(var(--ac) / 0.25)}
.border-r-accent-content\/30 {
    border-right-color: hsl(var(--ac) / 0.3)}
.border-r-accent-content\/40 {
    border-right-color: hsl(var(--ac) / 0.4)}
.border-r-accent-content\/5 {
    border-right-color: hsl(var(--ac) / 0.05)}
.border-r-accent-content\/50 {
    border-right-color: hsl(var(--ac) / 0.5)}
.border-r-accent-content\/60 {
    border-right-color: hsl(var(--ac) / 0.6)}
.border-r-accent-content\/70 {
    border-right-color: hsl(var(--ac) / 0.7)}
.border-r-accent-content\/75 {
    border-right-color: hsl(var(--ac) / 0.75)}
.border-r-accent-content\/80 {
    border-right-color: hsl(var(--ac) / 0.8)}
.border-r-accent-content\/90 {
    border-right-color: hsl(var(--ac) / 0.9)}
.border-r-accent-content\/95 {
    border-right-color: hsl(var(--ac) / 0.95)}
.border-r-accent-focus {
    border-right-color: hsl(var(--af) / 1)}
.border-r-accent-focus\/0 {
    border-right-color: hsl(var(--af) / 0)}
.border-r-accent-focus\/10 {
    border-right-color: hsl(var(--af) / 0.1)}
.border-r-accent-focus\/100 {
    border-right-color: hsl(var(--af) / 1)}
.border-r-accent-focus\/20 {
    border-right-color: hsl(var(--af) / 0.2)}
.border-r-accent-focus\/25 {
    border-right-color: hsl(var(--af) / 0.25)}
.border-r-accent-focus\/30 {
    border-right-color: hsl(var(--af) / 0.3)}
.border-r-accent-focus\/40 {
    border-right-color: hsl(var(--af) / 0.4)}
.border-r-accent-focus\/5 {
    border-right-color: hsl(var(--af) / 0.05)}
.border-r-accent-focus\/50 {
    border-right-color: hsl(var(--af) / 0.5)}
.border-r-accent-focus\/60 {
    border-right-color: hsl(var(--af) / 0.6)}
.border-r-accent-focus\/70 {
    border-right-color: hsl(var(--af) / 0.7)}
.border-r-accent-focus\/75 {
    border-right-color: hsl(var(--af) / 0.75)}
.border-r-accent-focus\/80 {
    border-right-color: hsl(var(--af) / 0.8)}
.border-r-accent-focus\/90 {
    border-right-color: hsl(var(--af) / 0.9)}
.border-r-accent-focus\/95 {
    border-right-color: hsl(var(--af) / 0.95)}
.border-r-accent\/0 {
    border-right-color: hsl(var(--a) / 0)}
.border-r-accent\/10 {
    border-right-color: hsl(var(--a) / 0.1)}
.border-r-accent\/100 {
    border-right-color: hsl(var(--a) / 1)}
.border-r-accent\/20 {
    border-right-color: hsl(var(--a) / 0.2)}
.border-r-accent\/25 {
    border-right-color: hsl(var(--a) / 0.25)}
.border-r-accent\/30 {
    border-right-color: hsl(var(--a) / 0.3)}
.border-r-accent\/40 {
    border-right-color: hsl(var(--a) / 0.4)}
.border-r-accent\/5 {
    border-right-color: hsl(var(--a) / 0.05)}
.border-r-accent\/50 {
    border-right-color: hsl(var(--a) / 0.5)}
.border-r-accent\/60 {
    border-right-color: hsl(var(--a) / 0.6)}
.border-r-accent\/70 {
    border-right-color: hsl(var(--a) / 0.7)}
.border-r-accent\/75 {
    border-right-color: hsl(var(--a) / 0.75)}
.border-r-accent\/80 {
    border-right-color: hsl(var(--a) / 0.8)}
.border-r-accent\/90 {
    border-right-color: hsl(var(--a) / 0.9)}
.border-r-accent\/95 {
    border-right-color: hsl(var(--a) / 0.95)}
.border-r-base-100 {
    border-right-color: hsl(var(--b1) / 1)}
.border-r-base-100\/0 {
    border-right-color: hsl(var(--b1) / 0)}
.border-r-base-100\/10 {
    border-right-color: hsl(var(--b1) / 0.1)}
.border-r-base-100\/100 {
    border-right-color: hsl(var(--b1) / 1)}
.border-r-base-100\/20 {
    border-right-color: hsl(var(--b1) / 0.2)}
.border-r-base-100\/25 {
    border-right-color: hsl(var(--b1) / 0.25)}
.border-r-base-100\/30 {
    border-right-color: hsl(var(--b1) / 0.3)}
.border-r-base-100\/40 {
    border-right-color: hsl(var(--b1) / 0.4)}
.border-r-base-100\/5 {
    border-right-color: hsl(var(--b1) / 0.05)}
.border-r-base-100\/50 {
    border-right-color: hsl(var(--b1) / 0.5)}
.border-r-base-100\/60 {
    border-right-color: hsl(var(--b1) / 0.6)}
.border-r-base-100\/70 {
    border-right-color: hsl(var(--b1) / 0.7)}
.border-r-base-100\/75 {
    border-right-color: hsl(var(--b1) / 0.75)}
.border-r-base-100\/80 {
    border-right-color: hsl(var(--b1) / 0.8)}
.border-r-base-100\/90 {
    border-right-color: hsl(var(--b1) / 0.9)}
.border-r-base-100\/95 {
    border-right-color: hsl(var(--b1) / 0.95)}
.border-r-base-200 {
    border-right-color: hsl(var(--b2) / 1)}
.border-r-base-200\/0 {
    border-right-color: hsl(var(--b2) / 0)}
.border-r-base-200\/10 {
    border-right-color: hsl(var(--b2) / 0.1)}
.border-r-base-200\/100 {
    border-right-color: hsl(var(--b2) / 1)}
.border-r-base-200\/20 {
    border-right-color: hsl(var(--b2) / 0.2)}
.border-r-base-200\/25 {
    border-right-color: hsl(var(--b2) / 0.25)}
.border-r-base-200\/30 {
    border-right-color: hsl(var(--b2) / 0.3)}
.border-r-base-200\/40 {
    border-right-color: hsl(var(--b2) / 0.4)}
.border-r-base-200\/5 {
    border-right-color: hsl(var(--b2) / 0.05)}
.border-r-base-200\/50 {
    border-right-color: hsl(var(--b2) / 0.5)}
.border-r-base-200\/60 {
    border-right-color: hsl(var(--b2) / 0.6)}
.border-r-base-200\/70 {
    border-right-color: hsl(var(--b2) / 0.7)}
.border-r-base-200\/75 {
    border-right-color: hsl(var(--b2) / 0.75)}
.border-r-base-200\/80 {
    border-right-color: hsl(var(--b2) / 0.8)}
.border-r-base-200\/90 {
    border-right-color: hsl(var(--b2) / 0.9)}
.border-r-base-200\/95 {
    border-right-color: hsl(var(--b2) / 0.95)}
.border-r-base-300 {
    border-right-color: hsl(var(--b3) / 1)}
.border-r-base-300\/0 {
    border-right-color: hsl(var(--b3) / 0)}
.border-r-base-300\/10 {
    border-right-color: hsl(var(--b3) / 0.1)}
.border-r-base-300\/100 {
    border-right-color: hsl(var(--b3) / 1)}
.border-r-base-300\/20 {
    border-right-color: hsl(var(--b3) / 0.2)}
.border-r-base-300\/25 {
    border-right-color: hsl(var(--b3) / 0.25)}
.border-r-base-300\/30 {
    border-right-color: hsl(var(--b3) / 0.3)}
.border-r-base-300\/40 {
    border-right-color: hsl(var(--b3) / 0.4)}
.border-r-base-300\/5 {
    border-right-color: hsl(var(--b3) / 0.05)}
.border-r-base-300\/50 {
    border-right-color: hsl(var(--b3) / 0.5)}
.border-r-base-300\/60 {
    border-right-color: hsl(var(--b3) / 0.6)}
.border-r-base-300\/70 {
    border-right-color: hsl(var(--b3) / 0.7)}
.border-r-base-300\/75 {
    border-right-color: hsl(var(--b3) / 0.75)}
.border-r-base-300\/80 {
    border-right-color: hsl(var(--b3) / 0.8)}
.border-r-base-300\/90 {
    border-right-color: hsl(var(--b3) / 0.9)}
.border-r-base-300\/95 {
    border-right-color: hsl(var(--b3) / 0.95)}
.border-r-base-content {
    border-right-color: hsl(var(--bc) / 1)}
.border-r-base-content\/0 {
    border-right-color: hsl(var(--bc) / 0)}
.border-r-base-content\/10 {
    border-right-color: hsl(var(--bc) / 0.1)}
.border-r-base-content\/100 {
    border-right-color: hsl(var(--bc) / 1)}
.border-r-base-content\/20 {
    border-right-color: hsl(var(--bc) / 0.2)}
.border-r-base-content\/25 {
    border-right-color: hsl(var(--bc) / 0.25)}
.border-r-base-content\/30 {
    border-right-color: hsl(var(--bc) / 0.3)}
.border-r-base-content\/40 {
    border-right-color: hsl(var(--bc) / 0.4)}
.border-r-base-content\/5 {
    border-right-color: hsl(var(--bc) / 0.05)}
.border-r-base-content\/50 {
    border-right-color: hsl(var(--bc) / 0.5)}
.border-r-base-content\/60 {
    border-right-color: hsl(var(--bc) / 0.6)}
.border-r-base-content\/70 {
    border-right-color: hsl(var(--bc) / 0.7)}
.border-r-base-content\/75 {
    border-right-color: hsl(var(--bc) / 0.75)}
.border-r-base-content\/80 {
    border-right-color: hsl(var(--bc) / 0.8)}
.border-r-base-content\/90 {
    border-right-color: hsl(var(--bc) / 0.9)}
.border-r-base-content\/95 {
    border-right-color: hsl(var(--bc) / 0.95)}
.border-r-current {
    border-right-color: currentColor}
.border-r-error {
    border-right-color: hsl(var(--er) / 1)}
.border-r-error-content {
    border-right-color: hsl(var(--erc) / 1)}
.border-r-error-content\/0 {
    border-right-color: hsl(var(--erc) / 0)}
.border-r-error-content\/10 {
    border-right-color: hsl(var(--erc) / 0.1)}
.border-r-error-content\/100 {
    border-right-color: hsl(var(--erc) / 1)}
.border-r-error-content\/20 {
    border-right-color: hsl(var(--erc) / 0.2)}
.border-r-error-content\/25 {
    border-right-color: hsl(var(--erc) / 0.25)}
.border-r-error-content\/30 {
    border-right-color: hsl(var(--erc) / 0.3)}
.border-r-error-content\/40 {
    border-right-color: hsl(var(--erc) / 0.4)}
.border-r-error-content\/5 {
    border-right-color: hsl(var(--erc) / 0.05)}
.border-r-error-content\/50 {
    border-right-color: hsl(var(--erc) / 0.5)}
.border-r-error-content\/60 {
    border-right-color: hsl(var(--erc) / 0.6)}
.border-r-error-content\/70 {
    border-right-color: hsl(var(--erc) / 0.7)}
.border-r-error-content\/75 {
    border-right-color: hsl(var(--erc) / 0.75)}
.border-r-error-content\/80 {
    border-right-color: hsl(var(--erc) / 0.8)}
.border-r-error-content\/90 {
    border-right-color: hsl(var(--erc) / 0.9)}
.border-r-error-content\/95 {
    border-right-color: hsl(var(--erc) / 0.95)}
.border-r-error\/0 {
    border-right-color: hsl(var(--er) / 0)}
.border-r-error\/10 {
    border-right-color: hsl(var(--er) / 0.1)}
.border-r-error\/100 {
    border-right-color: hsl(var(--er) / 1)}
.border-r-error\/20 {
    border-right-color: hsl(var(--er) / 0.2)}
.border-r-error\/25 {
    border-right-color: hsl(var(--er) / 0.25)}
.border-r-error\/30 {
    border-right-color: hsl(var(--er) / 0.3)}
.border-r-error\/40 {
    border-right-color: hsl(var(--er) / 0.4)}
.border-r-error\/5 {
    border-right-color: hsl(var(--er) / 0.05)}
.border-r-error\/50 {
    border-right-color: hsl(var(--er) / 0.5)}
.border-r-error\/60 {
    border-right-color: hsl(var(--er) / 0.6)}
.border-r-error\/70 {
    border-right-color: hsl(var(--er) / 0.7)}
.border-r-error\/75 {
    border-right-color: hsl(var(--er) / 0.75)}
.border-r-error\/80 {
    border-right-color: hsl(var(--er) / 0.8)}
.border-r-error\/90 {
    border-right-color: hsl(var(--er) / 0.9)}
.border-r-error\/95 {
    border-right-color: hsl(var(--er) / 0.95)}
.border-r-info {
    border-right-color: hsl(var(--in) / 1)}
.border-r-info-content {
    border-right-color: hsl(var(--inc) / 1)}
.border-r-info-content\/0 {
    border-right-color: hsl(var(--inc) / 0)}
.border-r-info-content\/10 {
    border-right-color: hsl(var(--inc) / 0.1)}
.border-r-info-content\/100 {
    border-right-color: hsl(var(--inc) / 1)}
.border-r-info-content\/20 {
    border-right-color: hsl(var(--inc) / 0.2)}
.border-r-info-content\/25 {
    border-right-color: hsl(var(--inc) / 0.25)}
.border-r-info-content\/30 {
    border-right-color: hsl(var(--inc) / 0.3)}
.border-r-info-content\/40 {
    border-right-color: hsl(var(--inc) / 0.4)}
.border-r-info-content\/5 {
    border-right-color: hsl(var(--inc) / 0.05)}
.border-r-info-content\/50 {
    border-right-color: hsl(var(--inc) / 0.5)}
.border-r-info-content\/60 {
    border-right-color: hsl(var(--inc) / 0.6)}
.border-r-info-content\/70 {
    border-right-color: hsl(var(--inc) / 0.7)}
.border-r-info-content\/75 {
    border-right-color: hsl(var(--inc) / 0.75)}
.border-r-info-content\/80 {
    border-right-color: hsl(var(--inc) / 0.8)}
.border-r-info-content\/90 {
    border-right-color: hsl(var(--inc) / 0.9)}
.border-r-info-content\/95 {
    border-right-color: hsl(var(--inc) / 0.95)}
.border-r-info\/0 {
    border-right-color: hsl(var(--in) / 0)}
.border-r-info\/10 {
    border-right-color: hsl(var(--in) / 0.1)}
.border-r-info\/100 {
    border-right-color: hsl(var(--in) / 1)}
.border-r-info\/20 {
    border-right-color: hsl(var(--in) / 0.2)}
.border-r-info\/25 {
    border-right-color: hsl(var(--in) / 0.25)}
.border-r-info\/30 {
    border-right-color: hsl(var(--in) / 0.3)}
.border-r-info\/40 {
    border-right-color: hsl(var(--in) / 0.4)}
.border-r-info\/5 {
    border-right-color: hsl(var(--in) / 0.05)}
.border-r-info\/50 {
    border-right-color: hsl(var(--in) / 0.5)}
.border-r-info\/60 {
    border-right-color: hsl(var(--in) / 0.6)}
.border-r-info\/70 {
    border-right-color: hsl(var(--in) / 0.7)}
.border-r-info\/75 {
    border-right-color: hsl(var(--in) / 0.75)}
.border-r-info\/80 {
    border-right-color: hsl(var(--in) / 0.8)}
.border-r-info\/90 {
    border-right-color: hsl(var(--in) / 0.9)}
.border-r-info\/95 {
    border-right-color: hsl(var(--in) / 0.95)}
.border-r-neutral {
    border-right-color: hsl(var(--n) / 1)}
.border-r-neutral-content {
    border-right-color: hsl(var(--nc) / 1)}
.border-r-neutral-content\/0 {
    border-right-color: hsl(var(--nc) / 0)}
.border-r-neutral-content\/10 {
    border-right-color: hsl(var(--nc) / 0.1)}
.border-r-neutral-content\/100 {
    border-right-color: hsl(var(--nc) / 1)}
.border-r-neutral-content\/20 {
    border-right-color: hsl(var(--nc) / 0.2)}
.border-r-neutral-content\/25 {
    border-right-color: hsl(var(--nc) / 0.25)}
.border-r-neutral-content\/30 {
    border-right-color: hsl(var(--nc) / 0.3)}
.border-r-neutral-content\/40 {
    border-right-color: hsl(var(--nc) / 0.4)}
.border-r-neutral-content\/5 {
    border-right-color: hsl(var(--nc) / 0.05)}
.border-r-neutral-content\/50 {
    border-right-color: hsl(var(--nc) / 0.5)}
.border-r-neutral-content\/60 {
    border-right-color: hsl(var(--nc) / 0.6)}
.border-r-neutral-content\/70 {
    border-right-color: hsl(var(--nc) / 0.7)}
.border-r-neutral-content\/75 {
    border-right-color: hsl(var(--nc) / 0.75)}
.border-r-neutral-content\/80 {
    border-right-color: hsl(var(--nc) / 0.8)}
.border-r-neutral-content\/90 {
    border-right-color: hsl(var(--nc) / 0.9)}
.border-r-neutral-content\/95 {
    border-right-color: hsl(var(--nc) / 0.95)}
.border-r-neutral-focus {
    border-right-color: hsl(var(--nf) / 1)}
.border-r-neutral-focus\/0 {
    border-right-color: hsl(var(--nf) / 0)}
.border-r-neutral-focus\/10 {
    border-right-color: hsl(var(--nf) / 0.1)}
.border-r-neutral-focus\/100 {
    border-right-color: hsl(var(--nf) / 1)}
.border-r-neutral-focus\/20 {
    border-right-color: hsl(var(--nf) / 0.2)}
.border-r-neutral-focus\/25 {
    border-right-color: hsl(var(--nf) / 0.25)}
.border-r-neutral-focus\/30 {
    border-right-color: hsl(var(--nf) / 0.3)}
.border-r-neutral-focus\/40 {
    border-right-color: hsl(var(--nf) / 0.4)}
.border-r-neutral-focus\/5 {
    border-right-color: hsl(var(--nf) / 0.05)}
.border-r-neutral-focus\/50 {
    border-right-color: hsl(var(--nf) / 0.5)}
.border-r-neutral-focus\/60 {
    border-right-color: hsl(var(--nf) / 0.6)}
.border-r-neutral-focus\/70 {
    border-right-color: hsl(var(--nf) / 0.7)}
.border-r-neutral-focus\/75 {
    border-right-color: hsl(var(--nf) / 0.75)}
.border-r-neutral-focus\/80 {
    border-right-color: hsl(var(--nf) / 0.8)}
.border-r-neutral-focus\/90 {
    border-right-color: hsl(var(--nf) / 0.9)}
.border-r-neutral-focus\/95 {
    border-right-color: hsl(var(--nf) / 0.95)}
.border-r-neutral\/0 {
    border-right-color: hsl(var(--n) / 0)}
.border-r-neutral\/10 {
    border-right-color: hsl(var(--n) / 0.1)}
.border-r-neutral\/100 {
    border-right-color: hsl(var(--n) / 1)}
.border-r-neutral\/20 {
    border-right-color: hsl(var(--n) / 0.2)}
.border-r-neutral\/25 {
    border-right-color: hsl(var(--n) / 0.25)}
.border-r-neutral\/30 {
    border-right-color: hsl(var(--n) / 0.3)}
.border-r-neutral\/40 {
    border-right-color: hsl(var(--n) / 0.4)}
.border-r-neutral\/5 {
    border-right-color: hsl(var(--n) / 0.05)}
.border-r-neutral\/50 {
    border-right-color: hsl(var(--n) / 0.5)}
.border-r-neutral\/60 {
    border-right-color: hsl(var(--n) / 0.6)}
.border-r-neutral\/70 {
    border-right-color: hsl(var(--n) / 0.7)}
.border-r-neutral\/75 {
    border-right-color: hsl(var(--n) / 0.75)}
.border-r-neutral\/80 {
    border-right-color: hsl(var(--n) / 0.8)}
.border-r-neutral\/90 {
    border-right-color: hsl(var(--n) / 0.9)}
.border-r-neutral\/95 {
    border-right-color: hsl(var(--n) / 0.95)}
.border-r-primary {
    border-right-color: hsl(var(--p) / 1)}
.border-r-primary-content {
    border-right-color: hsl(var(--pc) / 1)}
.border-r-primary-content\/0 {
    border-right-color: hsl(var(--pc) / 0)}
.border-r-primary-content\/10 {
    border-right-color: hsl(var(--pc) / 0.1)}
.border-r-primary-content\/100 {
    border-right-color: hsl(var(--pc) / 1)}
.border-r-primary-content\/20 {
    border-right-color: hsl(var(--pc) / 0.2)}
.border-r-primary-content\/25 {
    border-right-color: hsl(var(--pc) / 0.25)}
.border-r-primary-content\/30 {
    border-right-color: hsl(var(--pc) / 0.3)}
.border-r-primary-content\/40 {
    border-right-color: hsl(var(--pc) / 0.4)}
.border-r-primary-content\/5 {
    border-right-color: hsl(var(--pc) / 0.05)}
.border-r-primary-content\/50 {
    border-right-color: hsl(var(--pc) / 0.5)}
.border-r-primary-content\/60 {
    border-right-color: hsl(var(--pc) / 0.6)}
.border-r-primary-content\/70 {
    border-right-color: hsl(var(--pc) / 0.7)}
.border-r-primary-content\/75 {
    border-right-color: hsl(var(--pc) / 0.75)}
.border-r-primary-content\/80 {
    border-right-color: hsl(var(--pc) / 0.8)}
.border-r-primary-content\/90 {
    border-right-color: hsl(var(--pc) / 0.9)}
.border-r-primary-content\/95 {
    border-right-color: hsl(var(--pc) / 0.95)}
.border-r-primary-focus {
    border-right-color: hsl(var(--pf) / 1)}
.border-r-primary-focus\/0 {
    border-right-color: hsl(var(--pf) / 0)}
.border-r-primary-focus\/10 {
    border-right-color: hsl(var(--pf) / 0.1)}
.border-r-primary-focus\/100 {
    border-right-color: hsl(var(--pf) / 1)}
.border-r-primary-focus\/20 {
    border-right-color: hsl(var(--pf) / 0.2)}
.border-r-primary-focus\/25 {
    border-right-color: hsl(var(--pf) / 0.25)}
.border-r-primary-focus\/30 {
    border-right-color: hsl(var(--pf) / 0.3)}
.border-r-primary-focus\/40 {
    border-right-color: hsl(var(--pf) / 0.4)}
.border-r-primary-focus\/5 {
    border-right-color: hsl(var(--pf) / 0.05)}
.border-r-primary-focus\/50 {
    border-right-color: hsl(var(--pf) / 0.5)}
.border-r-primary-focus\/60 {
    border-right-color: hsl(var(--pf) / 0.6)}
.border-r-primary-focus\/70 {
    border-right-color: hsl(var(--pf) / 0.7)}
.border-r-primary-focus\/75 {
    border-right-color: hsl(var(--pf) / 0.75)}
.border-r-primary-focus\/80 {
    border-right-color: hsl(var(--pf) / 0.8)}
.border-r-primary-focus\/90 {
    border-right-color: hsl(var(--pf) / 0.9)}
.border-r-primary-focus\/95 {
    border-right-color: hsl(var(--pf) / 0.95)}
.border-r-primary\/0 {
    border-right-color: hsl(var(--p) / 0)}
.border-r-primary\/10 {
    border-right-color: hsl(var(--p) / 0.1)}
.border-r-primary\/100 {
    border-right-color: hsl(var(--p) / 1)}
.border-r-primary\/20 {
    border-right-color: hsl(var(--p) / 0.2)}
.border-r-primary\/25 {
    border-right-color: hsl(var(--p) / 0.25)}
.border-r-primary\/30 {
    border-right-color: hsl(var(--p) / 0.3)}
.border-r-primary\/40 {
    border-right-color: hsl(var(--p) / 0.4)}
.border-r-primary\/5 {
    border-right-color: hsl(var(--p) / 0.05)}
.border-r-primary\/50 {
    border-right-color: hsl(var(--p) / 0.5)}
.border-r-primary\/60 {
    border-right-color: hsl(var(--p) / 0.6)}
.border-r-primary\/70 {
    border-right-color: hsl(var(--p) / 0.7)}
.border-r-primary\/75 {
    border-right-color: hsl(var(--p) / 0.75)}
.border-r-primary\/80 {
    border-right-color: hsl(var(--p) / 0.8)}
.border-r-primary\/90 {
    border-right-color: hsl(var(--p) / 0.9)}
.border-r-primary\/95 {
    border-right-color: hsl(var(--p) / 0.95)}
.border-r-secondary {
    border-right-color: hsl(var(--s) / 1)}
.border-r-secondary-content {
    border-right-color: hsl(var(--sc) / 1)}
.border-r-secondary-content\/0 {
    border-right-color: hsl(var(--sc) / 0)}
.border-r-secondary-content\/10 {
    border-right-color: hsl(var(--sc) / 0.1)}
.border-r-secondary-content\/100 {
    border-right-color: hsl(var(--sc) / 1)}
.border-r-secondary-content\/20 {
    border-right-color: hsl(var(--sc) / 0.2)}
.border-r-secondary-content\/25 {
    border-right-color: hsl(var(--sc) / 0.25)}
.border-r-secondary-content\/30 {
    border-right-color: hsl(var(--sc) / 0.3)}
.border-r-secondary-content\/40 {
    border-right-color: hsl(var(--sc) / 0.4)}
.border-r-secondary-content\/5 {
    border-right-color: hsl(var(--sc) / 0.05)}
.border-r-secondary-content\/50 {
    border-right-color: hsl(var(--sc) / 0.5)}
.border-r-secondary-content\/60 {
    border-right-color: hsl(var(--sc) / 0.6)}
.border-r-secondary-content\/70 {
    border-right-color: hsl(var(--sc) / 0.7)}
.border-r-secondary-content\/75 {
    border-right-color: hsl(var(--sc) / 0.75)}
.border-r-secondary-content\/80 {
    border-right-color: hsl(var(--sc) / 0.8)}
.border-r-secondary-content\/90 {
    border-right-color: hsl(var(--sc) / 0.9)}
.border-r-secondary-content\/95 {
    border-right-color: hsl(var(--sc) / 0.95)}
.border-r-secondary-focus {
    border-right-color: hsl(var(--sf) / 1)}
.border-r-secondary-focus\/0 {
    border-right-color: hsl(var(--sf) / 0)}
.border-r-secondary-focus\/10 {
    border-right-color: hsl(var(--sf) / 0.1)}
.border-r-secondary-focus\/100 {
    border-right-color: hsl(var(--sf) / 1)}
.border-r-secondary-focus\/20 {
    border-right-color: hsl(var(--sf) / 0.2)}
.border-r-secondary-focus\/25 {
    border-right-color: hsl(var(--sf) / 0.25)}
.border-r-secondary-focus\/30 {
    border-right-color: hsl(var(--sf) / 0.3)}
.border-r-secondary-focus\/40 {
    border-right-color: hsl(var(--sf) / 0.4)}
.border-r-secondary-focus\/5 {
    border-right-color: hsl(var(--sf) / 0.05)}
.border-r-secondary-focus\/50 {
    border-right-color: hsl(var(--sf) / 0.5)}
.border-r-secondary-focus\/60 {
    border-right-color: hsl(var(--sf) / 0.6)}
.border-r-secondary-focus\/70 {
    border-right-color: hsl(var(--sf) / 0.7)}
.border-r-secondary-focus\/75 {
    border-right-color: hsl(var(--sf) / 0.75)}
.border-r-secondary-focus\/80 {
    border-right-color: hsl(var(--sf) / 0.8)}
.border-r-secondary-focus\/90 {
    border-right-color: hsl(var(--sf) / 0.9)}
.border-r-secondary-focus\/95 {
    border-right-color: hsl(var(--sf) / 0.95)}
.border-r-secondary\/0 {
    border-right-color: hsl(var(--s) / 0)}
.border-r-secondary\/10 {
    border-right-color: hsl(var(--s) / 0.1)}
.border-r-secondary\/100 {
    border-right-color: hsl(var(--s) / 1)}
.border-r-secondary\/20 {
    border-right-color: hsl(var(--s) / 0.2)}
.border-r-secondary\/25 {
    border-right-color: hsl(var(--s) / 0.25)}
.border-r-secondary\/30 {
    border-right-color: hsl(var(--s) / 0.3)}
.border-r-secondary\/40 {
    border-right-color: hsl(var(--s) / 0.4)}
.border-r-secondary\/5 {
    border-right-color: hsl(var(--s) / 0.05)}
.border-r-secondary\/50 {
    border-right-color: hsl(var(--s) / 0.5)}
.border-r-secondary\/60 {
    border-right-color: hsl(var(--s) / 0.6)}
.border-r-secondary\/70 {
    border-right-color: hsl(var(--s) / 0.7)}
.border-r-secondary\/75 {
    border-right-color: hsl(var(--s) / 0.75)}
.border-r-secondary\/80 {
    border-right-color: hsl(var(--s) / 0.8)}
.border-r-secondary\/90 {
    border-right-color: hsl(var(--s) / 0.9)}
.border-r-secondary\/95 {
    border-right-color: hsl(var(--s) / 0.95)}
.border-r-success {
    border-right-color: hsl(var(--su) / 1)}
.border-r-success-content {
    border-right-color: hsl(var(--suc) / 1)}
.border-r-success-content\/0 {
    border-right-color: hsl(var(--suc) / 0)}
.border-r-success-content\/10 {
    border-right-color: hsl(var(--suc) / 0.1)}
.border-r-success-content\/100 {
    border-right-color: hsl(var(--suc) / 1)}
.border-r-success-content\/20 {
    border-right-color: hsl(var(--suc) / 0.2)}
.border-r-success-content\/25 {
    border-right-color: hsl(var(--suc) / 0.25)}
.border-r-success-content\/30 {
    border-right-color: hsl(var(--suc) / 0.3)}
.border-r-success-content\/40 {
    border-right-color: hsl(var(--suc) / 0.4)}
.border-r-success-content\/5 {
    border-right-color: hsl(var(--suc) / 0.05)}
.border-r-success-content\/50 {
    border-right-color: hsl(var(--suc) / 0.5)}
.border-r-success-content\/60 {
    border-right-color: hsl(var(--suc) / 0.6)}
.border-r-success-content\/70 {
    border-right-color: hsl(var(--suc) / 0.7)}
.border-r-success-content\/75 {
    border-right-color: hsl(var(--suc) / 0.75)}
.border-r-success-content\/80 {
    border-right-color: hsl(var(--suc) / 0.8)}
.border-r-success-content\/90 {
    border-right-color: hsl(var(--suc) / 0.9)}
.border-r-success-content\/95 {
    border-right-color: hsl(var(--suc) / 0.95)}
.border-r-success\/0 {
    border-right-color: hsl(var(--su) / 0)}
.border-r-success\/10 {
    border-right-color: hsl(var(--su) / 0.1)}
.border-r-success\/100 {
    border-right-color: hsl(var(--su) / 1)}
.border-r-success\/20 {
    border-right-color: hsl(var(--su) / 0.2)}
.border-r-success\/25 {
    border-right-color: hsl(var(--su) / 0.25)}
.border-r-success\/30 {
    border-right-color: hsl(var(--su) / 0.3)}
.border-r-success\/40 {
    border-right-color: hsl(var(--su) / 0.4)}
.border-r-success\/5 {
    border-right-color: hsl(var(--su) / 0.05)}
.border-r-success\/50 {
    border-right-color: hsl(var(--su) / 0.5)}
.border-r-success\/60 {
    border-right-color: hsl(var(--su) / 0.6)}
.border-r-success\/70 {
    border-right-color: hsl(var(--su) / 0.7)}
.border-r-success\/75 {
    border-right-color: hsl(var(--su) / 0.75)}
.border-r-success\/80 {
    border-right-color: hsl(var(--su) / 0.8)}
.border-r-success\/90 {
    border-right-color: hsl(var(--su) / 0.9)}
.border-r-success\/95 {
    border-right-color: hsl(var(--su) / 0.95)}
.border-r-transparent {
    border-right-color: transparent}
.border-r-transparent\/0 {
    border-right-color: rgb(0 0 0 / 0)}
.border-r-transparent\/10 {
    border-right-color: rgb(0 0 0 / 0.1)}
.border-r-transparent\/100 {
    border-right-color: rgb(0 0 0 / 1)}
.border-r-transparent\/20 {
    border-right-color: rgb(0 0 0 / 0.2)}
.border-r-transparent\/25 {
    border-right-color: rgb(0 0 0 / 0.25)}
.border-r-transparent\/30 {
    border-right-color: rgb(0 0 0 / 0.3)}
.border-r-transparent\/40 {
    border-right-color: rgb(0 0 0 / 0.4)}
.border-r-transparent\/5 {
    border-right-color: rgb(0 0 0 / 0.05)}
.border-r-transparent\/50 {
    border-right-color: rgb(0 0 0 / 0.5)}
.border-r-transparent\/60 {
    border-right-color: rgb(0 0 0 / 0.6)}
.border-r-transparent\/70 {
    border-right-color: rgb(0 0 0 / 0.7)}
.border-r-transparent\/75 {
    border-right-color: rgb(0 0 0 / 0.75)}
.border-r-transparent\/80 {
    border-right-color: rgb(0 0 0 / 0.8)}
.border-r-transparent\/90 {
    border-right-color: rgb(0 0 0 / 0.9)}
.border-r-transparent\/95 {
    border-right-color: rgb(0 0 0 / 0.95)}
.border-r-warning {
    border-right-color: hsl(var(--wa) / 1)}
.border-r-warning-content {
    border-right-color: hsl(var(--wac) / 1)}
.border-r-warning-content\/0 {
    border-right-color: hsl(var(--wac) / 0)}
.border-r-warning-content\/10 {
    border-right-color: hsl(var(--wac) / 0.1)}
.border-r-warning-content\/100 {
    border-right-color: hsl(var(--wac) / 1)}
.border-r-warning-content\/20 {
    border-right-color: hsl(var(--wac) / 0.2)}
.border-r-warning-content\/25 {
    border-right-color: hsl(var(--wac) / 0.25)}
.border-r-warning-content\/30 {
    border-right-color: hsl(var(--wac) / 0.3)}
.border-r-warning-content\/40 {
    border-right-color: hsl(var(--wac) / 0.4)}
.border-r-warning-content\/5 {
    border-right-color: hsl(var(--wac) / 0.05)}
.border-r-warning-content\/50 {
    border-right-color: hsl(var(--wac) / 0.5)}
.border-r-warning-content\/60 {
    border-right-color: hsl(var(--wac) / 0.6)}
.border-r-warning-content\/70 {
    border-right-color: hsl(var(--wac) / 0.7)}
.border-r-warning-content\/75 {
    border-right-color: hsl(var(--wac) / 0.75)}
.border-r-warning-content\/80 {
    border-right-color: hsl(var(--wac) / 0.8)}
.border-r-warning-content\/90 {
    border-right-color: hsl(var(--wac) / 0.9)}
.border-r-warning-content\/95 {
    border-right-color: hsl(var(--wac) / 0.95)}
.border-r-warning\/0 {
    border-right-color: hsl(var(--wa) / 0)}
.border-r-warning\/10 {
    border-right-color: hsl(var(--wa) / 0.1)}
.border-r-warning\/100 {
    border-right-color: hsl(var(--wa) / 1)}
.border-r-warning\/20 {
    border-right-color: hsl(var(--wa) / 0.2)}
.border-r-warning\/25 {
    border-right-color: hsl(var(--wa) / 0.25)}
.border-r-warning\/30 {
    border-right-color: hsl(var(--wa) / 0.3)}
.border-r-warning\/40 {
    border-right-color: hsl(var(--wa) / 0.4)}
.border-r-warning\/5 {
    border-right-color: hsl(var(--wa) / 0.05)}
.border-r-warning\/50 {
    border-right-color: hsl(var(--wa) / 0.5)}
.border-r-warning\/60 {
    border-right-color: hsl(var(--wa) / 0.6)}
.border-r-warning\/70 {
    border-right-color: hsl(var(--wa) / 0.7)}
.border-r-warning\/75 {
    border-right-color: hsl(var(--wa) / 0.75)}
.border-r-warning\/80 {
    border-right-color: hsl(var(--wa) / 0.8)}
.border-r-warning\/90 {
    border-right-color: hsl(var(--wa) / 0.9)}
.border-r-warning\/95 {
    border-right-color: hsl(var(--wa) / 0.95)}
.border-s-accent {
    border-inline-start-color: hsl(var(--a) / 1)}
.border-s-accent-content {
    border-inline-start-color: hsl(var(--ac) / 1)}
.border-s-accent-content\/0 {
    border-inline-start-color: hsl(var(--ac) / 0)}
.border-s-accent-content\/10 {
    border-inline-start-color: hsl(var(--ac) / 0.1)}
.border-s-accent-content\/100 {
    border-inline-start-color: hsl(var(--ac) / 1)}
.border-s-accent-content\/20 {
    border-inline-start-color: hsl(var(--ac) / 0.2)}
.border-s-accent-content\/25 {
    border-inline-start-color: hsl(var(--ac) / 0.25)}
.border-s-accent-content\/30 {
    border-inline-start-color: hsl(var(--ac) / 0.3)}
.border-s-accent-content\/40 {
    border-inline-start-color: hsl(var(--ac) / 0.4)}
.border-s-accent-content\/5 {
    border-inline-start-color: hsl(var(--ac) / 0.05)}
.border-s-accent-content\/50 {
    border-inline-start-color: hsl(var(--ac) / 0.5)}
.border-s-accent-content\/60 {
    border-inline-start-color: hsl(var(--ac) / 0.6)}
.border-s-accent-content\/70 {
    border-inline-start-color: hsl(var(--ac) / 0.7)}
.border-s-accent-content\/75 {
    border-inline-start-color: hsl(var(--ac) / 0.75)}
.border-s-accent-content\/80 {
    border-inline-start-color: hsl(var(--ac) / 0.8)}
.border-s-accent-content\/90 {
    border-inline-start-color: hsl(var(--ac) / 0.9)}
.border-s-accent-content\/95 {
    border-inline-start-color: hsl(var(--ac) / 0.95)}
.border-s-accent-focus {
    border-inline-start-color: hsl(var(--af) / 1)}
.border-s-accent-focus\/0 {
    border-inline-start-color: hsl(var(--af) / 0)}
.border-s-accent-focus\/10 {
    border-inline-start-color: hsl(var(--af) / 0.1)}
.border-s-accent-focus\/100 {
    border-inline-start-color: hsl(var(--af) / 1)}
.border-s-accent-focus\/20 {
    border-inline-start-color: hsl(var(--af) / 0.2)}
.border-s-accent-focus\/25 {
    border-inline-start-color: hsl(var(--af) / 0.25)}
.border-s-accent-focus\/30 {
    border-inline-start-color: hsl(var(--af) / 0.3)}
.border-s-accent-focus\/40 {
    border-inline-start-color: hsl(var(--af) / 0.4)}
.border-s-accent-focus\/5 {
    border-inline-start-color: hsl(var(--af) / 0.05)}
.border-s-accent-focus\/50 {
    border-inline-start-color: hsl(var(--af) / 0.5)}
.border-s-accent-focus\/60 {
    border-inline-start-color: hsl(var(--af) / 0.6)}
.border-s-accent-focus\/70 {
    border-inline-start-color: hsl(var(--af) / 0.7)}
.border-s-accent-focus\/75 {
    border-inline-start-color: hsl(var(--af) / 0.75)}
.border-s-accent-focus\/80 {
    border-inline-start-color: hsl(var(--af) / 0.8)}
.border-s-accent-focus\/90 {
    border-inline-start-color: hsl(var(--af) / 0.9)}
.border-s-accent-focus\/95 {
    border-inline-start-color: hsl(var(--af) / 0.95)}
.border-s-accent\/0 {
    border-inline-start-color: hsl(var(--a) / 0)}
.border-s-accent\/10 {
    border-inline-start-color: hsl(var(--a) / 0.1)}
.border-s-accent\/100 {
    border-inline-start-color: hsl(var(--a) / 1)}
.border-s-accent\/20 {
    border-inline-start-color: hsl(var(--a) / 0.2)}
.border-s-accent\/25 {
    border-inline-start-color: hsl(var(--a) / 0.25)}
.border-s-accent\/30 {
    border-inline-start-color: hsl(var(--a) / 0.3)}
.border-s-accent\/40 {
    border-inline-start-color: hsl(var(--a) / 0.4)}
.border-s-accent\/5 {
    border-inline-start-color: hsl(var(--a) / 0.05)}
.border-s-accent\/50 {
    border-inline-start-color: hsl(var(--a) / 0.5)}
.border-s-accent\/60 {
    border-inline-start-color: hsl(var(--a) / 0.6)}
.border-s-accent\/70 {
    border-inline-start-color: hsl(var(--a) / 0.7)}
.border-s-accent\/75 {
    border-inline-start-color: hsl(var(--a) / 0.75)}
.border-s-accent\/80 {
    border-inline-start-color: hsl(var(--a) / 0.8)}
.border-s-accent\/90 {
    border-inline-start-color: hsl(var(--a) / 0.9)}
.border-s-accent\/95 {
    border-inline-start-color: hsl(var(--a) / 0.95)}
.border-s-base-100 {
    border-inline-start-color: hsl(var(--b1) / 1)}
.border-s-base-100\/0 {
    border-inline-start-color: hsl(var(--b1) / 0)}
.border-s-base-100\/10 {
    border-inline-start-color: hsl(var(--b1) / 0.1)}
.border-s-base-100\/100 {
    border-inline-start-color: hsl(var(--b1) / 1)}
.border-s-base-100\/20 {
    border-inline-start-color: hsl(var(--b1) / 0.2)}
.border-s-base-100\/25 {
    border-inline-start-color: hsl(var(--b1) / 0.25)}
.border-s-base-100\/30 {
    border-inline-start-color: hsl(var(--b1) / 0.3)}
.border-s-base-100\/40 {
    border-inline-start-color: hsl(var(--b1) / 0.4)}
.border-s-base-100\/5 {
    border-inline-start-color: hsl(var(--b1) / 0.05)}
.border-s-base-100\/50 {
    border-inline-start-color: hsl(var(--b1) / 0.5)}
.border-s-base-100\/60 {
    border-inline-start-color: hsl(var(--b1) / 0.6)}
.border-s-base-100\/70 {
    border-inline-start-color: hsl(var(--b1) / 0.7)}
.border-s-base-100\/75 {
    border-inline-start-color: hsl(var(--b1) / 0.75)}
.border-s-base-100\/80 {
    border-inline-start-color: hsl(var(--b1) / 0.8)}
.border-s-base-100\/90 {
    border-inline-start-color: hsl(var(--b1) / 0.9)}
.border-s-base-100\/95 {
    border-inline-start-color: hsl(var(--b1) / 0.95)}
.border-s-base-200 {
    border-inline-start-color: hsl(var(--b2) / 1)}
.border-s-base-200\/0 {
    border-inline-start-color: hsl(var(--b2) / 0)}
.border-s-base-200\/10 {
    border-inline-start-color: hsl(var(--b2) / 0.1)}
.border-s-base-200\/100 {
    border-inline-start-color: hsl(var(--b2) / 1)}
.border-s-base-200\/20 {
    border-inline-start-color: hsl(var(--b2) / 0.2)}
.border-s-base-200\/25 {
    border-inline-start-color: hsl(var(--b2) / 0.25)}
.border-s-base-200\/30 {
    border-inline-start-color: hsl(var(--b2) / 0.3)}
.border-s-base-200\/40 {
    border-inline-start-color: hsl(var(--b2) / 0.4)}
.border-s-base-200\/5 {
    border-inline-start-color: hsl(var(--b2) / 0.05)}
.border-s-base-200\/50 {
    border-inline-start-color: hsl(var(--b2) / 0.5)}
.border-s-base-200\/60 {
    border-inline-start-color: hsl(var(--b2) / 0.6)}
.border-s-base-200\/70 {
    border-inline-start-color: hsl(var(--b2) / 0.7)}
.border-s-base-200\/75 {
    border-inline-start-color: hsl(var(--b2) / 0.75)}
.border-s-base-200\/80 {
    border-inline-start-color: hsl(var(--b2) / 0.8)}
.border-s-base-200\/90 {
    border-inline-start-color: hsl(var(--b2) / 0.9)}
.border-s-base-200\/95 {
    border-inline-start-color: hsl(var(--b2) / 0.95)}
.border-s-base-300 {
    border-inline-start-color: hsl(var(--b3) / 1)}
.border-s-base-300\/0 {
    border-inline-start-color: hsl(var(--b3) / 0)}
.border-s-base-300\/10 {
    border-inline-start-color: hsl(var(--b3) / 0.1)}
.border-s-base-300\/100 {
    border-inline-start-color: hsl(var(--b3) / 1)}
.border-s-base-300\/20 {
    border-inline-start-color: hsl(var(--b3) / 0.2)}
.border-s-base-300\/25 {
    border-inline-start-color: hsl(var(--b3) / 0.25)}
.border-s-base-300\/30 {
    border-inline-start-color: hsl(var(--b3) / 0.3)}
.border-s-base-300\/40 {
    border-inline-start-color: hsl(var(--b3) / 0.4)}
.border-s-base-300\/5 {
    border-inline-start-color: hsl(var(--b3) / 0.05)}
.border-s-base-300\/50 {
    border-inline-start-color: hsl(var(--b3) / 0.5)}
.border-s-base-300\/60 {
    border-inline-start-color: hsl(var(--b3) / 0.6)}
.border-s-base-300\/70 {
    border-inline-start-color: hsl(var(--b3) / 0.7)}
.border-s-base-300\/75 {
    border-inline-start-color: hsl(var(--b3) / 0.75)}
.border-s-base-300\/80 {
    border-inline-start-color: hsl(var(--b3) / 0.8)}
.border-s-base-300\/90 {
    border-inline-start-color: hsl(var(--b3) / 0.9)}
.border-s-base-300\/95 {
    border-inline-start-color: hsl(var(--b3) / 0.95)}
.border-s-base-content {
    border-inline-start-color: hsl(var(--bc) / 1)}
.border-s-base-content\/0 {
    border-inline-start-color: hsl(var(--bc) / 0)}
.border-s-base-content\/10 {
    border-inline-start-color: hsl(var(--bc) / 0.1)}
.border-s-base-content\/100 {
    border-inline-start-color: hsl(var(--bc) / 1)}
.border-s-base-content\/20 {
    border-inline-start-color: hsl(var(--bc) / 0.2)}
.border-s-base-content\/25 {
    border-inline-start-color: hsl(var(--bc) / 0.25)}
.border-s-base-content\/30 {
    border-inline-start-color: hsl(var(--bc) / 0.3)}
.border-s-base-content\/40 {
    border-inline-start-color: hsl(var(--bc) / 0.4)}
.border-s-base-content\/5 {
    border-inline-start-color: hsl(var(--bc) / 0.05)}
.border-s-base-content\/50 {
    border-inline-start-color: hsl(var(--bc) / 0.5)}
.border-s-base-content\/60 {
    border-inline-start-color: hsl(var(--bc) / 0.6)}
.border-s-base-content\/70 {
    border-inline-start-color: hsl(var(--bc) / 0.7)}
.border-s-base-content\/75 {
    border-inline-start-color: hsl(var(--bc) / 0.75)}
.border-s-base-content\/80 {
    border-inline-start-color: hsl(var(--bc) / 0.8)}
.border-s-base-content\/90 {
    border-inline-start-color: hsl(var(--bc) / 0.9)}
.border-s-base-content\/95 {
    border-inline-start-color: hsl(var(--bc) / 0.95)}
.border-s-current {
    border-inline-start-color: currentColor}
.border-s-error {
    border-inline-start-color: hsl(var(--er) / 1)}
.border-s-error-content {
    border-inline-start-color: hsl(var(--erc) / 1)}
.border-s-error-content\/0 {
    border-inline-start-color: hsl(var(--erc) / 0)}
.border-s-error-content\/10 {
    border-inline-start-color: hsl(var(--erc) / 0.1)}
.border-s-error-content\/100 {
    border-inline-start-color: hsl(var(--erc) / 1)}
.border-s-error-content\/20 {
    border-inline-start-color: hsl(var(--erc) / 0.2)}
.border-s-error-content\/25 {
    border-inline-start-color: hsl(var(--erc) / 0.25)}
.border-s-error-content\/30 {
    border-inline-start-color: hsl(var(--erc) / 0.3)}
.border-s-error-content\/40 {
    border-inline-start-color: hsl(var(--erc) / 0.4)}
.border-s-error-content\/5 {
    border-inline-start-color: hsl(var(--erc) / 0.05)}
.border-s-error-content\/50 {
    border-inline-start-color: hsl(var(--erc) / 0.5)}
.border-s-error-content\/60 {
    border-inline-start-color: hsl(var(--erc) / 0.6)}
.border-s-error-content\/70 {
    border-inline-start-color: hsl(var(--erc) / 0.7)}
.border-s-error-content\/75 {
    border-inline-start-color: hsl(var(--erc) / 0.75)}
.border-s-error-content\/80 {
    border-inline-start-color: hsl(var(--erc) / 0.8)}
.border-s-error-content\/90 {
    border-inline-start-color: hsl(var(--erc) / 0.9)}
.border-s-error-content\/95 {
    border-inline-start-color: hsl(var(--erc) / 0.95)}
.border-s-error\/0 {
    border-inline-start-color: hsl(var(--er) / 0)}
.border-s-error\/10 {
    border-inline-start-color: hsl(var(--er) / 0.1)}
.border-s-error\/100 {
    border-inline-start-color: hsl(var(--er) / 1)}
.border-s-error\/20 {
    border-inline-start-color: hsl(var(--er) / 0.2)}
.border-s-error\/25 {
    border-inline-start-color: hsl(var(--er) / 0.25)}
.border-s-error\/30 {
    border-inline-start-color: hsl(var(--er) / 0.3)}
.border-s-error\/40 {
    border-inline-start-color: hsl(var(--er) / 0.4)}
.border-s-error\/5 {
    border-inline-start-color: hsl(var(--er) / 0.05)}
.border-s-error\/50 {
    border-inline-start-color: hsl(var(--er) / 0.5)}
.border-s-error\/60 {
    border-inline-start-color: hsl(var(--er) / 0.6)}
.border-s-error\/70 {
    border-inline-start-color: hsl(var(--er) / 0.7)}
.border-s-error\/75 {
    border-inline-start-color: hsl(var(--er) / 0.75)}
.border-s-error\/80 {
    border-inline-start-color: hsl(var(--er) / 0.8)}
.border-s-error\/90 {
    border-inline-start-color: hsl(var(--er) / 0.9)}
.border-s-error\/95 {
    border-inline-start-color: hsl(var(--er) / 0.95)}
.border-s-info {
    border-inline-start-color: hsl(var(--in) / 1)}
.border-s-info-content {
    border-inline-start-color: hsl(var(--inc) / 1)}
.border-s-info-content\/0 {
    border-inline-start-color: hsl(var(--inc) / 0)}
.border-s-info-content\/10 {
    border-inline-start-color: hsl(var(--inc) / 0.1)}
.border-s-info-content\/100 {
    border-inline-start-color: hsl(var(--inc) / 1)}
.border-s-info-content\/20 {
    border-inline-start-color: hsl(var(--inc) / 0.2)}
.border-s-info-content\/25 {
    border-inline-start-color: hsl(var(--inc) / 0.25)}
.border-s-info-content\/30 {
    border-inline-start-color: hsl(var(--inc) / 0.3)}
.border-s-info-content\/40 {
    border-inline-start-color: hsl(var(--inc) / 0.4)}
.border-s-info-content\/5 {
    border-inline-start-color: hsl(var(--inc) / 0.05)}
.border-s-info-content\/50 {
    border-inline-start-color: hsl(var(--inc) / 0.5)}
.border-s-info-content\/60 {
    border-inline-start-color: hsl(var(--inc) / 0.6)}
.border-s-info-content\/70 {
    border-inline-start-color: hsl(var(--inc) / 0.7)}
.border-s-info-content\/75 {
    border-inline-start-color: hsl(var(--inc) / 0.75)}
.border-s-info-content\/80 {
    border-inline-start-color: hsl(var(--inc) / 0.8)}
.border-s-info-content\/90 {
    border-inline-start-color: hsl(var(--inc) / 0.9)}
.border-s-info-content\/95 {
    border-inline-start-color: hsl(var(--inc) / 0.95)}
.border-s-info\/0 {
    border-inline-start-color: hsl(var(--in) / 0)}
.border-s-info\/10 {
    border-inline-start-color: hsl(var(--in) / 0.1)}
.border-s-info\/100 {
    border-inline-start-color: hsl(var(--in) / 1)}
.border-s-info\/20 {
    border-inline-start-color: hsl(var(--in) / 0.2)}
.border-s-info\/25 {
    border-inline-start-color: hsl(var(--in) / 0.25)}
.border-s-info\/30 {
    border-inline-start-color: hsl(var(--in) / 0.3)}
.border-s-info\/40 {
    border-inline-start-color: hsl(var(--in) / 0.4)}
.border-s-info\/5 {
    border-inline-start-color: hsl(var(--in) / 0.05)}
.border-s-info\/50 {
    border-inline-start-color: hsl(var(--in) / 0.5)}
.border-s-info\/60 {
    border-inline-start-color: hsl(var(--in) / 0.6)}
.border-s-info\/70 {
    border-inline-start-color: hsl(var(--in) / 0.7)}
.border-s-info\/75 {
    border-inline-start-color: hsl(var(--in) / 0.75)}
.border-s-info\/80 {
    border-inline-start-color: hsl(var(--in) / 0.8)}
.border-s-info\/90 {
    border-inline-start-color: hsl(var(--in) / 0.9)}
.border-s-info\/95 {
    border-inline-start-color: hsl(var(--in) / 0.95)}
.border-s-neutral {
    border-inline-start-color: hsl(var(--n) / 1)}
.border-s-neutral-content {
    border-inline-start-color: hsl(var(--nc) / 1)}
.border-s-neutral-content\/0 {
    border-inline-start-color: hsl(var(--nc) / 0)}
.border-s-neutral-content\/10 {
    border-inline-start-color: hsl(var(--nc) / 0.1)}
.border-s-neutral-content\/100 {
    border-inline-start-color: hsl(var(--nc) / 1)}
.border-s-neutral-content\/20 {
    border-inline-start-color: hsl(var(--nc) / 0.2)}
.border-s-neutral-content\/25 {
    border-inline-start-color: hsl(var(--nc) / 0.25)}
.border-s-neutral-content\/30 {
    border-inline-start-color: hsl(var(--nc) / 0.3)}
.border-s-neutral-content\/40 {
    border-inline-start-color: hsl(var(--nc) / 0.4)}
.border-s-neutral-content\/5 {
    border-inline-start-color: hsl(var(--nc) / 0.05)}
.border-s-neutral-content\/50 {
    border-inline-start-color: hsl(var(--nc) / 0.5)}
.border-s-neutral-content\/60 {
    border-inline-start-color: hsl(var(--nc) / 0.6)}
.border-s-neutral-content\/70 {
    border-inline-start-color: hsl(var(--nc) / 0.7)}
.border-s-neutral-content\/75 {
    border-inline-start-color: hsl(var(--nc) / 0.75)}
.border-s-neutral-content\/80 {
    border-inline-start-color: hsl(var(--nc) / 0.8)}
.border-s-neutral-content\/90 {
    border-inline-start-color: hsl(var(--nc) / 0.9)}
.border-s-neutral-content\/95 {
    border-inline-start-color: hsl(var(--nc) / 0.95)}
.border-s-neutral-focus {
    border-inline-start-color: hsl(var(--nf) / 1)}
.border-s-neutral-focus\/0 {
    border-inline-start-color: hsl(var(--nf) / 0)}
.border-s-neutral-focus\/10 {
    border-inline-start-color: hsl(var(--nf) / 0.1)}
.border-s-neutral-focus\/100 {
    border-inline-start-color: hsl(var(--nf) / 1)}
.border-s-neutral-focus\/20 {
    border-inline-start-color: hsl(var(--nf) / 0.2)}
.border-s-neutral-focus\/25 {
    border-inline-start-color: hsl(var(--nf) / 0.25)}
.border-s-neutral-focus\/30 {
    border-inline-start-color: hsl(var(--nf) / 0.3)}
.border-s-neutral-focus\/40 {
    border-inline-start-color: hsl(var(--nf) / 0.4)}
.border-s-neutral-focus\/5 {
    border-inline-start-color: hsl(var(--nf) / 0.05)}
.border-s-neutral-focus\/50 {
    border-inline-start-color: hsl(var(--nf) / 0.5)}
.border-s-neutral-focus\/60 {
    border-inline-start-color: hsl(var(--nf) / 0.6)}
.border-s-neutral-focus\/70 {
    border-inline-start-color: hsl(var(--nf) / 0.7)}
.border-s-neutral-focus\/75 {
    border-inline-start-color: hsl(var(--nf) / 0.75)}
.border-s-neutral-focus\/80 {
    border-inline-start-color: hsl(var(--nf) / 0.8)}
.border-s-neutral-focus\/90 {
    border-inline-start-color: hsl(var(--nf) / 0.9)}
.border-s-neutral-focus\/95 {
    border-inline-start-color: hsl(var(--nf) / 0.95)}
.border-s-neutral\/0 {
    border-inline-start-color: hsl(var(--n) / 0)}
.border-s-neutral\/10 {
    border-inline-start-color: hsl(var(--n) / 0.1)}
.border-s-neutral\/100 {
    border-inline-start-color: hsl(var(--n) / 1)}
.border-s-neutral\/20 {
    border-inline-start-color: hsl(var(--n) / 0.2)}
.border-s-neutral\/25 {
    border-inline-start-color: hsl(var(--n) / 0.25)}
.border-s-neutral\/30 {
    border-inline-start-color: hsl(var(--n) / 0.3)}
.border-s-neutral\/40 {
    border-inline-start-color: hsl(var(--n) / 0.4)}
.border-s-neutral\/5 {
    border-inline-start-color: hsl(var(--n) / 0.05)}
.border-s-neutral\/50 {
    border-inline-start-color: hsl(var(--n) / 0.5)}
.border-s-neutral\/60 {
    border-inline-start-color: hsl(var(--n) / 0.6)}
.border-s-neutral\/70 {
    border-inline-start-color: hsl(var(--n) / 0.7)}
.border-s-neutral\/75 {
    border-inline-start-color: hsl(var(--n) / 0.75)}
.border-s-neutral\/80 {
    border-inline-start-color: hsl(var(--n) / 0.8)}
.border-s-neutral\/90 {
    border-inline-start-color: hsl(var(--n) / 0.9)}
.border-s-neutral\/95 {
    border-inline-start-color: hsl(var(--n) / 0.95)}
.border-s-primary {
    border-inline-start-color: hsl(var(--p) / 1)}
.border-s-primary-content {
    border-inline-start-color: hsl(var(--pc) / 1)}
.border-s-primary-content\/0 {
    border-inline-start-color: hsl(var(--pc) / 0)}
.border-s-primary-content\/10 {
    border-inline-start-color: hsl(var(--pc) / 0.1)}
.border-s-primary-content\/100 {
    border-inline-start-color: hsl(var(--pc) / 1)}
.border-s-primary-content\/20 {
    border-inline-start-color: hsl(var(--pc) / 0.2)}
.border-s-primary-content\/25 {
    border-inline-start-color: hsl(var(--pc) / 0.25)}
.border-s-primary-content\/30 {
    border-inline-start-color: hsl(var(--pc) / 0.3)}
.border-s-primary-content\/40 {
    border-inline-start-color: hsl(var(--pc) / 0.4)}
.border-s-primary-content\/5 {
    border-inline-start-color: hsl(var(--pc) / 0.05)}
.border-s-primary-content\/50 {
    border-inline-start-color: hsl(var(--pc) / 0.5)}
.border-s-primary-content\/60 {
    border-inline-start-color: hsl(var(--pc) / 0.6)}
.border-s-primary-content\/70 {
    border-inline-start-color: hsl(var(--pc) / 0.7)}
.border-s-primary-content\/75 {
    border-inline-start-color: hsl(var(--pc) / 0.75)}
.border-s-primary-content\/80 {
    border-inline-start-color: hsl(var(--pc) / 0.8)}
.border-s-primary-content\/90 {
    border-inline-start-color: hsl(var(--pc) / 0.9)}
.border-s-primary-content\/95 {
    border-inline-start-color: hsl(var(--pc) / 0.95)}
.border-s-primary-focus {
    border-inline-start-color: hsl(var(--pf) / 1)}
.border-s-primary-focus\/0 {
    border-inline-start-color: hsl(var(--pf) / 0)}
.border-s-primary-focus\/10 {
    border-inline-start-color: hsl(var(--pf) / 0.1)}
.border-s-primary-focus\/100 {
    border-inline-start-color: hsl(var(--pf) / 1)}
.border-s-primary-focus\/20 {
    border-inline-start-color: hsl(var(--pf) / 0.2)}
.border-s-primary-focus\/25 {
    border-inline-start-color: hsl(var(--pf) / 0.25)}
.border-s-primary-focus\/30 {
    border-inline-start-color: hsl(var(--pf) / 0.3)}
.border-s-primary-focus\/40 {
    border-inline-start-color: hsl(var(--pf) / 0.4)}
.border-s-primary-focus\/5 {
    border-inline-start-color: hsl(var(--pf) / 0.05)}
.border-s-primary-focus\/50 {
    border-inline-start-color: hsl(var(--pf) / 0.5)}
.border-s-primary-focus\/60 {
    border-inline-start-color: hsl(var(--pf) / 0.6)}
.border-s-primary-focus\/70 {
    border-inline-start-color: hsl(var(--pf) / 0.7)}
.border-s-primary-focus\/75 {
    border-inline-start-color: hsl(var(--pf) / 0.75)}
.border-s-primary-focus\/80 {
    border-inline-start-color: hsl(var(--pf) / 0.8)}
.border-s-primary-focus\/90 {
    border-inline-start-color: hsl(var(--pf) / 0.9)}
.border-s-primary-focus\/95 {
    border-inline-start-color: hsl(var(--pf) / 0.95)}
.border-s-primary\/0 {
    border-inline-start-color: hsl(var(--p) / 0)}
.border-s-primary\/10 {
    border-inline-start-color: hsl(var(--p) / 0.1)}
.border-s-primary\/100 {
    border-inline-start-color: hsl(var(--p) / 1)}
.border-s-primary\/20 {
    border-inline-start-color: hsl(var(--p) / 0.2)}
.border-s-primary\/25 {
    border-inline-start-color: hsl(var(--p) / 0.25)}
.border-s-primary\/30 {
    border-inline-start-color: hsl(var(--p) / 0.3)}
.border-s-primary\/40 {
    border-inline-start-color: hsl(var(--p) / 0.4)}
.border-s-primary\/5 {
    border-inline-start-color: hsl(var(--p) / 0.05)}
.border-s-primary\/50 {
    border-inline-start-color: hsl(var(--p) / 0.5)}
.border-s-primary\/60 {
    border-inline-start-color: hsl(var(--p) / 0.6)}
.border-s-primary\/70 {
    border-inline-start-color: hsl(var(--p) / 0.7)}
.border-s-primary\/75 {
    border-inline-start-color: hsl(var(--p) / 0.75)}
.border-s-primary\/80 {
    border-inline-start-color: hsl(var(--p) / 0.8)}
.border-s-primary\/90 {
    border-inline-start-color: hsl(var(--p) / 0.9)}
.border-s-primary\/95 {
    border-inline-start-color: hsl(var(--p) / 0.95)}
.border-s-secondary {
    border-inline-start-color: hsl(var(--s) / 1)}
.border-s-secondary-content {
    border-inline-start-color: hsl(var(--sc) / 1)}
.border-s-secondary-content\/0 {
    border-inline-start-color: hsl(var(--sc) / 0)}
.border-s-secondary-content\/10 {
    border-inline-start-color: hsl(var(--sc) / 0.1)}
.border-s-secondary-content\/100 {
    border-inline-start-color: hsl(var(--sc) / 1)}
.border-s-secondary-content\/20 {
    border-inline-start-color: hsl(var(--sc) / 0.2)}
.border-s-secondary-content\/25 {
    border-inline-start-color: hsl(var(--sc) / 0.25)}
.border-s-secondary-content\/30 {
    border-inline-start-color: hsl(var(--sc) / 0.3)}
.border-s-secondary-content\/40 {
    border-inline-start-color: hsl(var(--sc) / 0.4)}
.border-s-secondary-content\/5 {
    border-inline-start-color: hsl(var(--sc) / 0.05)}
.border-s-secondary-content\/50 {
    border-inline-start-color: hsl(var(--sc) / 0.5)}
.border-s-secondary-content\/60 {
    border-inline-start-color: hsl(var(--sc) / 0.6)}
.border-s-secondary-content\/70 {
    border-inline-start-color: hsl(var(--sc) / 0.7)}
.border-s-secondary-content\/75 {
    border-inline-start-color: hsl(var(--sc) / 0.75)}
.border-s-secondary-content\/80 {
    border-inline-start-color: hsl(var(--sc) / 0.8)}
.border-s-secondary-content\/90 {
    border-inline-start-color: hsl(var(--sc) / 0.9)}
.border-s-secondary-content\/95 {
    border-inline-start-color: hsl(var(--sc) / 0.95)}
.border-s-secondary-focus {
    border-inline-start-color: hsl(var(--sf) / 1)}
.border-s-secondary-focus\/0 {
    border-inline-start-color: hsl(var(--sf) / 0)}
.border-s-secondary-focus\/10 {
    border-inline-start-color: hsl(var(--sf) / 0.1)}
.border-s-secondary-focus\/100 {
    border-inline-start-color: hsl(var(--sf) / 1)}
.border-s-secondary-focus\/20 {
    border-inline-start-color: hsl(var(--sf) / 0.2)}
.border-s-secondary-focus\/25 {
    border-inline-start-color: hsl(var(--sf) / 0.25)}
.border-s-secondary-focus\/30 {
    border-inline-start-color: hsl(var(--sf) / 0.3)}
.border-s-secondary-focus\/40 {
    border-inline-start-color: hsl(var(--sf) / 0.4)}
.border-s-secondary-focus\/5 {
    border-inline-start-color: hsl(var(--sf) / 0.05)}
.border-s-secondary-focus\/50 {
    border-inline-start-color: hsl(var(--sf) / 0.5)}
.border-s-secondary-focus\/60 {
    border-inline-start-color: hsl(var(--sf) / 0.6)}
.border-s-secondary-focus\/70 {
    border-inline-start-color: hsl(var(--sf) / 0.7)}
.border-s-secondary-focus\/75 {
    border-inline-start-color: hsl(var(--sf) / 0.75)}
.border-s-secondary-focus\/80 {
    border-inline-start-color: hsl(var(--sf) / 0.8)}
.border-s-secondary-focus\/90 {
    border-inline-start-color: hsl(var(--sf) / 0.9)}
.border-s-secondary-focus\/95 {
    border-inline-start-color: hsl(var(--sf) / 0.95)}
.border-s-secondary\/0 {
    border-inline-start-color: hsl(var(--s) / 0)}
.border-s-secondary\/10 {
    border-inline-start-color: hsl(var(--s) / 0.1)}
.border-s-secondary\/100 {
    border-inline-start-color: hsl(var(--s) / 1)}
.border-s-secondary\/20 {
    border-inline-start-color: hsl(var(--s) / 0.2)}
.border-s-secondary\/25 {
    border-inline-start-color: hsl(var(--s) / 0.25)}
.border-s-secondary\/30 {
    border-inline-start-color: hsl(var(--s) / 0.3)}
.border-s-secondary\/40 {
    border-inline-start-color: hsl(var(--s) / 0.4)}
.border-s-secondary\/5 {
    border-inline-start-color: hsl(var(--s) / 0.05)}
.border-s-secondary\/50 {
    border-inline-start-color: hsl(var(--s) / 0.5)}
.border-s-secondary\/60 {
    border-inline-start-color: hsl(var(--s) / 0.6)}
.border-s-secondary\/70 {
    border-inline-start-color: hsl(var(--s) / 0.7)}
.border-s-secondary\/75 {
    border-inline-start-color: hsl(var(--s) / 0.75)}
.border-s-secondary\/80 {
    border-inline-start-color: hsl(var(--s) / 0.8)}
.border-s-secondary\/90 {
    border-inline-start-color: hsl(var(--s) / 0.9)}
.border-s-secondary\/95 {
    border-inline-start-color: hsl(var(--s) / 0.95)}
.border-s-success {
    border-inline-start-color: hsl(var(--su) / 1)}
.border-s-success-content {
    border-inline-start-color: hsl(var(--suc) / 1)}
.border-s-success-content\/0 {
    border-inline-start-color: hsl(var(--suc) / 0)}
.border-s-success-content\/10 {
    border-inline-start-color: hsl(var(--suc) / 0.1)}
.border-s-success-content\/100 {
    border-inline-start-color: hsl(var(--suc) / 1)}
.border-s-success-content\/20 {
    border-inline-start-color: hsl(var(--suc) / 0.2)}
.border-s-success-content\/25 {
    border-inline-start-color: hsl(var(--suc) / 0.25)}
.border-s-success-content\/30 {
    border-inline-start-color: hsl(var(--suc) / 0.3)}
.border-s-success-content\/40 {
    border-inline-start-color: hsl(var(--suc) / 0.4)}
.border-s-success-content\/5 {
    border-inline-start-color: hsl(var(--suc) / 0.05)}
.border-s-success-content\/50 {
    border-inline-start-color: hsl(var(--suc) / 0.5)}
.border-s-success-content\/60 {
    border-inline-start-color: hsl(var(--suc) / 0.6)}
.border-s-success-content\/70 {
    border-inline-start-color: hsl(var(--suc) / 0.7)}
.border-s-success-content\/75 {
    border-inline-start-color: hsl(var(--suc) / 0.75)}
.border-s-success-content\/80 {
    border-inline-start-color: hsl(var(--suc) / 0.8)}
.border-s-success-content\/90 {
    border-inline-start-color: hsl(var(--suc) / 0.9)}
.border-s-success-content\/95 {
    border-inline-start-color: hsl(var(--suc) / 0.95)}
.border-s-success\/0 {
    border-inline-start-color: hsl(var(--su) / 0)}
.border-s-success\/10 {
    border-inline-start-color: hsl(var(--su) / 0.1)}
.border-s-success\/100 {
    border-inline-start-color: hsl(var(--su) / 1)}
.border-s-success\/20 {
    border-inline-start-color: hsl(var(--su) / 0.2)}
.border-s-success\/25 {
    border-inline-start-color: hsl(var(--su) / 0.25)}
.border-s-success\/30 {
    border-inline-start-color: hsl(var(--su) / 0.3)}
.border-s-success\/40 {
    border-inline-start-color: hsl(var(--su) / 0.4)}
.border-s-success\/5 {
    border-inline-start-color: hsl(var(--su) / 0.05)}
.border-s-success\/50 {
    border-inline-start-color: hsl(var(--su) / 0.5)}
.border-s-success\/60 {
    border-inline-start-color: hsl(var(--su) / 0.6)}
.border-s-success\/70 {
    border-inline-start-color: hsl(var(--su) / 0.7)}
.border-s-success\/75 {
    border-inline-start-color: hsl(var(--su) / 0.75)}
.border-s-success\/80 {
    border-inline-start-color: hsl(var(--su) / 0.8)}
.border-s-success\/90 {
    border-inline-start-color: hsl(var(--su) / 0.9)}
.border-s-success\/95 {
    border-inline-start-color: hsl(var(--su) / 0.95)}
.border-s-transparent {
    border-inline-start-color: transparent}
.border-s-transparent\/0 {
    border-inline-start-color: rgb(0 0 0 / 0)}
.border-s-transparent\/10 {
    border-inline-start-color: rgb(0 0 0 / 0.1)}
.border-s-transparent\/100 {
    border-inline-start-color: rgb(0 0 0 / 1)}
.border-s-transparent\/20 {
    border-inline-start-color: rgb(0 0 0 / 0.2)}
.border-s-transparent\/25 {
    border-inline-start-color: rgb(0 0 0 / 0.25)}
.border-s-transparent\/30 {
    border-inline-start-color: rgb(0 0 0 / 0.3)}
.border-s-transparent\/40 {
    border-inline-start-color: rgb(0 0 0 / 0.4)}
.border-s-transparent\/5 {
    border-inline-start-color: rgb(0 0 0 / 0.05)}
.border-s-transparent\/50 {
    border-inline-start-color: rgb(0 0 0 / 0.5)}
.border-s-transparent\/60 {
    border-inline-start-color: rgb(0 0 0 / 0.6)}
.border-s-transparent\/70 {
    border-inline-start-color: rgb(0 0 0 / 0.7)}
.border-s-transparent\/75 {
    border-inline-start-color: rgb(0 0 0 / 0.75)}
.border-s-transparent\/80 {
    border-inline-start-color: rgb(0 0 0 / 0.8)}
.border-s-transparent\/90 {
    border-inline-start-color: rgb(0 0 0 / 0.9)}
.border-s-transparent\/95 {
    border-inline-start-color: rgb(0 0 0 / 0.95)}
.border-s-warning {
    border-inline-start-color: hsl(var(--wa) / 1)}
.border-s-warning-content {
    border-inline-start-color: hsl(var(--wac) / 1)}
.border-s-warning-content\/0 {
    border-inline-start-color: hsl(var(--wac) / 0)}
.border-s-warning-content\/10 {
    border-inline-start-color: hsl(var(--wac) / 0.1)}
.border-s-warning-content\/100 {
    border-inline-start-color: hsl(var(--wac) / 1)}
.border-s-warning-content\/20 {
    border-inline-start-color: hsl(var(--wac) / 0.2)}
.border-s-warning-content\/25 {
    border-inline-start-color: hsl(var(--wac) / 0.25)}
.border-s-warning-content\/30 {
    border-inline-start-color: hsl(var(--wac) / 0.3)}
.border-s-warning-content\/40 {
    border-inline-start-color: hsl(var(--wac) / 0.4)}
.border-s-warning-content\/5 {
    border-inline-start-color: hsl(var(--wac) / 0.05)}
.border-s-warning-content\/50 {
    border-inline-start-color: hsl(var(--wac) / 0.5)}
.border-s-warning-content\/60 {
    border-inline-start-color: hsl(var(--wac) / 0.6)}
.border-s-warning-content\/70 {
    border-inline-start-color: hsl(var(--wac) / 0.7)}
.border-s-warning-content\/75 {
    border-inline-start-color: hsl(var(--wac) / 0.75)}
.border-s-warning-content\/80 {
    border-inline-start-color: hsl(var(--wac) / 0.8)}
.border-s-warning-content\/90 {
    border-inline-start-color: hsl(var(--wac) / 0.9)}
.border-s-warning-content\/95 {
    border-inline-start-color: hsl(var(--wac) / 0.95)}
.border-s-warning\/0 {
    border-inline-start-color: hsl(var(--wa) / 0)}
.border-s-warning\/10 {
    border-inline-start-color: hsl(var(--wa) / 0.1)}
.border-s-warning\/100 {
    border-inline-start-color: hsl(var(--wa) / 1)}
.border-s-warning\/20 {
    border-inline-start-color: hsl(var(--wa) / 0.2)}
.border-s-warning\/25 {
    border-inline-start-color: hsl(var(--wa) / 0.25)}
.border-s-warning\/30 {
    border-inline-start-color: hsl(var(--wa) / 0.3)}
.border-s-warning\/40 {
    border-inline-start-color: hsl(var(--wa) / 0.4)}
.border-s-warning\/5 {
    border-inline-start-color: hsl(var(--wa) / 0.05)}
.border-s-warning\/50 {
    border-inline-start-color: hsl(var(--wa) / 0.5)}
.border-s-warning\/60 {
    border-inline-start-color: hsl(var(--wa) / 0.6)}
.border-s-warning\/70 {
    border-inline-start-color: hsl(var(--wa) / 0.7)}
.border-s-warning\/75 {
    border-inline-start-color: hsl(var(--wa) / 0.75)}
.border-s-warning\/80 {
    border-inline-start-color: hsl(var(--wa) / 0.8)}
.border-s-warning\/90 {
    border-inline-start-color: hsl(var(--wa) / 0.9)}
.border-s-warning\/95 {
    border-inline-start-color: hsl(var(--wa) / 0.95)}
.border-t-accent {
    border-top-color: hsl(var(--a) / 1)}
.border-t-accent-content {
    border-top-color: hsl(var(--ac) / 1)}
.border-t-accent-content\/0 {
    border-top-color: hsl(var(--ac) / 0)}
.border-t-accent-content\/10 {
    border-top-color: hsl(var(--ac) / 0.1)}
.border-t-accent-content\/100 {
    border-top-color: hsl(var(--ac) / 1)}
.border-t-accent-content\/20 {
    border-top-color: hsl(var(--ac) / 0.2)}
.border-t-accent-content\/25 {
    border-top-color: hsl(var(--ac) / 0.25)}
.border-t-accent-content\/30 {
    border-top-color: hsl(var(--ac) / 0.3)}
.border-t-accent-content\/40 {
    border-top-color: hsl(var(--ac) / 0.4)}
.border-t-accent-content\/5 {
    border-top-color: hsl(var(--ac) / 0.05)}
.border-t-accent-content\/50 {
    border-top-color: hsl(var(--ac) / 0.5)}
.border-t-accent-content\/60 {
    border-top-color: hsl(var(--ac) / 0.6)}
.border-t-accent-content\/70 {
    border-top-color: hsl(var(--ac) / 0.7)}
.border-t-accent-content\/75 {
    border-top-color: hsl(var(--ac) / 0.75)}
.border-t-accent-content\/80 {
    border-top-color: hsl(var(--ac) / 0.8)}
.border-t-accent-content\/90 {
    border-top-color: hsl(var(--ac) / 0.9)}
.border-t-accent-content\/95 {
    border-top-color: hsl(var(--ac) / 0.95)}
.border-t-accent-focus {
    border-top-color: hsl(var(--af) / 1)}
.border-t-accent-focus\/0 {
    border-top-color: hsl(var(--af) / 0)}
.border-t-accent-focus\/10 {
    border-top-color: hsl(var(--af) / 0.1)}
.border-t-accent-focus\/100 {
    border-top-color: hsl(var(--af) / 1)}
.border-t-accent-focus\/20 {
    border-top-color: hsl(var(--af) / 0.2)}
.border-t-accent-focus\/25 {
    border-top-color: hsl(var(--af) / 0.25)}
.border-t-accent-focus\/30 {
    border-top-color: hsl(var(--af) / 0.3)}
.border-t-accent-focus\/40 {
    border-top-color: hsl(var(--af) / 0.4)}
.border-t-accent-focus\/5 {
    border-top-color: hsl(var(--af) / 0.05)}
.border-t-accent-focus\/50 {
    border-top-color: hsl(var(--af) / 0.5)}
.border-t-accent-focus\/60 {
    border-top-color: hsl(var(--af) / 0.6)}
.border-t-accent-focus\/70 {
    border-top-color: hsl(var(--af) / 0.7)}
.border-t-accent-focus\/75 {
    border-top-color: hsl(var(--af) / 0.75)}
.border-t-accent-focus\/80 {
    border-top-color: hsl(var(--af) / 0.8)}
.border-t-accent-focus\/90 {
    border-top-color: hsl(var(--af) / 0.9)}
.border-t-accent-focus\/95 {
    border-top-color: hsl(var(--af) / 0.95)}
.border-t-accent\/0 {
    border-top-color: hsl(var(--a) / 0)}
.border-t-accent\/10 {
    border-top-color: hsl(var(--a) / 0.1)}
.border-t-accent\/100 {
    border-top-color: hsl(var(--a) / 1)}
.border-t-accent\/20 {
    border-top-color: hsl(var(--a) / 0.2)}
.border-t-accent\/25 {
    border-top-color: hsl(var(--a) / 0.25)}
.border-t-accent\/30 {
    border-top-color: hsl(var(--a) / 0.3)}
.border-t-accent\/40 {
    border-top-color: hsl(var(--a) / 0.4)}
.border-t-accent\/5 {
    border-top-color: hsl(var(--a) / 0.05)}
.border-t-accent\/50 {
    border-top-color: hsl(var(--a) / 0.5)}
.border-t-accent\/60 {
    border-top-color: hsl(var(--a) / 0.6)}
.border-t-accent\/70 {
    border-top-color: hsl(var(--a) / 0.7)}
.border-t-accent\/75 {
    border-top-color: hsl(var(--a) / 0.75)}
.border-t-accent\/80 {
    border-top-color: hsl(var(--a) / 0.8)}
.border-t-accent\/90 {
    border-top-color: hsl(var(--a) / 0.9)}
.border-t-accent\/95 {
    border-top-color: hsl(var(--a) / 0.95)}
.border-t-base-100 {
    border-top-color: hsl(var(--b1) / 1)}
.border-t-base-100\/0 {
    border-top-color: hsl(var(--b1) / 0)}
.border-t-base-100\/10 {
    border-top-color: hsl(var(--b1) / 0.1)}
.border-t-base-100\/100 {
    border-top-color: hsl(var(--b1) / 1)}
.border-t-base-100\/20 {
    border-top-color: hsl(var(--b1) / 0.2)}
.border-t-base-100\/25 {
    border-top-color: hsl(var(--b1) / 0.25)}
.border-t-base-100\/30 {
    border-top-color: hsl(var(--b1) / 0.3)}
.border-t-base-100\/40 {
    border-top-color: hsl(var(--b1) / 0.4)}
.border-t-base-100\/5 {
    border-top-color: hsl(var(--b1) / 0.05)}
.border-t-base-100\/50 {
    border-top-color: hsl(var(--b1) / 0.5)}
.border-t-base-100\/60 {
    border-top-color: hsl(var(--b1) / 0.6)}
.border-t-base-100\/70 {
    border-top-color: hsl(var(--b1) / 0.7)}
.border-t-base-100\/75 {
    border-top-color: hsl(var(--b1) / 0.75)}
.border-t-base-100\/80 {
    border-top-color: hsl(var(--b1) / 0.8)}
.border-t-base-100\/90 {
    border-top-color: hsl(var(--b1) / 0.9)}
.border-t-base-100\/95 {
    border-top-color: hsl(var(--b1) / 0.95)}
.border-t-base-200 {
    border-top-color: hsl(var(--b2) / 1)}
.border-t-base-200\/0 {
    border-top-color: hsl(var(--b2) / 0)}
.border-t-base-200\/10 {
    border-top-color: hsl(var(--b2) / 0.1)}
.border-t-base-200\/100 {
    border-top-color: hsl(var(--b2) / 1)}
.border-t-base-200\/20 {
    border-top-color: hsl(var(--b2) / 0.2)}
.border-t-base-200\/25 {
    border-top-color: hsl(var(--b2) / 0.25)}
.border-t-base-200\/30 {
    border-top-color: hsl(var(--b2) / 0.3)}
.border-t-base-200\/40 {
    border-top-color: hsl(var(--b2) / 0.4)}
.border-t-base-200\/5 {
    border-top-color: hsl(var(--b2) / 0.05)}
.border-t-base-200\/50 {
    border-top-color: hsl(var(--b2) / 0.5)}
.border-t-base-200\/60 {
    border-top-color: hsl(var(--b2) / 0.6)}
.border-t-base-200\/70 {
    border-top-color: hsl(var(--b2) / 0.7)}
.border-t-base-200\/75 {
    border-top-color: hsl(var(--b2) / 0.75)}
.border-t-base-200\/80 {
    border-top-color: hsl(var(--b2) / 0.8)}
.border-t-base-200\/90 {
    border-top-color: hsl(var(--b2) / 0.9)}
.border-t-base-200\/95 {
    border-top-color: hsl(var(--b2) / 0.95)}
.border-t-base-300 {
    border-top-color: hsl(var(--b3) / 1)}
.border-t-base-300\/0 {
    border-top-color: hsl(var(--b3) / 0)}
.border-t-base-300\/10 {
    border-top-color: hsl(var(--b3) / 0.1)}
.border-t-base-300\/100 {
    border-top-color: hsl(var(--b3) / 1)}
.border-t-base-300\/20 {
    border-top-color: hsl(var(--b3) / 0.2)}
.border-t-base-300\/25 {
    border-top-color: hsl(var(--b3) / 0.25)}
.border-t-base-300\/30 {
    border-top-color: hsl(var(--b3) / 0.3)}
.border-t-base-300\/40 {
    border-top-color: hsl(var(--b3) / 0.4)}
.border-t-base-300\/5 {
    border-top-color: hsl(var(--b3) / 0.05)}
.border-t-base-300\/50 {
    border-top-color: hsl(var(--b3) / 0.5)}
.border-t-base-300\/60 {
    border-top-color: hsl(var(--b3) / 0.6)}
.border-t-base-300\/70 {
    border-top-color: hsl(var(--b3) / 0.7)}
.border-t-base-300\/75 {
    border-top-color: hsl(var(--b3) / 0.75)}
.border-t-base-300\/80 {
    border-top-color: hsl(var(--b3) / 0.8)}
.border-t-base-300\/90 {
    border-top-color: hsl(var(--b3) / 0.9)}
.border-t-base-300\/95 {
    border-top-color: hsl(var(--b3) / 0.95)}
.border-t-base-content {
    border-top-color: hsl(var(--bc) / 1)}
.border-t-base-content\/0 {
    border-top-color: hsl(var(--bc) / 0)}
.border-t-base-content\/10 {
    border-top-color: hsl(var(--bc) / 0.1)}
.border-t-base-content\/100 {
    border-top-color: hsl(var(--bc) / 1)}
.border-t-base-content\/20 {
    border-top-color: hsl(var(--bc) / 0.2)}
.border-t-base-content\/25 {
    border-top-color: hsl(var(--bc) / 0.25)}
.border-t-base-content\/30 {
    border-top-color: hsl(var(--bc) / 0.3)}
.border-t-base-content\/40 {
    border-top-color: hsl(var(--bc) / 0.4)}
.border-t-base-content\/5 {
    border-top-color: hsl(var(--bc) / 0.05)}
.border-t-base-content\/50 {
    border-top-color: hsl(var(--bc) / 0.5)}
.border-t-base-content\/60 {
    border-top-color: hsl(var(--bc) / 0.6)}
.border-t-base-content\/70 {
    border-top-color: hsl(var(--bc) / 0.7)}
.border-t-base-content\/75 {
    border-top-color: hsl(var(--bc) / 0.75)}
.border-t-base-content\/80 {
    border-top-color: hsl(var(--bc) / 0.8)}
.border-t-base-content\/90 {
    border-top-color: hsl(var(--bc) / 0.9)}
.border-t-base-content\/95 {
    border-top-color: hsl(var(--bc) / 0.95)}
.border-t-current {
    border-top-color: currentColor}
.border-t-error {
    border-top-color: hsl(var(--er) / 1)}
.border-t-error-content {
    border-top-color: hsl(var(--erc) / 1)}
.border-t-error-content\/0 {
    border-top-color: hsl(var(--erc) / 0)}
.border-t-error-content\/10 {
    border-top-color: hsl(var(--erc) / 0.1)}
.border-t-error-content\/100 {
    border-top-color: hsl(var(--erc) / 1)}
.border-t-error-content\/20 {
    border-top-color: hsl(var(--erc) / 0.2)}
.border-t-error-content\/25 {
    border-top-color: hsl(var(--erc) / 0.25)}
.border-t-error-content\/30 {
    border-top-color: hsl(var(--erc) / 0.3)}
.border-t-error-content\/40 {
    border-top-color: hsl(var(--erc) / 0.4)}
.border-t-error-content\/5 {
    border-top-color: hsl(var(--erc) / 0.05)}
.border-t-error-content\/50 {
    border-top-color: hsl(var(--erc) / 0.5)}
.border-t-error-content\/60 {
    border-top-color: hsl(var(--erc) / 0.6)}
.border-t-error-content\/70 {
    border-top-color: hsl(var(--erc) / 0.7)}
.border-t-error-content\/75 {
    border-top-color: hsl(var(--erc) / 0.75)}
.border-t-error-content\/80 {
    border-top-color: hsl(var(--erc) / 0.8)}
.border-t-error-content\/90 {
    border-top-color: hsl(var(--erc) / 0.9)}
.border-t-error-content\/95 {
    border-top-color: hsl(var(--erc) / 0.95)}
.border-t-error\/0 {
    border-top-color: hsl(var(--er) / 0)}
.border-t-error\/10 {
    border-top-color: hsl(var(--er) / 0.1)}
.border-t-error\/100 {
    border-top-color: hsl(var(--er) / 1)}
.border-t-error\/20 {
    border-top-color: hsl(var(--er) / 0.2)}
.border-t-error\/25 {
    border-top-color: hsl(var(--er) / 0.25)}
.border-t-error\/30 {
    border-top-color: hsl(var(--er) / 0.3)}
.border-t-error\/40 {
    border-top-color: hsl(var(--er) / 0.4)}
.border-t-error\/5 {
    border-top-color: hsl(var(--er) / 0.05)}
.border-t-error\/50 {
    border-top-color: hsl(var(--er) / 0.5)}
.border-t-error\/60 {
    border-top-color: hsl(var(--er) / 0.6)}
.border-t-error\/70 {
    border-top-color: hsl(var(--er) / 0.7)}
.border-t-error\/75 {
    border-top-color: hsl(var(--er) / 0.75)}
.border-t-error\/80 {
    border-top-color: hsl(var(--er) / 0.8)}
.border-t-error\/90 {
    border-top-color: hsl(var(--er) / 0.9)}
.border-t-error\/95 {
    border-top-color: hsl(var(--er) / 0.95)}
.border-t-info {
    border-top-color: hsl(var(--in) / 1)}
.border-t-info-content {
    border-top-color: hsl(var(--inc) / 1)}
.border-t-info-content\/0 {
    border-top-color: hsl(var(--inc) / 0)}
.border-t-info-content\/10 {
    border-top-color: hsl(var(--inc) / 0.1)}
.border-t-info-content\/100 {
    border-top-color: hsl(var(--inc) / 1)}
.border-t-info-content\/20 {
    border-top-color: hsl(var(--inc) / 0.2)}
.border-t-info-content\/25 {
    border-top-color: hsl(var(--inc) / 0.25)}
.border-t-info-content\/30 {
    border-top-color: hsl(var(--inc) / 0.3)}
.border-t-info-content\/40 {
    border-top-color: hsl(var(--inc) / 0.4)}
.border-t-info-content\/5 {
    border-top-color: hsl(var(--inc) / 0.05)}
.border-t-info-content\/50 {
    border-top-color: hsl(var(--inc) / 0.5)}
.border-t-info-content\/60 {
    border-top-color: hsl(var(--inc) / 0.6)}
.border-t-info-content\/70 {
    border-top-color: hsl(var(--inc) / 0.7)}
.border-t-info-content\/75 {
    border-top-color: hsl(var(--inc) / 0.75)}
.border-t-info-content\/80 {
    border-top-color: hsl(var(--inc) / 0.8)}
.border-t-info-content\/90 {
    border-top-color: hsl(var(--inc) / 0.9)}
.border-t-info-content\/95 {
    border-top-color: hsl(var(--inc) / 0.95)}
.border-t-info\/0 {
    border-top-color: hsl(var(--in) / 0)}
.border-t-info\/10 {
    border-top-color: hsl(var(--in) / 0.1)}
.border-t-info\/100 {
    border-top-color: hsl(var(--in) / 1)}
.border-t-info\/20 {
    border-top-color: hsl(var(--in) / 0.2)}
.border-t-info\/25 {
    border-top-color: hsl(var(--in) / 0.25)}
.border-t-info\/30 {
    border-top-color: hsl(var(--in) / 0.3)}
.border-t-info\/40 {
    border-top-color: hsl(var(--in) / 0.4)}
.border-t-info\/5 {
    border-top-color: hsl(var(--in) / 0.05)}
.border-t-info\/50 {
    border-top-color: hsl(var(--in) / 0.5)}
.border-t-info\/60 {
    border-top-color: hsl(var(--in) / 0.6)}
.border-t-info\/70 {
    border-top-color: hsl(var(--in) / 0.7)}
.border-t-info\/75 {
    border-top-color: hsl(var(--in) / 0.75)}
.border-t-info\/80 {
    border-top-color: hsl(var(--in) / 0.8)}
.border-t-info\/90 {
    border-top-color: hsl(var(--in) / 0.9)}
.border-t-info\/95 {
    border-top-color: hsl(var(--in) / 0.95)}
.border-t-neutral {
    border-top-color: hsl(var(--n) / 1)}
.border-t-neutral-content {
    border-top-color: hsl(var(--nc) / 1)}
.border-t-neutral-content\/0 {
    border-top-color: hsl(var(--nc) / 0)}
.border-t-neutral-content\/10 {
    border-top-color: hsl(var(--nc) / 0.1)}
.border-t-neutral-content\/100 {
    border-top-color: hsl(var(--nc) / 1)}
.border-t-neutral-content\/20 {
    border-top-color: hsl(var(--nc) / 0.2)}
.border-t-neutral-content\/25 {
    border-top-color: hsl(var(--nc) / 0.25)}
.border-t-neutral-content\/30 {
    border-top-color: hsl(var(--nc) / 0.3)}
.border-t-neutral-content\/40 {
    border-top-color: hsl(var(--nc) / 0.4)}
.border-t-neutral-content\/5 {
    border-top-color: hsl(var(--nc) / 0.05)}
.border-t-neutral-content\/50 {
    border-top-color: hsl(var(--nc) / 0.5)}
.border-t-neutral-content\/60 {
    border-top-color: hsl(var(--nc) / 0.6)}
.border-t-neutral-content\/70 {
    border-top-color: hsl(var(--nc) / 0.7)}
.border-t-neutral-content\/75 {
    border-top-color: hsl(var(--nc) / 0.75)}
.border-t-neutral-content\/80 {
    border-top-color: hsl(var(--nc) / 0.8)}
.border-t-neutral-content\/90 {
    border-top-color: hsl(var(--nc) / 0.9)}
.border-t-neutral-content\/95 {
    border-top-color: hsl(var(--nc) / 0.95)}
.border-t-neutral-focus {
    border-top-color: hsl(var(--nf) / 1)}
.border-t-neutral-focus\/0 {
    border-top-color: hsl(var(--nf) / 0)}
.border-t-neutral-focus\/10 {
    border-top-color: hsl(var(--nf) / 0.1)}
.border-t-neutral-focus\/100 {
    border-top-color: hsl(var(--nf) / 1)}
.border-t-neutral-focus\/20 {
    border-top-color: hsl(var(--nf) / 0.2)}
.border-t-neutral-focus\/25 {
    border-top-color: hsl(var(--nf) / 0.25)}
.border-t-neutral-focus\/30 {
    border-top-color: hsl(var(--nf) / 0.3)}
.border-t-neutral-focus\/40 {
    border-top-color: hsl(var(--nf) / 0.4)}
.border-t-neutral-focus\/5 {
    border-top-color: hsl(var(--nf) / 0.05)}
.border-t-neutral-focus\/50 {
    border-top-color: hsl(var(--nf) / 0.5)}
.border-t-neutral-focus\/60 {
    border-top-color: hsl(var(--nf) / 0.6)}
.border-t-neutral-focus\/70 {
    border-top-color: hsl(var(--nf) / 0.7)}
.border-t-neutral-focus\/75 {
    border-top-color: hsl(var(--nf) / 0.75)}
.border-t-neutral-focus\/80 {
    border-top-color: hsl(var(--nf) / 0.8)}
.border-t-neutral-focus\/90 {
    border-top-color: hsl(var(--nf) / 0.9)}
.border-t-neutral-focus\/95 {
    border-top-color: hsl(var(--nf) / 0.95)}
.border-t-neutral\/0 {
    border-top-color: hsl(var(--n) / 0)}
.border-t-neutral\/10 {
    border-top-color: hsl(var(--n) / 0.1)}
.border-t-neutral\/100 {
    border-top-color: hsl(var(--n) / 1)}
.border-t-neutral\/20 {
    border-top-color: hsl(var(--n) / 0.2)}
.border-t-neutral\/25 {
    border-top-color: hsl(var(--n) / 0.25)}
.border-t-neutral\/30 {
    border-top-color: hsl(var(--n) / 0.3)}
.border-t-neutral\/40 {
    border-top-color: hsl(var(--n) / 0.4)}
.border-t-neutral\/5 {
    border-top-color: hsl(var(--n) / 0.05)}
.border-t-neutral\/50 {
    border-top-color: hsl(var(--n) / 0.5)}
.border-t-neutral\/60 {
    border-top-color: hsl(var(--n) / 0.6)}
.border-t-neutral\/70 {
    border-top-color: hsl(var(--n) / 0.7)}
.border-t-neutral\/75 {
    border-top-color: hsl(var(--n) / 0.75)}
.border-t-neutral\/80 {
    border-top-color: hsl(var(--n) / 0.8)}
.border-t-neutral\/90 {
    border-top-color: hsl(var(--n) / 0.9)}
.border-t-neutral\/95 {
    border-top-color: hsl(var(--n) / 0.95)}
.border-t-primary {
    border-top-color: hsl(var(--p) / 1)}
.border-t-primary-content {
    border-top-color: hsl(var(--pc) / 1)}
.border-t-primary-content\/0 {
    border-top-color: hsl(var(--pc) / 0)}
.border-t-primary-content\/10 {
    border-top-color: hsl(var(--pc) / 0.1)}
.border-t-primary-content\/100 {
    border-top-color: hsl(var(--pc) / 1)}
.border-t-primary-content\/20 {
    border-top-color: hsl(var(--pc) / 0.2)}
.border-t-primary-content\/25 {
    border-top-color: hsl(var(--pc) / 0.25)}
.border-t-primary-content\/30 {
    border-top-color: hsl(var(--pc) / 0.3)}
.border-t-primary-content\/40 {
    border-top-color: hsl(var(--pc) / 0.4)}
.border-t-primary-content\/5 {
    border-top-color: hsl(var(--pc) / 0.05)}
.border-t-primary-content\/50 {
    border-top-color: hsl(var(--pc) / 0.5)}
.border-t-primary-content\/60 {
    border-top-color: hsl(var(--pc) / 0.6)}
.border-t-primary-content\/70 {
    border-top-color: hsl(var(--pc) / 0.7)}
.border-t-primary-content\/75 {
    border-top-color: hsl(var(--pc) / 0.75)}
.border-t-primary-content\/80 {
    border-top-color: hsl(var(--pc) / 0.8)}
.border-t-primary-content\/90 {
    border-top-color: hsl(var(--pc) / 0.9)}
.border-t-primary-content\/95 {
    border-top-color: hsl(var(--pc) / 0.95)}
.border-t-primary-focus {
    border-top-color: hsl(var(--pf) / 1)}
.border-t-primary-focus\/0 {
    border-top-color: hsl(var(--pf) / 0)}
.border-t-primary-focus\/10 {
    border-top-color: hsl(var(--pf) / 0.1)}
.border-t-primary-focus\/100 {
    border-top-color: hsl(var(--pf) / 1)}
.border-t-primary-focus\/20 {
    border-top-color: hsl(var(--pf) / 0.2)}
.border-t-primary-focus\/25 {
    border-top-color: hsl(var(--pf) / 0.25)}
.border-t-primary-focus\/30 {
    border-top-color: hsl(var(--pf) / 0.3)}
.border-t-primary-focus\/40 {
    border-top-color: hsl(var(--pf) / 0.4)}
.border-t-primary-focus\/5 {
    border-top-color: hsl(var(--pf) / 0.05)}
.border-t-primary-focus\/50 {
    border-top-color: hsl(var(--pf) / 0.5)}
.border-t-primary-focus\/60 {
    border-top-color: hsl(var(--pf) / 0.6)}
.border-t-primary-focus\/70 {
    border-top-color: hsl(var(--pf) / 0.7)}
.border-t-primary-focus\/75 {
    border-top-color: hsl(var(--pf) / 0.75)}
.border-t-primary-focus\/80 {
    border-top-color: hsl(var(--pf) / 0.8)}
.border-t-primary-focus\/90 {
    border-top-color: hsl(var(--pf) / 0.9)}
.border-t-primary-focus\/95 {
    border-top-color: hsl(var(--pf) / 0.95)}
.border-t-primary\/0 {
    border-top-color: hsl(var(--p) / 0)}
.border-t-primary\/10 {
    border-top-color: hsl(var(--p) / 0.1)}
.border-t-primary\/100 {
    border-top-color: hsl(var(--p) / 1)}
.border-t-primary\/20 {
    border-top-color: hsl(var(--p) / 0.2)}
.border-t-primary\/25 {
    border-top-color: hsl(var(--p) / 0.25)}
.border-t-primary\/30 {
    border-top-color: hsl(var(--p) / 0.3)}
.border-t-primary\/40 {
    border-top-color: hsl(var(--p) / 0.4)}
.border-t-primary\/5 {
    border-top-color: hsl(var(--p) / 0.05)}
.border-t-primary\/50 {
    border-top-color: hsl(var(--p) / 0.5)}
.border-t-primary\/60 {
    border-top-color: hsl(var(--p) / 0.6)}
.border-t-primary\/70 {
    border-top-color: hsl(var(--p) / 0.7)}
.border-t-primary\/75 {
    border-top-color: hsl(var(--p) / 0.75)}
.border-t-primary\/80 {
    border-top-color: hsl(var(--p) / 0.8)}
.border-t-primary\/90 {
    border-top-color: hsl(var(--p) / 0.9)}
.border-t-primary\/95 {
    border-top-color: hsl(var(--p) / 0.95)}
.border-t-secondary {
    border-top-color: hsl(var(--s) / 1)}
.border-t-secondary-content {
    border-top-color: hsl(var(--sc) / 1)}
.border-t-secondary-content\/0 {
    border-top-color: hsl(var(--sc) / 0)}
.border-t-secondary-content\/10 {
    border-top-color: hsl(var(--sc) / 0.1)}
.border-t-secondary-content\/100 {
    border-top-color: hsl(var(--sc) / 1)}
.border-t-secondary-content\/20 {
    border-top-color: hsl(var(--sc) / 0.2)}
.border-t-secondary-content\/25 {
    border-top-color: hsl(var(--sc) / 0.25)}
.border-t-secondary-content\/30 {
    border-top-color: hsl(var(--sc) / 0.3)}
.border-t-secondary-content\/40 {
    border-top-color: hsl(var(--sc) / 0.4)}
.border-t-secondary-content\/5 {
    border-top-color: hsl(var(--sc) / 0.05)}
.border-t-secondary-content\/50 {
    border-top-color: hsl(var(--sc) / 0.5)}
.border-t-secondary-content\/60 {
    border-top-color: hsl(var(--sc) / 0.6)}
.border-t-secondary-content\/70 {
    border-top-color: hsl(var(--sc) / 0.7)}
.border-t-secondary-content\/75 {
    border-top-color: hsl(var(--sc) / 0.75)}
.border-t-secondary-content\/80 {
    border-top-color: hsl(var(--sc) / 0.8)}
.border-t-secondary-content\/90 {
    border-top-color: hsl(var(--sc) / 0.9)}
.border-t-secondary-content\/95 {
    border-top-color: hsl(var(--sc) / 0.95)}
.border-t-secondary-focus {
    border-top-color: hsl(var(--sf) / 1)}
.border-t-secondary-focus\/0 {
    border-top-color: hsl(var(--sf) / 0)}
.border-t-secondary-focus\/10 {
    border-top-color: hsl(var(--sf) / 0.1)}
.border-t-secondary-focus\/100 {
    border-top-color: hsl(var(--sf) / 1)}
.border-t-secondary-focus\/20 {
    border-top-color: hsl(var(--sf) / 0.2)}
.border-t-secondary-focus\/25 {
    border-top-color: hsl(var(--sf) / 0.25)}
.border-t-secondary-focus\/30 {
    border-top-color: hsl(var(--sf) / 0.3)}
.border-t-secondary-focus\/40 {
    border-top-color: hsl(var(--sf) / 0.4)}
.border-t-secondary-focus\/5 {
    border-top-color: hsl(var(--sf) / 0.05)}
.border-t-secondary-focus\/50 {
    border-top-color: hsl(var(--sf) / 0.5)}
.border-t-secondary-focus\/60 {
    border-top-color: hsl(var(--sf) / 0.6)}
.border-t-secondary-focus\/70 {
    border-top-color: hsl(var(--sf) / 0.7)}
.border-t-secondary-focus\/75 {
    border-top-color: hsl(var(--sf) / 0.75)}
.border-t-secondary-focus\/80 {
    border-top-color: hsl(var(--sf) / 0.8)}
.border-t-secondary-focus\/90 {
    border-top-color: hsl(var(--sf) / 0.9)}
.border-t-secondary-focus\/95 {
    border-top-color: hsl(var(--sf) / 0.95)}
.border-t-secondary\/0 {
    border-top-color: hsl(var(--s) / 0)}
.border-t-secondary\/10 {
    border-top-color: hsl(var(--s) / 0.1)}
.border-t-secondary\/100 {
    border-top-color: hsl(var(--s) / 1)}
.border-t-secondary\/20 {
    border-top-color: hsl(var(--s) / 0.2)}
.border-t-secondary\/25 {
    border-top-color: hsl(var(--s) / 0.25)}
.border-t-secondary\/30 {
    border-top-color: hsl(var(--s) / 0.3)}
.border-t-secondary\/40 {
    border-top-color: hsl(var(--s) / 0.4)}
.border-t-secondary\/5 {
    border-top-color: hsl(var(--s) / 0.05)}
.border-t-secondary\/50 {
    border-top-color: hsl(var(--s) / 0.5)}
.border-t-secondary\/60 {
    border-top-color: hsl(var(--s) / 0.6)}
.border-t-secondary\/70 {
    border-top-color: hsl(var(--s) / 0.7)}
.border-t-secondary\/75 {
    border-top-color: hsl(var(--s) / 0.75)}
.border-t-secondary\/80 {
    border-top-color: hsl(var(--s) / 0.8)}
.border-t-secondary\/90 {
    border-top-color: hsl(var(--s) / 0.9)}
.border-t-secondary\/95 {
    border-top-color: hsl(var(--s) / 0.95)}
.border-t-success {
    border-top-color: hsl(var(--su) / 1)}
.border-t-success-content {
    border-top-color: hsl(var(--suc) / 1)}
.border-t-success-content\/0 {
    border-top-color: hsl(var(--suc) / 0)}
.border-t-success-content\/10 {
    border-top-color: hsl(var(--suc) / 0.1)}
.border-t-success-content\/100 {
    border-top-color: hsl(var(--suc) / 1)}
.border-t-success-content\/20 {
    border-top-color: hsl(var(--suc) / 0.2)}
.border-t-success-content\/25 {
    border-top-color: hsl(var(--suc) / 0.25)}
.border-t-success-content\/30 {
    border-top-color: hsl(var(--suc) / 0.3)}
.border-t-success-content\/40 {
    border-top-color: hsl(var(--suc) / 0.4)}
.border-t-success-content\/5 {
    border-top-color: hsl(var(--suc) / 0.05)}
.border-t-success-content\/50 {
    border-top-color: hsl(var(--suc) / 0.5)}
.border-t-success-content\/60 {
    border-top-color: hsl(var(--suc) / 0.6)}
.border-t-success-content\/70 {
    border-top-color: hsl(var(--suc) / 0.7)}
.border-t-success-content\/75 {
    border-top-color: hsl(var(--suc) / 0.75)}
.border-t-success-content\/80 {
    border-top-color: hsl(var(--suc) / 0.8)}
.border-t-success-content\/90 {
    border-top-color: hsl(var(--suc) / 0.9)}
.border-t-success-content\/95 {
    border-top-color: hsl(var(--suc) / 0.95)}
.border-t-success\/0 {
    border-top-color: hsl(var(--su) / 0)}
.border-t-success\/10 {
    border-top-color: hsl(var(--su) / 0.1)}
.border-t-success\/100 {
    border-top-color: hsl(var(--su) / 1)}
.border-t-success\/20 {
    border-top-color: hsl(var(--su) / 0.2)}
.border-t-success\/25 {
    border-top-color: hsl(var(--su) / 0.25)}
.border-t-success\/30 {
    border-top-color: hsl(var(--su) / 0.3)}
.border-t-success\/40 {
    border-top-color: hsl(var(--su) / 0.4)}
.border-t-success\/5 {
    border-top-color: hsl(var(--su) / 0.05)}
.border-t-success\/50 {
    border-top-color: hsl(var(--su) / 0.5)}
.border-t-success\/60 {
    border-top-color: hsl(var(--su) / 0.6)}
.border-t-success\/70 {
    border-top-color: hsl(var(--su) / 0.7)}
.border-t-success\/75 {
    border-top-color: hsl(var(--su) / 0.75)}
.border-t-success\/80 {
    border-top-color: hsl(var(--su) / 0.8)}
.border-t-success\/90 {
    border-top-color: hsl(var(--su) / 0.9)}
.border-t-success\/95 {
    border-top-color: hsl(var(--su) / 0.95)}
.border-t-transparent {
    border-top-color: transparent}
.border-t-transparent\/0 {
    border-top-color: rgb(0 0 0 / 0)}
.border-t-transparent\/10 {
    border-top-color: rgb(0 0 0 / 0.1)}
.border-t-transparent\/100 {
    border-top-color: rgb(0 0 0 / 1)}
.border-t-transparent\/20 {
    border-top-color: rgb(0 0 0 / 0.2)}
.border-t-transparent\/25 {
    border-top-color: rgb(0 0 0 / 0.25)}
.border-t-transparent\/30 {
    border-top-color: rgb(0 0 0 / 0.3)}
.border-t-transparent\/40 {
    border-top-color: rgb(0 0 0 / 0.4)}
.border-t-transparent\/5 {
    border-top-color: rgb(0 0 0 / 0.05)}
.border-t-transparent\/50 {
    border-top-color: rgb(0 0 0 / 0.5)}
.border-t-transparent\/60 {
    border-top-color: rgb(0 0 0 / 0.6)}
.border-t-transparent\/70 {
    border-top-color: rgb(0 0 0 / 0.7)}
.border-t-transparent\/75 {
    border-top-color: rgb(0 0 0 / 0.75)}
.border-t-transparent\/80 {
    border-top-color: rgb(0 0 0 / 0.8)}
.border-t-transparent\/90 {
    border-top-color: rgb(0 0 0 / 0.9)}
.border-t-transparent\/95 {
    border-top-color: rgb(0 0 0 / 0.95)}
.border-t-warning {
    border-top-color: hsl(var(--wa) / 1)}
.border-t-warning-content {
    border-top-color: hsl(var(--wac) / 1)}
.border-t-warning-content\/0 {
    border-top-color: hsl(var(--wac) / 0)}
.border-t-warning-content\/10 {
    border-top-color: hsl(var(--wac) / 0.1)}
.border-t-warning-content\/100 {
    border-top-color: hsl(var(--wac) / 1)}
.border-t-warning-content\/20 {
    border-top-color: hsl(var(--wac) / 0.2)}
.border-t-warning-content\/25 {
    border-top-color: hsl(var(--wac) / 0.25)}
.border-t-warning-content\/30 {
    border-top-color: hsl(var(--wac) / 0.3)}
.border-t-warning-content\/40 {
    border-top-color: hsl(var(--wac) / 0.4)}
.border-t-warning-content\/5 {
    border-top-color: hsl(var(--wac) / 0.05)}
.border-t-warning-content\/50 {
    border-top-color: hsl(var(--wac) / 0.5)}
.border-t-warning-content\/60 {
    border-top-color: hsl(var(--wac) / 0.6)}
.border-t-warning-content\/70 {
    border-top-color: hsl(var(--wac) / 0.7)}
.border-t-warning-content\/75 {
    border-top-color: hsl(var(--wac) / 0.75)}
.border-t-warning-content\/80 {
    border-top-color: hsl(var(--wac) / 0.8)}
.border-t-warning-content\/90 {
    border-top-color: hsl(var(--wac) / 0.9)}
.border-t-warning-content\/95 {
    border-top-color: hsl(var(--wac) / 0.95)}
.border-t-warning\/0 {
    border-top-color: hsl(var(--wa) / 0)}
.border-t-warning\/10 {
    border-top-color: hsl(var(--wa) / 0.1)}
.border-t-warning\/100 {
    border-top-color: hsl(var(--wa) / 1)}
.border-t-warning\/20 {
    border-top-color: hsl(var(--wa) / 0.2)}
.border-t-warning\/25 {
    border-top-color: hsl(var(--wa) / 0.25)}
.border-t-warning\/30 {
    border-top-color: hsl(var(--wa) / 0.3)}
.border-t-warning\/40 {
    border-top-color: hsl(var(--wa) / 0.4)}
.border-t-warning\/5 {
    border-top-color: hsl(var(--wa) / 0.05)}
.border-t-warning\/50 {
    border-top-color: hsl(var(--wa) / 0.5)}
.border-t-warning\/60 {
    border-top-color: hsl(var(--wa) / 0.6)}
.border-t-warning\/70 {
    border-top-color: hsl(var(--wa) / 0.7)}
.border-t-warning\/75 {
    border-top-color: hsl(var(--wa) / 0.75)}
.border-t-warning\/80 {
    border-top-color: hsl(var(--wa) / 0.8)}
.border-t-warning\/90 {
    border-top-color: hsl(var(--wa) / 0.9)}
.border-t-warning\/95 {
    border-top-color: hsl(var(--wa) / 0.95)}
.bg-accent {
    background-color: hsl(var(--a) / 1)}
.bg-accent-content {
    background-color: hsl(var(--ac) / 1)}
.bg-accent-content\/0 {
    background-color: hsl(var(--ac) / 0)}
.bg-accent-content\/10 {
    background-color: hsl(var(--ac) / 0.1)}
.bg-accent-content\/100 {
    background-color: hsl(var(--ac) / 1)}
.bg-accent-content\/20 {
    background-color: hsl(var(--ac) / 0.2)}
.bg-accent-content\/25 {
    background-color: hsl(var(--ac) / 0.25)}
.bg-accent-content\/30 {
    background-color: hsl(var(--ac) / 0.3)}
.bg-accent-content\/40 {
    background-color: hsl(var(--ac) / 0.4)}
.bg-accent-content\/5 {
    background-color: hsl(var(--ac) / 0.05)}
.bg-accent-content\/50 {
    background-color: hsl(var(--ac) / 0.5)}
.bg-accent-content\/60 {
    background-color: hsl(var(--ac) / 0.6)}
.bg-accent-content\/70 {
    background-color: hsl(var(--ac) / 0.7)}
.bg-accent-content\/75 {
    background-color: hsl(var(--ac) / 0.75)}
.bg-accent-content\/80 {
    background-color: hsl(var(--ac) / 0.8)}
.bg-accent-content\/90 {
    background-color: hsl(var(--ac) / 0.9)}
.bg-accent-content\/95 {
    background-color: hsl(var(--ac) / 0.95)}
.bg-accent-focus {
    background-color: hsl(var(--af) / 1)}
.bg-accent-focus\/0 {
    background-color: hsl(var(--af) / 0)}
.bg-accent-focus\/10 {
    background-color: hsl(var(--af) / 0.1)}
.bg-accent-focus\/100 {
    background-color: hsl(var(--af) / 1)}
.bg-accent-focus\/20 {
    background-color: hsl(var(--af) / 0.2)}
.bg-accent-focus\/25 {
    background-color: hsl(var(--af) / 0.25)}
.bg-accent-focus\/30 {
    background-color: hsl(var(--af) / 0.3)}
.bg-accent-focus\/40 {
    background-color: hsl(var(--af) / 0.4)}
.bg-accent-focus\/5 {
    background-color: hsl(var(--af) / 0.05)}
.bg-accent-focus\/50 {
    background-color: hsl(var(--af) / 0.5)}
.bg-accent-focus\/60 {
    background-color: hsl(var(--af) / 0.6)}
.bg-accent-focus\/70 {
    background-color: hsl(var(--af) / 0.7)}
.bg-accent-focus\/75 {
    background-color: hsl(var(--af) / 0.75)}
.bg-accent-focus\/80 {
    background-color: hsl(var(--af) / 0.8)}
.bg-accent-focus\/90 {
    background-color: hsl(var(--af) / 0.9)}
.bg-accent-focus\/95 {
    background-color: hsl(var(--af) / 0.95)}
.bg-accent\/0 {
    background-color: hsl(var(--a) / 0)}
.bg-accent\/10 {
    background-color: hsl(var(--a) / 0.1)}
.bg-accent\/100 {
    background-color: hsl(var(--a) / 1)}
.bg-accent\/20 {
    background-color: hsl(var(--a) / 0.2)}
.bg-accent\/25 {
    background-color: hsl(var(--a) / 0.25)}
.bg-accent\/30 {
    background-color: hsl(var(--a) / 0.3)}
.bg-accent\/40 {
    background-color: hsl(var(--a) / 0.4)}
.bg-accent\/5 {
    background-color: hsl(var(--a) / 0.05)}
.bg-accent\/50 {
    background-color: hsl(var(--a) / 0.5)}
.bg-accent\/60 {
    background-color: hsl(var(--a) / 0.6)}
.bg-accent\/70 {
    background-color: hsl(var(--a) / 0.7)}
.bg-accent\/75 {
    background-color: hsl(var(--a) / 0.75)}
.bg-accent\/80 {
    background-color: hsl(var(--a) / 0.8)}
.bg-accent\/90 {
    background-color: hsl(var(--a) / 0.9)}
.bg-accent\/95 {
    background-color: hsl(var(--a) / 0.95)}
.bg-base-100 {
    background-color: hsl(var(--b1) / 1)}
.bg-base-100\/0 {
    background-color: hsl(var(--b1) / 0)}
.bg-base-100\/10 {
    background-color: hsl(var(--b1) / 0.1)}
.bg-base-100\/100 {
    background-color: hsl(var(--b1) / 1)}
.bg-base-100\/20 {
    background-color: hsl(var(--b1) / 0.2)}
.bg-base-100\/25 {
    background-color: hsl(var(--b1) / 0.25)}
.bg-base-100\/30 {
    background-color: hsl(var(--b1) / 0.3)}
.bg-base-100\/40 {
    background-color: hsl(var(--b1) / 0.4)}
.bg-base-100\/5 {
    background-color: hsl(var(--b1) / 0.05)}
.bg-base-100\/50 {
    background-color: hsl(var(--b1) / 0.5)}
.bg-base-100\/60 {
    background-color: hsl(var(--b1) / 0.6)}
.bg-base-100\/70 {
    background-color: hsl(var(--b1) / 0.7)}
.bg-base-100\/75 {
    background-color: hsl(var(--b1) / 0.75)}
.bg-base-100\/80 {
    background-color: hsl(var(--b1) / 0.8)}
.bg-base-100\/90 {
    background-color: hsl(var(--b1) / 0.9)}
.bg-base-100\/95 {
    background-color: hsl(var(--b1) / 0.95)}
.bg-base-200 {
    background-color: hsl(var(--b2) / 1)}
.bg-base-200\/0 {
    background-color: hsl(var(--b2) / 0)}
.bg-base-200\/10 {
    background-color: hsl(var(--b2) / 0.1)}
.bg-base-200\/100 {
    background-color: hsl(var(--b2) / 1)}
.bg-base-200\/20 {
    background-color: hsl(var(--b2) / 0.2)}
.bg-base-200\/25 {
    background-color: hsl(var(--b2) / 0.25)}
.bg-base-200\/30 {
    background-color: hsl(var(--b2) / 0.3)}
.bg-base-200\/40 {
    background-color: hsl(var(--b2) / 0.4)}
.bg-base-200\/5 {
    background-color: hsl(var(--b2) / 0.05)}
.bg-base-200\/50 {
    background-color: hsl(var(--b2) / 0.5)}
.bg-base-200\/60 {
    background-color: hsl(var(--b2) / 0.6)}
.bg-base-200\/70 {
    background-color: hsl(var(--b2) / 0.7)}
.bg-base-200\/75 {
    background-color: hsl(var(--b2) / 0.75)}
.bg-base-200\/80 {
    background-color: hsl(var(--b2) / 0.8)}
.bg-base-200\/90 {
    background-color: hsl(var(--b2) / 0.9)}
.bg-base-200\/95 {
    background-color: hsl(var(--b2) / 0.95)}
.bg-base-300 {
    background-color: hsl(var(--b3) / 1)}
.bg-base-300\/0 {
    background-color: hsl(var(--b3) / 0)}
.bg-base-300\/10 {
    background-color: hsl(var(--b3) / 0.1)}
.bg-base-300\/100 {
    background-color: hsl(var(--b3) / 1)}
.bg-base-300\/20 {
    background-color: hsl(var(--b3) / 0.2)}
.bg-base-300\/25 {
    background-color: hsl(var(--b3) / 0.25)}
.bg-base-300\/30 {
    background-color: hsl(var(--b3) / 0.3)}
.bg-base-300\/40 {
    background-color: hsl(var(--b3) / 0.4)}
.bg-base-300\/5 {
    background-color: hsl(var(--b3) / 0.05)}
.bg-base-300\/50 {
    background-color: hsl(var(--b3) / 0.5)}
.bg-base-300\/60 {
    background-color: hsl(var(--b3) / 0.6)}
.bg-base-300\/70 {
    background-color: hsl(var(--b3) / 0.7)}
.bg-base-300\/75 {
    background-color: hsl(var(--b3) / 0.75)}
.bg-base-300\/80 {
    background-color: hsl(var(--b3) / 0.8)}
.bg-base-300\/90 {
    background-color: hsl(var(--b3) / 0.9)}
.bg-base-300\/95 {
    background-color: hsl(var(--b3) / 0.95)}
.bg-base-content {
    background-color: hsl(var(--bc) / 1)}
.bg-base-content\/0 {
    background-color: hsl(var(--bc) / 0)}
.bg-base-content\/10 {
    background-color: hsl(var(--bc) / 0.1)}
.bg-base-content\/100 {
    background-color: hsl(var(--bc) / 1)}
.bg-base-content\/20 {
    background-color: hsl(var(--bc) / 0.2)}
.bg-base-content\/25 {
    background-color: hsl(var(--bc) / 0.25)}
.bg-base-content\/30 {
    background-color: hsl(var(--bc) / 0.3)}
.bg-base-content\/40 {
    background-color: hsl(var(--bc) / 0.4)}
.bg-base-content\/5 {
    background-color: hsl(var(--bc) / 0.05)}
.bg-base-content\/50 {
    background-color: hsl(var(--bc) / 0.5)}
.bg-base-content\/60 {
    background-color: hsl(var(--bc) / 0.6)}
.bg-base-content\/70 {
    background-color: hsl(var(--bc) / 0.7)}
.bg-base-content\/75 {
    background-color: hsl(var(--bc) / 0.75)}
.bg-base-content\/80 {
    background-color: hsl(var(--bc) / 0.8)}
.bg-base-content\/90 {
    background-color: hsl(var(--bc) / 0.9)}
.bg-base-content\/95 {
    background-color: hsl(var(--bc) / 0.95)}
.bg-current {
    background-color: currentColor}
.bg-error {
    background-color: hsl(var(--er) / 1)}
.bg-error-content {
    background-color: hsl(var(--erc) / 1)}
.bg-error-content\/0 {
    background-color: hsl(var(--erc) / 0)}
.bg-error-content\/10 {
    background-color: hsl(var(--erc) / 0.1)}
.bg-error-content\/100 {
    background-color: hsl(var(--erc) / 1)}
.bg-error-content\/20 {
    background-color: hsl(var(--erc) / 0.2)}
.bg-error-content\/25 {
    background-color: hsl(var(--erc) / 0.25)}
.bg-error-content\/30 {
    background-color: hsl(var(--erc) / 0.3)}
.bg-error-content\/40 {
    background-color: hsl(var(--erc) / 0.4)}
.bg-error-content\/5 {
    background-color: hsl(var(--erc) / 0.05)}
.bg-error-content\/50 {
    background-color: hsl(var(--erc) / 0.5)}
.bg-error-content\/60 {
    background-color: hsl(var(--erc) / 0.6)}
.bg-error-content\/70 {
    background-color: hsl(var(--erc) / 0.7)}
.bg-error-content\/75 {
    background-color: hsl(var(--erc) / 0.75)}
.bg-error-content\/80 {
    background-color: hsl(var(--erc) / 0.8)}
.bg-error-content\/90 {
    background-color: hsl(var(--erc) / 0.9)}
.bg-error-content\/95 {
    background-color: hsl(var(--erc) / 0.95)}
.bg-error\/0 {
    background-color: hsl(var(--er) / 0)}
.bg-error\/10 {
    background-color: hsl(var(--er) / 0.1)}
.bg-error\/100 {
    background-color: hsl(var(--er) / 1)}
.bg-error\/20 {
    background-color: hsl(var(--er) / 0.2)}
.bg-error\/25 {
    background-color: hsl(var(--er) / 0.25)}
.bg-error\/30 {
    background-color: hsl(var(--er) / 0.3)}
.bg-error\/40 {
    background-color: hsl(var(--er) / 0.4)}
.bg-error\/5 {
    background-color: hsl(var(--er) / 0.05)}
.bg-error\/50 {
    background-color: hsl(var(--er) / 0.5)}
.bg-error\/60 {
    background-color: hsl(var(--er) / 0.6)}
.bg-error\/70 {
    background-color: hsl(var(--er) / 0.7)}
.bg-error\/75 {
    background-color: hsl(var(--er) / 0.75)}
.bg-error\/80 {
    background-color: hsl(var(--er) / 0.8)}
.bg-error\/90 {
    background-color: hsl(var(--er) / 0.9)}
.bg-error\/95 {
    background-color: hsl(var(--er) / 0.95)}
.bg-info {
    background-color: hsl(var(--in) / 1)}
.bg-info-content {
    background-color: hsl(var(--inc) / 1)}
.bg-info-content\/0 {
    background-color: hsl(var(--inc) / 0)}
.bg-info-content\/10 {
    background-color: hsl(var(--inc) / 0.1)}
.bg-info-content\/100 {
    background-color: hsl(var(--inc) / 1)}
.bg-info-content\/20 {
    background-color: hsl(var(--inc) / 0.2)}
.bg-info-content\/25 {
    background-color: hsl(var(--inc) / 0.25)}
.bg-info-content\/30 {
    background-color: hsl(var(--inc) / 0.3)}
.bg-info-content\/40 {
    background-color: hsl(var(--inc) / 0.4)}
.bg-info-content\/5 {
    background-color: hsl(var(--inc) / 0.05)}
.bg-info-content\/50 {
    background-color: hsl(var(--inc) / 0.5)}
.bg-info-content\/60 {
    background-color: hsl(var(--inc) / 0.6)}
.bg-info-content\/70 {
    background-color: hsl(var(--inc) / 0.7)}
.bg-info-content\/75 {
    background-color: hsl(var(--inc) / 0.75)}
.bg-info-content\/80 {
    background-color: hsl(var(--inc) / 0.8)}
.bg-info-content\/90 {
    background-color: hsl(var(--inc) / 0.9)}
.bg-info-content\/95 {
    background-color: hsl(var(--inc) / 0.95)}
.bg-info\/0 {
    background-color: hsl(var(--in) / 0)}
.bg-info\/10 {
    background-color: hsl(var(--in) / 0.1)}
.bg-info\/100 {
    background-color: hsl(var(--in) / 1)}
.bg-info\/20 {
    background-color: hsl(var(--in) / 0.2)}
.bg-info\/25 {
    background-color: hsl(var(--in) / 0.25)}
.bg-info\/30 {
    background-color: hsl(var(--in) / 0.3)}
.bg-info\/40 {
    background-color: hsl(var(--in) / 0.4)}
.bg-info\/5 {
    background-color: hsl(var(--in) / 0.05)}
.bg-info\/50 {
    background-color: hsl(var(--in) / 0.5)}
.bg-info\/60 {
    background-color: hsl(var(--in) / 0.6)}
.bg-info\/70 {
    background-color: hsl(var(--in) / 0.7)}
.bg-info\/75 {
    background-color: hsl(var(--in) / 0.75)}
.bg-info\/80 {
    background-color: hsl(var(--in) / 0.8)}
.bg-info\/90 {
    background-color: hsl(var(--in) / 0.9)}
.bg-info\/95 {
    background-color: hsl(var(--in) / 0.95)}
.bg-neutral {
    background-color: hsl(var(--n) / 1)}
.bg-neutral-content {
    background-color: hsl(var(--nc) / 1)}
.bg-neutral-content\/0 {
    background-color: hsl(var(--nc) / 0)}
.bg-neutral-content\/10 {
    background-color: hsl(var(--nc) / 0.1)}
.bg-neutral-content\/100 {
    background-color: hsl(var(--nc) / 1)}
.bg-neutral-content\/20 {
    background-color: hsl(var(--nc) / 0.2)}
.bg-neutral-content\/25 {
    background-color: hsl(var(--nc) / 0.25)}
.bg-neutral-content\/30 {
    background-color: hsl(var(--nc) / 0.3)}
.bg-neutral-content\/40 {
    background-color: hsl(var(--nc) / 0.4)}
.bg-neutral-content\/5 {
    background-color: hsl(var(--nc) / 0.05)}
.bg-neutral-content\/50 {
    background-color: hsl(var(--nc) / 0.5)}
.bg-neutral-content\/60 {
    background-color: hsl(var(--nc) / 0.6)}
.bg-neutral-content\/70 {
    background-color: hsl(var(--nc) / 0.7)}
.bg-neutral-content\/75 {
    background-color: hsl(var(--nc) / 0.75)}
.bg-neutral-content\/80 {
    background-color: hsl(var(--nc) / 0.8)}
.bg-neutral-content\/90 {
    background-color: hsl(var(--nc) / 0.9)}
.bg-neutral-content\/95 {
    background-color: hsl(var(--nc) / 0.95)}
.bg-neutral-focus {
    background-color: hsl(var(--nf) / 1)}
.bg-neutral-focus\/0 {
    background-color: hsl(var(--nf) / 0)}
.bg-neutral-focus\/10 {
    background-color: hsl(var(--nf) / 0.1)}
.bg-neutral-focus\/100 {
    background-color: hsl(var(--nf) / 1)}
.bg-neutral-focus\/20 {
    background-color: hsl(var(--nf) / 0.2)}
.bg-neutral-focus\/25 {
    background-color: hsl(var(--nf) / 0.25)}
.bg-neutral-focus\/30 {
    background-color: hsl(var(--nf) / 0.3)}
.bg-neutral-focus\/40 {
    background-color: hsl(var(--nf) / 0.4)}
.bg-neutral-focus\/5 {
    background-color: hsl(var(--nf) / 0.05)}
.bg-neutral-focus\/50 {
    background-color: hsl(var(--nf) / 0.5)}
.bg-neutral-focus\/60 {
    background-color: hsl(var(--nf) / 0.6)}
.bg-neutral-focus\/70 {
    background-color: hsl(var(--nf) / 0.7)}
.bg-neutral-focus\/75 {
    background-color: hsl(var(--nf) / 0.75)}
.bg-neutral-focus\/80 {
    background-color: hsl(var(--nf) / 0.8)}
.bg-neutral-focus\/90 {
    background-color: hsl(var(--nf) / 0.9)}
.bg-neutral-focus\/95 {
    background-color: hsl(var(--nf) / 0.95)}
.bg-neutral\/0 {
    background-color: hsl(var(--n) / 0)}
.bg-neutral\/10 {
    background-color: hsl(var(--n) / 0.1)}
.bg-neutral\/100 {
    background-color: hsl(var(--n) / 1)}
.bg-neutral\/20 {
    background-color: hsl(var(--n) / 0.2)}
.bg-neutral\/25 {
    background-color: hsl(var(--n) / 0.25)}
.bg-neutral\/30 {
    background-color: hsl(var(--n) / 0.3)}
.bg-neutral\/40 {
    background-color: hsl(var(--n) / 0.4)}
.bg-neutral\/5 {
    background-color: hsl(var(--n) / 0.05)}
.bg-neutral\/50 {
    background-color: hsl(var(--n) / 0.5)}
.bg-neutral\/60 {
    background-color: hsl(var(--n) / 0.6)}
.bg-neutral\/70 {
    background-color: hsl(var(--n) / 0.7)}
.bg-neutral\/75 {
    background-color: hsl(var(--n) / 0.75)}
.bg-neutral\/80 {
    background-color: hsl(var(--n) / 0.8)}
.bg-neutral\/90 {
    background-color: hsl(var(--n) / 0.9)}
.bg-neutral\/95 {
    background-color: hsl(var(--n) / 0.95)}
.bg-primary {
    background-color: hsl(var(--p) / 1)}
.bg-primary-content {
    background-color: hsl(var(--pc) / 1)}
.bg-primary-content\/0 {
    background-color: hsl(var(--pc) / 0)}
.bg-primary-content\/10 {
    background-color: hsl(var(--pc) / 0.1)}
.bg-primary-content\/100 {
    background-color: hsl(var(--pc) / 1)}
.bg-primary-content\/20 {
    background-color: hsl(var(--pc) / 0.2)}
.bg-primary-content\/25 {
    background-color: hsl(var(--pc) / 0.25)}
.bg-primary-content\/30 {
    background-color: hsl(var(--pc) / 0.3)}
.bg-primary-content\/40 {
    background-color: hsl(var(--pc) / 0.4)}
.bg-primary-content\/5 {
    background-color: hsl(var(--pc) / 0.05)}
.bg-primary-content\/50 {
    background-color: hsl(var(--pc) / 0.5)}
.bg-primary-content\/60 {
    background-color: hsl(var(--pc) / 0.6)}
.bg-primary-content\/70 {
    background-color: hsl(var(--pc) / 0.7)}
.bg-primary-content\/75 {
    background-color: hsl(var(--pc) / 0.75)}
.bg-primary-content\/80 {
    background-color: hsl(var(--pc) / 0.8)}
.bg-primary-content\/90 {
    background-color: hsl(var(--pc) / 0.9)}
.bg-primary-content\/95 {
    background-color: hsl(var(--pc) / 0.95)}
.bg-primary-focus {
    background-color: hsl(var(--pf) / 1)}
.bg-primary-focus\/0 {
    background-color: hsl(var(--pf) / 0)}
.bg-primary-focus\/10 {
    background-color: hsl(var(--pf) / 0.1)}
.bg-primary-focus\/100 {
    background-color: hsl(var(--pf) / 1)}
.bg-primary-focus\/20 {
    background-color: hsl(var(--pf) / 0.2)}
.bg-primary-focus\/25 {
    background-color: hsl(var(--pf) / 0.25)}
.bg-primary-focus\/30 {
    background-color: hsl(var(--pf) / 0.3)}
.bg-primary-focus\/40 {
    background-color: hsl(var(--pf) / 0.4)}
.bg-primary-focus\/5 {
    background-color: hsl(var(--pf) / 0.05)}
.bg-primary-focus\/50 {
    background-color: hsl(var(--pf) / 0.5)}
.bg-primary-focus\/60 {
    background-color: hsl(var(--pf) / 0.6)}
.bg-primary-focus\/70 {
    background-color: hsl(var(--pf) / 0.7)}
.bg-primary-focus\/75 {
    background-color: hsl(var(--pf) / 0.75)}
.bg-primary-focus\/80 {
    background-color: hsl(var(--pf) / 0.8)}
.bg-primary-focus\/90 {
    background-color: hsl(var(--pf) / 0.9)}
.bg-primary-focus\/95 {
    background-color: hsl(var(--pf) / 0.95)}
.bg-primary\/0 {
    background-color: hsl(var(--p) / 0)}
.bg-primary\/10 {
    background-color: hsl(var(--p) / 0.1)}
.bg-primary\/100 {
    background-color: hsl(var(--p) / 1)}
.bg-primary\/20 {
    background-color: hsl(var(--p) / 0.2)}
.bg-primary\/25 {
    background-color: hsl(var(--p) / 0.25)}
.bg-primary\/30 {
    background-color: hsl(var(--p) / 0.3)}
.bg-primary\/40 {
    background-color: hsl(var(--p) / 0.4)}
.bg-primary\/5 {
    background-color: hsl(var(--p) / 0.05)}
.bg-primary\/50 {
    background-color: hsl(var(--p) / 0.5)}
.bg-primary\/60 {
    background-color: hsl(var(--p) / 0.6)}
.bg-primary\/70 {
    background-color: hsl(var(--p) / 0.7)}
.bg-primary\/75 {
    background-color: hsl(var(--p) / 0.75)}
.bg-primary\/80 {
    background-color: hsl(var(--p) / 0.8)}
.bg-primary\/90 {
    background-color: hsl(var(--p) / 0.9)}
.bg-primary\/95 {
    background-color: hsl(var(--p) / 0.95)}
.bg-secondary {
    background-color: hsl(var(--s) / 1)}
.bg-secondary-content {
    background-color: hsl(var(--sc) / 1)}
.bg-secondary-content\/0 {
    background-color: hsl(var(--sc) / 0)}
.bg-secondary-content\/10 {
    background-color: hsl(var(--sc) / 0.1)}
.bg-secondary-content\/100 {
    background-color: hsl(var(--sc) / 1)}
.bg-secondary-content\/20 {
    background-color: hsl(var(--sc) / 0.2)}
.bg-secondary-content\/25 {
    background-color: hsl(var(--sc) / 0.25)}
.bg-secondary-content\/30 {
    background-color: hsl(var(--sc) / 0.3)}
.bg-secondary-content\/40 {
    background-color: hsl(var(--sc) / 0.4)}
.bg-secondary-content\/5 {
    background-color: hsl(var(--sc) / 0.05)}
.bg-secondary-content\/50 {
    background-color: hsl(var(--sc) / 0.5)}
.bg-secondary-content\/60 {
    background-color: hsl(var(--sc) / 0.6)}
.bg-secondary-content\/70 {
    background-color: hsl(var(--sc) / 0.7)}
.bg-secondary-content\/75 {
    background-color: hsl(var(--sc) / 0.75)}
.bg-secondary-content\/80 {
    background-color: hsl(var(--sc) / 0.8)}
.bg-secondary-content\/90 {
    background-color: hsl(var(--sc) / 0.9)}
.bg-secondary-content\/95 {
    background-color: hsl(var(--sc) / 0.95)}
.bg-secondary-focus {
    background-color: hsl(var(--sf) / 1)}
.bg-secondary-focus\/0 {
    background-color: hsl(var(--sf) / 0)}
.bg-secondary-focus\/10 {
    background-color: hsl(var(--sf) / 0.1)}
.bg-secondary-focus\/100 {
    background-color: hsl(var(--sf) / 1)}
.bg-secondary-focus\/20 {
    background-color: hsl(var(--sf) / 0.2)}
.bg-secondary-focus\/25 {
    background-color: hsl(var(--sf) / 0.25)}
.bg-secondary-focus\/30 {
    background-color: hsl(var(--sf) / 0.3)}
.bg-secondary-focus\/40 {
    background-color: hsl(var(--sf) / 0.4)}
.bg-secondary-focus\/5 {
    background-color: hsl(var(--sf) / 0.05)}
.bg-secondary-focus\/50 {
    background-color: hsl(var(--sf) / 0.5)}
.bg-secondary-focus\/60 {
    background-color: hsl(var(--sf) / 0.6)}
.bg-secondary-focus\/70 {
    background-color: hsl(var(--sf) / 0.7)}
.bg-secondary-focus\/75 {
    background-color: hsl(var(--sf) / 0.75)}
.bg-secondary-focus\/80 {
    background-color: hsl(var(--sf) / 0.8)}
.bg-secondary-focus\/90 {
    background-color: hsl(var(--sf) / 0.9)}
.bg-secondary-focus\/95 {
    background-color: hsl(var(--sf) / 0.95)}
.bg-secondary\/0 {
    background-color: hsl(var(--s) / 0)}
.bg-secondary\/10 {
    background-color: hsl(var(--s) / 0.1)}
.bg-secondary\/100 {
    background-color: hsl(var(--s) / 1)}
.bg-secondary\/20 {
    background-color: hsl(var(--s) / 0.2)}
.bg-secondary\/25 {
    background-color: hsl(var(--s) / 0.25)}
.bg-secondary\/30 {
    background-color: hsl(var(--s) / 0.3)}
.bg-secondary\/40 {
    background-color: hsl(var(--s) / 0.4)}
.bg-secondary\/5 {
    background-color: hsl(var(--s) / 0.05)}
.bg-secondary\/50 {
    background-color: hsl(var(--s) / 0.5)}
.bg-secondary\/60 {
    background-color: hsl(var(--s) / 0.6)}
.bg-secondary\/70 {
    background-color: hsl(var(--s) / 0.7)}
.bg-secondary\/75 {
    background-color: hsl(var(--s) / 0.75)}
.bg-secondary\/80 {
    background-color: hsl(var(--s) / 0.8)}
.bg-secondary\/90 {
    background-color: hsl(var(--s) / 0.9)}
.bg-secondary\/95 {
    background-color: hsl(var(--s) / 0.95)}
.bg-success {
    background-color: hsl(var(--su) / 1)}
.bg-success-content {
    background-color: hsl(var(--suc) / 1)}
.bg-success-content\/0 {
    background-color: hsl(var(--suc) / 0)}
.bg-success-content\/10 {
    background-color: hsl(var(--suc) / 0.1)}
.bg-success-content\/100 {
    background-color: hsl(var(--suc) / 1)}
.bg-success-content\/20 {
    background-color: hsl(var(--suc) / 0.2)}
.bg-success-content\/25 {
    background-color: hsl(var(--suc) / 0.25)}
.bg-success-content\/30 {
    background-color: hsl(var(--suc) / 0.3)}
.bg-success-content\/40 {
    background-color: hsl(var(--suc) / 0.4)}
.bg-success-content\/5 {
    background-color: hsl(var(--suc) / 0.05)}
.bg-success-content\/50 {
    background-color: hsl(var(--suc) / 0.5)}
.bg-success-content\/60 {
    background-color: hsl(var(--suc) / 0.6)}
.bg-success-content\/70 {
    background-color: hsl(var(--suc) / 0.7)}
.bg-success-content\/75 {
    background-color: hsl(var(--suc) / 0.75)}
.bg-success-content\/80 {
    background-color: hsl(var(--suc) / 0.8)}
.bg-success-content\/90 {
    background-color: hsl(var(--suc) / 0.9)}
.bg-success-content\/95 {
    background-color: hsl(var(--suc) / 0.95)}
.bg-success\/0 {
    background-color: hsl(var(--su) / 0)}
.bg-success\/10 {
    background-color: hsl(var(--su) / 0.1)}
.bg-success\/100 {
    background-color: hsl(var(--su) / 1)}
.bg-success\/20 {
    background-color: hsl(var(--su) / 0.2)}
.bg-success\/25 {
    background-color: hsl(var(--su) / 0.25)}
.bg-success\/30 {
    background-color: hsl(var(--su) / 0.3)}
.bg-success\/40 {
    background-color: hsl(var(--su) / 0.4)}
.bg-success\/5 {
    background-color: hsl(var(--su) / 0.05)}
.bg-success\/50 {
    background-color: hsl(var(--su) / 0.5)}
.bg-success\/60 {
    background-color: hsl(var(--su) / 0.6)}
.bg-success\/70 {
    background-color: hsl(var(--su) / 0.7)}
.bg-success\/75 {
    background-color: hsl(var(--su) / 0.75)}
.bg-success\/80 {
    background-color: hsl(var(--su) / 0.8)}
.bg-success\/90 {
    background-color: hsl(var(--su) / 0.9)}
.bg-success\/95 {
    background-color: hsl(var(--su) / 0.95)}
.bg-transparent {
    background-color: transparent}
.bg-transparent\/0 {
    background-color: rgb(0 0 0 / 0)}
.bg-transparent\/10 {
    background-color: rgb(0 0 0 / 0.1)}
.bg-transparent\/100 {
    background-color: rgb(0 0 0 / 1)}
.bg-transparent\/20 {
    background-color: rgb(0 0 0 / 0.2)}
.bg-transparent\/25 {
    background-color: rgb(0 0 0 / 0.25)}
.bg-transparent\/30 {
    background-color: rgb(0 0 0 / 0.3)}
.bg-transparent\/40 {
    background-color: rgb(0 0 0 / 0.4)}
.bg-transparent\/5 {
    background-color: rgb(0 0 0 / 0.05)}
.bg-transparent\/50 {
    background-color: rgb(0 0 0 / 0.5)}
.bg-transparent\/60 {
    background-color: rgb(0 0 0 / 0.6)}
.bg-transparent\/70 {
    background-color: rgb(0 0 0 / 0.7)}
.bg-transparent\/75 {
    background-color: rgb(0 0 0 / 0.75)}
.bg-transparent\/80 {
    background-color: rgb(0 0 0 / 0.8)}
.bg-transparent\/90 {
    background-color: rgb(0 0 0 / 0.9)}
.bg-transparent\/95 {
    background-color: rgb(0 0 0 / 0.95)}
.bg-warning {
    background-color: hsl(var(--wa) / 1)}
.bg-warning-content {
    background-color: hsl(var(--wac) / 1)}
.bg-warning-content\/0 {
    background-color: hsl(var(--wac) / 0)}
.bg-warning-content\/10 {
    background-color: hsl(var(--wac) / 0.1)}
.bg-warning-content\/100 {
    background-color: hsl(var(--wac) / 1)}
.bg-warning-content\/20 {
    background-color: hsl(var(--wac) / 0.2)}
.bg-warning-content\/25 {
    background-color: hsl(var(--wac) / 0.25)}
.bg-warning-content\/30 {
    background-color: hsl(var(--wac) / 0.3)}
.bg-warning-content\/40 {
    background-color: hsl(var(--wac) / 0.4)}
.bg-warning-content\/5 {
    background-color: hsl(var(--wac) / 0.05)}
.bg-warning-content\/50 {
    background-color: hsl(var(--wac) / 0.5)}
.bg-warning-content\/60 {
    background-color: hsl(var(--wac) / 0.6)}
.bg-warning-content\/70 {
    background-color: hsl(var(--wac) / 0.7)}
.bg-warning-content\/75 {
    background-color: hsl(var(--wac) / 0.75)}
.bg-warning-content\/80 {
    background-color: hsl(var(--wac) / 0.8)}
.bg-warning-content\/90 {
    background-color: hsl(var(--wac) / 0.9)}
.bg-warning-content\/95 {
    background-color: hsl(var(--wac) / 0.95)}
.bg-warning\/0 {
    background-color: hsl(var(--wa) / 0)}
.bg-warning\/10 {
    background-color: hsl(var(--wa) / 0.1)}
.bg-warning\/100 {
    background-color: hsl(var(--wa) / 1)}
.bg-warning\/20 {
    background-color: hsl(var(--wa) / 0.2)}
.bg-warning\/25 {
    background-color: hsl(var(--wa) / 0.25)}
.bg-warning\/30 {
    background-color: hsl(var(--wa) / 0.3)}
.bg-warning\/40 {
    background-color: hsl(var(--wa) / 0.4)}
.bg-warning\/5 {
    background-color: hsl(var(--wa) / 0.05)}
.bg-warning\/50 {
    background-color: hsl(var(--wa) / 0.5)}
.bg-warning\/60 {
    background-color: hsl(var(--wa) / 0.6)}
.bg-warning\/70 {
    background-color: hsl(var(--wa) / 0.7)}
.bg-warning\/75 {
    background-color: hsl(var(--wa) / 0.75)}
.bg-warning\/80 {
    background-color: hsl(var(--wa) / 0.8)}
.bg-warning\/90 {
    background-color: hsl(var(--wa) / 0.9)}
.bg-warning\/95 {
    background-color: hsl(var(--wa) / 0.95)}
.bg-gradient-to-b {
    background-image: linear-gradient(to bottom, var(--tw-gradient-stops))}
.bg-gradient-to-bl {
    background-image: linear-gradient(to bottom left, var(--tw-gradient-stops))}
.bg-gradient-to-br {
    background-image: linear-gradient(to bottom right, var(--tw-gradient-stops))}
.bg-gradient-to-l {
    background-image: linear-gradient(to left, var(--tw-gradient-stops))}
.bg-gradient-to-r {
    background-image: linear-gradient(to right, var(--tw-gradient-stops))}
.bg-gradient-to-t {
    background-image: linear-gradient(to top, var(--tw-gradient-stops))}
.bg-gradient-to-tl {
    background-image: linear-gradient(to top left, var(--tw-gradient-stops))}
.bg-gradient-to-tr {
    background-image: linear-gradient(to top right, var(--tw-gradient-stops))}
.bg-none {
    background-image: none}
.from-accent {
    --tw-gradient-from: hsl(var(--a) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content {
    --tw-gradient-from: hsl(var(--ac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/0 {
    --tw-gradient-from: hsl(var(--ac) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/10 {
    --tw-gradient-from: hsl(var(--ac) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/100 {
    --tw-gradient-from: hsl(var(--ac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/20 {
    --tw-gradient-from: hsl(var(--ac) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/25 {
    --tw-gradient-from: hsl(var(--ac) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/30 {
    --tw-gradient-from: hsl(var(--ac) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/40 {
    --tw-gradient-from: hsl(var(--ac) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/5 {
    --tw-gradient-from: hsl(var(--ac) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/50 {
    --tw-gradient-from: hsl(var(--ac) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/60 {
    --tw-gradient-from: hsl(var(--ac) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/70 {
    --tw-gradient-from: hsl(var(--ac) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/75 {
    --tw-gradient-from: hsl(var(--ac) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/80 {
    --tw-gradient-from: hsl(var(--ac) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/90 {
    --tw-gradient-from: hsl(var(--ac) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-content\/95 {
    --tw-gradient-from: hsl(var(--ac) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus {
    --tw-gradient-from: hsl(var(--af) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/0 {
    --tw-gradient-from: hsl(var(--af) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/10 {
    --tw-gradient-from: hsl(var(--af) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/100 {
    --tw-gradient-from: hsl(var(--af) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/20 {
    --tw-gradient-from: hsl(var(--af) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/25 {
    --tw-gradient-from: hsl(var(--af) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/30 {
    --tw-gradient-from: hsl(var(--af) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/40 {
    --tw-gradient-from: hsl(var(--af) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/5 {
    --tw-gradient-from: hsl(var(--af) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/50 {
    --tw-gradient-from: hsl(var(--af) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/60 {
    --tw-gradient-from: hsl(var(--af) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/70 {
    --tw-gradient-from: hsl(var(--af) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/75 {
    --tw-gradient-from: hsl(var(--af) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/80 {
    --tw-gradient-from: hsl(var(--af) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/90 {
    --tw-gradient-from: hsl(var(--af) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent-focus\/95 {
    --tw-gradient-from: hsl(var(--af) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/0 {
    --tw-gradient-from: hsl(var(--a) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/10 {
    --tw-gradient-from: hsl(var(--a) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/100 {
    --tw-gradient-from: hsl(var(--a) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/20 {
    --tw-gradient-from: hsl(var(--a) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/25 {
    --tw-gradient-from: hsl(var(--a) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/30 {
    --tw-gradient-from: hsl(var(--a) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/40 {
    --tw-gradient-from: hsl(var(--a) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/5 {
    --tw-gradient-from: hsl(var(--a) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/50 {
    --tw-gradient-from: hsl(var(--a) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/60 {
    --tw-gradient-from: hsl(var(--a) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/70 {
    --tw-gradient-from: hsl(var(--a) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/75 {
    --tw-gradient-from: hsl(var(--a) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/80 {
    --tw-gradient-from: hsl(var(--a) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/90 {
    --tw-gradient-from: hsl(var(--a) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-accent\/95 {
    --tw-gradient-from: hsl(var(--a) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100 {
    --tw-gradient-from: hsl(var(--b1) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/0 {
    --tw-gradient-from: hsl(var(--b1) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/10 {
    --tw-gradient-from: hsl(var(--b1) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/100 {
    --tw-gradient-from: hsl(var(--b1) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/20 {
    --tw-gradient-from: hsl(var(--b1) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/25 {
    --tw-gradient-from: hsl(var(--b1) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/30 {
    --tw-gradient-from: hsl(var(--b1) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/40 {
    --tw-gradient-from: hsl(var(--b1) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/5 {
    --tw-gradient-from: hsl(var(--b1) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/50 {
    --tw-gradient-from: hsl(var(--b1) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/60 {
    --tw-gradient-from: hsl(var(--b1) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/70 {
    --tw-gradient-from: hsl(var(--b1) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/75 {
    --tw-gradient-from: hsl(var(--b1) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/80 {
    --tw-gradient-from: hsl(var(--b1) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/90 {
    --tw-gradient-from: hsl(var(--b1) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-100\/95 {
    --tw-gradient-from: hsl(var(--b1) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200 {
    --tw-gradient-from: hsl(var(--b2) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/0 {
    --tw-gradient-from: hsl(var(--b2) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/10 {
    --tw-gradient-from: hsl(var(--b2) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/100 {
    --tw-gradient-from: hsl(var(--b2) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/20 {
    --tw-gradient-from: hsl(var(--b2) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/25 {
    --tw-gradient-from: hsl(var(--b2) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/30 {
    --tw-gradient-from: hsl(var(--b2) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/40 {
    --tw-gradient-from: hsl(var(--b2) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/5 {
    --tw-gradient-from: hsl(var(--b2) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/50 {
    --tw-gradient-from: hsl(var(--b2) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/60 {
    --tw-gradient-from: hsl(var(--b2) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/70 {
    --tw-gradient-from: hsl(var(--b2) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/75 {
    --tw-gradient-from: hsl(var(--b2) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/80 {
    --tw-gradient-from: hsl(var(--b2) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/90 {
    --tw-gradient-from: hsl(var(--b2) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-200\/95 {
    --tw-gradient-from: hsl(var(--b2) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300 {
    --tw-gradient-from: hsl(var(--b3) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/0 {
    --tw-gradient-from: hsl(var(--b3) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/10 {
    --tw-gradient-from: hsl(var(--b3) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/100 {
    --tw-gradient-from: hsl(var(--b3) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/20 {
    --tw-gradient-from: hsl(var(--b3) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/25 {
    --tw-gradient-from: hsl(var(--b3) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/30 {
    --tw-gradient-from: hsl(var(--b3) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/40 {
    --tw-gradient-from: hsl(var(--b3) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/5 {
    --tw-gradient-from: hsl(var(--b3) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/50 {
    --tw-gradient-from: hsl(var(--b3) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/60 {
    --tw-gradient-from: hsl(var(--b3) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/70 {
    --tw-gradient-from: hsl(var(--b3) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/75 {
    --tw-gradient-from: hsl(var(--b3) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/80 {
    --tw-gradient-from: hsl(var(--b3) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/90 {
    --tw-gradient-from: hsl(var(--b3) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-300\/95 {
    --tw-gradient-from: hsl(var(--b3) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content {
    --tw-gradient-from: hsl(var(--bc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/0 {
    --tw-gradient-from: hsl(var(--bc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/10 {
    --tw-gradient-from: hsl(var(--bc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/100 {
    --tw-gradient-from: hsl(var(--bc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/20 {
    --tw-gradient-from: hsl(var(--bc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/25 {
    --tw-gradient-from: hsl(var(--bc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/30 {
    --tw-gradient-from: hsl(var(--bc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/40 {
    --tw-gradient-from: hsl(var(--bc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/5 {
    --tw-gradient-from: hsl(var(--bc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/50 {
    --tw-gradient-from: hsl(var(--bc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/60 {
    --tw-gradient-from: hsl(var(--bc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/70 {
    --tw-gradient-from: hsl(var(--bc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/75 {
    --tw-gradient-from: hsl(var(--bc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/80 {
    --tw-gradient-from: hsl(var(--bc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/90 {
    --tw-gradient-from: hsl(var(--bc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-base-content\/95 {
    --tw-gradient-from: hsl(var(--bc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-current {
    --tw-gradient-from: currentColor var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(255 255 255 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error {
    --tw-gradient-from: hsl(var(--er) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content {
    --tw-gradient-from: hsl(var(--erc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/0 {
    --tw-gradient-from: hsl(var(--erc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/10 {
    --tw-gradient-from: hsl(var(--erc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/100 {
    --tw-gradient-from: hsl(var(--erc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/20 {
    --tw-gradient-from: hsl(var(--erc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/25 {
    --tw-gradient-from: hsl(var(--erc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/30 {
    --tw-gradient-from: hsl(var(--erc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/40 {
    --tw-gradient-from: hsl(var(--erc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/5 {
    --tw-gradient-from: hsl(var(--erc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/50 {
    --tw-gradient-from: hsl(var(--erc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/60 {
    --tw-gradient-from: hsl(var(--erc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/70 {
    --tw-gradient-from: hsl(var(--erc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/75 {
    --tw-gradient-from: hsl(var(--erc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/80 {
    --tw-gradient-from: hsl(var(--erc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/90 {
    --tw-gradient-from: hsl(var(--erc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error-content\/95 {
    --tw-gradient-from: hsl(var(--erc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/0 {
    --tw-gradient-from: hsl(var(--er) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/10 {
    --tw-gradient-from: hsl(var(--er) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/100 {
    --tw-gradient-from: hsl(var(--er) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/20 {
    --tw-gradient-from: hsl(var(--er) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/25 {
    --tw-gradient-from: hsl(var(--er) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/30 {
    --tw-gradient-from: hsl(var(--er) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/40 {
    --tw-gradient-from: hsl(var(--er) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/5 {
    --tw-gradient-from: hsl(var(--er) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/50 {
    --tw-gradient-from: hsl(var(--er) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/60 {
    --tw-gradient-from: hsl(var(--er) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/70 {
    --tw-gradient-from: hsl(var(--er) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/75 {
    --tw-gradient-from: hsl(var(--er) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/80 {
    --tw-gradient-from: hsl(var(--er) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/90 {
    --tw-gradient-from: hsl(var(--er) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-error\/95 {
    --tw-gradient-from: hsl(var(--er) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info {
    --tw-gradient-from: hsl(var(--in) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content {
    --tw-gradient-from: hsl(var(--inc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/0 {
    --tw-gradient-from: hsl(var(--inc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/10 {
    --tw-gradient-from: hsl(var(--inc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/100 {
    --tw-gradient-from: hsl(var(--inc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/20 {
    --tw-gradient-from: hsl(var(--inc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/25 {
    --tw-gradient-from: hsl(var(--inc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/30 {
    --tw-gradient-from: hsl(var(--inc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/40 {
    --tw-gradient-from: hsl(var(--inc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/5 {
    --tw-gradient-from: hsl(var(--inc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/50 {
    --tw-gradient-from: hsl(var(--inc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/60 {
    --tw-gradient-from: hsl(var(--inc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/70 {
    --tw-gradient-from: hsl(var(--inc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/75 {
    --tw-gradient-from: hsl(var(--inc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/80 {
    --tw-gradient-from: hsl(var(--inc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/90 {
    --tw-gradient-from: hsl(var(--inc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info-content\/95 {
    --tw-gradient-from: hsl(var(--inc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/0 {
    --tw-gradient-from: hsl(var(--in) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/10 {
    --tw-gradient-from: hsl(var(--in) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/100 {
    --tw-gradient-from: hsl(var(--in) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/20 {
    --tw-gradient-from: hsl(var(--in) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/25 {
    --tw-gradient-from: hsl(var(--in) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/30 {
    --tw-gradient-from: hsl(var(--in) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/40 {
    --tw-gradient-from: hsl(var(--in) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/5 {
    --tw-gradient-from: hsl(var(--in) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/50 {
    --tw-gradient-from: hsl(var(--in) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/60 {
    --tw-gradient-from: hsl(var(--in) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/70 {
    --tw-gradient-from: hsl(var(--in) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/75 {
    --tw-gradient-from: hsl(var(--in) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/80 {
    --tw-gradient-from: hsl(var(--in) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/90 {
    --tw-gradient-from: hsl(var(--in) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-info\/95 {
    --tw-gradient-from: hsl(var(--in) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral {
    --tw-gradient-from: hsl(var(--n) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content {
    --tw-gradient-from: hsl(var(--nc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/0 {
    --tw-gradient-from: hsl(var(--nc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/10 {
    --tw-gradient-from: hsl(var(--nc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/100 {
    --tw-gradient-from: hsl(var(--nc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/20 {
    --tw-gradient-from: hsl(var(--nc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/25 {
    --tw-gradient-from: hsl(var(--nc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/30 {
    --tw-gradient-from: hsl(var(--nc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/40 {
    --tw-gradient-from: hsl(var(--nc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/5 {
    --tw-gradient-from: hsl(var(--nc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/50 {
    --tw-gradient-from: hsl(var(--nc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/60 {
    --tw-gradient-from: hsl(var(--nc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/70 {
    --tw-gradient-from: hsl(var(--nc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/75 {
    --tw-gradient-from: hsl(var(--nc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/80 {
    --tw-gradient-from: hsl(var(--nc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/90 {
    --tw-gradient-from: hsl(var(--nc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-content\/95 {
    --tw-gradient-from: hsl(var(--nc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus {
    --tw-gradient-from: hsl(var(--nf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/0 {
    --tw-gradient-from: hsl(var(--nf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/10 {
    --tw-gradient-from: hsl(var(--nf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/100 {
    --tw-gradient-from: hsl(var(--nf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/20 {
    --tw-gradient-from: hsl(var(--nf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/25 {
    --tw-gradient-from: hsl(var(--nf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/30 {
    --tw-gradient-from: hsl(var(--nf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/40 {
    --tw-gradient-from: hsl(var(--nf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/5 {
    --tw-gradient-from: hsl(var(--nf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/50 {
    --tw-gradient-from: hsl(var(--nf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/60 {
    --tw-gradient-from: hsl(var(--nf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/70 {
    --tw-gradient-from: hsl(var(--nf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/75 {
    --tw-gradient-from: hsl(var(--nf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/80 {
    --tw-gradient-from: hsl(var(--nf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/90 {
    --tw-gradient-from: hsl(var(--nf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral-focus\/95 {
    --tw-gradient-from: hsl(var(--nf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/0 {
    --tw-gradient-from: hsl(var(--n) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/10 {
    --tw-gradient-from: hsl(var(--n) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/100 {
    --tw-gradient-from: hsl(var(--n) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/20 {
    --tw-gradient-from: hsl(var(--n) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/25 {
    --tw-gradient-from: hsl(var(--n) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/30 {
    --tw-gradient-from: hsl(var(--n) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/40 {
    --tw-gradient-from: hsl(var(--n) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/5 {
    --tw-gradient-from: hsl(var(--n) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/50 {
    --tw-gradient-from: hsl(var(--n) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/60 {
    --tw-gradient-from: hsl(var(--n) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/70 {
    --tw-gradient-from: hsl(var(--n) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/75 {
    --tw-gradient-from: hsl(var(--n) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/80 {
    --tw-gradient-from: hsl(var(--n) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/90 {
    --tw-gradient-from: hsl(var(--n) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-neutral\/95 {
    --tw-gradient-from: hsl(var(--n) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary {
    --tw-gradient-from: hsl(var(--p) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content {
    --tw-gradient-from: hsl(var(--pc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/0 {
    --tw-gradient-from: hsl(var(--pc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/10 {
    --tw-gradient-from: hsl(var(--pc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/100 {
    --tw-gradient-from: hsl(var(--pc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/20 {
    --tw-gradient-from: hsl(var(--pc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/25 {
    --tw-gradient-from: hsl(var(--pc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/30 {
    --tw-gradient-from: hsl(var(--pc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/40 {
    --tw-gradient-from: hsl(var(--pc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/5 {
    --tw-gradient-from: hsl(var(--pc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/50 {
    --tw-gradient-from: hsl(var(--pc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/60 {
    --tw-gradient-from: hsl(var(--pc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/70 {
    --tw-gradient-from: hsl(var(--pc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/75 {
    --tw-gradient-from: hsl(var(--pc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/80 {
    --tw-gradient-from: hsl(var(--pc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/90 {
    --tw-gradient-from: hsl(var(--pc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-content\/95 {
    --tw-gradient-from: hsl(var(--pc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus {
    --tw-gradient-from: hsl(var(--pf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/0 {
    --tw-gradient-from: hsl(var(--pf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/10 {
    --tw-gradient-from: hsl(var(--pf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/100 {
    --tw-gradient-from: hsl(var(--pf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/20 {
    --tw-gradient-from: hsl(var(--pf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/25 {
    --tw-gradient-from: hsl(var(--pf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/30 {
    --tw-gradient-from: hsl(var(--pf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/40 {
    --tw-gradient-from: hsl(var(--pf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/5 {
    --tw-gradient-from: hsl(var(--pf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/50 {
    --tw-gradient-from: hsl(var(--pf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/60 {
    --tw-gradient-from: hsl(var(--pf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/70 {
    --tw-gradient-from: hsl(var(--pf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/75 {
    --tw-gradient-from: hsl(var(--pf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/80 {
    --tw-gradient-from: hsl(var(--pf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/90 {
    --tw-gradient-from: hsl(var(--pf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary-focus\/95 {
    --tw-gradient-from: hsl(var(--pf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/0 {
    --tw-gradient-from: hsl(var(--p) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/10 {
    --tw-gradient-from: hsl(var(--p) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/100 {
    --tw-gradient-from: hsl(var(--p) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/20 {
    --tw-gradient-from: hsl(var(--p) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/25 {
    --tw-gradient-from: hsl(var(--p) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/30 {
    --tw-gradient-from: hsl(var(--p) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/40 {
    --tw-gradient-from: hsl(var(--p) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/5 {
    --tw-gradient-from: hsl(var(--p) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/50 {
    --tw-gradient-from: hsl(var(--p) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/60 {
    --tw-gradient-from: hsl(var(--p) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/70 {
    --tw-gradient-from: hsl(var(--p) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/75 {
    --tw-gradient-from: hsl(var(--p) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/80 {
    --tw-gradient-from: hsl(var(--p) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/90 {
    --tw-gradient-from: hsl(var(--p) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-primary\/95 {
    --tw-gradient-from: hsl(var(--p) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary {
    --tw-gradient-from: hsl(var(--s) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content {
    --tw-gradient-from: hsl(var(--sc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/0 {
    --tw-gradient-from: hsl(var(--sc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/10 {
    --tw-gradient-from: hsl(var(--sc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/100 {
    --tw-gradient-from: hsl(var(--sc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/20 {
    --tw-gradient-from: hsl(var(--sc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/25 {
    --tw-gradient-from: hsl(var(--sc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/30 {
    --tw-gradient-from: hsl(var(--sc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/40 {
    --tw-gradient-from: hsl(var(--sc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/5 {
    --tw-gradient-from: hsl(var(--sc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/50 {
    --tw-gradient-from: hsl(var(--sc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/60 {
    --tw-gradient-from: hsl(var(--sc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/70 {
    --tw-gradient-from: hsl(var(--sc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/75 {
    --tw-gradient-from: hsl(var(--sc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/80 {
    --tw-gradient-from: hsl(var(--sc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/90 {
    --tw-gradient-from: hsl(var(--sc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-content\/95 {
    --tw-gradient-from: hsl(var(--sc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus {
    --tw-gradient-from: hsl(var(--sf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/0 {
    --tw-gradient-from: hsl(var(--sf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/10 {
    --tw-gradient-from: hsl(var(--sf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/100 {
    --tw-gradient-from: hsl(var(--sf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/20 {
    --tw-gradient-from: hsl(var(--sf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/25 {
    --tw-gradient-from: hsl(var(--sf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/30 {
    --tw-gradient-from: hsl(var(--sf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/40 {
    --tw-gradient-from: hsl(var(--sf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/5 {
    --tw-gradient-from: hsl(var(--sf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/50 {
    --tw-gradient-from: hsl(var(--sf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/60 {
    --tw-gradient-from: hsl(var(--sf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/70 {
    --tw-gradient-from: hsl(var(--sf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/75 {
    --tw-gradient-from: hsl(var(--sf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/80 {
    --tw-gradient-from: hsl(var(--sf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/90 {
    --tw-gradient-from: hsl(var(--sf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary-focus\/95 {
    --tw-gradient-from: hsl(var(--sf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/0 {
    --tw-gradient-from: hsl(var(--s) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/10 {
    --tw-gradient-from: hsl(var(--s) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/100 {
    --tw-gradient-from: hsl(var(--s) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/20 {
    --tw-gradient-from: hsl(var(--s) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/25 {
    --tw-gradient-from: hsl(var(--s) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/30 {
    --tw-gradient-from: hsl(var(--s) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/40 {
    --tw-gradient-from: hsl(var(--s) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/5 {
    --tw-gradient-from: hsl(var(--s) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/50 {
    --tw-gradient-from: hsl(var(--s) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/60 {
    --tw-gradient-from: hsl(var(--s) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/70 {
    --tw-gradient-from: hsl(var(--s) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/75 {
    --tw-gradient-from: hsl(var(--s) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/80 {
    --tw-gradient-from: hsl(var(--s) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/90 {
    --tw-gradient-from: hsl(var(--s) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-secondary\/95 {
    --tw-gradient-from: hsl(var(--s) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success {
    --tw-gradient-from: hsl(var(--su) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content {
    --tw-gradient-from: hsl(var(--suc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/0 {
    --tw-gradient-from: hsl(var(--suc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/10 {
    --tw-gradient-from: hsl(var(--suc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/100 {
    --tw-gradient-from: hsl(var(--suc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/20 {
    --tw-gradient-from: hsl(var(--suc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/25 {
    --tw-gradient-from: hsl(var(--suc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/30 {
    --tw-gradient-from: hsl(var(--suc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/40 {
    --tw-gradient-from: hsl(var(--suc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/5 {
    --tw-gradient-from: hsl(var(--suc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/50 {
    --tw-gradient-from: hsl(var(--suc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/60 {
    --tw-gradient-from: hsl(var(--suc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/70 {
    --tw-gradient-from: hsl(var(--suc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/75 {
    --tw-gradient-from: hsl(var(--suc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/80 {
    --tw-gradient-from: hsl(var(--suc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/90 {
    --tw-gradient-from: hsl(var(--suc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success-content\/95 {
    --tw-gradient-from: hsl(var(--suc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/0 {
    --tw-gradient-from: hsl(var(--su) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/10 {
    --tw-gradient-from: hsl(var(--su) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/100 {
    --tw-gradient-from: hsl(var(--su) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/20 {
    --tw-gradient-from: hsl(var(--su) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/25 {
    --tw-gradient-from: hsl(var(--su) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/30 {
    --tw-gradient-from: hsl(var(--su) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/40 {
    --tw-gradient-from: hsl(var(--su) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/5 {
    --tw-gradient-from: hsl(var(--su) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/50 {
    --tw-gradient-from: hsl(var(--su) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/60 {
    --tw-gradient-from: hsl(var(--su) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/70 {
    --tw-gradient-from: hsl(var(--su) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/75 {
    --tw-gradient-from: hsl(var(--su) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/80 {
    --tw-gradient-from: hsl(var(--su) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/90 {
    --tw-gradient-from: hsl(var(--su) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-success\/95 {
    --tw-gradient-from: hsl(var(--su) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent {
    --tw-gradient-from: transparent var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/0 {
    --tw-gradient-from: rgb(0 0 0 / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/10 {
    --tw-gradient-from: rgb(0 0 0 / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/100 {
    --tw-gradient-from: rgb(0 0 0 / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/20 {
    --tw-gradient-from: rgb(0 0 0 / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/25 {
    --tw-gradient-from: rgb(0 0 0 / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/30 {
    --tw-gradient-from: rgb(0 0 0 / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/40 {
    --tw-gradient-from: rgb(0 0 0 / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/5 {
    --tw-gradient-from: rgb(0 0 0 / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/50 {
    --tw-gradient-from: rgb(0 0 0 / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/60 {
    --tw-gradient-from: rgb(0 0 0 / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/70 {
    --tw-gradient-from: rgb(0 0 0 / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/75 {
    --tw-gradient-from: rgb(0 0 0 / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/80 {
    --tw-gradient-from: rgb(0 0 0 / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/90 {
    --tw-gradient-from: rgb(0 0 0 / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-transparent\/95 {
    --tw-gradient-from: rgb(0 0 0 / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning {
    --tw-gradient-from: hsl(var(--wa) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content {
    --tw-gradient-from: hsl(var(--wac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/0 {
    --tw-gradient-from: hsl(var(--wac) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/10 {
    --tw-gradient-from: hsl(var(--wac) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/100 {
    --tw-gradient-from: hsl(var(--wac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/20 {
    --tw-gradient-from: hsl(var(--wac) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/25 {
    --tw-gradient-from: hsl(var(--wac) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/30 {
    --tw-gradient-from: hsl(var(--wac) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/40 {
    --tw-gradient-from: hsl(var(--wac) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/5 {
    --tw-gradient-from: hsl(var(--wac) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/50 {
    --tw-gradient-from: hsl(var(--wac) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/60 {
    --tw-gradient-from: hsl(var(--wac) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/70 {
    --tw-gradient-from: hsl(var(--wac) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/75 {
    --tw-gradient-from: hsl(var(--wac) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/80 {
    --tw-gradient-from: hsl(var(--wac) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/90 {
    --tw-gradient-from: hsl(var(--wac) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning-content\/95 {
    --tw-gradient-from: hsl(var(--wac) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/0 {
    --tw-gradient-from: hsl(var(--wa) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/10 {
    --tw-gradient-from: hsl(var(--wa) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/100 {
    --tw-gradient-from: hsl(var(--wa) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/20 {
    --tw-gradient-from: hsl(var(--wa) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/25 {
    --tw-gradient-from: hsl(var(--wa) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/30 {
    --tw-gradient-from: hsl(var(--wa) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/40 {
    --tw-gradient-from: hsl(var(--wa) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/5 {
    --tw-gradient-from: hsl(var(--wa) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/50 {
    --tw-gradient-from: hsl(var(--wa) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/60 {
    --tw-gradient-from: hsl(var(--wa) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/70 {
    --tw-gradient-from: hsl(var(--wa) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/75 {
    --tw-gradient-from: hsl(var(--wa) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/80 {
    --tw-gradient-from: hsl(var(--wa) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/90 {
    --tw-gradient-from: hsl(var(--wa) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-warning\/95 {
    --tw-gradient-from: hsl(var(--wa) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.from-0\% {
    --tw-gradient-from-position: 0%}
.from-10\% {
    --tw-gradient-from-position: 10%}
.from-100\% {
    --tw-gradient-from-position: 100%}
.from-15\% {
    --tw-gradient-from-position: 15%}
.from-20\% {
    --tw-gradient-from-position: 20%}
.from-25\% {
    --tw-gradient-from-position: 25%}
.from-30\% {
    --tw-gradient-from-position: 30%}
.from-35\% {
    --tw-gradient-from-position: 35%}
.from-40\% {
    --tw-gradient-from-position: 40%}
.from-45\% {
    --tw-gradient-from-position: 45%}
.from-5\% {
    --tw-gradient-from-position: 5%}
.from-50\% {
    --tw-gradient-from-position: 50%}
.from-55\% {
    --tw-gradient-from-position: 55%}
.from-60\% {
    --tw-gradient-from-position: 60%}
.from-65\% {
    --tw-gradient-from-position: 65%}
.from-70\% {
    --tw-gradient-from-position: 70%}
.from-75\% {
    --tw-gradient-from-position: 75%}
.from-80\% {
    --tw-gradient-from-position: 80%}
.from-85\% {
    --tw-gradient-from-position: 85%}
.from-90\% {
    --tw-gradient-from-position: 90%}
.from-95\% {
    --tw-gradient-from-position: 95%}
.via-accent {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/0 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/10 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/100 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/20 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/25 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/30 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/40 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/5 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/50 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/60 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/70 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/75 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/80 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/90 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-content\/95 {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/0 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/10 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/100 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/20 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/25 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/30 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/40 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/5 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/50 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/60 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/70 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/75 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/80 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/90 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent-focus\/95 {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/0 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/10 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/100 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/20 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/25 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/30 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/40 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/5 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/50 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/60 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/70 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/75 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/80 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/90 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-accent\/95 {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/0 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/10 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/100 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/20 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/25 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/30 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/40 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/5 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/50 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/60 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/70 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/75 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/80 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/90 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-100\/95 {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/0 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/10 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/100 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/20 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/25 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/30 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/40 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/5 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/50 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/60 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/70 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/75 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/80 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/90 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-200\/95 {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/0 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/10 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/100 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/20 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/25 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/30 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/40 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/5 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/50 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/60 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/70 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/75 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/80 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/90 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-300\/95 {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/0 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/10 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/100 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/20 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/25 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/30 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/40 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/5 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/50 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/60 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/70 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/75 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/80 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/90 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-base-content\/95 {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-current {
    --tw-gradient-to: rgb(255 255 255 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), currentColor var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/0 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/10 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/100 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/20 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/25 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/30 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/40 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/5 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/50 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/60 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/70 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/75 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/80 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/90 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error-content\/95 {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/0 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/10 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/100 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/20 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/25 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/30 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/40 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/5 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/50 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/60 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/70 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/75 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/80 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/90 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-error\/95 {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/0 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/10 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/100 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/20 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/25 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/30 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/40 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/5 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/50 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/60 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/70 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/75 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/80 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/90 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info-content\/95 {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/0 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/10 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/100 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/20 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/25 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/30 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/40 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/5 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/50 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/60 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/70 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/75 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/80 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/90 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-info\/95 {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/0 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/10 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/100 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/20 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/25 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/30 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/40 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/5 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/50 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/60 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/70 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/75 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/80 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/90 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-content\/95 {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/0 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/10 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/100 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/20 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/25 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/30 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/40 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/5 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/50 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/60 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/70 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/75 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/80 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/90 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral-focus\/95 {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/0 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/10 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/100 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/20 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/25 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/30 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/40 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/5 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/50 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/60 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/70 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/75 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/80 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/90 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-neutral\/95 {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/0 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/10 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/100 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/20 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/25 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/30 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/40 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/5 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/50 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/60 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/70 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/75 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/80 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/90 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-content\/95 {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/0 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/10 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/100 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/20 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/25 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/30 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/40 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/5 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/50 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/60 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/70 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/75 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/80 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/90 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary-focus\/95 {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/0 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/10 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/100 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/20 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/25 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/30 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/40 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/5 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/50 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/60 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/70 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/75 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/80 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/90 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-primary\/95 {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/0 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/10 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/100 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/20 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/25 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/30 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/40 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/5 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/50 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/60 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/70 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/75 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/80 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/90 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-content\/95 {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/0 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/10 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/100 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/20 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/25 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/30 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/40 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/5 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/50 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/60 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/70 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/75 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/80 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/90 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary-focus\/95 {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/0 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/10 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/100 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/20 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/25 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/30 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/40 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/5 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/50 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/60 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/70 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/75 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/80 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/90 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-secondary\/95 {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/0 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/10 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/100 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/20 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/25 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/30 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/40 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/5 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/50 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/60 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/70 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/75 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/80 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/90 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success-content\/95 {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/0 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/10 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/100 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/20 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/25 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/30 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/40 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/5 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/50 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/60 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/70 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/75 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/80 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/90 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-success\/95 {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), transparent var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/0 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/10 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/100 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/20 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/25 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/30 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/40 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/5 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/50 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/60 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/70 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/75 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/80 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/90 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-transparent\/95 {
    --tw-gradient-to: rgb(0 0 0 / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), rgb(0 0 0 / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/0 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/10 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/100 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/20 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/25 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/30 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/40 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/5 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/50 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/60 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/70 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/75 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/80 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/90 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning-content\/95 {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/0 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/10 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/100 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/20 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/25 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/30 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/40 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/5 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/50 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/60 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/70 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/75 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/80 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/90 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-warning\/95 {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.via-0\% {
    --tw-gradient-via-position: 0%}
.via-10\% {
    --tw-gradient-via-position: 10%}
.via-100\% {
    --tw-gradient-via-position: 100%}
.via-15\% {
    --tw-gradient-via-position: 15%}
.via-20\% {
    --tw-gradient-via-position: 20%}
.via-25\% {
    --tw-gradient-via-position: 25%}
.via-30\% {
    --tw-gradient-via-position: 30%}
.via-35\% {
    --tw-gradient-via-position: 35%}
.via-40\% {
    --tw-gradient-via-position: 40%}
.via-45\% {
    --tw-gradient-via-position: 45%}
.via-5\% {
    --tw-gradient-via-position: 5%}
.via-50\% {
    --tw-gradient-via-position: 50%}
.via-55\% {
    --tw-gradient-via-position: 55%}
.via-60\% {
    --tw-gradient-via-position: 60%}
.via-65\% {
    --tw-gradient-via-position: 65%}
.via-70\% {
    --tw-gradient-via-position: 70%}
.via-75\% {
    --tw-gradient-via-position: 75%}
.via-80\% {
    --tw-gradient-via-position: 80%}
.via-85\% {
    --tw-gradient-via-position: 85%}
.via-90\% {
    --tw-gradient-via-position: 90%}
.via-95\% {
    --tw-gradient-via-position: 95%}
.to-accent {
    --tw-gradient-to: hsl(var(--a) / 1) var(--tw-gradient-to-position)}
.to-accent-content {
    --tw-gradient-to: hsl(var(--ac) / 1) var(--tw-gradient-to-position)}
.to-accent-content\/0 {
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position)}
.to-accent-content\/10 {
    --tw-gradient-to: hsl(var(--ac) / 0.1) var(--tw-gradient-to-position)}
.to-accent-content\/100 {
    --tw-gradient-to: hsl(var(--ac) / 1) var(--tw-gradient-to-position)}
.to-accent-content\/20 {
    --tw-gradient-to: hsl(var(--ac) / 0.2) var(--tw-gradient-to-position)}
.to-accent-content\/25 {
    --tw-gradient-to: hsl(var(--ac) / 0.25) var(--tw-gradient-to-position)}
.to-accent-content\/30 {
    --tw-gradient-to: hsl(var(--ac) / 0.3) var(--tw-gradient-to-position)}
.to-accent-content\/40 {
    --tw-gradient-to: hsl(var(--ac) / 0.4) var(--tw-gradient-to-position)}
.to-accent-content\/5 {
    --tw-gradient-to: hsl(var(--ac) / 0.05) var(--tw-gradient-to-position)}
.to-accent-content\/50 {
    --tw-gradient-to: hsl(var(--ac) / 0.5) var(--tw-gradient-to-position)}
.to-accent-content\/60 {
    --tw-gradient-to: hsl(var(--ac) / 0.6) var(--tw-gradient-to-position)}
.to-accent-content\/70 {
    --tw-gradient-to: hsl(var(--ac) / 0.7) var(--tw-gradient-to-position)}
.to-accent-content\/75 {
    --tw-gradient-to: hsl(var(--ac) / 0.75) var(--tw-gradient-to-position)}
.to-accent-content\/80 {
    --tw-gradient-to: hsl(var(--ac) / 0.8) var(--tw-gradient-to-position)}
.to-accent-content\/90 {
    --tw-gradient-to: hsl(var(--ac) / 0.9) var(--tw-gradient-to-position)}
.to-accent-content\/95 {
    --tw-gradient-to: hsl(var(--ac) / 0.95) var(--tw-gradient-to-position)}
.to-accent-focus {
    --tw-gradient-to: hsl(var(--af) / 1) var(--tw-gradient-to-position)}
.to-accent-focus\/0 {
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position)}
.to-accent-focus\/10 {
    --tw-gradient-to: hsl(var(--af) / 0.1) var(--tw-gradient-to-position)}
.to-accent-focus\/100 {
    --tw-gradient-to: hsl(var(--af) / 1) var(--tw-gradient-to-position)}
.to-accent-focus\/20 {
    --tw-gradient-to: hsl(var(--af) / 0.2) var(--tw-gradient-to-position)}
.to-accent-focus\/25 {
    --tw-gradient-to: hsl(var(--af) / 0.25) var(--tw-gradient-to-position)}
.to-accent-focus\/30 {
    --tw-gradient-to: hsl(var(--af) / 0.3) var(--tw-gradient-to-position)}
.to-accent-focus\/40 {
    --tw-gradient-to: hsl(var(--af) / 0.4) var(--tw-gradient-to-position)}
.to-accent-focus\/5 {
    --tw-gradient-to: hsl(var(--af) / 0.05) var(--tw-gradient-to-position)}
.to-accent-focus\/50 {
    --tw-gradient-to: hsl(var(--af) / 0.5) var(--tw-gradient-to-position)}
.to-accent-focus\/60 {
    --tw-gradient-to: hsl(var(--af) / 0.6) var(--tw-gradient-to-position)}
.to-accent-focus\/70 {
    --tw-gradient-to: hsl(var(--af) / 0.7) var(--tw-gradient-to-position)}
.to-accent-focus\/75 {
    --tw-gradient-to: hsl(var(--af) / 0.75) var(--tw-gradient-to-position)}
.to-accent-focus\/80 {
    --tw-gradient-to: hsl(var(--af) / 0.8) var(--tw-gradient-to-position)}
.to-accent-focus\/90 {
    --tw-gradient-to: hsl(var(--af) / 0.9) var(--tw-gradient-to-position)}
.to-accent-focus\/95 {
    --tw-gradient-to: hsl(var(--af) / 0.95) var(--tw-gradient-to-position)}
.to-accent\/0 {
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position)}
.to-accent\/10 {
    --tw-gradient-to: hsl(var(--a) / 0.1) var(--tw-gradient-to-position)}
.to-accent\/100 {
    --tw-gradient-to: hsl(var(--a) / 1) var(--tw-gradient-to-position)}
.to-accent\/20 {
    --tw-gradient-to: hsl(var(--a) / 0.2) var(--tw-gradient-to-position)}
.to-accent\/25 {
    --tw-gradient-to: hsl(var(--a) / 0.25) var(--tw-gradient-to-position)}
.to-accent\/30 {
    --tw-gradient-to: hsl(var(--a) / 0.3) var(--tw-gradient-to-position)}
.to-accent\/40 {
    --tw-gradient-to: hsl(var(--a) / 0.4) var(--tw-gradient-to-position)}
.to-accent\/5 {
    --tw-gradient-to: hsl(var(--a) / 0.05) var(--tw-gradient-to-position)}
.to-accent\/50 {
    --tw-gradient-to: hsl(var(--a) / 0.5) var(--tw-gradient-to-position)}
.to-accent\/60 {
    --tw-gradient-to: hsl(var(--a) / 0.6) var(--tw-gradient-to-position)}
.to-accent\/70 {
    --tw-gradient-to: hsl(var(--a) / 0.7) var(--tw-gradient-to-position)}
.to-accent\/75 {
    --tw-gradient-to: hsl(var(--a) / 0.75) var(--tw-gradient-to-position)}
.to-accent\/80 {
    --tw-gradient-to: hsl(var(--a) / 0.8) var(--tw-gradient-to-position)}
.to-accent\/90 {
    --tw-gradient-to: hsl(var(--a) / 0.9) var(--tw-gradient-to-position)}
.to-accent\/95 {
    --tw-gradient-to: hsl(var(--a) / 0.95) var(--tw-gradient-to-position)}
.to-base-100 {
    --tw-gradient-to: hsl(var(--b1) / 1) var(--tw-gradient-to-position)}
.to-base-100\/0 {
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position)}
.to-base-100\/10 {
    --tw-gradient-to: hsl(var(--b1) / 0.1) var(--tw-gradient-to-position)}
.to-base-100\/100 {
    --tw-gradient-to: hsl(var(--b1) / 1) var(--tw-gradient-to-position)}
.to-base-100\/20 {
    --tw-gradient-to: hsl(var(--b1) / 0.2) var(--tw-gradient-to-position)}
.to-base-100\/25 {
    --tw-gradient-to: hsl(var(--b1) / 0.25) var(--tw-gradient-to-position)}
.to-base-100\/30 {
    --tw-gradient-to: hsl(var(--b1) / 0.3) var(--tw-gradient-to-position)}
.to-base-100\/40 {
    --tw-gradient-to: hsl(var(--b1) / 0.4) var(--tw-gradient-to-position)}
.to-base-100\/5 {
    --tw-gradient-to: hsl(var(--b1) / 0.05) var(--tw-gradient-to-position)}
.to-base-100\/50 {
    --tw-gradient-to: hsl(var(--b1) / 0.5) var(--tw-gradient-to-position)}
.to-base-100\/60 {
    --tw-gradient-to: hsl(var(--b1) / 0.6) var(--tw-gradient-to-position)}
.to-base-100\/70 {
    --tw-gradient-to: hsl(var(--b1) / 0.7) var(--tw-gradient-to-position)}
.to-base-100\/75 {
    --tw-gradient-to: hsl(var(--b1) / 0.75) var(--tw-gradient-to-position)}
.to-base-100\/80 {
    --tw-gradient-to: hsl(var(--b1) / 0.8) var(--tw-gradient-to-position)}
.to-base-100\/90 {
    --tw-gradient-to: hsl(var(--b1) / 0.9) var(--tw-gradient-to-position)}
.to-base-100\/95 {
    --tw-gradient-to: hsl(var(--b1) / 0.95) var(--tw-gradient-to-position)}
.to-base-200 {
    --tw-gradient-to: hsl(var(--b2) / 1) var(--tw-gradient-to-position)}
.to-base-200\/0 {
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position)}
.to-base-200\/10 {
    --tw-gradient-to: hsl(var(--b2) / 0.1) var(--tw-gradient-to-position)}
.to-base-200\/100 {
    --tw-gradient-to: hsl(var(--b2) / 1) var(--tw-gradient-to-position)}
.to-base-200\/20 {
    --tw-gradient-to: hsl(var(--b2) / 0.2) var(--tw-gradient-to-position)}
.to-base-200\/25 {
    --tw-gradient-to: hsl(var(--b2) / 0.25) var(--tw-gradient-to-position)}
.to-base-200\/30 {
    --tw-gradient-to: hsl(var(--b2) / 0.3) var(--tw-gradient-to-position)}
.to-base-200\/40 {
    --tw-gradient-to: hsl(var(--b2) / 0.4) var(--tw-gradient-to-position)}
.to-base-200\/5 {
    --tw-gradient-to: hsl(var(--b2) / 0.05) var(--tw-gradient-to-position)}
.to-base-200\/50 {
    --tw-gradient-to: hsl(var(--b2) / 0.5) var(--tw-gradient-to-position)}
.to-base-200\/60 {
    --tw-gradient-to: hsl(var(--b2) / 0.6) var(--tw-gradient-to-position)}
.to-base-200\/70 {
    --tw-gradient-to: hsl(var(--b2) / 0.7) var(--tw-gradient-to-position)}
.to-base-200\/75 {
    --tw-gradient-to: hsl(var(--b2) / 0.75) var(--tw-gradient-to-position)}
.to-base-200\/80 {
    --tw-gradient-to: hsl(var(--b2) / 0.8) var(--tw-gradient-to-position)}
.to-base-200\/90 {
    --tw-gradient-to: hsl(var(--b2) / 0.9) var(--tw-gradient-to-position)}
.to-base-200\/95 {
    --tw-gradient-to: hsl(var(--b2) / 0.95) var(--tw-gradient-to-position)}
.to-base-300 {
    --tw-gradient-to: hsl(var(--b3) / 1) var(--tw-gradient-to-position)}
.to-base-300\/0 {
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position)}
.to-base-300\/10 {
    --tw-gradient-to: hsl(var(--b3) / 0.1) var(--tw-gradient-to-position)}
.to-base-300\/100 {
    --tw-gradient-to: hsl(var(--b3) / 1) var(--tw-gradient-to-position)}
.to-base-300\/20 {
    --tw-gradient-to: hsl(var(--b3) / 0.2) var(--tw-gradient-to-position)}
.to-base-300\/25 {
    --tw-gradient-to: hsl(var(--b3) / 0.25) var(--tw-gradient-to-position)}
.to-base-300\/30 {
    --tw-gradient-to: hsl(var(--b3) / 0.3) var(--tw-gradient-to-position)}
.to-base-300\/40 {
    --tw-gradient-to: hsl(var(--b3) / 0.4) var(--tw-gradient-to-position)}
.to-base-300\/5 {
    --tw-gradient-to: hsl(var(--b3) / 0.05) var(--tw-gradient-to-position)}
.to-base-300\/50 {
    --tw-gradient-to: hsl(var(--b3) / 0.5) var(--tw-gradient-to-position)}
.to-base-300\/60 {
    --tw-gradient-to: hsl(var(--b3) / 0.6) var(--tw-gradient-to-position)}
.to-base-300\/70 {
    --tw-gradient-to: hsl(var(--b3) / 0.7) var(--tw-gradient-to-position)}
.to-base-300\/75 {
    --tw-gradient-to: hsl(var(--b3) / 0.75) var(--tw-gradient-to-position)}
.to-base-300\/80 {
    --tw-gradient-to: hsl(var(--b3) / 0.8) var(--tw-gradient-to-position)}
.to-base-300\/90 {
    --tw-gradient-to: hsl(var(--b3) / 0.9) var(--tw-gradient-to-position)}
.to-base-300\/95 {
    --tw-gradient-to: hsl(var(--b3) / 0.95) var(--tw-gradient-to-position)}
.to-base-content {
    --tw-gradient-to: hsl(var(--bc) / 1) var(--tw-gradient-to-position)}
.to-base-content\/0 {
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position)}
.to-base-content\/10 {
    --tw-gradient-to: hsl(var(--bc) / 0.1) var(--tw-gradient-to-position)}
.to-base-content\/100 {
    --tw-gradient-to: hsl(var(--bc) / 1) var(--tw-gradient-to-position)}
.to-base-content\/20 {
    --tw-gradient-to: hsl(var(--bc) / 0.2) var(--tw-gradient-to-position)}
.to-base-content\/25 {
    --tw-gradient-to: hsl(var(--bc) / 0.25) var(--tw-gradient-to-position)}
.to-base-content\/30 {
    --tw-gradient-to: hsl(var(--bc) / 0.3) var(--tw-gradient-to-position)}
.to-base-content\/40 {
    --tw-gradient-to: hsl(var(--bc) / 0.4) var(--tw-gradient-to-position)}
.to-base-content\/5 {
    --tw-gradient-to: hsl(var(--bc) / 0.05) var(--tw-gradient-to-position)}
.to-base-content\/50 {
    --tw-gradient-to: hsl(var(--bc) / 0.5) var(--tw-gradient-to-position)}
.to-base-content\/60 {
    --tw-gradient-to: hsl(var(--bc) / 0.6) var(--tw-gradient-to-position)}
.to-base-content\/70 {
    --tw-gradient-to: hsl(var(--bc) / 0.7) var(--tw-gradient-to-position)}
.to-base-content\/75 {
    --tw-gradient-to: hsl(var(--bc) / 0.75) var(--tw-gradient-to-position)}
.to-base-content\/80 {
    --tw-gradient-to: hsl(var(--bc) / 0.8) var(--tw-gradient-to-position)}
.to-base-content\/90 {
    --tw-gradient-to: hsl(var(--bc) / 0.9) var(--tw-gradient-to-position)}
.to-base-content\/95 {
    --tw-gradient-to: hsl(var(--bc) / 0.95) var(--tw-gradient-to-position)}
.to-current {
    --tw-gradient-to: currentColor var(--tw-gradient-to-position)}
.to-error {
    --tw-gradient-to: hsl(var(--er) / 1) var(--tw-gradient-to-position)}
.to-error-content {
    --tw-gradient-to: hsl(var(--erc) / 1) var(--tw-gradient-to-position)}
.to-error-content\/0 {
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position)}
.to-error-content\/10 {
    --tw-gradient-to: hsl(var(--erc) / 0.1) var(--tw-gradient-to-position)}
.to-error-content\/100 {
    --tw-gradient-to: hsl(var(--erc) / 1) var(--tw-gradient-to-position)}
.to-error-content\/20 {
    --tw-gradient-to: hsl(var(--erc) / 0.2) var(--tw-gradient-to-position)}
.to-error-content\/25 {
    --tw-gradient-to: hsl(var(--erc) / 0.25) var(--tw-gradient-to-position)}
.to-error-content\/30 {
    --tw-gradient-to: hsl(var(--erc) / 0.3) var(--tw-gradient-to-position)}
.to-error-content\/40 {
    --tw-gradient-to: hsl(var(--erc) / 0.4) var(--tw-gradient-to-position)}
.to-error-content\/5 {
    --tw-gradient-to: hsl(var(--erc) / 0.05) var(--tw-gradient-to-position)}
.to-error-content\/50 {
    --tw-gradient-to: hsl(var(--erc) / 0.5) var(--tw-gradient-to-position)}
.to-error-content\/60 {
    --tw-gradient-to: hsl(var(--erc) / 0.6) var(--tw-gradient-to-position)}
.to-error-content\/70 {
    --tw-gradient-to: hsl(var(--erc) / 0.7) var(--tw-gradient-to-position)}
.to-error-content\/75 {
    --tw-gradient-to: hsl(var(--erc) / 0.75) var(--tw-gradient-to-position)}
.to-error-content\/80 {
    --tw-gradient-to: hsl(var(--erc) / 0.8) var(--tw-gradient-to-position)}
.to-error-content\/90 {
    --tw-gradient-to: hsl(var(--erc) / 0.9) var(--tw-gradient-to-position)}
.to-error-content\/95 {
    --tw-gradient-to: hsl(var(--erc) / 0.95) var(--tw-gradient-to-position)}
.to-error\/0 {
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position)}
.to-error\/10 {
    --tw-gradient-to: hsl(var(--er) / 0.1) var(--tw-gradient-to-position)}
.to-error\/100 {
    --tw-gradient-to: hsl(var(--er) / 1) var(--tw-gradient-to-position)}
.to-error\/20 {
    --tw-gradient-to: hsl(var(--er) / 0.2) var(--tw-gradient-to-position)}
.to-error\/25 {
    --tw-gradient-to: hsl(var(--er) / 0.25) var(--tw-gradient-to-position)}
.to-error\/30 {
    --tw-gradient-to: hsl(var(--er) / 0.3) var(--tw-gradient-to-position)}
.to-error\/40 {
    --tw-gradient-to: hsl(var(--er) / 0.4) var(--tw-gradient-to-position)}
.to-error\/5 {
    --tw-gradient-to: hsl(var(--er) / 0.05) var(--tw-gradient-to-position)}
.to-error\/50 {
    --tw-gradient-to: hsl(var(--er) / 0.5) var(--tw-gradient-to-position)}
.to-error\/60 {
    --tw-gradient-to: hsl(var(--er) / 0.6) var(--tw-gradient-to-position)}
.to-error\/70 {
    --tw-gradient-to: hsl(var(--er) / 0.7) var(--tw-gradient-to-position)}
.to-error\/75 {
    --tw-gradient-to: hsl(var(--er) / 0.75) var(--tw-gradient-to-position)}
.to-error\/80 {
    --tw-gradient-to: hsl(var(--er) / 0.8) var(--tw-gradient-to-position)}
.to-error\/90 {
    --tw-gradient-to: hsl(var(--er) / 0.9) var(--tw-gradient-to-position)}
.to-error\/95 {
    --tw-gradient-to: hsl(var(--er) / 0.95) var(--tw-gradient-to-position)}
.to-info {
    --tw-gradient-to: hsl(var(--in) / 1) var(--tw-gradient-to-position)}
.to-info-content {
    --tw-gradient-to: hsl(var(--inc) / 1) var(--tw-gradient-to-position)}
.to-info-content\/0 {
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position)}
.to-info-content\/10 {
    --tw-gradient-to: hsl(var(--inc) / 0.1) var(--tw-gradient-to-position)}
.to-info-content\/100 {
    --tw-gradient-to: hsl(var(--inc) / 1) var(--tw-gradient-to-position)}
.to-info-content\/20 {
    --tw-gradient-to: hsl(var(--inc) / 0.2) var(--tw-gradient-to-position)}
.to-info-content\/25 {
    --tw-gradient-to: hsl(var(--inc) / 0.25) var(--tw-gradient-to-position)}
.to-info-content\/30 {
    --tw-gradient-to: hsl(var(--inc) / 0.3) var(--tw-gradient-to-position)}
.to-info-content\/40 {
    --tw-gradient-to: hsl(var(--inc) / 0.4) var(--tw-gradient-to-position)}
.to-info-content\/5 {
    --tw-gradient-to: hsl(var(--inc) / 0.05) var(--tw-gradient-to-position)}
.to-info-content\/50 {
    --tw-gradient-to: hsl(var(--inc) / 0.5) var(--tw-gradient-to-position)}
.to-info-content\/60 {
    --tw-gradient-to: hsl(var(--inc) / 0.6) var(--tw-gradient-to-position)}
.to-info-content\/70 {
    --tw-gradient-to: hsl(var(--inc) / 0.7) var(--tw-gradient-to-position)}
.to-info-content\/75 {
    --tw-gradient-to: hsl(var(--inc) / 0.75) var(--tw-gradient-to-position)}
.to-info-content\/80 {
    --tw-gradient-to: hsl(var(--inc) / 0.8) var(--tw-gradient-to-position)}
.to-info-content\/90 {
    --tw-gradient-to: hsl(var(--inc) / 0.9) var(--tw-gradient-to-position)}
.to-info-content\/95 {
    --tw-gradient-to: hsl(var(--inc) / 0.95) var(--tw-gradient-to-position)}
.to-info\/0 {
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position)}
.to-info\/10 {
    --tw-gradient-to: hsl(var(--in) / 0.1) var(--tw-gradient-to-position)}
.to-info\/100 {
    --tw-gradient-to: hsl(var(--in) / 1) var(--tw-gradient-to-position)}
.to-info\/20 {
    --tw-gradient-to: hsl(var(--in) / 0.2) var(--tw-gradient-to-position)}
.to-info\/25 {
    --tw-gradient-to: hsl(var(--in) / 0.25) var(--tw-gradient-to-position)}
.to-info\/30 {
    --tw-gradient-to: hsl(var(--in) / 0.3) var(--tw-gradient-to-position)}
.to-info\/40 {
    --tw-gradient-to: hsl(var(--in) / 0.4) var(--tw-gradient-to-position)}
.to-info\/5 {
    --tw-gradient-to: hsl(var(--in) / 0.05) var(--tw-gradient-to-position)}
.to-info\/50 {
    --tw-gradient-to: hsl(var(--in) / 0.5) var(--tw-gradient-to-position)}
.to-info\/60 {
    --tw-gradient-to: hsl(var(--in) / 0.6) var(--tw-gradient-to-position)}
.to-info\/70 {
    --tw-gradient-to: hsl(var(--in) / 0.7) var(--tw-gradient-to-position)}
.to-info\/75 {
    --tw-gradient-to: hsl(var(--in) / 0.75) var(--tw-gradient-to-position)}
.to-info\/80 {
    --tw-gradient-to: hsl(var(--in) / 0.8) var(--tw-gradient-to-position)}
.to-info\/90 {
    --tw-gradient-to: hsl(var(--in) / 0.9) var(--tw-gradient-to-position)}
.to-info\/95 {
    --tw-gradient-to: hsl(var(--in) / 0.95) var(--tw-gradient-to-position)}
.to-neutral {
    --tw-gradient-to: hsl(var(--n) / 1) var(--tw-gradient-to-position)}
.to-neutral-content {
    --tw-gradient-to: hsl(var(--nc) / 1) var(--tw-gradient-to-position)}
.to-neutral-content\/0 {
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position)}
.to-neutral-content\/10 {
    --tw-gradient-to: hsl(var(--nc) / 0.1) var(--tw-gradient-to-position)}
.to-neutral-content\/100 {
    --tw-gradient-to: hsl(var(--nc) / 1) var(--tw-gradient-to-position)}
.to-neutral-content\/20 {
    --tw-gradient-to: hsl(var(--nc) / 0.2) var(--tw-gradient-to-position)}
.to-neutral-content\/25 {
    --tw-gradient-to: hsl(var(--nc) / 0.25) var(--tw-gradient-to-position)}
.to-neutral-content\/30 {
    --tw-gradient-to: hsl(var(--nc) / 0.3) var(--tw-gradient-to-position)}
.to-neutral-content\/40 {
    --tw-gradient-to: hsl(var(--nc) / 0.4) var(--tw-gradient-to-position)}
.to-neutral-content\/5 {
    --tw-gradient-to: hsl(var(--nc) / 0.05) var(--tw-gradient-to-position)}
.to-neutral-content\/50 {
    --tw-gradient-to: hsl(var(--nc) / 0.5) var(--tw-gradient-to-position)}
.to-neutral-content\/60 {
    --tw-gradient-to: hsl(var(--nc) / 0.6) var(--tw-gradient-to-position)}
.to-neutral-content\/70 {
    --tw-gradient-to: hsl(var(--nc) / 0.7) var(--tw-gradient-to-position)}
.to-neutral-content\/75 {
    --tw-gradient-to: hsl(var(--nc) / 0.75) var(--tw-gradient-to-position)}
.to-neutral-content\/80 {
    --tw-gradient-to: hsl(var(--nc) / 0.8) var(--tw-gradient-to-position)}
.to-neutral-content\/90 {
    --tw-gradient-to: hsl(var(--nc) / 0.9) var(--tw-gradient-to-position)}
.to-neutral-content\/95 {
    --tw-gradient-to: hsl(var(--nc) / 0.95) var(--tw-gradient-to-position)}
.to-neutral-focus {
    --tw-gradient-to: hsl(var(--nf) / 1) var(--tw-gradient-to-position)}
.to-neutral-focus\/0 {
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position)}
.to-neutral-focus\/10 {
    --tw-gradient-to: hsl(var(--nf) / 0.1) var(--tw-gradient-to-position)}
.to-neutral-focus\/100 {
    --tw-gradient-to: hsl(var(--nf) / 1) var(--tw-gradient-to-position)}
.to-neutral-focus\/20 {
    --tw-gradient-to: hsl(var(--nf) / 0.2) var(--tw-gradient-to-position)}
.to-neutral-focus\/25 {
    --tw-gradient-to: hsl(var(--nf) / 0.25) var(--tw-gradient-to-position)}
.to-neutral-focus\/30 {
    --tw-gradient-to: hsl(var(--nf) / 0.3) var(--tw-gradient-to-position)}
.to-neutral-focus\/40 {
    --tw-gradient-to: hsl(var(--nf) / 0.4) var(--tw-gradient-to-position)}
.to-neutral-focus\/5 {
    --tw-gradient-to: hsl(var(--nf) / 0.05) var(--tw-gradient-to-position)}
.to-neutral-focus\/50 {
    --tw-gradient-to: hsl(var(--nf) / 0.5) var(--tw-gradient-to-position)}
.to-neutral-focus\/60 {
    --tw-gradient-to: hsl(var(--nf) / 0.6) var(--tw-gradient-to-position)}
.to-neutral-focus\/70 {
    --tw-gradient-to: hsl(var(--nf) / 0.7) var(--tw-gradient-to-position)}
.to-neutral-focus\/75 {
    --tw-gradient-to: hsl(var(--nf) / 0.75) var(--tw-gradient-to-position)}
.to-neutral-focus\/80 {
    --tw-gradient-to: hsl(var(--nf) / 0.8) var(--tw-gradient-to-position)}
.to-neutral-focus\/90 {
    --tw-gradient-to: hsl(var(--nf) / 0.9) var(--tw-gradient-to-position)}
.to-neutral-focus\/95 {
    --tw-gradient-to: hsl(var(--nf) / 0.95) var(--tw-gradient-to-position)}
.to-neutral\/0 {
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position)}
.to-neutral\/10 {
    --tw-gradient-to: hsl(var(--n) / 0.1) var(--tw-gradient-to-position)}
.to-neutral\/100 {
    --tw-gradient-to: hsl(var(--n) / 1) var(--tw-gradient-to-position)}
.to-neutral\/20 {
    --tw-gradient-to: hsl(var(--n) / 0.2) var(--tw-gradient-to-position)}
.to-neutral\/25 {
    --tw-gradient-to: hsl(var(--n) / 0.25) var(--tw-gradient-to-position)}
.to-neutral\/30 {
    --tw-gradient-to: hsl(var(--n) / 0.3) var(--tw-gradient-to-position)}
.to-neutral\/40 {
    --tw-gradient-to: hsl(var(--n) / 0.4) var(--tw-gradient-to-position)}
.to-neutral\/5 {
    --tw-gradient-to: hsl(var(--n) / 0.05) var(--tw-gradient-to-position)}
.to-neutral\/50 {
    --tw-gradient-to: hsl(var(--n) / 0.5) var(--tw-gradient-to-position)}
.to-neutral\/60 {
    --tw-gradient-to: hsl(var(--n) / 0.6) var(--tw-gradient-to-position)}
.to-neutral\/70 {
    --tw-gradient-to: hsl(var(--n) / 0.7) var(--tw-gradient-to-position)}
.to-neutral\/75 {
    --tw-gradient-to: hsl(var(--n) / 0.75) var(--tw-gradient-to-position)}
.to-neutral\/80 {
    --tw-gradient-to: hsl(var(--n) / 0.8) var(--tw-gradient-to-position)}
.to-neutral\/90 {
    --tw-gradient-to: hsl(var(--n) / 0.9) var(--tw-gradient-to-position)}
.to-neutral\/95 {
    --tw-gradient-to: hsl(var(--n) / 0.95) var(--tw-gradient-to-position)}
.to-primary {
    --tw-gradient-to: hsl(var(--p) / 1) var(--tw-gradient-to-position)}
.to-primary-content {
    --tw-gradient-to: hsl(var(--pc) / 1) var(--tw-gradient-to-position)}
.to-primary-content\/0 {
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position)}
.to-primary-content\/10 {
    --tw-gradient-to: hsl(var(--pc) / 0.1) var(--tw-gradient-to-position)}
.to-primary-content\/100 {
    --tw-gradient-to: hsl(var(--pc) / 1) var(--tw-gradient-to-position)}
.to-primary-content\/20 {
    --tw-gradient-to: hsl(var(--pc) / 0.2) var(--tw-gradient-to-position)}
.to-primary-content\/25 {
    --tw-gradient-to: hsl(var(--pc) / 0.25) var(--tw-gradient-to-position)}
.to-primary-content\/30 {
    --tw-gradient-to: hsl(var(--pc) / 0.3) var(--tw-gradient-to-position)}
.to-primary-content\/40 {
    --tw-gradient-to: hsl(var(--pc) / 0.4) var(--tw-gradient-to-position)}
.to-primary-content\/5 {
    --tw-gradient-to: hsl(var(--pc) / 0.05) var(--tw-gradient-to-position)}
.to-primary-content\/50 {
    --tw-gradient-to: hsl(var(--pc) / 0.5) var(--tw-gradient-to-position)}
.to-primary-content\/60 {
    --tw-gradient-to: hsl(var(--pc) / 0.6) var(--tw-gradient-to-position)}
.to-primary-content\/70 {
    --tw-gradient-to: hsl(var(--pc) / 0.7) var(--tw-gradient-to-position)}
.to-primary-content\/75 {
    --tw-gradient-to: hsl(var(--pc) / 0.75) var(--tw-gradient-to-position)}
.to-primary-content\/80 {
    --tw-gradient-to: hsl(var(--pc) / 0.8) var(--tw-gradient-to-position)}
.to-primary-content\/90 {
    --tw-gradient-to: hsl(var(--pc) / 0.9) var(--tw-gradient-to-position)}
.to-primary-content\/95 {
    --tw-gradient-to: hsl(var(--pc) / 0.95) var(--tw-gradient-to-position)}
.to-primary-focus {
    --tw-gradient-to: hsl(var(--pf) / 1) var(--tw-gradient-to-position)}
.to-primary-focus\/0 {
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position)}
.to-primary-focus\/10 {
    --tw-gradient-to: hsl(var(--pf) / 0.1) var(--tw-gradient-to-position)}
.to-primary-focus\/100 {
    --tw-gradient-to: hsl(var(--pf) / 1) var(--tw-gradient-to-position)}
.to-primary-focus\/20 {
    --tw-gradient-to: hsl(var(--pf) / 0.2) var(--tw-gradient-to-position)}
.to-primary-focus\/25 {
    --tw-gradient-to: hsl(var(--pf) / 0.25) var(--tw-gradient-to-position)}
.to-primary-focus\/30 {
    --tw-gradient-to: hsl(var(--pf) / 0.3) var(--tw-gradient-to-position)}
.to-primary-focus\/40 {
    --tw-gradient-to: hsl(var(--pf) / 0.4) var(--tw-gradient-to-position)}
.to-primary-focus\/5 {
    --tw-gradient-to: hsl(var(--pf) / 0.05) var(--tw-gradient-to-position)}
.to-primary-focus\/50 {
    --tw-gradient-to: hsl(var(--pf) / 0.5) var(--tw-gradient-to-position)}
.to-primary-focus\/60 {
    --tw-gradient-to: hsl(var(--pf) / 0.6) var(--tw-gradient-to-position)}
.to-primary-focus\/70 {
    --tw-gradient-to: hsl(var(--pf) / 0.7) var(--tw-gradient-to-position)}
.to-primary-focus\/75 {
    --tw-gradient-to: hsl(var(--pf) / 0.75) var(--tw-gradient-to-position)}
.to-primary-focus\/80 {
    --tw-gradient-to: hsl(var(--pf) / 0.8) var(--tw-gradient-to-position)}
.to-primary-focus\/90 {
    --tw-gradient-to: hsl(var(--pf) / 0.9) var(--tw-gradient-to-position)}
.to-primary-focus\/95 {
    --tw-gradient-to: hsl(var(--pf) / 0.95) var(--tw-gradient-to-position)}
.to-primary\/0 {
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position)}
.to-primary\/10 {
    --tw-gradient-to: hsl(var(--p) / 0.1) var(--tw-gradient-to-position)}
.to-primary\/100 {
    --tw-gradient-to: hsl(var(--p) / 1) var(--tw-gradient-to-position)}
.to-primary\/20 {
    --tw-gradient-to: hsl(var(--p) / 0.2) var(--tw-gradient-to-position)}
.to-primary\/25 {
    --tw-gradient-to: hsl(var(--p) / 0.25) var(--tw-gradient-to-position)}
.to-primary\/30 {
    --tw-gradient-to: hsl(var(--p) / 0.3) var(--tw-gradient-to-position)}
.to-primary\/40 {
    --tw-gradient-to: hsl(var(--p) / 0.4) var(--tw-gradient-to-position)}
.to-primary\/5 {
    --tw-gradient-to: hsl(var(--p) / 0.05) var(--tw-gradient-to-position)}
.to-primary\/50 {
    --tw-gradient-to: hsl(var(--p) / 0.5) var(--tw-gradient-to-position)}
.to-primary\/60 {
    --tw-gradient-to: hsl(var(--p) / 0.6) var(--tw-gradient-to-position)}
.to-primary\/70 {
    --tw-gradient-to: hsl(var(--p) / 0.7) var(--tw-gradient-to-position)}
.to-primary\/75 {
    --tw-gradient-to: hsl(var(--p) / 0.75) var(--tw-gradient-to-position)}
.to-primary\/80 {
    --tw-gradient-to: hsl(var(--p) / 0.8) var(--tw-gradient-to-position)}
.to-primary\/90 {
    --tw-gradient-to: hsl(var(--p) / 0.9) var(--tw-gradient-to-position)}
.to-primary\/95 {
    --tw-gradient-to: hsl(var(--p) / 0.95) var(--tw-gradient-to-position)}
.to-secondary {
    --tw-gradient-to: hsl(var(--s) / 1) var(--tw-gradient-to-position)}
.to-secondary-content {
    --tw-gradient-to: hsl(var(--sc) / 1) var(--tw-gradient-to-position)}
.to-secondary-content\/0 {
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position)}
.to-secondary-content\/10 {
    --tw-gradient-to: hsl(var(--sc) / 0.1) var(--tw-gradient-to-position)}
.to-secondary-content\/100 {
    --tw-gradient-to: hsl(var(--sc) / 1) var(--tw-gradient-to-position)}
.to-secondary-content\/20 {
    --tw-gradient-to: hsl(var(--sc) / 0.2) var(--tw-gradient-to-position)}
.to-secondary-content\/25 {
    --tw-gradient-to: hsl(var(--sc) / 0.25) var(--tw-gradient-to-position)}
.to-secondary-content\/30 {
    --tw-gradient-to: hsl(var(--sc) / 0.3) var(--tw-gradient-to-position)}
.to-secondary-content\/40 {
    --tw-gradient-to: hsl(var(--sc) / 0.4) var(--tw-gradient-to-position)}
.to-secondary-content\/5 {
    --tw-gradient-to: hsl(var(--sc) / 0.05) var(--tw-gradient-to-position)}
.to-secondary-content\/50 {
    --tw-gradient-to: hsl(var(--sc) / 0.5) var(--tw-gradient-to-position)}
.to-secondary-content\/60 {
    --tw-gradient-to: hsl(var(--sc) / 0.6) var(--tw-gradient-to-position)}
.to-secondary-content\/70 {
    --tw-gradient-to: hsl(var(--sc) / 0.7) var(--tw-gradient-to-position)}
.to-secondary-content\/75 {
    --tw-gradient-to: hsl(var(--sc) / 0.75) var(--tw-gradient-to-position)}
.to-secondary-content\/80 {
    --tw-gradient-to: hsl(var(--sc) / 0.8) var(--tw-gradient-to-position)}
.to-secondary-content\/90 {
    --tw-gradient-to: hsl(var(--sc) / 0.9) var(--tw-gradient-to-position)}
.to-secondary-content\/95 {
    --tw-gradient-to: hsl(var(--sc) / 0.95) var(--tw-gradient-to-position)}
.to-secondary-focus {
    --tw-gradient-to: hsl(var(--sf) / 1) var(--tw-gradient-to-position)}
.to-secondary-focus\/0 {
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position)}
.to-secondary-focus\/10 {
    --tw-gradient-to: hsl(var(--sf) / 0.1) var(--tw-gradient-to-position)}
.to-secondary-focus\/100 {
    --tw-gradient-to: hsl(var(--sf) / 1) var(--tw-gradient-to-position)}
.to-secondary-focus\/20 {
    --tw-gradient-to: hsl(var(--sf) / 0.2) var(--tw-gradient-to-position)}
.to-secondary-focus\/25 {
    --tw-gradient-to: hsl(var(--sf) / 0.25) var(--tw-gradient-to-position)}
.to-secondary-focus\/30 {
    --tw-gradient-to: hsl(var(--sf) / 0.3) var(--tw-gradient-to-position)}
.to-secondary-focus\/40 {
    --tw-gradient-to: hsl(var(--sf) / 0.4) var(--tw-gradient-to-position)}
.to-secondary-focus\/5 {
    --tw-gradient-to: hsl(var(--sf) / 0.05) var(--tw-gradient-to-position)}
.to-secondary-focus\/50 {
    --tw-gradient-to: hsl(var(--sf) / 0.5) var(--tw-gradient-to-position)}
.to-secondary-focus\/60 {
    --tw-gradient-to: hsl(var(--sf) / 0.6) var(--tw-gradient-to-position)}
.to-secondary-focus\/70 {
    --tw-gradient-to: hsl(var(--sf) / 0.7) var(--tw-gradient-to-position)}
.to-secondary-focus\/75 {
    --tw-gradient-to: hsl(var(--sf) / 0.75) var(--tw-gradient-to-position)}
.to-secondary-focus\/80 {
    --tw-gradient-to: hsl(var(--sf) / 0.8) var(--tw-gradient-to-position)}
.to-secondary-focus\/90 {
    --tw-gradient-to: hsl(var(--sf) / 0.9) var(--tw-gradient-to-position)}
.to-secondary-focus\/95 {
    --tw-gradient-to: hsl(var(--sf) / 0.95) var(--tw-gradient-to-position)}
.to-secondary\/0 {
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position)}
.to-secondary\/10 {
    --tw-gradient-to: hsl(var(--s) / 0.1) var(--tw-gradient-to-position)}
.to-secondary\/100 {
    --tw-gradient-to: hsl(var(--s) / 1) var(--tw-gradient-to-position)}
.to-secondary\/20 {
    --tw-gradient-to: hsl(var(--s) / 0.2) var(--tw-gradient-to-position)}
.to-secondary\/25 {
    --tw-gradient-to: hsl(var(--s) / 0.25) var(--tw-gradient-to-position)}
.to-secondary\/30 {
    --tw-gradient-to: hsl(var(--s) / 0.3) var(--tw-gradient-to-position)}
.to-secondary\/40 {
    --tw-gradient-to: hsl(var(--s) / 0.4) var(--tw-gradient-to-position)}
.to-secondary\/5 {
    --tw-gradient-to: hsl(var(--s) / 0.05) var(--tw-gradient-to-position)}
.to-secondary\/50 {
    --tw-gradient-to: hsl(var(--s) / 0.5) var(--tw-gradient-to-position)}
.to-secondary\/60 {
    --tw-gradient-to: hsl(var(--s) / 0.6) var(--tw-gradient-to-position)}
.to-secondary\/70 {
    --tw-gradient-to: hsl(var(--s) / 0.7) var(--tw-gradient-to-position)}
.to-secondary\/75 {
    --tw-gradient-to: hsl(var(--s) / 0.75) var(--tw-gradient-to-position)}
.to-secondary\/80 {
    --tw-gradient-to: hsl(var(--s) / 0.8) var(--tw-gradient-to-position)}
.to-secondary\/90 {
    --tw-gradient-to: hsl(var(--s) / 0.9) var(--tw-gradient-to-position)}
.to-secondary\/95 {
    --tw-gradient-to: hsl(var(--s) / 0.95) var(--tw-gradient-to-position)}
.to-success {
    --tw-gradient-to: hsl(var(--su) / 1) var(--tw-gradient-to-position)}
.to-success-content {
    --tw-gradient-to: hsl(var(--suc) / 1) var(--tw-gradient-to-position)}
.to-success-content\/0 {
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position)}
.to-success-content\/10 {
    --tw-gradient-to: hsl(var(--suc) / 0.1) var(--tw-gradient-to-position)}
.to-success-content\/100 {
    --tw-gradient-to: hsl(var(--suc) / 1) var(--tw-gradient-to-position)}
.to-success-content\/20 {
    --tw-gradient-to: hsl(var(--suc) / 0.2) var(--tw-gradient-to-position)}
.to-success-content\/25 {
    --tw-gradient-to: hsl(var(--suc) / 0.25) var(--tw-gradient-to-position)}
.to-success-content\/30 {
    --tw-gradient-to: hsl(var(--suc) / 0.3) var(--tw-gradient-to-position)}
.to-success-content\/40 {
    --tw-gradient-to: hsl(var(--suc) / 0.4) var(--tw-gradient-to-position)}
.to-success-content\/5 {
    --tw-gradient-to: hsl(var(--suc) / 0.05) var(--tw-gradient-to-position)}
.to-success-content\/50 {
    --tw-gradient-to: hsl(var(--suc) / 0.5) var(--tw-gradient-to-position)}
.to-success-content\/60 {
    --tw-gradient-to: hsl(var(--suc) / 0.6) var(--tw-gradient-to-position)}
.to-success-content\/70 {
    --tw-gradient-to: hsl(var(--suc) / 0.7) var(--tw-gradient-to-position)}
.to-success-content\/75 {
    --tw-gradient-to: hsl(var(--suc) / 0.75) var(--tw-gradient-to-position)}
.to-success-content\/80 {
    --tw-gradient-to: hsl(var(--suc) / 0.8) var(--tw-gradient-to-position)}
.to-success-content\/90 {
    --tw-gradient-to: hsl(var(--suc) / 0.9) var(--tw-gradient-to-position)}
.to-success-content\/95 {
    --tw-gradient-to: hsl(var(--suc) / 0.95) var(--tw-gradient-to-position)}
.to-success\/0 {
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position)}
.to-success\/10 {
    --tw-gradient-to: hsl(var(--su) / 0.1) var(--tw-gradient-to-position)}
.to-success\/100 {
    --tw-gradient-to: hsl(var(--su) / 1) var(--tw-gradient-to-position)}
.to-success\/20 {
    --tw-gradient-to: hsl(var(--su) / 0.2) var(--tw-gradient-to-position)}
.to-success\/25 {
    --tw-gradient-to: hsl(var(--su) / 0.25) var(--tw-gradient-to-position)}
.to-success\/30 {
    --tw-gradient-to: hsl(var(--su) / 0.3) var(--tw-gradient-to-position)}
.to-success\/40 {
    --tw-gradient-to: hsl(var(--su) / 0.4) var(--tw-gradient-to-position)}
.to-success\/5 {
    --tw-gradient-to: hsl(var(--su) / 0.05) var(--tw-gradient-to-position)}
.to-success\/50 {
    --tw-gradient-to: hsl(var(--su) / 0.5) var(--tw-gradient-to-position)}
.to-success\/60 {
    --tw-gradient-to: hsl(var(--su) / 0.6) var(--tw-gradient-to-position)}
.to-success\/70 {
    --tw-gradient-to: hsl(var(--su) / 0.7) var(--tw-gradient-to-position)}
.to-success\/75 {
    --tw-gradient-to: hsl(var(--su) / 0.75) var(--tw-gradient-to-position)}
.to-success\/80 {
    --tw-gradient-to: hsl(var(--su) / 0.8) var(--tw-gradient-to-position)}
.to-success\/90 {
    --tw-gradient-to: hsl(var(--su) / 0.9) var(--tw-gradient-to-position)}
.to-success\/95 {
    --tw-gradient-to: hsl(var(--su) / 0.95) var(--tw-gradient-to-position)}
.to-transparent {
    --tw-gradient-to: transparent var(--tw-gradient-to-position)}
.to-transparent\/0 {
    --tw-gradient-to: rgb(0 0 0 / 0) var(--tw-gradient-to-position)}
.to-transparent\/10 {
    --tw-gradient-to: rgb(0 0 0 / 0.1) var(--tw-gradient-to-position)}
.to-transparent\/100 {
    --tw-gradient-to: rgb(0 0 0 / 1) var(--tw-gradient-to-position)}
.to-transparent\/20 {
    --tw-gradient-to: rgb(0 0 0 / 0.2) var(--tw-gradient-to-position)}
.to-transparent\/25 {
    --tw-gradient-to: rgb(0 0 0 / 0.25) var(--tw-gradient-to-position)}
.to-transparent\/30 {
    --tw-gradient-to: rgb(0 0 0 / 0.3) var(--tw-gradient-to-position)}
.to-transparent\/40 {
    --tw-gradient-to: rgb(0 0 0 / 0.4) var(--tw-gradient-to-position)}
.to-transparent\/5 {
    --tw-gradient-to: rgb(0 0 0 / 0.05) var(--tw-gradient-to-position)}
.to-transparent\/50 {
    --tw-gradient-to: rgb(0 0 0 / 0.5) var(--tw-gradient-to-position)}
.to-transparent\/60 {
    --tw-gradient-to: rgb(0 0 0 / 0.6) var(--tw-gradient-to-position)}
.to-transparent\/70 {
    --tw-gradient-to: rgb(0 0 0 / 0.7) var(--tw-gradient-to-position)}
.to-transparent\/75 {
    --tw-gradient-to: rgb(0 0 0 / 0.75) var(--tw-gradient-to-position)}
.to-transparent\/80 {
    --tw-gradient-to: rgb(0 0 0 / 0.8) var(--tw-gradient-to-position)}
.to-transparent\/90 {
    --tw-gradient-to: rgb(0 0 0 / 0.9) var(--tw-gradient-to-position)}
.to-transparent\/95 {
    --tw-gradient-to: rgb(0 0 0 / 0.95) var(--tw-gradient-to-position)}
.to-warning {
    --tw-gradient-to: hsl(var(--wa) / 1) var(--tw-gradient-to-position)}
.to-warning-content {
    --tw-gradient-to: hsl(var(--wac) / 1) var(--tw-gradient-to-position)}
.to-warning-content\/0 {
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position)}
.to-warning-content\/10 {
    --tw-gradient-to: hsl(var(--wac) / 0.1) var(--tw-gradient-to-position)}
.to-warning-content\/100 {
    --tw-gradient-to: hsl(var(--wac) / 1) var(--tw-gradient-to-position)}
.to-warning-content\/20 {
    --tw-gradient-to: hsl(var(--wac) / 0.2) var(--tw-gradient-to-position)}
.to-warning-content\/25 {
    --tw-gradient-to: hsl(var(--wac) / 0.25) var(--tw-gradient-to-position)}
.to-warning-content\/30 {
    --tw-gradient-to: hsl(var(--wac) / 0.3) var(--tw-gradient-to-position)}
.to-warning-content\/40 {
    --tw-gradient-to: hsl(var(--wac) / 0.4) var(--tw-gradient-to-position)}
.to-warning-content\/5 {
    --tw-gradient-to: hsl(var(--wac) / 0.05) var(--tw-gradient-to-position)}
.to-warning-content\/50 {
    --tw-gradient-to: hsl(var(--wac) / 0.5) var(--tw-gradient-to-position)}
.to-warning-content\/60 {
    --tw-gradient-to: hsl(var(--wac) / 0.6) var(--tw-gradient-to-position)}
.to-warning-content\/70 {
    --tw-gradient-to: hsl(var(--wac) / 0.7) var(--tw-gradient-to-position)}
.to-warning-content\/75 {
    --tw-gradient-to: hsl(var(--wac) / 0.75) var(--tw-gradient-to-position)}
.to-warning-content\/80 {
    --tw-gradient-to: hsl(var(--wac) / 0.8) var(--tw-gradient-to-position)}
.to-warning-content\/90 {
    --tw-gradient-to: hsl(var(--wac) / 0.9) var(--tw-gradient-to-position)}
.to-warning-content\/95 {
    --tw-gradient-to: hsl(var(--wac) / 0.95) var(--tw-gradient-to-position)}
.to-warning\/0 {
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position)}
.to-warning\/10 {
    --tw-gradient-to: hsl(var(--wa) / 0.1) var(--tw-gradient-to-position)}
.to-warning\/100 {
    --tw-gradient-to: hsl(var(--wa) / 1) var(--tw-gradient-to-position)}
.to-warning\/20 {
    --tw-gradient-to: hsl(var(--wa) / 0.2) var(--tw-gradient-to-position)}
.to-warning\/25 {
    --tw-gradient-to: hsl(var(--wa) / 0.25) var(--tw-gradient-to-position)}
.to-warning\/30 {
    --tw-gradient-to: hsl(var(--wa) / 0.3) var(--tw-gradient-to-position)}
.to-warning\/40 {
    --tw-gradient-to: hsl(var(--wa) / 0.4) var(--tw-gradient-to-position)}
.to-warning\/5 {
    --tw-gradient-to: hsl(var(--wa) / 0.05) var(--tw-gradient-to-position)}
.to-warning\/50 {
    --tw-gradient-to: hsl(var(--wa) / 0.5) var(--tw-gradient-to-position)}
.to-warning\/60 {
    --tw-gradient-to: hsl(var(--wa) / 0.6) var(--tw-gradient-to-position)}
.to-warning\/70 {
    --tw-gradient-to: hsl(var(--wa) / 0.7) var(--tw-gradient-to-position)}
.to-warning\/75 {
    --tw-gradient-to: hsl(var(--wa) / 0.75) var(--tw-gradient-to-position)}
.to-warning\/80 {
    --tw-gradient-to: hsl(var(--wa) / 0.8) var(--tw-gradient-to-position)}
.to-warning\/90 {
    --tw-gradient-to: hsl(var(--wa) / 0.9) var(--tw-gradient-to-position)}
.to-warning\/95 {
    --tw-gradient-to: hsl(var(--wa) / 0.95) var(--tw-gradient-to-position)}
.to-0\% {
    --tw-gradient-to-position: 0%}
.to-10\% {
    --tw-gradient-to-position: 10%}
.to-100\% {
    --tw-gradient-to-position: 100%}
.to-15\% {
    --tw-gradient-to-position: 15%}
.to-20\% {
    --tw-gradient-to-position: 20%}
.to-25\% {
    --tw-gradient-to-position: 25%}
.to-30\% {
    --tw-gradient-to-position: 30%}
.to-35\% {
    --tw-gradient-to-position: 35%}
.to-40\% {
    --tw-gradient-to-position: 40%}
.to-45\% {
    --tw-gradient-to-position: 45%}
.to-5\% {
    --tw-gradient-to-position: 5%}
.to-50\% {
    --tw-gradient-to-position: 50%}
.to-55\% {
    --tw-gradient-to-position: 55%}
.to-60\% {
    --tw-gradient-to-position: 60%}
.to-65\% {
    --tw-gradient-to-position: 65%}
.to-70\% {
    --tw-gradient-to-position: 70%}
.to-75\% {
    --tw-gradient-to-position: 75%}
.to-80\% {
    --tw-gradient-to-position: 80%}
.to-85\% {
    --tw-gradient-to-position: 85%}
.to-90\% {
    --tw-gradient-to-position: 90%}
.to-95\% {
    --tw-gradient-to-position: 95%}
.text-accent {
    color: hsl(var(--a) / 1)}
.text-accent-content {
    color: hsl(var(--ac) / 1)}
.text-accent-content\/0 {
    color: hsl(var(--ac) / 0)}
.text-accent-content\/10 {
    color: hsl(var(--ac) / 0.1)}
.text-accent-content\/100 {
    color: hsl(var(--ac) / 1)}
.text-accent-content\/20 {
    color: hsl(var(--ac) / 0.2)}
.text-accent-content\/25 {
    color: hsl(var(--ac) / 0.25)}
.text-accent-content\/30 {
    color: hsl(var(--ac) / 0.3)}
.text-accent-content\/40 {
    color: hsl(var(--ac) / 0.4)}
.text-accent-content\/5 {
    color: hsl(var(--ac) / 0.05)}
.text-accent-content\/50 {
    color: hsl(var(--ac) / 0.5)}
.text-accent-content\/60 {
    color: hsl(var(--ac) / 0.6)}
.text-accent-content\/70 {
    color: hsl(var(--ac) / 0.7)}
.text-accent-content\/75 {
    color: hsl(var(--ac) / 0.75)}
.text-accent-content\/80 {
    color: hsl(var(--ac) / 0.8)}
.text-accent-content\/90 {
    color: hsl(var(--ac) / 0.9)}
.text-accent-content\/95 {
    color: hsl(var(--ac) / 0.95)}
.text-accent-focus {
    color: hsl(var(--af) / 1)}
.text-accent-focus\/0 {
    color: hsl(var(--af) / 0)}
.text-accent-focus\/10 {
    color: hsl(var(--af) / 0.1)}
.text-accent-focus\/100 {
    color: hsl(var(--af) / 1)}
.text-accent-focus\/20 {
    color: hsl(var(--af) / 0.2)}
.text-accent-focus\/25 {
    color: hsl(var(--af) / 0.25)}
.text-accent-focus\/30 {
    color: hsl(var(--af) / 0.3)}
.text-accent-focus\/40 {
    color: hsl(var(--af) / 0.4)}
.text-accent-focus\/5 {
    color: hsl(var(--af) / 0.05)}
.text-accent-focus\/50 {
    color: hsl(var(--af) / 0.5)}
.text-accent-focus\/60 {
    color: hsl(var(--af) / 0.6)}
.text-accent-focus\/70 {
    color: hsl(var(--af) / 0.7)}
.text-accent-focus\/75 {
    color: hsl(var(--af) / 0.75)}
.text-accent-focus\/80 {
    color: hsl(var(--af) / 0.8)}
.text-accent-focus\/90 {
    color: hsl(var(--af) / 0.9)}
.text-accent-focus\/95 {
    color: hsl(var(--af) / 0.95)}
.text-accent\/0 {
    color: hsl(var(--a) / 0)}
.text-accent\/10 {
    color: hsl(var(--a) / 0.1)}
.text-accent\/100 {
    color: hsl(var(--a) / 1)}
.text-accent\/20 {
    color: hsl(var(--a) / 0.2)}
.text-accent\/25 {
    color: hsl(var(--a) / 0.25)}
.text-accent\/30 {
    color: hsl(var(--a) / 0.3)}
.text-accent\/40 {
    color: hsl(var(--a) / 0.4)}
.text-accent\/5 {
    color: hsl(var(--a) / 0.05)}
.text-accent\/50 {
    color: hsl(var(--a) / 0.5)}
.text-accent\/60 {
    color: hsl(var(--a) / 0.6)}
.text-accent\/70 {
    color: hsl(var(--a) / 0.7)}
.text-accent\/75 {
    color: hsl(var(--a) / 0.75)}
.text-accent\/80 {
    color: hsl(var(--a) / 0.8)}
.text-accent\/90 {
    color: hsl(var(--a) / 0.9)}
.text-accent\/95 {
    color: hsl(var(--a) / 0.95)}
.text-base-100 {
    color: hsl(var(--b1) / 1)}
.text-base-100\/0 {
    color: hsl(var(--b1) / 0)}
.text-base-100\/10 {
    color: hsl(var(--b1) / 0.1)}
.text-base-100\/100 {
    color: hsl(var(--b1) / 1)}
.text-base-100\/20 {
    color: hsl(var(--b1) / 0.2)}
.text-base-100\/25 {
    color: hsl(var(--b1) / 0.25)}
.text-base-100\/30 {
    color: hsl(var(--b1) / 0.3)}
.text-base-100\/40 {
    color: hsl(var(--b1) / 0.4)}
.text-base-100\/5 {
    color: hsl(var(--b1) / 0.05)}
.text-base-100\/50 {
    color: hsl(var(--b1) / 0.5)}
.text-base-100\/60 {
    color: hsl(var(--b1) / 0.6)}
.text-base-100\/70 {
    color: hsl(var(--b1) / 0.7)}
.text-base-100\/75 {
    color: hsl(var(--b1) / 0.75)}
.text-base-100\/80 {
    color: hsl(var(--b1) / 0.8)}
.text-base-100\/90 {
    color: hsl(var(--b1) / 0.9)}
.text-base-100\/95 {
    color: hsl(var(--b1) / 0.95)}
.text-base-200 {
    color: hsl(var(--b2) / 1)}
.text-base-200\/0 {
    color: hsl(var(--b2) / 0)}
.text-base-200\/10 {
    color: hsl(var(--b2) / 0.1)}
.text-base-200\/100 {
    color: hsl(var(--b2) / 1)}
.text-base-200\/20 {
    color: hsl(var(--b2) / 0.2)}
.text-base-200\/25 {
    color: hsl(var(--b2) / 0.25)}
.text-base-200\/30 {
    color: hsl(var(--b2) / 0.3)}
.text-base-200\/40 {
    color: hsl(var(--b2) / 0.4)}
.text-base-200\/5 {
    color: hsl(var(--b2) / 0.05)}
.text-base-200\/50 {
    color: hsl(var(--b2) / 0.5)}
.text-base-200\/60 {
    color: hsl(var(--b2) / 0.6)}
.text-base-200\/70 {
    color: hsl(var(--b2) / 0.7)}
.text-base-200\/75 {
    color: hsl(var(--b2) / 0.75)}
.text-base-200\/80 {
    color: hsl(var(--b2) / 0.8)}
.text-base-200\/90 {
    color: hsl(var(--b2) / 0.9)}
.text-base-200\/95 {
    color: hsl(var(--b2) / 0.95)}
.text-base-300 {
    color: hsl(var(--b3) / 1)}
.text-base-300\/0 {
    color: hsl(var(--b3) / 0)}
.text-base-300\/10 {
    color: hsl(var(--b3) / 0.1)}
.text-base-300\/100 {
    color: hsl(var(--b3) / 1)}
.text-base-300\/20 {
    color: hsl(var(--b3) / 0.2)}
.text-base-300\/25 {
    color: hsl(var(--b3) / 0.25)}
.text-base-300\/30 {
    color: hsl(var(--b3) / 0.3)}
.text-base-300\/40 {
    color: hsl(var(--b3) / 0.4)}
.text-base-300\/5 {
    color: hsl(var(--b3) / 0.05)}
.text-base-300\/50 {
    color: hsl(var(--b3) / 0.5)}
.text-base-300\/60 {
    color: hsl(var(--b3) / 0.6)}
.text-base-300\/70 {
    color: hsl(var(--b3) / 0.7)}
.text-base-300\/75 {
    color: hsl(var(--b3) / 0.75)}
.text-base-300\/80 {
    color: hsl(var(--b3) / 0.8)}
.text-base-300\/90 {
    color: hsl(var(--b3) / 0.9)}
.text-base-300\/95 {
    color: hsl(var(--b3) / 0.95)}
.text-base-content {
    color: hsl(var(--bc) / 1)}
.text-base-content\/0 {
    color: hsl(var(--bc) / 0)}
.text-base-content\/10 {
    color: hsl(var(--bc) / 0.1)}
.text-base-content\/100 {
    color: hsl(var(--bc) / 1)}
.text-base-content\/20 {
    color: hsl(var(--bc) / 0.2)}
.text-base-content\/25 {
    color: hsl(var(--bc) / 0.25)}
.text-base-content\/30 {
    color: hsl(var(--bc) / 0.3)}
.text-base-content\/40 {
    color: hsl(var(--bc) / 0.4)}
.text-base-content\/5 {
    color: hsl(var(--bc) / 0.05)}
.text-base-content\/50 {
    color: hsl(var(--bc) / 0.5)}
.text-base-content\/60 {
    color: hsl(var(--bc) / 0.6)}
.text-base-content\/70 {
    color: hsl(var(--bc) / 0.7)}
.text-base-content\/75 {
    color: hsl(var(--bc) / 0.75)}
.text-base-content\/80 {
    color: hsl(var(--bc) / 0.8)}
.text-base-content\/90 {
    color: hsl(var(--bc) / 0.9)}
.text-base-content\/95 {
    color: hsl(var(--bc) / 0.95)}
.text-current {
    color: currentColor}
.text-error {
    color: hsl(var(--er) / 1)}
.text-error-content {
    color: hsl(var(--erc) / 1)}
.text-error-content\/0 {
    color: hsl(var(--erc) / 0)}
.text-error-content\/10 {
    color: hsl(var(--erc) / 0.1)}
.text-error-content\/100 {
    color: hsl(var(--erc) / 1)}
.text-error-content\/20 {
    color: hsl(var(--erc) / 0.2)}
.text-error-content\/25 {
    color: hsl(var(--erc) / 0.25)}
.text-error-content\/30 {
    color: hsl(var(--erc) / 0.3)}
.text-error-content\/40 {
    color: hsl(var(--erc) / 0.4)}
.text-error-content\/5 {
    color: hsl(var(--erc) / 0.05)}
.text-error-content\/50 {
    color: hsl(var(--erc) / 0.5)}
.text-error-content\/60 {
    color: hsl(var(--erc) / 0.6)}
.text-error-content\/70 {
    color: hsl(var(--erc) / 0.7)}
.text-error-content\/75 {
    color: hsl(var(--erc) / 0.75)}
.text-error-content\/80 {
    color: hsl(var(--erc) / 0.8)}
.text-error-content\/90 {
    color: hsl(var(--erc) / 0.9)}
.text-error-content\/95 {
    color: hsl(var(--erc) / 0.95)}
.text-error\/0 {
    color: hsl(var(--er) / 0)}
.text-error\/10 {
    color: hsl(var(--er) / 0.1)}
.text-error\/100 {
    color: hsl(var(--er) / 1)}
.text-error\/20 {
    color: hsl(var(--er) / 0.2)}
.text-error\/25 {
    color: hsl(var(--er) / 0.25)}
.text-error\/30 {
    color: hsl(var(--er) / 0.3)}
.text-error\/40 {
    color: hsl(var(--er) / 0.4)}
.text-error\/5 {
    color: hsl(var(--er) / 0.05)}
.text-error\/50 {
    color: hsl(var(--er) / 0.5)}
.text-error\/60 {
    color: hsl(var(--er) / 0.6)}
.text-error\/70 {
    color: hsl(var(--er) / 0.7)}
.text-error\/75 {
    color: hsl(var(--er) / 0.75)}
.text-error\/80 {
    color: hsl(var(--er) / 0.8)}
.text-error\/90 {
    color: hsl(var(--er) / 0.9)}
.text-error\/95 {
    color: hsl(var(--er) / 0.95)}
.text-info {
    color: hsl(var(--in) / 1)}
.text-info-content {
    color: hsl(var(--inc) / 1)}
.text-info-content\/0 {
    color: hsl(var(--inc) / 0)}
.text-info-content\/10 {
    color: hsl(var(--inc) / 0.1)}
.text-info-content\/100 {
    color: hsl(var(--inc) / 1)}
.text-info-content\/20 {
    color: hsl(var(--inc) / 0.2)}
.text-info-content\/25 {
    color: hsl(var(--inc) / 0.25)}
.text-info-content\/30 {
    color: hsl(var(--inc) / 0.3)}
.text-info-content\/40 {
    color: hsl(var(--inc) / 0.4)}
.text-info-content\/5 {
    color: hsl(var(--inc) / 0.05)}
.text-info-content\/50 {
    color: hsl(var(--inc) / 0.5)}
.text-info-content\/60 {
    color: hsl(var(--inc) / 0.6)}
.text-info-content\/70 {
    color: hsl(var(--inc) / 0.7)}
.text-info-content\/75 {
    color: hsl(var(--inc) / 0.75)}
.text-info-content\/80 {
    color: hsl(var(--inc) / 0.8)}
.text-info-content\/90 {
    color: hsl(var(--inc) / 0.9)}
.text-info-content\/95 {
    color: hsl(var(--inc) / 0.95)}
.text-info\/0 {
    color: hsl(var(--in) / 0)}
.text-info\/10 {
    color: hsl(var(--in) / 0.1)}
.text-info\/100 {
    color: hsl(var(--in) / 1)}
.text-info\/20 {
    color: hsl(var(--in) / 0.2)}
.text-info\/25 {
    color: hsl(var(--in) / 0.25)}
.text-info\/30 {
    color: hsl(var(--in) / 0.3)}
.text-info\/40 {
    color: hsl(var(--in) / 0.4)}
.text-info\/5 {
    color: hsl(var(--in) / 0.05)}
.text-info\/50 {
    color: hsl(var(--in) / 0.5)}
.text-info\/60 {
    color: hsl(var(--in) / 0.6)}
.text-info\/70 {
    color: hsl(var(--in) / 0.7)}
.text-info\/75 {
    color: hsl(var(--in) / 0.75)}
.text-info\/80 {
    color: hsl(var(--in) / 0.8)}
.text-info\/90 {
    color: hsl(var(--in) / 0.9)}
.text-info\/95 {
    color: hsl(var(--in) / 0.95)}
.text-neutral {
    color: hsl(var(--n) / 1)}
.text-neutral-content {
    color: hsl(var(--nc) / 1)}
.text-neutral-content\/0 {
    color: hsl(var(--nc) / 0)}
.text-neutral-content\/10 {
    color: hsl(var(--nc) / 0.1)}
.text-neutral-content\/100 {
    color: hsl(var(--nc) / 1)}
.text-neutral-content\/20 {
    color: hsl(var(--nc) / 0.2)}
.text-neutral-content\/25 {
    color: hsl(var(--nc) / 0.25)}
.text-neutral-content\/30 {
    color: hsl(var(--nc) / 0.3)}
.text-neutral-content\/40 {
    color: hsl(var(--nc) / 0.4)}
.text-neutral-content\/5 {
    color: hsl(var(--nc) / 0.05)}
.text-neutral-content\/50 {
    color: hsl(var(--nc) / 0.5)}
.text-neutral-content\/60 {
    color: hsl(var(--nc) / 0.6)}
.text-neutral-content\/70 {
    color: hsl(var(--nc) / 0.7)}
.text-neutral-content\/75 {
    color: hsl(var(--nc) / 0.75)}
.text-neutral-content\/80 {
    color: hsl(var(--nc) / 0.8)}
.text-neutral-content\/90 {
    color: hsl(var(--nc) / 0.9)}
.text-neutral-content\/95 {
    color: hsl(var(--nc) / 0.95)}
.text-neutral-focus {
    color: hsl(var(--nf) / 1)}
.text-neutral-focus\/0 {
    color: hsl(var(--nf) / 0)}
.text-neutral-focus\/10 {
    color: hsl(var(--nf) / 0.1)}
.text-neutral-focus\/100 {
    color: hsl(var(--nf) / 1)}
.text-neutral-focus\/20 {
    color: hsl(var(--nf) / 0.2)}
.text-neutral-focus\/25 {
    color: hsl(var(--nf) / 0.25)}
.text-neutral-focus\/30 {
    color: hsl(var(--nf) / 0.3)}
.text-neutral-focus\/40 {
    color: hsl(var(--nf) / 0.4)}
.text-neutral-focus\/5 {
    color: hsl(var(--nf) / 0.05)}
.text-neutral-focus\/50 {
    color: hsl(var(--nf) / 0.5)}
.text-neutral-focus\/60 {
    color: hsl(var(--nf) / 0.6)}
.text-neutral-focus\/70 {
    color: hsl(var(--nf) / 0.7)}
.text-neutral-focus\/75 {
    color: hsl(var(--nf) / 0.75)}
.text-neutral-focus\/80 {
    color: hsl(var(--nf) / 0.8)}
.text-neutral-focus\/90 {
    color: hsl(var(--nf) / 0.9)}
.text-neutral-focus\/95 {
    color: hsl(var(--nf) / 0.95)}
.text-neutral\/0 {
    color: hsl(var(--n) / 0)}
.text-neutral\/10 {
    color: hsl(var(--n) / 0.1)}
.text-neutral\/100 {
    color: hsl(var(--n) / 1)}
.text-neutral\/20 {
    color: hsl(var(--n) / 0.2)}
.text-neutral\/25 {
    color: hsl(var(--n) / 0.25)}
.text-neutral\/30 {
    color: hsl(var(--n) / 0.3)}
.text-neutral\/40 {
    color: hsl(var(--n) / 0.4)}
.text-neutral\/5 {
    color: hsl(var(--n) / 0.05)}
.text-neutral\/50 {
    color: hsl(var(--n) / 0.5)}
.text-neutral\/60 {
    color: hsl(var(--n) / 0.6)}
.text-neutral\/70 {
    color: hsl(var(--n) / 0.7)}
.text-neutral\/75 {
    color: hsl(var(--n) / 0.75)}
.text-neutral\/80 {
    color: hsl(var(--n) / 0.8)}
.text-neutral\/90 {
    color: hsl(var(--n) / 0.9)}
.text-neutral\/95 {
    color: hsl(var(--n) / 0.95)}
.text-primary {
    color: hsl(var(--p) / 1)}
.text-primary-content {
    color: hsl(var(--pc) / 1)}
.text-primary-content\/0 {
    color: hsl(var(--pc) / 0)}
.text-primary-content\/10 {
    color: hsl(var(--pc) / 0.1)}
.text-primary-content\/100 {
    color: hsl(var(--pc) / 1)}
.text-primary-content\/20 {
    color: hsl(var(--pc) / 0.2)}
.text-primary-content\/25 {
    color: hsl(var(--pc) / 0.25)}
.text-primary-content\/30 {
    color: hsl(var(--pc) / 0.3)}
.text-primary-content\/40 {
    color: hsl(var(--pc) / 0.4)}
.text-primary-content\/5 {
    color: hsl(var(--pc) / 0.05)}
.text-primary-content\/50 {
    color: hsl(var(--pc) / 0.5)}
.text-primary-content\/60 {
    color: hsl(var(--pc) / 0.6)}
.text-primary-content\/70 {
    color: hsl(var(--pc) / 0.7)}
.text-primary-content\/75 {
    color: hsl(var(--pc) / 0.75)}
.text-primary-content\/80 {
    color: hsl(var(--pc) / 0.8)}
.text-primary-content\/90 {
    color: hsl(var(--pc) / 0.9)}
.text-primary-content\/95 {
    color: hsl(var(--pc) / 0.95)}
.text-primary-focus {
    color: hsl(var(--pf) / 1)}
.text-primary-focus\/0 {
    color: hsl(var(--pf) / 0)}
.text-primary-focus\/10 {
    color: hsl(var(--pf) / 0.1)}
.text-primary-focus\/100 {
    color: hsl(var(--pf) / 1)}
.text-primary-focus\/20 {
    color: hsl(var(--pf) / 0.2)}
.text-primary-focus\/25 {
    color: hsl(var(--pf) / 0.25)}
.text-primary-focus\/30 {
    color: hsl(var(--pf) / 0.3)}
.text-primary-focus\/40 {
    color: hsl(var(--pf) / 0.4)}
.text-primary-focus\/5 {
    color: hsl(var(--pf) / 0.05)}
.text-primary-focus\/50 {
    color: hsl(var(--pf) / 0.5)}
.text-primary-focus\/60 {
    color: hsl(var(--pf) / 0.6)}
.text-primary-focus\/70 {
    color: hsl(var(--pf) / 0.7)}
.text-primary-focus\/75 {
    color: hsl(var(--pf) / 0.75)}
.text-primary-focus\/80 {
    color: hsl(var(--pf) / 0.8)}
.text-primary-focus\/90 {
    color: hsl(var(--pf) / 0.9)}
.text-primary-focus\/95 {
    color: hsl(var(--pf) / 0.95)}
.text-primary\/0 {
    color: hsl(var(--p) / 0)}
.text-primary\/10 {
    color: hsl(var(--p) / 0.1)}
.text-primary\/100 {
    color: hsl(var(--p) / 1)}
.text-primary\/20 {
    color: hsl(var(--p) / 0.2)}
.text-primary\/25 {
    color: hsl(var(--p) / 0.25)}
.text-primary\/30 {
    color: hsl(var(--p) / 0.3)}
.text-primary\/40 {
    color: hsl(var(--p) / 0.4)}
.text-primary\/5 {
    color: hsl(var(--p) / 0.05)}
.text-primary\/50 {
    color: hsl(var(--p) / 0.5)}
.text-primary\/60 {
    color: hsl(var(--p) / 0.6)}
.text-primary\/70 {
    color: hsl(var(--p) / 0.7)}
.text-primary\/75 {
    color: hsl(var(--p) / 0.75)}
.text-primary\/80 {
    color: hsl(var(--p) / 0.8)}
.text-primary\/90 {
    color: hsl(var(--p) / 0.9)}
.text-primary\/95 {
    color: hsl(var(--p) / 0.95)}
.text-secondary {
    color: hsl(var(--s) / 1)}
.text-secondary-content {
    color: hsl(var(--sc) / 1)}
.text-secondary-content\/0 {
    color: hsl(var(--sc) / 0)}
.text-secondary-content\/10 {
    color: hsl(var(--sc) / 0.1)}
.text-secondary-content\/100 {
    color: hsl(var(--sc) / 1)}
.text-secondary-content\/20 {
    color: hsl(var(--sc) / 0.2)}
.text-secondary-content\/25 {
    color: hsl(var(--sc) / 0.25)}
.text-secondary-content\/30 {
    color: hsl(var(--sc) / 0.3)}
.text-secondary-content\/40 {
    color: hsl(var(--sc) / 0.4)}
.text-secondary-content\/5 {
    color: hsl(var(--sc) / 0.05)}
.text-secondary-content\/50 {
    color: hsl(var(--sc) / 0.5)}
.text-secondary-content\/60 {
    color: hsl(var(--sc) / 0.6)}
.text-secondary-content\/70 {
    color: hsl(var(--sc) / 0.7)}
.text-secondary-content\/75 {
    color: hsl(var(--sc) / 0.75)}
.text-secondary-content\/80 {
    color: hsl(var(--sc) / 0.8)}
.text-secondary-content\/90 {
    color: hsl(var(--sc) / 0.9)}
.text-secondary-content\/95 {
    color: hsl(var(--sc) / 0.95)}
.text-secondary-focus {
    color: hsl(var(--sf) / 1)}
.text-secondary-focus\/0 {
    color: hsl(var(--sf) / 0)}
.text-secondary-focus\/10 {
    color: hsl(var(--sf) / 0.1)}
.text-secondary-focus\/100 {
    color: hsl(var(--sf) / 1)}
.text-secondary-focus\/20 {
    color: hsl(var(--sf) / 0.2)}
.text-secondary-focus\/25 {
    color: hsl(var(--sf) / 0.25)}
.text-secondary-focus\/30 {
    color: hsl(var(--sf) / 0.3)}
.text-secondary-focus\/40 {
    color: hsl(var(--sf) / 0.4)}
.text-secondary-focus\/5 {
    color: hsl(var(--sf) / 0.05)}
.text-secondary-focus\/50 {
    color: hsl(var(--sf) / 0.5)}
.text-secondary-focus\/60 {
    color: hsl(var(--sf) / 0.6)}
.text-secondary-focus\/70 {
    color: hsl(var(--sf) / 0.7)}
.text-secondary-focus\/75 {
    color: hsl(var(--sf) / 0.75)}
.text-secondary-focus\/80 {
    color: hsl(var(--sf) / 0.8)}
.text-secondary-focus\/90 {
    color: hsl(var(--sf) / 0.9)}
.text-secondary-focus\/95 {
    color: hsl(var(--sf) / 0.95)}
.text-secondary\/0 {
    color: hsl(var(--s) / 0)}
.text-secondary\/10 {
    color: hsl(var(--s) / 0.1)}
.text-secondary\/100 {
    color: hsl(var(--s) / 1)}
.text-secondary\/20 {
    color: hsl(var(--s) / 0.2)}
.text-secondary\/25 {
    color: hsl(var(--s) / 0.25)}
.text-secondary\/30 {
    color: hsl(var(--s) / 0.3)}
.text-secondary\/40 {
    color: hsl(var(--s) / 0.4)}
.text-secondary\/5 {
    color: hsl(var(--s) / 0.05)}
.text-secondary\/50 {
    color: hsl(var(--s) / 0.5)}
.text-secondary\/60 {
    color: hsl(var(--s) / 0.6)}
.text-secondary\/70 {
    color: hsl(var(--s) / 0.7)}
.text-secondary\/75 {
    color: hsl(var(--s) / 0.75)}
.text-secondary\/80 {
    color: hsl(var(--s) / 0.8)}
.text-secondary\/90 {
    color: hsl(var(--s) / 0.9)}
.text-secondary\/95 {
    color: hsl(var(--s) / 0.95)}
.text-success {
    color: hsl(var(--su) / 1)}
.text-success-content {
    color: hsl(var(--suc) / 1)}
.text-success-content\/0 {
    color: hsl(var(--suc) / 0)}
.text-success-content\/10 {
    color: hsl(var(--suc) / 0.1)}
.text-success-content\/100 {
    color: hsl(var(--suc) / 1)}
.text-success-content\/20 {
    color: hsl(var(--suc) / 0.2)}
.text-success-content\/25 {
    color: hsl(var(--suc) / 0.25)}
.text-success-content\/30 {
    color: hsl(var(--suc) / 0.3)}
.text-success-content\/40 {
    color: hsl(var(--suc) / 0.4)}
.text-success-content\/5 {
    color: hsl(var(--suc) / 0.05)}
.text-success-content\/50 {
    color: hsl(var(--suc) / 0.5)}
.text-success-content\/60 {
    color: hsl(var(--suc) / 0.6)}
.text-success-content\/70 {
    color: hsl(var(--suc) / 0.7)}
.text-success-content\/75 {
    color: hsl(var(--suc) / 0.75)}
.text-success-content\/80 {
    color: hsl(var(--suc) / 0.8)}
.text-success-content\/90 {
    color: hsl(var(--suc) / 0.9)}
.text-success-content\/95 {
    color: hsl(var(--suc) / 0.95)}
.text-success\/0 {
    color: hsl(var(--su) / 0)}
.text-success\/10 {
    color: hsl(var(--su) / 0.1)}
.text-success\/100 {
    color: hsl(var(--su) / 1)}
.text-success\/20 {
    color: hsl(var(--su) / 0.2)}
.text-success\/25 {
    color: hsl(var(--su) / 0.25)}
.text-success\/30 {
    color: hsl(var(--su) / 0.3)}
.text-success\/40 {
    color: hsl(var(--su) / 0.4)}
.text-success\/5 {
    color: hsl(var(--su) / 0.05)}
.text-success\/50 {
    color: hsl(var(--su) / 0.5)}
.text-success\/60 {
    color: hsl(var(--su) / 0.6)}
.text-success\/70 {
    color: hsl(var(--su) / 0.7)}
.text-success\/75 {
    color: hsl(var(--su) / 0.75)}
.text-success\/80 {
    color: hsl(var(--su) / 0.8)}
.text-success\/90 {
    color: hsl(var(--su) / 0.9)}
.text-success\/95 {
    color: hsl(var(--su) / 0.95)}
.text-transparent {
    color: transparent}
.text-transparent\/0 {
    color: rgb(0 0 0 / 0)}
.text-transparent\/10 {
    color: rgb(0 0 0 / 0.1)}
.text-transparent\/100 {
    color: rgb(0 0 0 / 1)}
.text-transparent\/20 {
    color: rgb(0 0 0 / 0.2)}
.text-transparent\/25 {
    color: rgb(0 0 0 / 0.25)}
.text-transparent\/30 {
    color: rgb(0 0 0 / 0.3)}
.text-transparent\/40 {
    color: rgb(0 0 0 / 0.4)}
.text-transparent\/5 {
    color: rgb(0 0 0 / 0.05)}
.text-transparent\/50 {
    color: rgb(0 0 0 / 0.5)}
.text-transparent\/60 {
    color: rgb(0 0 0 / 0.6)}
.text-transparent\/70 {
    color: rgb(0 0 0 / 0.7)}
.text-transparent\/75 {
    color: rgb(0 0 0 / 0.75)}
.text-transparent\/80 {
    color: rgb(0 0 0 / 0.8)}
.text-transparent\/90 {
    color: rgb(0 0 0 / 0.9)}
.text-transparent\/95 {
    color: rgb(0 0 0 / 0.95)}
.text-warning {
    color: hsl(var(--wa) / 1)}
.text-warning-content {
    color: hsl(var(--wac) / 1)}
.text-warning-content\/0 {
    color: hsl(var(--wac) / 0)}
.text-warning-content\/10 {
    color: hsl(var(--wac) / 0.1)}
.text-warning-content\/100 {
    color: hsl(var(--wac) / 1)}
.text-warning-content\/20 {
    color: hsl(var(--wac) / 0.2)}
.text-warning-content\/25 {
    color: hsl(var(--wac) / 0.25)}
.text-warning-content\/30 {
    color: hsl(var(--wac) / 0.3)}
.text-warning-content\/40 {
    color: hsl(var(--wac) / 0.4)}
.text-warning-content\/5 {
    color: hsl(var(--wac) / 0.05)}
.text-warning-content\/50 {
    color: hsl(var(--wac) / 0.5)}
.text-warning-content\/60 {
    color: hsl(var(--wac) / 0.6)}
.text-warning-content\/70 {
    color: hsl(var(--wac) / 0.7)}
.text-warning-content\/75 {
    color: hsl(var(--wac) / 0.75)}
.text-warning-content\/80 {
    color: hsl(var(--wac) / 0.8)}
.text-warning-content\/90 {
    color: hsl(var(--wac) / 0.9)}
.text-warning-content\/95 {
    color: hsl(var(--wac) / 0.95)}
.text-warning\/0 {
    color: hsl(var(--wa) / 0)}
.text-warning\/10 {
    color: hsl(var(--wa) / 0.1)}
.text-warning\/100 {
    color: hsl(var(--wa) / 1)}
.text-warning\/20 {
    color: hsl(var(--wa) / 0.2)}
.text-warning\/25 {
    color: hsl(var(--wa) / 0.25)}
.text-warning\/30 {
    color: hsl(var(--wa) / 0.3)}
.text-warning\/40 {
    color: hsl(var(--wa) / 0.4)}
.text-warning\/5 {
    color: hsl(var(--wa) / 0.05)}
.text-warning\/50 {
    color: hsl(var(--wa) / 0.5)}
.text-warning\/60 {
    color: hsl(var(--wa) / 0.6)}
.text-warning\/70 {
    color: hsl(var(--wa) / 0.7)}
.text-warning\/75 {
    color: hsl(var(--wa) / 0.75)}
.text-warning\/80 {
    color: hsl(var(--wa) / 0.8)}
.text-warning\/90 {
    color: hsl(var(--wa) / 0.9)}
.text-warning\/95 {
    color: hsl(var(--wa) / 0.95)}
.placeholder-accent::placeholder {
    color: hsl(var(--a) / 1)}
.placeholder-accent-content::placeholder {
    color: hsl(var(--ac) / 1)}
.placeholder-accent-content\/0::placeholder {
    color: hsl(var(--ac) / 0)}
.placeholder-accent-content\/10::placeholder {
    color: hsl(var(--ac) / 0.1)}
.placeholder-accent-content\/100::placeholder {
    color: hsl(var(--ac) / 1)}
.placeholder-accent-content\/20::placeholder {
    color: hsl(var(--ac) / 0.2)}
.placeholder-accent-content\/25::placeholder {
    color: hsl(var(--ac) / 0.25)}
.placeholder-accent-content\/30::placeholder {
    color: hsl(var(--ac) / 0.3)}
.placeholder-accent-content\/40::placeholder {
    color: hsl(var(--ac) / 0.4)}
.placeholder-accent-content\/5::placeholder {
    color: hsl(var(--ac) / 0.05)}
.placeholder-accent-content\/50::placeholder {
    color: hsl(var(--ac) / 0.5)}
.placeholder-accent-content\/60::placeholder {
    color: hsl(var(--ac) / 0.6)}
.placeholder-accent-content\/70::placeholder {
    color: hsl(var(--ac) / 0.7)}
.placeholder-accent-content\/75::placeholder {
    color: hsl(var(--ac) / 0.75)}
.placeholder-accent-content\/80::placeholder {
    color: hsl(var(--ac) / 0.8)}
.placeholder-accent-content\/90::placeholder {
    color: hsl(var(--ac) / 0.9)}
.placeholder-accent-content\/95::placeholder {
    color: hsl(var(--ac) / 0.95)}
.placeholder-accent-focus::placeholder {
    color: hsl(var(--af) / 1)}
.placeholder-accent-focus\/0::placeholder {
    color: hsl(var(--af) / 0)}
.placeholder-accent-focus\/10::placeholder {
    color: hsl(var(--af) / 0.1)}
.placeholder-accent-focus\/100::placeholder {
    color: hsl(var(--af) / 1)}
.placeholder-accent-focus\/20::placeholder {
    color: hsl(var(--af) / 0.2)}
.placeholder-accent-focus\/25::placeholder {
    color: hsl(var(--af) / 0.25)}
.placeholder-accent-focus\/30::placeholder {
    color: hsl(var(--af) / 0.3)}
.placeholder-accent-focus\/40::placeholder {
    color: hsl(var(--af) / 0.4)}
.placeholder-accent-focus\/5::placeholder {
    color: hsl(var(--af) / 0.05)}
.placeholder-accent-focus\/50::placeholder {
    color: hsl(var(--af) / 0.5)}
.placeholder-accent-focus\/60::placeholder {
    color: hsl(var(--af) / 0.6)}
.placeholder-accent-focus\/70::placeholder {
    color: hsl(var(--af) / 0.7)}
.placeholder-accent-focus\/75::placeholder {
    color: hsl(var(--af) / 0.75)}
.placeholder-accent-focus\/80::placeholder {
    color: hsl(var(--af) / 0.8)}
.placeholder-accent-focus\/90::placeholder {
    color: hsl(var(--af) / 0.9)}
.placeholder-accent-focus\/95::placeholder {
    color: hsl(var(--af) / 0.95)}
.placeholder-accent\/0::placeholder {
    color: hsl(var(--a) / 0)}
.placeholder-accent\/10::placeholder {
    color: hsl(var(--a) / 0.1)}
.placeholder-accent\/100::placeholder {
    color: hsl(var(--a) / 1)}
.placeholder-accent\/20::placeholder {
    color: hsl(var(--a) / 0.2)}
.placeholder-accent\/25::placeholder {
    color: hsl(var(--a) / 0.25)}
.placeholder-accent\/30::placeholder {
    color: hsl(var(--a) / 0.3)}
.placeholder-accent\/40::placeholder {
    color: hsl(var(--a) / 0.4)}
.placeholder-accent\/5::placeholder {
    color: hsl(var(--a) / 0.05)}
.placeholder-accent\/50::placeholder {
    color: hsl(var(--a) / 0.5)}
.placeholder-accent\/60::placeholder {
    color: hsl(var(--a) / 0.6)}
.placeholder-accent\/70::placeholder {
    color: hsl(var(--a) / 0.7)}
.placeholder-accent\/75::placeholder {
    color: hsl(var(--a) / 0.75)}
.placeholder-accent\/80::placeholder {
    color: hsl(var(--a) / 0.8)}
.placeholder-accent\/90::placeholder {
    color: hsl(var(--a) / 0.9)}
.placeholder-accent\/95::placeholder {
    color: hsl(var(--a) / 0.95)}
.placeholder-base-100::placeholder {
    color: hsl(var(--b1) / 1)}
.placeholder-base-100\/0::placeholder {
    color: hsl(var(--b1) / 0)}
.placeholder-base-100\/10::placeholder {
    color: hsl(var(--b1) / 0.1)}
.placeholder-base-100\/100::placeholder {
    color: hsl(var(--b1) / 1)}
.placeholder-base-100\/20::placeholder {
    color: hsl(var(--b1) / 0.2)}
.placeholder-base-100\/25::placeholder {
    color: hsl(var(--b1) / 0.25)}
.placeholder-base-100\/30::placeholder {
    color: hsl(var(--b1) / 0.3)}
.placeholder-base-100\/40::placeholder {
    color: hsl(var(--b1) / 0.4)}
.placeholder-base-100\/5::placeholder {
    color: hsl(var(--b1) / 0.05)}
.placeholder-base-100\/50::placeholder {
    color: hsl(var(--b1) / 0.5)}
.placeholder-base-100\/60::placeholder {
    color: hsl(var(--b1) / 0.6)}
.placeholder-base-100\/70::placeholder {
    color: hsl(var(--b1) / 0.7)}
.placeholder-base-100\/75::placeholder {
    color: hsl(var(--b1) / 0.75)}
.placeholder-base-100\/80::placeholder {
    color: hsl(var(--b1) / 0.8)}
.placeholder-base-100\/90::placeholder {
    color: hsl(var(--b1) / 0.9)}
.placeholder-base-100\/95::placeholder {
    color: hsl(var(--b1) / 0.95)}
.placeholder-base-200::placeholder {
    color: hsl(var(--b2) / 1)}
.placeholder-base-200\/0::placeholder {
    color: hsl(var(--b2) / 0)}
.placeholder-base-200\/10::placeholder {
    color: hsl(var(--b2) / 0.1)}
.placeholder-base-200\/100::placeholder {
    color: hsl(var(--b2) / 1)}
.placeholder-base-200\/20::placeholder {
    color: hsl(var(--b2) / 0.2)}
.placeholder-base-200\/25::placeholder {
    color: hsl(var(--b2) / 0.25)}
.placeholder-base-200\/30::placeholder {
    color: hsl(var(--b2) / 0.3)}
.placeholder-base-200\/40::placeholder {
    color: hsl(var(--b2) / 0.4)}
.placeholder-base-200\/5::placeholder {
    color: hsl(var(--b2) / 0.05)}
.placeholder-base-200\/50::placeholder {
    color: hsl(var(--b2) / 0.5)}
.placeholder-base-200\/60::placeholder {
    color: hsl(var(--b2) / 0.6)}
.placeholder-base-200\/70::placeholder {
    color: hsl(var(--b2) / 0.7)}
.placeholder-base-200\/75::placeholder {
    color: hsl(var(--b2) / 0.75)}
.placeholder-base-200\/80::placeholder {
    color: hsl(var(--b2) / 0.8)}
.placeholder-base-200\/90::placeholder {
    color: hsl(var(--b2) / 0.9)}
.placeholder-base-200\/95::placeholder {
    color: hsl(var(--b2) / 0.95)}
.placeholder-base-300::placeholder {
    color: hsl(var(--b3) / 1)}
.placeholder-base-300\/0::placeholder {
    color: hsl(var(--b3) / 0)}
.placeholder-base-300\/10::placeholder {
    color: hsl(var(--b3) / 0.1)}
.placeholder-base-300\/100::placeholder {
    color: hsl(var(--b3) / 1)}
.placeholder-base-300\/20::placeholder {
    color: hsl(var(--b3) / 0.2)}
.placeholder-base-300\/25::placeholder {
    color: hsl(var(--b3) / 0.25)}
.placeholder-base-300\/30::placeholder {
    color: hsl(var(--b3) / 0.3)}
.placeholder-base-300\/40::placeholder {
    color: hsl(var(--b3) / 0.4)}
.placeholder-base-300\/5::placeholder {
    color: hsl(var(--b3) / 0.05)}
.placeholder-base-300\/50::placeholder {
    color: hsl(var(--b3) / 0.5)}
.placeholder-base-300\/60::placeholder {
    color: hsl(var(--b3) / 0.6)}
.placeholder-base-300\/70::placeholder {
    color: hsl(var(--b3) / 0.7)}
.placeholder-base-300\/75::placeholder {
    color: hsl(var(--b3) / 0.75)}
.placeholder-base-300\/80::placeholder {
    color: hsl(var(--b3) / 0.8)}
.placeholder-base-300\/90::placeholder {
    color: hsl(var(--b3) / 0.9)}
.placeholder-base-300\/95::placeholder {
    color: hsl(var(--b3) / 0.95)}
.placeholder-base-content::placeholder {
    color: hsl(var(--bc) / 1)}
.placeholder-base-content\/0::placeholder {
    color: hsl(var(--bc) / 0)}
.placeholder-base-content\/10::placeholder {
    color: hsl(var(--bc) / 0.1)}
.placeholder-base-content\/100::placeholder {
    color: hsl(var(--bc) / 1)}
.placeholder-base-content\/20::placeholder {
    color: hsl(var(--bc) / 0.2)}
.placeholder-base-content\/25::placeholder {
    color: hsl(var(--bc) / 0.25)}
.placeholder-base-content\/30::placeholder {
    color: hsl(var(--bc) / 0.3)}
.placeholder-base-content\/40::placeholder {
    color: hsl(var(--bc) / 0.4)}
.placeholder-base-content\/5::placeholder {
    color: hsl(var(--bc) / 0.05)}
.placeholder-base-content\/50::placeholder {
    color: hsl(var(--bc) / 0.5)}
.placeholder-base-content\/60::placeholder {
    color: hsl(var(--bc) / 0.6)}
.placeholder-base-content\/70::placeholder {
    color: hsl(var(--bc) / 0.7)}
.placeholder-base-content\/75::placeholder {
    color: hsl(var(--bc) / 0.75)}
.placeholder-base-content\/80::placeholder {
    color: hsl(var(--bc) / 0.8)}
.placeholder-base-content\/90::placeholder {
    color: hsl(var(--bc) / 0.9)}
.placeholder-base-content\/95::placeholder {
    color: hsl(var(--bc) / 0.95)}
.placeholder-current::placeholder {
    color: currentColor}
.placeholder-error::placeholder {
    color: hsl(var(--er) / 1)}
.placeholder-error-content::placeholder {
    color: hsl(var(--erc) / 1)}
.placeholder-error-content\/0::placeholder {
    color: hsl(var(--erc) / 0)}
.placeholder-error-content\/10::placeholder {
    color: hsl(var(--erc) / 0.1)}
.placeholder-error-content\/100::placeholder {
    color: hsl(var(--erc) / 1)}
.placeholder-error-content\/20::placeholder {
    color: hsl(var(--erc) / 0.2)}
.placeholder-error-content\/25::placeholder {
    color: hsl(var(--erc) / 0.25)}
.placeholder-error-content\/30::placeholder {
    color: hsl(var(--erc) / 0.3)}
.placeholder-error-content\/40::placeholder {
    color: hsl(var(--erc) / 0.4)}
.placeholder-error-content\/5::placeholder {
    color: hsl(var(--erc) / 0.05)}
.placeholder-error-content\/50::placeholder {
    color: hsl(var(--erc) / 0.5)}
.placeholder-error-content\/60::placeholder {
    color: hsl(var(--erc) / 0.6)}
.placeholder-error-content\/70::placeholder {
    color: hsl(var(--erc) / 0.7)}
.placeholder-error-content\/75::placeholder {
    color: hsl(var(--erc) / 0.75)}
.placeholder-error-content\/80::placeholder {
    color: hsl(var(--erc) / 0.8)}
.placeholder-error-content\/90::placeholder {
    color: hsl(var(--erc) / 0.9)}
.placeholder-error-content\/95::placeholder {
    color: hsl(var(--erc) / 0.95)}
.placeholder-error\/0::placeholder {
    color: hsl(var(--er) / 0)}
.placeholder-error\/10::placeholder {
    color: hsl(var(--er) / 0.1)}
.placeholder-error\/100::placeholder {
    color: hsl(var(--er) / 1)}
.placeholder-error\/20::placeholder {
    color: hsl(var(--er) / 0.2)}
.placeholder-error\/25::placeholder {
    color: hsl(var(--er) / 0.25)}
.placeholder-error\/30::placeholder {
    color: hsl(var(--er) / 0.3)}
.placeholder-error\/40::placeholder {
    color: hsl(var(--er) / 0.4)}
.placeholder-error\/5::placeholder {
    color: hsl(var(--er) / 0.05)}
.placeholder-error\/50::placeholder {
    color: hsl(var(--er) / 0.5)}
.placeholder-error\/60::placeholder {
    color: hsl(var(--er) / 0.6)}
.placeholder-error\/70::placeholder {
    color: hsl(var(--er) / 0.7)}
.placeholder-error\/75::placeholder {
    color: hsl(var(--er) / 0.75)}
.placeholder-error\/80::placeholder {
    color: hsl(var(--er) / 0.8)}
.placeholder-error\/90::placeholder {
    color: hsl(var(--er) / 0.9)}
.placeholder-error\/95::placeholder {
    color: hsl(var(--er) / 0.95)}
.placeholder-info::placeholder {
    color: hsl(var(--in) / 1)}
.placeholder-info-content::placeholder {
    color: hsl(var(--inc) / 1)}
.placeholder-info-content\/0::placeholder {
    color: hsl(var(--inc) / 0)}
.placeholder-info-content\/10::placeholder {
    color: hsl(var(--inc) / 0.1)}
.placeholder-info-content\/100::placeholder {
    color: hsl(var(--inc) / 1)}
.placeholder-info-content\/20::placeholder {
    color: hsl(var(--inc) / 0.2)}
.placeholder-info-content\/25::placeholder {
    color: hsl(var(--inc) / 0.25)}
.placeholder-info-content\/30::placeholder {
    color: hsl(var(--inc) / 0.3)}
.placeholder-info-content\/40::placeholder {
    color: hsl(var(--inc) / 0.4)}
.placeholder-info-content\/5::placeholder {
    color: hsl(var(--inc) / 0.05)}
.placeholder-info-content\/50::placeholder {
    color: hsl(var(--inc) / 0.5)}
.placeholder-info-content\/60::placeholder {
    color: hsl(var(--inc) / 0.6)}
.placeholder-info-content\/70::placeholder {
    color: hsl(var(--inc) / 0.7)}
.placeholder-info-content\/75::placeholder {
    color: hsl(var(--inc) / 0.75)}
.placeholder-info-content\/80::placeholder {
    color: hsl(var(--inc) / 0.8)}
.placeholder-info-content\/90::placeholder {
    color: hsl(var(--inc) / 0.9)}
.placeholder-info-content\/95::placeholder {
    color: hsl(var(--inc) / 0.95)}
.placeholder-info\/0::placeholder {
    color: hsl(var(--in) / 0)}
.placeholder-info\/10::placeholder {
    color: hsl(var(--in) / 0.1)}
.placeholder-info\/100::placeholder {
    color: hsl(var(--in) / 1)}
.placeholder-info\/20::placeholder {
    color: hsl(var(--in) / 0.2)}
.placeholder-info\/25::placeholder {
    color: hsl(var(--in) / 0.25)}
.placeholder-info\/30::placeholder {
    color: hsl(var(--in) / 0.3)}
.placeholder-info\/40::placeholder {
    color: hsl(var(--in) / 0.4)}
.placeholder-info\/5::placeholder {
    color: hsl(var(--in) / 0.05)}
.placeholder-info\/50::placeholder {
    color: hsl(var(--in) / 0.5)}
.placeholder-info\/60::placeholder {
    color: hsl(var(--in) / 0.6)}
.placeholder-info\/70::placeholder {
    color: hsl(var(--in) / 0.7)}
.placeholder-info\/75::placeholder {
    color: hsl(var(--in) / 0.75)}
.placeholder-info\/80::placeholder {
    color: hsl(var(--in) / 0.8)}
.placeholder-info\/90::placeholder {
    color: hsl(var(--in) / 0.9)}
.placeholder-info\/95::placeholder {
    color: hsl(var(--in) / 0.95)}
.placeholder-neutral::placeholder {
    color: hsl(var(--n) / 1)}
.placeholder-neutral-content::placeholder {
    color: hsl(var(--nc) / 1)}
.placeholder-neutral-content\/0::placeholder {
    color: hsl(var(--nc) / 0)}
.placeholder-neutral-content\/10::placeholder {
    color: hsl(var(--nc) / 0.1)}
.placeholder-neutral-content\/100::placeholder {
    color: hsl(var(--nc) / 1)}
.placeholder-neutral-content\/20::placeholder {
    color: hsl(var(--nc) / 0.2)}
.placeholder-neutral-content\/25::placeholder {
    color: hsl(var(--nc) / 0.25)}
.placeholder-neutral-content\/30::placeholder {
    color: hsl(var(--nc) / 0.3)}
.placeholder-neutral-content\/40::placeholder {
    color: hsl(var(--nc) / 0.4)}
.placeholder-neutral-content\/5::placeholder {
    color: hsl(var(--nc) / 0.05)}
.placeholder-neutral-content\/50::placeholder {
    color: hsl(var(--nc) / 0.5)}
.placeholder-neutral-content\/60::placeholder {
    color: hsl(var(--nc) / 0.6)}
.placeholder-neutral-content\/70::placeholder {
    color: hsl(var(--nc) / 0.7)}
.placeholder-neutral-content\/75::placeholder {
    color: hsl(var(--nc) / 0.75)}
.placeholder-neutral-content\/80::placeholder {
    color: hsl(var(--nc) / 0.8)}
.placeholder-neutral-content\/90::placeholder {
    color: hsl(var(--nc) / 0.9)}
.placeholder-neutral-content\/95::placeholder {
    color: hsl(var(--nc) / 0.95)}
.placeholder-neutral-focus::placeholder {
    color: hsl(var(--nf) / 1)}
.placeholder-neutral-focus\/0::placeholder {
    color: hsl(var(--nf) / 0)}
.placeholder-neutral-focus\/10::placeholder {
    color: hsl(var(--nf) / 0.1)}
.placeholder-neutral-focus\/100::placeholder {
    color: hsl(var(--nf) / 1)}
.placeholder-neutral-focus\/20::placeholder {
    color: hsl(var(--nf) / 0.2)}
.placeholder-neutral-focus\/25::placeholder {
    color: hsl(var(--nf) / 0.25)}
.placeholder-neutral-focus\/30::placeholder {
    color: hsl(var(--nf) / 0.3)}
.placeholder-neutral-focus\/40::placeholder {
    color: hsl(var(--nf) / 0.4)}
.placeholder-neutral-focus\/5::placeholder {
    color: hsl(var(--nf) / 0.05)}
.placeholder-neutral-focus\/50::placeholder {
    color: hsl(var(--nf) / 0.5)}
.placeholder-neutral-focus\/60::placeholder {
    color: hsl(var(--nf) / 0.6)}
.placeholder-neutral-focus\/70::placeholder {
    color: hsl(var(--nf) / 0.7)}
.placeholder-neutral-focus\/75::placeholder {
    color: hsl(var(--nf) / 0.75)}
.placeholder-neutral-focus\/80::placeholder {
    color: hsl(var(--nf) / 0.8)}
.placeholder-neutral-focus\/90::placeholder {
    color: hsl(var(--nf) / 0.9)}
.placeholder-neutral-focus\/95::placeholder {
    color: hsl(var(--nf) / 0.95)}
.placeholder-neutral\/0::placeholder {
    color: hsl(var(--n) / 0)}
.placeholder-neutral\/10::placeholder {
    color: hsl(var(--n) / 0.1)}
.placeholder-neutral\/100::placeholder {
    color: hsl(var(--n) / 1)}
.placeholder-neutral\/20::placeholder {
    color: hsl(var(--n) / 0.2)}
.placeholder-neutral\/25::placeholder {
    color: hsl(var(--n) / 0.25)}
.placeholder-neutral\/30::placeholder {
    color: hsl(var(--n) / 0.3)}
.placeholder-neutral\/40::placeholder {
    color: hsl(var(--n) / 0.4)}
.placeholder-neutral\/5::placeholder {
    color: hsl(var(--n) / 0.05)}
.placeholder-neutral\/50::placeholder {
    color: hsl(var(--n) / 0.5)}
.placeholder-neutral\/60::placeholder {
    color: hsl(var(--n) / 0.6)}
.placeholder-neutral\/70::placeholder {
    color: hsl(var(--n) / 0.7)}
.placeholder-neutral\/75::placeholder {
    color: hsl(var(--n) / 0.75)}
.placeholder-neutral\/80::placeholder {
    color: hsl(var(--n) / 0.8)}
.placeholder-neutral\/90::placeholder {
    color: hsl(var(--n) / 0.9)}
.placeholder-neutral\/95::placeholder {
    color: hsl(var(--n) / 0.95)}
.placeholder-primary::placeholder {
    color: hsl(var(--p) / 1)}
.placeholder-primary-content::placeholder {
    color: hsl(var(--pc) / 1)}
.placeholder-primary-content\/0::placeholder {
    color: hsl(var(--pc) / 0)}
.placeholder-primary-content\/10::placeholder {
    color: hsl(var(--pc) / 0.1)}
.placeholder-primary-content\/100::placeholder {
    color: hsl(var(--pc) / 1)}
.placeholder-primary-content\/20::placeholder {
    color: hsl(var(--pc) / 0.2)}
.placeholder-primary-content\/25::placeholder {
    color: hsl(var(--pc) / 0.25)}
.placeholder-primary-content\/30::placeholder {
    color: hsl(var(--pc) / 0.3)}
.placeholder-primary-content\/40::placeholder {
    color: hsl(var(--pc) / 0.4)}
.placeholder-primary-content\/5::placeholder {
    color: hsl(var(--pc) / 0.05)}
.placeholder-primary-content\/50::placeholder {
    color: hsl(var(--pc) / 0.5)}
.placeholder-primary-content\/60::placeholder {
    color: hsl(var(--pc) / 0.6)}
.placeholder-primary-content\/70::placeholder {
    color: hsl(var(--pc) / 0.7)}
.placeholder-primary-content\/75::placeholder {
    color: hsl(var(--pc) / 0.75)}
.placeholder-primary-content\/80::placeholder {
    color: hsl(var(--pc) / 0.8)}
.placeholder-primary-content\/90::placeholder {
    color: hsl(var(--pc) / 0.9)}
.placeholder-primary-content\/95::placeholder {
    color: hsl(var(--pc) / 0.95)}
.placeholder-primary-focus::placeholder {
    color: hsl(var(--pf) / 1)}
.placeholder-primary-focus\/0::placeholder {
    color: hsl(var(--pf) / 0)}
.placeholder-primary-focus\/10::placeholder {
    color: hsl(var(--pf) / 0.1)}
.placeholder-primary-focus\/100::placeholder {
    color: hsl(var(--pf) / 1)}
.placeholder-primary-focus\/20::placeholder {
    color: hsl(var(--pf) / 0.2)}
.placeholder-primary-focus\/25::placeholder {
    color: hsl(var(--pf) / 0.25)}
.placeholder-primary-focus\/30::placeholder {
    color: hsl(var(--pf) / 0.3)}
.placeholder-primary-focus\/40::placeholder {
    color: hsl(var(--pf) / 0.4)}
.placeholder-primary-focus\/5::placeholder {
    color: hsl(var(--pf) / 0.05)}
.placeholder-primary-focus\/50::placeholder {
    color: hsl(var(--pf) / 0.5)}
.placeholder-primary-focus\/60::placeholder {
    color: hsl(var(--pf) / 0.6)}
.placeholder-primary-focus\/70::placeholder {
    color: hsl(var(--pf) / 0.7)}
.placeholder-primary-focus\/75::placeholder {
    color: hsl(var(--pf) / 0.75)}
.placeholder-primary-focus\/80::placeholder {
    color: hsl(var(--pf) / 0.8)}
.placeholder-primary-focus\/90::placeholder {
    color: hsl(var(--pf) / 0.9)}
.placeholder-primary-focus\/95::placeholder {
    color: hsl(var(--pf) / 0.95)}
.placeholder-primary\/0::placeholder {
    color: hsl(var(--p) / 0)}
.placeholder-primary\/10::placeholder {
    color: hsl(var(--p) / 0.1)}
.placeholder-primary\/100::placeholder {
    color: hsl(var(--p) / 1)}
.placeholder-primary\/20::placeholder {
    color: hsl(var(--p) / 0.2)}
.placeholder-primary\/25::placeholder {
    color: hsl(var(--p) / 0.25)}
.placeholder-primary\/30::placeholder {
    color: hsl(var(--p) / 0.3)}
.placeholder-primary\/40::placeholder {
    color: hsl(var(--p) / 0.4)}
.placeholder-primary\/5::placeholder {
    color: hsl(var(--p) / 0.05)}
.placeholder-primary\/50::placeholder {
    color: hsl(var(--p) / 0.5)}
.placeholder-primary\/60::placeholder {
    color: hsl(var(--p) / 0.6)}
.placeholder-primary\/70::placeholder {
    color: hsl(var(--p) / 0.7)}
.placeholder-primary\/75::placeholder {
    color: hsl(var(--p) / 0.75)}
.placeholder-primary\/80::placeholder {
    color: hsl(var(--p) / 0.8)}
.placeholder-primary\/90::placeholder {
    color: hsl(var(--p) / 0.9)}
.placeholder-primary\/95::placeholder {
    color: hsl(var(--p) / 0.95)}
.placeholder-secondary::placeholder {
    color: hsl(var(--s) / 1)}
.placeholder-secondary-content::placeholder {
    color: hsl(var(--sc) / 1)}
.placeholder-secondary-content\/0::placeholder {
    color: hsl(var(--sc) / 0)}
.placeholder-secondary-content\/10::placeholder {
    color: hsl(var(--sc) / 0.1)}
.placeholder-secondary-content\/100::placeholder {
    color: hsl(var(--sc) / 1)}
.placeholder-secondary-content\/20::placeholder {
    color: hsl(var(--sc) / 0.2)}
.placeholder-secondary-content\/25::placeholder {
    color: hsl(var(--sc) / 0.25)}
.placeholder-secondary-content\/30::placeholder {
    color: hsl(var(--sc) / 0.3)}
.placeholder-secondary-content\/40::placeholder {
    color: hsl(var(--sc) / 0.4)}
.placeholder-secondary-content\/5::placeholder {
    color: hsl(var(--sc) / 0.05)}
.placeholder-secondary-content\/50::placeholder {
    color: hsl(var(--sc) / 0.5)}
.placeholder-secondary-content\/60::placeholder {
    color: hsl(var(--sc) / 0.6)}
.placeholder-secondary-content\/70::placeholder {
    color: hsl(var(--sc) / 0.7)}
.placeholder-secondary-content\/75::placeholder {
    color: hsl(var(--sc) / 0.75)}
.placeholder-secondary-content\/80::placeholder {
    color: hsl(var(--sc) / 0.8)}
.placeholder-secondary-content\/90::placeholder {
    color: hsl(var(--sc) / 0.9)}
.placeholder-secondary-content\/95::placeholder {
    color: hsl(var(--sc) / 0.95)}
.placeholder-secondary-focus::placeholder {
    color: hsl(var(--sf) / 1)}
.placeholder-secondary-focus\/0::placeholder {
    color: hsl(var(--sf) / 0)}
.placeholder-secondary-focus\/10::placeholder {
    color: hsl(var(--sf) / 0.1)}
.placeholder-secondary-focus\/100::placeholder {
    color: hsl(var(--sf) / 1)}
.placeholder-secondary-focus\/20::placeholder {
    color: hsl(var(--sf) / 0.2)}
.placeholder-secondary-focus\/25::placeholder {
    color: hsl(var(--sf) / 0.25)}
.placeholder-secondary-focus\/30::placeholder {
    color: hsl(var(--sf) / 0.3)}
.placeholder-secondary-focus\/40::placeholder {
    color: hsl(var(--sf) / 0.4)}
.placeholder-secondary-focus\/5::placeholder {
    color: hsl(var(--sf) / 0.05)}
.placeholder-secondary-focus\/50::placeholder {
    color: hsl(var(--sf) / 0.5)}
.placeholder-secondary-focus\/60::placeholder {
    color: hsl(var(--sf) / 0.6)}
.placeholder-secondary-focus\/70::placeholder {
    color: hsl(var(--sf) / 0.7)}
.placeholder-secondary-focus\/75::placeholder {
    color: hsl(var(--sf) / 0.75)}
.placeholder-secondary-focus\/80::placeholder {
    color: hsl(var(--sf) / 0.8)}
.placeholder-secondary-focus\/90::placeholder {
    color: hsl(var(--sf) / 0.9)}
.placeholder-secondary-focus\/95::placeholder {
    color: hsl(var(--sf) / 0.95)}
.placeholder-secondary\/0::placeholder {
    color: hsl(var(--s) / 0)}
.placeholder-secondary\/10::placeholder {
    color: hsl(var(--s) / 0.1)}
.placeholder-secondary\/100::placeholder {
    color: hsl(var(--s) / 1)}
.placeholder-secondary\/20::placeholder {
    color: hsl(var(--s) / 0.2)}
.placeholder-secondary\/25::placeholder {
    color: hsl(var(--s) / 0.25)}
.placeholder-secondary\/30::placeholder {
    color: hsl(var(--s) / 0.3)}
.placeholder-secondary\/40::placeholder {
    color: hsl(var(--s) / 0.4)}
.placeholder-secondary\/5::placeholder {
    color: hsl(var(--s) / 0.05)}
.placeholder-secondary\/50::placeholder {
    color: hsl(var(--s) / 0.5)}
.placeholder-secondary\/60::placeholder {
    color: hsl(var(--s) / 0.6)}
.placeholder-secondary\/70::placeholder {
    color: hsl(var(--s) / 0.7)}
.placeholder-secondary\/75::placeholder {
    color: hsl(var(--s) / 0.75)}
.placeholder-secondary\/80::placeholder {
    color: hsl(var(--s) / 0.8)}
.placeholder-secondary\/90::placeholder {
    color: hsl(var(--s) / 0.9)}
.placeholder-secondary\/95::placeholder {
    color: hsl(var(--s) / 0.95)}
.placeholder-success::placeholder {
    color: hsl(var(--su) / 1)}
.placeholder-success-content::placeholder {
    color: hsl(var(--suc) / 1)}
.placeholder-success-content\/0::placeholder {
    color: hsl(var(--suc) / 0)}
.placeholder-success-content\/10::placeholder {
    color: hsl(var(--suc) / 0.1)}
.placeholder-success-content\/100::placeholder {
    color: hsl(var(--suc) / 1)}
.placeholder-success-content\/20::placeholder {
    color: hsl(var(--suc) / 0.2)}
.placeholder-success-content\/25::placeholder {
    color: hsl(var(--suc) / 0.25)}
.placeholder-success-content\/30::placeholder {
    color: hsl(var(--suc) / 0.3)}
.placeholder-success-content\/40::placeholder {
    color: hsl(var(--suc) / 0.4)}
.placeholder-success-content\/5::placeholder {
    color: hsl(var(--suc) / 0.05)}
.placeholder-success-content\/50::placeholder {
    color: hsl(var(--suc) / 0.5)}
.placeholder-success-content\/60::placeholder {
    color: hsl(var(--suc) / 0.6)}
.placeholder-success-content\/70::placeholder {
    color: hsl(var(--suc) / 0.7)}
.placeholder-success-content\/75::placeholder {
    color: hsl(var(--suc) / 0.75)}
.placeholder-success-content\/80::placeholder {
    color: hsl(var(--suc) / 0.8)}
.placeholder-success-content\/90::placeholder {
    color: hsl(var(--suc) / 0.9)}
.placeholder-success-content\/95::placeholder {
    color: hsl(var(--suc) / 0.95)}
.placeholder-success\/0::placeholder {
    color: hsl(var(--su) / 0)}
.placeholder-success\/10::placeholder {
    color: hsl(var(--su) / 0.1)}
.placeholder-success\/100::placeholder {
    color: hsl(var(--su) / 1)}
.placeholder-success\/20::placeholder {
    color: hsl(var(--su) / 0.2)}
.placeholder-success\/25::placeholder {
    color: hsl(var(--su) / 0.25)}
.placeholder-success\/30::placeholder {
    color: hsl(var(--su) / 0.3)}
.placeholder-success\/40::placeholder {
    color: hsl(var(--su) / 0.4)}
.placeholder-success\/5::placeholder {
    color: hsl(var(--su) / 0.05)}
.placeholder-success\/50::placeholder {
    color: hsl(var(--su) / 0.5)}
.placeholder-success\/60::placeholder {
    color: hsl(var(--su) / 0.6)}
.placeholder-success\/70::placeholder {
    color: hsl(var(--su) / 0.7)}
.placeholder-success\/75::placeholder {
    color: hsl(var(--su) / 0.75)}
.placeholder-success\/80::placeholder {
    color: hsl(var(--su) / 0.8)}
.placeholder-success\/90::placeholder {
    color: hsl(var(--su) / 0.9)}
.placeholder-success\/95::placeholder {
    color: hsl(var(--su) / 0.95)}
.placeholder-transparent::placeholder {
    color: transparent}
.placeholder-transparent\/0::placeholder {
    color: rgb(0 0 0 / 0)}
.placeholder-transparent\/10::placeholder {
    color: rgb(0 0 0 / 0.1)}
.placeholder-transparent\/100::placeholder {
    color: rgb(0 0 0 / 1)}
.placeholder-transparent\/20::placeholder {
    color: rgb(0 0 0 / 0.2)}
.placeholder-transparent\/25::placeholder {
    color: rgb(0 0 0 / 0.25)}
.placeholder-transparent\/30::placeholder {
    color: rgb(0 0 0 / 0.3)}
.placeholder-transparent\/40::placeholder {
    color: rgb(0 0 0 / 0.4)}
.placeholder-transparent\/5::placeholder {
    color: rgb(0 0 0 / 0.05)}
.placeholder-transparent\/50::placeholder {
    color: rgb(0 0 0 / 0.5)}
.placeholder-transparent\/60::placeholder {
    color: rgb(0 0 0 / 0.6)}
.placeholder-transparent\/70::placeholder {
    color: rgb(0 0 0 / 0.7)}
.placeholder-transparent\/75::placeholder {
    color: rgb(0 0 0 / 0.75)}
.placeholder-transparent\/80::placeholder {
    color: rgb(0 0 0 / 0.8)}
.placeholder-transparent\/90::placeholder {
    color: rgb(0 0 0 / 0.9)}
.placeholder-transparent\/95::placeholder {
    color: rgb(0 0 0 / 0.95)}
.placeholder-warning::placeholder {
    color: hsl(var(--wa) / 1)}
.placeholder-warning-content::placeholder {
    color: hsl(var(--wac) / 1)}
.placeholder-warning-content\/0::placeholder {
    color: hsl(var(--wac) / 0)}
.placeholder-warning-content\/10::placeholder {
    color: hsl(var(--wac) / 0.1)}
.placeholder-warning-content\/100::placeholder {
    color: hsl(var(--wac) / 1)}
.placeholder-warning-content\/20::placeholder {
    color: hsl(var(--wac) / 0.2)}
.placeholder-warning-content\/25::placeholder {
    color: hsl(var(--wac) / 0.25)}
.placeholder-warning-content\/30::placeholder {
    color: hsl(var(--wac) / 0.3)}
.placeholder-warning-content\/40::placeholder {
    color: hsl(var(--wac) / 0.4)}
.placeholder-warning-content\/5::placeholder {
    color: hsl(var(--wac) / 0.05)}
.placeholder-warning-content\/50::placeholder {
    color: hsl(var(--wac) / 0.5)}
.placeholder-warning-content\/60::placeholder {
    color: hsl(var(--wac) / 0.6)}
.placeholder-warning-content\/70::placeholder {
    color: hsl(var(--wac) / 0.7)}
.placeholder-warning-content\/75::placeholder {
    color: hsl(var(--wac) / 0.75)}
.placeholder-warning-content\/80::placeholder {
    color: hsl(var(--wac) / 0.8)}
.placeholder-warning-content\/90::placeholder {
    color: hsl(var(--wac) / 0.9)}
.placeholder-warning-content\/95::placeholder {
    color: hsl(var(--wac) / 0.95)}
.placeholder-warning\/0::placeholder {
    color: hsl(var(--wa) / 0)}
.placeholder-warning\/10::placeholder {
    color: hsl(var(--wa) / 0.1)}
.placeholder-warning\/100::placeholder {
    color: hsl(var(--wa) / 1)}
.placeholder-warning\/20::placeholder {
    color: hsl(var(--wa) / 0.2)}
.placeholder-warning\/25::placeholder {
    color: hsl(var(--wa) / 0.25)}
.placeholder-warning\/30::placeholder {
    color: hsl(var(--wa) / 0.3)}
.placeholder-warning\/40::placeholder {
    color: hsl(var(--wa) / 0.4)}
.placeholder-warning\/5::placeholder {
    color: hsl(var(--wa) / 0.05)}
.placeholder-warning\/50::placeholder {
    color: hsl(var(--wa) / 0.5)}
.placeholder-warning\/60::placeholder {
    color: hsl(var(--wa) / 0.6)}
.placeholder-warning\/70::placeholder {
    color: hsl(var(--wa) / 0.7)}
.placeholder-warning\/75::placeholder {
    color: hsl(var(--wa) / 0.75)}
.placeholder-warning\/80::placeholder {
    color: hsl(var(--wa) / 0.8)}
.placeholder-warning\/90::placeholder {
    color: hsl(var(--wa) / 0.9)}
.placeholder-warning\/95::placeholder {
    color: hsl(var(--wa) / 0.95)}
.ring {
    --tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);
    --tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(3px + var(--tw-ring-offset-width)) var(--tw-ring-color);
    box-shadow: var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)}
.ring-0 {
    --tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);
    --tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(0px + var(--tw-ring-offset-width)) var(--tw-ring-color);
    box-shadow: var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)}
.ring-1 {
    --tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);
    --tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(1px + var(--tw-ring-offset-width)) var(--tw-ring-color);
    box-shadow: var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)}
.ring-2 {
    --tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);
    --tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color);
    box-shadow: var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)}
.ring-4 {
    --tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);
    --tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(4px + var(--tw-ring-offset-width)) var(--tw-ring-color);
    box-shadow: var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)}
.ring-8 {
    --tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);
    --tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(8px + var(--tw-ring-offset-width)) var(--tw-ring-color);
    box-shadow: var(--tw-ring-offset-shadow), var(--tw-ring-shadow), var(--tw-shadow, 0 0 #0000)}
.ring-inset {
    --tw-ring-inset: inset}
.ring-accent {
    --tw-ring-color: hsl(var(--a) / 1)}
.ring-accent-content {
    --tw-ring-color: hsl(var(--ac) / 1)}
.ring-accent-content\/0 {
    --tw-ring-color: hsl(var(--ac) / 0)}
.ring-accent-content\/10 {
    --tw-ring-color: hsl(var(--ac) / 0.1)}
.ring-accent-content\/100 {
    --tw-ring-color: hsl(var(--ac) / 1)}
.ring-accent-content\/20 {
    --tw-ring-color: hsl(var(--ac) / 0.2)}
.ring-accent-content\/25 {
    --tw-ring-color: hsl(var(--ac) / 0.25)}
.ring-accent-content\/30 {
    --tw-ring-color: hsl(var(--ac) / 0.3)}
.ring-accent-content\/40 {
    --tw-ring-color: hsl(var(--ac) / 0.4)}
.ring-accent-content\/5 {
    --tw-ring-color: hsl(var(--ac) / 0.05)}
.ring-accent-content\/50 {
    --tw-ring-color: hsl(var(--ac) / 0.5)}
.ring-accent-content\/60 {
    --tw-ring-color: hsl(var(--ac) / 0.6)}
.ring-accent-content\/70 {
    --tw-ring-color: hsl(var(--ac) / 0.7)}
.ring-accent-content\/75 {
    --tw-ring-color: hsl(var(--ac) / 0.75)}
.ring-accent-content\/80 {
    --tw-ring-color: hsl(var(--ac) / 0.8)}
.ring-accent-content\/90 {
    --tw-ring-color: hsl(var(--ac) / 0.9)}
.ring-accent-content\/95 {
    --tw-ring-color: hsl(var(--ac) / 0.95)}
.ring-accent-focus {
    --tw-ring-color: hsl(var(--af) / 1)}
.ring-accent-focus\/0 {
    --tw-ring-color: hsl(var(--af) / 0)}
.ring-accent-focus\/10 {
    --tw-ring-color: hsl(var(--af) / 0.1)}
.ring-accent-focus\/100 {
    --tw-ring-color: hsl(var(--af) / 1)}
.ring-accent-focus\/20 {
    --tw-ring-color: hsl(var(--af) / 0.2)}
.ring-accent-focus\/25 {
    --tw-ring-color: hsl(var(--af) / 0.25)}
.ring-accent-focus\/30 {
    --tw-ring-color: hsl(var(--af) / 0.3)}
.ring-accent-focus\/40 {
    --tw-ring-color: hsl(var(--af) / 0.4)}
.ring-accent-focus\/5 {
    --tw-ring-color: hsl(var(--af) / 0.05)}
.ring-accent-focus\/50 {
    --tw-ring-color: hsl(var(--af) / 0.5)}
.ring-accent-focus\/60 {
    --tw-ring-color: hsl(var(--af) / 0.6)}
.ring-accent-focus\/70 {
    --tw-ring-color: hsl(var(--af) / 0.7)}
.ring-accent-focus\/75 {
    --tw-ring-color: hsl(var(--af) / 0.75)}
.ring-accent-focus\/80 {
    --tw-ring-color: hsl(var(--af) / 0.8)}
.ring-accent-focus\/90 {
    --tw-ring-color: hsl(var(--af) / 0.9)}
.ring-accent-focus\/95 {
    --tw-ring-color: hsl(var(--af) / 0.95)}
.ring-accent\/0 {
    --tw-ring-color: hsl(var(--a) / 0)}
.ring-accent\/10 {
    --tw-ring-color: hsl(var(--a) / 0.1)}
.ring-accent\/100 {
    --tw-ring-color: hsl(var(--a) / 1)}
.ring-accent\/20 {
    --tw-ring-color: hsl(var(--a) / 0.2)}
.ring-accent\/25 {
    --tw-ring-color: hsl(var(--a) / 0.25)}
.ring-accent\/30 {
    --tw-ring-color: hsl(var(--a) / 0.3)}
.ring-accent\/40 {
    --tw-ring-color: hsl(var(--a) / 0.4)}
.ring-accent\/5 {
    --tw-ring-color: hsl(var(--a) / 0.05)}
.ring-accent\/50 {
    --tw-ring-color: hsl(var(--a) / 0.5)}
.ring-accent\/60 {
    --tw-ring-color: hsl(var(--a) / 0.6)}
.ring-accent\/70 {
    --tw-ring-color: hsl(var(--a) / 0.7)}
.ring-accent\/75 {
    --tw-ring-color: hsl(var(--a) / 0.75)}
.ring-accent\/80 {
    --tw-ring-color: hsl(var(--a) / 0.8)}
.ring-accent\/90 {
    --tw-ring-color: hsl(var(--a) / 0.9)}
.ring-accent\/95 {
    --tw-ring-color: hsl(var(--a) / 0.95)}
.ring-base-100 {
    --tw-ring-color: hsl(var(--b1) / 1)}
.ring-base-100\/0 {
    --tw-ring-color: hsl(var(--b1) / 0)}
.ring-base-100\/10 {
    --tw-ring-color: hsl(var(--b1) / 0.1)}
.ring-base-100\/100 {
    --tw-ring-color: hsl(var(--b1) / 1)}
.ring-base-100\/20 {
    --tw-ring-color: hsl(var(--b1) / 0.2)}
.ring-base-100\/25 {
    --tw-ring-color: hsl(var(--b1) / 0.25)}
.ring-base-100\/30 {
    --tw-ring-color: hsl(var(--b1) / 0.3)}
.ring-base-100\/40 {
    --tw-ring-color: hsl(var(--b1) / 0.4)}
.ring-base-100\/5 {
    --tw-ring-color: hsl(var(--b1) / 0.05)}
.ring-base-100\/50 {
    --tw-ring-color: hsl(var(--b1) / 0.5)}
.ring-base-100\/60 {
    --tw-ring-color: hsl(var(--b1) / 0.6)}
.ring-base-100\/70 {
    --tw-ring-color: hsl(var(--b1) / 0.7)}
.ring-base-100\/75 {
    --tw-ring-color: hsl(var(--b1) / 0.75)}
.ring-base-100\/80 {
    --tw-ring-color: hsl(var(--b1) / 0.8)}
.ring-base-100\/90 {
    --tw-ring-color: hsl(var(--b1) / 0.9)}
.ring-base-100\/95 {
    --tw-ring-color: hsl(var(--b1) / 0.95)}
.ring-base-200 {
    --tw-ring-color: hsl(var(--b2) / 1)}
.ring-base-200\/0 {
    --tw-ring-color: hsl(var(--b2) / 0)}
.ring-base-200\/10 {
    --tw-ring-color: hsl(var(--b2) / 0.1)}
.ring-base-200\/100 {
    --tw-ring-color: hsl(var(--b2) / 1)}
.ring-base-200\/20 {
    --tw-ring-color: hsl(var(--b2) / 0.2)}
.ring-base-200\/25 {
    --tw-ring-color: hsl(var(--b2) / 0.25)}
.ring-base-200\/30 {
    --tw-ring-color: hsl(var(--b2) / 0.3)}
.ring-base-200\/40 {
    --tw-ring-color: hsl(var(--b2) / 0.4)}
.ring-base-200\/5 {
    --tw-ring-color: hsl(var(--b2) / 0.05)}
.ring-base-200\/50 {
    --tw-ring-color: hsl(var(--b2) / 0.5)}
.ring-base-200\/60 {
    --tw-ring-color: hsl(var(--b2) / 0.6)}
.ring-base-200\/70 {
    --tw-ring-color: hsl(var(--b2) / 0.7)}
.ring-base-200\/75 {
    --tw-ring-color: hsl(var(--b2) / 0.75)}
.ring-base-200\/80 {
    --tw-ring-color: hsl(var(--b2) / 0.8)}
.ring-base-200\/90 {
    --tw-ring-color: hsl(var(--b2) / 0.9)}
.ring-base-200\/95 {
    --tw-ring-color: hsl(var(--b2) / 0.95)}
.ring-base-300 {
    --tw-ring-color: hsl(var(--b3) / 1)}
.ring-base-300\/0 {
    --tw-ring-color: hsl(var(--b3) / 0)}
.ring-base-300\/10 {
    --tw-ring-color: hsl(var(--b3) / 0.1)}
.ring-base-300\/100 {
    --tw-ring-color: hsl(var(--b3) / 1)}
.ring-base-300\/20 {
    --tw-ring-color: hsl(var(--b3) / 0.2)}
.ring-base-300\/25 {
    --tw-ring-color: hsl(var(--b3) / 0.25)}
.ring-base-300\/30 {
    --tw-ring-color: hsl(var(--b3) / 0.3)}
.ring-base-300\/40 {
    --tw-ring-color: hsl(var(--b3) / 0.4)}
.ring-base-300\/5 {
    --tw-ring-color: hsl(var(--b3) / 0.05)}
.ring-base-300\/50 {
    --tw-ring-color: hsl(var(--b3) / 0.5)}
.ring-base-300\/60 {
    --tw-ring-color: hsl(var(--b3) / 0.6)}
.ring-base-300\/70 {
    --tw-ring-color: hsl(var(--b3) / 0.7)}
.ring-base-300\/75 {
    --tw-ring-color: hsl(var(--b3) / 0.75)}
.ring-base-300\/80 {
    --tw-ring-color: hsl(var(--b3) / 0.8)}
.ring-base-300\/90 {
    --tw-ring-color: hsl(var(--b3) / 0.9)}
.ring-base-300\/95 {
    --tw-ring-color: hsl(var(--b3) / 0.95)}
.ring-base-content {
    --tw-ring-color: hsl(var(--bc) / 1)}
.ring-base-content\/0 {
    --tw-ring-color: hsl(var(--bc) / 0)}
.ring-base-content\/10 {
    --tw-ring-color: hsl(var(--bc) / 0.1)}
.ring-base-content\/100 {
    --tw-ring-color: hsl(var(--bc) / 1)}
.ring-base-content\/20 {
    --tw-ring-color: hsl(var(--bc) / 0.2)}
.ring-base-content\/25 {
    --tw-ring-color: hsl(var(--bc) / 0.25)}
.ring-base-content\/30 {
    --tw-ring-color: hsl(var(--bc) / 0.3)}
.ring-base-content\/40 {
    --tw-ring-color: hsl(var(--bc) / 0.4)}
.ring-base-content\/5 {
    --tw-ring-color: hsl(var(--bc) / 0.05)}
.ring-base-content\/50 {
    --tw-ring-color: hsl(var(--bc) / 0.5)}
.ring-base-content\/60 {
    --tw-ring-color: hsl(var(--bc) / 0.6)}
.ring-base-content\/70 {
    --tw-ring-color: hsl(var(--bc) / 0.7)}
.ring-base-content\/75 {
    --tw-ring-color: hsl(var(--bc) / 0.75)}
.ring-base-content\/80 {
    --tw-ring-color: hsl(var(--bc) / 0.8)}
.ring-base-content\/90 {
    --tw-ring-color: hsl(var(--bc) / 0.9)}
.ring-base-content\/95 {
    --tw-ring-color: hsl(var(--bc) / 0.95)}
.ring-current {
    --tw-ring-color: currentColor}
.ring-error {
    --tw-ring-color: hsl(var(--er) / 1)}
.ring-error-content {
    --tw-ring-color: hsl(var(--erc) / 1)}
.ring-error-content\/0 {
    --tw-ring-color: hsl(var(--erc) / 0)}
.ring-error-content\/10 {
    --tw-ring-color: hsl(var(--erc) / 0.1)}
.ring-error-content\/100 {
    --tw-ring-color: hsl(var(--erc) / 1)}
.ring-error-content\/20 {
    --tw-ring-color: hsl(var(--erc) / 0.2)}
.ring-error-content\/25 {
    --tw-ring-color: hsl(var(--erc) / 0.25)}
.ring-error-content\/30 {
    --tw-ring-color: hsl(var(--erc) / 0.3)}
.ring-error-content\/40 {
    --tw-ring-color: hsl(var(--erc) / 0.4)}
.ring-error-content\/5 {
    --tw-ring-color: hsl(var(--erc) / 0.05)}
.ring-error-content\/50 {
    --tw-ring-color: hsl(var(--erc) / 0.5)}
.ring-error-content\/60 {
    --tw-ring-color: hsl(var(--erc) / 0.6)}
.ring-error-content\/70 {
    --tw-ring-color: hsl(var(--erc) / 0.7)}
.ring-error-content\/75 {
    --tw-ring-color: hsl(var(--erc) / 0.75)}
.ring-error-content\/80 {
    --tw-ring-color: hsl(var(--erc) / 0.8)}
.ring-error-content\/90 {
    --tw-ring-color: hsl(var(--erc) / 0.9)}
.ring-error-content\/95 {
    --tw-ring-color: hsl(var(--erc) / 0.95)}
.ring-error\/0 {
    --tw-ring-color: hsl(var(--er) / 0)}
.ring-error\/10 {
    --tw-ring-color: hsl(var(--er) / 0.1)}
.ring-error\/100 {
    --tw-ring-color: hsl(var(--er) / 1)}
.ring-error\/20 {
    --tw-ring-color: hsl(var(--er) / 0.2)}
.ring-error\/25 {
    --tw-ring-color: hsl(var(--er) / 0.25)}
.ring-error\/30 {
    --tw-ring-color: hsl(var(--er) / 0.3)}
.ring-error\/40 {
    --tw-ring-color: hsl(var(--er) / 0.4)}
.ring-error\/5 {
    --tw-ring-color: hsl(var(--er) / 0.05)}
.ring-error\/50 {
    --tw-ring-color: hsl(var(--er) / 0.5)}
.ring-error\/60 {
    --tw-ring-color: hsl(var(--er) / 0.6)}
.ring-error\/70 {
    --tw-ring-color: hsl(var(--er) / 0.7)}
.ring-error\/75 {
    --tw-ring-color: hsl(var(--er) / 0.75)}
.ring-error\/80 {
    --tw-ring-color: hsl(var(--er) / 0.8)}
.ring-error\/90 {
    --tw-ring-color: hsl(var(--er) / 0.9)}
.ring-error\/95 {
    --tw-ring-color: hsl(var(--er) / 0.95)}
.ring-info {
    --tw-ring-color: hsl(var(--in) / 1)}
.ring-info-content {
    --tw-ring-color: hsl(var(--inc) / 1)}
.ring-info-content\/0 {
    --tw-ring-color: hsl(var(--inc) / 0)}
.ring-info-content\/10 {
    --tw-ring-color: hsl(var(--inc) / 0.1)}
.ring-info-content\/100 {
    --tw-ring-color: hsl(var(--inc) / 1)}
.ring-info-content\/20 {
    --tw-ring-color: hsl(var(--inc) / 0.2)}
.ring-info-content\/25 {
    --tw-ring-color: hsl(var(--inc) / 0.25)}
.ring-info-content\/30 {
    --tw-ring-color: hsl(var(--inc) / 0.3)}
.ring-info-content\/40 {
    --tw-ring-color: hsl(var(--inc) / 0.4)}
.ring-info-content\/5 {
    --tw-ring-color: hsl(var(--inc) / 0.05)}
.ring-info-content\/50 {
    --tw-ring-color: hsl(var(--inc) / 0.5)}
.ring-info-content\/60 {
    --tw-ring-color: hsl(var(--inc) / 0.6)}
.ring-info-content\/70 {
    --tw-ring-color: hsl(var(--inc) / 0.7)}
.ring-info-content\/75 {
    --tw-ring-color: hsl(var(--inc) / 0.75)}
.ring-info-content\/80 {
    --tw-ring-color: hsl(var(--inc) / 0.8)}
.ring-info-content\/90 {
    --tw-ring-color: hsl(var(--inc) / 0.9)}
.ring-info-content\/95 {
    --tw-ring-color: hsl(var(--inc) / 0.95)}
.ring-info\/0 {
    --tw-ring-color: hsl(var(--in) / 0)}
.ring-info\/10 {
    --tw-ring-color: hsl(var(--in) / 0.1)}
.ring-info\/100 {
    --tw-ring-color: hsl(var(--in) / 1)}
.ring-info\/20 {
    --tw-ring-color: hsl(var(--in) / 0.2)}
.ring-info\/25 {
    --tw-ring-color: hsl(var(--in) / 0.25)}
.ring-info\/30 {
    --tw-ring-color: hsl(var(--in) / 0.3)}
.ring-info\/40 {
    --tw-ring-color: hsl(var(--in) / 0.4)}
.ring-info\/5 {
    --tw-ring-color: hsl(var(--in) / 0.05)}
.ring-info\/50 {
    --tw-ring-color: hsl(var(--in) / 0.5)}
.ring-info\/60 {
    --tw-ring-color: hsl(var(--in) / 0.6)}
.ring-info\/70 {
    --tw-ring-color: hsl(var(--in) / 0.7)}
.ring-info\/75 {
    --tw-ring-color: hsl(var(--in) / 0.75)}
.ring-info\/80 {
    --tw-ring-color: hsl(var(--in) / 0.8)}
.ring-info\/90 {
    --tw-ring-color: hsl(var(--in) / 0.9)}
.ring-info\/95 {
    --tw-ring-color: hsl(var(--in) / 0.95)}
.ring-neutral {
    --tw-ring-color: hsl(var(--n) / 1)}
.ring-neutral-content {
    --tw-ring-color: hsl(var(--nc) / 1)}
.ring-neutral-content\/0 {
    --tw-ring-color: hsl(var(--nc) / 0)}
.ring-neutral-content\/10 {
    --tw-ring-color: hsl(var(--nc) / 0.1)}
.ring-neutral-content\/100 {
    --tw-ring-color: hsl(var(--nc) / 1)}
.ring-neutral-content\/20 {
    --tw-ring-color: hsl(var(--nc) / 0.2)}
.ring-neutral-content\/25 {
    --tw-ring-color: hsl(var(--nc) / 0.25)}
.ring-neutral-content\/30 {
    --tw-ring-color: hsl(var(--nc) / 0.3)}
.ring-neutral-content\/40 {
    --tw-ring-color: hsl(var(--nc) / 0.4)}
.ring-neutral-content\/5 {
    --tw-ring-color: hsl(var(--nc) / 0.05)}
.ring-neutral-content\/50 {
    --tw-ring-color: hsl(var(--nc) / 0.5)}
.ring-neutral-content\/60 {
    --tw-ring-color: hsl(var(--nc) / 0.6)}
.ring-neutral-content\/70 {
    --tw-ring-color: hsl(var(--nc) / 0.7)}
.ring-neutral-content\/75 {
    --tw-ring-color: hsl(var(--nc) / 0.75)}
.ring-neutral-content\/80 {
    --tw-ring-color: hsl(var(--nc) / 0.8)}
.ring-neutral-content\/90 {
    --tw-ring-color: hsl(var(--nc) / 0.9)}
.ring-neutral-content\/95 {
    --tw-ring-color: hsl(var(--nc) / 0.95)}
.ring-neutral-focus {
    --tw-ring-color: hsl(var(--nf) / 1)}
.ring-neutral-focus\/0 {
    --tw-ring-color: hsl(var(--nf) / 0)}
.ring-neutral-focus\/10 {
    --tw-ring-color: hsl(var(--nf) / 0.1)}
.ring-neutral-focus\/100 {
    --tw-ring-color: hsl(var(--nf) / 1)}
.ring-neutral-focus\/20 {
    --tw-ring-color: hsl(var(--nf) / 0.2)}
.ring-neutral-focus\/25 {
    --tw-ring-color: hsl(var(--nf) / 0.25)}
.ring-neutral-focus\/30 {
    --tw-ring-color: hsl(var(--nf) / 0.3)}
.ring-neutral-focus\/40 {
    --tw-ring-color: hsl(var(--nf) / 0.4)}
.ring-neutral-focus\/5 {
    --tw-ring-color: hsl(var(--nf) / 0.05)}
.ring-neutral-focus\/50 {
    --tw-ring-color: hsl(var(--nf) / 0.5)}
.ring-neutral-focus\/60 {
    --tw-ring-color: hsl(var(--nf) / 0.6)}
.ring-neutral-focus\/70 {
    --tw-ring-color: hsl(var(--nf) / 0.7)}
.ring-neutral-focus\/75 {
    --tw-ring-color: hsl(var(--nf) / 0.75)}
.ring-neutral-focus\/80 {
    --tw-ring-color: hsl(var(--nf) / 0.8)}
.ring-neutral-focus\/90 {
    --tw-ring-color: hsl(var(--nf) / 0.9)}
.ring-neutral-focus\/95 {
    --tw-ring-color: hsl(var(--nf) / 0.95)}
.ring-neutral\/0 {
    --tw-ring-color: hsl(var(--n) / 0)}
.ring-neutral\/10 {
    --tw-ring-color: hsl(var(--n) / 0.1)}
.ring-neutral\/100 {
    --tw-ring-color: hsl(var(--n) / 1)}
.ring-neutral\/20 {
    --tw-ring-color: hsl(var(--n) / 0.2)}
.ring-neutral\/25 {
    --tw-ring-color: hsl(var(--n) / 0.25)}
.ring-neutral\/30 {
    --tw-ring-color: hsl(var(--n) / 0.3)}
.ring-neutral\/40 {
    --tw-ring-color: hsl(var(--n) / 0.4)}
.ring-neutral\/5 {
    --tw-ring-color: hsl(var(--n) / 0.05)}
.ring-neutral\/50 {
    --tw-ring-color: hsl(var(--n) / 0.5)}
.ring-neutral\/60 {
    --tw-ring-color: hsl(var(--n) / 0.6)}
.ring-neutral\/70 {
    --tw-ring-color: hsl(var(--n) / 0.7)}
.ring-neutral\/75 {
    --tw-ring-color: hsl(var(--n) / 0.75)}
.ring-neutral\/80 {
    --tw-ring-color: hsl(var(--n) / 0.8)}
.ring-neutral\/90 {
    --tw-ring-color: hsl(var(--n) / 0.9)}
.ring-neutral\/95 {
    --tw-ring-color: hsl(var(--n) / 0.95)}
.ring-primary {
    --tw-ring-color: hsl(var(--p) / 1)}
.ring-primary-content {
    --tw-ring-color: hsl(var(--pc) / 1)}
.ring-primary-content\/0 {
    --tw-ring-color: hsl(var(--pc) / 0)}
.ring-primary-content\/10 {
    --tw-ring-color: hsl(var(--pc) / 0.1)}
.ring-primary-content\/100 {
    --tw-ring-color: hsl(var(--pc) / 1)}
.ring-primary-content\/20 {
    --tw-ring-color: hsl(var(--pc) / 0.2)}
.ring-primary-content\/25 {
    --tw-ring-color: hsl(var(--pc) / 0.25)}
.ring-primary-content\/30 {
    --tw-ring-color: hsl(var(--pc) / 0.3)}
.ring-primary-content\/40 {
    --tw-ring-color: hsl(var(--pc) / 0.4)}
.ring-primary-content\/5 {
    --tw-ring-color: hsl(var(--pc) / 0.05)}
.ring-primary-content\/50 {
    --tw-ring-color: hsl(var(--pc) / 0.5)}
.ring-primary-content\/60 {
    --tw-ring-color: hsl(var(--pc) / 0.6)}
.ring-primary-content\/70 {
    --tw-ring-color: hsl(var(--pc) / 0.7)}
.ring-primary-content\/75 {
    --tw-ring-color: hsl(var(--pc) / 0.75)}
.ring-primary-content\/80 {
    --tw-ring-color: hsl(var(--pc) / 0.8)}
.ring-primary-content\/90 {
    --tw-ring-color: hsl(var(--pc) / 0.9)}
.ring-primary-content\/95 {
    --tw-ring-color: hsl(var(--pc) / 0.95)}
.ring-primary-focus {
    --tw-ring-color: hsl(var(--pf) / 1)}
.ring-primary-focus\/0 {
    --tw-ring-color: hsl(var(--pf) / 0)}
.ring-primary-focus\/10 {
    --tw-ring-color: hsl(var(--pf) / 0.1)}
.ring-primary-focus\/100 {
    --tw-ring-color: hsl(var(--pf) / 1)}
.ring-primary-focus\/20 {
    --tw-ring-color: hsl(var(--pf) / 0.2)}
.ring-primary-focus\/25 {
    --tw-ring-color: hsl(var(--pf) / 0.25)}
.ring-primary-focus\/30 {
    --tw-ring-color: hsl(var(--pf) / 0.3)}
.ring-primary-focus\/40 {
    --tw-ring-color: hsl(var(--pf) / 0.4)}
.ring-primary-focus\/5 {
    --tw-ring-color: hsl(var(--pf) / 0.05)}
.ring-primary-focus\/50 {
    --tw-ring-color: hsl(var(--pf) / 0.5)}
.ring-primary-focus\/60 {
    --tw-ring-color: hsl(var(--pf) / 0.6)}
.ring-primary-focus\/70 {
    --tw-ring-color: hsl(var(--pf) / 0.7)}
.ring-primary-focus\/75 {
    --tw-ring-color: hsl(var(--pf) / 0.75)}
.ring-primary-focus\/80 {
    --tw-ring-color: hsl(var(--pf) / 0.8)}
.ring-primary-focus\/90 {
    --tw-ring-color: hsl(var(--pf) / 0.9)}
.ring-primary-focus\/95 {
    --tw-ring-color: hsl(var(--pf) / 0.95)}
.ring-primary\/0 {
    --tw-ring-color: hsl(var(--p) / 0)}
.ring-primary\/10 {
    --tw-ring-color: hsl(var(--p) / 0.1)}
.ring-primary\/100 {
    --tw-ring-color: hsl(var(--p) / 1)}
.ring-primary\/20 {
    --tw-ring-color: hsl(var(--p) / 0.2)}
.ring-primary\/25 {
    --tw-ring-color: hsl(var(--p) / 0.25)}
.ring-primary\/30 {
    --tw-ring-color: hsl(var(--p) / 0.3)}
.ring-primary\/40 {
    --tw-ring-color: hsl(var(--p) / 0.4)}
.ring-primary\/5 {
    --tw-ring-color: hsl(var(--p) / 0.05)}
.ring-primary\/50 {
    --tw-ring-color: hsl(var(--p) / 0.5)}
.ring-primary\/60 {
    --tw-ring-color: hsl(var(--p) / 0.6)}
.ring-primary\/70 {
    --tw-ring-color: hsl(var(--p) / 0.7)}
.ring-primary\/75 {
    --tw-ring-color: hsl(var(--p) / 0.75)}
.ring-primary\/80 {
    --tw-ring-color: hsl(var(--p) / 0.8)}
.ring-primary\/90 {
    --tw-ring-color: hsl(var(--p) / 0.9)}
.ring-primary\/95 {
    --tw-ring-color: hsl(var(--p) / 0.95)}
.ring-secondary {
    --tw-ring-color: hsl(var(--s) / 1)}
.ring-secondary-content {
    --tw-ring-color: hsl(var(--sc) / 1)}
.ring-secondary-content\/0 {
    --tw-ring-color: hsl(var(--sc) / 0)}
.ring-secondary-content\/10 {
    --tw-ring-color: hsl(var(--sc) / 0.1)}
.ring-secondary-content\/100 {
    --tw-ring-color: hsl(var(--sc) / 1)}
.ring-secondary-content\/20 {
    --tw-ring-color: hsl(var(--sc) / 0.2)}
.ring-secondary-content\/25 {
    --tw-ring-color: hsl(var(--sc) / 0.25)}
.ring-secondary-content\/30 {
    --tw-ring-color: hsl(var(--sc) / 0.3)}
.ring-secondary-content\/40 {
    --tw-ring-color: hsl(var(--sc) / 0.4)}
.ring-secondary-content\/5 {
    --tw-ring-color: hsl(var(--sc) / 0.05)}
.ring-secondary-content\/50 {
    --tw-ring-color: hsl(var(--sc) / 0.5)}
.ring-secondary-content\/60 {
    --tw-ring-color: hsl(var(--sc) / 0.6)}
.ring-secondary-content\/70 {
    --tw-ring-color: hsl(var(--sc) / 0.7)}
.ring-secondary-content\/75 {
    --tw-ring-color: hsl(var(--sc) / 0.75)}
.ring-secondary-content\/80 {
    --tw-ring-color: hsl(var(--sc) / 0.8)}
.ring-secondary-content\/90 {
    --tw-ring-color: hsl(var(--sc) / 0.9)}
.ring-secondary-content\/95 {
    --tw-ring-color: hsl(var(--sc) / 0.95)}
.ring-secondary-focus {
    --tw-ring-color: hsl(var(--sf) / 1)}
.ring-secondary-focus\/0 {
    --tw-ring-color: hsl(var(--sf) / 0)}
.ring-secondary-focus\/10 {
    --tw-ring-color: hsl(var(--sf) / 0.1)}
.ring-secondary-focus\/100 {
    --tw-ring-color: hsl(var(--sf) / 1)}
.ring-secondary-focus\/20 {
    --tw-ring-color: hsl(var(--sf) / 0.2)}
.ring-secondary-focus\/25 {
    --tw-ring-color: hsl(var(--sf) / 0.25)}
.ring-secondary-focus\/30 {
    --tw-ring-color: hsl(var(--sf) / 0.3)}
.ring-secondary-focus\/40 {
    --tw-ring-color: hsl(var(--sf) / 0.4)}
.ring-secondary-focus\/5 {
    --tw-ring-color: hsl(var(--sf) / 0.05)}
.ring-secondary-focus\/50 {
    --tw-ring-color: hsl(var(--sf) / 0.5)}
.ring-secondary-focus\/60 {
    --tw-ring-color: hsl(var(--sf) / 0.6)}
.ring-secondary-focus\/70 {
    --tw-ring-color: hsl(var(--sf) / 0.7)}
.ring-secondary-focus\/75 {
    --tw-ring-color: hsl(var(--sf) / 0.75)}
.ring-secondary-focus\/80 {
    --tw-ring-color: hsl(var(--sf) / 0.8)}
.ring-secondary-focus\/90 {
    --tw-ring-color: hsl(var(--sf) / 0.9)}
.ring-secondary-focus\/95 {
    --tw-ring-color: hsl(var(--sf) / 0.95)}
.ring-secondary\/0 {
    --tw-ring-color: hsl(var(--s) / 0)}
.ring-secondary\/10 {
    --tw-ring-color: hsl(var(--s) / 0.1)}
.ring-secondary\/100 {
    --tw-ring-color: hsl(var(--s) / 1)}
.ring-secondary\/20 {
    --tw-ring-color: hsl(var(--s) / 0.2)}
.ring-secondary\/25 {
    --tw-ring-color: hsl(var(--s) / 0.25)}
.ring-secondary\/30 {
    --tw-ring-color: hsl(var(--s) / 0.3)}
.ring-secondary\/40 {
    --tw-ring-color: hsl(var(--s) / 0.4)}
.ring-secondary\/5 {
    --tw-ring-color: hsl(var(--s) / 0.05)}
.ring-secondary\/50 {
    --tw-ring-color: hsl(var(--s) / 0.5)}
.ring-secondary\/60 {
    --tw-ring-color: hsl(var(--s) / 0.6)}
.ring-secondary\/70 {
    --tw-ring-color: hsl(var(--s) / 0.7)}
.ring-secondary\/75 {
    --tw-ring-color: hsl(var(--s) / 0.75)}
.ring-secondary\/80 {
    --tw-ring-color: hsl(var(--s) / 0.8)}
.ring-secondary\/90 {
    --tw-ring-color: hsl(var(--s) / 0.9)}
.ring-secondary\/95 {
    --tw-ring-color: hsl(var(--s) / 0.95)}
.ring-success {
    --tw-ring-color: hsl(var(--su) / 1)}
.ring-success-content {
    --tw-ring-color: hsl(var(--suc) / 1)}
.ring-success-content\/0 {
    --tw-ring-color: hsl(var(--suc) / 0)}
.ring-success-content\/10 {
    --tw-ring-color: hsl(var(--suc) / 0.1)}
.ring-success-content\/100 {
    --tw-ring-color: hsl(var(--suc) / 1)}
.ring-success-content\/20 {
    --tw-ring-color: hsl(var(--suc) / 0.2)}
.ring-success-content\/25 {
    --tw-ring-color: hsl(var(--suc) / 0.25)}
.ring-success-content\/30 {
    --tw-ring-color: hsl(var(--suc) / 0.3)}
.ring-success-content\/40 {
    --tw-ring-color: hsl(var(--suc) / 0.4)}
.ring-success-content\/5 {
    --tw-ring-color: hsl(var(--suc) / 0.05)}
.ring-success-content\/50 {
    --tw-ring-color: hsl(var(--suc) / 0.5)}
.ring-success-content\/60 {
    --tw-ring-color: hsl(var(--suc) / 0.6)}
.ring-success-content\/70 {
    --tw-ring-color: hsl(var(--suc) / 0.7)}
.ring-success-content\/75 {
    --tw-ring-color: hsl(var(--suc) / 0.75)}
.ring-success-content\/80 {
    --tw-ring-color: hsl(var(--suc) / 0.8)}
.ring-success-content\/90 {
    --tw-ring-color: hsl(var(--suc) / 0.9)}
.ring-success-content\/95 {
    --tw-ring-color: hsl(var(--suc) / 0.95)}
.ring-success\/0 {
    --tw-ring-color: hsl(var(--su) / 0)}
.ring-success\/10 {
    --tw-ring-color: hsl(var(--su) / 0.1)}
.ring-success\/100 {
    --tw-ring-color: hsl(var(--su) / 1)}
.ring-success\/20 {
    --tw-ring-color: hsl(var(--su) / 0.2)}
.ring-success\/25 {
    --tw-ring-color: hsl(var(--su) / 0.25)}
.ring-success\/30 {
    --tw-ring-color: hsl(var(--su) / 0.3)}
.ring-success\/40 {
    --tw-ring-color: hsl(var(--su) / 0.4)}
.ring-success\/5 {
    --tw-ring-color: hsl(var(--su) / 0.05)}
.ring-success\/50 {
    --tw-ring-color: hsl(var(--su) / 0.5)}
.ring-success\/60 {
    --tw-ring-color: hsl(var(--su) / 0.6)}
.ring-success\/70 {
    --tw-ring-color: hsl(var(--su) / 0.7)}
.ring-success\/75 {
    --tw-ring-color: hsl(var(--su) / 0.75)}
.ring-success\/80 {
    --tw-ring-color: hsl(var(--su) / 0.8)}
.ring-success\/90 {
    --tw-ring-color: hsl(var(--su) / 0.9)}
.ring-success\/95 {
    --tw-ring-color: hsl(var(--su) / 0.95)}
.ring-transparent {
    --tw-ring-color: transparent}
.ring-transparent\/0 {
    --tw-ring-color: rgb(0 0 0 / 0)}
.ring-transparent\/10 {
    --tw-ring-color: rgb(0 0 0 / 0.1)}
.ring-transparent\/100 {
    --tw-ring-color: rgb(0 0 0 / 1)}
.ring-transparent\/20 {
    --tw-ring-color: rgb(0 0 0 / 0.2)}
.ring-transparent\/25 {
    --tw-ring-color: rgb(0 0 0 / 0.25)}
.ring-transparent\/30 {
    --tw-ring-color: rgb(0 0 0 / 0.3)}
.ring-transparent\/40 {
    --tw-ring-color: rgb(0 0 0 / 0.4)}
.ring-transparent\/5 {
    --tw-ring-color: rgb(0 0 0 / 0.05)}
.ring-transparent\/50 {
    --tw-ring-color: rgb(0 0 0 / 0.5)}
.ring-transparent\/60 {
    --tw-ring-color: rgb(0 0 0 / 0.6)}
.ring-transparent\/70 {
    --tw-ring-color: rgb(0 0 0 / 0.7)}
.ring-transparent\/75 {
    --tw-ring-color: rgb(0 0 0 / 0.75)}
.ring-transparent\/80 {
    --tw-ring-color: rgb(0 0 0 / 0.8)}
.ring-transparent\/90 {
    --tw-ring-color: rgb(0 0 0 / 0.9)}
.ring-transparent\/95 {
    --tw-ring-color: rgb(0 0 0 / 0.95)}
.ring-warning {
    --tw-ring-color: hsl(var(--wa) / 1)}
.ring-warning-content {
    --tw-ring-color: hsl(var(--wac) / 1)}
.ring-warning-content\/0 {
    --tw-ring-color: hsl(var(--wac) / 0)}
.ring-warning-content\/10 {
    --tw-ring-color: hsl(var(--wac) / 0.1)}
.ring-warning-content\/100 {
    --tw-ring-color: hsl(var(--wac) / 1)}
.ring-warning-content\/20 {
    --tw-ring-color: hsl(var(--wac) / 0.2)}
.ring-warning-content\/25 {
    --tw-ring-color: hsl(var(--wac) / 0.25)}
.ring-warning-content\/30 {
    --tw-ring-color: hsl(var(--wac) / 0.3)}
.ring-warning-content\/40 {
    --tw-ring-color: hsl(var(--wac) / 0.4)}
.ring-warning-content\/5 {
    --tw-ring-color: hsl(var(--wac) / 0.05)}
.ring-warning-content\/50 {
    --tw-ring-color: hsl(var(--wac) / 0.5)}
.ring-warning-content\/60 {
    --tw-ring-color: hsl(var(--wac) / 0.6)}
.ring-warning-content\/70 {
    --tw-ring-color: hsl(var(--wac) / 0.7)}
.ring-warning-content\/75 {
    --tw-ring-color: hsl(var(--wac) / 0.75)}
.ring-warning-content\/80 {
    --tw-ring-color: hsl(var(--wac) / 0.8)}
.ring-warning-content\/90 {
    --tw-ring-color: hsl(var(--wac) / 0.9)}
.ring-warning-content\/95 {
    --tw-ring-color: hsl(var(--wac) / 0.95)}
.ring-warning\/0 {
    --tw-ring-color: hsl(var(--wa) / 0)}
.ring-warning\/10 {
    --tw-ring-color: hsl(var(--wa) / 0.1)}
.ring-warning\/100 {
    --tw-ring-color: hsl(var(--wa) / 1)}
.ring-warning\/20 {
    --tw-ring-color: hsl(var(--wa) / 0.2)}
.ring-warning\/25 {
    --tw-ring-color: hsl(var(--wa) / 0.25)}
.ring-warning\/30 {
    --tw-ring-color: hsl(var(--wa) / 0.3)}
.ring-warning\/40 {
    --tw-ring-color: hsl(var(--wa) / 0.4)}
.ring-warning\/5 {
    --tw-ring-color: hsl(var(--wa) / 0.05)}
.ring-warning\/50 {
    --tw-ring-color: hsl(var(--wa) / 0.5)}
.ring-warning\/60 {
    --tw-ring-color: hsl(var(--wa) / 0.6)}
.ring-warning\/70 {
    --tw-ring-color: hsl(var(--wa) / 0.7)}
.ring-warning\/75 {
    --tw-ring-color: hsl(var(--wa) / 0.75)}
.ring-warning\/80 {
    --tw-ring-color: hsl(var(--wa) / 0.8)}
.ring-warning\/90 {
    --tw-ring-color: hsl(var(--wa) / 0.9)}
.ring-warning\/95 {
    --tw-ring-color: hsl(var(--wa) / 0.95)}
.ring-offset-0 {
    --tw-ring-offset-width: 0px}
.ring-offset-1 {
    --tw-ring-offset-width: 1px}
.ring-offset-2 {
    --tw-ring-offset-width: 2px}
.ring-offset-4 {
    --tw-ring-offset-width: 4px}
.ring-offset-8 {
    --tw-ring-offset-width: 8px}
.ring-offset-accent {
    --tw-ring-offset-color: hsl(var(--a) / 1)}
.ring-offset-accent-content {
    --tw-ring-offset-color: hsl(var(--ac) / 1)}
.ring-offset-accent-content\/0 {
    --tw-ring-offset-color: hsl(var(--ac) / 0)}
.ring-offset-accent-content\/10 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.1)}
.ring-offset-accent-content\/100 {
    --tw-ring-offset-color: hsl(var(--ac) / 1)}
.ring-offset-accent-content\/20 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.2)}
.ring-offset-accent-content\/25 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.25)}
.ring-offset-accent-content\/30 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.3)}
.ring-offset-accent-content\/40 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.4)}
.ring-offset-accent-content\/5 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.05)}
.ring-offset-accent-content\/50 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.5)}
.ring-offset-accent-content\/60 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.6)}
.ring-offset-accent-content\/70 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.7)}
.ring-offset-accent-content\/75 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.75)}
.ring-offset-accent-content\/80 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.8)}
.ring-offset-accent-content\/90 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.9)}
.ring-offset-accent-content\/95 {
    --tw-ring-offset-color: hsl(var(--ac) / 0.95)}
.ring-offset-accent-focus {
    --tw-ring-offset-color: hsl(var(--af) / 1)}
.ring-offset-accent-focus\/0 {
    --tw-ring-offset-color: hsl(var(--af) / 0)}
.ring-offset-accent-focus\/10 {
    --tw-ring-offset-color: hsl(var(--af) / 0.1)}
.ring-offset-accent-focus\/100 {
    --tw-ring-offset-color: hsl(var(--af) / 1)}
.ring-offset-accent-focus\/20 {
    --tw-ring-offset-color: hsl(var(--af) / 0.2)}
.ring-offset-accent-focus\/25 {
    --tw-ring-offset-color: hsl(var(--af) / 0.25)}
.ring-offset-accent-focus\/30 {
    --tw-ring-offset-color: hsl(var(--af) / 0.3)}
.ring-offset-accent-focus\/40 {
    --tw-ring-offset-color: hsl(var(--af) / 0.4)}
.ring-offset-accent-focus\/5 {
    --tw-ring-offset-color: hsl(var(--af) / 0.05)}
.ring-offset-accent-focus\/50 {
    --tw-ring-offset-color: hsl(var(--af) / 0.5)}
.ring-offset-accent-focus\/60 {
    --tw-ring-offset-color: hsl(var(--af) / 0.6)}
.ring-offset-accent-focus\/70 {
    --tw-ring-offset-color: hsl(var(--af) / 0.7)}
.ring-offset-accent-focus\/75 {
    --tw-ring-offset-color: hsl(var(--af) / 0.75)}
.ring-offset-accent-focus\/80 {
    --tw-ring-offset-color: hsl(var(--af) / 0.8)}
.ring-offset-accent-focus\/90 {
    --tw-ring-offset-color: hsl(var(--af) / 0.9)}
.ring-offset-accent-focus\/95 {
    --tw-ring-offset-color: hsl(var(--af) / 0.95)}
.ring-offset-accent\/0 {
    --tw-ring-offset-color: hsl(var(--a) / 0)}
.ring-offset-accent\/10 {
    --tw-ring-offset-color: hsl(var(--a) / 0.1)}
.ring-offset-accent\/100 {
    --tw-ring-offset-color: hsl(var(--a) / 1)}
.ring-offset-accent\/20 {
    --tw-ring-offset-color: hsl(var(--a) / 0.2)}
.ring-offset-accent\/25 {
    --tw-ring-offset-color: hsl(var(--a) / 0.25)}
.ring-offset-accent\/30 {
    --tw-ring-offset-color: hsl(var(--a) / 0.3)}
.ring-offset-accent\/40 {
    --tw-ring-offset-color: hsl(var(--a) / 0.4)}
.ring-offset-accent\/5 {
    --tw-ring-offset-color: hsl(var(--a) / 0.05)}
.ring-offset-accent\/50 {
    --tw-ring-offset-color: hsl(var(--a) / 0.5)}
.ring-offset-accent\/60 {
    --tw-ring-offset-color: hsl(var(--a) / 0.6)}
.ring-offset-accent\/70 {
    --tw-ring-offset-color: hsl(var(--a) / 0.7)}
.ring-offset-accent\/75 {
    --tw-ring-offset-color: hsl(var(--a) / 0.75)}
.ring-offset-accent\/80 {
    --tw-ring-offset-color: hsl(var(--a) / 0.8)}
.ring-offset-accent\/90 {
    --tw-ring-offset-color: hsl(var(--a) / 0.9)}
.ring-offset-accent\/95 {
    --tw-ring-offset-color: hsl(var(--a) / 0.95)}
.ring-offset-base-100 {
    --tw-ring-offset-color: hsl(var(--b1) / 1)}
.ring-offset-base-100\/0 {
    --tw-ring-offset-color: hsl(var(--b1) / 0)}
.ring-offset-base-100\/10 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.1)}
.ring-offset-base-100\/100 {
    --tw-ring-offset-color: hsl(var(--b1) / 1)}
.ring-offset-base-100\/20 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.2)}
.ring-offset-base-100\/25 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.25)}
.ring-offset-base-100\/30 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.3)}
.ring-offset-base-100\/40 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.4)}
.ring-offset-base-100\/5 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.05)}
.ring-offset-base-100\/50 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.5)}
.ring-offset-base-100\/60 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.6)}
.ring-offset-base-100\/70 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.7)}
.ring-offset-base-100\/75 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.75)}
.ring-offset-base-100\/80 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.8)}
.ring-offset-base-100\/90 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.9)}
.ring-offset-base-100\/95 {
    --tw-ring-offset-color: hsl(var(--b1) / 0.95)}
.ring-offset-base-200 {
    --tw-ring-offset-color: hsl(var(--b2) / 1)}
.ring-offset-base-200\/0 {
    --tw-ring-offset-color: hsl(var(--b2) / 0)}
.ring-offset-base-200\/10 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.1)}
.ring-offset-base-200\/100 {
    --tw-ring-offset-color: hsl(var(--b2) / 1)}
.ring-offset-base-200\/20 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.2)}
.ring-offset-base-200\/25 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.25)}
.ring-offset-base-200\/30 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.3)}
.ring-offset-base-200\/40 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.4)}
.ring-offset-base-200\/5 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.05)}
.ring-offset-base-200\/50 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.5)}
.ring-offset-base-200\/60 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.6)}
.ring-offset-base-200\/70 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.7)}
.ring-offset-base-200\/75 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.75)}
.ring-offset-base-200\/80 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.8)}
.ring-offset-base-200\/90 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.9)}
.ring-offset-base-200\/95 {
    --tw-ring-offset-color: hsl(var(--b2) / 0.95)}
.ring-offset-base-300 {
    --tw-ring-offset-color: hsl(var(--b3) / 1)}
.ring-offset-base-300\/0 {
    --tw-ring-offset-color: hsl(var(--b3) / 0)}
.ring-offset-base-300\/10 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.1)}
.ring-offset-base-300\/100 {
    --tw-ring-offset-color: hsl(var(--b3) / 1)}
.ring-offset-base-300\/20 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.2)}
.ring-offset-base-300\/25 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.25)}
.ring-offset-base-300\/30 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.3)}
.ring-offset-base-300\/40 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.4)}
.ring-offset-base-300\/5 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.05)}
.ring-offset-base-300\/50 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.5)}
.ring-offset-base-300\/60 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.6)}
.ring-offset-base-300\/70 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.7)}
.ring-offset-base-300\/75 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.75)}
.ring-offset-base-300\/80 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.8)}
.ring-offset-base-300\/90 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.9)}
.ring-offset-base-300\/95 {
    --tw-ring-offset-color: hsl(var(--b3) / 0.95)}
.ring-offset-base-content {
    --tw-ring-offset-color: hsl(var(--bc) / 1)}
.ring-offset-base-content\/0 {
    --tw-ring-offset-color: hsl(var(--bc) / 0)}
.ring-offset-base-content\/10 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.1)}
.ring-offset-base-content\/100 {
    --tw-ring-offset-color: hsl(var(--bc) / 1)}
.ring-offset-base-content\/20 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.2)}
.ring-offset-base-content\/25 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.25)}
.ring-offset-base-content\/30 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.3)}
.ring-offset-base-content\/40 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.4)}
.ring-offset-base-content\/5 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.05)}
.ring-offset-base-content\/50 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.5)}
.ring-offset-base-content\/60 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.6)}
.ring-offset-base-content\/70 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.7)}
.ring-offset-base-content\/75 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.75)}
.ring-offset-base-content\/80 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.8)}
.ring-offset-base-content\/90 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.9)}
.ring-offset-base-content\/95 {
    --tw-ring-offset-color: hsl(var(--bc) / 0.95)}
.ring-offset-current {
    --tw-ring-offset-color: currentColor}
.ring-offset-error {
    --tw-ring-offset-color: hsl(var(--er) / 1)}
.ring-offset-error-content {
    --tw-ring-offset-color: hsl(var(--erc) / 1)}
.ring-offset-error-content\/0 {
    --tw-ring-offset-color: hsl(var(--erc) / 0)}
.ring-offset-error-content\/10 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.1)}
.ring-offset-error-content\/100 {
    --tw-ring-offset-color: hsl(var(--erc) / 1)}
.ring-offset-error-content\/20 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.2)}
.ring-offset-error-content\/25 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.25)}
.ring-offset-error-content\/30 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.3)}
.ring-offset-error-content\/40 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.4)}
.ring-offset-error-content\/5 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.05)}
.ring-offset-error-content\/50 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.5)}
.ring-offset-error-content\/60 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.6)}
.ring-offset-error-content\/70 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.7)}
.ring-offset-error-content\/75 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.75)}
.ring-offset-error-content\/80 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.8)}
.ring-offset-error-content\/90 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.9)}
.ring-offset-error-content\/95 {
    --tw-ring-offset-color: hsl(var(--erc) / 0.95)}
.ring-offset-error\/0 {
    --tw-ring-offset-color: hsl(var(--er) / 0)}
.ring-offset-error\/10 {
    --tw-ring-offset-color: hsl(var(--er) / 0.1)}
.ring-offset-error\/100 {
    --tw-ring-offset-color: hsl(var(--er) / 1)}
.ring-offset-error\/20 {
    --tw-ring-offset-color: hsl(var(--er) / 0.2)}
.ring-offset-error\/25 {
    --tw-ring-offset-color: hsl(var(--er) / 0.25)}
.ring-offset-error\/30 {
    --tw-ring-offset-color: hsl(var(--er) / 0.3)}
.ring-offset-error\/40 {
    --tw-ring-offset-color: hsl(var(--er) / 0.4)}
.ring-offset-error\/5 {
    --tw-ring-offset-color: hsl(var(--er) / 0.05)}
.ring-offset-error\/50 {
    --tw-ring-offset-color: hsl(var(--er) / 0.5)}
.ring-offset-error\/60 {
    --tw-ring-offset-color: hsl(var(--er) / 0.6)}
.ring-offset-error\/70 {
    --tw-ring-offset-color: hsl(var(--er) / 0.7)}
.ring-offset-error\/75 {
    --tw-ring-offset-color: hsl(var(--er) / 0.75)}
.ring-offset-error\/80 {
    --tw-ring-offset-color: hsl(var(--er) / 0.8)}
.ring-offset-error\/90 {
    --tw-ring-offset-color: hsl(var(--er) / 0.9)}
.ring-offset-error\/95 {
    --tw-ring-offset-color: hsl(var(--er) / 0.95)}
.ring-offset-info {
    --tw-ring-offset-color: hsl(var(--in) / 1)}
.ring-offset-info-content {
    --tw-ring-offset-color: hsl(var(--inc) / 1)}
.ring-offset-info-content\/0 {
    --tw-ring-offset-color: hsl(var(--inc) / 0)}
.ring-offset-info-content\/10 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.1)}
.ring-offset-info-content\/100 {
    --tw-ring-offset-color: hsl(var(--inc) / 1)}
.ring-offset-info-content\/20 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.2)}
.ring-offset-info-content\/25 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.25)}
.ring-offset-info-content\/30 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.3)}
.ring-offset-info-content\/40 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.4)}
.ring-offset-info-content\/5 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.05)}
.ring-offset-info-content\/50 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.5)}
.ring-offset-info-content\/60 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.6)}
.ring-offset-info-content\/70 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.7)}
.ring-offset-info-content\/75 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.75)}
.ring-offset-info-content\/80 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.8)}
.ring-offset-info-content\/90 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.9)}
.ring-offset-info-content\/95 {
    --tw-ring-offset-color: hsl(var(--inc) / 0.95)}
.ring-offset-info\/0 {
    --tw-ring-offset-color: hsl(var(--in) / 0)}
.ring-offset-info\/10 {
    --tw-ring-offset-color: hsl(var(--in) / 0.1)}
.ring-offset-info\/100 {
    --tw-ring-offset-color: hsl(var(--in) / 1)}
.ring-offset-info\/20 {
    --tw-ring-offset-color: hsl(var(--in) / 0.2)}
.ring-offset-info\/25 {
    --tw-ring-offset-color: hsl(var(--in) / 0.25)}
.ring-offset-info\/30 {
    --tw-ring-offset-color: hsl(var(--in) / 0.3)}
.ring-offset-info\/40 {
    --tw-ring-offset-color: hsl(var(--in) / 0.4)}
.ring-offset-info\/5 {
    --tw-ring-offset-color: hsl(var(--in) / 0.05)}
.ring-offset-info\/50 {
    --tw-ring-offset-color: hsl(var(--in) / 0.5)}
.ring-offset-info\/60 {
    --tw-ring-offset-color: hsl(var(--in) / 0.6)}
.ring-offset-info\/70 {
    --tw-ring-offset-color: hsl(var(--in) / 0.7)}
.ring-offset-info\/75 {
    --tw-ring-offset-color: hsl(var(--in) / 0.75)}
.ring-offset-info\/80 {
    --tw-ring-offset-color: hsl(var(--in) / 0.8)}
.ring-offset-info\/90 {
    --tw-ring-offset-color: hsl(var(--in) / 0.9)}
.ring-offset-info\/95 {
    --tw-ring-offset-color: hsl(var(--in) / 0.95)}
.ring-offset-neutral {
    --tw-ring-offset-color: hsl(var(--n) / 1)}
.ring-offset-neutral-content {
    --tw-ring-offset-color: hsl(var(--nc) / 1)}
.ring-offset-neutral-content\/0 {
    --tw-ring-offset-color: hsl(var(--nc) / 0)}
.ring-offset-neutral-content\/10 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.1)}
.ring-offset-neutral-content\/100 {
    --tw-ring-offset-color: hsl(var(--nc) / 1)}
.ring-offset-neutral-content\/20 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.2)}
.ring-offset-neutral-content\/25 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.25)}
.ring-offset-neutral-content\/30 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.3)}
.ring-offset-neutral-content\/40 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.4)}
.ring-offset-neutral-content\/5 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.05)}
.ring-offset-neutral-content\/50 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.5)}
.ring-offset-neutral-content\/60 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.6)}
.ring-offset-neutral-content\/70 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.7)}
.ring-offset-neutral-content\/75 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.75)}
.ring-offset-neutral-content\/80 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.8)}
.ring-offset-neutral-content\/90 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.9)}
.ring-offset-neutral-content\/95 {
    --tw-ring-offset-color: hsl(var(--nc) / 0.95)}
.ring-offset-neutral-focus {
    --tw-ring-offset-color: hsl(var(--nf) / 1)}
.ring-offset-neutral-focus\/0 {
    --tw-ring-offset-color: hsl(var(--nf) / 0)}
.ring-offset-neutral-focus\/10 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.1)}
.ring-offset-neutral-focus\/100 {
    --tw-ring-offset-color: hsl(var(--nf) / 1)}
.ring-offset-neutral-focus\/20 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.2)}
.ring-offset-neutral-focus\/25 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.25)}
.ring-offset-neutral-focus\/30 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.3)}
.ring-offset-neutral-focus\/40 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.4)}
.ring-offset-neutral-focus\/5 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.05)}
.ring-offset-neutral-focus\/50 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.5)}
.ring-offset-neutral-focus\/60 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.6)}
.ring-offset-neutral-focus\/70 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.7)}
.ring-offset-neutral-focus\/75 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.75)}
.ring-offset-neutral-focus\/80 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.8)}
.ring-offset-neutral-focus\/90 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.9)}
.ring-offset-neutral-focus\/95 {
    --tw-ring-offset-color: hsl(var(--nf) / 0.95)}
.ring-offset-neutral\/0 {
    --tw-ring-offset-color: hsl(var(--n) / 0)}
.ring-offset-neutral\/10 {
    --tw-ring-offset-color: hsl(var(--n) / 0.1)}
.ring-offset-neutral\/100 {
    --tw-ring-offset-color: hsl(var(--n) / 1)}
.ring-offset-neutral\/20 {
    --tw-ring-offset-color: hsl(var(--n) / 0.2)}
.ring-offset-neutral\/25 {
    --tw-ring-offset-color: hsl(var(--n) / 0.25)}
.ring-offset-neutral\/30 {
    --tw-ring-offset-color: hsl(var(--n) / 0.3)}
.ring-offset-neutral\/40 {
    --tw-ring-offset-color: hsl(var(--n) / 0.4)}
.ring-offset-neutral\/5 {
    --tw-ring-offset-color: hsl(var(--n) / 0.05)}
.ring-offset-neutral\/50 {
    --tw-ring-offset-color: hsl(var(--n) / 0.5)}
.ring-offset-neutral\/60 {
    --tw-ring-offset-color: hsl(var(--n) / 0.6)}
.ring-offset-neutral\/70 {
    --tw-ring-offset-color: hsl(var(--n) / 0.7)}
.ring-offset-neutral\/75 {
    --tw-ring-offset-color: hsl(var(--n) / 0.75)}
.ring-offset-neutral\/80 {
    --tw-ring-offset-color: hsl(var(--n) / 0.8)}
.ring-offset-neutral\/90 {
    --tw-ring-offset-color: hsl(var(--n) / 0.9)}
.ring-offset-neutral\/95 {
    --tw-ring-offset-color: hsl(var(--n) / 0.95)}
.ring-offset-primary {
    --tw-ring-offset-color: hsl(var(--p) / 1)}
.ring-offset-primary-content {
    --tw-ring-offset-color: hsl(var(--pc) / 1)}
.ring-offset-primary-content\/0 {
    --tw-ring-offset-color: hsl(var(--pc) / 0)}
.ring-offset-primary-content\/10 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.1)}
.ring-offset-primary-content\/100 {
    --tw-ring-offset-color: hsl(var(--pc) / 1)}
.ring-offset-primary-content\/20 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.2)}
.ring-offset-primary-content\/25 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.25)}
.ring-offset-primary-content\/30 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.3)}
.ring-offset-primary-content\/40 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.4)}
.ring-offset-primary-content\/5 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.05)}
.ring-offset-primary-content\/50 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.5)}
.ring-offset-primary-content\/60 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.6)}
.ring-offset-primary-content\/70 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.7)}
.ring-offset-primary-content\/75 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.75)}
.ring-offset-primary-content\/80 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.8)}
.ring-offset-primary-content\/90 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.9)}
.ring-offset-primary-content\/95 {
    --tw-ring-offset-color: hsl(var(--pc) / 0.95)}
.ring-offset-primary-focus {
    --tw-ring-offset-color: hsl(var(--pf) / 1)}
.ring-offset-primary-focus\/0 {
    --tw-ring-offset-color: hsl(var(--pf) / 0)}
.ring-offset-primary-focus\/10 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.1)}
.ring-offset-primary-focus\/100 {
    --tw-ring-offset-color: hsl(var(--pf) / 1)}
.ring-offset-primary-focus\/20 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.2)}
.ring-offset-primary-focus\/25 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.25)}
.ring-offset-primary-focus\/30 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.3)}
.ring-offset-primary-focus\/40 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.4)}
.ring-offset-primary-focus\/5 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.05)}
.ring-offset-primary-focus\/50 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.5)}
.ring-offset-primary-focus\/60 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.6)}
.ring-offset-primary-focus\/70 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.7)}
.ring-offset-primary-focus\/75 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.75)}
.ring-offset-primary-focus\/80 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.8)}
.ring-offset-primary-focus\/90 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.9)}
.ring-offset-primary-focus\/95 {
    --tw-ring-offset-color: hsl(var(--pf) / 0.95)}
.ring-offset-primary\/0 {
    --tw-ring-offset-color: hsl(var(--p) / 0)}
.ring-offset-primary\/10 {
    --tw-ring-offset-color: hsl(var(--p) / 0.1)}
.ring-offset-primary\/100 {
    --tw-ring-offset-color: hsl(var(--p) / 1)}
.ring-offset-primary\/20 {
    --tw-ring-offset-color: hsl(var(--p) / 0.2)}
.ring-offset-primary\/25 {
    --tw-ring-offset-color: hsl(var(--p) / 0.25)}
.ring-offset-primary\/30 {
    --tw-ring-offset-color: hsl(var(--p) / 0.3)}
.ring-offset-primary\/40 {
    --tw-ring-offset-color: hsl(var(--p) / 0.4)}
.ring-offset-primary\/5 {
    --tw-ring-offset-color: hsl(var(--p) / 0.05)}
.ring-offset-primary\/50 {
    --tw-ring-offset-color: hsl(var(--p) / 0.5)}
.ring-offset-primary\/60 {
    --tw-ring-offset-color: hsl(var(--p) / 0.6)}
.ring-offset-primary\/70 {
    --tw-ring-offset-color: hsl(var(--p) / 0.7)}
.ring-offset-primary\/75 {
    --tw-ring-offset-color: hsl(var(--p) / 0.75)}
.ring-offset-primary\/80 {
    --tw-ring-offset-color: hsl(var(--p) / 0.8)}
.ring-offset-primary\/90 {
    --tw-ring-offset-color: hsl(var(--p) / 0.9)}
.ring-offset-primary\/95 {
    --tw-ring-offset-color: hsl(var(--p) / 0.95)}
.ring-offset-secondary {
    --tw-ring-offset-color: hsl(var(--s) / 1)}
.ring-offset-secondary-content {
    --tw-ring-offset-color: hsl(var(--sc) / 1)}
.ring-offset-secondary-content\/0 {
    --tw-ring-offset-color: hsl(var(--sc) / 0)}
.ring-offset-secondary-content\/10 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.1)}
.ring-offset-secondary-content\/100 {
    --tw-ring-offset-color: hsl(var(--sc) / 1)}
.ring-offset-secondary-content\/20 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.2)}
.ring-offset-secondary-content\/25 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.25)}
.ring-offset-secondary-content\/30 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.3)}
.ring-offset-secondary-content\/40 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.4)}
.ring-offset-secondary-content\/5 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.05)}
.ring-offset-secondary-content\/50 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.5)}
.ring-offset-secondary-content\/60 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.6)}
.ring-offset-secondary-content\/70 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.7)}
.ring-offset-secondary-content\/75 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.75)}
.ring-offset-secondary-content\/80 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.8)}
.ring-offset-secondary-content\/90 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.9)}
.ring-offset-secondary-content\/95 {
    --tw-ring-offset-color: hsl(var(--sc) / 0.95)}
.ring-offset-secondary-focus {
    --tw-ring-offset-color: hsl(var(--sf) / 1)}
.ring-offset-secondary-focus\/0 {
    --tw-ring-offset-color: hsl(var(--sf) / 0)}
.ring-offset-secondary-focus\/10 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.1)}
.ring-offset-secondary-focus\/100 {
    --tw-ring-offset-color: hsl(var(--sf) / 1)}
.ring-offset-secondary-focus\/20 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.2)}
.ring-offset-secondary-focus\/25 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.25)}
.ring-offset-secondary-focus\/30 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.3)}
.ring-offset-secondary-focus\/40 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.4)}
.ring-offset-secondary-focus\/5 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.05)}
.ring-offset-secondary-focus\/50 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.5)}
.ring-offset-secondary-focus\/60 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.6)}
.ring-offset-secondary-focus\/70 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.7)}
.ring-offset-secondary-focus\/75 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.75)}
.ring-offset-secondary-focus\/80 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.8)}
.ring-offset-secondary-focus\/90 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.9)}
.ring-offset-secondary-focus\/95 {
    --tw-ring-offset-color: hsl(var(--sf) / 0.95)}
.ring-offset-secondary\/0 {
    --tw-ring-offset-color: hsl(var(--s) / 0)}
.ring-offset-secondary\/10 {
    --tw-ring-offset-color: hsl(var(--s) / 0.1)}
.ring-offset-secondary\/100 {
    --tw-ring-offset-color: hsl(var(--s) / 1)}
.ring-offset-secondary\/20 {
    --tw-ring-offset-color: hsl(var(--s) / 0.2)}
.ring-offset-secondary\/25 {
    --tw-ring-offset-color: hsl(var(--s) / 0.25)}
.ring-offset-secondary\/30 {
    --tw-ring-offset-color: hsl(var(--s) / 0.3)}
.ring-offset-secondary\/40 {
    --tw-ring-offset-color: hsl(var(--s) / 0.4)}
.ring-offset-secondary\/5 {
    --tw-ring-offset-color: hsl(var(--s) / 0.05)}
.ring-offset-secondary\/50 {
    --tw-ring-offset-color: hsl(var(--s) / 0.5)}
.ring-offset-secondary\/60 {
    --tw-ring-offset-color: hsl(var(--s) / 0.6)}
.ring-offset-secondary\/70 {
    --tw-ring-offset-color: hsl(var(--s) / 0.7)}
.ring-offset-secondary\/75 {
    --tw-ring-offset-color: hsl(var(--s) / 0.75)}
.ring-offset-secondary\/80 {
    --tw-ring-offset-color: hsl(var(--s) / 0.8)}
.ring-offset-secondary\/90 {
    --tw-ring-offset-color: hsl(var(--s) / 0.9)}
.ring-offset-secondary\/95 {
    --tw-ring-offset-color: hsl(var(--s) / 0.95)}
.ring-offset-success {
    --tw-ring-offset-color: hsl(var(--su) / 1)}
.ring-offset-success-content {
    --tw-ring-offset-color: hsl(var(--suc) / 1)}
.ring-offset-success-content\/0 {
    --tw-ring-offset-color: hsl(var(--suc) / 0)}
.ring-offset-success-content\/10 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.1)}
.ring-offset-success-content\/100 {
    --tw-ring-offset-color: hsl(var(--suc) / 1)}
.ring-offset-success-content\/20 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.2)}
.ring-offset-success-content\/25 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.25)}
.ring-offset-success-content\/30 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.3)}
.ring-offset-success-content\/40 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.4)}
.ring-offset-success-content\/5 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.05)}
.ring-offset-success-content\/50 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.5)}
.ring-offset-success-content\/60 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.6)}
.ring-offset-success-content\/70 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.7)}
.ring-offset-success-content\/75 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.75)}
.ring-offset-success-content\/80 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.8)}
.ring-offset-success-content\/90 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.9)}
.ring-offset-success-content\/95 {
    --tw-ring-offset-color: hsl(var(--suc) / 0.95)}
.ring-offset-success\/0 {
    --tw-ring-offset-color: hsl(var(--su) / 0)}
.ring-offset-success\/10 {
    --tw-ring-offset-color: hsl(var(--su) / 0.1)}
.ring-offset-success\/100 {
    --tw-ring-offset-color: hsl(var(--su) / 1)}
.ring-offset-success\/20 {
    --tw-ring-offset-color: hsl(var(--su) / 0.2)}
.ring-offset-success\/25 {
    --tw-ring-offset-color: hsl(var(--su) / 0.25)}
.ring-offset-success\/30 {
    --tw-ring-offset-color: hsl(var(--su) / 0.3)}
.ring-offset-success\/40 {
    --tw-ring-offset-color: hsl(var(--su) / 0.4)}
.ring-offset-success\/5 {
    --tw-ring-offset-color: hsl(var(--su) / 0.05)}
.ring-offset-success\/50 {
    --tw-ring-offset-color: hsl(var(--su) / 0.5)}
.ring-offset-success\/60 {
    --tw-ring-offset-color: hsl(var(--su) / 0.6)}
.ring-offset-success\/70 {
    --tw-ring-offset-color: hsl(var(--su) / 0.7)}
.ring-offset-success\/75 {
    --tw-ring-offset-color: hsl(var(--su) / 0.75)}
.ring-offset-success\/80 {
    --tw-ring-offset-color: hsl(var(--su) / 0.8)}
.ring-offset-success\/90 {
    --tw-ring-offset-color: hsl(var(--su) / 0.9)}
.ring-offset-success\/95 {
    --tw-ring-offset-color: hsl(var(--su) / 0.95)}
.ring-offset-transparent {
    --tw-ring-offset-color: transparent}
.ring-offset-transparent\/0 {
    --tw-ring-offset-color: rgb(0 0 0 / 0)}
.ring-offset-transparent\/10 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.1)}
.ring-offset-transparent\/100 {
    --tw-ring-offset-color: rgb(0 0 0 / 1)}
.ring-offset-transparent\/20 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.2)}
.ring-offset-transparent\/25 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.25)}
.ring-offset-transparent\/30 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.3)}
.ring-offset-transparent\/40 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.4)}
.ring-offset-transparent\/5 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.05)}
.ring-offset-transparent\/50 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.5)}
.ring-offset-transparent\/60 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.6)}
.ring-offset-transparent\/70 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.7)}
.ring-offset-transparent\/75 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.75)}
.ring-offset-transparent\/80 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.8)}
.ring-offset-transparent\/90 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.9)}
.ring-offset-transparent\/95 {
    --tw-ring-offset-color: rgb(0 0 0 / 0.95)}
.ring-offset-warning {
    --tw-ring-offset-color: hsl(var(--wa) / 1)}
.ring-offset-warning-content {
    --tw-ring-offset-color: hsl(var(--wac) / 1)}
.ring-offset-warning-content\/0 {
    --tw-ring-offset-color: hsl(var(--wac) / 0)}
.ring-offset-warning-content\/10 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.1)}
.ring-offset-warning-content\/100 {
    --tw-ring-offset-color: hsl(var(--wac) / 1)}
.ring-offset-warning-content\/20 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.2)}
.ring-offset-warning-content\/25 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.25)}
.ring-offset-warning-content\/30 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.3)}
.ring-offset-warning-content\/40 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.4)}
.ring-offset-warning-content\/5 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.05)}
.ring-offset-warning-content\/50 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.5)}
.ring-offset-warning-content\/60 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.6)}
.ring-offset-warning-content\/70 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.7)}
.ring-offset-warning-content\/75 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.75)}
.ring-offset-warning-content\/80 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.8)}
.ring-offset-warning-content\/90 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.9)}
.ring-offset-warning-content\/95 {
    --tw-ring-offset-color: hsl(var(--wac) / 0.95)}
.ring-offset-warning\/0 {
    --tw-ring-offset-color: hsl(var(--wa) / 0)}
.ring-offset-warning\/10 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.1)}
.ring-offset-warning\/100 {
    --tw-ring-offset-color: hsl(var(--wa) / 1)}
.ring-offset-warning\/20 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.2)}
.ring-offset-warning\/25 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.25)}
.ring-offset-warning\/30 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.3)}
.ring-offset-warning\/40 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.4)}
.ring-offset-warning\/5 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.05)}
.ring-offset-warning\/50 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.5)}
.ring-offset-warning\/60 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.6)}
.ring-offset-warning\/70 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.7)}
.ring-offset-warning\/75 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.75)}
.ring-offset-warning\/80 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.8)}
.ring-offset-warning\/90 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.9)}
.ring-offset-warning\/95 {
    --tw-ring-offset-color: hsl(var(--wa) / 0.95)}
.transition {
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-duration: 150ms}
.transition-all {
    transition-property: all;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-duration: 150ms}
.transition-colors {
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-duration: 150ms}
.transition-none {
    transition-property: none}
.transition-opacity {
    transition-property: opacity;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-duration: 150ms}
.transition-shadow {
    transition-property: box-shadow;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-duration: 150ms}
.transition-transform {
    transition-property: transform;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-duration: 150ms}
.rounded-box {
    border-radius: var(--rounded-box, 1rem)}
.rounded-t-box {
    border-top-left-radius: var(--rounded-box, 1rem);
    border-top-right-radius: var(--rounded-box, 1rem)}
.rounded-b-box {
    border-bottom-left-radius: var(--rounded-box, 1rem);
    border-bottom-right-radius: var(--rounded-box, 1rem)}
.rounded-l-box {
    border-top-left-radius: var(--rounded-box, 1rem);
    border-bottom-left-radius: var(--rounded-box, 1rem)}
.rounded-r-box {
    border-top-right-radius: var(--rounded-box, 1rem);
    border-bottom-right-radius: var(--rounded-box, 1rem)}
.rounded-br-box {
    border-bottom-right-radius: var(--rounded-box, 1rem)}
.rounded-tr-box {
    border-top-right-radius: var(--rounded-box, 1rem)}
.rounded-bl-box {
    border-bottom-left-radius: var(--rounded-box, 1rem)}
.rounded-tl-box {
    border-top-left-radius: var(--rounded-box, 1rem)}
.rounded-btn {
    border-radius: var(--rounded-btn, 0.5rem)}
.rounded-badge {
    border-radius: var(--rounded-badge, 1.9rem)}
.glass,
  .glass.btn-active {
    border: none;
    backdrop-filter: blur(var(--glass-blur, 40px));
    background-color: transparent;
    background-image: linear-gradient(
        135deg,
        rgb(255 255 255 / var(--glass-opacity, 30%)) 0%,
        rgb(0 0 0 / 0%) 100%
      ),
      linear-gradient(
        var(--glass-reflex-degree, 100deg),
        rgb(255 255 255 / var(--glass-reflex-opacity, 10%)) 25%,
        rgb(0 0 0 / 0%) 25%
      );
    box-shadow: 0 0 0 1px rgb(255 255 255 / var(--glass-border-opacity, 10%)) inset,
      0 0 0 2px rgb(0 0 0 / 5%);
    text-shadow: 0 1px rgb(0 0 0 / var(--glass-text-shadow-opacity, 5%))}
@media (hover: hover) {
    .glass.btn-active {
        border: none;
        backdrop-filter: blur(var(--glass-blur, 40px));
        background-color: transparent;
        background-image: linear-gradient(
          135deg,
          rgb(255 255 255 / var(--glass-opacity, 30%)) 0%,
          rgb(0 0 0 / 0%) 100%
        ),
        linear-gradient(
          var(--glass-reflex-degree, 100deg),
          rgb(255 255 255 / var(--glass-reflex-opacity, 10%)) 25%,
          rgb(0 0 0 / 0%) 25%
        );
        box-shadow: 0 0 0 1px rgb(255 255 255 / var(--glass-border-opacity, 10%)) inset,
        0 0 0 2px rgb(0 0 0 / 5%);
        text-shadow: 0 1px rgb(0 0 0 / var(--glass-text-shadow-opacity, 5%))}}
.min-h-6 {
    min-height: 1.5rem}
.min-h-8 {
    min-height: 2rem}
.min-h-12 {
    min-height: 3rem}
.min-h-16 {
    min-height: 4rem}
.no-animation {
    --btn-focus-scale: 1;
    --animation-btn: 0;
    --animation-input: 0}
.tab-border-none {
    --tab-border: 0px}
.tab-border {
    --tab-border: 1px}
.tab-border-2 {
    --tab-border: 2px}
.tab-border-3 {
    --tab-border: 3px}
.tab-rounded-none {
    --tab-radius: 0}
.tab-rounded-lg {
    --tab-radius: 0.5rem}
.artboard-demo {
    display: flex;
    flex: none;
    flex-direction: column;
    align-items: center;
    justify-content: center}
.artboard.phone {
    width: 320px}
.artboard.phone-1 {
    width: 320px;
    height: 568px}
.artboard.phone-1.horizontal,
      .artboard.phone-1.artboard-horizontal {
    width: 568px;
    height: 320px}
.artboard.phone-2 {
    width: 375px;
    height: 667px}
.artboard.phone-2.horizontal,
      .artboard.phone-2.artboard-horizontal {
    width: 667px;
    height: 375px}
.artboard.phone-3 {
    width: 414px;
    height: 736px}
.artboard.phone-3.horizontal,
      .artboard.phone-3.artboard-horizontal {
    width: 736px;
    height: 414px}
.artboard.phone-4 {
    width: 375px;
    height: 812px}
.artboard.phone-4.horizontal,
      .artboard.phone-4.artboard-horizontal {
    width: 812px;
    height: 375px}
.artboard.phone-5 {
    width: 414px;
    height: 896px}
.artboard.phone-5.horizontal,
      .artboard.phone-5.artboard-horizontal {
    width: 896px;
    height: 414px}
.artboard.phone-6 {
    width: 320px;
    height: 1024px}
.artboard.phone-6.horizontal,
      .artboard.phone-6.artboard-horizontal {
    width: 1024px;
    height: 320px}
.badge-xs {
    height: 0.75rem;
    font-size: 0.75rem;
    line-height: .75rem;
    padding-left: 0.313rem;
    padding-right: 0.313rem}
.badge-sm {
    height: 1rem;
    font-size: 0.75rem;
    line-height: 1rem;
    padding-left: 0.438rem;
    padding-right: 0.438rem}
.badge-md {
    height: 1.25rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    padding-left: 0.563rem;
    padding-right: 0.563rem}
.badge-lg {
    height: 1.5rem;
    font-size: 1rem;
    line-height: 1.5rem;
    padding-left: 0.688rem;
    padding-right: 0.688rem}
.btm-nav-xs {
    height: 2.5rem}
.btm-nav-xs > *:where(.active) {
    border-top-width: 1px}
.btm-nav-xs .btm-nav-label {
    font-size: 0.75rem;
    line-height: 1rem}
.btm-nav-sm {
    height: 3rem}
.btm-nav-sm > *:where(.active) {
    border-top-width: 2px}
.btm-nav-sm .btm-nav-label {
    font-size: 0.75rem;
    line-height: 1rem}
.btm-nav-md {
    height: 4rem}
.btm-nav-md > *:where(.active) {
    border-top-width: 2px}
.btm-nav-md .btm-nav-label {
    font-size: 0.875rem;
    line-height: 1.25rem}
.btm-nav-lg {
    height: 5rem}
.btm-nav-lg > *:where(.active) {
    border-top-width: 4px}
.btm-nav-lg .btm-nav-label {
    font-size: 1rem;
    line-height: 1.5rem}
.btn-xs {
    height: 1.5rem;
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    min-height: 1.5rem;
    font-size: 0.75rem}
.btn-sm {
    height: 2rem;
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    min-height: 2rem;
    font-size: 0.875rem}
.btn-md {
    height: 3rem;
    padding-left: 1rem;
    padding-right: 1rem;
    min-height: 3rem;
    font-size: 0.875rem}
.btn-lg {
    height: 4rem;
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    min-height: 4rem;
    font-size: 1.125rem}
.btn-wide {
    width: 16rem}
.btn-block {
    width: 100%}
.btn-square:where(.btn-xs) {
    height: 1.5rem;
    width: 1.5rem;
    padding: 0px}
.btn-square:where(.btn-sm) {
    height: 2rem;
    width: 2rem;
    padding: 0px}
.btn-square:where(.btn-md) {
    height: 3rem;
    width: 3rem;
    padding: 0px}
.btn-square:where(.btn-lg) {
    height: 4rem;
    width: 4rem;
    padding: 0px}
.btn-circle:where(.btn-xs) {
    height: 1.5rem;
    width: 1.5rem;
    border-radius: 9999px;
    padding: 0px}
.btn-circle:where(.btn-sm) {
    height: 2rem;
    width: 2rem;
    border-radius: 9999px;
    padding: 0px}
.btn-circle:where(.btn-md) {
    height: 3rem;
    width: 3rem;
    border-radius: 9999px;
    padding: 0px}
.btn-circle:where(.btn-lg) {
    height: 4rem;
    width: 4rem;
    border-radius: 9999px;
    padding: 0px}
.btn-group-horizontal {
    flex-direction: row}
.btn-group-vertical {
    flex-direction: column}
.card-side {
    align-items: stretch;
    flex-direction: row}
.card-side :where(figure:first-child) {
    overflow: hidden;
    border-start-start-radius: inherit;
    border-start-end-radius: unset;
    border-end-start-radius: inherit;
    border-end-end-radius: unset}
.card-side :where(figure:last-child) {
    overflow: hidden;
    border-start-start-radius: unset;
    border-start-end-radius: inherit;
    border-end-start-radius: unset;
    border-end-end-radius: inherit}
.card-side figure > * {
    max-width: unset}
:where(.card-side figure > *) {
    width: 100%;
    height: 100%;
    object-fit: cover}
.checkbox-xs {
    height: 1rem;
    width: 1rem}
.checkbox-sm {
    height: 1.25rem;
    width: 1.25rem}
.checkbox-md {
    height: 1.5rem;
    width: 1.5rem}
.checkbox-lg {
    height: 2rem;
    width: 2rem}
.divider-horizontal {
    flex-direction: column}
.divider-horizontal:before {
    height: 100%;
    width: 0.125rem}
.divider-horizontal:after {
    height: 100%;
    width: 0.125rem}
.divider-vertical {
    flex-direction: row}
.divider-vertical:before {
    height: 0.125rem;
    width: 100%}
.divider-vertical:after {
    height: 0.125rem;
    width: 100%}
.drawer-open > .drawer-toggle {
    display: none}
.drawer-open > .drawer-toggle ~ .drawer-side {
    pointer-events: auto;
    visibility: visible;
    position: sticky;
    display: block;
    width: auto;
    overscroll-behavior: auto}
.drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(0%)}
[dir="rtl"] .drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
    transform: translateX(0%)}
.drawer-open > .drawer-toggle:checked ~ .drawer-side {
    pointer-events: auto;
    visibility: visible}
.file-input-xs {
    height: 1.5rem;
    padding-right: 0.5rem;
    font-size: 0.75rem;
    line-height: 1rem;
    line-height: 1.625}
.file-input-xs::file-selector-button {
    margin-right: 0.5rem;
    height: 1.375rem;
    min-height: 1.375rem;
    font-size: 0.75rem}
.file-input-sm {
    height: 2rem;
    padding-right: 0.75rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2}
.file-input-sm::file-selector-button {
    margin-right: 0.75rem;
    height: 1.875rem;
    min-height: 1.875rem;
    font-size: 0.875rem}
.file-input-md {
    height: 3rem;
    padding-right: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2}
.file-input-md::file-selector-button {
    margin-right: 1rem;
    height: 2.875rem;
    min-height: 2.875rem;
    font-size: 0.875rem}
.file-input-lg {
    height: 4rem;
    padding-right: 1.5rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    line-height: 2}
.file-input-lg::file-selector-button {
    margin-right: 1.5rem;
    height: 3.875rem;
    min-height: 3.875rem;
    font-size: 1.125rem}
.indicator :where(.indicator-item) {
    right: 0px;
    left: auto;
    top: 0px;
    bottom: auto;
    --tw-translate-x: 50%;
    --tw-translate-y: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.indicator :where(.indicator-item.indicator-start) {
    right: auto;
    left: 0px;
    --tw-translate-x: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.indicator :where(.indicator-item.indicator-center) {
    right: 50%;
    left: 50%;
    --tw-translate-x: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.indicator :where(.indicator-item.indicator-end) {
    right: 0px;
    left: auto;
    --tw-translate-x: 50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.indicator :where(.indicator-item.indicator-bottom) {
    top: auto;
    bottom: 0px;
    --tw-translate-y: 50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.indicator :where(.indicator-item.indicator-middle) {
    top: 50%;
    bottom: 50%;
    --tw-translate-y: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.indicator :where(.indicator-item.indicator-top) {
    top: 0px;
    bottom: auto;
    --tw-translate-y: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.input-xs {
    height: 1.5rem;
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    font-size: 0.75rem;
    line-height: 1rem;
    line-height: 1.625}
.input-md {
    height: 3rem;
    padding-left: 1rem;
    padding-right: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2}
.input-lg {
    height: 4rem;
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    line-height: 2}
.input-sm {
    height: 2rem;
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    font-size: 0.875rem;
    line-height: 2rem}
.join.join-vertical {
    flex-direction: column}
.join.join-vertical .join-item:first-child:not(:last-child),
  .join.join-vertical *:first-child:not(:last-child) .join-item {
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0;
    border-top-left-radius: inherit;
    border-top-right-radius: inherit}
.join.join-vertical .join-item:last-child:not(:first-child),
  .join.join-vertical *:last-child:not(:first-child) .join-item {
    border-top-left-radius: 0;
    border-top-right-radius: 0;
    border-bottom-left-radius: inherit;
    border-bottom-right-radius: inherit}
.join.join-horizontal {
    flex-direction: row}
.join.join-horizontal .join-item:first-child:not(:last-child),
  .join.join-horizontal *:first-child:not(:last-child) .join-item {
    border-bottom-right-radius: 0;
    border-top-right-radius: 0;
    border-bottom-left-radius: inherit;
    border-top-left-radius: inherit}
.join.join-horizontal .join-item:last-child:not(:first-child),
  .join.join-horizontal *:last-child:not(:first-child) .join-item {
    border-bottom-left-radius: 0;
    border-top-left-radius: 0;
    border-bottom-right-radius: inherit;
    border-top-right-radius: inherit}
.kbd-xs {
    padding-left: 0.25rem;
    padding-right: 0.25rem;
    font-size: 0.75rem;
    line-height: 1rem;
    min-height: 1.2em;
    min-width: 1.2em}
.kbd-sm {
    padding-left: 0.25rem;
    padding-right: 0.25rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    min-height: 1.6em;
    min-width: 1.6em}
.kbd-md {
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    font-size: 1rem;
    line-height: 1.5rem;
    min-height: 2.2em;
    min-width: 2.2em}
.kbd-lg {
    padding-left: 1rem;
    padding-right: 1rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    min-height: 2.5em;
    min-width: 2.5em}
.menu-horizontal {
    display: inline-flex;
    flex-direction: row}
.menu-horizontal > li:not(.menu-title) > details > ul {
    position: absolute}
.menu-vertical {
    display: flex;
    flex-direction: column}
.menu-vertical > li:not(.menu-title) > details > ul {
    position: relative}
.modal-top {
    place-items: start}
.modal-middle {
    place-items: center}
.modal-bottom {
    place-items: end}
.radio-xs {
    height: 1rem;
    width: 1rem}
.radio-sm {
    height: 1.25rem;
    width: 1.25rem}
.radio-md {
    height: 1.5rem;
    width: 1.5rem}
.radio-lg {
    height: 2rem;
    width: 2rem}
.range-xs {
    height: 1rem}
.range-xs::-webkit-slider-runnable-track {
    height: 0.25rem}
.range-xs::-moz-range-track {
    height: 0.25rem}
.range-xs::-webkit-slider-thumb {
    height: 1rem;
    width: 1rem;
    --filler-offset: 0.4rem}
.range-xs::-moz-range-thumb {
    height: 1rem;
    width: 1rem;
    --filler-offset: 0.4rem}
.range-sm {
    height: 1.25rem}
.range-sm::-webkit-slider-runnable-track {
    height: 0.25rem}
.range-sm::-moz-range-track {
    height: 0.25rem}
.range-sm::-webkit-slider-thumb {
    height: 1.25rem;
    width: 1.25rem;
    --filler-offset: 0.5rem}
.range-sm::-moz-range-thumb {
    height: 1.25rem;
    width: 1.25rem;
    --filler-offset: 0.5rem}
.range-md {
    height: 1.5rem}
.range-md::-webkit-slider-runnable-track {
    height: 0.5rem}
.range-md::-moz-range-track {
    height: 0.5rem}
.range-md::-webkit-slider-thumb {
    height: 1.5rem;
    width: 1.5rem;
    --filler-offset: 0.6rem}
.range-md::-moz-range-thumb {
    height: 1.5rem;
    width: 1.5rem;
    --filler-offset: 0.6rem}
.range-lg {
    height: 2rem}
.range-lg::-webkit-slider-runnable-track {
    height: 1rem}
.range-lg::-moz-range-track {
    height: 1rem}
.range-lg::-webkit-slider-thumb {
    height: 2rem;
    width: 2rem;
    --filler-offset: 1rem}
.range-lg::-moz-range-thumb {
    height: 2rem;
    width: 2rem;
    --filler-offset: 1rem}
.rating-xs input {
    height: 0.75rem;
    width: 0.75rem}
.rating-sm input {
    height: 1rem;
    width: 1rem}
.rating-md input {
    height: 1.5rem;
    width: 1.5rem}
.rating-lg input {
    height: 2.5rem;
    width: 2.5rem}
.rating-half.rating-xs input:not(.rating-hidden) {
    width: 0.375rem}
.rating-half.rating-sm input:not(.rating-hidden) {
    width: 0.5rem}
.rating-half.rating-md input:not(.rating-hidden) {
    width: 0.75rem}
.rating-half.rating-lg input:not(.rating-hidden) {
    width: 1.25rem}
.select-md {
    height: 3rem;
    padding-left: 1rem;
    padding-right: 2.5rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2;
    min-height: 3rem}
[dir="rtl"] .select-md {
    padding-right: 1rem;
    padding-left: 2.5rem}
.select-lg {
    height: 4rem;
    padding-left: 1.5rem;
    padding-right: 2rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    line-height: 2;
    min-height: 4rem}
[dir="rtl"] .select-lg {
    padding-right: 1.5rem;
    padding-left: 2rem}
.select-sm {
    height: 2rem;
    padding-left: 0.75rem;
    padding-right: 2rem;
    font-size: 0.875rem;
    line-height: 2rem;
    min-height: 2rem}
[dir="rtl"] .select-sm {
    padding-right: 0.75rem;
    padding-left: 2rem}
.select-xs {
    height: 1.5rem;
    padding-left: 0.5rem;
    padding-right: 2rem;
    font-size: 0.75rem;
    line-height: 1rem;
    line-height: 1.625;
    min-height: 1.5rem}
[dir="rtl"] .select-xs {
    padding-right: 0.5rem;
    padding-left: 2rem}
.stats-horizontal {
    grid-auto-flow: column}
.stats-vertical {
    grid-auto-flow: row}
.steps-horizontal {
    grid-auto-columns: 1fr;
    display: inline-grid;
    grid-auto-flow: column;
    overflow: hidden;
    overflow-x: auto}
.steps-horizontal .step {
    display: grid;
    grid-template-columns: repeat(1, minmax(0, 1fr));
    grid-template-rows: repeat(2, minmax(0, 1fr));
    place-items: center;
    text-align: center}
.steps-vertical {
    grid-auto-rows: 1fr;
    grid-auto-flow: row}
.steps-vertical .step {
    display: grid;
    grid-template-columns: repeat(2, minmax(0, 1fr));
    grid-template-rows: repeat(1, minmax(0, 1fr))}
.tab-md {
    height: 2rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2;
    --tab-padding: 1rem}
.tab-lg {
    height: 3rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    line-height: 2;
    --tab-padding: 1.25rem}
.tab-sm {
    height: 1.5rem;
    font-size: 0.875rem;
    line-height: .75rem;
    --tab-padding: 0.75rem}
.tab-xs {
    height: 1.25rem;
    font-size: 0.75rem;
    line-height: .75rem;
    --tab-padding: 0.5rem}
.textarea-xs {
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    font-size: 0.75rem;
    line-height: 1rem;
    line-height: 1.625}
.textarea-sm {
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    font-size: 0.875rem;
    line-height: 2rem}
.textarea-md {
    padding-top: 0.75rem;
    padding-bottom: 0.75rem;
    padding-left: 1rem;
    padding-right: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    line-height: 2}
.textarea-lg {
    padding-top: 1rem;
    padding-bottom: 1rem;
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    line-height: 2}
:where(.toast) {
    right: 0px;
    left: auto;
    top: auto;
    bottom: 0px;
    --tw-translate-x: 0px;
    --tw-translate-y: 0px;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toast:where(.toast-start) {
    right: auto;
    left: 0px;
    --tw-translate-x: 0px;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toast:where(.toast-center) {
    right: 50%;
    left: 50%;
    --tw-translate-x: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toast:where(.toast-end) {
    right: 0px;
    left: auto;
    --tw-translate-x: 0px;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toast:where(.toast-bottom) {
    top: auto;
    bottom: 0px;
    --tw-translate-y: 0px;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toast:where(.toast-middle) {
    top: 50%;
    bottom: auto;
    --tw-translate-y: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toast:where(.toast-top) {
    top: 0px;
    bottom: auto;
    --tw-translate-y: 0px;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}
.toggle-xs {
    --handleoffset: 0.5rem;
    height: 1rem;
    width: 1.5rem}
.toggle-sm {
    --handleoffset: 0.75rem;
    height: 1.25rem;
    width: 2rem}
.toggle-md {
    --handleoffset: 1.5rem;
    height: 1.5rem;
    width: 3rem}
.toggle-lg {
    --handleoffset: 2rem;
    height: 2rem;
    width: 4rem}
.tooltip {
    position: relative;
    display: inline-block;
    --tooltip-offset: calc(100% + 1px + var(--tooltip-tail, 0px))}
.tooltip:before {
    position: absolute;
    pointer-events: none;
    z-index: 1;
    content: var(--tw-content);
    --tw-content: attr(data-tip)}
.tooltip:before, .tooltip-top:before {
    transform: translateX(-50%);
    top: auto;
    left: 50%;
    right: auto;
    bottom: var(--tooltip-offset)}
.tooltip-bottom:before {
    transform: translateX(-50%);
    top: var(--tooltip-offset);
    left: 50%;
    right: auto;
    bottom: auto}
.tooltip-left:before {
    transform: translateY(-50%);
    top: 50%;
    left: auto;
    right: var(--tooltip-offset);
    bottom: auto}
.tooltip-right:before {
    transform: translateY(-50%);
    top: 50%;
    left: var(--tooltip-offset);
    right: auto;
    bottom: auto}
.artboard-demo {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--bc) / var(--tw-text-opacity));
    border-radius: var(--rounded-box, 1rem);
    box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.1), 0 1px 2px 0 rgba(0, 0, 0, 0.06)}
.avatar.online:before {
    content: "";
    position: absolute;
    z-index: 10;
    display: block;
    border-radius: 9999px;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    outline-style: solid;
    outline-width: 2px;
    outline-color: hsl(var(--b1) / 1);
    width: 15%;
    height: 15%;
    top: 7%;
    right: 7%}
.avatar.offline:before {
    content: "";
    position: absolute;
    z-index: 10;
    display: block;
    border-radius: 9999px;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b3) / var(--tw-bg-opacity));
    outline-style: solid;
    outline-width: 2px;
    outline-color: hsl(var(--b1) / 1);
    width: 15%;
    height: 15%;
    top: 7%;
    right: 7%}
.btn-group .btn:not(:first-child):not(:last-child) {
    border-top-left-radius: 0;
    border-top-right-radius: 0;
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0}
.btn-group .btn:first-child:not(:last-child) {
    margin-top: -0px;
    margin-left: -1px;
    border-top-left-radius: var(--rounded-btn, 0.5rem);
    border-top-right-radius: 0;
    border-bottom-left-radius: var(--rounded-btn, 0.5rem);
    border-bottom-right-radius: 0}
.btn-group .btn:last-child:not(:first-child) {
    border-top-left-radius: 0;
    border-top-right-radius: var(--rounded-btn, 0.5rem);
    border-bottom-left-radius: 0;
    border-bottom-right-radius: var(--rounded-btn, 0.5rem)}
.btn-group-horizontal .btn:not(:first-child):not(:last-child) {
    border-top-left-radius: 0;
    border-top-right-radius: 0;
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0}
.btn-group-horizontal .btn:first-child:not(:last-child) {
    margin-top: -0px;
    margin-left: -1px;
    border-top-left-radius: var(--rounded-btn, 0.5rem);
    border-top-right-radius: 0;
    border-bottom-left-radius: var(--rounded-btn, 0.5rem);
    border-bottom-right-radius: 0}
.btn-group-horizontal .btn:last-child:not(:first-child) {
    border-top-left-radius: 0;
    border-top-right-radius: var(--rounded-btn, 0.5rem);
    border-bottom-left-radius: 0;
    border-bottom-right-radius: var(--rounded-btn, 0.5rem)}
.btn-group-vertical .btn:first-child:not(:last-child) {
    margin-top: -1px;
    margin-left: -0px;
    border-top-left-radius: var(--rounded-btn, 0.5rem);
    border-top-right-radius: var(--rounded-btn, 0.5rem);
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0}
.btn-group-vertical .btn:last-child:not(:first-child) {
    border-top-left-radius: 0;
    border-top-right-radius: 0;
    border-bottom-left-radius: var(--rounded-btn, 0.5rem);
    border-bottom-right-radius: var(--rounded-btn, 0.5rem)}
.card-compact .card-body {
    padding: 1rem;
    font-size: 0.875rem;
    line-height: 1.25rem}
.card-compact .card-title {
    margin-bottom: 0.25rem}
.card-normal .card-body {
    padding: var(--padding-card, 2rem);
    font-size: 1rem;
    line-height: 1.5rem}
.card-normal .card-title {
    margin-bottom: 0.75rem}
.divider-horizontal {
    margin-top: 0px;
    margin-bottom: 0px;
    margin-left: 1rem;
    margin-right: 1rem;
    height: auto;
    width: 1rem}
.divider-vertical {
    margin-left: 0px;
    margin-right: 0px;
    margin-top: 1rem;
    margin-bottom: 1rem;
    height: 1rem;
    width: auto}
.drawer-open > .drawer-toggle ~ .drawer-side > .drawer-overlay {
    cursor: default;
    background-color: transparent}
.join.join-vertical > :where(*:not(:first-child)) {
    margin-left: 0px;
    margin-right: 0px;
    margin-top: -1px}
.join.join-horizontal > :where(*:not(:first-child)) {
    margin-top: 0px;
    margin-bottom: 0px;
    margin-left: -1px}
.menu-horizontal > li:not(.menu-title) > details > ul {
    margin-top: 1rem;
    margin-left: 0px;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    padding-right: 0.5rem}
.menu-horizontal > li > details > ul:before {
    content: none}
:where(.menu-horizontal > li:not(.menu-title) > details > ul) {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--b1) / var(--tw-bg-opacity));
    --tw-shadow: 0 20px 25px -5px rgb(0 0 0 / 0.1), 0 8px 10px -6px rgb(0 0 0 / 0.1);
    --tw-shadow-colored: 0 20px 25px -5px var(--tw-shadow-color), 0 8px 10px -6px var(--tw-shadow-color);
    box-shadow: var(--tw-ring-offset-shadow, 0 0 #0000), var(--tw-ring-shadow, 0 0 #0000), var(--tw-shadow);
    border-radius: var(--rounded-box, 1rem)}
.menu-vertical > li:not(.menu-title) > details > ul {
    margin-top: 0px;
    margin-left: 1rem;
    padding-top: 0px;
    padding-bottom: 0px;
    padding-right: 0px}
.menu-xs :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),
  .menu-xs :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
    border-radius: 0.25rem;
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
    font-size: 0.75rem;
    line-height: 1rem}
.menu-xs .menu-title {
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    padding-top: 0.25rem;
    padding-bottom: 0.25rem}
.menu-sm :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),
  .menu-sm :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    border-radius: var(--rounded-btn, 0.5rem)}
.menu-sm .menu-title {
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem}
.menu-md :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),
  .menu-md :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    border-radius: var(--rounded-btn, 0.5rem)}
.menu-md .menu-title {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem}
.menu-lg :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),
  .menu-lg :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    padding-top: 0.75rem;
    padding-bottom: 0.75rem;
    font-size: 1.125rem;
    line-height: 1.75rem;
    border-radius: var(--rounded-btn, 0.5rem)}
.menu-lg .menu-title {
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    padding-top: 0.75rem;
    padding-bottom: 0.75rem}
.modal-top :where(.modal-box) {
    width: 100%;
    max-width: none;
    --tw-translate-y: -2.5rem;
    --tw-scale-x: 1;
    --tw-scale-y: 1;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    border-top-left-radius: 0px;
    border-top-right-radius: 0px;
    border-bottom-left-radius: var(--rounded-box, 1rem);
    border-bottom-right-radius: var(--rounded-box, 1rem)}
.modal-middle :where(.modal-box) {
    width: 91.666667%;
    max-width: 32rem;
    --tw-translate-y: 0px;
    --tw-scale-x: .9;
    --tw-scale-y: .9;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    border-top-left-radius: var(--rounded-box, 1rem);
    border-top-right-radius: var(--rounded-box, 1rem);
    border-bottom-left-radius: var(--rounded-box, 1rem);
    border-bottom-right-radius: var(--rounded-box, 1rem)}
.modal-bottom :where(.modal-box) {
    width: 100%;
    max-width: none;
    --tw-translate-y: 2.5rem;
    --tw-scale-x: 1;
    --tw-scale-y: 1;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    border-bottom-right-radius: 0px;
    border-bottom-left-radius: 0px;
    border-top-left-radius: var(--rounded-box, 1rem);
    border-top-right-radius: var(--rounded-box, 1rem)}
.stats-horizontal > :not([hidden]) ~ :not([hidden]) {
    --tw-divide-x-reverse: 0;
    border-right-width: calc(1px * var(--tw-divide-x-reverse));
    border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
    --tw-divide-y-reverse: 0;
    border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
    border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}
.stats-horizontal {
    overflow-x: auto}
.stats-vertical > :not([hidden]) ~ :not([hidden]) {
    --tw-divide-y-reverse: 0;
    border-top-width: calc(1px * calc(1 - var(--tw-divide-y-reverse)));
    border-bottom-width: calc(1px * var(--tw-divide-y-reverse));
    --tw-divide-x-reverse: 0;
    border-right-width: calc(0px * var(--tw-divide-x-reverse));
    border-left-width: calc(0px * calc(1 - var(--tw-divide-x-reverse)))}
.stats-vertical {
    overflow-y: auto}
.steps-horizontal .step {
    grid-template-rows: 40px 1fr;
    grid-template-columns: auto;
    min-width: 4rem}
.steps-horizontal .step:before {
    height: 0.5rem;
    width: 100%;
    --tw-translate-y: 0px;
    --tw-translate-x: 0px;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    content: "";
    margin-left: -100%}
.steps-vertical .step {
    gap: 0.5rem;
    grid-template-columns: 40px 1fr;
    grid-template-rows: auto;
    min-height: 4rem;
    justify-items: start}
.steps-vertical .step:before {
    height: 100%;
    width: 0.5rem;
    --tw-translate-y: -50%;
    --tw-translate-x: -50%;
    transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
    margin-left: 50%}
[dir="rtl"] .steps-vertical .step:before {
    margin-right: auto}
.table-xs :not(thead):not(tfoot) tr {
    font-size: 0.75rem;
    line-height: 1rem}
.table-xs :where(th, td) {
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    padding-top: 0.25rem;
    padding-bottom: 0.25rem}
.table-sm :not(thead):not(tfoot) tr {
    font-size: 0.875rem;
    line-height: 1.25rem}
.table-sm :where(th, td) {
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem}
.table-md :not(thead):not(tfoot) tr {
    font-size: 0.875rem;
    line-height: 1.25rem}
.table-md :where(th, td) {
    padding-left: 1rem;
    padding-right: 1rem;
    padding-top: 0.75rem;
    padding-bottom: 0.75rem}
.table-lg :not(thead):not(tfoot) tr {
    font-size: 1rem;
    line-height: 1.5rem}
.table-lg :where(th, td) {
    padding-left: 1.5rem;
    padding-right: 1.5rem;
    padding-top: 1rem;
    padding-bottom: 1rem}
.tooltip {
    position: relative;
    display: inline-block;
    text-align: center;
    --tooltip-tail: 0.1875rem;
    --tooltip-color: hsl(var(--n));
    --tooltip-text-color: hsl(var(--nc));
    --tooltip-tail-offset: calc(100% + 0.0625rem - var(--tooltip-tail))}
.tooltip:before,
.tooltip:after {
    opacity: 0;
    transition-property: color, background-color, border-color, text-decoration-color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter;
    transition-delay: 100ms;
    transition-duration: 200ms;
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1)}
.tooltip:after {
    position: absolute;
    content: "";
    border-style: solid;
    border-width: var(--tooltip-tail, 0);
    width: 0;
    height: 0;
    display: block}
.tooltip:before {
    max-width: 20rem;
    border-radius: 0.25rem;
    padding-left: 0.5rem;
    padding-right: 0.5rem;
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
    font-size: 0.875rem;
    line-height: 1.25rem;
    background-color: var(--tooltip-color);
    color: var(--tooltip-text-color);
    width: max-content}
.tooltip.tooltip-open:before,
.tooltip.tooltip-open:after,
.tooltip:hover:before,
.tooltip:hover:after {
    opacity: 1;
    transition-delay: 75ms}
.tooltip:has(:focus-visible):after,
.tooltip:has(:focus-visible):before {
    opacity: 1;
    transition-delay: 75ms}
.tooltip:not([data-tip]):hover:before,
.tooltip:not([data-tip]):hover:after {
    visibility: hidden;
    opacity: 0}
.tooltip:after, .tooltip-top:after {
    transform: translateX(-50%);
    border-color: var(--tooltip-color) transparent transparent transparent;
    top: auto;
    left: 50%;
    right: auto;
    bottom: var(--tooltip-tail-offset)}
.tooltip-bottom:after {
    transform: translateX(-50%);
    border-color: transparent transparent var(--tooltip-color) transparent;
    top: var(--tooltip-tail-offset);
    left: 50%;
    right: auto;
    bottom: auto}
.tooltip-left:after {
    transform: translateY(-50%);
    border-color: transparent transparent transparent var(--tooltip-color);
    top: 50%;
    left: auto;
    right: calc(var(--tooltip-tail-offset) + 0.0625rem);
    bottom: auto}
.tooltip-right:after {
    transform: translateY(-50%);
    border-color: transparent var(--tooltip-color) transparent transparent;
    top: 50%;
    left: calc(var(--tooltip-tail-offset) + 0.0625rem);
    right: auto;
    bottom: auto}
@media (hover:hover) {
    .hover\:checkbox-success:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity))}
    .hover\:checkbox-warning:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity))}
    .hover\:checkbox-info:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity))}
    .hover\:checkbox-error:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity))}
    .hover\:link-success:hover:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--su) / var(--tw-text-opacity))}
    .hover\:link-info:hover:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--in) / var(--tw-text-opacity))}
    .hover\:link-warning:hover:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--wa) / var(--tw-text-opacity))}
    .hover\:link-error:hover:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--er) / var(--tw-text-opacity))}
    .hover\:radio-success:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity))}
    .hover\:radio-warning:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity))}
    .hover\:radio-info:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity))}
    .hover\:radio-error:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity))}}
@media (hover: hover) {
    .hover\:btn-info:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--in) / var(--tw-bg-opacity))}
    .hover\:btn-success:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--su) / var(--tw-bg-opacity))}
    .hover\:btn-warning:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
    .hover\:btn-error:hover:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--er) / var(--tw-bg-opacity))}
    .hover\:btn-success:hover.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--su) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--suc) / var(--tw-text-opacity))}
    .hover\:btn-info:hover.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--in) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--inc) / var(--tw-text-opacity))}
    .hover\:btn-warning:hover.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--wa) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--wac) / var(--tw-text-opacity))}
    .hover\:btn-error:hover.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--er) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--erc) / var(--tw-text-opacity))}}
.hover\:alert-info:hover {
    border-color: hsl(var(--in) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--in));
    --alert-bg-mix: hsl(var(--b1))}
.hover\:alert-success:hover {
    border-color: hsl(var(--su) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--su));
    --alert-bg-mix: hsl(var(--b1))}
.hover\:alert-warning:hover {
    border-color: hsl(var(--wa) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--wa));
    --alert-bg-mix: hsl(var(--b1))}
.hover\:alert-error:hover {
    border-color: hsl(var(--er) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--er));
    --alert-bg-mix: hsl(var(--b1))}
.hover\:badge-info:hover {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:badge-success:hover {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:badge-warning:hover {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:badge-error:hover {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:badge-info:hover.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.hover\:badge-success:hover.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.hover\:badge-warning:hover.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.hover\:badge-error:hover.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.hover\:btn-info:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity));
    outline-color: hsl(var(--in) / 1)}
.hover\:btn-info:hover.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.hover\:btn-success:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity));
    outline-color: hsl(var(--su) / 1)}
.hover\:btn-success:hover.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.hover\:btn-warning:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity));
    outline-color: hsl(var(--wa) / 1)}
.hover\:btn-warning:hover.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.hover\:btn-error:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity));
    outline-color: hsl(var(--er) / 1)}
.hover\:btn-error:hover.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.hover\:btn-success:hover.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.hover\:btn-success:hover.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:btn-info:hover.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.hover\:btn-info:hover.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:btn-warning:hover.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.hover\:btn-warning:hover.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:btn-error:hover.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.hover\:btn-error:hover.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:chat-bubble-info:hover {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:chat-bubble-success:hover {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:chat-bubble-warning:hover {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:chat-bubble-error:hover {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:checkbox-success:hover {
    --chkbg: var(--su);
    --chkfg: var(--suc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.hover\:checkbox-success:hover:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.hover\:checkbox-success:hover:checked,.hover\:checkbox-success:hover[checked="true"],.hover\:checkbox-success:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:checkbox-warning:hover {
    --chkbg: var(--wa);
    --chkfg: var(--wac);
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.hover\:checkbox-warning:hover:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.hover\:checkbox-warning:hover:checked,.hover\:checkbox-warning:hover[checked="true"],.hover\:checkbox-warning:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:checkbox-info:hover {
    --chkbg: var(--in);
    --chkfg: var(--inc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.hover\:checkbox-info:hover:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.hover\:checkbox-info:hover:checked,.hover\:checkbox-info:hover[checked="true"],.hover\:checkbox-info:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:checkbox-error:hover {
    --chkbg: var(--er);
    --chkfg: var(--erc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.hover\:checkbox-error:hover:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.hover\:checkbox-error:hover:checked,.hover\:checkbox-error:hover[checked="true"],.hover\:checkbox-error:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:file-input-info:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.hover\:file-input-info:hover:focus {
    outline-color: hsl(var(--in) / 1)}
.hover\:file-input-info:hover::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:file-input-success:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.hover\:file-input-success:hover:focus {
    outline-color: hsl(var(--su) / 1)}
.hover\:file-input-success:hover::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:file-input-warning:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.hover\:file-input-warning:hover:focus {
    outline-color: hsl(var(--wa) / 1)}
.hover\:file-input-warning:hover::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:file-input-error:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.hover\:file-input-error:hover:focus {
    outline-color: hsl(var(--er) / 1)}
.hover\:file-input-error:hover::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:input-info:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.hover\:input-info:hover:focus {
    outline-color: hsl(var(--in) / 1)}
.hover\:input-success:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.hover\:input-success:hover:focus {
    outline-color: hsl(var(--su) / 1)}
.hover\:input-warning:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.hover\:input-warning:hover:focus {
    outline-color: hsl(var(--wa) / 1)}
.hover\:input-error:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.hover\:input-error:hover:focus {
    outline-color: hsl(var(--er) / 1)}
.hover\:link-success:hover {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.hover\:link-info:hover {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.hover\:link-warning:hover {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.hover\:link-error:hover {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.hover\:progress-info:hover::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.hover\:progress-success:hover::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.hover\:progress-warning:hover::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.hover\:progress-error:hover::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.hover\:progress-info:hover:indeterminate {
    --progress-color: hsl(var(--in))}
.hover\:progress-success:hover:indeterminate {
    --progress-color: hsl(var(--su))}
.hover\:progress-warning:hover:indeterminate {
    --progress-color: hsl(var(--wa))}
.hover\:progress-error:hover:indeterminate {
    --progress-color: hsl(var(--er))}
.hover\:progress-info:hover::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.hover\:progress-success:hover::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.hover\:progress-warning:hover::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.hover\:progress-error:hover::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.hover\:radio-success:hover {
    --chkbg: var(--su);
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.hover\:radio-success:hover:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.hover\:radio-success:hover:checked,.hover\:radio-success:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:radio-warning:hover {
    --chkbg: var(--wa);
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.hover\:radio-warning:hover:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.hover\:radio-warning:hover:checked,.hover\:radio-warning:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:radio-info:hover {
    --chkbg: var(--in);
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.hover\:radio-info:hover:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.hover\:radio-info:hover:checked,.hover\:radio-info:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:radio-error:hover {
    --chkbg: var(--er);
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.hover\:radio-error:hover:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.hover\:radio-error:hover:checked,.hover\:radio-error:hover[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:range-success:hover {
    --range-shdw: var(--su)}
.hover\:range-warning:hover {
    --range-shdw: var(--wa)}
.hover\:range-info:hover {
    --range-shdw: var(--in)}
.hover\:range-error:hover {
    --range-shdw: var(--er)}
.hover\:select-info:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.hover\:select-info:hover:focus {
    outline-color: hsl(var(--in) / 1)}
.hover\:select-success:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.hover\:select-success:hover:focus {
    outline-color: hsl(var(--su) / 1)}
.hover\:select-warning:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.hover\:select-warning:hover:focus {
    outline-color: hsl(var(--wa) / 1)}
.hover\:select-error:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.hover\:select-error:hover:focus {
    outline-color: hsl(var(--er) / 1)}
.steps .hover\:step-info:hover + .hover\:step-info:hover:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.steps .hover\:step-info:hover:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.steps .hover\:step-success:hover + .hover\:step-success:hover:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.steps .hover\:step-success:hover:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.steps .hover\:step-warning:hover + .hover\:step-warning:hover:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.steps .hover\:step-warning:hover:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.steps .hover\:step-error:hover + .hover\:step-error:hover:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.steps .hover\:step-error:hover:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:textarea-info:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.hover\:textarea-info:hover:focus {
    outline-color: hsl(var(--in) / 1)}
.hover\:textarea-success:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.hover\:textarea-success:hover:focus {
    outline-color: hsl(var(--su) / 1)}
.hover\:textarea-warning:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.hover\:textarea-warning:hover:focus {
    outline-color: hsl(var(--wa) / 1)}
.hover\:textarea-error:hover {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.hover\:textarea-error:hover:focus {
    outline-color: hsl(var(--er) / 1)}
.hover\:toggle-success:hover:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.hover\:toggle-success:hover:checked,.hover\:toggle-success:hover[checked="true"],.hover\:toggle-success:hover[aria-checked="true"] {
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.hover\:toggle-warning:hover:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.hover\:toggle-warning:hover:checked,.hover\:toggle-warning:hover[checked="true"],.hover\:toggle-warning:hover[aria-checked="true"] {
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.hover\:toggle-info:hover:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.hover\:toggle-info:hover:checked,.hover\:toggle-info:hover[checked="true"],.hover\:toggle-info:hover[aria-checked="true"] {
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.hover\:toggle-error:hover:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.hover\:toggle-error:hover:checked,.hover\:toggle-error:hover[checked="true"],.hover\:toggle-error:hover[aria-checked="true"] {
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.hover\:tooltip-info:hover.tooltip {
    --tooltip-color: hsl(var(--in));
    --tooltip-text-color: hsl(var(--inc))}
.hover\:tooltip-success:hover.tooltip {
    --tooltip-color: hsl(var(--su));
    --tooltip-text-color: hsl(var(--suc))}
.hover\:tooltip-warning:hover.tooltip {
    --tooltip-color: hsl(var(--wa));
    --tooltip-text-color: hsl(var(--wac))}
.hover\:tooltip-error:hover.tooltip {
    --tooltip-color: hsl(var(--er));
    --tooltip-text-color: hsl(var(--erc))}
@media (hover:hover) {
    .focus\:checkbox-success:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity))}
    .focus\:checkbox-warning:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity))}
    .focus\:checkbox-info:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity))}
    .focus\:checkbox-error:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity))}
    .focus\:link-success:focus:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--su) / var(--tw-text-opacity))}
    .focus\:link-info:focus:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--in) / var(--tw-text-opacity))}
    .focus\:link-warning:focus:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--wa) / var(--tw-text-opacity))}
    .focus\:link-error:focus:hover {
        --tw-text-opacity: 1;
        color: hsl(var(--er) / var(--tw-text-opacity))}
    .focus\:radio-success:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity))}
    .focus\:radio-warning:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity))}
    .focus\:radio-info:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity))}
    .focus\:radio-error:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity))}}
@media (hover: hover) {
    .focus\:btn-info:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--in) / var(--tw-bg-opacity))}
    .focus\:btn-success:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--su) / var(--tw-bg-opacity))}
    .focus\:btn-warning:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
    .focus\:btn-error:focus:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--er) / var(--tw-bg-opacity))}
    .focus\:btn-success:focus.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--su) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--su) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--suc) / var(--tw-text-opacity))}
    .focus\:btn-info:focus.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--in) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--in) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--inc) / var(--tw-text-opacity))}
    .focus\:btn-warning:focus.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--wa) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--wa) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--wac) / var(--tw-text-opacity))}
    .focus\:btn-error:focus.btn-outline:hover {
        --tw-border-opacity: 1;
        border-color: hsl(var(--er) / var(--tw-border-opacity));
        --tw-bg-opacity: 1;
        background-color: hsl(var(--er) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--erc) / var(--tw-text-opacity))}}
.focus\:alert-info:focus {
    border-color: hsl(var(--in) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--in));
    --alert-bg-mix: hsl(var(--b1))}
.focus\:alert-success:focus {
    border-color: hsl(var(--su) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--su));
    --alert-bg-mix: hsl(var(--b1))}
.focus\:alert-warning:focus {
    border-color: hsl(var(--wa) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--wa));
    --alert-bg-mix: hsl(var(--b1))}
.focus\:alert-error:focus {
    border-color: hsl(var(--er) / 0.2);
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity));
    --alert-bg: hsl(var(--er));
    --alert-bg-mix: hsl(var(--b1))}
.focus\:badge-info:focus {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:badge-success:focus {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:badge-warning:focus {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:badge-error:focus {
    border-color: transparent;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:badge-info:focus.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.focus\:badge-success:focus.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.focus\:badge-warning:focus.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.focus\:badge-error:focus.badge-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.focus\:btn-info:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity));
    outline-color: hsl(var(--in) / 1)}
.focus\:btn-info:focus.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.focus\:btn-success:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity));
    outline-color: hsl(var(--su) / 1)}
.focus\:btn-success:focus.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.focus\:btn-warning:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity));
    outline-color: hsl(var(--wa) / 1)}
.focus\:btn-warning:focus.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.focus\:btn-error:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity));
    outline-color: hsl(var(--er) / 1)}
.focus\:btn-error:focus.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.focus\:btn-success:focus.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.focus\:btn-success:focus.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:btn-info:focus.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.focus\:btn-info:focus.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:btn-warning:focus.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.focus\:btn-warning:focus.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:btn-error:focus.btn-outline {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.focus\:btn-error:focus.btn-outline.btn-active {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:chat-bubble-info:focus {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:chat-bubble-success:focus {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:chat-bubble-warning:focus {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:chat-bubble-error:focus {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:checkbox-success:focus {
    --chkbg: var(--su);
    --chkfg: var(--suc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.focus\:checkbox-success:focus:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.focus\:checkbox-success:focus:checked,.focus\:checkbox-success:focus[checked="true"],.focus\:checkbox-success:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:checkbox-warning:focus {
    --chkbg: var(--wa);
    --chkfg: var(--wac);
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.focus\:checkbox-warning:focus:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.focus\:checkbox-warning:focus:checked,.focus\:checkbox-warning:focus[checked="true"],.focus\:checkbox-warning:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:checkbox-info:focus {
    --chkbg: var(--in);
    --chkfg: var(--inc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.focus\:checkbox-info:focus:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.focus\:checkbox-info:focus:checked,.focus\:checkbox-info:focus[checked="true"],.focus\:checkbox-info:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:checkbox-error:focus {
    --chkbg: var(--er);
    --chkfg: var(--erc);
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.focus\:checkbox-error:focus:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.focus\:checkbox-error:focus:checked,.focus\:checkbox-error:focus[checked="true"],.focus\:checkbox-error:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:file-input-info:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.focus\:file-input-info:focus:focus {
    outline-color: hsl(var(--in) / 1)}
.focus\:file-input-info:focus::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:file-input-success:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.focus\:file-input-success:focus:focus {
    outline-color: hsl(var(--su) / 1)}
.focus\:file-input-success:focus::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:file-input-warning:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.focus\:file-input-warning:focus:focus {
    outline-color: hsl(var(--wa) / 1)}
.focus\:file-input-warning:focus::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:file-input-error:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.focus\:file-input-error:focus:focus {
    outline-color: hsl(var(--er) / 1)}
.focus\:file-input-error:focus::file-selector-button {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:input-info:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.focus\:input-info:focus:focus {
    outline-color: hsl(var(--in) / 1)}
.focus\:input-success:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.focus\:input-success:focus:focus {
    outline-color: hsl(var(--su) / 1)}
.focus\:input-warning:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.focus\:input-warning:focus:focus {
    outline-color: hsl(var(--wa) / 1)}
.focus\:input-error:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.focus\:input-error:focus:focus {
    outline-color: hsl(var(--er) / 1)}
.focus\:link-success:focus {
    --tw-text-opacity: 1;
    color: hsl(var(--su) / var(--tw-text-opacity))}
.focus\:link-info:focus {
    --tw-text-opacity: 1;
    color: hsl(var(--in) / var(--tw-text-opacity))}
.focus\:link-warning:focus {
    --tw-text-opacity: 1;
    color: hsl(var(--wa) / var(--tw-text-opacity))}
.focus\:link-error:focus {
    --tw-text-opacity: 1;
    color: hsl(var(--er) / var(--tw-text-opacity))}
.focus\:progress-info:focus::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.focus\:progress-success:focus::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.focus\:progress-warning:focus::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.focus\:progress-error:focus::-moz-progress-bar {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.focus\:progress-info:focus:indeterminate {
    --progress-color: hsl(var(--in))}
.focus\:progress-success:focus:indeterminate {
    --progress-color: hsl(var(--su))}
.focus\:progress-warning:focus:indeterminate {
    --progress-color: hsl(var(--wa))}
.focus\:progress-error:focus:indeterminate {
    --progress-color: hsl(var(--er))}
.focus\:progress-info:focus::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.focus\:progress-success:focus::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.focus\:progress-warning:focus::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.focus\:progress-error:focus::-webkit-progress-value {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.focus\:radio-success:focus {
    --chkbg: var(--su);
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.focus\:radio-success:focus:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.focus\:radio-success:focus:checked,.focus\:radio-success:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:radio-warning:focus {
    --chkbg: var(--wa);
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.focus\:radio-warning:focus:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.focus\:radio-warning:focus:checked,.focus\:radio-warning:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:radio-info:focus {
    --chkbg: var(--in);
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.focus\:radio-info:focus:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.focus\:radio-info:focus:checked,.focus\:radio-info:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:radio-error:focus {
    --chkbg: var(--er);
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.focus\:radio-error:focus:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.focus\:radio-error:focus:checked,.focus\:radio-error:focus[aria-checked="true"] {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:range-success:focus {
    --range-shdw: var(--su)}
.focus\:range-warning:focus {
    --range-shdw: var(--wa)}
.focus\:range-info:focus {
    --range-shdw: var(--in)}
.focus\:range-error:focus {
    --range-shdw: var(--er)}
.focus\:select-info:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.focus\:select-info:focus:focus {
    outline-color: hsl(var(--in) / 1)}
.focus\:select-success:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.focus\:select-success:focus:focus {
    outline-color: hsl(var(--su) / 1)}
.focus\:select-warning:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.focus\:select-warning:focus:focus {
    outline-color: hsl(var(--wa) / 1)}
.focus\:select-error:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.focus\:select-error:focus:focus {
    outline-color: hsl(var(--er) / 1)}
.steps .focus\:step-info:focus + .focus\:step-info:focus:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity))}
.steps .focus\:step-info:focus:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.steps .focus\:step-success:focus + .focus\:step-success:focus:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity))}
.steps .focus\:step-success:focus:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.steps .focus\:step-warning:focus + .focus\:step-warning:focus:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity))}
.steps .focus\:step-warning:focus:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.steps .focus\:step-error:focus + .focus\:step-error:focus:before {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity))}
.steps .focus\:step-error:focus:after {
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:textarea-info:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--in) / var(--tw-border-opacity))}
.focus\:textarea-info:focus:focus {
    outline-color: hsl(var(--in) / 1)}
.focus\:textarea-success:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--su) / var(--tw-border-opacity))}
.focus\:textarea-success:focus:focus {
    outline-color: hsl(var(--su) / 1)}
.focus\:textarea-warning:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--wa) / var(--tw-border-opacity))}
.focus\:textarea-warning:focus:focus {
    outline-color: hsl(var(--wa) / 1)}
.focus\:textarea-error:focus {
    --tw-border-opacity: 1;
    border-color: hsl(var(--er) / var(--tw-border-opacity))}
.focus\:textarea-error:focus:focus {
    outline-color: hsl(var(--er) / 1)}
.focus\:toggle-success:focus:focus-visible {
    outline-color: hsl(var(--su) / 1)}
.focus\:toggle-success:focus:checked,.focus\:toggle-success:focus[checked="true"],.focus\:toggle-success:focus[aria-checked="true"] {
    border-color: hsl(var(--su) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--su) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--suc) / var(--tw-text-opacity))}
.focus\:toggle-warning:focus:focus-visible {
    outline-color: hsl(var(--wa) / 1)}
.focus\:toggle-warning:focus:checked,.focus\:toggle-warning:focus[checked="true"],.focus\:toggle-warning:focus[aria-checked="true"] {
    border-color: hsl(var(--wa) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--wa) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--wac) / var(--tw-text-opacity))}
.focus\:toggle-info:focus:focus-visible {
    outline-color: hsl(var(--in) / 1)}
.focus\:toggle-info:focus:checked,.focus\:toggle-info:focus[checked="true"],.focus\:toggle-info:focus[aria-checked="true"] {
    border-color: hsl(var(--in) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--in) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--inc) / var(--tw-text-opacity))}
.focus\:toggle-error:focus:focus-visible {
    outline-color: hsl(var(--er) / 1)}
.focus\:toggle-error:focus:checked,.focus\:toggle-error:focus[checked="true"],.focus\:toggle-error:focus[aria-checked="true"] {
    border-color: hsl(var(--er) / var(--tw-border-opacity));
    --tw-border-opacity: 0.1;
    --tw-bg-opacity: 1;
    background-color: hsl(var(--er) / var(--tw-bg-opacity));
    --tw-text-opacity: 1;
    color: hsl(var(--erc) / var(--tw-text-opacity))}
.focus\:tooltip-info:focus.tooltip {
    --tooltip-color: hsl(var(--in));
    --tooltip-text-color: hsl(var(--inc))}
.focus\:tooltip-success:focus.tooltip {
    --tooltip-color: hsl(var(--su));
    --tooltip-text-color: hsl(var(--suc))}
.focus\:tooltip-warning:focus.tooltip {
    --tooltip-color: hsl(var(--wa));
    --tooltip-text-color: hsl(var(--wac))}
.focus\:tooltip-error:focus.tooltip {
    --tooltip-color: hsl(var(--er));
    --tooltip-text-color: hsl(var(--erc))}
@media (min-width: 640px) {
    .sm\:input-group-md {
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .sm\:input-group-lg {
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .sm\:input-group-sm {
        font-size: 0.875rem;
        line-height: 2rem}
    .sm\:stats {
        display: inline-grid;
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b1) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        border-radius: var(--rounded-box, 1rem)}
    :where(.sm\:stats) {
        grid-auto-flow: column;
        overflow-x: auto}
    .sm\:loading-sm {
        width: 1.25rem}
    .sm\:loading-md {
        width: 1.5rem}
    .sm\:loading-lg {
        width: 2.5rem}
    :where(.sm\:stats) > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}}
@media (min-width: 768px) {
    .md\:input-group-md {
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .md\:input-group-lg {
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .md\:input-group-sm {
        font-size: 0.875rem;
        line-height: 2rem}
    .md\:stats {
        display: inline-grid;
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b1) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        border-radius: var(--rounded-box, 1rem)}
    :where(.md\:stats) {
        grid-auto-flow: column;
        overflow-x: auto}
    .md\:loading-sm {
        width: 1.25rem}
    .md\:loading-md {
        width: 1.5rem}
    .md\:loading-lg {
        width: 2.5rem}
    :where(.md\:stats) > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}}
@media (min-width: 1024px) {
    .lg\:input-group-md {
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .lg\:input-group-lg {
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .lg\:input-group-sm {
        font-size: 0.875rem;
        line-height: 2rem}
    .lg\:stats {
        display: inline-grid;
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b1) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        border-radius: var(--rounded-box, 1rem)}
    :where(.lg\:stats) {
        grid-auto-flow: column;
        overflow-x: auto}
    .lg\:loading-sm {
        width: 1.25rem}
    .lg\:loading-md {
        width: 1.5rem}
    .lg\:loading-lg {
        width: 2.5rem}
    :where(.lg\:stats) > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}}
@media (min-width: 1280px) {
    .xl\:input-group-md {
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .xl\:input-group-lg {
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .xl\:input-group-sm {
        font-size: 0.875rem;
        line-height: 2rem}
    .xl\:stats {
        display: inline-grid;
        --tw-bg-opacity: 1;
        background-color: hsl(var(--b1) / var(--tw-bg-opacity));
        --tw-text-opacity: 1;
        color: hsl(var(--bc) / var(--tw-text-opacity));
        border-radius: var(--rounded-box, 1rem)}
    :where(.xl\:stats) {
        grid-auto-flow: column;
        overflow-x: auto}
    .xl\:loading-sm {
        width: 1.25rem}
    .xl\:loading-md {
        width: 1.5rem}
    .xl\:loading-lg {
        width: 2.5rem}
    :where(.xl\:stats) > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}}
.hover\:divide-base-100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 1)}
.hover\:divide-base-100\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0)}
.hover\:divide-base-100\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.1)}
.hover\:divide-base-100\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 1)}
.hover\:divide-base-100\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.2)}
.hover\:divide-base-100\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.25)}
.hover\:divide-base-100\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.3)}
.hover\:divide-base-100\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.4)}
.hover\:divide-base-100\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.05)}
.hover\:divide-base-100\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.5)}
.hover\:divide-base-100\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.6)}
.hover\:divide-base-100\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.7)}
.hover\:divide-base-100\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.75)}
.hover\:divide-base-100\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.8)}
.hover\:divide-base-100\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.9)}
.hover\:divide-base-100\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.95)}
.hover\:divide-base-200:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 1)}
.hover\:divide-base-200\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0)}
.hover\:divide-base-200\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.1)}
.hover\:divide-base-200\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 1)}
.hover\:divide-base-200\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.2)}
.hover\:divide-base-200\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.25)}
.hover\:divide-base-200\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.3)}
.hover\:divide-base-200\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.4)}
.hover\:divide-base-200\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.05)}
.hover\:divide-base-200\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.5)}
.hover\:divide-base-200\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.6)}
.hover\:divide-base-200\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.7)}
.hover\:divide-base-200\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.75)}
.hover\:divide-base-200\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.8)}
.hover\:divide-base-200\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.9)}
.hover\:divide-base-200\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.95)}
.hover\:divide-base-300:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 1)}
.hover\:divide-base-300\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0)}
.hover\:divide-base-300\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.1)}
.hover\:divide-base-300\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 1)}
.hover\:divide-base-300\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.2)}
.hover\:divide-base-300\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.25)}
.hover\:divide-base-300\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.3)}
.hover\:divide-base-300\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.4)}
.hover\:divide-base-300\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.05)}
.hover\:divide-base-300\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.5)}
.hover\:divide-base-300\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.6)}
.hover\:divide-base-300\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.7)}
.hover\:divide-base-300\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.75)}
.hover\:divide-base-300\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.8)}
.hover\:divide-base-300\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.9)}
.hover\:divide-base-300\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.95)}
.hover\:divide-base-content:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 1)}
.hover\:divide-base-content\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0)}
.hover\:divide-base-content\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.1)}
.hover\:divide-base-content\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 1)}
.hover\:divide-base-content\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.2)}
.hover\:divide-base-content\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.25)}
.hover\:divide-base-content\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.3)}
.hover\:divide-base-content\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.4)}
.hover\:divide-base-content\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.05)}
.hover\:divide-base-content\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.5)}
.hover\:divide-base-content\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.6)}
.hover\:divide-base-content\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.7)}
.hover\:divide-base-content\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.75)}
.hover\:divide-base-content\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.8)}
.hover\:divide-base-content\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.9)}
.hover\:divide-base-content\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.95)}
.hover\:divide-error:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 1)}
.hover\:divide-error-content:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 1)}
.hover\:divide-error-content\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0)}
.hover\:divide-error-content\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.1)}
.hover\:divide-error-content\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 1)}
.hover\:divide-error-content\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.2)}
.hover\:divide-error-content\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.25)}
.hover\:divide-error-content\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.3)}
.hover\:divide-error-content\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.4)}
.hover\:divide-error-content\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.05)}
.hover\:divide-error-content\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.5)}
.hover\:divide-error-content\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.6)}
.hover\:divide-error-content\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.7)}
.hover\:divide-error-content\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.75)}
.hover\:divide-error-content\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.8)}
.hover\:divide-error-content\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.9)}
.hover\:divide-error-content\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.95)}
.hover\:divide-error\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0)}
.hover\:divide-error\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.1)}
.hover\:divide-error\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 1)}
.hover\:divide-error\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.2)}
.hover\:divide-error\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.25)}
.hover\:divide-error\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.3)}
.hover\:divide-error\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.4)}
.hover\:divide-error\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.05)}
.hover\:divide-error\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.5)}
.hover\:divide-error\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.6)}
.hover\:divide-error\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.7)}
.hover\:divide-error\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.75)}
.hover\:divide-error\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.8)}
.hover\:divide-error\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.9)}
.hover\:divide-error\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.95)}
.hover\:divide-info:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 1)}
.hover\:divide-info-content:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 1)}
.hover\:divide-info-content\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0)}
.hover\:divide-info-content\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.1)}
.hover\:divide-info-content\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 1)}
.hover\:divide-info-content\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.2)}
.hover\:divide-info-content\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.25)}
.hover\:divide-info-content\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.3)}
.hover\:divide-info-content\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.4)}
.hover\:divide-info-content\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.05)}
.hover\:divide-info-content\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.5)}
.hover\:divide-info-content\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.6)}
.hover\:divide-info-content\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.7)}
.hover\:divide-info-content\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.75)}
.hover\:divide-info-content\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.8)}
.hover\:divide-info-content\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.9)}
.hover\:divide-info-content\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.95)}
.hover\:divide-info\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0)}
.hover\:divide-info\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.1)}
.hover\:divide-info\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 1)}
.hover\:divide-info\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.2)}
.hover\:divide-info\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.25)}
.hover\:divide-info\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.3)}
.hover\:divide-info\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.4)}
.hover\:divide-info\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.05)}
.hover\:divide-info\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.5)}
.hover\:divide-info\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.6)}
.hover\:divide-info\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.7)}
.hover\:divide-info\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.75)}
.hover\:divide-info\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.8)}
.hover\:divide-info\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.9)}
.hover\:divide-info\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.95)}
.hover\:divide-success:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 1)}
.hover\:divide-success-content:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 1)}
.hover\:divide-success-content\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0)}
.hover\:divide-success-content\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.1)}
.hover\:divide-success-content\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 1)}
.hover\:divide-success-content\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.2)}
.hover\:divide-success-content\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.25)}
.hover\:divide-success-content\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.3)}
.hover\:divide-success-content\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.4)}
.hover\:divide-success-content\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.05)}
.hover\:divide-success-content\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.5)}
.hover\:divide-success-content\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.6)}
.hover\:divide-success-content\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.7)}
.hover\:divide-success-content\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.75)}
.hover\:divide-success-content\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.8)}
.hover\:divide-success-content\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.9)}
.hover\:divide-success-content\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.95)}
.hover\:divide-success\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0)}
.hover\:divide-success\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.1)}
.hover\:divide-success\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 1)}
.hover\:divide-success\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.2)}
.hover\:divide-success\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.25)}
.hover\:divide-success\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.3)}
.hover\:divide-success\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.4)}
.hover\:divide-success\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.05)}
.hover\:divide-success\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.5)}
.hover\:divide-success\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.6)}
.hover\:divide-success\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.7)}
.hover\:divide-success\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.75)}
.hover\:divide-success\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.8)}
.hover\:divide-success\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.9)}
.hover\:divide-success\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.95)}
.hover\:divide-warning:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 1)}
.hover\:divide-warning-content:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 1)}
.hover\:divide-warning-content\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0)}
.hover\:divide-warning-content\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.1)}
.hover\:divide-warning-content\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 1)}
.hover\:divide-warning-content\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.2)}
.hover\:divide-warning-content\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.25)}
.hover\:divide-warning-content\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.3)}
.hover\:divide-warning-content\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.4)}
.hover\:divide-warning-content\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.05)}
.hover\:divide-warning-content\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.5)}
.hover\:divide-warning-content\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.6)}
.hover\:divide-warning-content\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.7)}
.hover\:divide-warning-content\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.75)}
.hover\:divide-warning-content\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.8)}
.hover\:divide-warning-content\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.9)}
.hover\:divide-warning-content\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.95)}
.hover\:divide-warning\/0:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0)}
.hover\:divide-warning\/10:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.1)}
.hover\:divide-warning\/100:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 1)}
.hover\:divide-warning\/20:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.2)}
.hover\:divide-warning\/25:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.25)}
.hover\:divide-warning\/30:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.3)}
.hover\:divide-warning\/40:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.4)}
.hover\:divide-warning\/5:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.05)}
.hover\:divide-warning\/50:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.5)}
.hover\:divide-warning\/60:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.6)}
.hover\:divide-warning\/70:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.7)}
.hover\:divide-warning\/75:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.75)}
.hover\:divide-warning\/80:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.8)}
.hover\:divide-warning\/90:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.9)}
.hover\:divide-warning\/95:hover > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.95)}
.hover\:border-accent:hover {
    border-color: hsl(var(--a) / 1)}
.hover\:border-accent-content:hover {
    border-color: hsl(var(--ac) / 1)}
.hover\:border-accent-content\/0:hover {
    border-color: hsl(var(--ac) / 0)}
.hover\:border-accent-content\/10:hover {
    border-color: hsl(var(--ac) / 0.1)}
.hover\:border-accent-content\/100:hover {
    border-color: hsl(var(--ac) / 1)}
.hover\:border-accent-content\/20:hover {
    border-color: hsl(var(--ac) / 0.2)}
.hover\:border-accent-content\/25:hover {
    border-color: hsl(var(--ac) / 0.25)}
.hover\:border-accent-content\/30:hover {
    border-color: hsl(var(--ac) / 0.3)}
.hover\:border-accent-content\/40:hover {
    border-color: hsl(var(--ac) / 0.4)}
.hover\:border-accent-content\/5:hover {
    border-color: hsl(var(--ac) / 0.05)}
.hover\:border-accent-content\/50:hover {
    border-color: hsl(var(--ac) / 0.5)}
.hover\:border-accent-content\/60:hover {
    border-color: hsl(var(--ac) / 0.6)}
.hover\:border-accent-content\/70:hover {
    border-color: hsl(var(--ac) / 0.7)}
.hover\:border-accent-content\/75:hover {
    border-color: hsl(var(--ac) / 0.75)}
.hover\:border-accent-content\/80:hover {
    border-color: hsl(var(--ac) / 0.8)}
.hover\:border-accent-content\/90:hover {
    border-color: hsl(var(--ac) / 0.9)}
.hover\:border-accent-content\/95:hover {
    border-color: hsl(var(--ac) / 0.95)}
.hover\:border-accent-focus:hover {
    border-color: hsl(var(--af) / 1)}
.hover\:border-accent-focus\/0:hover {
    border-color: hsl(var(--af) / 0)}
.hover\:border-accent-focus\/10:hover {
    border-color: hsl(var(--af) / 0.1)}
.hover\:border-accent-focus\/100:hover {
    border-color: hsl(var(--af) / 1)}
.hover\:border-accent-focus\/20:hover {
    border-color: hsl(var(--af) / 0.2)}
.hover\:border-accent-focus\/25:hover {
    border-color: hsl(var(--af) / 0.25)}
.hover\:border-accent-focus\/30:hover {
    border-color: hsl(var(--af) / 0.3)}
.hover\:border-accent-focus\/40:hover {
    border-color: hsl(var(--af) / 0.4)}
.hover\:border-accent-focus\/5:hover {
    border-color: hsl(var(--af) / 0.05)}
.hover\:border-accent-focus\/50:hover {
    border-color: hsl(var(--af) / 0.5)}
.hover\:border-accent-focus\/60:hover {
    border-color: hsl(var(--af) / 0.6)}
.hover\:border-accent-focus\/70:hover {
    border-color: hsl(var(--af) / 0.7)}
.hover\:border-accent-focus\/75:hover {
    border-color: hsl(var(--af) / 0.75)}
.hover\:border-accent-focus\/80:hover {
    border-color: hsl(var(--af) / 0.8)}
.hover\:border-accent-focus\/90:hover {
    border-color: hsl(var(--af) / 0.9)}
.hover\:border-accent-focus\/95:hover {
    border-color: hsl(var(--af) / 0.95)}
.hover\:border-accent\/0:hover {
    border-color: hsl(var(--a) / 0)}
.hover\:border-accent\/10:hover {
    border-color: hsl(var(--a) / 0.1)}
.hover\:border-accent\/100:hover {
    border-color: hsl(var(--a) / 1)}
.hover\:border-accent\/20:hover {
    border-color: hsl(var(--a) / 0.2)}
.hover\:border-accent\/25:hover {
    border-color: hsl(var(--a) / 0.25)}
.hover\:border-accent\/30:hover {
    border-color: hsl(var(--a) / 0.3)}
.hover\:border-accent\/40:hover {
    border-color: hsl(var(--a) / 0.4)}
.hover\:border-accent\/5:hover {
    border-color: hsl(var(--a) / 0.05)}
.hover\:border-accent\/50:hover {
    border-color: hsl(var(--a) / 0.5)}
.hover\:border-accent\/60:hover {
    border-color: hsl(var(--a) / 0.6)}
.hover\:border-accent\/70:hover {
    border-color: hsl(var(--a) / 0.7)}
.hover\:border-accent\/75:hover {
    border-color: hsl(var(--a) / 0.75)}
.hover\:border-accent\/80:hover {
    border-color: hsl(var(--a) / 0.8)}
.hover\:border-accent\/90:hover {
    border-color: hsl(var(--a) / 0.9)}
.hover\:border-accent\/95:hover {
    border-color: hsl(var(--a) / 0.95)}
.hover\:border-base-100:hover {
    border-color: hsl(var(--b1) / 1)}
.hover\:border-base-100\/0:hover {
    border-color: hsl(var(--b1) / 0)}
.hover\:border-base-100\/10:hover {
    border-color: hsl(var(--b1) / 0.1)}
.hover\:border-base-100\/100:hover {
    border-color: hsl(var(--b1) / 1)}
.hover\:border-base-100\/20:hover {
    border-color: hsl(var(--b1) / 0.2)}
.hover\:border-base-100\/25:hover {
    border-color: hsl(var(--b1) / 0.25)}
.hover\:border-base-100\/30:hover {
    border-color: hsl(var(--b1) / 0.3)}
.hover\:border-base-100\/40:hover {
    border-color: hsl(var(--b1) / 0.4)}
.hover\:border-base-100\/5:hover {
    border-color: hsl(var(--b1) / 0.05)}
.hover\:border-base-100\/50:hover {
    border-color: hsl(var(--b1) / 0.5)}
.hover\:border-base-100\/60:hover {
    border-color: hsl(var(--b1) / 0.6)}
.hover\:border-base-100\/70:hover {
    border-color: hsl(var(--b1) / 0.7)}
.hover\:border-base-100\/75:hover {
    border-color: hsl(var(--b1) / 0.75)}
.hover\:border-base-100\/80:hover {
    border-color: hsl(var(--b1) / 0.8)}
.hover\:border-base-100\/90:hover {
    border-color: hsl(var(--b1) / 0.9)}
.hover\:border-base-100\/95:hover {
    border-color: hsl(var(--b1) / 0.95)}
.hover\:border-base-200:hover {
    border-color: hsl(var(--b2) / 1)}
.hover\:border-base-200\/0:hover {
    border-color: hsl(var(--b2) / 0)}
.hover\:border-base-200\/10:hover {
    border-color: hsl(var(--b2) / 0.1)}
.hover\:border-base-200\/100:hover {
    border-color: hsl(var(--b2) / 1)}
.hover\:border-base-200\/20:hover {
    border-color: hsl(var(--b2) / 0.2)}
.hover\:border-base-200\/25:hover {
    border-color: hsl(var(--b2) / 0.25)}
.hover\:border-base-200\/30:hover {
    border-color: hsl(var(--b2) / 0.3)}
.hover\:border-base-200\/40:hover {
    border-color: hsl(var(--b2) / 0.4)}
.hover\:border-base-200\/5:hover {
    border-color: hsl(var(--b2) / 0.05)}
.hover\:border-base-200\/50:hover {
    border-color: hsl(var(--b2) / 0.5)}
.hover\:border-base-200\/60:hover {
    border-color: hsl(var(--b2) / 0.6)}
.hover\:border-base-200\/70:hover {
    border-color: hsl(var(--b2) / 0.7)}
.hover\:border-base-200\/75:hover {
    border-color: hsl(var(--b2) / 0.75)}
.hover\:border-base-200\/80:hover {
    border-color: hsl(var(--b2) / 0.8)}
.hover\:border-base-200\/90:hover {
    border-color: hsl(var(--b2) / 0.9)}
.hover\:border-base-200\/95:hover {
    border-color: hsl(var(--b2) / 0.95)}
.hover\:border-base-300:hover {
    border-color: hsl(var(--b3) / 1)}
.hover\:border-base-300\/0:hover {
    border-color: hsl(var(--b3) / 0)}
.hover\:border-base-300\/10:hover {
    border-color: hsl(var(--b3) / 0.1)}
.hover\:border-base-300\/100:hover {
    border-color: hsl(var(--b3) / 1)}
.hover\:border-base-300\/20:hover {
    border-color: hsl(var(--b3) / 0.2)}
.hover\:border-base-300\/25:hover {
    border-color: hsl(var(--b3) / 0.25)}
.hover\:border-base-300\/30:hover {
    border-color: hsl(var(--b3) / 0.3)}
.hover\:border-base-300\/40:hover {
    border-color: hsl(var(--b3) / 0.4)}
.hover\:border-base-300\/5:hover {
    border-color: hsl(var(--b3) / 0.05)}
.hover\:border-base-300\/50:hover {
    border-color: hsl(var(--b3) / 0.5)}
.hover\:border-base-300\/60:hover {
    border-color: hsl(var(--b3) / 0.6)}
.hover\:border-base-300\/70:hover {
    border-color: hsl(var(--b3) / 0.7)}
.hover\:border-base-300\/75:hover {
    border-color: hsl(var(--b3) / 0.75)}
.hover\:border-base-300\/80:hover {
    border-color: hsl(var(--b3) / 0.8)}
.hover\:border-base-300\/90:hover {
    border-color: hsl(var(--b3) / 0.9)}
.hover\:border-base-300\/95:hover {
    border-color: hsl(var(--b3) / 0.95)}
.hover\:border-base-content:hover {
    border-color: hsl(var(--bc) / 1)}
.hover\:border-base-content\/0:hover {
    border-color: hsl(var(--bc) / 0)}
.hover\:border-base-content\/10:hover {
    border-color: hsl(var(--bc) / 0.1)}
.hover\:border-base-content\/100:hover {
    border-color: hsl(var(--bc) / 1)}
.hover\:border-base-content\/20:hover {
    border-color: hsl(var(--bc) / 0.2)}
.hover\:border-base-content\/25:hover {
    border-color: hsl(var(--bc) / 0.25)}
.hover\:border-base-content\/30:hover {
    border-color: hsl(var(--bc) / 0.3)}
.hover\:border-base-content\/40:hover {
    border-color: hsl(var(--bc) / 0.4)}
.hover\:border-base-content\/5:hover {
    border-color: hsl(var(--bc) / 0.05)}
.hover\:border-base-content\/50:hover {
    border-color: hsl(var(--bc) / 0.5)}
.hover\:border-base-content\/60:hover {
    border-color: hsl(var(--bc) / 0.6)}
.hover\:border-base-content\/70:hover {
    border-color: hsl(var(--bc) / 0.7)}
.hover\:border-base-content\/75:hover {
    border-color: hsl(var(--bc) / 0.75)}
.hover\:border-base-content\/80:hover {
    border-color: hsl(var(--bc) / 0.8)}
.hover\:border-base-content\/90:hover {
    border-color: hsl(var(--bc) / 0.9)}
.hover\:border-base-content\/95:hover {
    border-color: hsl(var(--bc) / 0.95)}
.hover\:border-error:hover {
    border-color: hsl(var(--er) / 1)}
.hover\:border-error-content:hover {
    border-color: hsl(var(--erc) / 1)}
.hover\:border-error-content\/0:hover {
    border-color: hsl(var(--erc) / 0)}
.hover\:border-error-content\/10:hover {
    border-color: hsl(var(--erc) / 0.1)}
.hover\:border-error-content\/100:hover {
    border-color: hsl(var(--erc) / 1)}
.hover\:border-error-content\/20:hover {
    border-color: hsl(var(--erc) / 0.2)}
.hover\:border-error-content\/25:hover {
    border-color: hsl(var(--erc) / 0.25)}
.hover\:border-error-content\/30:hover {
    border-color: hsl(var(--erc) / 0.3)}
.hover\:border-error-content\/40:hover {
    border-color: hsl(var(--erc) / 0.4)}
.hover\:border-error-content\/5:hover {
    border-color: hsl(var(--erc) / 0.05)}
.hover\:border-error-content\/50:hover {
    border-color: hsl(var(--erc) / 0.5)}
.hover\:border-error-content\/60:hover {
    border-color: hsl(var(--erc) / 0.6)}
.hover\:border-error-content\/70:hover {
    border-color: hsl(var(--erc) / 0.7)}
.hover\:border-error-content\/75:hover {
    border-color: hsl(var(--erc) / 0.75)}
.hover\:border-error-content\/80:hover {
    border-color: hsl(var(--erc) / 0.8)}
.hover\:border-error-content\/90:hover {
    border-color: hsl(var(--erc) / 0.9)}
.hover\:border-error-content\/95:hover {
    border-color: hsl(var(--erc) / 0.95)}
.hover\:border-error\/0:hover {
    border-color: hsl(var(--er) / 0)}
.hover\:border-error\/10:hover {
    border-color: hsl(var(--er) / 0.1)}
.hover\:border-error\/100:hover {
    border-color: hsl(var(--er) / 1)}
.hover\:border-error\/20:hover {
    border-color: hsl(var(--er) / 0.2)}
.hover\:border-error\/25:hover {
    border-color: hsl(var(--er) / 0.25)}
.hover\:border-error\/30:hover {
    border-color: hsl(var(--er) / 0.3)}
.hover\:border-error\/40:hover {
    border-color: hsl(var(--er) / 0.4)}
.hover\:border-error\/5:hover {
    border-color: hsl(var(--er) / 0.05)}
.hover\:border-error\/50:hover {
    border-color: hsl(var(--er) / 0.5)}
.hover\:border-error\/60:hover {
    border-color: hsl(var(--er) / 0.6)}
.hover\:border-error\/70:hover {
    border-color: hsl(var(--er) / 0.7)}
.hover\:border-error\/75:hover {
    border-color: hsl(var(--er) / 0.75)}
.hover\:border-error\/80:hover {
    border-color: hsl(var(--er) / 0.8)}
.hover\:border-error\/90:hover {
    border-color: hsl(var(--er) / 0.9)}
.hover\:border-error\/95:hover {
    border-color: hsl(var(--er) / 0.95)}
.hover\:border-info:hover {
    border-color: hsl(var(--in) / 1)}
.hover\:border-info-content:hover {
    border-color: hsl(var(--inc) / 1)}
.hover\:border-info-content\/0:hover {
    border-color: hsl(var(--inc) / 0)}
.hover\:border-info-content\/10:hover {
    border-color: hsl(var(--inc) / 0.1)}
.hover\:border-info-content\/100:hover {
    border-color: hsl(var(--inc) / 1)}
.hover\:border-info-content\/20:hover {
    border-color: hsl(var(--inc) / 0.2)}
.hover\:border-info-content\/25:hover {
    border-color: hsl(var(--inc) / 0.25)}
.hover\:border-info-content\/30:hover {
    border-color: hsl(var(--inc) / 0.3)}
.hover\:border-info-content\/40:hover {
    border-color: hsl(var(--inc) / 0.4)}
.hover\:border-info-content\/5:hover {
    border-color: hsl(var(--inc) / 0.05)}
.hover\:border-info-content\/50:hover {
    border-color: hsl(var(--inc) / 0.5)}
.hover\:border-info-content\/60:hover {
    border-color: hsl(var(--inc) / 0.6)}
.hover\:border-info-content\/70:hover {
    border-color: hsl(var(--inc) / 0.7)}
.hover\:border-info-content\/75:hover {
    border-color: hsl(var(--inc) / 0.75)}
.hover\:border-info-content\/80:hover {
    border-color: hsl(var(--inc) / 0.8)}
.hover\:border-info-content\/90:hover {
    border-color: hsl(var(--inc) / 0.9)}
.hover\:border-info-content\/95:hover {
    border-color: hsl(var(--inc) / 0.95)}
.hover\:border-info\/0:hover {
    border-color: hsl(var(--in) / 0)}
.hover\:border-info\/10:hover {
    border-color: hsl(var(--in) / 0.1)}
.hover\:border-info\/100:hover {
    border-color: hsl(var(--in) / 1)}
.hover\:border-info\/20:hover {
    border-color: hsl(var(--in) / 0.2)}
.hover\:border-info\/25:hover {
    border-color: hsl(var(--in) / 0.25)}
.hover\:border-info\/30:hover {
    border-color: hsl(var(--in) / 0.3)}
.hover\:border-info\/40:hover {
    border-color: hsl(var(--in) / 0.4)}
.hover\:border-info\/5:hover {
    border-color: hsl(var(--in) / 0.05)}
.hover\:border-info\/50:hover {
    border-color: hsl(var(--in) / 0.5)}
.hover\:border-info\/60:hover {
    border-color: hsl(var(--in) / 0.6)}
.hover\:border-info\/70:hover {
    border-color: hsl(var(--in) / 0.7)}
.hover\:border-info\/75:hover {
    border-color: hsl(var(--in) / 0.75)}
.hover\:border-info\/80:hover {
    border-color: hsl(var(--in) / 0.8)}
.hover\:border-info\/90:hover {
    border-color: hsl(var(--in) / 0.9)}
.hover\:border-info\/95:hover {
    border-color: hsl(var(--in) / 0.95)}
.hover\:border-neutral:hover {
    border-color: hsl(var(--n) / 1)}
.hover\:border-neutral-content:hover {
    border-color: hsl(var(--nc) / 1)}
.hover\:border-neutral-content\/0:hover {
    border-color: hsl(var(--nc) / 0)}
.hover\:border-neutral-content\/10:hover {
    border-color: hsl(var(--nc) / 0.1)}
.hover\:border-neutral-content\/100:hover {
    border-color: hsl(var(--nc) / 1)}
.hover\:border-neutral-content\/20:hover {
    border-color: hsl(var(--nc) / 0.2)}
.hover\:border-neutral-content\/25:hover {
    border-color: hsl(var(--nc) / 0.25)}
.hover\:border-neutral-content\/30:hover {
    border-color: hsl(var(--nc) / 0.3)}
.hover\:border-neutral-content\/40:hover {
    border-color: hsl(var(--nc) / 0.4)}
.hover\:border-neutral-content\/5:hover {
    border-color: hsl(var(--nc) / 0.05)}
.hover\:border-neutral-content\/50:hover {
    border-color: hsl(var(--nc) / 0.5)}
.hover\:border-neutral-content\/60:hover {
    border-color: hsl(var(--nc) / 0.6)}
.hover\:border-neutral-content\/70:hover {
    border-color: hsl(var(--nc) / 0.7)}
.hover\:border-neutral-content\/75:hover {
    border-color: hsl(var(--nc) / 0.75)}
.hover\:border-neutral-content\/80:hover {
    border-color: hsl(var(--nc) / 0.8)}
.hover\:border-neutral-content\/90:hover {
    border-color: hsl(var(--nc) / 0.9)}
.hover\:border-neutral-content\/95:hover {
    border-color: hsl(var(--nc) / 0.95)}
.hover\:border-neutral-focus:hover {
    border-color: hsl(var(--nf) / 1)}
.hover\:border-neutral-focus\/0:hover {
    border-color: hsl(var(--nf) / 0)}
.hover\:border-neutral-focus\/10:hover {
    border-color: hsl(var(--nf) / 0.1)}
.hover\:border-neutral-focus\/100:hover {
    border-color: hsl(var(--nf) / 1)}
.hover\:border-neutral-focus\/20:hover {
    border-color: hsl(var(--nf) / 0.2)}
.hover\:border-neutral-focus\/25:hover {
    border-color: hsl(var(--nf) / 0.25)}
.hover\:border-neutral-focus\/30:hover {
    border-color: hsl(var(--nf) / 0.3)}
.hover\:border-neutral-focus\/40:hover {
    border-color: hsl(var(--nf) / 0.4)}
.hover\:border-neutral-focus\/5:hover {
    border-color: hsl(var(--nf) / 0.05)}
.hover\:border-neutral-focus\/50:hover {
    border-color: hsl(var(--nf) / 0.5)}
.hover\:border-neutral-focus\/60:hover {
    border-color: hsl(var(--nf) / 0.6)}
.hover\:border-neutral-focus\/70:hover {
    border-color: hsl(var(--nf) / 0.7)}
.hover\:border-neutral-focus\/75:hover {
    border-color: hsl(var(--nf) / 0.75)}
.hover\:border-neutral-focus\/80:hover {
    border-color: hsl(var(--nf) / 0.8)}
.hover\:border-neutral-focus\/90:hover {
    border-color: hsl(var(--nf) / 0.9)}
.hover\:border-neutral-focus\/95:hover {
    border-color: hsl(var(--nf) / 0.95)}
.hover\:border-neutral\/0:hover {
    border-color: hsl(var(--n) / 0)}
.hover\:border-neutral\/10:hover {
    border-color: hsl(var(--n) / 0.1)}
.hover\:border-neutral\/100:hover {
    border-color: hsl(var(--n) / 1)}
.hover\:border-neutral\/20:hover {
    border-color: hsl(var(--n) / 0.2)}
.hover\:border-neutral\/25:hover {
    border-color: hsl(var(--n) / 0.25)}
.hover\:border-neutral\/30:hover {
    border-color: hsl(var(--n) / 0.3)}
.hover\:border-neutral\/40:hover {
    border-color: hsl(var(--n) / 0.4)}
.hover\:border-neutral\/5:hover {
    border-color: hsl(var(--n) / 0.05)}
.hover\:border-neutral\/50:hover {
    border-color: hsl(var(--n) / 0.5)}
.hover\:border-neutral\/60:hover {
    border-color: hsl(var(--n) / 0.6)}
.hover\:border-neutral\/70:hover {
    border-color: hsl(var(--n) / 0.7)}
.hover\:border-neutral\/75:hover {
    border-color: hsl(var(--n) / 0.75)}
.hover\:border-neutral\/80:hover {
    border-color: hsl(var(--n) / 0.8)}
.hover\:border-neutral\/90:hover {
    border-color: hsl(var(--n) / 0.9)}
.hover\:border-neutral\/95:hover {
    border-color: hsl(var(--n) / 0.95)}
.hover\:border-primary:hover {
    border-color: hsl(var(--p) / 1)}
.hover\:border-primary-content:hover {
    border-color: hsl(var(--pc) / 1)}
.hover\:border-primary-content\/0:hover {
    border-color: hsl(var(--pc) / 0)}
.hover\:border-primary-content\/10:hover {
    border-color: hsl(var(--pc) / 0.1)}
.hover\:border-primary-content\/100:hover {
    border-color: hsl(var(--pc) / 1)}
.hover\:border-primary-content\/20:hover {
    border-color: hsl(var(--pc) / 0.2)}
.hover\:border-primary-content\/25:hover {
    border-color: hsl(var(--pc) / 0.25)}
.hover\:border-primary-content\/30:hover {
    border-color: hsl(var(--pc) / 0.3)}
.hover\:border-primary-content\/40:hover {
    border-color: hsl(var(--pc) / 0.4)}
.hover\:border-primary-content\/5:hover {
    border-color: hsl(var(--pc) / 0.05)}
.hover\:border-primary-content\/50:hover {
    border-color: hsl(var(--pc) / 0.5)}
.hover\:border-primary-content\/60:hover {
    border-color: hsl(var(--pc) / 0.6)}
.hover\:border-primary-content\/70:hover {
    border-color: hsl(var(--pc) / 0.7)}
.hover\:border-primary-content\/75:hover {
    border-color: hsl(var(--pc) / 0.75)}
.hover\:border-primary-content\/80:hover {
    border-color: hsl(var(--pc) / 0.8)}
.hover\:border-primary-content\/90:hover {
    border-color: hsl(var(--pc) / 0.9)}
.hover\:border-primary-content\/95:hover {
    border-color: hsl(var(--pc) / 0.95)}
.hover\:border-primary-focus:hover {
    border-color: hsl(var(--pf) / 1)}
.hover\:border-primary-focus\/0:hover {
    border-color: hsl(var(--pf) / 0)}
.hover\:border-primary-focus\/10:hover {
    border-color: hsl(var(--pf) / 0.1)}
.hover\:border-primary-focus\/100:hover {
    border-color: hsl(var(--pf) / 1)}
.hover\:border-primary-focus\/20:hover {
    border-color: hsl(var(--pf) / 0.2)}
.hover\:border-primary-focus\/25:hover {
    border-color: hsl(var(--pf) / 0.25)}
.hover\:border-primary-focus\/30:hover {
    border-color: hsl(var(--pf) / 0.3)}
.hover\:border-primary-focus\/40:hover {
    border-color: hsl(var(--pf) / 0.4)}
.hover\:border-primary-focus\/5:hover {
    border-color: hsl(var(--pf) / 0.05)}
.hover\:border-primary-focus\/50:hover {
    border-color: hsl(var(--pf) / 0.5)}
.hover\:border-primary-focus\/60:hover {
    border-color: hsl(var(--pf) / 0.6)}
.hover\:border-primary-focus\/70:hover {
    border-color: hsl(var(--pf) / 0.7)}
.hover\:border-primary-focus\/75:hover {
    border-color: hsl(var(--pf) / 0.75)}
.hover\:border-primary-focus\/80:hover {
    border-color: hsl(var(--pf) / 0.8)}
.hover\:border-primary-focus\/90:hover {
    border-color: hsl(var(--pf) / 0.9)}
.hover\:border-primary-focus\/95:hover {
    border-color: hsl(var(--pf) / 0.95)}
.hover\:border-primary\/0:hover {
    border-color: hsl(var(--p) / 0)}
.hover\:border-primary\/10:hover {
    border-color: hsl(var(--p) / 0.1)}
.hover\:border-primary\/100:hover {
    border-color: hsl(var(--p) / 1)}
.hover\:border-primary\/20:hover {
    border-color: hsl(var(--p) / 0.2)}
.hover\:border-primary\/25:hover {
    border-color: hsl(var(--p) / 0.25)}
.hover\:border-primary\/30:hover {
    border-color: hsl(var(--p) / 0.3)}
.hover\:border-primary\/40:hover {
    border-color: hsl(var(--p) / 0.4)}
.hover\:border-primary\/5:hover {
    border-color: hsl(var(--p) / 0.05)}
.hover\:border-primary\/50:hover {
    border-color: hsl(var(--p) / 0.5)}
.hover\:border-primary\/60:hover {
    border-color: hsl(var(--p) / 0.6)}
.hover\:border-primary\/70:hover {
    border-color: hsl(var(--p) / 0.7)}
.hover\:border-primary\/75:hover {
    border-color: hsl(var(--p) / 0.75)}
.hover\:border-primary\/80:hover {
    border-color: hsl(var(--p) / 0.8)}
.hover\:border-primary\/90:hover {
    border-color: hsl(var(--p) / 0.9)}
.hover\:border-primary\/95:hover {
    border-color: hsl(var(--p) / 0.95)}
.hover\:border-secondary:hover {
    border-color: hsl(var(--s) / 1)}
.hover\:border-secondary-content:hover {
    border-color: hsl(var(--sc) / 1)}
.hover\:border-secondary-content\/0:hover {
    border-color: hsl(var(--sc) / 0)}
.hover\:border-secondary-content\/10:hover {
    border-color: hsl(var(--sc) / 0.1)}
.hover\:border-secondary-content\/100:hover {
    border-color: hsl(var(--sc) / 1)}
.hover\:border-secondary-content\/20:hover {
    border-color: hsl(var(--sc) / 0.2)}
.hover\:border-secondary-content\/25:hover {
    border-color: hsl(var(--sc) / 0.25)}
.hover\:border-secondary-content\/30:hover {
    border-color: hsl(var(--sc) / 0.3)}
.hover\:border-secondary-content\/40:hover {
    border-color: hsl(var(--sc) / 0.4)}
.hover\:border-secondary-content\/5:hover {
    border-color: hsl(var(--sc) / 0.05)}
.hover\:border-secondary-content\/50:hover {
    border-color: hsl(var(--sc) / 0.5)}
.hover\:border-secondary-content\/60:hover {
    border-color: hsl(var(--sc) / 0.6)}
.hover\:border-secondary-content\/70:hover {
    border-color: hsl(var(--sc) / 0.7)}
.hover\:border-secondary-content\/75:hover {
    border-color: hsl(var(--sc) / 0.75)}
.hover\:border-secondary-content\/80:hover {
    border-color: hsl(var(--sc) / 0.8)}
.hover\:border-secondary-content\/90:hover {
    border-color: hsl(var(--sc) / 0.9)}
.hover\:border-secondary-content\/95:hover {
    border-color: hsl(var(--sc) / 0.95)}
.hover\:border-secondary-focus:hover {
    border-color: hsl(var(--sf) / 1)}
.hover\:border-secondary-focus\/0:hover {
    border-color: hsl(var(--sf) / 0)}
.hover\:border-secondary-focus\/10:hover {
    border-color: hsl(var(--sf) / 0.1)}
.hover\:border-secondary-focus\/100:hover {
    border-color: hsl(var(--sf) / 1)}
.hover\:border-secondary-focus\/20:hover {
    border-color: hsl(var(--sf) / 0.2)}
.hover\:border-secondary-focus\/25:hover {
    border-color: hsl(var(--sf) / 0.25)}
.hover\:border-secondary-focus\/30:hover {
    border-color: hsl(var(--sf) / 0.3)}
.hover\:border-secondary-focus\/40:hover {
    border-color: hsl(var(--sf) / 0.4)}
.hover\:border-secondary-focus\/5:hover {
    border-color: hsl(var(--sf) / 0.05)}
.hover\:border-secondary-focus\/50:hover {
    border-color: hsl(var(--sf) / 0.5)}
.hover\:border-secondary-focus\/60:hover {
    border-color: hsl(var(--sf) / 0.6)}
.hover\:border-secondary-focus\/70:hover {
    border-color: hsl(var(--sf) / 0.7)}
.hover\:border-secondary-focus\/75:hover {
    border-color: hsl(var(--sf) / 0.75)}
.hover\:border-secondary-focus\/80:hover {
    border-color: hsl(var(--sf) / 0.8)}
.hover\:border-secondary-focus\/90:hover {
    border-color: hsl(var(--sf) / 0.9)}
.hover\:border-secondary-focus\/95:hover {
    border-color: hsl(var(--sf) / 0.95)}
.hover\:border-secondary\/0:hover {
    border-color: hsl(var(--s) / 0)}
.hover\:border-secondary\/10:hover {
    border-color: hsl(var(--s) / 0.1)}
.hover\:border-secondary\/100:hover {
    border-color: hsl(var(--s) / 1)}
.hover\:border-secondary\/20:hover {
    border-color: hsl(var(--s) / 0.2)}
.hover\:border-secondary\/25:hover {
    border-color: hsl(var(--s) / 0.25)}
.hover\:border-secondary\/30:hover {
    border-color: hsl(var(--s) / 0.3)}
.hover\:border-secondary\/40:hover {
    border-color: hsl(var(--s) / 0.4)}
.hover\:border-secondary\/5:hover {
    border-color: hsl(var(--s) / 0.05)}
.hover\:border-secondary\/50:hover {
    border-color: hsl(var(--s) / 0.5)}
.hover\:border-secondary\/60:hover {
    border-color: hsl(var(--s) / 0.6)}
.hover\:border-secondary\/70:hover {
    border-color: hsl(var(--s) / 0.7)}
.hover\:border-secondary\/75:hover {
    border-color: hsl(var(--s) / 0.75)}
.hover\:border-secondary\/80:hover {
    border-color: hsl(var(--s) / 0.8)}
.hover\:border-secondary\/90:hover {
    border-color: hsl(var(--s) / 0.9)}
.hover\:border-secondary\/95:hover {
    border-color: hsl(var(--s) / 0.95)}
.hover\:border-success:hover {
    border-color: hsl(var(--su) / 1)}
.hover\:border-success-content:hover {
    border-color: hsl(var(--suc) / 1)}
.hover\:border-success-content\/0:hover {
    border-color: hsl(var(--suc) / 0)}
.hover\:border-success-content\/10:hover {
    border-color: hsl(var(--suc) / 0.1)}
.hover\:border-success-content\/100:hover {
    border-color: hsl(var(--suc) / 1)}
.hover\:border-success-content\/20:hover {
    border-color: hsl(var(--suc) / 0.2)}
.hover\:border-success-content\/25:hover {
    border-color: hsl(var(--suc) / 0.25)}
.hover\:border-success-content\/30:hover {
    border-color: hsl(var(--suc) / 0.3)}
.hover\:border-success-content\/40:hover {
    border-color: hsl(var(--suc) / 0.4)}
.hover\:border-success-content\/5:hover {
    border-color: hsl(var(--suc) / 0.05)}
.hover\:border-success-content\/50:hover {
    border-color: hsl(var(--suc) / 0.5)}
.hover\:border-success-content\/60:hover {
    border-color: hsl(var(--suc) / 0.6)}
.hover\:border-success-content\/70:hover {
    border-color: hsl(var(--suc) / 0.7)}
.hover\:border-success-content\/75:hover {
    border-color: hsl(var(--suc) / 0.75)}
.hover\:border-success-content\/80:hover {
    border-color: hsl(var(--suc) / 0.8)}
.hover\:border-success-content\/90:hover {
    border-color: hsl(var(--suc) / 0.9)}
.hover\:border-success-content\/95:hover {
    border-color: hsl(var(--suc) / 0.95)}
.hover\:border-success\/0:hover {
    border-color: hsl(var(--su) / 0)}
.hover\:border-success\/10:hover {
    border-color: hsl(var(--su) / 0.1)}
.hover\:border-success\/100:hover {
    border-color: hsl(var(--su) / 1)}
.hover\:border-success\/20:hover {
    border-color: hsl(var(--su) / 0.2)}
.hover\:border-success\/25:hover {
    border-color: hsl(var(--su) / 0.25)}
.hover\:border-success\/30:hover {
    border-color: hsl(var(--su) / 0.3)}
.hover\:border-success\/40:hover {
    border-color: hsl(var(--su) / 0.4)}
.hover\:border-success\/5:hover {
    border-color: hsl(var(--su) / 0.05)}
.hover\:border-success\/50:hover {
    border-color: hsl(var(--su) / 0.5)}
.hover\:border-success\/60:hover {
    border-color: hsl(var(--su) / 0.6)}
.hover\:border-success\/70:hover {
    border-color: hsl(var(--su) / 0.7)}
.hover\:border-success\/75:hover {
    border-color: hsl(var(--su) / 0.75)}
.hover\:border-success\/80:hover {
    border-color: hsl(var(--su) / 0.8)}
.hover\:border-success\/90:hover {
    border-color: hsl(var(--su) / 0.9)}
.hover\:border-success\/95:hover {
    border-color: hsl(var(--su) / 0.95)}
.hover\:border-warning:hover {
    border-color: hsl(var(--wa) / 1)}
.hover\:border-warning-content:hover {
    border-color: hsl(var(--wac) / 1)}
.hover\:border-warning-content\/0:hover {
    border-color: hsl(var(--wac) / 0)}
.hover\:border-warning-content\/10:hover {
    border-color: hsl(var(--wac) / 0.1)}
.hover\:border-warning-content\/100:hover {
    border-color: hsl(var(--wac) / 1)}
.hover\:border-warning-content\/20:hover {
    border-color: hsl(var(--wac) / 0.2)}
.hover\:border-warning-content\/25:hover {
    border-color: hsl(var(--wac) / 0.25)}
.hover\:border-warning-content\/30:hover {
    border-color: hsl(var(--wac) / 0.3)}
.hover\:border-warning-content\/40:hover {
    border-color: hsl(var(--wac) / 0.4)}
.hover\:border-warning-content\/5:hover {
    border-color: hsl(var(--wac) / 0.05)}
.hover\:border-warning-content\/50:hover {
    border-color: hsl(var(--wac) / 0.5)}
.hover\:border-warning-content\/60:hover {
    border-color: hsl(var(--wac) / 0.6)}
.hover\:border-warning-content\/70:hover {
    border-color: hsl(var(--wac) / 0.7)}
.hover\:border-warning-content\/75:hover {
    border-color: hsl(var(--wac) / 0.75)}
.hover\:border-warning-content\/80:hover {
    border-color: hsl(var(--wac) / 0.8)}
.hover\:border-warning-content\/90:hover {
    border-color: hsl(var(--wac) / 0.9)}
.hover\:border-warning-content\/95:hover {
    border-color: hsl(var(--wac) / 0.95)}
.hover\:border-warning\/0:hover {
    border-color: hsl(var(--wa) / 0)}
.hover\:border-warning\/10:hover {
    border-color: hsl(var(--wa) / 0.1)}
.hover\:border-warning\/100:hover {
    border-color: hsl(var(--wa) / 1)}
.hover\:border-warning\/20:hover {
    border-color: hsl(var(--wa) / 0.2)}
.hover\:border-warning\/25:hover {
    border-color: hsl(var(--wa) / 0.25)}
.hover\:border-warning\/30:hover {
    border-color: hsl(var(--wa) / 0.3)}
.hover\:border-warning\/40:hover {
    border-color: hsl(var(--wa) / 0.4)}
.hover\:border-warning\/5:hover {
    border-color: hsl(var(--wa) / 0.05)}
.hover\:border-warning\/50:hover {
    border-color: hsl(var(--wa) / 0.5)}
.hover\:border-warning\/60:hover {
    border-color: hsl(var(--wa) / 0.6)}
.hover\:border-warning\/70:hover {
    border-color: hsl(var(--wa) / 0.7)}
.hover\:border-warning\/75:hover {
    border-color: hsl(var(--wa) / 0.75)}
.hover\:border-warning\/80:hover {
    border-color: hsl(var(--wa) / 0.8)}
.hover\:border-warning\/90:hover {
    border-color: hsl(var(--wa) / 0.9)}
.hover\:border-warning\/95:hover {
    border-color: hsl(var(--wa) / 0.95)}
.hover\:border-x-base-100:hover {
    border-left-color: hsl(var(--b1) / 1);
    border-right-color: hsl(var(--b1) / 1)}
.hover\:border-x-base-100\/0:hover {
    border-left-color: hsl(var(--b1) / 0);
    border-right-color: hsl(var(--b1) / 0)}
.hover\:border-x-base-100\/10:hover {
    border-left-color: hsl(var(--b1) / 0.1);
    border-right-color: hsl(var(--b1) / 0.1)}
.hover\:border-x-base-100\/100:hover {
    border-left-color: hsl(var(--b1) / 1);
    border-right-color: hsl(var(--b1) / 1)}
.hover\:border-x-base-100\/20:hover {
    border-left-color: hsl(var(--b1) / 0.2);
    border-right-color: hsl(var(--b1) / 0.2)}
.hover\:border-x-base-100\/25:hover {
    border-left-color: hsl(var(--b1) / 0.25);
    border-right-color: hsl(var(--b1) / 0.25)}
.hover\:border-x-base-100\/30:hover {
    border-left-color: hsl(var(--b1) / 0.3);
    border-right-color: hsl(var(--b1) / 0.3)}
.hover\:border-x-base-100\/40:hover {
    border-left-color: hsl(var(--b1) / 0.4);
    border-right-color: hsl(var(--b1) / 0.4)}
.hover\:border-x-base-100\/5:hover {
    border-left-color: hsl(var(--b1) / 0.05);
    border-right-color: hsl(var(--b1) / 0.05)}
.hover\:border-x-base-100\/50:hover {
    border-left-color: hsl(var(--b1) / 0.5);
    border-right-color: hsl(var(--b1) / 0.5)}
.hover\:border-x-base-100\/60:hover {
    border-left-color: hsl(var(--b1) / 0.6);
    border-right-color: hsl(var(--b1) / 0.6)}
.hover\:border-x-base-100\/70:hover {
    border-left-color: hsl(var(--b1) / 0.7);
    border-right-color: hsl(var(--b1) / 0.7)}
.hover\:border-x-base-100\/75:hover {
    border-left-color: hsl(var(--b1) / 0.75);
    border-right-color: hsl(var(--b1) / 0.75)}
.hover\:border-x-base-100\/80:hover {
    border-left-color: hsl(var(--b1) / 0.8);
    border-right-color: hsl(var(--b1) / 0.8)}
.hover\:border-x-base-100\/90:hover {
    border-left-color: hsl(var(--b1) / 0.9);
    border-right-color: hsl(var(--b1) / 0.9)}
.hover\:border-x-base-100\/95:hover {
    border-left-color: hsl(var(--b1) / 0.95);
    border-right-color: hsl(var(--b1) / 0.95)}
.hover\:border-x-base-200:hover {
    border-left-color: hsl(var(--b2) / 1);
    border-right-color: hsl(var(--b2) / 1)}
.hover\:border-x-base-200\/0:hover {
    border-left-color: hsl(var(--b2) / 0);
    border-right-color: hsl(var(--b2) / 0)}
.hover\:border-x-base-200\/10:hover {
    border-left-color: hsl(var(--b2) / 0.1);
    border-right-color: hsl(var(--b2) / 0.1)}
.hover\:border-x-base-200\/100:hover {
    border-left-color: hsl(var(--b2) / 1);
    border-right-color: hsl(var(--b2) / 1)}
.hover\:border-x-base-200\/20:hover {
    border-left-color: hsl(var(--b2) / 0.2);
    border-right-color: hsl(var(--b2) / 0.2)}
.hover\:border-x-base-200\/25:hover {
    border-left-color: hsl(var(--b2) / 0.25);
    border-right-color: hsl(var(--b2) / 0.25)}
.hover\:border-x-base-200\/30:hover {
    border-left-color: hsl(var(--b2) / 0.3);
    border-right-color: hsl(var(--b2) / 0.3)}
.hover\:border-x-base-200\/40:hover {
    border-left-color: hsl(var(--b2) / 0.4);
    border-right-color: hsl(var(--b2) / 0.4)}
.hover\:border-x-base-200\/5:hover {
    border-left-color: hsl(var(--b2) / 0.05);
    border-right-color: hsl(var(--b2) / 0.05)}
.hover\:border-x-base-200\/50:hover {
    border-left-color: hsl(var(--b2) / 0.5);
    border-right-color: hsl(var(--b2) / 0.5)}
.hover\:border-x-base-200\/60:hover {
    border-left-color: hsl(var(--b2) / 0.6);
    border-right-color: hsl(var(--b2) / 0.6)}
.hover\:border-x-base-200\/70:hover {
    border-left-color: hsl(var(--b2) / 0.7);
    border-right-color: hsl(var(--b2) / 0.7)}
.hover\:border-x-base-200\/75:hover {
    border-left-color: hsl(var(--b2) / 0.75);
    border-right-color: hsl(var(--b2) / 0.75)}
.hover\:border-x-base-200\/80:hover {
    border-left-color: hsl(var(--b2) / 0.8);
    border-right-color: hsl(var(--b2) / 0.8)}
.hover\:border-x-base-200\/90:hover {
    border-left-color: hsl(var(--b2) / 0.9);
    border-right-color: hsl(var(--b2) / 0.9)}
.hover\:border-x-base-200\/95:hover {
    border-left-color: hsl(var(--b2) / 0.95);
    border-right-color: hsl(var(--b2) / 0.95)}
.hover\:border-x-base-300:hover {
    border-left-color: hsl(var(--b3) / 1);
    border-right-color: hsl(var(--b3) / 1)}
.hover\:border-x-base-300\/0:hover {
    border-left-color: hsl(var(--b3) / 0);
    border-right-color: hsl(var(--b3) / 0)}
.hover\:border-x-base-300\/10:hover {
    border-left-color: hsl(var(--b3) / 0.1);
    border-right-color: hsl(var(--b3) / 0.1)}
.hover\:border-x-base-300\/100:hover {
    border-left-color: hsl(var(--b3) / 1);
    border-right-color: hsl(var(--b3) / 1)}
.hover\:border-x-base-300\/20:hover {
    border-left-color: hsl(var(--b3) / 0.2);
    border-right-color: hsl(var(--b3) / 0.2)}
.hover\:border-x-base-300\/25:hover {
    border-left-color: hsl(var(--b3) / 0.25);
    border-right-color: hsl(var(--b3) / 0.25)}
.hover\:border-x-base-300\/30:hover {
    border-left-color: hsl(var(--b3) / 0.3);
    border-right-color: hsl(var(--b3) / 0.3)}
.hover\:border-x-base-300\/40:hover {
    border-left-color: hsl(var(--b3) / 0.4);
    border-right-color: hsl(var(--b3) / 0.4)}
.hover\:border-x-base-300\/5:hover {
    border-left-color: hsl(var(--b3) / 0.05);
    border-right-color: hsl(var(--b3) / 0.05)}
.hover\:border-x-base-300\/50:hover {
    border-left-color: hsl(var(--b3) / 0.5);
    border-right-color: hsl(var(--b3) / 0.5)}
.hover\:border-x-base-300\/60:hover {
    border-left-color: hsl(var(--b3) / 0.6);
    border-right-color: hsl(var(--b3) / 0.6)}
.hover\:border-x-base-300\/70:hover {
    border-left-color: hsl(var(--b3) / 0.7);
    border-right-color: hsl(var(--b3) / 0.7)}
.hover\:border-x-base-300\/75:hover {
    border-left-color: hsl(var(--b3) / 0.75);
    border-right-color: hsl(var(--b3) / 0.75)}
.hover\:border-x-base-300\/80:hover {
    border-left-color: hsl(var(--b3) / 0.8);
    border-right-color: hsl(var(--b3) / 0.8)}
.hover\:border-x-base-300\/90:hover {
    border-left-color: hsl(var(--b3) / 0.9);
    border-right-color: hsl(var(--b3) / 0.9)}
.hover\:border-x-base-300\/95:hover {
    border-left-color: hsl(var(--b3) / 0.95);
    border-right-color: hsl(var(--b3) / 0.95)}
.hover\:border-x-base-content:hover {
    border-left-color: hsl(var(--bc) / 1);
    border-right-color: hsl(var(--bc) / 1)}
.hover\:border-x-base-content\/0:hover {
    border-left-color: hsl(var(--bc) / 0);
    border-right-color: hsl(var(--bc) / 0)}
.hover\:border-x-base-content\/10:hover {
    border-left-color: hsl(var(--bc) / 0.1);
    border-right-color: hsl(var(--bc) / 0.1)}
.hover\:border-x-base-content\/100:hover {
    border-left-color: hsl(var(--bc) / 1);
    border-right-color: hsl(var(--bc) / 1)}
.hover\:border-x-base-content\/20:hover {
    border-left-color: hsl(var(--bc) / 0.2);
    border-right-color: hsl(var(--bc) / 0.2)}
.hover\:border-x-base-content\/25:hover {
    border-left-color: hsl(var(--bc) / 0.25);
    border-right-color: hsl(var(--bc) / 0.25)}
.hover\:border-x-base-content\/30:hover {
    border-left-color: hsl(var(--bc) / 0.3);
    border-right-color: hsl(var(--bc) / 0.3)}
.hover\:border-x-base-content\/40:hover {
    border-left-color: hsl(var(--bc) / 0.4);
    border-right-color: hsl(var(--bc) / 0.4)}
.hover\:border-x-base-content\/5:hover {
    border-left-color: hsl(var(--bc) / 0.05);
    border-right-color: hsl(var(--bc) / 0.05)}
.hover\:border-x-base-content\/50:hover {
    border-left-color: hsl(var(--bc) / 0.5);
    border-right-color: hsl(var(--bc) / 0.5)}
.hover\:border-x-base-content\/60:hover {
    border-left-color: hsl(var(--bc) / 0.6);
    border-right-color: hsl(var(--bc) / 0.6)}
.hover\:border-x-base-content\/70:hover {
    border-left-color: hsl(var(--bc) / 0.7);
    border-right-color: hsl(var(--bc) / 0.7)}
.hover\:border-x-base-content\/75:hover {
    border-left-color: hsl(var(--bc) / 0.75);
    border-right-color: hsl(var(--bc) / 0.75)}
.hover\:border-x-base-content\/80:hover {
    border-left-color: hsl(var(--bc) / 0.8);
    border-right-color: hsl(var(--bc) / 0.8)}
.hover\:border-x-base-content\/90:hover {
    border-left-color: hsl(var(--bc) / 0.9);
    border-right-color: hsl(var(--bc) / 0.9)}
.hover\:border-x-base-content\/95:hover {
    border-left-color: hsl(var(--bc) / 0.95);
    border-right-color: hsl(var(--bc) / 0.95)}
.hover\:border-x-error:hover {
    border-left-color: hsl(var(--er) / 1);
    border-right-color: hsl(var(--er) / 1)}
.hover\:border-x-error-content:hover {
    border-left-color: hsl(var(--erc) / 1);
    border-right-color: hsl(var(--erc) / 1)}
.hover\:border-x-error-content\/0:hover {
    border-left-color: hsl(var(--erc) / 0);
    border-right-color: hsl(var(--erc) / 0)}
.hover\:border-x-error-content\/10:hover {
    border-left-color: hsl(var(--erc) / 0.1);
    border-right-color: hsl(var(--erc) / 0.1)}
.hover\:border-x-error-content\/100:hover {
    border-left-color: hsl(var(--erc) / 1);
    border-right-color: hsl(var(--erc) / 1)}
.hover\:border-x-error-content\/20:hover {
    border-left-color: hsl(var(--erc) / 0.2);
    border-right-color: hsl(var(--erc) / 0.2)}
.hover\:border-x-error-content\/25:hover {
    border-left-color: hsl(var(--erc) / 0.25);
    border-right-color: hsl(var(--erc) / 0.25)}
.hover\:border-x-error-content\/30:hover {
    border-left-color: hsl(var(--erc) / 0.3);
    border-right-color: hsl(var(--erc) / 0.3)}
.hover\:border-x-error-content\/40:hover {
    border-left-color: hsl(var(--erc) / 0.4);
    border-right-color: hsl(var(--erc) / 0.4)}
.hover\:border-x-error-content\/5:hover {
    border-left-color: hsl(var(--erc) / 0.05);
    border-right-color: hsl(var(--erc) / 0.05)}
.hover\:border-x-error-content\/50:hover {
    border-left-color: hsl(var(--erc) / 0.5);
    border-right-color: hsl(var(--erc) / 0.5)}
.hover\:border-x-error-content\/60:hover {
    border-left-color: hsl(var(--erc) / 0.6);
    border-right-color: hsl(var(--erc) / 0.6)}
.hover\:border-x-error-content\/70:hover {
    border-left-color: hsl(var(--erc) / 0.7);
    border-right-color: hsl(var(--erc) / 0.7)}
.hover\:border-x-error-content\/75:hover {
    border-left-color: hsl(var(--erc) / 0.75);
    border-right-color: hsl(var(--erc) / 0.75)}
.hover\:border-x-error-content\/80:hover {
    border-left-color: hsl(var(--erc) / 0.8);
    border-right-color: hsl(var(--erc) / 0.8)}
.hover\:border-x-error-content\/90:hover {
    border-left-color: hsl(var(--erc) / 0.9);
    border-right-color: hsl(var(--erc) / 0.9)}
.hover\:border-x-error-content\/95:hover {
    border-left-color: hsl(var(--erc) / 0.95);
    border-right-color: hsl(var(--erc) / 0.95)}
.hover\:border-x-error\/0:hover {
    border-left-color: hsl(var(--er) / 0);
    border-right-color: hsl(var(--er) / 0)}
.hover\:border-x-error\/10:hover {
    border-left-color: hsl(var(--er) / 0.1);
    border-right-color: hsl(var(--er) / 0.1)}
.hover\:border-x-error\/100:hover {
    border-left-color: hsl(var(--er) / 1);
    border-right-color: hsl(var(--er) / 1)}
.hover\:border-x-error\/20:hover {
    border-left-color: hsl(var(--er) / 0.2);
    border-right-color: hsl(var(--er) / 0.2)}
.hover\:border-x-error\/25:hover {
    border-left-color: hsl(var(--er) / 0.25);
    border-right-color: hsl(var(--er) / 0.25)}
.hover\:border-x-error\/30:hover {
    border-left-color: hsl(var(--er) / 0.3);
    border-right-color: hsl(var(--er) / 0.3)}
.hover\:border-x-error\/40:hover {
    border-left-color: hsl(var(--er) / 0.4);
    border-right-color: hsl(var(--er) / 0.4)}
.hover\:border-x-error\/5:hover {
    border-left-color: hsl(var(--er) / 0.05);
    border-right-color: hsl(var(--er) / 0.05)}
.hover\:border-x-error\/50:hover {
    border-left-color: hsl(var(--er) / 0.5);
    border-right-color: hsl(var(--er) / 0.5)}
.hover\:border-x-error\/60:hover {
    border-left-color: hsl(var(--er) / 0.6);
    border-right-color: hsl(var(--er) / 0.6)}
.hover\:border-x-error\/70:hover {
    border-left-color: hsl(var(--er) / 0.7);
    border-right-color: hsl(var(--er) / 0.7)}
.hover\:border-x-error\/75:hover {
    border-left-color: hsl(var(--er) / 0.75);
    border-right-color: hsl(var(--er) / 0.75)}
.hover\:border-x-error\/80:hover {
    border-left-color: hsl(var(--er) / 0.8);
    border-right-color: hsl(var(--er) / 0.8)}
.hover\:border-x-error\/90:hover {
    border-left-color: hsl(var(--er) / 0.9);
    border-right-color: hsl(var(--er) / 0.9)}
.hover\:border-x-error\/95:hover {
    border-left-color: hsl(var(--er) / 0.95);
    border-right-color: hsl(var(--er) / 0.95)}
.hover\:border-x-info:hover {
    border-left-color: hsl(var(--in) / 1);
    border-right-color: hsl(var(--in) / 1)}
.hover\:border-x-info-content:hover {
    border-left-color: hsl(var(--inc) / 1);
    border-right-color: hsl(var(--inc) / 1)}
.hover\:border-x-info-content\/0:hover {
    border-left-color: hsl(var(--inc) / 0);
    border-right-color: hsl(var(--inc) / 0)}
.hover\:border-x-info-content\/10:hover {
    border-left-color: hsl(var(--inc) / 0.1);
    border-right-color: hsl(var(--inc) / 0.1)}
.hover\:border-x-info-content\/100:hover {
    border-left-color: hsl(var(--inc) / 1);
    border-right-color: hsl(var(--inc) / 1)}
.hover\:border-x-info-content\/20:hover {
    border-left-color: hsl(var(--inc) / 0.2);
    border-right-color: hsl(var(--inc) / 0.2)}
.hover\:border-x-info-content\/25:hover {
    border-left-color: hsl(var(--inc) / 0.25);
    border-right-color: hsl(var(--inc) / 0.25)}
.hover\:border-x-info-content\/30:hover {
    border-left-color: hsl(var(--inc) / 0.3);
    border-right-color: hsl(var(--inc) / 0.3)}
.hover\:border-x-info-content\/40:hover {
    border-left-color: hsl(var(--inc) / 0.4);
    border-right-color: hsl(var(--inc) / 0.4)}
.hover\:border-x-info-content\/5:hover {
    border-left-color: hsl(var(--inc) / 0.05);
    border-right-color: hsl(var(--inc) / 0.05)}
.hover\:border-x-info-content\/50:hover {
    border-left-color: hsl(var(--inc) / 0.5);
    border-right-color: hsl(var(--inc) / 0.5)}
.hover\:border-x-info-content\/60:hover {
    border-left-color: hsl(var(--inc) / 0.6);
    border-right-color: hsl(var(--inc) / 0.6)}
.hover\:border-x-info-content\/70:hover {
    border-left-color: hsl(var(--inc) / 0.7);
    border-right-color: hsl(var(--inc) / 0.7)}
.hover\:border-x-info-content\/75:hover {
    border-left-color: hsl(var(--inc) / 0.75);
    border-right-color: hsl(var(--inc) / 0.75)}
.hover\:border-x-info-content\/80:hover {
    border-left-color: hsl(var(--inc) / 0.8);
    border-right-color: hsl(var(--inc) / 0.8)}
.hover\:border-x-info-content\/90:hover {
    border-left-color: hsl(var(--inc) / 0.9);
    border-right-color: hsl(var(--inc) / 0.9)}
.hover\:border-x-info-content\/95:hover {
    border-left-color: hsl(var(--inc) / 0.95);
    border-right-color: hsl(var(--inc) / 0.95)}
.hover\:border-x-info\/0:hover {
    border-left-color: hsl(var(--in) / 0);
    border-right-color: hsl(var(--in) / 0)}
.hover\:border-x-info\/10:hover {
    border-left-color: hsl(var(--in) / 0.1);
    border-right-color: hsl(var(--in) / 0.1)}
.hover\:border-x-info\/100:hover {
    border-left-color: hsl(var(--in) / 1);
    border-right-color: hsl(var(--in) / 1)}
.hover\:border-x-info\/20:hover {
    border-left-color: hsl(var(--in) / 0.2);
    border-right-color: hsl(var(--in) / 0.2)}
.hover\:border-x-info\/25:hover {
    border-left-color: hsl(var(--in) / 0.25);
    border-right-color: hsl(var(--in) / 0.25)}
.hover\:border-x-info\/30:hover {
    border-left-color: hsl(var(--in) / 0.3);
    border-right-color: hsl(var(--in) / 0.3)}
.hover\:border-x-info\/40:hover {
    border-left-color: hsl(var(--in) / 0.4);
    border-right-color: hsl(var(--in) / 0.4)}
.hover\:border-x-info\/5:hover {
    border-left-color: hsl(var(--in) / 0.05);
    border-right-color: hsl(var(--in) / 0.05)}
.hover\:border-x-info\/50:hover {
    border-left-color: hsl(var(--in) / 0.5);
    border-right-color: hsl(var(--in) / 0.5)}
.hover\:border-x-info\/60:hover {
    border-left-color: hsl(var(--in) / 0.6);
    border-right-color: hsl(var(--in) / 0.6)}
.hover\:border-x-info\/70:hover {
    border-left-color: hsl(var(--in) / 0.7);
    border-right-color: hsl(var(--in) / 0.7)}
.hover\:border-x-info\/75:hover {
    border-left-color: hsl(var(--in) / 0.75);
    border-right-color: hsl(var(--in) / 0.75)}
.hover\:border-x-info\/80:hover {
    border-left-color: hsl(var(--in) / 0.8);
    border-right-color: hsl(var(--in) / 0.8)}
.hover\:border-x-info\/90:hover {
    border-left-color: hsl(var(--in) / 0.9);
    border-right-color: hsl(var(--in) / 0.9)}
.hover\:border-x-info\/95:hover {
    border-left-color: hsl(var(--in) / 0.95);
    border-right-color: hsl(var(--in) / 0.95)}
.hover\:border-x-success:hover {
    border-left-color: hsl(var(--su) / 1);
    border-right-color: hsl(var(--su) / 1)}
.hover\:border-x-success-content:hover {
    border-left-color: hsl(var(--suc) / 1);
    border-right-color: hsl(var(--suc) / 1)}
.hover\:border-x-success-content\/0:hover {
    border-left-color: hsl(var(--suc) / 0);
    border-right-color: hsl(var(--suc) / 0)}
.hover\:border-x-success-content\/10:hover {
    border-left-color: hsl(var(--suc) / 0.1);
    border-right-color: hsl(var(--suc) / 0.1)}
.hover\:border-x-success-content\/100:hover {
    border-left-color: hsl(var(--suc) / 1);
    border-right-color: hsl(var(--suc) / 1)}
.hover\:border-x-success-content\/20:hover {
    border-left-color: hsl(var(--suc) / 0.2);
    border-right-color: hsl(var(--suc) / 0.2)}
.hover\:border-x-success-content\/25:hover {
    border-left-color: hsl(var(--suc) / 0.25);
    border-right-color: hsl(var(--suc) / 0.25)}
.hover\:border-x-success-content\/30:hover {
    border-left-color: hsl(var(--suc) / 0.3);
    border-right-color: hsl(var(--suc) / 0.3)}
.hover\:border-x-success-content\/40:hover {
    border-left-color: hsl(var(--suc) / 0.4);
    border-right-color: hsl(var(--suc) / 0.4)}
.hover\:border-x-success-content\/5:hover {
    border-left-color: hsl(var(--suc) / 0.05);
    border-right-color: hsl(var(--suc) / 0.05)}
.hover\:border-x-success-content\/50:hover {
    border-left-color: hsl(var(--suc) / 0.5);
    border-right-color: hsl(var(--suc) / 0.5)}
.hover\:border-x-success-content\/60:hover {
    border-left-color: hsl(var(--suc) / 0.6);
    border-right-color: hsl(var(--suc) / 0.6)}
.hover\:border-x-success-content\/70:hover {
    border-left-color: hsl(var(--suc) / 0.7);
    border-right-color: hsl(var(--suc) / 0.7)}
.hover\:border-x-success-content\/75:hover {
    border-left-color: hsl(var(--suc) / 0.75);
    border-right-color: hsl(var(--suc) / 0.75)}
.hover\:border-x-success-content\/80:hover {
    border-left-color: hsl(var(--suc) / 0.8);
    border-right-color: hsl(var(--suc) / 0.8)}
.hover\:border-x-success-content\/90:hover {
    border-left-color: hsl(var(--suc) / 0.9);
    border-right-color: hsl(var(--suc) / 0.9)}
.hover\:border-x-success-content\/95:hover {
    border-left-color: hsl(var(--suc) / 0.95);
    border-right-color: hsl(var(--suc) / 0.95)}
.hover\:border-x-success\/0:hover {
    border-left-color: hsl(var(--su) / 0);
    border-right-color: hsl(var(--su) / 0)}
.hover\:border-x-success\/10:hover {
    border-left-color: hsl(var(--su) / 0.1);
    border-right-color: hsl(var(--su) / 0.1)}
.hover\:border-x-success\/100:hover {
    border-left-color: hsl(var(--su) / 1);
    border-right-color: hsl(var(--su) / 1)}
.hover\:border-x-success\/20:hover {
    border-left-color: hsl(var(--su) / 0.2);
    border-right-color: hsl(var(--su) / 0.2)}
.hover\:border-x-success\/25:hover {
    border-left-color: hsl(var(--su) / 0.25);
    border-right-color: hsl(var(--su) / 0.25)}
.hover\:border-x-success\/30:hover {
    border-left-color: hsl(var(--su) / 0.3);
    border-right-color: hsl(var(--su) / 0.3)}
.hover\:border-x-success\/40:hover {
    border-left-color: hsl(var(--su) / 0.4);
    border-right-color: hsl(var(--su) / 0.4)}
.hover\:border-x-success\/5:hover {
    border-left-color: hsl(var(--su) / 0.05);
    border-right-color: hsl(var(--su) / 0.05)}
.hover\:border-x-success\/50:hover {
    border-left-color: hsl(var(--su) / 0.5);
    border-right-color: hsl(var(--su) / 0.5)}
.hover\:border-x-success\/60:hover {
    border-left-color: hsl(var(--su) / 0.6);
    border-right-color: hsl(var(--su) / 0.6)}
.hover\:border-x-success\/70:hover {
    border-left-color: hsl(var(--su) / 0.7);
    border-right-color: hsl(var(--su) / 0.7)}
.hover\:border-x-success\/75:hover {
    border-left-color: hsl(var(--su) / 0.75);
    border-right-color: hsl(var(--su) / 0.75)}
.hover\:border-x-success\/80:hover {
    border-left-color: hsl(var(--su) / 0.8);
    border-right-color: hsl(var(--su) / 0.8)}
.hover\:border-x-success\/90:hover {
    border-left-color: hsl(var(--su) / 0.9);
    border-right-color: hsl(var(--su) / 0.9)}
.hover\:border-x-success\/95:hover {
    border-left-color: hsl(var(--su) / 0.95);
    border-right-color: hsl(var(--su) / 0.95)}
.hover\:border-x-warning:hover {
    border-left-color: hsl(var(--wa) / 1);
    border-right-color: hsl(var(--wa) / 1)}
.hover\:border-x-warning-content:hover {
    border-left-color: hsl(var(--wac) / 1);
    border-right-color: hsl(var(--wac) / 1)}
.hover\:border-x-warning-content\/0:hover {
    border-left-color: hsl(var(--wac) / 0);
    border-right-color: hsl(var(--wac) / 0)}
.hover\:border-x-warning-content\/10:hover {
    border-left-color: hsl(var(--wac) / 0.1);
    border-right-color: hsl(var(--wac) / 0.1)}
.hover\:border-x-warning-content\/100:hover {
    border-left-color: hsl(var(--wac) / 1);
    border-right-color: hsl(var(--wac) / 1)}
.hover\:border-x-warning-content\/20:hover {
    border-left-color: hsl(var(--wac) / 0.2);
    border-right-color: hsl(var(--wac) / 0.2)}
.hover\:border-x-warning-content\/25:hover {
    border-left-color: hsl(var(--wac) / 0.25);
    border-right-color: hsl(var(--wac) / 0.25)}
.hover\:border-x-warning-content\/30:hover {
    border-left-color: hsl(var(--wac) / 0.3);
    border-right-color: hsl(var(--wac) / 0.3)}
.hover\:border-x-warning-content\/40:hover {
    border-left-color: hsl(var(--wac) / 0.4);
    border-right-color: hsl(var(--wac) / 0.4)}
.hover\:border-x-warning-content\/5:hover {
    border-left-color: hsl(var(--wac) / 0.05);
    border-right-color: hsl(var(--wac) / 0.05)}
.hover\:border-x-warning-content\/50:hover {
    border-left-color: hsl(var(--wac) / 0.5);
    border-right-color: hsl(var(--wac) / 0.5)}
.hover\:border-x-warning-content\/60:hover {
    border-left-color: hsl(var(--wac) / 0.6);
    border-right-color: hsl(var(--wac) / 0.6)}
.hover\:border-x-warning-content\/70:hover {
    border-left-color: hsl(var(--wac) / 0.7);
    border-right-color: hsl(var(--wac) / 0.7)}
.hover\:border-x-warning-content\/75:hover {
    border-left-color: hsl(var(--wac) / 0.75);
    border-right-color: hsl(var(--wac) / 0.75)}
.hover\:border-x-warning-content\/80:hover {
    border-left-color: hsl(var(--wac) / 0.8);
    border-right-color: hsl(var(--wac) / 0.8)}
.hover\:border-x-warning-content\/90:hover {
    border-left-color: hsl(var(--wac) / 0.9);
    border-right-color: hsl(var(--wac) / 0.9)}
.hover\:border-x-warning-content\/95:hover {
    border-left-color: hsl(var(--wac) / 0.95);
    border-right-color: hsl(var(--wac) / 0.95)}
.hover\:border-x-warning\/0:hover {
    border-left-color: hsl(var(--wa) / 0);
    border-right-color: hsl(var(--wa) / 0)}
.hover\:border-x-warning\/10:hover {
    border-left-color: hsl(var(--wa) / 0.1);
    border-right-color: hsl(var(--wa) / 0.1)}
.hover\:border-x-warning\/100:hover {
    border-left-color: hsl(var(--wa) / 1);
    border-right-color: hsl(var(--wa) / 1)}
.hover\:border-x-warning\/20:hover {
    border-left-color: hsl(var(--wa) / 0.2);
    border-right-color: hsl(var(--wa) / 0.2)}
.hover\:border-x-warning\/25:hover {
    border-left-color: hsl(var(--wa) / 0.25);
    border-right-color: hsl(var(--wa) / 0.25)}
.hover\:border-x-warning\/30:hover {
    border-left-color: hsl(var(--wa) / 0.3);
    border-right-color: hsl(var(--wa) / 0.3)}
.hover\:border-x-warning\/40:hover {
    border-left-color: hsl(var(--wa) / 0.4);
    border-right-color: hsl(var(--wa) / 0.4)}
.hover\:border-x-warning\/5:hover {
    border-left-color: hsl(var(--wa) / 0.05);
    border-right-color: hsl(var(--wa) / 0.05)}
.hover\:border-x-warning\/50:hover {
    border-left-color: hsl(var(--wa) / 0.5);
    border-right-color: hsl(var(--wa) / 0.5)}
.hover\:border-x-warning\/60:hover {
    border-left-color: hsl(var(--wa) / 0.6);
    border-right-color: hsl(var(--wa) / 0.6)}
.hover\:border-x-warning\/70:hover {
    border-left-color: hsl(var(--wa) / 0.7);
    border-right-color: hsl(var(--wa) / 0.7)}
.hover\:border-x-warning\/75:hover {
    border-left-color: hsl(var(--wa) / 0.75);
    border-right-color: hsl(var(--wa) / 0.75)}
.hover\:border-x-warning\/80:hover {
    border-left-color: hsl(var(--wa) / 0.8);
    border-right-color: hsl(var(--wa) / 0.8)}
.hover\:border-x-warning\/90:hover {
    border-left-color: hsl(var(--wa) / 0.9);
    border-right-color: hsl(var(--wa) / 0.9)}
.hover\:border-x-warning\/95:hover {
    border-left-color: hsl(var(--wa) / 0.95);
    border-right-color: hsl(var(--wa) / 0.95)}
.hover\:border-y-base-100:hover {
    border-top-color: hsl(var(--b1) / 1);
    border-bottom-color: hsl(var(--b1) / 1)}
.hover\:border-y-base-100\/0:hover {
    border-top-color: hsl(var(--b1) / 0);
    border-bottom-color: hsl(var(--b1) / 0)}
.hover\:border-y-base-100\/10:hover {
    border-top-color: hsl(var(--b1) / 0.1);
    border-bottom-color: hsl(var(--b1) / 0.1)}
.hover\:border-y-base-100\/100:hover {
    border-top-color: hsl(var(--b1) / 1);
    border-bottom-color: hsl(var(--b1) / 1)}
.hover\:border-y-base-100\/20:hover {
    border-top-color: hsl(var(--b1) / 0.2);
    border-bottom-color: hsl(var(--b1) / 0.2)}
.hover\:border-y-base-100\/25:hover {
    border-top-color: hsl(var(--b1) / 0.25);
    border-bottom-color: hsl(var(--b1) / 0.25)}
.hover\:border-y-base-100\/30:hover {
    border-top-color: hsl(var(--b1) / 0.3);
    border-bottom-color: hsl(var(--b1) / 0.3)}
.hover\:border-y-base-100\/40:hover {
    border-top-color: hsl(var(--b1) / 0.4);
    border-bottom-color: hsl(var(--b1) / 0.4)}
.hover\:border-y-base-100\/5:hover {
    border-top-color: hsl(var(--b1) / 0.05);
    border-bottom-color: hsl(var(--b1) / 0.05)}
.hover\:border-y-base-100\/50:hover {
    border-top-color: hsl(var(--b1) / 0.5);
    border-bottom-color: hsl(var(--b1) / 0.5)}
.hover\:border-y-base-100\/60:hover {
    border-top-color: hsl(var(--b1) / 0.6);
    border-bottom-color: hsl(var(--b1) / 0.6)}
.hover\:border-y-base-100\/70:hover {
    border-top-color: hsl(var(--b1) / 0.7);
    border-bottom-color: hsl(var(--b1) / 0.7)}
.hover\:border-y-base-100\/75:hover {
    border-top-color: hsl(var(--b1) / 0.75);
    border-bottom-color: hsl(var(--b1) / 0.75)}
.hover\:border-y-base-100\/80:hover {
    border-top-color: hsl(var(--b1) / 0.8);
    border-bottom-color: hsl(var(--b1) / 0.8)}
.hover\:border-y-base-100\/90:hover {
    border-top-color: hsl(var(--b1) / 0.9);
    border-bottom-color: hsl(var(--b1) / 0.9)}
.hover\:border-y-base-100\/95:hover {
    border-top-color: hsl(var(--b1) / 0.95);
    border-bottom-color: hsl(var(--b1) / 0.95)}
.hover\:border-y-base-200:hover {
    border-top-color: hsl(var(--b2) / 1);
    border-bottom-color: hsl(var(--b2) / 1)}
.hover\:border-y-base-200\/0:hover {
    border-top-color: hsl(var(--b2) / 0);
    border-bottom-color: hsl(var(--b2) / 0)}
.hover\:border-y-base-200\/10:hover {
    border-top-color: hsl(var(--b2) / 0.1);
    border-bottom-color: hsl(var(--b2) / 0.1)}
.hover\:border-y-base-200\/100:hover {
    border-top-color: hsl(var(--b2) / 1);
    border-bottom-color: hsl(var(--b2) / 1)}
.hover\:border-y-base-200\/20:hover {
    border-top-color: hsl(var(--b2) / 0.2);
    border-bottom-color: hsl(var(--b2) / 0.2)}
.hover\:border-y-base-200\/25:hover {
    border-top-color: hsl(var(--b2) / 0.25);
    border-bottom-color: hsl(var(--b2) / 0.25)}
.hover\:border-y-base-200\/30:hover {
    border-top-color: hsl(var(--b2) / 0.3);
    border-bottom-color: hsl(var(--b2) / 0.3)}
.hover\:border-y-base-200\/40:hover {
    border-top-color: hsl(var(--b2) / 0.4);
    border-bottom-color: hsl(var(--b2) / 0.4)}
.hover\:border-y-base-200\/5:hover {
    border-top-color: hsl(var(--b2) / 0.05);
    border-bottom-color: hsl(var(--b2) / 0.05)}
.hover\:border-y-base-200\/50:hover {
    border-top-color: hsl(var(--b2) / 0.5);
    border-bottom-color: hsl(var(--b2) / 0.5)}
.hover\:border-y-base-200\/60:hover {
    border-top-color: hsl(var(--b2) / 0.6);
    border-bottom-color: hsl(var(--b2) / 0.6)}
.hover\:border-y-base-200\/70:hover {
    border-top-color: hsl(var(--b2) / 0.7);
    border-bottom-color: hsl(var(--b2) / 0.7)}
.hover\:border-y-base-200\/75:hover {
    border-top-color: hsl(var(--b2) / 0.75);
    border-bottom-color: hsl(var(--b2) / 0.75)}
.hover\:border-y-base-200\/80:hover {
    border-top-color: hsl(var(--b2) / 0.8);
    border-bottom-color: hsl(var(--b2) / 0.8)}
.hover\:border-y-base-200\/90:hover {
    border-top-color: hsl(var(--b2) / 0.9);
    border-bottom-color: hsl(var(--b2) / 0.9)}
.hover\:border-y-base-200\/95:hover {
    border-top-color: hsl(var(--b2) / 0.95);
    border-bottom-color: hsl(var(--b2) / 0.95)}
.hover\:border-y-base-300:hover {
    border-top-color: hsl(var(--b3) / 1);
    border-bottom-color: hsl(var(--b3) / 1)}
.hover\:border-y-base-300\/0:hover {
    border-top-color: hsl(var(--b3) / 0);
    border-bottom-color: hsl(var(--b3) / 0)}
.hover\:border-y-base-300\/10:hover {
    border-top-color: hsl(var(--b3) / 0.1);
    border-bottom-color: hsl(var(--b3) / 0.1)}
.hover\:border-y-base-300\/100:hover {
    border-top-color: hsl(var(--b3) / 1);
    border-bottom-color: hsl(var(--b3) / 1)}
.hover\:border-y-base-300\/20:hover {
    border-top-color: hsl(var(--b3) / 0.2);
    border-bottom-color: hsl(var(--b3) / 0.2)}
.hover\:border-y-base-300\/25:hover {
    border-top-color: hsl(var(--b3) / 0.25);
    border-bottom-color: hsl(var(--b3) / 0.25)}
.hover\:border-y-base-300\/30:hover {
    border-top-color: hsl(var(--b3) / 0.3);
    border-bottom-color: hsl(var(--b3) / 0.3)}
.hover\:border-y-base-300\/40:hover {
    border-top-color: hsl(var(--b3) / 0.4);
    border-bottom-color: hsl(var(--b3) / 0.4)}
.hover\:border-y-base-300\/5:hover {
    border-top-color: hsl(var(--b3) / 0.05);
    border-bottom-color: hsl(var(--b3) / 0.05)}
.hover\:border-y-base-300\/50:hover {
    border-top-color: hsl(var(--b3) / 0.5);
    border-bottom-color: hsl(var(--b3) / 0.5)}
.hover\:border-y-base-300\/60:hover {
    border-top-color: hsl(var(--b3) / 0.6);
    border-bottom-color: hsl(var(--b3) / 0.6)}
.hover\:border-y-base-300\/70:hover {
    border-top-color: hsl(var(--b3) / 0.7);
    border-bottom-color: hsl(var(--b3) / 0.7)}
.hover\:border-y-base-300\/75:hover {
    border-top-color: hsl(var(--b3) / 0.75);
    border-bottom-color: hsl(var(--b3) / 0.75)}
.hover\:border-y-base-300\/80:hover {
    border-top-color: hsl(var(--b3) / 0.8);
    border-bottom-color: hsl(var(--b3) / 0.8)}
.hover\:border-y-base-300\/90:hover {
    border-top-color: hsl(var(--b3) / 0.9);
    border-bottom-color: hsl(var(--b3) / 0.9)}
.hover\:border-y-base-300\/95:hover {
    border-top-color: hsl(var(--b3) / 0.95);
    border-bottom-color: hsl(var(--b3) / 0.95)}
.hover\:border-y-base-content:hover {
    border-top-color: hsl(var(--bc) / 1);
    border-bottom-color: hsl(var(--bc) / 1)}
.hover\:border-y-base-content\/0:hover {
    border-top-color: hsl(var(--bc) / 0);
    border-bottom-color: hsl(var(--bc) / 0)}
.hover\:border-y-base-content\/10:hover {
    border-top-color: hsl(var(--bc) / 0.1);
    border-bottom-color: hsl(var(--bc) / 0.1)}
.hover\:border-y-base-content\/100:hover {
    border-top-color: hsl(var(--bc) / 1);
    border-bottom-color: hsl(var(--bc) / 1)}
.hover\:border-y-base-content\/20:hover {
    border-top-color: hsl(var(--bc) / 0.2);
    border-bottom-color: hsl(var(--bc) / 0.2)}
.hover\:border-y-base-content\/25:hover {
    border-top-color: hsl(var(--bc) / 0.25);
    border-bottom-color: hsl(var(--bc) / 0.25)}
.hover\:border-y-base-content\/30:hover {
    border-top-color: hsl(var(--bc) / 0.3);
    border-bottom-color: hsl(var(--bc) / 0.3)}
.hover\:border-y-base-content\/40:hover {
    border-top-color: hsl(var(--bc) / 0.4);
    border-bottom-color: hsl(var(--bc) / 0.4)}
.hover\:border-y-base-content\/5:hover {
    border-top-color: hsl(var(--bc) / 0.05);
    border-bottom-color: hsl(var(--bc) / 0.05)}
.hover\:border-y-base-content\/50:hover {
    border-top-color: hsl(var(--bc) / 0.5);
    border-bottom-color: hsl(var(--bc) / 0.5)}
.hover\:border-y-base-content\/60:hover {
    border-top-color: hsl(var(--bc) / 0.6);
    border-bottom-color: hsl(var(--bc) / 0.6)}
.hover\:border-y-base-content\/70:hover {
    border-top-color: hsl(var(--bc) / 0.7);
    border-bottom-color: hsl(var(--bc) / 0.7)}
.hover\:border-y-base-content\/75:hover {
    border-top-color: hsl(var(--bc) / 0.75);
    border-bottom-color: hsl(var(--bc) / 0.75)}
.hover\:border-y-base-content\/80:hover {
    border-top-color: hsl(var(--bc) / 0.8);
    border-bottom-color: hsl(var(--bc) / 0.8)}
.hover\:border-y-base-content\/90:hover {
    border-top-color: hsl(var(--bc) / 0.9);
    border-bottom-color: hsl(var(--bc) / 0.9)}
.hover\:border-y-base-content\/95:hover {
    border-top-color: hsl(var(--bc) / 0.95);
    border-bottom-color: hsl(var(--bc) / 0.95)}
.hover\:border-y-error:hover {
    border-top-color: hsl(var(--er) / 1);
    border-bottom-color: hsl(var(--er) / 1)}
.hover\:border-y-error-content:hover {
    border-top-color: hsl(var(--erc) / 1);
    border-bottom-color: hsl(var(--erc) / 1)}
.hover\:border-y-error-content\/0:hover {
    border-top-color: hsl(var(--erc) / 0);
    border-bottom-color: hsl(var(--erc) / 0)}
.hover\:border-y-error-content\/10:hover {
    border-top-color: hsl(var(--erc) / 0.1);
    border-bottom-color: hsl(var(--erc) / 0.1)}
.hover\:border-y-error-content\/100:hover {
    border-top-color: hsl(var(--erc) / 1);
    border-bottom-color: hsl(var(--erc) / 1)}
.hover\:border-y-error-content\/20:hover {
    border-top-color: hsl(var(--erc) / 0.2);
    border-bottom-color: hsl(var(--erc) / 0.2)}
.hover\:border-y-error-content\/25:hover {
    border-top-color: hsl(var(--erc) / 0.25);
    border-bottom-color: hsl(var(--erc) / 0.25)}
.hover\:border-y-error-content\/30:hover {
    border-top-color: hsl(var(--erc) / 0.3);
    border-bottom-color: hsl(var(--erc) / 0.3)}
.hover\:border-y-error-content\/40:hover {
    border-top-color: hsl(var(--erc) / 0.4);
    border-bottom-color: hsl(var(--erc) / 0.4)}
.hover\:border-y-error-content\/5:hover {
    border-top-color: hsl(var(--erc) / 0.05);
    border-bottom-color: hsl(var(--erc) / 0.05)}
.hover\:border-y-error-content\/50:hover {
    border-top-color: hsl(var(--erc) / 0.5);
    border-bottom-color: hsl(var(--erc) / 0.5)}
.hover\:border-y-error-content\/60:hover {
    border-top-color: hsl(var(--erc) / 0.6);
    border-bottom-color: hsl(var(--erc) / 0.6)}
.hover\:border-y-error-content\/70:hover {
    border-top-color: hsl(var(--erc) / 0.7);
    border-bottom-color: hsl(var(--erc) / 0.7)}
.hover\:border-y-error-content\/75:hover {
    border-top-color: hsl(var(--erc) / 0.75);
    border-bottom-color: hsl(var(--erc) / 0.75)}
.hover\:border-y-error-content\/80:hover {
    border-top-color: hsl(var(--erc) / 0.8);
    border-bottom-color: hsl(var(--erc) / 0.8)}
.hover\:border-y-error-content\/90:hover {
    border-top-color: hsl(var(--erc) / 0.9);
    border-bottom-color: hsl(var(--erc) / 0.9)}
.hover\:border-y-error-content\/95:hover {
    border-top-color: hsl(var(--erc) / 0.95);
    border-bottom-color: hsl(var(--erc) / 0.95)}
.hover\:border-y-error\/0:hover {
    border-top-color: hsl(var(--er) / 0);
    border-bottom-color: hsl(var(--er) / 0)}
.hover\:border-y-error\/10:hover {
    border-top-color: hsl(var(--er) / 0.1);
    border-bottom-color: hsl(var(--er) / 0.1)}
.hover\:border-y-error\/100:hover {
    border-top-color: hsl(var(--er) / 1);
    border-bottom-color: hsl(var(--er) / 1)}
.hover\:border-y-error\/20:hover {
    border-top-color: hsl(var(--er) / 0.2);
    border-bottom-color: hsl(var(--er) / 0.2)}
.hover\:border-y-error\/25:hover {
    border-top-color: hsl(var(--er) / 0.25);
    border-bottom-color: hsl(var(--er) / 0.25)}
.hover\:border-y-error\/30:hover {
    border-top-color: hsl(var(--er) / 0.3);
    border-bottom-color: hsl(var(--er) / 0.3)}
.hover\:border-y-error\/40:hover {
    border-top-color: hsl(var(--er) / 0.4);
    border-bottom-color: hsl(var(--er) / 0.4)}
.hover\:border-y-error\/5:hover {
    border-top-color: hsl(var(--er) / 0.05);
    border-bottom-color: hsl(var(--er) / 0.05)}
.hover\:border-y-error\/50:hover {
    border-top-color: hsl(var(--er) / 0.5);
    border-bottom-color: hsl(var(--er) / 0.5)}
.hover\:border-y-error\/60:hover {
    border-top-color: hsl(var(--er) / 0.6);
    border-bottom-color: hsl(var(--er) / 0.6)}
.hover\:border-y-error\/70:hover {
    border-top-color: hsl(var(--er) / 0.7);
    border-bottom-color: hsl(var(--er) / 0.7)}
.hover\:border-y-error\/75:hover {
    border-top-color: hsl(var(--er) / 0.75);
    border-bottom-color: hsl(var(--er) / 0.75)}
.hover\:border-y-error\/80:hover {
    border-top-color: hsl(var(--er) / 0.8);
    border-bottom-color: hsl(var(--er) / 0.8)}
.hover\:border-y-error\/90:hover {
    border-top-color: hsl(var(--er) / 0.9);
    border-bottom-color: hsl(var(--er) / 0.9)}
.hover\:border-y-error\/95:hover {
    border-top-color: hsl(var(--er) / 0.95);
    border-bottom-color: hsl(var(--er) / 0.95)}
.hover\:border-y-info:hover {
    border-top-color: hsl(var(--in) / 1);
    border-bottom-color: hsl(var(--in) / 1)}
.hover\:border-y-info-content:hover {
    border-top-color: hsl(var(--inc) / 1);
    border-bottom-color: hsl(var(--inc) / 1)}
.hover\:border-y-info-content\/0:hover {
    border-top-color: hsl(var(--inc) / 0);
    border-bottom-color: hsl(var(--inc) / 0)}
.hover\:border-y-info-content\/10:hover {
    border-top-color: hsl(var(--inc) / 0.1);
    border-bottom-color: hsl(var(--inc) / 0.1)}
.hover\:border-y-info-content\/100:hover {
    border-top-color: hsl(var(--inc) / 1);
    border-bottom-color: hsl(var(--inc) / 1)}
.hover\:border-y-info-content\/20:hover {
    border-top-color: hsl(var(--inc) / 0.2);
    border-bottom-color: hsl(var(--inc) / 0.2)}
.hover\:border-y-info-content\/25:hover {
    border-top-color: hsl(var(--inc) / 0.25);
    border-bottom-color: hsl(var(--inc) / 0.25)}
.hover\:border-y-info-content\/30:hover {
    border-top-color: hsl(var(--inc) / 0.3);
    border-bottom-color: hsl(var(--inc) / 0.3)}
.hover\:border-y-info-content\/40:hover {
    border-top-color: hsl(var(--inc) / 0.4);
    border-bottom-color: hsl(var(--inc) / 0.4)}
.hover\:border-y-info-content\/5:hover {
    border-top-color: hsl(var(--inc) / 0.05);
    border-bottom-color: hsl(var(--inc) / 0.05)}
.hover\:border-y-info-content\/50:hover {
    border-top-color: hsl(var(--inc) / 0.5);
    border-bottom-color: hsl(var(--inc) / 0.5)}
.hover\:border-y-info-content\/60:hover {
    border-top-color: hsl(var(--inc) / 0.6);
    border-bottom-color: hsl(var(--inc) / 0.6)}
.hover\:border-y-info-content\/70:hover {
    border-top-color: hsl(var(--inc) / 0.7);
    border-bottom-color: hsl(var(--inc) / 0.7)}
.hover\:border-y-info-content\/75:hover {
    border-top-color: hsl(var(--inc) / 0.75);
    border-bottom-color: hsl(var(--inc) / 0.75)}
.hover\:border-y-info-content\/80:hover {
    border-top-color: hsl(var(--inc) / 0.8);
    border-bottom-color: hsl(var(--inc) / 0.8)}
.hover\:border-y-info-content\/90:hover {
    border-top-color: hsl(var(--inc) / 0.9);
    border-bottom-color: hsl(var(--inc) / 0.9)}
.hover\:border-y-info-content\/95:hover {
    border-top-color: hsl(var(--inc) / 0.95);
    border-bottom-color: hsl(var(--inc) / 0.95)}
.hover\:border-y-info\/0:hover {
    border-top-color: hsl(var(--in) / 0);
    border-bottom-color: hsl(var(--in) / 0)}
.hover\:border-y-info\/10:hover {
    border-top-color: hsl(var(--in) / 0.1);
    border-bottom-color: hsl(var(--in) / 0.1)}
.hover\:border-y-info\/100:hover {
    border-top-color: hsl(var(--in) / 1);
    border-bottom-color: hsl(var(--in) / 1)}
.hover\:border-y-info\/20:hover {
    border-top-color: hsl(var(--in) / 0.2);
    border-bottom-color: hsl(var(--in) / 0.2)}
.hover\:border-y-info\/25:hover {
    border-top-color: hsl(var(--in) / 0.25);
    border-bottom-color: hsl(var(--in) / 0.25)}
.hover\:border-y-info\/30:hover {
    border-top-color: hsl(var(--in) / 0.3);
    border-bottom-color: hsl(var(--in) / 0.3)}
.hover\:border-y-info\/40:hover {
    border-top-color: hsl(var(--in) / 0.4);
    border-bottom-color: hsl(var(--in) / 0.4)}
.hover\:border-y-info\/5:hover {
    border-top-color: hsl(var(--in) / 0.05);
    border-bottom-color: hsl(var(--in) / 0.05)}
.hover\:border-y-info\/50:hover {
    border-top-color: hsl(var(--in) / 0.5);
    border-bottom-color: hsl(var(--in) / 0.5)}
.hover\:border-y-info\/60:hover {
    border-top-color: hsl(var(--in) / 0.6);
    border-bottom-color: hsl(var(--in) / 0.6)}
.hover\:border-y-info\/70:hover {
    border-top-color: hsl(var(--in) / 0.7);
    border-bottom-color: hsl(var(--in) / 0.7)}
.hover\:border-y-info\/75:hover {
    border-top-color: hsl(var(--in) / 0.75);
    border-bottom-color: hsl(var(--in) / 0.75)}
.hover\:border-y-info\/80:hover {
    border-top-color: hsl(var(--in) / 0.8);
    border-bottom-color: hsl(var(--in) / 0.8)}
.hover\:border-y-info\/90:hover {
    border-top-color: hsl(var(--in) / 0.9);
    border-bottom-color: hsl(var(--in) / 0.9)}
.hover\:border-y-info\/95:hover {
    border-top-color: hsl(var(--in) / 0.95);
    border-bottom-color: hsl(var(--in) / 0.95)}
.hover\:border-y-success:hover {
    border-top-color: hsl(var(--su) / 1);
    border-bottom-color: hsl(var(--su) / 1)}
.hover\:border-y-success-content:hover {
    border-top-color: hsl(var(--suc) / 1);
    border-bottom-color: hsl(var(--suc) / 1)}
.hover\:border-y-success-content\/0:hover {
    border-top-color: hsl(var(--suc) / 0);
    border-bottom-color: hsl(var(--suc) / 0)}
.hover\:border-y-success-content\/10:hover {
    border-top-color: hsl(var(--suc) / 0.1);
    border-bottom-color: hsl(var(--suc) / 0.1)}
.hover\:border-y-success-content\/100:hover {
    border-top-color: hsl(var(--suc) / 1);
    border-bottom-color: hsl(var(--suc) / 1)}
.hover\:border-y-success-content\/20:hover {
    border-top-color: hsl(var(--suc) / 0.2);
    border-bottom-color: hsl(var(--suc) / 0.2)}
.hover\:border-y-success-content\/25:hover {
    border-top-color: hsl(var(--suc) / 0.25);
    border-bottom-color: hsl(var(--suc) / 0.25)}
.hover\:border-y-success-content\/30:hover {
    border-top-color: hsl(var(--suc) / 0.3);
    border-bottom-color: hsl(var(--suc) / 0.3)}
.hover\:border-y-success-content\/40:hover {
    border-top-color: hsl(var(--suc) / 0.4);
    border-bottom-color: hsl(var(--suc) / 0.4)}
.hover\:border-y-success-content\/5:hover {
    border-top-color: hsl(var(--suc) / 0.05);
    border-bottom-color: hsl(var(--suc) / 0.05)}
.hover\:border-y-success-content\/50:hover {
    border-top-color: hsl(var(--suc) / 0.5);
    border-bottom-color: hsl(var(--suc) / 0.5)}
.hover\:border-y-success-content\/60:hover {
    border-top-color: hsl(var(--suc) / 0.6);
    border-bottom-color: hsl(var(--suc) / 0.6)}
.hover\:border-y-success-content\/70:hover {
    border-top-color: hsl(var(--suc) / 0.7);
    border-bottom-color: hsl(var(--suc) / 0.7)}
.hover\:border-y-success-content\/75:hover {
    border-top-color: hsl(var(--suc) / 0.75);
    border-bottom-color: hsl(var(--suc) / 0.75)}
.hover\:border-y-success-content\/80:hover {
    border-top-color: hsl(var(--suc) / 0.8);
    border-bottom-color: hsl(var(--suc) / 0.8)}
.hover\:border-y-success-content\/90:hover {
    border-top-color: hsl(var(--suc) / 0.9);
    border-bottom-color: hsl(var(--suc) / 0.9)}
.hover\:border-y-success-content\/95:hover {
    border-top-color: hsl(var(--suc) / 0.95);
    border-bottom-color: hsl(var(--suc) / 0.95)}
.hover\:border-y-success\/0:hover {
    border-top-color: hsl(var(--su) / 0);
    border-bottom-color: hsl(var(--su) / 0)}
.hover\:border-y-success\/10:hover {
    border-top-color: hsl(var(--su) / 0.1);
    border-bottom-color: hsl(var(--su) / 0.1)}
.hover\:border-y-success\/100:hover {
    border-top-color: hsl(var(--su) / 1);
    border-bottom-color: hsl(var(--su) / 1)}
.hover\:border-y-success\/20:hover {
    border-top-color: hsl(var(--su) / 0.2);
    border-bottom-color: hsl(var(--su) / 0.2)}
.hover\:border-y-success\/25:hover {
    border-top-color: hsl(var(--su) / 0.25);
    border-bottom-color: hsl(var(--su) / 0.25)}
.hover\:border-y-success\/30:hover {
    border-top-color: hsl(var(--su) / 0.3);
    border-bottom-color: hsl(var(--su) / 0.3)}
.hover\:border-y-success\/40:hover {
    border-top-color: hsl(var(--su) / 0.4);
    border-bottom-color: hsl(var(--su) / 0.4)}
.hover\:border-y-success\/5:hover {
    border-top-color: hsl(var(--su) / 0.05);
    border-bottom-color: hsl(var(--su) / 0.05)}
.hover\:border-y-success\/50:hover {
    border-top-color: hsl(var(--su) / 0.5);
    border-bottom-color: hsl(var(--su) / 0.5)}
.hover\:border-y-success\/60:hover {
    border-top-color: hsl(var(--su) / 0.6);
    border-bottom-color: hsl(var(--su) / 0.6)}
.hover\:border-y-success\/70:hover {
    border-top-color: hsl(var(--su) / 0.7);
    border-bottom-color: hsl(var(--su) / 0.7)}
.hover\:border-y-success\/75:hover {
    border-top-color: hsl(var(--su) / 0.75);
    border-bottom-color: hsl(var(--su) / 0.75)}
.hover\:border-y-success\/80:hover {
    border-top-color: hsl(var(--su) / 0.8);
    border-bottom-color: hsl(var(--su) / 0.8)}
.hover\:border-y-success\/90:hover {
    border-top-color: hsl(var(--su) / 0.9);
    border-bottom-color: hsl(var(--su) / 0.9)}
.hover\:border-y-success\/95:hover {
    border-top-color: hsl(var(--su) / 0.95);
    border-bottom-color: hsl(var(--su) / 0.95)}
.hover\:border-y-warning:hover {
    border-top-color: hsl(var(--wa) / 1);
    border-bottom-color: hsl(var(--wa) / 1)}
.hover\:border-y-warning-content:hover {
    border-top-color: hsl(var(--wac) / 1);
    border-bottom-color: hsl(var(--wac) / 1)}
.hover\:border-y-warning-content\/0:hover {
    border-top-color: hsl(var(--wac) / 0);
    border-bottom-color: hsl(var(--wac) / 0)}
.hover\:border-y-warning-content\/10:hover {
    border-top-color: hsl(var(--wac) / 0.1);
    border-bottom-color: hsl(var(--wac) / 0.1)}
.hover\:border-y-warning-content\/100:hover {
    border-top-color: hsl(var(--wac) / 1);
    border-bottom-color: hsl(var(--wac) / 1)}
.hover\:border-y-warning-content\/20:hover {
    border-top-color: hsl(var(--wac) / 0.2);
    border-bottom-color: hsl(var(--wac) / 0.2)}
.hover\:border-y-warning-content\/25:hover {
    border-top-color: hsl(var(--wac) / 0.25);
    border-bottom-color: hsl(var(--wac) / 0.25)}
.hover\:border-y-warning-content\/30:hover {
    border-top-color: hsl(var(--wac) / 0.3);
    border-bottom-color: hsl(var(--wac) / 0.3)}
.hover\:border-y-warning-content\/40:hover {
    border-top-color: hsl(var(--wac) / 0.4);
    border-bottom-color: hsl(var(--wac) / 0.4)}
.hover\:border-y-warning-content\/5:hover {
    border-top-color: hsl(var(--wac) / 0.05);
    border-bottom-color: hsl(var(--wac) / 0.05)}
.hover\:border-y-warning-content\/50:hover {
    border-top-color: hsl(var(--wac) / 0.5);
    border-bottom-color: hsl(var(--wac) / 0.5)}
.hover\:border-y-warning-content\/60:hover {
    border-top-color: hsl(var(--wac) / 0.6);
    border-bottom-color: hsl(var(--wac) / 0.6)}
.hover\:border-y-warning-content\/70:hover {
    border-top-color: hsl(var(--wac) / 0.7);
    border-bottom-color: hsl(var(--wac) / 0.7)}
.hover\:border-y-warning-content\/75:hover {
    border-top-color: hsl(var(--wac) / 0.75);
    border-bottom-color: hsl(var(--wac) / 0.75)}
.hover\:border-y-warning-content\/80:hover {
    border-top-color: hsl(var(--wac) / 0.8);
    border-bottom-color: hsl(var(--wac) / 0.8)}
.hover\:border-y-warning-content\/90:hover {
    border-top-color: hsl(var(--wac) / 0.9);
    border-bottom-color: hsl(var(--wac) / 0.9)}
.hover\:border-y-warning-content\/95:hover {
    border-top-color: hsl(var(--wac) / 0.95);
    border-bottom-color: hsl(var(--wac) / 0.95)}
.hover\:border-y-warning\/0:hover {
    border-top-color: hsl(var(--wa) / 0);
    border-bottom-color: hsl(var(--wa) / 0)}
.hover\:border-y-warning\/10:hover {
    border-top-color: hsl(var(--wa) / 0.1);
    border-bottom-color: hsl(var(--wa) / 0.1)}
.hover\:border-y-warning\/100:hover {
    border-top-color: hsl(var(--wa) / 1);
    border-bottom-color: hsl(var(--wa) / 1)}
.hover\:border-y-warning\/20:hover {
    border-top-color: hsl(var(--wa) / 0.2);
    border-bottom-color: hsl(var(--wa) / 0.2)}
.hover\:border-y-warning\/25:hover {
    border-top-color: hsl(var(--wa) / 0.25);
    border-bottom-color: hsl(var(--wa) / 0.25)}
.hover\:border-y-warning\/30:hover {
    border-top-color: hsl(var(--wa) / 0.3);
    border-bottom-color: hsl(var(--wa) / 0.3)}
.hover\:border-y-warning\/40:hover {
    border-top-color: hsl(var(--wa) / 0.4);
    border-bottom-color: hsl(var(--wa) / 0.4)}
.hover\:border-y-warning\/5:hover {
    border-top-color: hsl(var(--wa) / 0.05);
    border-bottom-color: hsl(var(--wa) / 0.05)}
.hover\:border-y-warning\/50:hover {
    border-top-color: hsl(var(--wa) / 0.5);
    border-bottom-color: hsl(var(--wa) / 0.5)}
.hover\:border-y-warning\/60:hover {
    border-top-color: hsl(var(--wa) / 0.6);
    border-bottom-color: hsl(var(--wa) / 0.6)}
.hover\:border-y-warning\/70:hover {
    border-top-color: hsl(var(--wa) / 0.7);
    border-bottom-color: hsl(var(--wa) / 0.7)}
.hover\:border-y-warning\/75:hover {
    border-top-color: hsl(var(--wa) / 0.75);
    border-bottom-color: hsl(var(--wa) / 0.75)}
.hover\:border-y-warning\/80:hover {
    border-top-color: hsl(var(--wa) / 0.8);
    border-bottom-color: hsl(var(--wa) / 0.8)}
.hover\:border-y-warning\/90:hover {
    border-top-color: hsl(var(--wa) / 0.9);
    border-bottom-color: hsl(var(--wa) / 0.9)}
.hover\:border-y-warning\/95:hover {
    border-top-color: hsl(var(--wa) / 0.95);
    border-bottom-color: hsl(var(--wa) / 0.95)}
.hover\:border-b-base-100:hover {
    border-bottom-color: hsl(var(--b1) / 1)}
.hover\:border-b-base-100\/0:hover {
    border-bottom-color: hsl(var(--b1) / 0)}
.hover\:border-b-base-100\/10:hover {
    border-bottom-color: hsl(var(--b1) / 0.1)}
.hover\:border-b-base-100\/100:hover {
    border-bottom-color: hsl(var(--b1) / 1)}
.hover\:border-b-base-100\/20:hover {
    border-bottom-color: hsl(var(--b1) / 0.2)}
.hover\:border-b-base-100\/25:hover {
    border-bottom-color: hsl(var(--b1) / 0.25)}
.hover\:border-b-base-100\/30:hover {
    border-bottom-color: hsl(var(--b1) / 0.3)}
.hover\:border-b-base-100\/40:hover {
    border-bottom-color: hsl(var(--b1) / 0.4)}
.hover\:border-b-base-100\/5:hover {
    border-bottom-color: hsl(var(--b1) / 0.05)}
.hover\:border-b-base-100\/50:hover {
    border-bottom-color: hsl(var(--b1) / 0.5)}
.hover\:border-b-base-100\/60:hover {
    border-bottom-color: hsl(var(--b1) / 0.6)}
.hover\:border-b-base-100\/70:hover {
    border-bottom-color: hsl(var(--b1) / 0.7)}
.hover\:border-b-base-100\/75:hover {
    border-bottom-color: hsl(var(--b1) / 0.75)}
.hover\:border-b-base-100\/80:hover {
    border-bottom-color: hsl(var(--b1) / 0.8)}
.hover\:border-b-base-100\/90:hover {
    border-bottom-color: hsl(var(--b1) / 0.9)}
.hover\:border-b-base-100\/95:hover {
    border-bottom-color: hsl(var(--b1) / 0.95)}
.hover\:border-b-base-200:hover {
    border-bottom-color: hsl(var(--b2) / 1)}
.hover\:border-b-base-200\/0:hover {
    border-bottom-color: hsl(var(--b2) / 0)}
.hover\:border-b-base-200\/10:hover {
    border-bottom-color: hsl(var(--b2) / 0.1)}
.hover\:border-b-base-200\/100:hover {
    border-bottom-color: hsl(var(--b2) / 1)}
.hover\:border-b-base-200\/20:hover {
    border-bottom-color: hsl(var(--b2) / 0.2)}
.hover\:border-b-base-200\/25:hover {
    border-bottom-color: hsl(var(--b2) / 0.25)}
.hover\:border-b-base-200\/30:hover {
    border-bottom-color: hsl(var(--b2) / 0.3)}
.hover\:border-b-base-200\/40:hover {
    border-bottom-color: hsl(var(--b2) / 0.4)}
.hover\:border-b-base-200\/5:hover {
    border-bottom-color: hsl(var(--b2) / 0.05)}
.hover\:border-b-base-200\/50:hover {
    border-bottom-color: hsl(var(--b2) / 0.5)}
.hover\:border-b-base-200\/60:hover {
    border-bottom-color: hsl(var(--b2) / 0.6)}
.hover\:border-b-base-200\/70:hover {
    border-bottom-color: hsl(var(--b2) / 0.7)}
.hover\:border-b-base-200\/75:hover {
    border-bottom-color: hsl(var(--b2) / 0.75)}
.hover\:border-b-base-200\/80:hover {
    border-bottom-color: hsl(var(--b2) / 0.8)}
.hover\:border-b-base-200\/90:hover {
    border-bottom-color: hsl(var(--b2) / 0.9)}
.hover\:border-b-base-200\/95:hover {
    border-bottom-color: hsl(var(--b2) / 0.95)}
.hover\:border-b-base-300:hover {
    border-bottom-color: hsl(var(--b3) / 1)}
.hover\:border-b-base-300\/0:hover {
    border-bottom-color: hsl(var(--b3) / 0)}
.hover\:border-b-base-300\/10:hover {
    border-bottom-color: hsl(var(--b3) / 0.1)}
.hover\:border-b-base-300\/100:hover {
    border-bottom-color: hsl(var(--b3) / 1)}
.hover\:border-b-base-300\/20:hover {
    border-bottom-color: hsl(var(--b3) / 0.2)}
.hover\:border-b-base-300\/25:hover {
    border-bottom-color: hsl(var(--b3) / 0.25)}
.hover\:border-b-base-300\/30:hover {
    border-bottom-color: hsl(var(--b3) / 0.3)}
.hover\:border-b-base-300\/40:hover {
    border-bottom-color: hsl(var(--b3) / 0.4)}
.hover\:border-b-base-300\/5:hover {
    border-bottom-color: hsl(var(--b3) / 0.05)}
.hover\:border-b-base-300\/50:hover {
    border-bottom-color: hsl(var(--b3) / 0.5)}
.hover\:border-b-base-300\/60:hover {
    border-bottom-color: hsl(var(--b3) / 0.6)}
.hover\:border-b-base-300\/70:hover {
    border-bottom-color: hsl(var(--b3) / 0.7)}
.hover\:border-b-base-300\/75:hover {
    border-bottom-color: hsl(var(--b3) / 0.75)}
.hover\:border-b-base-300\/80:hover {
    border-bottom-color: hsl(var(--b3) / 0.8)}
.hover\:border-b-base-300\/90:hover {
    border-bottom-color: hsl(var(--b3) / 0.9)}
.hover\:border-b-base-300\/95:hover {
    border-bottom-color: hsl(var(--b3) / 0.95)}
.hover\:border-b-base-content:hover {
    border-bottom-color: hsl(var(--bc) / 1)}
.hover\:border-b-base-content\/0:hover {
    border-bottom-color: hsl(var(--bc) / 0)}
.hover\:border-b-base-content\/10:hover {
    border-bottom-color: hsl(var(--bc) / 0.1)}
.hover\:border-b-base-content\/100:hover {
    border-bottom-color: hsl(var(--bc) / 1)}
.hover\:border-b-base-content\/20:hover {
    border-bottom-color: hsl(var(--bc) / 0.2)}
.hover\:border-b-base-content\/25:hover {
    border-bottom-color: hsl(var(--bc) / 0.25)}
.hover\:border-b-base-content\/30:hover {
    border-bottom-color: hsl(var(--bc) / 0.3)}
.hover\:border-b-base-content\/40:hover {
    border-bottom-color: hsl(var(--bc) / 0.4)}
.hover\:border-b-base-content\/5:hover {
    border-bottom-color: hsl(var(--bc) / 0.05)}
.hover\:border-b-base-content\/50:hover {
    border-bottom-color: hsl(var(--bc) / 0.5)}
.hover\:border-b-base-content\/60:hover {
    border-bottom-color: hsl(var(--bc) / 0.6)}
.hover\:border-b-base-content\/70:hover {
    border-bottom-color: hsl(var(--bc) / 0.7)}
.hover\:border-b-base-content\/75:hover {
    border-bottom-color: hsl(var(--bc) / 0.75)}
.hover\:border-b-base-content\/80:hover {
    border-bottom-color: hsl(var(--bc) / 0.8)}
.hover\:border-b-base-content\/90:hover {
    border-bottom-color: hsl(var(--bc) / 0.9)}
.hover\:border-b-base-content\/95:hover {
    border-bottom-color: hsl(var(--bc) / 0.95)}
.hover\:border-b-error:hover {
    border-bottom-color: hsl(var(--er) / 1)}
.hover\:border-b-error-content:hover {
    border-bottom-color: hsl(var(--erc) / 1)}
.hover\:border-b-error-content\/0:hover {
    border-bottom-color: hsl(var(--erc) / 0)}
.hover\:border-b-error-content\/10:hover {
    border-bottom-color: hsl(var(--erc) / 0.1)}
.hover\:border-b-error-content\/100:hover {
    border-bottom-color: hsl(var(--erc) / 1)}
.hover\:border-b-error-content\/20:hover {
    border-bottom-color: hsl(var(--erc) / 0.2)}
.hover\:border-b-error-content\/25:hover {
    border-bottom-color: hsl(var(--erc) / 0.25)}
.hover\:border-b-error-content\/30:hover {
    border-bottom-color: hsl(var(--erc) / 0.3)}
.hover\:border-b-error-content\/40:hover {
    border-bottom-color: hsl(var(--erc) / 0.4)}
.hover\:border-b-error-content\/5:hover {
    border-bottom-color: hsl(var(--erc) / 0.05)}
.hover\:border-b-error-content\/50:hover {
    border-bottom-color: hsl(var(--erc) / 0.5)}
.hover\:border-b-error-content\/60:hover {
    border-bottom-color: hsl(var(--erc) / 0.6)}
.hover\:border-b-error-content\/70:hover {
    border-bottom-color: hsl(var(--erc) / 0.7)}
.hover\:border-b-error-content\/75:hover {
    border-bottom-color: hsl(var(--erc) / 0.75)}
.hover\:border-b-error-content\/80:hover {
    border-bottom-color: hsl(var(--erc) / 0.8)}
.hover\:border-b-error-content\/90:hover {
    border-bottom-color: hsl(var(--erc) / 0.9)}
.hover\:border-b-error-content\/95:hover {
    border-bottom-color: hsl(var(--erc) / 0.95)}
.hover\:border-b-error\/0:hover {
    border-bottom-color: hsl(var(--er) / 0)}
.hover\:border-b-error\/10:hover {
    border-bottom-color: hsl(var(--er) / 0.1)}
.hover\:border-b-error\/100:hover {
    border-bottom-color: hsl(var(--er) / 1)}
.hover\:border-b-error\/20:hover {
    border-bottom-color: hsl(var(--er) / 0.2)}
.hover\:border-b-error\/25:hover {
    border-bottom-color: hsl(var(--er) / 0.25)}
.hover\:border-b-error\/30:hover {
    border-bottom-color: hsl(var(--er) / 0.3)}
.hover\:border-b-error\/40:hover {
    border-bottom-color: hsl(var(--er) / 0.4)}
.hover\:border-b-error\/5:hover {
    border-bottom-color: hsl(var(--er) / 0.05)}
.hover\:border-b-error\/50:hover {
    border-bottom-color: hsl(var(--er) / 0.5)}
.hover\:border-b-error\/60:hover {
    border-bottom-color: hsl(var(--er) / 0.6)}
.hover\:border-b-error\/70:hover {
    border-bottom-color: hsl(var(--er) / 0.7)}
.hover\:border-b-error\/75:hover {
    border-bottom-color: hsl(var(--er) / 0.75)}
.hover\:border-b-error\/80:hover {
    border-bottom-color: hsl(var(--er) / 0.8)}
.hover\:border-b-error\/90:hover {
    border-bottom-color: hsl(var(--er) / 0.9)}
.hover\:border-b-error\/95:hover {
    border-bottom-color: hsl(var(--er) / 0.95)}
.hover\:border-b-info:hover {
    border-bottom-color: hsl(var(--in) / 1)}
.hover\:border-b-info-content:hover {
    border-bottom-color: hsl(var(--inc) / 1)}
.hover\:border-b-info-content\/0:hover {
    border-bottom-color: hsl(var(--inc) / 0)}
.hover\:border-b-info-content\/10:hover {
    border-bottom-color: hsl(var(--inc) / 0.1)}
.hover\:border-b-info-content\/100:hover {
    border-bottom-color: hsl(var(--inc) / 1)}
.hover\:border-b-info-content\/20:hover {
    border-bottom-color: hsl(var(--inc) / 0.2)}
.hover\:border-b-info-content\/25:hover {
    border-bottom-color: hsl(var(--inc) / 0.25)}
.hover\:border-b-info-content\/30:hover {
    border-bottom-color: hsl(var(--inc) / 0.3)}
.hover\:border-b-info-content\/40:hover {
    border-bottom-color: hsl(var(--inc) / 0.4)}
.hover\:border-b-info-content\/5:hover {
    border-bottom-color: hsl(var(--inc) / 0.05)}
.hover\:border-b-info-content\/50:hover {
    border-bottom-color: hsl(var(--inc) / 0.5)}
.hover\:border-b-info-content\/60:hover {
    border-bottom-color: hsl(var(--inc) / 0.6)}
.hover\:border-b-info-content\/70:hover {
    border-bottom-color: hsl(var(--inc) / 0.7)}
.hover\:border-b-info-content\/75:hover {
    border-bottom-color: hsl(var(--inc) / 0.75)}
.hover\:border-b-info-content\/80:hover {
    border-bottom-color: hsl(var(--inc) / 0.8)}
.hover\:border-b-info-content\/90:hover {
    border-bottom-color: hsl(var(--inc) / 0.9)}
.hover\:border-b-info-content\/95:hover {
    border-bottom-color: hsl(var(--inc) / 0.95)}
.hover\:border-b-info\/0:hover {
    border-bottom-color: hsl(var(--in) / 0)}
.hover\:border-b-info\/10:hover {
    border-bottom-color: hsl(var(--in) / 0.1)}
.hover\:border-b-info\/100:hover {
    border-bottom-color: hsl(var(--in) / 1)}
.hover\:border-b-info\/20:hover {
    border-bottom-color: hsl(var(--in) / 0.2)}
.hover\:border-b-info\/25:hover {
    border-bottom-color: hsl(var(--in) / 0.25)}
.hover\:border-b-info\/30:hover {
    border-bottom-color: hsl(var(--in) / 0.3)}
.hover\:border-b-info\/40:hover {
    border-bottom-color: hsl(var(--in) / 0.4)}
.hover\:border-b-info\/5:hover {
    border-bottom-color: hsl(var(--in) / 0.05)}
.hover\:border-b-info\/50:hover {
    border-bottom-color: hsl(var(--in) / 0.5)}
.hover\:border-b-info\/60:hover {
    border-bottom-color: hsl(var(--in) / 0.6)}
.hover\:border-b-info\/70:hover {
    border-bottom-color: hsl(var(--in) / 0.7)}
.hover\:border-b-info\/75:hover {
    border-bottom-color: hsl(var(--in) / 0.75)}
.hover\:border-b-info\/80:hover {
    border-bottom-color: hsl(var(--in) / 0.8)}
.hover\:border-b-info\/90:hover {
    border-bottom-color: hsl(var(--in) / 0.9)}
.hover\:border-b-info\/95:hover {
    border-bottom-color: hsl(var(--in) / 0.95)}
.hover\:border-b-success:hover {
    border-bottom-color: hsl(var(--su) / 1)}
.hover\:border-b-success-content:hover {
    border-bottom-color: hsl(var(--suc) / 1)}
.hover\:border-b-success-content\/0:hover {
    border-bottom-color: hsl(var(--suc) / 0)}
.hover\:border-b-success-content\/10:hover {
    border-bottom-color: hsl(var(--suc) / 0.1)}
.hover\:border-b-success-content\/100:hover {
    border-bottom-color: hsl(var(--suc) / 1)}
.hover\:border-b-success-content\/20:hover {
    border-bottom-color: hsl(var(--suc) / 0.2)}
.hover\:border-b-success-content\/25:hover {
    border-bottom-color: hsl(var(--suc) / 0.25)}
.hover\:border-b-success-content\/30:hover {
    border-bottom-color: hsl(var(--suc) / 0.3)}
.hover\:border-b-success-content\/40:hover {
    border-bottom-color: hsl(var(--suc) / 0.4)}
.hover\:border-b-success-content\/5:hover {
    border-bottom-color: hsl(var(--suc) / 0.05)}
.hover\:border-b-success-content\/50:hover {
    border-bottom-color: hsl(var(--suc) / 0.5)}
.hover\:border-b-success-content\/60:hover {
    border-bottom-color: hsl(var(--suc) / 0.6)}
.hover\:border-b-success-content\/70:hover {
    border-bottom-color: hsl(var(--suc) / 0.7)}
.hover\:border-b-success-content\/75:hover {
    border-bottom-color: hsl(var(--suc) / 0.75)}
.hover\:border-b-success-content\/80:hover {
    border-bottom-color: hsl(var(--suc) / 0.8)}
.hover\:border-b-success-content\/90:hover {
    border-bottom-color: hsl(var(--suc) / 0.9)}
.hover\:border-b-success-content\/95:hover {
    border-bottom-color: hsl(var(--suc) / 0.95)}
.hover\:border-b-success\/0:hover {
    border-bottom-color: hsl(var(--su) / 0)}
.hover\:border-b-success\/10:hover {
    border-bottom-color: hsl(var(--su) / 0.1)}
.hover\:border-b-success\/100:hover {
    border-bottom-color: hsl(var(--su) / 1)}
.hover\:border-b-success\/20:hover {
    border-bottom-color: hsl(var(--su) / 0.2)}
.hover\:border-b-success\/25:hover {
    border-bottom-color: hsl(var(--su) / 0.25)}
.hover\:border-b-success\/30:hover {
    border-bottom-color: hsl(var(--su) / 0.3)}
.hover\:border-b-success\/40:hover {
    border-bottom-color: hsl(var(--su) / 0.4)}
.hover\:border-b-success\/5:hover {
    border-bottom-color: hsl(var(--su) / 0.05)}
.hover\:border-b-success\/50:hover {
    border-bottom-color: hsl(var(--su) / 0.5)}
.hover\:border-b-success\/60:hover {
    border-bottom-color: hsl(var(--su) / 0.6)}
.hover\:border-b-success\/70:hover {
    border-bottom-color: hsl(var(--su) / 0.7)}
.hover\:border-b-success\/75:hover {
    border-bottom-color: hsl(var(--su) / 0.75)}
.hover\:border-b-success\/80:hover {
    border-bottom-color: hsl(var(--su) / 0.8)}
.hover\:border-b-success\/90:hover {
    border-bottom-color: hsl(var(--su) / 0.9)}
.hover\:border-b-success\/95:hover {
    border-bottom-color: hsl(var(--su) / 0.95)}
.hover\:border-b-warning:hover {
    border-bottom-color: hsl(var(--wa) / 1)}
.hover\:border-b-warning-content:hover {
    border-bottom-color: hsl(var(--wac) / 1)}
.hover\:border-b-warning-content\/0:hover {
    border-bottom-color: hsl(var(--wac) / 0)}
.hover\:border-b-warning-content\/10:hover {
    border-bottom-color: hsl(var(--wac) / 0.1)}
.hover\:border-b-warning-content\/100:hover {
    border-bottom-color: hsl(var(--wac) / 1)}
.hover\:border-b-warning-content\/20:hover {
    border-bottom-color: hsl(var(--wac) / 0.2)}
.hover\:border-b-warning-content\/25:hover {
    border-bottom-color: hsl(var(--wac) / 0.25)}
.hover\:border-b-warning-content\/30:hover {
    border-bottom-color: hsl(var(--wac) / 0.3)}
.hover\:border-b-warning-content\/40:hover {
    border-bottom-color: hsl(var(--wac) / 0.4)}
.hover\:border-b-warning-content\/5:hover {
    border-bottom-color: hsl(var(--wac) / 0.05)}
.hover\:border-b-warning-content\/50:hover {
    border-bottom-color: hsl(var(--wac) / 0.5)}
.hover\:border-b-warning-content\/60:hover {
    border-bottom-color: hsl(var(--wac) / 0.6)}
.hover\:border-b-warning-content\/70:hover {
    border-bottom-color: hsl(var(--wac) / 0.7)}
.hover\:border-b-warning-content\/75:hover {
    border-bottom-color: hsl(var(--wac) / 0.75)}
.hover\:border-b-warning-content\/80:hover {
    border-bottom-color: hsl(var(--wac) / 0.8)}
.hover\:border-b-warning-content\/90:hover {
    border-bottom-color: hsl(var(--wac) / 0.9)}
.hover\:border-b-warning-content\/95:hover {
    border-bottom-color: hsl(var(--wac) / 0.95)}
.hover\:border-b-warning\/0:hover {
    border-bottom-color: hsl(var(--wa) / 0)}
.hover\:border-b-warning\/10:hover {
    border-bottom-color: hsl(var(--wa) / 0.1)}
.hover\:border-b-warning\/100:hover {
    border-bottom-color: hsl(var(--wa) / 1)}
.hover\:border-b-warning\/20:hover {
    border-bottom-color: hsl(var(--wa) / 0.2)}
.hover\:border-b-warning\/25:hover {
    border-bottom-color: hsl(var(--wa) / 0.25)}
.hover\:border-b-warning\/30:hover {
    border-bottom-color: hsl(var(--wa) / 0.3)}
.hover\:border-b-warning\/40:hover {
    border-bottom-color: hsl(var(--wa) / 0.4)}
.hover\:border-b-warning\/5:hover {
    border-bottom-color: hsl(var(--wa) / 0.05)}
.hover\:border-b-warning\/50:hover {
    border-bottom-color: hsl(var(--wa) / 0.5)}
.hover\:border-b-warning\/60:hover {
    border-bottom-color: hsl(var(--wa) / 0.6)}
.hover\:border-b-warning\/70:hover {
    border-bottom-color: hsl(var(--wa) / 0.7)}
.hover\:border-b-warning\/75:hover {
    border-bottom-color: hsl(var(--wa) / 0.75)}
.hover\:border-b-warning\/80:hover {
    border-bottom-color: hsl(var(--wa) / 0.8)}
.hover\:border-b-warning\/90:hover {
    border-bottom-color: hsl(var(--wa) / 0.9)}
.hover\:border-b-warning\/95:hover {
    border-bottom-color: hsl(var(--wa) / 0.95)}
.hover\:border-e-base-100:hover {
    border-inline-end-color: hsl(var(--b1) / 1)}
.hover\:border-e-base-100\/0:hover {
    border-inline-end-color: hsl(var(--b1) / 0)}
.hover\:border-e-base-100\/10:hover {
    border-inline-end-color: hsl(var(--b1) / 0.1)}
.hover\:border-e-base-100\/100:hover {
    border-inline-end-color: hsl(var(--b1) / 1)}
.hover\:border-e-base-100\/20:hover {
    border-inline-end-color: hsl(var(--b1) / 0.2)}
.hover\:border-e-base-100\/25:hover {
    border-inline-end-color: hsl(var(--b1) / 0.25)}
.hover\:border-e-base-100\/30:hover {
    border-inline-end-color: hsl(var(--b1) / 0.3)}
.hover\:border-e-base-100\/40:hover {
    border-inline-end-color: hsl(var(--b1) / 0.4)}
.hover\:border-e-base-100\/5:hover {
    border-inline-end-color: hsl(var(--b1) / 0.05)}
.hover\:border-e-base-100\/50:hover {
    border-inline-end-color: hsl(var(--b1) / 0.5)}
.hover\:border-e-base-100\/60:hover {
    border-inline-end-color: hsl(var(--b1) / 0.6)}
.hover\:border-e-base-100\/70:hover {
    border-inline-end-color: hsl(var(--b1) / 0.7)}
.hover\:border-e-base-100\/75:hover {
    border-inline-end-color: hsl(var(--b1) / 0.75)}
.hover\:border-e-base-100\/80:hover {
    border-inline-end-color: hsl(var(--b1) / 0.8)}
.hover\:border-e-base-100\/90:hover {
    border-inline-end-color: hsl(var(--b1) / 0.9)}
.hover\:border-e-base-100\/95:hover {
    border-inline-end-color: hsl(var(--b1) / 0.95)}
.hover\:border-e-base-200:hover {
    border-inline-end-color: hsl(var(--b2) / 1)}
.hover\:border-e-base-200\/0:hover {
    border-inline-end-color: hsl(var(--b2) / 0)}
.hover\:border-e-base-200\/10:hover {
    border-inline-end-color: hsl(var(--b2) / 0.1)}
.hover\:border-e-base-200\/100:hover {
    border-inline-end-color: hsl(var(--b2) / 1)}
.hover\:border-e-base-200\/20:hover {
    border-inline-end-color: hsl(var(--b2) / 0.2)}
.hover\:border-e-base-200\/25:hover {
    border-inline-end-color: hsl(var(--b2) / 0.25)}
.hover\:border-e-base-200\/30:hover {
    border-inline-end-color: hsl(var(--b2) / 0.3)}
.hover\:border-e-base-200\/40:hover {
    border-inline-end-color: hsl(var(--b2) / 0.4)}
.hover\:border-e-base-200\/5:hover {
    border-inline-end-color: hsl(var(--b2) / 0.05)}
.hover\:border-e-base-200\/50:hover {
    border-inline-end-color: hsl(var(--b2) / 0.5)}
.hover\:border-e-base-200\/60:hover {
    border-inline-end-color: hsl(var(--b2) / 0.6)}
.hover\:border-e-base-200\/70:hover {
    border-inline-end-color: hsl(var(--b2) / 0.7)}
.hover\:border-e-base-200\/75:hover {
    border-inline-end-color: hsl(var(--b2) / 0.75)}
.hover\:border-e-base-200\/80:hover {
    border-inline-end-color: hsl(var(--b2) / 0.8)}
.hover\:border-e-base-200\/90:hover {
    border-inline-end-color: hsl(var(--b2) / 0.9)}
.hover\:border-e-base-200\/95:hover {
    border-inline-end-color: hsl(var(--b2) / 0.95)}
.hover\:border-e-base-300:hover {
    border-inline-end-color: hsl(var(--b3) / 1)}
.hover\:border-e-base-300\/0:hover {
    border-inline-end-color: hsl(var(--b3) / 0)}
.hover\:border-e-base-300\/10:hover {
    border-inline-end-color: hsl(var(--b3) / 0.1)}
.hover\:border-e-base-300\/100:hover {
    border-inline-end-color: hsl(var(--b3) / 1)}
.hover\:border-e-base-300\/20:hover {
    border-inline-end-color: hsl(var(--b3) / 0.2)}
.hover\:border-e-base-300\/25:hover {
    border-inline-end-color: hsl(var(--b3) / 0.25)}
.hover\:border-e-base-300\/30:hover {
    border-inline-end-color: hsl(var(--b3) / 0.3)}
.hover\:border-e-base-300\/40:hover {
    border-inline-end-color: hsl(var(--b3) / 0.4)}
.hover\:border-e-base-300\/5:hover {
    border-inline-end-color: hsl(var(--b3) / 0.05)}
.hover\:border-e-base-300\/50:hover {
    border-inline-end-color: hsl(var(--b3) / 0.5)}
.hover\:border-e-base-300\/60:hover {
    border-inline-end-color: hsl(var(--b3) / 0.6)}
.hover\:border-e-base-300\/70:hover {
    border-inline-end-color: hsl(var(--b3) / 0.7)}
.hover\:border-e-base-300\/75:hover {
    border-inline-end-color: hsl(var(--b3) / 0.75)}
.hover\:border-e-base-300\/80:hover {
    border-inline-end-color: hsl(var(--b3) / 0.8)}
.hover\:border-e-base-300\/90:hover {
    border-inline-end-color: hsl(var(--b3) / 0.9)}
.hover\:border-e-base-300\/95:hover {
    border-inline-end-color: hsl(var(--b3) / 0.95)}
.hover\:border-e-base-content:hover {
    border-inline-end-color: hsl(var(--bc) / 1)}
.hover\:border-e-base-content\/0:hover {
    border-inline-end-color: hsl(var(--bc) / 0)}
.hover\:border-e-base-content\/10:hover {
    border-inline-end-color: hsl(var(--bc) / 0.1)}
.hover\:border-e-base-content\/100:hover {
    border-inline-end-color: hsl(var(--bc) / 1)}
.hover\:border-e-base-content\/20:hover {
    border-inline-end-color: hsl(var(--bc) / 0.2)}
.hover\:border-e-base-content\/25:hover {
    border-inline-end-color: hsl(var(--bc) / 0.25)}
.hover\:border-e-base-content\/30:hover {
    border-inline-end-color: hsl(var(--bc) / 0.3)}
.hover\:border-e-base-content\/40:hover {
    border-inline-end-color: hsl(var(--bc) / 0.4)}
.hover\:border-e-base-content\/5:hover {
    border-inline-end-color: hsl(var(--bc) / 0.05)}
.hover\:border-e-base-content\/50:hover {
    border-inline-end-color: hsl(var(--bc) / 0.5)}
.hover\:border-e-base-content\/60:hover {
    border-inline-end-color: hsl(var(--bc) / 0.6)}
.hover\:border-e-base-content\/70:hover {
    border-inline-end-color: hsl(var(--bc) / 0.7)}
.hover\:border-e-base-content\/75:hover {
    border-inline-end-color: hsl(var(--bc) / 0.75)}
.hover\:border-e-base-content\/80:hover {
    border-inline-end-color: hsl(var(--bc) / 0.8)}
.hover\:border-e-base-content\/90:hover {
    border-inline-end-color: hsl(var(--bc) / 0.9)}
.hover\:border-e-base-content\/95:hover {
    border-inline-end-color: hsl(var(--bc) / 0.95)}
.hover\:border-e-error:hover {
    border-inline-end-color: hsl(var(--er) / 1)}
.hover\:border-e-error-content:hover {
    border-inline-end-color: hsl(var(--erc) / 1)}
.hover\:border-e-error-content\/0:hover {
    border-inline-end-color: hsl(var(--erc) / 0)}
.hover\:border-e-error-content\/10:hover {
    border-inline-end-color: hsl(var(--erc) / 0.1)}
.hover\:border-e-error-content\/100:hover {
    border-inline-end-color: hsl(var(--erc) / 1)}
.hover\:border-e-error-content\/20:hover {
    border-inline-end-color: hsl(var(--erc) / 0.2)}
.hover\:border-e-error-content\/25:hover {
    border-inline-end-color: hsl(var(--erc) / 0.25)}
.hover\:border-e-error-content\/30:hover {
    border-inline-end-color: hsl(var(--erc) / 0.3)}
.hover\:border-e-error-content\/40:hover {
    border-inline-end-color: hsl(var(--erc) / 0.4)}
.hover\:border-e-error-content\/5:hover {
    border-inline-end-color: hsl(var(--erc) / 0.05)}
.hover\:border-e-error-content\/50:hover {
    border-inline-end-color: hsl(var(--erc) / 0.5)}
.hover\:border-e-error-content\/60:hover {
    border-inline-end-color: hsl(var(--erc) / 0.6)}
.hover\:border-e-error-content\/70:hover {
    border-inline-end-color: hsl(var(--erc) / 0.7)}
.hover\:border-e-error-content\/75:hover {
    border-inline-end-color: hsl(var(--erc) / 0.75)}
.hover\:border-e-error-content\/80:hover {
    border-inline-end-color: hsl(var(--erc) / 0.8)}
.hover\:border-e-error-content\/90:hover {
    border-inline-end-color: hsl(var(--erc) / 0.9)}
.hover\:border-e-error-content\/95:hover {
    border-inline-end-color: hsl(var(--erc) / 0.95)}
.hover\:border-e-error\/0:hover {
    border-inline-end-color: hsl(var(--er) / 0)}
.hover\:border-e-error\/10:hover {
    border-inline-end-color: hsl(var(--er) / 0.1)}
.hover\:border-e-error\/100:hover {
    border-inline-end-color: hsl(var(--er) / 1)}
.hover\:border-e-error\/20:hover {
    border-inline-end-color: hsl(var(--er) / 0.2)}
.hover\:border-e-error\/25:hover {
    border-inline-end-color: hsl(var(--er) / 0.25)}
.hover\:border-e-error\/30:hover {
    border-inline-end-color: hsl(var(--er) / 0.3)}
.hover\:border-e-error\/40:hover {
    border-inline-end-color: hsl(var(--er) / 0.4)}
.hover\:border-e-error\/5:hover {
    border-inline-end-color: hsl(var(--er) / 0.05)}
.hover\:border-e-error\/50:hover {
    border-inline-end-color: hsl(var(--er) / 0.5)}
.hover\:border-e-error\/60:hover {
    border-inline-end-color: hsl(var(--er) / 0.6)}
.hover\:border-e-error\/70:hover {
    border-inline-end-color: hsl(var(--er) / 0.7)}
.hover\:border-e-error\/75:hover {
    border-inline-end-color: hsl(var(--er) / 0.75)}
.hover\:border-e-error\/80:hover {
    border-inline-end-color: hsl(var(--er) / 0.8)}
.hover\:border-e-error\/90:hover {
    border-inline-end-color: hsl(var(--er) / 0.9)}
.hover\:border-e-error\/95:hover {
    border-inline-end-color: hsl(var(--er) / 0.95)}
.hover\:border-e-info:hover {
    border-inline-end-color: hsl(var(--in) / 1)}
.hover\:border-e-info-content:hover {
    border-inline-end-color: hsl(var(--inc) / 1)}
.hover\:border-e-info-content\/0:hover {
    border-inline-end-color: hsl(var(--inc) / 0)}
.hover\:border-e-info-content\/10:hover {
    border-inline-end-color: hsl(var(--inc) / 0.1)}
.hover\:border-e-info-content\/100:hover {
    border-inline-end-color: hsl(var(--inc) / 1)}
.hover\:border-e-info-content\/20:hover {
    border-inline-end-color: hsl(var(--inc) / 0.2)}
.hover\:border-e-info-content\/25:hover {
    border-inline-end-color: hsl(var(--inc) / 0.25)}
.hover\:border-e-info-content\/30:hover {
    border-inline-end-color: hsl(var(--inc) / 0.3)}
.hover\:border-e-info-content\/40:hover {
    border-inline-end-color: hsl(var(--inc) / 0.4)}
.hover\:border-e-info-content\/5:hover {
    border-inline-end-color: hsl(var(--inc) / 0.05)}
.hover\:border-e-info-content\/50:hover {
    border-inline-end-color: hsl(var(--inc) / 0.5)}
.hover\:border-e-info-content\/60:hover {
    border-inline-end-color: hsl(var(--inc) / 0.6)}
.hover\:border-e-info-content\/70:hover {
    border-inline-end-color: hsl(var(--inc) / 0.7)}
.hover\:border-e-info-content\/75:hover {
    border-inline-end-color: hsl(var(--inc) / 0.75)}
.hover\:border-e-info-content\/80:hover {
    border-inline-end-color: hsl(var(--inc) / 0.8)}
.hover\:border-e-info-content\/90:hover {
    border-inline-end-color: hsl(var(--inc) / 0.9)}
.hover\:border-e-info-content\/95:hover {
    border-inline-end-color: hsl(var(--inc) / 0.95)}
.hover\:border-e-info\/0:hover {
    border-inline-end-color: hsl(var(--in) / 0)}
.hover\:border-e-info\/10:hover {
    border-inline-end-color: hsl(var(--in) / 0.1)}
.hover\:border-e-info\/100:hover {
    border-inline-end-color: hsl(var(--in) / 1)}
.hover\:border-e-info\/20:hover {
    border-inline-end-color: hsl(var(--in) / 0.2)}
.hover\:border-e-info\/25:hover {
    border-inline-end-color: hsl(var(--in) / 0.25)}
.hover\:border-e-info\/30:hover {
    border-inline-end-color: hsl(var(--in) / 0.3)}
.hover\:border-e-info\/40:hover {
    border-inline-end-color: hsl(var(--in) / 0.4)}
.hover\:border-e-info\/5:hover {
    border-inline-end-color: hsl(var(--in) / 0.05)}
.hover\:border-e-info\/50:hover {
    border-inline-end-color: hsl(var(--in) / 0.5)}
.hover\:border-e-info\/60:hover {
    border-inline-end-color: hsl(var(--in) / 0.6)}
.hover\:border-e-info\/70:hover {
    border-inline-end-color: hsl(var(--in) / 0.7)}
.hover\:border-e-info\/75:hover {
    border-inline-end-color: hsl(var(--in) / 0.75)}
.hover\:border-e-info\/80:hover {
    border-inline-end-color: hsl(var(--in) / 0.8)}
.hover\:border-e-info\/90:hover {
    border-inline-end-color: hsl(var(--in) / 0.9)}
.hover\:border-e-info\/95:hover {
    border-inline-end-color: hsl(var(--in) / 0.95)}
.hover\:border-e-success:hover {
    border-inline-end-color: hsl(var(--su) / 1)}
.hover\:border-e-success-content:hover {
    border-inline-end-color: hsl(var(--suc) / 1)}
.hover\:border-e-success-content\/0:hover {
    border-inline-end-color: hsl(var(--suc) / 0)}
.hover\:border-e-success-content\/10:hover {
    border-inline-end-color: hsl(var(--suc) / 0.1)}
.hover\:border-e-success-content\/100:hover {
    border-inline-end-color: hsl(var(--suc) / 1)}
.hover\:border-e-success-content\/20:hover {
    border-inline-end-color: hsl(var(--suc) / 0.2)}
.hover\:border-e-success-content\/25:hover {
    border-inline-end-color: hsl(var(--suc) / 0.25)}
.hover\:border-e-success-content\/30:hover {
    border-inline-end-color: hsl(var(--suc) / 0.3)}
.hover\:border-e-success-content\/40:hover {
    border-inline-end-color: hsl(var(--suc) / 0.4)}
.hover\:border-e-success-content\/5:hover {
    border-inline-end-color: hsl(var(--suc) / 0.05)}
.hover\:border-e-success-content\/50:hover {
    border-inline-end-color: hsl(var(--suc) / 0.5)}
.hover\:border-e-success-content\/60:hover {
    border-inline-end-color: hsl(var(--suc) / 0.6)}
.hover\:border-e-success-content\/70:hover {
    border-inline-end-color: hsl(var(--suc) / 0.7)}
.hover\:border-e-success-content\/75:hover {
    border-inline-end-color: hsl(var(--suc) / 0.75)}
.hover\:border-e-success-content\/80:hover {
    border-inline-end-color: hsl(var(--suc) / 0.8)}
.hover\:border-e-success-content\/90:hover {
    border-inline-end-color: hsl(var(--suc) / 0.9)}
.hover\:border-e-success-content\/95:hover {
    border-inline-end-color: hsl(var(--suc) / 0.95)}
.hover\:border-e-success\/0:hover {
    border-inline-end-color: hsl(var(--su) / 0)}
.hover\:border-e-success\/10:hover {
    border-inline-end-color: hsl(var(--su) / 0.1)}
.hover\:border-e-success\/100:hover {
    border-inline-end-color: hsl(var(--su) / 1)}
.hover\:border-e-success\/20:hover {
    border-inline-end-color: hsl(var(--su) / 0.2)}
.hover\:border-e-success\/25:hover {
    border-inline-end-color: hsl(var(--su) / 0.25)}
.hover\:border-e-success\/30:hover {
    border-inline-end-color: hsl(var(--su) / 0.3)}
.hover\:border-e-success\/40:hover {
    border-inline-end-color: hsl(var(--su) / 0.4)}
.hover\:border-e-success\/5:hover {
    border-inline-end-color: hsl(var(--su) / 0.05)}
.hover\:border-e-success\/50:hover {
    border-inline-end-color: hsl(var(--su) / 0.5)}
.hover\:border-e-success\/60:hover {
    border-inline-end-color: hsl(var(--su) / 0.6)}
.hover\:border-e-success\/70:hover {
    border-inline-end-color: hsl(var(--su) / 0.7)}
.hover\:border-e-success\/75:hover {
    border-inline-end-color: hsl(var(--su) / 0.75)}
.hover\:border-e-success\/80:hover {
    border-inline-end-color: hsl(var(--su) / 0.8)}
.hover\:border-e-success\/90:hover {
    border-inline-end-color: hsl(var(--su) / 0.9)}
.hover\:border-e-success\/95:hover {
    border-inline-end-color: hsl(var(--su) / 0.95)}
.hover\:border-e-warning:hover {
    border-inline-end-color: hsl(var(--wa) / 1)}
.hover\:border-e-warning-content:hover {
    border-inline-end-color: hsl(var(--wac) / 1)}
.hover\:border-e-warning-content\/0:hover {
    border-inline-end-color: hsl(var(--wac) / 0)}
.hover\:border-e-warning-content\/10:hover {
    border-inline-end-color: hsl(var(--wac) / 0.1)}
.hover\:border-e-warning-content\/100:hover {
    border-inline-end-color: hsl(var(--wac) / 1)}
.hover\:border-e-warning-content\/20:hover {
    border-inline-end-color: hsl(var(--wac) / 0.2)}
.hover\:border-e-warning-content\/25:hover {
    border-inline-end-color: hsl(var(--wac) / 0.25)}
.hover\:border-e-warning-content\/30:hover {
    border-inline-end-color: hsl(var(--wac) / 0.3)}
.hover\:border-e-warning-content\/40:hover {
    border-inline-end-color: hsl(var(--wac) / 0.4)}
.hover\:border-e-warning-content\/5:hover {
    border-inline-end-color: hsl(var(--wac) / 0.05)}
.hover\:border-e-warning-content\/50:hover {
    border-inline-end-color: hsl(var(--wac) / 0.5)}
.hover\:border-e-warning-content\/60:hover {
    border-inline-end-color: hsl(var(--wac) / 0.6)}
.hover\:border-e-warning-content\/70:hover {
    border-inline-end-color: hsl(var(--wac) / 0.7)}
.hover\:border-e-warning-content\/75:hover {
    border-inline-end-color: hsl(var(--wac) / 0.75)}
.hover\:border-e-warning-content\/80:hover {
    border-inline-end-color: hsl(var(--wac) / 0.8)}
.hover\:border-e-warning-content\/90:hover {
    border-inline-end-color: hsl(var(--wac) / 0.9)}
.hover\:border-e-warning-content\/95:hover {
    border-inline-end-color: hsl(var(--wac) / 0.95)}
.hover\:border-e-warning\/0:hover {
    border-inline-end-color: hsl(var(--wa) / 0)}
.hover\:border-e-warning\/10:hover {
    border-inline-end-color: hsl(var(--wa) / 0.1)}
.hover\:border-e-warning\/100:hover {
    border-inline-end-color: hsl(var(--wa) / 1)}
.hover\:border-e-warning\/20:hover {
    border-inline-end-color: hsl(var(--wa) / 0.2)}
.hover\:border-e-warning\/25:hover {
    border-inline-end-color: hsl(var(--wa) / 0.25)}
.hover\:border-e-warning\/30:hover {
    border-inline-end-color: hsl(var(--wa) / 0.3)}
.hover\:border-e-warning\/40:hover {
    border-inline-end-color: hsl(var(--wa) / 0.4)}
.hover\:border-e-warning\/5:hover {
    border-inline-end-color: hsl(var(--wa) / 0.05)}
.hover\:border-e-warning\/50:hover {
    border-inline-end-color: hsl(var(--wa) / 0.5)}
.hover\:border-e-warning\/60:hover {
    border-inline-end-color: hsl(var(--wa) / 0.6)}
.hover\:border-e-warning\/70:hover {
    border-inline-end-color: hsl(var(--wa) / 0.7)}
.hover\:border-e-warning\/75:hover {
    border-inline-end-color: hsl(var(--wa) / 0.75)}
.hover\:border-e-warning\/80:hover {
    border-inline-end-color: hsl(var(--wa) / 0.8)}
.hover\:border-e-warning\/90:hover {
    border-inline-end-color: hsl(var(--wa) / 0.9)}
.hover\:border-e-warning\/95:hover {
    border-inline-end-color: hsl(var(--wa) / 0.95)}
.hover\:border-l-base-100:hover {
    border-left-color: hsl(var(--b1) / 1)}
.hover\:border-l-base-100\/0:hover {
    border-left-color: hsl(var(--b1) / 0)}
.hover\:border-l-base-100\/10:hover {
    border-left-color: hsl(var(--b1) / 0.1)}
.hover\:border-l-base-100\/100:hover {
    border-left-color: hsl(var(--b1) / 1)}
.hover\:border-l-base-100\/20:hover {
    border-left-color: hsl(var(--b1) / 0.2)}
.hover\:border-l-base-100\/25:hover {
    border-left-color: hsl(var(--b1) / 0.25)}
.hover\:border-l-base-100\/30:hover {
    border-left-color: hsl(var(--b1) / 0.3)}
.hover\:border-l-base-100\/40:hover {
    border-left-color: hsl(var(--b1) / 0.4)}
.hover\:border-l-base-100\/5:hover {
    border-left-color: hsl(var(--b1) / 0.05)}
.hover\:border-l-base-100\/50:hover {
    border-left-color: hsl(var(--b1) / 0.5)}
.hover\:border-l-base-100\/60:hover {
    border-left-color: hsl(var(--b1) / 0.6)}
.hover\:border-l-base-100\/70:hover {
    border-left-color: hsl(var(--b1) / 0.7)}
.hover\:border-l-base-100\/75:hover {
    border-left-color: hsl(var(--b1) / 0.75)}
.hover\:border-l-base-100\/80:hover {
    border-left-color: hsl(var(--b1) / 0.8)}
.hover\:border-l-base-100\/90:hover {
    border-left-color: hsl(var(--b1) / 0.9)}
.hover\:border-l-base-100\/95:hover {
    border-left-color: hsl(var(--b1) / 0.95)}
.hover\:border-l-base-200:hover {
    border-left-color: hsl(var(--b2) / 1)}
.hover\:border-l-base-200\/0:hover {
    border-left-color: hsl(var(--b2) / 0)}
.hover\:border-l-base-200\/10:hover {
    border-left-color: hsl(var(--b2) / 0.1)}
.hover\:border-l-base-200\/100:hover {
    border-left-color: hsl(var(--b2) / 1)}
.hover\:border-l-base-200\/20:hover {
    border-left-color: hsl(var(--b2) / 0.2)}
.hover\:border-l-base-200\/25:hover {
    border-left-color: hsl(var(--b2) / 0.25)}
.hover\:border-l-base-200\/30:hover {
    border-left-color: hsl(var(--b2) / 0.3)}
.hover\:border-l-base-200\/40:hover {
    border-left-color: hsl(var(--b2) / 0.4)}
.hover\:border-l-base-200\/5:hover {
    border-left-color: hsl(var(--b2) / 0.05)}
.hover\:border-l-base-200\/50:hover {
    border-left-color: hsl(var(--b2) / 0.5)}
.hover\:border-l-base-200\/60:hover {
    border-left-color: hsl(var(--b2) / 0.6)}
.hover\:border-l-base-200\/70:hover {
    border-left-color: hsl(var(--b2) / 0.7)}
.hover\:border-l-base-200\/75:hover {
    border-left-color: hsl(var(--b2) / 0.75)}
.hover\:border-l-base-200\/80:hover {
    border-left-color: hsl(var(--b2) / 0.8)}
.hover\:border-l-base-200\/90:hover {
    border-left-color: hsl(var(--b2) / 0.9)}
.hover\:border-l-base-200\/95:hover {
    border-left-color: hsl(var(--b2) / 0.95)}
.hover\:border-l-base-300:hover {
    border-left-color: hsl(var(--b3) / 1)}
.hover\:border-l-base-300\/0:hover {
    border-left-color: hsl(var(--b3) / 0)}
.hover\:border-l-base-300\/10:hover {
    border-left-color: hsl(var(--b3) / 0.1)}
.hover\:border-l-base-300\/100:hover {
    border-left-color: hsl(var(--b3) / 1)}
.hover\:border-l-base-300\/20:hover {
    border-left-color: hsl(var(--b3) / 0.2)}
.hover\:border-l-base-300\/25:hover {
    border-left-color: hsl(var(--b3) / 0.25)}
.hover\:border-l-base-300\/30:hover {
    border-left-color: hsl(var(--b3) / 0.3)}
.hover\:border-l-base-300\/40:hover {
    border-left-color: hsl(var(--b3) / 0.4)}
.hover\:border-l-base-300\/5:hover {
    border-left-color: hsl(var(--b3) / 0.05)}
.hover\:border-l-base-300\/50:hover {
    border-left-color: hsl(var(--b3) / 0.5)}
.hover\:border-l-base-300\/60:hover {
    border-left-color: hsl(var(--b3) / 0.6)}
.hover\:border-l-base-300\/70:hover {
    border-left-color: hsl(var(--b3) / 0.7)}
.hover\:border-l-base-300\/75:hover {
    border-left-color: hsl(var(--b3) / 0.75)}
.hover\:border-l-base-300\/80:hover {
    border-left-color: hsl(var(--b3) / 0.8)}
.hover\:border-l-base-300\/90:hover {
    border-left-color: hsl(var(--b3) / 0.9)}
.hover\:border-l-base-300\/95:hover {
    border-left-color: hsl(var(--b3) / 0.95)}
.hover\:border-l-base-content:hover {
    border-left-color: hsl(var(--bc) / 1)}
.hover\:border-l-base-content\/0:hover {
    border-left-color: hsl(var(--bc) / 0)}
.hover\:border-l-base-content\/10:hover {
    border-left-color: hsl(var(--bc) / 0.1)}
.hover\:border-l-base-content\/100:hover {
    border-left-color: hsl(var(--bc) / 1)}
.hover\:border-l-base-content\/20:hover {
    border-left-color: hsl(var(--bc) / 0.2)}
.hover\:border-l-base-content\/25:hover {
    border-left-color: hsl(var(--bc) / 0.25)}
.hover\:border-l-base-content\/30:hover {
    border-left-color: hsl(var(--bc) / 0.3)}
.hover\:border-l-base-content\/40:hover {
    border-left-color: hsl(var(--bc) / 0.4)}
.hover\:border-l-base-content\/5:hover {
    border-left-color: hsl(var(--bc) / 0.05)}
.hover\:border-l-base-content\/50:hover {
    border-left-color: hsl(var(--bc) / 0.5)}
.hover\:border-l-base-content\/60:hover {
    border-left-color: hsl(var(--bc) / 0.6)}
.hover\:border-l-base-content\/70:hover {
    border-left-color: hsl(var(--bc) / 0.7)}
.hover\:border-l-base-content\/75:hover {
    border-left-color: hsl(var(--bc) / 0.75)}
.hover\:border-l-base-content\/80:hover {
    border-left-color: hsl(var(--bc) / 0.8)}
.hover\:border-l-base-content\/90:hover {
    border-left-color: hsl(var(--bc) / 0.9)}
.hover\:border-l-base-content\/95:hover {
    border-left-color: hsl(var(--bc) / 0.95)}
.hover\:border-l-error:hover {
    border-left-color: hsl(var(--er) / 1)}
.hover\:border-l-error-content:hover {
    border-left-color: hsl(var(--erc) / 1)}
.hover\:border-l-error-content\/0:hover {
    border-left-color: hsl(var(--erc) / 0)}
.hover\:border-l-error-content\/10:hover {
    border-left-color: hsl(var(--erc) / 0.1)}
.hover\:border-l-error-content\/100:hover {
    border-left-color: hsl(var(--erc) / 1)}
.hover\:border-l-error-content\/20:hover {
    border-left-color: hsl(var(--erc) / 0.2)}
.hover\:border-l-error-content\/25:hover {
    border-left-color: hsl(var(--erc) / 0.25)}
.hover\:border-l-error-content\/30:hover {
    border-left-color: hsl(var(--erc) / 0.3)}
.hover\:border-l-error-content\/40:hover {
    border-left-color: hsl(var(--erc) / 0.4)}
.hover\:border-l-error-content\/5:hover {
    border-left-color: hsl(var(--erc) / 0.05)}
.hover\:border-l-error-content\/50:hover {
    border-left-color: hsl(var(--erc) / 0.5)}
.hover\:border-l-error-content\/60:hover {
    border-left-color: hsl(var(--erc) / 0.6)}
.hover\:border-l-error-content\/70:hover {
    border-left-color: hsl(var(--erc) / 0.7)}
.hover\:border-l-error-content\/75:hover {
    border-left-color: hsl(var(--erc) / 0.75)}
.hover\:border-l-error-content\/80:hover {
    border-left-color: hsl(var(--erc) / 0.8)}
.hover\:border-l-error-content\/90:hover {
    border-left-color: hsl(var(--erc) / 0.9)}
.hover\:border-l-error-content\/95:hover {
    border-left-color: hsl(var(--erc) / 0.95)}
.hover\:border-l-error\/0:hover {
    border-left-color: hsl(var(--er) / 0)}
.hover\:border-l-error\/10:hover {
    border-left-color: hsl(var(--er) / 0.1)}
.hover\:border-l-error\/100:hover {
    border-left-color: hsl(var(--er) / 1)}
.hover\:border-l-error\/20:hover {
    border-left-color: hsl(var(--er) / 0.2)}
.hover\:border-l-error\/25:hover {
    border-left-color: hsl(var(--er) / 0.25)}
.hover\:border-l-error\/30:hover {
    border-left-color: hsl(var(--er) / 0.3)}
.hover\:border-l-error\/40:hover {
    border-left-color: hsl(var(--er) / 0.4)}
.hover\:border-l-error\/5:hover {
    border-left-color: hsl(var(--er) / 0.05)}
.hover\:border-l-error\/50:hover {
    border-left-color: hsl(var(--er) / 0.5)}
.hover\:border-l-error\/60:hover {
    border-left-color: hsl(var(--er) / 0.6)}
.hover\:border-l-error\/70:hover {
    border-left-color: hsl(var(--er) / 0.7)}
.hover\:border-l-error\/75:hover {
    border-left-color: hsl(var(--er) / 0.75)}
.hover\:border-l-error\/80:hover {
    border-left-color: hsl(var(--er) / 0.8)}
.hover\:border-l-error\/90:hover {
    border-left-color: hsl(var(--er) / 0.9)}
.hover\:border-l-error\/95:hover {
    border-left-color: hsl(var(--er) / 0.95)}
.hover\:border-l-info:hover {
    border-left-color: hsl(var(--in) / 1)}
.hover\:border-l-info-content:hover {
    border-left-color: hsl(var(--inc) / 1)}
.hover\:border-l-info-content\/0:hover {
    border-left-color: hsl(var(--inc) / 0)}
.hover\:border-l-info-content\/10:hover {
    border-left-color: hsl(var(--inc) / 0.1)}
.hover\:border-l-info-content\/100:hover {
    border-left-color: hsl(var(--inc) / 1)}
.hover\:border-l-info-content\/20:hover {
    border-left-color: hsl(var(--inc) / 0.2)}
.hover\:border-l-info-content\/25:hover {
    border-left-color: hsl(var(--inc) / 0.25)}
.hover\:border-l-info-content\/30:hover {
    border-left-color: hsl(var(--inc) / 0.3)}
.hover\:border-l-info-content\/40:hover {
    border-left-color: hsl(var(--inc) / 0.4)}
.hover\:border-l-info-content\/5:hover {
    border-left-color: hsl(var(--inc) / 0.05)}
.hover\:border-l-info-content\/50:hover {
    border-left-color: hsl(var(--inc) / 0.5)}
.hover\:border-l-info-content\/60:hover {
    border-left-color: hsl(var(--inc) / 0.6)}
.hover\:border-l-info-content\/70:hover {
    border-left-color: hsl(var(--inc) / 0.7)}
.hover\:border-l-info-content\/75:hover {
    border-left-color: hsl(var(--inc) / 0.75)}
.hover\:border-l-info-content\/80:hover {
    border-left-color: hsl(var(--inc) / 0.8)}
.hover\:border-l-info-content\/90:hover {
    border-left-color: hsl(var(--inc) / 0.9)}
.hover\:border-l-info-content\/95:hover {
    border-left-color: hsl(var(--inc) / 0.95)}
.hover\:border-l-info\/0:hover {
    border-left-color: hsl(var(--in) / 0)}
.hover\:border-l-info\/10:hover {
    border-left-color: hsl(var(--in) / 0.1)}
.hover\:border-l-info\/100:hover {
    border-left-color: hsl(var(--in) / 1)}
.hover\:border-l-info\/20:hover {
    border-left-color: hsl(var(--in) / 0.2)}
.hover\:border-l-info\/25:hover {
    border-left-color: hsl(var(--in) / 0.25)}
.hover\:border-l-info\/30:hover {
    border-left-color: hsl(var(--in) / 0.3)}
.hover\:border-l-info\/40:hover {
    border-left-color: hsl(var(--in) / 0.4)}
.hover\:border-l-info\/5:hover {
    border-left-color: hsl(var(--in) / 0.05)}
.hover\:border-l-info\/50:hover {
    border-left-color: hsl(var(--in) / 0.5)}
.hover\:border-l-info\/60:hover {
    border-left-color: hsl(var(--in) / 0.6)}
.hover\:border-l-info\/70:hover {
    border-left-color: hsl(var(--in) / 0.7)}
.hover\:border-l-info\/75:hover {
    border-left-color: hsl(var(--in) / 0.75)}
.hover\:border-l-info\/80:hover {
    border-left-color: hsl(var(--in) / 0.8)}
.hover\:border-l-info\/90:hover {
    border-left-color: hsl(var(--in) / 0.9)}
.hover\:border-l-info\/95:hover {
    border-left-color: hsl(var(--in) / 0.95)}
.hover\:border-l-success:hover {
    border-left-color: hsl(var(--su) / 1)}
.hover\:border-l-success-content:hover {
    border-left-color: hsl(var(--suc) / 1)}
.hover\:border-l-success-content\/0:hover {
    border-left-color: hsl(var(--suc) / 0)}
.hover\:border-l-success-content\/10:hover {
    border-left-color: hsl(var(--suc) / 0.1)}
.hover\:border-l-success-content\/100:hover {
    border-left-color: hsl(var(--suc) / 1)}
.hover\:border-l-success-content\/20:hover {
    border-left-color: hsl(var(--suc) / 0.2)}
.hover\:border-l-success-content\/25:hover {
    border-left-color: hsl(var(--suc) / 0.25)}
.hover\:border-l-success-content\/30:hover {
    border-left-color: hsl(var(--suc) / 0.3)}
.hover\:border-l-success-content\/40:hover {
    border-left-color: hsl(var(--suc) / 0.4)}
.hover\:border-l-success-content\/5:hover {
    border-left-color: hsl(var(--suc) / 0.05)}
.hover\:border-l-success-content\/50:hover {
    border-left-color: hsl(var(--suc) / 0.5)}
.hover\:border-l-success-content\/60:hover {
    border-left-color: hsl(var(--suc) / 0.6)}
.hover\:border-l-success-content\/70:hover {
    border-left-color: hsl(var(--suc) / 0.7)}
.hover\:border-l-success-content\/75:hover {
    border-left-color: hsl(var(--suc) / 0.75)}
.hover\:border-l-success-content\/80:hover {
    border-left-color: hsl(var(--suc) / 0.8)}
.hover\:border-l-success-content\/90:hover {
    border-left-color: hsl(var(--suc) / 0.9)}
.hover\:border-l-success-content\/95:hover {
    border-left-color: hsl(var(--suc) / 0.95)}
.hover\:border-l-success\/0:hover {
    border-left-color: hsl(var(--su) / 0)}
.hover\:border-l-success\/10:hover {
    border-left-color: hsl(var(--su) / 0.1)}
.hover\:border-l-success\/100:hover {
    border-left-color: hsl(var(--su) / 1)}
.hover\:border-l-success\/20:hover {
    border-left-color: hsl(var(--su) / 0.2)}
.hover\:border-l-success\/25:hover {
    border-left-color: hsl(var(--su) / 0.25)}
.hover\:border-l-success\/30:hover {
    border-left-color: hsl(var(--su) / 0.3)}
.hover\:border-l-success\/40:hover {
    border-left-color: hsl(var(--su) / 0.4)}
.hover\:border-l-success\/5:hover {
    border-left-color: hsl(var(--su) / 0.05)}
.hover\:border-l-success\/50:hover {
    border-left-color: hsl(var(--su) / 0.5)}
.hover\:border-l-success\/60:hover {
    border-left-color: hsl(var(--su) / 0.6)}
.hover\:border-l-success\/70:hover {
    border-left-color: hsl(var(--su) / 0.7)}
.hover\:border-l-success\/75:hover {
    border-left-color: hsl(var(--su) / 0.75)}
.hover\:border-l-success\/80:hover {
    border-left-color: hsl(var(--su) / 0.8)}
.hover\:border-l-success\/90:hover {
    border-left-color: hsl(var(--su) / 0.9)}
.hover\:border-l-success\/95:hover {
    border-left-color: hsl(var(--su) / 0.95)}
.hover\:border-l-warning:hover {
    border-left-color: hsl(var(--wa) / 1)}
.hover\:border-l-warning-content:hover {
    border-left-color: hsl(var(--wac) / 1)}
.hover\:border-l-warning-content\/0:hover {
    border-left-color: hsl(var(--wac) / 0)}
.hover\:border-l-warning-content\/10:hover {
    border-left-color: hsl(var(--wac) / 0.1)}
.hover\:border-l-warning-content\/100:hover {
    border-left-color: hsl(var(--wac) / 1)}
.hover\:border-l-warning-content\/20:hover {
    border-left-color: hsl(var(--wac) / 0.2)}
.hover\:border-l-warning-content\/25:hover {
    border-left-color: hsl(var(--wac) / 0.25)}
.hover\:border-l-warning-content\/30:hover {
    border-left-color: hsl(var(--wac) / 0.3)}
.hover\:border-l-warning-content\/40:hover {
    border-left-color: hsl(var(--wac) / 0.4)}
.hover\:border-l-warning-content\/5:hover {
    border-left-color: hsl(var(--wac) / 0.05)}
.hover\:border-l-warning-content\/50:hover {
    border-left-color: hsl(var(--wac) / 0.5)}
.hover\:border-l-warning-content\/60:hover {
    border-left-color: hsl(var(--wac) / 0.6)}
.hover\:border-l-warning-content\/70:hover {
    border-left-color: hsl(var(--wac) / 0.7)}
.hover\:border-l-warning-content\/75:hover {
    border-left-color: hsl(var(--wac) / 0.75)}
.hover\:border-l-warning-content\/80:hover {
    border-left-color: hsl(var(--wac) / 0.8)}
.hover\:border-l-warning-content\/90:hover {
    border-left-color: hsl(var(--wac) / 0.9)}
.hover\:border-l-warning-content\/95:hover {
    border-left-color: hsl(var(--wac) / 0.95)}
.hover\:border-l-warning\/0:hover {
    border-left-color: hsl(var(--wa) / 0)}
.hover\:border-l-warning\/10:hover {
    border-left-color: hsl(var(--wa) / 0.1)}
.hover\:border-l-warning\/100:hover {
    border-left-color: hsl(var(--wa) / 1)}
.hover\:border-l-warning\/20:hover {
    border-left-color: hsl(var(--wa) / 0.2)}
.hover\:border-l-warning\/25:hover {
    border-left-color: hsl(var(--wa) / 0.25)}
.hover\:border-l-warning\/30:hover {
    border-left-color: hsl(var(--wa) / 0.3)}
.hover\:border-l-warning\/40:hover {
    border-left-color: hsl(var(--wa) / 0.4)}
.hover\:border-l-warning\/5:hover {
    border-left-color: hsl(var(--wa) / 0.05)}
.hover\:border-l-warning\/50:hover {
    border-left-color: hsl(var(--wa) / 0.5)}
.hover\:border-l-warning\/60:hover {
    border-left-color: hsl(var(--wa) / 0.6)}
.hover\:border-l-warning\/70:hover {
    border-left-color: hsl(var(--wa) / 0.7)}
.hover\:border-l-warning\/75:hover {
    border-left-color: hsl(var(--wa) / 0.75)}
.hover\:border-l-warning\/80:hover {
    border-left-color: hsl(var(--wa) / 0.8)}
.hover\:border-l-warning\/90:hover {
    border-left-color: hsl(var(--wa) / 0.9)}
.hover\:border-l-warning\/95:hover {
    border-left-color: hsl(var(--wa) / 0.95)}
.hover\:border-r-base-100:hover {
    border-right-color: hsl(var(--b1) / 1)}
.hover\:border-r-base-100\/0:hover {
    border-right-color: hsl(var(--b1) / 0)}
.hover\:border-r-base-100\/10:hover {
    border-right-color: hsl(var(--b1) / 0.1)}
.hover\:border-r-base-100\/100:hover {
    border-right-color: hsl(var(--b1) / 1)}
.hover\:border-r-base-100\/20:hover {
    border-right-color: hsl(var(--b1) / 0.2)}
.hover\:border-r-base-100\/25:hover {
    border-right-color: hsl(var(--b1) / 0.25)}
.hover\:border-r-base-100\/30:hover {
    border-right-color: hsl(var(--b1) / 0.3)}
.hover\:border-r-base-100\/40:hover {
    border-right-color: hsl(var(--b1) / 0.4)}
.hover\:border-r-base-100\/5:hover {
    border-right-color: hsl(var(--b1) / 0.05)}
.hover\:border-r-base-100\/50:hover {
    border-right-color: hsl(var(--b1) / 0.5)}
.hover\:border-r-base-100\/60:hover {
    border-right-color: hsl(var(--b1) / 0.6)}
.hover\:border-r-base-100\/70:hover {
    border-right-color: hsl(var(--b1) / 0.7)}
.hover\:border-r-base-100\/75:hover {
    border-right-color: hsl(var(--b1) / 0.75)}
.hover\:border-r-base-100\/80:hover {
    border-right-color: hsl(var(--b1) / 0.8)}
.hover\:border-r-base-100\/90:hover {
    border-right-color: hsl(var(--b1) / 0.9)}
.hover\:border-r-base-100\/95:hover {
    border-right-color: hsl(var(--b1) / 0.95)}
.hover\:border-r-base-200:hover {
    border-right-color: hsl(var(--b2) / 1)}
.hover\:border-r-base-200\/0:hover {
    border-right-color: hsl(var(--b2) / 0)}
.hover\:border-r-base-200\/10:hover {
    border-right-color: hsl(var(--b2) / 0.1)}
.hover\:border-r-base-200\/100:hover {
    border-right-color: hsl(var(--b2) / 1)}
.hover\:border-r-base-200\/20:hover {
    border-right-color: hsl(var(--b2) / 0.2)}
.hover\:border-r-base-200\/25:hover {
    border-right-color: hsl(var(--b2) / 0.25)}
.hover\:border-r-base-200\/30:hover {
    border-right-color: hsl(var(--b2) / 0.3)}
.hover\:border-r-base-200\/40:hover {
    border-right-color: hsl(var(--b2) / 0.4)}
.hover\:border-r-base-200\/5:hover {
    border-right-color: hsl(var(--b2) / 0.05)}
.hover\:border-r-base-200\/50:hover {
    border-right-color: hsl(var(--b2) / 0.5)}
.hover\:border-r-base-200\/60:hover {
    border-right-color: hsl(var(--b2) / 0.6)}
.hover\:border-r-base-200\/70:hover {
    border-right-color: hsl(var(--b2) / 0.7)}
.hover\:border-r-base-200\/75:hover {
    border-right-color: hsl(var(--b2) / 0.75)}
.hover\:border-r-base-200\/80:hover {
    border-right-color: hsl(var(--b2) / 0.8)}
.hover\:border-r-base-200\/90:hover {
    border-right-color: hsl(var(--b2) / 0.9)}
.hover\:border-r-base-200\/95:hover {
    border-right-color: hsl(var(--b2) / 0.95)}
.hover\:border-r-base-300:hover {
    border-right-color: hsl(var(--b3) / 1)}
.hover\:border-r-base-300\/0:hover {
    border-right-color: hsl(var(--b3) / 0)}
.hover\:border-r-base-300\/10:hover {
    border-right-color: hsl(var(--b3) / 0.1)}
.hover\:border-r-base-300\/100:hover {
    border-right-color: hsl(var(--b3) / 1)}
.hover\:border-r-base-300\/20:hover {
    border-right-color: hsl(var(--b3) / 0.2)}
.hover\:border-r-base-300\/25:hover {
    border-right-color: hsl(var(--b3) / 0.25)}
.hover\:border-r-base-300\/30:hover {
    border-right-color: hsl(var(--b3) / 0.3)}
.hover\:border-r-base-300\/40:hover {
    border-right-color: hsl(var(--b3) / 0.4)}
.hover\:border-r-base-300\/5:hover {
    border-right-color: hsl(var(--b3) / 0.05)}
.hover\:border-r-base-300\/50:hover {
    border-right-color: hsl(var(--b3) / 0.5)}
.hover\:border-r-base-300\/60:hover {
    border-right-color: hsl(var(--b3) / 0.6)}
.hover\:border-r-base-300\/70:hover {
    border-right-color: hsl(var(--b3) / 0.7)}
.hover\:border-r-base-300\/75:hover {
    border-right-color: hsl(var(--b3) / 0.75)}
.hover\:border-r-base-300\/80:hover {
    border-right-color: hsl(var(--b3) / 0.8)}
.hover\:border-r-base-300\/90:hover {
    border-right-color: hsl(var(--b3) / 0.9)}
.hover\:border-r-base-300\/95:hover {
    border-right-color: hsl(var(--b3) / 0.95)}
.hover\:border-r-base-content:hover {
    border-right-color: hsl(var(--bc) / 1)}
.hover\:border-r-base-content\/0:hover {
    border-right-color: hsl(var(--bc) / 0)}
.hover\:border-r-base-content\/10:hover {
    border-right-color: hsl(var(--bc) / 0.1)}
.hover\:border-r-base-content\/100:hover {
    border-right-color: hsl(var(--bc) / 1)}
.hover\:border-r-base-content\/20:hover {
    border-right-color: hsl(var(--bc) / 0.2)}
.hover\:border-r-base-content\/25:hover {
    border-right-color: hsl(var(--bc) / 0.25)}
.hover\:border-r-base-content\/30:hover {
    border-right-color: hsl(var(--bc) / 0.3)}
.hover\:border-r-base-content\/40:hover {
    border-right-color: hsl(var(--bc) / 0.4)}
.hover\:border-r-base-content\/5:hover {
    border-right-color: hsl(var(--bc) / 0.05)}
.hover\:border-r-base-content\/50:hover {
    border-right-color: hsl(var(--bc) / 0.5)}
.hover\:border-r-base-content\/60:hover {
    border-right-color: hsl(var(--bc) / 0.6)}
.hover\:border-r-base-content\/70:hover {
    border-right-color: hsl(var(--bc) / 0.7)}
.hover\:border-r-base-content\/75:hover {
    border-right-color: hsl(var(--bc) / 0.75)}
.hover\:border-r-base-content\/80:hover {
    border-right-color: hsl(var(--bc) / 0.8)}
.hover\:border-r-base-content\/90:hover {
    border-right-color: hsl(var(--bc) / 0.9)}
.hover\:border-r-base-content\/95:hover {
    border-right-color: hsl(var(--bc) / 0.95)}
.hover\:border-r-error:hover {
    border-right-color: hsl(var(--er) / 1)}
.hover\:border-r-error-content:hover {
    border-right-color: hsl(var(--erc) / 1)}
.hover\:border-r-error-content\/0:hover {
    border-right-color: hsl(var(--erc) / 0)}
.hover\:border-r-error-content\/10:hover {
    border-right-color: hsl(var(--erc) / 0.1)}
.hover\:border-r-error-content\/100:hover {
    border-right-color: hsl(var(--erc) / 1)}
.hover\:border-r-error-content\/20:hover {
    border-right-color: hsl(var(--erc) / 0.2)}
.hover\:border-r-error-content\/25:hover {
    border-right-color: hsl(var(--erc) / 0.25)}
.hover\:border-r-error-content\/30:hover {
    border-right-color: hsl(var(--erc) / 0.3)}
.hover\:border-r-error-content\/40:hover {
    border-right-color: hsl(var(--erc) / 0.4)}
.hover\:border-r-error-content\/5:hover {
    border-right-color: hsl(var(--erc) / 0.05)}
.hover\:border-r-error-content\/50:hover {
    border-right-color: hsl(var(--erc) / 0.5)}
.hover\:border-r-error-content\/60:hover {
    border-right-color: hsl(var(--erc) / 0.6)}
.hover\:border-r-error-content\/70:hover {
    border-right-color: hsl(var(--erc) / 0.7)}
.hover\:border-r-error-content\/75:hover {
    border-right-color: hsl(var(--erc) / 0.75)}
.hover\:border-r-error-content\/80:hover {
    border-right-color: hsl(var(--erc) / 0.8)}
.hover\:border-r-error-content\/90:hover {
    border-right-color: hsl(var(--erc) / 0.9)}
.hover\:border-r-error-content\/95:hover {
    border-right-color: hsl(var(--erc) / 0.95)}
.hover\:border-r-error\/0:hover {
    border-right-color: hsl(var(--er) / 0)}
.hover\:border-r-error\/10:hover {
    border-right-color: hsl(var(--er) / 0.1)}
.hover\:border-r-error\/100:hover {
    border-right-color: hsl(var(--er) / 1)}
.hover\:border-r-error\/20:hover {
    border-right-color: hsl(var(--er) / 0.2)}
.hover\:border-r-error\/25:hover {
    border-right-color: hsl(var(--er) / 0.25)}
.hover\:border-r-error\/30:hover {
    border-right-color: hsl(var(--er) / 0.3)}
.hover\:border-r-error\/40:hover {
    border-right-color: hsl(var(--er) / 0.4)}
.hover\:border-r-error\/5:hover {
    border-right-color: hsl(var(--er) / 0.05)}
.hover\:border-r-error\/50:hover {
    border-right-color: hsl(var(--er) / 0.5)}
.hover\:border-r-error\/60:hover {
    border-right-color: hsl(var(--er) / 0.6)}
.hover\:border-r-error\/70:hover {
    border-right-color: hsl(var(--er) / 0.7)}
.hover\:border-r-error\/75:hover {
    border-right-color: hsl(var(--er) / 0.75)}
.hover\:border-r-error\/80:hover {
    border-right-color: hsl(var(--er) / 0.8)}
.hover\:border-r-error\/90:hover {
    border-right-color: hsl(var(--er) / 0.9)}
.hover\:border-r-error\/95:hover {
    border-right-color: hsl(var(--er) / 0.95)}
.hover\:border-r-info:hover {
    border-right-color: hsl(var(--in) / 1)}
.hover\:border-r-info-content:hover {
    border-right-color: hsl(var(--inc) / 1)}
.hover\:border-r-info-content\/0:hover {
    border-right-color: hsl(var(--inc) / 0)}
.hover\:border-r-info-content\/10:hover {
    border-right-color: hsl(var(--inc) / 0.1)}
.hover\:border-r-info-content\/100:hover {
    border-right-color: hsl(var(--inc) / 1)}
.hover\:border-r-info-content\/20:hover {
    border-right-color: hsl(var(--inc) / 0.2)}
.hover\:border-r-info-content\/25:hover {
    border-right-color: hsl(var(--inc) / 0.25)}
.hover\:border-r-info-content\/30:hover {
    border-right-color: hsl(var(--inc) / 0.3)}
.hover\:border-r-info-content\/40:hover {
    border-right-color: hsl(var(--inc) / 0.4)}
.hover\:border-r-info-content\/5:hover {
    border-right-color: hsl(var(--inc) / 0.05)}
.hover\:border-r-info-content\/50:hover {
    border-right-color: hsl(var(--inc) / 0.5)}
.hover\:border-r-info-content\/60:hover {
    border-right-color: hsl(var(--inc) / 0.6)}
.hover\:border-r-info-content\/70:hover {
    border-right-color: hsl(var(--inc) / 0.7)}
.hover\:border-r-info-content\/75:hover {
    border-right-color: hsl(var(--inc) / 0.75)}
.hover\:border-r-info-content\/80:hover {
    border-right-color: hsl(var(--inc) / 0.8)}
.hover\:border-r-info-content\/90:hover {
    border-right-color: hsl(var(--inc) / 0.9)}
.hover\:border-r-info-content\/95:hover {
    border-right-color: hsl(var(--inc) / 0.95)}
.hover\:border-r-info\/0:hover {
    border-right-color: hsl(var(--in) / 0)}
.hover\:border-r-info\/10:hover {
    border-right-color: hsl(var(--in) / 0.1)}
.hover\:border-r-info\/100:hover {
    border-right-color: hsl(var(--in) / 1)}
.hover\:border-r-info\/20:hover {
    border-right-color: hsl(var(--in) / 0.2)}
.hover\:border-r-info\/25:hover {
    border-right-color: hsl(var(--in) / 0.25)}
.hover\:border-r-info\/30:hover {
    border-right-color: hsl(var(--in) / 0.3)}
.hover\:border-r-info\/40:hover {
    border-right-color: hsl(var(--in) / 0.4)}
.hover\:border-r-info\/5:hover {
    border-right-color: hsl(var(--in) / 0.05)}
.hover\:border-r-info\/50:hover {
    border-right-color: hsl(var(--in) / 0.5)}
.hover\:border-r-info\/60:hover {
    border-right-color: hsl(var(--in) / 0.6)}
.hover\:border-r-info\/70:hover {
    border-right-color: hsl(var(--in) / 0.7)}
.hover\:border-r-info\/75:hover {
    border-right-color: hsl(var(--in) / 0.75)}
.hover\:border-r-info\/80:hover {
    border-right-color: hsl(var(--in) / 0.8)}
.hover\:border-r-info\/90:hover {
    border-right-color: hsl(var(--in) / 0.9)}
.hover\:border-r-info\/95:hover {
    border-right-color: hsl(var(--in) / 0.95)}
.hover\:border-r-success:hover {
    border-right-color: hsl(var(--su) / 1)}
.hover\:border-r-success-content:hover {
    border-right-color: hsl(var(--suc) / 1)}
.hover\:border-r-success-content\/0:hover {
    border-right-color: hsl(var(--suc) / 0)}
.hover\:border-r-success-content\/10:hover {
    border-right-color: hsl(var(--suc) / 0.1)}
.hover\:border-r-success-content\/100:hover {
    border-right-color: hsl(var(--suc) / 1)}
.hover\:border-r-success-content\/20:hover {
    border-right-color: hsl(var(--suc) / 0.2)}
.hover\:border-r-success-content\/25:hover {
    border-right-color: hsl(var(--suc) / 0.25)}
.hover\:border-r-success-content\/30:hover {
    border-right-color: hsl(var(--suc) / 0.3)}
.hover\:border-r-success-content\/40:hover {
    border-right-color: hsl(var(--suc) / 0.4)}
.hover\:border-r-success-content\/5:hover {
    border-right-color: hsl(var(--suc) / 0.05)}
.hover\:border-r-success-content\/50:hover {
    border-right-color: hsl(var(--suc) / 0.5)}
.hover\:border-r-success-content\/60:hover {
    border-right-color: hsl(var(--suc) / 0.6)}
.hover\:border-r-success-content\/70:hover {
    border-right-color: hsl(var(--suc) / 0.7)}
.hover\:border-r-success-content\/75:hover {
    border-right-color: hsl(var(--suc) / 0.75)}
.hover\:border-r-success-content\/80:hover {
    border-right-color: hsl(var(--suc) / 0.8)}
.hover\:border-r-success-content\/90:hover {
    border-right-color: hsl(var(--suc) / 0.9)}
.hover\:border-r-success-content\/95:hover {
    border-right-color: hsl(var(--suc) / 0.95)}
.hover\:border-r-success\/0:hover {
    border-right-color: hsl(var(--su) / 0)}
.hover\:border-r-success\/10:hover {
    border-right-color: hsl(var(--su) / 0.1)}
.hover\:border-r-success\/100:hover {
    border-right-color: hsl(var(--su) / 1)}
.hover\:border-r-success\/20:hover {
    border-right-color: hsl(var(--su) / 0.2)}
.hover\:border-r-success\/25:hover {
    border-right-color: hsl(var(--su) / 0.25)}
.hover\:border-r-success\/30:hover {
    border-right-color: hsl(var(--su) / 0.3)}
.hover\:border-r-success\/40:hover {
    border-right-color: hsl(var(--su) / 0.4)}
.hover\:border-r-success\/5:hover {
    border-right-color: hsl(var(--su) / 0.05)}
.hover\:border-r-success\/50:hover {
    border-right-color: hsl(var(--su) / 0.5)}
.hover\:border-r-success\/60:hover {
    border-right-color: hsl(var(--su) / 0.6)}
.hover\:border-r-success\/70:hover {
    border-right-color: hsl(var(--su) / 0.7)}
.hover\:border-r-success\/75:hover {
    border-right-color: hsl(var(--su) / 0.75)}
.hover\:border-r-success\/80:hover {
    border-right-color: hsl(var(--su) / 0.8)}
.hover\:border-r-success\/90:hover {
    border-right-color: hsl(var(--su) / 0.9)}
.hover\:border-r-success\/95:hover {
    border-right-color: hsl(var(--su) / 0.95)}
.hover\:border-r-warning:hover {
    border-right-color: hsl(var(--wa) / 1)}
.hover\:border-r-warning-content:hover {
    border-right-color: hsl(var(--wac) / 1)}
.hover\:border-r-warning-content\/0:hover {
    border-right-color: hsl(var(--wac) / 0)}
.hover\:border-r-warning-content\/10:hover {
    border-right-color: hsl(var(--wac) / 0.1)}
.hover\:border-r-warning-content\/100:hover {
    border-right-color: hsl(var(--wac) / 1)}
.hover\:border-r-warning-content\/20:hover {
    border-right-color: hsl(var(--wac) / 0.2)}
.hover\:border-r-warning-content\/25:hover {
    border-right-color: hsl(var(--wac) / 0.25)}
.hover\:border-r-warning-content\/30:hover {
    border-right-color: hsl(var(--wac) / 0.3)}
.hover\:border-r-warning-content\/40:hover {
    border-right-color: hsl(var(--wac) / 0.4)}
.hover\:border-r-warning-content\/5:hover {
    border-right-color: hsl(var(--wac) / 0.05)}
.hover\:border-r-warning-content\/50:hover {
    border-right-color: hsl(var(--wac) / 0.5)}
.hover\:border-r-warning-content\/60:hover {
    border-right-color: hsl(var(--wac) / 0.6)}
.hover\:border-r-warning-content\/70:hover {
    border-right-color: hsl(var(--wac) / 0.7)}
.hover\:border-r-warning-content\/75:hover {
    border-right-color: hsl(var(--wac) / 0.75)}
.hover\:border-r-warning-content\/80:hover {
    border-right-color: hsl(var(--wac) / 0.8)}
.hover\:border-r-warning-content\/90:hover {
    border-right-color: hsl(var(--wac) / 0.9)}
.hover\:border-r-warning-content\/95:hover {
    border-right-color: hsl(var(--wac) / 0.95)}
.hover\:border-r-warning\/0:hover {
    border-right-color: hsl(var(--wa) / 0)}
.hover\:border-r-warning\/10:hover {
    border-right-color: hsl(var(--wa) / 0.1)}
.hover\:border-r-warning\/100:hover {
    border-right-color: hsl(var(--wa) / 1)}
.hover\:border-r-warning\/20:hover {
    border-right-color: hsl(var(--wa) / 0.2)}
.hover\:border-r-warning\/25:hover {
    border-right-color: hsl(var(--wa) / 0.25)}
.hover\:border-r-warning\/30:hover {
    border-right-color: hsl(var(--wa) / 0.3)}
.hover\:border-r-warning\/40:hover {
    border-right-color: hsl(var(--wa) / 0.4)}
.hover\:border-r-warning\/5:hover {
    border-right-color: hsl(var(--wa) / 0.05)}
.hover\:border-r-warning\/50:hover {
    border-right-color: hsl(var(--wa) / 0.5)}
.hover\:border-r-warning\/60:hover {
    border-right-color: hsl(var(--wa) / 0.6)}
.hover\:border-r-warning\/70:hover {
    border-right-color: hsl(var(--wa) / 0.7)}
.hover\:border-r-warning\/75:hover {
    border-right-color: hsl(var(--wa) / 0.75)}
.hover\:border-r-warning\/80:hover {
    border-right-color: hsl(var(--wa) / 0.8)}
.hover\:border-r-warning\/90:hover {
    border-right-color: hsl(var(--wa) / 0.9)}
.hover\:border-r-warning\/95:hover {
    border-right-color: hsl(var(--wa) / 0.95)}
.hover\:border-s-base-100:hover {
    border-inline-start-color: hsl(var(--b1) / 1)}
.hover\:border-s-base-100\/0:hover {
    border-inline-start-color: hsl(var(--b1) / 0)}
.hover\:border-s-base-100\/10:hover {
    border-inline-start-color: hsl(var(--b1) / 0.1)}
.hover\:border-s-base-100\/100:hover {
    border-inline-start-color: hsl(var(--b1) / 1)}
.hover\:border-s-base-100\/20:hover {
    border-inline-start-color: hsl(var(--b1) / 0.2)}
.hover\:border-s-base-100\/25:hover {
    border-inline-start-color: hsl(var(--b1) / 0.25)}
.hover\:border-s-base-100\/30:hover {
    border-inline-start-color: hsl(var(--b1) / 0.3)}
.hover\:border-s-base-100\/40:hover {
    border-inline-start-color: hsl(var(--b1) / 0.4)}
.hover\:border-s-base-100\/5:hover {
    border-inline-start-color: hsl(var(--b1) / 0.05)}
.hover\:border-s-base-100\/50:hover {
    border-inline-start-color: hsl(var(--b1) / 0.5)}
.hover\:border-s-base-100\/60:hover {
    border-inline-start-color: hsl(var(--b1) / 0.6)}
.hover\:border-s-base-100\/70:hover {
    border-inline-start-color: hsl(var(--b1) / 0.7)}
.hover\:border-s-base-100\/75:hover {
    border-inline-start-color: hsl(var(--b1) / 0.75)}
.hover\:border-s-base-100\/80:hover {
    border-inline-start-color: hsl(var(--b1) / 0.8)}
.hover\:border-s-base-100\/90:hover {
    border-inline-start-color: hsl(var(--b1) / 0.9)}
.hover\:border-s-base-100\/95:hover {
    border-inline-start-color: hsl(var(--b1) / 0.95)}
.hover\:border-s-base-200:hover {
    border-inline-start-color: hsl(var(--b2) / 1)}
.hover\:border-s-base-200\/0:hover {
    border-inline-start-color: hsl(var(--b2) / 0)}
.hover\:border-s-base-200\/10:hover {
    border-inline-start-color: hsl(var(--b2) / 0.1)}
.hover\:border-s-base-200\/100:hover {
    border-inline-start-color: hsl(var(--b2) / 1)}
.hover\:border-s-base-200\/20:hover {
    border-inline-start-color: hsl(var(--b2) / 0.2)}
.hover\:border-s-base-200\/25:hover {
    border-inline-start-color: hsl(var(--b2) / 0.25)}
.hover\:border-s-base-200\/30:hover {
    border-inline-start-color: hsl(var(--b2) / 0.3)}
.hover\:border-s-base-200\/40:hover {
    border-inline-start-color: hsl(var(--b2) / 0.4)}
.hover\:border-s-base-200\/5:hover {
    border-inline-start-color: hsl(var(--b2) / 0.05)}
.hover\:border-s-base-200\/50:hover {
    border-inline-start-color: hsl(var(--b2) / 0.5)}
.hover\:border-s-base-200\/60:hover {
    border-inline-start-color: hsl(var(--b2) / 0.6)}
.hover\:border-s-base-200\/70:hover {
    border-inline-start-color: hsl(var(--b2) / 0.7)}
.hover\:border-s-base-200\/75:hover {
    border-inline-start-color: hsl(var(--b2) / 0.75)}
.hover\:border-s-base-200\/80:hover {
    border-inline-start-color: hsl(var(--b2) / 0.8)}
.hover\:border-s-base-200\/90:hover {
    border-inline-start-color: hsl(var(--b2) / 0.9)}
.hover\:border-s-base-200\/95:hover {
    border-inline-start-color: hsl(var(--b2) / 0.95)}
.hover\:border-s-base-300:hover {
    border-inline-start-color: hsl(var(--b3) / 1)}
.hover\:border-s-base-300\/0:hover {
    border-inline-start-color: hsl(var(--b3) / 0)}
.hover\:border-s-base-300\/10:hover {
    border-inline-start-color: hsl(var(--b3) / 0.1)}
.hover\:border-s-base-300\/100:hover {
    border-inline-start-color: hsl(var(--b3) / 1)}
.hover\:border-s-base-300\/20:hover {
    border-inline-start-color: hsl(var(--b3) / 0.2)}
.hover\:border-s-base-300\/25:hover {
    border-inline-start-color: hsl(var(--b3) / 0.25)}
.hover\:border-s-base-300\/30:hover {
    border-inline-start-color: hsl(var(--b3) / 0.3)}
.hover\:border-s-base-300\/40:hover {
    border-inline-start-color: hsl(var(--b3) / 0.4)}
.hover\:border-s-base-300\/5:hover {
    border-inline-start-color: hsl(var(--b3) / 0.05)}
.hover\:border-s-base-300\/50:hover {
    border-inline-start-color: hsl(var(--b3) / 0.5)}
.hover\:border-s-base-300\/60:hover {
    border-inline-start-color: hsl(var(--b3) / 0.6)}
.hover\:border-s-base-300\/70:hover {
    border-inline-start-color: hsl(var(--b3) / 0.7)}
.hover\:border-s-base-300\/75:hover {
    border-inline-start-color: hsl(var(--b3) / 0.75)}
.hover\:border-s-base-300\/80:hover {
    border-inline-start-color: hsl(var(--b3) / 0.8)}
.hover\:border-s-base-300\/90:hover {
    border-inline-start-color: hsl(var(--b3) / 0.9)}
.hover\:border-s-base-300\/95:hover {
    border-inline-start-color: hsl(var(--b3) / 0.95)}
.hover\:border-s-base-content:hover {
    border-inline-start-color: hsl(var(--bc) / 1)}
.hover\:border-s-base-content\/0:hover {
    border-inline-start-color: hsl(var(--bc) / 0)}
.hover\:border-s-base-content\/10:hover {
    border-inline-start-color: hsl(var(--bc) / 0.1)}
.hover\:border-s-base-content\/100:hover {
    border-inline-start-color: hsl(var(--bc) / 1)}
.hover\:border-s-base-content\/20:hover {
    border-inline-start-color: hsl(var(--bc) / 0.2)}
.hover\:border-s-base-content\/25:hover {
    border-inline-start-color: hsl(var(--bc) / 0.25)}
.hover\:border-s-base-content\/30:hover {
    border-inline-start-color: hsl(var(--bc) / 0.3)}
.hover\:border-s-base-content\/40:hover {
    border-inline-start-color: hsl(var(--bc) / 0.4)}
.hover\:border-s-base-content\/5:hover {
    border-inline-start-color: hsl(var(--bc) / 0.05)}
.hover\:border-s-base-content\/50:hover {
    border-inline-start-color: hsl(var(--bc) / 0.5)}
.hover\:border-s-base-content\/60:hover {
    border-inline-start-color: hsl(var(--bc) / 0.6)}
.hover\:border-s-base-content\/70:hover {
    border-inline-start-color: hsl(var(--bc) / 0.7)}
.hover\:border-s-base-content\/75:hover {
    border-inline-start-color: hsl(var(--bc) / 0.75)}
.hover\:border-s-base-content\/80:hover {
    border-inline-start-color: hsl(var(--bc) / 0.8)}
.hover\:border-s-base-content\/90:hover {
    border-inline-start-color: hsl(var(--bc) / 0.9)}
.hover\:border-s-base-content\/95:hover {
    border-inline-start-color: hsl(var(--bc) / 0.95)}
.hover\:border-s-error:hover {
    border-inline-start-color: hsl(var(--er) / 1)}
.hover\:border-s-error-content:hover {
    border-inline-start-color: hsl(var(--erc) / 1)}
.hover\:border-s-error-content\/0:hover {
    border-inline-start-color: hsl(var(--erc) / 0)}
.hover\:border-s-error-content\/10:hover {
    border-inline-start-color: hsl(var(--erc) / 0.1)}
.hover\:border-s-error-content\/100:hover {
    border-inline-start-color: hsl(var(--erc) / 1)}
.hover\:border-s-error-content\/20:hover {
    border-inline-start-color: hsl(var(--erc) / 0.2)}
.hover\:border-s-error-content\/25:hover {
    border-inline-start-color: hsl(var(--erc) / 0.25)}
.hover\:border-s-error-content\/30:hover {
    border-inline-start-color: hsl(var(--erc) / 0.3)}
.hover\:border-s-error-content\/40:hover {
    border-inline-start-color: hsl(var(--erc) / 0.4)}
.hover\:border-s-error-content\/5:hover {
    border-inline-start-color: hsl(var(--erc) / 0.05)}
.hover\:border-s-error-content\/50:hover {
    border-inline-start-color: hsl(var(--erc) / 0.5)}
.hover\:border-s-error-content\/60:hover {
    border-inline-start-color: hsl(var(--erc) / 0.6)}
.hover\:border-s-error-content\/70:hover {
    border-inline-start-color: hsl(var(--erc) / 0.7)}
.hover\:border-s-error-content\/75:hover {
    border-inline-start-color: hsl(var(--erc) / 0.75)}
.hover\:border-s-error-content\/80:hover {
    border-inline-start-color: hsl(var(--erc) / 0.8)}
.hover\:border-s-error-content\/90:hover {
    border-inline-start-color: hsl(var(--erc) / 0.9)}
.hover\:border-s-error-content\/95:hover {
    border-inline-start-color: hsl(var(--erc) / 0.95)}
.hover\:border-s-error\/0:hover {
    border-inline-start-color: hsl(var(--er) / 0)}
.hover\:border-s-error\/10:hover {
    border-inline-start-color: hsl(var(--er) / 0.1)}
.hover\:border-s-error\/100:hover {
    border-inline-start-color: hsl(var(--er) / 1)}
.hover\:border-s-error\/20:hover {
    border-inline-start-color: hsl(var(--er) / 0.2)}
.hover\:border-s-error\/25:hover {
    border-inline-start-color: hsl(var(--er) / 0.25)}
.hover\:border-s-error\/30:hover {
    border-inline-start-color: hsl(var(--er) / 0.3)}
.hover\:border-s-error\/40:hover {
    border-inline-start-color: hsl(var(--er) / 0.4)}
.hover\:border-s-error\/5:hover {
    border-inline-start-color: hsl(var(--er) / 0.05)}
.hover\:border-s-error\/50:hover {
    border-inline-start-color: hsl(var(--er) / 0.5)}
.hover\:border-s-error\/60:hover {
    border-inline-start-color: hsl(var(--er) / 0.6)}
.hover\:border-s-error\/70:hover {
    border-inline-start-color: hsl(var(--er) / 0.7)}
.hover\:border-s-error\/75:hover {
    border-inline-start-color: hsl(var(--er) / 0.75)}
.hover\:border-s-error\/80:hover {
    border-inline-start-color: hsl(var(--er) / 0.8)}
.hover\:border-s-error\/90:hover {
    border-inline-start-color: hsl(var(--er) / 0.9)}
.hover\:border-s-error\/95:hover {
    border-inline-start-color: hsl(var(--er) / 0.95)}
.hover\:border-s-info:hover {
    border-inline-start-color: hsl(var(--in) / 1)}
.hover\:border-s-info-content:hover {
    border-inline-start-color: hsl(var(--inc) / 1)}
.hover\:border-s-info-content\/0:hover {
    border-inline-start-color: hsl(var(--inc) / 0)}
.hover\:border-s-info-content\/10:hover {
    border-inline-start-color: hsl(var(--inc) / 0.1)}
.hover\:border-s-info-content\/100:hover {
    border-inline-start-color: hsl(var(--inc) / 1)}
.hover\:border-s-info-content\/20:hover {
    border-inline-start-color: hsl(var(--inc) / 0.2)}
.hover\:border-s-info-content\/25:hover {
    border-inline-start-color: hsl(var(--inc) / 0.25)}
.hover\:border-s-info-content\/30:hover {
    border-inline-start-color: hsl(var(--inc) / 0.3)}
.hover\:border-s-info-content\/40:hover {
    border-inline-start-color: hsl(var(--inc) / 0.4)}
.hover\:border-s-info-content\/5:hover {
    border-inline-start-color: hsl(var(--inc) / 0.05)}
.hover\:border-s-info-content\/50:hover {
    border-inline-start-color: hsl(var(--inc) / 0.5)}
.hover\:border-s-info-content\/60:hover {
    border-inline-start-color: hsl(var(--inc) / 0.6)}
.hover\:border-s-info-content\/70:hover {
    border-inline-start-color: hsl(var(--inc) / 0.7)}
.hover\:border-s-info-content\/75:hover {
    border-inline-start-color: hsl(var(--inc) / 0.75)}
.hover\:border-s-info-content\/80:hover {
    border-inline-start-color: hsl(var(--inc) / 0.8)}
.hover\:border-s-info-content\/90:hover {
    border-inline-start-color: hsl(var(--inc) / 0.9)}
.hover\:border-s-info-content\/95:hover {
    border-inline-start-color: hsl(var(--inc) / 0.95)}
.hover\:border-s-info\/0:hover {
    border-inline-start-color: hsl(var(--in) / 0)}
.hover\:border-s-info\/10:hover {
    border-inline-start-color: hsl(var(--in) / 0.1)}
.hover\:border-s-info\/100:hover {
    border-inline-start-color: hsl(var(--in) / 1)}
.hover\:border-s-info\/20:hover {
    border-inline-start-color: hsl(var(--in) / 0.2)}
.hover\:border-s-info\/25:hover {
    border-inline-start-color: hsl(var(--in) / 0.25)}
.hover\:border-s-info\/30:hover {
    border-inline-start-color: hsl(var(--in) / 0.3)}
.hover\:border-s-info\/40:hover {
    border-inline-start-color: hsl(var(--in) / 0.4)}
.hover\:border-s-info\/5:hover {
    border-inline-start-color: hsl(var(--in) / 0.05)}
.hover\:border-s-info\/50:hover {
    border-inline-start-color: hsl(var(--in) / 0.5)}
.hover\:border-s-info\/60:hover {
    border-inline-start-color: hsl(var(--in) / 0.6)}
.hover\:border-s-info\/70:hover {
    border-inline-start-color: hsl(var(--in) / 0.7)}
.hover\:border-s-info\/75:hover {
    border-inline-start-color: hsl(var(--in) / 0.75)}
.hover\:border-s-info\/80:hover {
    border-inline-start-color: hsl(var(--in) / 0.8)}
.hover\:border-s-info\/90:hover {
    border-inline-start-color: hsl(var(--in) / 0.9)}
.hover\:border-s-info\/95:hover {
    border-inline-start-color: hsl(var(--in) / 0.95)}
.hover\:border-s-success:hover {
    border-inline-start-color: hsl(var(--su) / 1)}
.hover\:border-s-success-content:hover {
    border-inline-start-color: hsl(var(--suc) / 1)}
.hover\:border-s-success-content\/0:hover {
    border-inline-start-color: hsl(var(--suc) / 0)}
.hover\:border-s-success-content\/10:hover {
    border-inline-start-color: hsl(var(--suc) / 0.1)}
.hover\:border-s-success-content\/100:hover {
    border-inline-start-color: hsl(var(--suc) / 1)}
.hover\:border-s-success-content\/20:hover {
    border-inline-start-color: hsl(var(--suc) / 0.2)}
.hover\:border-s-success-content\/25:hover {
    border-inline-start-color: hsl(var(--suc) / 0.25)}
.hover\:border-s-success-content\/30:hover {
    border-inline-start-color: hsl(var(--suc) / 0.3)}
.hover\:border-s-success-content\/40:hover {
    border-inline-start-color: hsl(var(--suc) / 0.4)}
.hover\:border-s-success-content\/5:hover {
    border-inline-start-color: hsl(var(--suc) / 0.05)}
.hover\:border-s-success-content\/50:hover {
    border-inline-start-color: hsl(var(--suc) / 0.5)}
.hover\:border-s-success-content\/60:hover {
    border-inline-start-color: hsl(var(--suc) / 0.6)}
.hover\:border-s-success-content\/70:hover {
    border-inline-start-color: hsl(var(--suc) / 0.7)}
.hover\:border-s-success-content\/75:hover {
    border-inline-start-color: hsl(var(--suc) / 0.75)}
.hover\:border-s-success-content\/80:hover {
    border-inline-start-color: hsl(var(--suc) / 0.8)}
.hover\:border-s-success-content\/90:hover {
    border-inline-start-color: hsl(var(--suc) / 0.9)}
.hover\:border-s-success-content\/95:hover {
    border-inline-start-color: hsl(var(--suc) / 0.95)}
.hover\:border-s-success\/0:hover {
    border-inline-start-color: hsl(var(--su) / 0)}
.hover\:border-s-success\/10:hover {
    border-inline-start-color: hsl(var(--su) / 0.1)}
.hover\:border-s-success\/100:hover {
    border-inline-start-color: hsl(var(--su) / 1)}
.hover\:border-s-success\/20:hover {
    border-inline-start-color: hsl(var(--su) / 0.2)}
.hover\:border-s-success\/25:hover {
    border-inline-start-color: hsl(var(--su) / 0.25)}
.hover\:border-s-success\/30:hover {
    border-inline-start-color: hsl(var(--su) / 0.3)}
.hover\:border-s-success\/40:hover {
    border-inline-start-color: hsl(var(--su) / 0.4)}
.hover\:border-s-success\/5:hover {
    border-inline-start-color: hsl(var(--su) / 0.05)}
.hover\:border-s-success\/50:hover {
    border-inline-start-color: hsl(var(--su) / 0.5)}
.hover\:border-s-success\/60:hover {
    border-inline-start-color: hsl(var(--su) / 0.6)}
.hover\:border-s-success\/70:hover {
    border-inline-start-color: hsl(var(--su) / 0.7)}
.hover\:border-s-success\/75:hover {
    border-inline-start-color: hsl(var(--su) / 0.75)}
.hover\:border-s-success\/80:hover {
    border-inline-start-color: hsl(var(--su) / 0.8)}
.hover\:border-s-success\/90:hover {
    border-inline-start-color: hsl(var(--su) / 0.9)}
.hover\:border-s-success\/95:hover {
    border-inline-start-color: hsl(var(--su) / 0.95)}
.hover\:border-s-warning:hover {
    border-inline-start-color: hsl(var(--wa) / 1)}
.hover\:border-s-warning-content:hover {
    border-inline-start-color: hsl(var(--wac) / 1)}
.hover\:border-s-warning-content\/0:hover {
    border-inline-start-color: hsl(var(--wac) / 0)}
.hover\:border-s-warning-content\/10:hover {
    border-inline-start-color: hsl(var(--wac) / 0.1)}
.hover\:border-s-warning-content\/100:hover {
    border-inline-start-color: hsl(var(--wac) / 1)}
.hover\:border-s-warning-content\/20:hover {
    border-inline-start-color: hsl(var(--wac) / 0.2)}
.hover\:border-s-warning-content\/25:hover {
    border-inline-start-color: hsl(var(--wac) / 0.25)}
.hover\:border-s-warning-content\/30:hover {
    border-inline-start-color: hsl(var(--wac) / 0.3)}
.hover\:border-s-warning-content\/40:hover {
    border-inline-start-color: hsl(var(--wac) / 0.4)}
.hover\:border-s-warning-content\/5:hover {
    border-inline-start-color: hsl(var(--wac) / 0.05)}
.hover\:border-s-warning-content\/50:hover {
    border-inline-start-color: hsl(var(--wac) / 0.5)}
.hover\:border-s-warning-content\/60:hover {
    border-inline-start-color: hsl(var(--wac) / 0.6)}
.hover\:border-s-warning-content\/70:hover {
    border-inline-start-color: hsl(var(--wac) / 0.7)}
.hover\:border-s-warning-content\/75:hover {
    border-inline-start-color: hsl(var(--wac) / 0.75)}
.hover\:border-s-warning-content\/80:hover {
    border-inline-start-color: hsl(var(--wac) / 0.8)}
.hover\:border-s-warning-content\/90:hover {
    border-inline-start-color: hsl(var(--wac) / 0.9)}
.hover\:border-s-warning-content\/95:hover {
    border-inline-start-color: hsl(var(--wac) / 0.95)}
.hover\:border-s-warning\/0:hover {
    border-inline-start-color: hsl(var(--wa) / 0)}
.hover\:border-s-warning\/10:hover {
    border-inline-start-color: hsl(var(--wa) / 0.1)}
.hover\:border-s-warning\/100:hover {
    border-inline-start-color: hsl(var(--wa) / 1)}
.hover\:border-s-warning\/20:hover {
    border-inline-start-color: hsl(var(--wa) / 0.2)}
.hover\:border-s-warning\/25:hover {
    border-inline-start-color: hsl(var(--wa) / 0.25)}
.hover\:border-s-warning\/30:hover {
    border-inline-start-color: hsl(var(--wa) / 0.3)}
.hover\:border-s-warning\/40:hover {
    border-inline-start-color: hsl(var(--wa) / 0.4)}
.hover\:border-s-warning\/5:hover {
    border-inline-start-color: hsl(var(--wa) / 0.05)}
.hover\:border-s-warning\/50:hover {
    border-inline-start-color: hsl(var(--wa) / 0.5)}
.hover\:border-s-warning\/60:hover {
    border-inline-start-color: hsl(var(--wa) / 0.6)}
.hover\:border-s-warning\/70:hover {
    border-inline-start-color: hsl(var(--wa) / 0.7)}
.hover\:border-s-warning\/75:hover {
    border-inline-start-color: hsl(var(--wa) / 0.75)}
.hover\:border-s-warning\/80:hover {
    border-inline-start-color: hsl(var(--wa) / 0.8)}
.hover\:border-s-warning\/90:hover {
    border-inline-start-color: hsl(var(--wa) / 0.9)}
.hover\:border-s-warning\/95:hover {
    border-inline-start-color: hsl(var(--wa) / 0.95)}
.hover\:border-t-base-100:hover {
    border-top-color: hsl(var(--b1) / 1)}
.hover\:border-t-base-100\/0:hover {
    border-top-color: hsl(var(--b1) / 0)}
.hover\:border-t-base-100\/10:hover {
    border-top-color: hsl(var(--b1) / 0.1)}
.hover\:border-t-base-100\/100:hover {
    border-top-color: hsl(var(--b1) / 1)}
.hover\:border-t-base-100\/20:hover {
    border-top-color: hsl(var(--b1) / 0.2)}
.hover\:border-t-base-100\/25:hover {
    border-top-color: hsl(var(--b1) / 0.25)}
.hover\:border-t-base-100\/30:hover {
    border-top-color: hsl(var(--b1) / 0.3)}
.hover\:border-t-base-100\/40:hover {
    border-top-color: hsl(var(--b1) / 0.4)}
.hover\:border-t-base-100\/5:hover {
    border-top-color: hsl(var(--b1) / 0.05)}
.hover\:border-t-base-100\/50:hover {
    border-top-color: hsl(var(--b1) / 0.5)}
.hover\:border-t-base-100\/60:hover {
    border-top-color: hsl(var(--b1) / 0.6)}
.hover\:border-t-base-100\/70:hover {
    border-top-color: hsl(var(--b1) / 0.7)}
.hover\:border-t-base-100\/75:hover {
    border-top-color: hsl(var(--b1) / 0.75)}
.hover\:border-t-base-100\/80:hover {
    border-top-color: hsl(var(--b1) / 0.8)}
.hover\:border-t-base-100\/90:hover {
    border-top-color: hsl(var(--b1) / 0.9)}
.hover\:border-t-base-100\/95:hover {
    border-top-color: hsl(var(--b1) / 0.95)}
.hover\:border-t-base-200:hover {
    border-top-color: hsl(var(--b2) / 1)}
.hover\:border-t-base-200\/0:hover {
    border-top-color: hsl(var(--b2) / 0)}
.hover\:border-t-base-200\/10:hover {
    border-top-color: hsl(var(--b2) / 0.1)}
.hover\:border-t-base-200\/100:hover {
    border-top-color: hsl(var(--b2) / 1)}
.hover\:border-t-base-200\/20:hover {
    border-top-color: hsl(var(--b2) / 0.2)}
.hover\:border-t-base-200\/25:hover {
    border-top-color: hsl(var(--b2) / 0.25)}
.hover\:border-t-base-200\/30:hover {
    border-top-color: hsl(var(--b2) / 0.3)}
.hover\:border-t-base-200\/40:hover {
    border-top-color: hsl(var(--b2) / 0.4)}
.hover\:border-t-base-200\/5:hover {
    border-top-color: hsl(var(--b2) / 0.05)}
.hover\:border-t-base-200\/50:hover {
    border-top-color: hsl(var(--b2) / 0.5)}
.hover\:border-t-base-200\/60:hover {
    border-top-color: hsl(var(--b2) / 0.6)}
.hover\:border-t-base-200\/70:hover {
    border-top-color: hsl(var(--b2) / 0.7)}
.hover\:border-t-base-200\/75:hover {
    border-top-color: hsl(var(--b2) / 0.75)}
.hover\:border-t-base-200\/80:hover {
    border-top-color: hsl(var(--b2) / 0.8)}
.hover\:border-t-base-200\/90:hover {
    border-top-color: hsl(var(--b2) / 0.9)}
.hover\:border-t-base-200\/95:hover {
    border-top-color: hsl(var(--b2) / 0.95)}
.hover\:border-t-base-300:hover {
    border-top-color: hsl(var(--b3) / 1)}
.hover\:border-t-base-300\/0:hover {
    border-top-color: hsl(var(--b3) / 0)}
.hover\:border-t-base-300\/10:hover {
    border-top-color: hsl(var(--b3) / 0.1)}
.hover\:border-t-base-300\/100:hover {
    border-top-color: hsl(var(--b3) / 1)}
.hover\:border-t-base-300\/20:hover {
    border-top-color: hsl(var(--b3) / 0.2)}
.hover\:border-t-base-300\/25:hover {
    border-top-color: hsl(var(--b3) / 0.25)}
.hover\:border-t-base-300\/30:hover {
    border-top-color: hsl(var(--b3) / 0.3)}
.hover\:border-t-base-300\/40:hover {
    border-top-color: hsl(var(--b3) / 0.4)}
.hover\:border-t-base-300\/5:hover {
    border-top-color: hsl(var(--b3) / 0.05)}
.hover\:border-t-base-300\/50:hover {
    border-top-color: hsl(var(--b3) / 0.5)}
.hover\:border-t-base-300\/60:hover {
    border-top-color: hsl(var(--b3) / 0.6)}
.hover\:border-t-base-300\/70:hover {
    border-top-color: hsl(var(--b3) / 0.7)}
.hover\:border-t-base-300\/75:hover {
    border-top-color: hsl(var(--b3) / 0.75)}
.hover\:border-t-base-300\/80:hover {
    border-top-color: hsl(var(--b3) / 0.8)}
.hover\:border-t-base-300\/90:hover {
    border-top-color: hsl(var(--b3) / 0.9)}
.hover\:border-t-base-300\/95:hover {
    border-top-color: hsl(var(--b3) / 0.95)}
.hover\:border-t-base-content:hover {
    border-top-color: hsl(var(--bc) / 1)}
.hover\:border-t-base-content\/0:hover {
    border-top-color: hsl(var(--bc) / 0)}
.hover\:border-t-base-content\/10:hover {
    border-top-color: hsl(var(--bc) / 0.1)}
.hover\:border-t-base-content\/100:hover {
    border-top-color: hsl(var(--bc) / 1)}
.hover\:border-t-base-content\/20:hover {
    border-top-color: hsl(var(--bc) / 0.2)}
.hover\:border-t-base-content\/25:hover {
    border-top-color: hsl(var(--bc) / 0.25)}
.hover\:border-t-base-content\/30:hover {
    border-top-color: hsl(var(--bc) / 0.3)}
.hover\:border-t-base-content\/40:hover {
    border-top-color: hsl(var(--bc) / 0.4)}
.hover\:border-t-base-content\/5:hover {
    border-top-color: hsl(var(--bc) / 0.05)}
.hover\:border-t-base-content\/50:hover {
    border-top-color: hsl(var(--bc) / 0.5)}
.hover\:border-t-base-content\/60:hover {
    border-top-color: hsl(var(--bc) / 0.6)}
.hover\:border-t-base-content\/70:hover {
    border-top-color: hsl(var(--bc) / 0.7)}
.hover\:border-t-base-content\/75:hover {
    border-top-color: hsl(var(--bc) / 0.75)}
.hover\:border-t-base-content\/80:hover {
    border-top-color: hsl(var(--bc) / 0.8)}
.hover\:border-t-base-content\/90:hover {
    border-top-color: hsl(var(--bc) / 0.9)}
.hover\:border-t-base-content\/95:hover {
    border-top-color: hsl(var(--bc) / 0.95)}
.hover\:border-t-error:hover {
    border-top-color: hsl(var(--er) / 1)}
.hover\:border-t-error-content:hover {
    border-top-color: hsl(var(--erc) / 1)}
.hover\:border-t-error-content\/0:hover {
    border-top-color: hsl(var(--erc) / 0)}
.hover\:border-t-error-content\/10:hover {
    border-top-color: hsl(var(--erc) / 0.1)}
.hover\:border-t-error-content\/100:hover {
    border-top-color: hsl(var(--erc) / 1)}
.hover\:border-t-error-content\/20:hover {
    border-top-color: hsl(var(--erc) / 0.2)}
.hover\:border-t-error-content\/25:hover {
    border-top-color: hsl(var(--erc) / 0.25)}
.hover\:border-t-error-content\/30:hover {
    border-top-color: hsl(var(--erc) / 0.3)}
.hover\:border-t-error-content\/40:hover {
    border-top-color: hsl(var(--erc) / 0.4)}
.hover\:border-t-error-content\/5:hover {
    border-top-color: hsl(var(--erc) / 0.05)}
.hover\:border-t-error-content\/50:hover {
    border-top-color: hsl(var(--erc) / 0.5)}
.hover\:border-t-error-content\/60:hover {
    border-top-color: hsl(var(--erc) / 0.6)}
.hover\:border-t-error-content\/70:hover {
    border-top-color: hsl(var(--erc) / 0.7)}
.hover\:border-t-error-content\/75:hover {
    border-top-color: hsl(var(--erc) / 0.75)}
.hover\:border-t-error-content\/80:hover {
    border-top-color: hsl(var(--erc) / 0.8)}
.hover\:border-t-error-content\/90:hover {
    border-top-color: hsl(var(--erc) / 0.9)}
.hover\:border-t-error-content\/95:hover {
    border-top-color: hsl(var(--erc) / 0.95)}
.hover\:border-t-error\/0:hover {
    border-top-color: hsl(var(--er) / 0)}
.hover\:border-t-error\/10:hover {
    border-top-color: hsl(var(--er) / 0.1)}
.hover\:border-t-error\/100:hover {
    border-top-color: hsl(var(--er) / 1)}
.hover\:border-t-error\/20:hover {
    border-top-color: hsl(var(--er) / 0.2)}
.hover\:border-t-error\/25:hover {
    border-top-color: hsl(var(--er) / 0.25)}
.hover\:border-t-error\/30:hover {
    border-top-color: hsl(var(--er) / 0.3)}
.hover\:border-t-error\/40:hover {
    border-top-color: hsl(var(--er) / 0.4)}
.hover\:border-t-error\/5:hover {
    border-top-color: hsl(var(--er) / 0.05)}
.hover\:border-t-error\/50:hover {
    border-top-color: hsl(var(--er) / 0.5)}
.hover\:border-t-error\/60:hover {
    border-top-color: hsl(var(--er) / 0.6)}
.hover\:border-t-error\/70:hover {
    border-top-color: hsl(var(--er) / 0.7)}
.hover\:border-t-error\/75:hover {
    border-top-color: hsl(var(--er) / 0.75)}
.hover\:border-t-error\/80:hover {
    border-top-color: hsl(var(--er) / 0.8)}
.hover\:border-t-error\/90:hover {
    border-top-color: hsl(var(--er) / 0.9)}
.hover\:border-t-error\/95:hover {
    border-top-color: hsl(var(--er) / 0.95)}
.hover\:border-t-info:hover {
    border-top-color: hsl(var(--in) / 1)}
.hover\:border-t-info-content:hover {
    border-top-color: hsl(var(--inc) / 1)}
.hover\:border-t-info-content\/0:hover {
    border-top-color: hsl(var(--inc) / 0)}
.hover\:border-t-info-content\/10:hover {
    border-top-color: hsl(var(--inc) / 0.1)}
.hover\:border-t-info-content\/100:hover {
    border-top-color: hsl(var(--inc) / 1)}
.hover\:border-t-info-content\/20:hover {
    border-top-color: hsl(var(--inc) / 0.2)}
.hover\:border-t-info-content\/25:hover {
    border-top-color: hsl(var(--inc) / 0.25)}
.hover\:border-t-info-content\/30:hover {
    border-top-color: hsl(var(--inc) / 0.3)}
.hover\:border-t-info-content\/40:hover {
    border-top-color: hsl(var(--inc) / 0.4)}
.hover\:border-t-info-content\/5:hover {
    border-top-color: hsl(var(--inc) / 0.05)}
.hover\:border-t-info-content\/50:hover {
    border-top-color: hsl(var(--inc) / 0.5)}
.hover\:border-t-info-content\/60:hover {
    border-top-color: hsl(var(--inc) / 0.6)}
.hover\:border-t-info-content\/70:hover {
    border-top-color: hsl(var(--inc) / 0.7)}
.hover\:border-t-info-content\/75:hover {
    border-top-color: hsl(var(--inc) / 0.75)}
.hover\:border-t-info-content\/80:hover {
    border-top-color: hsl(var(--inc) / 0.8)}
.hover\:border-t-info-content\/90:hover {
    border-top-color: hsl(var(--inc) / 0.9)}
.hover\:border-t-info-content\/95:hover {
    border-top-color: hsl(var(--inc) / 0.95)}
.hover\:border-t-info\/0:hover {
    border-top-color: hsl(var(--in) / 0)}
.hover\:border-t-info\/10:hover {
    border-top-color: hsl(var(--in) / 0.1)}
.hover\:border-t-info\/100:hover {
    border-top-color: hsl(var(--in) / 1)}
.hover\:border-t-info\/20:hover {
    border-top-color: hsl(var(--in) / 0.2)}
.hover\:border-t-info\/25:hover {
    border-top-color: hsl(var(--in) / 0.25)}
.hover\:border-t-info\/30:hover {
    border-top-color: hsl(var(--in) / 0.3)}
.hover\:border-t-info\/40:hover {
    border-top-color: hsl(var(--in) / 0.4)}
.hover\:border-t-info\/5:hover {
    border-top-color: hsl(var(--in) / 0.05)}
.hover\:border-t-info\/50:hover {
    border-top-color: hsl(var(--in) / 0.5)}
.hover\:border-t-info\/60:hover {
    border-top-color: hsl(var(--in) / 0.6)}
.hover\:border-t-info\/70:hover {
    border-top-color: hsl(var(--in) / 0.7)}
.hover\:border-t-info\/75:hover {
    border-top-color: hsl(var(--in) / 0.75)}
.hover\:border-t-info\/80:hover {
    border-top-color: hsl(var(--in) / 0.8)}
.hover\:border-t-info\/90:hover {
    border-top-color: hsl(var(--in) / 0.9)}
.hover\:border-t-info\/95:hover {
    border-top-color: hsl(var(--in) / 0.95)}
.hover\:border-t-success:hover {
    border-top-color: hsl(var(--su) / 1)}
.hover\:border-t-success-content:hover {
    border-top-color: hsl(var(--suc) / 1)}
.hover\:border-t-success-content\/0:hover {
    border-top-color: hsl(var(--suc) / 0)}
.hover\:border-t-success-content\/10:hover {
    border-top-color: hsl(var(--suc) / 0.1)}
.hover\:border-t-success-content\/100:hover {
    border-top-color: hsl(var(--suc) / 1)}
.hover\:border-t-success-content\/20:hover {
    border-top-color: hsl(var(--suc) / 0.2)}
.hover\:border-t-success-content\/25:hover {
    border-top-color: hsl(var(--suc) / 0.25)}
.hover\:border-t-success-content\/30:hover {
    border-top-color: hsl(var(--suc) / 0.3)}
.hover\:border-t-success-content\/40:hover {
    border-top-color: hsl(var(--suc) / 0.4)}
.hover\:border-t-success-content\/5:hover {
    border-top-color: hsl(var(--suc) / 0.05)}
.hover\:border-t-success-content\/50:hover {
    border-top-color: hsl(var(--suc) / 0.5)}
.hover\:border-t-success-content\/60:hover {
    border-top-color: hsl(var(--suc) / 0.6)}
.hover\:border-t-success-content\/70:hover {
    border-top-color: hsl(var(--suc) / 0.7)}
.hover\:border-t-success-content\/75:hover {
    border-top-color: hsl(var(--suc) / 0.75)}
.hover\:border-t-success-content\/80:hover {
    border-top-color: hsl(var(--suc) / 0.8)}
.hover\:border-t-success-content\/90:hover {
    border-top-color: hsl(var(--suc) / 0.9)}
.hover\:border-t-success-content\/95:hover {
    border-top-color: hsl(var(--suc) / 0.95)}
.hover\:border-t-success\/0:hover {
    border-top-color: hsl(var(--su) / 0)}
.hover\:border-t-success\/10:hover {
    border-top-color: hsl(var(--su) / 0.1)}
.hover\:border-t-success\/100:hover {
    border-top-color: hsl(var(--su) / 1)}
.hover\:border-t-success\/20:hover {
    border-top-color: hsl(var(--su) / 0.2)}
.hover\:border-t-success\/25:hover {
    border-top-color: hsl(var(--su) / 0.25)}
.hover\:border-t-success\/30:hover {
    border-top-color: hsl(var(--su) / 0.3)}
.hover\:border-t-success\/40:hover {
    border-top-color: hsl(var(--su) / 0.4)}
.hover\:border-t-success\/5:hover {
    border-top-color: hsl(var(--su) / 0.05)}
.hover\:border-t-success\/50:hover {
    border-top-color: hsl(var(--su) / 0.5)}
.hover\:border-t-success\/60:hover {
    border-top-color: hsl(var(--su) / 0.6)}
.hover\:border-t-success\/70:hover {
    border-top-color: hsl(var(--su) / 0.7)}
.hover\:border-t-success\/75:hover {
    border-top-color: hsl(var(--su) / 0.75)}
.hover\:border-t-success\/80:hover {
    border-top-color: hsl(var(--su) / 0.8)}
.hover\:border-t-success\/90:hover {
    border-top-color: hsl(var(--su) / 0.9)}
.hover\:border-t-success\/95:hover {
    border-top-color: hsl(var(--su) / 0.95)}
.hover\:border-t-warning:hover {
    border-top-color: hsl(var(--wa) / 1)}
.hover\:border-t-warning-content:hover {
    border-top-color: hsl(var(--wac) / 1)}
.hover\:border-t-warning-content\/0:hover {
    border-top-color: hsl(var(--wac) / 0)}
.hover\:border-t-warning-content\/10:hover {
    border-top-color: hsl(var(--wac) / 0.1)}
.hover\:border-t-warning-content\/100:hover {
    border-top-color: hsl(var(--wac) / 1)}
.hover\:border-t-warning-content\/20:hover {
    border-top-color: hsl(var(--wac) / 0.2)}
.hover\:border-t-warning-content\/25:hover {
    border-top-color: hsl(var(--wac) / 0.25)}
.hover\:border-t-warning-content\/30:hover {
    border-top-color: hsl(var(--wac) / 0.3)}
.hover\:border-t-warning-content\/40:hover {
    border-top-color: hsl(var(--wac) / 0.4)}
.hover\:border-t-warning-content\/5:hover {
    border-top-color: hsl(var(--wac) / 0.05)}
.hover\:border-t-warning-content\/50:hover {
    border-top-color: hsl(var(--wac) / 0.5)}
.hover\:border-t-warning-content\/60:hover {
    border-top-color: hsl(var(--wac) / 0.6)}
.hover\:border-t-warning-content\/70:hover {
    border-top-color: hsl(var(--wac) / 0.7)}
.hover\:border-t-warning-content\/75:hover {
    border-top-color: hsl(var(--wac) / 0.75)}
.hover\:border-t-warning-content\/80:hover {
    border-top-color: hsl(var(--wac) / 0.8)}
.hover\:border-t-warning-content\/90:hover {
    border-top-color: hsl(var(--wac) / 0.9)}
.hover\:border-t-warning-content\/95:hover {
    border-top-color: hsl(var(--wac) / 0.95)}
.hover\:border-t-warning\/0:hover {
    border-top-color: hsl(var(--wa) / 0)}
.hover\:border-t-warning\/10:hover {
    border-top-color: hsl(var(--wa) / 0.1)}
.hover\:border-t-warning\/100:hover {
    border-top-color: hsl(var(--wa) / 1)}
.hover\:border-t-warning\/20:hover {
    border-top-color: hsl(var(--wa) / 0.2)}
.hover\:border-t-warning\/25:hover {
    border-top-color: hsl(var(--wa) / 0.25)}
.hover\:border-t-warning\/30:hover {
    border-top-color: hsl(var(--wa) / 0.3)}
.hover\:border-t-warning\/40:hover {
    border-top-color: hsl(var(--wa) / 0.4)}
.hover\:border-t-warning\/5:hover {
    border-top-color: hsl(var(--wa) / 0.05)}
.hover\:border-t-warning\/50:hover {
    border-top-color: hsl(var(--wa) / 0.5)}
.hover\:border-t-warning\/60:hover {
    border-top-color: hsl(var(--wa) / 0.6)}
.hover\:border-t-warning\/70:hover {
    border-top-color: hsl(var(--wa) / 0.7)}
.hover\:border-t-warning\/75:hover {
    border-top-color: hsl(var(--wa) / 0.75)}
.hover\:border-t-warning\/80:hover {
    border-top-color: hsl(var(--wa) / 0.8)}
.hover\:border-t-warning\/90:hover {
    border-top-color: hsl(var(--wa) / 0.9)}
.hover\:border-t-warning\/95:hover {
    border-top-color: hsl(var(--wa) / 0.95)}
.hover\:bg-accent:hover {
    background-color: hsl(var(--a) / 1)}
.hover\:bg-accent-content:hover {
    background-color: hsl(var(--ac) / 1)}
.hover\:bg-accent-content\/0:hover {
    background-color: hsl(var(--ac) / 0)}
.hover\:bg-accent-content\/10:hover {
    background-color: hsl(var(--ac) / 0.1)}
.hover\:bg-accent-content\/100:hover {
    background-color: hsl(var(--ac) / 1)}
.hover\:bg-accent-content\/20:hover {
    background-color: hsl(var(--ac) / 0.2)}
.hover\:bg-accent-content\/25:hover {
    background-color: hsl(var(--ac) / 0.25)}
.hover\:bg-accent-content\/30:hover {
    background-color: hsl(var(--ac) / 0.3)}
.hover\:bg-accent-content\/40:hover {
    background-color: hsl(var(--ac) / 0.4)}
.hover\:bg-accent-content\/5:hover {
    background-color: hsl(var(--ac) / 0.05)}
.hover\:bg-accent-content\/50:hover {
    background-color: hsl(var(--ac) / 0.5)}
.hover\:bg-accent-content\/60:hover {
    background-color: hsl(var(--ac) / 0.6)}
.hover\:bg-accent-content\/70:hover {
    background-color: hsl(var(--ac) / 0.7)}
.hover\:bg-accent-content\/75:hover {
    background-color: hsl(var(--ac) / 0.75)}
.hover\:bg-accent-content\/80:hover {
    background-color: hsl(var(--ac) / 0.8)}
.hover\:bg-accent-content\/90:hover {
    background-color: hsl(var(--ac) / 0.9)}
.hover\:bg-accent-content\/95:hover {
    background-color: hsl(var(--ac) / 0.95)}
.hover\:bg-accent-focus:hover {
    background-color: hsl(var(--af) / 1)}
.hover\:bg-accent-focus\/0:hover {
    background-color: hsl(var(--af) / 0)}
.hover\:bg-accent-focus\/10:hover {
    background-color: hsl(var(--af) / 0.1)}
.hover\:bg-accent-focus\/100:hover {
    background-color: hsl(var(--af) / 1)}
.hover\:bg-accent-focus\/20:hover {
    background-color: hsl(var(--af) / 0.2)}
.hover\:bg-accent-focus\/25:hover {
    background-color: hsl(var(--af) / 0.25)}
.hover\:bg-accent-focus\/30:hover {
    background-color: hsl(var(--af) / 0.3)}
.hover\:bg-accent-focus\/40:hover {
    background-color: hsl(var(--af) / 0.4)}
.hover\:bg-accent-focus\/5:hover {
    background-color: hsl(var(--af) / 0.05)}
.hover\:bg-accent-focus\/50:hover {
    background-color: hsl(var(--af) / 0.5)}
.hover\:bg-accent-focus\/60:hover {
    background-color: hsl(var(--af) / 0.6)}
.hover\:bg-accent-focus\/70:hover {
    background-color: hsl(var(--af) / 0.7)}
.hover\:bg-accent-focus\/75:hover {
    background-color: hsl(var(--af) / 0.75)}
.hover\:bg-accent-focus\/80:hover {
    background-color: hsl(var(--af) / 0.8)}
.hover\:bg-accent-focus\/90:hover {
    background-color: hsl(var(--af) / 0.9)}
.hover\:bg-accent-focus\/95:hover {
    background-color: hsl(var(--af) / 0.95)}
.hover\:bg-accent\/0:hover {
    background-color: hsl(var(--a) / 0)}
.hover\:bg-accent\/10:hover {
    background-color: hsl(var(--a) / 0.1)}
.hover\:bg-accent\/100:hover {
    background-color: hsl(var(--a) / 1)}
.hover\:bg-accent\/20:hover {
    background-color: hsl(var(--a) / 0.2)}
.hover\:bg-accent\/25:hover {
    background-color: hsl(var(--a) / 0.25)}
.hover\:bg-accent\/30:hover {
    background-color: hsl(var(--a) / 0.3)}
.hover\:bg-accent\/40:hover {
    background-color: hsl(var(--a) / 0.4)}
.hover\:bg-accent\/5:hover {
    background-color: hsl(var(--a) / 0.05)}
.hover\:bg-accent\/50:hover {
    background-color: hsl(var(--a) / 0.5)}
.hover\:bg-accent\/60:hover {
    background-color: hsl(var(--a) / 0.6)}
.hover\:bg-accent\/70:hover {
    background-color: hsl(var(--a) / 0.7)}
.hover\:bg-accent\/75:hover {
    background-color: hsl(var(--a) / 0.75)}
.hover\:bg-accent\/80:hover {
    background-color: hsl(var(--a) / 0.8)}
.hover\:bg-accent\/90:hover {
    background-color: hsl(var(--a) / 0.9)}
.hover\:bg-accent\/95:hover {
    background-color: hsl(var(--a) / 0.95)}
.hover\:bg-base-100:hover {
    background-color: hsl(var(--b1) / 1)}
.hover\:bg-base-100\/0:hover {
    background-color: hsl(var(--b1) / 0)}
.hover\:bg-base-100\/10:hover {
    background-color: hsl(var(--b1) / 0.1)}
.hover\:bg-base-100\/100:hover {
    background-color: hsl(var(--b1) / 1)}
.hover\:bg-base-100\/20:hover {
    background-color: hsl(var(--b1) / 0.2)}
.hover\:bg-base-100\/25:hover {
    background-color: hsl(var(--b1) / 0.25)}
.hover\:bg-base-100\/30:hover {
    background-color: hsl(var(--b1) / 0.3)}
.hover\:bg-base-100\/40:hover {
    background-color: hsl(var(--b1) / 0.4)}
.hover\:bg-base-100\/5:hover {
    background-color: hsl(var(--b1) / 0.05)}
.hover\:bg-base-100\/50:hover {
    background-color: hsl(var(--b1) / 0.5)}
.hover\:bg-base-100\/60:hover {
    background-color: hsl(var(--b1) / 0.6)}
.hover\:bg-base-100\/70:hover {
    background-color: hsl(var(--b1) / 0.7)}
.hover\:bg-base-100\/75:hover {
    background-color: hsl(var(--b1) / 0.75)}
.hover\:bg-base-100\/80:hover {
    background-color: hsl(var(--b1) / 0.8)}
.hover\:bg-base-100\/90:hover {
    background-color: hsl(var(--b1) / 0.9)}
.hover\:bg-base-100\/95:hover {
    background-color: hsl(var(--b1) / 0.95)}
.hover\:bg-base-200:hover {
    background-color: hsl(var(--b2) / 1)}
.hover\:bg-base-200\/0:hover {
    background-color: hsl(var(--b2) / 0)}
.hover\:bg-base-200\/10:hover {
    background-color: hsl(var(--b2) / 0.1)}
.hover\:bg-base-200\/100:hover {
    background-color: hsl(var(--b2) / 1)}
.hover\:bg-base-200\/20:hover {
    background-color: hsl(var(--b2) / 0.2)}
.hover\:bg-base-200\/25:hover {
    background-color: hsl(var(--b2) / 0.25)}
.hover\:bg-base-200\/30:hover {
    background-color: hsl(var(--b2) / 0.3)}
.hover\:bg-base-200\/40:hover {
    background-color: hsl(var(--b2) / 0.4)}
.hover\:bg-base-200\/5:hover {
    background-color: hsl(var(--b2) / 0.05)}
.hover\:bg-base-200\/50:hover {
    background-color: hsl(var(--b2) / 0.5)}
.hover\:bg-base-200\/60:hover {
    background-color: hsl(var(--b2) / 0.6)}
.hover\:bg-base-200\/70:hover {
    background-color: hsl(var(--b2) / 0.7)}
.hover\:bg-base-200\/75:hover {
    background-color: hsl(var(--b2) / 0.75)}
.hover\:bg-base-200\/80:hover {
    background-color: hsl(var(--b2) / 0.8)}
.hover\:bg-base-200\/90:hover {
    background-color: hsl(var(--b2) / 0.9)}
.hover\:bg-base-200\/95:hover {
    background-color: hsl(var(--b2) / 0.95)}
.hover\:bg-base-300:hover {
    background-color: hsl(var(--b3) / 1)}
.hover\:bg-base-300\/0:hover {
    background-color: hsl(var(--b3) / 0)}
.hover\:bg-base-300\/10:hover {
    background-color: hsl(var(--b3) / 0.1)}
.hover\:bg-base-300\/100:hover {
    background-color: hsl(var(--b3) / 1)}
.hover\:bg-base-300\/20:hover {
    background-color: hsl(var(--b3) / 0.2)}
.hover\:bg-base-300\/25:hover {
    background-color: hsl(var(--b3) / 0.25)}
.hover\:bg-base-300\/30:hover {
    background-color: hsl(var(--b3) / 0.3)}
.hover\:bg-base-300\/40:hover {
    background-color: hsl(var(--b3) / 0.4)}
.hover\:bg-base-300\/5:hover {
    background-color: hsl(var(--b3) / 0.05)}
.hover\:bg-base-300\/50:hover {
    background-color: hsl(var(--b3) / 0.5)}
.hover\:bg-base-300\/60:hover {
    background-color: hsl(var(--b3) / 0.6)}
.hover\:bg-base-300\/70:hover {
    background-color: hsl(var(--b3) / 0.7)}
.hover\:bg-base-300\/75:hover {
    background-color: hsl(var(--b3) / 0.75)}
.hover\:bg-base-300\/80:hover {
    background-color: hsl(var(--b3) / 0.8)}
.hover\:bg-base-300\/90:hover {
    background-color: hsl(var(--b3) / 0.9)}
.hover\:bg-base-300\/95:hover {
    background-color: hsl(var(--b3) / 0.95)}
.hover\:bg-base-content:hover {
    background-color: hsl(var(--bc) / 1)}
.hover\:bg-base-content\/0:hover {
    background-color: hsl(var(--bc) / 0)}
.hover\:bg-base-content\/10:hover {
    background-color: hsl(var(--bc) / 0.1)}
.hover\:bg-base-content\/100:hover {
    background-color: hsl(var(--bc) / 1)}
.hover\:bg-base-content\/20:hover {
    background-color: hsl(var(--bc) / 0.2)}
.hover\:bg-base-content\/25:hover {
    background-color: hsl(var(--bc) / 0.25)}
.hover\:bg-base-content\/30:hover {
    background-color: hsl(var(--bc) / 0.3)}
.hover\:bg-base-content\/40:hover {
    background-color: hsl(var(--bc) / 0.4)}
.hover\:bg-base-content\/5:hover {
    background-color: hsl(var(--bc) / 0.05)}
.hover\:bg-base-content\/50:hover {
    background-color: hsl(var(--bc) / 0.5)}
.hover\:bg-base-content\/60:hover {
    background-color: hsl(var(--bc) / 0.6)}
.hover\:bg-base-content\/70:hover {
    background-color: hsl(var(--bc) / 0.7)}
.hover\:bg-base-content\/75:hover {
    background-color: hsl(var(--bc) / 0.75)}
.hover\:bg-base-content\/80:hover {
    background-color: hsl(var(--bc) / 0.8)}
.hover\:bg-base-content\/90:hover {
    background-color: hsl(var(--bc) / 0.9)}
.hover\:bg-base-content\/95:hover {
    background-color: hsl(var(--bc) / 0.95)}
.hover\:bg-error:hover {
    background-color: hsl(var(--er) / 1)}
.hover\:bg-error-content:hover {
    background-color: hsl(var(--erc) / 1)}
.hover\:bg-error-content\/0:hover {
    background-color: hsl(var(--erc) / 0)}
.hover\:bg-error-content\/10:hover {
    background-color: hsl(var(--erc) / 0.1)}
.hover\:bg-error-content\/100:hover {
    background-color: hsl(var(--erc) / 1)}
.hover\:bg-error-content\/20:hover {
    background-color: hsl(var(--erc) / 0.2)}
.hover\:bg-error-content\/25:hover {
    background-color: hsl(var(--erc) / 0.25)}
.hover\:bg-error-content\/30:hover {
    background-color: hsl(var(--erc) / 0.3)}
.hover\:bg-error-content\/40:hover {
    background-color: hsl(var(--erc) / 0.4)}
.hover\:bg-error-content\/5:hover {
    background-color: hsl(var(--erc) / 0.05)}
.hover\:bg-error-content\/50:hover {
    background-color: hsl(var(--erc) / 0.5)}
.hover\:bg-error-content\/60:hover {
    background-color: hsl(var(--erc) / 0.6)}
.hover\:bg-error-content\/70:hover {
    background-color: hsl(var(--erc) / 0.7)}
.hover\:bg-error-content\/75:hover {
    background-color: hsl(var(--erc) / 0.75)}
.hover\:bg-error-content\/80:hover {
    background-color: hsl(var(--erc) / 0.8)}
.hover\:bg-error-content\/90:hover {
    background-color: hsl(var(--erc) / 0.9)}
.hover\:bg-error-content\/95:hover {
    background-color: hsl(var(--erc) / 0.95)}
.hover\:bg-error\/0:hover {
    background-color: hsl(var(--er) / 0)}
.hover\:bg-error\/10:hover {
    background-color: hsl(var(--er) / 0.1)}
.hover\:bg-error\/100:hover {
    background-color: hsl(var(--er) / 1)}
.hover\:bg-error\/20:hover {
    background-color: hsl(var(--er) / 0.2)}
.hover\:bg-error\/25:hover {
    background-color: hsl(var(--er) / 0.25)}
.hover\:bg-error\/30:hover {
    background-color: hsl(var(--er) / 0.3)}
.hover\:bg-error\/40:hover {
    background-color: hsl(var(--er) / 0.4)}
.hover\:bg-error\/5:hover {
    background-color: hsl(var(--er) / 0.05)}
.hover\:bg-error\/50:hover {
    background-color: hsl(var(--er) / 0.5)}
.hover\:bg-error\/60:hover {
    background-color: hsl(var(--er) / 0.6)}
.hover\:bg-error\/70:hover {
    background-color: hsl(var(--er) / 0.7)}
.hover\:bg-error\/75:hover {
    background-color: hsl(var(--er) / 0.75)}
.hover\:bg-error\/80:hover {
    background-color: hsl(var(--er) / 0.8)}
.hover\:bg-error\/90:hover {
    background-color: hsl(var(--er) / 0.9)}
.hover\:bg-error\/95:hover {
    background-color: hsl(var(--er) / 0.95)}
.hover\:bg-info:hover {
    background-color: hsl(var(--in) / 1)}
.hover\:bg-info-content:hover {
    background-color: hsl(var(--inc) / 1)}
.hover\:bg-info-content\/0:hover {
    background-color: hsl(var(--inc) / 0)}
.hover\:bg-info-content\/10:hover {
    background-color: hsl(var(--inc) / 0.1)}
.hover\:bg-info-content\/100:hover {
    background-color: hsl(var(--inc) / 1)}
.hover\:bg-info-content\/20:hover {
    background-color: hsl(var(--inc) / 0.2)}
.hover\:bg-info-content\/25:hover {
    background-color: hsl(var(--inc) / 0.25)}
.hover\:bg-info-content\/30:hover {
    background-color: hsl(var(--inc) / 0.3)}
.hover\:bg-info-content\/40:hover {
    background-color: hsl(var(--inc) / 0.4)}
.hover\:bg-info-content\/5:hover {
    background-color: hsl(var(--inc) / 0.05)}
.hover\:bg-info-content\/50:hover {
    background-color: hsl(var(--inc) / 0.5)}
.hover\:bg-info-content\/60:hover {
    background-color: hsl(var(--inc) / 0.6)}
.hover\:bg-info-content\/70:hover {
    background-color: hsl(var(--inc) / 0.7)}
.hover\:bg-info-content\/75:hover {
    background-color: hsl(var(--inc) / 0.75)}
.hover\:bg-info-content\/80:hover {
    background-color: hsl(var(--inc) / 0.8)}
.hover\:bg-info-content\/90:hover {
    background-color: hsl(var(--inc) / 0.9)}
.hover\:bg-info-content\/95:hover {
    background-color: hsl(var(--inc) / 0.95)}
.hover\:bg-info\/0:hover {
    background-color: hsl(var(--in) / 0)}
.hover\:bg-info\/10:hover {
    background-color: hsl(var(--in) / 0.1)}
.hover\:bg-info\/100:hover {
    background-color: hsl(var(--in) / 1)}
.hover\:bg-info\/20:hover {
    background-color: hsl(var(--in) / 0.2)}
.hover\:bg-info\/25:hover {
    background-color: hsl(var(--in) / 0.25)}
.hover\:bg-info\/30:hover {
    background-color: hsl(var(--in) / 0.3)}
.hover\:bg-info\/40:hover {
    background-color: hsl(var(--in) / 0.4)}
.hover\:bg-info\/5:hover {
    background-color: hsl(var(--in) / 0.05)}
.hover\:bg-info\/50:hover {
    background-color: hsl(var(--in) / 0.5)}
.hover\:bg-info\/60:hover {
    background-color: hsl(var(--in) / 0.6)}
.hover\:bg-info\/70:hover {
    background-color: hsl(var(--in) / 0.7)}
.hover\:bg-info\/75:hover {
    background-color: hsl(var(--in) / 0.75)}
.hover\:bg-info\/80:hover {
    background-color: hsl(var(--in) / 0.8)}
.hover\:bg-info\/90:hover {
    background-color: hsl(var(--in) / 0.9)}
.hover\:bg-info\/95:hover {
    background-color: hsl(var(--in) / 0.95)}
.hover\:bg-neutral:hover {
    background-color: hsl(var(--n) / 1)}
.hover\:bg-neutral-content:hover {
    background-color: hsl(var(--nc) / 1)}
.hover\:bg-neutral-content\/0:hover {
    background-color: hsl(var(--nc) / 0)}
.hover\:bg-neutral-content\/10:hover {
    background-color: hsl(var(--nc) / 0.1)}
.hover\:bg-neutral-content\/100:hover {
    background-color: hsl(var(--nc) / 1)}
.hover\:bg-neutral-content\/20:hover {
    background-color: hsl(var(--nc) / 0.2)}
.hover\:bg-neutral-content\/25:hover {
    background-color: hsl(var(--nc) / 0.25)}
.hover\:bg-neutral-content\/30:hover {
    background-color: hsl(var(--nc) / 0.3)}
.hover\:bg-neutral-content\/40:hover {
    background-color: hsl(var(--nc) / 0.4)}
.hover\:bg-neutral-content\/5:hover {
    background-color: hsl(var(--nc) / 0.05)}
.hover\:bg-neutral-content\/50:hover {
    background-color: hsl(var(--nc) / 0.5)}
.hover\:bg-neutral-content\/60:hover {
    background-color: hsl(var(--nc) / 0.6)}
.hover\:bg-neutral-content\/70:hover {
    background-color: hsl(var(--nc) / 0.7)}
.hover\:bg-neutral-content\/75:hover {
    background-color: hsl(var(--nc) / 0.75)}
.hover\:bg-neutral-content\/80:hover {
    background-color: hsl(var(--nc) / 0.8)}
.hover\:bg-neutral-content\/90:hover {
    background-color: hsl(var(--nc) / 0.9)}
.hover\:bg-neutral-content\/95:hover {
    background-color: hsl(var(--nc) / 0.95)}
.hover\:bg-neutral-focus:hover {
    background-color: hsl(var(--nf) / 1)}
.hover\:bg-neutral-focus\/0:hover {
    background-color: hsl(var(--nf) / 0)}
.hover\:bg-neutral-focus\/10:hover {
    background-color: hsl(var(--nf) / 0.1)}
.hover\:bg-neutral-focus\/100:hover {
    background-color: hsl(var(--nf) / 1)}
.hover\:bg-neutral-focus\/20:hover {
    background-color: hsl(var(--nf) / 0.2)}
.hover\:bg-neutral-focus\/25:hover {
    background-color: hsl(var(--nf) / 0.25)}
.hover\:bg-neutral-focus\/30:hover {
    background-color: hsl(var(--nf) / 0.3)}
.hover\:bg-neutral-focus\/40:hover {
    background-color: hsl(var(--nf) / 0.4)}
.hover\:bg-neutral-focus\/5:hover {
    background-color: hsl(var(--nf) / 0.05)}
.hover\:bg-neutral-focus\/50:hover {
    background-color: hsl(var(--nf) / 0.5)}
.hover\:bg-neutral-focus\/60:hover {
    background-color: hsl(var(--nf) / 0.6)}
.hover\:bg-neutral-focus\/70:hover {
    background-color: hsl(var(--nf) / 0.7)}
.hover\:bg-neutral-focus\/75:hover {
    background-color: hsl(var(--nf) / 0.75)}
.hover\:bg-neutral-focus\/80:hover {
    background-color: hsl(var(--nf) / 0.8)}
.hover\:bg-neutral-focus\/90:hover {
    background-color: hsl(var(--nf) / 0.9)}
.hover\:bg-neutral-focus\/95:hover {
    background-color: hsl(var(--nf) / 0.95)}
.hover\:bg-neutral\/0:hover {
    background-color: hsl(var(--n) / 0)}
.hover\:bg-neutral\/10:hover {
    background-color: hsl(var(--n) / 0.1)}
.hover\:bg-neutral\/100:hover {
    background-color: hsl(var(--n) / 1)}
.hover\:bg-neutral\/20:hover {
    background-color: hsl(var(--n) / 0.2)}
.hover\:bg-neutral\/25:hover {
    background-color: hsl(var(--n) / 0.25)}
.hover\:bg-neutral\/30:hover {
    background-color: hsl(var(--n) / 0.3)}
.hover\:bg-neutral\/40:hover {
    background-color: hsl(var(--n) / 0.4)}
.hover\:bg-neutral\/5:hover {
    background-color: hsl(var(--n) / 0.05)}
.hover\:bg-neutral\/50:hover {
    background-color: hsl(var(--n) / 0.5)}
.hover\:bg-neutral\/60:hover {
    background-color: hsl(var(--n) / 0.6)}
.hover\:bg-neutral\/70:hover {
    background-color: hsl(var(--n) / 0.7)}
.hover\:bg-neutral\/75:hover {
    background-color: hsl(var(--n) / 0.75)}
.hover\:bg-neutral\/80:hover {
    background-color: hsl(var(--n) / 0.8)}
.hover\:bg-neutral\/90:hover {
    background-color: hsl(var(--n) / 0.9)}
.hover\:bg-neutral\/95:hover {
    background-color: hsl(var(--n) / 0.95)}
.hover\:bg-primary:hover {
    background-color: hsl(var(--p) / 1)}
.hover\:bg-primary-content:hover {
    background-color: hsl(var(--pc) / 1)}
.hover\:bg-primary-content\/0:hover {
    background-color: hsl(var(--pc) / 0)}
.hover\:bg-primary-content\/10:hover {
    background-color: hsl(var(--pc) / 0.1)}
.hover\:bg-primary-content\/100:hover {
    background-color: hsl(var(--pc) / 1)}
.hover\:bg-primary-content\/20:hover {
    background-color: hsl(var(--pc) / 0.2)}
.hover\:bg-primary-content\/25:hover {
    background-color: hsl(var(--pc) / 0.25)}
.hover\:bg-primary-content\/30:hover {
    background-color: hsl(var(--pc) / 0.3)}
.hover\:bg-primary-content\/40:hover {
    background-color: hsl(var(--pc) / 0.4)}
.hover\:bg-primary-content\/5:hover {
    background-color: hsl(var(--pc) / 0.05)}
.hover\:bg-primary-content\/50:hover {
    background-color: hsl(var(--pc) / 0.5)}
.hover\:bg-primary-content\/60:hover {
    background-color: hsl(var(--pc) / 0.6)}
.hover\:bg-primary-content\/70:hover {
    background-color: hsl(var(--pc) / 0.7)}
.hover\:bg-primary-content\/75:hover {
    background-color: hsl(var(--pc) / 0.75)}
.hover\:bg-primary-content\/80:hover {
    background-color: hsl(var(--pc) / 0.8)}
.hover\:bg-primary-content\/90:hover {
    background-color: hsl(var(--pc) / 0.9)}
.hover\:bg-primary-content\/95:hover {
    background-color: hsl(var(--pc) / 0.95)}
.hover\:bg-primary-focus:hover {
    background-color: hsl(var(--pf) / 1)}
.hover\:bg-primary-focus\/0:hover {
    background-color: hsl(var(--pf) / 0)}
.hover\:bg-primary-focus\/10:hover {
    background-color: hsl(var(--pf) / 0.1)}
.hover\:bg-primary-focus\/100:hover {
    background-color: hsl(var(--pf) / 1)}
.hover\:bg-primary-focus\/20:hover {
    background-color: hsl(var(--pf) / 0.2)}
.hover\:bg-primary-focus\/25:hover {
    background-color: hsl(var(--pf) / 0.25)}
.hover\:bg-primary-focus\/30:hover {
    background-color: hsl(var(--pf) / 0.3)}
.hover\:bg-primary-focus\/40:hover {
    background-color: hsl(var(--pf) / 0.4)}
.hover\:bg-primary-focus\/5:hover {
    background-color: hsl(var(--pf) / 0.05)}
.hover\:bg-primary-focus\/50:hover {
    background-color: hsl(var(--pf) / 0.5)}
.hover\:bg-primary-focus\/60:hover {
    background-color: hsl(var(--pf) / 0.6)}
.hover\:bg-primary-focus\/70:hover {
    background-color: hsl(var(--pf) / 0.7)}
.hover\:bg-primary-focus\/75:hover {
    background-color: hsl(var(--pf) / 0.75)}
.hover\:bg-primary-focus\/80:hover {
    background-color: hsl(var(--pf) / 0.8)}
.hover\:bg-primary-focus\/90:hover {
    background-color: hsl(var(--pf) / 0.9)}
.hover\:bg-primary-focus\/95:hover {
    background-color: hsl(var(--pf) / 0.95)}
.hover\:bg-primary\/0:hover {
    background-color: hsl(var(--p) / 0)}
.hover\:bg-primary\/10:hover {
    background-color: hsl(var(--p) / 0.1)}
.hover\:bg-primary\/100:hover {
    background-color: hsl(var(--p) / 1)}
.hover\:bg-primary\/20:hover {
    background-color: hsl(var(--p) / 0.2)}
.hover\:bg-primary\/25:hover {
    background-color: hsl(var(--p) / 0.25)}
.hover\:bg-primary\/30:hover {
    background-color: hsl(var(--p) / 0.3)}
.hover\:bg-primary\/40:hover {
    background-color: hsl(var(--p) / 0.4)}
.hover\:bg-primary\/5:hover {
    background-color: hsl(var(--p) / 0.05)}
.hover\:bg-primary\/50:hover {
    background-color: hsl(var(--p) / 0.5)}
.hover\:bg-primary\/60:hover {
    background-color: hsl(var(--p) / 0.6)}
.hover\:bg-primary\/70:hover {
    background-color: hsl(var(--p) / 0.7)}
.hover\:bg-primary\/75:hover {
    background-color: hsl(var(--p) / 0.75)}
.hover\:bg-primary\/80:hover {
    background-color: hsl(var(--p) / 0.8)}
.hover\:bg-primary\/90:hover {
    background-color: hsl(var(--p) / 0.9)}
.hover\:bg-primary\/95:hover {
    background-color: hsl(var(--p) / 0.95)}
.hover\:bg-secondary:hover {
    background-color: hsl(var(--s) / 1)}
.hover\:bg-secondary-content:hover {
    background-color: hsl(var(--sc) / 1)}
.hover\:bg-secondary-content\/0:hover {
    background-color: hsl(var(--sc) / 0)}
.hover\:bg-secondary-content\/10:hover {
    background-color: hsl(var(--sc) / 0.1)}
.hover\:bg-secondary-content\/100:hover {
    background-color: hsl(var(--sc) / 1)}
.hover\:bg-secondary-content\/20:hover {
    background-color: hsl(var(--sc) / 0.2)}
.hover\:bg-secondary-content\/25:hover {
    background-color: hsl(var(--sc) / 0.25)}
.hover\:bg-secondary-content\/30:hover {
    background-color: hsl(var(--sc) / 0.3)}
.hover\:bg-secondary-content\/40:hover {
    background-color: hsl(var(--sc) / 0.4)}
.hover\:bg-secondary-content\/5:hover {
    background-color: hsl(var(--sc) / 0.05)}
.hover\:bg-secondary-content\/50:hover {
    background-color: hsl(var(--sc) / 0.5)}
.hover\:bg-secondary-content\/60:hover {
    background-color: hsl(var(--sc) / 0.6)}
.hover\:bg-secondary-content\/70:hover {
    background-color: hsl(var(--sc) / 0.7)}
.hover\:bg-secondary-content\/75:hover {
    background-color: hsl(var(--sc) / 0.75)}
.hover\:bg-secondary-content\/80:hover {
    background-color: hsl(var(--sc) / 0.8)}
.hover\:bg-secondary-content\/90:hover {
    background-color: hsl(var(--sc) / 0.9)}
.hover\:bg-secondary-content\/95:hover {
    background-color: hsl(var(--sc) / 0.95)}
.hover\:bg-secondary-focus:hover {
    background-color: hsl(var(--sf) / 1)}
.hover\:bg-secondary-focus\/0:hover {
    background-color: hsl(var(--sf) / 0)}
.hover\:bg-secondary-focus\/10:hover {
    background-color: hsl(var(--sf) / 0.1)}
.hover\:bg-secondary-focus\/100:hover {
    background-color: hsl(var(--sf) / 1)}
.hover\:bg-secondary-focus\/20:hover {
    background-color: hsl(var(--sf) / 0.2)}
.hover\:bg-secondary-focus\/25:hover {
    background-color: hsl(var(--sf) / 0.25)}
.hover\:bg-secondary-focus\/30:hover {
    background-color: hsl(var(--sf) / 0.3)}
.hover\:bg-secondary-focus\/40:hover {
    background-color: hsl(var(--sf) / 0.4)}
.hover\:bg-secondary-focus\/5:hover {
    background-color: hsl(var(--sf) / 0.05)}
.hover\:bg-secondary-focus\/50:hover {
    background-color: hsl(var(--sf) / 0.5)}
.hover\:bg-secondary-focus\/60:hover {
    background-color: hsl(var(--sf) / 0.6)}
.hover\:bg-secondary-focus\/70:hover {
    background-color: hsl(var(--sf) / 0.7)}
.hover\:bg-secondary-focus\/75:hover {
    background-color: hsl(var(--sf) / 0.75)}
.hover\:bg-secondary-focus\/80:hover {
    background-color: hsl(var(--sf) / 0.8)}
.hover\:bg-secondary-focus\/90:hover {
    background-color: hsl(var(--sf) / 0.9)}
.hover\:bg-secondary-focus\/95:hover {
    background-color: hsl(var(--sf) / 0.95)}
.hover\:bg-secondary\/0:hover {
    background-color: hsl(var(--s) / 0)}
.hover\:bg-secondary\/10:hover {
    background-color: hsl(var(--s) / 0.1)}
.hover\:bg-secondary\/100:hover {
    background-color: hsl(var(--s) / 1)}
.hover\:bg-secondary\/20:hover {
    background-color: hsl(var(--s) / 0.2)}
.hover\:bg-secondary\/25:hover {
    background-color: hsl(var(--s) / 0.25)}
.hover\:bg-secondary\/30:hover {
    background-color: hsl(var(--s) / 0.3)}
.hover\:bg-secondary\/40:hover {
    background-color: hsl(var(--s) / 0.4)}
.hover\:bg-secondary\/5:hover {
    background-color: hsl(var(--s) / 0.05)}
.hover\:bg-secondary\/50:hover {
    background-color: hsl(var(--s) / 0.5)}
.hover\:bg-secondary\/60:hover {
    background-color: hsl(var(--s) / 0.6)}
.hover\:bg-secondary\/70:hover {
    background-color: hsl(var(--s) / 0.7)}
.hover\:bg-secondary\/75:hover {
    background-color: hsl(var(--s) / 0.75)}
.hover\:bg-secondary\/80:hover {
    background-color: hsl(var(--s) / 0.8)}
.hover\:bg-secondary\/90:hover {
    background-color: hsl(var(--s) / 0.9)}
.hover\:bg-secondary\/95:hover {
    background-color: hsl(var(--s) / 0.95)}
.hover\:bg-success:hover {
    background-color: hsl(var(--su) / 1)}
.hover\:bg-success-content:hover {
    background-color: hsl(var(--suc) / 1)}
.hover\:bg-success-content\/0:hover {
    background-color: hsl(var(--suc) / 0)}
.hover\:bg-success-content\/10:hover {
    background-color: hsl(var(--suc) / 0.1)}
.hover\:bg-success-content\/100:hover {
    background-color: hsl(var(--suc) / 1)}
.hover\:bg-success-content\/20:hover {
    background-color: hsl(var(--suc) / 0.2)}
.hover\:bg-success-content\/25:hover {
    background-color: hsl(var(--suc) / 0.25)}
.hover\:bg-success-content\/30:hover {
    background-color: hsl(var(--suc) / 0.3)}
.hover\:bg-success-content\/40:hover {
    background-color: hsl(var(--suc) / 0.4)}
.hover\:bg-success-content\/5:hover {
    background-color: hsl(var(--suc) / 0.05)}
.hover\:bg-success-content\/50:hover {
    background-color: hsl(var(--suc) / 0.5)}
.hover\:bg-success-content\/60:hover {
    background-color: hsl(var(--suc) / 0.6)}
.hover\:bg-success-content\/70:hover {
    background-color: hsl(var(--suc) / 0.7)}
.hover\:bg-success-content\/75:hover {
    background-color: hsl(var(--suc) / 0.75)}
.hover\:bg-success-content\/80:hover {
    background-color: hsl(var(--suc) / 0.8)}
.hover\:bg-success-content\/90:hover {
    background-color: hsl(var(--suc) / 0.9)}
.hover\:bg-success-content\/95:hover {
    background-color: hsl(var(--suc) / 0.95)}
.hover\:bg-success\/0:hover {
    background-color: hsl(var(--su) / 0)}
.hover\:bg-success\/10:hover {
    background-color: hsl(var(--su) / 0.1)}
.hover\:bg-success\/100:hover {
    background-color: hsl(var(--su) / 1)}
.hover\:bg-success\/20:hover {
    background-color: hsl(var(--su) / 0.2)}
.hover\:bg-success\/25:hover {
    background-color: hsl(var(--su) / 0.25)}
.hover\:bg-success\/30:hover {
    background-color: hsl(var(--su) / 0.3)}
.hover\:bg-success\/40:hover {
    background-color: hsl(var(--su) / 0.4)}
.hover\:bg-success\/5:hover {
    background-color: hsl(var(--su) / 0.05)}
.hover\:bg-success\/50:hover {
    background-color: hsl(var(--su) / 0.5)}
.hover\:bg-success\/60:hover {
    background-color: hsl(var(--su) / 0.6)}
.hover\:bg-success\/70:hover {
    background-color: hsl(var(--su) / 0.7)}
.hover\:bg-success\/75:hover {
    background-color: hsl(var(--su) / 0.75)}
.hover\:bg-success\/80:hover {
    background-color: hsl(var(--su) / 0.8)}
.hover\:bg-success\/90:hover {
    background-color: hsl(var(--su) / 0.9)}
.hover\:bg-success\/95:hover {
    background-color: hsl(var(--su) / 0.95)}
.hover\:bg-warning:hover {
    background-color: hsl(var(--wa) / 1)}
.hover\:bg-warning-content:hover {
    background-color: hsl(var(--wac) / 1)}
.hover\:bg-warning-content\/0:hover {
    background-color: hsl(var(--wac) / 0)}
.hover\:bg-warning-content\/10:hover {
    background-color: hsl(var(--wac) / 0.1)}
.hover\:bg-warning-content\/100:hover {
    background-color: hsl(var(--wac) / 1)}
.hover\:bg-warning-content\/20:hover {
    background-color: hsl(var(--wac) / 0.2)}
.hover\:bg-warning-content\/25:hover {
    background-color: hsl(var(--wac) / 0.25)}
.hover\:bg-warning-content\/30:hover {
    background-color: hsl(var(--wac) / 0.3)}
.hover\:bg-warning-content\/40:hover {
    background-color: hsl(var(--wac) / 0.4)}
.hover\:bg-warning-content\/5:hover {
    background-color: hsl(var(--wac) / 0.05)}
.hover\:bg-warning-content\/50:hover {
    background-color: hsl(var(--wac) / 0.5)}
.hover\:bg-warning-content\/60:hover {
    background-color: hsl(var(--wac) / 0.6)}
.hover\:bg-warning-content\/70:hover {
    background-color: hsl(var(--wac) / 0.7)}
.hover\:bg-warning-content\/75:hover {
    background-color: hsl(var(--wac) / 0.75)}
.hover\:bg-warning-content\/80:hover {
    background-color: hsl(var(--wac) / 0.8)}
.hover\:bg-warning-content\/90:hover {
    background-color: hsl(var(--wac) / 0.9)}
.hover\:bg-warning-content\/95:hover {
    background-color: hsl(var(--wac) / 0.95)}
.hover\:bg-warning\/0:hover {
    background-color: hsl(var(--wa) / 0)}
.hover\:bg-warning\/10:hover {
    background-color: hsl(var(--wa) / 0.1)}
.hover\:bg-warning\/100:hover {
    background-color: hsl(var(--wa) / 1)}
.hover\:bg-warning\/20:hover {
    background-color: hsl(var(--wa) / 0.2)}
.hover\:bg-warning\/25:hover {
    background-color: hsl(var(--wa) / 0.25)}
.hover\:bg-warning\/30:hover {
    background-color: hsl(var(--wa) / 0.3)}
.hover\:bg-warning\/40:hover {
    background-color: hsl(var(--wa) / 0.4)}
.hover\:bg-warning\/5:hover {
    background-color: hsl(var(--wa) / 0.05)}
.hover\:bg-warning\/50:hover {
    background-color: hsl(var(--wa) / 0.5)}
.hover\:bg-warning\/60:hover {
    background-color: hsl(var(--wa) / 0.6)}
.hover\:bg-warning\/70:hover {
    background-color: hsl(var(--wa) / 0.7)}
.hover\:bg-warning\/75:hover {
    background-color: hsl(var(--wa) / 0.75)}
.hover\:bg-warning\/80:hover {
    background-color: hsl(var(--wa) / 0.8)}
.hover\:bg-warning\/90:hover {
    background-color: hsl(var(--wa) / 0.9)}
.hover\:bg-warning\/95:hover {
    background-color: hsl(var(--wa) / 0.95)}
.hover\:from-accent:hover {
    --tw-gradient-from: hsl(var(--a) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content:hover {
    --tw-gradient-from: hsl(var(--ac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/0:hover {
    --tw-gradient-from: hsl(var(--ac) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/10:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/100:hover {
    --tw-gradient-from: hsl(var(--ac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/20:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/25:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/30:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/40:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/5:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/50:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/60:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/70:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/75:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/80:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/90:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-content\/95:hover {
    --tw-gradient-from: hsl(var(--ac) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus:hover {
    --tw-gradient-from: hsl(var(--af) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/0:hover {
    --tw-gradient-from: hsl(var(--af) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/10:hover {
    --tw-gradient-from: hsl(var(--af) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/100:hover {
    --tw-gradient-from: hsl(var(--af) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/20:hover {
    --tw-gradient-from: hsl(var(--af) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/25:hover {
    --tw-gradient-from: hsl(var(--af) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/30:hover {
    --tw-gradient-from: hsl(var(--af) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/40:hover {
    --tw-gradient-from: hsl(var(--af) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/5:hover {
    --tw-gradient-from: hsl(var(--af) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/50:hover {
    --tw-gradient-from: hsl(var(--af) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/60:hover {
    --tw-gradient-from: hsl(var(--af) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/70:hover {
    --tw-gradient-from: hsl(var(--af) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/75:hover {
    --tw-gradient-from: hsl(var(--af) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/80:hover {
    --tw-gradient-from: hsl(var(--af) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/90:hover {
    --tw-gradient-from: hsl(var(--af) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent-focus\/95:hover {
    --tw-gradient-from: hsl(var(--af) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/0:hover {
    --tw-gradient-from: hsl(var(--a) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/10:hover {
    --tw-gradient-from: hsl(var(--a) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/100:hover {
    --tw-gradient-from: hsl(var(--a) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/20:hover {
    --tw-gradient-from: hsl(var(--a) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/25:hover {
    --tw-gradient-from: hsl(var(--a) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/30:hover {
    --tw-gradient-from: hsl(var(--a) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/40:hover {
    --tw-gradient-from: hsl(var(--a) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/5:hover {
    --tw-gradient-from: hsl(var(--a) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/50:hover {
    --tw-gradient-from: hsl(var(--a) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/60:hover {
    --tw-gradient-from: hsl(var(--a) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/70:hover {
    --tw-gradient-from: hsl(var(--a) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/75:hover {
    --tw-gradient-from: hsl(var(--a) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/80:hover {
    --tw-gradient-from: hsl(var(--a) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/90:hover {
    --tw-gradient-from: hsl(var(--a) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-accent\/95:hover {
    --tw-gradient-from: hsl(var(--a) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100:hover {
    --tw-gradient-from: hsl(var(--b1) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/0:hover {
    --tw-gradient-from: hsl(var(--b1) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/10:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/100:hover {
    --tw-gradient-from: hsl(var(--b1) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/20:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/25:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/30:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/40:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/5:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/50:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/60:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/70:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/75:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/80:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/90:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-100\/95:hover {
    --tw-gradient-from: hsl(var(--b1) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200:hover {
    --tw-gradient-from: hsl(var(--b2) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/0:hover {
    --tw-gradient-from: hsl(var(--b2) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/10:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/100:hover {
    --tw-gradient-from: hsl(var(--b2) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/20:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/25:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/30:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/40:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/5:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/50:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/60:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/70:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/75:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/80:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/90:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-200\/95:hover {
    --tw-gradient-from: hsl(var(--b2) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300:hover {
    --tw-gradient-from: hsl(var(--b3) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/0:hover {
    --tw-gradient-from: hsl(var(--b3) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/10:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/100:hover {
    --tw-gradient-from: hsl(var(--b3) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/20:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/25:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/30:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/40:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/5:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/50:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/60:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/70:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/75:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/80:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/90:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-300\/95:hover {
    --tw-gradient-from: hsl(var(--b3) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content:hover {
    --tw-gradient-from: hsl(var(--bc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/0:hover {
    --tw-gradient-from: hsl(var(--bc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/10:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/100:hover {
    --tw-gradient-from: hsl(var(--bc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/20:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/25:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/30:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/40:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/5:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/50:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/60:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/70:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/75:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/80:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/90:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-base-content\/95:hover {
    --tw-gradient-from: hsl(var(--bc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error:hover {
    --tw-gradient-from: hsl(var(--er) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content:hover {
    --tw-gradient-from: hsl(var(--erc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/0:hover {
    --tw-gradient-from: hsl(var(--erc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/10:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/100:hover {
    --tw-gradient-from: hsl(var(--erc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/20:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/25:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/30:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/40:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/5:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/50:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/60:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/70:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/75:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/80:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/90:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error-content\/95:hover {
    --tw-gradient-from: hsl(var(--erc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/0:hover {
    --tw-gradient-from: hsl(var(--er) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/10:hover {
    --tw-gradient-from: hsl(var(--er) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/100:hover {
    --tw-gradient-from: hsl(var(--er) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/20:hover {
    --tw-gradient-from: hsl(var(--er) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/25:hover {
    --tw-gradient-from: hsl(var(--er) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/30:hover {
    --tw-gradient-from: hsl(var(--er) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/40:hover {
    --tw-gradient-from: hsl(var(--er) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/5:hover {
    --tw-gradient-from: hsl(var(--er) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/50:hover {
    --tw-gradient-from: hsl(var(--er) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/60:hover {
    --tw-gradient-from: hsl(var(--er) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/70:hover {
    --tw-gradient-from: hsl(var(--er) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/75:hover {
    --tw-gradient-from: hsl(var(--er) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/80:hover {
    --tw-gradient-from: hsl(var(--er) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/90:hover {
    --tw-gradient-from: hsl(var(--er) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-error\/95:hover {
    --tw-gradient-from: hsl(var(--er) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info:hover {
    --tw-gradient-from: hsl(var(--in) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content:hover {
    --tw-gradient-from: hsl(var(--inc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/0:hover {
    --tw-gradient-from: hsl(var(--inc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/10:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/100:hover {
    --tw-gradient-from: hsl(var(--inc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/20:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/25:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/30:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/40:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/5:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/50:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/60:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/70:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/75:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/80:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/90:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info-content\/95:hover {
    --tw-gradient-from: hsl(var(--inc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/0:hover {
    --tw-gradient-from: hsl(var(--in) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/10:hover {
    --tw-gradient-from: hsl(var(--in) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/100:hover {
    --tw-gradient-from: hsl(var(--in) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/20:hover {
    --tw-gradient-from: hsl(var(--in) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/25:hover {
    --tw-gradient-from: hsl(var(--in) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/30:hover {
    --tw-gradient-from: hsl(var(--in) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/40:hover {
    --tw-gradient-from: hsl(var(--in) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/5:hover {
    --tw-gradient-from: hsl(var(--in) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/50:hover {
    --tw-gradient-from: hsl(var(--in) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/60:hover {
    --tw-gradient-from: hsl(var(--in) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/70:hover {
    --tw-gradient-from: hsl(var(--in) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/75:hover {
    --tw-gradient-from: hsl(var(--in) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/80:hover {
    --tw-gradient-from: hsl(var(--in) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/90:hover {
    --tw-gradient-from: hsl(var(--in) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-info\/95:hover {
    --tw-gradient-from: hsl(var(--in) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral:hover {
    --tw-gradient-from: hsl(var(--n) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content:hover {
    --tw-gradient-from: hsl(var(--nc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/0:hover {
    --tw-gradient-from: hsl(var(--nc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/10:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/100:hover {
    --tw-gradient-from: hsl(var(--nc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/20:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/25:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/30:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/40:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/5:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/50:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/60:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/70:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/75:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/80:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/90:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-content\/95:hover {
    --tw-gradient-from: hsl(var(--nc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus:hover {
    --tw-gradient-from: hsl(var(--nf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/0:hover {
    --tw-gradient-from: hsl(var(--nf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/10:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/100:hover {
    --tw-gradient-from: hsl(var(--nf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/20:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/25:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/30:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/40:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/5:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/50:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/60:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/70:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/75:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/80:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/90:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral-focus\/95:hover {
    --tw-gradient-from: hsl(var(--nf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/0:hover {
    --tw-gradient-from: hsl(var(--n) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/10:hover {
    --tw-gradient-from: hsl(var(--n) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/100:hover {
    --tw-gradient-from: hsl(var(--n) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/20:hover {
    --tw-gradient-from: hsl(var(--n) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/25:hover {
    --tw-gradient-from: hsl(var(--n) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/30:hover {
    --tw-gradient-from: hsl(var(--n) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/40:hover {
    --tw-gradient-from: hsl(var(--n) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/5:hover {
    --tw-gradient-from: hsl(var(--n) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/50:hover {
    --tw-gradient-from: hsl(var(--n) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/60:hover {
    --tw-gradient-from: hsl(var(--n) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/70:hover {
    --tw-gradient-from: hsl(var(--n) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/75:hover {
    --tw-gradient-from: hsl(var(--n) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/80:hover {
    --tw-gradient-from: hsl(var(--n) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/90:hover {
    --tw-gradient-from: hsl(var(--n) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-neutral\/95:hover {
    --tw-gradient-from: hsl(var(--n) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary:hover {
    --tw-gradient-from: hsl(var(--p) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content:hover {
    --tw-gradient-from: hsl(var(--pc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/0:hover {
    --tw-gradient-from: hsl(var(--pc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/10:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/100:hover {
    --tw-gradient-from: hsl(var(--pc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/20:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/25:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/30:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/40:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/5:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/50:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/60:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/70:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/75:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/80:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/90:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-content\/95:hover {
    --tw-gradient-from: hsl(var(--pc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus:hover {
    --tw-gradient-from: hsl(var(--pf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/0:hover {
    --tw-gradient-from: hsl(var(--pf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/10:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/100:hover {
    --tw-gradient-from: hsl(var(--pf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/20:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/25:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/30:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/40:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/5:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/50:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/60:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/70:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/75:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/80:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/90:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary-focus\/95:hover {
    --tw-gradient-from: hsl(var(--pf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/0:hover {
    --tw-gradient-from: hsl(var(--p) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/10:hover {
    --tw-gradient-from: hsl(var(--p) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/100:hover {
    --tw-gradient-from: hsl(var(--p) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/20:hover {
    --tw-gradient-from: hsl(var(--p) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/25:hover {
    --tw-gradient-from: hsl(var(--p) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/30:hover {
    --tw-gradient-from: hsl(var(--p) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/40:hover {
    --tw-gradient-from: hsl(var(--p) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/5:hover {
    --tw-gradient-from: hsl(var(--p) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/50:hover {
    --tw-gradient-from: hsl(var(--p) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/60:hover {
    --tw-gradient-from: hsl(var(--p) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/70:hover {
    --tw-gradient-from: hsl(var(--p) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/75:hover {
    --tw-gradient-from: hsl(var(--p) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/80:hover {
    --tw-gradient-from: hsl(var(--p) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/90:hover {
    --tw-gradient-from: hsl(var(--p) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-primary\/95:hover {
    --tw-gradient-from: hsl(var(--p) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary:hover {
    --tw-gradient-from: hsl(var(--s) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content:hover {
    --tw-gradient-from: hsl(var(--sc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/0:hover {
    --tw-gradient-from: hsl(var(--sc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/10:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/100:hover {
    --tw-gradient-from: hsl(var(--sc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/20:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/25:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/30:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/40:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/5:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/50:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/60:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/70:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/75:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/80:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/90:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-content\/95:hover {
    --tw-gradient-from: hsl(var(--sc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus:hover {
    --tw-gradient-from: hsl(var(--sf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/0:hover {
    --tw-gradient-from: hsl(var(--sf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/10:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/100:hover {
    --tw-gradient-from: hsl(var(--sf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/20:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/25:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/30:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/40:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/5:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/50:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/60:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/70:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/75:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/80:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/90:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary-focus\/95:hover {
    --tw-gradient-from: hsl(var(--sf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/0:hover {
    --tw-gradient-from: hsl(var(--s) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/10:hover {
    --tw-gradient-from: hsl(var(--s) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/100:hover {
    --tw-gradient-from: hsl(var(--s) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/20:hover {
    --tw-gradient-from: hsl(var(--s) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/25:hover {
    --tw-gradient-from: hsl(var(--s) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/30:hover {
    --tw-gradient-from: hsl(var(--s) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/40:hover {
    --tw-gradient-from: hsl(var(--s) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/5:hover {
    --tw-gradient-from: hsl(var(--s) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/50:hover {
    --tw-gradient-from: hsl(var(--s) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/60:hover {
    --tw-gradient-from: hsl(var(--s) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/70:hover {
    --tw-gradient-from: hsl(var(--s) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/75:hover {
    --tw-gradient-from: hsl(var(--s) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/80:hover {
    --tw-gradient-from: hsl(var(--s) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/90:hover {
    --tw-gradient-from: hsl(var(--s) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-secondary\/95:hover {
    --tw-gradient-from: hsl(var(--s) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success:hover {
    --tw-gradient-from: hsl(var(--su) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content:hover {
    --tw-gradient-from: hsl(var(--suc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/0:hover {
    --tw-gradient-from: hsl(var(--suc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/10:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/100:hover {
    --tw-gradient-from: hsl(var(--suc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/20:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/25:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/30:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/40:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/5:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/50:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/60:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/70:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/75:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/80:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/90:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success-content\/95:hover {
    --tw-gradient-from: hsl(var(--suc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/0:hover {
    --tw-gradient-from: hsl(var(--su) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/10:hover {
    --tw-gradient-from: hsl(var(--su) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/100:hover {
    --tw-gradient-from: hsl(var(--su) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/20:hover {
    --tw-gradient-from: hsl(var(--su) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/25:hover {
    --tw-gradient-from: hsl(var(--su) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/30:hover {
    --tw-gradient-from: hsl(var(--su) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/40:hover {
    --tw-gradient-from: hsl(var(--su) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/5:hover {
    --tw-gradient-from: hsl(var(--su) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/50:hover {
    --tw-gradient-from: hsl(var(--su) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/60:hover {
    --tw-gradient-from: hsl(var(--su) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/70:hover {
    --tw-gradient-from: hsl(var(--su) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/75:hover {
    --tw-gradient-from: hsl(var(--su) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/80:hover {
    --tw-gradient-from: hsl(var(--su) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/90:hover {
    --tw-gradient-from: hsl(var(--su) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-success\/95:hover {
    --tw-gradient-from: hsl(var(--su) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning:hover {
    --tw-gradient-from: hsl(var(--wa) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content:hover {
    --tw-gradient-from: hsl(var(--wac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/0:hover {
    --tw-gradient-from: hsl(var(--wac) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/10:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/100:hover {
    --tw-gradient-from: hsl(var(--wac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/20:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/25:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/30:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/40:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/5:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/50:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/60:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/70:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/75:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/80:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/90:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning-content\/95:hover {
    --tw-gradient-from: hsl(var(--wac) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/0:hover {
    --tw-gradient-from: hsl(var(--wa) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/10:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/100:hover {
    --tw-gradient-from: hsl(var(--wa) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/20:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/25:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/30:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/40:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/5:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/50:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/60:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/70:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/75:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/80:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/90:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:from-warning\/95:hover {
    --tw-gradient-from: hsl(var(--wa) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.hover\:via-accent:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/0:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/10:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/100:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/20:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/25:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/30:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/40:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/5:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/50:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/60:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/70:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/75:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/80:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/90:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-content\/95:hover {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/0:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/10:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/100:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/20:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/25:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/30:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/40:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/5:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/50:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/60:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/70:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/75:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/80:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/90:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent-focus\/95:hover {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/0:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/10:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/100:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/20:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/25:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/30:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/40:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/5:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/50:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/60:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/70:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/75:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/80:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/90:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-accent\/95:hover {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/0:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/10:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/100:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/20:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/25:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/30:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/40:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/5:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/50:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/60:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/70:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/75:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/80:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/90:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-100\/95:hover {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/0:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/10:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/100:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/20:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/25:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/30:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/40:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/5:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/50:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/60:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/70:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/75:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/80:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/90:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-200\/95:hover {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/0:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/10:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/100:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/20:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/25:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/30:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/40:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/5:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/50:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/60:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/70:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/75:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/80:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/90:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-300\/95:hover {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/0:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/10:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/100:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/20:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/25:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/30:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/40:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/5:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/50:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/60:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/70:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/75:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/80:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/90:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-base-content\/95:hover {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/0:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/10:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/100:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/20:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/25:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/30:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/40:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/5:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/50:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/60:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/70:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/75:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/80:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/90:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error-content\/95:hover {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/0:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/10:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/100:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/20:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/25:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/30:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/40:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/5:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/50:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/60:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/70:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/75:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/80:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/90:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-error\/95:hover {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/0:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/10:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/100:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/20:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/25:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/30:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/40:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/5:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/50:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/60:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/70:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/75:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/80:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/90:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info-content\/95:hover {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/0:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/10:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/100:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/20:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/25:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/30:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/40:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/5:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/50:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/60:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/70:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/75:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/80:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/90:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-info\/95:hover {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/0:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/10:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/100:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/20:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/25:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/30:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/40:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/5:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/50:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/60:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/70:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/75:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/80:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/90:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-content\/95:hover {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/0:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/10:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/100:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/20:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/25:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/30:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/40:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/5:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/50:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/60:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/70:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/75:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/80:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/90:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral-focus\/95:hover {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/0:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/10:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/100:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/20:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/25:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/30:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/40:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/5:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/50:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/60:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/70:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/75:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/80:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/90:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-neutral\/95:hover {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/0:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/10:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/100:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/20:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/25:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/30:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/40:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/5:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/50:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/60:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/70:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/75:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/80:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/90:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-content\/95:hover {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/0:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/10:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/100:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/20:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/25:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/30:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/40:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/5:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/50:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/60:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/70:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/75:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/80:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/90:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary-focus\/95:hover {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/0:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/10:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/100:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/20:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/25:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/30:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/40:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/5:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/50:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/60:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/70:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/75:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/80:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/90:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-primary\/95:hover {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/0:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/10:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/100:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/20:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/25:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/30:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/40:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/5:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/50:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/60:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/70:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/75:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/80:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/90:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-content\/95:hover {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/0:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/10:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/100:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/20:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/25:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/30:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/40:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/5:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/50:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/60:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/70:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/75:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/80:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/90:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary-focus\/95:hover {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/0:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/10:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/100:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/20:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/25:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/30:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/40:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/5:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/50:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/60:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/70:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/75:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/80:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/90:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-secondary\/95:hover {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/0:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/10:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/100:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/20:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/25:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/30:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/40:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/5:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/50:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/60:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/70:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/75:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/80:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/90:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success-content\/95:hover {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/0:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/10:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/100:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/20:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/25:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/30:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/40:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/5:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/50:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/60:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/70:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/75:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/80:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/90:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-success\/95:hover {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/0:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/10:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/100:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/20:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/25:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/30:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/40:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/5:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/50:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/60:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/70:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/75:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/80:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/90:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning-content\/95:hover {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/0:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/10:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/100:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/20:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/25:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/30:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/40:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/5:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/50:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/60:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/70:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/75:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/80:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/90:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:via-warning\/95:hover {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.hover\:to-accent:hover {
    --tw-gradient-to: hsl(var(--a) / 1) var(--tw-gradient-to-position)}
.hover\:to-accent-content:hover {
    --tw-gradient-to: hsl(var(--ac) / 1) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/0:hover {
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/10:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/100:hover {
    --tw-gradient-to: hsl(var(--ac) / 1) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/20:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/25:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/30:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/40:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/5:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/50:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/60:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/70:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/75:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/80:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/90:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-accent-content\/95:hover {
    --tw-gradient-to: hsl(var(--ac) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-accent-focus:hover {
    --tw-gradient-to: hsl(var(--af) / 1) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/0:hover {
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/10:hover {
    --tw-gradient-to: hsl(var(--af) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/100:hover {
    --tw-gradient-to: hsl(var(--af) / 1) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/20:hover {
    --tw-gradient-to: hsl(var(--af) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/25:hover {
    --tw-gradient-to: hsl(var(--af) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/30:hover {
    --tw-gradient-to: hsl(var(--af) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/40:hover {
    --tw-gradient-to: hsl(var(--af) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/5:hover {
    --tw-gradient-to: hsl(var(--af) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/50:hover {
    --tw-gradient-to: hsl(var(--af) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/60:hover {
    --tw-gradient-to: hsl(var(--af) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/70:hover {
    --tw-gradient-to: hsl(var(--af) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/75:hover {
    --tw-gradient-to: hsl(var(--af) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/80:hover {
    --tw-gradient-to: hsl(var(--af) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/90:hover {
    --tw-gradient-to: hsl(var(--af) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-accent-focus\/95:hover {
    --tw-gradient-to: hsl(var(--af) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-accent\/0:hover {
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position)}
.hover\:to-accent\/10:hover {
    --tw-gradient-to: hsl(var(--a) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-accent\/100:hover {
    --tw-gradient-to: hsl(var(--a) / 1) var(--tw-gradient-to-position)}
.hover\:to-accent\/20:hover {
    --tw-gradient-to: hsl(var(--a) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-accent\/25:hover {
    --tw-gradient-to: hsl(var(--a) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-accent\/30:hover {
    --tw-gradient-to: hsl(var(--a) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-accent\/40:hover {
    --tw-gradient-to: hsl(var(--a) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-accent\/5:hover {
    --tw-gradient-to: hsl(var(--a) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-accent\/50:hover {
    --tw-gradient-to: hsl(var(--a) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-accent\/60:hover {
    --tw-gradient-to: hsl(var(--a) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-accent\/70:hover {
    --tw-gradient-to: hsl(var(--a) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-accent\/75:hover {
    --tw-gradient-to: hsl(var(--a) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-accent\/80:hover {
    --tw-gradient-to: hsl(var(--a) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-accent\/90:hover {
    --tw-gradient-to: hsl(var(--a) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-accent\/95:hover {
    --tw-gradient-to: hsl(var(--a) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-base-100:hover {
    --tw-gradient-to: hsl(var(--b1) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-100\/0:hover {
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position)}
.hover\:to-base-100\/10:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-base-100\/100:hover {
    --tw-gradient-to: hsl(var(--b1) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-100\/20:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-base-100\/25:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-base-100\/30:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-base-100\/40:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-base-100\/5:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-base-100\/50:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-base-100\/60:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-base-100\/70:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-base-100\/75:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-base-100\/80:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-base-100\/90:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-base-100\/95:hover {
    --tw-gradient-to: hsl(var(--b1) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-base-200:hover {
    --tw-gradient-to: hsl(var(--b2) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-200\/0:hover {
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position)}
.hover\:to-base-200\/10:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-base-200\/100:hover {
    --tw-gradient-to: hsl(var(--b2) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-200\/20:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-base-200\/25:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-base-200\/30:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-base-200\/40:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-base-200\/5:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-base-200\/50:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-base-200\/60:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-base-200\/70:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-base-200\/75:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-base-200\/80:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-base-200\/90:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-base-200\/95:hover {
    --tw-gradient-to: hsl(var(--b2) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-base-300:hover {
    --tw-gradient-to: hsl(var(--b3) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-300\/0:hover {
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position)}
.hover\:to-base-300\/10:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-base-300\/100:hover {
    --tw-gradient-to: hsl(var(--b3) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-300\/20:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-base-300\/25:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-base-300\/30:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-base-300\/40:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-base-300\/5:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-base-300\/50:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-base-300\/60:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-base-300\/70:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-base-300\/75:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-base-300\/80:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-base-300\/90:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-base-300\/95:hover {
    --tw-gradient-to: hsl(var(--b3) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-base-content:hover {
    --tw-gradient-to: hsl(var(--bc) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-content\/0:hover {
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position)}
.hover\:to-base-content\/10:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-base-content\/100:hover {
    --tw-gradient-to: hsl(var(--bc) / 1) var(--tw-gradient-to-position)}
.hover\:to-base-content\/20:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-base-content\/25:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-base-content\/30:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-base-content\/40:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-base-content\/5:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-base-content\/50:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-base-content\/60:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-base-content\/70:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-base-content\/75:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-base-content\/80:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-base-content\/90:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-base-content\/95:hover {
    --tw-gradient-to: hsl(var(--bc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-error:hover {
    --tw-gradient-to: hsl(var(--er) / 1) var(--tw-gradient-to-position)}
.hover\:to-error-content:hover {
    --tw-gradient-to: hsl(var(--erc) / 1) var(--tw-gradient-to-position)}
.hover\:to-error-content\/0:hover {
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position)}
.hover\:to-error-content\/10:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-error-content\/100:hover {
    --tw-gradient-to: hsl(var(--erc) / 1) var(--tw-gradient-to-position)}
.hover\:to-error-content\/20:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-error-content\/25:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-error-content\/30:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-error-content\/40:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-error-content\/5:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-error-content\/50:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-error-content\/60:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-error-content\/70:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-error-content\/75:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-error-content\/80:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-error-content\/90:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-error-content\/95:hover {
    --tw-gradient-to: hsl(var(--erc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-error\/0:hover {
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position)}
.hover\:to-error\/10:hover {
    --tw-gradient-to: hsl(var(--er) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-error\/100:hover {
    --tw-gradient-to: hsl(var(--er) / 1) var(--tw-gradient-to-position)}
.hover\:to-error\/20:hover {
    --tw-gradient-to: hsl(var(--er) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-error\/25:hover {
    --tw-gradient-to: hsl(var(--er) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-error\/30:hover {
    --tw-gradient-to: hsl(var(--er) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-error\/40:hover {
    --tw-gradient-to: hsl(var(--er) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-error\/5:hover {
    --tw-gradient-to: hsl(var(--er) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-error\/50:hover {
    --tw-gradient-to: hsl(var(--er) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-error\/60:hover {
    --tw-gradient-to: hsl(var(--er) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-error\/70:hover {
    --tw-gradient-to: hsl(var(--er) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-error\/75:hover {
    --tw-gradient-to: hsl(var(--er) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-error\/80:hover {
    --tw-gradient-to: hsl(var(--er) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-error\/90:hover {
    --tw-gradient-to: hsl(var(--er) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-error\/95:hover {
    --tw-gradient-to: hsl(var(--er) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-info:hover {
    --tw-gradient-to: hsl(var(--in) / 1) var(--tw-gradient-to-position)}
.hover\:to-info-content:hover {
    --tw-gradient-to: hsl(var(--inc) / 1) var(--tw-gradient-to-position)}
.hover\:to-info-content\/0:hover {
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position)}
.hover\:to-info-content\/10:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-info-content\/100:hover {
    --tw-gradient-to: hsl(var(--inc) / 1) var(--tw-gradient-to-position)}
.hover\:to-info-content\/20:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-info-content\/25:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-info-content\/30:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-info-content\/40:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-info-content\/5:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-info-content\/50:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-info-content\/60:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-info-content\/70:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-info-content\/75:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-info-content\/80:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-info-content\/90:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-info-content\/95:hover {
    --tw-gradient-to: hsl(var(--inc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-info\/0:hover {
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position)}
.hover\:to-info\/10:hover {
    --tw-gradient-to: hsl(var(--in) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-info\/100:hover {
    --tw-gradient-to: hsl(var(--in) / 1) var(--tw-gradient-to-position)}
.hover\:to-info\/20:hover {
    --tw-gradient-to: hsl(var(--in) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-info\/25:hover {
    --tw-gradient-to: hsl(var(--in) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-info\/30:hover {
    --tw-gradient-to: hsl(var(--in) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-info\/40:hover {
    --tw-gradient-to: hsl(var(--in) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-info\/5:hover {
    --tw-gradient-to: hsl(var(--in) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-info\/50:hover {
    --tw-gradient-to: hsl(var(--in) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-info\/60:hover {
    --tw-gradient-to: hsl(var(--in) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-info\/70:hover {
    --tw-gradient-to: hsl(var(--in) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-info\/75:hover {
    --tw-gradient-to: hsl(var(--in) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-info\/80:hover {
    --tw-gradient-to: hsl(var(--in) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-info\/90:hover {
    --tw-gradient-to: hsl(var(--in) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-info\/95:hover {
    --tw-gradient-to: hsl(var(--in) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-neutral:hover {
    --tw-gradient-to: hsl(var(--n) / 1) var(--tw-gradient-to-position)}
.hover\:to-neutral-content:hover {
    --tw-gradient-to: hsl(var(--nc) / 1) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/0:hover {
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/10:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/100:hover {
    --tw-gradient-to: hsl(var(--nc) / 1) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/20:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/25:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/30:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/40:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/5:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/50:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/60:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/70:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/75:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/80:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/90:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-neutral-content\/95:hover {
    --tw-gradient-to: hsl(var(--nc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus:hover {
    --tw-gradient-to: hsl(var(--nf) / 1) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/0:hover {
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/10:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/100:hover {
    --tw-gradient-to: hsl(var(--nf) / 1) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/20:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/25:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/30:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/40:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/5:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/50:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/60:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/70:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/75:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/80:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/90:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-neutral-focus\/95:hover {
    --tw-gradient-to: hsl(var(--nf) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-neutral\/0:hover {
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position)}
.hover\:to-neutral\/10:hover {
    --tw-gradient-to: hsl(var(--n) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-neutral\/100:hover {
    --tw-gradient-to: hsl(var(--n) / 1) var(--tw-gradient-to-position)}
.hover\:to-neutral\/20:hover {
    --tw-gradient-to: hsl(var(--n) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-neutral\/25:hover {
    --tw-gradient-to: hsl(var(--n) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-neutral\/30:hover {
    --tw-gradient-to: hsl(var(--n) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-neutral\/40:hover {
    --tw-gradient-to: hsl(var(--n) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-neutral\/5:hover {
    --tw-gradient-to: hsl(var(--n) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-neutral\/50:hover {
    --tw-gradient-to: hsl(var(--n) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-neutral\/60:hover {
    --tw-gradient-to: hsl(var(--n) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-neutral\/70:hover {
    --tw-gradient-to: hsl(var(--n) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-neutral\/75:hover {
    --tw-gradient-to: hsl(var(--n) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-neutral\/80:hover {
    --tw-gradient-to: hsl(var(--n) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-neutral\/90:hover {
    --tw-gradient-to: hsl(var(--n) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-neutral\/95:hover {
    --tw-gradient-to: hsl(var(--n) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-primary:hover {
    --tw-gradient-to: hsl(var(--p) / 1) var(--tw-gradient-to-position)}
.hover\:to-primary-content:hover {
    --tw-gradient-to: hsl(var(--pc) / 1) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/0:hover {
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/10:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/100:hover {
    --tw-gradient-to: hsl(var(--pc) / 1) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/20:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/25:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/30:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/40:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/5:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/50:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/60:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/70:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/75:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/80:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/90:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-primary-content\/95:hover {
    --tw-gradient-to: hsl(var(--pc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-primary-focus:hover {
    --tw-gradient-to: hsl(var(--pf) / 1) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/0:hover {
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/10:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/100:hover {
    --tw-gradient-to: hsl(var(--pf) / 1) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/20:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/25:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/30:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/40:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/5:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/50:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/60:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/70:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/75:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/80:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/90:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-primary-focus\/95:hover {
    --tw-gradient-to: hsl(var(--pf) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-primary\/0:hover {
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position)}
.hover\:to-primary\/10:hover {
    --tw-gradient-to: hsl(var(--p) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-primary\/100:hover {
    --tw-gradient-to: hsl(var(--p) / 1) var(--tw-gradient-to-position)}
.hover\:to-primary\/20:hover {
    --tw-gradient-to: hsl(var(--p) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-primary\/25:hover {
    --tw-gradient-to: hsl(var(--p) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-primary\/30:hover {
    --tw-gradient-to: hsl(var(--p) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-primary\/40:hover {
    --tw-gradient-to: hsl(var(--p) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-primary\/5:hover {
    --tw-gradient-to: hsl(var(--p) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-primary\/50:hover {
    --tw-gradient-to: hsl(var(--p) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-primary\/60:hover {
    --tw-gradient-to: hsl(var(--p) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-primary\/70:hover {
    --tw-gradient-to: hsl(var(--p) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-primary\/75:hover {
    --tw-gradient-to: hsl(var(--p) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-primary\/80:hover {
    --tw-gradient-to: hsl(var(--p) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-primary\/90:hover {
    --tw-gradient-to: hsl(var(--p) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-primary\/95:hover {
    --tw-gradient-to: hsl(var(--p) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-secondary:hover {
    --tw-gradient-to: hsl(var(--s) / 1) var(--tw-gradient-to-position)}
.hover\:to-secondary-content:hover {
    --tw-gradient-to: hsl(var(--sc) / 1) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/0:hover {
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/10:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/100:hover {
    --tw-gradient-to: hsl(var(--sc) / 1) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/20:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/25:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/30:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/40:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/5:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/50:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/60:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/70:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/75:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/80:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/90:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-secondary-content\/95:hover {
    --tw-gradient-to: hsl(var(--sc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus:hover {
    --tw-gradient-to: hsl(var(--sf) / 1) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/0:hover {
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/10:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/100:hover {
    --tw-gradient-to: hsl(var(--sf) / 1) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/20:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/25:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/30:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/40:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/5:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/50:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/60:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/70:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/75:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/80:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/90:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-secondary-focus\/95:hover {
    --tw-gradient-to: hsl(var(--sf) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-secondary\/0:hover {
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position)}
.hover\:to-secondary\/10:hover {
    --tw-gradient-to: hsl(var(--s) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-secondary\/100:hover {
    --tw-gradient-to: hsl(var(--s) / 1) var(--tw-gradient-to-position)}
.hover\:to-secondary\/20:hover {
    --tw-gradient-to: hsl(var(--s) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-secondary\/25:hover {
    --tw-gradient-to: hsl(var(--s) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-secondary\/30:hover {
    --tw-gradient-to: hsl(var(--s) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-secondary\/40:hover {
    --tw-gradient-to: hsl(var(--s) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-secondary\/5:hover {
    --tw-gradient-to: hsl(var(--s) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-secondary\/50:hover {
    --tw-gradient-to: hsl(var(--s) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-secondary\/60:hover {
    --tw-gradient-to: hsl(var(--s) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-secondary\/70:hover {
    --tw-gradient-to: hsl(var(--s) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-secondary\/75:hover {
    --tw-gradient-to: hsl(var(--s) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-secondary\/80:hover {
    --tw-gradient-to: hsl(var(--s) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-secondary\/90:hover {
    --tw-gradient-to: hsl(var(--s) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-secondary\/95:hover {
    --tw-gradient-to: hsl(var(--s) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-success:hover {
    --tw-gradient-to: hsl(var(--su) / 1) var(--tw-gradient-to-position)}
.hover\:to-success-content:hover {
    --tw-gradient-to: hsl(var(--suc) / 1) var(--tw-gradient-to-position)}
.hover\:to-success-content\/0:hover {
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position)}
.hover\:to-success-content\/10:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-success-content\/100:hover {
    --tw-gradient-to: hsl(var(--suc) / 1) var(--tw-gradient-to-position)}
.hover\:to-success-content\/20:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-success-content\/25:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-success-content\/30:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-success-content\/40:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-success-content\/5:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-success-content\/50:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-success-content\/60:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-success-content\/70:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-success-content\/75:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-success-content\/80:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-success-content\/90:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-success-content\/95:hover {
    --tw-gradient-to: hsl(var(--suc) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-success\/0:hover {
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position)}
.hover\:to-success\/10:hover {
    --tw-gradient-to: hsl(var(--su) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-success\/100:hover {
    --tw-gradient-to: hsl(var(--su) / 1) var(--tw-gradient-to-position)}
.hover\:to-success\/20:hover {
    --tw-gradient-to: hsl(var(--su) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-success\/25:hover {
    --tw-gradient-to: hsl(var(--su) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-success\/30:hover {
    --tw-gradient-to: hsl(var(--su) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-success\/40:hover {
    --tw-gradient-to: hsl(var(--su) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-success\/5:hover {
    --tw-gradient-to: hsl(var(--su) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-success\/50:hover {
    --tw-gradient-to: hsl(var(--su) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-success\/60:hover {
    --tw-gradient-to: hsl(var(--su) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-success\/70:hover {
    --tw-gradient-to: hsl(var(--su) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-success\/75:hover {
    --tw-gradient-to: hsl(var(--su) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-success\/80:hover {
    --tw-gradient-to: hsl(var(--su) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-success\/90:hover {
    --tw-gradient-to: hsl(var(--su) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-success\/95:hover {
    --tw-gradient-to: hsl(var(--su) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-warning:hover {
    --tw-gradient-to: hsl(var(--wa) / 1) var(--tw-gradient-to-position)}
.hover\:to-warning-content:hover {
    --tw-gradient-to: hsl(var(--wac) / 1) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/0:hover {
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/10:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/100:hover {
    --tw-gradient-to: hsl(var(--wac) / 1) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/20:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/25:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/30:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/40:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/5:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/50:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/60:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/70:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/75:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/80:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/90:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-warning-content\/95:hover {
    --tw-gradient-to: hsl(var(--wac) / 0.95) var(--tw-gradient-to-position)}
.hover\:to-warning\/0:hover {
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position)}
.hover\:to-warning\/10:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.1) var(--tw-gradient-to-position)}
.hover\:to-warning\/100:hover {
    --tw-gradient-to: hsl(var(--wa) / 1) var(--tw-gradient-to-position)}
.hover\:to-warning\/20:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.2) var(--tw-gradient-to-position)}
.hover\:to-warning\/25:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.25) var(--tw-gradient-to-position)}
.hover\:to-warning\/30:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.3) var(--tw-gradient-to-position)}
.hover\:to-warning\/40:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.4) var(--tw-gradient-to-position)}
.hover\:to-warning\/5:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.05) var(--tw-gradient-to-position)}
.hover\:to-warning\/50:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.5) var(--tw-gradient-to-position)}
.hover\:to-warning\/60:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.6) var(--tw-gradient-to-position)}
.hover\:to-warning\/70:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.7) var(--tw-gradient-to-position)}
.hover\:to-warning\/75:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.75) var(--tw-gradient-to-position)}
.hover\:to-warning\/80:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.8) var(--tw-gradient-to-position)}
.hover\:to-warning\/90:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.9) var(--tw-gradient-to-position)}
.hover\:to-warning\/95:hover {
    --tw-gradient-to: hsl(var(--wa) / 0.95) var(--tw-gradient-to-position)}
.hover\:text-accent:hover {
    color: hsl(var(--a) / 1)}
.hover\:text-accent-content:hover {
    color: hsl(var(--ac) / 1)}
.hover\:text-accent-content\/0:hover {
    color: hsl(var(--ac) / 0)}
.hover\:text-accent-content\/10:hover {
    color: hsl(var(--ac) / 0.1)}
.hover\:text-accent-content\/100:hover {
    color: hsl(var(--ac) / 1)}
.hover\:text-accent-content\/20:hover {
    color: hsl(var(--ac) / 0.2)}
.hover\:text-accent-content\/25:hover {
    color: hsl(var(--ac) / 0.25)}
.hover\:text-accent-content\/30:hover {
    color: hsl(var(--ac) / 0.3)}
.hover\:text-accent-content\/40:hover {
    color: hsl(var(--ac) / 0.4)}
.hover\:text-accent-content\/5:hover {
    color: hsl(var(--ac) / 0.05)}
.hover\:text-accent-content\/50:hover {
    color: hsl(var(--ac) / 0.5)}
.hover\:text-accent-content\/60:hover {
    color: hsl(var(--ac) / 0.6)}
.hover\:text-accent-content\/70:hover {
    color: hsl(var(--ac) / 0.7)}
.hover\:text-accent-content\/75:hover {
    color: hsl(var(--ac) / 0.75)}
.hover\:text-accent-content\/80:hover {
    color: hsl(var(--ac) / 0.8)}
.hover\:text-accent-content\/90:hover {
    color: hsl(var(--ac) / 0.9)}
.hover\:text-accent-content\/95:hover {
    color: hsl(var(--ac) / 0.95)}
.hover\:text-accent-focus:hover {
    color: hsl(var(--af) / 1)}
.hover\:text-accent-focus\/0:hover {
    color: hsl(var(--af) / 0)}
.hover\:text-accent-focus\/10:hover {
    color: hsl(var(--af) / 0.1)}
.hover\:text-accent-focus\/100:hover {
    color: hsl(var(--af) / 1)}
.hover\:text-accent-focus\/20:hover {
    color: hsl(var(--af) / 0.2)}
.hover\:text-accent-focus\/25:hover {
    color: hsl(var(--af) / 0.25)}
.hover\:text-accent-focus\/30:hover {
    color: hsl(var(--af) / 0.3)}
.hover\:text-accent-focus\/40:hover {
    color: hsl(var(--af) / 0.4)}
.hover\:text-accent-focus\/5:hover {
    color: hsl(var(--af) / 0.05)}
.hover\:text-accent-focus\/50:hover {
    color: hsl(var(--af) / 0.5)}
.hover\:text-accent-focus\/60:hover {
    color: hsl(var(--af) / 0.6)}
.hover\:text-accent-focus\/70:hover {
    color: hsl(var(--af) / 0.7)}
.hover\:text-accent-focus\/75:hover {
    color: hsl(var(--af) / 0.75)}
.hover\:text-accent-focus\/80:hover {
    color: hsl(var(--af) / 0.8)}
.hover\:text-accent-focus\/90:hover {
    color: hsl(var(--af) / 0.9)}
.hover\:text-accent-focus\/95:hover {
    color: hsl(var(--af) / 0.95)}
.hover\:text-accent\/0:hover {
    color: hsl(var(--a) / 0)}
.hover\:text-accent\/10:hover {
    color: hsl(var(--a) / 0.1)}
.hover\:text-accent\/100:hover {
    color: hsl(var(--a) / 1)}
.hover\:text-accent\/20:hover {
    color: hsl(var(--a) / 0.2)}
.hover\:text-accent\/25:hover {
    color: hsl(var(--a) / 0.25)}
.hover\:text-accent\/30:hover {
    color: hsl(var(--a) / 0.3)}
.hover\:text-accent\/40:hover {
    color: hsl(var(--a) / 0.4)}
.hover\:text-accent\/5:hover {
    color: hsl(var(--a) / 0.05)}
.hover\:text-accent\/50:hover {
    color: hsl(var(--a) / 0.5)}
.hover\:text-accent\/60:hover {
    color: hsl(var(--a) / 0.6)}
.hover\:text-accent\/70:hover {
    color: hsl(var(--a) / 0.7)}
.hover\:text-accent\/75:hover {
    color: hsl(var(--a) / 0.75)}
.hover\:text-accent\/80:hover {
    color: hsl(var(--a) / 0.8)}
.hover\:text-accent\/90:hover {
    color: hsl(var(--a) / 0.9)}
.hover\:text-accent\/95:hover {
    color: hsl(var(--a) / 0.95)}
.hover\:text-base-100:hover {
    color: hsl(var(--b1) / 1)}
.hover\:text-base-100\/0:hover {
    color: hsl(var(--b1) / 0)}
.hover\:text-base-100\/10:hover {
    color: hsl(var(--b1) / 0.1)}
.hover\:text-base-100\/100:hover {
    color: hsl(var(--b1) / 1)}
.hover\:text-base-100\/20:hover {
    color: hsl(var(--b1) / 0.2)}
.hover\:text-base-100\/25:hover {
    color: hsl(var(--b1) / 0.25)}
.hover\:text-base-100\/30:hover {
    color: hsl(var(--b1) / 0.3)}
.hover\:text-base-100\/40:hover {
    color: hsl(var(--b1) / 0.4)}
.hover\:text-base-100\/5:hover {
    color: hsl(var(--b1) / 0.05)}
.hover\:text-base-100\/50:hover {
    color: hsl(var(--b1) / 0.5)}
.hover\:text-base-100\/60:hover {
    color: hsl(var(--b1) / 0.6)}
.hover\:text-base-100\/70:hover {
    color: hsl(var(--b1) / 0.7)}
.hover\:text-base-100\/75:hover {
    color: hsl(var(--b1) / 0.75)}
.hover\:text-base-100\/80:hover {
    color: hsl(var(--b1) / 0.8)}
.hover\:text-base-100\/90:hover {
    color: hsl(var(--b1) / 0.9)}
.hover\:text-base-100\/95:hover {
    color: hsl(var(--b1) / 0.95)}
.hover\:text-base-200:hover {
    color: hsl(var(--b2) / 1)}
.hover\:text-base-200\/0:hover {
    color: hsl(var(--b2) / 0)}
.hover\:text-base-200\/10:hover {
    color: hsl(var(--b2) / 0.1)}
.hover\:text-base-200\/100:hover {
    color: hsl(var(--b2) / 1)}
.hover\:text-base-200\/20:hover {
    color: hsl(var(--b2) / 0.2)}
.hover\:text-base-200\/25:hover {
    color: hsl(var(--b2) / 0.25)}
.hover\:text-base-200\/30:hover {
    color: hsl(var(--b2) / 0.3)}
.hover\:text-base-200\/40:hover {
    color: hsl(var(--b2) / 0.4)}
.hover\:text-base-200\/5:hover {
    color: hsl(var(--b2) / 0.05)}
.hover\:text-base-200\/50:hover {
    color: hsl(var(--b2) / 0.5)}
.hover\:text-base-200\/60:hover {
    color: hsl(var(--b2) / 0.6)}
.hover\:text-base-200\/70:hover {
    color: hsl(var(--b2) / 0.7)}
.hover\:text-base-200\/75:hover {
    color: hsl(var(--b2) / 0.75)}
.hover\:text-base-200\/80:hover {
    color: hsl(var(--b2) / 0.8)}
.hover\:text-base-200\/90:hover {
    color: hsl(var(--b2) / 0.9)}
.hover\:text-base-200\/95:hover {
    color: hsl(var(--b2) / 0.95)}
.hover\:text-base-300:hover {
    color: hsl(var(--b3) / 1)}
.hover\:text-base-300\/0:hover {
    color: hsl(var(--b3) / 0)}
.hover\:text-base-300\/10:hover {
    color: hsl(var(--b3) / 0.1)}
.hover\:text-base-300\/100:hover {
    color: hsl(var(--b3) / 1)}
.hover\:text-base-300\/20:hover {
    color: hsl(var(--b3) / 0.2)}
.hover\:text-base-300\/25:hover {
    color: hsl(var(--b3) / 0.25)}
.hover\:text-base-300\/30:hover {
    color: hsl(var(--b3) / 0.3)}
.hover\:text-base-300\/40:hover {
    color: hsl(var(--b3) / 0.4)}
.hover\:text-base-300\/5:hover {
    color: hsl(var(--b3) / 0.05)}
.hover\:text-base-300\/50:hover {
    color: hsl(var(--b3) / 0.5)}
.hover\:text-base-300\/60:hover {
    color: hsl(var(--b3) / 0.6)}
.hover\:text-base-300\/70:hover {
    color: hsl(var(--b3) / 0.7)}
.hover\:text-base-300\/75:hover {
    color: hsl(var(--b3) / 0.75)}
.hover\:text-base-300\/80:hover {
    color: hsl(var(--b3) / 0.8)}
.hover\:text-base-300\/90:hover {
    color: hsl(var(--b3) / 0.9)}
.hover\:text-base-300\/95:hover {
    color: hsl(var(--b3) / 0.95)}
.hover\:text-base-content:hover {
    color: hsl(var(--bc) / 1)}
.hover\:text-base-content\/0:hover {
    color: hsl(var(--bc) / 0)}
.hover\:text-base-content\/10:hover {
    color: hsl(var(--bc) / 0.1)}
.hover\:text-base-content\/100:hover {
    color: hsl(var(--bc) / 1)}
.hover\:text-base-content\/20:hover {
    color: hsl(var(--bc) / 0.2)}
.hover\:text-base-content\/25:hover {
    color: hsl(var(--bc) / 0.25)}
.hover\:text-base-content\/30:hover {
    color: hsl(var(--bc) / 0.3)}
.hover\:text-base-content\/40:hover {
    color: hsl(var(--bc) / 0.4)}
.hover\:text-base-content\/5:hover {
    color: hsl(var(--bc) / 0.05)}
.hover\:text-base-content\/50:hover {
    color: hsl(var(--bc) / 0.5)}
.hover\:text-base-content\/60:hover {
    color: hsl(var(--bc) / 0.6)}
.hover\:text-base-content\/70:hover {
    color: hsl(var(--bc) / 0.7)}
.hover\:text-base-content\/75:hover {
    color: hsl(var(--bc) / 0.75)}
.hover\:text-base-content\/80:hover {
    color: hsl(var(--bc) / 0.8)}
.hover\:text-base-content\/90:hover {
    color: hsl(var(--bc) / 0.9)}
.hover\:text-base-content\/95:hover {
    color: hsl(var(--bc) / 0.95)}
.hover\:text-error:hover {
    color: hsl(var(--er) / 1)}
.hover\:text-error-content:hover {
    color: hsl(var(--erc) / 1)}
.hover\:text-error-content\/0:hover {
    color: hsl(var(--erc) / 0)}
.hover\:text-error-content\/10:hover {
    color: hsl(var(--erc) / 0.1)}
.hover\:text-error-content\/100:hover {
    color: hsl(var(--erc) / 1)}
.hover\:text-error-content\/20:hover {
    color: hsl(var(--erc) / 0.2)}
.hover\:text-error-content\/25:hover {
    color: hsl(var(--erc) / 0.25)}
.hover\:text-error-content\/30:hover {
    color: hsl(var(--erc) / 0.3)}
.hover\:text-error-content\/40:hover {
    color: hsl(var(--erc) / 0.4)}
.hover\:text-error-content\/5:hover {
    color: hsl(var(--erc) / 0.05)}
.hover\:text-error-content\/50:hover {
    color: hsl(var(--erc) / 0.5)}
.hover\:text-error-content\/60:hover {
    color: hsl(var(--erc) / 0.6)}
.hover\:text-error-content\/70:hover {
    color: hsl(var(--erc) / 0.7)}
.hover\:text-error-content\/75:hover {
    color: hsl(var(--erc) / 0.75)}
.hover\:text-error-content\/80:hover {
    color: hsl(var(--erc) / 0.8)}
.hover\:text-error-content\/90:hover {
    color: hsl(var(--erc) / 0.9)}
.hover\:text-error-content\/95:hover {
    color: hsl(var(--erc) / 0.95)}
.hover\:text-error\/0:hover {
    color: hsl(var(--er) / 0)}
.hover\:text-error\/10:hover {
    color: hsl(var(--er) / 0.1)}
.hover\:text-error\/100:hover {
    color: hsl(var(--er) / 1)}
.hover\:text-error\/20:hover {
    color: hsl(var(--er) / 0.2)}
.hover\:text-error\/25:hover {
    color: hsl(var(--er) / 0.25)}
.hover\:text-error\/30:hover {
    color: hsl(var(--er) / 0.3)}
.hover\:text-error\/40:hover {
    color: hsl(var(--er) / 0.4)}
.hover\:text-error\/5:hover {
    color: hsl(var(--er) / 0.05)}
.hover\:text-error\/50:hover {
    color: hsl(var(--er) / 0.5)}
.hover\:text-error\/60:hover {
    color: hsl(var(--er) / 0.6)}
.hover\:text-error\/70:hover {
    color: hsl(var(--er) / 0.7)}
.hover\:text-error\/75:hover {
    color: hsl(var(--er) / 0.75)}
.hover\:text-error\/80:hover {
    color: hsl(var(--er) / 0.8)}
.hover\:text-error\/90:hover {
    color: hsl(var(--er) / 0.9)}
.hover\:text-error\/95:hover {
    color: hsl(var(--er) / 0.95)}
.hover\:text-info:hover {
    color: hsl(var(--in) / 1)}
.hover\:text-info-content:hover {
    color: hsl(var(--inc) / 1)}
.hover\:text-info-content\/0:hover {
    color: hsl(var(--inc) / 0)}
.hover\:text-info-content\/10:hover {
    color: hsl(var(--inc) / 0.1)}
.hover\:text-info-content\/100:hover {
    color: hsl(var(--inc) / 1)}
.hover\:text-info-content\/20:hover {
    color: hsl(var(--inc) / 0.2)}
.hover\:text-info-content\/25:hover {
    color: hsl(var(--inc) / 0.25)}
.hover\:text-info-content\/30:hover {
    color: hsl(var(--inc) / 0.3)}
.hover\:text-info-content\/40:hover {
    color: hsl(var(--inc) / 0.4)}
.hover\:text-info-content\/5:hover {
    color: hsl(var(--inc) / 0.05)}
.hover\:text-info-content\/50:hover {
    color: hsl(var(--inc) / 0.5)}
.hover\:text-info-content\/60:hover {
    color: hsl(var(--inc) / 0.6)}
.hover\:text-info-content\/70:hover {
    color: hsl(var(--inc) / 0.7)}
.hover\:text-info-content\/75:hover {
    color: hsl(var(--inc) / 0.75)}
.hover\:text-info-content\/80:hover {
    color: hsl(var(--inc) / 0.8)}
.hover\:text-info-content\/90:hover {
    color: hsl(var(--inc) / 0.9)}
.hover\:text-info-content\/95:hover {
    color: hsl(var(--inc) / 0.95)}
.hover\:text-info\/0:hover {
    color: hsl(var(--in) / 0)}
.hover\:text-info\/10:hover {
    color: hsl(var(--in) / 0.1)}
.hover\:text-info\/100:hover {
    color: hsl(var(--in) / 1)}
.hover\:text-info\/20:hover {
    color: hsl(var(--in) / 0.2)}
.hover\:text-info\/25:hover {
    color: hsl(var(--in) / 0.25)}
.hover\:text-info\/30:hover {
    color: hsl(var(--in) / 0.3)}
.hover\:text-info\/40:hover {
    color: hsl(var(--in) / 0.4)}
.hover\:text-info\/5:hover {
    color: hsl(var(--in) / 0.05)}
.hover\:text-info\/50:hover {
    color: hsl(var(--in) / 0.5)}
.hover\:text-info\/60:hover {
    color: hsl(var(--in) / 0.6)}
.hover\:text-info\/70:hover {
    color: hsl(var(--in) / 0.7)}
.hover\:text-info\/75:hover {
    color: hsl(var(--in) / 0.75)}
.hover\:text-info\/80:hover {
    color: hsl(var(--in) / 0.8)}
.hover\:text-info\/90:hover {
    color: hsl(var(--in) / 0.9)}
.hover\:text-info\/95:hover {
    color: hsl(var(--in) / 0.95)}
.hover\:text-neutral:hover {
    color: hsl(var(--n) / 1)}
.hover\:text-neutral-content:hover {
    color: hsl(var(--nc) / 1)}
.hover\:text-neutral-content\/0:hover {
    color: hsl(var(--nc) / 0)}
.hover\:text-neutral-content\/10:hover {
    color: hsl(var(--nc) / 0.1)}
.hover\:text-neutral-content\/100:hover {
    color: hsl(var(--nc) / 1)}
.hover\:text-neutral-content\/20:hover {
    color: hsl(var(--nc) / 0.2)}
.hover\:text-neutral-content\/25:hover {
    color: hsl(var(--nc) / 0.25)}
.hover\:text-neutral-content\/30:hover {
    color: hsl(var(--nc) / 0.3)}
.hover\:text-neutral-content\/40:hover {
    color: hsl(var(--nc) / 0.4)}
.hover\:text-neutral-content\/5:hover {
    color: hsl(var(--nc) / 0.05)}
.hover\:text-neutral-content\/50:hover {
    color: hsl(var(--nc) / 0.5)}
.hover\:text-neutral-content\/60:hover {
    color: hsl(var(--nc) / 0.6)}
.hover\:text-neutral-content\/70:hover {
    color: hsl(var(--nc) / 0.7)}
.hover\:text-neutral-content\/75:hover {
    color: hsl(var(--nc) / 0.75)}
.hover\:text-neutral-content\/80:hover {
    color: hsl(var(--nc) / 0.8)}
.hover\:text-neutral-content\/90:hover {
    color: hsl(var(--nc) / 0.9)}
.hover\:text-neutral-content\/95:hover {
    color: hsl(var(--nc) / 0.95)}
.hover\:text-neutral-focus:hover {
    color: hsl(var(--nf) / 1)}
.hover\:text-neutral-focus\/0:hover {
    color: hsl(var(--nf) / 0)}
.hover\:text-neutral-focus\/10:hover {
    color: hsl(var(--nf) / 0.1)}
.hover\:text-neutral-focus\/100:hover {
    color: hsl(var(--nf) / 1)}
.hover\:text-neutral-focus\/20:hover {
    color: hsl(var(--nf) / 0.2)}
.hover\:text-neutral-focus\/25:hover {
    color: hsl(var(--nf) / 0.25)}
.hover\:text-neutral-focus\/30:hover {
    color: hsl(var(--nf) / 0.3)}
.hover\:text-neutral-focus\/40:hover {
    color: hsl(var(--nf) / 0.4)}
.hover\:text-neutral-focus\/5:hover {
    color: hsl(var(--nf) / 0.05)}
.hover\:text-neutral-focus\/50:hover {
    color: hsl(var(--nf) / 0.5)}
.hover\:text-neutral-focus\/60:hover {
    color: hsl(var(--nf) / 0.6)}
.hover\:text-neutral-focus\/70:hover {
    color: hsl(var(--nf) / 0.7)}
.hover\:text-neutral-focus\/75:hover {
    color: hsl(var(--nf) / 0.75)}
.hover\:text-neutral-focus\/80:hover {
    color: hsl(var(--nf) / 0.8)}
.hover\:text-neutral-focus\/90:hover {
    color: hsl(var(--nf) / 0.9)}
.hover\:text-neutral-focus\/95:hover {
    color: hsl(var(--nf) / 0.95)}
.hover\:text-neutral\/0:hover {
    color: hsl(var(--n) / 0)}
.hover\:text-neutral\/10:hover {
    color: hsl(var(--n) / 0.1)}
.hover\:text-neutral\/100:hover {
    color: hsl(var(--n) / 1)}
.hover\:text-neutral\/20:hover {
    color: hsl(var(--n) / 0.2)}
.hover\:text-neutral\/25:hover {
    color: hsl(var(--n) / 0.25)}
.hover\:text-neutral\/30:hover {
    color: hsl(var(--n) / 0.3)}
.hover\:text-neutral\/40:hover {
    color: hsl(var(--n) / 0.4)}
.hover\:text-neutral\/5:hover {
    color: hsl(var(--n) / 0.05)}
.hover\:text-neutral\/50:hover {
    color: hsl(var(--n) / 0.5)}
.hover\:text-neutral\/60:hover {
    color: hsl(var(--n) / 0.6)}
.hover\:text-neutral\/70:hover {
    color: hsl(var(--n) / 0.7)}
.hover\:text-neutral\/75:hover {
    color: hsl(var(--n) / 0.75)}
.hover\:text-neutral\/80:hover {
    color: hsl(var(--n) / 0.8)}
.hover\:text-neutral\/90:hover {
    color: hsl(var(--n) / 0.9)}
.hover\:text-neutral\/95:hover {
    color: hsl(var(--n) / 0.95)}
.hover\:text-primary:hover {
    color: hsl(var(--p) / 1)}
.hover\:text-primary-content:hover {
    color: hsl(var(--pc) / 1)}
.hover\:text-primary-content\/0:hover {
    color: hsl(var(--pc) / 0)}
.hover\:text-primary-content\/10:hover {
    color: hsl(var(--pc) / 0.1)}
.hover\:text-primary-content\/100:hover {
    color: hsl(var(--pc) / 1)}
.hover\:text-primary-content\/20:hover {
    color: hsl(var(--pc) / 0.2)}
.hover\:text-primary-content\/25:hover {
    color: hsl(var(--pc) / 0.25)}
.hover\:text-primary-content\/30:hover {
    color: hsl(var(--pc) / 0.3)}
.hover\:text-primary-content\/40:hover {
    color: hsl(var(--pc) / 0.4)}
.hover\:text-primary-content\/5:hover {
    color: hsl(var(--pc) / 0.05)}
.hover\:text-primary-content\/50:hover {
    color: hsl(var(--pc) / 0.5)}
.hover\:text-primary-content\/60:hover {
    color: hsl(var(--pc) / 0.6)}
.hover\:text-primary-content\/70:hover {
    color: hsl(var(--pc) / 0.7)}
.hover\:text-primary-content\/75:hover {
    color: hsl(var(--pc) / 0.75)}
.hover\:text-primary-content\/80:hover {
    color: hsl(var(--pc) / 0.8)}
.hover\:text-primary-content\/90:hover {
    color: hsl(var(--pc) / 0.9)}
.hover\:text-primary-content\/95:hover {
    color: hsl(var(--pc) / 0.95)}
.hover\:text-primary-focus:hover {
    color: hsl(var(--pf) / 1)}
.hover\:text-primary-focus\/0:hover {
    color: hsl(var(--pf) / 0)}
.hover\:text-primary-focus\/10:hover {
    color: hsl(var(--pf) / 0.1)}
.hover\:text-primary-focus\/100:hover {
    color: hsl(var(--pf) / 1)}
.hover\:text-primary-focus\/20:hover {
    color: hsl(var(--pf) / 0.2)}
.hover\:text-primary-focus\/25:hover {
    color: hsl(var(--pf) / 0.25)}
.hover\:text-primary-focus\/30:hover {
    color: hsl(var(--pf) / 0.3)}
.hover\:text-primary-focus\/40:hover {
    color: hsl(var(--pf) / 0.4)}
.hover\:text-primary-focus\/5:hover {
    color: hsl(var(--pf) / 0.05)}
.hover\:text-primary-focus\/50:hover {
    color: hsl(var(--pf) / 0.5)}
.hover\:text-primary-focus\/60:hover {
    color: hsl(var(--pf) / 0.6)}
.hover\:text-primary-focus\/70:hover {
    color: hsl(var(--pf) / 0.7)}
.hover\:text-primary-focus\/75:hover {
    color: hsl(var(--pf) / 0.75)}
.hover\:text-primary-focus\/80:hover {
    color: hsl(var(--pf) / 0.8)}
.hover\:text-primary-focus\/90:hover {
    color: hsl(var(--pf) / 0.9)}
.hover\:text-primary-focus\/95:hover {
    color: hsl(var(--pf) / 0.95)}
.hover\:text-primary\/0:hover {
    color: hsl(var(--p) / 0)}
.hover\:text-primary\/10:hover {
    color: hsl(var(--p) / 0.1)}
.hover\:text-primary\/100:hover {
    color: hsl(var(--p) / 1)}
.hover\:text-primary\/20:hover {
    color: hsl(var(--p) / 0.2)}
.hover\:text-primary\/25:hover {
    color: hsl(var(--p) / 0.25)}
.hover\:text-primary\/30:hover {
    color: hsl(var(--p) / 0.3)}
.hover\:text-primary\/40:hover {
    color: hsl(var(--p) / 0.4)}
.hover\:text-primary\/5:hover {
    color: hsl(var(--p) / 0.05)}
.hover\:text-primary\/50:hover {
    color: hsl(var(--p) / 0.5)}
.hover\:text-primary\/60:hover {
    color: hsl(var(--p) / 0.6)}
.hover\:text-primary\/70:hover {
    color: hsl(var(--p) / 0.7)}
.hover\:text-primary\/75:hover {
    color: hsl(var(--p) / 0.75)}
.hover\:text-primary\/80:hover {
    color: hsl(var(--p) / 0.8)}
.hover\:text-primary\/90:hover {
    color: hsl(var(--p) / 0.9)}
.hover\:text-primary\/95:hover {
    color: hsl(var(--p) / 0.95)}
.hover\:text-secondary:hover {
    color: hsl(var(--s) / 1)}
.hover\:text-secondary-content:hover {
    color: hsl(var(--sc) / 1)}
.hover\:text-secondary-content\/0:hover {
    color: hsl(var(--sc) / 0)}
.hover\:text-secondary-content\/10:hover {
    color: hsl(var(--sc) / 0.1)}
.hover\:text-secondary-content\/100:hover {
    color: hsl(var(--sc) / 1)}
.hover\:text-secondary-content\/20:hover {
    color: hsl(var(--sc) / 0.2)}
.hover\:text-secondary-content\/25:hover {
    color: hsl(var(--sc) / 0.25)}
.hover\:text-secondary-content\/30:hover {
    color: hsl(var(--sc) / 0.3)}
.hover\:text-secondary-content\/40:hover {
    color: hsl(var(--sc) / 0.4)}
.hover\:text-secondary-content\/5:hover {
    color: hsl(var(--sc) / 0.05)}
.hover\:text-secondary-content\/50:hover {
    color: hsl(var(--sc) / 0.5)}
.hover\:text-secondary-content\/60:hover {
    color: hsl(var(--sc) / 0.6)}
.hover\:text-secondary-content\/70:hover {
    color: hsl(var(--sc) / 0.7)}
.hover\:text-secondary-content\/75:hover {
    color: hsl(var(--sc) / 0.75)}
.hover\:text-secondary-content\/80:hover {
    color: hsl(var(--sc) / 0.8)}
.hover\:text-secondary-content\/90:hover {
    color: hsl(var(--sc) / 0.9)}
.hover\:text-secondary-content\/95:hover {
    color: hsl(var(--sc) / 0.95)}
.hover\:text-secondary-focus:hover {
    color: hsl(var(--sf) / 1)}
.hover\:text-secondary-focus\/0:hover {
    color: hsl(var(--sf) / 0)}
.hover\:text-secondary-focus\/10:hover {
    color: hsl(var(--sf) / 0.1)}
.hover\:text-secondary-focus\/100:hover {
    color: hsl(var(--sf) / 1)}
.hover\:text-secondary-focus\/20:hover {
    color: hsl(var(--sf) / 0.2)}
.hover\:text-secondary-focus\/25:hover {
    color: hsl(var(--sf) / 0.25)}
.hover\:text-secondary-focus\/30:hover {
    color: hsl(var(--sf) / 0.3)}
.hover\:text-secondary-focus\/40:hover {
    color: hsl(var(--sf) / 0.4)}
.hover\:text-secondary-focus\/5:hover {
    color: hsl(var(--sf) / 0.05)}
.hover\:text-secondary-focus\/50:hover {
    color: hsl(var(--sf) / 0.5)}
.hover\:text-secondary-focus\/60:hover {
    color: hsl(var(--sf) / 0.6)}
.hover\:text-secondary-focus\/70:hover {
    color: hsl(var(--sf) / 0.7)}
.hover\:text-secondary-focus\/75:hover {
    color: hsl(var(--sf) / 0.75)}
.hover\:text-secondary-focus\/80:hover {
    color: hsl(var(--sf) / 0.8)}
.hover\:text-secondary-focus\/90:hover {
    color: hsl(var(--sf) / 0.9)}
.hover\:text-secondary-focus\/95:hover {
    color: hsl(var(--sf) / 0.95)}
.hover\:text-secondary\/0:hover {
    color: hsl(var(--s) / 0)}
.hover\:text-secondary\/10:hover {
    color: hsl(var(--s) / 0.1)}
.hover\:text-secondary\/100:hover {
    color: hsl(var(--s) / 1)}
.hover\:text-secondary\/20:hover {
    color: hsl(var(--s) / 0.2)}
.hover\:text-secondary\/25:hover {
    color: hsl(var(--s) / 0.25)}
.hover\:text-secondary\/30:hover {
    color: hsl(var(--s) / 0.3)}
.hover\:text-secondary\/40:hover {
    color: hsl(var(--s) / 0.4)}
.hover\:text-secondary\/5:hover {
    color: hsl(var(--s) / 0.05)}
.hover\:text-secondary\/50:hover {
    color: hsl(var(--s) / 0.5)}
.hover\:text-secondary\/60:hover {
    color: hsl(var(--s) / 0.6)}
.hover\:text-secondary\/70:hover {
    color: hsl(var(--s) / 0.7)}
.hover\:text-secondary\/75:hover {
    color: hsl(var(--s) / 0.75)}
.hover\:text-secondary\/80:hover {
    color: hsl(var(--s) / 0.8)}
.hover\:text-secondary\/90:hover {
    color: hsl(var(--s) / 0.9)}
.hover\:text-secondary\/95:hover {
    color: hsl(var(--s) / 0.95)}
.hover\:text-success:hover {
    color: hsl(var(--su) / 1)}
.hover\:text-success-content:hover {
    color: hsl(var(--suc) / 1)}
.hover\:text-success-content\/0:hover {
    color: hsl(var(--suc) / 0)}
.hover\:text-success-content\/10:hover {
    color: hsl(var(--suc) / 0.1)}
.hover\:text-success-content\/100:hover {
    color: hsl(var(--suc) / 1)}
.hover\:text-success-content\/20:hover {
    color: hsl(var(--suc) / 0.2)}
.hover\:text-success-content\/25:hover {
    color: hsl(var(--suc) / 0.25)}
.hover\:text-success-content\/30:hover {
    color: hsl(var(--suc) / 0.3)}
.hover\:text-success-content\/40:hover {
    color: hsl(var(--suc) / 0.4)}
.hover\:text-success-content\/5:hover {
    color: hsl(var(--suc) / 0.05)}
.hover\:text-success-content\/50:hover {
    color: hsl(var(--suc) / 0.5)}
.hover\:text-success-content\/60:hover {
    color: hsl(var(--suc) / 0.6)}
.hover\:text-success-content\/70:hover {
    color: hsl(var(--suc) / 0.7)}
.hover\:text-success-content\/75:hover {
    color: hsl(var(--suc) / 0.75)}
.hover\:text-success-content\/80:hover {
    color: hsl(var(--suc) / 0.8)}
.hover\:text-success-content\/90:hover {
    color: hsl(var(--suc) / 0.9)}
.hover\:text-success-content\/95:hover {
    color: hsl(var(--suc) / 0.95)}
.hover\:text-success\/0:hover {
    color: hsl(var(--su) / 0)}
.hover\:text-success\/10:hover {
    color: hsl(var(--su) / 0.1)}
.hover\:text-success\/100:hover {
    color: hsl(var(--su) / 1)}
.hover\:text-success\/20:hover {
    color: hsl(var(--su) / 0.2)}
.hover\:text-success\/25:hover {
    color: hsl(var(--su) / 0.25)}
.hover\:text-success\/30:hover {
    color: hsl(var(--su) / 0.3)}
.hover\:text-success\/40:hover {
    color: hsl(var(--su) / 0.4)}
.hover\:text-success\/5:hover {
    color: hsl(var(--su) / 0.05)}
.hover\:text-success\/50:hover {
    color: hsl(var(--su) / 0.5)}
.hover\:text-success\/60:hover {
    color: hsl(var(--su) / 0.6)}
.hover\:text-success\/70:hover {
    color: hsl(var(--su) / 0.7)}
.hover\:text-success\/75:hover {
    color: hsl(var(--su) / 0.75)}
.hover\:text-success\/80:hover {
    color: hsl(var(--su) / 0.8)}
.hover\:text-success\/90:hover {
    color: hsl(var(--su) / 0.9)}
.hover\:text-success\/95:hover {
    color: hsl(var(--su) / 0.95)}
.hover\:text-warning:hover {
    color: hsl(var(--wa) / 1)}
.hover\:text-warning-content:hover {
    color: hsl(var(--wac) / 1)}
.hover\:text-warning-content\/0:hover {
    color: hsl(var(--wac) / 0)}
.hover\:text-warning-content\/10:hover {
    color: hsl(var(--wac) / 0.1)}
.hover\:text-warning-content\/100:hover {
    color: hsl(var(--wac) / 1)}
.hover\:text-warning-content\/20:hover {
    color: hsl(var(--wac) / 0.2)}
.hover\:text-warning-content\/25:hover {
    color: hsl(var(--wac) / 0.25)}
.hover\:text-warning-content\/30:hover {
    color: hsl(var(--wac) / 0.3)}
.hover\:text-warning-content\/40:hover {
    color: hsl(var(--wac) / 0.4)}
.hover\:text-warning-content\/5:hover {
    color: hsl(var(--wac) / 0.05)}
.hover\:text-warning-content\/50:hover {
    color: hsl(var(--wac) / 0.5)}
.hover\:text-warning-content\/60:hover {
    color: hsl(var(--wac) / 0.6)}
.hover\:text-warning-content\/70:hover {
    color: hsl(var(--wac) / 0.7)}
.hover\:text-warning-content\/75:hover {
    color: hsl(var(--wac) / 0.75)}
.hover\:text-warning-content\/80:hover {
    color: hsl(var(--wac) / 0.8)}
.hover\:text-warning-content\/90:hover {
    color: hsl(var(--wac) / 0.9)}
.hover\:text-warning-content\/95:hover {
    color: hsl(var(--wac) / 0.95)}
.hover\:text-warning\/0:hover {
    color: hsl(var(--wa) / 0)}
.hover\:text-warning\/10:hover {
    color: hsl(var(--wa) / 0.1)}
.hover\:text-warning\/100:hover {
    color: hsl(var(--wa) / 1)}
.hover\:text-warning\/20:hover {
    color: hsl(var(--wa) / 0.2)}
.hover\:text-warning\/25:hover {
    color: hsl(var(--wa) / 0.25)}
.hover\:text-warning\/30:hover {
    color: hsl(var(--wa) / 0.3)}
.hover\:text-warning\/40:hover {
    color: hsl(var(--wa) / 0.4)}
.hover\:text-warning\/5:hover {
    color: hsl(var(--wa) / 0.05)}
.hover\:text-warning\/50:hover {
    color: hsl(var(--wa) / 0.5)}
.hover\:text-warning\/60:hover {
    color: hsl(var(--wa) / 0.6)}
.hover\:text-warning\/70:hover {
    color: hsl(var(--wa) / 0.7)}
.hover\:text-warning\/75:hover {
    color: hsl(var(--wa) / 0.75)}
.hover\:text-warning\/80:hover {
    color: hsl(var(--wa) / 0.8)}
.hover\:text-warning\/90:hover {
    color: hsl(var(--wa) / 0.9)}
.hover\:text-warning\/95:hover {
    color: hsl(var(--wa) / 0.95)}
.hover\:placeholder-base-100:hover::placeholder {
    color: hsl(var(--b1) / 1)}
.hover\:placeholder-base-100\/0:hover::placeholder {
    color: hsl(var(--b1) / 0)}
.hover\:placeholder-base-100\/10:hover::placeholder {
    color: hsl(var(--b1) / 0.1)}
.hover\:placeholder-base-100\/100:hover::placeholder {
    color: hsl(var(--b1) / 1)}
.hover\:placeholder-base-100\/20:hover::placeholder {
    color: hsl(var(--b1) / 0.2)}
.hover\:placeholder-base-100\/25:hover::placeholder {
    color: hsl(var(--b1) / 0.25)}
.hover\:placeholder-base-100\/30:hover::placeholder {
    color: hsl(var(--b1) / 0.3)}
.hover\:placeholder-base-100\/40:hover::placeholder {
    color: hsl(var(--b1) / 0.4)}
.hover\:placeholder-base-100\/5:hover::placeholder {
    color: hsl(var(--b1) / 0.05)}
.hover\:placeholder-base-100\/50:hover::placeholder {
    color: hsl(var(--b1) / 0.5)}
.hover\:placeholder-base-100\/60:hover::placeholder {
    color: hsl(var(--b1) / 0.6)}
.hover\:placeholder-base-100\/70:hover::placeholder {
    color: hsl(var(--b1) / 0.7)}
.hover\:placeholder-base-100\/75:hover::placeholder {
    color: hsl(var(--b1) / 0.75)}
.hover\:placeholder-base-100\/80:hover::placeholder {
    color: hsl(var(--b1) / 0.8)}
.hover\:placeholder-base-100\/90:hover::placeholder {
    color: hsl(var(--b1) / 0.9)}
.hover\:placeholder-base-100\/95:hover::placeholder {
    color: hsl(var(--b1) / 0.95)}
.hover\:placeholder-base-200:hover::placeholder {
    color: hsl(var(--b2) / 1)}
.hover\:placeholder-base-200\/0:hover::placeholder {
    color: hsl(var(--b2) / 0)}
.hover\:placeholder-base-200\/10:hover::placeholder {
    color: hsl(var(--b2) / 0.1)}
.hover\:placeholder-base-200\/100:hover::placeholder {
    color: hsl(var(--b2) / 1)}
.hover\:placeholder-base-200\/20:hover::placeholder {
    color: hsl(var(--b2) / 0.2)}
.hover\:placeholder-base-200\/25:hover::placeholder {
    color: hsl(var(--b2) / 0.25)}
.hover\:placeholder-base-200\/30:hover::placeholder {
    color: hsl(var(--b2) / 0.3)}
.hover\:placeholder-base-200\/40:hover::placeholder {
    color: hsl(var(--b2) / 0.4)}
.hover\:placeholder-base-200\/5:hover::placeholder {
    color: hsl(var(--b2) / 0.05)}
.hover\:placeholder-base-200\/50:hover::placeholder {
    color: hsl(var(--b2) / 0.5)}
.hover\:placeholder-base-200\/60:hover::placeholder {
    color: hsl(var(--b2) / 0.6)}
.hover\:placeholder-base-200\/70:hover::placeholder {
    color: hsl(var(--b2) / 0.7)}
.hover\:placeholder-base-200\/75:hover::placeholder {
    color: hsl(var(--b2) / 0.75)}
.hover\:placeholder-base-200\/80:hover::placeholder {
    color: hsl(var(--b2) / 0.8)}
.hover\:placeholder-base-200\/90:hover::placeholder {
    color: hsl(var(--b2) / 0.9)}
.hover\:placeholder-base-200\/95:hover::placeholder {
    color: hsl(var(--b2) / 0.95)}
.hover\:placeholder-base-300:hover::placeholder {
    color: hsl(var(--b3) / 1)}
.hover\:placeholder-base-300\/0:hover::placeholder {
    color: hsl(var(--b3) / 0)}
.hover\:placeholder-base-300\/10:hover::placeholder {
    color: hsl(var(--b3) / 0.1)}
.hover\:placeholder-base-300\/100:hover::placeholder {
    color: hsl(var(--b3) / 1)}
.hover\:placeholder-base-300\/20:hover::placeholder {
    color: hsl(var(--b3) / 0.2)}
.hover\:placeholder-base-300\/25:hover::placeholder {
    color: hsl(var(--b3) / 0.25)}
.hover\:placeholder-base-300\/30:hover::placeholder {
    color: hsl(var(--b3) / 0.3)}
.hover\:placeholder-base-300\/40:hover::placeholder {
    color: hsl(var(--b3) / 0.4)}
.hover\:placeholder-base-300\/5:hover::placeholder {
    color: hsl(var(--b3) / 0.05)}
.hover\:placeholder-base-300\/50:hover::placeholder {
    color: hsl(var(--b3) / 0.5)}
.hover\:placeholder-base-300\/60:hover::placeholder {
    color: hsl(var(--b3) / 0.6)}
.hover\:placeholder-base-300\/70:hover::placeholder {
    color: hsl(var(--b3) / 0.7)}
.hover\:placeholder-base-300\/75:hover::placeholder {
    color: hsl(var(--b3) / 0.75)}
.hover\:placeholder-base-300\/80:hover::placeholder {
    color: hsl(var(--b3) / 0.8)}
.hover\:placeholder-base-300\/90:hover::placeholder {
    color: hsl(var(--b3) / 0.9)}
.hover\:placeholder-base-300\/95:hover::placeholder {
    color: hsl(var(--b3) / 0.95)}
.hover\:placeholder-base-content:hover::placeholder {
    color: hsl(var(--bc) / 1)}
.hover\:placeholder-base-content\/0:hover::placeholder {
    color: hsl(var(--bc) / 0)}
.hover\:placeholder-base-content\/10:hover::placeholder {
    color: hsl(var(--bc) / 0.1)}
.hover\:placeholder-base-content\/100:hover::placeholder {
    color: hsl(var(--bc) / 1)}
.hover\:placeholder-base-content\/20:hover::placeholder {
    color: hsl(var(--bc) / 0.2)}
.hover\:placeholder-base-content\/25:hover::placeholder {
    color: hsl(var(--bc) / 0.25)}
.hover\:placeholder-base-content\/30:hover::placeholder {
    color: hsl(var(--bc) / 0.3)}
.hover\:placeholder-base-content\/40:hover::placeholder {
    color: hsl(var(--bc) / 0.4)}
.hover\:placeholder-base-content\/5:hover::placeholder {
    color: hsl(var(--bc) / 0.05)}
.hover\:placeholder-base-content\/50:hover::placeholder {
    color: hsl(var(--bc) / 0.5)}
.hover\:placeholder-base-content\/60:hover::placeholder {
    color: hsl(var(--bc) / 0.6)}
.hover\:placeholder-base-content\/70:hover::placeholder {
    color: hsl(var(--bc) / 0.7)}
.hover\:placeholder-base-content\/75:hover::placeholder {
    color: hsl(var(--bc) / 0.75)}
.hover\:placeholder-base-content\/80:hover::placeholder {
    color: hsl(var(--bc) / 0.8)}
.hover\:placeholder-base-content\/90:hover::placeholder {
    color: hsl(var(--bc) / 0.9)}
.hover\:placeholder-base-content\/95:hover::placeholder {
    color: hsl(var(--bc) / 0.95)}
.hover\:placeholder-error:hover::placeholder {
    color: hsl(var(--er) / 1)}
.hover\:placeholder-error-content:hover::placeholder {
    color: hsl(var(--erc) / 1)}
.hover\:placeholder-error-content\/0:hover::placeholder {
    color: hsl(var(--erc) / 0)}
.hover\:placeholder-error-content\/10:hover::placeholder {
    color: hsl(var(--erc) / 0.1)}
.hover\:placeholder-error-content\/100:hover::placeholder {
    color: hsl(var(--erc) / 1)}
.hover\:placeholder-error-content\/20:hover::placeholder {
    color: hsl(var(--erc) / 0.2)}
.hover\:placeholder-error-content\/25:hover::placeholder {
    color: hsl(var(--erc) / 0.25)}
.hover\:placeholder-error-content\/30:hover::placeholder {
    color: hsl(var(--erc) / 0.3)}
.hover\:placeholder-error-content\/40:hover::placeholder {
    color: hsl(var(--erc) / 0.4)}
.hover\:placeholder-error-content\/5:hover::placeholder {
    color: hsl(var(--erc) / 0.05)}
.hover\:placeholder-error-content\/50:hover::placeholder {
    color: hsl(var(--erc) / 0.5)}
.hover\:placeholder-error-content\/60:hover::placeholder {
    color: hsl(var(--erc) / 0.6)}
.hover\:placeholder-error-content\/70:hover::placeholder {
    color: hsl(var(--erc) / 0.7)}
.hover\:placeholder-error-content\/75:hover::placeholder {
    color: hsl(var(--erc) / 0.75)}
.hover\:placeholder-error-content\/80:hover::placeholder {
    color: hsl(var(--erc) / 0.8)}
.hover\:placeholder-error-content\/90:hover::placeholder {
    color: hsl(var(--erc) / 0.9)}
.hover\:placeholder-error-content\/95:hover::placeholder {
    color: hsl(var(--erc) / 0.95)}
.hover\:placeholder-error\/0:hover::placeholder {
    color: hsl(var(--er) / 0)}
.hover\:placeholder-error\/10:hover::placeholder {
    color: hsl(var(--er) / 0.1)}
.hover\:placeholder-error\/100:hover::placeholder {
    color: hsl(var(--er) / 1)}
.hover\:placeholder-error\/20:hover::placeholder {
    color: hsl(var(--er) / 0.2)}
.hover\:placeholder-error\/25:hover::placeholder {
    color: hsl(var(--er) / 0.25)}
.hover\:placeholder-error\/30:hover::placeholder {
    color: hsl(var(--er) / 0.3)}
.hover\:placeholder-error\/40:hover::placeholder {
    color: hsl(var(--er) / 0.4)}
.hover\:placeholder-error\/5:hover::placeholder {
    color: hsl(var(--er) / 0.05)}
.hover\:placeholder-error\/50:hover::placeholder {
    color: hsl(var(--er) / 0.5)}
.hover\:placeholder-error\/60:hover::placeholder {
    color: hsl(var(--er) / 0.6)}
.hover\:placeholder-error\/70:hover::placeholder {
    color: hsl(var(--er) / 0.7)}
.hover\:placeholder-error\/75:hover::placeholder {
    color: hsl(var(--er) / 0.75)}
.hover\:placeholder-error\/80:hover::placeholder {
    color: hsl(var(--er) / 0.8)}
.hover\:placeholder-error\/90:hover::placeholder {
    color: hsl(var(--er) / 0.9)}
.hover\:placeholder-error\/95:hover::placeholder {
    color: hsl(var(--er) / 0.95)}
.hover\:placeholder-info:hover::placeholder {
    color: hsl(var(--in) / 1)}
.hover\:placeholder-info-content:hover::placeholder {
    color: hsl(var(--inc) / 1)}
.hover\:placeholder-info-content\/0:hover::placeholder {
    color: hsl(var(--inc) / 0)}
.hover\:placeholder-info-content\/10:hover::placeholder {
    color: hsl(var(--inc) / 0.1)}
.hover\:placeholder-info-content\/100:hover::placeholder {
    color: hsl(var(--inc) / 1)}
.hover\:placeholder-info-content\/20:hover::placeholder {
    color: hsl(var(--inc) / 0.2)}
.hover\:placeholder-info-content\/25:hover::placeholder {
    color: hsl(var(--inc) / 0.25)}
.hover\:placeholder-info-content\/30:hover::placeholder {
    color: hsl(var(--inc) / 0.3)}
.hover\:placeholder-info-content\/40:hover::placeholder {
    color: hsl(var(--inc) / 0.4)}
.hover\:placeholder-info-content\/5:hover::placeholder {
    color: hsl(var(--inc) / 0.05)}
.hover\:placeholder-info-content\/50:hover::placeholder {
    color: hsl(var(--inc) / 0.5)}
.hover\:placeholder-info-content\/60:hover::placeholder {
    color: hsl(var(--inc) / 0.6)}
.hover\:placeholder-info-content\/70:hover::placeholder {
    color: hsl(var(--inc) / 0.7)}
.hover\:placeholder-info-content\/75:hover::placeholder {
    color: hsl(var(--inc) / 0.75)}
.hover\:placeholder-info-content\/80:hover::placeholder {
    color: hsl(var(--inc) / 0.8)}
.hover\:placeholder-info-content\/90:hover::placeholder {
    color: hsl(var(--inc) / 0.9)}
.hover\:placeholder-info-content\/95:hover::placeholder {
    color: hsl(var(--inc) / 0.95)}
.hover\:placeholder-info\/0:hover::placeholder {
    color: hsl(var(--in) / 0)}
.hover\:placeholder-info\/10:hover::placeholder {
    color: hsl(var(--in) / 0.1)}
.hover\:placeholder-info\/100:hover::placeholder {
    color: hsl(var(--in) / 1)}
.hover\:placeholder-info\/20:hover::placeholder {
    color: hsl(var(--in) / 0.2)}
.hover\:placeholder-info\/25:hover::placeholder {
    color: hsl(var(--in) / 0.25)}
.hover\:placeholder-info\/30:hover::placeholder {
    color: hsl(var(--in) / 0.3)}
.hover\:placeholder-info\/40:hover::placeholder {
    color: hsl(var(--in) / 0.4)}
.hover\:placeholder-info\/5:hover::placeholder {
    color: hsl(var(--in) / 0.05)}
.hover\:placeholder-info\/50:hover::placeholder {
    color: hsl(var(--in) / 0.5)}
.hover\:placeholder-info\/60:hover::placeholder {
    color: hsl(var(--in) / 0.6)}
.hover\:placeholder-info\/70:hover::placeholder {
    color: hsl(var(--in) / 0.7)}
.hover\:placeholder-info\/75:hover::placeholder {
    color: hsl(var(--in) / 0.75)}
.hover\:placeholder-info\/80:hover::placeholder {
    color: hsl(var(--in) / 0.8)}
.hover\:placeholder-info\/90:hover::placeholder {
    color: hsl(var(--in) / 0.9)}
.hover\:placeholder-info\/95:hover::placeholder {
    color: hsl(var(--in) / 0.95)}
.hover\:placeholder-success:hover::placeholder {
    color: hsl(var(--su) / 1)}
.hover\:placeholder-success-content:hover::placeholder {
    color: hsl(var(--suc) / 1)}
.hover\:placeholder-success-content\/0:hover::placeholder {
    color: hsl(var(--suc) / 0)}
.hover\:placeholder-success-content\/10:hover::placeholder {
    color: hsl(var(--suc) / 0.1)}
.hover\:placeholder-success-content\/100:hover::placeholder {
    color: hsl(var(--suc) / 1)}
.hover\:placeholder-success-content\/20:hover::placeholder {
    color: hsl(var(--suc) / 0.2)}
.hover\:placeholder-success-content\/25:hover::placeholder {
    color: hsl(var(--suc) / 0.25)}
.hover\:placeholder-success-content\/30:hover::placeholder {
    color: hsl(var(--suc) / 0.3)}
.hover\:placeholder-success-content\/40:hover::placeholder {
    color: hsl(var(--suc) / 0.4)}
.hover\:placeholder-success-content\/5:hover::placeholder {
    color: hsl(var(--suc) / 0.05)}
.hover\:placeholder-success-content\/50:hover::placeholder {
    color: hsl(var(--suc) / 0.5)}
.hover\:placeholder-success-content\/60:hover::placeholder {
    color: hsl(var(--suc) / 0.6)}
.hover\:placeholder-success-content\/70:hover::placeholder {
    color: hsl(var(--suc) / 0.7)}
.hover\:placeholder-success-content\/75:hover::placeholder {
    color: hsl(var(--suc) / 0.75)}
.hover\:placeholder-success-content\/80:hover::placeholder {
    color: hsl(var(--suc) / 0.8)}
.hover\:placeholder-success-content\/90:hover::placeholder {
    color: hsl(var(--suc) / 0.9)}
.hover\:placeholder-success-content\/95:hover::placeholder {
    color: hsl(var(--suc) / 0.95)}
.hover\:placeholder-success\/0:hover::placeholder {
    color: hsl(var(--su) / 0)}
.hover\:placeholder-success\/10:hover::placeholder {
    color: hsl(var(--su) / 0.1)}
.hover\:placeholder-success\/100:hover::placeholder {
    color: hsl(var(--su) / 1)}
.hover\:placeholder-success\/20:hover::placeholder {
    color: hsl(var(--su) / 0.2)}
.hover\:placeholder-success\/25:hover::placeholder {
    color: hsl(var(--su) / 0.25)}
.hover\:placeholder-success\/30:hover::placeholder {
    color: hsl(var(--su) / 0.3)}
.hover\:placeholder-success\/40:hover::placeholder {
    color: hsl(var(--su) / 0.4)}
.hover\:placeholder-success\/5:hover::placeholder {
    color: hsl(var(--su) / 0.05)}
.hover\:placeholder-success\/50:hover::placeholder {
    color: hsl(var(--su) / 0.5)}
.hover\:placeholder-success\/60:hover::placeholder {
    color: hsl(var(--su) / 0.6)}
.hover\:placeholder-success\/70:hover::placeholder {
    color: hsl(var(--su) / 0.7)}
.hover\:placeholder-success\/75:hover::placeholder {
    color: hsl(var(--su) / 0.75)}
.hover\:placeholder-success\/80:hover::placeholder {
    color: hsl(var(--su) / 0.8)}
.hover\:placeholder-success\/90:hover::placeholder {
    color: hsl(var(--su) / 0.9)}
.hover\:placeholder-success\/95:hover::placeholder {
    color: hsl(var(--su) / 0.95)}
.hover\:placeholder-warning:hover::placeholder {
    color: hsl(var(--wa) / 1)}
.hover\:placeholder-warning-content:hover::placeholder {
    color: hsl(var(--wac) / 1)}
.hover\:placeholder-warning-content\/0:hover::placeholder {
    color: hsl(var(--wac) / 0)}
.hover\:placeholder-warning-content\/10:hover::placeholder {
    color: hsl(var(--wac) / 0.1)}
.hover\:placeholder-warning-content\/100:hover::placeholder {
    color: hsl(var(--wac) / 1)}
.hover\:placeholder-warning-content\/20:hover::placeholder {
    color: hsl(var(--wac) / 0.2)}
.hover\:placeholder-warning-content\/25:hover::placeholder {
    color: hsl(var(--wac) / 0.25)}
.hover\:placeholder-warning-content\/30:hover::placeholder {
    color: hsl(var(--wac) / 0.3)}
.hover\:placeholder-warning-content\/40:hover::placeholder {
    color: hsl(var(--wac) / 0.4)}
.hover\:placeholder-warning-content\/5:hover::placeholder {
    color: hsl(var(--wac) / 0.05)}
.hover\:placeholder-warning-content\/50:hover::placeholder {
    color: hsl(var(--wac) / 0.5)}
.hover\:placeholder-warning-content\/60:hover::placeholder {
    color: hsl(var(--wac) / 0.6)}
.hover\:placeholder-warning-content\/70:hover::placeholder {
    color: hsl(var(--wac) / 0.7)}
.hover\:placeholder-warning-content\/75:hover::placeholder {
    color: hsl(var(--wac) / 0.75)}
.hover\:placeholder-warning-content\/80:hover::placeholder {
    color: hsl(var(--wac) / 0.8)}
.hover\:placeholder-warning-content\/90:hover::placeholder {
    color: hsl(var(--wac) / 0.9)}
.hover\:placeholder-warning-content\/95:hover::placeholder {
    color: hsl(var(--wac) / 0.95)}
.hover\:placeholder-warning\/0:hover::placeholder {
    color: hsl(var(--wa) / 0)}
.hover\:placeholder-warning\/10:hover::placeholder {
    color: hsl(var(--wa) / 0.1)}
.hover\:placeholder-warning\/100:hover::placeholder {
    color: hsl(var(--wa) / 1)}
.hover\:placeholder-warning\/20:hover::placeholder {
    color: hsl(var(--wa) / 0.2)}
.hover\:placeholder-warning\/25:hover::placeholder {
    color: hsl(var(--wa) / 0.25)}
.hover\:placeholder-warning\/30:hover::placeholder {
    color: hsl(var(--wa) / 0.3)}
.hover\:placeholder-warning\/40:hover::placeholder {
    color: hsl(var(--wa) / 0.4)}
.hover\:placeholder-warning\/5:hover::placeholder {
    color: hsl(var(--wa) / 0.05)}
.hover\:placeholder-warning\/50:hover::placeholder {
    color: hsl(var(--wa) / 0.5)}
.hover\:placeholder-warning\/60:hover::placeholder {
    color: hsl(var(--wa) / 0.6)}
.hover\:placeholder-warning\/70:hover::placeholder {
    color: hsl(var(--wa) / 0.7)}
.hover\:placeholder-warning\/75:hover::placeholder {
    color: hsl(var(--wa) / 0.75)}
.hover\:placeholder-warning\/80:hover::placeholder {
    color: hsl(var(--wa) / 0.8)}
.hover\:placeholder-warning\/90:hover::placeholder {
    color: hsl(var(--wa) / 0.9)}
.hover\:placeholder-warning\/95:hover::placeholder {
    color: hsl(var(--wa) / 0.95)}
.hover\:ring-base-100:hover {
    --tw-ring-color: hsl(var(--b1) / 1)}
.hover\:ring-base-100\/0:hover {
    --tw-ring-color: hsl(var(--b1) / 0)}
.hover\:ring-base-100\/10:hover {
    --tw-ring-color: hsl(var(--b1) / 0.1)}
.hover\:ring-base-100\/100:hover {
    --tw-ring-color: hsl(var(--b1) / 1)}
.hover\:ring-base-100\/20:hover {
    --tw-ring-color: hsl(var(--b1) / 0.2)}
.hover\:ring-base-100\/25:hover {
    --tw-ring-color: hsl(var(--b1) / 0.25)}
.hover\:ring-base-100\/30:hover {
    --tw-ring-color: hsl(var(--b1) / 0.3)}
.hover\:ring-base-100\/40:hover {
    --tw-ring-color: hsl(var(--b1) / 0.4)}
.hover\:ring-base-100\/5:hover {
    --tw-ring-color: hsl(var(--b1) / 0.05)}
.hover\:ring-base-100\/50:hover {
    --tw-ring-color: hsl(var(--b1) / 0.5)}
.hover\:ring-base-100\/60:hover {
    --tw-ring-color: hsl(var(--b1) / 0.6)}
.hover\:ring-base-100\/70:hover {
    --tw-ring-color: hsl(var(--b1) / 0.7)}
.hover\:ring-base-100\/75:hover {
    --tw-ring-color: hsl(var(--b1) / 0.75)}
.hover\:ring-base-100\/80:hover {
    --tw-ring-color: hsl(var(--b1) / 0.8)}
.hover\:ring-base-100\/90:hover {
    --tw-ring-color: hsl(var(--b1) / 0.9)}
.hover\:ring-base-100\/95:hover {
    --tw-ring-color: hsl(var(--b1) / 0.95)}
.hover\:ring-base-200:hover {
    --tw-ring-color: hsl(var(--b2) / 1)}
.hover\:ring-base-200\/0:hover {
    --tw-ring-color: hsl(var(--b2) / 0)}
.hover\:ring-base-200\/10:hover {
    --tw-ring-color: hsl(var(--b2) / 0.1)}
.hover\:ring-base-200\/100:hover {
    --tw-ring-color: hsl(var(--b2) / 1)}
.hover\:ring-base-200\/20:hover {
    --tw-ring-color: hsl(var(--b2) / 0.2)}
.hover\:ring-base-200\/25:hover {
    --tw-ring-color: hsl(var(--b2) / 0.25)}
.hover\:ring-base-200\/30:hover {
    --tw-ring-color: hsl(var(--b2) / 0.3)}
.hover\:ring-base-200\/40:hover {
    --tw-ring-color: hsl(var(--b2) / 0.4)}
.hover\:ring-base-200\/5:hover {
    --tw-ring-color: hsl(var(--b2) / 0.05)}
.hover\:ring-base-200\/50:hover {
    --tw-ring-color: hsl(var(--b2) / 0.5)}
.hover\:ring-base-200\/60:hover {
    --tw-ring-color: hsl(var(--b2) / 0.6)}
.hover\:ring-base-200\/70:hover {
    --tw-ring-color: hsl(var(--b2) / 0.7)}
.hover\:ring-base-200\/75:hover {
    --tw-ring-color: hsl(var(--b2) / 0.75)}
.hover\:ring-base-200\/80:hover {
    --tw-ring-color: hsl(var(--b2) / 0.8)}
.hover\:ring-base-200\/90:hover {
    --tw-ring-color: hsl(var(--b2) / 0.9)}
.hover\:ring-base-200\/95:hover {
    --tw-ring-color: hsl(var(--b2) / 0.95)}
.hover\:ring-base-300:hover {
    --tw-ring-color: hsl(var(--b3) / 1)}
.hover\:ring-base-300\/0:hover {
    --tw-ring-color: hsl(var(--b3) / 0)}
.hover\:ring-base-300\/10:hover {
    --tw-ring-color: hsl(var(--b3) / 0.1)}
.hover\:ring-base-300\/100:hover {
    --tw-ring-color: hsl(var(--b3) / 1)}
.hover\:ring-base-300\/20:hover {
    --tw-ring-color: hsl(var(--b3) / 0.2)}
.hover\:ring-base-300\/25:hover {
    --tw-ring-color: hsl(var(--b3) / 0.25)}
.hover\:ring-base-300\/30:hover {
    --tw-ring-color: hsl(var(--b3) / 0.3)}
.hover\:ring-base-300\/40:hover {
    --tw-ring-color: hsl(var(--b3) / 0.4)}
.hover\:ring-base-300\/5:hover {
    --tw-ring-color: hsl(var(--b3) / 0.05)}
.hover\:ring-base-300\/50:hover {
    --tw-ring-color: hsl(var(--b3) / 0.5)}
.hover\:ring-base-300\/60:hover {
    --tw-ring-color: hsl(var(--b3) / 0.6)}
.hover\:ring-base-300\/70:hover {
    --tw-ring-color: hsl(var(--b3) / 0.7)}
.hover\:ring-base-300\/75:hover {
    --tw-ring-color: hsl(var(--b3) / 0.75)}
.hover\:ring-base-300\/80:hover {
    --tw-ring-color: hsl(var(--b3) / 0.8)}
.hover\:ring-base-300\/90:hover {
    --tw-ring-color: hsl(var(--b3) / 0.9)}
.hover\:ring-base-300\/95:hover {
    --tw-ring-color: hsl(var(--b3) / 0.95)}
.hover\:ring-base-content:hover {
    --tw-ring-color: hsl(var(--bc) / 1)}
.hover\:ring-base-content\/0:hover {
    --tw-ring-color: hsl(var(--bc) / 0)}
.hover\:ring-base-content\/10:hover {
    --tw-ring-color: hsl(var(--bc) / 0.1)}
.hover\:ring-base-content\/100:hover {
    --tw-ring-color: hsl(var(--bc) / 1)}
.hover\:ring-base-content\/20:hover {
    --tw-ring-color: hsl(var(--bc) / 0.2)}
.hover\:ring-base-content\/25:hover {
    --tw-ring-color: hsl(var(--bc) / 0.25)}
.hover\:ring-base-content\/30:hover {
    --tw-ring-color: hsl(var(--bc) / 0.3)}
.hover\:ring-base-content\/40:hover {
    --tw-ring-color: hsl(var(--bc) / 0.4)}
.hover\:ring-base-content\/5:hover {
    --tw-ring-color: hsl(var(--bc) / 0.05)}
.hover\:ring-base-content\/50:hover {
    --tw-ring-color: hsl(var(--bc) / 0.5)}
.hover\:ring-base-content\/60:hover {
    --tw-ring-color: hsl(var(--bc) / 0.6)}
.hover\:ring-base-content\/70:hover {
    --tw-ring-color: hsl(var(--bc) / 0.7)}
.hover\:ring-base-content\/75:hover {
    --tw-ring-color: hsl(var(--bc) / 0.75)}
.hover\:ring-base-content\/80:hover {
    --tw-ring-color: hsl(var(--bc) / 0.8)}
.hover\:ring-base-content\/90:hover {
    --tw-ring-color: hsl(var(--bc) / 0.9)}
.hover\:ring-base-content\/95:hover {
    --tw-ring-color: hsl(var(--bc) / 0.95)}
.hover\:ring-error:hover {
    --tw-ring-color: hsl(var(--er) / 1)}
.hover\:ring-error-content:hover {
    --tw-ring-color: hsl(var(--erc) / 1)}
.hover\:ring-error-content\/0:hover {
    --tw-ring-color: hsl(var(--erc) / 0)}
.hover\:ring-error-content\/10:hover {
    --tw-ring-color: hsl(var(--erc) / 0.1)}
.hover\:ring-error-content\/100:hover {
    --tw-ring-color: hsl(var(--erc) / 1)}
.hover\:ring-error-content\/20:hover {
    --tw-ring-color: hsl(var(--erc) / 0.2)}
.hover\:ring-error-content\/25:hover {
    --tw-ring-color: hsl(var(--erc) / 0.25)}
.hover\:ring-error-content\/30:hover {
    --tw-ring-color: hsl(var(--erc) / 0.3)}
.hover\:ring-error-content\/40:hover {
    --tw-ring-color: hsl(var(--erc) / 0.4)}
.hover\:ring-error-content\/5:hover {
    --tw-ring-color: hsl(var(--erc) / 0.05)}
.hover\:ring-error-content\/50:hover {
    --tw-ring-color: hsl(var(--erc) / 0.5)}
.hover\:ring-error-content\/60:hover {
    --tw-ring-color: hsl(var(--erc) / 0.6)}
.hover\:ring-error-content\/70:hover {
    --tw-ring-color: hsl(var(--erc) / 0.7)}
.hover\:ring-error-content\/75:hover {
    --tw-ring-color: hsl(var(--erc) / 0.75)}
.hover\:ring-error-content\/80:hover {
    --tw-ring-color: hsl(var(--erc) / 0.8)}
.hover\:ring-error-content\/90:hover {
    --tw-ring-color: hsl(var(--erc) / 0.9)}
.hover\:ring-error-content\/95:hover {
    --tw-ring-color: hsl(var(--erc) / 0.95)}
.hover\:ring-error\/0:hover {
    --tw-ring-color: hsl(var(--er) / 0)}
.hover\:ring-error\/10:hover {
    --tw-ring-color: hsl(var(--er) / 0.1)}
.hover\:ring-error\/100:hover {
    --tw-ring-color: hsl(var(--er) / 1)}
.hover\:ring-error\/20:hover {
    --tw-ring-color: hsl(var(--er) / 0.2)}
.hover\:ring-error\/25:hover {
    --tw-ring-color: hsl(var(--er) / 0.25)}
.hover\:ring-error\/30:hover {
    --tw-ring-color: hsl(var(--er) / 0.3)}
.hover\:ring-error\/40:hover {
    --tw-ring-color: hsl(var(--er) / 0.4)}
.hover\:ring-error\/5:hover {
    --tw-ring-color: hsl(var(--er) / 0.05)}
.hover\:ring-error\/50:hover {
    --tw-ring-color: hsl(var(--er) / 0.5)}
.hover\:ring-error\/60:hover {
    --tw-ring-color: hsl(var(--er) / 0.6)}
.hover\:ring-error\/70:hover {
    --tw-ring-color: hsl(var(--er) / 0.7)}
.hover\:ring-error\/75:hover {
    --tw-ring-color: hsl(var(--er) / 0.75)}
.hover\:ring-error\/80:hover {
    --tw-ring-color: hsl(var(--er) / 0.8)}
.hover\:ring-error\/90:hover {
    --tw-ring-color: hsl(var(--er) / 0.9)}
.hover\:ring-error\/95:hover {
    --tw-ring-color: hsl(var(--er) / 0.95)}
.hover\:ring-info:hover {
    --tw-ring-color: hsl(var(--in) / 1)}
.hover\:ring-info-content:hover {
    --tw-ring-color: hsl(var(--inc) / 1)}
.hover\:ring-info-content\/0:hover {
    --tw-ring-color: hsl(var(--inc) / 0)}
.hover\:ring-info-content\/10:hover {
    --tw-ring-color: hsl(var(--inc) / 0.1)}
.hover\:ring-info-content\/100:hover {
    --tw-ring-color: hsl(var(--inc) / 1)}
.hover\:ring-info-content\/20:hover {
    --tw-ring-color: hsl(var(--inc) / 0.2)}
.hover\:ring-info-content\/25:hover {
    --tw-ring-color: hsl(var(--inc) / 0.25)}
.hover\:ring-info-content\/30:hover {
    --tw-ring-color: hsl(var(--inc) / 0.3)}
.hover\:ring-info-content\/40:hover {
    --tw-ring-color: hsl(var(--inc) / 0.4)}
.hover\:ring-info-content\/5:hover {
    --tw-ring-color: hsl(var(--inc) / 0.05)}
.hover\:ring-info-content\/50:hover {
    --tw-ring-color: hsl(var(--inc) / 0.5)}
.hover\:ring-info-content\/60:hover {
    --tw-ring-color: hsl(var(--inc) / 0.6)}
.hover\:ring-info-content\/70:hover {
    --tw-ring-color: hsl(var(--inc) / 0.7)}
.hover\:ring-info-content\/75:hover {
    --tw-ring-color: hsl(var(--inc) / 0.75)}
.hover\:ring-info-content\/80:hover {
    --tw-ring-color: hsl(var(--inc) / 0.8)}
.hover\:ring-info-content\/90:hover {
    --tw-ring-color: hsl(var(--inc) / 0.9)}
.hover\:ring-info-content\/95:hover {
    --tw-ring-color: hsl(var(--inc) / 0.95)}
.hover\:ring-info\/0:hover {
    --tw-ring-color: hsl(var(--in) / 0)}
.hover\:ring-info\/10:hover {
    --tw-ring-color: hsl(var(--in) / 0.1)}
.hover\:ring-info\/100:hover {
    --tw-ring-color: hsl(var(--in) / 1)}
.hover\:ring-info\/20:hover {
    --tw-ring-color: hsl(var(--in) / 0.2)}
.hover\:ring-info\/25:hover {
    --tw-ring-color: hsl(var(--in) / 0.25)}
.hover\:ring-info\/30:hover {
    --tw-ring-color: hsl(var(--in) / 0.3)}
.hover\:ring-info\/40:hover {
    --tw-ring-color: hsl(var(--in) / 0.4)}
.hover\:ring-info\/5:hover {
    --tw-ring-color: hsl(var(--in) / 0.05)}
.hover\:ring-info\/50:hover {
    --tw-ring-color: hsl(var(--in) / 0.5)}
.hover\:ring-info\/60:hover {
    --tw-ring-color: hsl(var(--in) / 0.6)}
.hover\:ring-info\/70:hover {
    --tw-ring-color: hsl(var(--in) / 0.7)}
.hover\:ring-info\/75:hover {
    --tw-ring-color: hsl(var(--in) / 0.75)}
.hover\:ring-info\/80:hover {
    --tw-ring-color: hsl(var(--in) / 0.8)}
.hover\:ring-info\/90:hover {
    --tw-ring-color: hsl(var(--in) / 0.9)}
.hover\:ring-info\/95:hover {
    --tw-ring-color: hsl(var(--in) / 0.95)}
.hover\:ring-success:hover {
    --tw-ring-color: hsl(var(--su) / 1)}
.hover\:ring-success-content:hover {
    --tw-ring-color: hsl(var(--suc) / 1)}
.hover\:ring-success-content\/0:hover {
    --tw-ring-color: hsl(var(--suc) / 0)}
.hover\:ring-success-content\/10:hover {
    --tw-ring-color: hsl(var(--suc) / 0.1)}
.hover\:ring-success-content\/100:hover {
    --tw-ring-color: hsl(var(--suc) / 1)}
.hover\:ring-success-content\/20:hover {
    --tw-ring-color: hsl(var(--suc) / 0.2)}
.hover\:ring-success-content\/25:hover {
    --tw-ring-color: hsl(var(--suc) / 0.25)}
.hover\:ring-success-content\/30:hover {
    --tw-ring-color: hsl(var(--suc) / 0.3)}
.hover\:ring-success-content\/40:hover {
    --tw-ring-color: hsl(var(--suc) / 0.4)}
.hover\:ring-success-content\/5:hover {
    --tw-ring-color: hsl(var(--suc) / 0.05)}
.hover\:ring-success-content\/50:hover {
    --tw-ring-color: hsl(var(--suc) / 0.5)}
.hover\:ring-success-content\/60:hover {
    --tw-ring-color: hsl(var(--suc) / 0.6)}
.hover\:ring-success-content\/70:hover {
    --tw-ring-color: hsl(var(--suc) / 0.7)}
.hover\:ring-success-content\/75:hover {
    --tw-ring-color: hsl(var(--suc) / 0.75)}
.hover\:ring-success-content\/80:hover {
    --tw-ring-color: hsl(var(--suc) / 0.8)}
.hover\:ring-success-content\/90:hover {
    --tw-ring-color: hsl(var(--suc) / 0.9)}
.hover\:ring-success-content\/95:hover {
    --tw-ring-color: hsl(var(--suc) / 0.95)}
.hover\:ring-success\/0:hover {
    --tw-ring-color: hsl(var(--su) / 0)}
.hover\:ring-success\/10:hover {
    --tw-ring-color: hsl(var(--su) / 0.1)}
.hover\:ring-success\/100:hover {
    --tw-ring-color: hsl(var(--su) / 1)}
.hover\:ring-success\/20:hover {
    --tw-ring-color: hsl(var(--su) / 0.2)}
.hover\:ring-success\/25:hover {
    --tw-ring-color: hsl(var(--su) / 0.25)}
.hover\:ring-success\/30:hover {
    --tw-ring-color: hsl(var(--su) / 0.3)}
.hover\:ring-success\/40:hover {
    --tw-ring-color: hsl(var(--su) / 0.4)}
.hover\:ring-success\/5:hover {
    --tw-ring-color: hsl(var(--su) / 0.05)}
.hover\:ring-success\/50:hover {
    --tw-ring-color: hsl(var(--su) / 0.5)}
.hover\:ring-success\/60:hover {
    --tw-ring-color: hsl(var(--su) / 0.6)}
.hover\:ring-success\/70:hover {
    --tw-ring-color: hsl(var(--su) / 0.7)}
.hover\:ring-success\/75:hover {
    --tw-ring-color: hsl(var(--su) / 0.75)}
.hover\:ring-success\/80:hover {
    --tw-ring-color: hsl(var(--su) / 0.8)}
.hover\:ring-success\/90:hover {
    --tw-ring-color: hsl(var(--su) / 0.9)}
.hover\:ring-success\/95:hover {
    --tw-ring-color: hsl(var(--su) / 0.95)}
.hover\:ring-warning:hover {
    --tw-ring-color: hsl(var(--wa) / 1)}
.hover\:ring-warning-content:hover {
    --tw-ring-color: hsl(var(--wac) / 1)}
.hover\:ring-warning-content\/0:hover {
    --tw-ring-color: hsl(var(--wac) / 0)}
.hover\:ring-warning-content\/10:hover {
    --tw-ring-color: hsl(var(--wac) / 0.1)}
.hover\:ring-warning-content\/100:hover {
    --tw-ring-color: hsl(var(--wac) / 1)}
.hover\:ring-warning-content\/20:hover {
    --tw-ring-color: hsl(var(--wac) / 0.2)}
.hover\:ring-warning-content\/25:hover {
    --tw-ring-color: hsl(var(--wac) / 0.25)}
.hover\:ring-warning-content\/30:hover {
    --tw-ring-color: hsl(var(--wac) / 0.3)}
.hover\:ring-warning-content\/40:hover {
    --tw-ring-color: hsl(var(--wac) / 0.4)}
.hover\:ring-warning-content\/5:hover {
    --tw-ring-color: hsl(var(--wac) / 0.05)}
.hover\:ring-warning-content\/50:hover {
    --tw-ring-color: hsl(var(--wac) / 0.5)}
.hover\:ring-warning-content\/60:hover {
    --tw-ring-color: hsl(var(--wac) / 0.6)}
.hover\:ring-warning-content\/70:hover {
    --tw-ring-color: hsl(var(--wac) / 0.7)}
.hover\:ring-warning-content\/75:hover {
    --tw-ring-color: hsl(var(--wac) / 0.75)}
.hover\:ring-warning-content\/80:hover {
    --tw-ring-color: hsl(var(--wac) / 0.8)}
.hover\:ring-warning-content\/90:hover {
    --tw-ring-color: hsl(var(--wac) / 0.9)}
.hover\:ring-warning-content\/95:hover {
    --tw-ring-color: hsl(var(--wac) / 0.95)}
.hover\:ring-warning\/0:hover {
    --tw-ring-color: hsl(var(--wa) / 0)}
.hover\:ring-warning\/10:hover {
    --tw-ring-color: hsl(var(--wa) / 0.1)}
.hover\:ring-warning\/100:hover {
    --tw-ring-color: hsl(var(--wa) / 1)}
.hover\:ring-warning\/20:hover {
    --tw-ring-color: hsl(var(--wa) / 0.2)}
.hover\:ring-warning\/25:hover {
    --tw-ring-color: hsl(var(--wa) / 0.25)}
.hover\:ring-warning\/30:hover {
    --tw-ring-color: hsl(var(--wa) / 0.3)}
.hover\:ring-warning\/40:hover {
    --tw-ring-color: hsl(var(--wa) / 0.4)}
.hover\:ring-warning\/5:hover {
    --tw-ring-color: hsl(var(--wa) / 0.05)}
.hover\:ring-warning\/50:hover {
    --tw-ring-color: hsl(var(--wa) / 0.5)}
.hover\:ring-warning\/60:hover {
    --tw-ring-color: hsl(var(--wa) / 0.6)}
.hover\:ring-warning\/70:hover {
    --tw-ring-color: hsl(var(--wa) / 0.7)}
.hover\:ring-warning\/75:hover {
    --tw-ring-color: hsl(var(--wa) / 0.75)}
.hover\:ring-warning\/80:hover {
    --tw-ring-color: hsl(var(--wa) / 0.8)}
.hover\:ring-warning\/90:hover {
    --tw-ring-color: hsl(var(--wa) / 0.9)}
.hover\:ring-warning\/95:hover {
    --tw-ring-color: hsl(var(--wa) / 0.95)}
.hover\:ring-offset-base-100:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 1)}
.hover\:ring-offset-base-100\/0:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0)}
.hover\:ring-offset-base-100\/10:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.1)}
.hover\:ring-offset-base-100\/100:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 1)}
.hover\:ring-offset-base-100\/20:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.2)}
.hover\:ring-offset-base-100\/25:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.25)}
.hover\:ring-offset-base-100\/30:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.3)}
.hover\:ring-offset-base-100\/40:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.4)}
.hover\:ring-offset-base-100\/5:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.05)}
.hover\:ring-offset-base-100\/50:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.5)}
.hover\:ring-offset-base-100\/60:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.6)}
.hover\:ring-offset-base-100\/70:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.7)}
.hover\:ring-offset-base-100\/75:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.75)}
.hover\:ring-offset-base-100\/80:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.8)}
.hover\:ring-offset-base-100\/90:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.9)}
.hover\:ring-offset-base-100\/95:hover {
    --tw-ring-offset-color: hsl(var(--b1) / 0.95)}
.hover\:ring-offset-base-200:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 1)}
.hover\:ring-offset-base-200\/0:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0)}
.hover\:ring-offset-base-200\/10:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.1)}
.hover\:ring-offset-base-200\/100:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 1)}
.hover\:ring-offset-base-200\/20:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.2)}
.hover\:ring-offset-base-200\/25:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.25)}
.hover\:ring-offset-base-200\/30:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.3)}
.hover\:ring-offset-base-200\/40:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.4)}
.hover\:ring-offset-base-200\/5:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.05)}
.hover\:ring-offset-base-200\/50:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.5)}
.hover\:ring-offset-base-200\/60:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.6)}
.hover\:ring-offset-base-200\/70:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.7)}
.hover\:ring-offset-base-200\/75:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.75)}
.hover\:ring-offset-base-200\/80:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.8)}
.hover\:ring-offset-base-200\/90:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.9)}
.hover\:ring-offset-base-200\/95:hover {
    --tw-ring-offset-color: hsl(var(--b2) / 0.95)}
.hover\:ring-offset-base-300:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 1)}
.hover\:ring-offset-base-300\/0:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0)}
.hover\:ring-offset-base-300\/10:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.1)}
.hover\:ring-offset-base-300\/100:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 1)}
.hover\:ring-offset-base-300\/20:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.2)}
.hover\:ring-offset-base-300\/25:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.25)}
.hover\:ring-offset-base-300\/30:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.3)}
.hover\:ring-offset-base-300\/40:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.4)}
.hover\:ring-offset-base-300\/5:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.05)}
.hover\:ring-offset-base-300\/50:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.5)}
.hover\:ring-offset-base-300\/60:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.6)}
.hover\:ring-offset-base-300\/70:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.7)}
.hover\:ring-offset-base-300\/75:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.75)}
.hover\:ring-offset-base-300\/80:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.8)}
.hover\:ring-offset-base-300\/90:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.9)}
.hover\:ring-offset-base-300\/95:hover {
    --tw-ring-offset-color: hsl(var(--b3) / 0.95)}
.hover\:ring-offset-base-content:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 1)}
.hover\:ring-offset-base-content\/0:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0)}
.hover\:ring-offset-base-content\/10:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.1)}
.hover\:ring-offset-base-content\/100:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 1)}
.hover\:ring-offset-base-content\/20:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.2)}
.hover\:ring-offset-base-content\/25:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.25)}
.hover\:ring-offset-base-content\/30:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.3)}
.hover\:ring-offset-base-content\/40:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.4)}
.hover\:ring-offset-base-content\/5:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.05)}
.hover\:ring-offset-base-content\/50:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.5)}
.hover\:ring-offset-base-content\/60:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.6)}
.hover\:ring-offset-base-content\/70:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.7)}
.hover\:ring-offset-base-content\/75:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.75)}
.hover\:ring-offset-base-content\/80:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.8)}
.hover\:ring-offset-base-content\/90:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.9)}
.hover\:ring-offset-base-content\/95:hover {
    --tw-ring-offset-color: hsl(var(--bc) / 0.95)}
.hover\:ring-offset-error:hover {
    --tw-ring-offset-color: hsl(var(--er) / 1)}
.hover\:ring-offset-error-content:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 1)}
.hover\:ring-offset-error-content\/0:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0)}
.hover\:ring-offset-error-content\/10:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.1)}
.hover\:ring-offset-error-content\/100:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 1)}
.hover\:ring-offset-error-content\/20:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.2)}
.hover\:ring-offset-error-content\/25:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.25)}
.hover\:ring-offset-error-content\/30:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.3)}
.hover\:ring-offset-error-content\/40:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.4)}
.hover\:ring-offset-error-content\/5:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.05)}
.hover\:ring-offset-error-content\/50:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.5)}
.hover\:ring-offset-error-content\/60:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.6)}
.hover\:ring-offset-error-content\/70:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.7)}
.hover\:ring-offset-error-content\/75:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.75)}
.hover\:ring-offset-error-content\/80:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.8)}
.hover\:ring-offset-error-content\/90:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.9)}
.hover\:ring-offset-error-content\/95:hover {
    --tw-ring-offset-color: hsl(var(--erc) / 0.95)}
.hover\:ring-offset-error\/0:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0)}
.hover\:ring-offset-error\/10:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.1)}
.hover\:ring-offset-error\/100:hover {
    --tw-ring-offset-color: hsl(var(--er) / 1)}
.hover\:ring-offset-error\/20:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.2)}
.hover\:ring-offset-error\/25:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.25)}
.hover\:ring-offset-error\/30:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.3)}
.hover\:ring-offset-error\/40:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.4)}
.hover\:ring-offset-error\/5:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.05)}
.hover\:ring-offset-error\/50:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.5)}
.hover\:ring-offset-error\/60:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.6)}
.hover\:ring-offset-error\/70:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.7)}
.hover\:ring-offset-error\/75:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.75)}
.hover\:ring-offset-error\/80:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.8)}
.hover\:ring-offset-error\/90:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.9)}
.hover\:ring-offset-error\/95:hover {
    --tw-ring-offset-color: hsl(var(--er) / 0.95)}
.hover\:ring-offset-info:hover {
    --tw-ring-offset-color: hsl(var(--in) / 1)}
.hover\:ring-offset-info-content:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 1)}
.hover\:ring-offset-info-content\/0:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0)}
.hover\:ring-offset-info-content\/10:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.1)}
.hover\:ring-offset-info-content\/100:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 1)}
.hover\:ring-offset-info-content\/20:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.2)}
.hover\:ring-offset-info-content\/25:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.25)}
.hover\:ring-offset-info-content\/30:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.3)}
.hover\:ring-offset-info-content\/40:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.4)}
.hover\:ring-offset-info-content\/5:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.05)}
.hover\:ring-offset-info-content\/50:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.5)}
.hover\:ring-offset-info-content\/60:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.6)}
.hover\:ring-offset-info-content\/70:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.7)}
.hover\:ring-offset-info-content\/75:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.75)}
.hover\:ring-offset-info-content\/80:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.8)}
.hover\:ring-offset-info-content\/90:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.9)}
.hover\:ring-offset-info-content\/95:hover {
    --tw-ring-offset-color: hsl(var(--inc) / 0.95)}
.hover\:ring-offset-info\/0:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0)}
.hover\:ring-offset-info\/10:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.1)}
.hover\:ring-offset-info\/100:hover {
    --tw-ring-offset-color: hsl(var(--in) / 1)}
.hover\:ring-offset-info\/20:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.2)}
.hover\:ring-offset-info\/25:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.25)}
.hover\:ring-offset-info\/30:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.3)}
.hover\:ring-offset-info\/40:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.4)}
.hover\:ring-offset-info\/5:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.05)}
.hover\:ring-offset-info\/50:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.5)}
.hover\:ring-offset-info\/60:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.6)}
.hover\:ring-offset-info\/70:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.7)}
.hover\:ring-offset-info\/75:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.75)}
.hover\:ring-offset-info\/80:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.8)}
.hover\:ring-offset-info\/90:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.9)}
.hover\:ring-offset-info\/95:hover {
    --tw-ring-offset-color: hsl(var(--in) / 0.95)}
.hover\:ring-offset-success:hover {
    --tw-ring-offset-color: hsl(var(--su) / 1)}
.hover\:ring-offset-success-content:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 1)}
.hover\:ring-offset-success-content\/0:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0)}
.hover\:ring-offset-success-content\/10:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.1)}
.hover\:ring-offset-success-content\/100:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 1)}
.hover\:ring-offset-success-content\/20:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.2)}
.hover\:ring-offset-success-content\/25:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.25)}
.hover\:ring-offset-success-content\/30:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.3)}
.hover\:ring-offset-success-content\/40:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.4)}
.hover\:ring-offset-success-content\/5:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.05)}
.hover\:ring-offset-success-content\/50:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.5)}
.hover\:ring-offset-success-content\/60:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.6)}
.hover\:ring-offset-success-content\/70:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.7)}
.hover\:ring-offset-success-content\/75:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.75)}
.hover\:ring-offset-success-content\/80:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.8)}
.hover\:ring-offset-success-content\/90:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.9)}
.hover\:ring-offset-success-content\/95:hover {
    --tw-ring-offset-color: hsl(var(--suc) / 0.95)}
.hover\:ring-offset-success\/0:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0)}
.hover\:ring-offset-success\/10:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.1)}
.hover\:ring-offset-success\/100:hover {
    --tw-ring-offset-color: hsl(var(--su) / 1)}
.hover\:ring-offset-success\/20:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.2)}
.hover\:ring-offset-success\/25:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.25)}
.hover\:ring-offset-success\/30:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.3)}
.hover\:ring-offset-success\/40:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.4)}
.hover\:ring-offset-success\/5:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.05)}
.hover\:ring-offset-success\/50:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.5)}
.hover\:ring-offset-success\/60:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.6)}
.hover\:ring-offset-success\/70:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.7)}
.hover\:ring-offset-success\/75:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.75)}
.hover\:ring-offset-success\/80:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.8)}
.hover\:ring-offset-success\/90:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.9)}
.hover\:ring-offset-success\/95:hover {
    --tw-ring-offset-color: hsl(var(--su) / 0.95)}
.hover\:ring-offset-warning:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 1)}
.hover\:ring-offset-warning-content:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 1)}
.hover\:ring-offset-warning-content\/0:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0)}
.hover\:ring-offset-warning-content\/10:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.1)}
.hover\:ring-offset-warning-content\/100:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 1)}
.hover\:ring-offset-warning-content\/20:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.2)}
.hover\:ring-offset-warning-content\/25:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.25)}
.hover\:ring-offset-warning-content\/30:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.3)}
.hover\:ring-offset-warning-content\/40:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.4)}
.hover\:ring-offset-warning-content\/5:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.05)}
.hover\:ring-offset-warning-content\/50:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.5)}
.hover\:ring-offset-warning-content\/60:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.6)}
.hover\:ring-offset-warning-content\/70:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.7)}
.hover\:ring-offset-warning-content\/75:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.75)}
.hover\:ring-offset-warning-content\/80:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.8)}
.hover\:ring-offset-warning-content\/90:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.9)}
.hover\:ring-offset-warning-content\/95:hover {
    --tw-ring-offset-color: hsl(var(--wac) / 0.95)}
.hover\:ring-offset-warning\/0:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0)}
.hover\:ring-offset-warning\/10:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.1)}
.hover\:ring-offset-warning\/100:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 1)}
.hover\:ring-offset-warning\/20:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.2)}
.hover\:ring-offset-warning\/25:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.25)}
.hover\:ring-offset-warning\/30:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.3)}
.hover\:ring-offset-warning\/40:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.4)}
.hover\:ring-offset-warning\/5:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.05)}
.hover\:ring-offset-warning\/50:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.5)}
.hover\:ring-offset-warning\/60:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.6)}
.hover\:ring-offset-warning\/70:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.7)}
.hover\:ring-offset-warning\/75:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.75)}
.hover\:ring-offset-warning\/80:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.8)}
.hover\:ring-offset-warning\/90:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.9)}
.hover\:ring-offset-warning\/95:hover {
    --tw-ring-offset-color: hsl(var(--wa) / 0.95)}
.focus\:divide-base-100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 1)}
.focus\:divide-base-100\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0)}
.focus\:divide-base-100\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.1)}
.focus\:divide-base-100\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 1)}
.focus\:divide-base-100\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.2)}
.focus\:divide-base-100\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.25)}
.focus\:divide-base-100\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.3)}
.focus\:divide-base-100\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.4)}
.focus\:divide-base-100\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.05)}
.focus\:divide-base-100\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.5)}
.focus\:divide-base-100\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.6)}
.focus\:divide-base-100\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.7)}
.focus\:divide-base-100\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.75)}
.focus\:divide-base-100\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.8)}
.focus\:divide-base-100\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.9)}
.focus\:divide-base-100\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b1) / 0.95)}
.focus\:divide-base-200:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 1)}
.focus\:divide-base-200\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0)}
.focus\:divide-base-200\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.1)}
.focus\:divide-base-200\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 1)}
.focus\:divide-base-200\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.2)}
.focus\:divide-base-200\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.25)}
.focus\:divide-base-200\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.3)}
.focus\:divide-base-200\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.4)}
.focus\:divide-base-200\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.05)}
.focus\:divide-base-200\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.5)}
.focus\:divide-base-200\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.6)}
.focus\:divide-base-200\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.7)}
.focus\:divide-base-200\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.75)}
.focus\:divide-base-200\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.8)}
.focus\:divide-base-200\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.9)}
.focus\:divide-base-200\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b2) / 0.95)}
.focus\:divide-base-300:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 1)}
.focus\:divide-base-300\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0)}
.focus\:divide-base-300\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.1)}
.focus\:divide-base-300\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 1)}
.focus\:divide-base-300\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.2)}
.focus\:divide-base-300\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.25)}
.focus\:divide-base-300\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.3)}
.focus\:divide-base-300\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.4)}
.focus\:divide-base-300\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.05)}
.focus\:divide-base-300\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.5)}
.focus\:divide-base-300\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.6)}
.focus\:divide-base-300\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.7)}
.focus\:divide-base-300\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.75)}
.focus\:divide-base-300\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.8)}
.focus\:divide-base-300\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.9)}
.focus\:divide-base-300\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--b3) / 0.95)}
.focus\:divide-base-content:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 1)}
.focus\:divide-base-content\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0)}
.focus\:divide-base-content\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.1)}
.focus\:divide-base-content\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 1)}
.focus\:divide-base-content\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.2)}
.focus\:divide-base-content\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.25)}
.focus\:divide-base-content\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.3)}
.focus\:divide-base-content\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.4)}
.focus\:divide-base-content\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.05)}
.focus\:divide-base-content\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.5)}
.focus\:divide-base-content\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.6)}
.focus\:divide-base-content\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.7)}
.focus\:divide-base-content\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.75)}
.focus\:divide-base-content\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.8)}
.focus\:divide-base-content\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.9)}
.focus\:divide-base-content\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--bc) / 0.95)}
.focus\:divide-error:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 1)}
.focus\:divide-error-content:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 1)}
.focus\:divide-error-content\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0)}
.focus\:divide-error-content\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.1)}
.focus\:divide-error-content\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 1)}
.focus\:divide-error-content\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.2)}
.focus\:divide-error-content\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.25)}
.focus\:divide-error-content\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.3)}
.focus\:divide-error-content\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.4)}
.focus\:divide-error-content\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.05)}
.focus\:divide-error-content\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.5)}
.focus\:divide-error-content\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.6)}
.focus\:divide-error-content\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.7)}
.focus\:divide-error-content\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.75)}
.focus\:divide-error-content\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.8)}
.focus\:divide-error-content\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.9)}
.focus\:divide-error-content\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--erc) / 0.95)}
.focus\:divide-error\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0)}
.focus\:divide-error\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.1)}
.focus\:divide-error\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 1)}
.focus\:divide-error\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.2)}
.focus\:divide-error\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.25)}
.focus\:divide-error\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.3)}
.focus\:divide-error\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.4)}
.focus\:divide-error\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.05)}
.focus\:divide-error\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.5)}
.focus\:divide-error\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.6)}
.focus\:divide-error\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.7)}
.focus\:divide-error\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.75)}
.focus\:divide-error\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.8)}
.focus\:divide-error\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.9)}
.focus\:divide-error\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--er) / 0.95)}
.focus\:divide-info:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 1)}
.focus\:divide-info-content:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 1)}
.focus\:divide-info-content\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0)}
.focus\:divide-info-content\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.1)}
.focus\:divide-info-content\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 1)}
.focus\:divide-info-content\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.2)}
.focus\:divide-info-content\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.25)}
.focus\:divide-info-content\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.3)}
.focus\:divide-info-content\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.4)}
.focus\:divide-info-content\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.05)}
.focus\:divide-info-content\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.5)}
.focus\:divide-info-content\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.6)}
.focus\:divide-info-content\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.7)}
.focus\:divide-info-content\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.75)}
.focus\:divide-info-content\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.8)}
.focus\:divide-info-content\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.9)}
.focus\:divide-info-content\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--inc) / 0.95)}
.focus\:divide-info\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0)}
.focus\:divide-info\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.1)}
.focus\:divide-info\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 1)}
.focus\:divide-info\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.2)}
.focus\:divide-info\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.25)}
.focus\:divide-info\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.3)}
.focus\:divide-info\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.4)}
.focus\:divide-info\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.05)}
.focus\:divide-info\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.5)}
.focus\:divide-info\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.6)}
.focus\:divide-info\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.7)}
.focus\:divide-info\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.75)}
.focus\:divide-info\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.8)}
.focus\:divide-info\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.9)}
.focus\:divide-info\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--in) / 0.95)}
.focus\:divide-success:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 1)}
.focus\:divide-success-content:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 1)}
.focus\:divide-success-content\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0)}
.focus\:divide-success-content\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.1)}
.focus\:divide-success-content\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 1)}
.focus\:divide-success-content\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.2)}
.focus\:divide-success-content\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.25)}
.focus\:divide-success-content\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.3)}
.focus\:divide-success-content\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.4)}
.focus\:divide-success-content\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.05)}
.focus\:divide-success-content\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.5)}
.focus\:divide-success-content\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.6)}
.focus\:divide-success-content\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.7)}
.focus\:divide-success-content\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.75)}
.focus\:divide-success-content\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.8)}
.focus\:divide-success-content\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.9)}
.focus\:divide-success-content\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--suc) / 0.95)}
.focus\:divide-success\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0)}
.focus\:divide-success\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.1)}
.focus\:divide-success\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 1)}
.focus\:divide-success\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.2)}
.focus\:divide-success\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.25)}
.focus\:divide-success\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.3)}
.focus\:divide-success\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.4)}
.focus\:divide-success\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.05)}
.focus\:divide-success\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.5)}
.focus\:divide-success\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.6)}
.focus\:divide-success\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.7)}
.focus\:divide-success\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.75)}
.focus\:divide-success\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.8)}
.focus\:divide-success\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.9)}
.focus\:divide-success\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--su) / 0.95)}
.focus\:divide-warning:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 1)}
.focus\:divide-warning-content:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 1)}
.focus\:divide-warning-content\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0)}
.focus\:divide-warning-content\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.1)}
.focus\:divide-warning-content\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 1)}
.focus\:divide-warning-content\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.2)}
.focus\:divide-warning-content\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.25)}
.focus\:divide-warning-content\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.3)}
.focus\:divide-warning-content\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.4)}
.focus\:divide-warning-content\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.05)}
.focus\:divide-warning-content\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.5)}
.focus\:divide-warning-content\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.6)}
.focus\:divide-warning-content\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.7)}
.focus\:divide-warning-content\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.75)}
.focus\:divide-warning-content\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.8)}
.focus\:divide-warning-content\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.9)}
.focus\:divide-warning-content\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wac) / 0.95)}
.focus\:divide-warning\/0:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0)}
.focus\:divide-warning\/10:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.1)}
.focus\:divide-warning\/100:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 1)}
.focus\:divide-warning\/20:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.2)}
.focus\:divide-warning\/25:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.25)}
.focus\:divide-warning\/30:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.3)}
.focus\:divide-warning\/40:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.4)}
.focus\:divide-warning\/5:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.05)}
.focus\:divide-warning\/50:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.5)}
.focus\:divide-warning\/60:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.6)}
.focus\:divide-warning\/70:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.7)}
.focus\:divide-warning\/75:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.75)}
.focus\:divide-warning\/80:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.8)}
.focus\:divide-warning\/90:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.9)}
.focus\:divide-warning\/95:focus > :not([hidden]) ~ :not([hidden]) {
    border-color: hsl(var(--wa) / 0.95)}
.focus\:border-accent:focus {
    border-color: hsl(var(--a) / 1)}
.focus\:border-accent-content:focus {
    border-color: hsl(var(--ac) / 1)}
.focus\:border-accent-content\/0:focus {
    border-color: hsl(var(--ac) / 0)}
.focus\:border-accent-content\/10:focus {
    border-color: hsl(var(--ac) / 0.1)}
.focus\:border-accent-content\/100:focus {
    border-color: hsl(var(--ac) / 1)}
.focus\:border-accent-content\/20:focus {
    border-color: hsl(var(--ac) / 0.2)}
.focus\:border-accent-content\/25:focus {
    border-color: hsl(var(--ac) / 0.25)}
.focus\:border-accent-content\/30:focus {
    border-color: hsl(var(--ac) / 0.3)}
.focus\:border-accent-content\/40:focus {
    border-color: hsl(var(--ac) / 0.4)}
.focus\:border-accent-content\/5:focus {
    border-color: hsl(var(--ac) / 0.05)}
.focus\:border-accent-content\/50:focus {
    border-color: hsl(var(--ac) / 0.5)}
.focus\:border-accent-content\/60:focus {
    border-color: hsl(var(--ac) / 0.6)}
.focus\:border-accent-content\/70:focus {
    border-color: hsl(var(--ac) / 0.7)}
.focus\:border-accent-content\/75:focus {
    border-color: hsl(var(--ac) / 0.75)}
.focus\:border-accent-content\/80:focus {
    border-color: hsl(var(--ac) / 0.8)}
.focus\:border-accent-content\/90:focus {
    border-color: hsl(var(--ac) / 0.9)}
.focus\:border-accent-content\/95:focus {
    border-color: hsl(var(--ac) / 0.95)}
.focus\:border-accent-focus:focus {
    border-color: hsl(var(--af) / 1)}
.focus\:border-accent-focus\/0:focus {
    border-color: hsl(var(--af) / 0)}
.focus\:border-accent-focus\/10:focus {
    border-color: hsl(var(--af) / 0.1)}
.focus\:border-accent-focus\/100:focus {
    border-color: hsl(var(--af) / 1)}
.focus\:border-accent-focus\/20:focus {
    border-color: hsl(var(--af) / 0.2)}
.focus\:border-accent-focus\/25:focus {
    border-color: hsl(var(--af) / 0.25)}
.focus\:border-accent-focus\/30:focus {
    border-color: hsl(var(--af) / 0.3)}
.focus\:border-accent-focus\/40:focus {
    border-color: hsl(var(--af) / 0.4)}
.focus\:border-accent-focus\/5:focus {
    border-color: hsl(var(--af) / 0.05)}
.focus\:border-accent-focus\/50:focus {
    border-color: hsl(var(--af) / 0.5)}
.focus\:border-accent-focus\/60:focus {
    border-color: hsl(var(--af) / 0.6)}
.focus\:border-accent-focus\/70:focus {
    border-color: hsl(var(--af) / 0.7)}
.focus\:border-accent-focus\/75:focus {
    border-color: hsl(var(--af) / 0.75)}
.focus\:border-accent-focus\/80:focus {
    border-color: hsl(var(--af) / 0.8)}
.focus\:border-accent-focus\/90:focus {
    border-color: hsl(var(--af) / 0.9)}
.focus\:border-accent-focus\/95:focus {
    border-color: hsl(var(--af) / 0.95)}
.focus\:border-accent\/0:focus {
    border-color: hsl(var(--a) / 0)}
.focus\:border-accent\/10:focus {
    border-color: hsl(var(--a) / 0.1)}
.focus\:border-accent\/100:focus {
    border-color: hsl(var(--a) / 1)}
.focus\:border-accent\/20:focus {
    border-color: hsl(var(--a) / 0.2)}
.focus\:border-accent\/25:focus {
    border-color: hsl(var(--a) / 0.25)}
.focus\:border-accent\/30:focus {
    border-color: hsl(var(--a) / 0.3)}
.focus\:border-accent\/40:focus {
    border-color: hsl(var(--a) / 0.4)}
.focus\:border-accent\/5:focus {
    border-color: hsl(var(--a) / 0.05)}
.focus\:border-accent\/50:focus {
    border-color: hsl(var(--a) / 0.5)}
.focus\:border-accent\/60:focus {
    border-color: hsl(var(--a) / 0.6)}
.focus\:border-accent\/70:focus {
    border-color: hsl(var(--a) / 0.7)}
.focus\:border-accent\/75:focus {
    border-color: hsl(var(--a) / 0.75)}
.focus\:border-accent\/80:focus {
    border-color: hsl(var(--a) / 0.8)}
.focus\:border-accent\/90:focus {
    border-color: hsl(var(--a) / 0.9)}
.focus\:border-accent\/95:focus {
    border-color: hsl(var(--a) / 0.95)}
.focus\:border-base-100:focus {
    border-color: hsl(var(--b1) / 1)}
.focus\:border-base-100\/0:focus {
    border-color: hsl(var(--b1) / 0)}
.focus\:border-base-100\/10:focus {
    border-color: hsl(var(--b1) / 0.1)}
.focus\:border-base-100\/100:focus {
    border-color: hsl(var(--b1) / 1)}
.focus\:border-base-100\/20:focus {
    border-color: hsl(var(--b1) / 0.2)}
.focus\:border-base-100\/25:focus {
    border-color: hsl(var(--b1) / 0.25)}
.focus\:border-base-100\/30:focus {
    border-color: hsl(var(--b1) / 0.3)}
.focus\:border-base-100\/40:focus {
    border-color: hsl(var(--b1) / 0.4)}
.focus\:border-base-100\/5:focus {
    border-color: hsl(var(--b1) / 0.05)}
.focus\:border-base-100\/50:focus {
    border-color: hsl(var(--b1) / 0.5)}
.focus\:border-base-100\/60:focus {
    border-color: hsl(var(--b1) / 0.6)}
.focus\:border-base-100\/70:focus {
    border-color: hsl(var(--b1) / 0.7)}
.focus\:border-base-100\/75:focus {
    border-color: hsl(var(--b1) / 0.75)}
.focus\:border-base-100\/80:focus {
    border-color: hsl(var(--b1) / 0.8)}
.focus\:border-base-100\/90:focus {
    border-color: hsl(var(--b1) / 0.9)}
.focus\:border-base-100\/95:focus {
    border-color: hsl(var(--b1) / 0.95)}
.focus\:border-base-200:focus {
    border-color: hsl(var(--b2) / 1)}
.focus\:border-base-200\/0:focus {
    border-color: hsl(var(--b2) / 0)}
.focus\:border-base-200\/10:focus {
    border-color: hsl(var(--b2) / 0.1)}
.focus\:border-base-200\/100:focus {
    border-color: hsl(var(--b2) / 1)}
.focus\:border-base-200\/20:focus {
    border-color: hsl(var(--b2) / 0.2)}
.focus\:border-base-200\/25:focus {
    border-color: hsl(var(--b2) / 0.25)}
.focus\:border-base-200\/30:focus {
    border-color: hsl(var(--b2) / 0.3)}
.focus\:border-base-200\/40:focus {
    border-color: hsl(var(--b2) / 0.4)}
.focus\:border-base-200\/5:focus {
    border-color: hsl(var(--b2) / 0.05)}
.focus\:border-base-200\/50:focus {
    border-color: hsl(var(--b2) / 0.5)}
.focus\:border-base-200\/60:focus {
    border-color: hsl(var(--b2) / 0.6)}
.focus\:border-base-200\/70:focus {
    border-color: hsl(var(--b2) / 0.7)}
.focus\:border-base-200\/75:focus {
    border-color: hsl(var(--b2) / 0.75)}
.focus\:border-base-200\/80:focus {
    border-color: hsl(var(--b2) / 0.8)}
.focus\:border-base-200\/90:focus {
    border-color: hsl(var(--b2) / 0.9)}
.focus\:border-base-200\/95:focus {
    border-color: hsl(var(--b2) / 0.95)}
.focus\:border-base-300:focus {
    border-color: hsl(var(--b3) / 1)}
.focus\:border-base-300\/0:focus {
    border-color: hsl(var(--b3) / 0)}
.focus\:border-base-300\/10:focus {
    border-color: hsl(var(--b3) / 0.1)}
.focus\:border-base-300\/100:focus {
    border-color: hsl(var(--b3) / 1)}
.focus\:border-base-300\/20:focus {
    border-color: hsl(var(--b3) / 0.2)}
.focus\:border-base-300\/25:focus {
    border-color: hsl(var(--b3) / 0.25)}
.focus\:border-base-300\/30:focus {
    border-color: hsl(var(--b3) / 0.3)}
.focus\:border-base-300\/40:focus {
    border-color: hsl(var(--b3) / 0.4)}
.focus\:border-base-300\/5:focus {
    border-color: hsl(var(--b3) / 0.05)}
.focus\:border-base-300\/50:focus {
    border-color: hsl(var(--b3) / 0.5)}
.focus\:border-base-300\/60:focus {
    border-color: hsl(var(--b3) / 0.6)}
.focus\:border-base-300\/70:focus {
    border-color: hsl(var(--b3) / 0.7)}
.focus\:border-base-300\/75:focus {
    border-color: hsl(var(--b3) / 0.75)}
.focus\:border-base-300\/80:focus {
    border-color: hsl(var(--b3) / 0.8)}
.focus\:border-base-300\/90:focus {
    border-color: hsl(var(--b3) / 0.9)}
.focus\:border-base-300\/95:focus {
    border-color: hsl(var(--b3) / 0.95)}
.focus\:border-base-content:focus {
    border-color: hsl(var(--bc) / 1)}
.focus\:border-base-content\/0:focus {
    border-color: hsl(var(--bc) / 0)}
.focus\:border-base-content\/10:focus {
    border-color: hsl(var(--bc) / 0.1)}
.focus\:border-base-content\/100:focus {
    border-color: hsl(var(--bc) / 1)}
.focus\:border-base-content\/20:focus {
    border-color: hsl(var(--bc) / 0.2)}
.focus\:border-base-content\/25:focus {
    border-color: hsl(var(--bc) / 0.25)}
.focus\:border-base-content\/30:focus {
    border-color: hsl(var(--bc) / 0.3)}
.focus\:border-base-content\/40:focus {
    border-color: hsl(var(--bc) / 0.4)}
.focus\:border-base-content\/5:focus {
    border-color: hsl(var(--bc) / 0.05)}
.focus\:border-base-content\/50:focus {
    border-color: hsl(var(--bc) / 0.5)}
.focus\:border-base-content\/60:focus {
    border-color: hsl(var(--bc) / 0.6)}
.focus\:border-base-content\/70:focus {
    border-color: hsl(var(--bc) / 0.7)}
.focus\:border-base-content\/75:focus {
    border-color: hsl(var(--bc) / 0.75)}
.focus\:border-base-content\/80:focus {
    border-color: hsl(var(--bc) / 0.8)}
.focus\:border-base-content\/90:focus {
    border-color: hsl(var(--bc) / 0.9)}
.focus\:border-base-content\/95:focus {
    border-color: hsl(var(--bc) / 0.95)}
.focus\:border-error:focus {
    border-color: hsl(var(--er) / 1)}
.focus\:border-error-content:focus {
    border-color: hsl(var(--erc) / 1)}
.focus\:border-error-content\/0:focus {
    border-color: hsl(var(--erc) / 0)}
.focus\:border-error-content\/10:focus {
    border-color: hsl(var(--erc) / 0.1)}
.focus\:border-error-content\/100:focus {
    border-color: hsl(var(--erc) / 1)}
.focus\:border-error-content\/20:focus {
    border-color: hsl(var(--erc) / 0.2)}
.focus\:border-error-content\/25:focus {
    border-color: hsl(var(--erc) / 0.25)}
.focus\:border-error-content\/30:focus {
    border-color: hsl(var(--erc) / 0.3)}
.focus\:border-error-content\/40:focus {
    border-color: hsl(var(--erc) / 0.4)}
.focus\:border-error-content\/5:focus {
    border-color: hsl(var(--erc) / 0.05)}
.focus\:border-error-content\/50:focus {
    border-color: hsl(var(--erc) / 0.5)}
.focus\:border-error-content\/60:focus {
    border-color: hsl(var(--erc) / 0.6)}
.focus\:border-error-content\/70:focus {
    border-color: hsl(var(--erc) / 0.7)}
.focus\:border-error-content\/75:focus {
    border-color: hsl(var(--erc) / 0.75)}
.focus\:border-error-content\/80:focus {
    border-color: hsl(var(--erc) / 0.8)}
.focus\:border-error-content\/90:focus {
    border-color: hsl(var(--erc) / 0.9)}
.focus\:border-error-content\/95:focus {
    border-color: hsl(var(--erc) / 0.95)}
.focus\:border-error\/0:focus {
    border-color: hsl(var(--er) / 0)}
.focus\:border-error\/10:focus {
    border-color: hsl(var(--er) / 0.1)}
.focus\:border-error\/100:focus {
    border-color: hsl(var(--er) / 1)}
.focus\:border-error\/20:focus {
    border-color: hsl(var(--er) / 0.2)}
.focus\:border-error\/25:focus {
    border-color: hsl(var(--er) / 0.25)}
.focus\:border-error\/30:focus {
    border-color: hsl(var(--er) / 0.3)}
.focus\:border-error\/40:focus {
    border-color: hsl(var(--er) / 0.4)}
.focus\:border-error\/5:focus {
    border-color: hsl(var(--er) / 0.05)}
.focus\:border-error\/50:focus {
    border-color: hsl(var(--er) / 0.5)}
.focus\:border-error\/60:focus {
    border-color: hsl(var(--er) / 0.6)}
.focus\:border-error\/70:focus {
    border-color: hsl(var(--er) / 0.7)}
.focus\:border-error\/75:focus {
    border-color: hsl(var(--er) / 0.75)}
.focus\:border-error\/80:focus {
    border-color: hsl(var(--er) / 0.8)}
.focus\:border-error\/90:focus {
    border-color: hsl(var(--er) / 0.9)}
.focus\:border-error\/95:focus {
    border-color: hsl(var(--er) / 0.95)}
.focus\:border-info:focus {
    border-color: hsl(var(--in) / 1)}
.focus\:border-info-content:focus {
    border-color: hsl(var(--inc) / 1)}
.focus\:border-info-content\/0:focus {
    border-color: hsl(var(--inc) / 0)}
.focus\:border-info-content\/10:focus {
    border-color: hsl(var(--inc) / 0.1)}
.focus\:border-info-content\/100:focus {
    border-color: hsl(var(--inc) / 1)}
.focus\:border-info-content\/20:focus {
    border-color: hsl(var(--inc) / 0.2)}
.focus\:border-info-content\/25:focus {
    border-color: hsl(var(--inc) / 0.25)}
.focus\:border-info-content\/30:focus {
    border-color: hsl(var(--inc) / 0.3)}
.focus\:border-info-content\/40:focus {
    border-color: hsl(var(--inc) / 0.4)}
.focus\:border-info-content\/5:focus {
    border-color: hsl(var(--inc) / 0.05)}
.focus\:border-info-content\/50:focus {
    border-color: hsl(var(--inc) / 0.5)}
.focus\:border-info-content\/60:focus {
    border-color: hsl(var(--inc) / 0.6)}
.focus\:border-info-content\/70:focus {
    border-color: hsl(var(--inc) / 0.7)}
.focus\:border-info-content\/75:focus {
    border-color: hsl(var(--inc) / 0.75)}
.focus\:border-info-content\/80:focus {
    border-color: hsl(var(--inc) / 0.8)}
.focus\:border-info-content\/90:focus {
    border-color: hsl(var(--inc) / 0.9)}
.focus\:border-info-content\/95:focus {
    border-color: hsl(var(--inc) / 0.95)}
.focus\:border-info\/0:focus {
    border-color: hsl(var(--in) / 0)}
.focus\:border-info\/10:focus {
    border-color: hsl(var(--in) / 0.1)}
.focus\:border-info\/100:focus {
    border-color: hsl(var(--in) / 1)}
.focus\:border-info\/20:focus {
    border-color: hsl(var(--in) / 0.2)}
.focus\:border-info\/25:focus {
    border-color: hsl(var(--in) / 0.25)}
.focus\:border-info\/30:focus {
    border-color: hsl(var(--in) / 0.3)}
.focus\:border-info\/40:focus {
    border-color: hsl(var(--in) / 0.4)}
.focus\:border-info\/5:focus {
    border-color: hsl(var(--in) / 0.05)}
.focus\:border-info\/50:focus {
    border-color: hsl(var(--in) / 0.5)}
.focus\:border-info\/60:focus {
    border-color: hsl(var(--in) / 0.6)}
.focus\:border-info\/70:focus {
    border-color: hsl(var(--in) / 0.7)}
.focus\:border-info\/75:focus {
    border-color: hsl(var(--in) / 0.75)}
.focus\:border-info\/80:focus {
    border-color: hsl(var(--in) / 0.8)}
.focus\:border-info\/90:focus {
    border-color: hsl(var(--in) / 0.9)}
.focus\:border-info\/95:focus {
    border-color: hsl(var(--in) / 0.95)}
.focus\:border-neutral:focus {
    border-color: hsl(var(--n) / 1)}
.focus\:border-neutral-content:focus {
    border-color: hsl(var(--nc) / 1)}
.focus\:border-neutral-content\/0:focus {
    border-color: hsl(var(--nc) / 0)}
.focus\:border-neutral-content\/10:focus {
    border-color: hsl(var(--nc) / 0.1)}
.focus\:border-neutral-content\/100:focus {
    border-color: hsl(var(--nc) / 1)}
.focus\:border-neutral-content\/20:focus {
    border-color: hsl(var(--nc) / 0.2)}
.focus\:border-neutral-content\/25:focus {
    border-color: hsl(var(--nc) / 0.25)}
.focus\:border-neutral-content\/30:focus {
    border-color: hsl(var(--nc) / 0.3)}
.focus\:border-neutral-content\/40:focus {
    border-color: hsl(var(--nc) / 0.4)}
.focus\:border-neutral-content\/5:focus {
    border-color: hsl(var(--nc) / 0.05)}
.focus\:border-neutral-content\/50:focus {
    border-color: hsl(var(--nc) / 0.5)}
.focus\:border-neutral-content\/60:focus {
    border-color: hsl(var(--nc) / 0.6)}
.focus\:border-neutral-content\/70:focus {
    border-color: hsl(var(--nc) / 0.7)}
.focus\:border-neutral-content\/75:focus {
    border-color: hsl(var(--nc) / 0.75)}
.focus\:border-neutral-content\/80:focus {
    border-color: hsl(var(--nc) / 0.8)}
.focus\:border-neutral-content\/90:focus {
    border-color: hsl(var(--nc) / 0.9)}
.focus\:border-neutral-content\/95:focus {
    border-color: hsl(var(--nc) / 0.95)}
.focus\:border-neutral-focus:focus {
    border-color: hsl(var(--nf) / 1)}
.focus\:border-neutral-focus\/0:focus {
    border-color: hsl(var(--nf) / 0)}
.focus\:border-neutral-focus\/10:focus {
    border-color: hsl(var(--nf) / 0.1)}
.focus\:border-neutral-focus\/100:focus {
    border-color: hsl(var(--nf) / 1)}
.focus\:border-neutral-focus\/20:focus {
    border-color: hsl(var(--nf) / 0.2)}
.focus\:border-neutral-focus\/25:focus {
    border-color: hsl(var(--nf) / 0.25)}
.focus\:border-neutral-focus\/30:focus {
    border-color: hsl(var(--nf) / 0.3)}
.focus\:border-neutral-focus\/40:focus {
    border-color: hsl(var(--nf) / 0.4)}
.focus\:border-neutral-focus\/5:focus {
    border-color: hsl(var(--nf) / 0.05)}
.focus\:border-neutral-focus\/50:focus {
    border-color: hsl(var(--nf) / 0.5)}
.focus\:border-neutral-focus\/60:focus {
    border-color: hsl(var(--nf) / 0.6)}
.focus\:border-neutral-focus\/70:focus {
    border-color: hsl(var(--nf) / 0.7)}
.focus\:border-neutral-focus\/75:focus {
    border-color: hsl(var(--nf) / 0.75)}
.focus\:border-neutral-focus\/80:focus {
    border-color: hsl(var(--nf) / 0.8)}
.focus\:border-neutral-focus\/90:focus {
    border-color: hsl(var(--nf) / 0.9)}
.focus\:border-neutral-focus\/95:focus {
    border-color: hsl(var(--nf) / 0.95)}
.focus\:border-neutral\/0:focus {
    border-color: hsl(var(--n) / 0)}
.focus\:border-neutral\/10:focus {
    border-color: hsl(var(--n) / 0.1)}
.focus\:border-neutral\/100:focus {
    border-color: hsl(var(--n) / 1)}
.focus\:border-neutral\/20:focus {
    border-color: hsl(var(--n) / 0.2)}
.focus\:border-neutral\/25:focus {
    border-color: hsl(var(--n) / 0.25)}
.focus\:border-neutral\/30:focus {
    border-color: hsl(var(--n) / 0.3)}
.focus\:border-neutral\/40:focus {
    border-color: hsl(var(--n) / 0.4)}
.focus\:border-neutral\/5:focus {
    border-color: hsl(var(--n) / 0.05)}
.focus\:border-neutral\/50:focus {
    border-color: hsl(var(--n) / 0.5)}
.focus\:border-neutral\/60:focus {
    border-color: hsl(var(--n) / 0.6)}
.focus\:border-neutral\/70:focus {
    border-color: hsl(var(--n) / 0.7)}
.focus\:border-neutral\/75:focus {
    border-color: hsl(var(--n) / 0.75)}
.focus\:border-neutral\/80:focus {
    border-color: hsl(var(--n) / 0.8)}
.focus\:border-neutral\/90:focus {
    border-color: hsl(var(--n) / 0.9)}
.focus\:border-neutral\/95:focus {
    border-color: hsl(var(--n) / 0.95)}
.focus\:border-primary:focus {
    border-color: hsl(var(--p) / 1)}
.focus\:border-primary-content:focus {
    border-color: hsl(var(--pc) / 1)}
.focus\:border-primary-content\/0:focus {
    border-color: hsl(var(--pc) / 0)}
.focus\:border-primary-content\/10:focus {
    border-color: hsl(var(--pc) / 0.1)}
.focus\:border-primary-content\/100:focus {
    border-color: hsl(var(--pc) / 1)}
.focus\:border-primary-content\/20:focus {
    border-color: hsl(var(--pc) / 0.2)}
.focus\:border-primary-content\/25:focus {
    border-color: hsl(var(--pc) / 0.25)}
.focus\:border-primary-content\/30:focus {
    border-color: hsl(var(--pc) / 0.3)}
.focus\:border-primary-content\/40:focus {
    border-color: hsl(var(--pc) / 0.4)}
.focus\:border-primary-content\/5:focus {
    border-color: hsl(var(--pc) / 0.05)}
.focus\:border-primary-content\/50:focus {
    border-color: hsl(var(--pc) / 0.5)}
.focus\:border-primary-content\/60:focus {
    border-color: hsl(var(--pc) / 0.6)}
.focus\:border-primary-content\/70:focus {
    border-color: hsl(var(--pc) / 0.7)}
.focus\:border-primary-content\/75:focus {
    border-color: hsl(var(--pc) / 0.75)}
.focus\:border-primary-content\/80:focus {
    border-color: hsl(var(--pc) / 0.8)}
.focus\:border-primary-content\/90:focus {
    border-color: hsl(var(--pc) / 0.9)}
.focus\:border-primary-content\/95:focus {
    border-color: hsl(var(--pc) / 0.95)}
.focus\:border-primary-focus:focus {
    border-color: hsl(var(--pf) / 1)}
.focus\:border-primary-focus\/0:focus {
    border-color: hsl(var(--pf) / 0)}
.focus\:border-primary-focus\/10:focus {
    border-color: hsl(var(--pf) / 0.1)}
.focus\:border-primary-focus\/100:focus {
    border-color: hsl(var(--pf) / 1)}
.focus\:border-primary-focus\/20:focus {
    border-color: hsl(var(--pf) / 0.2)}
.focus\:border-primary-focus\/25:focus {
    border-color: hsl(var(--pf) / 0.25)}
.focus\:border-primary-focus\/30:focus {
    border-color: hsl(var(--pf) / 0.3)}
.focus\:border-primary-focus\/40:focus {
    border-color: hsl(var(--pf) / 0.4)}
.focus\:border-primary-focus\/5:focus {
    border-color: hsl(var(--pf) / 0.05)}
.focus\:border-primary-focus\/50:focus {
    border-color: hsl(var(--pf) / 0.5)}
.focus\:border-primary-focus\/60:focus {
    border-color: hsl(var(--pf) / 0.6)}
.focus\:border-primary-focus\/70:focus {
    border-color: hsl(var(--pf) / 0.7)}
.focus\:border-primary-focus\/75:focus {
    border-color: hsl(var(--pf) / 0.75)}
.focus\:border-primary-focus\/80:focus {
    border-color: hsl(var(--pf) / 0.8)}
.focus\:border-primary-focus\/90:focus {
    border-color: hsl(var(--pf) / 0.9)}
.focus\:border-primary-focus\/95:focus {
    border-color: hsl(var(--pf) / 0.95)}
.focus\:border-primary\/0:focus {
    border-color: hsl(var(--p) / 0)}
.focus\:border-primary\/10:focus {
    border-color: hsl(var(--p) / 0.1)}
.focus\:border-primary\/100:focus {
    border-color: hsl(var(--p) / 1)}
.focus\:border-primary\/20:focus {
    border-color: hsl(var(--p) / 0.2)}
.focus\:border-primary\/25:focus {
    border-color: hsl(var(--p) / 0.25)}
.focus\:border-primary\/30:focus {
    border-color: hsl(var(--p) / 0.3)}
.focus\:border-primary\/40:focus {
    border-color: hsl(var(--p) / 0.4)}
.focus\:border-primary\/5:focus {
    border-color: hsl(var(--p) / 0.05)}
.focus\:border-primary\/50:focus {
    border-color: hsl(var(--p) / 0.5)}
.focus\:border-primary\/60:focus {
    border-color: hsl(var(--p) / 0.6)}
.focus\:border-primary\/70:focus {
    border-color: hsl(var(--p) / 0.7)}
.focus\:border-primary\/75:focus {
    border-color: hsl(var(--p) / 0.75)}
.focus\:border-primary\/80:focus {
    border-color: hsl(var(--p) / 0.8)}
.focus\:border-primary\/90:focus {
    border-color: hsl(var(--p) / 0.9)}
.focus\:border-primary\/95:focus {
    border-color: hsl(var(--p) / 0.95)}
.focus\:border-secondary:focus {
    border-color: hsl(var(--s) / 1)}
.focus\:border-secondary-content:focus {
    border-color: hsl(var(--sc) / 1)}
.focus\:border-secondary-content\/0:focus {
    border-color: hsl(var(--sc) / 0)}
.focus\:border-secondary-content\/10:focus {
    border-color: hsl(var(--sc) / 0.1)}
.focus\:border-secondary-content\/100:focus {
    border-color: hsl(var(--sc) / 1)}
.focus\:border-secondary-content\/20:focus {
    border-color: hsl(var(--sc) / 0.2)}
.focus\:border-secondary-content\/25:focus {
    border-color: hsl(var(--sc) / 0.25)}
.focus\:border-secondary-content\/30:focus {
    border-color: hsl(var(--sc) / 0.3)}
.focus\:border-secondary-content\/40:focus {
    border-color: hsl(var(--sc) / 0.4)}
.focus\:border-secondary-content\/5:focus {
    border-color: hsl(var(--sc) / 0.05)}
.focus\:border-secondary-content\/50:focus {
    border-color: hsl(var(--sc) / 0.5)}
.focus\:border-secondary-content\/60:focus {
    border-color: hsl(var(--sc) / 0.6)}
.focus\:border-secondary-content\/70:focus {
    border-color: hsl(var(--sc) / 0.7)}
.focus\:border-secondary-content\/75:focus {
    border-color: hsl(var(--sc) / 0.75)}
.focus\:border-secondary-content\/80:focus {
    border-color: hsl(var(--sc) / 0.8)}
.focus\:border-secondary-content\/90:focus {
    border-color: hsl(var(--sc) / 0.9)}
.focus\:border-secondary-content\/95:focus {
    border-color: hsl(var(--sc) / 0.95)}
.focus\:border-secondary-focus:focus {
    border-color: hsl(var(--sf) / 1)}
.focus\:border-secondary-focus\/0:focus {
    border-color: hsl(var(--sf) / 0)}
.focus\:border-secondary-focus\/10:focus {
    border-color: hsl(var(--sf) / 0.1)}
.focus\:border-secondary-focus\/100:focus {
    border-color: hsl(var(--sf) / 1)}
.focus\:border-secondary-focus\/20:focus {
    border-color: hsl(var(--sf) / 0.2)}
.focus\:border-secondary-focus\/25:focus {
    border-color: hsl(var(--sf) / 0.25)}
.focus\:border-secondary-focus\/30:focus {
    border-color: hsl(var(--sf) / 0.3)}
.focus\:border-secondary-focus\/40:focus {
    border-color: hsl(var(--sf) / 0.4)}
.focus\:border-secondary-focus\/5:focus {
    border-color: hsl(var(--sf) / 0.05)}
.focus\:border-secondary-focus\/50:focus {
    border-color: hsl(var(--sf) / 0.5)}
.focus\:border-secondary-focus\/60:focus {
    border-color: hsl(var(--sf) / 0.6)}
.focus\:border-secondary-focus\/70:focus {
    border-color: hsl(var(--sf) / 0.7)}
.focus\:border-secondary-focus\/75:focus {
    border-color: hsl(var(--sf) / 0.75)}
.focus\:border-secondary-focus\/80:focus {
    border-color: hsl(var(--sf) / 0.8)}
.focus\:border-secondary-focus\/90:focus {
    border-color: hsl(var(--sf) / 0.9)}
.focus\:border-secondary-focus\/95:focus {
    border-color: hsl(var(--sf) / 0.95)}
.focus\:border-secondary\/0:focus {
    border-color: hsl(var(--s) / 0)}
.focus\:border-secondary\/10:focus {
    border-color: hsl(var(--s) / 0.1)}
.focus\:border-secondary\/100:focus {
    border-color: hsl(var(--s) / 1)}
.focus\:border-secondary\/20:focus {
    border-color: hsl(var(--s) / 0.2)}
.focus\:border-secondary\/25:focus {
    border-color: hsl(var(--s) / 0.25)}
.focus\:border-secondary\/30:focus {
    border-color: hsl(var(--s) / 0.3)}
.focus\:border-secondary\/40:focus {
    border-color: hsl(var(--s) / 0.4)}
.focus\:border-secondary\/5:focus {
    border-color: hsl(var(--s) / 0.05)}
.focus\:border-secondary\/50:focus {
    border-color: hsl(var(--s) / 0.5)}
.focus\:border-secondary\/60:focus {
    border-color: hsl(var(--s) / 0.6)}
.focus\:border-secondary\/70:focus {
    border-color: hsl(var(--s) / 0.7)}
.focus\:border-secondary\/75:focus {
    border-color: hsl(var(--s) / 0.75)}
.focus\:border-secondary\/80:focus {
    border-color: hsl(var(--s) / 0.8)}
.focus\:border-secondary\/90:focus {
    border-color: hsl(var(--s) / 0.9)}
.focus\:border-secondary\/95:focus {
    border-color: hsl(var(--s) / 0.95)}
.focus\:border-success:focus {
    border-color: hsl(var(--su) / 1)}
.focus\:border-success-content:focus {
    border-color: hsl(var(--suc) / 1)}
.focus\:border-success-content\/0:focus {
    border-color: hsl(var(--suc) / 0)}
.focus\:border-success-content\/10:focus {
    border-color: hsl(var(--suc) / 0.1)}
.focus\:border-success-content\/100:focus {
    border-color: hsl(var(--suc) / 1)}
.focus\:border-success-content\/20:focus {
    border-color: hsl(var(--suc) / 0.2)}
.focus\:border-success-content\/25:focus {
    border-color: hsl(var(--suc) / 0.25)}
.focus\:border-success-content\/30:focus {
    border-color: hsl(var(--suc) / 0.3)}
.focus\:border-success-content\/40:focus {
    border-color: hsl(var(--suc) / 0.4)}
.focus\:border-success-content\/5:focus {
    border-color: hsl(var(--suc) / 0.05)}
.focus\:border-success-content\/50:focus {
    border-color: hsl(var(--suc) / 0.5)}
.focus\:border-success-content\/60:focus {
    border-color: hsl(var(--suc) / 0.6)}
.focus\:border-success-content\/70:focus {
    border-color: hsl(var(--suc) / 0.7)}
.focus\:border-success-content\/75:focus {
    border-color: hsl(var(--suc) / 0.75)}
.focus\:border-success-content\/80:focus {
    border-color: hsl(var(--suc) / 0.8)}
.focus\:border-success-content\/90:focus {
    border-color: hsl(var(--suc) / 0.9)}
.focus\:border-success-content\/95:focus {
    border-color: hsl(var(--suc) / 0.95)}
.focus\:border-success\/0:focus {
    border-color: hsl(var(--su) / 0)}
.focus\:border-success\/10:focus {
    border-color: hsl(var(--su) / 0.1)}
.focus\:border-success\/100:focus {
    border-color: hsl(var(--su) / 1)}
.focus\:border-success\/20:focus {
    border-color: hsl(var(--su) / 0.2)}
.focus\:border-success\/25:focus {
    border-color: hsl(var(--su) / 0.25)}
.focus\:border-success\/30:focus {
    border-color: hsl(var(--su) / 0.3)}
.focus\:border-success\/40:focus {
    border-color: hsl(var(--su) / 0.4)}
.focus\:border-success\/5:focus {
    border-color: hsl(var(--su) / 0.05)}
.focus\:border-success\/50:focus {
    border-color: hsl(var(--su) / 0.5)}
.focus\:border-success\/60:focus {
    border-color: hsl(var(--su) / 0.6)}
.focus\:border-success\/70:focus {
    border-color: hsl(var(--su) / 0.7)}
.focus\:border-success\/75:focus {
    border-color: hsl(var(--su) / 0.75)}
.focus\:border-success\/80:focus {
    border-color: hsl(var(--su) / 0.8)}
.focus\:border-success\/90:focus {
    border-color: hsl(var(--su) / 0.9)}
.focus\:border-success\/95:focus {
    border-color: hsl(var(--su) / 0.95)}
.focus\:border-warning:focus {
    border-color: hsl(var(--wa) / 1)}
.focus\:border-warning-content:focus {
    border-color: hsl(var(--wac) / 1)}
.focus\:border-warning-content\/0:focus {
    border-color: hsl(var(--wac) / 0)}
.focus\:border-warning-content\/10:focus {
    border-color: hsl(var(--wac) / 0.1)}
.focus\:border-warning-content\/100:focus {
    border-color: hsl(var(--wac) / 1)}
.focus\:border-warning-content\/20:focus {
    border-color: hsl(var(--wac) / 0.2)}
.focus\:border-warning-content\/25:focus {
    border-color: hsl(var(--wac) / 0.25)}
.focus\:border-warning-content\/30:focus {
    border-color: hsl(var(--wac) / 0.3)}
.focus\:border-warning-content\/40:focus {
    border-color: hsl(var(--wac) / 0.4)}
.focus\:border-warning-content\/5:focus {
    border-color: hsl(var(--wac) / 0.05)}
.focus\:border-warning-content\/50:focus {
    border-color: hsl(var(--wac) / 0.5)}
.focus\:border-warning-content\/60:focus {
    border-color: hsl(var(--wac) / 0.6)}
.focus\:border-warning-content\/70:focus {
    border-color: hsl(var(--wac) / 0.7)}
.focus\:border-warning-content\/75:focus {
    border-color: hsl(var(--wac) / 0.75)}
.focus\:border-warning-content\/80:focus {
    border-color: hsl(var(--wac) / 0.8)}
.focus\:border-warning-content\/90:focus {
    border-color: hsl(var(--wac) / 0.9)}
.focus\:border-warning-content\/95:focus {
    border-color: hsl(var(--wac) / 0.95)}
.focus\:border-warning\/0:focus {
    border-color: hsl(var(--wa) / 0)}
.focus\:border-warning\/10:focus {
    border-color: hsl(var(--wa) / 0.1)}
.focus\:border-warning\/100:focus {
    border-color: hsl(var(--wa) / 1)}
.focus\:border-warning\/20:focus {
    border-color: hsl(var(--wa) / 0.2)}
.focus\:border-warning\/25:focus {
    border-color: hsl(var(--wa) / 0.25)}
.focus\:border-warning\/30:focus {
    border-color: hsl(var(--wa) / 0.3)}
.focus\:border-warning\/40:focus {
    border-color: hsl(var(--wa) / 0.4)}
.focus\:border-warning\/5:focus {
    border-color: hsl(var(--wa) / 0.05)}
.focus\:border-warning\/50:focus {
    border-color: hsl(var(--wa) / 0.5)}
.focus\:border-warning\/60:focus {
    border-color: hsl(var(--wa) / 0.6)}
.focus\:border-warning\/70:focus {
    border-color: hsl(var(--wa) / 0.7)}
.focus\:border-warning\/75:focus {
    border-color: hsl(var(--wa) / 0.75)}
.focus\:border-warning\/80:focus {
    border-color: hsl(var(--wa) / 0.8)}
.focus\:border-warning\/90:focus {
    border-color: hsl(var(--wa) / 0.9)}
.focus\:border-warning\/95:focus {
    border-color: hsl(var(--wa) / 0.95)}
.focus\:border-x-base-100:focus {
    border-left-color: hsl(var(--b1) / 1);
    border-right-color: hsl(var(--b1) / 1)}
.focus\:border-x-base-100\/0:focus {
    border-left-color: hsl(var(--b1) / 0);
    border-right-color: hsl(var(--b1) / 0)}
.focus\:border-x-base-100\/10:focus {
    border-left-color: hsl(var(--b1) / 0.1);
    border-right-color: hsl(var(--b1) / 0.1)}
.focus\:border-x-base-100\/100:focus {
    border-left-color: hsl(var(--b1) / 1);
    border-right-color: hsl(var(--b1) / 1)}
.focus\:border-x-base-100\/20:focus {
    border-left-color: hsl(var(--b1) / 0.2);
    border-right-color: hsl(var(--b1) / 0.2)}
.focus\:border-x-base-100\/25:focus {
    border-left-color: hsl(var(--b1) / 0.25);
    border-right-color: hsl(var(--b1) / 0.25)}
.focus\:border-x-base-100\/30:focus {
    border-left-color: hsl(var(--b1) / 0.3);
    border-right-color: hsl(var(--b1) / 0.3)}
.focus\:border-x-base-100\/40:focus {
    border-left-color: hsl(var(--b1) / 0.4);
    border-right-color: hsl(var(--b1) / 0.4)}
.focus\:border-x-base-100\/5:focus {
    border-left-color: hsl(var(--b1) / 0.05);
    border-right-color: hsl(var(--b1) / 0.05)}
.focus\:border-x-base-100\/50:focus {
    border-left-color: hsl(var(--b1) / 0.5);
    border-right-color: hsl(var(--b1) / 0.5)}
.focus\:border-x-base-100\/60:focus {
    border-left-color: hsl(var(--b1) / 0.6);
    border-right-color: hsl(var(--b1) / 0.6)}
.focus\:border-x-base-100\/70:focus {
    border-left-color: hsl(var(--b1) / 0.7);
    border-right-color: hsl(var(--b1) / 0.7)}
.focus\:border-x-base-100\/75:focus {
    border-left-color: hsl(var(--b1) / 0.75);
    border-right-color: hsl(var(--b1) / 0.75)}
.focus\:border-x-base-100\/80:focus {
    border-left-color: hsl(var(--b1) / 0.8);
    border-right-color: hsl(var(--b1) / 0.8)}
.focus\:border-x-base-100\/90:focus {
    border-left-color: hsl(var(--b1) / 0.9);
    border-right-color: hsl(var(--b1) / 0.9)}
.focus\:border-x-base-100\/95:focus {
    border-left-color: hsl(var(--b1) / 0.95);
    border-right-color: hsl(var(--b1) / 0.95)}
.focus\:border-x-base-200:focus {
    border-left-color: hsl(var(--b2) / 1);
    border-right-color: hsl(var(--b2) / 1)}
.focus\:border-x-base-200\/0:focus {
    border-left-color: hsl(var(--b2) / 0);
    border-right-color: hsl(var(--b2) / 0)}
.focus\:border-x-base-200\/10:focus {
    border-left-color: hsl(var(--b2) / 0.1);
    border-right-color: hsl(var(--b2) / 0.1)}
.focus\:border-x-base-200\/100:focus {
    border-left-color: hsl(var(--b2) / 1);
    border-right-color: hsl(var(--b2) / 1)}
.focus\:border-x-base-200\/20:focus {
    border-left-color: hsl(var(--b2) / 0.2);
    border-right-color: hsl(var(--b2) / 0.2)}
.focus\:border-x-base-200\/25:focus {
    border-left-color: hsl(var(--b2) / 0.25);
    border-right-color: hsl(var(--b2) / 0.25)}
.focus\:border-x-base-200\/30:focus {
    border-left-color: hsl(var(--b2) / 0.3);
    border-right-color: hsl(var(--b2) / 0.3)}
.focus\:border-x-base-200\/40:focus {
    border-left-color: hsl(var(--b2) / 0.4);
    border-right-color: hsl(var(--b2) / 0.4)}
.focus\:border-x-base-200\/5:focus {
    border-left-color: hsl(var(--b2) / 0.05);
    border-right-color: hsl(var(--b2) / 0.05)}
.focus\:border-x-base-200\/50:focus {
    border-left-color: hsl(var(--b2) / 0.5);
    border-right-color: hsl(var(--b2) / 0.5)}
.focus\:border-x-base-200\/60:focus {
    border-left-color: hsl(var(--b2) / 0.6);
    border-right-color: hsl(var(--b2) / 0.6)}
.focus\:border-x-base-200\/70:focus {
    border-left-color: hsl(var(--b2) / 0.7);
    border-right-color: hsl(var(--b2) / 0.7)}
.focus\:border-x-base-200\/75:focus {
    border-left-color: hsl(var(--b2) / 0.75);
    border-right-color: hsl(var(--b2) / 0.75)}
.focus\:border-x-base-200\/80:focus {
    border-left-color: hsl(var(--b2) / 0.8);
    border-right-color: hsl(var(--b2) / 0.8)}
.focus\:border-x-base-200\/90:focus {
    border-left-color: hsl(var(--b2) / 0.9);
    border-right-color: hsl(var(--b2) / 0.9)}
.focus\:border-x-base-200\/95:focus {
    border-left-color: hsl(var(--b2) / 0.95);
    border-right-color: hsl(var(--b2) / 0.95)}
.focus\:border-x-base-300:focus {
    border-left-color: hsl(var(--b3) / 1);
    border-right-color: hsl(var(--b3) / 1)}
.focus\:border-x-base-300\/0:focus {
    border-left-color: hsl(var(--b3) / 0);
    border-right-color: hsl(var(--b3) / 0)}
.focus\:border-x-base-300\/10:focus {
    border-left-color: hsl(var(--b3) / 0.1);
    border-right-color: hsl(var(--b3) / 0.1)}
.focus\:border-x-base-300\/100:focus {
    border-left-color: hsl(var(--b3) / 1);
    border-right-color: hsl(var(--b3) / 1)}
.focus\:border-x-base-300\/20:focus {
    border-left-color: hsl(var(--b3) / 0.2);
    border-right-color: hsl(var(--b3) / 0.2)}
.focus\:border-x-base-300\/25:focus {
    border-left-color: hsl(var(--b3) / 0.25);
    border-right-color: hsl(var(--b3) / 0.25)}
.focus\:border-x-base-300\/30:focus {
    border-left-color: hsl(var(--b3) / 0.3);
    border-right-color: hsl(var(--b3) / 0.3)}
.focus\:border-x-base-300\/40:focus {
    border-left-color: hsl(var(--b3) / 0.4);
    border-right-color: hsl(var(--b3) / 0.4)}
.focus\:border-x-base-300\/5:focus {
    border-left-color: hsl(var(--b3) / 0.05);
    border-right-color: hsl(var(--b3) / 0.05)}
.focus\:border-x-base-300\/50:focus {
    border-left-color: hsl(var(--b3) / 0.5);
    border-right-color: hsl(var(--b3) / 0.5)}
.focus\:border-x-base-300\/60:focus {
    border-left-color: hsl(var(--b3) / 0.6);
    border-right-color: hsl(var(--b3) / 0.6)}
.focus\:border-x-base-300\/70:focus {
    border-left-color: hsl(var(--b3) / 0.7);
    border-right-color: hsl(var(--b3) / 0.7)}
.focus\:border-x-base-300\/75:focus {
    border-left-color: hsl(var(--b3) / 0.75);
    border-right-color: hsl(var(--b3) / 0.75)}
.focus\:border-x-base-300\/80:focus {
    border-left-color: hsl(var(--b3) / 0.8);
    border-right-color: hsl(var(--b3) / 0.8)}
.focus\:border-x-base-300\/90:focus {
    border-left-color: hsl(var(--b3) / 0.9);
    border-right-color: hsl(var(--b3) / 0.9)}
.focus\:border-x-base-300\/95:focus {
    border-left-color: hsl(var(--b3) / 0.95);
    border-right-color: hsl(var(--b3) / 0.95)}
.focus\:border-x-base-content:focus {
    border-left-color: hsl(var(--bc) / 1);
    border-right-color: hsl(var(--bc) / 1)}
.focus\:border-x-base-content\/0:focus {
    border-left-color: hsl(var(--bc) / 0);
    border-right-color: hsl(var(--bc) / 0)}
.focus\:border-x-base-content\/10:focus {
    border-left-color: hsl(var(--bc) / 0.1);
    border-right-color: hsl(var(--bc) / 0.1)}
.focus\:border-x-base-content\/100:focus {
    border-left-color: hsl(var(--bc) / 1);
    border-right-color: hsl(var(--bc) / 1)}
.focus\:border-x-base-content\/20:focus {
    border-left-color: hsl(var(--bc) / 0.2);
    border-right-color: hsl(var(--bc) / 0.2)}
.focus\:border-x-base-content\/25:focus {
    border-left-color: hsl(var(--bc) / 0.25);
    border-right-color: hsl(var(--bc) / 0.25)}
.focus\:border-x-base-content\/30:focus {
    border-left-color: hsl(var(--bc) / 0.3);
    border-right-color: hsl(var(--bc) / 0.3)}
.focus\:border-x-base-content\/40:focus {
    border-left-color: hsl(var(--bc) / 0.4);
    border-right-color: hsl(var(--bc) / 0.4)}
.focus\:border-x-base-content\/5:focus {
    border-left-color: hsl(var(--bc) / 0.05);
    border-right-color: hsl(var(--bc) / 0.05)}
.focus\:border-x-base-content\/50:focus {
    border-left-color: hsl(var(--bc) / 0.5);
    border-right-color: hsl(var(--bc) / 0.5)}
.focus\:border-x-base-content\/60:focus {
    border-left-color: hsl(var(--bc) / 0.6);
    border-right-color: hsl(var(--bc) / 0.6)}
.focus\:border-x-base-content\/70:focus {
    border-left-color: hsl(var(--bc) / 0.7);
    border-right-color: hsl(var(--bc) / 0.7)}
.focus\:border-x-base-content\/75:focus {
    border-left-color: hsl(var(--bc) / 0.75);
    border-right-color: hsl(var(--bc) / 0.75)}
.focus\:border-x-base-content\/80:focus {
    border-left-color: hsl(var(--bc) / 0.8);
    border-right-color: hsl(var(--bc) / 0.8)}
.focus\:border-x-base-content\/90:focus {
    border-left-color: hsl(var(--bc) / 0.9);
    border-right-color: hsl(var(--bc) / 0.9)}
.focus\:border-x-base-content\/95:focus {
    border-left-color: hsl(var(--bc) / 0.95);
    border-right-color: hsl(var(--bc) / 0.95)}
.focus\:border-x-error:focus {
    border-left-color: hsl(var(--er) / 1);
    border-right-color: hsl(var(--er) / 1)}
.focus\:border-x-error-content:focus {
    border-left-color: hsl(var(--erc) / 1);
    border-right-color: hsl(var(--erc) / 1)}
.focus\:border-x-error-content\/0:focus {
    border-left-color: hsl(var(--erc) / 0);
    border-right-color: hsl(var(--erc) / 0)}
.focus\:border-x-error-content\/10:focus {
    border-left-color: hsl(var(--erc) / 0.1);
    border-right-color: hsl(var(--erc) / 0.1)}
.focus\:border-x-error-content\/100:focus {
    border-left-color: hsl(var(--erc) / 1);
    border-right-color: hsl(var(--erc) / 1)}
.focus\:border-x-error-content\/20:focus {
    border-left-color: hsl(var(--erc) / 0.2);
    border-right-color: hsl(var(--erc) / 0.2)}
.focus\:border-x-error-content\/25:focus {
    border-left-color: hsl(var(--erc) / 0.25);
    border-right-color: hsl(var(--erc) / 0.25)}
.focus\:border-x-error-content\/30:focus {
    border-left-color: hsl(var(--erc) / 0.3);
    border-right-color: hsl(var(--erc) / 0.3)}
.focus\:border-x-error-content\/40:focus {
    border-left-color: hsl(var(--erc) / 0.4);
    border-right-color: hsl(var(--erc) / 0.4)}
.focus\:border-x-error-content\/5:focus {
    border-left-color: hsl(var(--erc) / 0.05);
    border-right-color: hsl(var(--erc) / 0.05)}
.focus\:border-x-error-content\/50:focus {
    border-left-color: hsl(var(--erc) / 0.5);
    border-right-color: hsl(var(--erc) / 0.5)}
.focus\:border-x-error-content\/60:focus {
    border-left-color: hsl(var(--erc) / 0.6);
    border-right-color: hsl(var(--erc) / 0.6)}
.focus\:border-x-error-content\/70:focus {
    border-left-color: hsl(var(--erc) / 0.7);
    border-right-color: hsl(var(--erc) / 0.7)}
.focus\:border-x-error-content\/75:focus {
    border-left-color: hsl(var(--erc) / 0.75);
    border-right-color: hsl(var(--erc) / 0.75)}
.focus\:border-x-error-content\/80:focus {
    border-left-color: hsl(var(--erc) / 0.8);
    border-right-color: hsl(var(--erc) / 0.8)}
.focus\:border-x-error-content\/90:focus {
    border-left-color: hsl(var(--erc) / 0.9);
    border-right-color: hsl(var(--erc) / 0.9)}
.focus\:border-x-error-content\/95:focus {
    border-left-color: hsl(var(--erc) / 0.95);
    border-right-color: hsl(var(--erc) / 0.95)}
.focus\:border-x-error\/0:focus {
    border-left-color: hsl(var(--er) / 0);
    border-right-color: hsl(var(--er) / 0)}
.focus\:border-x-error\/10:focus {
    border-left-color: hsl(var(--er) / 0.1);
    border-right-color: hsl(var(--er) / 0.1)}
.focus\:border-x-error\/100:focus {
    border-left-color: hsl(var(--er) / 1);
    border-right-color: hsl(var(--er) / 1)}
.focus\:border-x-error\/20:focus {
    border-left-color: hsl(var(--er) / 0.2);
    border-right-color: hsl(var(--er) / 0.2)}
.focus\:border-x-error\/25:focus {
    border-left-color: hsl(var(--er) / 0.25);
    border-right-color: hsl(var(--er) / 0.25)}
.focus\:border-x-error\/30:focus {
    border-left-color: hsl(var(--er) / 0.3);
    border-right-color: hsl(var(--er) / 0.3)}
.focus\:border-x-error\/40:focus {
    border-left-color: hsl(var(--er) / 0.4);
    border-right-color: hsl(var(--er) / 0.4)}
.focus\:border-x-error\/5:focus {
    border-left-color: hsl(var(--er) / 0.05);
    border-right-color: hsl(var(--er) / 0.05)}
.focus\:border-x-error\/50:focus {
    border-left-color: hsl(var(--er) / 0.5);
    border-right-color: hsl(var(--er) / 0.5)}
.focus\:border-x-error\/60:focus {
    border-left-color: hsl(var(--er) / 0.6);
    border-right-color: hsl(var(--er) / 0.6)}
.focus\:border-x-error\/70:focus {
    border-left-color: hsl(var(--er) / 0.7);
    border-right-color: hsl(var(--er) / 0.7)}
.focus\:border-x-error\/75:focus {
    border-left-color: hsl(var(--er) / 0.75);
    border-right-color: hsl(var(--er) / 0.75)}
.focus\:border-x-error\/80:focus {
    border-left-color: hsl(var(--er) / 0.8);
    border-right-color: hsl(var(--er) / 0.8)}
.focus\:border-x-error\/90:focus {
    border-left-color: hsl(var(--er) / 0.9);
    border-right-color: hsl(var(--er) / 0.9)}
.focus\:border-x-error\/95:focus {
    border-left-color: hsl(var(--er) / 0.95);
    border-right-color: hsl(var(--er) / 0.95)}
.focus\:border-x-info:focus {
    border-left-color: hsl(var(--in) / 1);
    border-right-color: hsl(var(--in) / 1)}
.focus\:border-x-info-content:focus {
    border-left-color: hsl(var(--inc) / 1);
    border-right-color: hsl(var(--inc) / 1)}
.focus\:border-x-info-content\/0:focus {
    border-left-color: hsl(var(--inc) / 0);
    border-right-color: hsl(var(--inc) / 0)}
.focus\:border-x-info-content\/10:focus {
    border-left-color: hsl(var(--inc) / 0.1);
    border-right-color: hsl(var(--inc) / 0.1)}
.focus\:border-x-info-content\/100:focus {
    border-left-color: hsl(var(--inc) / 1);
    border-right-color: hsl(var(--inc) / 1)}
.focus\:border-x-info-content\/20:focus {
    border-left-color: hsl(var(--inc) / 0.2);
    border-right-color: hsl(var(--inc) / 0.2)}
.focus\:border-x-info-content\/25:focus {
    border-left-color: hsl(var(--inc) / 0.25);
    border-right-color: hsl(var(--inc) / 0.25)}
.focus\:border-x-info-content\/30:focus {
    border-left-color: hsl(var(--inc) / 0.3);
    border-right-color: hsl(var(--inc) / 0.3)}
.focus\:border-x-info-content\/40:focus {
    border-left-color: hsl(var(--inc) / 0.4);
    border-right-color: hsl(var(--inc) / 0.4)}
.focus\:border-x-info-content\/5:focus {
    border-left-color: hsl(var(--inc) / 0.05);
    border-right-color: hsl(var(--inc) / 0.05)}
.focus\:border-x-info-content\/50:focus {
    border-left-color: hsl(var(--inc) / 0.5);
    border-right-color: hsl(var(--inc) / 0.5)}
.focus\:border-x-info-content\/60:focus {
    border-left-color: hsl(var(--inc) / 0.6);
    border-right-color: hsl(var(--inc) / 0.6)}
.focus\:border-x-info-content\/70:focus {
    border-left-color: hsl(var(--inc) / 0.7);
    border-right-color: hsl(var(--inc) / 0.7)}
.focus\:border-x-info-content\/75:focus {
    border-left-color: hsl(var(--inc) / 0.75);
    border-right-color: hsl(var(--inc) / 0.75)}
.focus\:border-x-info-content\/80:focus {
    border-left-color: hsl(var(--inc) / 0.8);
    border-right-color: hsl(var(--inc) / 0.8)}
.focus\:border-x-info-content\/90:focus {
    border-left-color: hsl(var(--inc) / 0.9);
    border-right-color: hsl(var(--inc) / 0.9)}
.focus\:border-x-info-content\/95:focus {
    border-left-color: hsl(var(--inc) / 0.95);
    border-right-color: hsl(var(--inc) / 0.95)}
.focus\:border-x-info\/0:focus {
    border-left-color: hsl(var(--in) / 0);
    border-right-color: hsl(var(--in) / 0)}
.focus\:border-x-info\/10:focus {
    border-left-color: hsl(var(--in) / 0.1);
    border-right-color: hsl(var(--in) / 0.1)}
.focus\:border-x-info\/100:focus {
    border-left-color: hsl(var(--in) / 1);
    border-right-color: hsl(var(--in) / 1)}
.focus\:border-x-info\/20:focus {
    border-left-color: hsl(var(--in) / 0.2);
    border-right-color: hsl(var(--in) / 0.2)}
.focus\:border-x-info\/25:focus {
    border-left-color: hsl(var(--in) / 0.25);
    border-right-color: hsl(var(--in) / 0.25)}
.focus\:border-x-info\/30:focus {
    border-left-color: hsl(var(--in) / 0.3);
    border-right-color: hsl(var(--in) / 0.3)}
.focus\:border-x-info\/40:focus {
    border-left-color: hsl(var(--in) / 0.4);
    border-right-color: hsl(var(--in) / 0.4)}
.focus\:border-x-info\/5:focus {
    border-left-color: hsl(var(--in) / 0.05);
    border-right-color: hsl(var(--in) / 0.05)}
.focus\:border-x-info\/50:focus {
    border-left-color: hsl(var(--in) / 0.5);
    border-right-color: hsl(var(--in) / 0.5)}
.focus\:border-x-info\/60:focus {
    border-left-color: hsl(var(--in) / 0.6);
    border-right-color: hsl(var(--in) / 0.6)}
.focus\:border-x-info\/70:focus {
    border-left-color: hsl(var(--in) / 0.7);
    border-right-color: hsl(var(--in) / 0.7)}
.focus\:border-x-info\/75:focus {
    border-left-color: hsl(var(--in) / 0.75);
    border-right-color: hsl(var(--in) / 0.75)}
.focus\:border-x-info\/80:focus {
    border-left-color: hsl(var(--in) / 0.8);
    border-right-color: hsl(var(--in) / 0.8)}
.focus\:border-x-info\/90:focus {
    border-left-color: hsl(var(--in) / 0.9);
    border-right-color: hsl(var(--in) / 0.9)}
.focus\:border-x-info\/95:focus {
    border-left-color: hsl(var(--in) / 0.95);
    border-right-color: hsl(var(--in) / 0.95)}
.focus\:border-x-success:focus {
    border-left-color: hsl(var(--su) / 1);
    border-right-color: hsl(var(--su) / 1)}
.focus\:border-x-success-content:focus {
    border-left-color: hsl(var(--suc) / 1);
    border-right-color: hsl(var(--suc) / 1)}
.focus\:border-x-success-content\/0:focus {
    border-left-color: hsl(var(--suc) / 0);
    border-right-color: hsl(var(--suc) / 0)}
.focus\:border-x-success-content\/10:focus {
    border-left-color: hsl(var(--suc) / 0.1);
    border-right-color: hsl(var(--suc) / 0.1)}
.focus\:border-x-success-content\/100:focus {
    border-left-color: hsl(var(--suc) / 1);
    border-right-color: hsl(var(--suc) / 1)}
.focus\:border-x-success-content\/20:focus {
    border-left-color: hsl(var(--suc) / 0.2);
    border-right-color: hsl(var(--suc) / 0.2)}
.focus\:border-x-success-content\/25:focus {
    border-left-color: hsl(var(--suc) / 0.25);
    border-right-color: hsl(var(--suc) / 0.25)}
.focus\:border-x-success-content\/30:focus {
    border-left-color: hsl(var(--suc) / 0.3);
    border-right-color: hsl(var(--suc) / 0.3)}
.focus\:border-x-success-content\/40:focus {
    border-left-color: hsl(var(--suc) / 0.4);
    border-right-color: hsl(var(--suc) / 0.4)}
.focus\:border-x-success-content\/5:focus {
    border-left-color: hsl(var(--suc) / 0.05);
    border-right-color: hsl(var(--suc) / 0.05)}
.focus\:border-x-success-content\/50:focus {
    border-left-color: hsl(var(--suc) / 0.5);
    border-right-color: hsl(var(--suc) / 0.5)}
.focus\:border-x-success-content\/60:focus {
    border-left-color: hsl(var(--suc) / 0.6);
    border-right-color: hsl(var(--suc) / 0.6)}
.focus\:border-x-success-content\/70:focus {
    border-left-color: hsl(var(--suc) / 0.7);
    border-right-color: hsl(var(--suc) / 0.7)}
.focus\:border-x-success-content\/75:focus {
    border-left-color: hsl(var(--suc) / 0.75);
    border-right-color: hsl(var(--suc) / 0.75)}
.focus\:border-x-success-content\/80:focus {
    border-left-color: hsl(var(--suc) / 0.8);
    border-right-color: hsl(var(--suc) / 0.8)}
.focus\:border-x-success-content\/90:focus {
    border-left-color: hsl(var(--suc) / 0.9);
    border-right-color: hsl(var(--suc) / 0.9)}
.focus\:border-x-success-content\/95:focus {
    border-left-color: hsl(var(--suc) / 0.95);
    border-right-color: hsl(var(--suc) / 0.95)}
.focus\:border-x-success\/0:focus {
    border-left-color: hsl(var(--su) / 0);
    border-right-color: hsl(var(--su) / 0)}
.focus\:border-x-success\/10:focus {
    border-left-color: hsl(var(--su) / 0.1);
    border-right-color: hsl(var(--su) / 0.1)}
.focus\:border-x-success\/100:focus {
    border-left-color: hsl(var(--su) / 1);
    border-right-color: hsl(var(--su) / 1)}
.focus\:border-x-success\/20:focus {
    border-left-color: hsl(var(--su) / 0.2);
    border-right-color: hsl(var(--su) / 0.2)}
.focus\:border-x-success\/25:focus {
    border-left-color: hsl(var(--su) / 0.25);
    border-right-color: hsl(var(--su) / 0.25)}
.focus\:border-x-success\/30:focus {
    border-left-color: hsl(var(--su) / 0.3);
    border-right-color: hsl(var(--su) / 0.3)}
.focus\:border-x-success\/40:focus {
    border-left-color: hsl(var(--su) / 0.4);
    border-right-color: hsl(var(--su) / 0.4)}
.focus\:border-x-success\/5:focus {
    border-left-color: hsl(var(--su) / 0.05);
    border-right-color: hsl(var(--su) / 0.05)}
.focus\:border-x-success\/50:focus {
    border-left-color: hsl(var(--su) / 0.5);
    border-right-color: hsl(var(--su) / 0.5)}
.focus\:border-x-success\/60:focus {
    border-left-color: hsl(var(--su) / 0.6);
    border-right-color: hsl(var(--su) / 0.6)}
.focus\:border-x-success\/70:focus {
    border-left-color: hsl(var(--su) / 0.7);
    border-right-color: hsl(var(--su) / 0.7)}
.focus\:border-x-success\/75:focus {
    border-left-color: hsl(var(--su) / 0.75);
    border-right-color: hsl(var(--su) / 0.75)}
.focus\:border-x-success\/80:focus {
    border-left-color: hsl(var(--su) / 0.8);
    border-right-color: hsl(var(--su) / 0.8)}
.focus\:border-x-success\/90:focus {
    border-left-color: hsl(var(--su) / 0.9);
    border-right-color: hsl(var(--su) / 0.9)}
.focus\:border-x-success\/95:focus {
    border-left-color: hsl(var(--su) / 0.95);
    border-right-color: hsl(var(--su) / 0.95)}
.focus\:border-x-warning:focus {
    border-left-color: hsl(var(--wa) / 1);
    border-right-color: hsl(var(--wa) / 1)}
.focus\:border-x-warning-content:focus {
    border-left-color: hsl(var(--wac) / 1);
    border-right-color: hsl(var(--wac) / 1)}
.focus\:border-x-warning-content\/0:focus {
    border-left-color: hsl(var(--wac) / 0);
    border-right-color: hsl(var(--wac) / 0)}
.focus\:border-x-warning-content\/10:focus {
    border-left-color: hsl(var(--wac) / 0.1);
    border-right-color: hsl(var(--wac) / 0.1)}
.focus\:border-x-warning-content\/100:focus {
    border-left-color: hsl(var(--wac) / 1);
    border-right-color: hsl(var(--wac) / 1)}
.focus\:border-x-warning-content\/20:focus {
    border-left-color: hsl(var(--wac) / 0.2);
    border-right-color: hsl(var(--wac) / 0.2)}
.focus\:border-x-warning-content\/25:focus {
    border-left-color: hsl(var(--wac) / 0.25);
    border-right-color: hsl(var(--wac) / 0.25)}
.focus\:border-x-warning-content\/30:focus {
    border-left-color: hsl(var(--wac) / 0.3);
    border-right-color: hsl(var(--wac) / 0.3)}
.focus\:border-x-warning-content\/40:focus {
    border-left-color: hsl(var(--wac) / 0.4);
    border-right-color: hsl(var(--wac) / 0.4)}
.focus\:border-x-warning-content\/5:focus {
    border-left-color: hsl(var(--wac) / 0.05);
    border-right-color: hsl(var(--wac) / 0.05)}
.focus\:border-x-warning-content\/50:focus {
    border-left-color: hsl(var(--wac) / 0.5);
    border-right-color: hsl(var(--wac) / 0.5)}
.focus\:border-x-warning-content\/60:focus {
    border-left-color: hsl(var(--wac) / 0.6);
    border-right-color: hsl(var(--wac) / 0.6)}
.focus\:border-x-warning-content\/70:focus {
    border-left-color: hsl(var(--wac) / 0.7);
    border-right-color: hsl(var(--wac) / 0.7)}
.focus\:border-x-warning-content\/75:focus {
    border-left-color: hsl(var(--wac) / 0.75);
    border-right-color: hsl(var(--wac) / 0.75)}
.focus\:border-x-warning-content\/80:focus {
    border-left-color: hsl(var(--wac) / 0.8);
    border-right-color: hsl(var(--wac) / 0.8)}
.focus\:border-x-warning-content\/90:focus {
    border-left-color: hsl(var(--wac) / 0.9);
    border-right-color: hsl(var(--wac) / 0.9)}
.focus\:border-x-warning-content\/95:focus {
    border-left-color: hsl(var(--wac) / 0.95);
    border-right-color: hsl(var(--wac) / 0.95)}
.focus\:border-x-warning\/0:focus {
    border-left-color: hsl(var(--wa) / 0);
    border-right-color: hsl(var(--wa) / 0)}
.focus\:border-x-warning\/10:focus {
    border-left-color: hsl(var(--wa) / 0.1);
    border-right-color: hsl(var(--wa) / 0.1)}
.focus\:border-x-warning\/100:focus {
    border-left-color: hsl(var(--wa) / 1);
    border-right-color: hsl(var(--wa) / 1)}
.focus\:border-x-warning\/20:focus {
    border-left-color: hsl(var(--wa) / 0.2);
    border-right-color: hsl(var(--wa) / 0.2)}
.focus\:border-x-warning\/25:focus {
    border-left-color: hsl(var(--wa) / 0.25);
    border-right-color: hsl(var(--wa) / 0.25)}
.focus\:border-x-warning\/30:focus {
    border-left-color: hsl(var(--wa) / 0.3);
    border-right-color: hsl(var(--wa) / 0.3)}
.focus\:border-x-warning\/40:focus {
    border-left-color: hsl(var(--wa) / 0.4);
    border-right-color: hsl(var(--wa) / 0.4)}
.focus\:border-x-warning\/5:focus {
    border-left-color: hsl(var(--wa) / 0.05);
    border-right-color: hsl(var(--wa) / 0.05)}
.focus\:border-x-warning\/50:focus {
    border-left-color: hsl(var(--wa) / 0.5);
    border-right-color: hsl(var(--wa) / 0.5)}
.focus\:border-x-warning\/60:focus {
    border-left-color: hsl(var(--wa) / 0.6);
    border-right-color: hsl(var(--wa) / 0.6)}
.focus\:border-x-warning\/70:focus {
    border-left-color: hsl(var(--wa) / 0.7);
    border-right-color: hsl(var(--wa) / 0.7)}
.focus\:border-x-warning\/75:focus {
    border-left-color: hsl(var(--wa) / 0.75);
    border-right-color: hsl(var(--wa) / 0.75)}
.focus\:border-x-warning\/80:focus {
    border-left-color: hsl(var(--wa) / 0.8);
    border-right-color: hsl(var(--wa) / 0.8)}
.focus\:border-x-warning\/90:focus {
    border-left-color: hsl(var(--wa) / 0.9);
    border-right-color: hsl(var(--wa) / 0.9)}
.focus\:border-x-warning\/95:focus {
    border-left-color: hsl(var(--wa) / 0.95);
    border-right-color: hsl(var(--wa) / 0.95)}
.focus\:border-y-base-100:focus {
    border-top-color: hsl(var(--b1) / 1);
    border-bottom-color: hsl(var(--b1) / 1)}
.focus\:border-y-base-100\/0:focus {
    border-top-color: hsl(var(--b1) / 0);
    border-bottom-color: hsl(var(--b1) / 0)}
.focus\:border-y-base-100\/10:focus {
    border-top-color: hsl(var(--b1) / 0.1);
    border-bottom-color: hsl(var(--b1) / 0.1)}
.focus\:border-y-base-100\/100:focus {
    border-top-color: hsl(var(--b1) / 1);
    border-bottom-color: hsl(var(--b1) / 1)}
.focus\:border-y-base-100\/20:focus {
    border-top-color: hsl(var(--b1) / 0.2);
    border-bottom-color: hsl(var(--b1) / 0.2)}
.focus\:border-y-base-100\/25:focus {
    border-top-color: hsl(var(--b1) / 0.25);
    border-bottom-color: hsl(var(--b1) / 0.25)}
.focus\:border-y-base-100\/30:focus {
    border-top-color: hsl(var(--b1) / 0.3);
    border-bottom-color: hsl(var(--b1) / 0.3)}
.focus\:border-y-base-100\/40:focus {
    border-top-color: hsl(var(--b1) / 0.4);
    border-bottom-color: hsl(var(--b1) / 0.4)}
.focus\:border-y-base-100\/5:focus {
    border-top-color: hsl(var(--b1) / 0.05);
    border-bottom-color: hsl(var(--b1) / 0.05)}
.focus\:border-y-base-100\/50:focus {
    border-top-color: hsl(var(--b1) / 0.5);
    border-bottom-color: hsl(var(--b1) / 0.5)}
.focus\:border-y-base-100\/60:focus {
    border-top-color: hsl(var(--b1) / 0.6);
    border-bottom-color: hsl(var(--b1) / 0.6)}
.focus\:border-y-base-100\/70:focus {
    border-top-color: hsl(var(--b1) / 0.7);
    border-bottom-color: hsl(var(--b1) / 0.7)}
.focus\:border-y-base-100\/75:focus {
    border-top-color: hsl(var(--b1) / 0.75);
    border-bottom-color: hsl(var(--b1) / 0.75)}
.focus\:border-y-base-100\/80:focus {
    border-top-color: hsl(var(--b1) / 0.8);
    border-bottom-color: hsl(var(--b1) / 0.8)}
.focus\:border-y-base-100\/90:focus {
    border-top-color: hsl(var(--b1) / 0.9);
    border-bottom-color: hsl(var(--b1) / 0.9)}
.focus\:border-y-base-100\/95:focus {
    border-top-color: hsl(var(--b1) / 0.95);
    border-bottom-color: hsl(var(--b1) / 0.95)}
.focus\:border-y-base-200:focus {
    border-top-color: hsl(var(--b2) / 1);
    border-bottom-color: hsl(var(--b2) / 1)}
.focus\:border-y-base-200\/0:focus {
    border-top-color: hsl(var(--b2) / 0);
    border-bottom-color: hsl(var(--b2) / 0)}
.focus\:border-y-base-200\/10:focus {
    border-top-color: hsl(var(--b2) / 0.1);
    border-bottom-color: hsl(var(--b2) / 0.1)}
.focus\:border-y-base-200\/100:focus {
    border-top-color: hsl(var(--b2) / 1);
    border-bottom-color: hsl(var(--b2) / 1)}
.focus\:border-y-base-200\/20:focus {
    border-top-color: hsl(var(--b2) / 0.2);
    border-bottom-color: hsl(var(--b2) / 0.2)}
.focus\:border-y-base-200\/25:focus {
    border-top-color: hsl(var(--b2) / 0.25);
    border-bottom-color: hsl(var(--b2) / 0.25)}
.focus\:border-y-base-200\/30:focus {
    border-top-color: hsl(var(--b2) / 0.3);
    border-bottom-color: hsl(var(--b2) / 0.3)}
.focus\:border-y-base-200\/40:focus {
    border-top-color: hsl(var(--b2) / 0.4);
    border-bottom-color: hsl(var(--b2) / 0.4)}
.focus\:border-y-base-200\/5:focus {
    border-top-color: hsl(var(--b2) / 0.05);
    border-bottom-color: hsl(var(--b2) / 0.05)}
.focus\:border-y-base-200\/50:focus {
    border-top-color: hsl(var(--b2) / 0.5);
    border-bottom-color: hsl(var(--b2) / 0.5)}
.focus\:border-y-base-200\/60:focus {
    border-top-color: hsl(var(--b2) / 0.6);
    border-bottom-color: hsl(var(--b2) / 0.6)}
.focus\:border-y-base-200\/70:focus {
    border-top-color: hsl(var(--b2) / 0.7);
    border-bottom-color: hsl(var(--b2) / 0.7)}
.focus\:border-y-base-200\/75:focus {
    border-top-color: hsl(var(--b2) / 0.75);
    border-bottom-color: hsl(var(--b2) / 0.75)}
.focus\:border-y-base-200\/80:focus {
    border-top-color: hsl(var(--b2) / 0.8);
    border-bottom-color: hsl(var(--b2) / 0.8)}
.focus\:border-y-base-200\/90:focus {
    border-top-color: hsl(var(--b2) / 0.9);
    border-bottom-color: hsl(var(--b2) / 0.9)}
.focus\:border-y-base-200\/95:focus {
    border-top-color: hsl(var(--b2) / 0.95);
    border-bottom-color: hsl(var(--b2) / 0.95)}
.focus\:border-y-base-300:focus {
    border-top-color: hsl(var(--b3) / 1);
    border-bottom-color: hsl(var(--b3) / 1)}
.focus\:border-y-base-300\/0:focus {
    border-top-color: hsl(var(--b3) / 0);
    border-bottom-color: hsl(var(--b3) / 0)}
.focus\:border-y-base-300\/10:focus {
    border-top-color: hsl(var(--b3) / 0.1);
    border-bottom-color: hsl(var(--b3) / 0.1)}
.focus\:border-y-base-300\/100:focus {
    border-top-color: hsl(var(--b3) / 1);
    border-bottom-color: hsl(var(--b3) / 1)}
.focus\:border-y-base-300\/20:focus {
    border-top-color: hsl(var(--b3) / 0.2);
    border-bottom-color: hsl(var(--b3) / 0.2)}
.focus\:border-y-base-300\/25:focus {
    border-top-color: hsl(var(--b3) / 0.25);
    border-bottom-color: hsl(var(--b3) / 0.25)}
.focus\:border-y-base-300\/30:focus {
    border-top-color: hsl(var(--b3) / 0.3);
    border-bottom-color: hsl(var(--b3) / 0.3)}
.focus\:border-y-base-300\/40:focus {
    border-top-color: hsl(var(--b3) / 0.4);
    border-bottom-color: hsl(var(--b3) / 0.4)}
.focus\:border-y-base-300\/5:focus {
    border-top-color: hsl(var(--b3) / 0.05);
    border-bottom-color: hsl(var(--b3) / 0.05)}
.focus\:border-y-base-300\/50:focus {
    border-top-color: hsl(var(--b3) / 0.5);
    border-bottom-color: hsl(var(--b3) / 0.5)}
.focus\:border-y-base-300\/60:focus {
    border-top-color: hsl(var(--b3) / 0.6);
    border-bottom-color: hsl(var(--b3) / 0.6)}
.focus\:border-y-base-300\/70:focus {
    border-top-color: hsl(var(--b3) / 0.7);
    border-bottom-color: hsl(var(--b3) / 0.7)}
.focus\:border-y-base-300\/75:focus {
    border-top-color: hsl(var(--b3) / 0.75);
    border-bottom-color: hsl(var(--b3) / 0.75)}
.focus\:border-y-base-300\/80:focus {
    border-top-color: hsl(var(--b3) / 0.8);
    border-bottom-color: hsl(var(--b3) / 0.8)}
.focus\:border-y-base-300\/90:focus {
    border-top-color: hsl(var(--b3) / 0.9);
    border-bottom-color: hsl(var(--b3) / 0.9)}
.focus\:border-y-base-300\/95:focus {
    border-top-color: hsl(var(--b3) / 0.95);
    border-bottom-color: hsl(var(--b3) / 0.95)}
.focus\:border-y-base-content:focus {
    border-top-color: hsl(var(--bc) / 1);
    border-bottom-color: hsl(var(--bc) / 1)}
.focus\:border-y-base-content\/0:focus {
    border-top-color: hsl(var(--bc) / 0);
    border-bottom-color: hsl(var(--bc) / 0)}
.focus\:border-y-base-content\/10:focus {
    border-top-color: hsl(var(--bc) / 0.1);
    border-bottom-color: hsl(var(--bc) / 0.1)}
.focus\:border-y-base-content\/100:focus {
    border-top-color: hsl(var(--bc) / 1);
    border-bottom-color: hsl(var(--bc) / 1)}
.focus\:border-y-base-content\/20:focus {
    border-top-color: hsl(var(--bc) / 0.2);
    border-bottom-color: hsl(var(--bc) / 0.2)}
.focus\:border-y-base-content\/25:focus {
    border-top-color: hsl(var(--bc) / 0.25);
    border-bottom-color: hsl(var(--bc) / 0.25)}
.focus\:border-y-base-content\/30:focus {
    border-top-color: hsl(var(--bc) / 0.3);
    border-bottom-color: hsl(var(--bc) / 0.3)}
.focus\:border-y-base-content\/40:focus {
    border-top-color: hsl(var(--bc) / 0.4);
    border-bottom-color: hsl(var(--bc) / 0.4)}
.focus\:border-y-base-content\/5:focus {
    border-top-color: hsl(var(--bc) / 0.05);
    border-bottom-color: hsl(var(--bc) / 0.05)}
.focus\:border-y-base-content\/50:focus {
    border-top-color: hsl(var(--bc) / 0.5);
    border-bottom-color: hsl(var(--bc) / 0.5)}
.focus\:border-y-base-content\/60:focus {
    border-top-color: hsl(var(--bc) / 0.6);
    border-bottom-color: hsl(var(--bc) / 0.6)}
.focus\:border-y-base-content\/70:focus {
    border-top-color: hsl(var(--bc) / 0.7);
    border-bottom-color: hsl(var(--bc) / 0.7)}
.focus\:border-y-base-content\/75:focus {
    border-top-color: hsl(var(--bc) / 0.75);
    border-bottom-color: hsl(var(--bc) / 0.75)}
.focus\:border-y-base-content\/80:focus {
    border-top-color: hsl(var(--bc) / 0.8);
    border-bottom-color: hsl(var(--bc) / 0.8)}
.focus\:border-y-base-content\/90:focus {
    border-top-color: hsl(var(--bc) / 0.9);
    border-bottom-color: hsl(var(--bc) / 0.9)}
.focus\:border-y-base-content\/95:focus {
    border-top-color: hsl(var(--bc) / 0.95);
    border-bottom-color: hsl(var(--bc) / 0.95)}
.focus\:border-y-error:focus {
    border-top-color: hsl(var(--er) / 1);
    border-bottom-color: hsl(var(--er) / 1)}
.focus\:border-y-error-content:focus {
    border-top-color: hsl(var(--erc) / 1);
    border-bottom-color: hsl(var(--erc) / 1)}
.focus\:border-y-error-content\/0:focus {
    border-top-color: hsl(var(--erc) / 0);
    border-bottom-color: hsl(var(--erc) / 0)}
.focus\:border-y-error-content\/10:focus {
    border-top-color: hsl(var(--erc) / 0.1);
    border-bottom-color: hsl(var(--erc) / 0.1)}
.focus\:border-y-error-content\/100:focus {
    border-top-color: hsl(var(--erc) / 1);
    border-bottom-color: hsl(var(--erc) / 1)}
.focus\:border-y-error-content\/20:focus {
    border-top-color: hsl(var(--erc) / 0.2);
    border-bottom-color: hsl(var(--erc) / 0.2)}
.focus\:border-y-error-content\/25:focus {
    border-top-color: hsl(var(--erc) / 0.25);
    border-bottom-color: hsl(var(--erc) / 0.25)}
.focus\:border-y-error-content\/30:focus {
    border-top-color: hsl(var(--erc) / 0.3);
    border-bottom-color: hsl(var(--erc) / 0.3)}
.focus\:border-y-error-content\/40:focus {
    border-top-color: hsl(var(--erc) / 0.4);
    border-bottom-color: hsl(var(--erc) / 0.4)}
.focus\:border-y-error-content\/5:focus {
    border-top-color: hsl(var(--erc) / 0.05);
    border-bottom-color: hsl(var(--erc) / 0.05)}
.focus\:border-y-error-content\/50:focus {
    border-top-color: hsl(var(--erc) / 0.5);
    border-bottom-color: hsl(var(--erc) / 0.5)}
.focus\:border-y-error-content\/60:focus {
    border-top-color: hsl(var(--erc) / 0.6);
    border-bottom-color: hsl(var(--erc) / 0.6)}
.focus\:border-y-error-content\/70:focus {
    border-top-color: hsl(var(--erc) / 0.7);
    border-bottom-color: hsl(var(--erc) / 0.7)}
.focus\:border-y-error-content\/75:focus {
    border-top-color: hsl(var(--erc) / 0.75);
    border-bottom-color: hsl(var(--erc) / 0.75)}
.focus\:border-y-error-content\/80:focus {
    border-top-color: hsl(var(--erc) / 0.8);
    border-bottom-color: hsl(var(--erc) / 0.8)}
.focus\:border-y-error-content\/90:focus {
    border-top-color: hsl(var(--erc) / 0.9);
    border-bottom-color: hsl(var(--erc) / 0.9)}
.focus\:border-y-error-content\/95:focus {
    border-top-color: hsl(var(--erc) / 0.95);
    border-bottom-color: hsl(var(--erc) / 0.95)}
.focus\:border-y-error\/0:focus {
    border-top-color: hsl(var(--er) / 0);
    border-bottom-color: hsl(var(--er) / 0)}
.focus\:border-y-error\/10:focus {
    border-top-color: hsl(var(--er) / 0.1);
    border-bottom-color: hsl(var(--er) / 0.1)}
.focus\:border-y-error\/100:focus {
    border-top-color: hsl(var(--er) / 1);
    border-bottom-color: hsl(var(--er) / 1)}
.focus\:border-y-error\/20:focus {
    border-top-color: hsl(var(--er) / 0.2);
    border-bottom-color: hsl(var(--er) / 0.2)}
.focus\:border-y-error\/25:focus {
    border-top-color: hsl(var(--er) / 0.25);
    border-bottom-color: hsl(var(--er) / 0.25)}
.focus\:border-y-error\/30:focus {
    border-top-color: hsl(var(--er) / 0.3);
    border-bottom-color: hsl(var(--er) / 0.3)}
.focus\:border-y-error\/40:focus {
    border-top-color: hsl(var(--er) / 0.4);
    border-bottom-color: hsl(var(--er) / 0.4)}
.focus\:border-y-error\/5:focus {
    border-top-color: hsl(var(--er) / 0.05);
    border-bottom-color: hsl(var(--er) / 0.05)}
.focus\:border-y-error\/50:focus {
    border-top-color: hsl(var(--er) / 0.5);
    border-bottom-color: hsl(var(--er) / 0.5)}
.focus\:border-y-error\/60:focus {
    border-top-color: hsl(var(--er) / 0.6);
    border-bottom-color: hsl(var(--er) / 0.6)}
.focus\:border-y-error\/70:focus {
    border-top-color: hsl(var(--er) / 0.7);
    border-bottom-color: hsl(var(--er) / 0.7)}
.focus\:border-y-error\/75:focus {
    border-top-color: hsl(var(--er) / 0.75);
    border-bottom-color: hsl(var(--er) / 0.75)}
.focus\:border-y-error\/80:focus {
    border-top-color: hsl(var(--er) / 0.8);
    border-bottom-color: hsl(var(--er) / 0.8)}
.focus\:border-y-error\/90:focus {
    border-top-color: hsl(var(--er) / 0.9);
    border-bottom-color: hsl(var(--er) / 0.9)}
.focus\:border-y-error\/95:focus {
    border-top-color: hsl(var(--er) / 0.95);
    border-bottom-color: hsl(var(--er) / 0.95)}
.focus\:border-y-info:focus {
    border-top-color: hsl(var(--in) / 1);
    border-bottom-color: hsl(var(--in) / 1)}
.focus\:border-y-info-content:focus {
    border-top-color: hsl(var(--inc) / 1);
    border-bottom-color: hsl(var(--inc) / 1)}
.focus\:border-y-info-content\/0:focus {
    border-top-color: hsl(var(--inc) / 0);
    border-bottom-color: hsl(var(--inc) / 0)}
.focus\:border-y-info-content\/10:focus {
    border-top-color: hsl(var(--inc) / 0.1);
    border-bottom-color: hsl(var(--inc) / 0.1)}
.focus\:border-y-info-content\/100:focus {
    border-top-color: hsl(var(--inc) / 1);
    border-bottom-color: hsl(var(--inc) / 1)}
.focus\:border-y-info-content\/20:focus {
    border-top-color: hsl(var(--inc) / 0.2);
    border-bottom-color: hsl(var(--inc) / 0.2)}
.focus\:border-y-info-content\/25:focus {
    border-top-color: hsl(var(--inc) / 0.25);
    border-bottom-color: hsl(var(--inc) / 0.25)}
.focus\:border-y-info-content\/30:focus {
    border-top-color: hsl(var(--inc) / 0.3);
    border-bottom-color: hsl(var(--inc) / 0.3)}
.focus\:border-y-info-content\/40:focus {
    border-top-color: hsl(var(--inc) / 0.4);
    border-bottom-color: hsl(var(--inc) / 0.4)}
.focus\:border-y-info-content\/5:focus {
    border-top-color: hsl(var(--inc) / 0.05);
    border-bottom-color: hsl(var(--inc) / 0.05)}
.focus\:border-y-info-content\/50:focus {
    border-top-color: hsl(var(--inc) / 0.5);
    border-bottom-color: hsl(var(--inc) / 0.5)}
.focus\:border-y-info-content\/60:focus {
    border-top-color: hsl(var(--inc) / 0.6);
    border-bottom-color: hsl(var(--inc) / 0.6)}
.focus\:border-y-info-content\/70:focus {
    border-top-color: hsl(var(--inc) / 0.7);
    border-bottom-color: hsl(var(--inc) / 0.7)}
.focus\:border-y-info-content\/75:focus {
    border-top-color: hsl(var(--inc) / 0.75);
    border-bottom-color: hsl(var(--inc) / 0.75)}
.focus\:border-y-info-content\/80:focus {
    border-top-color: hsl(var(--inc) / 0.8);
    border-bottom-color: hsl(var(--inc) / 0.8)}
.focus\:border-y-info-content\/90:focus {
    border-top-color: hsl(var(--inc) / 0.9);
    border-bottom-color: hsl(var(--inc) / 0.9)}
.focus\:border-y-info-content\/95:focus {
    border-top-color: hsl(var(--inc) / 0.95);
    border-bottom-color: hsl(var(--inc) / 0.95)}
.focus\:border-y-info\/0:focus {
    border-top-color: hsl(var(--in) / 0);
    border-bottom-color: hsl(var(--in) / 0)}
.focus\:border-y-info\/10:focus {
    border-top-color: hsl(var(--in) / 0.1);
    border-bottom-color: hsl(var(--in) / 0.1)}
.focus\:border-y-info\/100:focus {
    border-top-color: hsl(var(--in) / 1);
    border-bottom-color: hsl(var(--in) / 1)}
.focus\:border-y-info\/20:focus {
    border-top-color: hsl(var(--in) / 0.2);
    border-bottom-color: hsl(var(--in) / 0.2)}
.focus\:border-y-info\/25:focus {
    border-top-color: hsl(var(--in) / 0.25);
    border-bottom-color: hsl(var(--in) / 0.25)}
.focus\:border-y-info\/30:focus {
    border-top-color: hsl(var(--in) / 0.3);
    border-bottom-color: hsl(var(--in) / 0.3)}
.focus\:border-y-info\/40:focus {
    border-top-color: hsl(var(--in) / 0.4);
    border-bottom-color: hsl(var(--in) / 0.4)}
.focus\:border-y-info\/5:focus {
    border-top-color: hsl(var(--in) / 0.05);
    border-bottom-color: hsl(var(--in) / 0.05)}
.focus\:border-y-info\/50:focus {
    border-top-color: hsl(var(--in) / 0.5);
    border-bottom-color: hsl(var(--in) / 0.5)}
.focus\:border-y-info\/60:focus {
    border-top-color: hsl(var(--in) / 0.6);
    border-bottom-color: hsl(var(--in) / 0.6)}
.focus\:border-y-info\/70:focus {
    border-top-color: hsl(var(--in) / 0.7);
    border-bottom-color: hsl(var(--in) / 0.7)}
.focus\:border-y-info\/75:focus {
    border-top-color: hsl(var(--in) / 0.75);
    border-bottom-color: hsl(var(--in) / 0.75)}
.focus\:border-y-info\/80:focus {
    border-top-color: hsl(var(--in) / 0.8);
    border-bottom-color: hsl(var(--in) / 0.8)}
.focus\:border-y-info\/90:focus {
    border-top-color: hsl(var(--in) / 0.9);
    border-bottom-color: hsl(var(--in) / 0.9)}
.focus\:border-y-info\/95:focus {
    border-top-color: hsl(var(--in) / 0.95);
    border-bottom-color: hsl(var(--in) / 0.95)}
.focus\:border-y-success:focus {
    border-top-color: hsl(var(--su) / 1);
    border-bottom-color: hsl(var(--su) / 1)}
.focus\:border-y-success-content:focus {
    border-top-color: hsl(var(--suc) / 1);
    border-bottom-color: hsl(var(--suc) / 1)}
.focus\:border-y-success-content\/0:focus {
    border-top-color: hsl(var(--suc) / 0);
    border-bottom-color: hsl(var(--suc) / 0)}
.focus\:border-y-success-content\/10:focus {
    border-top-color: hsl(var(--suc) / 0.1);
    border-bottom-color: hsl(var(--suc) / 0.1)}
.focus\:border-y-success-content\/100:focus {
    border-top-color: hsl(var(--suc) / 1);
    border-bottom-color: hsl(var(--suc) / 1)}
.focus\:border-y-success-content\/20:focus {
    border-top-color: hsl(var(--suc) / 0.2);
    border-bottom-color: hsl(var(--suc) / 0.2)}
.focus\:border-y-success-content\/25:focus {
    border-top-color: hsl(var(--suc) / 0.25);
    border-bottom-color: hsl(var(--suc) / 0.25)}
.focus\:border-y-success-content\/30:focus {
    border-top-color: hsl(var(--suc) / 0.3);
    border-bottom-color: hsl(var(--suc) / 0.3)}
.focus\:border-y-success-content\/40:focus {
    border-top-color: hsl(var(--suc) / 0.4);
    border-bottom-color: hsl(var(--suc) / 0.4)}
.focus\:border-y-success-content\/5:focus {
    border-top-color: hsl(var(--suc) / 0.05);
    border-bottom-color: hsl(var(--suc) / 0.05)}
.focus\:border-y-success-content\/50:focus {
    border-top-color: hsl(var(--suc) / 0.5);
    border-bottom-color: hsl(var(--suc) / 0.5)}
.focus\:border-y-success-content\/60:focus {
    border-top-color: hsl(var(--suc) / 0.6);
    border-bottom-color: hsl(var(--suc) / 0.6)}
.focus\:border-y-success-content\/70:focus {
    border-top-color: hsl(var(--suc) / 0.7);
    border-bottom-color: hsl(var(--suc) / 0.7)}
.focus\:border-y-success-content\/75:focus {
    border-top-color: hsl(var(--suc) / 0.75);
    border-bottom-color: hsl(var(--suc) / 0.75)}
.focus\:border-y-success-content\/80:focus {
    border-top-color: hsl(var(--suc) / 0.8);
    border-bottom-color: hsl(var(--suc) / 0.8)}
.focus\:border-y-success-content\/90:focus {
    border-top-color: hsl(var(--suc) / 0.9);
    border-bottom-color: hsl(var(--suc) / 0.9)}
.focus\:border-y-success-content\/95:focus {
    border-top-color: hsl(var(--suc) / 0.95);
    border-bottom-color: hsl(var(--suc) / 0.95)}
.focus\:border-y-success\/0:focus {
    border-top-color: hsl(var(--su) / 0);
    border-bottom-color: hsl(var(--su) / 0)}
.focus\:border-y-success\/10:focus {
    border-top-color: hsl(var(--su) / 0.1);
    border-bottom-color: hsl(var(--su) / 0.1)}
.focus\:border-y-success\/100:focus {
    border-top-color: hsl(var(--su) / 1);
    border-bottom-color: hsl(var(--su) / 1)}
.focus\:border-y-success\/20:focus {
    border-top-color: hsl(var(--su) / 0.2);
    border-bottom-color: hsl(var(--su) / 0.2)}
.focus\:border-y-success\/25:focus {
    border-top-color: hsl(var(--su) / 0.25);
    border-bottom-color: hsl(var(--su) / 0.25)}
.focus\:border-y-success\/30:focus {
    border-top-color: hsl(var(--su) / 0.3);
    border-bottom-color: hsl(var(--su) / 0.3)}
.focus\:border-y-success\/40:focus {
    border-top-color: hsl(var(--su) / 0.4);
    border-bottom-color: hsl(var(--su) / 0.4)}
.focus\:border-y-success\/5:focus {
    border-top-color: hsl(var(--su) / 0.05);
    border-bottom-color: hsl(var(--su) / 0.05)}
.focus\:border-y-success\/50:focus {
    border-top-color: hsl(var(--su) / 0.5);
    border-bottom-color: hsl(var(--su) / 0.5)}
.focus\:border-y-success\/60:focus {
    border-top-color: hsl(var(--su) / 0.6);
    border-bottom-color: hsl(var(--su) / 0.6)}
.focus\:border-y-success\/70:focus {
    border-top-color: hsl(var(--su) / 0.7);
    border-bottom-color: hsl(var(--su) / 0.7)}
.focus\:border-y-success\/75:focus {
    border-top-color: hsl(var(--su) / 0.75);
    border-bottom-color: hsl(var(--su) / 0.75)}
.focus\:border-y-success\/80:focus {
    border-top-color: hsl(var(--su) / 0.8);
    border-bottom-color: hsl(var(--su) / 0.8)}
.focus\:border-y-success\/90:focus {
    border-top-color: hsl(var(--su) / 0.9);
    border-bottom-color: hsl(var(--su) / 0.9)}
.focus\:border-y-success\/95:focus {
    border-top-color: hsl(var(--su) / 0.95);
    border-bottom-color: hsl(var(--su) / 0.95)}
.focus\:border-y-warning:focus {
    border-top-color: hsl(var(--wa) / 1);
    border-bottom-color: hsl(var(--wa) / 1)}
.focus\:border-y-warning-content:focus {
    border-top-color: hsl(var(--wac) / 1);
    border-bottom-color: hsl(var(--wac) / 1)}
.focus\:border-y-warning-content\/0:focus {
    border-top-color: hsl(var(--wac) / 0);
    border-bottom-color: hsl(var(--wac) / 0)}
.focus\:border-y-warning-content\/10:focus {
    border-top-color: hsl(var(--wac) / 0.1);
    border-bottom-color: hsl(var(--wac) / 0.1)}
.focus\:border-y-warning-content\/100:focus {
    border-top-color: hsl(var(--wac) / 1);
    border-bottom-color: hsl(var(--wac) / 1)}
.focus\:border-y-warning-content\/20:focus {
    border-top-color: hsl(var(--wac) / 0.2);
    border-bottom-color: hsl(var(--wac) / 0.2)}
.focus\:border-y-warning-content\/25:focus {
    border-top-color: hsl(var(--wac) / 0.25);
    border-bottom-color: hsl(var(--wac) / 0.25)}
.focus\:border-y-warning-content\/30:focus {
    border-top-color: hsl(var(--wac) / 0.3);
    border-bottom-color: hsl(var(--wac) / 0.3)}
.focus\:border-y-warning-content\/40:focus {
    border-top-color: hsl(var(--wac) / 0.4);
    border-bottom-color: hsl(var(--wac) / 0.4)}
.focus\:border-y-warning-content\/5:focus {
    border-top-color: hsl(var(--wac) / 0.05);
    border-bottom-color: hsl(var(--wac) / 0.05)}
.focus\:border-y-warning-content\/50:focus {
    border-top-color: hsl(var(--wac) / 0.5);
    border-bottom-color: hsl(var(--wac) / 0.5)}
.focus\:border-y-warning-content\/60:focus {
    border-top-color: hsl(var(--wac) / 0.6);
    border-bottom-color: hsl(var(--wac) / 0.6)}
.focus\:border-y-warning-content\/70:focus {
    border-top-color: hsl(var(--wac) / 0.7);
    border-bottom-color: hsl(var(--wac) / 0.7)}
.focus\:border-y-warning-content\/75:focus {
    border-top-color: hsl(var(--wac) / 0.75);
    border-bottom-color: hsl(var(--wac) / 0.75)}
.focus\:border-y-warning-content\/80:focus {
    border-top-color: hsl(var(--wac) / 0.8);
    border-bottom-color: hsl(var(--wac) / 0.8)}
.focus\:border-y-warning-content\/90:focus {
    border-top-color: hsl(var(--wac) / 0.9);
    border-bottom-color: hsl(var(--wac) / 0.9)}
.focus\:border-y-warning-content\/95:focus {
    border-top-color: hsl(var(--wac) / 0.95);
    border-bottom-color: hsl(var(--wac) / 0.95)}
.focus\:border-y-warning\/0:focus {
    border-top-color: hsl(var(--wa) / 0);
    border-bottom-color: hsl(var(--wa) / 0)}
.focus\:border-y-warning\/10:focus {
    border-top-color: hsl(var(--wa) / 0.1);
    border-bottom-color: hsl(var(--wa) / 0.1)}
.focus\:border-y-warning\/100:focus {
    border-top-color: hsl(var(--wa) / 1);
    border-bottom-color: hsl(var(--wa) / 1)}
.focus\:border-y-warning\/20:focus {
    border-top-color: hsl(var(--wa) / 0.2);
    border-bottom-color: hsl(var(--wa) / 0.2)}
.focus\:border-y-warning\/25:focus {
    border-top-color: hsl(var(--wa) / 0.25);
    border-bottom-color: hsl(var(--wa) / 0.25)}
.focus\:border-y-warning\/30:focus {
    border-top-color: hsl(var(--wa) / 0.3);
    border-bottom-color: hsl(var(--wa) / 0.3)}
.focus\:border-y-warning\/40:focus {
    border-top-color: hsl(var(--wa) / 0.4);
    border-bottom-color: hsl(var(--wa) / 0.4)}
.focus\:border-y-warning\/5:focus {
    border-top-color: hsl(var(--wa) / 0.05);
    border-bottom-color: hsl(var(--wa) / 0.05)}
.focus\:border-y-warning\/50:focus {
    border-top-color: hsl(var(--wa) / 0.5);
    border-bottom-color: hsl(var(--wa) / 0.5)}
.focus\:border-y-warning\/60:focus {
    border-top-color: hsl(var(--wa) / 0.6);
    border-bottom-color: hsl(var(--wa) / 0.6)}
.focus\:border-y-warning\/70:focus {
    border-top-color: hsl(var(--wa) / 0.7);
    border-bottom-color: hsl(var(--wa) / 0.7)}
.focus\:border-y-warning\/75:focus {
    border-top-color: hsl(var(--wa) / 0.75);
    border-bottom-color: hsl(var(--wa) / 0.75)}
.focus\:border-y-warning\/80:focus {
    border-top-color: hsl(var(--wa) / 0.8);
    border-bottom-color: hsl(var(--wa) / 0.8)}
.focus\:border-y-warning\/90:focus {
    border-top-color: hsl(var(--wa) / 0.9);
    border-bottom-color: hsl(var(--wa) / 0.9)}
.focus\:border-y-warning\/95:focus {
    border-top-color: hsl(var(--wa) / 0.95);
    border-bottom-color: hsl(var(--wa) / 0.95)}
.focus\:border-b-base-100:focus {
    border-bottom-color: hsl(var(--b1) / 1)}
.focus\:border-b-base-100\/0:focus {
    border-bottom-color: hsl(var(--b1) / 0)}
.focus\:border-b-base-100\/10:focus {
    border-bottom-color: hsl(var(--b1) / 0.1)}
.focus\:border-b-base-100\/100:focus {
    border-bottom-color: hsl(var(--b1) / 1)}
.focus\:border-b-base-100\/20:focus {
    border-bottom-color: hsl(var(--b1) / 0.2)}
.focus\:border-b-base-100\/25:focus {
    border-bottom-color: hsl(var(--b1) / 0.25)}
.focus\:border-b-base-100\/30:focus {
    border-bottom-color: hsl(var(--b1) / 0.3)}
.focus\:border-b-base-100\/40:focus {
    border-bottom-color: hsl(var(--b1) / 0.4)}
.focus\:border-b-base-100\/5:focus {
    border-bottom-color: hsl(var(--b1) / 0.05)}
.focus\:border-b-base-100\/50:focus {
    border-bottom-color: hsl(var(--b1) / 0.5)}
.focus\:border-b-base-100\/60:focus {
    border-bottom-color: hsl(var(--b1) / 0.6)}
.focus\:border-b-base-100\/70:focus {
    border-bottom-color: hsl(var(--b1) / 0.7)}
.focus\:border-b-base-100\/75:focus {
    border-bottom-color: hsl(var(--b1) / 0.75)}
.focus\:border-b-base-100\/80:focus {
    border-bottom-color: hsl(var(--b1) / 0.8)}
.focus\:border-b-base-100\/90:focus {
    border-bottom-color: hsl(var(--b1) / 0.9)}
.focus\:border-b-base-100\/95:focus {
    border-bottom-color: hsl(var(--b1) / 0.95)}
.focus\:border-b-base-200:focus {
    border-bottom-color: hsl(var(--b2) / 1)}
.focus\:border-b-base-200\/0:focus {
    border-bottom-color: hsl(var(--b2) / 0)}
.focus\:border-b-base-200\/10:focus {
    border-bottom-color: hsl(var(--b2) / 0.1)}
.focus\:border-b-base-200\/100:focus {
    border-bottom-color: hsl(var(--b2) / 1)}
.focus\:border-b-base-200\/20:focus {
    border-bottom-color: hsl(var(--b2) / 0.2)}
.focus\:border-b-base-200\/25:focus {
    border-bottom-color: hsl(var(--b2) / 0.25)}
.focus\:border-b-base-200\/30:focus {
    border-bottom-color: hsl(var(--b2) / 0.3)}
.focus\:border-b-base-200\/40:focus {
    border-bottom-color: hsl(var(--b2) / 0.4)}
.focus\:border-b-base-200\/5:focus {
    border-bottom-color: hsl(var(--b2) / 0.05)}
.focus\:border-b-base-200\/50:focus {
    border-bottom-color: hsl(var(--b2) / 0.5)}
.focus\:border-b-base-200\/60:focus {
    border-bottom-color: hsl(var(--b2) / 0.6)}
.focus\:border-b-base-200\/70:focus {
    border-bottom-color: hsl(var(--b2) / 0.7)}
.focus\:border-b-base-200\/75:focus {
    border-bottom-color: hsl(var(--b2) / 0.75)}
.focus\:border-b-base-200\/80:focus {
    border-bottom-color: hsl(var(--b2) / 0.8)}
.focus\:border-b-base-200\/90:focus {
    border-bottom-color: hsl(var(--b2) / 0.9)}
.focus\:border-b-base-200\/95:focus {
    border-bottom-color: hsl(var(--b2) / 0.95)}
.focus\:border-b-base-300:focus {
    border-bottom-color: hsl(var(--b3) / 1)}
.focus\:border-b-base-300\/0:focus {
    border-bottom-color: hsl(var(--b3) / 0)}
.focus\:border-b-base-300\/10:focus {
    border-bottom-color: hsl(var(--b3) / 0.1)}
.focus\:border-b-base-300\/100:focus {
    border-bottom-color: hsl(var(--b3) / 1)}
.focus\:border-b-base-300\/20:focus {
    border-bottom-color: hsl(var(--b3) / 0.2)}
.focus\:border-b-base-300\/25:focus {
    border-bottom-color: hsl(var(--b3) / 0.25)}
.focus\:border-b-base-300\/30:focus {
    border-bottom-color: hsl(var(--b3) / 0.3)}
.focus\:border-b-base-300\/40:focus {
    border-bottom-color: hsl(var(--b3) / 0.4)}
.focus\:border-b-base-300\/5:focus {
    border-bottom-color: hsl(var(--b3) / 0.05)}
.focus\:border-b-base-300\/50:focus {
    border-bottom-color: hsl(var(--b3) / 0.5)}
.focus\:border-b-base-300\/60:focus {
    border-bottom-color: hsl(var(--b3) / 0.6)}
.focus\:border-b-base-300\/70:focus {
    border-bottom-color: hsl(var(--b3) / 0.7)}
.focus\:border-b-base-300\/75:focus {
    border-bottom-color: hsl(var(--b3) / 0.75)}
.focus\:border-b-base-300\/80:focus {
    border-bottom-color: hsl(var(--b3) / 0.8)}
.focus\:border-b-base-300\/90:focus {
    border-bottom-color: hsl(var(--b3) / 0.9)}
.focus\:border-b-base-300\/95:focus {
    border-bottom-color: hsl(var(--b3) / 0.95)}
.focus\:border-b-base-content:focus {
    border-bottom-color: hsl(var(--bc) / 1)}
.focus\:border-b-base-content\/0:focus {
    border-bottom-color: hsl(var(--bc) / 0)}
.focus\:border-b-base-content\/10:focus {
    border-bottom-color: hsl(var(--bc) / 0.1)}
.focus\:border-b-base-content\/100:focus {
    border-bottom-color: hsl(var(--bc) / 1)}
.focus\:border-b-base-content\/20:focus {
    border-bottom-color: hsl(var(--bc) / 0.2)}
.focus\:border-b-base-content\/25:focus {
    border-bottom-color: hsl(var(--bc) / 0.25)}
.focus\:border-b-base-content\/30:focus {
    border-bottom-color: hsl(var(--bc) / 0.3)}
.focus\:border-b-base-content\/40:focus {
    border-bottom-color: hsl(var(--bc) / 0.4)}
.focus\:border-b-base-content\/5:focus {
    border-bottom-color: hsl(var(--bc) / 0.05)}
.focus\:border-b-base-content\/50:focus {
    border-bottom-color: hsl(var(--bc) / 0.5)}
.focus\:border-b-base-content\/60:focus {
    border-bottom-color: hsl(var(--bc) / 0.6)}
.focus\:border-b-base-content\/70:focus {
    border-bottom-color: hsl(var(--bc) / 0.7)}
.focus\:border-b-base-content\/75:focus {
    border-bottom-color: hsl(var(--bc) / 0.75)}
.focus\:border-b-base-content\/80:focus {
    border-bottom-color: hsl(var(--bc) / 0.8)}
.focus\:border-b-base-content\/90:focus {
    border-bottom-color: hsl(var(--bc) / 0.9)}
.focus\:border-b-base-content\/95:focus {
    border-bottom-color: hsl(var(--bc) / 0.95)}
.focus\:border-b-error:focus {
    border-bottom-color: hsl(var(--er) / 1)}
.focus\:border-b-error-content:focus {
    border-bottom-color: hsl(var(--erc) / 1)}
.focus\:border-b-error-content\/0:focus {
    border-bottom-color: hsl(var(--erc) / 0)}
.focus\:border-b-error-content\/10:focus {
    border-bottom-color: hsl(var(--erc) / 0.1)}
.focus\:border-b-error-content\/100:focus {
    border-bottom-color: hsl(var(--erc) / 1)}
.focus\:border-b-error-content\/20:focus {
    border-bottom-color: hsl(var(--erc) / 0.2)}
.focus\:border-b-error-content\/25:focus {
    border-bottom-color: hsl(var(--erc) / 0.25)}
.focus\:border-b-error-content\/30:focus {
    border-bottom-color: hsl(var(--erc) / 0.3)}
.focus\:border-b-error-content\/40:focus {
    border-bottom-color: hsl(var(--erc) / 0.4)}
.focus\:border-b-error-content\/5:focus {
    border-bottom-color: hsl(var(--erc) / 0.05)}
.focus\:border-b-error-content\/50:focus {
    border-bottom-color: hsl(var(--erc) / 0.5)}
.focus\:border-b-error-content\/60:focus {
    border-bottom-color: hsl(var(--erc) / 0.6)}
.focus\:border-b-error-content\/70:focus {
    border-bottom-color: hsl(var(--erc) / 0.7)}
.focus\:border-b-error-content\/75:focus {
    border-bottom-color: hsl(var(--erc) / 0.75)}
.focus\:border-b-error-content\/80:focus {
    border-bottom-color: hsl(var(--erc) / 0.8)}
.focus\:border-b-error-content\/90:focus {
    border-bottom-color: hsl(var(--erc) / 0.9)}
.focus\:border-b-error-content\/95:focus {
    border-bottom-color: hsl(var(--erc) / 0.95)}
.focus\:border-b-error\/0:focus {
    border-bottom-color: hsl(var(--er) / 0)}
.focus\:border-b-error\/10:focus {
    border-bottom-color: hsl(var(--er) / 0.1)}
.focus\:border-b-error\/100:focus {
    border-bottom-color: hsl(var(--er) / 1)}
.focus\:border-b-error\/20:focus {
    border-bottom-color: hsl(var(--er) / 0.2)}
.focus\:border-b-error\/25:focus {
    border-bottom-color: hsl(var(--er) / 0.25)}
.focus\:border-b-error\/30:focus {
    border-bottom-color: hsl(var(--er) / 0.3)}
.focus\:border-b-error\/40:focus {
    border-bottom-color: hsl(var(--er) / 0.4)}
.focus\:border-b-error\/5:focus {
    border-bottom-color: hsl(var(--er) / 0.05)}
.focus\:border-b-error\/50:focus {
    border-bottom-color: hsl(var(--er) / 0.5)}
.focus\:border-b-error\/60:focus {
    border-bottom-color: hsl(var(--er) / 0.6)}
.focus\:border-b-error\/70:focus {
    border-bottom-color: hsl(var(--er) / 0.7)}
.focus\:border-b-error\/75:focus {
    border-bottom-color: hsl(var(--er) / 0.75)}
.focus\:border-b-error\/80:focus {
    border-bottom-color: hsl(var(--er) / 0.8)}
.focus\:border-b-error\/90:focus {
    border-bottom-color: hsl(var(--er) / 0.9)}
.focus\:border-b-error\/95:focus {
    border-bottom-color: hsl(var(--er) / 0.95)}
.focus\:border-b-info:focus {
    border-bottom-color: hsl(var(--in) / 1)}
.focus\:border-b-info-content:focus {
    border-bottom-color: hsl(var(--inc) / 1)}
.focus\:border-b-info-content\/0:focus {
    border-bottom-color: hsl(var(--inc) / 0)}
.focus\:border-b-info-content\/10:focus {
    border-bottom-color: hsl(var(--inc) / 0.1)}
.focus\:border-b-info-content\/100:focus {
    border-bottom-color: hsl(var(--inc) / 1)}
.focus\:border-b-info-content\/20:focus {
    border-bottom-color: hsl(var(--inc) / 0.2)}
.focus\:border-b-info-content\/25:focus {
    border-bottom-color: hsl(var(--inc) / 0.25)}
.focus\:border-b-info-content\/30:focus {
    border-bottom-color: hsl(var(--inc) / 0.3)}
.focus\:border-b-info-content\/40:focus {
    border-bottom-color: hsl(var(--inc) / 0.4)}
.focus\:border-b-info-content\/5:focus {
    border-bottom-color: hsl(var(--inc) / 0.05)}
.focus\:border-b-info-content\/50:focus {
    border-bottom-color: hsl(var(--inc) / 0.5)}
.focus\:border-b-info-content\/60:focus {
    border-bottom-color: hsl(var(--inc) / 0.6)}
.focus\:border-b-info-content\/70:focus {
    border-bottom-color: hsl(var(--inc) / 0.7)}
.focus\:border-b-info-content\/75:focus {
    border-bottom-color: hsl(var(--inc) / 0.75)}
.focus\:border-b-info-content\/80:focus {
    border-bottom-color: hsl(var(--inc) / 0.8)}
.focus\:border-b-info-content\/90:focus {
    border-bottom-color: hsl(var(--inc) / 0.9)}
.focus\:border-b-info-content\/95:focus {
    border-bottom-color: hsl(var(--inc) / 0.95)}
.focus\:border-b-info\/0:focus {
    border-bottom-color: hsl(var(--in) / 0)}
.focus\:border-b-info\/10:focus {
    border-bottom-color: hsl(var(--in) / 0.1)}
.focus\:border-b-info\/100:focus {
    border-bottom-color: hsl(var(--in) / 1)}
.focus\:border-b-info\/20:focus {
    border-bottom-color: hsl(var(--in) / 0.2)}
.focus\:border-b-info\/25:focus {
    border-bottom-color: hsl(var(--in) / 0.25)}
.focus\:border-b-info\/30:focus {
    border-bottom-color: hsl(var(--in) / 0.3)}
.focus\:border-b-info\/40:focus {
    border-bottom-color: hsl(var(--in) / 0.4)}
.focus\:border-b-info\/5:focus {
    border-bottom-color: hsl(var(--in) / 0.05)}
.focus\:border-b-info\/50:focus {
    border-bottom-color: hsl(var(--in) / 0.5)}
.focus\:border-b-info\/60:focus {
    border-bottom-color: hsl(var(--in) / 0.6)}
.focus\:border-b-info\/70:focus {
    border-bottom-color: hsl(var(--in) / 0.7)}
.focus\:border-b-info\/75:focus {
    border-bottom-color: hsl(var(--in) / 0.75)}
.focus\:border-b-info\/80:focus {
    border-bottom-color: hsl(var(--in) / 0.8)}
.focus\:border-b-info\/90:focus {
    border-bottom-color: hsl(var(--in) / 0.9)}
.focus\:border-b-info\/95:focus {
    border-bottom-color: hsl(var(--in) / 0.95)}
.focus\:border-b-success:focus {
    border-bottom-color: hsl(var(--su) / 1)}
.focus\:border-b-success-content:focus {
    border-bottom-color: hsl(var(--suc) / 1)}
.focus\:border-b-success-content\/0:focus {
    border-bottom-color: hsl(var(--suc) / 0)}
.focus\:border-b-success-content\/10:focus {
    border-bottom-color: hsl(var(--suc) / 0.1)}
.focus\:border-b-success-content\/100:focus {
    border-bottom-color: hsl(var(--suc) / 1)}
.focus\:border-b-success-content\/20:focus {
    border-bottom-color: hsl(var(--suc) / 0.2)}
.focus\:border-b-success-content\/25:focus {
    border-bottom-color: hsl(var(--suc) / 0.25)}
.focus\:border-b-success-content\/30:focus {
    border-bottom-color: hsl(var(--suc) / 0.3)}
.focus\:border-b-success-content\/40:focus {
    border-bottom-color: hsl(var(--suc) / 0.4)}
.focus\:border-b-success-content\/5:focus {
    border-bottom-color: hsl(var(--suc) / 0.05)}
.focus\:border-b-success-content\/50:focus {
    border-bottom-color: hsl(var(--suc) / 0.5)}
.focus\:border-b-success-content\/60:focus {
    border-bottom-color: hsl(var(--suc) / 0.6)}
.focus\:border-b-success-content\/70:focus {
    border-bottom-color: hsl(var(--suc) / 0.7)}
.focus\:border-b-success-content\/75:focus {
    border-bottom-color: hsl(var(--suc) / 0.75)}
.focus\:border-b-success-content\/80:focus {
    border-bottom-color: hsl(var(--suc) / 0.8)}
.focus\:border-b-success-content\/90:focus {
    border-bottom-color: hsl(var(--suc) / 0.9)}
.focus\:border-b-success-content\/95:focus {
    border-bottom-color: hsl(var(--suc) / 0.95)}
.focus\:border-b-success\/0:focus {
    border-bottom-color: hsl(var(--su) / 0)}
.focus\:border-b-success\/10:focus {
    border-bottom-color: hsl(var(--su) / 0.1)}
.focus\:border-b-success\/100:focus {
    border-bottom-color: hsl(var(--su) / 1)}
.focus\:border-b-success\/20:focus {
    border-bottom-color: hsl(var(--su) / 0.2)}
.focus\:border-b-success\/25:focus {
    border-bottom-color: hsl(var(--su) / 0.25)}
.focus\:border-b-success\/30:focus {
    border-bottom-color: hsl(var(--su) / 0.3)}
.focus\:border-b-success\/40:focus {
    border-bottom-color: hsl(var(--su) / 0.4)}
.focus\:border-b-success\/5:focus {
    border-bottom-color: hsl(var(--su) / 0.05)}
.focus\:border-b-success\/50:focus {
    border-bottom-color: hsl(var(--su) / 0.5)}
.focus\:border-b-success\/60:focus {
    border-bottom-color: hsl(var(--su) / 0.6)}
.focus\:border-b-success\/70:focus {
    border-bottom-color: hsl(var(--su) / 0.7)}
.focus\:border-b-success\/75:focus {
    border-bottom-color: hsl(var(--su) / 0.75)}
.focus\:border-b-success\/80:focus {
    border-bottom-color: hsl(var(--su) / 0.8)}
.focus\:border-b-success\/90:focus {
    border-bottom-color: hsl(var(--su) / 0.9)}
.focus\:border-b-success\/95:focus {
    border-bottom-color: hsl(var(--su) / 0.95)}
.focus\:border-b-warning:focus {
    border-bottom-color: hsl(var(--wa) / 1)}
.focus\:border-b-warning-content:focus {
    border-bottom-color: hsl(var(--wac) / 1)}
.focus\:border-b-warning-content\/0:focus {
    border-bottom-color: hsl(var(--wac) / 0)}
.focus\:border-b-warning-content\/10:focus {
    border-bottom-color: hsl(var(--wac) / 0.1)}
.focus\:border-b-warning-content\/100:focus {
    border-bottom-color: hsl(var(--wac) / 1)}
.focus\:border-b-warning-content\/20:focus {
    border-bottom-color: hsl(var(--wac) / 0.2)}
.focus\:border-b-warning-content\/25:focus {
    border-bottom-color: hsl(var(--wac) / 0.25)}
.focus\:border-b-warning-content\/30:focus {
    border-bottom-color: hsl(var(--wac) / 0.3)}
.focus\:border-b-warning-content\/40:focus {
    border-bottom-color: hsl(var(--wac) / 0.4)}
.focus\:border-b-warning-content\/5:focus {
    border-bottom-color: hsl(var(--wac) / 0.05)}
.focus\:border-b-warning-content\/50:focus {
    border-bottom-color: hsl(var(--wac) / 0.5)}
.focus\:border-b-warning-content\/60:focus {
    border-bottom-color: hsl(var(--wac) / 0.6)}
.focus\:border-b-warning-content\/70:focus {
    border-bottom-color: hsl(var(--wac) / 0.7)}
.focus\:border-b-warning-content\/75:focus {
    border-bottom-color: hsl(var(--wac) / 0.75)}
.focus\:border-b-warning-content\/80:focus {
    border-bottom-color: hsl(var(--wac) / 0.8)}
.focus\:border-b-warning-content\/90:focus {
    border-bottom-color: hsl(var(--wac) / 0.9)}
.focus\:border-b-warning-content\/95:focus {
    border-bottom-color: hsl(var(--wac) / 0.95)}
.focus\:border-b-warning\/0:focus {
    border-bottom-color: hsl(var(--wa) / 0)}
.focus\:border-b-warning\/10:focus {
    border-bottom-color: hsl(var(--wa) / 0.1)}
.focus\:border-b-warning\/100:focus {
    border-bottom-color: hsl(var(--wa) / 1)}
.focus\:border-b-warning\/20:focus {
    border-bottom-color: hsl(var(--wa) / 0.2)}
.focus\:border-b-warning\/25:focus {
    border-bottom-color: hsl(var(--wa) / 0.25)}
.focus\:border-b-warning\/30:focus {
    border-bottom-color: hsl(var(--wa) / 0.3)}
.focus\:border-b-warning\/40:focus {
    border-bottom-color: hsl(var(--wa) / 0.4)}
.focus\:border-b-warning\/5:focus {
    border-bottom-color: hsl(var(--wa) / 0.05)}
.focus\:border-b-warning\/50:focus {
    border-bottom-color: hsl(var(--wa) / 0.5)}
.focus\:border-b-warning\/60:focus {
    border-bottom-color: hsl(var(--wa) / 0.6)}
.focus\:border-b-warning\/70:focus {
    border-bottom-color: hsl(var(--wa) / 0.7)}
.focus\:border-b-warning\/75:focus {
    border-bottom-color: hsl(var(--wa) / 0.75)}
.focus\:border-b-warning\/80:focus {
    border-bottom-color: hsl(var(--wa) / 0.8)}
.focus\:border-b-warning\/90:focus {
    border-bottom-color: hsl(var(--wa) / 0.9)}
.focus\:border-b-warning\/95:focus {
    border-bottom-color: hsl(var(--wa) / 0.95)}
.focus\:border-e-base-100:focus {
    border-inline-end-color: hsl(var(--b1) / 1)}
.focus\:border-e-base-100\/0:focus {
    border-inline-end-color: hsl(var(--b1) / 0)}
.focus\:border-e-base-100\/10:focus {
    border-inline-end-color: hsl(var(--b1) / 0.1)}
.focus\:border-e-base-100\/100:focus {
    border-inline-end-color: hsl(var(--b1) / 1)}
.focus\:border-e-base-100\/20:focus {
    border-inline-end-color: hsl(var(--b1) / 0.2)}
.focus\:border-e-base-100\/25:focus {
    border-inline-end-color: hsl(var(--b1) / 0.25)}
.focus\:border-e-base-100\/30:focus {
    border-inline-end-color: hsl(var(--b1) / 0.3)}
.focus\:border-e-base-100\/40:focus {
    border-inline-end-color: hsl(var(--b1) / 0.4)}
.focus\:border-e-base-100\/5:focus {
    border-inline-end-color: hsl(var(--b1) / 0.05)}
.focus\:border-e-base-100\/50:focus {
    border-inline-end-color: hsl(var(--b1) / 0.5)}
.focus\:border-e-base-100\/60:focus {
    border-inline-end-color: hsl(var(--b1) / 0.6)}
.focus\:border-e-base-100\/70:focus {
    border-inline-end-color: hsl(var(--b1) / 0.7)}
.focus\:border-e-base-100\/75:focus {
    border-inline-end-color: hsl(var(--b1) / 0.75)}
.focus\:border-e-base-100\/80:focus {
    border-inline-end-color: hsl(var(--b1) / 0.8)}
.focus\:border-e-base-100\/90:focus {
    border-inline-end-color: hsl(var(--b1) / 0.9)}
.focus\:border-e-base-100\/95:focus {
    border-inline-end-color: hsl(var(--b1) / 0.95)}
.focus\:border-e-base-200:focus {
    border-inline-end-color: hsl(var(--b2) / 1)}
.focus\:border-e-base-200\/0:focus {
    border-inline-end-color: hsl(var(--b2) / 0)}
.focus\:border-e-base-200\/10:focus {
    border-inline-end-color: hsl(var(--b2) / 0.1)}
.focus\:border-e-base-200\/100:focus {
    border-inline-end-color: hsl(var(--b2) / 1)}
.focus\:border-e-base-200\/20:focus {
    border-inline-end-color: hsl(var(--b2) / 0.2)}
.focus\:border-e-base-200\/25:focus {
    border-inline-end-color: hsl(var(--b2) / 0.25)}
.focus\:border-e-base-200\/30:focus {
    border-inline-end-color: hsl(var(--b2) / 0.3)}
.focus\:border-e-base-200\/40:focus {
    border-inline-end-color: hsl(var(--b2) / 0.4)}
.focus\:border-e-base-200\/5:focus {
    border-inline-end-color: hsl(var(--b2) / 0.05)}
.focus\:border-e-base-200\/50:focus {
    border-inline-end-color: hsl(var(--b2) / 0.5)}
.focus\:border-e-base-200\/60:focus {
    border-inline-end-color: hsl(var(--b2) / 0.6)}
.focus\:border-e-base-200\/70:focus {
    border-inline-end-color: hsl(var(--b2) / 0.7)}
.focus\:border-e-base-200\/75:focus {
    border-inline-end-color: hsl(var(--b2) / 0.75)}
.focus\:border-e-base-200\/80:focus {
    border-inline-end-color: hsl(var(--b2) / 0.8)}
.focus\:border-e-base-200\/90:focus {
    border-inline-end-color: hsl(var(--b2) / 0.9)}
.focus\:border-e-base-200\/95:focus {
    border-inline-end-color: hsl(var(--b2) / 0.95)}
.focus\:border-e-base-300:focus {
    border-inline-end-color: hsl(var(--b3) / 1)}
.focus\:border-e-base-300\/0:focus {
    border-inline-end-color: hsl(var(--b3) / 0)}
.focus\:border-e-base-300\/10:focus {
    border-inline-end-color: hsl(var(--b3) / 0.1)}
.focus\:border-e-base-300\/100:focus {
    border-inline-end-color: hsl(var(--b3) / 1)}
.focus\:border-e-base-300\/20:focus {
    border-inline-end-color: hsl(var(--b3) / 0.2)}
.focus\:border-e-base-300\/25:focus {
    border-inline-end-color: hsl(var(--b3) / 0.25)}
.focus\:border-e-base-300\/30:focus {
    border-inline-end-color: hsl(var(--b3) / 0.3)}
.focus\:border-e-base-300\/40:focus {
    border-inline-end-color: hsl(var(--b3) / 0.4)}
.focus\:border-e-base-300\/5:focus {
    border-inline-end-color: hsl(var(--b3) / 0.05)}
.focus\:border-e-base-300\/50:focus {
    border-inline-end-color: hsl(var(--b3) / 0.5)}
.focus\:border-e-base-300\/60:focus {
    border-inline-end-color: hsl(var(--b3) / 0.6)}
.focus\:border-e-base-300\/70:focus {
    border-inline-end-color: hsl(var(--b3) / 0.7)}
.focus\:border-e-base-300\/75:focus {
    border-inline-end-color: hsl(var(--b3) / 0.75)}
.focus\:border-e-base-300\/80:focus {
    border-inline-end-color: hsl(var(--b3) / 0.8)}
.focus\:border-e-base-300\/90:focus {
    border-inline-end-color: hsl(var(--b3) / 0.9)}
.focus\:border-e-base-300\/95:focus {
    border-inline-end-color: hsl(var(--b3) / 0.95)}
.focus\:border-e-base-content:focus {
    border-inline-end-color: hsl(var(--bc) / 1)}
.focus\:border-e-base-content\/0:focus {
    border-inline-end-color: hsl(var(--bc) / 0)}
.focus\:border-e-base-content\/10:focus {
    border-inline-end-color: hsl(var(--bc) / 0.1)}
.focus\:border-e-base-content\/100:focus {
    border-inline-end-color: hsl(var(--bc) / 1)}
.focus\:border-e-base-content\/20:focus {
    border-inline-end-color: hsl(var(--bc) / 0.2)}
.focus\:border-e-base-content\/25:focus {
    border-inline-end-color: hsl(var(--bc) / 0.25)}
.focus\:border-e-base-content\/30:focus {
    border-inline-end-color: hsl(var(--bc) / 0.3)}
.focus\:border-e-base-content\/40:focus {
    border-inline-end-color: hsl(var(--bc) / 0.4)}
.focus\:border-e-base-content\/5:focus {
    border-inline-end-color: hsl(var(--bc) / 0.05)}
.focus\:border-e-base-content\/50:focus {
    border-inline-end-color: hsl(var(--bc) / 0.5)}
.focus\:border-e-base-content\/60:focus {
    border-inline-end-color: hsl(var(--bc) / 0.6)}
.focus\:border-e-base-content\/70:focus {
    border-inline-end-color: hsl(var(--bc) / 0.7)}
.focus\:border-e-base-content\/75:focus {
    border-inline-end-color: hsl(var(--bc) / 0.75)}
.focus\:border-e-base-content\/80:focus {
    border-inline-end-color: hsl(var(--bc) / 0.8)}
.focus\:border-e-base-content\/90:focus {
    border-inline-end-color: hsl(var(--bc) / 0.9)}
.focus\:border-e-base-content\/95:focus {
    border-inline-end-color: hsl(var(--bc) / 0.95)}
.focus\:border-e-error:focus {
    border-inline-end-color: hsl(var(--er) / 1)}
.focus\:border-e-error-content:focus {
    border-inline-end-color: hsl(var(--erc) / 1)}
.focus\:border-e-error-content\/0:focus {
    border-inline-end-color: hsl(var(--erc) / 0)}
.focus\:border-e-error-content\/10:focus {
    border-inline-end-color: hsl(var(--erc) / 0.1)}
.focus\:border-e-error-content\/100:focus {
    border-inline-end-color: hsl(var(--erc) / 1)}
.focus\:border-e-error-content\/20:focus {
    border-inline-end-color: hsl(var(--erc) / 0.2)}
.focus\:border-e-error-content\/25:focus {
    border-inline-end-color: hsl(var(--erc) / 0.25)}
.focus\:border-e-error-content\/30:focus {
    border-inline-end-color: hsl(var(--erc) / 0.3)}
.focus\:border-e-error-content\/40:focus {
    border-inline-end-color: hsl(var(--erc) / 0.4)}
.focus\:border-e-error-content\/5:focus {
    border-inline-end-color: hsl(var(--erc) / 0.05)}
.focus\:border-e-error-content\/50:focus {
    border-inline-end-color: hsl(var(--erc) / 0.5)}
.focus\:border-e-error-content\/60:focus {
    border-inline-end-color: hsl(var(--erc) / 0.6)}
.focus\:border-e-error-content\/70:focus {
    border-inline-end-color: hsl(var(--erc) / 0.7)}
.focus\:border-e-error-content\/75:focus {
    border-inline-end-color: hsl(var(--erc) / 0.75)}
.focus\:border-e-error-content\/80:focus {
    border-inline-end-color: hsl(var(--erc) / 0.8)}
.focus\:border-e-error-content\/90:focus {
    border-inline-end-color: hsl(var(--erc) / 0.9)}
.focus\:border-e-error-content\/95:focus {
    border-inline-end-color: hsl(var(--erc) / 0.95)}
.focus\:border-e-error\/0:focus {
    border-inline-end-color: hsl(var(--er) / 0)}
.focus\:border-e-error\/10:focus {
    border-inline-end-color: hsl(var(--er) / 0.1)}
.focus\:border-e-error\/100:focus {
    border-inline-end-color: hsl(var(--er) / 1)}
.focus\:border-e-error\/20:focus {
    border-inline-end-color: hsl(var(--er) / 0.2)}
.focus\:border-e-error\/25:focus {
    border-inline-end-color: hsl(var(--er) / 0.25)}
.focus\:border-e-error\/30:focus {
    border-inline-end-color: hsl(var(--er) / 0.3)}
.focus\:border-e-error\/40:focus {
    border-inline-end-color: hsl(var(--er) / 0.4)}
.focus\:border-e-error\/5:focus {
    border-inline-end-color: hsl(var(--er) / 0.05)}
.focus\:border-e-error\/50:focus {
    border-inline-end-color: hsl(var(--er) / 0.5)}
.focus\:border-e-error\/60:focus {
    border-inline-end-color: hsl(var(--er) / 0.6)}
.focus\:border-e-error\/70:focus {
    border-inline-end-color: hsl(var(--er) / 0.7)}
.focus\:border-e-error\/75:focus {
    border-inline-end-color: hsl(var(--er) / 0.75)}
.focus\:border-e-error\/80:focus {
    border-inline-end-color: hsl(var(--er) / 0.8)}
.focus\:border-e-error\/90:focus {
    border-inline-end-color: hsl(var(--er) / 0.9)}
.focus\:border-e-error\/95:focus {
    border-inline-end-color: hsl(var(--er) / 0.95)}
.focus\:border-e-info:focus {
    border-inline-end-color: hsl(var(--in) / 1)}
.focus\:border-e-info-content:focus {
    border-inline-end-color: hsl(var(--inc) / 1)}
.focus\:border-e-info-content\/0:focus {
    border-inline-end-color: hsl(var(--inc) / 0)}
.focus\:border-e-info-content\/10:focus {
    border-inline-end-color: hsl(var(--inc) / 0.1)}
.focus\:border-e-info-content\/100:focus {
    border-inline-end-color: hsl(var(--inc) / 1)}
.focus\:border-e-info-content\/20:focus {
    border-inline-end-color: hsl(var(--inc) / 0.2)}
.focus\:border-e-info-content\/25:focus {
    border-inline-end-color: hsl(var(--inc) / 0.25)}
.focus\:border-e-info-content\/30:focus {
    border-inline-end-color: hsl(var(--inc) / 0.3)}
.focus\:border-e-info-content\/40:focus {
    border-inline-end-color: hsl(var(--inc) / 0.4)}
.focus\:border-e-info-content\/5:focus {
    border-inline-end-color: hsl(var(--inc) / 0.05)}
.focus\:border-e-info-content\/50:focus {
    border-inline-end-color: hsl(var(--inc) / 0.5)}
.focus\:border-e-info-content\/60:focus {
    border-inline-end-color: hsl(var(--inc) / 0.6)}
.focus\:border-e-info-content\/70:focus {
    border-inline-end-color: hsl(var(--inc) / 0.7)}
.focus\:border-e-info-content\/75:focus {
    border-inline-end-color: hsl(var(--inc) / 0.75)}
.focus\:border-e-info-content\/80:focus {
    border-inline-end-color: hsl(var(--inc) / 0.8)}
.focus\:border-e-info-content\/90:focus {
    border-inline-end-color: hsl(var(--inc) / 0.9)}
.focus\:border-e-info-content\/95:focus {
    border-inline-end-color: hsl(var(--inc) / 0.95)}
.focus\:border-e-info\/0:focus {
    border-inline-end-color: hsl(var(--in) / 0)}
.focus\:border-e-info\/10:focus {
    border-inline-end-color: hsl(var(--in) / 0.1)}
.focus\:border-e-info\/100:focus {
    border-inline-end-color: hsl(var(--in) / 1)}
.focus\:border-e-info\/20:focus {
    border-inline-end-color: hsl(var(--in) / 0.2)}
.focus\:border-e-info\/25:focus {
    border-inline-end-color: hsl(var(--in) / 0.25)}
.focus\:border-e-info\/30:focus {
    border-inline-end-color: hsl(var(--in) / 0.3)}
.focus\:border-e-info\/40:focus {
    border-inline-end-color: hsl(var(--in) / 0.4)}
.focus\:border-e-info\/5:focus {
    border-inline-end-color: hsl(var(--in) / 0.05)}
.focus\:border-e-info\/50:focus {
    border-inline-end-color: hsl(var(--in) / 0.5)}
.focus\:border-e-info\/60:focus {
    border-inline-end-color: hsl(var(--in) / 0.6)}
.focus\:border-e-info\/70:focus {
    border-inline-end-color: hsl(var(--in) / 0.7)}
.focus\:border-e-info\/75:focus {
    border-inline-end-color: hsl(var(--in) / 0.75)}
.focus\:border-e-info\/80:focus {
    border-inline-end-color: hsl(var(--in) / 0.8)}
.focus\:border-e-info\/90:focus {
    border-inline-end-color: hsl(var(--in) / 0.9)}
.focus\:border-e-info\/95:focus {
    border-inline-end-color: hsl(var(--in) / 0.95)}
.focus\:border-e-success:focus {
    border-inline-end-color: hsl(var(--su) / 1)}
.focus\:border-e-success-content:focus {
    border-inline-end-color: hsl(var(--suc) / 1)}
.focus\:border-e-success-content\/0:focus {
    border-inline-end-color: hsl(var(--suc) / 0)}
.focus\:border-e-success-content\/10:focus {
    border-inline-end-color: hsl(var(--suc) / 0.1)}
.focus\:border-e-success-content\/100:focus {
    border-inline-end-color: hsl(var(--suc) / 1)}
.focus\:border-e-success-content\/20:focus {
    border-inline-end-color: hsl(var(--suc) / 0.2)}
.focus\:border-e-success-content\/25:focus {
    border-inline-end-color: hsl(var(--suc) / 0.25)}
.focus\:border-e-success-content\/30:focus {
    border-inline-end-color: hsl(var(--suc) / 0.3)}
.focus\:border-e-success-content\/40:focus {
    border-inline-end-color: hsl(var(--suc) / 0.4)}
.focus\:border-e-success-content\/5:focus {
    border-inline-end-color: hsl(var(--suc) / 0.05)}
.focus\:border-e-success-content\/50:focus {
    border-inline-end-color: hsl(var(--suc) / 0.5)}
.focus\:border-e-success-content\/60:focus {
    border-inline-end-color: hsl(var(--suc) / 0.6)}
.focus\:border-e-success-content\/70:focus {
    border-inline-end-color: hsl(var(--suc) / 0.7)}
.focus\:border-e-success-content\/75:focus {
    border-inline-end-color: hsl(var(--suc) / 0.75)}
.focus\:border-e-success-content\/80:focus {
    border-inline-end-color: hsl(var(--suc) / 0.8)}
.focus\:border-e-success-content\/90:focus {
    border-inline-end-color: hsl(var(--suc) / 0.9)}
.focus\:border-e-success-content\/95:focus {
    border-inline-end-color: hsl(var(--suc) / 0.95)}
.focus\:border-e-success\/0:focus {
    border-inline-end-color: hsl(var(--su) / 0)}
.focus\:border-e-success\/10:focus {
    border-inline-end-color: hsl(var(--su) / 0.1)}
.focus\:border-e-success\/100:focus {
    border-inline-end-color: hsl(var(--su) / 1)}
.focus\:border-e-success\/20:focus {
    border-inline-end-color: hsl(var(--su) / 0.2)}
.focus\:border-e-success\/25:focus {
    border-inline-end-color: hsl(var(--su) / 0.25)}
.focus\:border-e-success\/30:focus {
    border-inline-end-color: hsl(var(--su) / 0.3)}
.focus\:border-e-success\/40:focus {
    border-inline-end-color: hsl(var(--su) / 0.4)}
.focus\:border-e-success\/5:focus {
    border-inline-end-color: hsl(var(--su) / 0.05)}
.focus\:border-e-success\/50:focus {
    border-inline-end-color: hsl(var(--su) / 0.5)}
.focus\:border-e-success\/60:focus {
    border-inline-end-color: hsl(var(--su) / 0.6)}
.focus\:border-e-success\/70:focus {
    border-inline-end-color: hsl(var(--su) / 0.7)}
.focus\:border-e-success\/75:focus {
    border-inline-end-color: hsl(var(--su) / 0.75)}
.focus\:border-e-success\/80:focus {
    border-inline-end-color: hsl(var(--su) / 0.8)}
.focus\:border-e-success\/90:focus {
    border-inline-end-color: hsl(var(--su) / 0.9)}
.focus\:border-e-success\/95:focus {
    border-inline-end-color: hsl(var(--su) / 0.95)}
.focus\:border-e-warning:focus {
    border-inline-end-color: hsl(var(--wa) / 1)}
.focus\:border-e-warning-content:focus {
    border-inline-end-color: hsl(var(--wac) / 1)}
.focus\:border-e-warning-content\/0:focus {
    border-inline-end-color: hsl(var(--wac) / 0)}
.focus\:border-e-warning-content\/10:focus {
    border-inline-end-color: hsl(var(--wac) / 0.1)}
.focus\:border-e-warning-content\/100:focus {
    border-inline-end-color: hsl(var(--wac) / 1)}
.focus\:border-e-warning-content\/20:focus {
    border-inline-end-color: hsl(var(--wac) / 0.2)}
.focus\:border-e-warning-content\/25:focus {
    border-inline-end-color: hsl(var(--wac) / 0.25)}
.focus\:border-e-warning-content\/30:focus {
    border-inline-end-color: hsl(var(--wac) / 0.3)}
.focus\:border-e-warning-content\/40:focus {
    border-inline-end-color: hsl(var(--wac) / 0.4)}
.focus\:border-e-warning-content\/5:focus {
    border-inline-end-color: hsl(var(--wac) / 0.05)}
.focus\:border-e-warning-content\/50:focus {
    border-inline-end-color: hsl(var(--wac) / 0.5)}
.focus\:border-e-warning-content\/60:focus {
    border-inline-end-color: hsl(var(--wac) / 0.6)}
.focus\:border-e-warning-content\/70:focus {
    border-inline-end-color: hsl(var(--wac) / 0.7)}
.focus\:border-e-warning-content\/75:focus {
    border-inline-end-color: hsl(var(--wac) / 0.75)}
.focus\:border-e-warning-content\/80:focus {
    border-inline-end-color: hsl(var(--wac) / 0.8)}
.focus\:border-e-warning-content\/90:focus {
    border-inline-end-color: hsl(var(--wac) / 0.9)}
.focus\:border-e-warning-content\/95:focus {
    border-inline-end-color: hsl(var(--wac) / 0.95)}
.focus\:border-e-warning\/0:focus {
    border-inline-end-color: hsl(var(--wa) / 0)}
.focus\:border-e-warning\/10:focus {
    border-inline-end-color: hsl(var(--wa) / 0.1)}
.focus\:border-e-warning\/100:focus {
    border-inline-end-color: hsl(var(--wa) / 1)}
.focus\:border-e-warning\/20:focus {
    border-inline-end-color: hsl(var(--wa) / 0.2)}
.focus\:border-e-warning\/25:focus {
    border-inline-end-color: hsl(var(--wa) / 0.25)}
.focus\:border-e-warning\/30:focus {
    border-inline-end-color: hsl(var(--wa) / 0.3)}
.focus\:border-e-warning\/40:focus {
    border-inline-end-color: hsl(var(--wa) / 0.4)}
.focus\:border-e-warning\/5:focus {
    border-inline-end-color: hsl(var(--wa) / 0.05)}
.focus\:border-e-warning\/50:focus {
    border-inline-end-color: hsl(var(--wa) / 0.5)}
.focus\:border-e-warning\/60:focus {
    border-inline-end-color: hsl(var(--wa) / 0.6)}
.focus\:border-e-warning\/70:focus {
    border-inline-end-color: hsl(var(--wa) / 0.7)}
.focus\:border-e-warning\/75:focus {
    border-inline-end-color: hsl(var(--wa) / 0.75)}
.focus\:border-e-warning\/80:focus {
    border-inline-end-color: hsl(var(--wa) / 0.8)}
.focus\:border-e-warning\/90:focus {
    border-inline-end-color: hsl(var(--wa) / 0.9)}
.focus\:border-e-warning\/95:focus {
    border-inline-end-color: hsl(var(--wa) / 0.95)}
.focus\:border-l-base-100:focus {
    border-left-color: hsl(var(--b1) / 1)}
.focus\:border-l-base-100\/0:focus {
    border-left-color: hsl(var(--b1) / 0)}
.focus\:border-l-base-100\/10:focus {
    border-left-color: hsl(var(--b1) / 0.1)}
.focus\:border-l-base-100\/100:focus {
    border-left-color: hsl(var(--b1) / 1)}
.focus\:border-l-base-100\/20:focus {
    border-left-color: hsl(var(--b1) / 0.2)}
.focus\:border-l-base-100\/25:focus {
    border-left-color: hsl(var(--b1) / 0.25)}
.focus\:border-l-base-100\/30:focus {
    border-left-color: hsl(var(--b1) / 0.3)}
.focus\:border-l-base-100\/40:focus {
    border-left-color: hsl(var(--b1) / 0.4)}
.focus\:border-l-base-100\/5:focus {
    border-left-color: hsl(var(--b1) / 0.05)}
.focus\:border-l-base-100\/50:focus {
    border-left-color: hsl(var(--b1) / 0.5)}
.focus\:border-l-base-100\/60:focus {
    border-left-color: hsl(var(--b1) / 0.6)}
.focus\:border-l-base-100\/70:focus {
    border-left-color: hsl(var(--b1) / 0.7)}
.focus\:border-l-base-100\/75:focus {
    border-left-color: hsl(var(--b1) / 0.75)}
.focus\:border-l-base-100\/80:focus {
    border-left-color: hsl(var(--b1) / 0.8)}
.focus\:border-l-base-100\/90:focus {
    border-left-color: hsl(var(--b1) / 0.9)}
.focus\:border-l-base-100\/95:focus {
    border-left-color: hsl(var(--b1) / 0.95)}
.focus\:border-l-base-200:focus {
    border-left-color: hsl(var(--b2) / 1)}
.focus\:border-l-base-200\/0:focus {
    border-left-color: hsl(var(--b2) / 0)}
.focus\:border-l-base-200\/10:focus {
    border-left-color: hsl(var(--b2) / 0.1)}
.focus\:border-l-base-200\/100:focus {
    border-left-color: hsl(var(--b2) / 1)}
.focus\:border-l-base-200\/20:focus {
    border-left-color: hsl(var(--b2) / 0.2)}
.focus\:border-l-base-200\/25:focus {
    border-left-color: hsl(var(--b2) / 0.25)}
.focus\:border-l-base-200\/30:focus {
    border-left-color: hsl(var(--b2) / 0.3)}
.focus\:border-l-base-200\/40:focus {
    border-left-color: hsl(var(--b2) / 0.4)}
.focus\:border-l-base-200\/5:focus {
    border-left-color: hsl(var(--b2) / 0.05)}
.focus\:border-l-base-200\/50:focus {
    border-left-color: hsl(var(--b2) / 0.5)}
.focus\:border-l-base-200\/60:focus {
    border-left-color: hsl(var(--b2) / 0.6)}
.focus\:border-l-base-200\/70:focus {
    border-left-color: hsl(var(--b2) / 0.7)}
.focus\:border-l-base-200\/75:focus {
    border-left-color: hsl(var(--b2) / 0.75)}
.focus\:border-l-base-200\/80:focus {
    border-left-color: hsl(var(--b2) / 0.8)}
.focus\:border-l-base-200\/90:focus {
    border-left-color: hsl(var(--b2) / 0.9)}
.focus\:border-l-base-200\/95:focus {
    border-left-color: hsl(var(--b2) / 0.95)}
.focus\:border-l-base-300:focus {
    border-left-color: hsl(var(--b3) / 1)}
.focus\:border-l-base-300\/0:focus {
    border-left-color: hsl(var(--b3) / 0)}
.focus\:border-l-base-300\/10:focus {
    border-left-color: hsl(var(--b3) / 0.1)}
.focus\:border-l-base-300\/100:focus {
    border-left-color: hsl(var(--b3) / 1)}
.focus\:border-l-base-300\/20:focus {
    border-left-color: hsl(var(--b3) / 0.2)}
.focus\:border-l-base-300\/25:focus {
    border-left-color: hsl(var(--b3) / 0.25)}
.focus\:border-l-base-300\/30:focus {
    border-left-color: hsl(var(--b3) / 0.3)}
.focus\:border-l-base-300\/40:focus {
    border-left-color: hsl(var(--b3) / 0.4)}
.focus\:border-l-base-300\/5:focus {
    border-left-color: hsl(var(--b3) / 0.05)}
.focus\:border-l-base-300\/50:focus {
    border-left-color: hsl(var(--b3) / 0.5)}
.focus\:border-l-base-300\/60:focus {
    border-left-color: hsl(var(--b3) / 0.6)}
.focus\:border-l-base-300\/70:focus {
    border-left-color: hsl(var(--b3) / 0.7)}
.focus\:border-l-base-300\/75:focus {
    border-left-color: hsl(var(--b3) / 0.75)}
.focus\:border-l-base-300\/80:focus {
    border-left-color: hsl(var(--b3) / 0.8)}
.focus\:border-l-base-300\/90:focus {
    border-left-color: hsl(var(--b3) / 0.9)}
.focus\:border-l-base-300\/95:focus {
    border-left-color: hsl(var(--b3) / 0.95)}
.focus\:border-l-base-content:focus {
    border-left-color: hsl(var(--bc) / 1)}
.focus\:border-l-base-content\/0:focus {
    border-left-color: hsl(var(--bc) / 0)}
.focus\:border-l-base-content\/10:focus {
    border-left-color: hsl(var(--bc) / 0.1)}
.focus\:border-l-base-content\/100:focus {
    border-left-color: hsl(var(--bc) / 1)}
.focus\:border-l-base-content\/20:focus {
    border-left-color: hsl(var(--bc) / 0.2)}
.focus\:border-l-base-content\/25:focus {
    border-left-color: hsl(var(--bc) / 0.25)}
.focus\:border-l-base-content\/30:focus {
    border-left-color: hsl(var(--bc) / 0.3)}
.focus\:border-l-base-content\/40:focus {
    border-left-color: hsl(var(--bc) / 0.4)}
.focus\:border-l-base-content\/5:focus {
    border-left-color: hsl(var(--bc) / 0.05)}
.focus\:border-l-base-content\/50:focus {
    border-left-color: hsl(var(--bc) / 0.5)}
.focus\:border-l-base-content\/60:focus {
    border-left-color: hsl(var(--bc) / 0.6)}
.focus\:border-l-base-content\/70:focus {
    border-left-color: hsl(var(--bc) / 0.7)}
.focus\:border-l-base-content\/75:focus {
    border-left-color: hsl(var(--bc) / 0.75)}
.focus\:border-l-base-content\/80:focus {
    border-left-color: hsl(var(--bc) / 0.8)}
.focus\:border-l-base-content\/90:focus {
    border-left-color: hsl(var(--bc) / 0.9)}
.focus\:border-l-base-content\/95:focus {
    border-left-color: hsl(var(--bc) / 0.95)}
.focus\:border-l-error:focus {
    border-left-color: hsl(var(--er) / 1)}
.focus\:border-l-error-content:focus {
    border-left-color: hsl(var(--erc) / 1)}
.focus\:border-l-error-content\/0:focus {
    border-left-color: hsl(var(--erc) / 0)}
.focus\:border-l-error-content\/10:focus {
    border-left-color: hsl(var(--erc) / 0.1)}
.focus\:border-l-error-content\/100:focus {
    border-left-color: hsl(var(--erc) / 1)}
.focus\:border-l-error-content\/20:focus {
    border-left-color: hsl(var(--erc) / 0.2)}
.focus\:border-l-error-content\/25:focus {
    border-left-color: hsl(var(--erc) / 0.25)}
.focus\:border-l-error-content\/30:focus {
    border-left-color: hsl(var(--erc) / 0.3)}
.focus\:border-l-error-content\/40:focus {
    border-left-color: hsl(var(--erc) / 0.4)}
.focus\:border-l-error-content\/5:focus {
    border-left-color: hsl(var(--erc) / 0.05)}
.focus\:border-l-error-content\/50:focus {
    border-left-color: hsl(var(--erc) / 0.5)}
.focus\:border-l-error-content\/60:focus {
    border-left-color: hsl(var(--erc) / 0.6)}
.focus\:border-l-error-content\/70:focus {
    border-left-color: hsl(var(--erc) / 0.7)}
.focus\:border-l-error-content\/75:focus {
    border-left-color: hsl(var(--erc) / 0.75)}
.focus\:border-l-error-content\/80:focus {
    border-left-color: hsl(var(--erc) / 0.8)}
.focus\:border-l-error-content\/90:focus {
    border-left-color: hsl(var(--erc) / 0.9)}
.focus\:border-l-error-content\/95:focus {
    border-left-color: hsl(var(--erc) / 0.95)}
.focus\:border-l-error\/0:focus {
    border-left-color: hsl(var(--er) / 0)}
.focus\:border-l-error\/10:focus {
    border-left-color: hsl(var(--er) / 0.1)}
.focus\:border-l-error\/100:focus {
    border-left-color: hsl(var(--er) / 1)}
.focus\:border-l-error\/20:focus {
    border-left-color: hsl(var(--er) / 0.2)}
.focus\:border-l-error\/25:focus {
    border-left-color: hsl(var(--er) / 0.25)}
.focus\:border-l-error\/30:focus {
    border-left-color: hsl(var(--er) / 0.3)}
.focus\:border-l-error\/40:focus {
    border-left-color: hsl(var(--er) / 0.4)}
.focus\:border-l-error\/5:focus {
    border-left-color: hsl(var(--er) / 0.05)}
.focus\:border-l-error\/50:focus {
    border-left-color: hsl(var(--er) / 0.5)}
.focus\:border-l-error\/60:focus {
    border-left-color: hsl(var(--er) / 0.6)}
.focus\:border-l-error\/70:focus {
    border-left-color: hsl(var(--er) / 0.7)}
.focus\:border-l-error\/75:focus {
    border-left-color: hsl(var(--er) / 0.75)}
.focus\:border-l-error\/80:focus {
    border-left-color: hsl(var(--er) / 0.8)}
.focus\:border-l-error\/90:focus {
    border-left-color: hsl(var(--er) / 0.9)}
.focus\:border-l-error\/95:focus {
    border-left-color: hsl(var(--er) / 0.95)}
.focus\:border-l-info:focus {
    border-left-color: hsl(var(--in) / 1)}
.focus\:border-l-info-content:focus {
    border-left-color: hsl(var(--inc) / 1)}
.focus\:border-l-info-content\/0:focus {
    border-left-color: hsl(var(--inc) / 0)}
.focus\:border-l-info-content\/10:focus {
    border-left-color: hsl(var(--inc) / 0.1)}
.focus\:border-l-info-content\/100:focus {
    border-left-color: hsl(var(--inc) / 1)}
.focus\:border-l-info-content\/20:focus {
    border-left-color: hsl(var(--inc) / 0.2)}
.focus\:border-l-info-content\/25:focus {
    border-left-color: hsl(var(--inc) / 0.25)}
.focus\:border-l-info-content\/30:focus {
    border-left-color: hsl(var(--inc) / 0.3)}
.focus\:border-l-info-content\/40:focus {
    border-left-color: hsl(var(--inc) / 0.4)}
.focus\:border-l-info-content\/5:focus {
    border-left-color: hsl(var(--inc) / 0.05)}
.focus\:border-l-info-content\/50:focus {
    border-left-color: hsl(var(--inc) / 0.5)}
.focus\:border-l-info-content\/60:focus {
    border-left-color: hsl(var(--inc) / 0.6)}
.focus\:border-l-info-content\/70:focus {
    border-left-color: hsl(var(--inc) / 0.7)}
.focus\:border-l-info-content\/75:focus {
    border-left-color: hsl(var(--inc) / 0.75)}
.focus\:border-l-info-content\/80:focus {
    border-left-color: hsl(var(--inc) / 0.8)}
.focus\:border-l-info-content\/90:focus {
    border-left-color: hsl(var(--inc) / 0.9)}
.focus\:border-l-info-content\/95:focus {
    border-left-color: hsl(var(--inc) / 0.95)}
.focus\:border-l-info\/0:focus {
    border-left-color: hsl(var(--in) / 0)}
.focus\:border-l-info\/10:focus {
    border-left-color: hsl(var(--in) / 0.1)}
.focus\:border-l-info\/100:focus {
    border-left-color: hsl(var(--in) / 1)}
.focus\:border-l-info\/20:focus {
    border-left-color: hsl(var(--in) / 0.2)}
.focus\:border-l-info\/25:focus {
    border-left-color: hsl(var(--in) / 0.25)}
.focus\:border-l-info\/30:focus {
    border-left-color: hsl(var(--in) / 0.3)}
.focus\:border-l-info\/40:focus {
    border-left-color: hsl(var(--in) / 0.4)}
.focus\:border-l-info\/5:focus {
    border-left-color: hsl(var(--in) / 0.05)}
.focus\:border-l-info\/50:focus {
    border-left-color: hsl(var(--in) / 0.5)}
.focus\:border-l-info\/60:focus {
    border-left-color: hsl(var(--in) / 0.6)}
.focus\:border-l-info\/70:focus {
    border-left-color: hsl(var(--in) / 0.7)}
.focus\:border-l-info\/75:focus {
    border-left-color: hsl(var(--in) / 0.75)}
.focus\:border-l-info\/80:focus {
    border-left-color: hsl(var(--in) / 0.8)}
.focus\:border-l-info\/90:focus {
    border-left-color: hsl(var(--in) / 0.9)}
.focus\:border-l-info\/95:focus {
    border-left-color: hsl(var(--in) / 0.95)}
.focus\:border-l-success:focus {
    border-left-color: hsl(var(--su) / 1)}
.focus\:border-l-success-content:focus {
    border-left-color: hsl(var(--suc) / 1)}
.focus\:border-l-success-content\/0:focus {
    border-left-color: hsl(var(--suc) / 0)}
.focus\:border-l-success-content\/10:focus {
    border-left-color: hsl(var(--suc) / 0.1)}
.focus\:border-l-success-content\/100:focus {
    border-left-color: hsl(var(--suc) / 1)}
.focus\:border-l-success-content\/20:focus {
    border-left-color: hsl(var(--suc) / 0.2)}
.focus\:border-l-success-content\/25:focus {
    border-left-color: hsl(var(--suc) / 0.25)}
.focus\:border-l-success-content\/30:focus {
    border-left-color: hsl(var(--suc) / 0.3)}
.focus\:border-l-success-content\/40:focus {
    border-left-color: hsl(var(--suc) / 0.4)}
.focus\:border-l-success-content\/5:focus {
    border-left-color: hsl(var(--suc) / 0.05)}
.focus\:border-l-success-content\/50:focus {
    border-left-color: hsl(var(--suc) / 0.5)}
.focus\:border-l-success-content\/60:focus {
    border-left-color: hsl(var(--suc) / 0.6)}
.focus\:border-l-success-content\/70:focus {
    border-left-color: hsl(var(--suc) / 0.7)}
.focus\:border-l-success-content\/75:focus {
    border-left-color: hsl(var(--suc) / 0.75)}
.focus\:border-l-success-content\/80:focus {
    border-left-color: hsl(var(--suc) / 0.8)}
.focus\:border-l-success-content\/90:focus {
    border-left-color: hsl(var(--suc) / 0.9)}
.focus\:border-l-success-content\/95:focus {
    border-left-color: hsl(var(--suc) / 0.95)}
.focus\:border-l-success\/0:focus {
    border-left-color: hsl(var(--su) / 0)}
.focus\:border-l-success\/10:focus {
    border-left-color: hsl(var(--su) / 0.1)}
.focus\:border-l-success\/100:focus {
    border-left-color: hsl(var(--su) / 1)}
.focus\:border-l-success\/20:focus {
    border-left-color: hsl(var(--su) / 0.2)}
.focus\:border-l-success\/25:focus {
    border-left-color: hsl(var(--su) / 0.25)}
.focus\:border-l-success\/30:focus {
    border-left-color: hsl(var(--su) / 0.3)}
.focus\:border-l-success\/40:focus {
    border-left-color: hsl(var(--su) / 0.4)}
.focus\:border-l-success\/5:focus {
    border-left-color: hsl(var(--su) / 0.05)}
.focus\:border-l-success\/50:focus {
    border-left-color: hsl(var(--su) / 0.5)}
.focus\:border-l-success\/60:focus {
    border-left-color: hsl(var(--su) / 0.6)}
.focus\:border-l-success\/70:focus {
    border-left-color: hsl(var(--su) / 0.7)}
.focus\:border-l-success\/75:focus {
    border-left-color: hsl(var(--su) / 0.75)}
.focus\:border-l-success\/80:focus {
    border-left-color: hsl(var(--su) / 0.8)}
.focus\:border-l-success\/90:focus {
    border-left-color: hsl(var(--su) / 0.9)}
.focus\:border-l-success\/95:focus {
    border-left-color: hsl(var(--su) / 0.95)}
.focus\:border-l-warning:focus {
    border-left-color: hsl(var(--wa) / 1)}
.focus\:border-l-warning-content:focus {
    border-left-color: hsl(var(--wac) / 1)}
.focus\:border-l-warning-content\/0:focus {
    border-left-color: hsl(var(--wac) / 0)}
.focus\:border-l-warning-content\/10:focus {
    border-left-color: hsl(var(--wac) / 0.1)}
.focus\:border-l-warning-content\/100:focus {
    border-left-color: hsl(var(--wac) / 1)}
.focus\:border-l-warning-content\/20:focus {
    border-left-color: hsl(var(--wac) / 0.2)}
.focus\:border-l-warning-content\/25:focus {
    border-left-color: hsl(var(--wac) / 0.25)}
.focus\:border-l-warning-content\/30:focus {
    border-left-color: hsl(var(--wac) / 0.3)}
.focus\:border-l-warning-content\/40:focus {
    border-left-color: hsl(var(--wac) / 0.4)}
.focus\:border-l-warning-content\/5:focus {
    border-left-color: hsl(var(--wac) / 0.05)}
.focus\:border-l-warning-content\/50:focus {
    border-left-color: hsl(var(--wac) / 0.5)}
.focus\:border-l-warning-content\/60:focus {
    border-left-color: hsl(var(--wac) / 0.6)}
.focus\:border-l-warning-content\/70:focus {
    border-left-color: hsl(var(--wac) / 0.7)}
.focus\:border-l-warning-content\/75:focus {
    border-left-color: hsl(var(--wac) / 0.75)}
.focus\:border-l-warning-content\/80:focus {
    border-left-color: hsl(var(--wac) / 0.8)}
.focus\:border-l-warning-content\/90:focus {
    border-left-color: hsl(var(--wac) / 0.9)}
.focus\:border-l-warning-content\/95:focus {
    border-left-color: hsl(var(--wac) / 0.95)}
.focus\:border-l-warning\/0:focus {
    border-left-color: hsl(var(--wa) / 0)}
.focus\:border-l-warning\/10:focus {
    border-left-color: hsl(var(--wa) / 0.1)}
.focus\:border-l-warning\/100:focus {
    border-left-color: hsl(var(--wa) / 1)}
.focus\:border-l-warning\/20:focus {
    border-left-color: hsl(var(--wa) / 0.2)}
.focus\:border-l-warning\/25:focus {
    border-left-color: hsl(var(--wa) / 0.25)}
.focus\:border-l-warning\/30:focus {
    border-left-color: hsl(var(--wa) / 0.3)}
.focus\:border-l-warning\/40:focus {
    border-left-color: hsl(var(--wa) / 0.4)}
.focus\:border-l-warning\/5:focus {
    border-left-color: hsl(var(--wa) / 0.05)}
.focus\:border-l-warning\/50:focus {
    border-left-color: hsl(var(--wa) / 0.5)}
.focus\:border-l-warning\/60:focus {
    border-left-color: hsl(var(--wa) / 0.6)}
.focus\:border-l-warning\/70:focus {
    border-left-color: hsl(var(--wa) / 0.7)}
.focus\:border-l-warning\/75:focus {
    border-left-color: hsl(var(--wa) / 0.75)}
.focus\:border-l-warning\/80:focus {
    border-left-color: hsl(var(--wa) / 0.8)}
.focus\:border-l-warning\/90:focus {
    border-left-color: hsl(var(--wa) / 0.9)}
.focus\:border-l-warning\/95:focus {
    border-left-color: hsl(var(--wa) / 0.95)}
.focus\:border-r-base-100:focus {
    border-right-color: hsl(var(--b1) / 1)}
.focus\:border-r-base-100\/0:focus {
    border-right-color: hsl(var(--b1) / 0)}
.focus\:border-r-base-100\/10:focus {
    border-right-color: hsl(var(--b1) / 0.1)}
.focus\:border-r-base-100\/100:focus {
    border-right-color: hsl(var(--b1) / 1)}
.focus\:border-r-base-100\/20:focus {
    border-right-color: hsl(var(--b1) / 0.2)}
.focus\:border-r-base-100\/25:focus {
    border-right-color: hsl(var(--b1) / 0.25)}
.focus\:border-r-base-100\/30:focus {
    border-right-color: hsl(var(--b1) / 0.3)}
.focus\:border-r-base-100\/40:focus {
    border-right-color: hsl(var(--b1) / 0.4)}
.focus\:border-r-base-100\/5:focus {
    border-right-color: hsl(var(--b1) / 0.05)}
.focus\:border-r-base-100\/50:focus {
    border-right-color: hsl(var(--b1) / 0.5)}
.focus\:border-r-base-100\/60:focus {
    border-right-color: hsl(var(--b1) / 0.6)}
.focus\:border-r-base-100\/70:focus {
    border-right-color: hsl(var(--b1) / 0.7)}
.focus\:border-r-base-100\/75:focus {
    border-right-color: hsl(var(--b1) / 0.75)}
.focus\:border-r-base-100\/80:focus {
    border-right-color: hsl(var(--b1) / 0.8)}
.focus\:border-r-base-100\/90:focus {
    border-right-color: hsl(var(--b1) / 0.9)}
.focus\:border-r-base-100\/95:focus {
    border-right-color: hsl(var(--b1) / 0.95)}
.focus\:border-r-base-200:focus {
    border-right-color: hsl(var(--b2) / 1)}
.focus\:border-r-base-200\/0:focus {
    border-right-color: hsl(var(--b2) / 0)}
.focus\:border-r-base-200\/10:focus {
    border-right-color: hsl(var(--b2) / 0.1)}
.focus\:border-r-base-200\/100:focus {
    border-right-color: hsl(var(--b2) / 1)}
.focus\:border-r-base-200\/20:focus {
    border-right-color: hsl(var(--b2) / 0.2)}
.focus\:border-r-base-200\/25:focus {
    border-right-color: hsl(var(--b2) / 0.25)}
.focus\:border-r-base-200\/30:focus {
    border-right-color: hsl(var(--b2) / 0.3)}
.focus\:border-r-base-200\/40:focus {
    border-right-color: hsl(var(--b2) / 0.4)}
.focus\:border-r-base-200\/5:focus {
    border-right-color: hsl(var(--b2) / 0.05)}
.focus\:border-r-base-200\/50:focus {
    border-right-color: hsl(var(--b2) / 0.5)}
.focus\:border-r-base-200\/60:focus {
    border-right-color: hsl(var(--b2) / 0.6)}
.focus\:border-r-base-200\/70:focus {
    border-right-color: hsl(var(--b2) / 0.7)}
.focus\:border-r-base-200\/75:focus {
    border-right-color: hsl(var(--b2) / 0.75)}
.focus\:border-r-base-200\/80:focus {
    border-right-color: hsl(var(--b2) / 0.8)}
.focus\:border-r-base-200\/90:focus {
    border-right-color: hsl(var(--b2) / 0.9)}
.focus\:border-r-base-200\/95:focus {
    border-right-color: hsl(var(--b2) / 0.95)}
.focus\:border-r-base-300:focus {
    border-right-color: hsl(var(--b3) / 1)}
.focus\:border-r-base-300\/0:focus {
    border-right-color: hsl(var(--b3) / 0)}
.focus\:border-r-base-300\/10:focus {
    border-right-color: hsl(var(--b3) / 0.1)}
.focus\:border-r-base-300\/100:focus {
    border-right-color: hsl(var(--b3) / 1)}
.focus\:border-r-base-300\/20:focus {
    border-right-color: hsl(var(--b3) / 0.2)}
.focus\:border-r-base-300\/25:focus {
    border-right-color: hsl(var(--b3) / 0.25)}
.focus\:border-r-base-300\/30:focus {
    border-right-color: hsl(var(--b3) / 0.3)}
.focus\:border-r-base-300\/40:focus {
    border-right-color: hsl(var(--b3) / 0.4)}
.focus\:border-r-base-300\/5:focus {
    border-right-color: hsl(var(--b3) / 0.05)}
.focus\:border-r-base-300\/50:focus {
    border-right-color: hsl(var(--b3) / 0.5)}
.focus\:border-r-base-300\/60:focus {
    border-right-color: hsl(var(--b3) / 0.6)}
.focus\:border-r-base-300\/70:focus {
    border-right-color: hsl(var(--b3) / 0.7)}
.focus\:border-r-base-300\/75:focus {
    border-right-color: hsl(var(--b3) / 0.75)}
.focus\:border-r-base-300\/80:focus {
    border-right-color: hsl(var(--b3) / 0.8)}
.focus\:border-r-base-300\/90:focus {
    border-right-color: hsl(var(--b3) / 0.9)}
.focus\:border-r-base-300\/95:focus {
    border-right-color: hsl(var(--b3) / 0.95)}
.focus\:border-r-base-content:focus {
    border-right-color: hsl(var(--bc) / 1)}
.focus\:border-r-base-content\/0:focus {
    border-right-color: hsl(var(--bc) / 0)}
.focus\:border-r-base-content\/10:focus {
    border-right-color: hsl(var(--bc) / 0.1)}
.focus\:border-r-base-content\/100:focus {
    border-right-color: hsl(var(--bc) / 1)}
.focus\:border-r-base-content\/20:focus {
    border-right-color: hsl(var(--bc) / 0.2)}
.focus\:border-r-base-content\/25:focus {
    border-right-color: hsl(var(--bc) / 0.25)}
.focus\:border-r-base-content\/30:focus {
    border-right-color: hsl(var(--bc) / 0.3)}
.focus\:border-r-base-content\/40:focus {
    border-right-color: hsl(var(--bc) / 0.4)}
.focus\:border-r-base-content\/5:focus {
    border-right-color: hsl(var(--bc) / 0.05)}
.focus\:border-r-base-content\/50:focus {
    border-right-color: hsl(var(--bc) / 0.5)}
.focus\:border-r-base-content\/60:focus {
    border-right-color: hsl(var(--bc) / 0.6)}
.focus\:border-r-base-content\/70:focus {
    border-right-color: hsl(var(--bc) / 0.7)}
.focus\:border-r-base-content\/75:focus {
    border-right-color: hsl(var(--bc) / 0.75)}
.focus\:border-r-base-content\/80:focus {
    border-right-color: hsl(var(--bc) / 0.8)}
.focus\:border-r-base-content\/90:focus {
    border-right-color: hsl(var(--bc) / 0.9)}
.focus\:border-r-base-content\/95:focus {
    border-right-color: hsl(var(--bc) / 0.95)}
.focus\:border-r-error:focus {
    border-right-color: hsl(var(--er) / 1)}
.focus\:border-r-error-content:focus {
    border-right-color: hsl(var(--erc) / 1)}
.focus\:border-r-error-content\/0:focus {
    border-right-color: hsl(var(--erc) / 0)}
.focus\:border-r-error-content\/10:focus {
    border-right-color: hsl(var(--erc) / 0.1)}
.focus\:border-r-error-content\/100:focus {
    border-right-color: hsl(var(--erc) / 1)}
.focus\:border-r-error-content\/20:focus {
    border-right-color: hsl(var(--erc) / 0.2)}
.focus\:border-r-error-content\/25:focus {
    border-right-color: hsl(var(--erc) / 0.25)}
.focus\:border-r-error-content\/30:focus {
    border-right-color: hsl(var(--erc) / 0.3)}
.focus\:border-r-error-content\/40:focus {
    border-right-color: hsl(var(--erc) / 0.4)}
.focus\:border-r-error-content\/5:focus {
    border-right-color: hsl(var(--erc) / 0.05)}
.focus\:border-r-error-content\/50:focus {
    border-right-color: hsl(var(--erc) / 0.5)}
.focus\:border-r-error-content\/60:focus {
    border-right-color: hsl(var(--erc) / 0.6)}
.focus\:border-r-error-content\/70:focus {
    border-right-color: hsl(var(--erc) / 0.7)}
.focus\:border-r-error-content\/75:focus {
    border-right-color: hsl(var(--erc) / 0.75)}
.focus\:border-r-error-content\/80:focus {
    border-right-color: hsl(var(--erc) / 0.8)}
.focus\:border-r-error-content\/90:focus {
    border-right-color: hsl(var(--erc) / 0.9)}
.focus\:border-r-error-content\/95:focus {
    border-right-color: hsl(var(--erc) / 0.95)}
.focus\:border-r-error\/0:focus {
    border-right-color: hsl(var(--er) / 0)}
.focus\:border-r-error\/10:focus {
    border-right-color: hsl(var(--er) / 0.1)}
.focus\:border-r-error\/100:focus {
    border-right-color: hsl(var(--er) / 1)}
.focus\:border-r-error\/20:focus {
    border-right-color: hsl(var(--er) / 0.2)}
.focus\:border-r-error\/25:focus {
    border-right-color: hsl(var(--er) / 0.25)}
.focus\:border-r-error\/30:focus {
    border-right-color: hsl(var(--er) / 0.3)}
.focus\:border-r-error\/40:focus {
    border-right-color: hsl(var(--er) / 0.4)}
.focus\:border-r-error\/5:focus {
    border-right-color: hsl(var(--er) / 0.05)}
.focus\:border-r-error\/50:focus {
    border-right-color: hsl(var(--er) / 0.5)}
.focus\:border-r-error\/60:focus {
    border-right-color: hsl(var(--er) / 0.6)}
.focus\:border-r-error\/70:focus {
    border-right-color: hsl(var(--er) / 0.7)}
.focus\:border-r-error\/75:focus {
    border-right-color: hsl(var(--er) / 0.75)}
.focus\:border-r-error\/80:focus {
    border-right-color: hsl(var(--er) / 0.8)}
.focus\:border-r-error\/90:focus {
    border-right-color: hsl(var(--er) / 0.9)}
.focus\:border-r-error\/95:focus {
    border-right-color: hsl(var(--er) / 0.95)}
.focus\:border-r-info:focus {
    border-right-color: hsl(var(--in) / 1)}
.focus\:border-r-info-content:focus {
    border-right-color: hsl(var(--inc) / 1)}
.focus\:border-r-info-content\/0:focus {
    border-right-color: hsl(var(--inc) / 0)}
.focus\:border-r-info-content\/10:focus {
    border-right-color: hsl(var(--inc) / 0.1)}
.focus\:border-r-info-content\/100:focus {
    border-right-color: hsl(var(--inc) / 1)}
.focus\:border-r-info-content\/20:focus {
    border-right-color: hsl(var(--inc) / 0.2)}
.focus\:border-r-info-content\/25:focus {
    border-right-color: hsl(var(--inc) / 0.25)}
.focus\:border-r-info-content\/30:focus {
    border-right-color: hsl(var(--inc) / 0.3)}
.focus\:border-r-info-content\/40:focus {
    border-right-color: hsl(var(--inc) / 0.4)}
.focus\:border-r-info-content\/5:focus {
    border-right-color: hsl(var(--inc) / 0.05)}
.focus\:border-r-info-content\/50:focus {
    border-right-color: hsl(var(--inc) / 0.5)}
.focus\:border-r-info-content\/60:focus {
    border-right-color: hsl(var(--inc) / 0.6)}
.focus\:border-r-info-content\/70:focus {
    border-right-color: hsl(var(--inc) / 0.7)}
.focus\:border-r-info-content\/75:focus {
    border-right-color: hsl(var(--inc) / 0.75)}
.focus\:border-r-info-content\/80:focus {
    border-right-color: hsl(var(--inc) / 0.8)}
.focus\:border-r-info-content\/90:focus {
    border-right-color: hsl(var(--inc) / 0.9)}
.focus\:border-r-info-content\/95:focus {
    border-right-color: hsl(var(--inc) / 0.95)}
.focus\:border-r-info\/0:focus {
    border-right-color: hsl(var(--in) / 0)}
.focus\:border-r-info\/10:focus {
    border-right-color: hsl(var(--in) / 0.1)}
.focus\:border-r-info\/100:focus {
    border-right-color: hsl(var(--in) / 1)}
.focus\:border-r-info\/20:focus {
    border-right-color: hsl(var(--in) / 0.2)}
.focus\:border-r-info\/25:focus {
    border-right-color: hsl(var(--in) / 0.25)}
.focus\:border-r-info\/30:focus {
    border-right-color: hsl(var(--in) / 0.3)}
.focus\:border-r-info\/40:focus {
    border-right-color: hsl(var(--in) / 0.4)}
.focus\:border-r-info\/5:focus {
    border-right-color: hsl(var(--in) / 0.05)}
.focus\:border-r-info\/50:focus {
    border-right-color: hsl(var(--in) / 0.5)}
.focus\:border-r-info\/60:focus {
    border-right-color: hsl(var(--in) / 0.6)}
.focus\:border-r-info\/70:focus {
    border-right-color: hsl(var(--in) / 0.7)}
.focus\:border-r-info\/75:focus {
    border-right-color: hsl(var(--in) / 0.75)}
.focus\:border-r-info\/80:focus {
    border-right-color: hsl(var(--in) / 0.8)}
.focus\:border-r-info\/90:focus {
    border-right-color: hsl(var(--in) / 0.9)}
.focus\:border-r-info\/95:focus {
    border-right-color: hsl(var(--in) / 0.95)}
.focus\:border-r-success:focus {
    border-right-color: hsl(var(--su) / 1)}
.focus\:border-r-success-content:focus {
    border-right-color: hsl(var(--suc) / 1)}
.focus\:border-r-success-content\/0:focus {
    border-right-color: hsl(var(--suc) / 0)}
.focus\:border-r-success-content\/10:focus {
    border-right-color: hsl(var(--suc) / 0.1)}
.focus\:border-r-success-content\/100:focus {
    border-right-color: hsl(var(--suc) / 1)}
.focus\:border-r-success-content\/20:focus {
    border-right-color: hsl(var(--suc) / 0.2)}
.focus\:border-r-success-content\/25:focus {
    border-right-color: hsl(var(--suc) / 0.25)}
.focus\:border-r-success-content\/30:focus {
    border-right-color: hsl(var(--suc) / 0.3)}
.focus\:border-r-success-content\/40:focus {
    border-right-color: hsl(var(--suc) / 0.4)}
.focus\:border-r-success-content\/5:focus {
    border-right-color: hsl(var(--suc) / 0.05)}
.focus\:border-r-success-content\/50:focus {
    border-right-color: hsl(var(--suc) / 0.5)}
.focus\:border-r-success-content\/60:focus {
    border-right-color: hsl(var(--suc) / 0.6)}
.focus\:border-r-success-content\/70:focus {
    border-right-color: hsl(var(--suc) / 0.7)}
.focus\:border-r-success-content\/75:focus {
    border-right-color: hsl(var(--suc) / 0.75)}
.focus\:border-r-success-content\/80:focus {
    border-right-color: hsl(var(--suc) / 0.8)}
.focus\:border-r-success-content\/90:focus {
    border-right-color: hsl(var(--suc) / 0.9)}
.focus\:border-r-success-content\/95:focus {
    border-right-color: hsl(var(--suc) / 0.95)}
.focus\:border-r-success\/0:focus {
    border-right-color: hsl(var(--su) / 0)}
.focus\:border-r-success\/10:focus {
    border-right-color: hsl(var(--su) / 0.1)}
.focus\:border-r-success\/100:focus {
    border-right-color: hsl(var(--su) / 1)}
.focus\:border-r-success\/20:focus {
    border-right-color: hsl(var(--su) / 0.2)}
.focus\:border-r-success\/25:focus {
    border-right-color: hsl(var(--su) / 0.25)}
.focus\:border-r-success\/30:focus {
    border-right-color: hsl(var(--su) / 0.3)}
.focus\:border-r-success\/40:focus {
    border-right-color: hsl(var(--su) / 0.4)}
.focus\:border-r-success\/5:focus {
    border-right-color: hsl(var(--su) / 0.05)}
.focus\:border-r-success\/50:focus {
    border-right-color: hsl(var(--su) / 0.5)}
.focus\:border-r-success\/60:focus {
    border-right-color: hsl(var(--su) / 0.6)}
.focus\:border-r-success\/70:focus {
    border-right-color: hsl(var(--su) / 0.7)}
.focus\:border-r-success\/75:focus {
    border-right-color: hsl(var(--su) / 0.75)}
.focus\:border-r-success\/80:focus {
    border-right-color: hsl(var(--su) / 0.8)}
.focus\:border-r-success\/90:focus {
    border-right-color: hsl(var(--su) / 0.9)}
.focus\:border-r-success\/95:focus {
    border-right-color: hsl(var(--su) / 0.95)}
.focus\:border-r-warning:focus {
    border-right-color: hsl(var(--wa) / 1)}
.focus\:border-r-warning-content:focus {
    border-right-color: hsl(var(--wac) / 1)}
.focus\:border-r-warning-content\/0:focus {
    border-right-color: hsl(var(--wac) / 0)}
.focus\:border-r-warning-content\/10:focus {
    border-right-color: hsl(var(--wac) / 0.1)}
.focus\:border-r-warning-content\/100:focus {
    border-right-color: hsl(var(--wac) / 1)}
.focus\:border-r-warning-content\/20:focus {
    border-right-color: hsl(var(--wac) / 0.2)}
.focus\:border-r-warning-content\/25:focus {
    border-right-color: hsl(var(--wac) / 0.25)}
.focus\:border-r-warning-content\/30:focus {
    border-right-color: hsl(var(--wac) / 0.3)}
.focus\:border-r-warning-content\/40:focus {
    border-right-color: hsl(var(--wac) / 0.4)}
.focus\:border-r-warning-content\/5:focus {
    border-right-color: hsl(var(--wac) / 0.05)}
.focus\:border-r-warning-content\/50:focus {
    border-right-color: hsl(var(--wac) / 0.5)}
.focus\:border-r-warning-content\/60:focus {
    border-right-color: hsl(var(--wac) / 0.6)}
.focus\:border-r-warning-content\/70:focus {
    border-right-color: hsl(var(--wac) / 0.7)}
.focus\:border-r-warning-content\/75:focus {
    border-right-color: hsl(var(--wac) / 0.75)}
.focus\:border-r-warning-content\/80:focus {
    border-right-color: hsl(var(--wac) / 0.8)}
.focus\:border-r-warning-content\/90:focus {
    border-right-color: hsl(var(--wac) / 0.9)}
.focus\:border-r-warning-content\/95:focus {
    border-right-color: hsl(var(--wac) / 0.95)}
.focus\:border-r-warning\/0:focus {
    border-right-color: hsl(var(--wa) / 0)}
.focus\:border-r-warning\/10:focus {
    border-right-color: hsl(var(--wa) / 0.1)}
.focus\:border-r-warning\/100:focus {
    border-right-color: hsl(var(--wa) / 1)}
.focus\:border-r-warning\/20:focus {
    border-right-color: hsl(var(--wa) / 0.2)}
.focus\:border-r-warning\/25:focus {
    border-right-color: hsl(var(--wa) / 0.25)}
.focus\:border-r-warning\/30:focus {
    border-right-color: hsl(var(--wa) / 0.3)}
.focus\:border-r-warning\/40:focus {
    border-right-color: hsl(var(--wa) / 0.4)}
.focus\:border-r-warning\/5:focus {
    border-right-color: hsl(var(--wa) / 0.05)}
.focus\:border-r-warning\/50:focus {
    border-right-color: hsl(var(--wa) / 0.5)}
.focus\:border-r-warning\/60:focus {
    border-right-color: hsl(var(--wa) / 0.6)}
.focus\:border-r-warning\/70:focus {
    border-right-color: hsl(var(--wa) / 0.7)}
.focus\:border-r-warning\/75:focus {
    border-right-color: hsl(var(--wa) / 0.75)}
.focus\:border-r-warning\/80:focus {
    border-right-color: hsl(var(--wa) / 0.8)}
.focus\:border-r-warning\/90:focus {
    border-right-color: hsl(var(--wa) / 0.9)}
.focus\:border-r-warning\/95:focus {
    border-right-color: hsl(var(--wa) / 0.95)}
.focus\:border-s-base-100:focus {
    border-inline-start-color: hsl(var(--b1) / 1)}
.focus\:border-s-base-100\/0:focus {
    border-inline-start-color: hsl(var(--b1) / 0)}
.focus\:border-s-base-100\/10:focus {
    border-inline-start-color: hsl(var(--b1) / 0.1)}
.focus\:border-s-base-100\/100:focus {
    border-inline-start-color: hsl(var(--b1) / 1)}
.focus\:border-s-base-100\/20:focus {
    border-inline-start-color: hsl(var(--b1) / 0.2)}
.focus\:border-s-base-100\/25:focus {
    border-inline-start-color: hsl(var(--b1) / 0.25)}
.focus\:border-s-base-100\/30:focus {
    border-inline-start-color: hsl(var(--b1) / 0.3)}
.focus\:border-s-base-100\/40:focus {
    border-inline-start-color: hsl(var(--b1) / 0.4)}
.focus\:border-s-base-100\/5:focus {
    border-inline-start-color: hsl(var(--b1) / 0.05)}
.focus\:border-s-base-100\/50:focus {
    border-inline-start-color: hsl(var(--b1) / 0.5)}
.focus\:border-s-base-100\/60:focus {
    border-inline-start-color: hsl(var(--b1) / 0.6)}
.focus\:border-s-base-100\/70:focus {
    border-inline-start-color: hsl(var(--b1) / 0.7)}
.focus\:border-s-base-100\/75:focus {
    border-inline-start-color: hsl(var(--b1) / 0.75)}
.focus\:border-s-base-100\/80:focus {
    border-inline-start-color: hsl(var(--b1) / 0.8)}
.focus\:border-s-base-100\/90:focus {
    border-inline-start-color: hsl(var(--b1) / 0.9)}
.focus\:border-s-base-100\/95:focus {
    border-inline-start-color: hsl(var(--b1) / 0.95)}
.focus\:border-s-base-200:focus {
    border-inline-start-color: hsl(var(--b2) / 1)}
.focus\:border-s-base-200\/0:focus {
    border-inline-start-color: hsl(var(--b2) / 0)}
.focus\:border-s-base-200\/10:focus {
    border-inline-start-color: hsl(var(--b2) / 0.1)}
.focus\:border-s-base-200\/100:focus {
    border-inline-start-color: hsl(var(--b2) / 1)}
.focus\:border-s-base-200\/20:focus {
    border-inline-start-color: hsl(var(--b2) / 0.2)}
.focus\:border-s-base-200\/25:focus {
    border-inline-start-color: hsl(var(--b2) / 0.25)}
.focus\:border-s-base-200\/30:focus {
    border-inline-start-color: hsl(var(--b2) / 0.3)}
.focus\:border-s-base-200\/40:focus {
    border-inline-start-color: hsl(var(--b2) / 0.4)}
.focus\:border-s-base-200\/5:focus {
    border-inline-start-color: hsl(var(--b2) / 0.05)}
.focus\:border-s-base-200\/50:focus {
    border-inline-start-color: hsl(var(--b2) / 0.5)}
.focus\:border-s-base-200\/60:focus {
    border-inline-start-color: hsl(var(--b2) / 0.6)}
.focus\:border-s-base-200\/70:focus {
    border-inline-start-color: hsl(var(--b2) / 0.7)}
.focus\:border-s-base-200\/75:focus {
    border-inline-start-color: hsl(var(--b2) / 0.75)}
.focus\:border-s-base-200\/80:focus {
    border-inline-start-color: hsl(var(--b2) / 0.8)}
.focus\:border-s-base-200\/90:focus {
    border-inline-start-color: hsl(var(--b2) / 0.9)}
.focus\:border-s-base-200\/95:focus {
    border-inline-start-color: hsl(var(--b2) / 0.95)}
.focus\:border-s-base-300:focus {
    border-inline-start-color: hsl(var(--b3) / 1)}
.focus\:border-s-base-300\/0:focus {
    border-inline-start-color: hsl(var(--b3) / 0)}
.focus\:border-s-base-300\/10:focus {
    border-inline-start-color: hsl(var(--b3) / 0.1)}
.focus\:border-s-base-300\/100:focus {
    border-inline-start-color: hsl(var(--b3) / 1)}
.focus\:border-s-base-300\/20:focus {
    border-inline-start-color: hsl(var(--b3) / 0.2)}
.focus\:border-s-base-300\/25:focus {
    border-inline-start-color: hsl(var(--b3) / 0.25)}
.focus\:border-s-base-300\/30:focus {
    border-inline-start-color: hsl(var(--b3) / 0.3)}
.focus\:border-s-base-300\/40:focus {
    border-inline-start-color: hsl(var(--b3) / 0.4)}
.focus\:border-s-base-300\/5:focus {
    border-inline-start-color: hsl(var(--b3) / 0.05)}
.focus\:border-s-base-300\/50:focus {
    border-inline-start-color: hsl(var(--b3) / 0.5)}
.focus\:border-s-base-300\/60:focus {
    border-inline-start-color: hsl(var(--b3) / 0.6)}
.focus\:border-s-base-300\/70:focus {
    border-inline-start-color: hsl(var(--b3) / 0.7)}
.focus\:border-s-base-300\/75:focus {
    border-inline-start-color: hsl(var(--b3) / 0.75)}
.focus\:border-s-base-300\/80:focus {
    border-inline-start-color: hsl(var(--b3) / 0.8)}
.focus\:border-s-base-300\/90:focus {
    border-inline-start-color: hsl(var(--b3) / 0.9)}
.focus\:border-s-base-300\/95:focus {
    border-inline-start-color: hsl(var(--b3) / 0.95)}
.focus\:border-s-base-content:focus {
    border-inline-start-color: hsl(var(--bc) / 1)}
.focus\:border-s-base-content\/0:focus {
    border-inline-start-color: hsl(var(--bc) / 0)}
.focus\:border-s-base-content\/10:focus {
    border-inline-start-color: hsl(var(--bc) / 0.1)}
.focus\:border-s-base-content\/100:focus {
    border-inline-start-color: hsl(var(--bc) / 1)}
.focus\:border-s-base-content\/20:focus {
    border-inline-start-color: hsl(var(--bc) / 0.2)}
.focus\:border-s-base-content\/25:focus {
    border-inline-start-color: hsl(var(--bc) / 0.25)}
.focus\:border-s-base-content\/30:focus {
    border-inline-start-color: hsl(var(--bc) / 0.3)}
.focus\:border-s-base-content\/40:focus {
    border-inline-start-color: hsl(var(--bc) / 0.4)}
.focus\:border-s-base-content\/5:focus {
    border-inline-start-color: hsl(var(--bc) / 0.05)}
.focus\:border-s-base-content\/50:focus {
    border-inline-start-color: hsl(var(--bc) / 0.5)}
.focus\:border-s-base-content\/60:focus {
    border-inline-start-color: hsl(var(--bc) / 0.6)}
.focus\:border-s-base-content\/70:focus {
    border-inline-start-color: hsl(var(--bc) / 0.7)}
.focus\:border-s-base-content\/75:focus {
    border-inline-start-color: hsl(var(--bc) / 0.75)}
.focus\:border-s-base-content\/80:focus {
    border-inline-start-color: hsl(var(--bc) / 0.8)}
.focus\:border-s-base-content\/90:focus {
    border-inline-start-color: hsl(var(--bc) / 0.9)}
.focus\:border-s-base-content\/95:focus {
    border-inline-start-color: hsl(var(--bc) / 0.95)}
.focus\:border-s-error:focus {
    border-inline-start-color: hsl(var(--er) / 1)}
.focus\:border-s-error-content:focus {
    border-inline-start-color: hsl(var(--erc) / 1)}
.focus\:border-s-error-content\/0:focus {
    border-inline-start-color: hsl(var(--erc) / 0)}
.focus\:border-s-error-content\/10:focus {
    border-inline-start-color: hsl(var(--erc) / 0.1)}
.focus\:border-s-error-content\/100:focus {
    border-inline-start-color: hsl(var(--erc) / 1)}
.focus\:border-s-error-content\/20:focus {
    border-inline-start-color: hsl(var(--erc) / 0.2)}
.focus\:border-s-error-content\/25:focus {
    border-inline-start-color: hsl(var(--erc) / 0.25)}
.focus\:border-s-error-content\/30:focus {
    border-inline-start-color: hsl(var(--erc) / 0.3)}
.focus\:border-s-error-content\/40:focus {
    border-inline-start-color: hsl(var(--erc) / 0.4)}
.focus\:border-s-error-content\/5:focus {
    border-inline-start-color: hsl(var(--erc) / 0.05)}
.focus\:border-s-error-content\/50:focus {
    border-inline-start-color: hsl(var(--erc) / 0.5)}
.focus\:border-s-error-content\/60:focus {
    border-inline-start-color: hsl(var(--erc) / 0.6)}
.focus\:border-s-error-content\/70:focus {
    border-inline-start-color: hsl(var(--erc) / 0.7)}
.focus\:border-s-error-content\/75:focus {
    border-inline-start-color: hsl(var(--erc) / 0.75)}
.focus\:border-s-error-content\/80:focus {
    border-inline-start-color: hsl(var(--erc) / 0.8)}
.focus\:border-s-error-content\/90:focus {
    border-inline-start-color: hsl(var(--erc) / 0.9)}
.focus\:border-s-error-content\/95:focus {
    border-inline-start-color: hsl(var(--erc) / 0.95)}
.focus\:border-s-error\/0:focus {
    border-inline-start-color: hsl(var(--er) / 0)}
.focus\:border-s-error\/10:focus {
    border-inline-start-color: hsl(var(--er) / 0.1)}
.focus\:border-s-error\/100:focus {
    border-inline-start-color: hsl(var(--er) / 1)}
.focus\:border-s-error\/20:focus {
    border-inline-start-color: hsl(var(--er) / 0.2)}
.focus\:border-s-error\/25:focus {
    border-inline-start-color: hsl(var(--er) / 0.25)}
.focus\:border-s-error\/30:focus {
    border-inline-start-color: hsl(var(--er) / 0.3)}
.focus\:border-s-error\/40:focus {
    border-inline-start-color: hsl(var(--er) / 0.4)}
.focus\:border-s-error\/5:focus {
    border-inline-start-color: hsl(var(--er) / 0.05)}
.focus\:border-s-error\/50:focus {
    border-inline-start-color: hsl(var(--er) / 0.5)}
.focus\:border-s-error\/60:focus {
    border-inline-start-color: hsl(var(--er) / 0.6)}
.focus\:border-s-error\/70:focus {
    border-inline-start-color: hsl(var(--er) / 0.7)}
.focus\:border-s-error\/75:focus {
    border-inline-start-color: hsl(var(--er) / 0.75)}
.focus\:border-s-error\/80:focus {
    border-inline-start-color: hsl(var(--er) / 0.8)}
.focus\:border-s-error\/90:focus {
    border-inline-start-color: hsl(var(--er) / 0.9)}
.focus\:border-s-error\/95:focus {
    border-inline-start-color: hsl(var(--er) / 0.95)}
.focus\:border-s-info:focus {
    border-inline-start-color: hsl(var(--in) / 1)}
.focus\:border-s-info-content:focus {
    border-inline-start-color: hsl(var(--inc) / 1)}
.focus\:border-s-info-content\/0:focus {
    border-inline-start-color: hsl(var(--inc) / 0)}
.focus\:border-s-info-content\/10:focus {
    border-inline-start-color: hsl(var(--inc) / 0.1)}
.focus\:border-s-info-content\/100:focus {
    border-inline-start-color: hsl(var(--inc) / 1)}
.focus\:border-s-info-content\/20:focus {
    border-inline-start-color: hsl(var(--inc) / 0.2)}
.focus\:border-s-info-content\/25:focus {
    border-inline-start-color: hsl(var(--inc) / 0.25)}
.focus\:border-s-info-content\/30:focus {
    border-inline-start-color: hsl(var(--inc) / 0.3)}
.focus\:border-s-info-content\/40:focus {
    border-inline-start-color: hsl(var(--inc) / 0.4)}
.focus\:border-s-info-content\/5:focus {
    border-inline-start-color: hsl(var(--inc) / 0.05)}
.focus\:border-s-info-content\/50:focus {
    border-inline-start-color: hsl(var(--inc) / 0.5)}
.focus\:border-s-info-content\/60:focus {
    border-inline-start-color: hsl(var(--inc) / 0.6)}
.focus\:border-s-info-content\/70:focus {
    border-inline-start-color: hsl(var(--inc) / 0.7)}
.focus\:border-s-info-content\/75:focus {
    border-inline-start-color: hsl(var(--inc) / 0.75)}
.focus\:border-s-info-content\/80:focus {
    border-inline-start-color: hsl(var(--inc) / 0.8)}
.focus\:border-s-info-content\/90:focus {
    border-inline-start-color: hsl(var(--inc) / 0.9)}
.focus\:border-s-info-content\/95:focus {
    border-inline-start-color: hsl(var(--inc) / 0.95)}
.focus\:border-s-info\/0:focus {
    border-inline-start-color: hsl(var(--in) / 0)}
.focus\:border-s-info\/10:focus {
    border-inline-start-color: hsl(var(--in) / 0.1)}
.focus\:border-s-info\/100:focus {
    border-inline-start-color: hsl(var(--in) / 1)}
.focus\:border-s-info\/20:focus {
    border-inline-start-color: hsl(var(--in) / 0.2)}
.focus\:border-s-info\/25:focus {
    border-inline-start-color: hsl(var(--in) / 0.25)}
.focus\:border-s-info\/30:focus {
    border-inline-start-color: hsl(var(--in) / 0.3)}
.focus\:border-s-info\/40:focus {
    border-inline-start-color: hsl(var(--in) / 0.4)}
.focus\:border-s-info\/5:focus {
    border-inline-start-color: hsl(var(--in) / 0.05)}
.focus\:border-s-info\/50:focus {
    border-inline-start-color: hsl(var(--in) / 0.5)}
.focus\:border-s-info\/60:focus {
    border-inline-start-color: hsl(var(--in) / 0.6)}
.focus\:border-s-info\/70:focus {
    border-inline-start-color: hsl(var(--in) / 0.7)}
.focus\:border-s-info\/75:focus {
    border-inline-start-color: hsl(var(--in) / 0.75)}
.focus\:border-s-info\/80:focus {
    border-inline-start-color: hsl(var(--in) / 0.8)}
.focus\:border-s-info\/90:focus {
    border-inline-start-color: hsl(var(--in) / 0.9)}
.focus\:border-s-info\/95:focus {
    border-inline-start-color: hsl(var(--in) / 0.95)}
.focus\:border-s-success:focus {
    border-inline-start-color: hsl(var(--su) / 1)}
.focus\:border-s-success-content:focus {
    border-inline-start-color: hsl(var(--suc) / 1)}
.focus\:border-s-success-content\/0:focus {
    border-inline-start-color: hsl(var(--suc) / 0)}
.focus\:border-s-success-content\/10:focus {
    border-inline-start-color: hsl(var(--suc) / 0.1)}
.focus\:border-s-success-content\/100:focus {
    border-inline-start-color: hsl(var(--suc) / 1)}
.focus\:border-s-success-content\/20:focus {
    border-inline-start-color: hsl(var(--suc) / 0.2)}
.focus\:border-s-success-content\/25:focus {
    border-inline-start-color: hsl(var(--suc) / 0.25)}
.focus\:border-s-success-content\/30:focus {
    border-inline-start-color: hsl(var(--suc) / 0.3)}
.focus\:border-s-success-content\/40:focus {
    border-inline-start-color: hsl(var(--suc) / 0.4)}
.focus\:border-s-success-content\/5:focus {
    border-inline-start-color: hsl(var(--suc) / 0.05)}
.focus\:border-s-success-content\/50:focus {
    border-inline-start-color: hsl(var(--suc) / 0.5)}
.focus\:border-s-success-content\/60:focus {
    border-inline-start-color: hsl(var(--suc) / 0.6)}
.focus\:border-s-success-content\/70:focus {
    border-inline-start-color: hsl(var(--suc) / 0.7)}
.focus\:border-s-success-content\/75:focus {
    border-inline-start-color: hsl(var(--suc) / 0.75)}
.focus\:border-s-success-content\/80:focus {
    border-inline-start-color: hsl(var(--suc) / 0.8)}
.focus\:border-s-success-content\/90:focus {
    border-inline-start-color: hsl(var(--suc) / 0.9)}
.focus\:border-s-success-content\/95:focus {
    border-inline-start-color: hsl(var(--suc) / 0.95)}
.focus\:border-s-success\/0:focus {
    border-inline-start-color: hsl(var(--su) / 0)}
.focus\:border-s-success\/10:focus {
    border-inline-start-color: hsl(var(--su) / 0.1)}
.focus\:border-s-success\/100:focus {
    border-inline-start-color: hsl(var(--su) / 1)}
.focus\:border-s-success\/20:focus {
    border-inline-start-color: hsl(var(--su) / 0.2)}
.focus\:border-s-success\/25:focus {
    border-inline-start-color: hsl(var(--su) / 0.25)}
.focus\:border-s-success\/30:focus {
    border-inline-start-color: hsl(var(--su) / 0.3)}
.focus\:border-s-success\/40:focus {
    border-inline-start-color: hsl(var(--su) / 0.4)}
.focus\:border-s-success\/5:focus {
    border-inline-start-color: hsl(var(--su) / 0.05)}
.focus\:border-s-success\/50:focus {
    border-inline-start-color: hsl(var(--su) / 0.5)}
.focus\:border-s-success\/60:focus {
    border-inline-start-color: hsl(var(--su) / 0.6)}
.focus\:border-s-success\/70:focus {
    border-inline-start-color: hsl(var(--su) / 0.7)}
.focus\:border-s-success\/75:focus {
    border-inline-start-color: hsl(var(--su) / 0.75)}
.focus\:border-s-success\/80:focus {
    border-inline-start-color: hsl(var(--su) / 0.8)}
.focus\:border-s-success\/90:focus {
    border-inline-start-color: hsl(var(--su) / 0.9)}
.focus\:border-s-success\/95:focus {
    border-inline-start-color: hsl(var(--su) / 0.95)}
.focus\:border-s-warning:focus {
    border-inline-start-color: hsl(var(--wa) / 1)}
.focus\:border-s-warning-content:focus {
    border-inline-start-color: hsl(var(--wac) / 1)}
.focus\:border-s-warning-content\/0:focus {
    border-inline-start-color: hsl(var(--wac) / 0)}
.focus\:border-s-warning-content\/10:focus {
    border-inline-start-color: hsl(var(--wac) / 0.1)}
.focus\:border-s-warning-content\/100:focus {
    border-inline-start-color: hsl(var(--wac) / 1)}
.focus\:border-s-warning-content\/20:focus {
    border-inline-start-color: hsl(var(--wac) / 0.2)}
.focus\:border-s-warning-content\/25:focus {
    border-inline-start-color: hsl(var(--wac) / 0.25)}
.focus\:border-s-warning-content\/30:focus {
    border-inline-start-color: hsl(var(--wac) / 0.3)}
.focus\:border-s-warning-content\/40:focus {
    border-inline-start-color: hsl(var(--wac) / 0.4)}
.focus\:border-s-warning-content\/5:focus {
    border-inline-start-color: hsl(var(--wac) / 0.05)}
.focus\:border-s-warning-content\/50:focus {
    border-inline-start-color: hsl(var(--wac) / 0.5)}
.focus\:border-s-warning-content\/60:focus {
    border-inline-start-color: hsl(var(--wac) / 0.6)}
.focus\:border-s-warning-content\/70:focus {
    border-inline-start-color: hsl(var(--wac) / 0.7)}
.focus\:border-s-warning-content\/75:focus {
    border-inline-start-color: hsl(var(--wac) / 0.75)}
.focus\:border-s-warning-content\/80:focus {
    border-inline-start-color: hsl(var(--wac) / 0.8)}
.focus\:border-s-warning-content\/90:focus {
    border-inline-start-color: hsl(var(--wac) / 0.9)}
.focus\:border-s-warning-content\/95:focus {
    border-inline-start-color: hsl(var(--wac) / 0.95)}
.focus\:border-s-warning\/0:focus {
    border-inline-start-color: hsl(var(--wa) / 0)}
.focus\:border-s-warning\/10:focus {
    border-inline-start-color: hsl(var(--wa) / 0.1)}
.focus\:border-s-warning\/100:focus {
    border-inline-start-color: hsl(var(--wa) / 1)}
.focus\:border-s-warning\/20:focus {
    border-inline-start-color: hsl(var(--wa) / 0.2)}
.focus\:border-s-warning\/25:focus {
    border-inline-start-color: hsl(var(--wa) / 0.25)}
.focus\:border-s-warning\/30:focus {
    border-inline-start-color: hsl(var(--wa) / 0.3)}
.focus\:border-s-warning\/40:focus {
    border-inline-start-color: hsl(var(--wa) / 0.4)}
.focus\:border-s-warning\/5:focus {
    border-inline-start-color: hsl(var(--wa) / 0.05)}
.focus\:border-s-warning\/50:focus {
    border-inline-start-color: hsl(var(--wa) / 0.5)}
.focus\:border-s-warning\/60:focus {
    border-inline-start-color: hsl(var(--wa) / 0.6)}
.focus\:border-s-warning\/70:focus {
    border-inline-start-color: hsl(var(--wa) / 0.7)}
.focus\:border-s-warning\/75:focus {
    border-inline-start-color: hsl(var(--wa) / 0.75)}
.focus\:border-s-warning\/80:focus {
    border-inline-start-color: hsl(var(--wa) / 0.8)}
.focus\:border-s-warning\/90:focus {
    border-inline-start-color: hsl(var(--wa) / 0.9)}
.focus\:border-s-warning\/95:focus {
    border-inline-start-color: hsl(var(--wa) / 0.95)}
.focus\:border-t-base-100:focus {
    border-top-color: hsl(var(--b1) / 1)}
.focus\:border-t-base-100\/0:focus {
    border-top-color: hsl(var(--b1) / 0)}
.focus\:border-t-base-100\/10:focus {
    border-top-color: hsl(var(--b1) / 0.1)}
.focus\:border-t-base-100\/100:focus {
    border-top-color: hsl(var(--b1) / 1)}
.focus\:border-t-base-100\/20:focus {
    border-top-color: hsl(var(--b1) / 0.2)}
.focus\:border-t-base-100\/25:focus {
    border-top-color: hsl(var(--b1) / 0.25)}
.focus\:border-t-base-100\/30:focus {
    border-top-color: hsl(var(--b1) / 0.3)}
.focus\:border-t-base-100\/40:focus {
    border-top-color: hsl(var(--b1) / 0.4)}
.focus\:border-t-base-100\/5:focus {
    border-top-color: hsl(var(--b1) / 0.05)}
.focus\:border-t-base-100\/50:focus {
    border-top-color: hsl(var(--b1) / 0.5)}
.focus\:border-t-base-100\/60:focus {
    border-top-color: hsl(var(--b1) / 0.6)}
.focus\:border-t-base-100\/70:focus {
    border-top-color: hsl(var(--b1) / 0.7)}
.focus\:border-t-base-100\/75:focus {
    border-top-color: hsl(var(--b1) / 0.75)}
.focus\:border-t-base-100\/80:focus {
    border-top-color: hsl(var(--b1) / 0.8)}
.focus\:border-t-base-100\/90:focus {
    border-top-color: hsl(var(--b1) / 0.9)}
.focus\:border-t-base-100\/95:focus {
    border-top-color: hsl(var(--b1) / 0.95)}
.focus\:border-t-base-200:focus {
    border-top-color: hsl(var(--b2) / 1)}
.focus\:border-t-base-200\/0:focus {
    border-top-color: hsl(var(--b2) / 0)}
.focus\:border-t-base-200\/10:focus {
    border-top-color: hsl(var(--b2) / 0.1)}
.focus\:border-t-base-200\/100:focus {
    border-top-color: hsl(var(--b2) / 1)}
.focus\:border-t-base-200\/20:focus {
    border-top-color: hsl(var(--b2) / 0.2)}
.focus\:border-t-base-200\/25:focus {
    border-top-color: hsl(var(--b2) / 0.25)}
.focus\:border-t-base-200\/30:focus {
    border-top-color: hsl(var(--b2) / 0.3)}
.focus\:border-t-base-200\/40:focus {
    border-top-color: hsl(var(--b2) / 0.4)}
.focus\:border-t-base-200\/5:focus {
    border-top-color: hsl(var(--b2) / 0.05)}
.focus\:border-t-base-200\/50:focus {
    border-top-color: hsl(var(--b2) / 0.5)}
.focus\:border-t-base-200\/60:focus {
    border-top-color: hsl(var(--b2) / 0.6)}
.focus\:border-t-base-200\/70:focus {
    border-top-color: hsl(var(--b2) / 0.7)}
.focus\:border-t-base-200\/75:focus {
    border-top-color: hsl(var(--b2) / 0.75)}
.focus\:border-t-base-200\/80:focus {
    border-top-color: hsl(var(--b2) / 0.8)}
.focus\:border-t-base-200\/90:focus {
    border-top-color: hsl(var(--b2) / 0.9)}
.focus\:border-t-base-200\/95:focus {
    border-top-color: hsl(var(--b2) / 0.95)}
.focus\:border-t-base-300:focus {
    border-top-color: hsl(var(--b3) / 1)}
.focus\:border-t-base-300\/0:focus {
    border-top-color: hsl(var(--b3) / 0)}
.focus\:border-t-base-300\/10:focus {
    border-top-color: hsl(var(--b3) / 0.1)}
.focus\:border-t-base-300\/100:focus {
    border-top-color: hsl(var(--b3) / 1)}
.focus\:border-t-base-300\/20:focus {
    border-top-color: hsl(var(--b3) / 0.2)}
.focus\:border-t-base-300\/25:focus {
    border-top-color: hsl(var(--b3) / 0.25)}
.focus\:border-t-base-300\/30:focus {
    border-top-color: hsl(var(--b3) / 0.3)}
.focus\:border-t-base-300\/40:focus {
    border-top-color: hsl(var(--b3) / 0.4)}
.focus\:border-t-base-300\/5:focus {
    border-top-color: hsl(var(--b3) / 0.05)}
.focus\:border-t-base-300\/50:focus {
    border-top-color: hsl(var(--b3) / 0.5)}
.focus\:border-t-base-300\/60:focus {
    border-top-color: hsl(var(--b3) / 0.6)}
.focus\:border-t-base-300\/70:focus {
    border-top-color: hsl(var(--b3) / 0.7)}
.focus\:border-t-base-300\/75:focus {
    border-top-color: hsl(var(--b3) / 0.75)}
.focus\:border-t-base-300\/80:focus {
    border-top-color: hsl(var(--b3) / 0.8)}
.focus\:border-t-base-300\/90:focus {
    border-top-color: hsl(var(--b3) / 0.9)}
.focus\:border-t-base-300\/95:focus {
    border-top-color: hsl(var(--b3) / 0.95)}
.focus\:border-t-base-content:focus {
    border-top-color: hsl(var(--bc) / 1)}
.focus\:border-t-base-content\/0:focus {
    border-top-color: hsl(var(--bc) / 0)}
.focus\:border-t-base-content\/10:focus {
    border-top-color: hsl(var(--bc) / 0.1)}
.focus\:border-t-base-content\/100:focus {
    border-top-color: hsl(var(--bc) / 1)}
.focus\:border-t-base-content\/20:focus {
    border-top-color: hsl(var(--bc) / 0.2)}
.focus\:border-t-base-content\/25:focus {
    border-top-color: hsl(var(--bc) / 0.25)}
.focus\:border-t-base-content\/30:focus {
    border-top-color: hsl(var(--bc) / 0.3)}
.focus\:border-t-base-content\/40:focus {
    border-top-color: hsl(var(--bc) / 0.4)}
.focus\:border-t-base-content\/5:focus {
    border-top-color: hsl(var(--bc) / 0.05)}
.focus\:border-t-base-content\/50:focus {
    border-top-color: hsl(var(--bc) / 0.5)}
.focus\:border-t-base-content\/60:focus {
    border-top-color: hsl(var(--bc) / 0.6)}
.focus\:border-t-base-content\/70:focus {
    border-top-color: hsl(var(--bc) / 0.7)}
.focus\:border-t-base-content\/75:focus {
    border-top-color: hsl(var(--bc) / 0.75)}
.focus\:border-t-base-content\/80:focus {
    border-top-color: hsl(var(--bc) / 0.8)}
.focus\:border-t-base-content\/90:focus {
    border-top-color: hsl(var(--bc) / 0.9)}
.focus\:border-t-base-content\/95:focus {
    border-top-color: hsl(var(--bc) / 0.95)}
.focus\:border-t-error:focus {
    border-top-color: hsl(var(--er) / 1)}
.focus\:border-t-error-content:focus {
    border-top-color: hsl(var(--erc) / 1)}
.focus\:border-t-error-content\/0:focus {
    border-top-color: hsl(var(--erc) / 0)}
.focus\:border-t-error-content\/10:focus {
    border-top-color: hsl(var(--erc) / 0.1)}
.focus\:border-t-error-content\/100:focus {
    border-top-color: hsl(var(--erc) / 1)}
.focus\:border-t-error-content\/20:focus {
    border-top-color: hsl(var(--erc) / 0.2)}
.focus\:border-t-error-content\/25:focus {
    border-top-color: hsl(var(--erc) / 0.25)}
.focus\:border-t-error-content\/30:focus {
    border-top-color: hsl(var(--erc) / 0.3)}
.focus\:border-t-error-content\/40:focus {
    border-top-color: hsl(var(--erc) / 0.4)}
.focus\:border-t-error-content\/5:focus {
    border-top-color: hsl(var(--erc) / 0.05)}
.focus\:border-t-error-content\/50:focus {
    border-top-color: hsl(var(--erc) / 0.5)}
.focus\:border-t-error-content\/60:focus {
    border-top-color: hsl(var(--erc) / 0.6)}
.focus\:border-t-error-content\/70:focus {
    border-top-color: hsl(var(--erc) / 0.7)}
.focus\:border-t-error-content\/75:focus {
    border-top-color: hsl(var(--erc) / 0.75)}
.focus\:border-t-error-content\/80:focus {
    border-top-color: hsl(var(--erc) / 0.8)}
.focus\:border-t-error-content\/90:focus {
    border-top-color: hsl(var(--erc) / 0.9)}
.focus\:border-t-error-content\/95:focus {
    border-top-color: hsl(var(--erc) / 0.95)}
.focus\:border-t-error\/0:focus {
    border-top-color: hsl(var(--er) / 0)}
.focus\:border-t-error\/10:focus {
    border-top-color: hsl(var(--er) / 0.1)}
.focus\:border-t-error\/100:focus {
    border-top-color: hsl(var(--er) / 1)}
.focus\:border-t-error\/20:focus {
    border-top-color: hsl(var(--er) / 0.2)}
.focus\:border-t-error\/25:focus {
    border-top-color: hsl(var(--er) / 0.25)}
.focus\:border-t-error\/30:focus {
    border-top-color: hsl(var(--er) / 0.3)}
.focus\:border-t-error\/40:focus {
    border-top-color: hsl(var(--er) / 0.4)}
.focus\:border-t-error\/5:focus {
    border-top-color: hsl(var(--er) / 0.05)}
.focus\:border-t-error\/50:focus {
    border-top-color: hsl(var(--er) / 0.5)}
.focus\:border-t-error\/60:focus {
    border-top-color: hsl(var(--er) / 0.6)}
.focus\:border-t-error\/70:focus {
    border-top-color: hsl(var(--er) / 0.7)}
.focus\:border-t-error\/75:focus {
    border-top-color: hsl(var(--er) / 0.75)}
.focus\:border-t-error\/80:focus {
    border-top-color: hsl(var(--er) / 0.8)}
.focus\:border-t-error\/90:focus {
    border-top-color: hsl(var(--er) / 0.9)}
.focus\:border-t-error\/95:focus {
    border-top-color: hsl(var(--er) / 0.95)}
.focus\:border-t-info:focus {
    border-top-color: hsl(var(--in) / 1)}
.focus\:border-t-info-content:focus {
    border-top-color: hsl(var(--inc) / 1)}
.focus\:border-t-info-content\/0:focus {
    border-top-color: hsl(var(--inc) / 0)}
.focus\:border-t-info-content\/10:focus {
    border-top-color: hsl(var(--inc) / 0.1)}
.focus\:border-t-info-content\/100:focus {
    border-top-color: hsl(var(--inc) / 1)}
.focus\:border-t-info-content\/20:focus {
    border-top-color: hsl(var(--inc) / 0.2)}
.focus\:border-t-info-content\/25:focus {
    border-top-color: hsl(var(--inc) / 0.25)}
.focus\:border-t-info-content\/30:focus {
    border-top-color: hsl(var(--inc) / 0.3)}
.focus\:border-t-info-content\/40:focus {
    border-top-color: hsl(var(--inc) / 0.4)}
.focus\:border-t-info-content\/5:focus {
    border-top-color: hsl(var(--inc) / 0.05)}
.focus\:border-t-info-content\/50:focus {
    border-top-color: hsl(var(--inc) / 0.5)}
.focus\:border-t-info-content\/60:focus {
    border-top-color: hsl(var(--inc) / 0.6)}
.focus\:border-t-info-content\/70:focus {
    border-top-color: hsl(var(--inc) / 0.7)}
.focus\:border-t-info-content\/75:focus {
    border-top-color: hsl(var(--inc) / 0.75)}
.focus\:border-t-info-content\/80:focus {
    border-top-color: hsl(var(--inc) / 0.8)}
.focus\:border-t-info-content\/90:focus {
    border-top-color: hsl(var(--inc) / 0.9)}
.focus\:border-t-info-content\/95:focus {
    border-top-color: hsl(var(--inc) / 0.95)}
.focus\:border-t-info\/0:focus {
    border-top-color: hsl(var(--in) / 0)}
.focus\:border-t-info\/10:focus {
    border-top-color: hsl(var(--in) / 0.1)}
.focus\:border-t-info\/100:focus {
    border-top-color: hsl(var(--in) / 1)}
.focus\:border-t-info\/20:focus {
    border-top-color: hsl(var(--in) / 0.2)}
.focus\:border-t-info\/25:focus {
    border-top-color: hsl(var(--in) / 0.25)}
.focus\:border-t-info\/30:focus {
    border-top-color: hsl(var(--in) / 0.3)}
.focus\:border-t-info\/40:focus {
    border-top-color: hsl(var(--in) / 0.4)}
.focus\:border-t-info\/5:focus {
    border-top-color: hsl(var(--in) / 0.05)}
.focus\:border-t-info\/50:focus {
    border-top-color: hsl(var(--in) / 0.5)}
.focus\:border-t-info\/60:focus {
    border-top-color: hsl(var(--in) / 0.6)}
.focus\:border-t-info\/70:focus {
    border-top-color: hsl(var(--in) / 0.7)}
.focus\:border-t-info\/75:focus {
    border-top-color: hsl(var(--in) / 0.75)}
.focus\:border-t-info\/80:focus {
    border-top-color: hsl(var(--in) / 0.8)}
.focus\:border-t-info\/90:focus {
    border-top-color: hsl(var(--in) / 0.9)}
.focus\:border-t-info\/95:focus {
    border-top-color: hsl(var(--in) / 0.95)}
.focus\:border-t-success:focus {
    border-top-color: hsl(var(--su) / 1)}
.focus\:border-t-success-content:focus {
    border-top-color: hsl(var(--suc) / 1)}
.focus\:border-t-success-content\/0:focus {
    border-top-color: hsl(var(--suc) / 0)}
.focus\:border-t-success-content\/10:focus {
    border-top-color: hsl(var(--suc) / 0.1)}
.focus\:border-t-success-content\/100:focus {
    border-top-color: hsl(var(--suc) / 1)}
.focus\:border-t-success-content\/20:focus {
    border-top-color: hsl(var(--suc) / 0.2)}
.focus\:border-t-success-content\/25:focus {
    border-top-color: hsl(var(--suc) / 0.25)}
.focus\:border-t-success-content\/30:focus {
    border-top-color: hsl(var(--suc) / 0.3)}
.focus\:border-t-success-content\/40:focus {
    border-top-color: hsl(var(--suc) / 0.4)}
.focus\:border-t-success-content\/5:focus {
    border-top-color: hsl(var(--suc) / 0.05)}
.focus\:border-t-success-content\/50:focus {
    border-top-color: hsl(var(--suc) / 0.5)}
.focus\:border-t-success-content\/60:focus {
    border-top-color: hsl(var(--suc) / 0.6)}
.focus\:border-t-success-content\/70:focus {
    border-top-color: hsl(var(--suc) / 0.7)}
.focus\:border-t-success-content\/75:focus {
    border-top-color: hsl(var(--suc) / 0.75)}
.focus\:border-t-success-content\/80:focus {
    border-top-color: hsl(var(--suc) / 0.8)}
.focus\:border-t-success-content\/90:focus {
    border-top-color: hsl(var(--suc) / 0.9)}
.focus\:border-t-success-content\/95:focus {
    border-top-color: hsl(var(--suc) / 0.95)}
.focus\:border-t-success\/0:focus {
    border-top-color: hsl(var(--su) / 0)}
.focus\:border-t-success\/10:focus {
    border-top-color: hsl(var(--su) / 0.1)}
.focus\:border-t-success\/100:focus {
    border-top-color: hsl(var(--su) / 1)}
.focus\:border-t-success\/20:focus {
    border-top-color: hsl(var(--su) / 0.2)}
.focus\:border-t-success\/25:focus {
    border-top-color: hsl(var(--su) / 0.25)}
.focus\:border-t-success\/30:focus {
    border-top-color: hsl(var(--su) / 0.3)}
.focus\:border-t-success\/40:focus {
    border-top-color: hsl(var(--su) / 0.4)}
.focus\:border-t-success\/5:focus {
    border-top-color: hsl(var(--su) / 0.05)}
.focus\:border-t-success\/50:focus {
    border-top-color: hsl(var(--su) / 0.5)}
.focus\:border-t-success\/60:focus {
    border-top-color: hsl(var(--su) / 0.6)}
.focus\:border-t-success\/70:focus {
    border-top-color: hsl(var(--su) / 0.7)}
.focus\:border-t-success\/75:focus {
    border-top-color: hsl(var(--su) / 0.75)}
.focus\:border-t-success\/80:focus {
    border-top-color: hsl(var(--su) / 0.8)}
.focus\:border-t-success\/90:focus {
    border-top-color: hsl(var(--su) / 0.9)}
.focus\:border-t-success\/95:focus {
    border-top-color: hsl(var(--su) / 0.95)}
.focus\:border-t-warning:focus {
    border-top-color: hsl(var(--wa) / 1)}
.focus\:border-t-warning-content:focus {
    border-top-color: hsl(var(--wac) / 1)}
.focus\:border-t-warning-content\/0:focus {
    border-top-color: hsl(var(--wac) / 0)}
.focus\:border-t-warning-content\/10:focus {
    border-top-color: hsl(var(--wac) / 0.1)}
.focus\:border-t-warning-content\/100:focus {
    border-top-color: hsl(var(--wac) / 1)}
.focus\:border-t-warning-content\/20:focus {
    border-top-color: hsl(var(--wac) / 0.2)}
.focus\:border-t-warning-content\/25:focus {
    border-top-color: hsl(var(--wac) / 0.25)}
.focus\:border-t-warning-content\/30:focus {
    border-top-color: hsl(var(--wac) / 0.3)}
.focus\:border-t-warning-content\/40:focus {
    border-top-color: hsl(var(--wac) / 0.4)}
.focus\:border-t-warning-content\/5:focus {
    border-top-color: hsl(var(--wac) / 0.05)}
.focus\:border-t-warning-content\/50:focus {
    border-top-color: hsl(var(--wac) / 0.5)}
.focus\:border-t-warning-content\/60:focus {
    border-top-color: hsl(var(--wac) / 0.6)}
.focus\:border-t-warning-content\/70:focus {
    border-top-color: hsl(var(--wac) / 0.7)}
.focus\:border-t-warning-content\/75:focus {
    border-top-color: hsl(var(--wac) / 0.75)}
.focus\:border-t-warning-content\/80:focus {
    border-top-color: hsl(var(--wac) / 0.8)}
.focus\:border-t-warning-content\/90:focus {
    border-top-color: hsl(var(--wac) / 0.9)}
.focus\:border-t-warning-content\/95:focus {
    border-top-color: hsl(var(--wac) / 0.95)}
.focus\:border-t-warning\/0:focus {
    border-top-color: hsl(var(--wa) / 0)}
.focus\:border-t-warning\/10:focus {
    border-top-color: hsl(var(--wa) / 0.1)}
.focus\:border-t-warning\/100:focus {
    border-top-color: hsl(var(--wa) / 1)}
.focus\:border-t-warning\/20:focus {
    border-top-color: hsl(var(--wa) / 0.2)}
.focus\:border-t-warning\/25:focus {
    border-top-color: hsl(var(--wa) / 0.25)}
.focus\:border-t-warning\/30:focus {
    border-top-color: hsl(var(--wa) / 0.3)}
.focus\:border-t-warning\/40:focus {
    border-top-color: hsl(var(--wa) / 0.4)}
.focus\:border-t-warning\/5:focus {
    border-top-color: hsl(var(--wa) / 0.05)}
.focus\:border-t-warning\/50:focus {
    border-top-color: hsl(var(--wa) / 0.5)}
.focus\:border-t-warning\/60:focus {
    border-top-color: hsl(var(--wa) / 0.6)}
.focus\:border-t-warning\/70:focus {
    border-top-color: hsl(var(--wa) / 0.7)}
.focus\:border-t-warning\/75:focus {
    border-top-color: hsl(var(--wa) / 0.75)}
.focus\:border-t-warning\/80:focus {
    border-top-color: hsl(var(--wa) / 0.8)}
.focus\:border-t-warning\/90:focus {
    border-top-color: hsl(var(--wa) / 0.9)}
.focus\:border-t-warning\/95:focus {
    border-top-color: hsl(var(--wa) / 0.95)}
.focus\:bg-accent:focus {
    background-color: hsl(var(--a) / 1)}
.focus\:bg-accent-content:focus {
    background-color: hsl(var(--ac) / 1)}
.focus\:bg-accent-content\/0:focus {
    background-color: hsl(var(--ac) / 0)}
.focus\:bg-accent-content\/10:focus {
    background-color: hsl(var(--ac) / 0.1)}
.focus\:bg-accent-content\/100:focus {
    background-color: hsl(var(--ac) / 1)}
.focus\:bg-accent-content\/20:focus {
    background-color: hsl(var(--ac) / 0.2)}
.focus\:bg-accent-content\/25:focus {
    background-color: hsl(var(--ac) / 0.25)}
.focus\:bg-accent-content\/30:focus {
    background-color: hsl(var(--ac) / 0.3)}
.focus\:bg-accent-content\/40:focus {
    background-color: hsl(var(--ac) / 0.4)}
.focus\:bg-accent-content\/5:focus {
    background-color: hsl(var(--ac) / 0.05)}
.focus\:bg-accent-content\/50:focus {
    background-color: hsl(var(--ac) / 0.5)}
.focus\:bg-accent-content\/60:focus {
    background-color: hsl(var(--ac) / 0.6)}
.focus\:bg-accent-content\/70:focus {
    background-color: hsl(var(--ac) / 0.7)}
.focus\:bg-accent-content\/75:focus {
    background-color: hsl(var(--ac) / 0.75)}
.focus\:bg-accent-content\/80:focus {
    background-color: hsl(var(--ac) / 0.8)}
.focus\:bg-accent-content\/90:focus {
    background-color: hsl(var(--ac) / 0.9)}
.focus\:bg-accent-content\/95:focus {
    background-color: hsl(var(--ac) / 0.95)}
.focus\:bg-accent-focus:focus {
    background-color: hsl(var(--af) / 1)}
.focus\:bg-accent-focus\/0:focus {
    background-color: hsl(var(--af) / 0)}
.focus\:bg-accent-focus\/10:focus {
    background-color: hsl(var(--af) / 0.1)}
.focus\:bg-accent-focus\/100:focus {
    background-color: hsl(var(--af) / 1)}
.focus\:bg-accent-focus\/20:focus {
    background-color: hsl(var(--af) / 0.2)}
.focus\:bg-accent-focus\/25:focus {
    background-color: hsl(var(--af) / 0.25)}
.focus\:bg-accent-focus\/30:focus {
    background-color: hsl(var(--af) / 0.3)}
.focus\:bg-accent-focus\/40:focus {
    background-color: hsl(var(--af) / 0.4)}
.focus\:bg-accent-focus\/5:focus {
    background-color: hsl(var(--af) / 0.05)}
.focus\:bg-accent-focus\/50:focus {
    background-color: hsl(var(--af) / 0.5)}
.focus\:bg-accent-focus\/60:focus {
    background-color: hsl(var(--af) / 0.6)}
.focus\:bg-accent-focus\/70:focus {
    background-color: hsl(var(--af) / 0.7)}
.focus\:bg-accent-focus\/75:focus {
    background-color: hsl(var(--af) / 0.75)}
.focus\:bg-accent-focus\/80:focus {
    background-color: hsl(var(--af) / 0.8)}
.focus\:bg-accent-focus\/90:focus {
    background-color: hsl(var(--af) / 0.9)}
.focus\:bg-accent-focus\/95:focus {
    background-color: hsl(var(--af) / 0.95)}
.focus\:bg-accent\/0:focus {
    background-color: hsl(var(--a) / 0)}
.focus\:bg-accent\/10:focus {
    background-color: hsl(var(--a) / 0.1)}
.focus\:bg-accent\/100:focus {
    background-color: hsl(var(--a) / 1)}
.focus\:bg-accent\/20:focus {
    background-color: hsl(var(--a) / 0.2)}
.focus\:bg-accent\/25:focus {
    background-color: hsl(var(--a) / 0.25)}
.focus\:bg-accent\/30:focus {
    background-color: hsl(var(--a) / 0.3)}
.focus\:bg-accent\/40:focus {
    background-color: hsl(var(--a) / 0.4)}
.focus\:bg-accent\/5:focus {
    background-color: hsl(var(--a) / 0.05)}
.focus\:bg-accent\/50:focus {
    background-color: hsl(var(--a) / 0.5)}
.focus\:bg-accent\/60:focus {
    background-color: hsl(var(--a) / 0.6)}
.focus\:bg-accent\/70:focus {
    background-color: hsl(var(--a) / 0.7)}
.focus\:bg-accent\/75:focus {
    background-color: hsl(var(--a) / 0.75)}
.focus\:bg-accent\/80:focus {
    background-color: hsl(var(--a) / 0.8)}
.focus\:bg-accent\/90:focus {
    background-color: hsl(var(--a) / 0.9)}
.focus\:bg-accent\/95:focus {
    background-color: hsl(var(--a) / 0.95)}
.focus\:bg-base-100:focus {
    background-color: hsl(var(--b1) / 1)}
.focus\:bg-base-100\/0:focus {
    background-color: hsl(var(--b1) / 0)}
.focus\:bg-base-100\/10:focus {
    background-color: hsl(var(--b1) / 0.1)}
.focus\:bg-base-100\/100:focus {
    background-color: hsl(var(--b1) / 1)}
.focus\:bg-base-100\/20:focus {
    background-color: hsl(var(--b1) / 0.2)}
.focus\:bg-base-100\/25:focus {
    background-color: hsl(var(--b1) / 0.25)}
.focus\:bg-base-100\/30:focus {
    background-color: hsl(var(--b1) / 0.3)}
.focus\:bg-base-100\/40:focus {
    background-color: hsl(var(--b1) / 0.4)}
.focus\:bg-base-100\/5:focus {
    background-color: hsl(var(--b1) / 0.05)}
.focus\:bg-base-100\/50:focus {
    background-color: hsl(var(--b1) / 0.5)}
.focus\:bg-base-100\/60:focus {
    background-color: hsl(var(--b1) / 0.6)}
.focus\:bg-base-100\/70:focus {
    background-color: hsl(var(--b1) / 0.7)}
.focus\:bg-base-100\/75:focus {
    background-color: hsl(var(--b1) / 0.75)}
.focus\:bg-base-100\/80:focus {
    background-color: hsl(var(--b1) / 0.8)}
.focus\:bg-base-100\/90:focus {
    background-color: hsl(var(--b1) / 0.9)}
.focus\:bg-base-100\/95:focus {
    background-color: hsl(var(--b1) / 0.95)}
.focus\:bg-base-200:focus {
    background-color: hsl(var(--b2) / 1)}
.focus\:bg-base-200\/0:focus {
    background-color: hsl(var(--b2) / 0)}
.focus\:bg-base-200\/10:focus {
    background-color: hsl(var(--b2) / 0.1)}
.focus\:bg-base-200\/100:focus {
    background-color: hsl(var(--b2) / 1)}
.focus\:bg-base-200\/20:focus {
    background-color: hsl(var(--b2) / 0.2)}
.focus\:bg-base-200\/25:focus {
    background-color: hsl(var(--b2) / 0.25)}
.focus\:bg-base-200\/30:focus {
    background-color: hsl(var(--b2) / 0.3)}
.focus\:bg-base-200\/40:focus {
    background-color: hsl(var(--b2) / 0.4)}
.focus\:bg-base-200\/5:focus {
    background-color: hsl(var(--b2) / 0.05)}
.focus\:bg-base-200\/50:focus {
    background-color: hsl(var(--b2) / 0.5)}
.focus\:bg-base-200\/60:focus {
    background-color: hsl(var(--b2) / 0.6)}
.focus\:bg-base-200\/70:focus {
    background-color: hsl(var(--b2) / 0.7)}
.focus\:bg-base-200\/75:focus {
    background-color: hsl(var(--b2) / 0.75)}
.focus\:bg-base-200\/80:focus {
    background-color: hsl(var(--b2) / 0.8)}
.focus\:bg-base-200\/90:focus {
    background-color: hsl(var(--b2) / 0.9)}
.focus\:bg-base-200\/95:focus {
    background-color: hsl(var(--b2) / 0.95)}
.focus\:bg-base-300:focus {
    background-color: hsl(var(--b3) / 1)}
.focus\:bg-base-300\/0:focus {
    background-color: hsl(var(--b3) / 0)}
.focus\:bg-base-300\/10:focus {
    background-color: hsl(var(--b3) / 0.1)}
.focus\:bg-base-300\/100:focus {
    background-color: hsl(var(--b3) / 1)}
.focus\:bg-base-300\/20:focus {
    background-color: hsl(var(--b3) / 0.2)}
.focus\:bg-base-300\/25:focus {
    background-color: hsl(var(--b3) / 0.25)}
.focus\:bg-base-300\/30:focus {
    background-color: hsl(var(--b3) / 0.3)}
.focus\:bg-base-300\/40:focus {
    background-color: hsl(var(--b3) / 0.4)}
.focus\:bg-base-300\/5:focus {
    background-color: hsl(var(--b3) / 0.05)}
.focus\:bg-base-300\/50:focus {
    background-color: hsl(var(--b3) / 0.5)}
.focus\:bg-base-300\/60:focus {
    background-color: hsl(var(--b3) / 0.6)}
.focus\:bg-base-300\/70:focus {
    background-color: hsl(var(--b3) / 0.7)}
.focus\:bg-base-300\/75:focus {
    background-color: hsl(var(--b3) / 0.75)}
.focus\:bg-base-300\/80:focus {
    background-color: hsl(var(--b3) / 0.8)}
.focus\:bg-base-300\/90:focus {
    background-color: hsl(var(--b3) / 0.9)}
.focus\:bg-base-300\/95:focus {
    background-color: hsl(var(--b3) / 0.95)}
.focus\:bg-base-content:focus {
    background-color: hsl(var(--bc) / 1)}
.focus\:bg-base-content\/0:focus {
    background-color: hsl(var(--bc) / 0)}
.focus\:bg-base-content\/10:focus {
    background-color: hsl(var(--bc) / 0.1)}
.focus\:bg-base-content\/100:focus {
    background-color: hsl(var(--bc) / 1)}
.focus\:bg-base-content\/20:focus {
    background-color: hsl(var(--bc) / 0.2)}
.focus\:bg-base-content\/25:focus {
    background-color: hsl(var(--bc) / 0.25)}
.focus\:bg-base-content\/30:focus {
    background-color: hsl(var(--bc) / 0.3)}
.focus\:bg-base-content\/40:focus {
    background-color: hsl(var(--bc) / 0.4)}
.focus\:bg-base-content\/5:focus {
    background-color: hsl(var(--bc) / 0.05)}
.focus\:bg-base-content\/50:focus {
    background-color: hsl(var(--bc) / 0.5)}
.focus\:bg-base-content\/60:focus {
    background-color: hsl(var(--bc) / 0.6)}
.focus\:bg-base-content\/70:focus {
    background-color: hsl(var(--bc) / 0.7)}
.focus\:bg-base-content\/75:focus {
    background-color: hsl(var(--bc) / 0.75)}
.focus\:bg-base-content\/80:focus {
    background-color: hsl(var(--bc) / 0.8)}
.focus\:bg-base-content\/90:focus {
    background-color: hsl(var(--bc) / 0.9)}
.focus\:bg-base-content\/95:focus {
    background-color: hsl(var(--bc) / 0.95)}
.focus\:bg-error:focus {
    background-color: hsl(var(--er) / 1)}
.focus\:bg-error-content:focus {
    background-color: hsl(var(--erc) / 1)}
.focus\:bg-error-content\/0:focus {
    background-color: hsl(var(--erc) / 0)}
.focus\:bg-error-content\/10:focus {
    background-color: hsl(var(--erc) / 0.1)}
.focus\:bg-error-content\/100:focus {
    background-color: hsl(var(--erc) / 1)}
.focus\:bg-error-content\/20:focus {
    background-color: hsl(var(--erc) / 0.2)}
.focus\:bg-error-content\/25:focus {
    background-color: hsl(var(--erc) / 0.25)}
.focus\:bg-error-content\/30:focus {
    background-color: hsl(var(--erc) / 0.3)}
.focus\:bg-error-content\/40:focus {
    background-color: hsl(var(--erc) / 0.4)}
.focus\:bg-error-content\/5:focus {
    background-color: hsl(var(--erc) / 0.05)}
.focus\:bg-error-content\/50:focus {
    background-color: hsl(var(--erc) / 0.5)}
.focus\:bg-error-content\/60:focus {
    background-color: hsl(var(--erc) / 0.6)}
.focus\:bg-error-content\/70:focus {
    background-color: hsl(var(--erc) / 0.7)}
.focus\:bg-error-content\/75:focus {
    background-color: hsl(var(--erc) / 0.75)}
.focus\:bg-error-content\/80:focus {
    background-color: hsl(var(--erc) / 0.8)}
.focus\:bg-error-content\/90:focus {
    background-color: hsl(var(--erc) / 0.9)}
.focus\:bg-error-content\/95:focus {
    background-color: hsl(var(--erc) / 0.95)}
.focus\:bg-error\/0:focus {
    background-color: hsl(var(--er) / 0)}
.focus\:bg-error\/10:focus {
    background-color: hsl(var(--er) / 0.1)}
.focus\:bg-error\/100:focus {
    background-color: hsl(var(--er) / 1)}
.focus\:bg-error\/20:focus {
    background-color: hsl(var(--er) / 0.2)}
.focus\:bg-error\/25:focus {
    background-color: hsl(var(--er) / 0.25)}
.focus\:bg-error\/30:focus {
    background-color: hsl(var(--er) / 0.3)}
.focus\:bg-error\/40:focus {
    background-color: hsl(var(--er) / 0.4)}
.focus\:bg-error\/5:focus {
    background-color: hsl(var(--er) / 0.05)}
.focus\:bg-error\/50:focus {
    background-color: hsl(var(--er) / 0.5)}
.focus\:bg-error\/60:focus {
    background-color: hsl(var(--er) / 0.6)}
.focus\:bg-error\/70:focus {
    background-color: hsl(var(--er) / 0.7)}
.focus\:bg-error\/75:focus {
    background-color: hsl(var(--er) / 0.75)}
.focus\:bg-error\/80:focus {
    background-color: hsl(var(--er) / 0.8)}
.focus\:bg-error\/90:focus {
    background-color: hsl(var(--er) / 0.9)}
.focus\:bg-error\/95:focus {
    background-color: hsl(var(--er) / 0.95)}
.focus\:bg-info:focus {
    background-color: hsl(var(--in) / 1)}
.focus\:bg-info-content:focus {
    background-color: hsl(var(--inc) / 1)}
.focus\:bg-info-content\/0:focus {
    background-color: hsl(var(--inc) / 0)}
.focus\:bg-info-content\/10:focus {
    background-color: hsl(var(--inc) / 0.1)}
.focus\:bg-info-content\/100:focus {
    background-color: hsl(var(--inc) / 1)}
.focus\:bg-info-content\/20:focus {
    background-color: hsl(var(--inc) / 0.2)}
.focus\:bg-info-content\/25:focus {
    background-color: hsl(var(--inc) / 0.25)}
.focus\:bg-info-content\/30:focus {
    background-color: hsl(var(--inc) / 0.3)}
.focus\:bg-info-content\/40:focus {
    background-color: hsl(var(--inc) / 0.4)}
.focus\:bg-info-content\/5:focus {
    background-color: hsl(var(--inc) / 0.05)}
.focus\:bg-info-content\/50:focus {
    background-color: hsl(var(--inc) / 0.5)}
.focus\:bg-info-content\/60:focus {
    background-color: hsl(var(--inc) / 0.6)}
.focus\:bg-info-content\/70:focus {
    background-color: hsl(var(--inc) / 0.7)}
.focus\:bg-info-content\/75:focus {
    background-color: hsl(var(--inc) / 0.75)}
.focus\:bg-info-content\/80:focus {
    background-color: hsl(var(--inc) / 0.8)}
.focus\:bg-info-content\/90:focus {
    background-color: hsl(var(--inc) / 0.9)}
.focus\:bg-info-content\/95:focus {
    background-color: hsl(var(--inc) / 0.95)}
.focus\:bg-info\/0:focus {
    background-color: hsl(var(--in) / 0)}
.focus\:bg-info\/10:focus {
    background-color: hsl(var(--in) / 0.1)}
.focus\:bg-info\/100:focus {
    background-color: hsl(var(--in) / 1)}
.focus\:bg-info\/20:focus {
    background-color: hsl(var(--in) / 0.2)}
.focus\:bg-info\/25:focus {
    background-color: hsl(var(--in) / 0.25)}
.focus\:bg-info\/30:focus {
    background-color: hsl(var(--in) / 0.3)}
.focus\:bg-info\/40:focus {
    background-color: hsl(var(--in) / 0.4)}
.focus\:bg-info\/5:focus {
    background-color: hsl(var(--in) / 0.05)}
.focus\:bg-info\/50:focus {
    background-color: hsl(var(--in) / 0.5)}
.focus\:bg-info\/60:focus {
    background-color: hsl(var(--in) / 0.6)}
.focus\:bg-info\/70:focus {
    background-color: hsl(var(--in) / 0.7)}
.focus\:bg-info\/75:focus {
    background-color: hsl(var(--in) / 0.75)}
.focus\:bg-info\/80:focus {
    background-color: hsl(var(--in) / 0.8)}
.focus\:bg-info\/90:focus {
    background-color: hsl(var(--in) / 0.9)}
.focus\:bg-info\/95:focus {
    background-color: hsl(var(--in) / 0.95)}
.focus\:bg-neutral:focus {
    background-color: hsl(var(--n) / 1)}
.focus\:bg-neutral-content:focus {
    background-color: hsl(var(--nc) / 1)}
.focus\:bg-neutral-content\/0:focus {
    background-color: hsl(var(--nc) / 0)}
.focus\:bg-neutral-content\/10:focus {
    background-color: hsl(var(--nc) / 0.1)}
.focus\:bg-neutral-content\/100:focus {
    background-color: hsl(var(--nc) / 1)}
.focus\:bg-neutral-content\/20:focus {
    background-color: hsl(var(--nc) / 0.2)}
.focus\:bg-neutral-content\/25:focus {
    background-color: hsl(var(--nc) / 0.25)}
.focus\:bg-neutral-content\/30:focus {
    background-color: hsl(var(--nc) / 0.3)}
.focus\:bg-neutral-content\/40:focus {
    background-color: hsl(var(--nc) / 0.4)}
.focus\:bg-neutral-content\/5:focus {
    background-color: hsl(var(--nc) / 0.05)}
.focus\:bg-neutral-content\/50:focus {
    background-color: hsl(var(--nc) / 0.5)}
.focus\:bg-neutral-content\/60:focus {
    background-color: hsl(var(--nc) / 0.6)}
.focus\:bg-neutral-content\/70:focus {
    background-color: hsl(var(--nc) / 0.7)}
.focus\:bg-neutral-content\/75:focus {
    background-color: hsl(var(--nc) / 0.75)}
.focus\:bg-neutral-content\/80:focus {
    background-color: hsl(var(--nc) / 0.8)}
.focus\:bg-neutral-content\/90:focus {
    background-color: hsl(var(--nc) / 0.9)}
.focus\:bg-neutral-content\/95:focus {
    background-color: hsl(var(--nc) / 0.95)}
.focus\:bg-neutral-focus:focus {
    background-color: hsl(var(--nf) / 1)}
.focus\:bg-neutral-focus\/0:focus {
    background-color: hsl(var(--nf) / 0)}
.focus\:bg-neutral-focus\/10:focus {
    background-color: hsl(var(--nf) / 0.1)}
.focus\:bg-neutral-focus\/100:focus {
    background-color: hsl(var(--nf) / 1)}
.focus\:bg-neutral-focus\/20:focus {
    background-color: hsl(var(--nf) / 0.2)}
.focus\:bg-neutral-focus\/25:focus {
    background-color: hsl(var(--nf) / 0.25)}
.focus\:bg-neutral-focus\/30:focus {
    background-color: hsl(var(--nf) / 0.3)}
.focus\:bg-neutral-focus\/40:focus {
    background-color: hsl(var(--nf) / 0.4)}
.focus\:bg-neutral-focus\/5:focus {
    background-color: hsl(var(--nf) / 0.05)}
.focus\:bg-neutral-focus\/50:focus {
    background-color: hsl(var(--nf) / 0.5)}
.focus\:bg-neutral-focus\/60:focus {
    background-color: hsl(var(--nf) / 0.6)}
.focus\:bg-neutral-focus\/70:focus {
    background-color: hsl(var(--nf) / 0.7)}
.focus\:bg-neutral-focus\/75:focus {
    background-color: hsl(var(--nf) / 0.75)}
.focus\:bg-neutral-focus\/80:focus {
    background-color: hsl(var(--nf) / 0.8)}
.focus\:bg-neutral-focus\/90:focus {
    background-color: hsl(var(--nf) / 0.9)}
.focus\:bg-neutral-focus\/95:focus {
    background-color: hsl(var(--nf) / 0.95)}
.focus\:bg-neutral\/0:focus {
    background-color: hsl(var(--n) / 0)}
.focus\:bg-neutral\/10:focus {
    background-color: hsl(var(--n) / 0.1)}
.focus\:bg-neutral\/100:focus {
    background-color: hsl(var(--n) / 1)}
.focus\:bg-neutral\/20:focus {
    background-color: hsl(var(--n) / 0.2)}
.focus\:bg-neutral\/25:focus {
    background-color: hsl(var(--n) / 0.25)}
.focus\:bg-neutral\/30:focus {
    background-color: hsl(var(--n) / 0.3)}
.focus\:bg-neutral\/40:focus {
    background-color: hsl(var(--n) / 0.4)}
.focus\:bg-neutral\/5:focus {
    background-color: hsl(var(--n) / 0.05)}
.focus\:bg-neutral\/50:focus {
    background-color: hsl(var(--n) / 0.5)}
.focus\:bg-neutral\/60:focus {
    background-color: hsl(var(--n) / 0.6)}
.focus\:bg-neutral\/70:focus {
    background-color: hsl(var(--n) / 0.7)}
.focus\:bg-neutral\/75:focus {
    background-color: hsl(var(--n) / 0.75)}
.focus\:bg-neutral\/80:focus {
    background-color: hsl(var(--n) / 0.8)}
.focus\:bg-neutral\/90:focus {
    background-color: hsl(var(--n) / 0.9)}
.focus\:bg-neutral\/95:focus {
    background-color: hsl(var(--n) / 0.95)}
.focus\:bg-primary:focus {
    background-color: hsl(var(--p) / 1)}
.focus\:bg-primary-content:focus {
    background-color: hsl(var(--pc) / 1)}
.focus\:bg-primary-content\/0:focus {
    background-color: hsl(var(--pc) / 0)}
.focus\:bg-primary-content\/10:focus {
    background-color: hsl(var(--pc) / 0.1)}
.focus\:bg-primary-content\/100:focus {
    background-color: hsl(var(--pc) / 1)}
.focus\:bg-primary-content\/20:focus {
    background-color: hsl(var(--pc) / 0.2)}
.focus\:bg-primary-content\/25:focus {
    background-color: hsl(var(--pc) / 0.25)}
.focus\:bg-primary-content\/30:focus {
    background-color: hsl(var(--pc) / 0.3)}
.focus\:bg-primary-content\/40:focus {
    background-color: hsl(var(--pc) / 0.4)}
.focus\:bg-primary-content\/5:focus {
    background-color: hsl(var(--pc) / 0.05)}
.focus\:bg-primary-content\/50:focus {
    background-color: hsl(var(--pc) / 0.5)}
.focus\:bg-primary-content\/60:focus {
    background-color: hsl(var(--pc) / 0.6)}
.focus\:bg-primary-content\/70:focus {
    background-color: hsl(var(--pc) / 0.7)}
.focus\:bg-primary-content\/75:focus {
    background-color: hsl(var(--pc) / 0.75)}
.focus\:bg-primary-content\/80:focus {
    background-color: hsl(var(--pc) / 0.8)}
.focus\:bg-primary-content\/90:focus {
    background-color: hsl(var(--pc) / 0.9)}
.focus\:bg-primary-content\/95:focus {
    background-color: hsl(var(--pc) / 0.95)}
.focus\:bg-primary-focus:focus {
    background-color: hsl(var(--pf) / 1)}
.focus\:bg-primary-focus\/0:focus {
    background-color: hsl(var(--pf) / 0)}
.focus\:bg-primary-focus\/10:focus {
    background-color: hsl(var(--pf) / 0.1)}
.focus\:bg-primary-focus\/100:focus {
    background-color: hsl(var(--pf) / 1)}
.focus\:bg-primary-focus\/20:focus {
    background-color: hsl(var(--pf) / 0.2)}
.focus\:bg-primary-focus\/25:focus {
    background-color: hsl(var(--pf) / 0.25)}
.focus\:bg-primary-focus\/30:focus {
    background-color: hsl(var(--pf) / 0.3)}
.focus\:bg-primary-focus\/40:focus {
    background-color: hsl(var(--pf) / 0.4)}
.focus\:bg-primary-focus\/5:focus {
    background-color: hsl(var(--pf) / 0.05)}
.focus\:bg-primary-focus\/50:focus {
    background-color: hsl(var(--pf) / 0.5)}
.focus\:bg-primary-focus\/60:focus {
    background-color: hsl(var(--pf) / 0.6)}
.focus\:bg-primary-focus\/70:focus {
    background-color: hsl(var(--pf) / 0.7)}
.focus\:bg-primary-focus\/75:focus {
    background-color: hsl(var(--pf) / 0.75)}
.focus\:bg-primary-focus\/80:focus {
    background-color: hsl(var(--pf) / 0.8)}
.focus\:bg-primary-focus\/90:focus {
    background-color: hsl(var(--pf) / 0.9)}
.focus\:bg-primary-focus\/95:focus {
    background-color: hsl(var(--pf) / 0.95)}
.focus\:bg-primary\/0:focus {
    background-color: hsl(var(--p) / 0)}
.focus\:bg-primary\/10:focus {
    background-color: hsl(var(--p) / 0.1)}
.focus\:bg-primary\/100:focus {
    background-color: hsl(var(--p) / 1)}
.focus\:bg-primary\/20:focus {
    background-color: hsl(var(--p) / 0.2)}
.focus\:bg-primary\/25:focus {
    background-color: hsl(var(--p) / 0.25)}
.focus\:bg-primary\/30:focus {
    background-color: hsl(var(--p) / 0.3)}
.focus\:bg-primary\/40:focus {
    background-color: hsl(var(--p) / 0.4)}
.focus\:bg-primary\/5:focus {
    background-color: hsl(var(--p) / 0.05)}
.focus\:bg-primary\/50:focus {
    background-color: hsl(var(--p) / 0.5)}
.focus\:bg-primary\/60:focus {
    background-color: hsl(var(--p) / 0.6)}
.focus\:bg-primary\/70:focus {
    background-color: hsl(var(--p) / 0.7)}
.focus\:bg-primary\/75:focus {
    background-color: hsl(var(--p) / 0.75)}
.focus\:bg-primary\/80:focus {
    background-color: hsl(var(--p) / 0.8)}
.focus\:bg-primary\/90:focus {
    background-color: hsl(var(--p) / 0.9)}
.focus\:bg-primary\/95:focus {
    background-color: hsl(var(--p) / 0.95)}
.focus\:bg-secondary:focus {
    background-color: hsl(var(--s) / 1)}
.focus\:bg-secondary-content:focus {
    background-color: hsl(var(--sc) / 1)}
.focus\:bg-secondary-content\/0:focus {
    background-color: hsl(var(--sc) / 0)}
.focus\:bg-secondary-content\/10:focus {
    background-color: hsl(var(--sc) / 0.1)}
.focus\:bg-secondary-content\/100:focus {
    background-color: hsl(var(--sc) / 1)}
.focus\:bg-secondary-content\/20:focus {
    background-color: hsl(var(--sc) / 0.2)}
.focus\:bg-secondary-content\/25:focus {
    background-color: hsl(var(--sc) / 0.25)}
.focus\:bg-secondary-content\/30:focus {
    background-color: hsl(var(--sc) / 0.3)}
.focus\:bg-secondary-content\/40:focus {
    background-color: hsl(var(--sc) / 0.4)}
.focus\:bg-secondary-content\/5:focus {
    background-color: hsl(var(--sc) / 0.05)}
.focus\:bg-secondary-content\/50:focus {
    background-color: hsl(var(--sc) / 0.5)}
.focus\:bg-secondary-content\/60:focus {
    background-color: hsl(var(--sc) / 0.6)}
.focus\:bg-secondary-content\/70:focus {
    background-color: hsl(var(--sc) / 0.7)}
.focus\:bg-secondary-content\/75:focus {
    background-color: hsl(var(--sc) / 0.75)}
.focus\:bg-secondary-content\/80:focus {
    background-color: hsl(var(--sc) / 0.8)}
.focus\:bg-secondary-content\/90:focus {
    background-color: hsl(var(--sc) / 0.9)}
.focus\:bg-secondary-content\/95:focus {
    background-color: hsl(var(--sc) / 0.95)}
.focus\:bg-secondary-focus:focus {
    background-color: hsl(var(--sf) / 1)}
.focus\:bg-secondary-focus\/0:focus {
    background-color: hsl(var(--sf) / 0)}
.focus\:bg-secondary-focus\/10:focus {
    background-color: hsl(var(--sf) / 0.1)}
.focus\:bg-secondary-focus\/100:focus {
    background-color: hsl(var(--sf) / 1)}
.focus\:bg-secondary-focus\/20:focus {
    background-color: hsl(var(--sf) / 0.2)}
.focus\:bg-secondary-focus\/25:focus {
    background-color: hsl(var(--sf) / 0.25)}
.focus\:bg-secondary-focus\/30:focus {
    background-color: hsl(var(--sf) / 0.3)}
.focus\:bg-secondary-focus\/40:focus {
    background-color: hsl(var(--sf) / 0.4)}
.focus\:bg-secondary-focus\/5:focus {
    background-color: hsl(var(--sf) / 0.05)}
.focus\:bg-secondary-focus\/50:focus {
    background-color: hsl(var(--sf) / 0.5)}
.focus\:bg-secondary-focus\/60:focus {
    background-color: hsl(var(--sf) / 0.6)}
.focus\:bg-secondary-focus\/70:focus {
    background-color: hsl(var(--sf) / 0.7)}
.focus\:bg-secondary-focus\/75:focus {
    background-color: hsl(var(--sf) / 0.75)}
.focus\:bg-secondary-focus\/80:focus {
    background-color: hsl(var(--sf) / 0.8)}
.focus\:bg-secondary-focus\/90:focus {
    background-color: hsl(var(--sf) / 0.9)}
.focus\:bg-secondary-focus\/95:focus {
    background-color: hsl(var(--sf) / 0.95)}
.focus\:bg-secondary\/0:focus {
    background-color: hsl(var(--s) / 0)}
.focus\:bg-secondary\/10:focus {
    background-color: hsl(var(--s) / 0.1)}
.focus\:bg-secondary\/100:focus {
    background-color: hsl(var(--s) / 1)}
.focus\:bg-secondary\/20:focus {
    background-color: hsl(var(--s) / 0.2)}
.focus\:bg-secondary\/25:focus {
    background-color: hsl(var(--s) / 0.25)}
.focus\:bg-secondary\/30:focus {
    background-color: hsl(var(--s) / 0.3)}
.focus\:bg-secondary\/40:focus {
    background-color: hsl(var(--s) / 0.4)}
.focus\:bg-secondary\/5:focus {
    background-color: hsl(var(--s) / 0.05)}
.focus\:bg-secondary\/50:focus {
    background-color: hsl(var(--s) / 0.5)}
.focus\:bg-secondary\/60:focus {
    background-color: hsl(var(--s) / 0.6)}
.focus\:bg-secondary\/70:focus {
    background-color: hsl(var(--s) / 0.7)}
.focus\:bg-secondary\/75:focus {
    background-color: hsl(var(--s) / 0.75)}
.focus\:bg-secondary\/80:focus {
    background-color: hsl(var(--s) / 0.8)}
.focus\:bg-secondary\/90:focus {
    background-color: hsl(var(--s) / 0.9)}
.focus\:bg-secondary\/95:focus {
    background-color: hsl(var(--s) / 0.95)}
.focus\:bg-success:focus {
    background-color: hsl(var(--su) / 1)}
.focus\:bg-success-content:focus {
    background-color: hsl(var(--suc) / 1)}
.focus\:bg-success-content\/0:focus {
    background-color: hsl(var(--suc) / 0)}
.focus\:bg-success-content\/10:focus {
    background-color: hsl(var(--suc) / 0.1)}
.focus\:bg-success-content\/100:focus {
    background-color: hsl(var(--suc) / 1)}
.focus\:bg-success-content\/20:focus {
    background-color: hsl(var(--suc) / 0.2)}
.focus\:bg-success-content\/25:focus {
    background-color: hsl(var(--suc) / 0.25)}
.focus\:bg-success-content\/30:focus {
    background-color: hsl(var(--suc) / 0.3)}
.focus\:bg-success-content\/40:focus {
    background-color: hsl(var(--suc) / 0.4)}
.focus\:bg-success-content\/5:focus {
    background-color: hsl(var(--suc) / 0.05)}
.focus\:bg-success-content\/50:focus {
    background-color: hsl(var(--suc) / 0.5)}
.focus\:bg-success-content\/60:focus {
    background-color: hsl(var(--suc) / 0.6)}
.focus\:bg-success-content\/70:focus {
    background-color: hsl(var(--suc) / 0.7)}
.focus\:bg-success-content\/75:focus {
    background-color: hsl(var(--suc) / 0.75)}
.focus\:bg-success-content\/80:focus {
    background-color: hsl(var(--suc) / 0.8)}
.focus\:bg-success-content\/90:focus {
    background-color: hsl(var(--suc) / 0.9)}
.focus\:bg-success-content\/95:focus {
    background-color: hsl(var(--suc) / 0.95)}
.focus\:bg-success\/0:focus {
    background-color: hsl(var(--su) / 0)}
.focus\:bg-success\/10:focus {
    background-color: hsl(var(--su) / 0.1)}
.focus\:bg-success\/100:focus {
    background-color: hsl(var(--su) / 1)}
.focus\:bg-success\/20:focus {
    background-color: hsl(var(--su) / 0.2)}
.focus\:bg-success\/25:focus {
    background-color: hsl(var(--su) / 0.25)}
.focus\:bg-success\/30:focus {
    background-color: hsl(var(--su) / 0.3)}
.focus\:bg-success\/40:focus {
    background-color: hsl(var(--su) / 0.4)}
.focus\:bg-success\/5:focus {
    background-color: hsl(var(--su) / 0.05)}
.focus\:bg-success\/50:focus {
    background-color: hsl(var(--su) / 0.5)}
.focus\:bg-success\/60:focus {
    background-color: hsl(var(--su) / 0.6)}
.focus\:bg-success\/70:focus {
    background-color: hsl(var(--su) / 0.7)}
.focus\:bg-success\/75:focus {
    background-color: hsl(var(--su) / 0.75)}
.focus\:bg-success\/80:focus {
    background-color: hsl(var(--su) / 0.8)}
.focus\:bg-success\/90:focus {
    background-color: hsl(var(--su) / 0.9)}
.focus\:bg-success\/95:focus {
    background-color: hsl(var(--su) / 0.95)}
.focus\:bg-warning:focus {
    background-color: hsl(var(--wa) / 1)}
.focus\:bg-warning-content:focus {
    background-color: hsl(var(--wac) / 1)}
.focus\:bg-warning-content\/0:focus {
    background-color: hsl(var(--wac) / 0)}
.focus\:bg-warning-content\/10:focus {
    background-color: hsl(var(--wac) / 0.1)}
.focus\:bg-warning-content\/100:focus {
    background-color: hsl(var(--wac) / 1)}
.focus\:bg-warning-content\/20:focus {
    background-color: hsl(var(--wac) / 0.2)}
.focus\:bg-warning-content\/25:focus {
    background-color: hsl(var(--wac) / 0.25)}
.focus\:bg-warning-content\/30:focus {
    background-color: hsl(var(--wac) / 0.3)}
.focus\:bg-warning-content\/40:focus {
    background-color: hsl(var(--wac) / 0.4)}
.focus\:bg-warning-content\/5:focus {
    background-color: hsl(var(--wac) / 0.05)}
.focus\:bg-warning-content\/50:focus {
    background-color: hsl(var(--wac) / 0.5)}
.focus\:bg-warning-content\/60:focus {
    background-color: hsl(var(--wac) / 0.6)}
.focus\:bg-warning-content\/70:focus {
    background-color: hsl(var(--wac) / 0.7)}
.focus\:bg-warning-content\/75:focus {
    background-color: hsl(var(--wac) / 0.75)}
.focus\:bg-warning-content\/80:focus {
    background-color: hsl(var(--wac) / 0.8)}
.focus\:bg-warning-content\/90:focus {
    background-color: hsl(var(--wac) / 0.9)}
.focus\:bg-warning-content\/95:focus {
    background-color: hsl(var(--wac) / 0.95)}
.focus\:bg-warning\/0:focus {
    background-color: hsl(var(--wa) / 0)}
.focus\:bg-warning\/10:focus {
    background-color: hsl(var(--wa) / 0.1)}
.focus\:bg-warning\/100:focus {
    background-color: hsl(var(--wa) / 1)}
.focus\:bg-warning\/20:focus {
    background-color: hsl(var(--wa) / 0.2)}
.focus\:bg-warning\/25:focus {
    background-color: hsl(var(--wa) / 0.25)}
.focus\:bg-warning\/30:focus {
    background-color: hsl(var(--wa) / 0.3)}
.focus\:bg-warning\/40:focus {
    background-color: hsl(var(--wa) / 0.4)}
.focus\:bg-warning\/5:focus {
    background-color: hsl(var(--wa) / 0.05)}
.focus\:bg-warning\/50:focus {
    background-color: hsl(var(--wa) / 0.5)}
.focus\:bg-warning\/60:focus {
    background-color: hsl(var(--wa) / 0.6)}
.focus\:bg-warning\/70:focus {
    background-color: hsl(var(--wa) / 0.7)}
.focus\:bg-warning\/75:focus {
    background-color: hsl(var(--wa) / 0.75)}
.focus\:bg-warning\/80:focus {
    background-color: hsl(var(--wa) / 0.8)}
.focus\:bg-warning\/90:focus {
    background-color: hsl(var(--wa) / 0.9)}
.focus\:bg-warning\/95:focus {
    background-color: hsl(var(--wa) / 0.95)}
.focus\:from-accent:focus {
    --tw-gradient-from: hsl(var(--a) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content:focus {
    --tw-gradient-from: hsl(var(--ac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/0:focus {
    --tw-gradient-from: hsl(var(--ac) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/10:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/100:focus {
    --tw-gradient-from: hsl(var(--ac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/20:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/25:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/30:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/40:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/5:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/50:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/60:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/70:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/75:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/80:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/90:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-content\/95:focus {
    --tw-gradient-from: hsl(var(--ac) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus:focus {
    --tw-gradient-from: hsl(var(--af) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/0:focus {
    --tw-gradient-from: hsl(var(--af) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/10:focus {
    --tw-gradient-from: hsl(var(--af) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/100:focus {
    --tw-gradient-from: hsl(var(--af) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/20:focus {
    --tw-gradient-from: hsl(var(--af) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/25:focus {
    --tw-gradient-from: hsl(var(--af) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/30:focus {
    --tw-gradient-from: hsl(var(--af) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/40:focus {
    --tw-gradient-from: hsl(var(--af) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/5:focus {
    --tw-gradient-from: hsl(var(--af) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/50:focus {
    --tw-gradient-from: hsl(var(--af) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/60:focus {
    --tw-gradient-from: hsl(var(--af) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/70:focus {
    --tw-gradient-from: hsl(var(--af) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/75:focus {
    --tw-gradient-from: hsl(var(--af) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/80:focus {
    --tw-gradient-from: hsl(var(--af) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/90:focus {
    --tw-gradient-from: hsl(var(--af) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent-focus\/95:focus {
    --tw-gradient-from: hsl(var(--af) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/0:focus {
    --tw-gradient-from: hsl(var(--a) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/10:focus {
    --tw-gradient-from: hsl(var(--a) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/100:focus {
    --tw-gradient-from: hsl(var(--a) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/20:focus {
    --tw-gradient-from: hsl(var(--a) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/25:focus {
    --tw-gradient-from: hsl(var(--a) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/30:focus {
    --tw-gradient-from: hsl(var(--a) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/40:focus {
    --tw-gradient-from: hsl(var(--a) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/5:focus {
    --tw-gradient-from: hsl(var(--a) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/50:focus {
    --tw-gradient-from: hsl(var(--a) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/60:focus {
    --tw-gradient-from: hsl(var(--a) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/70:focus {
    --tw-gradient-from: hsl(var(--a) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/75:focus {
    --tw-gradient-from: hsl(var(--a) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/80:focus {
    --tw-gradient-from: hsl(var(--a) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/90:focus {
    --tw-gradient-from: hsl(var(--a) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-accent\/95:focus {
    --tw-gradient-from: hsl(var(--a) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100:focus {
    --tw-gradient-from: hsl(var(--b1) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/0:focus {
    --tw-gradient-from: hsl(var(--b1) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/10:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/100:focus {
    --tw-gradient-from: hsl(var(--b1) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/20:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/25:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/30:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/40:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/5:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/50:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/60:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/70:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/75:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/80:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/90:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-100\/95:focus {
    --tw-gradient-from: hsl(var(--b1) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200:focus {
    --tw-gradient-from: hsl(var(--b2) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/0:focus {
    --tw-gradient-from: hsl(var(--b2) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/10:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/100:focus {
    --tw-gradient-from: hsl(var(--b2) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/20:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/25:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/30:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/40:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/5:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/50:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/60:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/70:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/75:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/80:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/90:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-200\/95:focus {
    --tw-gradient-from: hsl(var(--b2) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300:focus {
    --tw-gradient-from: hsl(var(--b3) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/0:focus {
    --tw-gradient-from: hsl(var(--b3) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/10:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/100:focus {
    --tw-gradient-from: hsl(var(--b3) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/20:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/25:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/30:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/40:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/5:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/50:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/60:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/70:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/75:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/80:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/90:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-300\/95:focus {
    --tw-gradient-from: hsl(var(--b3) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content:focus {
    --tw-gradient-from: hsl(var(--bc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/0:focus {
    --tw-gradient-from: hsl(var(--bc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/10:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/100:focus {
    --tw-gradient-from: hsl(var(--bc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/20:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/25:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/30:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/40:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/5:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/50:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/60:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/70:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/75:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/80:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/90:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-base-content\/95:focus {
    --tw-gradient-from: hsl(var(--bc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error:focus {
    --tw-gradient-from: hsl(var(--er) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content:focus {
    --tw-gradient-from: hsl(var(--erc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/0:focus {
    --tw-gradient-from: hsl(var(--erc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/10:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/100:focus {
    --tw-gradient-from: hsl(var(--erc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/20:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/25:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/30:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/40:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/5:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/50:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/60:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/70:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/75:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/80:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/90:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error-content\/95:focus {
    --tw-gradient-from: hsl(var(--erc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/0:focus {
    --tw-gradient-from: hsl(var(--er) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/10:focus {
    --tw-gradient-from: hsl(var(--er) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/100:focus {
    --tw-gradient-from: hsl(var(--er) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/20:focus {
    --tw-gradient-from: hsl(var(--er) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/25:focus {
    --tw-gradient-from: hsl(var(--er) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/30:focus {
    --tw-gradient-from: hsl(var(--er) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/40:focus {
    --tw-gradient-from: hsl(var(--er) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/5:focus {
    --tw-gradient-from: hsl(var(--er) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/50:focus {
    --tw-gradient-from: hsl(var(--er) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/60:focus {
    --tw-gradient-from: hsl(var(--er) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/70:focus {
    --tw-gradient-from: hsl(var(--er) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/75:focus {
    --tw-gradient-from: hsl(var(--er) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/80:focus {
    --tw-gradient-from: hsl(var(--er) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/90:focus {
    --tw-gradient-from: hsl(var(--er) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-error\/95:focus {
    --tw-gradient-from: hsl(var(--er) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info:focus {
    --tw-gradient-from: hsl(var(--in) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content:focus {
    --tw-gradient-from: hsl(var(--inc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/0:focus {
    --tw-gradient-from: hsl(var(--inc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/10:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/100:focus {
    --tw-gradient-from: hsl(var(--inc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/20:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/25:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/30:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/40:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/5:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/50:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/60:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/70:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/75:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/80:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/90:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info-content\/95:focus {
    --tw-gradient-from: hsl(var(--inc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/0:focus {
    --tw-gradient-from: hsl(var(--in) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/10:focus {
    --tw-gradient-from: hsl(var(--in) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/100:focus {
    --tw-gradient-from: hsl(var(--in) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/20:focus {
    --tw-gradient-from: hsl(var(--in) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/25:focus {
    --tw-gradient-from: hsl(var(--in) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/30:focus {
    --tw-gradient-from: hsl(var(--in) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/40:focus {
    --tw-gradient-from: hsl(var(--in) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/5:focus {
    --tw-gradient-from: hsl(var(--in) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/50:focus {
    --tw-gradient-from: hsl(var(--in) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/60:focus {
    --tw-gradient-from: hsl(var(--in) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/70:focus {
    --tw-gradient-from: hsl(var(--in) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/75:focus {
    --tw-gradient-from: hsl(var(--in) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/80:focus {
    --tw-gradient-from: hsl(var(--in) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/90:focus {
    --tw-gradient-from: hsl(var(--in) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-info\/95:focus {
    --tw-gradient-from: hsl(var(--in) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral:focus {
    --tw-gradient-from: hsl(var(--n) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content:focus {
    --tw-gradient-from: hsl(var(--nc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/0:focus {
    --tw-gradient-from: hsl(var(--nc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/10:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/100:focus {
    --tw-gradient-from: hsl(var(--nc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/20:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/25:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/30:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/40:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/5:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/50:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/60:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/70:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/75:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/80:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/90:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-content\/95:focus {
    --tw-gradient-from: hsl(var(--nc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus:focus {
    --tw-gradient-from: hsl(var(--nf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/0:focus {
    --tw-gradient-from: hsl(var(--nf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/10:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/100:focus {
    --tw-gradient-from: hsl(var(--nf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/20:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/25:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/30:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/40:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/5:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/50:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/60:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/70:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/75:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/80:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/90:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral-focus\/95:focus {
    --tw-gradient-from: hsl(var(--nf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/0:focus {
    --tw-gradient-from: hsl(var(--n) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/10:focus {
    --tw-gradient-from: hsl(var(--n) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/100:focus {
    --tw-gradient-from: hsl(var(--n) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/20:focus {
    --tw-gradient-from: hsl(var(--n) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/25:focus {
    --tw-gradient-from: hsl(var(--n) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/30:focus {
    --tw-gradient-from: hsl(var(--n) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/40:focus {
    --tw-gradient-from: hsl(var(--n) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/5:focus {
    --tw-gradient-from: hsl(var(--n) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/50:focus {
    --tw-gradient-from: hsl(var(--n) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/60:focus {
    --tw-gradient-from: hsl(var(--n) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/70:focus {
    --tw-gradient-from: hsl(var(--n) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/75:focus {
    --tw-gradient-from: hsl(var(--n) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/80:focus {
    --tw-gradient-from: hsl(var(--n) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/90:focus {
    --tw-gradient-from: hsl(var(--n) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-neutral\/95:focus {
    --tw-gradient-from: hsl(var(--n) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary:focus {
    --tw-gradient-from: hsl(var(--p) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content:focus {
    --tw-gradient-from: hsl(var(--pc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/0:focus {
    --tw-gradient-from: hsl(var(--pc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/10:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/100:focus {
    --tw-gradient-from: hsl(var(--pc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/20:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/25:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/30:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/40:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/5:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/50:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/60:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/70:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/75:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/80:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/90:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-content\/95:focus {
    --tw-gradient-from: hsl(var(--pc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus:focus {
    --tw-gradient-from: hsl(var(--pf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/0:focus {
    --tw-gradient-from: hsl(var(--pf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/10:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/100:focus {
    --tw-gradient-from: hsl(var(--pf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/20:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/25:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/30:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/40:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/5:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/50:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/60:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/70:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/75:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/80:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/90:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary-focus\/95:focus {
    --tw-gradient-from: hsl(var(--pf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/0:focus {
    --tw-gradient-from: hsl(var(--p) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/10:focus {
    --tw-gradient-from: hsl(var(--p) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/100:focus {
    --tw-gradient-from: hsl(var(--p) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/20:focus {
    --tw-gradient-from: hsl(var(--p) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/25:focus {
    --tw-gradient-from: hsl(var(--p) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/30:focus {
    --tw-gradient-from: hsl(var(--p) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/40:focus {
    --tw-gradient-from: hsl(var(--p) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/5:focus {
    --tw-gradient-from: hsl(var(--p) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/50:focus {
    --tw-gradient-from: hsl(var(--p) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/60:focus {
    --tw-gradient-from: hsl(var(--p) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/70:focus {
    --tw-gradient-from: hsl(var(--p) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/75:focus {
    --tw-gradient-from: hsl(var(--p) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/80:focus {
    --tw-gradient-from: hsl(var(--p) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/90:focus {
    --tw-gradient-from: hsl(var(--p) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-primary\/95:focus {
    --tw-gradient-from: hsl(var(--p) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary:focus {
    --tw-gradient-from: hsl(var(--s) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content:focus {
    --tw-gradient-from: hsl(var(--sc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/0:focus {
    --tw-gradient-from: hsl(var(--sc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/10:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/100:focus {
    --tw-gradient-from: hsl(var(--sc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/20:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/25:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/30:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/40:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/5:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/50:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/60:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/70:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/75:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/80:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/90:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-content\/95:focus {
    --tw-gradient-from: hsl(var(--sc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus:focus {
    --tw-gradient-from: hsl(var(--sf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/0:focus {
    --tw-gradient-from: hsl(var(--sf) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/10:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/100:focus {
    --tw-gradient-from: hsl(var(--sf) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/20:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/25:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/30:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/40:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/5:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/50:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/60:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/70:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/75:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/80:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/90:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary-focus\/95:focus {
    --tw-gradient-from: hsl(var(--sf) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/0:focus {
    --tw-gradient-from: hsl(var(--s) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/10:focus {
    --tw-gradient-from: hsl(var(--s) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/100:focus {
    --tw-gradient-from: hsl(var(--s) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/20:focus {
    --tw-gradient-from: hsl(var(--s) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/25:focus {
    --tw-gradient-from: hsl(var(--s) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/30:focus {
    --tw-gradient-from: hsl(var(--s) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/40:focus {
    --tw-gradient-from: hsl(var(--s) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/5:focus {
    --tw-gradient-from: hsl(var(--s) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/50:focus {
    --tw-gradient-from: hsl(var(--s) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/60:focus {
    --tw-gradient-from: hsl(var(--s) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/70:focus {
    --tw-gradient-from: hsl(var(--s) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/75:focus {
    --tw-gradient-from: hsl(var(--s) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/80:focus {
    --tw-gradient-from: hsl(var(--s) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/90:focus {
    --tw-gradient-from: hsl(var(--s) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-secondary\/95:focus {
    --tw-gradient-from: hsl(var(--s) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success:focus {
    --tw-gradient-from: hsl(var(--su) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content:focus {
    --tw-gradient-from: hsl(var(--suc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/0:focus {
    --tw-gradient-from: hsl(var(--suc) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/10:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/100:focus {
    --tw-gradient-from: hsl(var(--suc) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/20:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/25:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/30:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/40:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/5:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/50:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/60:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/70:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/75:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/80:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/90:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success-content\/95:focus {
    --tw-gradient-from: hsl(var(--suc) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/0:focus {
    --tw-gradient-from: hsl(var(--su) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/10:focus {
    --tw-gradient-from: hsl(var(--su) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/100:focus {
    --tw-gradient-from: hsl(var(--su) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/20:focus {
    --tw-gradient-from: hsl(var(--su) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/25:focus {
    --tw-gradient-from: hsl(var(--su) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/30:focus {
    --tw-gradient-from: hsl(var(--su) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/40:focus {
    --tw-gradient-from: hsl(var(--su) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/5:focus {
    --tw-gradient-from: hsl(var(--su) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/50:focus {
    --tw-gradient-from: hsl(var(--su) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/60:focus {
    --tw-gradient-from: hsl(var(--su) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/70:focus {
    --tw-gradient-from: hsl(var(--su) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/75:focus {
    --tw-gradient-from: hsl(var(--su) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/80:focus {
    --tw-gradient-from: hsl(var(--su) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/90:focus {
    --tw-gradient-from: hsl(var(--su) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-success\/95:focus {
    --tw-gradient-from: hsl(var(--su) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning:focus {
    --tw-gradient-from: hsl(var(--wa) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content:focus {
    --tw-gradient-from: hsl(var(--wac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/0:focus {
    --tw-gradient-from: hsl(var(--wac) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/10:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/100:focus {
    --tw-gradient-from: hsl(var(--wac) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/20:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/25:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/30:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/40:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/5:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/50:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/60:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/70:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/75:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/80:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/90:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning-content\/95:focus {
    --tw-gradient-from: hsl(var(--wac) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/0:focus {
    --tw-gradient-from: hsl(var(--wa) / 0) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/10:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/100:focus {
    --tw-gradient-from: hsl(var(--wa) / 1) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/20:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.2) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/25:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.25) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/30:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.3) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/40:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.4) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/5:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.05) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/50:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.5) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/60:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.6) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/70:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.7) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/75:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.75) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/80:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.8) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/90:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.9) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:from-warning\/95:focus {
    --tw-gradient-from: hsl(var(--wa) / 0.95) var(--tw-gradient-from-position);
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), var(--tw-gradient-to)}
.focus\:via-accent:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/0:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/10:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/100:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/20:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/25:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/30:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/40:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/5:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/50:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/60:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/70:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/75:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/80:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/90:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-content\/95:focus {
    --tw-gradient-to: hsl(var(--ac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--ac) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/0:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/10:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/100:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/20:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/25:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/30:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/40:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/5:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/50:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/60:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/70:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/75:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/80:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/90:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent-focus\/95:focus {
    --tw-gradient-to: hsl(var(--af) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--af) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/0:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/10:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/100:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/20:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/25:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/30:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/40:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/5:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/50:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/60:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/70:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/75:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/80:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/90:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-accent\/95:focus {
    --tw-gradient-to: hsl(var(--a) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--a) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/0:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/10:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/100:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/20:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/25:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/30:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/40:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/5:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/50:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/60:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/70:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/75:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/80:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/90:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-100\/95:focus {
    --tw-gradient-to: hsl(var(--b1) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b1) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/0:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/10:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/100:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/20:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/25:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/30:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/40:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/5:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/50:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/60:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/70:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/75:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/80:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/90:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-200\/95:focus {
    --tw-gradient-to: hsl(var(--b2) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b2) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/0:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/10:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/100:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/20:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/25:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/30:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/40:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/5:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/50:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/60:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/70:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/75:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/80:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/90:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-300\/95:focus {
    --tw-gradient-to: hsl(var(--b3) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--b3) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/0:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/10:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/100:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/20:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/25:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/30:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/40:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/5:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/50:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/60:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/70:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/75:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/80:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/90:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-base-content\/95:focus {
    --tw-gradient-to: hsl(var(--bc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--bc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/0:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/10:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/100:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/20:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/25:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/30:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/40:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/5:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/50:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/60:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/70:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/75:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/80:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/90:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error-content\/95:focus {
    --tw-gradient-to: hsl(var(--erc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--erc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/0:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/10:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/100:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/20:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/25:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/30:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/40:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/5:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/50:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/60:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/70:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/75:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/80:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/90:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-error\/95:focus {
    --tw-gradient-to: hsl(var(--er) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--er) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/0:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/10:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/100:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/20:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/25:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/30:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/40:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/5:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/50:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/60:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/70:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/75:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/80:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/90:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info-content\/95:focus {
    --tw-gradient-to: hsl(var(--inc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--inc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/0:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/10:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/100:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/20:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/25:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/30:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/40:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/5:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/50:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/60:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/70:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/75:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/80:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/90:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-info\/95:focus {
    --tw-gradient-to: hsl(var(--in) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--in) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/0:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/10:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/100:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/20:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/25:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/30:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/40:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/5:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/50:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/60:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/70:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/75:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/80:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/90:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-content\/95:focus {
    --tw-gradient-to: hsl(var(--nc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/0:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/10:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/100:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/20:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/25:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/30:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/40:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/5:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/50:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/60:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/70:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/75:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/80:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/90:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral-focus\/95:focus {
    --tw-gradient-to: hsl(var(--nf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--nf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/0:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/10:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/100:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/20:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/25:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/30:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/40:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/5:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/50:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/60:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/70:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/75:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/80:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/90:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-neutral\/95:focus {
    --tw-gradient-to: hsl(var(--n) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--n) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/0:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/10:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/100:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/20:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/25:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/30:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/40:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/5:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/50:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/60:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/70:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/75:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/80:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/90:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-content\/95:focus {
    --tw-gradient-to: hsl(var(--pc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/0:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/10:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/100:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/20:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/25:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/30:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/40:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/5:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/50:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/60:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/70:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/75:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/80:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/90:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary-focus\/95:focus {
    --tw-gradient-to: hsl(var(--pf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--pf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/0:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/10:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/100:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/20:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/25:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/30:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/40:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/5:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/50:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/60:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/70:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/75:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/80:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/90:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-primary\/95:focus {
    --tw-gradient-to: hsl(var(--p) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--p) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/0:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/10:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/100:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/20:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/25:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/30:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/40:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/5:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/50:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/60:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/70:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/75:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/80:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/90:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-content\/95:focus {
    --tw-gradient-to: hsl(var(--sc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/0:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/10:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/100:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/20:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/25:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/30:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/40:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/5:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/50:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/60:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/70:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/75:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/80:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/90:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary-focus\/95:focus {
    --tw-gradient-to: hsl(var(--sf) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--sf) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/0:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/10:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/100:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/20:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/25:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/30:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/40:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/5:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/50:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/60:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/70:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/75:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/80:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/90:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-secondary\/95:focus {
    --tw-gradient-to: hsl(var(--s) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--s) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/0:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/10:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/100:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/20:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/25:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/30:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/40:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/5:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/50:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/60:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/70:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/75:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/80:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/90:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success-content\/95:focus {
    --tw-gradient-to: hsl(var(--suc) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--suc) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/0:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/10:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/100:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/20:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/25:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/30:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/40:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/5:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/50:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/60:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/70:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/75:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/80:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/90:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-success\/95:focus {
    --tw-gradient-to: hsl(var(--su) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--su) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/0:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/10:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/100:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/20:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/25:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/30:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/40:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/5:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/50:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/60:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/70:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/75:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/80:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/90:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning-content\/95:focus {
    --tw-gradient-to: hsl(var(--wac) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wac) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/0:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/10:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/100:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 1) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/20:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.2) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/25:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.25) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/30:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.3) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/40:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.4) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/5:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.05) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/50:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.5) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/60:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.6) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/70:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.7) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/75:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.75) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/80:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.8) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/90:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.9) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:via-warning\/95:focus {
    --tw-gradient-to: hsl(var(--wa) / 0)  var(--tw-gradient-to-position);
    --tw-gradient-stops: var(--tw-gradient-from), hsl(var(--wa) / 0.95) var(--tw-gradient-via-position), var(--tw-gradient-to)}
.focus\:to-accent:focus {
    --tw-gradient-to: hsl(var(--a) / 1) var(--tw-gradient-to-position)}
.focus\:to-accent-content:focus {
    --tw-gradient-to: hsl(var(--ac) / 1) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/0:focus {
    --tw-gradient-to: hsl(var(--ac) / 0) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/10:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/100:focus {
    --tw-gradient-to: hsl(var(--ac) / 1) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/20:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/25:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/30:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/40:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/5:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/50:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/60:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/70:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/75:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/80:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/90:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-accent-content\/95:focus {
    --tw-gradient-to: hsl(var(--ac) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-accent-focus:focus {
    --tw-gradient-to: hsl(var(--af) / 1) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/0:focus {
    --tw-gradient-to: hsl(var(--af) / 0) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/10:focus {
    --tw-gradient-to: hsl(var(--af) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/100:focus {
    --tw-gradient-to: hsl(var(--af) / 1) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/20:focus {
    --tw-gradient-to: hsl(var(--af) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/25:focus {
    --tw-gradient-to: hsl(var(--af) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/30:focus {
    --tw-gradient-to: hsl(var(--af) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/40:focus {
    --tw-gradient-to: hsl(var(--af) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/5:focus {
    --tw-gradient-to: hsl(var(--af) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/50:focus {
    --tw-gradient-to: hsl(var(--af) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/60:focus {
    --tw-gradient-to: hsl(var(--af) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/70:focus {
    --tw-gradient-to: hsl(var(--af) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/75:focus {
    --tw-gradient-to: hsl(var(--af) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/80:focus {
    --tw-gradient-to: hsl(var(--af) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/90:focus {
    --tw-gradient-to: hsl(var(--af) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-accent-focus\/95:focus {
    --tw-gradient-to: hsl(var(--af) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-accent\/0:focus {
    --tw-gradient-to: hsl(var(--a) / 0) var(--tw-gradient-to-position)}
.focus\:to-accent\/10:focus {
    --tw-gradient-to: hsl(var(--a) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-accent\/100:focus {
    --tw-gradient-to: hsl(var(--a) / 1) var(--tw-gradient-to-position)}
.focus\:to-accent\/20:focus {
    --tw-gradient-to: hsl(var(--a) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-accent\/25:focus {
    --tw-gradient-to: hsl(var(--a) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-accent\/30:focus {
    --tw-gradient-to: hsl(var(--a) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-accent\/40:focus {
    --tw-gradient-to: hsl(var(--a) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-accent\/5:focus {
    --tw-gradient-to: hsl(var(--a) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-accent\/50:focus {
    --tw-gradient-to: hsl(var(--a) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-accent\/60:focus {
    --tw-gradient-to: hsl(var(--a) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-accent\/70:focus {
    --tw-gradient-to: hsl(var(--a) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-accent\/75:focus {
    --tw-gradient-to: hsl(var(--a) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-accent\/80:focus {
    --tw-gradient-to: hsl(var(--a) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-accent\/90:focus {
    --tw-gradient-to: hsl(var(--a) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-accent\/95:focus {
    --tw-gradient-to: hsl(var(--a) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-base-100:focus {
    --tw-gradient-to: hsl(var(--b1) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-100\/0:focus {
    --tw-gradient-to: hsl(var(--b1) / 0) var(--tw-gradient-to-position)}
.focus\:to-base-100\/10:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-base-100\/100:focus {
    --tw-gradient-to: hsl(var(--b1) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-100\/20:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-base-100\/25:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-base-100\/30:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-base-100\/40:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-base-100\/5:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-base-100\/50:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-base-100\/60:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-base-100\/70:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-base-100\/75:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-base-100\/80:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-base-100\/90:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-base-100\/95:focus {
    --tw-gradient-to: hsl(var(--b1) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-base-200:focus {
    --tw-gradient-to: hsl(var(--b2) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-200\/0:focus {
    --tw-gradient-to: hsl(var(--b2) / 0) var(--tw-gradient-to-position)}
.focus\:to-base-200\/10:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-base-200\/100:focus {
    --tw-gradient-to: hsl(var(--b2) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-200\/20:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-base-200\/25:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-base-200\/30:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-base-200\/40:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-base-200\/5:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-base-200\/50:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-base-200\/60:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-base-200\/70:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-base-200\/75:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-base-200\/80:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-base-200\/90:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-base-200\/95:focus {
    --tw-gradient-to: hsl(var(--b2) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-base-300:focus {
    --tw-gradient-to: hsl(var(--b3) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-300\/0:focus {
    --tw-gradient-to: hsl(var(--b3) / 0) var(--tw-gradient-to-position)}
.focus\:to-base-300\/10:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-base-300\/100:focus {
    --tw-gradient-to: hsl(var(--b3) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-300\/20:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-base-300\/25:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-base-300\/30:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-base-300\/40:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-base-300\/5:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-base-300\/50:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-base-300\/60:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-base-300\/70:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-base-300\/75:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-base-300\/80:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-base-300\/90:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-base-300\/95:focus {
    --tw-gradient-to: hsl(var(--b3) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-base-content:focus {
    --tw-gradient-to: hsl(var(--bc) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-content\/0:focus {
    --tw-gradient-to: hsl(var(--bc) / 0) var(--tw-gradient-to-position)}
.focus\:to-base-content\/10:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-base-content\/100:focus {
    --tw-gradient-to: hsl(var(--bc) / 1) var(--tw-gradient-to-position)}
.focus\:to-base-content\/20:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-base-content\/25:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-base-content\/30:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-base-content\/40:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-base-content\/5:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-base-content\/50:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-base-content\/60:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-base-content\/70:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-base-content\/75:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-base-content\/80:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-base-content\/90:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-base-content\/95:focus {
    --tw-gradient-to: hsl(var(--bc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-error:focus {
    --tw-gradient-to: hsl(var(--er) / 1) var(--tw-gradient-to-position)}
.focus\:to-error-content:focus {
    --tw-gradient-to: hsl(var(--erc) / 1) var(--tw-gradient-to-position)}
.focus\:to-error-content\/0:focus {
    --tw-gradient-to: hsl(var(--erc) / 0) var(--tw-gradient-to-position)}
.focus\:to-error-content\/10:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-error-content\/100:focus {
    --tw-gradient-to: hsl(var(--erc) / 1) var(--tw-gradient-to-position)}
.focus\:to-error-content\/20:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-error-content\/25:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-error-content\/30:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-error-content\/40:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-error-content\/5:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-error-content\/50:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-error-content\/60:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-error-content\/70:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-error-content\/75:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-error-content\/80:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-error-content\/90:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-error-content\/95:focus {
    --tw-gradient-to: hsl(var(--erc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-error\/0:focus {
    --tw-gradient-to: hsl(var(--er) / 0) var(--tw-gradient-to-position)}
.focus\:to-error\/10:focus {
    --tw-gradient-to: hsl(var(--er) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-error\/100:focus {
    --tw-gradient-to: hsl(var(--er) / 1) var(--tw-gradient-to-position)}
.focus\:to-error\/20:focus {
    --tw-gradient-to: hsl(var(--er) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-error\/25:focus {
    --tw-gradient-to: hsl(var(--er) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-error\/30:focus {
    --tw-gradient-to: hsl(var(--er) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-error\/40:focus {
    --tw-gradient-to: hsl(var(--er) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-error\/5:focus {
    --tw-gradient-to: hsl(var(--er) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-error\/50:focus {
    --tw-gradient-to: hsl(var(--er) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-error\/60:focus {
    --tw-gradient-to: hsl(var(--er) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-error\/70:focus {
    --tw-gradient-to: hsl(var(--er) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-error\/75:focus {
    --tw-gradient-to: hsl(var(--er) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-error\/80:focus {
    --tw-gradient-to: hsl(var(--er) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-error\/90:focus {
    --tw-gradient-to: hsl(var(--er) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-error\/95:focus {
    --tw-gradient-to: hsl(var(--er) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-info:focus {
    --tw-gradient-to: hsl(var(--in) / 1) var(--tw-gradient-to-position)}
.focus\:to-info-content:focus {
    --tw-gradient-to: hsl(var(--inc) / 1) var(--tw-gradient-to-position)}
.focus\:to-info-content\/0:focus {
    --tw-gradient-to: hsl(var(--inc) / 0) var(--tw-gradient-to-position)}
.focus\:to-info-content\/10:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-info-content\/100:focus {
    --tw-gradient-to: hsl(var(--inc) / 1) var(--tw-gradient-to-position)}
.focus\:to-info-content\/20:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-info-content\/25:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-info-content\/30:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-info-content\/40:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-info-content\/5:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-info-content\/50:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-info-content\/60:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-info-content\/70:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-info-content\/75:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-info-content\/80:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-info-content\/90:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-info-content\/95:focus {
    --tw-gradient-to: hsl(var(--inc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-info\/0:focus {
    --tw-gradient-to: hsl(var(--in) / 0) var(--tw-gradient-to-position)}
.focus\:to-info\/10:focus {
    --tw-gradient-to: hsl(var(--in) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-info\/100:focus {
    --tw-gradient-to: hsl(var(--in) / 1) var(--tw-gradient-to-position)}
.focus\:to-info\/20:focus {
    --tw-gradient-to: hsl(var(--in) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-info\/25:focus {
    --tw-gradient-to: hsl(var(--in) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-info\/30:focus {
    --tw-gradient-to: hsl(var(--in) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-info\/40:focus {
    --tw-gradient-to: hsl(var(--in) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-info\/5:focus {
    --tw-gradient-to: hsl(var(--in) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-info\/50:focus {
    --tw-gradient-to: hsl(var(--in) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-info\/60:focus {
    --tw-gradient-to: hsl(var(--in) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-info\/70:focus {
    --tw-gradient-to: hsl(var(--in) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-info\/75:focus {
    --tw-gradient-to: hsl(var(--in) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-info\/80:focus {
    --tw-gradient-to: hsl(var(--in) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-info\/90:focus {
    --tw-gradient-to: hsl(var(--in) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-info\/95:focus {
    --tw-gradient-to: hsl(var(--in) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-neutral:focus {
    --tw-gradient-to: hsl(var(--n) / 1) var(--tw-gradient-to-position)}
.focus\:to-neutral-content:focus {
    --tw-gradient-to: hsl(var(--nc) / 1) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/0:focus {
    --tw-gradient-to: hsl(var(--nc) / 0) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/10:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/100:focus {
    --tw-gradient-to: hsl(var(--nc) / 1) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/20:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/25:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/30:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/40:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/5:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/50:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/60:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/70:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/75:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/80:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/90:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-neutral-content\/95:focus {
    --tw-gradient-to: hsl(var(--nc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus:focus {
    --tw-gradient-to: hsl(var(--nf) / 1) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/0:focus {
    --tw-gradient-to: hsl(var(--nf) / 0) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/10:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/100:focus {
    --tw-gradient-to: hsl(var(--nf) / 1) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/20:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/25:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/30:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/40:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/5:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/50:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/60:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/70:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/75:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/80:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/90:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-neutral-focus\/95:focus {
    --tw-gradient-to: hsl(var(--nf) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-neutral\/0:focus {
    --tw-gradient-to: hsl(var(--n) / 0) var(--tw-gradient-to-position)}
.focus\:to-neutral\/10:focus {
    --tw-gradient-to: hsl(var(--n) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-neutral\/100:focus {
    --tw-gradient-to: hsl(var(--n) / 1) var(--tw-gradient-to-position)}
.focus\:to-neutral\/20:focus {
    --tw-gradient-to: hsl(var(--n) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-neutral\/25:focus {
    --tw-gradient-to: hsl(var(--n) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-neutral\/30:focus {
    --tw-gradient-to: hsl(var(--n) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-neutral\/40:focus {
    --tw-gradient-to: hsl(var(--n) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-neutral\/5:focus {
    --tw-gradient-to: hsl(var(--n) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-neutral\/50:focus {
    --tw-gradient-to: hsl(var(--n) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-neutral\/60:focus {
    --tw-gradient-to: hsl(var(--n) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-neutral\/70:focus {
    --tw-gradient-to: hsl(var(--n) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-neutral\/75:focus {
    --tw-gradient-to: hsl(var(--n) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-neutral\/80:focus {
    --tw-gradient-to: hsl(var(--n) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-neutral\/90:focus {
    --tw-gradient-to: hsl(var(--n) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-neutral\/95:focus {
    --tw-gradient-to: hsl(var(--n) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-primary:focus {
    --tw-gradient-to: hsl(var(--p) / 1) var(--tw-gradient-to-position)}
.focus\:to-primary-content:focus {
    --tw-gradient-to: hsl(var(--pc) / 1) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/0:focus {
    --tw-gradient-to: hsl(var(--pc) / 0) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/10:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/100:focus {
    --tw-gradient-to: hsl(var(--pc) / 1) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/20:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/25:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/30:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/40:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/5:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/50:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/60:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/70:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/75:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/80:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/90:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-primary-content\/95:focus {
    --tw-gradient-to: hsl(var(--pc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-primary-focus:focus {
    --tw-gradient-to: hsl(var(--pf) / 1) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/0:focus {
    --tw-gradient-to: hsl(var(--pf) / 0) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/10:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/100:focus {
    --tw-gradient-to: hsl(var(--pf) / 1) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/20:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/25:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/30:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/40:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/5:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/50:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/60:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/70:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/75:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/80:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/90:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-primary-focus\/95:focus {
    --tw-gradient-to: hsl(var(--pf) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-primary\/0:focus {
    --tw-gradient-to: hsl(var(--p) / 0) var(--tw-gradient-to-position)}
.focus\:to-primary\/10:focus {
    --tw-gradient-to: hsl(var(--p) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-primary\/100:focus {
    --tw-gradient-to: hsl(var(--p) / 1) var(--tw-gradient-to-position)}
.focus\:to-primary\/20:focus {
    --tw-gradient-to: hsl(var(--p) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-primary\/25:focus {
    --tw-gradient-to: hsl(var(--p) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-primary\/30:focus {
    --tw-gradient-to: hsl(var(--p) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-primary\/40:focus {
    --tw-gradient-to: hsl(var(--p) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-primary\/5:focus {
    --tw-gradient-to: hsl(var(--p) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-primary\/50:focus {
    --tw-gradient-to: hsl(var(--p) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-primary\/60:focus {
    --tw-gradient-to: hsl(var(--p) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-primary\/70:focus {
    --tw-gradient-to: hsl(var(--p) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-primary\/75:focus {
    --tw-gradient-to: hsl(var(--p) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-primary\/80:focus {
    --tw-gradient-to: hsl(var(--p) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-primary\/90:focus {
    --tw-gradient-to: hsl(var(--p) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-primary\/95:focus {
    --tw-gradient-to: hsl(var(--p) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-secondary:focus {
    --tw-gradient-to: hsl(var(--s) / 1) var(--tw-gradient-to-position)}
.focus\:to-secondary-content:focus {
    --tw-gradient-to: hsl(var(--sc) / 1) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/0:focus {
    --tw-gradient-to: hsl(var(--sc) / 0) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/10:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/100:focus {
    --tw-gradient-to: hsl(var(--sc) / 1) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/20:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/25:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/30:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/40:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/5:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/50:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/60:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/70:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/75:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/80:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/90:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-secondary-content\/95:focus {
    --tw-gradient-to: hsl(var(--sc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus:focus {
    --tw-gradient-to: hsl(var(--sf) / 1) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/0:focus {
    --tw-gradient-to: hsl(var(--sf) / 0) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/10:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/100:focus {
    --tw-gradient-to: hsl(var(--sf) / 1) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/20:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/25:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/30:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/40:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/5:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/50:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/60:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/70:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/75:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/80:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/90:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-secondary-focus\/95:focus {
    --tw-gradient-to: hsl(var(--sf) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-secondary\/0:focus {
    --tw-gradient-to: hsl(var(--s) / 0) var(--tw-gradient-to-position)}
.focus\:to-secondary\/10:focus {
    --tw-gradient-to: hsl(var(--s) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-secondary\/100:focus {
    --tw-gradient-to: hsl(var(--s) / 1) var(--tw-gradient-to-position)}
.focus\:to-secondary\/20:focus {
    --tw-gradient-to: hsl(var(--s) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-secondary\/25:focus {
    --tw-gradient-to: hsl(var(--s) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-secondary\/30:focus {
    --tw-gradient-to: hsl(var(--s) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-secondary\/40:focus {
    --tw-gradient-to: hsl(var(--s) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-secondary\/5:focus {
    --tw-gradient-to: hsl(var(--s) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-secondary\/50:focus {
    --tw-gradient-to: hsl(var(--s) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-secondary\/60:focus {
    --tw-gradient-to: hsl(var(--s) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-secondary\/70:focus {
    --tw-gradient-to: hsl(var(--s) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-secondary\/75:focus {
    --tw-gradient-to: hsl(var(--s) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-secondary\/80:focus {
    --tw-gradient-to: hsl(var(--s) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-secondary\/90:focus {
    --tw-gradient-to: hsl(var(--s) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-secondary\/95:focus {
    --tw-gradient-to: hsl(var(--s) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-success:focus {
    --tw-gradient-to: hsl(var(--su) / 1) var(--tw-gradient-to-position)}
.focus\:to-success-content:focus {
    --tw-gradient-to: hsl(var(--suc) / 1) var(--tw-gradient-to-position)}
.focus\:to-success-content\/0:focus {
    --tw-gradient-to: hsl(var(--suc) / 0) var(--tw-gradient-to-position)}
.focus\:to-success-content\/10:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-success-content\/100:focus {
    --tw-gradient-to: hsl(var(--suc) / 1) var(--tw-gradient-to-position)}
.focus\:to-success-content\/20:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-success-content\/25:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-success-content\/30:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-success-content\/40:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-success-content\/5:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-success-content\/50:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-success-content\/60:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-success-content\/70:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-success-content\/75:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-success-content\/80:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-success-content\/90:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-success-content\/95:focus {
    --tw-gradient-to: hsl(var(--suc) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-success\/0:focus {
    --tw-gradient-to: hsl(var(--su) / 0) var(--tw-gradient-to-position)}
.focus\:to-success\/10:focus {
    --tw-gradient-to: hsl(var(--su) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-success\/100:focus {
    --tw-gradient-to: hsl(var(--su) / 1) var(--tw-gradient-to-position)}
.focus\:to-success\/20:focus {
    --tw-gradient-to: hsl(var(--su) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-success\/25:focus {
    --tw-gradient-to: hsl(var(--su) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-success\/30:focus {
    --tw-gradient-to: hsl(var(--su) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-success\/40:focus {
    --tw-gradient-to: hsl(var(--su) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-success\/5:focus {
    --tw-gradient-to: hsl(var(--su) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-success\/50:focus {
    --tw-gradient-to: hsl(var(--su) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-success\/60:focus {
    --tw-gradient-to: hsl(var(--su) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-success\/70:focus {
    --tw-gradient-to: hsl(var(--su) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-success\/75:focus {
    --tw-gradient-to: hsl(var(--su) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-success\/80:focus {
    --tw-gradient-to: hsl(var(--su) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-success\/90:focus {
    --tw-gradient-to: hsl(var(--su) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-success\/95:focus {
    --tw-gradient-to: hsl(var(--su) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-warning:focus {
    --tw-gradient-to: hsl(var(--wa) / 1) var(--tw-gradient-to-position)}
.focus\:to-warning-content:focus {
    --tw-gradient-to: hsl(var(--wac) / 1) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/0:focus {
    --tw-gradient-to: hsl(var(--wac) / 0) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/10:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/100:focus {
    --tw-gradient-to: hsl(var(--wac) / 1) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/20:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/25:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/30:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/40:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/5:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/50:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/60:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/70:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/75:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/80:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/90:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-warning-content\/95:focus {
    --tw-gradient-to: hsl(var(--wac) / 0.95) var(--tw-gradient-to-position)}
.focus\:to-warning\/0:focus {
    --tw-gradient-to: hsl(var(--wa) / 0) var(--tw-gradient-to-position)}
.focus\:to-warning\/10:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.1) var(--tw-gradient-to-position)}
.focus\:to-warning\/100:focus {
    --tw-gradient-to: hsl(var(--wa) / 1) var(--tw-gradient-to-position)}
.focus\:to-warning\/20:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.2) var(--tw-gradient-to-position)}
.focus\:to-warning\/25:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.25) var(--tw-gradient-to-position)}
.focus\:to-warning\/30:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.3) var(--tw-gradient-to-position)}
.focus\:to-warning\/40:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.4) var(--tw-gradient-to-position)}
.focus\:to-warning\/5:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.05) var(--tw-gradient-to-position)}
.focus\:to-warning\/50:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.5) var(--tw-gradient-to-position)}
.focus\:to-warning\/60:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.6) var(--tw-gradient-to-position)}
.focus\:to-warning\/70:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.7) var(--tw-gradient-to-position)}
.focus\:to-warning\/75:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.75) var(--tw-gradient-to-position)}
.focus\:to-warning\/80:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.8) var(--tw-gradient-to-position)}
.focus\:to-warning\/90:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.9) var(--tw-gradient-to-position)}
.focus\:to-warning\/95:focus {
    --tw-gradient-to: hsl(var(--wa) / 0.95) var(--tw-gradient-to-position)}
.focus\:text-accent:focus {
    color: hsl(var(--a) / 1)}
.focus\:text-accent-content:focus {
    color: hsl(var(--ac) / 1)}
.focus\:text-accent-content\/0:focus {
    color: hsl(var(--ac) / 0)}
.focus\:text-accent-content\/10:focus {
    color: hsl(var(--ac) / 0.1)}
.focus\:text-accent-content\/100:focus {
    color: hsl(var(--ac) / 1)}
.focus\:text-accent-content\/20:focus {
    color: hsl(var(--ac) / 0.2)}
.focus\:text-accent-content\/25:focus {
    color: hsl(var(--ac) / 0.25)}
.focus\:text-accent-content\/30:focus {
    color: hsl(var(--ac) / 0.3)}
.focus\:text-accent-content\/40:focus {
    color: hsl(var(--ac) / 0.4)}
.focus\:text-accent-content\/5:focus {
    color: hsl(var(--ac) / 0.05)}
.focus\:text-accent-content\/50:focus {
    color: hsl(var(--ac) / 0.5)}
.focus\:text-accent-content\/60:focus {
    color: hsl(var(--ac) / 0.6)}
.focus\:text-accent-content\/70:focus {
    color: hsl(var(--ac) / 0.7)}
.focus\:text-accent-content\/75:focus {
    color: hsl(var(--ac) / 0.75)}
.focus\:text-accent-content\/80:focus {
    color: hsl(var(--ac) / 0.8)}
.focus\:text-accent-content\/90:focus {
    color: hsl(var(--ac) / 0.9)}
.focus\:text-accent-content\/95:focus {
    color: hsl(var(--ac) / 0.95)}
.focus\:text-accent-focus:focus {
    color: hsl(var(--af) / 1)}
.focus\:text-accent-focus\/0:focus {
    color: hsl(var(--af) / 0)}
.focus\:text-accent-focus\/10:focus {
    color: hsl(var(--af) / 0.1)}
.focus\:text-accent-focus\/100:focus {
    color: hsl(var(--af) / 1)}
.focus\:text-accent-focus\/20:focus {
    color: hsl(var(--af) / 0.2)}
.focus\:text-accent-focus\/25:focus {
    color: hsl(var(--af) / 0.25)}
.focus\:text-accent-focus\/30:focus {
    color: hsl(var(--af) / 0.3)}
.focus\:text-accent-focus\/40:focus {
    color: hsl(var(--af) / 0.4)}
.focus\:text-accent-focus\/5:focus {
    color: hsl(var(--af) / 0.05)}
.focus\:text-accent-focus\/50:focus {
    color: hsl(var(--af) / 0.5)}
.focus\:text-accent-focus\/60:focus {
    color: hsl(var(--af) / 0.6)}
.focus\:text-accent-focus\/70:focus {
    color: hsl(var(--af) / 0.7)}
.focus\:text-accent-focus\/75:focus {
    color: hsl(var(--af) / 0.75)}
.focus\:text-accent-focus\/80:focus {
    color: hsl(var(--af) / 0.8)}
.focus\:text-accent-focus\/90:focus {
    color: hsl(var(--af) / 0.9)}
.focus\:text-accent-focus\/95:focus {
    color: hsl(var(--af) / 0.95)}
.focus\:text-accent\/0:focus {
    color: hsl(var(--a) / 0)}
.focus\:text-accent\/10:focus {
    color: hsl(var(--a) / 0.1)}
.focus\:text-accent\/100:focus {
    color: hsl(var(--a) / 1)}
.focus\:text-accent\/20:focus {
    color: hsl(var(--a) / 0.2)}
.focus\:text-accent\/25:focus {
    color: hsl(var(--a) / 0.25)}
.focus\:text-accent\/30:focus {
    color: hsl(var(--a) / 0.3)}
.focus\:text-accent\/40:focus {
    color: hsl(var(--a) / 0.4)}
.focus\:text-accent\/5:focus {
    color: hsl(var(--a) / 0.05)}
.focus\:text-accent\/50:focus {
    color: hsl(var(--a) / 0.5)}
.focus\:text-accent\/60:focus {
    color: hsl(var(--a) / 0.6)}
.focus\:text-accent\/70:focus {
    color: hsl(var(--a) / 0.7)}
.focus\:text-accent\/75:focus {
    color: hsl(var(--a) / 0.75)}
.focus\:text-accent\/80:focus {
    color: hsl(var(--a) / 0.8)}
.focus\:text-accent\/90:focus {
    color: hsl(var(--a) / 0.9)}
.focus\:text-accent\/95:focus {
    color: hsl(var(--a) / 0.95)}
.focus\:text-base-100:focus {
    color: hsl(var(--b1) / 1)}
.focus\:text-base-100\/0:focus {
    color: hsl(var(--b1) / 0)}
.focus\:text-base-100\/10:focus {
    color: hsl(var(--b1) / 0.1)}
.focus\:text-base-100\/100:focus {
    color: hsl(var(--b1) / 1)}
.focus\:text-base-100\/20:focus {
    color: hsl(var(--b1) / 0.2)}
.focus\:text-base-100\/25:focus {
    color: hsl(var(--b1) / 0.25)}
.focus\:text-base-100\/30:focus {
    color: hsl(var(--b1) / 0.3)}
.focus\:text-base-100\/40:focus {
    color: hsl(var(--b1) / 0.4)}
.focus\:text-base-100\/5:focus {
    color: hsl(var(--b1) / 0.05)}
.focus\:text-base-100\/50:focus {
    color: hsl(var(--b1) / 0.5)}
.focus\:text-base-100\/60:focus {
    color: hsl(var(--b1) / 0.6)}
.focus\:text-base-100\/70:focus {
    color: hsl(var(--b1) / 0.7)}
.focus\:text-base-100\/75:focus {
    color: hsl(var(--b1) / 0.75)}
.focus\:text-base-100\/80:focus {
    color: hsl(var(--b1) / 0.8)}
.focus\:text-base-100\/90:focus {
    color: hsl(var(--b1) / 0.9)}
.focus\:text-base-100\/95:focus {
    color: hsl(var(--b1) / 0.95)}
.focus\:text-base-200:focus {
    color: hsl(var(--b2) / 1)}
.focus\:text-base-200\/0:focus {
    color: hsl(var(--b2) / 0)}
.focus\:text-base-200\/10:focus {
    color: hsl(var(--b2) / 0.1)}
.focus\:text-base-200\/100:focus {
    color: hsl(var(--b2) / 1)}
.focus\:text-base-200\/20:focus {
    color: hsl(var(--b2) / 0.2)}
.focus\:text-base-200\/25:focus {
    color: hsl(var(--b2) / 0.25)}
.focus\:text-base-200\/30:focus {
    color: hsl(var(--b2) / 0.3)}
.focus\:text-base-200\/40:focus {
    color: hsl(var(--b2) / 0.4)}
.focus\:text-base-200\/5:focus {
    color: hsl(var(--b2) / 0.05)}
.focus\:text-base-200\/50:focus {
    color: hsl(var(--b2) / 0.5)}
.focus\:text-base-200\/60:focus {
    color: hsl(var(--b2) / 0.6)}
.focus\:text-base-200\/70:focus {
    color: hsl(var(--b2) / 0.7)}
.focus\:text-base-200\/75:focus {
    color: hsl(var(--b2) / 0.75)}
.focus\:text-base-200\/80:focus {
    color: hsl(var(--b2) / 0.8)}
.focus\:text-base-200\/90:focus {
    color: hsl(var(--b2) / 0.9)}
.focus\:text-base-200\/95:focus {
    color: hsl(var(--b2) / 0.95)}
.focus\:text-base-300:focus {
    color: hsl(var(--b3) / 1)}
.focus\:text-base-300\/0:focus {
    color: hsl(var(--b3) / 0)}
.focus\:text-base-300\/10:focus {
    color: hsl(var(--b3) / 0.1)}
.focus\:text-base-300\/100:focus {
    color: hsl(var(--b3) / 1)}
.focus\:text-base-300\/20:focus {
    color: hsl(var(--b3) / 0.2)}
.focus\:text-base-300\/25:focus {
    color: hsl(var(--b3) / 0.25)}
.focus\:text-base-300\/30:focus {
    color: hsl(var(--b3) / 0.3)}
.focus\:text-base-300\/40:focus {
    color: hsl(var(--b3) / 0.4)}
.focus\:text-base-300\/5:focus {
    color: hsl(var(--b3) / 0.05)}
.focus\:text-base-300\/50:focus {
    color: hsl(var(--b3) / 0.5)}
.focus\:text-base-300\/60:focus {
    color: hsl(var(--b3) / 0.6)}
.focus\:text-base-300\/70:focus {
    color: hsl(var(--b3) / 0.7)}
.focus\:text-base-300\/75:focus {
    color: hsl(var(--b3) / 0.75)}
.focus\:text-base-300\/80:focus {
    color: hsl(var(--b3) / 0.8)}
.focus\:text-base-300\/90:focus {
    color: hsl(var(--b3) / 0.9)}
.focus\:text-base-300\/95:focus {
    color: hsl(var(--b3) / 0.95)}
.focus\:text-base-content:focus {
    color: hsl(var(--bc) / 1)}
.focus\:text-base-content\/0:focus {
    color: hsl(var(--bc) / 0)}
.focus\:text-base-content\/10:focus {
    color: hsl(var(--bc) / 0.1)}
.focus\:text-base-content\/100:focus {
    color: hsl(var(--bc) / 1)}
.focus\:text-base-content\/20:focus {
    color: hsl(var(--bc) / 0.2)}
.focus\:text-base-content\/25:focus {
    color: hsl(var(--bc) / 0.25)}
.focus\:text-base-content\/30:focus {
    color: hsl(var(--bc) / 0.3)}
.focus\:text-base-content\/40:focus {
    color: hsl(var(--bc) / 0.4)}
.focus\:text-base-content\/5:focus {
    color: hsl(var(--bc) / 0.05)}
.focus\:text-base-content\/50:focus {
    color: hsl(var(--bc) / 0.5)}
.focus\:text-base-content\/60:focus {
    color: hsl(var(--bc) / 0.6)}
.focus\:text-base-content\/70:focus {
    color: hsl(var(--bc) / 0.7)}
.focus\:text-base-content\/75:focus {
    color: hsl(var(--bc) / 0.75)}
.focus\:text-base-content\/80:focus {
    color: hsl(var(--bc) / 0.8)}
.focus\:text-base-content\/90:focus {
    color: hsl(var(--bc) / 0.9)}
.focus\:text-base-content\/95:focus {
    color: hsl(var(--bc) / 0.95)}
.focus\:text-error:focus {
    color: hsl(var(--er) / 1)}
.focus\:text-error-content:focus {
    color: hsl(var(--erc) / 1)}
.focus\:text-error-content\/0:focus {
    color: hsl(var(--erc) / 0)}
.focus\:text-error-content\/10:focus {
    color: hsl(var(--erc) / 0.1)}
.focus\:text-error-content\/100:focus {
    color: hsl(var(--erc) / 1)}
.focus\:text-error-content\/20:focus {
    color: hsl(var(--erc) / 0.2)}
.focus\:text-error-content\/25:focus {
    color: hsl(var(--erc) / 0.25)}
.focus\:text-error-content\/30:focus {
    color: hsl(var(--erc) / 0.3)}
.focus\:text-error-content\/40:focus {
    color: hsl(var(--erc) / 0.4)}
.focus\:text-error-content\/5:focus {
    color: hsl(var(--erc) / 0.05)}
.focus\:text-error-content\/50:focus {
    color: hsl(var(--erc) / 0.5)}
.focus\:text-error-content\/60:focus {
    color: hsl(var(--erc) / 0.6)}
.focus\:text-error-content\/70:focus {
    color: hsl(var(--erc) / 0.7)}
.focus\:text-error-content\/75:focus {
    color: hsl(var(--erc) / 0.75)}
.focus\:text-error-content\/80:focus {
    color: hsl(var(--erc) / 0.8)}
.focus\:text-error-content\/90:focus {
    color: hsl(var(--erc) / 0.9)}
.focus\:text-error-content\/95:focus {
    color: hsl(var(--erc) / 0.95)}
.focus\:text-error\/0:focus {
    color: hsl(var(--er) / 0)}
.focus\:text-error\/10:focus {
    color: hsl(var(--er) / 0.1)}
.focus\:text-error\/100:focus {
    color: hsl(var(--er) / 1)}
.focus\:text-error\/20:focus {
    color: hsl(var(--er) / 0.2)}
.focus\:text-error\/25:focus {
    color: hsl(var(--er) / 0.25)}
.focus\:text-error\/30:focus {
    color: hsl(var(--er) / 0.3)}
.focus\:text-error\/40:focus {
    color: hsl(var(--er) / 0.4)}
.focus\:text-error\/5:focus {
    color: hsl(var(--er) / 0.05)}
.focus\:text-error\/50:focus {
    color: hsl(var(--er) / 0.5)}
.focus\:text-error\/60:focus {
    color: hsl(var(--er) / 0.6)}
.focus\:text-error\/70:focus {
    color: hsl(var(--er) / 0.7)}
.focus\:text-error\/75:focus {
    color: hsl(var(--er) / 0.75)}
.focus\:text-error\/80:focus {
    color: hsl(var(--er) / 0.8)}
.focus\:text-error\/90:focus {
    color: hsl(var(--er) / 0.9)}
.focus\:text-error\/95:focus {
    color: hsl(var(--er) / 0.95)}
.focus\:text-info:focus {
    color: hsl(var(--in) / 1)}
.focus\:text-info-content:focus {
    color: hsl(var(--inc) / 1)}
.focus\:text-info-content\/0:focus {
    color: hsl(var(--inc) / 0)}
.focus\:text-info-content\/10:focus {
    color: hsl(var(--inc) / 0.1)}
.focus\:text-info-content\/100:focus {
    color: hsl(var(--inc) / 1)}
.focus\:text-info-content\/20:focus {
    color: hsl(var(--inc) / 0.2)}
.focus\:text-info-content\/25:focus {
    color: hsl(var(--inc) / 0.25)}
.focus\:text-info-content\/30:focus {
    color: hsl(var(--inc) / 0.3)}
.focus\:text-info-content\/40:focus {
    color: hsl(var(--inc) / 0.4)}
.focus\:text-info-content\/5:focus {
    color: hsl(var(--inc) / 0.05)}
.focus\:text-info-content\/50:focus {
    color: hsl(var(--inc) / 0.5)}
.focus\:text-info-content\/60:focus {
    color: hsl(var(--inc) / 0.6)}
.focus\:text-info-content\/70:focus {
    color: hsl(var(--inc) / 0.7)}
.focus\:text-info-content\/75:focus {
    color: hsl(var(--inc) / 0.75)}
.focus\:text-info-content\/80:focus {
    color: hsl(var(--inc) / 0.8)}
.focus\:text-info-content\/90:focus {
    color: hsl(var(--inc) / 0.9)}
.focus\:text-info-content\/95:focus {
    color: hsl(var(--inc) / 0.95)}
.focus\:text-info\/0:focus {
    color: hsl(var(--in) / 0)}
.focus\:text-info\/10:focus {
    color: hsl(var(--in) / 0.1)}
.focus\:text-info\/100:focus {
    color: hsl(var(--in) / 1)}
.focus\:text-info\/20:focus {
    color: hsl(var(--in) / 0.2)}
.focus\:text-info\/25:focus {
    color: hsl(var(--in) / 0.25)}
.focus\:text-info\/30:focus {
    color: hsl(var(--in) / 0.3)}
.focus\:text-info\/40:focus {
    color: hsl(var(--in) / 0.4)}
.focus\:text-info\/5:focus {
    color: hsl(var(--in) / 0.05)}
.focus\:text-info\/50:focus {
    color: hsl(var(--in) / 0.5)}
.focus\:text-info\/60:focus {
    color: hsl(var(--in) / 0.6)}
.focus\:text-info\/70:focus {
    color: hsl(var(--in) / 0.7)}
.focus\:text-info\/75:focus {
    color: hsl(var(--in) / 0.75)}
.focus\:text-info\/80:focus {
    color: hsl(var(--in) / 0.8)}
.focus\:text-info\/90:focus {
    color: hsl(var(--in) / 0.9)}
.focus\:text-info\/95:focus {
    color: hsl(var(--in) / 0.95)}
.focus\:text-neutral:focus {
    color: hsl(var(--n) / 1)}
.focus\:text-neutral-content:focus {
    color: hsl(var(--nc) / 1)}
.focus\:text-neutral-content\/0:focus {
    color: hsl(var(--nc) / 0)}
.focus\:text-neutral-content\/10:focus {
    color: hsl(var(--nc) / 0.1)}
.focus\:text-neutral-content\/100:focus {
    color: hsl(var(--nc) / 1)}
.focus\:text-neutral-content\/20:focus {
    color: hsl(var(--nc) / 0.2)}
.focus\:text-neutral-content\/25:focus {
    color: hsl(var(--nc) / 0.25)}
.focus\:text-neutral-content\/30:focus {
    color: hsl(var(--nc) / 0.3)}
.focus\:text-neutral-content\/40:focus {
    color: hsl(var(--nc) / 0.4)}
.focus\:text-neutral-content\/5:focus {
    color: hsl(var(--nc) / 0.05)}
.focus\:text-neutral-content\/50:focus {
    color: hsl(var(--nc) / 0.5)}
.focus\:text-neutral-content\/60:focus {
    color: hsl(var(--nc) / 0.6)}
.focus\:text-neutral-content\/70:focus {
    color: hsl(var(--nc) / 0.7)}
.focus\:text-neutral-content\/75:focus {
    color: hsl(var(--nc) / 0.75)}
.focus\:text-neutral-content\/80:focus {
    color: hsl(var(--nc) / 0.8)}
.focus\:text-neutral-content\/90:focus {
    color: hsl(var(--nc) / 0.9)}
.focus\:text-neutral-content\/95:focus {
    color: hsl(var(--nc) / 0.95)}
.focus\:text-neutral-focus:focus {
    color: hsl(var(--nf) / 1)}
.focus\:text-neutral-focus\/0:focus {
    color: hsl(var(--nf) / 0)}
.focus\:text-neutral-focus\/10:focus {
    color: hsl(var(--nf) / 0.1)}
.focus\:text-neutral-focus\/100:focus {
    color: hsl(var(--nf) / 1)}
.focus\:text-neutral-focus\/20:focus {
    color: hsl(var(--nf) / 0.2)}
.focus\:text-neutral-focus\/25:focus {
    color: hsl(var(--nf) / 0.25)}
.focus\:text-neutral-focus\/30:focus {
    color: hsl(var(--nf) / 0.3)}
.focus\:text-neutral-focus\/40:focus {
    color: hsl(var(--nf) / 0.4)}
.focus\:text-neutral-focus\/5:focus {
    color: hsl(var(--nf) / 0.05)}
.focus\:text-neutral-focus\/50:focus {
    color: hsl(var(--nf) / 0.5)}
.focus\:text-neutral-focus\/60:focus {
    color: hsl(var(--nf) / 0.6)}
.focus\:text-neutral-focus\/70:focus {
    color: hsl(var(--nf) / 0.7)}
.focus\:text-neutral-focus\/75:focus {
    color: hsl(var(--nf) / 0.75)}
.focus\:text-neutral-focus\/80:focus {
    color: hsl(var(--nf) / 0.8)}
.focus\:text-neutral-focus\/90:focus {
    color: hsl(var(--nf) / 0.9)}
.focus\:text-neutral-focus\/95:focus {
    color: hsl(var(--nf) / 0.95)}
.focus\:text-neutral\/0:focus {
    color: hsl(var(--n) / 0)}
.focus\:text-neutral\/10:focus {
    color: hsl(var(--n) / 0.1)}
.focus\:text-neutral\/100:focus {
    color: hsl(var(--n) / 1)}
.focus\:text-neutral\/20:focus {
    color: hsl(var(--n) / 0.2)}
.focus\:text-neutral\/25:focus {
    color: hsl(var(--n) / 0.25)}
.focus\:text-neutral\/30:focus {
    color: hsl(var(--n) / 0.3)}
.focus\:text-neutral\/40:focus {
    color: hsl(var(--n) / 0.4)}
.focus\:text-neutral\/5:focus {
    color: hsl(var(--n) / 0.05)}
.focus\:text-neutral\/50:focus {
    color: hsl(var(--n) / 0.5)}
.focus\:text-neutral\/60:focus {
    color: hsl(var(--n) / 0.6)}
.focus\:text-neutral\/70:focus {
    color: hsl(var(--n) / 0.7)}
.focus\:text-neutral\/75:focus {
    color: hsl(var(--n) / 0.75)}
.focus\:text-neutral\/80:focus {
    color: hsl(var(--n) / 0.8)}
.focus\:text-neutral\/90:focus {
    color: hsl(var(--n) / 0.9)}
.focus\:text-neutral\/95:focus {
    color: hsl(var(--n) / 0.95)}
.focus\:text-primary:focus {
    color: hsl(var(--p) / 1)}
.focus\:text-primary-content:focus {
    color: hsl(var(--pc) / 1)}
.focus\:text-primary-content\/0:focus {
    color: hsl(var(--pc) / 0)}
.focus\:text-primary-content\/10:focus {
    color: hsl(var(--pc) / 0.1)}
.focus\:text-primary-content\/100:focus {
    color: hsl(var(--pc) / 1)}
.focus\:text-primary-content\/20:focus {
    color: hsl(var(--pc) / 0.2)}
.focus\:text-primary-content\/25:focus {
    color: hsl(var(--pc) / 0.25)}
.focus\:text-primary-content\/30:focus {
    color: hsl(var(--pc) / 0.3)}
.focus\:text-primary-content\/40:focus {
    color: hsl(var(--pc) / 0.4)}
.focus\:text-primary-content\/5:focus {
    color: hsl(var(--pc) / 0.05)}
.focus\:text-primary-content\/50:focus {
    color: hsl(var(--pc) / 0.5)}
.focus\:text-primary-content\/60:focus {
    color: hsl(var(--pc) / 0.6)}
.focus\:text-primary-content\/70:focus {
    color: hsl(var(--pc) / 0.7)}
.focus\:text-primary-content\/75:focus {
    color: hsl(var(--pc) / 0.75)}
.focus\:text-primary-content\/80:focus {
    color: hsl(var(--pc) / 0.8)}
.focus\:text-primary-content\/90:focus {
    color: hsl(var(--pc) / 0.9)}
.focus\:text-primary-content\/95:focus {
    color: hsl(var(--pc) / 0.95)}
.focus\:text-primary-focus:focus {
    color: hsl(var(--pf) / 1)}
.focus\:text-primary-focus\/0:focus {
    color: hsl(var(--pf) / 0)}
.focus\:text-primary-focus\/10:focus {
    color: hsl(var(--pf) / 0.1)}
.focus\:text-primary-focus\/100:focus {
    color: hsl(var(--pf) / 1)}
.focus\:text-primary-focus\/20:focus {
    color: hsl(var(--pf) / 0.2)}
.focus\:text-primary-focus\/25:focus {
    color: hsl(var(--pf) / 0.25)}
.focus\:text-primary-focus\/30:focus {
    color: hsl(var(--pf) / 0.3)}
.focus\:text-primary-focus\/40:focus {
    color: hsl(var(--pf) / 0.4)}
.focus\:text-primary-focus\/5:focus {
    color: hsl(var(--pf) / 0.05)}
.focus\:text-primary-focus\/50:focus {
    color: hsl(var(--pf) / 0.5)}
.focus\:text-primary-focus\/60:focus {
    color: hsl(var(--pf) / 0.6)}
.focus\:text-primary-focus\/70:focus {
    color: hsl(var(--pf) / 0.7)}
.focus\:text-primary-focus\/75:focus {
    color: hsl(var(--pf) / 0.75)}
.focus\:text-primary-focus\/80:focus {
    color: hsl(var(--pf) / 0.8)}
.focus\:text-primary-focus\/90:focus {
    color: hsl(var(--pf) / 0.9)}
.focus\:text-primary-focus\/95:focus {
    color: hsl(var(--pf) / 0.95)}
.focus\:text-primary\/0:focus {
    color: hsl(var(--p) / 0)}
.focus\:text-primary\/10:focus {
    color: hsl(var(--p) / 0.1)}
.focus\:text-primary\/100:focus {
    color: hsl(var(--p) / 1)}
.focus\:text-primary\/20:focus {
    color: hsl(var(--p) / 0.2)}
.focus\:text-primary\/25:focus {
    color: hsl(var(--p) / 0.25)}
.focus\:text-primary\/30:focus {
    color: hsl(var(--p) / 0.3)}
.focus\:text-primary\/40:focus {
    color: hsl(var(--p) / 0.4)}
.focus\:text-primary\/5:focus {
    color: hsl(var(--p) / 0.05)}
.focus\:text-primary\/50:focus {
    color: hsl(var(--p) / 0.5)}
.focus\:text-primary\/60:focus {
    color: hsl(var(--p) / 0.6)}
.focus\:text-primary\/70:focus {
    color: hsl(var(--p) / 0.7)}
.focus\:text-primary\/75:focus {
    color: hsl(var(--p) / 0.75)}
.focus\:text-primary\/80:focus {
    color: hsl(var(--p) / 0.8)}
.focus\:text-primary\/90:focus {
    color: hsl(var(--p) / 0.9)}
.focus\:text-primary\/95:focus {
    color: hsl(var(--p) / 0.95)}
.focus\:text-secondary:focus {
    color: hsl(var(--s) / 1)}
.focus\:text-secondary-content:focus {
    color: hsl(var(--sc) / 1)}
.focus\:text-secondary-content\/0:focus {
    color: hsl(var(--sc) / 0)}
.focus\:text-secondary-content\/10:focus {
    color: hsl(var(--sc) / 0.1)}
.focus\:text-secondary-content\/100:focus {
    color: hsl(var(--sc) / 1)}
.focus\:text-secondary-content\/20:focus {
    color: hsl(var(--sc) / 0.2)}
.focus\:text-secondary-content\/25:focus {
    color: hsl(var(--sc) / 0.25)}
.focus\:text-secondary-content\/30:focus {
    color: hsl(var(--sc) / 0.3)}
.focus\:text-secondary-content\/40:focus {
    color: hsl(var(--sc) / 0.4)}
.focus\:text-secondary-content\/5:focus {
    color: hsl(var(--sc) / 0.05)}
.focus\:text-secondary-content\/50:focus {
    color: hsl(var(--sc) / 0.5)}
.focus\:text-secondary-content\/60:focus {
    color: hsl(var(--sc) / 0.6)}
.focus\:text-secondary-content\/70:focus {
    color: hsl(var(--sc) / 0.7)}
.focus\:text-secondary-content\/75:focus {
    color: hsl(var(--sc) / 0.75)}
.focus\:text-secondary-content\/80:focus {
    color: hsl(var(--sc) / 0.8)}
.focus\:text-secondary-content\/90:focus {
    color: hsl(var(--sc) / 0.9)}
.focus\:text-secondary-content\/95:focus {
    color: hsl(var(--sc) / 0.95)}
.focus\:text-secondary-focus:focus {
    color: hsl(var(--sf) / 1)}
.focus\:text-secondary-focus\/0:focus {
    color: hsl(var(--sf) / 0)}
.focus\:text-secondary-focus\/10:focus {
    color: hsl(var(--sf) / 0.1)}
.focus\:text-secondary-focus\/100:focus {
    color: hsl(var(--sf) / 1)}
.focus\:text-secondary-focus\/20:focus {
    color: hsl(var(--sf) / 0.2)}
.focus\:text-secondary-focus\/25:focus {
    color: hsl(var(--sf) / 0.25)}
.focus\:text-secondary-focus\/30:focus {
    color: hsl(var(--sf) / 0.3)}
.focus\:text-secondary-focus\/40:focus {
    color: hsl(var(--sf) / 0.4)}
.focus\:text-secondary-focus\/5:focus {
    color: hsl(var(--sf) / 0.05)}
.focus\:text-secondary-focus\/50:focus {
    color: hsl(var(--sf) / 0.5)}
.focus\:text-secondary-focus\/60:focus {
    color: hsl(var(--sf) / 0.6)}
.focus\:text-secondary-focus\/70:focus {
    color: hsl(var(--sf) / 0.7)}
.focus\:text-secondary-focus\/75:focus {
    color: hsl(var(--sf) / 0.75)}
.focus\:text-secondary-focus\/80:focus {
    color: hsl(var(--sf) / 0.8)}
.focus\:text-secondary-focus\/90:focus {
    color: hsl(var(--sf) / 0.9)}
.focus\:text-secondary-focus\/95:focus {
    color: hsl(var(--sf) / 0.95)}
.focus\:text-secondary\/0:focus {
    color: hsl(var(--s) / 0)}
.focus\:text-secondary\/10:focus {
    color: hsl(var(--s) / 0.1)}
.focus\:text-secondary\/100:focus {
    color: hsl(var(--s) / 1)}
.focus\:text-secondary\/20:focus {
    color: hsl(var(--s) / 0.2)}
.focus\:text-secondary\/25:focus {
    color: hsl(var(--s) / 0.25)}
.focus\:text-secondary\/30:focus {
    color: hsl(var(--s) / 0.3)}
.focus\:text-secondary\/40:focus {
    color: hsl(var(--s) / 0.4)}
.focus\:text-secondary\/5:focus {
    color: hsl(var(--s) / 0.05)}
.focus\:text-secondary\/50:focus {
    color: hsl(var(--s) / 0.5)}
.focus\:text-secondary\/60:focus {
    color: hsl(var(--s) / 0.6)}
.focus\:text-secondary\/70:focus {
    color: hsl(var(--s) / 0.7)}
.focus\:text-secondary\/75:focus {
    color: hsl(var(--s) / 0.75)}
.focus\:text-secondary\/80:focus {
    color: hsl(var(--s) / 0.8)}
.focus\:text-secondary\/90:focus {
    color: hsl(var(--s) / 0.9)}
.focus\:text-secondary\/95:focus {
    color: hsl(var(--s) / 0.95)}
.focus\:text-success:focus {
    color: hsl(var(--su) / 1)}
.focus\:text-success-content:focus {
    color: hsl(var(--suc) / 1)}
.focus\:text-success-content\/0:focus {
    color: hsl(var(--suc) / 0)}
.focus\:text-success-content\/10:focus {
    color: hsl(var(--suc) / 0.1)}
.focus\:text-success-content\/100:focus {
    color: hsl(var(--suc) / 1)}
.focus\:text-success-content\/20:focus {
    color: hsl(var(--suc) / 0.2)}
.focus\:text-success-content\/25:focus {
    color: hsl(var(--suc) / 0.25)}
.focus\:text-success-content\/30:focus {
    color: hsl(var(--suc) / 0.3)}
.focus\:text-success-content\/40:focus {
    color: hsl(var(--suc) / 0.4)}
.focus\:text-success-content\/5:focus {
    color: hsl(var(--suc) / 0.05)}
.focus\:text-success-content\/50:focus {
    color: hsl(var(--suc) / 0.5)}
.focus\:text-success-content\/60:focus {
    color: hsl(var(--suc) / 0.6)}
.focus\:text-success-content\/70:focus {
    color: hsl(var(--suc) / 0.7)}
.focus\:text-success-content\/75:focus {
    color: hsl(var(--suc) / 0.75)}
.focus\:text-success-content\/80:focus {
    color: hsl(var(--suc) / 0.8)}
.focus\:text-success-content\/90:focus {
    color: hsl(var(--suc) / 0.9)}
.focus\:text-success-content\/95:focus {
    color: hsl(var(--suc) / 0.95)}
.focus\:text-success\/0:focus {
    color: hsl(var(--su) / 0)}
.focus\:text-success\/10:focus {
    color: hsl(var(--su) / 0.1)}
.focus\:text-success\/100:focus {
    color: hsl(var(--su) / 1)}
.focus\:text-success\/20:focus {
    color: hsl(var(--su) / 0.2)}
.focus\:text-success\/25:focus {
    color: hsl(var(--su) / 0.25)}
.focus\:text-success\/30:focus {
    color: hsl(var(--su) / 0.3)}
.focus\:text-success\/40:focus {
    color: hsl(var(--su) / 0.4)}
.focus\:text-success\/5:focus {
    color: hsl(var(--su) / 0.05)}
.focus\:text-success\/50:focus {
    color: hsl(var(--su) / 0.5)}
.focus\:text-success\/60:focus {
    color: hsl(var(--su) / 0.6)}
.focus\:text-success\/70:focus {
    color: hsl(var(--su) / 0.7)}
.focus\:text-success\/75:focus {
    color: hsl(var(--su) / 0.75)}
.focus\:text-success\/80:focus {
    color: hsl(var(--su) / 0.8)}
.focus\:text-success\/90:focus {
    color: hsl(var(--su) / 0.9)}
.focus\:text-success\/95:focus {
    color: hsl(var(--su) / 0.95)}
.focus\:text-warning:focus {
    color: hsl(var(--wa) / 1)}
.focus\:text-warning-content:focus {
    color: hsl(var(--wac) / 1)}
.focus\:text-warning-content\/0:focus {
    color: hsl(var(--wac) / 0)}
.focus\:text-warning-content\/10:focus {
    color: hsl(var(--wac) / 0.1)}
.focus\:text-warning-content\/100:focus {
    color: hsl(var(--wac) / 1)}
.focus\:text-warning-content\/20:focus {
    color: hsl(var(--wac) / 0.2)}
.focus\:text-warning-content\/25:focus {
    color: hsl(var(--wac) / 0.25)}
.focus\:text-warning-content\/30:focus {
    color: hsl(var(--wac) / 0.3)}
.focus\:text-warning-content\/40:focus {
    color: hsl(var(--wac) / 0.4)}
.focus\:text-warning-content\/5:focus {
    color: hsl(var(--wac) / 0.05)}
.focus\:text-warning-content\/50:focus {
    color: hsl(var(--wac) / 0.5)}
.focus\:text-warning-content\/60:focus {
    color: hsl(var(--wac) / 0.6)}
.focus\:text-warning-content\/70:focus {
    color: hsl(var(--wac) / 0.7)}
.focus\:text-warning-content\/75:focus {
    color: hsl(var(--wac) / 0.75)}
.focus\:text-warning-content\/80:focus {
    color: hsl(var(--wac) / 0.8)}
.focus\:text-warning-content\/90:focus {
    color: hsl(var(--wac) / 0.9)}
.focus\:text-warning-content\/95:focus {
    color: hsl(var(--wac) / 0.95)}
.focus\:text-warning\/0:focus {
    color: hsl(var(--wa) / 0)}
.focus\:text-warning\/10:focus {
    color: hsl(var(--wa) / 0.1)}
.focus\:text-warning\/100:focus {
    color: hsl(var(--wa) / 1)}
.focus\:text-warning\/20:focus {
    color: hsl(var(--wa) / 0.2)}
.focus\:text-warning\/25:focus {
    color: hsl(var(--wa) / 0.25)}
.focus\:text-warning\/30:focus {
    color: hsl(var(--wa) / 0.3)}
.focus\:text-warning\/40:focus {
    color: hsl(var(--wa) / 0.4)}
.focus\:text-warning\/5:focus {
    color: hsl(var(--wa) / 0.05)}
.focus\:text-warning\/50:focus {
    color: hsl(var(--wa) / 0.5)}
.focus\:text-warning\/60:focus {
    color: hsl(var(--wa) / 0.6)}
.focus\:text-warning\/70:focus {
    color: hsl(var(--wa) / 0.7)}
.focus\:text-warning\/75:focus {
    color: hsl(var(--wa) / 0.75)}
.focus\:text-warning\/80:focus {
    color: hsl(var(--wa) / 0.8)}
.focus\:text-warning\/90:focus {
    color: hsl(var(--wa) / 0.9)}
.focus\:text-warning\/95:focus {
    color: hsl(var(--wa) / 0.95)}
.focus\:placeholder-base-100:focus::placeholder {
    color: hsl(var(--b1) / 1)}
.focus\:placeholder-base-100\/0:focus::placeholder {
    color: hsl(var(--b1) / 0)}
.focus\:placeholder-base-100\/10:focus::placeholder {
    color: hsl(var(--b1) / 0.1)}
.focus\:placeholder-base-100\/100:focus::placeholder {
    color: hsl(var(--b1) / 1)}
.focus\:placeholder-base-100\/20:focus::placeholder {
    color: hsl(var(--b1) / 0.2)}
.focus\:placeholder-base-100\/25:focus::placeholder {
    color: hsl(var(--b1) / 0.25)}
.focus\:placeholder-base-100\/30:focus::placeholder {
    color: hsl(var(--b1) / 0.3)}
.focus\:placeholder-base-100\/40:focus::placeholder {
    color: hsl(var(--b1) / 0.4)}
.focus\:placeholder-base-100\/5:focus::placeholder {
    color: hsl(var(--b1) / 0.05)}
.focus\:placeholder-base-100\/50:focus::placeholder {
    color: hsl(var(--b1) / 0.5)}
.focus\:placeholder-base-100\/60:focus::placeholder {
    color: hsl(var(--b1) / 0.6)}
.focus\:placeholder-base-100\/70:focus::placeholder {
    color: hsl(var(--b1) / 0.7)}
.focus\:placeholder-base-100\/75:focus::placeholder {
    color: hsl(var(--b1) / 0.75)}
.focus\:placeholder-base-100\/80:focus::placeholder {
    color: hsl(var(--b1) / 0.8)}
.focus\:placeholder-base-100\/90:focus::placeholder {
    color: hsl(var(--b1) / 0.9)}
.focus\:placeholder-base-100\/95:focus::placeholder {
    color: hsl(var(--b1) / 0.95)}
.focus\:placeholder-base-200:focus::placeholder {
    color: hsl(var(--b2) / 1)}
.focus\:placeholder-base-200\/0:focus::placeholder {
    color: hsl(var(--b2) / 0)}
.focus\:placeholder-base-200\/10:focus::placeholder {
    color: hsl(var(--b2) / 0.1)}
.focus\:placeholder-base-200\/100:focus::placeholder {
    color: hsl(var(--b2) / 1)}
.focus\:placeholder-base-200\/20:focus::placeholder {
    color: hsl(var(--b2) / 0.2)}
.focus\:placeholder-base-200\/25:focus::placeholder {
    color: hsl(var(--b2) / 0.25)}
.focus\:placeholder-base-200\/30:focus::placeholder {
    color: hsl(var(--b2) / 0.3)}
.focus\:placeholder-base-200\/40:focus::placeholder {
    color: hsl(var(--b2) / 0.4)}
.focus\:placeholder-base-200\/5:focus::placeholder {
    color: hsl(var(--b2) / 0.05)}
.focus\:placeholder-base-200\/50:focus::placeholder {
    color: hsl(var(--b2) / 0.5)}
.focus\:placeholder-base-200\/60:focus::placeholder {
    color: hsl(var(--b2) / 0.6)}
.focus\:placeholder-base-200\/70:focus::placeholder {
    color: hsl(var(--b2) / 0.7)}
.focus\:placeholder-base-200\/75:focus::placeholder {
    color: hsl(var(--b2) / 0.75)}
.focus\:placeholder-base-200\/80:focus::placeholder {
    color: hsl(var(--b2) / 0.8)}
.focus\:placeholder-base-200\/90:focus::placeholder {
    color: hsl(var(--b2) / 0.9)}
.focus\:placeholder-base-200\/95:focus::placeholder {
    color: hsl(var(--b2) / 0.95)}
.focus\:placeholder-base-300:focus::placeholder {
    color: hsl(var(--b3) / 1)}
.focus\:placeholder-base-300\/0:focus::placeholder {
    color: hsl(var(--b3) / 0)}
.focus\:placeholder-base-300\/10:focus::placeholder {
    color: hsl(var(--b3) / 0.1)}
.focus\:placeholder-base-300\/100:focus::placeholder {
    color: hsl(var(--b3) / 1)}
.focus\:placeholder-base-300\/20:focus::placeholder {
    color: hsl(var(--b3) / 0.2)}
.focus\:placeholder-base-300\/25:focus::placeholder {
    color: hsl(var(--b3) / 0.25)}
.focus\:placeholder-base-300\/30:focus::placeholder {
    color: hsl(var(--b3) / 0.3)}
.focus\:placeholder-base-300\/40:focus::placeholder {
    color: hsl(var(--b3) / 0.4)}
.focus\:placeholder-base-300\/5:focus::placeholder {
    color: hsl(var(--b3) / 0.05)}
.focus\:placeholder-base-300\/50:focus::placeholder {
    color: hsl(var(--b3) / 0.5)}
.focus\:placeholder-base-300\/60:focus::placeholder {
    color: hsl(var(--b3) / 0.6)}
.focus\:placeholder-base-300\/70:focus::placeholder {
    color: hsl(var(--b3) / 0.7)}
.focus\:placeholder-base-300\/75:focus::placeholder {
    color: hsl(var(--b3) / 0.75)}
.focus\:placeholder-base-300\/80:focus::placeholder {
    color: hsl(var(--b3) / 0.8)}
.focus\:placeholder-base-300\/90:focus::placeholder {
    color: hsl(var(--b3) / 0.9)}
.focus\:placeholder-base-300\/95:focus::placeholder {
    color: hsl(var(--b3) / 0.95)}
.focus\:placeholder-base-content:focus::placeholder {
    color: hsl(var(--bc) / 1)}
.focus\:placeholder-base-content\/0:focus::placeholder {
    color: hsl(var(--bc) / 0)}
.focus\:placeholder-base-content\/10:focus::placeholder {
    color: hsl(var(--bc) / 0.1)}
.focus\:placeholder-base-content\/100:focus::placeholder {
    color: hsl(var(--bc) / 1)}
.focus\:placeholder-base-content\/20:focus::placeholder {
    color: hsl(var(--bc) / 0.2)}
.focus\:placeholder-base-content\/25:focus::placeholder {
    color: hsl(var(--bc) / 0.25)}
.focus\:placeholder-base-content\/30:focus::placeholder {
    color: hsl(var(--bc) / 0.3)}
.focus\:placeholder-base-content\/40:focus::placeholder {
    color: hsl(var(--bc) / 0.4)}
.focus\:placeholder-base-content\/5:focus::placeholder {
    color: hsl(var(--bc) / 0.05)}
.focus\:placeholder-base-content\/50:focus::placeholder {
    color: hsl(var(--bc) / 0.5)}
.focus\:placeholder-base-content\/60:focus::placeholder {
    color: hsl(var(--bc) / 0.6)}
.focus\:placeholder-base-content\/70:focus::placeholder {
    color: hsl(var(--bc) / 0.7)}
.focus\:placeholder-base-content\/75:focus::placeholder {
    color: hsl(var(--bc) / 0.75)}
.focus\:placeholder-base-content\/80:focus::placeholder {
    color: hsl(var(--bc) / 0.8)}
.focus\:placeholder-base-content\/90:focus::placeholder {
    color: hsl(var(--bc) / 0.9)}
.focus\:placeholder-base-content\/95:focus::placeholder {
    color: hsl(var(--bc) / 0.95)}
.focus\:placeholder-error:focus::placeholder {
    color: hsl(var(--er) / 1)}
.focus\:placeholder-error-content:focus::placeholder {
    color: hsl(var(--erc) / 1)}
.focus\:placeholder-error-content\/0:focus::placeholder {
    color: hsl(var(--erc) / 0)}
.focus\:placeholder-error-content\/10:focus::placeholder {
    color: hsl(var(--erc) / 0.1)}
.focus\:placeholder-error-content\/100:focus::placeholder {
    color: hsl(var(--erc) / 1)}
.focus\:placeholder-error-content\/20:focus::placeholder {
    color: hsl(var(--erc) / 0.2)}
.focus\:placeholder-error-content\/25:focus::placeholder {
    color: hsl(var(--erc) / 0.25)}
.focus\:placeholder-error-content\/30:focus::placeholder {
    color: hsl(var(--erc) / 0.3)}
.focus\:placeholder-error-content\/40:focus::placeholder {
    color: hsl(var(--erc) / 0.4)}
.focus\:placeholder-error-content\/5:focus::placeholder {
    color: hsl(var(--erc) / 0.05)}
.focus\:placeholder-error-content\/50:focus::placeholder {
    color: hsl(var(--erc) / 0.5)}
.focus\:placeholder-error-content\/60:focus::placeholder {
    color: hsl(var(--erc) / 0.6)}
.focus\:placeholder-error-content\/70:focus::placeholder {
    color: hsl(var(--erc) / 0.7)}
.focus\:placeholder-error-content\/75:focus::placeholder {
    color: hsl(var(--erc) / 0.75)}
.focus\:placeholder-error-content\/80:focus::placeholder {
    color: hsl(var(--erc) / 0.8)}
.focus\:placeholder-error-content\/90:focus::placeholder {
    color: hsl(var(--erc) / 0.9)}
.focus\:placeholder-error-content\/95:focus::placeholder {
    color: hsl(var(--erc) / 0.95)}
.focus\:placeholder-error\/0:focus::placeholder {
    color: hsl(var(--er) / 0)}
.focus\:placeholder-error\/10:focus::placeholder {
    color: hsl(var(--er) / 0.1)}
.focus\:placeholder-error\/100:focus::placeholder {
    color: hsl(var(--er) / 1)}
.focus\:placeholder-error\/20:focus::placeholder {
    color: hsl(var(--er) / 0.2)}
.focus\:placeholder-error\/25:focus::placeholder {
    color: hsl(var(--er) / 0.25)}
.focus\:placeholder-error\/30:focus::placeholder {
    color: hsl(var(--er) / 0.3)}
.focus\:placeholder-error\/40:focus::placeholder {
    color: hsl(var(--er) / 0.4)}
.focus\:placeholder-error\/5:focus::placeholder {
    color: hsl(var(--er) / 0.05)}
.focus\:placeholder-error\/50:focus::placeholder {
    color: hsl(var(--er) / 0.5)}
.focus\:placeholder-error\/60:focus::placeholder {
    color: hsl(var(--er) / 0.6)}
.focus\:placeholder-error\/70:focus::placeholder {
    color: hsl(var(--er) / 0.7)}
.focus\:placeholder-error\/75:focus::placeholder {
    color: hsl(var(--er) / 0.75)}
.focus\:placeholder-error\/80:focus::placeholder {
    color: hsl(var(--er) / 0.8)}
.focus\:placeholder-error\/90:focus::placeholder {
    color: hsl(var(--er) / 0.9)}
.focus\:placeholder-error\/95:focus::placeholder {
    color: hsl(var(--er) / 0.95)}
.focus\:placeholder-info:focus::placeholder {
    color: hsl(var(--in) / 1)}
.focus\:placeholder-info-content:focus::placeholder {
    color: hsl(var(--inc) / 1)}
.focus\:placeholder-info-content\/0:focus::placeholder {
    color: hsl(var(--inc) / 0)}
.focus\:placeholder-info-content\/10:focus::placeholder {
    color: hsl(var(--inc) / 0.1)}
.focus\:placeholder-info-content\/100:focus::placeholder {
    color: hsl(var(--inc) / 1)}
.focus\:placeholder-info-content\/20:focus::placeholder {
    color: hsl(var(--inc) / 0.2)}
.focus\:placeholder-info-content\/25:focus::placeholder {
    color: hsl(var(--inc) / 0.25)}
.focus\:placeholder-info-content\/30:focus::placeholder {
    color: hsl(var(--inc) / 0.3)}
.focus\:placeholder-info-content\/40:focus::placeholder {
    color: hsl(var(--inc) / 0.4)}
.focus\:placeholder-info-content\/5:focus::placeholder {
    color: hsl(var(--inc) / 0.05)}
.focus\:placeholder-info-content\/50:focus::placeholder {
    color: hsl(var(--inc) / 0.5)}
.focus\:placeholder-info-content\/60:focus::placeholder {
    color: hsl(var(--inc) / 0.6)}
.focus\:placeholder-info-content\/70:focus::placeholder {
    color: hsl(var(--inc) / 0.7)}
.focus\:placeholder-info-content\/75:focus::placeholder {
    color: hsl(var(--inc) / 0.75)}
.focus\:placeholder-info-content\/80:focus::placeholder {
    color: hsl(var(--inc) / 0.8)}
.focus\:placeholder-info-content\/90:focus::placeholder {
    color: hsl(var(--inc) / 0.9)}
.focus\:placeholder-info-content\/95:focus::placeholder {
    color: hsl(var(--inc) / 0.95)}
.focus\:placeholder-info\/0:focus::placeholder {
    color: hsl(var(--in) / 0)}
.focus\:placeholder-info\/10:focus::placeholder {
    color: hsl(var(--in) / 0.1)}
.focus\:placeholder-info\/100:focus::placeholder {
    color: hsl(var(--in) / 1)}
.focus\:placeholder-info\/20:focus::placeholder {
    color: hsl(var(--in) / 0.2)}
.focus\:placeholder-info\/25:focus::placeholder {
    color: hsl(var(--in) / 0.25)}
.focus\:placeholder-info\/30:focus::placeholder {
    color: hsl(var(--in) / 0.3)}
.focus\:placeholder-info\/40:focus::placeholder {
    color: hsl(var(--in) / 0.4)}
.focus\:placeholder-info\/5:focus::placeholder {
    color: hsl(var(--in) / 0.05)}
.focus\:placeholder-info\/50:focus::placeholder {
    color: hsl(var(--in) / 0.5)}
.focus\:placeholder-info\/60:focus::placeholder {
    color: hsl(var(--in) / 0.6)}
.focus\:placeholder-info\/70:focus::placeholder {
    color: hsl(var(--in) / 0.7)}
.focus\:placeholder-info\/75:focus::placeholder {
    color: hsl(var(--in) / 0.75)}
.focus\:placeholder-info\/80:focus::placeholder {
    color: hsl(var(--in) / 0.8)}
.focus\:placeholder-info\/90:focus::placeholder {
    color: hsl(var(--in) / 0.9)}
.focus\:placeholder-info\/95:focus::placeholder {
    color: hsl(var(--in) / 0.95)}
.focus\:placeholder-success:focus::placeholder {
    color: hsl(var(--su) / 1)}
.focus\:placeholder-success-content:focus::placeholder {
    color: hsl(var(--suc) / 1)}
.focus\:placeholder-success-content\/0:focus::placeholder {
    color: hsl(var(--suc) / 0)}
.focus\:placeholder-success-content\/10:focus::placeholder {
    color: hsl(var(--suc) / 0.1)}
.focus\:placeholder-success-content\/100:focus::placeholder {
    color: hsl(var(--suc) / 1)}
.focus\:placeholder-success-content\/20:focus::placeholder {
    color: hsl(var(--suc) / 0.2)}
.focus\:placeholder-success-content\/25:focus::placeholder {
    color: hsl(var(--suc) / 0.25)}
.focus\:placeholder-success-content\/30:focus::placeholder {
    color: hsl(var(--suc) / 0.3)}
.focus\:placeholder-success-content\/40:focus::placeholder {
    color: hsl(var(--suc) / 0.4)}
.focus\:placeholder-success-content\/5:focus::placeholder {
    color: hsl(var(--suc) / 0.05)}
.focus\:placeholder-success-content\/50:focus::placeholder {
    color: hsl(var(--suc) / 0.5)}
.focus\:placeholder-success-content\/60:focus::placeholder {
    color: hsl(var(--suc) / 0.6)}
.focus\:placeholder-success-content\/70:focus::placeholder {
    color: hsl(var(--suc) / 0.7)}
.focus\:placeholder-success-content\/75:focus::placeholder {
    color: hsl(var(--suc) / 0.75)}
.focus\:placeholder-success-content\/80:focus::placeholder {
    color: hsl(var(--suc) / 0.8)}
.focus\:placeholder-success-content\/90:focus::placeholder {
    color: hsl(var(--suc) / 0.9)}
.focus\:placeholder-success-content\/95:focus::placeholder {
    color: hsl(var(--suc) / 0.95)}
.focus\:placeholder-success\/0:focus::placeholder {
    color: hsl(var(--su) / 0)}
.focus\:placeholder-success\/10:focus::placeholder {
    color: hsl(var(--su) / 0.1)}
.focus\:placeholder-success\/100:focus::placeholder {
    color: hsl(var(--su) / 1)}
.focus\:placeholder-success\/20:focus::placeholder {
    color: hsl(var(--su) / 0.2)}
.focus\:placeholder-success\/25:focus::placeholder {
    color: hsl(var(--su) / 0.25)}
.focus\:placeholder-success\/30:focus::placeholder {
    color: hsl(var(--su) / 0.3)}
.focus\:placeholder-success\/40:focus::placeholder {
    color: hsl(var(--su) / 0.4)}
.focus\:placeholder-success\/5:focus::placeholder {
    color: hsl(var(--su) / 0.05)}
.focus\:placeholder-success\/50:focus::placeholder {
    color: hsl(var(--su) / 0.5)}
.focus\:placeholder-success\/60:focus::placeholder {
    color: hsl(var(--su) / 0.6)}
.focus\:placeholder-success\/70:focus::placeholder {
    color: hsl(var(--su) / 0.7)}
.focus\:placeholder-success\/75:focus::placeholder {
    color: hsl(var(--su) / 0.75)}
.focus\:placeholder-success\/80:focus::placeholder {
    color: hsl(var(--su) / 0.8)}
.focus\:placeholder-success\/90:focus::placeholder {
    color: hsl(var(--su) / 0.9)}
.focus\:placeholder-success\/95:focus::placeholder {
    color: hsl(var(--su) / 0.95)}
.focus\:placeholder-warning:focus::placeholder {
    color: hsl(var(--wa) / 1)}
.focus\:placeholder-warning-content:focus::placeholder {
    color: hsl(var(--wac) / 1)}
.focus\:placeholder-warning-content\/0:focus::placeholder {
    color: hsl(var(--wac) / 0)}
.focus\:placeholder-warning-content\/10:focus::placeholder {
    color: hsl(var(--wac) / 0.1)}
.focus\:placeholder-warning-content\/100:focus::placeholder {
    color: hsl(var(--wac) / 1)}
.focus\:placeholder-warning-content\/20:focus::placeholder {
    color: hsl(var(--wac) / 0.2)}
.focus\:placeholder-warning-content\/25:focus::placeholder {
    color: hsl(var(--wac) / 0.25)}
.focus\:placeholder-warning-content\/30:focus::placeholder {
    color: hsl(var(--wac) / 0.3)}
.focus\:placeholder-warning-content\/40:focus::placeholder {
    color: hsl(var(--wac) / 0.4)}
.focus\:placeholder-warning-content\/5:focus::placeholder {
    color: hsl(var(--wac) / 0.05)}
.focus\:placeholder-warning-content\/50:focus::placeholder {
    color: hsl(var(--wac) / 0.5)}
.focus\:placeholder-warning-content\/60:focus::placeholder {
    color: hsl(var(--wac) / 0.6)}
.focus\:placeholder-warning-content\/70:focus::placeholder {
    color: hsl(var(--wac) / 0.7)}
.focus\:placeholder-warning-content\/75:focus::placeholder {
    color: hsl(var(--wac) / 0.75)}
.focus\:placeholder-warning-content\/80:focus::placeholder {
    color: hsl(var(--wac) / 0.8)}
.focus\:placeholder-warning-content\/90:focus::placeholder {
    color: hsl(var(--wac) / 0.9)}
.focus\:placeholder-warning-content\/95:focus::placeholder {
    color: hsl(var(--wac) / 0.95)}
.focus\:placeholder-warning\/0:focus::placeholder {
    color: hsl(var(--wa) / 0)}
.focus\:placeholder-warning\/10:focus::placeholder {
    color: hsl(var(--wa) / 0.1)}
.focus\:placeholder-warning\/100:focus::placeholder {
    color: hsl(var(--wa) / 1)}
.focus\:placeholder-warning\/20:focus::placeholder {
    color: hsl(var(--wa) / 0.2)}
.focus\:placeholder-warning\/25:focus::placeholder {
    color: hsl(var(--wa) / 0.25)}
.focus\:placeholder-warning\/30:focus::placeholder {
    color: hsl(var(--wa) / 0.3)}
.focus\:placeholder-warning\/40:focus::placeholder {
    color: hsl(var(--wa) / 0.4)}
.focus\:placeholder-warning\/5:focus::placeholder {
    color: hsl(var(--wa) / 0.05)}
.focus\:placeholder-warning\/50:focus::placeholder {
    color: hsl(var(--wa) / 0.5)}
.focus\:placeholder-warning\/60:focus::placeholder {
    color: hsl(var(--wa) / 0.6)}
.focus\:placeholder-warning\/70:focus::placeholder {
    color: hsl(var(--wa) / 0.7)}
.focus\:placeholder-warning\/75:focus::placeholder {
    color: hsl(var(--wa) / 0.75)}
.focus\:placeholder-warning\/80:focus::placeholder {
    color: hsl(var(--wa) / 0.8)}
.focus\:placeholder-warning\/90:focus::placeholder {
    color: hsl(var(--wa) / 0.9)}
.focus\:placeholder-warning\/95:focus::placeholder {
    color: hsl(var(--wa) / 0.95)}
.focus\:ring-base-100:focus {
    --tw-ring-color: hsl(var(--b1) / 1)}
.focus\:ring-base-100\/0:focus {
    --tw-ring-color: hsl(var(--b1) / 0)}
.focus\:ring-base-100\/10:focus {
    --tw-ring-color: hsl(var(--b1) / 0.1)}
.focus\:ring-base-100\/100:focus {
    --tw-ring-color: hsl(var(--b1) / 1)}
.focus\:ring-base-100\/20:focus {
    --tw-ring-color: hsl(var(--b1) / 0.2)}
.focus\:ring-base-100\/25:focus {
    --tw-ring-color: hsl(var(--b1) / 0.25)}
.focus\:ring-base-100\/30:focus {
    --tw-ring-color: hsl(var(--b1) / 0.3)}
.focus\:ring-base-100\/40:focus {
    --tw-ring-color: hsl(var(--b1) / 0.4)}
.focus\:ring-base-100\/5:focus {
    --tw-ring-color: hsl(var(--b1) / 0.05)}
.focus\:ring-base-100\/50:focus {
    --tw-ring-color: hsl(var(--b1) / 0.5)}
.focus\:ring-base-100\/60:focus {
    --tw-ring-color: hsl(var(--b1) / 0.6)}
.focus\:ring-base-100\/70:focus {
    --tw-ring-color: hsl(var(--b1) / 0.7)}
.focus\:ring-base-100\/75:focus {
    --tw-ring-color: hsl(var(--b1) / 0.75)}
.focus\:ring-base-100\/80:focus {
    --tw-ring-color: hsl(var(--b1) / 0.8)}
.focus\:ring-base-100\/90:focus {
    --tw-ring-color: hsl(var(--b1) / 0.9)}
.focus\:ring-base-100\/95:focus {
    --tw-ring-color: hsl(var(--b1) / 0.95)}
.focus\:ring-base-200:focus {
    --tw-ring-color: hsl(var(--b2) / 1)}
.focus\:ring-base-200\/0:focus {
    --tw-ring-color: hsl(var(--b2) / 0)}
.focus\:ring-base-200\/10:focus {
    --tw-ring-color: hsl(var(--b2) / 0.1)}
.focus\:ring-base-200\/100:focus {
    --tw-ring-color: hsl(var(--b2) / 1)}
.focus\:ring-base-200\/20:focus {
    --tw-ring-color: hsl(var(--b2) / 0.2)}
.focus\:ring-base-200\/25:focus {
    --tw-ring-color: hsl(var(--b2) / 0.25)}
.focus\:ring-base-200\/30:focus {
    --tw-ring-color: hsl(var(--b2) / 0.3)}
.focus\:ring-base-200\/40:focus {
    --tw-ring-color: hsl(var(--b2) / 0.4)}
.focus\:ring-base-200\/5:focus {
    --tw-ring-color: hsl(var(--b2) / 0.05)}
.focus\:ring-base-200\/50:focus {
    --tw-ring-color: hsl(var(--b2) / 0.5)}
.focus\:ring-base-200\/60:focus {
    --tw-ring-color: hsl(var(--b2) / 0.6)}
.focus\:ring-base-200\/70:focus {
    --tw-ring-color: hsl(var(--b2) / 0.7)}
.focus\:ring-base-200\/75:focus {
    --tw-ring-color: hsl(var(--b2) / 0.75)}
.focus\:ring-base-200\/80:focus {
    --tw-ring-color: hsl(var(--b2) / 0.8)}
.focus\:ring-base-200\/90:focus {
    --tw-ring-color: hsl(var(--b2) / 0.9)}
.focus\:ring-base-200\/95:focus {
    --tw-ring-color: hsl(var(--b2) / 0.95)}
.focus\:ring-base-300:focus {
    --tw-ring-color: hsl(var(--b3) / 1)}
.focus\:ring-base-300\/0:focus {
    --tw-ring-color: hsl(var(--b3) / 0)}
.focus\:ring-base-300\/10:focus {
    --tw-ring-color: hsl(var(--b3) / 0.1)}
.focus\:ring-base-300\/100:focus {
    --tw-ring-color: hsl(var(--b3) / 1)}
.focus\:ring-base-300\/20:focus {
    --tw-ring-color: hsl(var(--b3) / 0.2)}
.focus\:ring-base-300\/25:focus {
    --tw-ring-color: hsl(var(--b3) / 0.25)}
.focus\:ring-base-300\/30:focus {
    --tw-ring-color: hsl(var(--b3) / 0.3)}
.focus\:ring-base-300\/40:focus {
    --tw-ring-color: hsl(var(--b3) / 0.4)}
.focus\:ring-base-300\/5:focus {
    --tw-ring-color: hsl(var(--b3) / 0.05)}
.focus\:ring-base-300\/50:focus {
    --tw-ring-color: hsl(var(--b3) / 0.5)}
.focus\:ring-base-300\/60:focus {
    --tw-ring-color: hsl(var(--b3) / 0.6)}
.focus\:ring-base-300\/70:focus {
    --tw-ring-color: hsl(var(--b3) / 0.7)}
.focus\:ring-base-300\/75:focus {
    --tw-ring-color: hsl(var(--b3) / 0.75)}
.focus\:ring-base-300\/80:focus {
    --tw-ring-color: hsl(var(--b3) / 0.8)}
.focus\:ring-base-300\/90:focus {
    --tw-ring-color: hsl(var(--b3) / 0.9)}
.focus\:ring-base-300\/95:focus {
    --tw-ring-color: hsl(var(--b3) / 0.95)}
.focus\:ring-base-content:focus {
    --tw-ring-color: hsl(var(--bc) / 1)}
.focus\:ring-base-content\/0:focus {
    --tw-ring-color: hsl(var(--bc) / 0)}
.focus\:ring-base-content\/10:focus {
    --tw-ring-color: hsl(var(--bc) / 0.1)}
.focus\:ring-base-content\/100:focus {
    --tw-ring-color: hsl(var(--bc) / 1)}
.focus\:ring-base-content\/20:focus {
    --tw-ring-color: hsl(var(--bc) / 0.2)}
.focus\:ring-base-content\/25:focus {
    --tw-ring-color: hsl(var(--bc) / 0.25)}
.focus\:ring-base-content\/30:focus {
    --tw-ring-color: hsl(var(--bc) / 0.3)}
.focus\:ring-base-content\/40:focus {
    --tw-ring-color: hsl(var(--bc) / 0.4)}
.focus\:ring-base-content\/5:focus {
    --tw-ring-color: hsl(var(--bc) / 0.05)}
.focus\:ring-base-content\/50:focus {
    --tw-ring-color: hsl(var(--bc) / 0.5)}
.focus\:ring-base-content\/60:focus {
    --tw-ring-color: hsl(var(--bc) / 0.6)}
.focus\:ring-base-content\/70:focus {
    --tw-ring-color: hsl(var(--bc) / 0.7)}
.focus\:ring-base-content\/75:focus {
    --tw-ring-color: hsl(var(--bc) / 0.75)}
.focus\:ring-base-content\/80:focus {
    --tw-ring-color: hsl(var(--bc) / 0.8)}
.focus\:ring-base-content\/90:focus {
    --tw-ring-color: hsl(var(--bc) / 0.9)}
.focus\:ring-base-content\/95:focus {
    --tw-ring-color: hsl(var(--bc) / 0.95)}
.focus\:ring-error:focus {
    --tw-ring-color: hsl(var(--er) / 1)}
.focus\:ring-error-content:focus {
    --tw-ring-color: hsl(var(--erc) / 1)}
.focus\:ring-error-content\/0:focus {
    --tw-ring-color: hsl(var(--erc) / 0)}
.focus\:ring-error-content\/10:focus {
    --tw-ring-color: hsl(var(--erc) / 0.1)}
.focus\:ring-error-content\/100:focus {
    --tw-ring-color: hsl(var(--erc) / 1)}
.focus\:ring-error-content\/20:focus {
    --tw-ring-color: hsl(var(--erc) / 0.2)}
.focus\:ring-error-content\/25:focus {
    --tw-ring-color: hsl(var(--erc) / 0.25)}
.focus\:ring-error-content\/30:focus {
    --tw-ring-color: hsl(var(--erc) / 0.3)}
.focus\:ring-error-content\/40:focus {
    --tw-ring-color: hsl(var(--erc) / 0.4)}
.focus\:ring-error-content\/5:focus {
    --tw-ring-color: hsl(var(--erc) / 0.05)}
.focus\:ring-error-content\/50:focus {
    --tw-ring-color: hsl(var(--erc) / 0.5)}
.focus\:ring-error-content\/60:focus {
    --tw-ring-color: hsl(var(--erc) / 0.6)}
.focus\:ring-error-content\/70:focus {
    --tw-ring-color: hsl(var(--erc) / 0.7)}
.focus\:ring-error-content\/75:focus {
    --tw-ring-color: hsl(var(--erc) / 0.75)}
.focus\:ring-error-content\/80:focus {
    --tw-ring-color: hsl(var(--erc) / 0.8)}
.focus\:ring-error-content\/90:focus {
    --tw-ring-color: hsl(var(--erc) / 0.9)}
.focus\:ring-error-content\/95:focus {
    --tw-ring-color: hsl(var(--erc) / 0.95)}
.focus\:ring-error\/0:focus {
    --tw-ring-color: hsl(var(--er) / 0)}
.focus\:ring-error\/10:focus {
    --tw-ring-color: hsl(var(--er) / 0.1)}
.focus\:ring-error\/100:focus {
    --tw-ring-color: hsl(var(--er) / 1)}
.focus\:ring-error\/20:focus {
    --tw-ring-color: hsl(var(--er) / 0.2)}
.focus\:ring-error\/25:focus {
    --tw-ring-color: hsl(var(--er) / 0.25)}
.focus\:ring-error\/30:focus {
    --tw-ring-color: hsl(var(--er) / 0.3)}
.focus\:ring-error\/40:focus {
    --tw-ring-color: hsl(var(--er) / 0.4)}
.focus\:ring-error\/5:focus {
    --tw-ring-color: hsl(var(--er) / 0.05)}
.focus\:ring-error\/50:focus {
    --tw-ring-color: hsl(var(--er) / 0.5)}
.focus\:ring-error\/60:focus {
    --tw-ring-color: hsl(var(--er) / 0.6)}
.focus\:ring-error\/70:focus {
    --tw-ring-color: hsl(var(--er) / 0.7)}
.focus\:ring-error\/75:focus {
    --tw-ring-color: hsl(var(--er) / 0.75)}
.focus\:ring-error\/80:focus {
    --tw-ring-color: hsl(var(--er) / 0.8)}
.focus\:ring-error\/90:focus {
    --tw-ring-color: hsl(var(--er) / 0.9)}
.focus\:ring-error\/95:focus {
    --tw-ring-color: hsl(var(--er) / 0.95)}
.focus\:ring-info:focus {
    --tw-ring-color: hsl(var(--in) / 1)}
.focus\:ring-info-content:focus {
    --tw-ring-color: hsl(var(--inc) / 1)}
.focus\:ring-info-content\/0:focus {
    --tw-ring-color: hsl(var(--inc) / 0)}
.focus\:ring-info-content\/10:focus {
    --tw-ring-color: hsl(var(--inc) / 0.1)}
.focus\:ring-info-content\/100:focus {
    --tw-ring-color: hsl(var(--inc) / 1)}
.focus\:ring-info-content\/20:focus {
    --tw-ring-color: hsl(var(--inc) / 0.2)}
.focus\:ring-info-content\/25:focus {
    --tw-ring-color: hsl(var(--inc) / 0.25)}
.focus\:ring-info-content\/30:focus {
    --tw-ring-color: hsl(var(--inc) / 0.3)}
.focus\:ring-info-content\/40:focus {
    --tw-ring-color: hsl(var(--inc) / 0.4)}
.focus\:ring-info-content\/5:focus {
    --tw-ring-color: hsl(var(--inc) / 0.05)}
.focus\:ring-info-content\/50:focus {
    --tw-ring-color: hsl(var(--inc) / 0.5)}
.focus\:ring-info-content\/60:focus {
    --tw-ring-color: hsl(var(--inc) / 0.6)}
.focus\:ring-info-content\/70:focus {
    --tw-ring-color: hsl(var(--inc) / 0.7)}
.focus\:ring-info-content\/75:focus {
    --tw-ring-color: hsl(var(--inc) / 0.75)}
.focus\:ring-info-content\/80:focus {
    --tw-ring-color: hsl(var(--inc) / 0.8)}
.focus\:ring-info-content\/90:focus {
    --tw-ring-color: hsl(var(--inc) / 0.9)}
.focus\:ring-info-content\/95:focus {
    --tw-ring-color: hsl(var(--inc) / 0.95)}
.focus\:ring-info\/0:focus {
    --tw-ring-color: hsl(var(--in) / 0)}
.focus\:ring-info\/10:focus {
    --tw-ring-color: hsl(var(--in) / 0.1)}
.focus\:ring-info\/100:focus {
    --tw-ring-color: hsl(var(--in) / 1)}
.focus\:ring-info\/20:focus {
    --tw-ring-color: hsl(var(--in) / 0.2)}
.focus\:ring-info\/25:focus {
    --tw-ring-color: hsl(var(--in) / 0.25)}
.focus\:ring-info\/30:focus {
    --tw-ring-color: hsl(var(--in) / 0.3)}
.focus\:ring-info\/40:focus {
    --tw-ring-color: hsl(var(--in) / 0.4)}
.focus\:ring-info\/5:focus {
    --tw-ring-color: hsl(var(--in) / 0.05)}
.focus\:ring-info\/50:focus {
    --tw-ring-color: hsl(var(--in) / 0.5)}
.focus\:ring-info\/60:focus {
    --tw-ring-color: hsl(var(--in) / 0.6)}
.focus\:ring-info\/70:focus {
    --tw-ring-color: hsl(var(--in) / 0.7)}
.focus\:ring-info\/75:focus {
    --tw-ring-color: hsl(var(--in) / 0.75)}
.focus\:ring-info\/80:focus {
    --tw-ring-color: hsl(var(--in) / 0.8)}
.focus\:ring-info\/90:focus {
    --tw-ring-color: hsl(var(--in) / 0.9)}
.focus\:ring-info\/95:focus {
    --tw-ring-color: hsl(var(--in) / 0.95)}
.focus\:ring-success:focus {
    --tw-ring-color: hsl(var(--su) / 1)}
.focus\:ring-success-content:focus {
    --tw-ring-color: hsl(var(--suc) / 1)}
.focus\:ring-success-content\/0:focus {
    --tw-ring-color: hsl(var(--suc) / 0)}
.focus\:ring-success-content\/10:focus {
    --tw-ring-color: hsl(var(--suc) / 0.1)}
.focus\:ring-success-content\/100:focus {
    --tw-ring-color: hsl(var(--suc) / 1)}
.focus\:ring-success-content\/20:focus {
    --tw-ring-color: hsl(var(--suc) / 0.2)}
.focus\:ring-success-content\/25:focus {
    --tw-ring-color: hsl(var(--suc) / 0.25)}
.focus\:ring-success-content\/30:focus {
    --tw-ring-color: hsl(var(--suc) / 0.3)}
.focus\:ring-success-content\/40:focus {
    --tw-ring-color: hsl(var(--suc) / 0.4)}
.focus\:ring-success-content\/5:focus {
    --tw-ring-color: hsl(var(--suc) / 0.05)}
.focus\:ring-success-content\/50:focus {
    --tw-ring-color: hsl(var(--suc) / 0.5)}
.focus\:ring-success-content\/60:focus {
    --tw-ring-color: hsl(var(--suc) / 0.6)}
.focus\:ring-success-content\/70:focus {
    --tw-ring-color: hsl(var(--suc) / 0.7)}
.focus\:ring-success-content\/75:focus {
    --tw-ring-color: hsl(var(--suc) / 0.75)}
.focus\:ring-success-content\/80:focus {
    --tw-ring-color: hsl(var(--suc) / 0.8)}
.focus\:ring-success-content\/90:focus {
    --tw-ring-color: hsl(var(--suc) / 0.9)}
.focus\:ring-success-content\/95:focus {
    --tw-ring-color: hsl(var(--suc) / 0.95)}
.focus\:ring-success\/0:focus {
    --tw-ring-color: hsl(var(--su) / 0)}
.focus\:ring-success\/10:focus {
    --tw-ring-color: hsl(var(--su) / 0.1)}
.focus\:ring-success\/100:focus {
    --tw-ring-color: hsl(var(--su) / 1)}
.focus\:ring-success\/20:focus {
    --tw-ring-color: hsl(var(--su) / 0.2)}
.focus\:ring-success\/25:focus {
    --tw-ring-color: hsl(var(--su) / 0.25)}
.focus\:ring-success\/30:focus {
    --tw-ring-color: hsl(var(--su) / 0.3)}
.focus\:ring-success\/40:focus {
    --tw-ring-color: hsl(var(--su) / 0.4)}
.focus\:ring-success\/5:focus {
    --tw-ring-color: hsl(var(--su) / 0.05)}
.focus\:ring-success\/50:focus {
    --tw-ring-color: hsl(var(--su) / 0.5)}
.focus\:ring-success\/60:focus {
    --tw-ring-color: hsl(var(--su) / 0.6)}
.focus\:ring-success\/70:focus {
    --tw-ring-color: hsl(var(--su) / 0.7)}
.focus\:ring-success\/75:focus {
    --tw-ring-color: hsl(var(--su) / 0.75)}
.focus\:ring-success\/80:focus {
    --tw-ring-color: hsl(var(--su) / 0.8)}
.focus\:ring-success\/90:focus {
    --tw-ring-color: hsl(var(--su) / 0.9)}
.focus\:ring-success\/95:focus {
    --tw-ring-color: hsl(var(--su) / 0.95)}
.focus\:ring-warning:focus {
    --tw-ring-color: hsl(var(--wa) / 1)}
.focus\:ring-warning-content:focus {
    --tw-ring-color: hsl(var(--wac) / 1)}
.focus\:ring-warning-content\/0:focus {
    --tw-ring-color: hsl(var(--wac) / 0)}
.focus\:ring-warning-content\/10:focus {
    --tw-ring-color: hsl(var(--wac) / 0.1)}
.focus\:ring-warning-content\/100:focus {
    --tw-ring-color: hsl(var(--wac) / 1)}
.focus\:ring-warning-content\/20:focus {
    --tw-ring-color: hsl(var(--wac) / 0.2)}
.focus\:ring-warning-content\/25:focus {
    --tw-ring-color: hsl(var(--wac) / 0.25)}
.focus\:ring-warning-content\/30:focus {
    --tw-ring-color: hsl(var(--wac) / 0.3)}
.focus\:ring-warning-content\/40:focus {
    --tw-ring-color: hsl(var(--wac) / 0.4)}
.focus\:ring-warning-content\/5:focus {
    --tw-ring-color: hsl(var(--wac) / 0.05)}
.focus\:ring-warning-content\/50:focus {
    --tw-ring-color: hsl(var(--wac) / 0.5)}
.focus\:ring-warning-content\/60:focus {
    --tw-ring-color: hsl(var(--wac) / 0.6)}
.focus\:ring-warning-content\/70:focus {
    --tw-ring-color: hsl(var(--wac) / 0.7)}
.focus\:ring-warning-content\/75:focus {
    --tw-ring-color: hsl(var(--wac) / 0.75)}
.focus\:ring-warning-content\/80:focus {
    --tw-ring-color: hsl(var(--wac) / 0.8)}
.focus\:ring-warning-content\/90:focus {
    --tw-ring-color: hsl(var(--wac) / 0.9)}
.focus\:ring-warning-content\/95:focus {
    --tw-ring-color: hsl(var(--wac) / 0.95)}
.focus\:ring-warning\/0:focus {
    --tw-ring-color: hsl(var(--wa) / 0)}
.focus\:ring-warning\/10:focus {
    --tw-ring-color: hsl(var(--wa) / 0.1)}
.focus\:ring-warning\/100:focus {
    --tw-ring-color: hsl(var(--wa) / 1)}
.focus\:ring-warning\/20:focus {
    --tw-ring-color: hsl(var(--wa) / 0.2)}
.focus\:ring-warning\/25:focus {
    --tw-ring-color: hsl(var(--wa) / 0.25)}
.focus\:ring-warning\/30:focus {
    --tw-ring-color: hsl(var(--wa) / 0.3)}
.focus\:ring-warning\/40:focus {
    --tw-ring-color: hsl(var(--wa) / 0.4)}
.focus\:ring-warning\/5:focus {
    --tw-ring-color: hsl(var(--wa) / 0.05)}
.focus\:ring-warning\/50:focus {
    --tw-ring-color: hsl(var(--wa) / 0.5)}
.focus\:ring-warning\/60:focus {
    --tw-ring-color: hsl(var(--wa) / 0.6)}
.focus\:ring-warning\/70:focus {
    --tw-ring-color: hsl(var(--wa) / 0.7)}
.focus\:ring-warning\/75:focus {
    --tw-ring-color: hsl(var(--wa) / 0.75)}
.focus\:ring-warning\/80:focus {
    --tw-ring-color: hsl(var(--wa) / 0.8)}
.focus\:ring-warning\/90:focus {
    --tw-ring-color: hsl(var(--wa) / 0.9)}
.focus\:ring-warning\/95:focus {
    --tw-ring-color: hsl(var(--wa) / 0.95)}
.focus\:ring-offset-base-100:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 1)}
.focus\:ring-offset-base-100\/0:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0)}
.focus\:ring-offset-base-100\/10:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.1)}
.focus\:ring-offset-base-100\/100:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 1)}
.focus\:ring-offset-base-100\/20:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.2)}
.focus\:ring-offset-base-100\/25:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.25)}
.focus\:ring-offset-base-100\/30:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.3)}
.focus\:ring-offset-base-100\/40:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.4)}
.focus\:ring-offset-base-100\/5:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.05)}
.focus\:ring-offset-base-100\/50:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.5)}
.focus\:ring-offset-base-100\/60:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.6)}
.focus\:ring-offset-base-100\/70:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.7)}
.focus\:ring-offset-base-100\/75:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.75)}
.focus\:ring-offset-base-100\/80:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.8)}
.focus\:ring-offset-base-100\/90:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.9)}
.focus\:ring-offset-base-100\/95:focus {
    --tw-ring-offset-color: hsl(var(--b1) / 0.95)}
.focus\:ring-offset-base-200:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 1)}
.focus\:ring-offset-base-200\/0:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0)}
.focus\:ring-offset-base-200\/10:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.1)}
.focus\:ring-offset-base-200\/100:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 1)}
.focus\:ring-offset-base-200\/20:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.2)}
.focus\:ring-offset-base-200\/25:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.25)}
.focus\:ring-offset-base-200\/30:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.3)}
.focus\:ring-offset-base-200\/40:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.4)}
.focus\:ring-offset-base-200\/5:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.05)}
.focus\:ring-offset-base-200\/50:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.5)}
.focus\:ring-offset-base-200\/60:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.6)}
.focus\:ring-offset-base-200\/70:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.7)}
.focus\:ring-offset-base-200\/75:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.75)}
.focus\:ring-offset-base-200\/80:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.8)}
.focus\:ring-offset-base-200\/90:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.9)}
.focus\:ring-offset-base-200\/95:focus {
    --tw-ring-offset-color: hsl(var(--b2) / 0.95)}
.focus\:ring-offset-base-300:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 1)}
.focus\:ring-offset-base-300\/0:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0)}
.focus\:ring-offset-base-300\/10:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.1)}
.focus\:ring-offset-base-300\/100:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 1)}
.focus\:ring-offset-base-300\/20:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.2)}
.focus\:ring-offset-base-300\/25:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.25)}
.focus\:ring-offset-base-300\/30:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.3)}
.focus\:ring-offset-base-300\/40:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.4)}
.focus\:ring-offset-base-300\/5:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.05)}
.focus\:ring-offset-base-300\/50:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.5)}
.focus\:ring-offset-base-300\/60:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.6)}
.focus\:ring-offset-base-300\/70:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.7)}
.focus\:ring-offset-base-300\/75:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.75)}
.focus\:ring-offset-base-300\/80:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.8)}
.focus\:ring-offset-base-300\/90:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.9)}
.focus\:ring-offset-base-300\/95:focus {
    --tw-ring-offset-color: hsl(var(--b3) / 0.95)}
.focus\:ring-offset-base-content:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 1)}
.focus\:ring-offset-base-content\/0:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0)}
.focus\:ring-offset-base-content\/10:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.1)}
.focus\:ring-offset-base-content\/100:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 1)}
.focus\:ring-offset-base-content\/20:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.2)}
.focus\:ring-offset-base-content\/25:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.25)}
.focus\:ring-offset-base-content\/30:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.3)}
.focus\:ring-offset-base-content\/40:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.4)}
.focus\:ring-offset-base-content\/5:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.05)}
.focus\:ring-offset-base-content\/50:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.5)}
.focus\:ring-offset-base-content\/60:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.6)}
.focus\:ring-offset-base-content\/70:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.7)}
.focus\:ring-offset-base-content\/75:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.75)}
.focus\:ring-offset-base-content\/80:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.8)}
.focus\:ring-offset-base-content\/90:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.9)}
.focus\:ring-offset-base-content\/95:focus {
    --tw-ring-offset-color: hsl(var(--bc) / 0.95)}
.focus\:ring-offset-error:focus {
    --tw-ring-offset-color: hsl(var(--er) / 1)}
.focus\:ring-offset-error-content:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 1)}
.focus\:ring-offset-error-content\/0:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0)}
.focus\:ring-offset-error-content\/10:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.1)}
.focus\:ring-offset-error-content\/100:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 1)}
.focus\:ring-offset-error-content\/20:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.2)}
.focus\:ring-offset-error-content\/25:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.25)}
.focus\:ring-offset-error-content\/30:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.3)}
.focus\:ring-offset-error-content\/40:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.4)}
.focus\:ring-offset-error-content\/5:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.05)}
.focus\:ring-offset-error-content\/50:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.5)}
.focus\:ring-offset-error-content\/60:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.6)}
.focus\:ring-offset-error-content\/70:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.7)}
.focus\:ring-offset-error-content\/75:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.75)}
.focus\:ring-offset-error-content\/80:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.8)}
.focus\:ring-offset-error-content\/90:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.9)}
.focus\:ring-offset-error-content\/95:focus {
    --tw-ring-offset-color: hsl(var(--erc) / 0.95)}
.focus\:ring-offset-error\/0:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0)}
.focus\:ring-offset-error\/10:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.1)}
.focus\:ring-offset-error\/100:focus {
    --tw-ring-offset-color: hsl(var(--er) / 1)}
.focus\:ring-offset-error\/20:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.2)}
.focus\:ring-offset-error\/25:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.25)}
.focus\:ring-offset-error\/30:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.3)}
.focus\:ring-offset-error\/40:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.4)}
.focus\:ring-offset-error\/5:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.05)}
.focus\:ring-offset-error\/50:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.5)}
.focus\:ring-offset-error\/60:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.6)}
.focus\:ring-offset-error\/70:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.7)}
.focus\:ring-offset-error\/75:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.75)}
.focus\:ring-offset-error\/80:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.8)}
.focus\:ring-offset-error\/90:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.9)}
.focus\:ring-offset-error\/95:focus {
    --tw-ring-offset-color: hsl(var(--er) / 0.95)}
.focus\:ring-offset-info:focus {
    --tw-ring-offset-color: hsl(var(--in) / 1)}
.focus\:ring-offset-info-content:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 1)}
.focus\:ring-offset-info-content\/0:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0)}
.focus\:ring-offset-info-content\/10:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.1)}
.focus\:ring-offset-info-content\/100:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 1)}
.focus\:ring-offset-info-content\/20:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.2)}
.focus\:ring-offset-info-content\/25:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.25)}
.focus\:ring-offset-info-content\/30:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.3)}
.focus\:ring-offset-info-content\/40:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.4)}
.focus\:ring-offset-info-content\/5:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.05)}
.focus\:ring-offset-info-content\/50:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.5)}
.focus\:ring-offset-info-content\/60:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.6)}
.focus\:ring-offset-info-content\/70:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.7)}
.focus\:ring-offset-info-content\/75:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.75)}
.focus\:ring-offset-info-content\/80:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.8)}
.focus\:ring-offset-info-content\/90:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.9)}
.focus\:ring-offset-info-content\/95:focus {
    --tw-ring-offset-color: hsl(var(--inc) / 0.95)}
.focus\:ring-offset-info\/0:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0)}
.focus\:ring-offset-info\/10:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.1)}
.focus\:ring-offset-info\/100:focus {
    --tw-ring-offset-color: hsl(var(--in) / 1)}
.focus\:ring-offset-info\/20:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.2)}
.focus\:ring-offset-info\/25:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.25)}
.focus\:ring-offset-info\/30:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.3)}
.focus\:ring-offset-info\/40:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.4)}
.focus\:ring-offset-info\/5:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.05)}
.focus\:ring-offset-info\/50:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.5)}
.focus\:ring-offset-info\/60:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.6)}
.focus\:ring-offset-info\/70:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.7)}
.focus\:ring-offset-info\/75:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.75)}
.focus\:ring-offset-info\/80:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.8)}
.focus\:ring-offset-info\/90:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.9)}
.focus\:ring-offset-info\/95:focus {
    --tw-ring-offset-color: hsl(var(--in) / 0.95)}
.focus\:ring-offset-success:focus {
    --tw-ring-offset-color: hsl(var(--su) / 1)}
.focus\:ring-offset-success-content:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 1)}
.focus\:ring-offset-success-content\/0:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0)}
.focus\:ring-offset-success-content\/10:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.1)}
.focus\:ring-offset-success-content\/100:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 1)}
.focus\:ring-offset-success-content\/20:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.2)}
.focus\:ring-offset-success-content\/25:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.25)}
.focus\:ring-offset-success-content\/30:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.3)}
.focus\:ring-offset-success-content\/40:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.4)}
.focus\:ring-offset-success-content\/5:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.05)}
.focus\:ring-offset-success-content\/50:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.5)}
.focus\:ring-offset-success-content\/60:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.6)}
.focus\:ring-offset-success-content\/70:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.7)}
.focus\:ring-offset-success-content\/75:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.75)}
.focus\:ring-offset-success-content\/80:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.8)}
.focus\:ring-offset-success-content\/90:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.9)}
.focus\:ring-offset-success-content\/95:focus {
    --tw-ring-offset-color: hsl(var(--suc) / 0.95)}
.focus\:ring-offset-success\/0:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0)}
.focus\:ring-offset-success\/10:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.1)}
.focus\:ring-offset-success\/100:focus {
    --tw-ring-offset-color: hsl(var(--su) / 1)}
.focus\:ring-offset-success\/20:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.2)}
.focus\:ring-offset-success\/25:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.25)}
.focus\:ring-offset-success\/30:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.3)}
.focus\:ring-offset-success\/40:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.4)}
.focus\:ring-offset-success\/5:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.05)}
.focus\:ring-offset-success\/50:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.5)}
.focus\:ring-offset-success\/60:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.6)}
.focus\:ring-offset-success\/70:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.7)}
.focus\:ring-offset-success\/75:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.75)}
.focus\:ring-offset-success\/80:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.8)}
.focus\:ring-offset-success\/90:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.9)}
.focus\:ring-offset-success\/95:focus {
    --tw-ring-offset-color: hsl(var(--su) / 0.95)}
.focus\:ring-offset-warning:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 1)}
.focus\:ring-offset-warning-content:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 1)}
.focus\:ring-offset-warning-content\/0:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0)}
.focus\:ring-offset-warning-content\/10:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.1)}
.focus\:ring-offset-warning-content\/100:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 1)}
.focus\:ring-offset-warning-content\/20:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.2)}
.focus\:ring-offset-warning-content\/25:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.25)}
.focus\:ring-offset-warning-content\/30:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.3)}
.focus\:ring-offset-warning-content\/40:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.4)}
.focus\:ring-offset-warning-content\/5:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.05)}
.focus\:ring-offset-warning-content\/50:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.5)}
.focus\:ring-offset-warning-content\/60:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.6)}
.focus\:ring-offset-warning-content\/70:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.7)}
.focus\:ring-offset-warning-content\/75:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.75)}
.focus\:ring-offset-warning-content\/80:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.8)}
.focus\:ring-offset-warning-content\/90:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.9)}
.focus\:ring-offset-warning-content\/95:focus {
    --tw-ring-offset-color: hsl(var(--wac) / 0.95)}
.focus\:ring-offset-warning\/0:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0)}
.focus\:ring-offset-warning\/10:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.1)}
.focus\:ring-offset-warning\/100:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 1)}
.focus\:ring-offset-warning\/20:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.2)}
.focus\:ring-offset-warning\/25:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.25)}
.focus\:ring-offset-warning\/30:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.3)}
.focus\:ring-offset-warning\/40:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.4)}
.focus\:ring-offset-warning\/5:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.05)}
.focus\:ring-offset-warning\/50:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.5)}
.focus\:ring-offset-warning\/60:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.6)}
.focus\:ring-offset-warning\/70:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.7)}
.focus\:ring-offset-warning\/75:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.75)}
.focus\:ring-offset-warning\/80:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.8)}
.focus\:ring-offset-warning\/90:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.9)}
.focus\:ring-offset-warning\/95:focus {
    --tw-ring-offset-color: hsl(var(--wa) / 0.95)}
@media (min-width: 640px) {
    .sm\:tab-rounded-lg {
        --tab-radius: 0.5rem}
    .sm\:badge-sm {
        height: 1rem;
        font-size: 0.75rem;
        line-height: 1rem;
        padding-left: 0.438rem;
        padding-right: 0.438rem}
    .sm\:badge-md {
        height: 1.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        padding-left: 0.563rem;
        padding-right: 0.563rem}
    .sm\:badge-lg {
        height: 1.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        padding-left: 0.688rem;
        padding-right: 0.688rem}
    .sm\:btm-nav-sm {
        height: 3rem}
    .sm\:btm-nav-sm > *:where(.active) {
        border-top-width: 2px}
    .sm\:btm-nav-sm .btm-nav-label {
        font-size: 0.75rem;
        line-height: 1rem}
    .sm\:btm-nav-md {
        height: 4rem}
    .sm\:btm-nav-md > *:where(.active) {
        border-top-width: 2px}
    .sm\:btm-nav-md .btm-nav-label {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .sm\:btm-nav-lg {
        height: 5rem}
    .sm\:btm-nav-lg > *:where(.active) {
        border-top-width: 4px}
    .sm\:btm-nav-lg .btm-nav-label {
        font-size: 1rem;
        line-height: 1.5rem}
    .sm\:btn-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        min-height: 2rem;
        font-size: 0.875rem}
    .sm\:btn-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        min-height: 3rem;
        font-size: 0.875rem}
    .sm\:btn-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        min-height: 4rem;
        font-size: 1.125rem}
    .btn-square:where(.sm\:btn-sm) {
        height: 2rem;
        width: 2rem;
        padding: 0px}
    .btn-square:where(.sm\:btn-md) {
        height: 3rem;
        width: 3rem;
        padding: 0px}
    .btn-square:where(.sm\:btn-lg) {
        height: 4rem;
        width: 4rem;
        padding: 0px}
    .btn-circle:where(.sm\:btn-sm) {
        height: 2rem;
        width: 2rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.sm\:btn-md) {
        height: 3rem;
        width: 3rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.sm\:btn-lg) {
        height: 4rem;
        width: 4rem;
        border-radius: 9999px;
        padding: 0px}
    .sm\:card-side {
        align-items: stretch;
        flex-direction: row}
    .sm\:card-side :where(figure:first-child) {
        overflow: hidden;
        border-start-start-radius: inherit;
        border-start-end-radius: unset;
        border-end-start-radius: inherit;
        border-end-end-radius: unset}
    .sm\:card-side :where(figure:last-child) {
        overflow: hidden;
        border-start-start-radius: unset;
        border-start-end-radius: inherit;
        border-end-start-radius: unset;
        border-end-end-radius: inherit}
    .sm\:card-side figure > * {
        max-width: unset}
    :where(.sm\:card-side figure > *) {
        width: 100%;
        height: 100%;
        object-fit: cover}
    .sm\:checkbox-sm {
        height: 1.25rem;
        width: 1.25rem}
    .sm\:checkbox-md {
        height: 1.5rem;
        width: 1.5rem}
    .sm\:checkbox-lg {
        height: 2rem;
        width: 2rem}
    .sm\:drawer-open > .drawer-toggle {
        display: none}
    .sm\:drawer-open > .drawer-toggle ~ .drawer-side {
        pointer-events: auto;
        visibility: visible;
        position: sticky;
        display: block;
        width: auto;
        overscroll-behavior: auto}
    .sm\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    [dir="rtl"] .sm\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    .sm\:drawer-open > .drawer-toggle:checked ~ .drawer-side {
        pointer-events: auto;
        visibility: visible}
    .sm\:file-input-sm {
        height: 2rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .sm\:file-input-sm::file-selector-button {
        margin-right: 0.75rem;
        height: 1.875rem;
        min-height: 1.875rem;
        font-size: 0.875rem}
    .sm\:file-input-md {
        height: 3rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .sm\:file-input-md::file-selector-button {
        margin-right: 1rem;
        height: 2.875rem;
        min-height: 2.875rem;
        font-size: 0.875rem}
    .sm\:file-input-lg {
        height: 4rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .sm\:file-input-lg::file-selector-button {
        margin-right: 1.5rem;
        height: 3.875rem;
        min-height: 3.875rem;
        font-size: 1.125rem}
    .sm\:input-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .sm\:input-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .sm\:input-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .sm\:kbd-sm {
        padding-left: 0.25rem;
        padding-right: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        min-height: 1.6em;
        min-width: 1.6em}
    .sm\:kbd-md {
        padding-left: 0.5rem;
        padding-right: 0.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        min-height: 2.2em;
        min-width: 2.2em}
    .sm\:kbd-lg {
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        min-height: 2.5em;
        min-width: 2.5em}
    .sm\:modal-top {
        place-items: start}
    .sm\:modal-middle {
        place-items: center}
    .sm\:modal-bottom {
        place-items: end}
    .sm\:radio-sm {
        height: 1.25rem;
        width: 1.25rem}
    .sm\:radio-md {
        height: 1.5rem;
        width: 1.5rem}
    .sm\:radio-lg {
        height: 2rem;
        width: 2rem}
    .sm\:range-sm {
        height: 1.25rem}
    .sm\:range-sm::-webkit-slider-runnable-track {
        height: 0.25rem}
    .sm\:range-sm::-moz-range-track {
        height: 0.25rem}
    .sm\:range-sm::-webkit-slider-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .sm\:range-sm::-moz-range-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .sm\:range-md {
        height: 1.5rem}
    .sm\:range-md::-webkit-slider-runnable-track {
        height: 0.5rem}
    .sm\:range-md::-moz-range-track {
        height: 0.5rem}
    .sm\:range-md::-webkit-slider-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .sm\:range-md::-moz-range-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .sm\:range-lg {
        height: 2rem}
    .sm\:range-lg::-webkit-slider-runnable-track {
        height: 1rem}
    .sm\:range-lg::-moz-range-track {
        height: 1rem}
    .sm\:range-lg::-webkit-slider-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .sm\:range-lg::-moz-range-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .sm\:rating-sm input {
        height: 1rem;
        width: 1rem}
    .sm\:rating-md input {
        height: 1.5rem;
        width: 1.5rem}
    .sm\:rating-lg input {
        height: 2.5rem;
        width: 2.5rem}
    .sm\:rating-sm.rating-half input:not(.rating-hidden) {
        width: 0.5rem}
    .sm\:rating-md.rating-half input:not(.rating-hidden) {
        width: 0.75rem}
    .sm\:rating-lg.rating-half input:not(.rating-hidden) {
        width: 1.25rem}
    .sm\:select-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 2.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        min-height: 3rem}
    [dir="rtl"] .sm\:select-md {
        padding-right: 1rem;
        padding-left: 2.5rem}
    .sm\:select-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 2rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        min-height: 4rem}
    [dir="rtl"] .sm\:select-lg {
        padding-right: 1.5rem;
        padding-left: 2rem}
    .sm\:select-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 2rem;
        font-size: 0.875rem;
        line-height: 2rem;
        min-height: 2rem}
    [dir="rtl"] .sm\:select-sm {
        padding-right: 0.75rem;
        padding-left: 2rem}
    .sm\:stats-horizontal {
        grid-auto-flow: column}
    .sm\:stats-vertical {
        grid-auto-flow: row}
    .sm\:tab-md {
        height: 2rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        --tab-padding: 1rem}
    .sm\:tab-lg {
        height: 3rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        --tab-padding: 1.25rem}
    .sm\:tab-sm {
        height: 1.5rem;
        font-size: 0.875rem;
        line-height: .75rem;
        --tab-padding: 0.75rem}
    .sm\:textarea-sm {
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .sm\:textarea-md {
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .sm\:textarea-lg {
        padding-top: 1rem;
        padding-bottom: 1rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .sm\:toggle-sm {
        --handleoffset: 0.75rem;
        height: 1.25rem;
        width: 2rem}
    .sm\:toggle-md {
        --handleoffset: 1.5rem;
        height: 1.5rem;
        width: 3rem}
    .sm\:toggle-lg {
        --handleoffset: 2rem;
        height: 2rem;
        width: 4rem}
    .sm\:card-compact .card-body {
        padding: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem}
    .sm\:card-compact .card-title {
        margin-bottom: 0.25rem}
    .sm\:card-normal .card-body {
        padding: var(--padding-card, 2rem);
        font-size: 1rem;
        line-height: 1.5rem}
    .sm\:card-normal .card-title {
        margin-bottom: 0.75rem}
    .sm\:drawer-open > .drawer-toggle ~ .drawer-side > .drawer-overlay {
        cursor: default;
        background-color: transparent}
    .sm\:menu-sm :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.sm\:menu-sm :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .sm\:menu-sm .menu-title {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .sm\:menu-md :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.sm\:menu-md :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .sm\:menu-md .menu-title {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .sm\:menu-lg :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.sm\:menu-lg :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .sm\:menu-lg .menu-title {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .sm\:modal-top :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: -2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: 0px;
        border-top-right-radius: 0px;
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .sm\:modal-middle :where(.modal-box) {
        width: 91.666667%;
        max-width: 32rem;
        --tw-translate-y: 0px;
        --tw-scale-x: .9;
        --tw-scale-y: .9;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem);
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .sm\:modal-bottom :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: 2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-bottom-right-radius: 0px;
        border-bottom-left-radius: 0px;
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem)}
    .sm\:stats-horizontal > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}
    .sm\:stats-horizontal {
        overflow-x: auto}
    .sm\:stats-vertical > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-y-reverse: 0;
        border-top-width: calc(1px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(1px * var(--tw-divide-y-reverse));
        --tw-divide-x-reverse: 0;
        border-right-width: calc(0px * var(--tw-divide-x-reverse));
        border-left-width: calc(0px * calc(1 - var(--tw-divide-x-reverse)))}
    .sm\:stats-vertical {
        overflow-y: auto}
    .sm\:table-sm :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .sm\:table-sm :where(th, td) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .sm\:table-md :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .sm\:table-md :where(th, td) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .sm\:table-lg :not(thead):not(tfoot) tr {
        font-size: 1rem;
        line-height: 1.5rem}
    .sm\:table-lg :where(th, td) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 1rem;
        padding-bottom: 1rem}}
@media (min-width: 768px) {
    .md\:tab-rounded-lg {
        --tab-radius: 0.5rem}
    .md\:badge-sm {
        height: 1rem;
        font-size: 0.75rem;
        line-height: 1rem;
        padding-left: 0.438rem;
        padding-right: 0.438rem}
    .md\:badge-md {
        height: 1.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        padding-left: 0.563rem;
        padding-right: 0.563rem}
    .md\:badge-lg {
        height: 1.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        padding-left: 0.688rem;
        padding-right: 0.688rem}
    .md\:btm-nav-sm {
        height: 3rem}
    .md\:btm-nav-sm > *:where(.active) {
        border-top-width: 2px}
    .md\:btm-nav-sm .btm-nav-label {
        font-size: 0.75rem;
        line-height: 1rem}
    .md\:btm-nav-md {
        height: 4rem}
    .md\:btm-nav-md > *:where(.active) {
        border-top-width: 2px}
    .md\:btm-nav-md .btm-nav-label {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .md\:btm-nav-lg {
        height: 5rem}
    .md\:btm-nav-lg > *:where(.active) {
        border-top-width: 4px}
    .md\:btm-nav-lg .btm-nav-label {
        font-size: 1rem;
        line-height: 1.5rem}
    .md\:btn-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        min-height: 2rem;
        font-size: 0.875rem}
    .md\:btn-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        min-height: 3rem;
        font-size: 0.875rem}
    .md\:btn-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        min-height: 4rem;
        font-size: 1.125rem}
    .btn-square:where(.md\:btn-sm) {
        height: 2rem;
        width: 2rem;
        padding: 0px}
    .btn-square:where(.md\:btn-md) {
        height: 3rem;
        width: 3rem;
        padding: 0px}
    .btn-square:where(.md\:btn-lg) {
        height: 4rem;
        width: 4rem;
        padding: 0px}
    .btn-circle:where(.md\:btn-sm) {
        height: 2rem;
        width: 2rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.md\:btn-md) {
        height: 3rem;
        width: 3rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.md\:btn-lg) {
        height: 4rem;
        width: 4rem;
        border-radius: 9999px;
        padding: 0px}
    .md\:card-side {
        align-items: stretch;
        flex-direction: row}
    .md\:card-side :where(figure:first-child) {
        overflow: hidden;
        border-start-start-radius: inherit;
        border-start-end-radius: unset;
        border-end-start-radius: inherit;
        border-end-end-radius: unset}
    .md\:card-side :where(figure:last-child) {
        overflow: hidden;
        border-start-start-radius: unset;
        border-start-end-radius: inherit;
        border-end-start-radius: unset;
        border-end-end-radius: inherit}
    .md\:card-side figure > * {
        max-width: unset}
    :where(.md\:card-side figure > *) {
        width: 100%;
        height: 100%;
        object-fit: cover}
    .md\:checkbox-sm {
        height: 1.25rem;
        width: 1.25rem}
    .md\:checkbox-md {
        height: 1.5rem;
        width: 1.5rem}
    .md\:checkbox-lg {
        height: 2rem;
        width: 2rem}
    .md\:drawer-open > .drawer-toggle {
        display: none}
    .md\:drawer-open > .drawer-toggle ~ .drawer-side {
        pointer-events: auto;
        visibility: visible;
        position: sticky;
        display: block;
        width: auto;
        overscroll-behavior: auto}
    .md\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    [dir="rtl"] .md\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    .md\:drawer-open > .drawer-toggle:checked ~ .drawer-side {
        pointer-events: auto;
        visibility: visible}
    .md\:file-input-sm {
        height: 2rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .md\:file-input-sm::file-selector-button {
        margin-right: 0.75rem;
        height: 1.875rem;
        min-height: 1.875rem;
        font-size: 0.875rem}
    .md\:file-input-md {
        height: 3rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .md\:file-input-md::file-selector-button {
        margin-right: 1rem;
        height: 2.875rem;
        min-height: 2.875rem;
        font-size: 0.875rem}
    .md\:file-input-lg {
        height: 4rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .md\:file-input-lg::file-selector-button {
        margin-right: 1.5rem;
        height: 3.875rem;
        min-height: 3.875rem;
        font-size: 1.125rem}
    .md\:input-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .md\:input-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .md\:input-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .md\:kbd-sm {
        padding-left: 0.25rem;
        padding-right: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        min-height: 1.6em;
        min-width: 1.6em}
    .md\:kbd-md {
        padding-left: 0.5rem;
        padding-right: 0.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        min-height: 2.2em;
        min-width: 2.2em}
    .md\:kbd-lg {
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        min-height: 2.5em;
        min-width: 2.5em}
    .md\:modal-top {
        place-items: start}
    .md\:modal-middle {
        place-items: center}
    .md\:modal-bottom {
        place-items: end}
    .md\:radio-sm {
        height: 1.25rem;
        width: 1.25rem}
    .md\:radio-md {
        height: 1.5rem;
        width: 1.5rem}
    .md\:radio-lg {
        height: 2rem;
        width: 2rem}
    .md\:range-sm {
        height: 1.25rem}
    .md\:range-sm::-webkit-slider-runnable-track {
        height: 0.25rem}
    .md\:range-sm::-moz-range-track {
        height: 0.25rem}
    .md\:range-sm::-webkit-slider-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .md\:range-sm::-moz-range-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .md\:range-md {
        height: 1.5rem}
    .md\:range-md::-webkit-slider-runnable-track {
        height: 0.5rem}
    .md\:range-md::-moz-range-track {
        height: 0.5rem}
    .md\:range-md::-webkit-slider-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .md\:range-md::-moz-range-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .md\:range-lg {
        height: 2rem}
    .md\:range-lg::-webkit-slider-runnable-track {
        height: 1rem}
    .md\:range-lg::-moz-range-track {
        height: 1rem}
    .md\:range-lg::-webkit-slider-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .md\:range-lg::-moz-range-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .md\:rating-sm input {
        height: 1rem;
        width: 1rem}
    .md\:rating-md input {
        height: 1.5rem;
        width: 1.5rem}
    .md\:rating-lg input {
        height: 2.5rem;
        width: 2.5rem}
    .md\:rating-sm.rating-half input:not(.rating-hidden) {
        width: 0.5rem}
    .md\:rating-md.rating-half input:not(.rating-hidden) {
        width: 0.75rem}
    .md\:rating-lg.rating-half input:not(.rating-hidden) {
        width: 1.25rem}
    .md\:select-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 2.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        min-height: 3rem}
    [dir="rtl"] .md\:select-md {
        padding-right: 1rem;
        padding-left: 2.5rem}
    .md\:select-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 2rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        min-height: 4rem}
    [dir="rtl"] .md\:select-lg {
        padding-right: 1.5rem;
        padding-left: 2rem}
    .md\:select-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 2rem;
        font-size: 0.875rem;
        line-height: 2rem;
        min-height: 2rem}
    [dir="rtl"] .md\:select-sm {
        padding-right: 0.75rem;
        padding-left: 2rem}
    .md\:stats-horizontal {
        grid-auto-flow: column}
    .md\:stats-vertical {
        grid-auto-flow: row}
    .md\:tab-md {
        height: 2rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        --tab-padding: 1rem}
    .md\:tab-lg {
        height: 3rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        --tab-padding: 1.25rem}
    .md\:tab-sm {
        height: 1.5rem;
        font-size: 0.875rem;
        line-height: .75rem;
        --tab-padding: 0.75rem}
    .md\:textarea-sm {
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .md\:textarea-md {
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .md\:textarea-lg {
        padding-top: 1rem;
        padding-bottom: 1rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .md\:toggle-sm {
        --handleoffset: 0.75rem;
        height: 1.25rem;
        width: 2rem}
    .md\:toggle-md {
        --handleoffset: 1.5rem;
        height: 1.5rem;
        width: 3rem}
    .md\:toggle-lg {
        --handleoffset: 2rem;
        height: 2rem;
        width: 4rem}
    .md\:card-compact .card-body {
        padding: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem}
    .md\:card-compact .card-title {
        margin-bottom: 0.25rem}
    .md\:card-normal .card-body {
        padding: var(--padding-card, 2rem);
        font-size: 1rem;
        line-height: 1.5rem}
    .md\:card-normal .card-title {
        margin-bottom: 0.75rem}
    .md\:drawer-open > .drawer-toggle ~ .drawer-side > .drawer-overlay {
        cursor: default;
        background-color: transparent}
    .md\:menu-sm :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.md\:menu-sm :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .md\:menu-sm .menu-title {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .md\:menu-md :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.md\:menu-md :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .md\:menu-md .menu-title {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .md\:menu-lg :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.md\:menu-lg :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .md\:menu-lg .menu-title {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .md\:modal-top :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: -2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: 0px;
        border-top-right-radius: 0px;
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .md\:modal-middle :where(.modal-box) {
        width: 91.666667%;
        max-width: 32rem;
        --tw-translate-y: 0px;
        --tw-scale-x: .9;
        --tw-scale-y: .9;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem);
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .md\:modal-bottom :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: 2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-bottom-right-radius: 0px;
        border-bottom-left-radius: 0px;
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem)}
    .md\:stats-horizontal > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}
    .md\:stats-horizontal {
        overflow-x: auto}
    .md\:stats-vertical > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-y-reverse: 0;
        border-top-width: calc(1px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(1px * var(--tw-divide-y-reverse));
        --tw-divide-x-reverse: 0;
        border-right-width: calc(0px * var(--tw-divide-x-reverse));
        border-left-width: calc(0px * calc(1 - var(--tw-divide-x-reverse)))}
    .md\:stats-vertical {
        overflow-y: auto}
    .md\:table-sm :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .md\:table-sm :where(th, td) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .md\:table-md :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .md\:table-md :where(th, td) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .md\:table-lg :not(thead):not(tfoot) tr {
        font-size: 1rem;
        line-height: 1.5rem}
    .md\:table-lg :where(th, td) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 1rem;
        padding-bottom: 1rem}}
@media (min-width: 1024px) {
    .lg\:tab-rounded-lg {
        --tab-radius: 0.5rem}
    .lg\:badge-sm {
        height: 1rem;
        font-size: 0.75rem;
        line-height: 1rem;
        padding-left: 0.438rem;
        padding-right: 0.438rem}
    .lg\:badge-md {
        height: 1.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        padding-left: 0.563rem;
        padding-right: 0.563rem}
    .lg\:badge-lg {
        height: 1.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        padding-left: 0.688rem;
        padding-right: 0.688rem}
    .lg\:btm-nav-sm {
        height: 3rem}
    .lg\:btm-nav-sm > *:where(.active) {
        border-top-width: 2px}
    .lg\:btm-nav-sm .btm-nav-label {
        font-size: 0.75rem;
        line-height: 1rem}
    .lg\:btm-nav-md {
        height: 4rem}
    .lg\:btm-nav-md > *:where(.active) {
        border-top-width: 2px}
    .lg\:btm-nav-md .btm-nav-label {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .lg\:btm-nav-lg {
        height: 5rem}
    .lg\:btm-nav-lg > *:where(.active) {
        border-top-width: 4px}
    .lg\:btm-nav-lg .btm-nav-label {
        font-size: 1rem;
        line-height: 1.5rem}
    .lg\:btn-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        min-height: 2rem;
        font-size: 0.875rem}
    .lg\:btn-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        min-height: 3rem;
        font-size: 0.875rem}
    .lg\:btn-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        min-height: 4rem;
        font-size: 1.125rem}
    .btn-square:where(.lg\:btn-sm) {
        height: 2rem;
        width: 2rem;
        padding: 0px}
    .btn-square:where(.lg\:btn-md) {
        height: 3rem;
        width: 3rem;
        padding: 0px}
    .btn-square:where(.lg\:btn-lg) {
        height: 4rem;
        width: 4rem;
        padding: 0px}
    .btn-circle:where(.lg\:btn-sm) {
        height: 2rem;
        width: 2rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.lg\:btn-md) {
        height: 3rem;
        width: 3rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.lg\:btn-lg) {
        height: 4rem;
        width: 4rem;
        border-radius: 9999px;
        padding: 0px}
    .lg\:card-side {
        align-items: stretch;
        flex-direction: row}
    .lg\:card-side :where(figure:first-child) {
        overflow: hidden;
        border-start-start-radius: inherit;
        border-start-end-radius: unset;
        border-end-start-radius: inherit;
        border-end-end-radius: unset}
    .lg\:card-side :where(figure:last-child) {
        overflow: hidden;
        border-start-start-radius: unset;
        border-start-end-radius: inherit;
        border-end-start-radius: unset;
        border-end-end-radius: inherit}
    .lg\:card-side figure > * {
        max-width: unset}
    :where(.lg\:card-side figure > *) {
        width: 100%;
        height: 100%;
        object-fit: cover}
    .lg\:checkbox-sm {
        height: 1.25rem;
        width: 1.25rem}
    .lg\:checkbox-md {
        height: 1.5rem;
        width: 1.5rem}
    .lg\:checkbox-lg {
        height: 2rem;
        width: 2rem}
    .lg\:drawer-open > .drawer-toggle {
        display: none}
    .lg\:drawer-open > .drawer-toggle ~ .drawer-side {
        pointer-events: auto;
        visibility: visible;
        position: sticky;
        display: block;
        width: auto;
        overscroll-behavior: auto}
    .lg\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    [dir="rtl"] .lg\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    .lg\:drawer-open > .drawer-toggle:checked ~ .drawer-side {
        pointer-events: auto;
        visibility: visible}
    .lg\:file-input-sm {
        height: 2rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .lg\:file-input-sm::file-selector-button {
        margin-right: 0.75rem;
        height: 1.875rem;
        min-height: 1.875rem;
        font-size: 0.875rem}
    .lg\:file-input-md {
        height: 3rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .lg\:file-input-md::file-selector-button {
        margin-right: 1rem;
        height: 2.875rem;
        min-height: 2.875rem;
        font-size: 0.875rem}
    .lg\:file-input-lg {
        height: 4rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .lg\:file-input-lg::file-selector-button {
        margin-right: 1.5rem;
        height: 3.875rem;
        min-height: 3.875rem;
        font-size: 1.125rem}
    .lg\:input-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .lg\:input-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .lg\:input-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .lg\:kbd-sm {
        padding-left: 0.25rem;
        padding-right: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        min-height: 1.6em;
        min-width: 1.6em}
    .lg\:kbd-md {
        padding-left: 0.5rem;
        padding-right: 0.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        min-height: 2.2em;
        min-width: 2.2em}
    .lg\:kbd-lg {
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        min-height: 2.5em;
        min-width: 2.5em}
    .lg\:modal-top {
        place-items: start}
    .lg\:modal-middle {
        place-items: center}
    .lg\:modal-bottom {
        place-items: end}
    .lg\:radio-sm {
        height: 1.25rem;
        width: 1.25rem}
    .lg\:radio-md {
        height: 1.5rem;
        width: 1.5rem}
    .lg\:radio-lg {
        height: 2rem;
        width: 2rem}
    .lg\:range-sm {
        height: 1.25rem}
    .lg\:range-sm::-webkit-slider-runnable-track {
        height: 0.25rem}
    .lg\:range-sm::-moz-range-track {
        height: 0.25rem}
    .lg\:range-sm::-webkit-slider-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .lg\:range-sm::-moz-range-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .lg\:range-md {
        height: 1.5rem}
    .lg\:range-md::-webkit-slider-runnable-track {
        height: 0.5rem}
    .lg\:range-md::-moz-range-track {
        height: 0.5rem}
    .lg\:range-md::-webkit-slider-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .lg\:range-md::-moz-range-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .lg\:range-lg {
        height: 2rem}
    .lg\:range-lg::-webkit-slider-runnable-track {
        height: 1rem}
    .lg\:range-lg::-moz-range-track {
        height: 1rem}
    .lg\:range-lg::-webkit-slider-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .lg\:range-lg::-moz-range-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .lg\:rating-sm input {
        height: 1rem;
        width: 1rem}
    .lg\:rating-md input {
        height: 1.5rem;
        width: 1.5rem}
    .lg\:rating-lg input {
        height: 2.5rem;
        width: 2.5rem}
    .lg\:rating-sm.rating-half input:not(.rating-hidden) {
        width: 0.5rem}
    .lg\:rating-md.rating-half input:not(.rating-hidden) {
        width: 0.75rem}
    .lg\:rating-lg.rating-half input:not(.rating-hidden) {
        width: 1.25rem}
    .lg\:select-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 2.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        min-height: 3rem}
    [dir="rtl"] .lg\:select-md {
        padding-right: 1rem;
        padding-left: 2.5rem}
    .lg\:select-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 2rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        min-height: 4rem}
    [dir="rtl"] .lg\:select-lg {
        padding-right: 1.5rem;
        padding-left: 2rem}
    .lg\:select-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 2rem;
        font-size: 0.875rem;
        line-height: 2rem;
        min-height: 2rem}
    [dir="rtl"] .lg\:select-sm {
        padding-right: 0.75rem;
        padding-left: 2rem}
    .lg\:stats-horizontal {
        grid-auto-flow: column}
    .lg\:stats-vertical {
        grid-auto-flow: row}
    .lg\:tab-md {
        height: 2rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        --tab-padding: 1rem}
    .lg\:tab-lg {
        height: 3rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        --tab-padding: 1.25rem}
    .lg\:tab-sm {
        height: 1.5rem;
        font-size: 0.875rem;
        line-height: .75rem;
        --tab-padding: 0.75rem}
    .lg\:textarea-sm {
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .lg\:textarea-md {
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .lg\:textarea-lg {
        padding-top: 1rem;
        padding-bottom: 1rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .lg\:toggle-sm {
        --handleoffset: 0.75rem;
        height: 1.25rem;
        width: 2rem}
    .lg\:toggle-md {
        --handleoffset: 1.5rem;
        height: 1.5rem;
        width: 3rem}
    .lg\:toggle-lg {
        --handleoffset: 2rem;
        height: 2rem;
        width: 4rem}
    .lg\:card-compact .card-body {
        padding: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem}
    .lg\:card-compact .card-title {
        margin-bottom: 0.25rem}
    .lg\:card-normal .card-body {
        padding: var(--padding-card, 2rem);
        font-size: 1rem;
        line-height: 1.5rem}
    .lg\:card-normal .card-title {
        margin-bottom: 0.75rem}
    .lg\:drawer-open > .drawer-toggle ~ .drawer-side > .drawer-overlay {
        cursor: default;
        background-color: transparent}
    .lg\:menu-sm :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.lg\:menu-sm :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .lg\:menu-sm .menu-title {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .lg\:menu-md :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.lg\:menu-md :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .lg\:menu-md .menu-title {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .lg\:menu-lg :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.lg\:menu-lg :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .lg\:menu-lg .menu-title {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .lg\:modal-top :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: -2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: 0px;
        border-top-right-radius: 0px;
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .lg\:modal-middle :where(.modal-box) {
        width: 91.666667%;
        max-width: 32rem;
        --tw-translate-y: 0px;
        --tw-scale-x: .9;
        --tw-scale-y: .9;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem);
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .lg\:modal-bottom :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: 2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-bottom-right-radius: 0px;
        border-bottom-left-radius: 0px;
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem)}
    .lg\:stats-horizontal > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}
    .lg\:stats-horizontal {
        overflow-x: auto}
    .lg\:stats-vertical > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-y-reverse: 0;
        border-top-width: calc(1px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(1px * var(--tw-divide-y-reverse));
        --tw-divide-x-reverse: 0;
        border-right-width: calc(0px * var(--tw-divide-x-reverse));
        border-left-width: calc(0px * calc(1 - var(--tw-divide-x-reverse)))}
    .lg\:stats-vertical {
        overflow-y: auto}
    .lg\:table-sm :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .lg\:table-sm :where(th, td) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .lg\:table-md :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .lg\:table-md :where(th, td) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .lg\:table-lg :not(thead):not(tfoot) tr {
        font-size: 1rem;
        line-height: 1.5rem}
    .lg\:table-lg :where(th, td) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 1rem;
        padding-bottom: 1rem}}
@media (min-width: 1280px) {
    .xl\:tab-rounded-lg {
        --tab-radius: 0.5rem}
    .xl\:badge-sm {
        height: 1rem;
        font-size: 0.75rem;
        line-height: 1rem;
        padding-left: 0.438rem;
        padding-right: 0.438rem}
    .xl\:badge-md {
        height: 1.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        padding-left: 0.563rem;
        padding-right: 0.563rem}
    .xl\:badge-lg {
        height: 1.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        padding-left: 0.688rem;
        padding-right: 0.688rem}
    .xl\:btm-nav-sm {
        height: 3rem}
    .xl\:btm-nav-sm > *:where(.active) {
        border-top-width: 2px}
    .xl\:btm-nav-sm .btm-nav-label {
        font-size: 0.75rem;
        line-height: 1rem}
    .xl\:btm-nav-md {
        height: 4rem}
    .xl\:btm-nav-md > *:where(.active) {
        border-top-width: 2px}
    .xl\:btm-nav-md .btm-nav-label {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .xl\:btm-nav-lg {
        height: 5rem}
    .xl\:btm-nav-lg > *:where(.active) {
        border-top-width: 4px}
    .xl\:btm-nav-lg .btm-nav-label {
        font-size: 1rem;
        line-height: 1.5rem}
    .xl\:btn-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        min-height: 2rem;
        font-size: 0.875rem}
    .xl\:btn-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        min-height: 3rem;
        font-size: 0.875rem}
    .xl\:btn-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        min-height: 4rem;
        font-size: 1.125rem}
    .btn-square:where(.xl\:btn-sm) {
        height: 2rem;
        width: 2rem;
        padding: 0px}
    .btn-square:where(.xl\:btn-md) {
        height: 3rem;
        width: 3rem;
        padding: 0px}
    .btn-square:where(.xl\:btn-lg) {
        height: 4rem;
        width: 4rem;
        padding: 0px}
    .btn-circle:where(.xl\:btn-sm) {
        height: 2rem;
        width: 2rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.xl\:btn-md) {
        height: 3rem;
        width: 3rem;
        border-radius: 9999px;
        padding: 0px}
    .btn-circle:where(.xl\:btn-lg) {
        height: 4rem;
        width: 4rem;
        border-radius: 9999px;
        padding: 0px}
    .xl\:card-side {
        align-items: stretch;
        flex-direction: row}
    .xl\:card-side :where(figure:first-child) {
        overflow: hidden;
        border-start-start-radius: inherit;
        border-start-end-radius: unset;
        border-end-start-radius: inherit;
        border-end-end-radius: unset}
    .xl\:card-side :where(figure:last-child) {
        overflow: hidden;
        border-start-start-radius: unset;
        border-start-end-radius: inherit;
        border-end-start-radius: unset;
        border-end-end-radius: inherit}
    .xl\:card-side figure > * {
        max-width: unset}
    :where(.xl\:card-side figure > *) {
        width: 100%;
        height: 100%;
        object-fit: cover}
    .xl\:checkbox-sm {
        height: 1.25rem;
        width: 1.25rem}
    .xl\:checkbox-md {
        height: 1.5rem;
        width: 1.5rem}
    .xl\:checkbox-lg {
        height: 2rem;
        width: 2rem}
    .xl\:drawer-open > .drawer-toggle {
        display: none}
    .xl\:drawer-open > .drawer-toggle ~ .drawer-side {
        pointer-events: auto;
        visibility: visible;
        position: sticky;
        display: block;
        width: auto;
        overscroll-behavior: auto}
    .xl\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    [dir="rtl"] .xl\:drawer-open > .drawer-toggle ~ .drawer-side > *:not(.drawer-overlay) {
        transform: translateX(0%)}
    .xl\:drawer-open > .drawer-toggle:checked ~ .drawer-side {
        pointer-events: auto;
        visibility: visible}
    .xl\:file-input-sm {
        height: 2rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .xl\:file-input-sm::file-selector-button {
        margin-right: 0.75rem;
        height: 1.875rem;
        min-height: 1.875rem;
        font-size: 0.875rem}
    .xl\:file-input-md {
        height: 3rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .xl\:file-input-md::file-selector-button {
        margin-right: 1rem;
        height: 2.875rem;
        min-height: 2.875rem;
        font-size: 0.875rem}
    .xl\:file-input-lg {
        height: 4rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .xl\:file-input-lg::file-selector-button {
        margin-right: 1.5rem;
        height: 3.875rem;
        min-height: 3.875rem;
        font-size: 1.125rem}
    .xl\:input-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .xl\:input-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .xl\:input-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .xl\:kbd-sm {
        padding-left: 0.25rem;
        padding-right: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        min-height: 1.6em;
        min-width: 1.6em}
    .xl\:kbd-md {
        padding-left: 0.5rem;
        padding-right: 0.5rem;
        font-size: 1rem;
        line-height: 1.5rem;
        min-height: 2.2em;
        min-width: 2.2em}
    .xl\:kbd-lg {
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        min-height: 2.5em;
        min-width: 2.5em}
    .xl\:modal-top {
        place-items: start}
    .xl\:modal-middle {
        place-items: center}
    .xl\:modal-bottom {
        place-items: end}
    .xl\:radio-sm {
        height: 1.25rem;
        width: 1.25rem}
    .xl\:radio-md {
        height: 1.5rem;
        width: 1.5rem}
    .xl\:radio-lg {
        height: 2rem;
        width: 2rem}
    .xl\:range-sm {
        height: 1.25rem}
    .xl\:range-sm::-webkit-slider-runnable-track {
        height: 0.25rem}
    .xl\:range-sm::-moz-range-track {
        height: 0.25rem}
    .xl\:range-sm::-webkit-slider-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .xl\:range-sm::-moz-range-thumb {
        height: 1.25rem;
        width: 1.25rem;
        --filler-offset: 0.5rem}
    .xl\:range-md {
        height: 1.5rem}
    .xl\:range-md::-webkit-slider-runnable-track {
        height: 0.5rem}
    .xl\:range-md::-moz-range-track {
        height: 0.5rem}
    .xl\:range-md::-webkit-slider-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .xl\:range-md::-moz-range-thumb {
        height: 1.5rem;
        width: 1.5rem;
        --filler-offset: 0.6rem}
    .xl\:range-lg {
        height: 2rem}
    .xl\:range-lg::-webkit-slider-runnable-track {
        height: 1rem}
    .xl\:range-lg::-moz-range-track {
        height: 1rem}
    .xl\:range-lg::-webkit-slider-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .xl\:range-lg::-moz-range-thumb {
        height: 2rem;
        width: 2rem;
        --filler-offset: 1rem}
    .xl\:rating-sm input {
        height: 1rem;
        width: 1rem}
    .xl\:rating-md input {
        height: 1.5rem;
        width: 1.5rem}
    .xl\:rating-lg input {
        height: 2.5rem;
        width: 2.5rem}
    .xl\:rating-sm.rating-half input:not(.rating-hidden) {
        width: 0.5rem}
    .xl\:rating-md.rating-half input:not(.rating-hidden) {
        width: 0.75rem}
    .xl\:rating-lg.rating-half input:not(.rating-hidden) {
        width: 1.25rem}
    .xl\:select-md {
        height: 3rem;
        padding-left: 1rem;
        padding-right: 2.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        min-height: 3rem}
    [dir="rtl"] .xl\:select-md {
        padding-right: 1rem;
        padding-left: 2.5rem}
    .xl\:select-lg {
        height: 4rem;
        padding-left: 1.5rem;
        padding-right: 2rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        min-height: 4rem}
    [dir="rtl"] .xl\:select-lg {
        padding-right: 1.5rem;
        padding-left: 2rem}
    .xl\:select-sm {
        height: 2rem;
        padding-left: 0.75rem;
        padding-right: 2rem;
        font-size: 0.875rem;
        line-height: 2rem;
        min-height: 2rem}
    [dir="rtl"] .xl\:select-sm {
        padding-right: 0.75rem;
        padding-left: 2rem}
    .xl\:stats-horizontal {
        grid-auto-flow: column}
    .xl\:stats-vertical {
        grid-auto-flow: row}
    .xl\:tab-md {
        height: 2rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2;
        --tab-padding: 1rem}
    .xl\:tab-lg {
        height: 3rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2;
        --tab-padding: 1.25rem}
    .xl\:tab-sm {
        height: 1.5rem;
        font-size: 0.875rem;
        line-height: .75rem;
        --tab-padding: 0.75rem}
    .xl\:textarea-sm {
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        font-size: 0.875rem;
        line-height: 2rem}
    .xl\:textarea-md {
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        padding-left: 1rem;
        padding-right: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        line-height: 2}
    .xl\:textarea-lg {
        padding-top: 1rem;
        padding-bottom: 1rem;
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        line-height: 2}
    .xl\:toggle-sm {
        --handleoffset: 0.75rem;
        height: 1.25rem;
        width: 2rem}
    .xl\:toggle-md {
        --handleoffset: 1.5rem;
        height: 1.5rem;
        width: 3rem}
    .xl\:toggle-lg {
        --handleoffset: 2rem;
        height: 2rem;
        width: 4rem}
    .xl\:card-compact .card-body {
        padding: 1rem;
        font-size: 0.875rem;
        line-height: 1.25rem}
    .xl\:card-compact .card-title {
        margin-bottom: 0.25rem}
    .xl\:card-normal .card-body {
        padding: var(--padding-card, 2rem);
        font-size: 1rem;
        line-height: 1.5rem}
    .xl\:card-normal .card-title {
        margin-bottom: 0.75rem}
    .xl\:drawer-open > .drawer-toggle ~ .drawer-side > .drawer-overlay {
        cursor: default;
        background-color: transparent}
    .xl\:menu-sm :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.xl\:menu-sm :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.25rem;
        padding-bottom: 0.25rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .xl\:menu-sm .menu-title {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .xl\:menu-md :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.xl\:menu-md :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem;
        font-size: 0.875rem;
        line-height: 1.25rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .xl\:menu-md .menu-title {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .xl\:menu-lg :where(li:not(.menu-title) > *:not(ul):not(details):not(.menu-title)),.xl\:menu-lg :where(li:not(.menu-title) > details > summary:not(.menu-title)) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem;
        font-size: 1.125rem;
        line-height: 1.75rem;
        border-radius: var(--rounded-btn, 0.5rem)}
    .xl\:menu-lg .menu-title {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .xl\:modal-top :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: -2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: 0px;
        border-top-right-radius: 0px;
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .xl\:modal-middle :where(.modal-box) {
        width: 91.666667%;
        max-width: 32rem;
        --tw-translate-y: 0px;
        --tw-scale-x: .9;
        --tw-scale-y: .9;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem);
        border-bottom-left-radius: var(--rounded-box, 1rem);
        border-bottom-right-radius: var(--rounded-box, 1rem)}
    .xl\:modal-bottom :where(.modal-box) {
        width: 100%;
        max-width: none;
        --tw-translate-y: 2.5rem;
        --tw-scale-x: 1;
        --tw-scale-y: 1;
        transform: translate(var(--tw-translate-x), var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));
        border-bottom-right-radius: 0px;
        border-bottom-left-radius: 0px;
        border-top-left-radius: var(--rounded-box, 1rem);
        border-top-right-radius: var(--rounded-box, 1rem)}
    .xl\:stats-horizontal > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-x-reverse: 0;
        border-right-width: calc(1px * var(--tw-divide-x-reverse));
        border-left-width: calc(1px * calc(1 - var(--tw-divide-x-reverse)));
        --tw-divide-y-reverse: 0;
        border-top-width: calc(0px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(0px * var(--tw-divide-y-reverse))}
    .xl\:stats-horizontal {
        overflow-x: auto}
    .xl\:stats-vertical > :not([hidden]) ~ :not([hidden]) {
        --tw-divide-y-reverse: 0;
        border-top-width: calc(1px * calc(1 - var(--tw-divide-y-reverse)));
        border-bottom-width: calc(1px * var(--tw-divide-y-reverse));
        --tw-divide-x-reverse: 0;
        border-right-width: calc(0px * var(--tw-divide-x-reverse));
        border-left-width: calc(0px * calc(1 - var(--tw-divide-x-reverse)))}
    .xl\:stats-vertical {
        overflow-y: auto}
    .xl\:table-sm :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .xl\:table-sm :where(th, td) {
        padding-left: 0.75rem;
        padding-right: 0.75rem;
        padding-top: 0.5rem;
        padding-bottom: 0.5rem}
    .xl\:table-md :not(thead):not(tfoot) tr {
        font-size: 0.875rem;
        line-height: 1.25rem}
    .xl\:table-md :where(th, td) {
        padding-left: 1rem;
        padding-right: 1rem;
        padding-top: 0.75rem;
        padding-bottom: 0.75rem}
    .xl\:table-lg :not(thead):not(tfoot) tr {
        font-size: 1rem;
        line-height: 1.5rem}
    .xl\:table-lg :where(th, td) {
        padding-left: 1.5rem;
        padding-right: 1.5rem;
        padding-top: 1rem;
        padding-bottom: 1rem}}